(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    109746,       3527]*)
(*NotebookOutlinePosition[    124203,       3926]*)
(*  CellTagsIndexPosition[    123722,       3907]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 4.2", "Title",
  TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
  TextAlignment->Center],

Cell[TextData[StyleBox["Short manual",
  FontWeight->"Bold"]], "Subtitle",
  PageBreakBelow->True,
  TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
  TextAlignment->Center],

Cell[TextData[{
  "HTML version: ",
  ButtonBox["http://www.feyncalc.org/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Introduction", "Section",
  PageBreakAbove->True,
  CellTags->{"Short Overview", "intro"}],

Cell[TextData[{
  "FeynCalc 4.2 is a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 or 4.0 package for algebraic calculations in high energy physics. It \
provides tools for Lorentz structure manipulation, Dirac algebra \
manipulation, color factor calculation, Feynman rule derivation, Feynman loop \
integral calculation (e.g. Passarino-Veltman reduction) and noncommutative \
algebra calculation, as well as various tabulations of Feynman parameter \
integrals, Mellin ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["transforms", 
          ButtonBox["\[Dagger]",
            ButtonData:>"introfoot",
            ButtonStyle->"Hyperlink"]], TraditionalForm]]],
  ", convolutions, lagrangians and Feynman rules. Also, translation \
facilities are provided to change the FeynCalc syntax to and from FORM syntax \
and to generate FORTRAN code."
}], "Text",
  CellTags->"intro"],

Cell[TextData[{
  "Some important functions for input of objects like ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Nu]"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", " ", "\[Nu]"}]], ",", " ", 
          SuperscriptBox["\[CurlyEpsilon]", 
            RowBox[{"\[Mu]", " ", "\[Nu]", " ", "\[Rho]", " ", "\[Sigma]"}]], 
          ",", " ", 
          RowBox[{"etc", "."}]}], TraditionalForm]]],
  " (abbreviations in parentheses) are:"
}], "Text",
  CellTags->"intro"],

Cell[TextData[{
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink"],
  " (",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink"],
  "), ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink"],
  " (",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink"],
  "), ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  " (",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"AddOnsLink"],
  "), ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"AddOnsLink"],
  " (",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"AddOnsLink"],
  "), ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"AddOnsLink"],
  " (",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"AddOnsLink"],
  "), ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink"],
  " (",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"AddOnsLink"],
  "). "
}], "Text"],

Cell["Some important functions for manipulations are:", "Text",
  CellTags->"intro"],

Cell[TextData[{
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink"],
  " ,",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Series2"],
  " , ",
  ButtonBox["ScalarProductCancel",
    ButtonData:>"ScalarProductCancel",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  CellTags->"intro"],

Cell["\<\
Some important functions for Feyman loop integral \
manipulations:\
\>", "Text",
  CellTags->"intro"],

Cell[TextData[{
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Text",
  CellTags->"intro"],

Cell[TextData[{
  "FeynCalc 4.2 is still compatible with FeynArts 2.2 if the FeynArts \
function ToFA1Convention is used on the generated amplitudes. While ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink"],
  "  work fine for not too complicated self-energy, triangle and box \
diagrams, the reader should be aware of the package ",
  ButtonBox["FormCalc",
    ButtonData:>{
      URL[ "http://www.feynarts.de/formcalc/"], None},
    ButtonStyle->"Hyperlink"],
  " by Thomas Hahn, which is more effective for the adventurous researcher \
calculating thousands of diagrams involving four external gauge bosons. \
Notice however that FormCalc (presently) does not reduce the \
Passarino-Veltman integrals to scalar integrals. Also FormCalc is limited to \
the 't Hooft Feynman gauge. FeynCalc provides algebraic redution of \
Passarino-Veltman integrals to scalar integrals via the function ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Text",
  CellTags->"intro"],

Cell[TextData[{
  "On a Pentium II 300 MHz computer with 128 MB of RAM running ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 for Linux, this noteboook needs less than 15 minutes to evaluate and \
less than 16 MB of memory."
}], "Text",
  CellTags->"intro"],

Cell["\<\
For more detailed information concerning FeynCalc functions you can \
investigate all of the source code in the file \"FeynCalc.m\" in the \
directory \"HighEnergyPhysics\" and the further packages in the \
sub-directories \"fctools\", \"fctables\" and others. A complete list of \
FeynCalc objects is stored in the variable $FeynCalcStuff. Not all functions \
are equally well documented in this notebook. Some are very special ones \
(e.g. for tools for 2-loop QCD diagrams originating in twist-2 OPE), some are \
still experimental.\
\>", "Text",
  CellTags->"intro"],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "scheme used in OneLoop is the naive one, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              SuperscriptBox["\[Gamma]", "5"], ",", 
              SuperscriptBox["\[Gamma]", "\[Mu]"]}], ")"}], "=", "0"}], 
        TraditionalForm]]],
  " in 4 and D dimensions. See also ",
  ButtonBox["ToLarin",
    ButtonData:>"ToLarin",
    ButtonStyle->"AddOnsLink"]
}], "Text",
  CellTags->"intro"],

Cell["The metric used is the one from Bjorken and Drell (+---).", "Text",
  CellTags->"intro"],

Cell[TextData[{
  "For suggestions and bug reports please see ",
  ButtonBox["http://www.feyncalc.org/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/"], None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Text",
  CellTags->"intro"],

Cell[TextData[{
  "\[Dagger] Mellin transforms (e.g. all integrals - except 57,58 and 59, and \
correcting a minor misprint in 14) of Appendix 7 from ",
  ButtonBox["hep-ph/9810241",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9810241"], None},
    ButtonStyle->"Hyperlink"],
  " are tabulated in ",
  ButtonBox["Integrate2.",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink"]
}], "SmallText",
  CellTags->{"intro", "introfoot"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Starting FeynCalc", "Section",
  CellTags->{"Load FeynCalc", "start"}],

Cell[TextData[{
  "If the directory \"HighEnergyPhysics\" has been put in the directory \
\"Applications\" in your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " installation tree, you should be able to load FeynCalc from a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session with no further ado."
}], "Text",
  CellTags->"start"],

Cell["This loads FeynCalc:", "MathCaption",
  CellTags->"start"],

Cell[BoxData[
    RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input",
  CellLabel->"In[5]:=  (16:33:37 on 8/23/02)",
  CellTags->"start"],

Cell[TextData[{
  "You can also put the last input line into your \"init.m\" file. This will \
cause automatic loading of FeynCalc on every ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " startup."
}], "Text",
  CellTags->"start"],

Cell[TextData[{
  "By loading FeynCalc the setting of the default output format type is \
automatically set to TraditionalForm. You might want to reverse this after \
quitting FeynCalc by clicking on the ",
  StyleBox["Cell", "TextNormal",
    FontWeight->"Bold"],
  " menu, then go to the ",
  StyleBox["Default Output Format Type ", "TextNormal",
    FontWeight->"Bold"],
  "item and select ",
  StyleBox["StandardForm",
    FontWeight->"Bold"],
  ", or by evaluating"
}], "Text",
  CellTags->"start"],

Cell[BoxData[
    StyleBox[
      RowBox[{"SetOptions", "[", 
        RowBox[{"$FrontEnd", ",", 
          RowBox[{"\"\<CommonDefaultFormatTypes\>\"", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<Output\>\"", "\[Rule]", "StandardForm"}], 
              "}"}]}]}], "]"}],
      "Input"]], "Input",
  CellLabel->"In[20]:=  (15:29:47 on 9/21/02)",
  CellTags->"start"],

Cell[TextData[{
  "This defines typesetting rules for ",
  StyleBox["q1",
    FontFamily->"Courier"],
  " and ",
  StyleBox["q2",
    FontFamily->"Courier"],
  " for the output format type TraditionalForm:"
}], "MathCaption",
  CellTags->"start"],

Cell[BoxData[
    RowBox[{"q1", "/:", 
      RowBox[{"MakeBoxes", "[", 
        RowBox[{"q1", ",", "TraditionalForm"}], "]"}], ":=", 
      RowBox[{"InterpretationBox", "[", 
        RowBox[{
          RowBox[{"SubscriptBox", "[", 
            RowBox[{"q", ",", "1"}], "]"}], ",", "q1"}], "]"}]}]], "Input",
  CellLabel->"In[21]:=  (15:30:15 on 9/21/02)",
  CellTags->"start"],

Cell[BoxData[
    RowBox[{"q2", "/:", 
      RowBox[{"MakeBoxes", "[", 
        RowBox[{"q2", ",", "TraditionalForm"}], "]"}], ":=", 
      RowBox[{"InterpretationBox", "[", 
        RowBox[{
          RowBox[{"SubscriptBox", "[", 
            RowBox[{"q", ",", "2"}], "]"}], ",", "q2"}], "]"}]}]], "Input",
  CellLabel->"In[22]:=  (15:30:17 on 9/21/02)",
  CellTags->"start"],

Cell[TextData[{
  "Evaluating ",
  StyleBox["q1",
    FontFamily->"Courier"],
  " and ",
  StyleBox["q2",
    FontFamily->"Courier"],
  " now returns formatted TraditionalForm output:"
}], "MathCaption",
  CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"q1", ",", "q2"}], "}"}]], "Input",
  CellLabel->"In[23]:=  (15:30:19 on 9/21/02)",
  CellTags->"start"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          InterpretationBox[
            SubscriptBox["q", "1"],
            q1], ",", 
          InterpretationBox[
            SubscriptBox["q", "2"],
            q2]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[23]=  (15:30:19 on 9/21/02)"]
}, Closed]],

Cell["\<\
Enforcing StandardForm gives 'raw' output usable for copying, \
pasting and re-evaluating:\
\>", "MathCaption",
  CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[9]:=  (16:34:08 on 8/23/02)",
  CellTags->"start"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"q1", ",", "q2"}], "}"}]], "Output",
  CellLabel->"Out[9]//StandardForm=  (16:34:08 on 8/23/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Extra subpackages", "Section",
  PageBreakAbove->True,
  CellTags->"About Extras"],

Cell[TextData[{
  "In the directory  \"HighEnergyPhysics\" are various files and \
subdirectories belonging to third-party packages which are not necessary to \
run \"standard\" FeynCalc calculations and which do not use the autoloading \
conventions of FeynCalc. These packages are not initialized by default when \
starting FeynCalc. Loading them can be switched on and off by setting \
appropiate variables to ",
  StyleBox["False",
    FontFamily->"Courier"],
  " or ",
  StyleBox["True",
    FontFamily->"Courier"],
  "."
}], "Text",
  CellTags->"About Extras"],

Cell["\<\
E.g. evaluating this before loading FeynCalc causes FeynCalc to \
load the subpackage TARCER:\
\>", "MathCaption",
  CellTags->"About Extras"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LoadTARCER", "=", "True"}], ";"}]], "Input",
  CellTags->"About Extras"],

Cell["\<\
You can also put the last input line into your \"FCConfig.m\" file.\
\
\>", "Text",
  CellTags->"About Extras"],

Cell[BoxData[
    FormBox[
      StyleBox[
        FrameBox[
          StyleBox[GridBox[{
                {
                  StyleBox["$LoadTARCER",
                    FontFamily->"Courier"], 
                  RowBox[{
                  "Specifies", " ", "if", " ", "TARCER", " ", "should", " ", 
                    "be", " ", "loaded", " ", "by", " ", "FeynCalc"}]},
                {
                  StyleBox["$LoadPhi",
                    FontFamily->"Courier"], 
                  RowBox[{
                  "Specifies", " ", "if", " ", "PHI", " ", "should", " ", 
                    "be", " ", "loaded", " ", "by", " ", "FeynCalc"}]},
                {
                  StyleBox["$LoadFeynArts",
                    FontFamily->"Courier"], 
                  RowBox[{"Specifies", " ", "if", " ", 
                    SuperscriptBox["FeynArts", 
                      ButtonBox["\[Dagger]",
                        ButtonData:>"extrafoot",
                        ButtonStyle->"Hyperlink"]], " ", "should", " ", "be", 
                    " ", "loaded", " ", "by", " ", "FeynCalc"}]}
                },
              ColumnAlignments->{Right, Left}],
            GridBoxOptions->{RowLines->False}]],
        "2ColumnBox"], TraditionalForm]], "Text",
  Background->None,
  CellTags->"About Extras"],

Cell["Variables switching loading of extra packages on and off. ", "Caption",
  CellTags->"About Extras"],

Cell[CellGroupData[{

Cell[TextData[{
  "TARCER -\n",
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " program for the reduction of two-loop propagator integrals"
}], "Subsection",
  CellTags->"tarcer"],

Cell["R. Mertig, R. Scharf", "InspectorLevel2",
  CellTags->"tarcer"],

Cell[TextData[{
  ButtonBox["hep-ph/9801383",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"],
  " at Los Alamos"
}], "Text",
  CellDingbat->"\[GrayCircle]",
  CellTags->"tarcer"],

Cell[TextData[{
  "published in ",
  StyleBox["Comp. Phys. Comm.",
    CellDingbat->"\[GrayCircle]"],
  " ",
  StyleBox["111",
    FontWeight->"Bold"],
  " (1998) 265-173"
}], "Text",
  CellDingbat->"\[GrayCircle]",
  CellTags->"tarcer"],

Cell["Loading TARCER", "Subsubsection",
  CellTags->"tarcer"],

Cell[TextData[{
  "If FeynCalc is not needed, simply evaluate the file \"TARCER.nb\". This \
produces a file of the form \"tarcer*.mx\". Later it is then only necessary \
to load this file (e.g. ",
  StyleBox["<<\"tarcer06.mx\"",
    FontFamily->"Courier"],
  "). You can later change parameters in \"TARCER.nb\" and reevaluate it. \
Alternatively you can get a preproduced .mx file at ",
  ButtonBox["http://www.feyncalc.org/tarcer/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/tarcer/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/tarcer/"],
  "."
}], "Text",
  CellTags->"tarcer"],

Cell[TextData[{
  "If instructed to do so, FeynCalc automatically loads any tarcer*.mx file \
found the directory \"Tarcer\" (if several are present, the last one found by \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s FileNames is loaded). To have FeynCalc load TARCER, evaluate"
}], "Text",
  CellTags->"tarcer"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LoadTARCER", "=", "True"}], ";"}]], "Input",
  CellTags->"tarcer"],

Cell[TextData[{
  "before ",
  ButtonBox["loading FeynCalc",
    ButtonData:>"Load FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"start"],
  ". You can also put the last input line into your \"FCConfig.m\" file."
}], "Text",
  CellTags->"tarcer"],

Cell[TextData[{
  "An example notebook is also available at ",
  ButtonBox["http://www.feyncalc.org/tarcer/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/tarcer/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/tarcer/"],
  "."
}], "Text",
  CellTags->"tarcer"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "PHI -\nA ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " package for ChPT calculations"
}], "Subsection",
  CellTags->"phi"],

Cell["F. J. Orellana", "InspectorLevel2",
  CellTags->"phi"],

Cell[TextData[{
  "PHI is a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " package for manipulating covariant derivatives etc. of effective theories \
like Chiral Perturbation Theory. Moreover it provides an alternative way of \
creating coupling definitions for FeynArts. This is convenient if one needs \
to generate a large number of Feynman rules from different lagrangians."
}], "Text",
  CellTags->"phi"],

Cell["To have FeynCalc load PHI, evaluate", "Text",
  CellTags->"phi"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LoadPhi", "=", "True"}], ";"}]], "Input",
  CellTags->"phi"],

Cell[TextData[{
  "before ",
  ButtonBox["loading FeynCalc",
    ButtonData:>"Load FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"start"],
  ". You can also put the last input line into your \"FCConfig.m\" file."
}], "Text",
  CellTags->"phi"],

Cell[TextData[{
  "More information is available at ",
  ButtonBox["http://www.feyncalc.org/phi/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/phi/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/phi/"],
  "."
}], "Text",
  CellTags->"phi"]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynArts", "Subsection",
  CellTags->{"fa", "extrafoor"}],

Cell["Hagen Eck, Sepp K\[UDoubleDot]blbeck and Thomas Hahn", "InspectorLevel2",
  CellTags->{"fa", "extrafoor"}],

Cell[TextData[{
  ButtonBox["hep-ph/0012260",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/0012260"], None},
    ButtonStyle->"Hyperlink"],
  " at Los Alamos"
}], "Text",
  CellDingbat->"\[GrayCircle]",
  CellTags->{"fa", "extrafoor"}],

Cell[TextData[{
  "old version published in ",
  StyleBox["Comp. Phys. Comm. ",
    CellDingbat->"\[GrayCircle]"],
  StyleBox["60",
    CellDingbat->"\[GrayCircle]",
    FontWeight->"Bold"],
  StyleBox[" (1990) 165",
    CellDingbat->"\[GrayCircle]"]
}], "Text",
  CellDingbat->"\[GrayCircle]",
  CellTags->{"fa", "extrafoor"}],

Cell["\<\
FeynArts is a Mathematica package written by Hagen Eck, Sepp K\
\[UDoubleDot]blbeck and Thomas Hahn for the generation and visualization of \
Feynman diagrams and amplitudes. The current version is being developed by \
Thomas Hahn independently of FeynCalc. As it turns out, it can, however, be \
used with FeynCalc with only a few modifications, which are done \
automatically when it is loaded by FeynCalc.\
\>", "Text",
  CellTags->{"fa", "extrafoor"}],

Cell["\<\
One use of FeynCalc and FeynArts together is to use FeynCalc to \
process the output you get from FeynArts. Another use is to use FeynCalc and \
PHI to generate the many Feynman rules needed in effective theories like ChPT \
and feed them to FeynArts.\
\>", "Text",
  CellTags->{"fa", "extrafoor"}],

Cell[TextData[{
  "To have FeynArts loaded by FeynCalc, you need first of all to get FeynArts \
at ",
  ButtonBox["http://www.feynarts.de/",
    ButtonData:>{
      URL[ "http://www.feynarts.de/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"Download FeynArts"],
  " and put it in the directory \"HighEnergyPhysics\". Then evaluate"
}], "Text",
  CellTags->{"fa", "extrafoor"}],

Cell[BoxData[{
    RowBox[{
      RowBox[{"$LoadPhi", "=", "True"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"$LoadFeynArts", "=", "True"}], ";"}]}], "Input",
  CellTags->{"fa", "extrafoor"}],

Cell[TextData[{
  "before ",
  ButtonBox["loading FeynCalc",
    ButtonData:>"Load FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"start"],
  ". You can also put the last input line into your \"FCConfig.m\" file."
}], "Text",
  CellTags->{"fa", "extrafoor"}],

Cell[TextData[{
  "More information about FeynArts is available at ",
  ButtonBox["http://www.feynarts.de/",
    ButtonData:>{
      URL[ "http://www.feynarts.de/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"Download FeynArts"],
  ". More information about using FeynArts, FeynCalc and PHI is available at \
",
  ButtonBox["http://www.feyncalc.org/phi/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/phi/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/phi/"],
  "."
}], "Text",
  CellTags->{"fa", "extrafoor"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example Calculations", "Section",
  PageBreakAbove->True,
  CellTags->{"Example Calculations", "ex"}],

Cell[TextData[{
  "This section is under development ... Please send suggestions to ",
  ButtonBox["feyncalc@feyncalc.org",
    ButtonData:>{
      URL[ "mailto:feyncalc@feyncalc.org"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/examples"],
  " and/or upload at ",
  ButtonBox["www.feyncalc.org/examples",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/examples"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/examples"],
  "..."
}], "Commentary",
  CellTags->"ex"],

Cell[TextData[{
  "The examples are from various areas of research or education. More are \
welcome. See also ",
  ButtonBox["www.feyncalc.org/examples",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/examples"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/examples"],
  "."
}], "Text",
  CellTags->"ex"],

Cell[CellGroupData[{

Cell[TextData[{
  "Tree level ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Gamma]", " ", "\[Gamma]"}], " ", "\[Rule]", " ", 
          RowBox[{
            SuperscriptBox["e", "+"], 
            SuperscriptBox["e", "-"]}]}], TraditionalForm]]],
  " in the standard model"
}], "Subsection",
  CellTags->"Tree level"],

Cell[TextData[{
  "Tree level pair production ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Gamma]", " ", "\[Gamma]"}], " ", "\[Rule]", " ", 
          RowBox[{
            SuperscriptBox["e", "+"], 
            SuperscriptBox["e", "-"]}]}], TraditionalForm]]]
}], "Text",
  CellTags->"Tree level"],

Cell["Define typesetting rules for p1, p2, k1, k2", "Text",
  CellTags->"Tree level"],

Cell["\<\
p1/: MakeBoxes[p1, fmt_] := InterpretationBox[SubscriptBox[p, 1], p1]\
\>", "Input",
  CellLabel->"In[23]:=  (21:45:30 on 5/16/01)",
  CellTags->"Tree level"],

Cell["\<\
k1/: MakeBoxes[k1, fmt_] := InterpretationBox[SubscriptBox[k, 1], k1]\
\>", "Input",
  CellLabel->"In[24]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell["\<\
p2/: MakeBoxes[p2, fmt_] := InterpretationBox[SubscriptBox[p, 2], p2]\
\>", "Input",
  CellLabel->"In[25]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell["\<\
k2/: MakeBoxes[k2, fmt_] := InterpretationBox[SubscriptBox[k, 2], k2]\
\>", "Input",
  CellLabel->"In[26]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell["Define slashes of external momenta to save future typing.", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"{", 
        RowBox[{"p1sl", ",", "p2sl", ",", "k1sl", ",", "k2sl"}], "}"}], "=", 
      RowBox[{"DiracSlash", "/@", 
        RowBox[{"{", 
          RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2"}], "}"}]}]}]], "Input",\

  CellLabel->"In[27]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              InterpretationBox[
                SubscriptBox["k", "1"],
                k1],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              InterpretationBox[
                SubscriptBox["k", "2"],
                k2],
              "TraditionalForm"]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[27]=  (21:45:31 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Kinematic definitions so that only ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["p", "1"], ".", 
          SubscriptBox["k", "1"]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["p", "1"], ".", 
          SubscriptBox["k", "2"]}], TraditionalForm]]],
  " appear in final results. I.e., define ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["p", "1", "2"], "=", 
          RowBox[{
            SubsuperscriptBox["p", "2", "2"], "=", " ", 
            SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["k", "1", "2"], "=", 
          RowBox[{
            SubsuperscriptBox["k", "2", "2"], "=", " ", "0"}]}], 
        TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["k", "2"]}], " ", "=", " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["p", "1"], "\[CenterDot]", 
              SubscriptBox["p", "2"]}], "+", 
            SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], " ", "=", " ", 
          RowBox[{
            SubscriptBox["k", "2"], "\[CenterDot]", 
            SubscriptBox["p", "1"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "2"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], " ", "=", " ", 
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["p", "1"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], "=", " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["k", "2"], "\[CenterDot]", 
              SubscriptBox["p", "1"]}], "-", 
            SuperscriptBox["m", "2"], "+", 
            RowBox[{
              SubscriptBox["k", "1"], "\[CenterDot]", 
              SubscriptBox["p", "1"]}]}]}], TraditionalForm]]]
}], "Text",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p1", ",", "p1"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p2", ",", "p2"}], "]"}], "=", 
          SuperscriptBox["m", "2"]}]}], ";"}]], "Input",
  CellLabel->"In[28]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "k1"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k2", ",", "k2"}], "]"}], "=", "0"}]}], ";"}]], "Input",
  CellLabel->"In[29]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "k2"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p1", ",", "p2"}], "]"}], "+", 
          SuperscriptBox["m", "2"]}]}], ";"}]], "Input",
  CellLabel->"In[30]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "p2"}], "]"}], "=", 
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k2", ",", "p1"}], "]"}]}], ";"}]], "Input",
  CellLabel->"In[31]:=  (21:45:31 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k2", ",", "p2"}], "]"}], "=", 
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "p1"}], "]"}]}], ";"}]], "Input",
  CellLabel->"In[32]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p1", ",", "p2"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k2", ",", "p1"}], "]"}], "-", 
          SuperscriptBox["m", "2"], "+", 
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k1", ",", "p1"}], "]"}]}]}], ";"}]], "Input",
  CellLabel->"In[33]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell["Define the amplitudes.", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f1", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input",
  CellLabel->"In[34]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[34]=  (21:45:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input",
  CellLabel->"In[35]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[35]=  (21:45:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f3", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input",
  CellLabel->"In[36]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[36]=  (21:45:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f4", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input",
  CellLabel->"In[37]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[37]=  (21:45:32 on 5/16/01)"]
}, Closed]],

Cell[" Calculation of the four traces.", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell["tr1 = Tr[(p1sl + m) . f1 . (p2sl - m) . f3]", "Input",
  CellLabel->"In[38]:=  (21:45:32 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[38]=  (21:45:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell["tr2 = Tr[(p1sl + m) . f1 . (p2sl - m) . f4]", "Input",
  CellLabel->"In[39]:=  (21:45:33 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[39]=  (21:45:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell["tr3 = Tr[(p1sl + m) . f2 . (p2sl - m) . f3]", "Input",
  CellLabel->"In[40]:=  (21:45:33 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[40]=  (21:45:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell["tr4 = Tr[(p1sl + m) . f2 . (p2sl - m) . f4]", "Input",
  CellLabel->"In[41]:=  (21:45:34 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[41]=  (21:45:34 on 5/16/01)"]
}, Closed]],

Cell["Divide by the denominators and add everything up.", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"matrixsq", "=", 
      RowBox[{
        FractionBox["tr1", 
          SuperscriptBox[
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], "2"]], "+", 
        FractionBox["tr4", 
          SuperscriptBox[
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}], "2"]], "+", 
        FractionBox[
          RowBox[{"tr2", "+", "tr3"}], 
          RowBox[{
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], " ", 
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}]}]]}]}]], "Input",\

  CellLabel->"In[42]:=  (21:45:34 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", "8"}], " ", 
                  SuperscriptBox["m", "4"]}], "+", 
                RowBox[{"4", " ", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["k", "1"],
                        k1],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox[
                        StyleBox["\<\"p\"\>",
                          FontSlant->"Italic"], "1"],
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}], "+", 
                RowBox[{"4", " ", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["k", "2"],
                        k2],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox[
                        StyleBox["\<\"p\"\>",
                          FontSlant->"Italic"], "1"],
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "1"],
                      k1],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "2"],
                      k2],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[42]=  (21:45:34 on 5/16/01)"]
}, Closed]],

Cell["Collect with respect to powers of m.", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r", "=", 
      RowBox[{"Collect2", "[", 
        RowBox[{"matrixsq", ",", "m"}], "]"}]}]], "Input",
  CellLabel->"In[43]:=  (21:45:34 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"8", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "1"],
                          k1],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox[
                          StyleBox["\<\"p\"\>",
                            FontSlant->"Italic"], "1"],
                        "TraditionalForm"]}], "+", 
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "2"],
                          k2],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox[
                          StyleBox["\<\"p\"\>",
                            FontSlant->"Italic"], "1"],
                        "TraditionalForm"]}]}], ")"}], "2"], " ", 
              SuperscriptBox["m", "4"]}], 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "1"],
                          k1],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox[
                          StyleBox["\<\"p\"\>",
                            FontSlant->"Italic"], "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "2"],
                          k2],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox[
                          StyleBox["\<\"p\"\>",
                            FontSlant->"Italic"], "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}]]}], "+", 
        FractionBox[
          RowBox[{"16", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "1"],
                      k1],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "2"],
                      k2],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}]}], ")"}], " ", 
            SuperscriptBox["m", "2"]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}]], "+", 
        FractionBox[
          RowBox[{"8", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["k", "1"],
                            k1],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox[
                            StyleBox["\<\"p\"\>",
                              FontSlant->"Italic"], "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["k", "2"],
                            k2],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox[
                            StyleBox["\<\"p\"\>",
                              FontSlant->"Italic"], "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], ")"}]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[43]=  (21:45:35 on 5/16/01)"]
}, Closed]],

Cell["\<\
This is a possibility to check the symmetry in k1 and k2 explicitly.\
\>", "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"(", 
      RowBox[{"r", " ", "-", " ", 
        RowBox[{"(", 
          RowBox[{"r", " ", "/.", " ", 
            RowBox[{"{", " ", 
              RowBox[{
                RowBox[{"k1", "\[RuleDelayed]", "k2"}], ",", " ", 
                RowBox[{"k2", "\[RuleDelayed]", "k1"}]}], "}"}]}], ")"}]}], 
      ")"}]], "Input",
  CellLabel->"In[44]:=  (21:45:35 on 5/16/01)",
  CellTags->"Tree level"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[44]=  (21:45:35 on 5/16/01)"]
}, Closed]],

Cell["\<\
Remove the definitions for the scalar products and for the typesetting \
definitions.\
\>", "Text",
  CellTags->"Tree level"],

Cell[BoxData[
    RowBox[{"ClearScalarProducts", ";", " ", 
      RowBox[{"Clear", "[", 
        RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2", ",", "r"}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[45]:=  (21:45:35 on 5/16/01)",
  CellTags->"Tree level"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ClearScalarProducts",
    ButtonData:>"ClearScalarProducts",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  CellTags->"Tree level"]
}, Closed]],

Cell[CellGroupData[{

Cell["A moment integral", "Subsection",
  CellTags->"A moment integral"],

Cell[TextData[{
  "Moments of ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["A",
                FontSlant->"Plain"], "^"],
            FontSlant->"Plain"], "qq", 
          RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
  " from ",
  ButtonBox["hep-ph/9801224",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801224"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  CellTags->"A moment integral"],

Cell["\<\
This is an example of integration and of translation from FORM syntax.\
\>", "Text",
  CellTags->"A moment integral"],

Cell[TextData[{
  "Due to current conservation the first moment of ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["A",
                FontSlant->"Plain"], "^"],
            FontSlant->"Plain"], "qq", 
          RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
  "should vanish."
}], "Text",
  CellTags->"A moment integral"],

Cell["Translate FORM output (jqq12.log) to FeynCalc", "Text",
  CellTags->"A moment integral"],

Cell[TextData[{
  "The FORM file ",
  "jqq12.log ",
  "was generated by one of the authors of ",
  ButtonBox["hep-ph/9801224",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801224"], None},
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"e", " ", "=", " ", "Epsilon"}]], "Input",
  CellLabel->"In[47]:=  (21:48:35 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      TagBox["\[CurlyEpsilon]",
        TraditionalForm], TraditionalForm]], "Output",
  CellLabel->"Out[47]=  (21:48:35 on 5/16/01)"]
}, Closed]],

Cell["\<\
Special FORM convenctions like ln(1-x) can be translated automatically by \
FORM2FeynCalc, but they have to be listed once in the option Replace of \
FORM2FeynCalc :\
\>", "Text",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"FORM2FeynCalc", ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"\"\<ln(x)\>\"", "\[Rule]", "\"\<Log[x]\>\""}], ",", 
                RowBox[{"\"\<ln(1+x)\>\"", "\[Rule]", "\"\<Log[1+x]\>\""}], 
                ",", 
                RowBox[{"\"\<ln(1-x)\>\"", "\[Rule]", "\"\<Log[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li2(-x)\>\"", "\[Rule]", "\"\<Li2[-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li2(1-x)\>\"", "\[Rule]", "\"\<Li2[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li3(1-x)\>\"", "\[Rule]", "\"\<Li3[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li3(-x)\>\"", "\[Rule]", "\"\<Li3[-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(1-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,1-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(x^2)\>\"", "\[Rule]", "\"\<Nielsen[1,2,x^2]\>\""}], 
                ",", 
                RowBox[{"\"\<Z2\>\"", "\[Rule]", "\"\<Zeta2\>\""}], ",", 
                RowBox[{"\"\<Z3\>\"", "\[Rule]", "\"\<Zeta[3]\>\""}], ",", 
                RowBox[{"\"\<[1+x]^-1\>\"", "\[Rule]", "\"\<(1+x)^-1\>\""}], 
                ",", 
                RowBox[{
                "\"\<[(1-x)+]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], ",", 
                RowBox[{"\"\<[1-x]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], 
                ",", 
                RowBox[{
                "\"\<delta\>\"", "\[Rule]", "\"\<DeltaFunction[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Ca\>\"", "\[Rule]", "\"\<CA\>\""}], ",", 
                RowBox[{"\"\<Cf\>\"", "\[Rule]", "\"\<CF\>\""}]}], "}"}]}], 
          ",", 
          RowBox[{"Dot", "\[Rule]", "Times"}], ",", 
          RowBox[{"HoldForm", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[48]:=  (21:48:35 on 5/16/01)",
  CellTags->"A moment integral"],

Cell["\<\
Copy the FORM output from jqq12.log into the next cell and put \
FORM2CFeynCalc[\" \"]//ReleaseHold around it.\
\>", "Text",
  CellTags->"A moment integral"],

Cell["\<\
aqq=FORM2FeynCalc[\"

          + [(-1)^m]*e^-1*O1*Ca*Cf * (
          - 8
          + 4*x*Z2
          + 8*x
          + 8*Z2*[1+x]^-1
          - 4*Z2
          - 4*ln(x)
          - 4*ln(x)*x
          - 8*ln(x)*ln(1 + x)
          + 8*ln(x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 + x)*[1+x]^-1
          + 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          - 4*ln(x)*ln(x)*[1+x]^-1
          - 8*Li2( - x)
          + 8*Li2( - x)*x
          + 16*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*e^-1*O1*Cf^2 * (
          + 16
          - 8*x*Z2
          - 16*x
          - 16*Z2*[1+x]^-1
          + 8*Z2
          + 8*ln(x)
          + 8*ln(x)*x
          + 16*ln(x)*ln(1 + x)
          - 16*ln(x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 + x)*[1+x]^-1
          - 4*ln(x)*ln(x)
          + 4*ln(x)*ln(x)*x
          + 8*ln(x)*ln(x)*[1+x]^-1
          + 16*Li2( - x)
          - 16*Li2( - x)*x
          - 32*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*O1*Ca*Cf * (
          - 41/3
          - 6*x*Z2
          - 2*x*Z3
          + 41/3*x
          + 16/3*x^2*Z2
          + 6*Z2
          + 4*Z3*[1+x]^-1
          - 6*Z3
          - 8*ln(1 - x)
          + 8*ln(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)
          + 8*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(1 + x)*x*Z2
          + 8*ln(1 + x)*Z2*[1+x]^-1
          - 8*ln(1 + x)*Li2(1 - x)
          + 8*ln(1 + x)*Li2(1 - x)*x
          + 16*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          + 16*ln(1 + x)*Li2( - x)*x
          + 16*ln(1 + x)*Li2( - x)*[1+x]^-1
          - 37/3*ln(x)
          - 25/3*ln(x)*x
          - 4*ln(x)*ln(1 - x)
          - 4*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(1 - x)*ln(1 - x)
          + 4*ln(x)*ln(1 - x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          - 8*ln(x)*ln(1 - x)*ln(1 + x)
          + 8*ln(x)*ln(1 - x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          + 4/3*ln(x)*ln(1 + x)*x^-1
          + 4*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*x
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          - 6*ln(x)*ln(x)
          - 8/3*ln(x)*ln(x)*x^2
          + 2*ln(x)*ln(x)*ln(1 - x)
          - 2*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          - 6*ln(x)*ln(x)*ln(1 + x)
          + 2*ln(x)*ln(x)*ln(1 + x)*x
          + 8*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          + ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          - 2*ln(x)*ln(x)*ln(x)*[1+x]^-1
          + 4*ln(x)*Li2(1 - x)
          - 4*ln(x)*Li2(1 - x)*x
          - 8*ln(x)*Li2(1 - x)*[1+x]^-1
          - 4*ln(x)*Li2( - x)
          + 4*ln(x)*Li2( - x)*x
          + 8*ln(x)*Li2( - x)*[1+x]^-1
          - 4*Li2(1 - x)
          - 4*Li2(1 - x)*x
          + 4/3*Li2( - x)*x^-1
          + 4*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          + 8*Li3(1 - x)
          - 8*Li3(1 - x)*x
          - 16*Li3(1 - x)*[1+x]^-1
          - 4*Li3( - x)
          - 4*Li3( - x)*x
          + 4*S12(1 - x)
          - 4*S12(1 - x)*x
          - 8*S12(1 - x)*[1+x]^-1
          - 8*S12( - x)
          + 24*S12( - x)*x
          + 32*S12( - x)*[1+x]^-1
          + 4*S12(x^2)
          - 4*S12(x^2)*x
          - 8*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O1*Cf^2 * (
          + 82/3
          + 12*x*Z2
          + 4*x*Z3
          - 82/3*x
          - 32/3*x^2*Z2
          - 12*Z2
          - 8*Z3*[1+x]^-1
          + 12*Z3
          + 16*ln(1 - x)
          - 16*ln(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)
          - 16*ln(1 - x)*Li2(1 - x)*x
          - 32*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          - 16*ln(1 + x)*x*Z2
          - 16*ln(1 + x)*Z2*[1+x]^-1
          + 16*ln(1 + x)*Li2(1 - x)
          - 16*ln(1 + x)*Li2(1 - x)*x
          - 32*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          - 32*ln(1 + x)*Li2( - x)*x
          - 32*ln(1 + x)*Li2( - x)*[1+x]^-1
          + 74/3*ln(x)
          + 50/3*ln(x)*x
          + 8*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)
          - 8*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 16*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          + 16*ln(x)*ln(1 - x)*ln(1 + x)
          - 16*ln(x)*ln(1 - x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          - 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*x
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          + 12*ln(x)*ln(x)
          + 16/3*ln(x)*ln(x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          + 12*ln(x)*ln(x)*ln(1 + x)
          - 4*ln(x)*ln(x)*ln(1 + x)*x
          - 16*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          - 2*ln(x)*ln(x)*ln(x)
          + 2*ln(x)*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*ln(x)*[1+x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 8*ln(x)*Li2(1 - x)*x
          + 16*ln(x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*x
          - 16*ln(x)*Li2( - x)*[1+x]^-1
          + 8*Li2(1 - x)
          + 8*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          - 16*Li3(1 - x)
          + 16*Li3(1 - x)*x
          + 32*Li3(1 - x)*[1+x]^-1
          + 8*Li3( - x)
          + 8*Li3( - x)*x
          - 8*S12(1 - x)
          + 8*S12(1 - x)*x
          + 16*S12(1 - x)*[1+x]^-1
          + 16*S12( - x)
          - 48*S12( - x)*x
          - 64*S12( - x)*[1+x]^-1
          - 8*S12(x^2)
          + 8*S12(x^2)*x
          + 16*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O2*Ca*Cf * (
          + 8/3
          - 8/3*x
          - 16/3*x^2*Z2
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          + 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 8/3*ln(x)*ln(x)*x^2
          + 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          )

       + [(-1)^m]*O2*Cf^2 * (
          - 16/3
          + 16/3*x
          + 32/3*x^2*Z2
          + 16/3*ln(x)
          - 32/3*ln(x)*x
          - 16/3*ln(x)*ln(1 + x)*x^-1
          + 16*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          - 16/3*ln(x)*ln(x)*x^2
          - 16/3*Li2( - x)*x^-1
          + 16*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          )

       + e^-2*delta*O1*Ca*Cf * (
          - 22
          )

       + e^-2*delta*O1*Cf*Tf * (
          + 8
          )

       + e^-2*delta*O1*Cf^2 * (
          - 18
          + 32*Z2
          )

       + e^-2*O1*Ca*Cf * (
          + 44/3
          + 44/3*x
          - 88/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf*Tf * (
          - 16/3
          - 16/3*x
          + 32/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf^2 * (
          + 40
          + 8*x
          - 48*[(1-x)+]^-1
          + 32*ln(1 - x)
          + 32*ln(1 - x)*x
          - 64*ln(1 - x)*[(1-x)+]^-1
          - 24*ln(x)
          - 24*ln(x)*x
          + 32*ln(x)*[1-x]^-1
          )

       + e^-1*delta*O1*Ca*Cf * (
          + 325/6
          - 44/3*Z2
          - 12*Z3
          )

       + e^-1*delta*O1*Cf*Tf * (
          - 58/3
          + 16/3*Z2
          )

       + e^-1*delta*O1*Cf^2 * (
          + 87/2
          - 36*Z2
          - 8*Z3
          )

       + e^-1*O1*Ca*Cf * (
          - 158/9
          + 4*x*Z2
          + 22/9*x
          - 8*Z2*[(1-x)+]^-1
          + 4*Z2
          + 238/9*[(1-x)+]^-1
          + 44/3*ln(1 - x)
          + 44/3*ln(1 - x)*x
          - 88/3*ln(1 - x)*[(1-x)+]^-1
          + 34/3*ln(x)
          + 34/3*ln(x)*x
          - 44/3*ln(x)*[1-x]^-1
          - 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf*Tf * (
          + 88/9
          - 56/9*x
          - 56/9*[(1-x)+]^-1
          - 16/3*ln(1 - x)
          - 16/3*ln(1 - x)*x
          + 32/3*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(x)
          - 8/3*ln(x)*x
          + 16/3*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf^2 * (
          - 40
          - 4*x
          + 56*[(1-x)+]^-1
          - 4*ln(1 - x)
          + 44*ln(1 - x)*x
          - 24*ln(1 - x)*[(1-x)+]^-1
          + 24*ln(1 - x)*ln(1 - x)
          + 24*ln(1 - x)*ln(1 - x)*x
          - 48*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 36*ln(x)
          - 20*ln(x)*x
          - 36*ln(x)*[1-x]^-1
          - 16*ln(x)*ln(1 - x)*[1-x]^-1
          - 14*ln(x)*ln(x)
          - 14*ln(x)*ln(x)*x
          + 16*ln(x)*ln(x)*[1-x]^-1
          - 8*Li2(1 - x)
          - 8*Li2(1 - x)*x
          )

       + e^-1*O2*Ca*Cf * (
          + 20/3
          - 88/3*x
          )

       + e^-1*O2*Cf*Tf * (
          - 16/3
          + 32/3*x
          )

       + e^-1*O2*Cf^2 * (
          - 16
          - 8*x
          - 32*ln(1 - x)*x
          + 16*ln(x)*x
          )

       + delta*O1*Ca*Cf * (
          - 7081/72
          + 301/18*Z2
          + 49/5*Z2^2
          + 28*Z3
          )

       + delta*O1*Cf*Tf * (
          + 569/18
          - 46/9*Z2
          - 8*Z3
          )

       + delta*O1*Cf^2 * (
          - 541/8
          + 97/2*Z2
          - 74/5*Z2^2
          + 54*Z3
          )

       + O1*Ca*Cf * (
          + 941/27
          + 17/3*x*Z2
          - 8*x*Z3
          - 580/27*x
          - 16/3*x^2*Z2
          - 24*x^2*Z3
          - 28/3*Z2*[(1-x)+]^-1
          - 1/3*Z2
          + 12*Z3*[1-x]^-1
          + 22*Z3*[(1-x)+]^-1
          - 20*Z3
          - 670/27*[(1-x)+]^-1
          - 122/9*ln(1 - x)
          + 6*ln(1 - x)*x*Z2
          - 32/9*ln(1 - x)*x
          + 8*ln(1 - x)*x^2*Z2
          - 10*ln(1 - x)*Z2*[(1-x)+]^-1
          + 2*ln(1 - x)*Z2
          + 238/9*ln(1 - x)*[(1-x)+]^-1
          + 22/3*ln(1 - x)*ln(1 - x)
          + 22/3*ln(1 - x)*ln(1 - x)*x
          - 44/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 2*ln(1 - x)*Li2(1 - x)
          - 2*ln(1 - x)*Li2(1 - x)*x
          + 8*ln(1 - x)*Li2(1 - x)*x^2
          - 2*ln(1 - x)*Li2(1 - x)*[1-x]^-1
          + 14/9*ln(x)
          - 2*ln(x)*x*Z2
          + 176/9*ln(x)*x
          - 8*ln(x)*x^2*Z2
          + 10*ln(x)*Z2*[1-x]^-1
          - 6*ln(x)*Z2
          + 101/9*ln(x)*[1-x]^-1
          + 28/3*ln(x)*ln(1 - x)
          + 10/3*ln(x)*ln(1 - x)*x
          - 38/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 12*ln(x)*ln(1 + x)
          - 4/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 47/6*ln(x)*ln(x)
          + 47/6*ln(x)*ln(x)*x
          + 8/3*ln(x)*ln(x)*x^2
          - 11/3*ln(x)*ln(x)*[1-x]^-1
          + ln(x)*ln(x)*ln(1 - x)
          - ln(x)*ln(x)*ln(1 - x)*x
          + 4*ln(x)*ln(x)*ln(1 - x)*x^2
          - ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          + 2*ln(x)*ln(x)*ln(x)*[1-x]^-1
          + 8*ln(x)*Li2(1 - x)
          + 16*ln(x)*Li2(1 - x)*x^2
          - 12*ln(x)*Li2(1 - x)*[1-x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*[1-x]^-1
          - 12*Li2(1 - x)*x
          + 4*Li2(1 - x)*[1-x]^-1
          - 12*Li2( - x)
          - 4/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          - 6*Li3(1 - x)
          + 6*Li3(1 - x)*x
          - 24*Li3(1 - x)*x^2
          + 6*Li3(1 - x)*[1-x]^-1
          - 16*Li3( - x)
          + 16*Li3( - x)*[1-x]^-1
          + 14*S12(1 - x)
          - 6*S12(1 - x)*x
          + 24*S12(1 - x)*x^2
          - 14*S12(1 - x)*[1-x]^-1
          )

       + O1*Cf*Tf * (
          - 88/27
          - 4/3*x*Z2
          + 20/27*x
          + 8/3*Z2*[(1-x)+]^-1
          - 4/3*Z2
          + 128/27*[(1-x)+]^-1
          + 88/9*ln(1 - x)
          - 56/9*ln(1 - x)*x
          - 56/9*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(1 - x)*ln(1 - x)
          - 8/3*ln(1 - x)*ln(1 - x)*x
          + 16/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 44/9*ln(x)
          - 28/9*ln(x)*x
          - 28/9*ln(x)*[1-x]^-1
          - 8/3*ln(x)*ln(1 - x)
          - 8/3*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 2/3*ln(x)*ln(x)
          - 2/3*ln(x)*ln(x)*x
          + 4/3*ln(x)*ln(x)*[1-x]^-1
          )

       + O1*Cf^2 * (
          + 188/3
          - 4*x*Z2
          - 24*x*Z3
          + 4/3*x
          + 32/3*x^2*Z2
          + 48*x^2*Z3
          - 4*Z2*[(1-x)+]^-1
          + 20*Z2
          - 24*Z3*[1-x]^-1
          + 24*Z3
          - 56*[(1-x)+]^-1
          - 38*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          + 38*ln(1 - x)*x
          - 16*ln(1 - x)*x^2*Z2
          - 8*ln(1 - x)*Z2*[(1-x)+]^-1
          + 8*ln(1 - x)*Z2
          + 28*ln(1 - x)*[(1-x)+]^-1
          - 13*ln(1 - x)*ln(1 - x)
          + 31*ln(1 - x)*ln(1 - x)*x
          - 6*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*x
          - 56/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          - 12*ln(1 - x)*Li2(1 - x)
          + 4*ln(1 - x)*Li2(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x^2
          - 68/3*ln(x)
          - 14*ln(x)*x*Z2
          - 68/3*ln(x)*x
          + 16*ln(x)*x^2*Z2
          + 2*ln(x)*Z2
          + 44*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(1 - x)
          + 34*ln(x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*[1-x]^-1
          + 6*ln(x)*ln(1 - x)*ln(1 - x)
          + 6*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*ln(1 - x)*[1-x]^-1
          + 24*ln(x)*ln(1 + x)
          + 8/3*ln(x)*ln(1 + x)*x^-1
          + 32*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          + 11*ln(x)*ln(x)
          - 23*ln(x)*ln(x)*x
          - 16/3*ln(x)*ln(x)*x^2
          - 15*ln(x)*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(x)*ln(1 - x)
          + 2*ln(x)*ln(x)*ln(1 - x)*x
          - 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - 5*ln(x)*ln(x)*ln(x)
          - 5*ln(x)*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*ln(x)*[1-x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 24*ln(x)*Li2(1 - x)*x
          - 32*ln(x)*Li2(1 - x)*x^2
          - 16*ln(x)*Li2(1 - x)*[1-x]^-1
          - 16*ln(x)*Li2( - x)
          + 16*ln(x)*Li2( - x)*[1-x]^-1
          - 24*Li2(1 - x)
          + 48*Li2(1 - x)*x
          - 4*Li2(1 - x)*[1-x]^-1
          + 24*Li2( - x)
          + 8/3*Li2( - x)*x^-1
          + 32*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          + 20*Li3(1 - x)
          - 28*Li3(1 - x)*x
          + 48*Li3(1 - x)*x^2
          + 32*Li3( - x)
          - 32*Li3( - x)*[1-x]^-1
          - 8*S12(1 - x)
          + 56*S12(1 - x)*x
          - 48*S12(1 - x)*x^2
          - 32*S12(1 - x)*[1-x]^-1
          )

       + O2*Ca*Cf * (
          - 26/9
          - 8*x*Z2
          - 24*x*Z3
          + 232/9*x
          + 16/3*x^2*Z2
          + 24*x^2*Z3
          + 4*Z2
          - 16/3*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          - 64/3*ln(1 - x)*x
          - 8*ln(1 - x)*x^2*Z2
          + 8*ln(1 - x)*Li2(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)*x^2
          + 6*ln(x)
          - 8*ln(x)*x*Z2
          - 28*ln(x)*x
          + 8*ln(x)*x^2*Z2
          - 4*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          - 8/3*ln(x)*ln(1 + x)*x^-1
          + 8*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          - 8/3*ln(x)*ln(x)*x^2
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*x^2
          + 16*ln(x)*Li2(1 - x)*x
          - 16*ln(x)*Li2(1 - x)*x^2
          - 8*Li2(1 - x)
          + 16*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          + 8*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          - 24*Li3(1 - x)*x
          + 24*Li3(1 - x)*x^2
          + 24*S12(1 - x)*x
          - 24*S12(1 - x)*x^2
          )

       + O2*Cf*Tf * (
          - 8/9
          - 32/9*x
          - 16/3*ln(1 - x)
          + 32/3*ln(1 - x)*x
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          )

       + O2*Cf^2 * (
          - 32/3
          + 16*x*Z2
          + 48*x*Z3
          - 16/3*x
          - 32/3*x^2*Z2
          - 48*x^2*Z3
          - 8*Z2
          - 16*ln(1 - x)*x*Z2
          - 44*ln(1 - x)*x
          + 16*ln(1 - x)*x^2*Z2
          - 24*ln(1 - x)*ln(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*x^2
          - 40/3*ln(x)
          + 16*ln(x)*x*Z2
          + 68/3*ln(x)*x
          - 16*ln(x)*x^2*Z2
          + 8*ln(x)*ln(1 - x)
          - 32*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          + 12*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*x^2
          - 8*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 32*ln(x)*Li2(1 - x)*x
          + 32*ln(x)*Li2(1 - x)*x^2
          + 16*Li2(1 - x)
          - 48*Li2(1 - x)*x
          + 16/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          + 48*Li3(1 - x)*x
          - 48*Li3(1 - x)*x^2
          - 48*S12(1 - x)*x
          + 48*S12(1 - x)*x^2
          )
          
          \"]//ReleaseHold;\
\>", "Input",
  CellLabel->"In[49]:=  (21:48:36 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", "aqq", "]"}]], "Input",
  CellLabel->"In[50]:=  (21:48:37 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          FractionBox[
            RowBox[{"O1", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"32", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "18"}], ")"}], 
              " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}], " ", 
              SubsuperscriptBox["C", "F", "2"]}], 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"]], "+", 
          RowBox[{"\[LeftSkeleton]", "29", "\[RightSkeleton]"}], "+", 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            SubscriptBox["C", "A"], " ", "O1", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", "x"}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "3"], "(", "x", ")"}]}], "-", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"\[LeftSkeleton]", "3", "\[RightSkeleton]"}], 
                        "3"], "(", "x", ")"}]}], 
                  RowBox[{"x", "+", "1"}]], "+", 
                RowBox[{"\[LeftSkeleton]", "101", "\[RightSkeleton]"}]}], 
              ")"}], " ", 
            SubscriptBox["C", "F"]}]}],
        Short], TraditionalForm]], "Output",
  CellLabel->"Out[50]//Short=  (21:48:38 on 5/16/01)"]
}, Closed]],

Cell["\<\
Comment: By default all bracketed FORM expressions are translated to a Hold[] \
expression.\
\>", "Commentary",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell["FORM2FeynCalc[\"-[(-1)^m]\"]", "Input",
  CellLabel->"In[51]:=  (21:48:38 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"Hold", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[51]=  (21:48:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "ReleaseHold"}]], "Input",
  CellLabel->"In[52]:=  (21:48:38 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output",
  CellLabel->"Out[52]=  (21:48:38 on 5/16/01)"]
}, Closed]],

Cell["Calculate moments", "Text",
  CellTags->"A moment integral"],

Cell["\<\
The first moment should be zero, which is verified by Integrate2 in a \
reasonable amount of time. The variable O2 can be set to zero since it is the \
coefficient of unphysical matrix elements.\
\>", "Text",
  CellTags->"A moment integral"],

Cell[TextData[{
  "This calculates for m=1 the integral ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubsuperscriptBox["\[Integral]", "0", 
              RowBox[{"1", " "}]], " ", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox[
                  OverscriptBox[
                    StyleBox["A",
                      FontSlant->"Plain"], "^"],
                  FontSlant->"Plain"], "qq", 
                RowBox[{"NS", ",", "PHYS"}]], " ", 
              RowBox[{"\[DifferentialD]", "x"}]}]}], " "}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{"Integrate2", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"aqq", "/.", 
              RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
            RowBox[{"O2", "\[Rule]", "0"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}]], "Input",\

  CellLabel->"In[53]:=  (21:48:38 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"9.343999999999994`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[53]=  (21:48:48 on 5/16/01)"]
}, Closed]],

Cell["\<\
If you know that no partial fraction decomposition has to be done then \
Integrate3 will be much faster on an expanded or collected integral:\
\>", "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{"Expand", "[", 
        RowBox[{"Integrate3", "[", 
          RowBox[{
            RowBox[{"Expand", "[", 
              RowBox[{
                RowBox[{"aqq", "/.", 
                  RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
                RowBox[{"O2", "\[Rule]", "0"}]}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}], 
      "]"}]], "Input",
  CellLabel->"In[54]:=  (21:48:48 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"1.642000000000003`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[54]=  (21:48:50 on 5/16/01)"]
}, Closed]],

Cell["Define an auxiliary typesetting function.", "Text",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MakeBoxes", "[", 
          RowBox[{
            RowBox[{"big", "[", "a_", "]"}], ",", " ", "f_"}], "]"}], " ", ":=",
         " ", 
        RowBox[{"StyleBox", "[", 
          RowBox[{
            RowBox[{"MakeBoxes", "[", 
              RowBox[{"a", ",", " ", "f"}], "]"}], ",", " ", 
            RowBox[{"FontColor", " ", "\[Rule]", " ", 
              RowBox[{"RGBColor", "[", 
                RowBox[{"0", ",", " ", "0", ",", " ", "1"}], "]"}]}], ",", 
            "   ", 
            RowBox[{"FontSize", " ", "\[Rule]", " ", "18"}]}], "]"}]}], 
      ";"}]], "Input",
  CellLabel->"In[55]:=  (21:48:50 on 5/16/01)",
  CellTags->"A moment integral"],

Cell["The second moment takes a little bit longer.", "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Timing", "[", 
        RowBox[{"Collect2", "[", 
          RowBox[{
            RowBox[{"Integrate2", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{"x", " ", "aqq"}], "/.", 
                      RowBox[{"m", "\[Rule]", "2"}]}], "/.", 
                    RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
                  RowBox[{"O1", "\[Rule]", "1"}]}], ",", 
                RowBox[{"{", 
                  RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"CA", ",", "CF", ",", "Tf"}], "}"}], ",", 
            RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}], "]"}], "/.", 
      " ", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"CA", " ", "CF"}], " ", "\[RuleDelayed]", " ", 
            RowBox[{"big", "[", " ", 
              RowBox[{"CA", " ", "CF"}], "]"}]}], ",", 
          RowBox[{
            RowBox[{"CF", "^", "2"}], "\[RuleDelayed]", 
            RowBox[{"big", "[", 
              RowBox[{"CF", "^", "2"}], "]"}]}], ",", 
          RowBox[{
            RowBox[{"a_", " ", "CF", " ", "Tf"}], " ", "\[RuleDelayed]", 
            RowBox[{"HoldForm", "[", " ", 
              RowBox[{
                RowBox[{"big", "[", 
                  RowBox[{"CF", " ", "Tf"}], "]"}], " ", "a"}], "]"}]}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[56]:=  (21:48:50 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"11.186`", " ", "Second"}], ",", 
          RowBox[{
            TagBox[
              RowBox[{
                RowBox[{"(", 
                  StyleBox[
                    RowBox[{
                      SubscriptBox["C", "F"], " ", 
                      SubscriptBox["T", "f"]}],
                    FontSize->18,
                    FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"16", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "+", 
                    FractionBox["376", 
                      RowBox[{"27", " ", 
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm]}]], "-", 
                    FractionBox["64", 
                      RowBox[{"9", " ", 
                        SuperscriptBox[
                          TagBox["\[CurlyEpsilon]",
                            TraditionalForm], "2"]}]], "-", 
                    FractionBox["2086", "81"]}], ")"}]}],
              HoldForm], "+", 
            RowBox[{
              RowBox[{"(", 
                StyleBox[
                  RowBox[{
                    SubscriptBox["C", "A"], " ", 
                    SubscriptBox["C", "F"]}],
                  FontSize->18,
                  FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"44", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"]}], "-", 
                  FractionBox["1166", 
                    RowBox[{"27", " ", 
                      TagBox["\[CurlyEpsilon]",
                        TraditionalForm]}]], "+", 
                  FractionBox["176", 
                    RowBox[{"9", " ", 
                      SuperscriptBox[
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm], "2"]}]], "-", 
                  RowBox[{"12", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], "+", 
                  FractionBox["12133", "162"]}], ")"}]}], "+", 
            RowBox[{
              StyleBox[
                SubsuperscriptBox["C", "F", "2"],
                FontSize->18,
                FontColor->RGBColor[0, 0, 1]], " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{"32", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "+", 
                  FractionBox["752", 
                    RowBox[{"27", " ", 
                      TagBox["\[CurlyEpsilon]",
                        TraditionalForm]}]], "-", 
                  FractionBox["128", 
                    RowBox[{"9", " ", 
                      SuperscriptBox[
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm], "2"]}]], "+", 
                  RowBox[{"8", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], "-", 
                  FractionBox["3452", "81"]}], ")"}]}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[56]=  (21:49:02 on 5/16/01)"]
}, Closed]],

Cell["\<\
Try to do the general Mellin transform (replace m by OPEm, which has special \
properties).\
\>", "Text",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{
      RowBox[{"gqq", " ", "=", " ", 
        RowBox[{"Collect2", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"OPEm", "-", "1"}]], 
                  RowBox[{"(", 
                    RowBox[{"aqq", "/.", 
                      RowBox[{"m", "\[Rule]", "OPEm"}]}], ")"}]}], "/.", 
                RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
              RowBox[{"O1", "\[Rule]", "1"}]}], ",", "x"}], "]"}]}], 
      ";"}]], "Input",
  CellLabel->"In[57]:=  (21:49:02 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gqq", "//", "Length"}]], "Input",
  CellLabel->"In[58]:=  (21:49:06 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox["102", TraditionalForm]], "Output",
  CellLabel->"Out[58]=  (21:49:06 on 5/16/01)"]
}, Closed]],

Cell["\<\
There are 102 integrals. However, some of the Mellin transforms cannot be \
done at the moment.\
\>", "Text",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ww", "=", 
        RowBox[{"Integrate3", "[", 
          RowBox[{"gqq", ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
      ";"}]], "Input",
  CellLabel->"In[59]:=  (21:49:06 on 5/16/01)",
  CellTags->"A moment integral"],

Cell["\<\
If an integral cannot be performed the head Hold[Integrate3] is substituted \
for Integrate3. Thus, in order to find the list of integrals not done you can \
do:\
\>", "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{"ww", ",", 
        RowBox[{"Hold", "[", "Integrate3", "]"}]}], "]"}]], "Input",
  CellLabel->"In[60]:=  (21:49:07 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", 
                  RowBox[{"m", "-", "1"}]], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{
                      FormBox["1",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["2",
                        "TraditionalForm"]}]], "(", 
                  FormBox[
                    SuperscriptBox["x", "2"],
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", "m"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{
                      FormBox["1",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["2",
                        "TraditionalForm"]}]], "(", 
                  FormBox[
                    SuperscriptBox["x", "2"],
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{
                    SubscriptBox["S", 
                      RowBox[{
                        FormBox["1",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["2",
                          "TraditionalForm"]}]], "(", 
                    FormBox[
                      SuperscriptBox["x", "2"],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", 
                  RowBox[{"m", "-", "1"}]], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", "m"], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{
            RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"x", "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[60]=  (21:49:08 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Change the appearance and retain only those integrals with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["x", 
            RowBox[{"m", "-", "1"}]], ":"}], TraditionalForm]]]
}], "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Select2", "[", 
        RowBox[{"%", ",", 
          SuperscriptBox["x", 
            RowBox[{"OPEm", "-", "1"}]]}], "]"}], "/.", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"Hold", "[", "Integrate3", "]"}], "[", "z__", "]"}], 
        "\[RuleDelayed]", " ", 
        RowBox[{"HoldForm", "[", 
          RowBox[{"Integrate", "[", "z", "]"}], "]"}]}]}]], "Input",
  CellLabel->"In[61]:=  (21:49:08 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{
                    SubscriptBox["S", 
                      RowBox[{
                        FormBox["1",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["2",
                          "TraditionalForm"]}]], "(", 
                    FormBox[
                      SuperscriptBox["x", "2"],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{
                      SubscriptBox["S", 
                        RowBox[{
                          FormBox["1",
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox["2",
                            "TraditionalForm"]}]], "(", 
                      FormBox[
                        SuperscriptBox["x", "2"],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{"log", "(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", 
                    RowBox[{
                      SubscriptBox["Li", "2"], "(", 
                      RowBox[{"1", "-", "x"}], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"x", "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{"log", "(", 
                      RowBox[{"x", "+", "1"}], ")"}], " ", 
                    RowBox[{
                      SubscriptBox["Li", "2"], "(", 
                      RowBox[{"1", "-", "x"}], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[61]=  (21:49:08 on 5/16/01)"]
}, Closed]],

Cell["\<\
If you do know the answers for these integrals please share the knowledge.\
\>", "Text",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"e", ",", "ww", ",", "gqq", ",", "big", ",", "aqq"}], 
      "]"}]], "Input",
  CellLabel->"In[62]:=  (21:49:08 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"FORM2FeynCalc", ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[63]:=  (21:49:08 on 5/16/01)",
  CellTags->"A moment integral"],

Cell[TextData[{
  "See also: ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Cases2",
    ButtonData:>"Cases2",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Epsilon",
    ButtonData:>"Epsilon",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate3",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Integrate3",
    ButtonData:>"Integrate3",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Li2",
    ButtonData:>"Li2",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Li3",
    ButtonData:>"Li3",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Nielsen",
    ButtonData:>"Nielsen",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Zeta2",
    ButtonData:>"Zeta2",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  CellTags->"A moment integral"]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"AllBrackets",
ShowPageBreaks->False,
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
CellGrouping->Manual,
WindowSize->{774, 671},
WindowMargins->{{Automatic, 96}, {Automatic, 4}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{597.563, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintSelectionHighlighting"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->True,
"FacingPages"->True,
"FirstPageFace"->Left,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{"MakeContentsNonDefaultOptions" -> {AutomaticCellTagging -> \
FromContent, PageNumbers -> False, SaveOptions -> True, SelectCellStyles -> \
{"Section", "Subsection"}, TOCCellStyles -> {"Contents", "Index"}}},
CellLabelAutoDelete->True,
ShowCellTags->True,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Short Overview"->{
    Cell[2368, 78, 96, 2, 70, "Section",
      PageBreakAbove->True,
      CellTags->{"Short Overview", "intro"}]},
  "intro"->{
    Cell[2368, 78, 96, 2, 70, "Section",
      PageBreakAbove->True,
      CellTags->{"Short Overview", "intro"}],
    Cell[2467, 82, 903, 20, 113, "Text",
      CellTags->"intro"],
    Cell[3373, 104, 580, 15, 39, "Text",
      CellTags->"intro"],
    Cell[5353, 185, 84, 1, 39, "Text",
      CellTags->"intro"],
    Cell[5440, 188, 1708, 71, 56, "Text",
      CellTags->"intro"],
    Cell[7151, 261, 111, 4, 39, "Text",
      CellTags->"intro"],
    Cell[7265, 267, 342, 14, 39, "Text",
      CellTags->"intro"],
    Cell[7610, 283, 1152, 27, 107, "Text",
      CellTags->"intro"],
    Cell[8765, 312, 273, 7, 56, "Text",
      CellTags->"intro"],
    Cell[9041, 321, 580, 10, 90, "Text",
      CellTags->"intro"],
    Cell[9624, 333, 570, 19, 39, "Text",
      CellTags->"intro"],
    Cell[10197, 354, 94, 1, 39, "Text",
      CellTags->"intro"],
    Cell[10294, 357, 239, 8, 39, "Text",
      CellTags->"intro"],
    Cell[10536, 367, 457, 12, 39, "SmallText",
      CellTags->{"intro", "introfoot"}]},
  "introfoot"->{
    Cell[10536, 367, 457, 12, 39, "SmallText",
      CellTags->{"intro", "introfoot"}]},
  "Load FeynCalc"->{
    Cell[11030, 384, 76, 1, 56, "Section",
      CellTags->{"Load FeynCalc", "start"}]},
  "start"->{
    Cell[11030, 384, 76, 1, 56, "Section",
      CellTags->{"Load FeynCalc", "start"}],
    Cell[11109, 387, 357, 10, 56, "Text",
      CellTags->"start"],
    Cell[11469, 399, 64, 1, 45, "MathCaption",
      CellTags->"start"],
    Cell[11536, 402, 142, 3, 56, "Input",
      CellTags->"start"],
    Cell[11681, 407, 241, 7, 39, "Text",
      CellTags->"start"],
    Cell[11925, 416, 503, 14, 58, "Text",
      CellTags->"start"],
    Cell[12431, 432, 385, 10, 56, "Input",
      CellTags->"start"],
    Cell[12819, 444, 246, 9, 45, "MathCaption",
      CellTags->"start"],
    Cell[13068, 455, 376, 9, 56, "Input",
      CellTags->"start"],
    Cell[13447, 466, 376, 9, 56, "Input",
      CellTags->"start"],
    Cell[13826, 477, 224, 9, 45, "MathCaption",
      CellTags->"start"],
    Cell[14075, 490, 150, 4, 56, "Input",
      CellTags->"start"],
    Cell[14562, 509, 142, 4, 45, "MathCaption",
      CellTags->"start"],
    Cell[14729, 517, 132, 3, 56, "Input",
      CellTags->"start"]},
  "About Extras"->{
    Cell[15056, 531, 88, 2, 56, "Section",
      PageBreakAbove->True,
      CellTags->"About Extras"],
    Cell[15147, 535, 566, 14, 73, "Text",
      CellTags->"About Extras"],
    Cell[15716, 551, 152, 4, 45, "MathCaption",
      CellTags->"About Extras"],
    Cell[15871, 557, 115, 3, 43, "Input",
      CellTags->"About Extras"],
    Cell[15989, 562, 121, 4, 39, "Text",
      CellTags->"About Extras"],
    Cell[16113, 568, 1310, 31, 97, "Text",
      CellTags->"About Extras"],
    Cell[17426, 601, 105, 1, 31, "Caption",
      CellTags->"About Extras"]},
  "tarcer"->{
    Cell[17556, 606, 195, 7, 74, "Subsection",
      CellTags->"tarcer"],
    Cell[17754, 615, 69, 1, 46, "InspectorLevel2",
      CellTags->"tarcer"],
    Cell[17826, 618, 240, 8, 39, "Text",
      CellTags->"tarcer"],
    Cell[18069, 628, 237, 10, 39, "Text",
      CellTags->"tarcer"],
    Cell[18309, 640, 61, 1, 49, "Subsubsection",
      CellTags->"tarcer"],
    Cell[18373, 643, 629, 15, 73, "Text",
      CellTags->"tarcer"],
    Cell[19005, 660, 330, 8, 56, "Text",
      CellTags->"tarcer"],
    Cell[19338, 670, 109, 3, 43, "Input",
      CellTags->"tarcer"],
    Cell[19450, 675, 258, 8, 39, "Text",
      CellTags->"tarcer"],
    Cell[19711, 685, 303, 9, 39, "Text",
      CellTags->"tarcer"]},
  "phi"->{
    Cell[20051, 699, 154, 6, 74, "Subsection",
      CellTags->"phi"],
    Cell[20208, 707, 60, 1, 46, "InspectorLevel2",
      CellTags->"phi"],
    Cell[20271, 710, 422, 9, 73, "Text",
      CellTags->"phi"],
    Cell[20696, 721, 70, 1, 39, "Text",
      CellTags->"phi"],
    Cell[20769, 724, 103, 3, 43, "Input",
      CellTags->"phi"],
    Cell[20875, 729, 255, 8, 39, "Text",
      CellTags->"phi"],
    Cell[21133, 739, 283, 9, 39, "Text",
      CellTags->"phi"]},
  "fa"->{
    Cell[21453, 753, 63, 1, 53, "Subsection",
      CellTags->{"fa", "extrafoor"}],
    Cell[21519, 756, 112, 1, 46, "InspectorLevel2",
      CellTags->{"fa", "extrafoor"}],
    Cell[21634, 759, 251, 8, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[21888, 769, 327, 11, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22218, 782, 465, 8, 73, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22686, 792, 308, 6, 56, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22997, 800, 387, 10, 56, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[23387, 812, 209, 5, 63, "Input",
      CellTags->{"fa", "extrafoor"}],
    Cell[23599, 819, 269, 8, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[23871, 829, 565, 16, 56, "Text",
      CellTags->{"fa", "extrafoor"}]},
  "extrafoor"->{
    Cell[21453, 753, 63, 1, 53, "Subsection",
      CellTags->{"fa", "extrafoor"}],
    Cell[21519, 756, 112, 1, 46, "InspectorLevel2",
      CellTags->{"fa", "extrafoor"}],
    Cell[21634, 759, 251, 8, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[21888, 769, 327, 11, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22218, 782, 465, 8, 73, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22686, 792, 308, 6, 56, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[22997, 800, 387, 10, 56, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[23387, 812, 209, 5, 63, "Input",
      CellTags->{"fa", "extrafoor"}],
    Cell[23599, 819, 269, 8, 39, "Text",
      CellTags->{"fa", "extrafoor"}],
    Cell[23871, 829, 565, 16, 56, "Text",
      CellTags->{"fa", "extrafoor"}]},
  "Example Calculations"->{
    Cell[24485, 851, 107, 2, 56, "Section",
      PageBreakAbove->True,
      CellTags->{"Example Calculations", "ex"}]},
  "ex"->{
    Cell[24485, 851, 107, 2, 56, "Section",
      PageBreakAbove->True,
      CellTags->{"Example Calculations", "ex"}],
    Cell[24595, 855, 540, 15, 47, "Commentary",
      CellTags->"ex"],
    Cell[25138, 872, 345, 10, 39, "Text",
      CellTags->"ex"]},
  "Tree level"->{
    Cell[25508, 886, 342, 11, 53, "Subsection",
      CellTags->"Tree level"],
    Cell[25853, 899, 324, 10, 39, "Text",
      CellTags->"Tree level"],
    Cell[26180, 911, 85, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[26268, 914, 168, 4, 58, "Input",
      CellTags->"Tree level"],
    Cell[26439, 920, 168, 4, 58, "Input",
      CellTags->"Tree level"],
    Cell[26610, 926, 168, 4, 58, "Input",
      CellTags->"Tree level"],
    Cell[26781, 932, 168, 4, 58, "Input",
      CellTags->"Tree level"],
    Cell[26952, 938, 99, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[27076, 943, 337, 9, 56, "Input",
      CellTags->"Tree level"],
    Cell[28842, 997, 2399, 77, 56, "Text",
      CellTags->"Tree level"],
    Cell[31244, 1076, 371, 10, 56, "Input",
      CellTags->"Tree level"],
    Cell[31618, 1088, 339, 9, 56, "Input",
      CellTags->"Tree level"],
    Cell[31960, 1099, 371, 10, 56, "Input",
      CellTags->"Tree level"],
    Cell[32334, 1111, 306, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[32643, 1121, 306, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[32952, 1131, 464, 12, 56, "Input",
      CellTags->"Tree level"],
    Cell[33419, 1145, 64, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[33508, 1150, 336, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[35042, 1198, 336, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[36576, 1246, 336, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[38110, 1294, 336, 8, 56, "Input",
      CellTags->"Tree level"],
    Cell[39622, 1340, 74, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[39721, 1345, 134, 2, 58, "Input",
      CellTags->"Tree level"],
    Cell[41644, 1402, 134, 2, 58, "Input",
      CellTags->"Tree level"],
    Cell[43164, 1448, 134, 2, 58, "Input",
      CellTags->"Tree level"],
    Cell[44684, 1494, 134, 2, 58, "Input",
      CellTags->"Tree level"],
    Cell[46585, 1549, 91, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[46701, 1554, 883, 24, 92, "Input",
      CellTags->"Tree level"],
    Cell[54395, 1767, 78, 1, 39, "Text",
      CellTags->"Tree level"],
    Cell[54498, 1772, 201, 5, 56, "Input",
      CellTags->"Tree level"],
    Cell[61773, 1964, 118, 3, 39, "Text",
      CellTags->"Tree level"],
    Cell[61916, 1971, 423, 11, 56, "Input",
      CellTags->"Tree level"],
    Cell[62464, 1989, 135, 4, 39, "Text",
      CellTags->"Tree level"],
    Cell[62602, 1995, 260, 6, 56, "Input",
      CellTags->"Tree level"],
    Cell[62865, 2003, 649, 27, 39, "Text",
      CellTags->"Tree level"]},
  "A moment integral"->{
    Cell[63551, 2035, 72, 1, 53, "Subsection",
      CellTags->"A moment integral"],
    Cell[63626, 2038, 504, 17, 45, "Text",
      CellTags->"A moment integral"],
    Cell[64133, 2057, 127, 3, 39, "Text",
      CellTags->"A moment integral"],
    Cell[64263, 2062, 409, 13, 45, "Text",
      CellTags->"A moment integral"],
    Cell[64675, 2077, 94, 1, 39, "Text",
      CellTags->"A moment integral"],
    Cell[64772, 2080, 288, 10, 39, "Text",
      CellTags->"A moment integral"],
    Cell[65085, 2094, 149, 3, 56, "Input",
      CellTags->"A moment integral"],
    Cell[65413, 2106, 222, 5, 52, "Text",
      CellTags->"A moment integral"],
    Cell[65638, 2113, 2187, 46, 176, "Input",
      CellTags->"A moment integral"],
    Cell[67828, 2161, 167, 4, 39, "Text",
      CellTags->"A moment integral"],
    Cell[67998, 2167, 17184, 630, 12578, "Input",
      CellTags->"A moment integral"],
    Cell[85207, 2801, 144, 3, 56, "Input",
      CellTags->"A moment integral"],
    Cell[87022, 2851, 154, 4, 47, "Commentary",
      CellTags->"A moment integral"],
    Cell[87201, 2859, 126, 2, 58, "Input",
      CellTags->"A moment integral"],
    Cell[87635, 2876, 144, 3, 56, "Input",
      CellTags->"A moment integral"],
    Cell[88015, 2890, 66, 1, 39, "Text",
      CellTags->"A moment integral"],
    Cell[88084, 2893, 251, 5, 56, "Text",
      CellTags->"A moment integral"],
    Cell[88338, 2900, 644, 19, 53, "Text",
      CellTags->"A moment integral"],
    Cell[89007, 2923, 438, 12, 56, "Input",
      CellTags->"A moment integral"],
    Cell[89686, 2946, 198, 4, 39, "Text",
      CellTags->"A moment integral"],
    Cell[89909, 2954, 547, 14, 56, "Input",
      CellTags->"A moment integral"],
    Cell[90697, 2979, 90, 1, 39, "Text",
      CellTags->"A moment integral"],
    Cell[90790, 2982, 713, 18, 56, "Input",
      CellTags->"A moment integral"],
    Cell[91506, 3002, 93, 1, 39, "Text",
      CellTags->"A moment integral"],
    Cell[91624, 3007, 1538, 38, 96, "Input",
      CellTags->"A moment integral"],
    Cell[96804, 3138, 148, 4, 39, "Text",
      CellTags->"A moment integral"],
    Cell[96955, 3144, 653, 17, 57, "Input",
      CellTags->"A moment integral"],
    Cell[97633, 3165, 141, 3, 56, "Input",
      CellTags->"A moment integral"],
    Cell[97901, 3175, 152, 4, 39, "Text",
      CellTags->"A moment integral"],
    Cell[98056, 3181, 317, 9, 56, "Input",
      CellTags->"A moment integral"],
    Cell[98376, 3192, 218, 5, 39, "Text",
      CellTags->"A moment integral"],
    Cell[98619, 3201, 217, 5, 56, "Input",
      CellTags->"A moment integral"],
    Cell[103474, 3326, 268, 8, 39, "Text",
      CellTags->"A moment integral"],
    Cell[103767, 3338, 497, 13, 57, "Input",
      CellTags->"A moment integral"],
    Cell[107832, 3446, 131, 3, 39, "Text",
      CellTags->"A moment integral"],
    Cell[107966, 3451, 213, 5, 56, "Input",
      CellTags->"A moment integral"],
    Cell[108182, 3458, 281, 7, 56, "Input",
      CellTags->"A moment integral"],
    Cell[108466, 3467, 1240, 55, 39, "Text",
      CellTags->"A moment integral"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Short Overview", 111766, 3580},
  {"intro", 111895, 3584},
  {"introfoot", 112918, 3614},
  {"Load FeynCalc", 113028, 3617},
  {"start", 113130, 3620},
  {"About Extras", 114146, 3649},
  {"tarcer", 114723, 3665},
  {"phi", 115442, 3686},
  {"fa", 115924, 3701},
  {"extrafoor", 116752, 3722},
  {"Example Calculations", 117591, 3743},
  {"ex", 117723, 3747},
  {"Tree level", 117999, 3755},
  {"A moment integral", 120603, 3826}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 54, 1, 105, "Title"],
Cell[1784, 55, 88, 1, 64, "Subtitle"],
Cell[1875, 58, 123, 3, 66, "Subtitle",
  PageBreakBelow->True],
Cell[2001, 63, 81, 1, 53, "Subsubtitle"],
Cell[2085, 66, 258, 8, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2368, 78, 96, 2, 70, "Section",
  PageBreakAbove->True,
  CellTags->{"Short Overview", "intro"}],
Cell[2467, 82, 903, 20, 113, "Text",
  CellTags->"intro"],
Cell[3373, 104, 580, 15, 39, "Text",
  CellTags->"intro"],
Cell[3956, 121, 1394, 62, 42, "Text"],
Cell[5353, 185, 84, 1, 39, "Text",
  CellTags->"intro"],
Cell[5440, 188, 1708, 71, 56, "Text",
  CellTags->"intro"],
Cell[7151, 261, 111, 4, 39, "Text",
  CellTags->"intro"],
Cell[7265, 267, 342, 14, 39, "Text",
  CellTags->"intro"],
Cell[7610, 283, 1152, 27, 107, "Text",
  CellTags->"intro"],
Cell[8765, 312, 273, 7, 56, "Text",
  CellTags->"intro"],
Cell[9041, 321, 580, 10, 90, "Text",
  CellTags->"intro"],
Cell[9624, 333, 570, 19, 39, "Text",
  CellTags->"intro"],
Cell[10197, 354, 94, 1, 39, "Text",
  CellTags->"intro"],
Cell[10294, 357, 239, 8, 39, "Text",
  CellTags->"intro"],
Cell[10536, 367, 457, 12, 39, "SmallText",
  CellTags->{"intro", "introfoot"}]
}, Closed]],

Cell[CellGroupData[{
Cell[11030, 384, 76, 1, 56, "Section",
  CellTags->{"Load FeynCalc", "start"}],
Cell[11109, 387, 357, 10, 56, "Text",
  CellTags->"start"],
Cell[11469, 399, 64, 1, 45, "MathCaption",
  CellTags->"start"],
Cell[11536, 402, 142, 3, 56, "Input",
  CellTags->"start"],
Cell[11681, 407, 241, 7, 39, "Text",
  CellTags->"start"],
Cell[11925, 416, 503, 14, 58, "Text",
  CellTags->"start"],
Cell[12431, 432, 385, 10, 56, "Input",
  CellTags->"start"],
Cell[12819, 444, 246, 9, 45, "MathCaption",
  CellTags->"start"],
Cell[13068, 455, 376, 9, 56, "Input",
  CellTags->"start"],
Cell[13447, 466, 376, 9, 56, "Input",
  CellTags->"start"],
Cell[13826, 477, 224, 9, 45, "MathCaption",
  CellTags->"start"],

Cell[CellGroupData[{
Cell[14075, 490, 150, 4, 56, "Input",
  CellTags->"start"],
Cell[14228, 496, 319, 10, 37, "Output"]
}, Closed]],
Cell[14562, 509, 142, 4, 45, "MathCaption",
  CellTags->"start"],

Cell[CellGroupData[{
Cell[14729, 517, 132, 3, 56, "Input",
  CellTags->"start"],
Cell[14864, 522, 143, 3, 37, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[15056, 531, 88, 2, 56, "Section",
  PageBreakAbove->True,
  CellTags->"About Extras"],
Cell[15147, 535, 566, 14, 73, "Text",
  CellTags->"About Extras"],
Cell[15716, 551, 152, 4, 45, "MathCaption",
  CellTags->"About Extras"],
Cell[15871, 557, 115, 3, 43, "Input",
  CellTags->"About Extras"],
Cell[15989, 562, 121, 4, 39, "Text",
  CellTags->"About Extras"],
Cell[16113, 568, 1310, 31, 97, "Text",
  CellTags->"About Extras"],
Cell[17426, 601, 105, 1, 31, "Caption",
  CellTags->"About Extras"],

Cell[CellGroupData[{
Cell[17556, 606, 195, 7, 74, "Subsection",
  CellTags->"tarcer"],
Cell[17754, 615, 69, 1, 46, "InspectorLevel2",
  CellTags->"tarcer"],
Cell[17826, 618, 240, 8, 39, "Text",
  CellTags->"tarcer"],
Cell[18069, 628, 237, 10, 39, "Text",
  CellTags->"tarcer"],
Cell[18309, 640, 61, 1, 49, "Subsubsection",
  CellTags->"tarcer"],
Cell[18373, 643, 629, 15, 73, "Text",
  CellTags->"tarcer"],
Cell[19005, 660, 330, 8, 56, "Text",
  CellTags->"tarcer"],
Cell[19338, 670, 109, 3, 43, "Input",
  CellTags->"tarcer"],
Cell[19450, 675, 258, 8, 39, "Text",
  CellTags->"tarcer"],
Cell[19711, 685, 303, 9, 39, "Text",
  CellTags->"tarcer"]
}, Closed]],

Cell[CellGroupData[{
Cell[20051, 699, 154, 6, 74, "Subsection",
  CellTags->"phi"],
Cell[20208, 707, 60, 1, 46, "InspectorLevel2",
  CellTags->"phi"],
Cell[20271, 710, 422, 9, 73, "Text",
  CellTags->"phi"],
Cell[20696, 721, 70, 1, 39, "Text",
  CellTags->"phi"],
Cell[20769, 724, 103, 3, 43, "Input",
  CellTags->"phi"],
Cell[20875, 729, 255, 8, 39, "Text",
  CellTags->"phi"],
Cell[21133, 739, 283, 9, 39, "Text",
  CellTags->"phi"]
}, Closed]],

Cell[CellGroupData[{
Cell[21453, 753, 63, 1, 53, "Subsection",
  CellTags->{"fa", "extrafoor"}],
Cell[21519, 756, 112, 1, 46, "InspectorLevel2",
  CellTags->{"fa", "extrafoor"}],
Cell[21634, 759, 251, 8, 39, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[21888, 769, 327, 11, 39, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[22218, 782, 465, 8, 73, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[22686, 792, 308, 6, 56, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[22997, 800, 387, 10, 56, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[23387, 812, 209, 5, 63, "Input",
  CellTags->{"fa", "extrafoor"}],
Cell[23599, 819, 269, 8, 39, "Text",
  CellTags->{"fa", "extrafoor"}],
Cell[23871, 829, 565, 16, 56, "Text",
  CellTags->{"fa", "extrafoor"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[24485, 851, 107, 2, 56, "Section",
  PageBreakAbove->True,
  CellTags->{"Example Calculations", "ex"}],
Cell[24595, 855, 540, 15, 47, "Commentary",
  CellTags->"ex"],
Cell[25138, 872, 345, 10, 39, "Text",
  CellTags->"ex"],

Cell[CellGroupData[{
Cell[25508, 886, 342, 11, 53, "Subsection",
  CellTags->"Tree level"],
Cell[25853, 899, 324, 10, 39, "Text",
  CellTags->"Tree level"],
Cell[26180, 911, 85, 1, 39, "Text",
  CellTags->"Tree level"],
Cell[26268, 914, 168, 4, 58, "Input",
  CellTags->"Tree level"],
Cell[26439, 920, 168, 4, 58, "Input",
  CellTags->"Tree level"],
Cell[26610, 926, 168, 4, 58, "Input",
  CellTags->"Tree level"],
Cell[26781, 932, 168, 4, 58, "Input",
  CellTags->"Tree level"],
Cell[26952, 938, 99, 1, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[27076, 943, 337, 9, 56, "Input",
  CellTags->"Tree level"],
Cell[27416, 954, 1411, 40, 38, "Output"]
}, Closed]],
Cell[28842, 997, 2399, 77, 56, "Text",
  CellTags->"Tree level"],
Cell[31244, 1076, 371, 10, 56, "Input",
  CellTags->"Tree level"],
Cell[31618, 1088, 339, 9, 56, "Input",
  CellTags->"Tree level"],
Cell[31960, 1099, 371, 10, 56, "Input",
  CellTags->"Tree level"],
Cell[32334, 1111, 306, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[32643, 1121, 306, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[32952, 1131, 464, 12, 56, "Input",
  CellTags->"Tree level"],
Cell[33419, 1145, 64, 1, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[33508, 1150, 336, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[33847, 1160, 1158, 33, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[35042, 1198, 336, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[35381, 1208, 1158, 33, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[36576, 1246, 336, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[36915, 1256, 1158, 33, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[38110, 1294, 336, 8, 56, "Input",
  CellTags->"Tree level"],
Cell[38449, 1304, 1158, 33, 38, "Output"]
}, Closed]],
Cell[39622, 1340, 74, 1, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[39721, 1345, 134, 2, 58, "Input",
  CellTags->"Tree level"],
Cell[39858, 1349, 1749, 48, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[41644, 1402, 134, 2, 58, "Input",
  CellTags->"Tree level"],
Cell[41781, 1406, 1346, 37, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[43164, 1448, 134, 2, 58, "Input",
  CellTags->"Tree level"],
Cell[43301, 1452, 1346, 37, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[44684, 1494, 134, 2, 58, "Input",
  CellTags->"Tree level"],
Cell[44821, 1498, 1749, 48, 39, "Output"]
}, Closed]],
Cell[46585, 1549, 91, 1, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[46701, 1554, 883, 24, 92, "Input",
  CellTags->"Tree level"],
Cell[47587, 1580, 6793, 184, 61, "Output"]
}, Closed]],
Cell[54395, 1767, 78, 1, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[54498, 1772, 201, 5, 56, "Input",
  CellTags->"Tree level"],
Cell[54702, 1779, 7056, 182, 61, "Output"]
}, Closed]],
Cell[61773, 1964, 118, 3, 39, "Text",
  CellTags->"Tree level"],

Cell[CellGroupData[{
Cell[61916, 1971, 423, 11, 56, "Input",
  CellTags->"Tree level"],
Cell[62342, 1984, 107, 2, 37, "Output"]
}, Closed]],
Cell[62464, 1989, 135, 4, 39, "Text",
  CellTags->"Tree level"],
Cell[62602, 1995, 260, 6, 56, "Input",
  CellTags->"Tree level"],
Cell[62865, 2003, 649, 27, 39, "Text",
  CellTags->"Tree level"]
}, Closed]],

Cell[CellGroupData[{
Cell[63551, 2035, 72, 1, 53, "Subsection",
  CellTags->"A moment integral"],
Cell[63626, 2038, 504, 17, 45, "Text",
  CellTags->"A moment integral"],
Cell[64133, 2057, 127, 3, 39, "Text",
  CellTags->"A moment integral"],
Cell[64263, 2062, 409, 13, 45, "Text",
  CellTags->"A moment integral"],
Cell[64675, 2077, 94, 1, 39, "Text",
  CellTags->"A moment integral"],
Cell[64772, 2080, 288, 10, 39, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[65085, 2094, 149, 3, 56, "Input",
  CellTags->"A moment integral"],
Cell[65237, 2099, 161, 4, 37, "Output"]
}, Closed]],
Cell[65413, 2106, 222, 5, 52, "Text",
  CellTags->"A moment integral"],
Cell[65638, 2113, 2187, 46, 176, "Input",
  CellTags->"A moment integral"],
Cell[67828, 2161, 167, 4, 39, "Text",
  CellTags->"A moment integral"],
Cell[67998, 2167, 17184, 630, 12578, "Input",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[85207, 2801, 144, 3, 56, "Input",
  CellTags->"A moment integral"],
Cell[85354, 2806, 1653, 42, 61, "Output"]
}, Closed]],
Cell[87022, 2851, 154, 4, 47, "Commentary",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[87201, 2859, 126, 2, 58, "Input",
  CellTags->"A moment integral"],
Cell[87330, 2863, 268, 8, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[87635, 2876, 144, 3, 56, "Input",
  CellTags->"A moment integral"],
Cell[87782, 2881, 218, 6, 37, "Output"]
}, Closed]],
Cell[88015, 2890, 66, 1, 39, "Text",
  CellTags->"A moment integral"],
Cell[88084, 2893, 251, 5, 56, "Text",
  CellTags->"A moment integral"],
Cell[88338, 2900, 644, 19, 53, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[89007, 2923, 438, 12, 56, "Input",
  CellTags->"A moment integral"],
Cell[89448, 2937, 223, 6, 37, "Output"]
}, Closed]],
Cell[89686, 2946, 198, 4, 39, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[89909, 2954, 547, 14, 56, "Input",
  CellTags->"A moment integral"],
Cell[90459, 2970, 223, 6, 37, "Output"]
}, Closed]],
Cell[90697, 2979, 90, 1, 39, "Text",
  CellTags->"A moment integral"],
Cell[90790, 2982, 713, 18, 56, "Input",
  CellTags->"A moment integral"],
Cell[91506, 3002, 93, 1, 39, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[91624, 3007, 1538, 38, 96, "Input",
  CellTags->"A moment integral"],
Cell[93165, 3047, 3624, 88, 90, "Output"]
}, Closed]],
Cell[96804, 3138, 148, 4, 39, "Text",
  CellTags->"A moment integral"],
Cell[96955, 3144, 653, 17, 57, "Input",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[97633, 3165, 141, 3, 56, "Input",
  CellTags->"A moment integral"],
Cell[97777, 3170, 109, 2, 37, "Output"]
}, Closed]],
Cell[97901, 3175, 152, 4, 39, "Text",
  CellTags->"A moment integral"],
Cell[98056, 3181, 317, 9, 56, "Input",
  CellTags->"A moment integral"],
Cell[98376, 3192, 218, 5, 39, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[98619, 3201, 217, 5, 56, "Input",
  CellTags->"A moment integral"],
Cell[98839, 3208, 4620, 115, 137, "Output"]
}, Closed]],
Cell[103474, 3326, 268, 8, 39, "Text",
  CellTags->"A moment integral"],

Cell[CellGroupData[{
Cell[103767, 3338, 497, 13, 57, "Input",
  CellTags->"A moment integral"],
Cell[104267, 3353, 3550, 90, 96, "Output"]
}, Closed]],
Cell[107832, 3446, 131, 3, 39, "Text",
  CellTags->"A moment integral"],
Cell[107966, 3451, 213, 5, 56, "Input",
  CellTags->"A moment integral"],
Cell[108182, 3458, 281, 7, 56, "Input",
  CellTags->"A moment integral"],
Cell[108466, 3467, 1240, 55, 39, "Text",
  CellTags->"A moment integral"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

