(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   3498227,     109376]*)
(*NotebookOutlinePosition[   3878720,     119316]*)
(*  CellTagsIndexPosition[   3862101,     118841]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 4.2", "Title",
  TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
  TextAlignment->Center],

Cell[TextData[StyleBox["Reference",
  FontWeight->"Bold"]], "Subtitle",
  PageBreakBelow->True,
  TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
  TextAlignment->Center],

Cell[TextData[{
  "HTML version: ",
  ButtonBox["http://www.feyncalc.org/",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Functions", "Section",
  CellTags->"Functions"],

Cell[CellGroupData[{

Cell["Evaluation time and memory usage", "Commentary"],

Cell["\<\
This is only for timing purposes and can be omitted in \
general.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"absolutestarttime", "=", 
      RowBox[{"AbsoluteTime", "[", "]"}]}]], "Input",
  CellLabel->"In[64]:=  (21:51:49 on 5/16/01)"],

Cell[BoxData[
    FormBox["3.199038709`15.5256*^9", TraditionalForm]], "Output",
  CellLabel->"Out[64]=  (21:51:49 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Abbreviation", "ObjectName",
  PageBreakAbove->True,
  CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Abbreviation"],

Cell["\<\
Abbreviation is a function used by OneLoop and PaVeReduce for \
generating smaller files when saving results to the hard disk. The convention \
is that a definition like GP = GluonPropagator should be accompanied by the \
definition Abbreviation[GluonPropagator] = HoldForm[GP].\
\>", "Text",
  CellTags->"Abbreviation"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["$Abbreviations",
    ButtonData:>"$Abbreviations",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$Abbreviations"],
  ", ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["WriteOut",
    ButtonData:>"WriteOut",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"WriteOut"],
  ", ",
  ButtonBox["WriteOutPaVe",
    ButtonData:>"WriteOutPaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"WriteOutPaVe"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["QuarkPropagator",
    ButtonData:>"QuarkPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkPropagator"],
  "."
}], "Text",
  CellTags->"Abbreviation"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GP", "[", 
      RowBox[{"p", ",", " ", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", " ", "a"}], "}"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", " ", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[32]:=  (18:07:05 on 9/2/02)",
  CellTags->"Abbreviation"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (18:07:05 on 9/2/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Amplitude", "ObjectName",
  CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Amplitude"],

Cell["\<\
Amplitude is a database of Feynman amplitudes. Amplitude[\"name\"] \
returns the amplitude corresponding to the string \"name\". A list of all \
defined names is obtained with Amplitude[]. New amplitudes can be added to \
the file \"Amplitude.m\". It is strongly recommended to use names that \
reflect the process. The option Gauge \[Rule] 1 means `t Hooft Feynman gauge; \
Polarization \[Rule] 0 gives unpolarized OPE-type amplitudes, Polarization \
\[Rule] 1 the polarized ones.\
\>", "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Amplitude", "]"}]], "Input",
  CellLabel->"In[70]:=  (21:51:49 on 5/16/01)",
  CellTags->"Amplitude"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
          RowBox[{"QuarkMass", "\[Rule]", "0"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[70]=  (21:51:49 on 5/16/01)",
  CellTags->"Amplitude"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmp",
    ButtonData:>"FeynAmp",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmp"],
  "."
}], "Text",
  CellTags->"Amplitude"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Amplitude"],

Cell[BoxData[
    RowBox[{"Amplitude", "[", "]"}]], "Input",
  CellLabel->"In[71]:=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"],

Cell["This is the amplitude of a gluon self-energy diagram.", "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Amplitude", "[", "\"\<se1g1\>\"", "]"}]], "Input",
  CellLabel->"In[71]:=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            SubsuperscriptBox["g", "s", "2"], " ", 
            RowBox[{
              SuperscriptBox["V", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], "(", 
              RowBox[{
                FormBox[
                  RowBox[{"-", "p"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"p", "-", "q"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["V", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], "(", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"q", "-", "p"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"-", "q"}],
                  "TraditionalForm"]}], ")"}], " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["f",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["f", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["f", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["f",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
            SuperscriptBox["q", "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[71]=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[72]:=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
              SuperscriptBox["q", "2"]}]], 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["g", "s", "2"], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Alpha]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["e",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["f",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["e",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["f",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[72]=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"]
}, Closed]],

Cell[TextData[{
  "This is the amplitude for graph 6.2 from the paper Z.Phys C ",
  StyleBox["70:",
    FontWeight->"Bold"],
  "637-654, 1996",
  StyleBox[".",
    FontWeight->"Bold"]
}], "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{"q1", ",", "q2", ",", 
        RowBox[{"EpsEvaluate", "[", 
          RowBox[{"Trick", "[", 
            RowBox[{"Explicit", "[", 
              RowBox[{"Amplitude", "[", "\"\<gg2\>\"", "]"}], "]"}], "]"}], 
          "]"}]}], "]"}]], "Input",
  CellLabel->"In[73]:=  (21:51:50 on 5/16/01)",
  CellTags->"Amplitude"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        RowBox[{
          StyleBox[
            RowBox[{
              SuperscriptBox["\[DifferentialD]", "D"], 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}],
            ZeroWidthTimes->True], "\[Integral]", 
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], 
            FormBox[
              SubscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]}], "(", 
        FormBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", 
                SubsuperscriptBox["g", "s", "4"], " ", 
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Lambda]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Lambda]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", 
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]]}], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{"2", " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "1"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "1"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "2"}], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "5"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "5"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", 
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]]}], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SubsuperscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2", 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", 
                    FormBox[
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{"-", "1"}], ")"}], "m"],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox[
                            StyleBox["\<\"q\"\>",
                              FontSlant->"Italic"], "2"],
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{"m", "-", "1"}]],
                  "TraditionalForm"], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "11"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "7"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "10"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "7"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "12"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "10"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "11"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "12"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], ")"}], "/", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1", "2"], ".", 
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2", "2"], ".", 
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
              ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"], "-", 
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"], "-", "p"}], ")"}], 
                "2"]}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[73]=  (21:51:51 on 5/16/01)",
  CellTags->"Amplitude"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Amputate", "ObjectName",
  CellTags->"Amputate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Amputate"],

Cell["\<\
Amputate[exp,q1,q2,...] amputates Eps and DiracGamma. \
Amputate[exp,q1,q2,Pair\[Rule]{p}] amputates also p.q1 and p.q2; \
Pair\[Rule]All amputates all except OPEDelta.\
\>", "Text",
  CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Amputate", "]"}]], "Input",
  CellLabel->"In[74]:=  (21:51:51 on 5/16/01)",
  CellTags->"Amputate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Pair", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[74]=  (21:51:51 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also:  ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"Amputate"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
      RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[75]:=  (21:51:52 on 5/16/01)",
  CellTags->"Amputate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[75]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Amputate", "[", 
      RowBox[{"%", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[76]:=  (21:51:52 on 5/16/01)",
  CellTags->"Amputate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$41", "\[InvisibleApplication]", 
                  RowBox[{"(", "1", ")"}]}],
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$41", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[76]=  (21:51:52 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["AnomalousDimension", "ObjectName",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"AnomalousDimension"],

Cell["\<\
AnomalousDimension[name] is a database of anomalous dimensions of \
twist 2 operators.\
\>", "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "AnomalousDimension", "]"}]], "Input",
  CellLabel->"In[77]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[77]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["SplittingFunction",
    ButtonData:>"SplittingFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SplittingFunction"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumT"],
  "."
}], "Text",
  CellTags->"AnomalousDimension"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"AnomalousDimension"],

Cell["Polarized case", "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", " ", 
      RowBox[{"AnomalousDimension", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}]], "Input",
  CellLabel->"In[78]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[78]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"NS", ",", "qq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gnsqq0", "]"}]], "Input",
  CellLabel->"In[79]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"8", " ", 
              RowBox[{
                SubscriptBox["S", "1"], "(", 
                FormBox[
                  RowBox[{"m", "-", "1"}],
                  "TraditionalForm"], ")"}]}], "+", 
            FractionBox["4", "m"], "+", 
            FractionBox["4", 
              RowBox[{"m", "+", "1"}]], "-", "6"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[79]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "qg", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsqg0", "]"}]], "Input",
  CellLabel->"In[80]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FractionBox["8", "m"], "-", 
            FractionBox["16", 
              RowBox[{"m", "+", "1"}]]}], ")"}], " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
  CellLabel->"Out[80]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgq0", "]"}]], "Input",
  CellLabel->"In[81]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["4", 
              RowBox[{"m", "+", "1"}]], "-", 
            FractionBox["8", "m"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[81]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgg0", "]"}]], "Input",
  CellLabel->"In[82]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"8", " ", 
            SubscriptBox["T", "f"]}], "3"], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["8", "m"], "+", 
              FractionBox["16", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["22", "3"]}], ")"}]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[82]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"PS", ",", "qq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gpsqq1", "]"}]], "Input",
  CellLabel->"In[83]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"16", " ", 
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["1", 
              RowBox[{"m", "+", "1"}]], "+", 
            FractionBox["3", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
            FractionBox["2", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
            FractionBox["1", "m"], "-", 
            FractionBox["1", 
              SuperscriptBox["m", "2"]], "+", 
            FractionBox["2", 
              SuperscriptBox["m", "3"]]}], ")"}], " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
  CellLabel->"Out[83]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"NS", ",", "qq", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gnsqq1", "]"}]], "Input",
  CellLabel->"In[84]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  SuperscriptBox["m", "2"]], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"m", "+", "1"}]], "-", 
                RowBox[{"24", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox["S", "12"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox["S", "21"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                FractionBox[
                  RowBox[{"32", " ", 
                    RowBox[{
                      SubscriptBox[
                        OverscriptBox["S", "~"], "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"], "+", 
                FractionBox[
                  RowBox[{"32", " ", 
                    RowBox[{
                      SubscriptBox[
                        OverscriptBox["S", "~"], "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"m", "+", "1"}]], "-", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "3"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"64", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "12"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "-", 
                FractionBox["40", "m"], "+", 
                FractionBox["40", 
                  RowBox[{"m", "+", "1"}]], "+", 
                FractionBox["16", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
                FractionBox["8", 
                  SuperscriptBox["m", "3"]], "+", 
                FractionBox["40", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "3"}], 
              ")"}]}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "-", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FractionBox["80", "9"], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{
                FractionBox["16", "3"], " ", 
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["8", 
                RowBox[{"9", " ", "m"}]], "+", 
              FractionBox["88", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["8", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["8", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["2", "3"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["536", "9"]}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                FractionBox["88", "3"], " ", 
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"16", " ", 
                RowBox[{
                  SubscriptBox["S", "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"16", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"32", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox["212", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["748", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["44", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["4", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["16", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
              FractionBox["17", "3"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[84]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "qg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsqg1", "]"}]], "Input",
  CellLabel->"In[85]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"8", " ", 
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox["14", "m"], "-", 
              FractionBox["19", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["1", 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox["8", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
              FractionBox["2", 
                SuperscriptBox["m", "3"]], "+", 
              FractionBox["4", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
        RowBox[{"16", " ", 
          SubscriptBox["C", "A"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}], "m"]}], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], "m"], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["4", "m"], "+", 
              FractionBox["3", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["3", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["8", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["2", 
                SuperscriptBox["m", "3"]], "+", 
              FractionBox["12", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[85]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gq", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgq1", "]"}]], "Input",
  CellLabel->"In[86]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox["15", "m"], "-", 
              FractionBox["6", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["12", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["3", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["2", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"3", " ", "m"}]]}], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["7", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["2", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["2", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "+", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{"8", " ", 
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{
                      SubsuperscriptBox["S", "1", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"]}], "+", 
              FractionBox[
                RowBox[{
                  SubsuperscriptBox["S", "1", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"3", " ", "m"}]], "-", 
              FractionBox[
                RowBox[{"5", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["56", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["20", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["28", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["38", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["6", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[86]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgg1", "]"}]], "Input",
  CellLabel->"In[87]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              RowBox[{
                FractionBox["134", "9"], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox["S", "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "21"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["107", 
                RowBox[{"9", " ", "m"}]], "+", 
              FractionBox["241", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["58", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["86", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["8", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["48", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
              FractionBox["16", "3"]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["5", "9"]}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox["14", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["19", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "+", 
              FractionBox["1", "3"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        RowBox[{"8", " ", 
          SubscriptBox["C", "F"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox["10", 
                  RowBox[{"m", "+", "1"}]]}], "+", 
              FractionBox["2", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "1", "+", 
              FractionBox["10", "m"], "-", 
              FractionBox["10", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]]}], ")"}], " ", 
          SubscriptBox["T", "f"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[87]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized (different representation)"
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "GSGG1", "]"}]], "Input",
  CellLabel->"In[88]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{
                    RowBox[{"m", " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"m", " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"m", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"48", " ", "m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"m", "+", "3"}], ")"}]}], "+",
                                         "469"}], ")"}]}], "+", "698"}], 
                                    ")"}]}], "+", "7"}], ")"}]}], "+", 
                          "258"}], ")"}]}], "+", "144"}], 
                  RowBox[{"9", " ", 
                    SuperscriptBox["m", "3"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", 
                    "\[NoBreak]", 
                    FormBox[
                      FractionBox["m", "2"],
                      "TraditionalForm"], "\[NoBreak]", ")"}]}], 
                RowBox[{"m", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              RowBox[{
                SubsuperscriptBox["S", "3", "'"], "\[NoBreak]", "(", 
                "\[NoBreak]", 
                FormBox[
                  FractionBox["m", "2"],
                  "TraditionalForm"], "\[NoBreak]", ")"}], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"m", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"67", " ", "m", " ", 
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{"m", "+", "1"}], ")"}], "2"]}], "+",
                             "144"}], ")"}]}], "+", "72"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox["m",
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"9", " ", 
                  SuperscriptBox["m", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", 
                  "\[NoBreak]", 
                  FormBox[
                    FractionBox["m", "2"],
                    "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{
                  OverscriptBox["S", "~"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  RowBox[{"m", " ", 
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"3", " ", "m", " ", 
                          RowBox[{"(", 
                            RowBox[{"m", "+", "1"}], ")"}]}], "+", "13"}], 
                      ")"}]}], "-", "3"}], 
                RowBox[{"9", " ", 
                  SuperscriptBox["m", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox[
                RowBox[{"5", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox["m",
                      "TraditionalForm"], ")"}]}], "9"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        FractionBox[
          RowBox[{"8", " ", 
            SubscriptBox["C", "F"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"m", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"m", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"m", " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"m", " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"m", "+", "3"}], ")"}]}], "+",
                                         "5"}], ")"}]}], "+", "1"}], ")"}]}], 
                            "-", "8"}], ")"}]}], "+", "2"}], ")"}]}], "+", 
                "4"}], ")"}], " ", 
            SubscriptBox["T", "f"]}], 
          RowBox[{
            SuperscriptBox["m", "3"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[88]=  (21:51:52 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Check that all odd moments give the same for the two representations of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Gamma]", 
            RowBox[{"S", ",", "gg", " "}], 
            RowBox[{"(", "1", ")", " "}]], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"%", "-", "%%"}], "/.", 
          RowBox[{"OPEm", "\[Rule]", "ij"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"ij", ",", "1", ",", "17", ",", "2"}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[89]:=  (21:51:52 on 5/16/01)",
  CellTags->"AnomalousDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
          "0", ",", "0"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[89]=  (21:51:53 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["AntiCommutator", "ObjectName",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"AntiCommutator"],

Cell["\<\
AntiCommutator[x, y] = c defines the anti-commutator of the non \
commuting objects x and y.\
\>", "Text",
  CellTags->"AntiCommutator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Commutator",
    ButtonData:>"Commutator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Commutator"],
  ", ",
  ButtonBox["CommutatorExplicit",
    ButtonData:>"CommutatorExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CommutatorExplicit"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"AntiCommutator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"AntiCommutator"],

Cell[TextData[{
  "This declares ",
  StyleBox["a",
    FontFamily->"Courier"],
  " and ",
  StyleBox["b",
    FontFamily->"Courier"],
  " as noncommutative variables."
}], "Text",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[99]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiCommutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[100]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "{", "\[NoBreak]", "a", "\[NoBreak]", ",", "\[NoBreak]", 
        "\[MediumSpace]", "\[NoBreak]", "b", "\[NoBreak]", "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[100]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[101]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "+", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[101]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          RowBox[{"a", "+", "b"}], ",", 
          RowBox[{
            RowBox[{"a", "-", 
              RowBox[{"2", "b"}]}], " "}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[102]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", 
              RowBox[{"2", " ", "b"}]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "-", 
              RowBox[{"2", " ", "b"}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[102]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          RowBox[{"a", "+", "b"}], ",", 
          RowBox[{
            RowBox[{"a", "-", 
              RowBox[{"2", "b"}]}], " "}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[103]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          SuperscriptBox["b", "2"]}], "-", 
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[103]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"c", ",", "d", ",", 
        OverscriptBox["c", "~"], ",", 
        OverscriptBox["d", "~"]}], "]"}]], "Input",
  CellLabel->"In[104]:=  (21:51:53 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell["Defining {c,d} = z results in replacements of c.d by z-d.c.", "Text",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"c", ",", "d"}], "]"}], " ", "=", " ", "z"}]], "Input",
  CellLabel->"In[105]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox["z", TraditionalForm]], "Output",
  CellLabel->"Out[105]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", " ", 
      RowBox[{"d", " ", ".", " ", "c", " ", ".", " ", "d"}], " ", 
      "]"}]], "Input",
  CellLabel->"In[106]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"d", " ", "z"}], "-", 
        RowBox[{
          SuperscriptBox["d", "2"], ".", "c"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[106]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          OverscriptBox["d", "~"], ",", 
          OverscriptBox["c", "~"]}], "]"}], " ", "=", " ", 
      OverscriptBox["z", "~"]}]], "Input",
  CellLabel->"In[107]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      OverscriptBox["z", "~"], TraditionalForm]], "Output",
  CellLabel->"Out[107]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", " ", 
      RowBox[{
        OverscriptBox["d", "~"], " ", ".", " ", 
        OverscriptBox["c", "~"], " ", ".", " ", 
        OverscriptBox["d", "~"]}], " ", "]"}]], "Input",
  CellLabel->"In[108]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          OverscriptBox["d", "~"], " ", 
          OverscriptBox["z", "~"]}], "-", 
        RowBox[{
          OverscriptBox["c", "~"], ".", 
          SuperscriptBox[
            OverscriptBox["d", "~"], "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[108]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", 
        OverscriptBox["c", "~"], ",", 
        OverscriptBox["d", "~"]}], "]"}]], "Input",
  CellLabel->"In[109]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    RowBox[{"Unset", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"c", ",", "d"}], "]"}], "]"}]], "Input",
  CellLabel->"In[110]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"],

Cell[BoxData[
    RowBox[{"Unset", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          OverscriptBox["d", "~"], ",", 
          OverscriptBox["c", "~"]}], "]"}], "]"}]], "Input",
  CellLabel->"In[111]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiCommutator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["AntiQuarkField", "ObjectName",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"AntiQuarkField"],

Cell["\<\
AntiQuarkField is the name of a fermionic field. AntiQuarkField is \
just a name with no functional properties. Only typeset rules are attached.\
\
\>", "Text",
  CellTags->"AntiQuarkField"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuantumField"],
  ", ",
  ButtonBox["QuarkField",
    ButtonData:>"QuarkField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkField"],
  "."
}], "Text",
  CellTags->"AntiQuarkField"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell[BoxData["AntiQuarkField"], "Input",
  CellLabel->"In[112]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiQuarkField"],

Cell[BoxData[
    FormBox[
      OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
  CellLabel->"Out[112]=  (21:51:54 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["AntiSymmetrize", "ObjectName",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"AntiSymmetrize"],

Cell["\<\
AntiSymmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with \
respect to the variables a1,a2, ... \
\>", "Text",
  CellTags->"AntiSymmetrize"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Symmetrize",
    ButtonData:>"Symmetrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Symmetrize"],
  "."
}], "Text",
  CellTags->"AntiSymmetrize"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiSymmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"a", ",", "b"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[113]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiSymmetrize"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"a", ",", "b"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[113]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiSymmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[114]:=  (21:51:54 on 5/16/01)",
  CellTags->"AntiSymmetrize"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "6"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "y", ",", "z"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "z", ",", "y"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "x", ",", "y"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[114]=  (21:51:54 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Anti5", "ObjectName",
  CellTags->"Anti5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Anti5"],

Cell[TextData[{
  "Anti5[exp] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "in exp to the right. Anti5[exp, n] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", 
          RowBox[{"5", " "}]], TraditionalForm]]],
  "n times to the right. Anti5[exp, -n] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " n times to the left."
}], "Text",
  CellTags->"Anti5"],

Cell[TextData[{
  "The naive ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "scheme is used."
}], "Text",
  CellTags->"Anti5"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracOrder",
    ButtonData:>"DiracOrder",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracOrder"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"Anti5"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"5", ",", "\[Mu]"}], "]"}], " "}]], "Input",
  CellLabel->"In[90]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[90]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", "%", "]"}]], "Input",
  CellLabel->"In[91]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[91]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", 
        RowBox[{"-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[92]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[92]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracMatrix", "[", 
      RowBox[{
      "5", ",", "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", 
        "\[Delta]"}], "]"}]], "Input",
  CellLabel->"In[93]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], 
        SuperscriptBox["\[Gamma]", "\[Alpha]"], 
        SuperscriptBox["\[Gamma]", "\[Beta]"], 
        SuperscriptBox["\[Gamma]", "\[Gamma]"], 
        SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[93]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", "2"}], "]"}]], "Input",
  CellLabel->"In[94]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[94]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%%", ",", "Infinity"}], "]"}]], "Input",
  CellLabel->"In[95]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[95]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", 
        RowBox[{"-", "Infinity"}]}], "]"}]], "Input",
  CellLabel->"In[96]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[96]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "In the naive ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "- scheme D-dimensional ",
  Cell[BoxData[
      FormBox["\[Gamma]", TraditionalForm]]],
  "-matrices anticommute with ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{"5", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[97]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[97]=  (21:51:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", "%", "]"}]], "Input",
  CellLabel->"In[98]:=  (21:51:53 on 5/16/01)",
  CellTags->"Anti5"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[98]=  (21:51:53 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Apart1", "ObjectName",
  CellTags->"Apart1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Apart1"],

Cell["\<\
Apart1[expr, x] is equivalent to Apart[expr, x], but it is faster \
than Apart if explicit complex numbers (I's) are in exp.\
\>", "Text",
  CellTags->"Apart1"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Apart2",
    ButtonData:>"Apart2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Apart2"],
  ", ",
  ButtonBox["Apart3",
    ButtonData:>"Apart3",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Apart3"],
  "."
}], "Text",
  CellTags->"Apart1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Apart1"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"1", "/", "24"}], "*", "11", "*", "I", "*", "Pi", "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"36", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"7", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"48", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"49", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "5"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"144", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "5"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", "Zeta2"}], "+", 
          RowBox[{
            RowBox[{"11", "/", "24"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", "I", "*", "Pi", "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}]}]}], ";"}]], "Input",
  CellLabel->"In[115]:=  (21:51:54 on 5/16/01)",
  CellTags->"Apart1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a1", "=", 
          RowBox[{"Apart1", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input",
  CellLabel->"In[116]:=  (21:51:54 on 5/16/01)",
  CellTags->"Apart1"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.29999999999999716`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[116]=  (21:51:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a2", "=", 
          RowBox[{"Apart", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input",
  CellLabel->"In[117]:=  (21:51:54 on 5/16/01)",
  CellTags->"Apart1"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"1.6329999999999956`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[117]=  (21:51:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a1", "-", "a2"}], "//", "Expand"}]], "Input",
  CellLabel->"In[118]:=  (21:51:58 on 5/16/01)",
  CellTags->"Apart1"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[118]=  (21:51:58 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "a1", ",", "a2"}], "]"}]], "Input",
  CellLabel->"In[119]:=  (21:51:58 on 5/16/01)",
  CellTags->"Apart1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Apart2", "ObjectName",
  CellTags->"Apart2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Apart2"],

Cell["\<\
Apart2[expr] partial fractions FeynAmpDenominators (and \
FAD's).\
\>", "Text",
  CellTags->"Apart2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominator"],
  "."
}], "Text",
  CellTags->"Apart2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Apart2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "M"}], "}"}], ",", 
        RowBox[{"q", "-", "p"}]}], "]"}]], "Input",
  CellLabel->"In[120]:=  (21:51:58 on 5/16/01)",
  CellTags->"Apart2"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", "p"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
                "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
                "\[InvisibleSpace]", 
                SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "-", 
                Power[ M, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[120]=  (21:51:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Apart2", "[", "%", "]"}]], "Input",
  CellLabel->"In[121]:=  (21:51:58 on 5/16/01)",
  CellTags->"Apart2"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FractionBox["1", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
          FractionBox["1", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["M", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
        RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[121]=  (21:51:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", "%", "]"}], "]"}]], "Input",
  CellLabel->"In[122]:=  (21:51:59 on 5/16/01)",
  CellTags->"Apart2"],

Cell[BoxData[
    FractionBox[
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "m"}], "}"}], ",", 
            RowBox[{
              RowBox[{"-", "p"}], "+", "q"}]}], "]"}], "-", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "M"}], "}"}], ",", 
            RowBox[{
              RowBox[{"-", "p"}], "+", "q"}]}], "]"}]}], 
      RowBox[{
        SuperscriptBox["m", "2"], "-", 
        SuperscriptBox["M", "2"]}]]], "Output",
  CellLabel->"Out[122]//StandardForm=  (21:51:59 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Apart3", "ObjectName",
  CellTags->"Apart3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Apart3"],

Cell["\<\
Apart3[expr, x] is equivalent to Map2[Factor2, \
Collect2[Apart1[expr,x],x]].\
\>", "Text",
  CellTags->"Apart3"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Apart1",
    ButtonData:>"Apart1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Apart1"],
  "."
}], "Text",
  CellTags->"Apart3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Apart3"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"11", "/", "24"}], " ", "I", " ", "\[Pi]", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"36", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"7", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"48", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"49", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "5"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"144", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "5"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", "Zeta2"}], "+", 
          RowBox[{
            RowBox[{"11", "/", "24"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", "I", " ", "\[Pi]", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}]}]}], ";"}]], "Input",
  CellLabel->"In[123]:=  (21:51:59 on 5/16/01)",
  CellTags->"Apart3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a1", "=", 
          RowBox[{"Apart3", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input",
  CellLabel->"In[124]:=  (21:51:59 on 5/16/01)",
  CellTags->"Apart3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.47099999999999653`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[124]=  (21:52:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a2", "=", 
          RowBox[{"Apart", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input",
  CellLabel->"In[125]:=  (21:52:00 on 5/16/01)",
  CellTags->"Apart3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"1.631999999999998`", " ", "Second"}], ",", "Null"}], "}"}],
       TraditionalForm]], "Output",
  CellLabel->"Out[125]=  (21:52:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a1", "-", "a2"}], "//", "Expand"}]], "Input",
  CellLabel->"In[126]:=  (21:52:02 on 5/16/01)",
  CellTags->"Apart3"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[126]=  (21:52:02 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "a1", ",", "a2"}], "]"}]], "Input",
  CellLabel->"In[127]:=  (21:52:02 on 5/16/01)",
  CellTags->"Apart3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["A0", "ObjectName",
  PageBreakAbove->True,
  CellTags->"A0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"A0"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"A0", "[", 
            SuperscriptBox["m", "2"], "]"}], " ", "is", "the", "Passarino"}], 
        "-", 
        RowBox[{"Veltman", "one"}], "-", 
        RowBox[{"point", " ", " ", "integral", " ", " ", 
          SubscriptBox["A", "0."]}]}], TraditionalForm]]]], "Text",
  CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
  CellLabel->"In[65]:=  (21:51:49 on 5/16/01)",
  CellTags->"A0"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"A0ToB0", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[65]=  (21:51:49 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  "."
}], "Text",
  CellTags->"A0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"A0"],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["A", "0"], TraditionalForm]]],
  " is not expressed in terms of ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "0"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      SuperscriptBox["m", "2"], "]"}]], "Input",
  CellLabel->"In[15]:=  (16:35:18 on 9/10/02)",
  CellTags->"A0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[15]=  (16:35:18 on 9/10/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"A0", ",", 
          RowBox[{"A0ToB0", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[16]:=  (16:35:23 on 9/10/02)",
  CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      SuperscriptBox["m", "2"], "]"}]], "Input",
  CellLabel->"In[17]:=  (16:35:27 on 9/10/02)",
  CellTags->"A0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[17]=  (16:35:27 on 9/10/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"A0", ",", 
          RowBox[{"A0ToB0", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[18]:=  (16:35:29 on 9/10/02)",
  CellTags->"A0"],

Cell[TextData[{
  "According to the rules of dimensional regularization ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["A", "0"], "(", "0", ")"}], TraditionalForm]]],
  " is set to 0."
}], "Text",
  CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", "0", "]"}]], "Input",
  CellLabel->"In[19]:=  (16:35:35 on 9/10/02)",
  CellTags->"A0"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[19]=  (16:35:35 on 9/10/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      RowBox[{"SmallVariable", "[", 
        RowBox[{"M", "^", "2"}], "]"}], "]"}]], "Input",
  CellLabel->"In[20]:=  (16:35:40 on 9/10/02)",
  CellTags->"A0"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[20]=  (16:35:40 on 9/10/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["A0ToB0", "ObjectName",
  PageBreakAbove->True,
  CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"A0ToB0"],

Cell[TextData[{
  "A0ToB0 is an option for A0. If set to True, A0[",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "] is expressed by (1 + B0[0, ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "]) ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
  CellLabel->"In[21]:=  (16:35:56 on 9/10/02)",
  CellTags->"A0ToB0"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[21]=  (16:35:56 on 9/10/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["A0",
    ButtonData:>"A0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"A0"],
  ", ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  "."
}], "Text",
  CellTags->"A0ToB0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["BackgroundGluonVertex", "ObjectName",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"BackgroundGluonVertex"],

Cell[TextData[{
  "BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or \
BackgroundGluonVertex[ p,mu,a , q,nu,b , k,la,c ] yields the 3-gluon vertex \
in the background field gauge, where the first set of arguments corresponds \
to the external background field. BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, \
{k,la,c}, {s,si,d}] or BackgroundGluonVertex[{mu,a}, {nu,b}, {la,c}, {",
  "si",
  ",d}] or BackgroundGluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
BackgroundGluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex, \
with {p,mu,a} and {k,la,c} denoting the external background fields. The \
gauge, dimension and the name of the coupling constant are determined by the \
options Gauge, Dimension and CouplingConstant. The Feynman rules are taken \
from L.Abbot ",
  StyleBox["NPB ",
    FontWeight->"Bold"],
  "185 (1981), 189-203; except that all momenta are incoming. Note that \
Abbots coupling constant convention is consistent with the default setting of \
GluonVertex."
}], "Text",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "BackgroundGluonVertex", "]"}]], "Input",
  CellLabel->"In[159]:=  (21:52:06 on 5/16/01)",
  CellTags->"BackgroundGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[159]=  (21:52:06 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text",
  CellTags->"BackgroundGluonVertex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}]}], "]"}]], "Input",\

  CellLabel->"In[160]:=  (21:52:06 on 5/16/01)",
  CellTags->"BackgroundGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"-", 
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"]}], "+", "p", "-", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", 
                        FormBox[
                          TagBox[
                            StyleBox["\<\"k\"\>",
                              FontSlant->"Italic"],
                            (StyleForm[ #, FontSlant -> "Italic"]&)],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          TagBox[
                            StyleBox["\<\"k\"\>",
                              FontSlant->"Italic"],
                            (StyleForm[ #, FontSlant -> "Italic"]&)],
                          "TraditionalForm"], "-", "p", "+", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[160]=  (21:52:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[161]:=  (21:52:06 on 5/16/01)",
  CellTags->"BackgroundGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[161]=  (21:52:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
  CellLabel->"In[162]:=  (21:52:07 on 5/16/01)",
  CellTags->"BackgroundGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"p", "-", "q"}], ")"}], "-", 
                        FractionBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"], "\[Alpha]"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", 
                        FormBox[
                          TagBox[
                            StyleBox["\<\"k\"\>",
                              FontSlant->"Italic"],
                            (StyleForm[ #, FontSlant -> "Italic"]&)],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"], "-", "p"}], ")"}], "+", 
                        FractionBox["q", "\[Alpha]"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[162]=  (21:52:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
  CellLabel->"In[163]:=  (21:52:07 on 5/16/01)",
  CellTags->"BackgroundGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  FractionBox[
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Lambda]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Sigma]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "\[Alpha]"], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Lambda]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Sigma]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "\[Alpha]"], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[163]=  (21:52:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Bracket", "ObjectName",
  CellTags->"Bracket"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Bracket"],

Cell["Bracket is an option of Convolute.", "Text",
  CellTags->"Bracket"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Convolute"],
  "."
}], "Text",
  CellTags->"Bracket"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["BReduce", "ObjectName",
  PageBreakAbove->True,
  CellTags->"BReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"BReduce"],

Cell["\<\
BReduce is an option for B0, B00, B1, B11 determining whether \
reductions to A0 and B0 will be done. \
\>", "Text",
  CellTags->"BReduce"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["A0",
    ButtonData:>"A0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"A0"],
  ", ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["B11",
    ButtonData:>"B11",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B11"],
  "."
}], "Text",
  CellTags->"BReduce"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"BReduce"],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "0"], TraditionalForm]]],
  " is not expressed in terms of ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["A", "0"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "s", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[28]:=  (16:39:37 on 9/10/02)",
  CellTags->"BReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[28]=  (16:39:37 on 9/10/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "//", "Options"}]], "Input",
  CellLabel->"In[23]:=  (16:37:41 on 9/10/02)"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[23]=  (16:37:41 on 9/10/02)"]
}, Closed]],

Cell["\<\
With BReduce\[Rule]True, transformation is done.\
\>", "Text",
  CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "s", ",", "s", ",", 
        RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[30]:=  (16:41:35 on 9/10/02)",
  CellTags->"BReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            FormBox[
              SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
              "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "s", 
            "\[NoBreak]", ")"}], "s"], "-", "1"}], TraditionalForm]], "Output",\

  CellLabel->"Out[30]=  (16:41:35 on 9/10/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B0", "ObjectName",
  CellTags->"B0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B0"],

Cell[TextData[{
  "B0[pp, ma^2, mb^2] is the Passarino-Veltman two-point integral ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "0"], TraditionalForm]]],
  ". All arguments are scalars and have dimension mass^2. If the option \
BReduce is set to True certain B0's are reduced to A0's. Setting the option \
B0Unique to True simplifies B0[a,0,a] and B0[0,0,a]."
}], "Text",
  CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B0", "]"}]], "Input",
  CellLabel->"In[135]:=  (21:52:03 on 5/16/01)",
  CellTags->"B0"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[135]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B11",
    ButtonData:>"B11",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B11"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  "."
}], "Text",
  CellTags->"B0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[136]:=  (21:52:03 on 5/16/01)",
  CellTags->"B0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[136]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"B0", ",", 
          RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[137]:=  (21:52:03 on 5/16/01)",
  CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[138]:=  (21:52:03 on 5/16/01)",
  CellTags->"B0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[138]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{
        SuperscriptBox["m", "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[139]:=  (21:52:04 on 5/16/01)",
  CellTags->"B0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}], "+", "2"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[139]=  (21:52:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", 
        RowBox[{"m", "^", "2"}], ",", 
        RowBox[{"m", "^", "2"}]}], "]"}]], "Input",
  CellLabel->"In[140]:=  (21:52:04 on 5/16/01)",
  CellTags->"B0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[140]=  (21:52:04 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B0Real", "ObjectName",
  PageBreakAbove->True,
  CellTags->"B0Real"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B0Real"],

Cell["\<\
B0Real is an option of B0 (default False). If set to True, B0 is \
assumed to be real and the relation B0[a,0,a] = 2 + B0[0,a,a]  is \
applied.\
\>", "Text",
  CellTags->"B0Real"],

Cell[TextData[{
  "See also: ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  "."
}], "Text",
  CellTags->"B0Real"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B0Real"],

Cell["By default the arguments are not assumed real.", "Text",
  CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"s", ",", "0", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[32]:=  (16:45:03 on 9/10/02)",
  CellTags->"B0Real"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", "s", ",", "0", ",", "s", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (16:45:03 on 9/10/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "//", "Options"}]], "Input",
  CellLabel->"In[33]:=  (16:45:39 on 9/10/02)",
  CellTags->"B0Real"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[33]=  (16:45:39 on 9/10/02)"]
}, Closed]],

Cell["\<\
With B0Real\[Rule]True, transformation is done.\
\>", "Text",
  CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"s", ",", "0", ",", "s", ",", 
        RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[34]:=  (16:45:54 on 9/10/02)",
  CellTags->"B0Real"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", 
        "2"}], TraditionalForm]], "Output",
  CellLabel->"Out[34]=  (16:45:54 on 9/10/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B0Unique", "ObjectName",
  PageBreakAbove->True,
  CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B0Unique"],

Cell["\<\
B0Unique is an option of B0. If set to True, B0[0,0,m2] is replaced \
with (B0[0,m2,m2]+1) and B0[m2,0,m2] simplifies to (B0[0,m2,m2]+2).\
\>", \
"Text",
  CellTags->"B0Unique"],

Cell[TextData[{
  "See also: ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  "."
}], "Text",
  CellTags->"B0Unique"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B0Unique"],

Cell["By default transformation is done.", "Text",
  CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "0", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[36]:=  (16:48:41 on 9/10/02)",
  CellTags->"B0Unique"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", 
        "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[36]=  (16:48:41 on 9/10/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "//", "Options"}]], "Input",
  CellLabel->"In[37]:=  (16:48:46 on 9/10/02)",
  CellTags->"B0Unique"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[37]=  (16:48:46 on 9/10/02)"]
}, Closed]],

Cell["\<\
With B0Real\[Rule]False, nothing happens.\
\>", "Text",
  CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "0", ",", "s", ",", 
        RowBox[{"B0Unique", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[38]:=  (16:49:23 on 9/10/02)",
  CellTags->"B0Unique"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", "0", ",", "s", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[38]=  (16:49:23 on 9/10/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B00", "ObjectName",
  CellTags->"B00"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B00"],

Cell[TextData[{
  "B00[pp, ma^2,mb^2] is the Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "00"], TraditionalForm]]],
  "-function, i.e., the coefficient function of the metric tensor. All \
arguments are scalars and have dimension mass^2."
}], "Text",
  CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B00", "]"}]], "Input",
  CellLabel->"In[128]:=  (21:52:02 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[128]=  (21:52:02 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  "."
}], "Text",
  CellTags->"B00"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B00"],

Cell[TextData[{
  "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e., ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "2"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", "p", "]"}]], "Input",
  CellLabel->"In[129]:=  (21:52:02 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[129]=  (21:52:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input",
  CellLabel->"In[130]:=  (21:52:02 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["m", "2"]}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["M", "2"]}], "-", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["M", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox["M", "2"]}], "+", 
              SuperscriptBox["M", "2"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            SuperscriptBox["M", "2"], "-", 
                            SuperscriptBox["m", "2"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SubscriptBox["B", "0"], "(", 
                              FormBox[
                                SuperscriptBox[
                                  FormBox["p",
                                    "TraditionalForm"], "2"],
                                "TraditionalForm"], ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}], "-", 
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "0", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                      RowBox[{"2", " ", 
                        FormBox[
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"],
                          "TraditionalForm"]}]], "-", 
                    RowBox[{
                      FractionBox["1", "2"], " ", 
                      RowBox[{
                        SubscriptBox["B", "0"], "(", 
                        FormBox[
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"],
                          "TraditionalForm"], ",", 
                        SuperscriptBox["m", "2"], ",", 
                        SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox["M", "2"], "+", 
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[130]=  (21:52:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[131]:=  (21:52:02 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"6", " ", 
                SuperscriptBox["m", "2"]}], "-", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"], "-", 
              RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[131]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[132]:=  (21:52:03 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"B\"\>", "\<\"00\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["M", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[132]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{"0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[133]:=  (21:52:03 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FractionBox[
          SuperscriptBox["m", "2"], "3"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[133]=  (21:52:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SuperscriptBox["M", "2"], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[134]:=  (21:52:03 on 5/16/01)",
  CellTags->"B00"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            SuperscriptBox["M", "2"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FractionBox[
          SuperscriptBox["m", "2"], "3"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[134]=  (21:52:03 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B1", "ObjectName",
  CellTags->"B1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B1"],

Cell[TextData[{
  "B1[pp, ma^2, mb^2] the Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "1"], TraditionalForm]]],
  "-function. All arguments are scalars and have dimension mass^2."
}], "Text",
  CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B1", "]"}]], "Input",
  CellLabel->"In[150]:=  (21:52:05 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[150]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B11",
    ButtonData:>"B11",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B11"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text",
  CellTags->"B1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input",
  CellLabel->"In[151]:=  (21:52:05 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["M", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], "-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
          RowBox[{"2", " ", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"]}]], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[151]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[152]:=  (21:52:05 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "1"], "(", 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[152]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"B1", ",", 
          RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[153]:=  (21:52:05 on 5/16/01)",
  CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[154]:=  (21:52:05 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", 
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[154]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ",", "0"}], "]"}]], "Input",
  CellLabel->"In[155]:=  (21:52:06 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}]}], "-", "2"}], ")"}]}], "-",
         
        FractionBox["1", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[155]=  (21:52:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{"0", ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[156]:=  (21:52:06 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}]}], "-", "1"}], ")"}]}], "+",
         
        FractionBox["1", "4"]}], TraditionalForm]], "Output",
  CellLabel->"Out[156]=  (21:52:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{"pp", ",", 
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
        SubsuperscriptBox["m", "2", "2"]}], "]"}]], "Input",
  CellLabel->"In[157]:=  (21:52:06 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "pp", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}], " ", 
            SubsuperscriptBox["m", "2", "2"]}], 
          RowBox[{"2", " ", "pp"}]], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "pp", ",", 
            SubsuperscriptBox["m", "e", "2"], ",", 
            SubsuperscriptBox["m", "2", "2"], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[157]=  (21:52:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", "0"}], "]"}]], "Input",\

  CellLabel->"In[158]:=  (21:52:06 on 5/16/01)",
  CellTags->"B1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "e", "2"], ")"}]}], "-", "2"}], 
            ")"}]}], "-", 
        FractionBox["1", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[158]=  (21:52:06 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B11", "ObjectName",
  CellTags->"B11"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"B11"],

Cell[TextData[{
  "B11[pp, ma^2, mb^2] is the Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "11"], TraditionalForm]]],
  "-function, i.e., the coefficient function of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], " ", 
          SuperscriptBox["p", "\[Nu]"]}], TraditionalForm]]],
  ". All arguments are scalars and have dimension mass",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Null]", "2"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B11", "]"}]], "Input",
  CellLabel->"In[141]:=  (21:52:04 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[141]=  (21:52:04 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  "."
}], "Text",
  CellTags->"B11"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"B11"],

Cell[TextData[{
  "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e. ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "2"], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", "p", "]"}]], "Input",
  CellLabel->"In[142]:=  (21:52:04 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[142]=  (21:52:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input",
  CellLabel->"In[143]:=  (21:52:04 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          RowBox[{"3", " ", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"]}]], 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            SuperscriptBox["M", "2"], "+", 
            RowBox[{
              SuperscriptBox["M", "2"], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "0", ",", 
                SuperscriptBox["M", "2"], ",", 
                SuperscriptBox["M", "2"], ")"}]}], "+", 
            RowBox[{
              FractionBox["1", "6"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "3"}], " ", 
                    SuperscriptBox["m", "2"]}], "-", 
                  RowBox[{"3", " ", 
                    SuperscriptBox["M", "2"]}], "+", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"]}], ")"}]}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          SuperscriptBox["M", "2"], "-", 
                          SuperscriptBox["m", "2"]}], ")"}], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            SubscriptBox["B", "0"], "(", 
                            FormBox[
                              SuperscriptBox[
                                FormBox["p",
                                  "TraditionalForm"], "2"],
                              "TraditionalForm"], ",", 
                            SuperscriptBox["m", "2"], ",", 
                            SuperscriptBox["M", "2"], ")"}], "-", 
                          RowBox[{
                            SubscriptBox["B", "0"], "(", "0", ",", 
                            SuperscriptBox["m", "2"], ",", 
                            SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                    RowBox[{"2", " ", 
                      FormBox[
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"],
                        "TraditionalForm"]}]], "-", 
                  RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{
                      SubscriptBox["B", "0"], "(", 
                      FormBox[
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"],
                        "TraditionalForm"], ",", 
                      SuperscriptBox["m", "2"], ",", 
                      SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["m", "2"], "-", 
                  SuperscriptBox["M", "2"], "+", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"]}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[143]=  (21:52:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[144]:=  (21:52:04 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}], " ", 
            SuperscriptBox["m", "2"]}], "+", 
          SuperscriptBox["m", "2"], "+", 
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "-", 
                RowBox[{"6", " ", 
                  SuperscriptBox["m", "2"]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{
              SubscriptBox["B", "0"], "(", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]}], 
        RowBox[{"3", " ", 
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[144]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"B11", ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[145]:=  (21:52:05 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[145]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input",
  CellLabel->"In[146]:=  (21:52:05 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"B\"\>", "\<\"11\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["M", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[146]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"B11", ",", 
        RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[147]:=  (21:52:05 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[147]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{"0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[148]:=  (21:52:05 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "3"], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[148]=  (21:52:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SuperscriptBox["M", "2"], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input",
  CellLabel->"In[149]:=  (21:52:05 on 5/16/01)",
  CellTags->"B11"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "3"], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", 
          SuperscriptBox["M", "2"], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[149]=  (21:52:05 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CA", "ObjectName",
  CellTags->"CA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CA"],

Cell[TextData[{
  "CA is one of the Casimir operator eigenvalues of SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") (CA = ",
  StyleBox["N",
    FontSlant->"Italic"],
  ")."
}], "Text",
  CellTags->"CA"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"CA"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CA"],

Cell[CellGroupData[{

Cell[BoxData["CA"], "Input",
  CellLabel->"In[173]:=  (21:52:09 on 5/16/01)",
  CellTags->"CA"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "A"], TraditionalForm]], "Output",
  CellLabel->"Out[173]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CA", ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[174]:=  (21:52:09 on 5/16/01)",
  CellTags->"CA"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output",
  CellLabel->"Out[174]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
  CellLabel->"In[175]:=  (21:52:09 on 5/16/01)",
  CellTags->"CA"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output",
  CellLabel->"Out[175]=  (21:52:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Calc", "ObjectName",
  CellTags->"Calc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Calc"],

Cell["\<\
Calc[exp] performs several basic simplifications. Calc[exp] is the \
same as DotSimplify[DiracSimplify[Contract[DiracSimplify[Explicit[ \
SUNSimplify[Trick[exp], Explicit\[Rule]False] ]]]]].\
\>", "Text",
  CellTags->"Calc"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  "."
}], "Text",
  CellTags->"Calc"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"Calc"],

Cell[TextData[{
  "This calculates ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
  " in 4 dimensions and ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
  " in D dimensions."
}], "Text",
  CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}], ",", " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
  CellLabel->"In[167]:=  (21:52:07 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", "D"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[167]=  (21:52:08 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This simplifies ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["f", "abc"], " ", 
          RowBox[{
            SubscriptBox["f", "abe"], "."}]}], TraditionalForm]]]
}], "Text",
  CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "e"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[168]:=  (21:52:08 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[168]=  (21:52:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[169]:=  (21:52:08 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"q", "-", "p"}],
                "TraditionalForm"], ")"}], "\[Nu]"], ")"}], " ", 
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"p", "+", "r"}],
                "TraditionalForm"], ")"}], "\[Mu]"], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[169]=  (21:52:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[170]:=  (21:52:08 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[170]=  (21:52:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", 
        RowBox[{
          RowBox[{"-", "p"}], "-", "q"}], ",", "3"}], "]"}]], "Input",
  CellLabel->"In[171]:=  (21:52:08 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li3",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox[
              RowBox[{
                RowBox[{"-", "p"}], "-", "q"}],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[171]=  (21:52:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{"%", " ", 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "li1"}], "]"}]}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"q", ",", "li2"}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "p"}], "-", "q"}], ",", "li3"}], "]"}]}], 
      "]"}]], "Input",
  CellLabel->"In[172]:=  (21:52:08 on 5/16/01)",
  CellTags->"Calc"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[172]=  (21:52:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CalculateCounterTerm ***unfinished***", "ObjectName",
  CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CalculateCounterTerm"],

Cell["CalculateCounterTerm[exp, k] calculates the residue of exp.", "Text",
  CellTags->"CalculateCounterTerm"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "CalculateCounterTerm"}]], "Input",
  CellLabel->"In[3]:=  (15:41:09 on 9/17/02)",
  CellTags->"CalculateCounterTerm"],

Cell[BoxData["\<\"CalculateCounterTerm[exp, k] calculates the residue of exp.\
\"\>"], "Print",
  CellTags->"Info3241266069-5635180"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CancelQP", "ObjectName",
  CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CancelQP"],

Cell[TextData[{
  "CancelQP is an option for OneLoop. If set to True, cancelation of all ",
  StyleBox["q",
    FontSlant->"Italic"],
  ".",
  StyleBox["p",
    FontSlant->"Italic"],
  "'s and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["q", "2"], TraditionalForm]]],
  " is performed."
}], "Text",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{
              RowBox[{"p", "+", 
                RowBox[{"2", "q2"}]}], ",", 
              RowBox[{"p", "+", "q2"}]}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q2", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q2", ",", "M"}], "}"}]}], "]"}]}], "//", 
        "FCI"}]}]], "Input",
  CellLabel->"In[41]:=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"(", "\[NoBreak]", 
          FormBox[
            RowBox[{"p", "+", "q2"}],
            "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
          "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            RowBox[{"p", "+", 
              RowBox[{"2", " ", "q2"}]}],
            "TraditionalForm"], "\[NoBreak]", ")"}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox["q2", "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q2", "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[41]=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q2", ",", "t", ",", 
        RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[42]:=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox[
              RowBox[{"3", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "-", 
            FractionBox[
              RowBox[{"3", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
                    "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox["M", "2"],
                    "TraditionalForm"], "\[NoBreak]", ")"}]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"3", " ", 
                      SuperscriptBox["m", "2"]}], "-", 
                    RowBox[{"7", " ", 
                      SuperscriptBox["M", "2"]}]}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "-", 
            FractionBox[
              RowBox[{"5", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "+", 
            FractionBox[
              RowBox[{"5", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[42]=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q2", ",", "t", ",", 
        RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[43]:=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                RowBox[{"3", " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", "0", ",", 
                    SuperscriptBox["m", "2"], ")"}], " ", 
                  SuperscriptBox["m", "2"]}], 
                RowBox[{"2", " ", 
                  SuperscriptBox["M", "2"]}]]}], "+", 
            FractionBox[
              RowBox[{"7", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "+", 
            FractionBox[
              RowBox[{"3", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "-", 
            FractionBox[
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
                    "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox["m", "2"],
                    "TraditionalForm"], "\[NoBreak]", ")"}]}], 
              SuperscriptBox["M", "2"]], "+", 
            FractionBox[
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
                    "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox["M", "2"],
                    "TraditionalForm"], "\[NoBreak]", ")"}]}], 
              SuperscriptBox["M", "2"]], "-", 
            RowBox[{
              FractionBox["7", "2"], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "0", ",", 
                SuperscriptBox["m", "2"], ",", 
                SuperscriptBox["M", "2"], ")"}]}], "-", 
            FractionBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"3", " ", 
                      SuperscriptBox["m", "2"]}], "-", 
                    RowBox[{"7", " ", 
                      SuperscriptBox["M", "2"]}]}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "-", 
            FractionBox[
              RowBox[{"5", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]], "+", 
            FractionBox[
              RowBox[{"5", " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox["M", "2"]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[43]=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"q2", ",", "p"}], "]"}], 
          RowBox[{"SPD", "[", 
            RowBox[{"q1", ",", "p"}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q1", ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q2", ",", "m"}], "}"}], ",", 
              RowBox[{"q1", "-", "p"}], ",", 
              RowBox[{"q2", "-", "p"}], ",", 
              RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",\

  CellLabel->"In[44]:=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q1",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q2",
              "TraditionalForm"]}]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q1", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q2", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[44]=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q2", ",", "t", ",", 
        RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[45]:=  (17:10:05 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    FormBox[
                      SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                      "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                    FormBox[
                      SuperscriptBox[
                        FormBox["q1",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"], "-", 
                        RowBox[{"2", " ", 
                          RowBox[{
                            FormBox["p",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q1",
                              "TraditionalForm"]}]}], "+", 
                        SuperscriptBox[
                          FormBox["q1",
                            "TraditionalForm"], "2"]}],
                      "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                    FormBox["0",
                      "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                    FormBox[
                      SuperscriptBox["m", "2"],
                      "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                    FormBox["0",
                      "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q1",
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}], 
                RowBox[{"2", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["m", "2"], "-", 
                      SuperscriptBox[
                        FormBox["q1",
                          "TraditionalForm"], "2"]}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q1",
                            "TraditionalForm"]}]}], "+", 
                      SuperscriptBox[
                        FormBox["q1",
                          "TraditionalForm"], "2"]}], ")"}]}]]}], "+", 
            FractionBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["q1",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ",", "0", ",", "0", ")"}],
                 " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{
                  FormBox[
                    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                    "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q1",
                            "TraditionalForm"]}]}], "+", 
                      SuperscriptBox[
                        FormBox["q1",
                          "TraditionalForm"], "2"]}],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox["0",
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox["m", "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox["0",
                    "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[45]=  (17:10:06 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q2", ",", "t", ",", 
        RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[46]:=  (17:10:06 on 9/17/02)",
  CellTags->"CancelQP"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox[
              RowBox[{
                RowBox[{"PaVe", "(", 
                  RowBox[{"2", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"], ",", 
                        SuperscriptBox[
                          FormBox["q1",
                            "TraditionalForm"], "2"], ",", 
                        RowBox[{
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"], "-", 
                          RowBox[{"2", " ", 
                            RowBox[{
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q1",
                                "TraditionalForm"]}]}], "+", 
                          SuperscriptBox[
                            FormBox["q1",
                              "TraditionalForm"], "2"]}]}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"0", ",", 
                        SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], 
                " ", 
                SuperscriptBox[
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q1",
                        "TraditionalForm"]}],
                    "TraditionalForm"], "2"]}], 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]], "-", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                RowBox[{"PaVe", "(", 
                  RowBox[{"1", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"], ",", 
                        SuperscriptBox[
                          FormBox["q1",
                            "TraditionalForm"], "2"], ",", 
                        RowBox[{
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"], "-", 
                          RowBox[{"2", " ", 
                            RowBox[{
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q1",
                                "TraditionalForm"]}]}], "+", 
                          SuperscriptBox[
                            FormBox["q1",
                              "TraditionalForm"], "2"]}]}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"0", ",", 
                        SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], 
                " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]], "-", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                RowBox[{"PaVe", "(", 
                  RowBox[{"2", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"], ",", 
                        SuperscriptBox[
                          FormBox["q1",
                            "TraditionalForm"], "2"], ",", 
                        RowBox[{
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"], "-", 
                          RowBox[{"2", " ", 
                            RowBox[{
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q1",
                                "TraditionalForm"]}]}], "+", 
                          SuperscriptBox[
                            FormBox["q1",
                              "TraditionalForm"], "2"]}]}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"0", ",", 
                        SuperscriptBox["m", "2"], ",", "0"}], "}"}]}], ")"}], 
                " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{
                  FormBox[
                    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                    "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q1",
                            "TraditionalForm"]}]}], "+", 
                      SuperscriptBox[
                        FormBox["q1",
                          "TraditionalForm"], "2"]}],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox["0",
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox[
                    SuperscriptBox["m", "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                  FormBox["0",
                    "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q1",
                    "TraditionalForm"]}]}], 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], "-", 
                    RowBox[{"2", " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q1",
                          "TraditionalForm"]}]}], "+", 
                    SuperscriptBox[
                      FormBox["q1",
                        "TraditionalForm"], "2"]}], ")"}]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[46]=  (17:10:06 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[47]:=  (17:10:06 on 9/17/02)",
  CellTags->"CancelQP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Cases2", "ObjectName",
  CellTags->"Cases2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Cases2"],

Cell["\<\
Cases2[expr, f] returns a list of all objects in expr with head f. \
Cases2[expr,f] is equivalent to Cases2[{expr},f[___],Infinity]//Union. \
Cases2[expr, f, g, ...] or Cases2[expr, {f,g, ...}] is equivalent to \
Cases[{expr},f[___] | g[___] ...] .\
\>", "Text",
  CellTags->"Cases2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"Cases2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "a", "]"}], "+", 
          SuperscriptBox[
            RowBox[{"f", "[", "b", "]"}], "2"], "+", 
          RowBox[{"f", "[", 
            RowBox[{"c", ",", "d"}], "]"}]}], ",", "f"}], "]"}]], "Input",
  CellLabel->"In[176]:=  (21:52:09 on 5/16/01)",
  CellTags->"Cases2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"f", "(", "a", ")"}], ",", 
          RowBox[{"f", "(", "b", ")"}], ",", 
          RowBox[{"f", "(", 
            RowBox[{"c", ",", "d"}], ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[176]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Sin", "[", "x", "]"}], " ", 
            RowBox[{"Sin", "[", 
              RowBox[{"y", "-", "z"}], "]"}]}], "+", 
          RowBox[{"g", "[", "y", "]"}]}], ",", "Sin", ",", "g"}], 
      "]"}]], "Input",
  CellLabel->"In[177]:=  (21:52:09 on 5/16/01)",
  CellTags->"Cases2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"g", "(", "y", ")"}], ",", 
          RowBox[{"sin", "(", "x", ")"}], ",", 
          RowBox[{"sin", "(", 
            RowBox[{"y", "-", "z"}], ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[177]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Sin", "[", "x", "]"}], " ", 
            RowBox[{"Sin", "[", 
              RowBox[{"y", "-", "z"}], "]"}]}], "+", 
          RowBox[{"g", "[", "x", "]"}], "+", 
          RowBox[{"g", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"Sin", ",", "g"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[178]:=  (21:52:09 on 5/16/01)",
  CellTags->"Cases2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"g", "(", "x", ")"}], ",", 
          RowBox[{"g", "(", 
            RowBox[{"a", ",", "b", ",", "c"}], ")"}], ",", 
          RowBox[{"sin", "(", "x", ")"}], ",", 
          RowBox[{"sin", "(", 
            RowBox[{"y", "-", "z"}], ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[178]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
            RowBox[{"DiracSlash", "[", "q", "]"}]}], "+", 
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p", ",", "p"}], "]"}]}], ",", "Dot"}], "]"}]], "Input",
  CellLabel->"In[179]:=  (21:52:09 on 5/16/01)",
  CellTags->"Cases2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[179]=  (21:52:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CF", "ObjectName",
  CellTags->"CF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CF"],

Cell[TextData[{
  "CF is one of the Casimir operator eigenvalues of SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") (CF = (",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["N", "2"], TraditionalForm]]],
  "-1)/(2 ",
  StyleBox["N",
    FontSlant->"Italic"],
  "))."
}], "Text",
  CellTags->"CF"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"CF"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CF"],

Cell[CellGroupData[{

Cell[BoxData["CF"], "Input",
  CellLabel->"In[180]:=  (21:52:09 on 5/16/01)",
  CellTags->"CF"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output",
  CellLabel->"Out[180]=  (21:52:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CF", ",", " ", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[181]:=  (21:52:10 on 5/16/01)",
  CellTags->"CF"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["N", "2"], "-", "1"}], 
        RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
  CellLabel->"Out[181]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
  CellLabel->"In[182]:=  (21:52:10 on 5/16/01)",
  CellTags->"CF"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output",
  CellLabel->"Out[182]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CF", ",", " ", 
        RowBox[{"SUNNToCACF", " ", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[183]:=  (21:52:10 on 5/16/01)",
  CellTags->"CF"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output",
  CellLabel->"Out[183]=  (21:52:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ChangeDimension", "ObjectName",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ChangeDimension"],

Cell["\<\
ChangeDimension[exp, dim] changes all LorentzIndex and Momenta in \
exp to dimension dim (and also Levi-Civita-tensors, Dirac slashes and Dirac \
matrices).\
\>", "Text",
  CellTags->"ChangeDimension"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Momentum"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  "."
}], "Text",
  CellTags->"ChangeDimension"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ChangeDimension"],

Cell["\<\
Remember that LorentzIndex[mu, 4] is simplified to LorentzIndex[mu] \
and Momentum[p, 4] to Momentum[p]. Thus the fullowing objects are defined in \
four dimensions.\
\>", "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", " ", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Input",
  CellLabel->"In[184]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox["\[Mu]",
            "TraditionalForm"], ",", "p"}], "}"}], TraditionalForm]], "Output",\

  CellLabel->"Out[184]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChangeDimension", "[", 
      RowBox[{"%", ",", " ", "D"}], "]"}]], "Input",
  CellLabel->"In[185]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox["\[Mu]",
            "TraditionalForm"], ",", "p"}], "}"}], TraditionalForm]], "Output",\

  CellLabel->"Out[185]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[186]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[186]//StandardForm=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell["\<\
This changes all non-4-dimensional objects to 4-dimensional ones.\
\
\>", "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"%%", ",", " ", "4"}], "]"}], " ", "//", " ", 
      "StandardForm"}]], "Input",
  CellLabel->"In[187]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Output",
  CellLabel->"Out[187]//StandardForm=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell["Consider the following list of 4- and D-dimensional object.", "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"MT", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
        RowBox[{
          RowBox[{"GAD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"f", "[", "D", "]"}]}]}], "}"}]], "Input",
  CellLabel->"In[188]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
            RowBox[{"f", "(", "D", ")"}], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[188]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", 
      RowBox[{"Contract", "[", "%", "]"}], "]"}]], "Input",
  CellLabel->"In[189]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", 
          RowBox[{"D", " ", 
            RowBox[{"f", "(", "D", ")"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[189]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", 
      RowBox[{"Contract", "[", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%%", ",", "n"}], "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[190]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"n", ",", 
          RowBox[{"n", " ", 
            RowBox[{"f", "(", "D", ")"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[190]=  (21:52:10 on 5/16/01)"]
}, Closed]],

Cell["\<\
Any explicit occurence of D (like in f(D)) is not replaced by \
ChangeDimension.\
\>", "Text",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[191]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell["\<\
The option Dimension of Eps must be changed too, since with the \
default setting Dimension\[Rule]4 the arguments of Eps are automatically \
changed to 4 dimensions.\
\>", "Text",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[192]:=  (21:52:10 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", " ", 
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[193]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[193]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"a1", ",", "4"}], "]"}]}]], "Input",
  CellLabel->"In[194]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[194]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Contract", "[", 
        SuperscriptBox["a1", "2"], "]"}], "]"}]], "Input",
  CellLabel->"In[195]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[195]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      SuperscriptBox["a2", "2"], "]"}]], "Input",
  CellLabel->"In[196]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output",
  CellLabel->"Out[196]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[197]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"a1", ",", "a2"}], "]"}], ";"}]], "Input",
  CellLabel->"In[198]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChangeDimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ChargeConjugationMatrix", "ObjectName",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
  "ChargeConjugationMatrix denotes the charge conjugation matrix ",
  StyleBox["C",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChargeConjugationMatrixInv",
    ButtonData:>"ChargeConjugationMatrixInv",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ChargeConjugationMatrixInv"],
  "."
}], "Text",
  CellTags->"ChargeConjugationMatrix"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input",
  CellLabel->"In[206]:=  (21:52:13 on 5/16/01)",
  CellTags->"ChargeConjugationMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[206]=  (21:52:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[207]:=  (21:52:13 on 5/16/01)",
  CellTags->"ChargeConjugationMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "T"]}], TraditionalForm]], "Output",
  CellLabel->"Out[207]=  (21:52:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input",
  CellLabel->"In[208]:=  (21:52:13 on 5/16/01)",
  CellTags->"ChargeConjugationMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[208]=  (21:52:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[209]:=  (21:52:13 on 5/16/01)",
  CellTags->"ChargeConjugationMatrix"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox["5",
          "TraditionalForm"], "T"], TraditionalForm]], "Output",
  CellLabel->"Out[209]=  (21:52:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", "^", "2"}]], "Input",
  CellLabel->"In[210]:=  (21:52:14 on 5/16/01)",
  CellTags->"ChargeConjugationMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[210]=  (21:52:14 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ChargeConjugationMatrixInv", "ObjectName",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ChargeConjugationMatrixInv"],

Cell["\<\
ChargeConjugationMatrixInv is the inverse of \
ChargeConjugationMatrix. It is substituted immediately by \
-ChargeConjugationMatrix.\
\>", "Text",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChargeConjugationMatrix",
    ButtonData:>"ChargeConjugationMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ChargeConjugationMatrix"],
  "."
}], "Text",
  CellTags->"ChargeConjugationMatrixInv"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input",
  CellLabel->"In[199]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[199]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[200]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "T"]}], TraditionalForm]], "Output",
  CellLabel->"Out[200]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input",
  CellLabel->"In[201]:=  (21:52:11 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[201]=  (21:52:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[202]:=  (21:52:12 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox["5",
          "TraditionalForm"], "T"], TraditionalForm]], "Output",
  CellLabel->"Out[202]=  (21:52:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
    "ChargeConjugationMatrix", ".", "ChargeConjugationMatrixInv"}]], "Input",
  CellLabel->"In[203]:=  (21:52:12 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[203]=  (21:52:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "Calc"}]], "Input",
  CellLabel->"In[204]:=  (21:52:12 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[204]=  (21:52:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrixInv", "^", "2"}]], "Input",
  CellLabel->"In[205]:=  (21:52:12 on 5/16/01)",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[205]=  (21:52:12 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CheckContext", "ObjectName",
  CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CheckContext"],

Cell["\<\
CheckContext[string] yields True if the packaged associated with \
string is already loaded, and False otherwise.\
\>", "Text",
  CellTags->"CheckContext"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["MakeContext",
    ButtonData:>"MakeContext",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MakeContext"],
  ", ",
  ButtonBox["$FeynCalcStuff",
    ButtonData:>"$FeynCalcStuff",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$FeynCalcStuff"],
  "."
}], "Text",
  CellTags->"CheckContext"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CheckContext"],

Cell["\<\
If a FeynCalc symbol has not been used, the corresponding \
subpackage has not been loaded:\
\>", "Text",
  CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
  CellLabel->"In[2]:=  (17:13:49 on 9/17/02)",
  CellTags->"CheckContext"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[2]=  (17:13:50 on 9/17/02)"]
}, Closed]],

Cell["Using it makes FeynCalc load the subpackage:", "Text",
  CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData["CompleteSquare"], "Input",
  CellLabel->"In[3]:=  (17:13:55 on 9/17/02)",
  CellTags->"CheckContext"],

Cell[BoxData[
    FormBox["CompleteSquare", TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (17:13:55 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
  CellLabel->"In[4]:=  (17:13:59 on 9/17/02)",
  CellTags->"CheckContext"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (17:13:59 on 9/17/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ChiralityProjector", "ObjectName",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ChiralityProjector"],

Cell[TextData[{
  "ChiralityProjector[+1] denotes ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
  ". ChiralityProjector[-1] denotes ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"ChiralityProjector"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  "."
}], "Text",
  CellTags->"ChiralityProjector"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"ChiralityProjector", "[", 
          RowBox[{"+", "1"}], "]"}], ",", 
        RowBox[{"ChiralityProjector", "[", 
          RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[211]:=  (21:52:14 on 5/16/01)",
  CellTags->"ChiralityProjector"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "6"], ",", 
          SuperscriptBox["\[Gamma]", "7"]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[211]=  (21:52:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCI", "[", "%", "]"}]], "Input",
  CellLabel->"In[212]:=  (21:52:14 on 5/16/01)",
  CellTags->"ChiralityProjector"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "6"], ",", 
          SuperscriptBox["\[Gamma]", "7"]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[212]=  (21:52:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{"%", ",", 
        RowBox[{"DiracSubstitute67", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[213]:=  (21:52:15 on 5/16/01)",
  CellTags->"ChiralityProjector"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FractionBox[
              SuperscriptBox["\[Gamma]", "5"], "2"], "+", 
            FractionBox["1", "2"]}], ",", 
          RowBox[{
            FractionBox["1", "2"], "-", 
            FractionBox[
              SuperscriptBox["\[Gamma]", "5"], "2"]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[213]=  (21:52:15 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Chisholm", "ObjectName",
  CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Chisholm"],

Cell["\<\
Chisholm[x] substitutes products of three Dirac matrices or slashes \
by the Chisholm identity.\
\>", "Text",
  CellTags->"Chisholm"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["EpsChisholm",
    ButtonData:>"EpsChisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"EpsChisholm"],
  ", ",
  ButtonBox["ChisholmSpinor",
    ButtonData:>"ChisholmSpinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ChisholmSpinor"],
  "."
}], "Text",
  CellTags->"Chisholm"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Mu]", ",", "\[Nu]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[214]:=  (21:52:15 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$42",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$42",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Mu]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Mu]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[214]=  (21:52:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
  CellLabel->"In[215]:=  (21:52:16 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[215]=  (21:52:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", 
      RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
  CellLabel->"In[216]:=  (21:52:16 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$43",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$43",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[216]=  (21:52:17 on 5/16/01)"]
}, Closed]],

Cell["The $MU$ variables are unique indices.", "Text",
  CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t1", ".", "t1"}], "]"}]], "Input",
  CellLabel->"In[217]:=  (21:52:17 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox["16", TraditionalForm]], "Output",
  CellLabel->"Out[217]=  (21:52:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", " ", "=", 
      RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
  CellLabel->"In[218]:=  (21:52:17 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$44",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$44",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[218]=  (21:52:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t2", ".", "t3"}], "]"}]], "Input",
  CellLabel->"In[219]:=  (21:52:18 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox["16", TraditionalForm]], "Output",
  CellLabel->"Out[219]=  (21:52:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"DiracSlash", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]], "Input",
  CellLabel->"In[220]:=  (21:52:19 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["a",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["b",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["c",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[220]=  (21:52:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", "t4", "]"}]], "Input",
  CellLabel->"In[221]:=  (21:52:19 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$45",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["$MU$45",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["a",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["b",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["c",
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["c",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["a",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["b",
              "TraditionalForm"]}]}], "-", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["b",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["a",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["c",
              "TraditionalForm"]}]}], "+", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["a",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["b",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["c",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[221]=  (21:52:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", 
          ",", "\[Kappa]"}], "]"}]}]], "Input",
  CellLabel->"In[222]:=  (21:52:20 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Tau]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Kappa]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[222]=  (21:52:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", 
      RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
  CellLabel->"In[223]:=  (21:52:20 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$48",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$48",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$49",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$49",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$50",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$50",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$46",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$46",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[223]=  (21:52:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", " ", "=", 
      RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
  CellLabel->"In[224]:=  (21:52:23 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$53",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$53",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$54",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$54",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$55",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$55",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$51",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$51",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[224]=  (21:52:26 on 5/16/01)"]
}, Closed]],

Cell["Check that both a1.a1 and a2.a3 give the same.", "Text",
  CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"a1", ".", "a1"}], "]"}]], "Input",
  CellLabel->"In[225]:=  (21:52:26 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "2048"}], TraditionalForm]], "Output",
  CellLabel->"Out[225]=  (21:52:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"a2", ".", "a3"}], "]"}]], "Input",
  CellLabel->"In[226]:=  (21:52:26 on 5/16/01)",
  CellTags->"Chisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "2048"}], TraditionalForm]], "Output",
  CellLabel->"Out[226]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "a1", ",", "a2", ",", 
        "a3"}], "]"}]], "Input",
  CellLabel->"In[227]:=  (21:52:35 on 5/16/01)",
  CellTags->"Chisholm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ChisholmSpinor", "ObjectName",
  CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ChisholmSpinor"],

Cell["\<\
ChisholmSpinor[x] uses the Chisholm identity on a DiraGamma between \
spinors. As an optional second argument 1 or 2 may be given, indicating that \
ChisholmSpinor should only act on the first resp. second part of a product of \
spinor chains.\
\>", "Text",
  CellTags->"ChisholmSpinor"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["EpsChisholm",
    ButtonData:>"EpsChisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"EpsChisholm"],
  ", ",
  ButtonBox["Chisholm",
    ButtonData:>"Chisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Chisholm"],
  "."
}], "Text",
  CellTags->"ChisholmSpinor"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{"p1", ",", "m1"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ".", 
      RowBox[{"Spinor", "[", 
        RowBox[{"p2", ",", "m2"}], "]"}]}]], "Input",
  CellLabel->"In[4]:=  (20:37:32 on 9/2/02)",
  CellTags->"ChisholmSpinor"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            SubscriptBox[
              StyleBox["\<\"p\"\>",
                FontSlant->"Italic"], "1"],
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m1", 
          "\[NoBreak]", ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            SubscriptBox[
              StyleBox["\<\"p\"\>",
                FontSlant->"Italic"], "2"],
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m2", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (20:37:32 on 9/2/02)",
  CellTags->"ChisholmSpinor"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "ChisholmSpinor"}]], "Input",
  CellLabel->"In[5]:=  (20:37:32 on 9/2/02)",
  CellTags->"ChisholmSpinor"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"m1", " ", "m2", " ", 
              RowBox[{
                RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m1",
                   "\[NoBreak]", ")"}], ".", 
                SuperscriptBox["\[Gamma]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"p\"\>",
                        FontSlant->"Italic"], "2"],
                    "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m2",
                   "\[NoBreak]", ")"}]}]}], 
            RowBox[{
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"], "2"],
                "TraditionalForm"]}]]}], "+", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            RowBox[{
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m1", 
                "\[NoBreak]", ")"}], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["alpha1",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", "5"], ".", 
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "2"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m2", 
                "\[NoBreak]", ")"}]}], " ", 
            SuperscriptBox["\[Epsilon]", 
              RowBox[{
                FormBox[
                  FormBox["alpha1",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "2"],
                  "TraditionalForm"]}]]}], 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]], "+", 
        FractionBox[
          RowBox[{"m2", " ", 
            RowBox[{
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m1", 
                "\[NoBreak]", ")"}], ".", 
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "2"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m2", 
                "\[NoBreak]", ")"}]}], " ", 
            SubsuperscriptBox[
              StyleBox["\<\"p\"\>",
                FontSlant->"Italic"], "1", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]], "+", 
        FractionBox[
          RowBox[{"m1", " ", 
            RowBox[{
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m1", 
                "\[NoBreak]", ")"}], ".", 
              RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"p\"\>",
                      FontSlant->"Italic"], "2"],
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m2", 
                "\[NoBreak]", ")"}]}], " ", 
            SubsuperscriptBox[
              StyleBox["\<\"p\"\>",
                FontSlant->"Italic"], "2", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[5]=  (20:37:33 on 9/2/02)",
  CellTags->"ChisholmSpinor"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ClearScalarProducts", "ObjectName",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ClearScalarProducts"],

Cell["\<\
ClearScalarProducts removes all user-performed specific settings \
for ScalarProduct's.\
\>", "Text",
  CellTags->"ClearScalarProducts"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SPD"],
  "."
}], "Text",
  CellTags->"ClearScalarProducts"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "p"}], "]"}], "=", 
      RowBox[{"m", "^", "2"}]}]], "Input",
  CellLabel->"In[228]:=  (21:52:35 on 5/16/01)",
  CellTags->"ClearScalarProducts"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["m", "2"], TraditionalForm]], "Output",
  CellLabel->"Out[228]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[229]:=  (21:52:35 on 5/16/01)",
  CellTags->"ClearScalarProducts"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["m", "2"], TraditionalForm]], "Output",
  CellLabel->"Out[229]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[BoxData["ClearScalarProducts"], "Input",
  CellLabel->"In[230]:=  (21:52:35 on 5/16/01)",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[231]:=  (21:52:35 on 5/16/01)",
  CellTags->"ClearScalarProducts"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[231]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[232]:=  (21:52:35 on 5/16/01)",
  CellTags->"ClearScalarProducts"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[232]=  (21:52:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Collecting", "ObjectName",
  CellTags->"Collecting"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Collecting"],

Cell["\<\
Collecting is an option of ScalarProductCancel, Series2, TID and \
related functions. Setting it to True will trigger some kind of collecting of \
the result.\
\>", "Text",
  CellTags->"Collecting"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["ScalarProductCancel",
    ButtonData:>"ScalarProductCancel",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProductCancel"],
  ", ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Series2"],
  ", ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"],
  "."
}], "Text",
  CellTags->"Collecting"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Collect2", "ObjectName",
  CellTags->"Collect2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Collect2"],

Cell["\<\
Collect2[expr, x] collects together terms which are not free of any \
occurrence of x. Collect2[expr, {x1, x2, ...}] (or also Collect2[expr, x1, \
x2, ...]) collects together terms which are not free of any occurrence of x1, \
x2, .... The coefficients are put over a common denominator. If expr is \
expanded before collecting depends on the option Factoring, which may be set \
to Factor, Factor2, or any other function, which is applied to the \
coefficients. If expr is already expanded with respect to x (x1,x2, ...), the \
option Expanding can be set to False.\
\>", "Text",
  CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Collect2", "]"}]], "Input",
  CellLabel->"In[233]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Denominator", "\[Rule]", "False"}], ",", 
          RowBox[{"Dot", "\[Rule]", "False"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"IsolateNames", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[233]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  "."
}], "Text",
  CellTags->"Collect2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"t1", "=", 
          RowBox[{"a", "+", 
            RowBox[{"r", " ", "a"}], "+", 
            RowBox[{
              SuperscriptBox["k", "2"], " ", 
              RowBox[{"f", "[", "a", "]"}]}], "-", 
            RowBox[{"k", " ", 
              RowBox[{"f", "[", "a", "]"}]}], "+", 
            FractionBox["x", "2"], "-", 
            FractionBox["y", "w"]}]}], ",", "a"}], "]"}]], "Input",
  CellLabel->"In[234]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"w", " ", "x"}], "-", 
            RowBox[{"2", " ", "y"}]}], 
          RowBox[{"2", " ", "w"}]], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"]}], ")"}], " ", 
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"], " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[234]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{"t1", ",", "a", ",", 
        RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[235]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox["x", "2"], "-", 
        FractionBox["y", "w"], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], "-", 
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"]}], ")"}], " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[235]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{"t1", ",", "a", ",", 
        RowBox[{"Factoring", "\[Rule]", "Factor"}]}], "]"}]], "Input",
  CellLabel->"In[236]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"w", " ", "x"}], "-", 
            RowBox[{"2", " ", "y"}]}], 
          RowBox[{"2", " ", "w"}]], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "-", "1"}], ")"}], " ", 
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"], " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[236]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"2", " ", "a", " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "a"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"h", "-", "1"}], ")"}]}], "-", 
          RowBox[{
            SuperscriptBox["b", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
          SuperscriptBox["b", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[237]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "h"}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", "e", " ", "a"}], "-", 
        RowBox[{"2", " ", "b", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "h"}], ")"}], " ", "a"}], "+", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[237]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}],
         ",", "a", ",", 
        RowBox[{"IsolateNames", "\[Rule]", "L"}]}], "]"}]], "Input",
  CellLabel->"In[238]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "5"], "-", 
        RowBox[{"5", " ", 
          TagBox[
            RowBox[{"L", "(", "1", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "4"]}], "+", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            TagBox[
              RowBox[{"L", "(", "1", ")"}],
              HoldForm], "2"], " ", 
          SuperscriptBox["a", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            TagBox[
              RowBox[{"L", "(", "1", ")"}],
              HoldForm], "3"], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"5", " ", 
          SuperscriptBox[
            TagBox[
              RowBox[{"L", "(", "1", ")"}],
              HoldForm], "4"], " ", "a"}], "-", 
        SuperscriptBox[
          TagBox[
            RowBox[{"L", "(", "1", ")"}],
            HoldForm], "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[238]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
  CellLabel->"In[239]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "5"], "-", 
        RowBox[{"5", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c", "+", "d"}], ")"}], " ", 
          SuperscriptBox["a", "4"]}], "+", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"], " ", 
          SuperscriptBox["a", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"5", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"], " ", "a"}], "-",
         
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[239]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "L"}], "]"}]], "Input",
  CellLabel->"In[240]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"a", "-", "b", "-", "c"}], ")"}], "3"], "]"}], ",", "a",
         ",", 
        RowBox[{"Factoring", "\[Rule]", "fun"}]}], "]"}]], "Input",
  CellLabel->"In[241]:=  (21:52:35 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"fun", "(", "1", ")"}], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{
          RowBox[{"fun", "(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "3"}], " ", "b"}], "-", 
              RowBox[{"3", " ", "c"}]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{
          RowBox[{"fun", "(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"6", " ", "c", " ", "b"}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["c", "2"]}]}], ")"}], " ", "a"}], "+", 
        RowBox[{"fun", "(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["b", "3"]}], "-", 
            RowBox[{"3", " ", "c", " ", 
              SuperscriptBox["b", "2"]}], "-", 
            RowBox[{"3", " ", 
              SuperscriptBox["c", "2"], " ", "b"}], "-", 
            SuperscriptBox["c", "3"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[241]=  (21:52:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"fun", "\[Rule]", "FactorTerms"}]}]], "Input",
  CellLabel->"In[242]:=  (21:52:36 on 5/16/01)",
  CellTags->"Collect2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "3"], "-", 
        RowBox[{"3", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"3", " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              RowBox[{"2", " ", "c", " ", "b"}], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "-", 
        SuperscriptBox["b", "3"], "-", 
        SuperscriptBox["c", "3"], "-", 
        RowBox[{"3", " ", "b", " ", 
          SuperscriptBox["c", "2"]}], "-", 
        RowBox[{"3", " ", 
          SuperscriptBox["b", "2"], " ", "c"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[242]=  (21:52:36 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Collect3", "ObjectName",
  CellTags->"Collect3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Collect3"],

Cell[TextData[{
  "Collect3[expr, {x, y, ...}] collects terms involving the same powers of \
monomials ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["x",
            FontSlant->"Plain"], 
          SubscriptBox[
            StyleBox["n",
              FontSlant->"Plain"], "1"]], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["y",
            FontSlant->"Plain"], 
          SubscriptBox[
            StyleBox["n",
              FontSlant->"Plain"], "2"]], TraditionalForm]]],
  " ... An option Factor \[Rule] True/False can be  given, which factors the \
coefficients. The option Head (default Plus) determines the applied function \
to the list of monomials  mulitplied by their coefficients."
}], "Text",
  CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Collect3", "]"}]], "Input",
  CellLabel->"In[8]:=  (17:17:16 on 9/17/02)",
  CellTags->"Collect3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factor", "\[Rule]", "False"}], ",", 
          RowBox[{"Head", "\[Rule]", "Plus"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[8]=  (17:17:16 on 9/17/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Collect2"],
  ", ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  "."
}], "Text",
  CellTags->"Collect3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"2", " ", "a", " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "a"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"h", "-", "1"}], ")"}]}], "-", 
          RowBox[{
            SuperscriptBox["b", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
          SuperscriptBox["b", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[3]:=  (17:55:19 on 9/17/02)",
  CellTags->"Collect3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"2", "-", 
              RowBox[{"2", " ", "h"}]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", "e", " ", "a"}], "+", 
        RowBox[{"b", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"2", " ", "h"}], "-", "2"}], ")"}], " ", "a"}], "+", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (17:55:19 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect3", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}],
         ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[10]:=  (17:57:42 on 9/17/02)",
  CellTags->"Collect3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "5"], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "5"}], " ", "b"}], "-", 
              RowBox[{"5", " ", "c"}], "-", 
              RowBox[{"5", " ", "d"}]}], ")"}], " ", 
          SuperscriptBox["a", "4"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"10", " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"20", " ", "c", " ", "b"}], "+", 
              RowBox[{"20", " ", "d", " ", "b"}], "+", 
              RowBox[{"10", " ", 
                SuperscriptBox["c", "2"]}], "+", 
              RowBox[{"10", " ", 
                SuperscriptBox["d", "2"]}], "+", 
              RowBox[{"20", " ", "c", " ", "d"}]}], ")"}], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "10"}], " ", 
                SuperscriptBox["b", "3"]}], "-", 
              RowBox[{"30", " ", "c", " ", 
                SuperscriptBox["b", "2"]}], "-", 
              RowBox[{"30", " ", "d", " ", 
                SuperscriptBox["b", "2"]}], "-", 
              RowBox[{"30", " ", 
                SuperscriptBox["c", "2"], " ", "b"}], "-", 
              RowBox[{"30", " ", 
                SuperscriptBox["d", "2"], " ", "b"}], "-", 
              RowBox[{"60", " ", "c", " ", "d", " ", "b"}], "-", 
              RowBox[{"10", " ", 
                SuperscriptBox["c", "3"]}], "-", 
              RowBox[{"10", " ", 
                SuperscriptBox["d", "3"]}], "-", 
              RowBox[{"30", " ", "c", " ", 
                SuperscriptBox["d", "2"]}], "-", 
              RowBox[{"30", " ", 
                SuperscriptBox["c", "2"], " ", "d"}]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"5", " ", 
                SuperscriptBox["b", "4"]}], "+", 
              RowBox[{"20", " ", "c", " ", 
                SuperscriptBox["b", "3"]}], "+", 
              RowBox[{"20", " ", "d", " ", 
                SuperscriptBox["b", "3"]}], "+", 
              RowBox[{"30", " ", 
                SuperscriptBox["c", "2"], " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"30", " ", 
                SuperscriptBox["d", "2"], " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"60", " ", "c", " ", "d", " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"20", " ", 
                SuperscriptBox["c", "3"], " ", "b"}], "+", 
              RowBox[{"20", " ", 
                SuperscriptBox["d", "3"], " ", "b"}], "+", 
              RowBox[{"60", " ", "c", " ", 
                SuperscriptBox["d", "2"], " ", "b"}], "+", 
              RowBox[{"60", " ", 
                SuperscriptBox["c", "2"], " ", "d", " ", "b"}], "+", 
              RowBox[{"5", " ", 
                SuperscriptBox["c", "4"]}], "+", 
              RowBox[{"5", " ", 
                SuperscriptBox["d", "4"]}], "+", 
              RowBox[{"20", " ", "c", " ", 
                SuperscriptBox["d", "3"]}], "+", 
              RowBox[{"30", " ", 
                SuperscriptBox["c", "2"], " ", 
                SuperscriptBox["d", "2"]}], "+", 
              RowBox[{"20", " ", 
                SuperscriptBox["c", "3"], " ", "d"}]}], ")"}], " ", "a"}], 
        "-", 
        SuperscriptBox["b", "5"], "-", 
        SuperscriptBox["c", "5"], "-", 
        SuperscriptBox["d", "5"], "-", 
        RowBox[{"5", " ", "b", " ", 
          SuperscriptBox["c", "4"]}], "-", 
        RowBox[{"5", " ", "b", " ", 
          SuperscriptBox["d", "4"]}], "-", 
        RowBox[{"5", " ", "c", " ", 
          SuperscriptBox["d", "4"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["b", "2"], " ", 
          SuperscriptBox["c", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["b", "2"], " ", 
          SuperscriptBox["d", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["c", "2"], " ", 
          SuperscriptBox["d", "3"]}], "-", 
        RowBox[{"20", " ", "b", " ", "c", " ", 
          SuperscriptBox["d", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["b", "3"], " ", 
          SuperscriptBox["c", "2"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["b", "3"], " ", 
          SuperscriptBox["d", "2"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox["c", "3"], " ", 
          SuperscriptBox["d", "2"]}], "-", 
        RowBox[{"30", " ", "b", " ", 
          SuperscriptBox["c", "2"], " ", 
          SuperscriptBox["d", "2"]}], "-", 
        RowBox[{"30", " ", 
          SuperscriptBox["b", "2"], " ", "c", " ", 
          SuperscriptBox["d", "2"]}], "-", 
        RowBox[{"5", " ", 
          SuperscriptBox["b", "4"], " ", "c"}], "-", 
        RowBox[{"5", " ", 
          SuperscriptBox["b", "4"], " ", "d"}], "-", 
        RowBox[{"5", " ", 
          SuperscriptBox["c", "4"], " ", "d"}], "-", 
        RowBox[{"20", " ", "b", " ", 
          SuperscriptBox["c", "3"], " ", "d"}], "-", 
        RowBox[{"30", " ", 
          SuperscriptBox["b", "2"], " ", 
          SuperscriptBox["c", "2"], " ", "d"}], "-", 
        RowBox[{"20", " ", 
          SuperscriptBox["b", "3"], " ", "c", " ", "d"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[10]=  (17:57:42 on 9/17/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Combinations", "ObjectName",
  CellTags->"Combinations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Combinations"],

Cell["\<\
Combinations[l, n] returns a list of all possible sets containing n \
elements from the list l. (this function is probably in the combinatorics \
package, but we have enough in memory already).\
\>", "Text",
  CellTags->"Combinations"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Combinations"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Combinations", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "}"}], ",", "3"}], 
        "]"}], "//", "TableForm"}]], "Input",
  CellLabel->"In[22]:=  (18:23:40 on 9/17/02)",
  CellTags->"Combinations"],

Cell[BoxData[
    FormBox[
      TagBox[GridBox[{
            {"a", "b", "c"},
            {"a", "b", "d"},
            {"a", "c", "d"},
            {"b", "c", "d"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        Function[ BoxForm`e$, 
          TableForm[ BoxForm`e$]]], TraditionalForm]], "Output",
  CellLabel->"Out[22]//TableForm=  (18:23:40 on 9/17/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Combine", "ObjectName",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Combine"],

Cell["\<\
Combine[expr]puts terms in a sum over a common denominator,and \
cancels factors in the result. Combine is similar to Together, but accepts \
the option Expanding and works usually better than Together for polynomials \
involving rationals with sums in the denominator.\
\>", "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Combine", "]"}]], "Input",
  CellLabel->"In[243]:=  (21:52:36 on 5/16/01)",
  CellTags->"Combine"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[243]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Factor2",
    ButtonData:>"Factor2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Factor2"],
  "."
}], "Text",
  CellTags->"Combine"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Combine", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], 
      "]"}]], "Input",
  CellLabel->"In[244]:=  (21:52:36 on 5/16/01)",
  CellTags->"Combine"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
  CellLabel->"Out[244]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell["\<\
Here the result from Together where the numerator is automatically \
expanded.\
\>", "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Together", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], 
      "]"}]], "Input",
  CellLabel->"In[245]:=  (21:52:36 on 5/16/01)",
  CellTags->"Combine"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"a", " ", "c"}], "-", 
          RowBox[{"b", " ", "c"}], "-", 
          RowBox[{"a", " ", "d"}], "+", 
          RowBox[{"b", " ", "d"}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
  CellLabel->"Out[245]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell["\<\
If the option Expanding is set to True, the result of Combine is \
the same as Together, but uses a slightly different algorithm.\
\>", "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Combine", "[", 
      RowBox[{
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], ",", 
        RowBox[{"Expanding", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[246]:=  (21:52:36 on 5/16/01)",
  CellTags->"Combine"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"a", " ", "c"}], "-", 
          RowBox[{"b", " ", "c"}], "-", 
          RowBox[{"a", " ", "d"}], "+", 
          RowBox[{"b", " ", "d"}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
  CellLabel->"Out[246]=  (21:52:36 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CombineGraphs", "ObjectName",
  CellTags->"CombineGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CombineGraphs"],

Cell["CombineGraphs is an option for OneLoopSum.", "Text",
  CellTags->"CombineGraphs"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"CombineGraphs"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Commutator", "ObjectName",
  CellTags->"Commutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Commutator"],

Cell["\<\
Commutator[x, y] = c defines the commutator between the \
non-commuting objects x and y. \
\>", "Text",
  CellTags->"Commutator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiCommutator",
    ButtonData:>"AntiCommutator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AntiCommutator"],
  ", ",
  ButtonBox["CommutatorExplicit",
    ButtonData:>"CommutatorExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CommutatorExplicit"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"Commutator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Commutator"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[254]:=  (21:52:36 on 5/16/01)",
  CellTags->"Commutator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Commutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[255]:=  (21:52:36 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[255]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[256]:=  (21:52:36 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[256]=  (21:52:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DotSimplify", "[", 
        RowBox[{"Commutator", "[", 
          RowBox[{
            RowBox[{"a", "+", "b"}], ",", 
            RowBox[{"c", "+", "d"}]}], "]"}], "]"}], " "}]], "Input",
  CellLabel->"In[257]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "c"}], "+", 
        RowBox[{"a", ".", "d"}], "+", 
        RowBox[{"b", ".", "c"}], "+", 
        RowBox[{"b", ".", "d"}], "-", 
        RowBox[{"c", ".", "a"}], "-", 
        RowBox[{"c", ".", "b"}], "-", 
        RowBox[{"d", ".", "a"}], "-", 
        RowBox[{"d", ".", "b"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[257]=  (21:52:37 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[258]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell["Verify the Jacobi identity.", "Text",
  CellTags->"Commutator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Chi]", "=", "Commutator"}], ";", " ", 
      RowBox[{"DeclareNonCommutative", "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], ";"}]], "Input",
  CellLabel->"In[259]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Chi]", "[", 
        RowBox[{"x", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"y", ",", "z"}], "]"}]}], "]"}], "+", 
      RowBox[{"\[Chi]", "[", 
        RowBox[{"y", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"z", ",", "x"}], "]"}]}], "]"}], "+", 
      RowBox[{"\[Chi]", "[", 
        RowBox[{"z", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[260]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"[", "\[NoBreak]", "x", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "y", "\[NoBreak]", ",", "z", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}], "+", 
        RowBox[{"[", "\[NoBreak]", "y", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "z", "\[NoBreak]", ",", "x", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}], "+", 
        RowBox[{"[", "\[NoBreak]", "z", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "x", "\[NoBreak]", ",", "y", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[260]=  (21:52:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "%", "]"}]], "Input",
  CellLabel->"In[261]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[261]=  (21:52:37 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "\[Chi]", "]"}]], "Input",
  CellLabel->"In[262]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[263]:=  (21:52:37 on 5/16/01)",
  CellTags->"Commutator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CommutatorExplicit", "ObjectName",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CommutatorExplicit"],

Cell["\<\
CommutatorExplicit[exp] substitutes any Commutator and \
AntiCommutator in exp by their definitions.\
\>", "Text",
  CellTags->"CommutatorExplicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"CommutatorExplicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[247]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Commutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[248]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[248]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[249]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[249]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiCommutator", "[", 
      RowBox[{
        RowBox[{"a", "-", "c"}], ",", 
        RowBox[{"b", "-", "d"}]}], "]"}]], "Input",
  CellLabel->"In[250]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "\[NoBreak]", 
        FormBox[
          RowBox[{"a", "-", "c"}],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        "\[MediumSpace]", "\[NoBreak]", 
        FormBox[
          RowBox[{"b", "-", "d"}],
          "TraditionalForm"], "\[NoBreak]", "}"}], TraditionalForm]], "Output",\

  CellLabel->"Out[250]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[251]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", "c"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"b", "-", "d"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"b", "-", "d"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "-", "c"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[251]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CommutatorExplicit", "[", "%%", "]"}], "//", 
      "DotSimplify"}]], "Input",
  CellLabel->"In[252]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"a", ".", "d"}], "+", 
        RowBox[{"b", ".", "a"}], "-", 
        RowBox[{"b", ".", "c"}], "-", 
        RowBox[{"c", ".", "b"}], "+", 
        RowBox[{"c", ".", "d"}], "-", 
        RowBox[{"d", ".", "a"}], "+", 
        RowBox[{"d", ".", "c"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[252]=  (21:52:36 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[253]:=  (21:52:36 on 5/16/01)",
  CellTags->"CommutatorExplicit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CompleteSquare", "ObjectName",
  CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CompleteSquare"],

Cell[TextData[{
  "Completes the square of a second order polynomial in the momentum x. \
CompleteSquare[",
  "a",
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["p",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "+",
  "b",
  " ",
  "p",
  "+",
  "c",
  ", ",
  "p",
  "] \[Rule] -",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["b",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "/(4 ",
  "a",
  ")+",
  "c",
  "+",
  "a",
  " (",
  "b",
  "/(2 ",
  "a",
  ")+",
  "x",
  ")^2. CompleteSquare[",
  "a",
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["p",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "+",
  "b",
  " ",
  "p",
  "+",
  "c",
  ", ",
  "p",
  ", ",
  "q",
  "] \[Rule] {-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["b",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  "/(4 ",
  "a",
  ")+",
  "c",
  "+",
  "a",
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          StyleBox["q",
            FontSlant->"Plain"], "2"], TraditionalForm]]],
  ", ",
  "q",
  "\[Rule]",
  "b",
  "/(2 ",
  "a",
  ")+",
  "p",
  "}."
}], "Text",
  CellTags->"CompleteSquare"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"5", 
          RowBox[{"SP", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"2", "p"}], "+", 
                RowBox[{"3", "r"}]}], ",", 
              RowBox[{"p", "+", "r"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
  CellLabel->"In[31]:=  (18:31:06 on 9/17/02)"],

Cell[BoxData[
    FormBox[
      RowBox[{"5", " ", 
        RowBox[{"(", "\[NoBreak]", 
          FormBox[
            RowBox[{"p", "+", "r"}],
            "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
          "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            RowBox[{
              RowBox[{"2", " ", "p"}], "+", 
              RowBox[{"3", " ", "r"}]}],
            "TraditionalForm"], "\[NoBreak]", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[31]=  (18:31:06 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"CompleteSquare", "[", 
        RowBox[{"t1", ",", "p"}], "]"}]}]], "Input",
  CellLabel->"In[32]:=  (18:31:09 on 9/17/02)",
  CellTags->"CompleteSquare"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"10", " ", 
          RowBox[{"(", 
            TagBox[
              FormBox[
                RowBox[{"p", "+", 
                  FractionBox[
                    RowBox[{"5", " ", "r"}], "4"]}],
                "TraditionalForm"],
              TraditionalForm,
              Editable->True], ")", ".", "(", 
            TagBox[
              FormBox[
                RowBox[{"p", "+", 
                  FractionBox[
                    RowBox[{"5", " ", "r"}], "4"]}],
                "TraditionalForm"],
              TraditionalForm,
              Editable->True], ")"}]}], "-", 
        FractionBox[
          RowBox[{"5", " ", 
            SuperscriptBox[
              FormBox["r",
                "TraditionalForm"], "2"]}], "8"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (18:31:09 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"t1", "-", "t2"}], "//", "ScalarProductExpand"}], "//", 
      "Expand"}]], "Input",
  CellLabel->"In[34]:=  (18:31:29 on 9/17/02)",
  CellTags->"CompleteSquare"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[34]=  (18:31:29 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CompleteSquare", "[", 
      RowBox[{"t1", ",", "p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[35]:=  (18:31:35 on 9/17/02)",
  CellTags->"CompleteSquare"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"10", " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], "2"]}], "-", 
            FractionBox[
              RowBox[{"5", " ", 
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], "2"]}], "8"]}], ",", 
          RowBox[{"q", "\[Rule]", 
            RowBox[{"p", "+", 
              FractionBox[
                RowBox[{"5", " ", "r"}], "4"]}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[35]=  (18:31:35 on 9/17/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
  CellLabel->"In[36]:=  (18:31:37 on 9/17/02)",
  CellTags->"CompleteSquare"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ComplexConjugate", "ObjectName",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ComplexConjugate"],

Cell["\<\
ComplexConjugate[expr] complex conjugates expr. It operates on \
fermion lines, i.e., products of Spinor[..] .DiracMatrix[..] . Spinor[..], \
and changes all occuring LorentzIndex[mu] into \
LorentzIndex[ComplexIndex[mu]]. For taking the spin sum (i.e. constructing \
the traces) use FermionSpinSum. WARNING: In expr should be NO explicit I's in \
denominators!\
\>", "Text",
  CellTags->"ComplexConjugate"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ComplexIndex",
    ButtonData:>"ComplexIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ComplexIndex"],
  ", ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  "."
}], "Text",
  CellTags->"ComplexConjugate"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[264]:=  (21:52:37 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            SuperscriptBox["\[Mu]", "*"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            SuperscriptBox["\[Nu]", "*"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[264]=  (21:52:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[265]:=  (21:52:38 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"ComplexIndex", "[", "\[Nu]", "]"}], "]"}]}], 
      "]"}]], "Output",
  CellLabel->"Out[265]//StandardForm=  (21:52:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "5"}], "]"}]], "Input",
  CellLabel->"In[266]:=  (21:52:38 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[266]=  (21:52:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
  CellLabel->"In[267]:=  (21:52:38 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Nu]", "*"],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Mu]", "*"],
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[267]=  (21:52:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
  CellLabel->"In[268]:=  (21:52:38 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[268]=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
  CellLabel->"In[269]:=  (21:52:39 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[269]=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
  CellLabel->"In[270]:=  (21:52:39 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[270]=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"ComplexConjugate", "[", 
          RowBox[{"MetricTensor", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], 
      "]"}]], "Input",
  CellLabel->"In[271]:=  (21:52:39 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{
        RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"ComplexIndex", "[", "\[Nu]", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[271]//StandardForm=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorUBar", "[", 
        RowBox[{"k1", ",", "m"}], "]"}], ".", 
      RowBox[{"GA", "[", "\[Lambda]", "]"}], ".", 
      RowBox[{"SpinorU", "[", 
        RowBox[{"p1", ",", "m"}], "]"}]}]], "Input",
  CellLabel->"In[272]:=  (21:52:39 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"u\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "k1", 
          "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Lambda]"], ".", 
        RowBox[{
        "u", "\[NoBreak]", "(", "\[NoBreak]", "p1", "\[NoBreak]", ",", 
          "\[NoBreak]", "m", "\[NoBreak]", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[272]=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
  CellLabel->"In[273]:=  (21:52:39 on 5/16/01)",
  CellTags->"ComplexConjugate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox[
              SuperscriptBox["\[Lambda]", "*"],
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["k1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[273]=  (21:52:39 on 5/16/01)"]
}, Closed]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. \
Internally they are converted to Spinor objects.\
\>", "Text",
  CellTags->"ComplexConjugate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Contract", "ObjectName",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Contract"],

Cell["\<\
Contract[expr] contracts pairs of Lorentz indices of metric \
tensors, four-vectors and (depending on the optionEpsContract) of Levi-Civita \
tensors in expr. For the contraction of Dirac matrices with each other use \
DiracSimplify. Contract[exp1, exp2] contracts (exp1*exp2), where exp1 and \
exp2 may be larger products of sums of metric tensors and 4-vectors. \
\>", \
"Text",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Contract", "]"}]], "Input",
  CellLabel->"In[281]:=  (21:52:40 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"EpsContract", "\[Rule]", "True"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
          RowBox[{"MomentumCombine", "\[Rule]", "False"}], ",", 
          RowBox[{"Rename", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "0"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[281]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumCombine"],
  "."
}], "Text",
  CellTags->"Contract"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[282]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[282]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[283]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Nu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[283]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]], "Input",
  CellLabel->"In[284]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["p", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[284]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[285]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[285]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[286]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Mu]"}]], TraditionalForm]], "Output",\

  CellLabel->"Out[286]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell["The default dimension for MetricTensor is 4.", "Text",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[287]:=  (21:52:41 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output",
  CellLabel->"Out[287]=  (21:52:41 on 5/16/01)"]
}, Closed]],

Cell["\<\
A short way to enter D-dimensional metric tensors is given by MTD. \
The \".\" as multiplication operator is not necessary but just convenient for \
typesetting.\
\>", "Text",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[288]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], ".", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[288]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[289]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox["D", TraditionalForm]], "Output",
  CellLabel->"Out[289]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[290]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], ".", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[290]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"q", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[291]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        SubscriptBox["q", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[291]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", " ", "]"}]], "Input",
  CellLabel->"In[292]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[292]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"a", "-", "b"}], ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[293]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              TagBox[
                RowBox[{"a", "-", "b"}],
                HoldForm],
              "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"], " ", 
        SubscriptBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              TagBox[
                RowBox[{"p", "-", "q"}],
                HoldForm],
              "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[293]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[294]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["a",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["a",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["b",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["b",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[294]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
        "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Sigma]"}], "]"}]}]], "Input",
  CellLabel->"In[295]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["p", "\[Sigma]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[295]=  (21:52:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[296]:=  (21:52:42 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[296]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], 
          "]"}], " ", 
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
          "]"}]}], " "}]], "Input",
  CellLabel->"In[297]:=  (21:52:43 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[297]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input",
  CellLabel->"In[298]:=  (21:52:43 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "6"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Beta]", "\[NoBreak]", "\[Sigma]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[298]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
      RowBox[{
        RowBox[{"LCD", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], 
          "]"}], " ", 
        RowBox[{"LCD", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
          "]"}]}]}]], "Input",
  CellLabel->"In[299]:=  (21:52:43 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
          "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Alpha]", 
            "\[NoBreak]", "\[Beta]"}]], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
          "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Alpha]", 
            "\[NoBreak]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[299]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", "%", "]"}], "//", "Factor2"}]], "Input",
  CellLabel->"In[300]:=  (21:52:43 on 5/16/01)",
  CellTags->"Contract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[300]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[301]:=  (21:52:43 on 5/16/01)",
  CellTags->"Contract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Contract1", "ObjectName",
  CellTags->"Contract1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Contract1"],

Cell["\<\
Contract1[exp] contracts Upper and Lower indices. Neither Upper and \
Upper nor Lower and Lower indices are contracted.\
\>", "Text",
  CellTags->"Contract1"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Lower",
    ButtonData:>"Lower",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Lower"],
  ", ",
  ButtonBox["Upper",
    ButtonData:>"Upper",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Upper"],
  "."
}], "Text",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Contract1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FV", "[", 
            RowBox[{"p", ",", 
              RowBox[{"Lower", "[", "\[Mu]", "]"}]}], "]"}], 
          RowBox[{"FV", "[", 
            RowBox[{"q", ",", 
              RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], 
      "//", "Contract1"}]], "Input",
  CellLabel->"In[23]:=  (21:59:31 on 9/4/02)",
  CellTags->"Contract1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[23]=  (21:59:31 on 9/4/02)",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FV", "[", 
            RowBox[{"p", ",", 
              RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], 
          RowBox[{"FV", "[", 
            RowBox[{"q", ",", 
              RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], 
      "//", "Contract1"}]], "Input",
  CellLabel->"In[24]:=  (21:59:33 on 9/4/02)",
  CellTags->"Contract1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"Upper", "(", "\[Mu]", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"Upper", "(", "\[Mu]", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (21:59:33 on 9/4/02)",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MT", "[", 
          RowBox[{
            RowBox[{"Lower", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//",
       "Contract1"}]], "Input",
  CellLabel->"In[30]:=  (22:00:15 on 9/4/02)",
  CellTags->"Contract1"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output",
  CellLabel->"Out[30]=  (22:00:15 on 9/4/02)",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MT", "[", 
          RowBox[{
            RowBox[{"Upper", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//",
       "Contract1"}]], "Input",
  CellLabel->"In[26]:=  (21:59:40 on 9/4/02)",
  CellTags->"Contract1"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            RowBox[{"Upper", "(", "\[Mu]", ")"}],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            RowBox[{"Upper", "(", "\[Mu]", ")"}],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (21:59:40 on 9/4/02)",
  CellTags->"Contract1"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Convolute", "ObjectName",
  CellTags->"Convolute"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Convolute"],

Cell[TextData[{
  "Convolute[f, g, x] convolutes ",
  StyleBox["f",
    FontSlant->"Italic"],
  "(",
  StyleBox["x",
    FontSlant->"Italic"],
  ") and ",
  StyleBox["g",
    FontSlant->"Italic"],
  "(",
  StyleBox["x",
    FontSlant->"Italic"],
  "), i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", "1"], 
          RowBox[{
            SubscriptBox[
              StyleBox[
                RowBox[{
                  StyleBox["d",
                    FontSlant->"Plain"], 
                  StyleBox["x",
                    FontSlant->"Italic"]}]], "1"], " ", 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                SubscriptBox[
                  StyleBox[
                    RowBox[{
                      StyleBox["d",
                        FontSlant->"Plain"], "x"}]], "2"], " ", 
                RowBox[{"\[Delta]", "(", 
                  RowBox[{"x", " ", "-", " ", 
                    RowBox[{
                      SubscriptBox["x", "1"], " ", 
                      SubscriptBox["x", "2"]}]}], ")"}], " ", 
                RowBox[{"f", "(", 
                  SubscriptBox["x", "1"], ")"}], " ", 
                RowBox[{
                  RowBox[{"g", "(", 
                    SubscriptBox["x", "2"], ")"}], " ", "."}]}]}]}]}], 
        TraditionalForm]]],
  " Convolute[f, g] is equivalent to Convolute[f, g, x]. Convolute[exp, {x1, \
x2}] assumes that exp is polynomial in x1 and x2. Convolute uses \
table-look-up and does not do any integral calculations, only linear \
algebra."
}], "Text",
  CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
  CellLabel->"In[302]:=  (21:52:43 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Bracket", "\[Rule]", 
            RowBox[{"{", 
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "}"}]}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[302]=  (21:52:43 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PlusDistribution",
    ButtonData:>"PlusDistribution",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PlusDistribution"],
  ", ",
  ButtonBox["ConvoluteTable",
    ButtonData:>"ConvoluteTable",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ConvoluteTable"],
  "."
}], "Text",
  CellTags->"Convolute"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", "1"}], "]"}]], "Input",
  CellLabel->"In[303]:=  (21:52:43 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[303]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"x", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[304]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "x"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[304]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[305]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
  CellLabel->"Out[305]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
  CellLabel->"In[306]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[306]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"PlusDistribution", "[", 
          FractionBox["1", 
            RowBox[{"1", "-", "x"}]], "]"}]}], "]"}]], "Input",
  CellLabel->"In[307]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[307]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[308]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", "x"}], "-", "x", "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[308]=  (21:52:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
  CellLabel->"In[309]:=  (21:52:44 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[309]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[310]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[310]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[311]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[311]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[312]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["log", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[312]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[313]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", "x", "-", 
        RowBox[{"x", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[313]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox[
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[314]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[314]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[315]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", 
        "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[315]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[316]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[316]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
  CellLabel->"In[317]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}]]}], TraditionalForm]], "Output",\

  CellLabel->"Out[317]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
  CellLabel->"In[318]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[318]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"x", ",", " ", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[319]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "x", " ", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[319]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
  CellLabel->"In[320]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[320]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
  CellLabel->"In[321]:=  (21:52:45 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[321]=  (21:52:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[322]:=  (21:52:46 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", "x", "+", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[322]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[323]:=  (21:52:46 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
        RowBox[{"log", "(", "x", ")"}], "-", "x", "-", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[323]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[324]:=  (21:52:46 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", 
        "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[324]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"Log", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[325]:=  (21:52:46 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "x"}], "+", 
        RowBox[{"log", "(", "x", ")"}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[325]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox[
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
  CellLabel->"In[326]:=  (21:52:46 on 5/16/01)",
  CellTags->"Convolute"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"3", " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[326]=  (21:52:46 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ConvoluteTable ***unfinished***", "ObjectName",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ConvoluteTable"],

Cell["ConvoluteTable[f, g, x] yields the convolution of f and g.", "Text",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
  CellLabel->"In[302]:=  (21:52:43 on 5/16/01)",
  CellTags->"ConvoluteTable"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Bracket", "\[Rule]", 
            RowBox[{"{", 
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "}"}]}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[302]=  (21:52:43 on 5/16/01)",
  CellTags->"ConvoluteTable"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PlusDistribution",
    ButtonData:>"PlusDistribution",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PlusDistribution"],
  ", ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Convolute"],
  "."
}], "Text",
  CellTags->"ConvoluteTable"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ConvoluteTable", "[", 
      RowBox[{"1", ",", "1"}], "]"}]], "Input",
  CellLabel->"In[47]:=  (18:52:06 on 9/17/02)",
  CellTags->"ConvoluteTable"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[47]=  (18:52:06 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ConvoluteTable", "[", 
      RowBox[{"x", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[48]:=  (18:52:11 on 9/17/02)",
  CellTags->"ConvoluteTable"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "x"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[48]=  (18:52:11 on 9/17/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ConvoluteTable", "[", 
      RowBox[{"1", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[49]:=  (18:52:17 on 9/17/02)",
  CellTags->"ConvoluteTable"],

Cell[BoxData[
    FormBox[
      RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
  CellLabel->"Out[49]=  (18:52:17 on 9/17/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CovariantFieldDerivative", "ObjectName",
  CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CovariantFieldDerivative"],

Cell["\<\
CovariantFieldDerivative[f[x],x,{li1,li2,...},opts] is a covariant \
derivative of f[x] with respect to space-time variables x and with Lorentz \
indices li1, li2,... CovariantFieldDerivative has only typesetting \
definitions by default. The user is must supply his/her own definition of the \
actual function.\
\>", "Text",
  CellTags->"CovariantFieldDerivative"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CovariantD",
    ButtonData:>"CovariantD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CovariantD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["FieldDerivative",
    ButtonData:>"FieldDerivative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FieldDerivative"],
  "."
}], "Text",
  CellTags->"CovariantFieldDerivative"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantFieldDerivative", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"QuantumField", "[", 
            RowBox[{"A", ",", 
              RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ",", "x",
         ",", 
        RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
  CellLabel->"In[12]:=",
  CellTags->"CovariantFieldDerivative"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          TagBox[
            StyleBox["\<\"\[ScriptCapitalD]\"\>",
              FontSlant->"Italic"],
            (StyleForm[ #, FontSlant -> "Italic"]&)], 
          TagBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            TraditionalForm,
            Editable->True]], "(", 
        TagBox[
          FormBox[
            RowBox[{
              SubscriptBox["A", "\[Mu]"], "(", "x", ")"}],
            "TraditionalForm"],
          TraditionalForm,
          Editable->True], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[12]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CounterT", "ObjectName",
  CellTags->"CounterT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CounterT"],

Cell["\<\
CounterT is a factor used by GluonPropagator and QuarkPropagator \
when CounterTerms is set to All.\
\>", "Text",
  CellTags->"CounterT"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CounterTerm",
    ButtonData:>"CounterTerm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CounterTerm"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["QuarkPropagator",
    ButtonData:>"QuarkPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkPropagator"],
  "."
}], "Text",
  CellTags->"CounterT"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CounterT"],

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
  CellLabel->"In[303]:=  (21:52:43 on 5/16/01)",
  CellTags->"CounterT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CounterTerm", "ObjectName",
  CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CounterTerm"],

Cell["\<\
CounterTerm[name] is a database of counter terms. CounterTerm is \
also an option for the Feynman rule functions QuarkGluonVertex, \
GluonPropagator, QuarkPropagator.\
\>", "Text",
  CellTags->"CounterTerm"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CounterT",
    ButtonData:>"CounterT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CounterT"],
  ", ",
  ButtonBox["QuarkGluonVertex",
    ButtonData:>"QuarkGluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkGluonVertex"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["QuarkPropagator",
    ButtonData:>"QuarkPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkPropagator"],
  "."
}], "Text",
  CellTags->"CounterTerm"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CounterTerm", "[", "Zm", "]"}]], "Input",
  CellLabel->"In[55]:=  (18:54:47 on 9/17/02)",
  CellTags->"CounterTerm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["C", "F"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"4", " ", 
                    RowBox[{"(", 
                      RowBox[{
                        FractionBox[
                          RowBox[{"11", " ", 
                            SubscriptBox["C", "A"]}], "2"], "+", 
                        FractionBox[
                          RowBox[{"9", " ", 
                            SubscriptBox["C", "F"]}], "2"], "-", 
                        RowBox[{"2", " ", 
                          SubscriptBox["N", "f"], " ", 
                          SubscriptBox["T", "f"]}]}], ")"}]}], 
                  SuperscriptBox[
                    TagBox["\[CurlyEpsilon]",
                      TraditionalForm], "2"]], "+", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{"(", 
                      RowBox[{
                        FractionBox[
                          RowBox[{"97", " ", 
                            SubscriptBox["C", "A"]}], "12"], "+", 
                        FractionBox[
                          RowBox[{"3", " ", 
                            SubscriptBox["C", "F"]}], "4"], "-", 
                        FractionBox[
                          RowBox[{"5", " ", 
                            SubscriptBox["N", "f"], " ", 
                            SubscriptBox["T", "f"]}], "3"]}], ")"}]}], 
                  TagBox["\[CurlyEpsilon]",
                    TraditionalForm]]}], ")"}], " ", 
            SubsuperscriptBox["g", "s", "4"]}], 
          RowBox[{"256", " ", 
            SuperscriptBox["\[Pi]", "4"]}]], "+", 
        FractionBox[
          RowBox[{"3", " ", 
            SubscriptBox["C", "F"], " ", 
            SubsuperscriptBox["g", "s", "2"]}], 
          RowBox[{"8", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], " ", 
            SuperscriptBox["\[Pi]", "2"]}]], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[55]=  (18:54:47 on 9/17/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CouplingConstant", "ObjectName",
  CellTags->"CouplingConstant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CouplingConstant"],

Cell["\<\
In general, CouplingConstant is an option for several Feynman rule \
functions and for CovariantD and FieldStrength.
In the convention of the subpackage PHI, CouplingConstant is also the head of \
coupling constants.  CouplingConstant takes three extra optional arguments, \
with head RenormalizationState, RenormalizationScheme and ExpansionState \
respectively.  E.g. CouplingConstant[QED[1]] is the unit charge, \
CouplingConstant[ChPT2[4],1] is the first of the coupling constants of the \
lagrangian ChPT2[4].  CouplingConstant[a_,b_,c___][i_] := \
CouplingConstant[a,b,RenormalizationState[i],c].\
\>", "Text",
  CellTags->"CouplingConstant"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CovariantD",
    ButtonData:>"CovariantD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CovariantD"],
  ", ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FieldStrength"],
  "."
}], "Text",
  CellTags->"CouplingConstant"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CovariantD", "ObjectName",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CovariantD"],

Cell["\<\
CovariantD[mu] is a generic covariant derivative with Lorentz index \
mu. With the option-setting Explicit \[Rule] True, an explicit expression for \
a fermionic field is returned, depending on the setting on the other options.
CovariantD[x, mu] is a generic covariant derivative with respect to x^mu.
CovariantD[mu, a, b] is a covariant derivative for a bosonic field; acting on \
QuantumField[f,{},{a,b}], where f is some field name and a and b are two \
SU(N) indices. Again, with the option-setting Explicit \[Rule] True, an \
explicit expression is returned, depending on the setting on the other \
options.
CovariantD[OPEDelta, a, b] is a short form for  \
CovariantD[mu,a,b]*FourVector[OPEDelta, mu]. CovariantD[{OPEDelta, a, b}, \
{n}] yields the product of n operators, where n is an integer.  \
CovariantD[OPEDelta, a, b, {m, n}]  gives the expanded form of \
CovariantD[OPEDelta, a, b]^m up to order g^n for the gluon, where n is an \
integer and g the coupling constant  indicated by the setting of the option \
CouplingConstant. CovariantD[OPEDelta, {m, n}] gives the expanded form of  \
CovariantD[OPEDelta]^m up to order g^n of the fermionic field.\
\>", "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "CovariantD", "]"}]], "Input",
  CellLabel->"In[327]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"DummyIndex", "\[Rule]", "Automatic"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
          RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[327]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell["\<\
 Possible settings of PartialD are: LeftPartialD, \
LeftRigthPartialD, RightPartialD. The default setting of QuantumField is \
GaugeField.\
\>", "Text",
  CellTags->"CovariantD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"CovariantD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[328]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      SubscriptBox["D", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[328]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[329]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["D", "\[Mu]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output",
  CellLabel->"Out[329]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"\[Mu]", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[330]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "1"],
                    Subscripted[ 
                      c[ 1]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "1"],
                    Subscripted[ 
                      c[ 1]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[330]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell["\<\
The first argument of CovariantD is intepreted as type \
LorentzIndex, except for OPEDelta, which is type Momentum.\
\>", "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", "OPEDelta", "]"}]], "Input",
  CellLabel->"In[331]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      SubscriptBox["D", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[331]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[332]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["D", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output",
  CellLabel->"Out[332]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[333]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox[
                InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[ 
                    c[ 2]],
                  Editable->False],
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[ 
                    c[ 2]],
                  Editable->False],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[333]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[334]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "3"],
                    Subscripted[ 
                      c[ 3]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "3"],
                    Subscripted[ 
                      c[ 3]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[334]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
  CellLabel->"In[335]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{"a", "\[NoBreak]", "c8"}]]}], "-", 
            RowBox[{
              SubscriptBox["g", "s"], " ", 
              SubsuperscriptBox[
                FormBox["A",
                  "TraditionalForm"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e1",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubscriptBox["f", 
                RowBox[{"a", "\[NoBreak]", "c8", "\[NoBreak]", "e1"}]]}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{"b", "\[NoBreak]", "c8"}]]}], "-", 
            RowBox[{
              SubscriptBox["g", "s"], " ", 
              SubsuperscriptBox[
                FormBox["A",
                  "TraditionalForm"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e2",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubscriptBox["f", 
                RowBox[{"c8", "\[NoBreak]", "b", "\[NoBreak]", "e2"}]]}]}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[335]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This gives",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{"m", " ", "times", " ", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              "\[CapitalDelta]"]}], ",", " "}], TraditionalForm]]],
  "the partial derivative ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
  "contracted with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[CapitalDelta]", "\[Mu]"], "."}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "0"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[336]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ")"}], "m"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[336]=  (21:52:46 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "The expansion up to first order in the coupling constant ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"the", "sum", "is", "the", "FeynCalc", "OPESum"}], 
              ")"}], "."}]}], TraditionalForm]]]
}], "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[337]:=  (21:52:46 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], ")"}], "m"], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], "i"], ".", 
                    SubsuperscriptBox[
                      FormBox["A",
                        "TraditionalForm"], 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[337]=  (21:52:47 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "The expansion up to second order in the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["g", "s"], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[338]:=  (21:52:47 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], ")"}], "m"]}], "-", 
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"j", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], "j"], 
                      RowBox[{
                        RowBox[{
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                OverscriptBox["\[PartialD]", "\[RightArrow]"],
                                 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], "i"], ".", 
                          SubsuperscriptBox[
                            FormBox["A",
                              "TraditionalForm"], 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "1"],
                                  Subscripted[ 
                                    c[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]], ".", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                OverscriptBox["\[PartialD]", "\[RightArrow]"],
                                 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], 
                            RowBox[{"j", "-", "i"}]], ".", 
                          SubsuperscriptBox[
                            FormBox["A",
                              "TraditionalForm"], 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "2"],
                                  Subscripted[ 
                                    c[ 2]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]], ".", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                OverscriptBox["\[PartialD]", "\[RightArrow]"],
                                 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], 
                            RowBox[{
                              RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}], 
                        " ", 
                        SubscriptBox["f", 
                          RowBox[{
                            FormBox[
                              FormBox["a",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "1"],
                                  Subscripted[ 
                                    c[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["e", "1"],
                                  Subscripted[ 
                                    e[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]}]], " ", 
                        SubscriptBox["f", 
                          RowBox[{
                            FormBox[
                              FormBox["b",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "2"],
                                  Subscripted[ 
                                    c[ 2]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["e", "1"],
                                  Subscripted[ 
                                    e[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]}]]}]}],
                    HoldForm],
                  "TraditionalForm"]}],
              HoldForm],
            "TraditionalForm"]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], "i"], ".", 
                    SubsuperscriptBox[
                      FormBox["A",
                        "TraditionalForm"], 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[338]=  (21:52:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    SuperscriptBox[
      RowBox[{"CovariantD", "[", 
        RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}], "OPEm"]], "Input",
  CellLabel->"In[339]:=  (21:52:47 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          SubsuperscriptBox["D", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], "m"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[339]=  (21:52:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[340]:=  (21:52:47 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ")"}], "m"], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["c", "1"],
                          Subscripted[ 
                            c[ 1]],
                          Editable->False],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      SubscriptBox[
                        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"]], ")"}], "i"], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["c", "1"],
                          Subscripted[ 
                            c[ 1]],
                          Editable->False],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      SubscriptBox[
                        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"]], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}]}],
              HoldForm],
            "TraditionalForm"]}], "-", 
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"j", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], "j"], 
                      RowBox[{
                        SubscriptBox["T", 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "1"],
                                Subscripted[ 
                                  c[ 1]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SubscriptBox["T", 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "2"],
                                Subscripted[ 
                                  c[ 2]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], "i"], ".", 
                        SubsuperscriptBox[
                          FormBox["A",
                            "TraditionalForm"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "1"],
                                Subscripted[ 
                                  c[ 1]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], 
                          RowBox[{"j", "-", "i"}]], ".", 
                        SubsuperscriptBox[
                          FormBox["A",
                            "TraditionalForm"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "2"],
                                Subscripted[ 
                                  c[ 2]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}]}],
                    HoldForm],
                  "TraditionalForm"]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[340]=  (21:52:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CovariantD", "[", 
        RowBox[{"OPEDelta", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[341]:=  (21:52:47 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", "Gstrong", " ", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"SUNIndex", "[", 
              InterpretationBox[
                SubscriptBox["c", "4"],
                Subscripted[ 
                  c[ 4]],
                Editable->False], "]"}], "]"}], ".", 
          RowBox[{"QuantumField", "[", 
            RowBox[{"GaugeField", ",", 
              RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
              RowBox[{"SUNIndex", "[", 
                InterpretationBox[
                  SubscriptBox["c", "4"],
                  Subscripted[ 
                    c[ 4]],
                  Editable->False], "]"}]}], "]"}]}]}], "+", 
      RowBox[{"RightPartialD", "[", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}]}]], "Output",
  CellLabel->"Out[341]//StandardForm=  (21:52:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CovariantD", "[", 
        RowBox[{"\[Mu]", ",", "a", ",", "b", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[342]:=  (21:52:48 on 5/16/01)",
  CellTags->"CovariantD"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"RightPartialD", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "-", 
      RowBox[{"Gstrong", " ", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"GaugeField", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"SUNIndex", "[", 
              InterpretationBox[
                SubscriptBox["c", "5"],
                Subscripted[ 
                  c[ 5]],
                Editable->False], "]"}]}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", 
            InterpretationBox[
              SubscriptBox["c", "5"],
              Subscripted[ 
                c[ 5]],
              Editable->False]}], "]"}]}]}]], "Output",
  CellLabel->"Out[342]//StandardForm=  (21:52:48 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CrossProduct", "ObjectName",
  CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"CrossProduct"],

Cell["\<\
CrossProduct[a, b] denotes the three-dimensional cross-product of \
the three-vectors a and b.\
\>", "Text",
  CellTags->"CrossProduct"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DotProduct",
    ButtonData:>"DotProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotProduct"],
  ", ",
  ButtonBox["ThreeVector",
    ButtonData:>"ThreeVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ThreeVector"],
  "."
}], "Text",
  CellTags->"CrossProduct"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CrossProduct", "[", 
      RowBox[{
        RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
        RowBox[{"CrossProduct", "[", 
          RowBox[{
            RowBox[{"ThreeVector", "[", "b", "]"}], ",", 
            RowBox[{"ThreeVector", "[", "c", "]"}]}], "]"}]}], "]"}]], "Input",\

  CellLabel->"In[60]:=  (19:00:20 on 9/17/02)",
  CellTags->"CrossProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              OverscriptBox["a", "\[RightVector]"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              OverscriptBox["c", "\[RightVector]"],
              "TraditionalForm"]}], " ", 
          OverscriptBox["b", "\[RightVector]"]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              OverscriptBox["a", "\[RightVector]"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              OverscriptBox["b", "\[RightVector]"],
              "TraditionalForm"]}], " ", 
          OverscriptBox["c", "\[RightVector]"]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[60]=  (19:00:20 on 9/17/02)",
  CellTags->"CrossProduct"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["C0", "ObjectName",
  CellTags->"C0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"C0"],

Cell[TextData[{
  "C0[p10, p12, p20, m1^2, m2^2, m3^2] is the scalar Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["C", "0"], TraditionalForm]]],
  "function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2]), the \
first three arguments of C0 are the scalar products p10 = p1^2, p12 = \
(p1-p2).(p1-p2), p20 = p2^2."
}], "Text",
  CellTags->"C0"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text",
  CellTags->"C0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"C0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"C0", "[", 
      RowBox[{
      "a", ",", "b", ",", "c", ",", " ", "m12", ",", "m22", ",", "m32"}], 
      "]"}]], "Input",
  CellLabel->"In[164]:=  (21:52:07 on 5/16/01)",
  CellTags->"C0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", "\[NoBreak]",
         "c", "\[NoBreak]", ",", "\[NoBreak]", "m12", "\[NoBreak]", ",", 
        "\[NoBreak]", "m22", "\[NoBreak]", ",", "\[NoBreak]", "m32", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[164]=  (21:52:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"C0", "[", 
        RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}],
         "]"}], "//", "PaVeOrder"}]], "Input",
  CellLabel->"In[165]:=  (21:52:07 on 5/16/01)",
  CellTags->"C0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", "\[NoBreak]",
         "c", "\[NoBreak]", ",", "\[NoBreak]", "m12", "\[NoBreak]", ",", 
        "\[NoBreak]", "m22", "\[NoBreak]", ",", "\[NoBreak]", "m32", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[165]=  (21:52:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"C0", "[", 
          RowBox[{
          "b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
          "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"c", ",", "a"}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[166]:=  (21:52:07 on 5/16/01)",
  CellTags->"C0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "b", 
        "\[NoBreak]", ",", "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]",
         "a", "\[NoBreak]", ",", "\[NoBreak]", "m22", "\[NoBreak]", ",", 
        "\[NoBreak]", "m32", "\[NoBreak]", ",", "\[NoBreak]", "m12", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[166]=  (21:52:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DataType", "ObjectName",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DataType"],

Cell["\<\
DataType[exp, type] = True defines the object exp to have data-type \
type. DataType[exp1, exp2, ..., type] defines the objects exp1, exp2, ...to \
have data-type type. The default setting is DataType[__, _] := False. To \
assign a certain data-type, do, e.g., DataType[x, PositiveInteger] = \
True.\
\>", "Text",
  CellTags->"DataType"],

Cell[TextData[{
  "Currently used DataTypes: ",
  ButtonBox["NonCommutative",
    ButtonData:>"NonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NonCommutative"],
  ", ",
  ButtonBox["PositiveInteger",
    ButtonData:>"PositiveInteger",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PositiveInteger"],
  ", ",
  ButtonBox["NegativeInteger",
    ButtonData:>"NegativeInteger",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NegativeInteger"],
  ", ",
  ButtonBox["PositiveNumber",
    ButtonData:>"PositiveNumber",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PositiveNumber"],
  ", ",
  ButtonBox["FreeIndex",
    ButtonData:>"FreeIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FreeIndex"],
  ", ",
  ButtonBox["GrassmannParity",
    ButtonData:>"GrassmannParity",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GrassmannParity"]
}], "Text",
  CellTags->"DataType"],

Cell[TextData[{
  "PHI adds the DataTypes: ",
  ButtonBox["UMatrix",
    ButtonData:>"UMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"UMatrix"],
  ", ",
  ButtonBox["UScalar",
    ButtonData:>"UScalar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"UScalar"],
  "."
}], "Text",
  CellTags->"DataType"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  "."
}], "Text",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DataType"],

Cell["NonCommutative is just a data-type.", "Text",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", 
        "=", " ", "True"}], ";"}]], "Input",
  CellLabel->"In[346]:=  (21:52:48 on 5/16/01)",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"g", ".", 
          RowBox[{"(", 
            RowBox[{"2", "a"}], ")"}], ".", "f"}]}]}]], "Input",
  CellLabel->"In[347]:=  (21:52:48 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"g", ".", 
          RowBox[{"(", 
            RowBox[{"2", " ", "a"}], ")"}], ".", "f"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[347]=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell["\<\
Since \"f \"and \"g\" have DataType NonCommutative the function \
DotSimplify extracts only \"a\" out of the noncommutative product.\
\>", \
"Text",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t", "]"}]], "Input",
  CellLabel->"In[348]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"2", " ", "a", " ", 
          RowBox[{"g", ".", "f"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[348]=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "odd"}], "]"}], "=", 
        RowBox[{
          RowBox[{"DataType", "[", 
            RowBox[{"a", ",", "even"}], "]"}], "=", "True"}]}], 
      ";"}]], "Input",
  CellLabel->"In[349]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ptest1", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
              RowBox[{"DataType", "[", 
                RowBox[{"n", ",", "odd"}], "]"}]}], "\[RuleDelayed]", 
            RowBox[{"-", "1"}]}]}]}], ";"}]], "Input",
  CellLabel->"In[350]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ptest2", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
              RowBox[{"DataType", "[", 
                RowBox[{"n", ",", "even"}], "]"}]}], "\[RuleDelayed]", 
            "1"}]}]}], ";"}]], "Input",
  CellLabel->"In[351]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "a"}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "z"}]}]}]], "Input",
  CellLabel->"In[352]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "a"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
  CellLabel->"Out[352]=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest1", "[", "t", "]"}]], "Input",
  CellLabel->"In[353]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "a"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
  CellLabel->"Out[353]=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest2", "[", "%", "]"}]], "Input",
  CellLabel->"In[354]:=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "z"], TraditionalForm]], "Output",
  CellLabel->"Out[354]=  (21:52:49 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"ptest1", ",", "ptest2", ",", "t", ",", "a", ",", "m"}], 
        "]"}], ";"}]], "Input",
  CellLabel->"In[355]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "ganzeZahl"}], "]"}], "=", "True"}], 
      ";"}]], "Input",
  CellLabel->"In[356]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"f", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "^", "p_"}], "/;", 
                RowBox[{"DataType", "[", 
                  RowBox[{"p", ",", "ganzeZahl"}], "]"}]}], "\[RuleDelayed]", 
              "1"}], "}"}]}]}], ";"}]], "Input",
  CellLabel->"In[357]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"test", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", "n"}], " ", "x"}]}]}]], "Input",\

  CellLabel->"In[358]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output",
  CellLabel->"Out[358]=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f", "[", "test", "]"}]], "Input",
  CellLabel->"In[359]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[359]=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"f", ",", "test"}], "]"}], ";"}]], "Input",
  CellLabel->"In[360]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", 
        "=", " ", "False"}], ";"}]], "Input",
  CellLabel->"In[361]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "odd"}], "]"}], "=", 
        RowBox[{
          RowBox[{"DataType", "[", 
            RowBox[{"a", ",", "even"}], "]"}], "=", "False"}]}], 
      ";"}]], "Input",
  CellLabel->"In[362]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell["\<\
Certain FeynCalc objects have DataType PositiveInteger set to True.\
\
\>", "Text",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
  CellLabel->"In[363]:=  (21:52:50 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[363]=  (21:52:51 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell["PowerSimplify uses the DataType information.", "Text",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", " ", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[364]:=  (21:52:51 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[364]=  (21:52:51 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", " ", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", " ", 
            RowBox[{"SO", "[", "q", "]"}]}], ")"}], "^", "OPEm"}], 
      "]"}]], "Input",
  CellLabel->"In[365]:=  (21:52:51 on 5/16/01)",
  CellTags->"DataType"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["q",
                "TraditionalForm"]}], ")"}], "m"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[365]=  (21:52:51 on 5/16/01)",
  CellTags->"DataType"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DB0", "ObjectName",
  CellTags->"DB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DB0"],

Cell["\<\
DB0[p2, m1^2, m2^2] is the derivative of the two-point function \
B0[p2, m1^2, m2^2] with respect to p2.\
\>", "Text",
  CellTags->"DB0"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  "."
}], "Text",
  CellTags->"DB0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DB0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D", "[", 
      RowBox[{
        RowBox[{"B0", "[", 
          RowBox[{
            SubscriptBox["p", "2"], ",", 
            SuperscriptBox[
              SubscriptBox["m", "1"], "2"], ",", 
            SuperscriptBox[
              SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
        SubscriptBox["p", "2"]}], "]"}]], "Input",
  CellLabel->"In[2]:=  (23:01:29 on 9/19/02)",
  CellTags->"DB0"],

Cell[BoxData[
    FormBox[
      RowBox[{"DB0", "(", 
        RowBox[{
          SubscriptBox["p", "2"], ",", 
          SubsuperscriptBox["m", "1", "2"], ",", 
          SubsuperscriptBox["m", "2", "2"]}], ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[2]=  (23:01:29 on 9/19/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DB1", "ObjectName",
  CellTags->"DB1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DB1"],

Cell["\<\
DB1[p2,m1^2,m2^2] is the derivative of B1[p2,m1^2,m2^2] with \
respect to p2.\
\>", "Text",
  CellTags->"DB1"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B1"],
  "."
}], "Text",
  CellTags->"DB1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DB1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D", "[", 
      RowBox[{
        RowBox[{"B1", "[", 
          RowBox[{
            SubscriptBox["p", "2"], ",", 
            SuperscriptBox[
              SubscriptBox["m", "1"], "2"], ",", 
            SuperscriptBox[
              SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
        SubscriptBox["p", "2"]}], "]"}]], "Input",
  CellLabel->"In[4]:=  (23:02:39 on 9/19/02)",
  CellTags->"DB1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox["m", "2", "2"], "-", 
                SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
            RowBox[{"DB0", "(", 
              RowBox[{
                SubscriptBox["p", "2"], ",", 
                SubsuperscriptBox["m", "1", "2"], ",", 
                SubsuperscriptBox["m", "2", "2"]}], ")"}]}], 
          RowBox[{"2", " ", 
            SubscriptBox["p", "2"]}]], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"DB0", "(", 
            RowBox[{
              SubscriptBox["p", "2"], ",", 
              SubsuperscriptBox["m", "1", "2"], ",", 
              SubsuperscriptBox["m", "2", "2"]}], ")"}]}], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SubscriptBox["p", "2"], ",", 
                  SubsuperscriptBox["m", "1", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SubsuperscriptBox["m", "1", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox["m", "2", "2"], "-", 
                SubsuperscriptBox["m", "1", "2"]}], ")"}]}], 
          RowBox[{"2", " ", 
            SubsuperscriptBox["p", "2", "2"]}]]}], TraditionalForm]], "Output",\

  CellLabel->"Out[4]=  (23:02:39 on 9/19/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DeclareNonCommutative", "ObjectName",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DeclareNonCommutative"],

Cell["\<\
DeclareNonCommutative[a, b, ...] declares a,b, ... to be \
non-commutative, i.e., DataType[a,b, ..., NonCommutative] is set to \
True.\
\>", "Text",
  CellTags->"DeclareNonCommutative"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["UnDeclareNonCommutative",
    ButtonData:>"UnDeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"UnDeclareNonCommutative"],
  "."
}], "Text",
  CellTags->"DeclareNonCommutative"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DeclareNonCommutative"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
  CellLabel->"In[366]:=  (21:52:51 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell["\<\
As a side effect of DeclareNonCommutative x is declared to be of \
data type NonCommutative.\
\>", "Text",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
  CellLabel->"In[367]:=  (21:52:51 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[367]=  (21:52:51 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[368]:=  (21:52:51 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
      "]"}]], "Input",
  CellLabel->"In[369]:=  (21:52:51 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "True", ",", "True", ",", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[369]=  (21:52:51 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[370]:=  (21:52:51 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
      "]"}]], "Input",
  CellLabel->"In[371]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeclareNonCommutative"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "False", ",", "False", ",", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[371]=  (21:52:52 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DeltaFunction", "ObjectName",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DeltaFunction"],

Cell[TextData[{
  " DeltaFunction[x] is the Dirac delta-function ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
  ".\n*** After version 4 of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", there is a built-in function, DiracDelta, with comparable properties. \
***"
}], "Text",
  CellTags->"DeltaFunction"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Convolute"],
  ", ",
  ButtonBox["DeltaFunctionPrime",
    ButtonData:>"DeltaFunctionPrime",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunctionPrime"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["SimplifyDeltaFunction",
    ButtonData:>"SimplifyDeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyDeltaFunction"],
  "."
}], "Text",
  CellTags->"DeltaFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DeltaFunction", "[", 
      RowBox[{"1", "-", "x"}], "]"}]], "Input",
  CellLabel->"In[372]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Delta]", "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[372]=  (21:52:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[373]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[373]=  (21:52:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", "x", "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[374]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "0", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[374]=  (21:52:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[375]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[375]=  (21:52:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"DeltaFunction", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[376]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunction"],

Cell[BoxData[
    FormBox["x", TraditionalForm]], "Output",
  CellLabel->"Out[376]=  (21:52:52 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DeltaFunctionDoublePrime", "ObjectName",
  CellTags->"DeltaFunctionDoublePrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
  " DeltaFunctionDoublePrime[1-x] is the second derivative of the Dirac \
delta-function ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DeltaFunctionPrime",
    ButtonData:>"DeltaFunctionPrime",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunctionPrime"],
  "."
}], "Text",
  CellTags->"DeltaFunctionDoublePrime"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DeltaFunctionPrime", "ObjectName",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
  " DeltaFunctionPrime[1-x] is the derivative of the Dirac delta-function ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Convolute"],
  ", ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["SimplifyDeltaFunction",
    ButtonData:>"SimplifyDeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyDeltaFunction"],
  "."
}], "Text",
  CellTags->"DeltaFunctionPrime"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DeltaFunctionPrime", "[", 
      RowBox[{"1", "-", "x"}], "]"}]], "Input",
  CellLabel->"In[377]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunctionPrime"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[377]=  (21:52:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[378]:=  (21:52:52 on 5/16/01)",
  CellTags->"DeltaFunctionPrime"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "(", "1", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[378]=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          SuperscriptBox["x", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[379]:=  (21:52:53 on 5/16/01)",
  CellTags->"DeltaFunctionPrime"],

Cell[BoxData[
    FormBox["2", TraditionalForm]], "Output",
  CellLabel->"Out[379]=  (21:52:53 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DenominatorOrder", "ObjectName",
  CellTags->"DenominatorOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DenominatorOrder"],

Cell["\<\
DenominatorOrder is an option for OneLoop, if set to True the \
PropagatorDenominator will be ordered in a standard way.\
\>", "Text",
  CellTags->"DenominatorOrder"],

Cell[TextData[{
  "See also: ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text",
  CellTags->"DenominatorOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Dimension", "ObjectName",
  CellTags->"Dimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Dimension"],

Cell["\<\
Dimension is an option of several functions and denotes the number \
of space-time dimensions. Possible settings are: 4, n, d, D, ... ,the \
variable does not matter, but it should have Head Symbol.\
\>", "Text",
  CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "MetricTensor", "]"}]], "Input",
  CellLabel->"In[380]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[380]=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MetricTensor", "[", 
          RowBox[{"m", ",", "n", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
        RowBox[{"DiracMatrix", "[", 
          RowBox[{"\[Alpha]", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
      "FCI"}]], "Input",
  CellLabel->"In[381]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              FormBox["m",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["n",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[381]=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell["\<\
The dimension of the indices is not shown by default but can be \
inspected easily.\
\>", "Text",
  CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MetricTensor", "[", 
          RowBox[{"m", ",", "n", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
        RowBox[{"DiracMatrix", "[", 
          RowBox[{"\[Alpha]", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[382]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "d"}], "]"}], ",", "d"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"m", ",", "d"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"n", ",", "d"}], "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[382]//StandardForm=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell["\<\
Setting the global variable $LorentzIndices to True will display \
the dimension (if different from 4) as a subscript.\
\>", "Text",
  CellTags->"Dimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
  CellLabel->"In[383]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Alpha]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
  CellLabel->"In[384]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Alpha]",
                "TraditionalForm"], 
              FormBox["n",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[384]=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[385]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[385]//StandardForm=  (21:52:53 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
  CellLabel->"In[386]:=  (21:52:53 on 5/16/01)",
  CellTags->"Dimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DimensionalReduction", "ObjectName",
  CellTags->"DimensionalReduction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DimensionalReduction"],

Cell["\<\
DimensionalReduction is an option for TID and \
OneLoopSimplify.\
\>", "Text",
  CellTags->"DimensionalReduction"],

Cell[TextData[{
  "See also: ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text",
  CellTags->"DimensionalReduction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracBasis", "ObjectName",
  CellTags->"DiracBasis"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracBasis"],

Cell["\<\
DiracBasis[any] is a head which is wrapped around Dirac structures \
(and the 1) as a result of the function DiracReduce. Eventually you want to \
substitute DiracBasis by Identity (or set: DiracBasis[1] = S; \
DiracBasis[DiracMatrix[mu]] = P; etc.).\
\>", "Text",
  CellTags->"DiracBasis"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracReduce",
    ButtonData:>"DiracReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracReduce"],
  "."
}], "Text",
  CellTags->"DiracBasis"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracCanonical", "ObjectName",
  CellTags->"DiracCanonical"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracCanonical"],

Cell["\<\
DiracCanonical is an option for DiracSimplify. If set to True \
DiracSimplify uses the function DiracOrder internally.\
\>", "Text",
  CellTags->"DiracCanonical"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracCanonical"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracEquation", "ObjectName",
  CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracEquation"],

Cell["\<\
DiracEquation[exp] applies the Dirac equation without expanding \
exp. If that is needed, use DiracSimplify.\
\>", "Text",
  CellTags->"DiracEquation"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracEquation"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracEquation"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}], 
      ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "p", "]"}], "]"}]}]], "Input",
  CellLabel->"In[4]:=  (22:45:29 on 9/2/02)",
  CellTags->"DiracEquation"],

Cell[BoxData[
    RowBox[{"DiracEquation", "[", "%", "]"}]], "Input",
  CellLabel->"In[5]:=  (22:45:29 on 9/2/02)",
  CellTags->"DiracEquation"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracGamma", "ObjectName",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracGamma"],

Cell[TextData[{
  "DiracGamma[x, dim] is the head of all Dirac matrices and slashes (in the \
internal representation). Use DiracMatrix (or GA, GAD) and DiracSlash (or GS, \
GSD) for manual (short) input. DiracGamma[x, 4] simplifies to DiracGamma[x]. \
DiracGamma[5] is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  ". DiracGamma[6] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
        TraditionalForm]]],
  " DiracGamma[7] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"DiracGamma"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"DiracGamma"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGamma", "[", "5", "]"}]], "Input",
  CellLabel->"In[399]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
  CellLabel->"Out[399]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Input",
  CellLabel->"In[400]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Alpha]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[400]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "A Dirac-slash, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          SubscriptBox["q", "\[Mu]"]}], TraditionalForm]]],
  ", is displayed as ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], " "}]], "Input",
  CellLabel->"In[401]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[401]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[402]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[402]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"q", ",", "D"}], "]"}], ",", "D"}], "]"}], " "}]], "Input",\

  CellLabel->"In[403]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[403]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}]}]}]], "Input",
  CellLabel->"In[404]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[404]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", "=", 
      RowBox[{"DiracGammaExpand", "[", "a1", "]"}]}]], "Input",
  CellLabel->"In[405]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[405]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", "=", 
      RowBox[{
        RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"GSD", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ".", 
        RowBox[{"GSD", "[", "q", "]"}], ".", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}]}]}]], "Input",
  CellLabel->"In[406]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[406]=  (21:52:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a4", "=", 
      RowBox[{"DiracTrick", "[", "a3", "]"}]}]], "Input",
  CellLabel->"In[407]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "4"}], ")"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  RowBox[{"p", "-", "q"}],
                  "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
            "\[CenterDot]", "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[407]=  (21:52:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a5", "=", 
      RowBox[{"DiracSimplify", "[", "a4", "]"}]}]], "Input",
  CellLabel->"In[408]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGamma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"D", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"D", " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[408]=  (21:52:55 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], 
      "]"}]], "Input",
  CellLabel->"In[409]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGamma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracGammaCombine", "ObjectName",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracGammaCombine"],

Cell["\<\
DiracGammaCombine[exp] is (nearly) the inverse operation to \
DiracGammaExpand.\
\>", "Text",
  CellTags->"DiracGammaCombine"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"DiracGammaCombine"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaCombine", "[", 
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], " ", "+", " ", 
        RowBox[{"GS", "[", "q", "]"}]}], "]"}]], "Input",
  CellLabel->"In[387]:=  (21:52:53 on 5/16/01)",
  CellTags->"DiracGammaCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[387]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[388]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaCombine"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{"p", "+", "q"}], "]"}], "]"}]], "Output",
  CellLabel->"Out[388]//StandardForm=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaCombine", "[", 
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"GS", "[", "p", "]"}]}], " ", "-", " ", 
        RowBox[{"2", " ", 
          RowBox[{"GS", "[", "q", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[389]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "-", 
            RowBox[{"2", " ", "q"}]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[389]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[390]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaCombine"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{
          RowBox[{"2", " ", "p"}], "-", 
          RowBox[{"2", " ", "q"}]}], "]"}], "]"}]], "Output",
  CellLabel->"Out[390]//StandardForm=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaExpand", "[", "%%", "]"}]], "Input",
  CellLabel->"In[391]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[391]=  (21:52:54 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracGammaExpand", "ObjectName",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracGammaExpand"],

Cell["\<\
DiracGammaExpand[exp] expands all DiracGamma[Momentum[a+b+..]] in \
exp into (DiracGamma[Momentum[a]] + DiracGamma[Momentum[b]] + ...).\
\>", \
"Text",
  CellTags->"DiracGammaExpand"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracGammaCombine",
    ButtonData:>"DiracGammaCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaCombine"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"DiracGammaExpand"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", "-", "q"}], "]"}], "]"}]}]}]], "Input",
  CellLabel->"In[392]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[392]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell["\<\
Momentum is the head of p-q, i.e., it is treated as one \
four-momentum.\
\>", "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "t", "]"}]], "Input",
  CellLabel->"In[393]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
  CellLabel->"Out[393]//StandardForm=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell["With DiracGammaExpand the Momentum[p-q] gets expanded.", "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaExpand", "[", "t", "]"}]], "Input",
  CellLabel->"In[394]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[394]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell["The inverse operation is DiracGammaCombine.", "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"DiracGammaCombine", "[", 
        RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[395]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
  CellLabel->"Out[395]//StandardForm=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}]], "Input",
  CellLabel->"In[396]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "p", "]"}], "]"}], "-", 
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "q", "]"}], "]"}]}], ")"}]}]], "Output",
  CellLabel->"Out[396]//StandardForm=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell["In order to do non-commutative expansion use DiracSimplify.", "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t", "]"}]], "Input",
  CellLabel->"In[397]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[397]=  (21:52:54 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[398]:=  (21:52:54 on 5/16/01)",
  CellTags->"DiracGammaExpand"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracGammaT", "ObjectName",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracGammaT"],

Cell["\<\
DiracGammaT[x] denotes the transpose of DiracGamma[x]. \
Transpose[DiracGammaT[x]] gives DiracGamma[x]. Note that x must have Head \
LorentzIndex or Momentum.\
\>", "Text",
  CellTags->"DiracGammaT"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  "."
}], "Text",
  CellTags->"DiracGammaT"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaT", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
  CellLabel->"In[410]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGammaT"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], "T"], TraditionalForm]], "Output",
  CellLabel->"Out[410]=  (21:52:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Transpose", "[", "%", "]"}]], "Input",
  CellLabel->"In[411]:=  (21:52:55 on 5/16/01)",
  CellTags->"DiracGammaT"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[411]=  (21:52:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
      "Transpose"}]], "Input",
  CellLabel->"In[412]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracGammaT"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{
        "(", "\[NoBreak]", "\[Gamma]", "\[NoBreak]", "\[CenterDot]", 
          "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ")"}], "T"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[412]=  (21:52:56 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracMatrix", "ObjectName",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracMatrix"],

Cell[TextData[{
  "DiracMatrix[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes a Dirac gamma matrix with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ". DiracMatrix[",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Mu]", ",", "\[Nu]", ","}], " "}], TraditionalForm]]],
  "...] is a product of ",
  Cell[BoxData[
      FormBox["\[Gamma]", TraditionalForm]]],
  " matrices with Lorentz indices ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]", ",", " ", "..."}], 
        TraditionalForm]]],
  " DiracMatrix[5] is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  ". DiracMatrix[6] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", "2"}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            RowBox[{"+", " ", 
              SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], 
        TraditionalForm]]],
  ". DiracMatrix[7] is",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "1", "/", "2"}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            RowBox[{"-", " ", 
              SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], 
        TraditionalForm]]],
  "."
}], "Text",
  CellTags->"DiracMatrix"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GAD"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GS"],
  ", ",
  ButtonBox["GSD",
    ButtonData:>"GSD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GSD"],
  "."
}], "Text",
  CellTags->"DiracMatrix"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[413]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[413]=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This is how to enter the non-commutative product of two ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], TraditionalForm]]],
  ". The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Dot \".\" is used as non-commutative multiplication operator."
}], "Text",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[414]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[414]=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Alpha]", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[415]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Output",
  CellLabel->"Out[415]//StandardForm=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], "//", "FCE"}]], "Input",
  CellLabel->"In[416]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[416]=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[417]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Output",
  CellLabel->"Out[417]//StandardForm=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[418]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[418]=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[419]:=  (21:52:56 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
  CellLabel->"Out[419]//StandardForm=  (21:52:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]], "Input",
  CellLabel->"In[420]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[420]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", 
        RowBox[{"a", " ", ".", " ", "b"}], "]"}], "//", "FCI"}]], "Input",
  CellLabel->"In[421]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[421]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[422]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracMatrix"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"LorentzIndex", "[", "a", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"LorentzIndex", "[", "b", "]"}], "]"}]}]], "Output",
  CellLabel->"Out[422]//StandardForm=  (21:52:57 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracOrder", "ObjectName",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracOrder"],

Cell["\<\
DiracOrder[expr] orders the Dirac matrices in expr alphabetically. \
DiracOrder[expr,orderlist] orders the Dirac matrices in expr according to \
orderlist.\
\>", "Text",
  CellTags->"DiracOrder"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"DiracOrder"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Beta]", ",", "\[Alpha]"}], "]"}]}]], "Input",
  CellLabel->"In[423]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[423]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
  CellLabel->"In[424]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[424]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[425]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[425]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
  CellLabel->"In[426]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[426]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[427]:=  (21:52:57 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[427]=  (21:52:57 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], " "}], TraditionalForm]]],
  "is moved to the right."
}], "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t3", "]"}]], "Input",
  CellLabel->"In[428]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[428]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
  CellLabel->"In[429]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[429]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t4", "]"}]], "Input",
  CellLabel->"In[430]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[430]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[431]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[431]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This orders the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Alpha]"], 
          SuperscriptBox["\[Gamma]", "\[Beta]"], 
          SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]]],
  " in reverse order."
}], "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", 
      RowBox[{"t5", ",", 
        RowBox[{"{", 
          RowBox[{"\[Delta]", ",", "\[Beta]", ",", "\[Alpha]"}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[432]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Delta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Delta]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Delta]"}]]}], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Delta]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[432]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "%", "]"}]], "Input",
  CellLabel->"In[433]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[433]=  (21:52:58 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5"}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[434]:=  (21:52:58 on 5/16/01)",
  CellTags->"DiracOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracReduce", "ObjectName",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracReduce"],

Cell["\<\
DiracReduce[exp] reduces all four-dimensional Dirac matrices in exp \
to the standard basis (S,P,V,A,T) using the Chisholm identity (see Chisholm). \
In the result the basic Dirac structures are wrapped with a head DiracBasis. \
I.e., S corresponds to DiracBasis[1], P : DiracBasis[DiracMatrix[5]], V: \
DiracBasis[DiracMatrix[mu]], A: DiracBasis[DiracMatrix[mu, 5]], T: \
DiracBasis[DiracSigma[DiracMatrix[mu, nu]]]. By default DiracBasis is \
substituted to Identity. Notice that the result of DiracReduce is given in \
the FeynCalcExternal-way, i.e.,evtl. you may have to use FeynCalcInternal on \
the result.\
\>", "Text",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracReduce", "]"}]], "Input",
  CellLabel->"In[435]:=  (21:52:59 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"DiracBasis", "\[Rule]", "Identity"}], "}"}]}]}], "}"}],
       TraditionalForm]], "Output",
  CellLabel->"Out[435]=  (21:52:59 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracReduce"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[436]:=  (21:52:59 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[436]=  (21:52:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracReduce", "[", "t1", "]"}]], "Input",
  CellLabel->"In[437]:=  (21:52:59 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[437]=  (21:52:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
  CellLabel->"In[438]:=  (21:52:59 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[438]=  (21:52:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracReduce", "[", "t2", "]"}]], "Input",
  CellLabel->"In[439]:=  (21:52:59 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              RowBox[{"$MU", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
            "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"$MU", "\[InvisibleApplication]", 
                  RowBox[{"(", "1", ")"}]}],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Rho]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[439]=  (21:53:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[440]:=  (21:53:00 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"], 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[440]=  (21:53:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"DiracReduce", "[", "t3", "]"}]}]], "Input",
  CellLabel->"In[441]:=  (21:53:00 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          SuperscriptBox["\[Gamma]", "5"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
            "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", 
              "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[441]=  (21:53:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", " ", "=", 
      RowBox[{"Calc", "[", 
        RowBox[{"DiracSimplify", "[", 
          RowBox[{"DiracSigmaExplicit", "[", 
            RowBox[{"t4", ".", "t4"}], "]"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[442]:=  (21:53:03 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "128"}], TraditionalForm]], "Output",
  CellLabel->"Out[442]=  (21:53:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t4", ".", "t4"}], "]"}]], "Input",
  CellLabel->"In[443]:=  (21:53:05 on 5/16/01)",
  CellTags->"DiracReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "128"}], TraditionalForm]], "Output",
  CellLabel->"Out[443]=  (21:53:05 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4"}], "]"}]], "Input",
  CellLabel->"In[444]:=  (21:53:05 on 5/16/01)",
  CellTags->"DiracReduce"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSigma", "ObjectName",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSigma"],

Cell["\<\
DiracSigma[a, b] stands for i/2*(a . b - b . a) in 4 dimensions. a \
and b must have Head DiracGamma, DiracMatrix or DiracSlash. Only antisymmetry \
is implemented.\
\>", "Text",
  CellTags->"DiracSigma"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSigmaExplicit",
    ButtonData:>"DiracSigmaExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSigmaExplicit"],
  "."
}], "Text",
  CellTags->"DiracSigma"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
          RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[447]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigma"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[447]=  (21:53:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "t1", "]"}]], "Input",
  CellLabel->"In[448]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[448]=  (21:53:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GA", "[", "\[Beta]", "]"}], ",", 
          RowBox[{"GA", "[", "\[Alpha]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[449]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigma"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox["\[Sigma]", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[449]=  (21:53:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], ",", 
          RowBox[{"GS", "[", "q", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[450]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigma"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"p", "\[NoBreak]", "q"}]], TraditionalForm]], "Output",
  CellLabel->"Out[450]=  (21:53:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "t3", "]"}]], "Input",
  CellLabel->"In[451]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSigma"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[451]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
  CellLabel->"In[452]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSigma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSigmaExplicit", "ObjectName",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSigmaExplicit"],

Cell["\<\
DiracSigmaExplicit[exp] inserts in exp for all DiracSigma its \
definition. DiracSigmaExplict is also an option of DiracSimplify.\
\>", "Text",\

  CellTags->"DiracSigmaExplicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSigma",
    ButtonData:>"DiracSigma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSigma"],
  "."
}], "Text",
  CellTags->"DiracSigmaExplicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigma", "[", 
      RowBox[{
        RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[445]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigmaExplicit"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[445]=  (21:53:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[446]:=  (21:53:06 on 5/16/01)",
  CellTags->"DiracSigmaExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[446]=  (21:53:06 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSimpCombine", "ObjectName",
  CellTags->"DiracSimpCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSimpCombine"],

Cell["\<\
DiracSimpCombine is an option for DiracSimplify. If set to True, \
sums of DiracGamma's will be merged as much as possible in DiracGamma[ .. + \
.. + ]'s.\
\>", "Text",
  CellTags->"DiracSimpCombine"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracSimpCombine"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSimplify", "ObjectName",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSimplify"],

Cell["\<\
DiracSimplify[expr] simplifies products of Dirac matrices in expr \
and expands non-commutative products. Double Lorentz indices and four vectors \
are contracted. The Dirac equation is applied. All DiracMatrix[5], \
DiracMatrix[6] and DiracMatrix[7] are moved to the right. The order of the \
other Dirac matrices is not changed.\
\>", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracSimplify", "]"}]], "Input",
  CellLabel->"In[453]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracCanonical", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSigmaExplicit", "\[Rule]", "True"}], ",", 
          RowBox[{"DiracSimpCombine", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSubstitute67", "\[Rule]", "False"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
          RowBox[{"InsideDiracTrace", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[453]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"DiracSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracSimplify"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[454]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[454]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
  CellLabel->"In[455]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[455]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[456]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[456]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
  CellLabel->"In[457]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[457]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[458]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[458]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "is moved to the right."
}], "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t3", "]"}]], "Input",
  CellLabel->"In[459]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[459]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
  CellLabel->"In[460]:=  (21:53:07 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[460]=  (21:53:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t4", "]"}]], "Input",
  CellLabel->"In[461]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[461]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", 
          RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
  CellLabel->"In[462]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"a", "+", "b"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"c", "+", "d"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[462]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell["\<\
Contrary to DiracTrick DiracSimplify does non-commutative \
expansion.\
\>", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t5", "]"}]], "Input",
  CellLabel->"In[463]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[463]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
  CellLabel->"In[464]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"a", "+", "b"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"c", "+", "d"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[464]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", " ", "=", " ", 
      RowBox[{
        RowBox[{"SpinorVBar", "[", "p", "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], " ", ".", " ", 
        RowBox[{"SpinorUBar", "[", "q", "]"}]}]}]], "Input",
  CellLabel->"In[465]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"v\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{
          FormBox[
            OverscriptBox["\<\"u\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "q", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[465]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t6", "]"}]], "Input",
  CellLabel->"In[466]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[466]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "@@", 
      RowBox[{"Join", "[", 
        RowBox[{
          RowBox[{"{", "\[Mu]", "}"}], ",", 
          RowBox[{"Table", "[", 
            RowBox[{
              SubscriptBox["\[Nu]", "i"], ",", 
              RowBox[{"{", 
                RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[467]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "1"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "2"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "3"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "4"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "5"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "6"]], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[467]=  (21:53:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "%", "]"}]], "Input",
  CellLabel->"In[468]:=  (21:53:08 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"D", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{"12", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[468]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell["\<\
 With the option DiracCanonical an alphabetic ordering is \
done.\
\>", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}], ",", 
        RowBox[{"DiracCanonical", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[469]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[469]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell["\<\
Setting InsideDiracTrace\[Rule]True assumes that a trace is still \
to be taken later on.\
\>", "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
          "]"}], ",", 
        RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[470]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[470]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], ",", 
        RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[471]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[471]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
      "]"}]], "Input",
  CellLabel->"In[472]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSimplify2", "ObjectName",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSimplify2"],

Cell[TextData[{
  "DiracSimplify2[exp] simplifies the Dirac structure but leaves any ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " untouched."
}], "Text",
  CellTags->"DiracSimplify2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracSimplify2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{
      "\[Mu]", ",", "\[Nu]", ",", "\[Mu]", ",", "5", ",", "\[Alpha]", ",", 
        "\[Beta]", ",", "\[Alpha]"}], "]"}]], "Input",
  CellLabel->"In[473]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[473]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify2", "[", "%", "]"}]], "Input",
  CellLabel->"In[474]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSimplify2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[474]=  (21:53:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSlash", "ObjectName",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSlash"],

Cell[TextData[{
  "DiracSlash[p] is the contraction ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox["p", "\[Mu]"], 
            SubscriptBox["\[Gamma]", "\[Mu]"]}], " "}], TraditionalForm]]],
  "(FourVector[p, ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] DiracMatrix[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]). Products of those can be entered in the form DiracSlash[p1, p2, ...]."
}], "Text",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracSlash", "]"}]], "Input",
  CellLabel->"In[475]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[475]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GS"],
  ", ",
  ButtonBox["GSD",
    ButtonData:>"GSD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GSD"],
  "."
}], "Text",
  CellTags->"DiracSlash"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracSlash"],

Cell[TextData[{
  "This is q-slash, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          RowBox[{
            SubscriptBox["q", "\[Mu]"], "."}]}], TraditionalForm]]]
}], "Text",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", "q", "]"}]], "Input",
  CellLabel->"In[476]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[476]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
      RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[477]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[477]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[478]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[478]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", "p", "]"}]], "Input",
  CellLabel->"In[479]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[479]=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", "q", "]"}], "//", "StandardForm"}]], "Input",\

  CellLabel->"In[480]:=  (21:53:10 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", "q", "]"}], "]"}]], "Output",
  CellLabel->"Out[480]//StandardForm=  (21:53:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", 
        RowBox[{"q", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[481]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracSlash"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "n"}], "]"}], ",", "n"}], "]"}]], "Output",
  CellLabel->"Out[481]//StandardForm=  (21:53:11 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSpinor", "ObjectName",
  CellTags->"DiracSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSpinor"],

Cell["\<\
DiracSpinor is simply a quantity defined as noncommutative (with \
DeclareNonCommutative[DiracSpinor]). The convention intended is that \
DiracSpinor[p, m, ind] is a Dirac spinor for a fermion with momentum p and \
mass m and indices ind.\
\>", "Text",
  CellTags->"DiracSpinor"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  "."
}], "Text",
  CellTags->"DiracSpinor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracSubstitute67", "ObjectName",
  CellTags->"DiracSubstitute67"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracSubstitute67"],

Cell["\<\
DiracSubstitute67 is an option for DiracSimplify. If set to True \
the chirality-projectors DiracGamma[6] and DiracGamma[7] are substituted by \
their definitions.\
\>", "Text",
  CellTags->"DiracSubstitute67"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracSubstitute67"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracTrace", "ObjectName",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracTrace"],

Cell["\<\
DiracTrace[expr] is the head of Dirac traces. Whether the trace is \
evaluated depends on the option DiracTraceEvaluate. Direct trace evaluation \
should be performed with Tr. The argument expr may be a product of Dirac \
matrices or slashes separated by the Mathematica Dot (.).\
\>", "Text",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input",
  CellLabel->"In[482]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
          RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
          RowBox[{"LeviCivitaSign", "\[Rule]", 
            RowBox[{"-", "1"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[482]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "For comments regarding ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "schemes see the notes for ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"DiracTrace"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"DiracTrace"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[483]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[483]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[484]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[484]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
  CellLabel->"In[485]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[485]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{
          "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
          "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[486]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[486]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{
          "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", 
            "\[Delta]", ",", "\[Tau]", ",", "5"}], "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[487]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Mu]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[487]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
  CellLabel->"In[488]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
              ")"}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[488]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[489]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[489]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[490]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrace"],

Cell[BoxData[
    RowBox[{"4", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Output",
  CellLabel->"Out[490]//StandardForm=  (21:53:11 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracTraceEvaluate", "ObjectName",
  CellTags->"DiracTraceEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracTraceEvaluate"],

Cell["\<\
DiracTraceEvaluate is an option for DiracTrace and Tr. If set to \
False, DiracTrace remains unevaluated.\
\>", "Text",
  CellTags->"DiracTraceEvaluate"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"DiracTraceEvaluate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DiracTrick", "ObjectName",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DiracTrick"],

Cell["\<\
DiracTrick[exp] contracts gamma matrices with each other and \
performs several simplifications, but no expansion, use Calc or DiracSimplify \
for non-commutative expansion.\
\>", "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracTrick", "]"}]], "Input",
  CellLabel->"In[491]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[491]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"DiracTrick"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DiracTrick"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[492]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[492]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t1", "]"}]], "Input",
  CellLabel->"In[493]:=  (21:53:11 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[493]=  (21:53:11 on 5/16/01)"]
}, Closed]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[494]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[494]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t2", "]"}]], "Input",
  CellLabel->"In[495]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[495]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[496]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[496]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "is moved to the right."
}], "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t3", "]"}]], "Input",
  CellLabel->"In[497]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[497]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
  CellLabel->"In[498]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[498]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t4", "]"}]], "Input",
  CellLabel->"In[499]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
  CellLabel->"Out[499]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", 
          RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
  CellLabel->"In[500]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"a", "+", "b"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"c", "+", "d"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[500]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
  CellLabel->"In[501]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"a", "+", "b"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"c", "+", "d"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[501]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "t5", "]"}]], "Input",
  CellLabel->"In[502]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[502]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "@@", 
      RowBox[{"Join", "[", 
        RowBox[{
          RowBox[{"{", "\[Mu]", "}"}], ",", 
          RowBox[{"Table", "[", 
            RowBox[{
              SubscriptBox["\[Nu]", "i"], ",", 
              RowBox[{"{", 
                RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[503]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "1"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "2"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "3"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "4"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "5"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "6"]], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[503]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "%", "]"}]], "Input",
  CellLabel->"In[504]:=  (21:53:12 on 5/16/01)",
  CellTags->"DiracTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "12"}], ")"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  RowBox[{
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "3"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "4"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "5"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "6"],
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[504]=  (21:53:12 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Divideout", "ObjectName",
  CellTags->"Divideout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Divideout"],

Cell["\<\
Divideout is an option for OPEInt and OPEInsert. The setting is \
divided out at the end.\
\>", "Text",
  CellTags->"Divideout"],

Cell[TextData[{
  "See also: ",
  ButtonBox["OPEInt",
    ButtonData:>"OPEInt",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEInt"],
  ", ",
  ButtonBox["OPEInsert",
    ButtonData:>"OPEInsert",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEInsert"],
  "."
}], "Text",
  CellTags->"Divideout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DOT", "ObjectName",
  CellTags->"DOT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DOT"],

Cell["\<\
DOT[a, b, ...] is the FeynCalc function for non-commutative \
multiplication. By default it is set to the Mathematica Dot functions. By \
setting
DOT=.
this can be disabled. Note that then non-commutative products should to be \
entered like DOT[ DiracMatrix[mu], m + DiracSlash[p], DiracMatrix[mu] ], etc.\
\
\>", "Text",
  CellTags->"DOT"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"DOT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DotExpand", "ObjectName",
  CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DotExpand"],

Cell["DotExpand[expr] expands DOT products in expr.", "Text",
  CellTags->"DotExpand"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DOT",
    ButtonData:>"DOT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DOT"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  ", ",
  ButtonBox["UnDeclareNonCommutative",
    ButtonData:>"UnDeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"UnDeclareNonCommutative"],
  "."
}], "Text",
  CellTags->"DotExpand"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotExpand", "[", 
      RowBox[{"DOT", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", " ", "x"}], "+", 
            RowBox[{"b", " ", "y"}], "+", 
            RowBox[{"c", " ", "z"}]}], ",", 
          RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[13]:=",
  CellTags->"DotExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", "d", " ", "x"}], "+", 
        RowBox[{"a", " ", "e", " ", "x"}], "+", 
        RowBox[{"a", " ", "f", " ", "x"}], "+", 
        RowBox[{"b", " ", "d", " ", "y"}], "+", 
        RowBox[{"b", " ", "e", " ", "y"}], "+", 
        RowBox[{"b", " ", "f", " ", "y"}], "+", 
        RowBox[{"c", " ", "d", " ", "z"}], "+", 
        RowBox[{"c", " ", "e", " ", "z"}], "+", 
        RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[13]="]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeclareNonCommutative", "/@", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], 
          "}"}]}], ";"}]], "Input",
  CellLabel->"In[14]:=",
  CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotExpand", "[", 
      RowBox[{"DOT", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", " ", "x"}], "+", 
            RowBox[{"b", " ", "y"}], "+", 
            RowBox[{"c", " ", "z"}]}], ",", 
          RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[16]:=",
  CellTags->"DotExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{"a", ".", "d"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"a", ".", "e"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"a", ".", "f"}]}], "+", 
        RowBox[{"y", " ", 
          RowBox[{"b", ".", "d"}]}], "+", 
        RowBox[{"y", " ", 
          RowBox[{"b", ".", "e"}]}], "+", 
        RowBox[{"y", " ", 
          RowBox[{"b", ".", "f"}]}], "+", 
        RowBox[{"z", " ", 
          RowBox[{"c", ".", "d"}]}], "+", 
        RowBox[{"z", " ", 
          RowBox[{"c", ".", "e"}]}], "+", 
        RowBox[{"z", " ", 
          RowBox[{"c", ".", "f"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[16]="]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"UnDeclareNonCommutative", "/@", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], 
          "}"}]}], ";"}]], "Input",
  CellLabel->"In[17]:=",
  CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotExpand", "[", 
      RowBox[{"DOT", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", " ", "x"}], "+", 
            RowBox[{"b", " ", "y"}], "+", 
            RowBox[{"c", " ", "z"}]}], ",", 
          RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[19]:=",
  CellTags->"DotExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", "d", " ", "x"}], "+", 
        RowBox[{"a", " ", "e", " ", "x"}], "+", 
        RowBox[{"a", " ", "f", " ", "x"}], "+", 
        RowBox[{"b", " ", "d", " ", "y"}], "+", 
        RowBox[{"b", " ", "e", " ", "y"}], "+", 
        RowBox[{"b", " ", "f", " ", "y"}], "+", 
        RowBox[{"c", " ", "d", " ", "z"}], "+", 
        RowBox[{"c", " ", "e", " ", "z"}], "+", 
        RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[19]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DotPower", "ObjectName",
  CellTags->"DotPower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DotPower"],

Cell["\<\
DotPower is an option for DotSimplify. It determines whether \
non-commutative powers are represented by succesive multiplication or by \
Power.\
\>", "Text",
  CellTags->"DotPower"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"DotPower"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DotProduct", "ObjectName",
  CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DotProduct"],

Cell["\<\
DotProduct[x, y] denotes the three-dimensional dot-product. If x \
and y have Head List, DotProduct[x, a] (where a is a vector) performs Sum[ \
x[[k]] a[[k]], {k, 0, 3}].\
\>", "Text",
  CellTags->"DotProduct"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CrossProduct",
    ButtonData:>"CrossProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CrossProduct"],
  ", ",
  ButtonBox["ThreeVector",
    ButtonData:>"ThreeVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ThreeVector"],
  "."
}], "Text",
  CellTags->"DotProduct"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotProduct", "[", 
      RowBox[{
        RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
        RowBox[{"3", 
          RowBox[{"ThreeVector", "[", "b", "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[61]:=  (22:47:12 on 9/20/02)",
  CellTags->"DotProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{"3", " ", 
        RowBox[{
          FormBox[
            OverscriptBox["a", "\[RightVector]"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            OverscriptBox["b", "\[RightVector]"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[61]=  (22:47:12 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DotSimplifyRelations", "ObjectName",
  CellTags->"DotSimplifyRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DotSimplifyRelations"],

Cell["\<\
DotSimplifyRelations is an option for DotSimplify. Its setting may \
be a list of substitution rules of the form DotSimplifyRelations \[Rule] {a . \
b \[Rule] c, b^2 \[Rule] 0, ...}.\
\>", "Text",
  CellTags->"DotSimplifyRelations"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text",
  CellTags->"DotSimplifyRelations"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DotSimplify", "ObjectName",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DotSimplify"],

Cell["\<\
DotSimplify[expr] expands and reorders noncommutative terms in \
expr. Simplifying relations may be specified by the option \
DotSimplifyRelations or by Commutator and AntiCommutator definitions. Whether \
expr is expanded noncommutatively depends on the option Expanding.\
\>", \
"Text",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DotSimplify", "]"}]], "Input",
  CellLabel->"In[505]:=  (21:53:12 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"DotSimplifyRelations", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"DotPower", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[505]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiCommutator",
    ButtonData:>"AntiCommutator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AntiCommutator"],
  ", ",
  ButtonBox["Commutator",
    ButtonData:>"Commutator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Commutator"],
  ", ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  "."
}], "Text",
  CellTags->"DotSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              RowBox[{"GS", "[", "p", "]"}]}], "-", 
            RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
        RowBox[{"GA", "[", "\[Nu]", "]"}]}]}]], "Input",
  CellLabel->"In[506]:=  (21:53:12 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}],
             "-", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[506]=  (21:53:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t1", "]"}]], "Input",
  CellLabel->"In[507]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
              ")"}], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          RowBox[{"(", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
            ")"}], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[507]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[508]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"a", ".", 
        RowBox[{"(", 
          RowBox[{"b", "-", 
            RowBox[{"z", " ", "c"}]}], ")"}], ".", "a"}]}]], "Input",
  CellLabel->"In[509]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", ".", 
        RowBox[{"(", 
          RowBox[{"b", "-", 
            RowBox[{"c", " ", "z"}]}], ")"}], ".", "a"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[509]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
  CellLabel->"In[510]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[510]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Commutator", "[", 
        RowBox[{"a", ",", "c"}], "]"}], "=", "1"}]], "Input",
  CellLabel->"In[511]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[511]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
  CellLabel->"In[512]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"(", 
            RowBox[{"a", "+", 
              RowBox[{"c", ".", 
                SuperscriptBox["a", "2"]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[512]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Commutator", "[", 
        RowBox[{"a", ",", "c"}], "]"}], "=", "."}]], "Input",
  CellLabel->"In[513]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
  CellLabel->"In[514]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[514]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"b", ",", "a"}], "]"}], "=", "c"}]], "Input",
  CellLabel->"In[515]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox["c", TraditionalForm]], "Output",
  CellLabel->"Out[515]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
  CellLabel->"In[516]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "c"}], "-", 
        RowBox[{
          SuperscriptBox["a", "2"], ".", "b"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[516]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"b", ",", "a"}], "]"}], "=", "."}]], "Input",
  CellLabel->"In[517]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"DotSimplifyRelations", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"a", ".", "c"}], "\[Rule]", 
              RowBox[{"1", "/", "z"}]}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[518]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", "a"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[518]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
  CellLabel->"In[519]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"x", ".", "x", ".", "x"}], "]"}]], "Input",
  CellLabel->"In[520]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["x", "3"], TraditionalForm]], "Output",
  CellLabel->"Out[520]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{
        RowBox[{"x", ".", "x", ".", "x"}], ",", 
        RowBox[{"DotPower", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[521]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
  CellLabel->"Out[521]=  (21:53:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[522]:=  (21:53:13 on 5/16/01)",
  CellTags->"DotSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["DummyIndex", "ObjectName",
  CellTags->"DummyIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DummyIndex"],

Cell["\<\
DummyIndex is an option of CovariantD specifying an index to use as \
dummy summation index. If set to Automatic, unique indices are \
generated\
\>", "Text",
  CellTags->"DummyIndex"],

Cell[TextData[{
  "See also: ",
  ButtonBox["CovariantD",
    ButtonData:>"CovariantD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CovariantD"],
  "."
}], "Text",
  CellTags->"DummyIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["D0", "ObjectName",
  CellTags->"D0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"D0"],

Cell[TextData[{
  "D0[ p10, p12, p23, p30, p20, p13, m1^2, m2^2, m3^2, m4^2 ] is the \
Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["D", "0"], TraditionalForm]]],
  " function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2] \
[(q+p3)^2-m4^2] ), the first six arguments of D0 are the scalar products p10 \
= p1^2, p12 = (p1-p2)^2, p23 = (p2-p3)^2, p30 = p3^2, p20 = p2^2, p13 = \
(p1-p3)^2."
}], "Text",
  CellTags->"D0"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text",
  CellTags->"D0"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"D0"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D0", "[", 
      RowBox[{
      "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
        
        RowBox[{"m1", "^", "2"}], ",", 
        RowBox[{"m2", "^", "2"}], ",", 
        RowBox[{"m3", "^", "2"}], ",", 
        RowBox[{"m4", "^", "2"}]}], "]"}]], "Input",
  CellLabel->"In[343]:=  (21:52:48 on 5/16/01)",
  CellTags->"D0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "10"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "12"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "p23", 
        "\[NoBreak]", ",", "\[NoBreak]", "p30", "\[NoBreak]", ",", 
        "\[NoBreak]", "p20", "\[NoBreak]", ",", "\[NoBreak]", "p13", 
        "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[343]=  (21:52:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", 
            ",", 
            RowBox[{"m1", "^", "2"}], ",", 
            RowBox[{"m2", "^", "2"}], ",", 
            RowBox[{"m3", "^", "2"}], ",", 
            RowBox[{"m4", "^", "2"}]}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"p13", ",", "p20"}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[344]:=  (21:52:48 on 5/16/01)",
  CellTags->"D0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "10"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "p30", 
        "\[NoBreak]", ",", "\[NoBreak]", "p23", "\[NoBreak]", ",", 
        "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "12"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "p13", 
        "\[NoBreak]", ",", "\[NoBreak]", "p20", "\[NoBreak]", ",", 
        "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[344]=  (21:52:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", "%", "]"}]], "Input",
  CellLabel->"In[345]:=  (21:52:48 on 5/16/01)",
  CellTags->"D0"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "10"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SubscriptBox[
            StyleBox["\<\"p\"\>",
              FontSlant->"Italic"], "12"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "p23", 
        "\[NoBreak]", ",", "\[NoBreak]", "p30", "\[NoBreak]", ",", 
        "\[NoBreak]", "p20", "\[NoBreak]", ",", "\[NoBreak]", "p13", 
        "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[345]=  (21:52:48 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["D0Convention", "ObjectName",
  CellTags->"D0Convention"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"D0Convention"],

Cell["\<\
D0Convention is an option for Write2. If set to 1, the convention \
for the arguments of D0 is changed when writing a Fortran file with Write2: \
The fifth and sixth argument of D0 are interchanged and the square root is \
taken of the last four arguments.\
\>", "Text",
  CellTags->"D0Convention"],

Cell[TextData[{
  "See also: ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  "."
}], "Text",
  CellTags->"D0Convention"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Eps", "ObjectName",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Eps"],

Cell[TextData[{
  "Eps[a, b, c, d] is the head of the totally antisymmetric ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "\[Epsilon]"}], TraditionalForm]]],
  " (Levi-Civita) tensor. The a,b, ... may have head LorentzIndex, Momentum \
or Integer. In case of integers the Levi-Civita tensor is evaluated \
immediately. "
}], "Text",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Eps", "]"}]], "Input",
  CellLabel->"In[530]:=  (21:53:15 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[530]=  (21:53:15 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["EpsEvaluate",
    ButtonData:>"EpsEvaluate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"EpsEvaluate"],
  ", ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LC"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LCD"],
  ", ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeviCivita"],
  "."
}], "Text",
  CellTags->"Eps"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[531]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[531]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[532]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[532]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Eps", "[", 
        RowBox[{"b", ",", "a", ",", "c", ",", "d"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[533]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"Eps", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]}]], "Output",
  CellLabel->"Out[533]//StandardForm=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "]"}]], "Input",
  CellLabel->"In[534]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[534]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{"1", ",", "0", ",", "2", ",", "3"}], "]"}]], "Input",
  CellLabel->"In[535]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[535]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[536]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[537]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[537]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"a1", "a1"}], "]"}]], "Input",
  CellLabel->"In[538]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output",
  CellLabel->"Out[538]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[539]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[540]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[540]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{"a2", "a2"}], "]"}], "//", "Factor2"}]], "Input",
  CellLabel->"In[541]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[541]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g5", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["\[ImaginaryI]", "24"]}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}],
               "]"}], ".", 
            RowBox[{"GAD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}],
               "]"}]}]}], "//", "FCI"}]}]], "Input",
  CellLabel->"In[542]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[542]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g5p", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["\[ImaginaryI]", "24"]}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None]}], "]"}], ".", 
            RowBox[{"GAD", "[", 
              RowBox[{
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None]}], "]"}]}]}], "//", 
        "FCI"}]}]], "Input",
  CellLabel->"In[543]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox[
                  SuperscriptBox["\[Mu]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Nu]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Rho]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Alpha]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"]}]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[543]=  (21:53:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g52", "=", 
      RowBox[{"Factor2", "[", 
        RowBox[{"Calc", "[", 
          RowBox[{"g5", ".", "g5p"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[544]:=  (21:53:16 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[544]=  (21:53:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g52", "/.", 
      RowBox[{"D", "\[Rule]", "4"}]}]], "Input",
  CellLabel->"In[545]:=  (21:53:18 on 5/16/01)",
  CellTags->"Eps"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[545]=  (21:53:18 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "g5", ",", "g5p", ",", "g52"}], 
      "]"}]], "Input",
  CellLabel->"In[546]:=  (21:53:18 on 5/16/01)",
  CellTags->"Eps"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsChisholm", "ObjectName",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsChisholm"],

Cell["\<\
EpsChisholm[expr] substitutes for a gamma matrix contracted with a \
Levi-Civita tensor (Eps) the Chisholm identity.\
\>", "Text",
  CellTags->"EpsChisholm"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Chisholm",
    ButtonData:>"Chisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Chisholm"],
  "."
}], "Text",
  CellTags->"EpsChisholm"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[523]:=  (21:53:14 on 5/16/01)",
  CellTags->"EpsChisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$200",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$200",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[523]=  (21:53:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"EpsChisholm", "[", "%", "]"}]], "Input",
  CellLabel->"In[524]:=  (21:53:14 on 5/16/01)",
  CellTags->"EpsChisholm"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[524]=  (21:53:15 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsContract", "ObjectName",
  CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsContract"],

Cell["\<\
EpsContract is an option of Contract specifying whether Levi-Civita \
tensors Eps[...] will be contracted, i.e., products of two  Eps are replaced \
via the determinant formula.\
\>", "Text",
  CellTags->"EpsContract"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  "."
}], "Text",
  CellTags->"EpsContract"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[26]:=  (23:27:00 on 9/2/02)",
  CellTags->"EpsContract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (23:27:00 on 9/2/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"a1", " ", "a1"}], ",", 
        RowBox[{"EpsContract", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[27]:=  (23:27:02 on 9/2/02)",
  CellTags->"EpsContract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ")"}], "2"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[27]=  (23:27:02 on 9/2/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"a1", " ", "a1"}], ",", 
        RowBox[{"EpsContract", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[28]:=  (23:27:03 on 9/2/02)",
  CellTags->"EpsContract"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output",
  CellLabel->"Out[28]=  (23:27:03 on 9/2/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "a1", "]"}]], "Input",
  CellLabel->"In[29]:=  (23:27:04 on 9/2/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsDiscard", "ObjectName",
  CellTags->"EpsDiscard"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsDiscard"],

Cell["\<\
EpsDiscard is an option for FeynCalc2FORM. If set to True all \
Levi-Civita tensors are replaced by 0 after contraction.\
\>", "Text",
  CellTags->"EpsDiscard"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  "."
}], "Text",
  CellTags->"EpsDiscard"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsEvaluate", "ObjectName",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsEvaluate"],

Cell["\<\
EpsEvaluate[expr] applies total antisymmetry and linearity (w.r.t. \
Momentum's) to all Levi-Civita tensors (Eps') in expr.\
\>", "Text",
  CellTags->"EpsEvaluate"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  "."
}], "Text",
  CellTags->"EpsEvaluate"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
          "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{
            RowBox[{"p", "+", "q"}], ",", "\[Sigma]"}], "]"}]}], 
      "]"}]], "Input",
  CellLabel->"In[525]:=  (21:53:15 on 5/16/01)",
  CellTags->"EpsEvaluate"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            RowBox[{"p", "+", "q"}],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[525]=  (21:53:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"EpsEvaluate", "[", "%", "]"}]], "Input",
  CellLabel->"In[526]:=  (21:53:15 on 5/16/01)",
  CellTags->"EpsEvaluate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], "+", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[526]=  (21:53:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[527]:=  (21:53:15 on 5/16/01)",
  CellTags->"EpsEvaluate"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "+", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
          RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[527]//StandardForm=  (21:53:15 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsUncontract", "ObjectName",
  CellTags->"EpsUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsUncontract"],

Cell["\<\
EpsUncontract does Uncontract on scalar products involving \
Eps.\
\>", "Text",
  CellTags->"EpsUncontract"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["Uncontract",
    ButtonData:>"Uncontract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Uncontract"],
  "."
}], "Text",
  CellTags->"EpsUncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Epsilon", "ObjectName",
  CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Epsilon"],

Cell[TextData[{
  "Epsilon is (",
  StyleBox["n",
    FontSlant->"Italic"],
  "-4), where ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is the space-time dimension. Epsilon stands for a small positive number."
}], "Text",
  CellTags->"Epsilon"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Series2"],
  "."
}], "Text",
  CellTags->"Epsilon"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData["Epsilon"], "Input",
  CellLabel->"In[528]:=  (21:53:15 on 5/16/01)",
  CellTags->"Epsilon"],

Cell[BoxData[
    FormBox[
      TagBox["\[CurlyEpsilon]",
        TraditionalForm], TraditionalForm]], "Output",
  CellLabel->"Out[528]=  (21:53:15 on 5/16/01)"]
}, Closed]],

Cell["\<\
Epsilon has no functional properties, but some upvalues are \
changed:\
\>", "Text",
  CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "4"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "3"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "2"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "1"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", "0"}]}], 
      "}"}]], "Input",
  CellLabel->"In[529]:=  (21:53:15 on 5/16/01)",
  CellTags->"Epsilon"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[529]=  (21:53:15 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["EpsilonOrder", "ObjectName",
  CellTags->"EpsilonOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"EpsilonOrder"],

Cell[TextData[{
  "EpsilonOrder is an option of OPEIntDelta and RHI. The setting determines \
the order ",
  "n",
  " (",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["Epsilon", 
          StyleBox["n",
            FontSlant->"Plain"]], TraditionalForm]]],
  ") which should be kept."
}], "Text",
  CellTags->"EpsilonOrder"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEIntDelta",
    ButtonData:>"OPEIntDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEIntDelta"],
  ", ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"EpsilonOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Expanding", "ObjectName",
  CellTags->"Expanding"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Expanding"],

Cell["\<\
Expanding is an option for Calc, Contract, \
DiracSimplify,DotSimplify, SUNSimplify, etc. As option for Contract it \
specifies whether expansion w.r.t. LorentzIndex is done BEFORE contraction. \
If set to False in DiracSimplify or SUNSimplify, only a limited set of \
simplifications (multiplicative linearity etc.) is performed.\
\>", "Text",
  CellTags->"Expanding"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"Expanding"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExpandPartialD", "ObjectName",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExpandPartialD"],

Cell["\<\
ExpandPartialD[exp] expands all products of QuantumField's and \
partial differentiation operators in exp and applies the Leibniz rule.\
\>", \
"Text",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "ExpandPartialD", "]"}]], "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PartialDRelations", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
              "HighEnergyPhysics`fctools`ExpandPartialD`Private`a___", ".", 
                RowBox[{"\[PartialD]", "/", "\[PartialD]", 
                  SuperscriptBox[
                    
                    FormBox[
                      "HighEnergyPhysics`fctools`ExpandPartialD`Private`x_",
                      "TraditionalForm"], 
                    FormBox[
                      
                      FormBox[
                        "HighEnergyPhysics`fctools`ExpandPartialD`Private`mu_\
",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ".", 
                "HighEnergyPhysics`fctools`ExpandPartialD`Private`b___"}], 
              "\[RuleDelayed]", 
              RowBox[{"DOT", "(", 
                RowBox[{
                "HighEnergyPhysics`fctools`ExpandPartialD`Private`a", ",", 
                  RowBox[{
                  "HighEnergyPhysics`fctools`ExpandPartialD`Private`\
FieldDerivative", "(", 
                    RowBox[{
                      RowBox[{
                      "DOT", "(", 
                        "HighEnergyPhysics`fctools`ExpandPartialD`Private`b", 
                        ")"}], ",", 
                      "HighEnergyPhysics`fctools`ExpandPartialD`Private`x", 
                      ",", 
                      RowBox[{
                      "HighEnergyPhysics`fctools`ExpandPartialD`Private`\
LorentzIndex", "(", "HighEnergyPhysics`fctools`ExpandPartialD`Private`mu", 
                        ")"}]}], ")"}]}], ")"}]}], "}"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[5]="]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["PartialDRelations",
    ButtonData:>"PartialDRelations",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialDRelations"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"ExpandPartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[2]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[2]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[3]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], ".", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[3]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[4]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], 
          "]"}]}]}]], "Output",
  CellLabel->"Out[4]//StandardForm="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[5]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[5]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[6]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], "2"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["A", "\[Nu]"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[6]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "2"}], 
        ")"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[7]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "2"], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[7]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[8]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    SubscriptBox["\[PartialD]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]],
                    "TraditionalForm"], 
                  FormBox[
                    SubscriptBox["\[PartialD]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]],
                    "TraditionalForm"]}], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}]}]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    SubscriptBox["\[PartialD]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]],
                    "TraditionalForm"], 
                  FormBox[
                    SubscriptBox["\[PartialD]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]],
                    "TraditionalForm"]}], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], ".", 
            SubscriptBox["A", "\[Rho]"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[8]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"8", " ", 
      RowBox[{
        RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", 
        "3"}]}]], "Input",
  CellLabel->"In[9]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{"8", " ", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "3"]}], TraditionalForm]], "Output",
  CellLabel->"Out[9]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[10]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[10]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[11]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}]}], "+", 
        RowBox[{"3", " ", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}]}], "-", 
        RowBox[{"3", " ", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ".", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}]}], "+", 
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[11]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Tau]"}], "]"}], 
      " ", 
      RowBox[{"RightPartialD", "[", 
        RowBox[{"\[Alpha]", ",", "\[Mu]", ",", "\[Beta]", ",", "\[Nu]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[12]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Tau]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[12]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[13]:=",
  CellTags->"ExpandPartialD"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[13]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExpandScalarProduct, ScalarProductExpand", "ObjectName",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell["\<\
ExpandScalarProduct[expr] expands scalar products of sums of \
momenta in expr. ExpandScalarProduct does not use Expand on expr.
ScalarProductExpand=ExpandScalarProduct.\
\>", "Text",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "//", "Options"}]], "Input",
  CellLabel->"In[11]:=  (20:59:22 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[11]=  (20:59:22 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumExpand"],
  ", ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumCombine"],
  "."
}], "Text",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"SP", "[", 
        RowBox[{
          RowBox[{"p1", "+", "p2", "+", "p3"}], ",", 
          RowBox[{"p4", "+", "p5", "+", "p6"}]}], "]"}]}]], "Input",
  CellLabel->"In[6]:=  (20:54:53 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p1", "+", "p2", "+", "p3"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p4", "+", "p5", "+", "p6"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[6]=  (20:54:53 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "ScalarProductExpand"}]], "Input",
  CellLabel->"In[7]:=  (20:54:55 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p4",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p5",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p6",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p2",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p4",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p2",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p5",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p2",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p6",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p3",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p4",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p3",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p5",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p3",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p6",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[7]=  (20:54:55 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"p", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
  CellLabel->"In[559]:=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[559]=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
  CellLabel->"In[560]:=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[560]=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[561]:=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[561]=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
  CellLabel->"In[562]:=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[562]=  (21:53:21 on 5/16/01)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "-", "r"}]}], "]"}], "//", "FCI"}]], "Input",
  CellLabel->"In[9]:=  (20:57:36 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"q", "-", "r"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[9]=  (20:57:36 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"Pair", "\[Rule]", "ExpandScalarProduct"}]}]], "Input",
  CellLabel->"In[10]:=  (20:57:38 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[10]=  (20:57:38 on 9/20/02)",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Expand2", "ObjectName",
  CellTags->"Expand2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Expand2"],

Cell["\<\
Expand2[exp, x] expands all sums containing x. Expand2[exp, {x1, \
x2, ...}]  expands all sums containing x1, x2, ....\
\>", "Text",
  CellTags->"Expand2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Expand2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Expand2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"x1", "+", "x2", "+", "x3"}], ")"}], 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", "x1"}], "+", 
                RowBox[{"3", "x2"}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}], 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", "y1"}], "+", 
                RowBox[{"3", "y2"}]}], ")"}]}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x1", ",", "x2"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[12]:=  (23:18:04 on 9/19/02)",
  CellTags->"Expand2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["x1", "2"]}], "+", 
        RowBox[{"5", " ", "x2", " ", "x1"}], "+", 
        RowBox[{"2", " ", "x3", " ", "x1"}], "+", 
        RowBox[{"3", " ", 
          SuperscriptBox["x2", "2"]}], "+", 
        RowBox[{"3", " ", "x2", " ", "x3"}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"2", " ", "y1"}], "+", 
              RowBox[{"3", " ", "y2"}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[12]=  (23:18:04 on 9/19/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Explicit", "ObjectName",
  CellTags->"Explicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Explicit"],

Cell[TextData[{
  "Explicit is an option for FieldStrength, GluonVertex, SUNF, and \
Twist2GluonOperator. If set to True the full form of the operator is \
inserted. ",
  StyleBox["Explicit[exp]",
    FontVariations->{"CompatibilityType"->0}],
  " inserts explicit expressions of GluonVertex, Twist2GluonOperator, etc., \
in exp. SUNF's are replaced by SUNTrace objects."
}], "Text",
  CellTags->"Explicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text",
  CellTags->"Explicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
      "p", ",", "\[Mu]", ",", "a", ",", " ", "q", ",", "\[Nu]", ",", "b", ",",
         "r", ",", "\[Rho]", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[565]:=  (21:53:21 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[565]=  (21:53:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[566]:=  (21:53:22 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[566]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
      "]"}]], "Input",
  CellLabel->"In[567]:=  (21:53:22 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[567]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[568]:=  (21:53:22 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[568]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
  CellLabel->"In[569]:=  (21:53:22 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "a"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[569]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input",
  CellLabel->"In[570]:=  (21:53:22 on 5/16/01)",
  CellTags->"Explicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b1",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c10",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b1", "\[NoBreak]", "c10"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[570]=  (21:53:22 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExplicitLorentzIndex", "ObjectName",
  CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExplicitLorentzIndex"],

Cell["\<\
ExplicitLorentzIndex[ind] is an explicit Lorentz index, i.e., ind \
is an integer.\
\>", "Text",
  CellTags->"ExplicitLorentzIndex"],

Cell[TextData[{
  "See also: ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  "."
}], "Text",
  CellTags->"ExplicitLorentzIndex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "1", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[6]:=  (11:30:26 on 9/3/02)",
  CellTags->"ExplicitLorentzIndex"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox["1",
            "TraditionalForm"], "\[NoBreak]", "\[Mu]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[6]=  (11:30:26 on 9/3/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[7]:=  (11:30:28 on 9/3/02)",
  CellTags->"ExplicitLorentzIndex"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"ExplicitLorentzIndex", "[", "1", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[7]//StandardForm=  (11:30:28 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExplicitPartialD", "ObjectName",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExplicitPartialD"],

Cell["\<\
ExplicitPartialD[exp] inserts in exp the definition for \
LeftRightPartialD[z] (and LeftRightPartialD2[z]).\
\>", "Text",
  CellTags->"ExplicitPartialD"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD2",
    ButtonData:>"LeftRightPartialD2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD2"],
  "."
}], "Text",
  CellTags->"ExplicitPartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
  CellLabel->"In[571]:=  (21:53:22 on 5/16/01)",
  CellTags->"ExplicitPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[571]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",\

  CellLabel->"In[572]:=  (21:53:22 on 5/16/01)",
  CellTags->"ExplicitPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[572]=  (21:53:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], " ", 
      "]"}]], "Input",
  CellLabel->"In[573]:=  (21:53:23 on 5/16/01)",
  CellTags->"ExplicitPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[573]=  (21:53:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"16", " ", 
      RowBox[{
        RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", 
        "4"}]}]], "Input",
  CellLabel->"In[61]:=",
  CellTags->"ExplicitPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{"16", " ", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "4"]}], TraditionalForm]], "Output",
  CellLabel->"Out[61]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[62]:=",
  CellTags->"ExplicitPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[62]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExplicitSUNIndex", "ObjectName",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
  "ExplicitSUNIndex[ind] is a specific SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") index, i.e., ind is an integer."
}], "Text",
  CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  "."
}], "Text",
  CellTags->"ExplicitSUNIndex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitSUNIndex", "[", "1", "]"}]], "Input",
  CellLabel->"In[575]:=  (21:53:23 on 5/16/01)",
  CellTags->"ExplicitSUNIndex"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[575]=  (21:53:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"1", ",", "a"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[2]:=  (11:45:03 on 9/3/02)"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[2]//StandardForm=  (11:45:03 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExtraFactor", "ObjectName",
  CellTags->"ExtraFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExtraFactor"],

Cell["\<\
ExtraFactor is an option for FermionSpinSum. The setting \
ExtraFactor \[Rule] fa  multiplies the whole amplitude with the factor fa \
before squaring.\
\>", "Text",
  CellTags->"ExtraFactor"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  "."
}], "Text",
  CellTags->"ExtraFactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ExtraVariables", "ObjectName",
  CellTags->"ExtraVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ExtraVariables"],

Cell["\<\
ExtraVariables is an option for OneLoopSum; it may be set to a list \
of variables which are also bracketed out in the result, just like B0, C0, D0 \
and  PaVe.\
\>", "Text",
  CellTags->"ExtraVariables"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"ExtraVariables"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FactorFull", "ObjectName",
  CellTags->"FactorFull"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FactorFull"],

Cell[TextData[{
  "FactorFull is an option of Factor2 (default False). If set to False, \
products like (",
  StyleBox["a",
    FontSlant->"Italic"],
  "-",
  StyleBox["b",
    FontSlant->"Italic"],
  ") (",
  StyleBox["a",
    FontSlant->"Italic"],
  "+",
  StyleBox["b",
    FontSlant->"Italic"],
  ") will be replaced by (",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["a", "2"], TraditionalForm]]],
  "-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["b", "2"], TraditionalForm]]],
  ")."
}], "Text",
  CellTags->"FactorFull"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Factor2",
    ButtonData:>"Factor2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Factor2"],
  "."
}], "Text",
  CellTags->"FactorFull"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Factoring", "ObjectName",
  CellTags->"Factoring"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Factoring"],

Cell["\<\
Factoring is an option for Collect2, Contract, Tr and more \
functions. If set to True, the result will be factored, using Factor2. If set \
to any function f, this function will be used.\
\>", "Text",
  CellTags->"Factoring"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Collect2"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"Factoring"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Factorout", "ObjectName",
  CellTags->"Factorout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Factorout"],

Cell["Factorout is an option for OPEInt.", "Text",
  CellTags->"Factorout"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEInt",
    ButtonData:>"OPEInt",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEInt"],
  "."
}], "Text",
  CellTags->"Factorout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FactorTime", "ObjectName",
  CellTags->"FactorTime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FactorTime"],

Cell["\<\
FactorTime is an option for Factor2. It denotes the maximum time \
(in seconds) during which Factor2 tries to factor.\
\>", "Text",
  CellTags->"FactorTime"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Factor2",
    ButtonData:>"Factor2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Factor2"],
  "."
}], "Text",
  CellTags->"FactorTime"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Factor1", "ObjectName",
  CellTags->"Factor1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Factor1"],

Cell["\<\
Factor1[poly] factorizes common terms  in the summands of poly. It \
uses basically PolynomialGCD.\
\>", "Text",
  CellTags->"Factor1"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Factor2",
    ButtonData:>"Factor2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Factor2"],
  "."
}], "Text",
  CellTags->"Factor1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Factor1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
  CellLabel->"In[22]:=  (23:24:33 on 9/19/02)",
  CellTags->"Factor1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[22]=  (23:24:33 on 9/19/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factor1", "[", "t1", "]"}], ",", " ", 
          RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
  CellLabel->"In[24]:=  (23:24:51 on 9/19/02)",
  CellTags->"Factor1"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "x"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "x"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "x"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "x"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{"a", "-", "x"}], ")"}]}], " ", 
            RowBox[{"(", 
              RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (23:24:51 on 9/19/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"Expand", "[", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", "b"}], ")"}], 
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
  CellLabel->"In[25]:=  (23:25:00 on 9/19/02)",
  CellTags->"Factor1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[25]=  (23:25:00 on 9/19/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor", "[", "t3", "]"}]], "Input",
  CellLabel->"In[26]:=  (23:25:04 on 9/19/02)",
  CellTags->"Factor1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "b"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (23:25:04 on 9/19/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor1", "[", "t3", "]"}]], "Input",
  CellLabel->"In[28]:=  (23:25:11 on 9/19/02)",
  CellTags->"Factor1"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[28]=  (23:25:11 on 9/19/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
  CellLabel->"In[29]:=  (23:25:22 on 9/19/02)",
  CellTags->"Factor1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Factor2", "ObjectName",
  CellTags->"Factor2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Factor2"],

Cell["\<\
Factor2[poly] factors a polynomial in a standard way. Factor2 works \
sometimes better than Factor on polynomials involving rationals with sums in \
the denominator. Factor2 uses Factor internally and is in general slower than \
Factor. There are four possible settings of the option Method (0,1,2,3). In \
general Factor will work faster than Factor2.\
\>", "Text",
  CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Factor2", "]"}]], "Input",
  CellLabel->"In[577]:=  (21:53:23 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FactorFull", "\[Rule]", "False"}], ",", 
          RowBox[{"Method", "\[Rule]", "3"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[577]=  (21:53:23 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Collect2"],
  "."
}], "Text",
  CellTags->"Factor2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
  CellLabel->"In[578]:=  (21:53:23 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[578]=  (21:53:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factor2", "[", "t1", "]"}], ",", " ", 
          RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
  CellLabel->"In[579]:=  (21:53:23 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "x"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "x"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{"a", "-", "x"}], ")"}]}], " ", 
            RowBox[{"(", 
              RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[579]=  (21:53:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"Expand", "[", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", "b"}], ")"}], 
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
  CellLabel->"In[580]:=  (21:53:23 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[580]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor", "[", "t3", "]"}]], "Input",
  CellLabel->"In[581]:=  (21:53:24 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "b"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[581]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", "t3", "]"}]], "Input",
  CellLabel->"In[582]:=  (21:53:24 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[582]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"t3", ",", 
        RowBox[{"FactorFull", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[583]:=  (21:53:24 on 5/16/01)",
  CellTags->"Factor2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "b"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[583]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
  CellLabel->"In[584]:=  (21:53:24 on 5/16/01)",
  CellTags->"Factor2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FAD", "ObjectName",
  CellTags->"FAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FAD"],

Cell["\<\
FAD is the FeynCalc external form of FeynAmpDenominator and denotes \
an inverse propagator. FAD[q, q-p, ...] is 1/(q^2 (q-p)^2 ...). FAD[{q1,m}, \
{q1-p,m}, q2, ...] is 1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... ). \
Translation into FeynCalc internal form is performed by \
FeynCalcInternal.\
\>", "Text",
  CellTags->"FAD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text",
  CellTags->"FAD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FAD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
  CellLabel->"In[585]:=  (21:53:24 on 5/16/01)",
  CellTags->"FAD"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ p, 
                    Times[ -1, q]], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[585]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"p", "-", "q"}], ",", "m"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[586]:=  (21:53:24 on 5/16/01)",
  CellTags->"FAD"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["p", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ p, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ p, 
                    Times[ -1, q]], 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[586]=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{"q", ",", 
              RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input",
  CellLabel->"In[587]:=  (21:53:24 on 5/16/01)",
  CellTags->"FAD"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Output",
  CellLabel->"Out[587]//StandardForm=  (21:53:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"q", ",", 
            RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[588]:=  (21:53:24 on 5/16/01)",
  CellTags->"FAD"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", 
              RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"p", ",", "D"}], "]"}], "-", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], 
      "]"}]], "Output",
  CellLabel->"Out[588]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", "p", "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{"p", "-", "q"}], "]"}]}], " ", "//", " ", 
        "FeynAmpDenominatorCombine"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[589]:=  (21:53:25 on 5/16/01)",
  CellTags->"FAD"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", 
              RowBox[{"p", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"p", ",", "D"}], "]"}], "-", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], 
      "]"}]], "Output",
  CellLabel->"Out[589]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FC", "ObjectName",
  CellTags->"FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FC"],

Cell["\<\
FC changes the output format to FeynCalcForm. To change to \
InputForm use FI.\
\>", "Text",
  CellTags->"FC"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcForm",
    ButtonData:>"FeynCalcForm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcForm"],
  ", ",
  ButtonBox["FI",
    ButtonData:>"FI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FI"],
  ", ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text",
  CellTags->"FC"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FC"],

Cell[CellGroupData[{

Cell[BoxData["FI"], "Input",
  CellLabel->"In[23]:=  (13:15:48 on 9/3/02)",
  CellTags->"FC"],

Cell["InputForm", "Output",
  CellLabel->"Out[23]=  (13:15:48 on 9/3/02)",
  CellTags->"FC"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[24]:=  (13:15:53 on 9/3/02)",
  CellTags->"FC"],

Cell["{DiracGamma[5], DiracGamma[Momentum[p]]}", "Output",
  CellLabel->"Out[24]=  (13:15:53 on 9/3/02)",
  CellTags->"FC"]
}, Closed]],

Cell[BoxData["FC"], "Input",
  CellLabel->"In[25]:=  (13:15:54 on 9/3/02)",
  CellTags->"FC"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[26]:=  (13:15:55 on 9/3/02)",
  CellTags->"FC"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (13:15:55 on 9/3/02)",
  CellTags->"FC"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FCE", "ObjectName",
  CellTags->"FCE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FCE"],

Cell["\<\
FCE[exp] translates exp from the internal FeynCalc representation \
to a short form.\
\>", "Text",
  CellTags->"FCE"],

Cell["FCE is equivalent to FeynCalcExternal.", "Text",
  CellTags->"FCE"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text",
  CellTags->"FCE"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FCE"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCE", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}], 
      "]"}]], "Input",
  CellLabel->"In[590]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[590]=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[591]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "5", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}]}], "}"}]], "Output",
  CellLabel->"Out[591]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[592]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], ",", 
          
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[592]=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[593]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[593]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input",
  CellLabel->"In[594]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[594]=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[595]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[595]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[596]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCE"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[596]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FCI", "ObjectName",
  CellTags->"FCI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FCI"],

Cell["\<\
FCI[exp] translates exp into the internal FeynCalc \
(datatype-)representation.FCI is equivalent to FeynCalcInternal.\
\>", "Text",\

  CellTags->"FCI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  "."
}], "Text",
  CellTags->"FCI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FCI"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[597]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCI"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], ",", 
          
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[597]=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[598]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCI"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[598]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input",
  CellLabel->"In[599]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCI"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[599]=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[600]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCI"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[600]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[601]:=  (21:53:25 on 5/16/01)",
  CellTags->"FCI"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[601]//StandardForm=  (21:53:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FCIntegral", "ObjectName",
  CellTags->"FCIntegral"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FCIntegral"],

Cell["\<\
FCIntegral is the head of integrals in a setting of the option \
IntegralTable of FeynAmpDenominatorSimplify. Currently implemented only for \
2-loop integrals.\
\>", "Text",
  CellTags->"FCIntegral"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["IntegralTable",
    ButtonData:>"IntegralTable",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IntegralTable"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text",
  CellTags->"FCIntegral"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FCIntegrate", "ObjectName",
  CellTags->"FCIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FCIntegrate"],

Cell["\<\
FCIntegrate is an option of certain Feynman integral related \
functions. It determines which integration function is used to evaluate \
analytic integrals. Possible settings include Integrate, NIntegrate, \
(Dot[Integratedx@@#2, #1] &).\
\>", "Text",
  CellTags->"FCIntegrate"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCNIntegrate",
    ButtonData:>"FCNIntegrate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCNIntegrate"],
  "."
}], "Text",
  CellTags->"FCIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FCNIntegrate", "ObjectName",
  CellTags->"FCNIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FCNIntegrate"],

Cell["\<\
FCNIntegrate is an option of certain Feynman integral related \
functions which may return output containing both integrals that can be \
evaluated and integrals that can only be evaluated numerically. It then \
determines which integration function is used to evaluate numeric integrals. \
Possible settings include NIntegrate, (0*#1)&, (Dot[Integratedx@@#2, #1] \
&).\
\>", "Text",
  CellTags->"FCNIntegrate"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCIntegrate",
    ButtonData:>"FCIntegrate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCIntegrate"],
  "."
}], "Text",
  CellTags->"FCNIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FC2RHI ***unfinished***", "ObjectName",
  CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FC2RHI"],

Cell["\<\
FC2RHI[exp, k1, k2] transforms all 2-loop OPE-integrals in \
FeynAmpDenominator form to the RHI-integrals.  FC2RHI[exp] is equivalent to \
FC2RHI[exp,q1,q2]. The option IncludePair governs the inclusion  of scalar \
products p.k1, p.k2 and k1.k2 (setting True).\
\>", "Text",
  CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FC2RHI", "]"}]], "Input",
  CellLabel->"In[31]:=  (23:27:47 on 9/19/02)",
  CellTags->"FC2RHI"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
          RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[31]=  (23:27:47 on 9/19/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"FC2RHI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FC2RHI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[607]:=  (21:53:26 on 5/16/01)",
  CellTags->"FC2RHI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FC2TLI ***unfinished***", "ObjectName",
  CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FC2TLI"],

Cell["\<\
FC2TLI[exp, k1, k2] transforms all 2-loop OPE-integrals in \
FeynAmpDenominator form to the TLI-integrals.  The option IncludePair governs \
the inclusion  of scalar products p.k1, p.k2 and k1.k2 (setting True).\
\>", \
"Text",
  CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FC2TLI", "]"}]], "Input",
  CellLabel->"In[33]:=  (23:31:04 on 9/19/02)",
  CellTags->"FC2TLI"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
          RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[33]=  (23:31:04 on 9/19/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  "."
}], "Text",
  CellTags->"FC2TLI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FC2TLI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[607]:=  (21:53:26 on 5/16/01)",
  CellTags->"FC2TLI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FDS", "ObjectName",
  CellTags->"FDS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FDS"],

Cell["FDS is shorthand for FeynAmpDenominatorSimplify.", "Text",
  CellTags->"FDS"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text",
  CellTags->"FDS"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FermionSpinSum", "ObjectName",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FermionSpinSum"],

Cell["\<\
FermionSpinSum[x] constructs the Traces out of squared ampliudes.\
\
\>", "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FermionSpinSum", "]"}]], "Input",
  CellLabel->"In[602]:=  (21:53:25 on 5/16/01)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"SpinPolarizationSum", "\[Rule]", "Identity"}], ",", 
          RowBox[{"SpinorCollect", "\[Rule]", "False"}], ",", 
          RowBox[{"ExtraFactor", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[602]=  (21:53:26 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["ComplexConjugate",
    ButtonData:>"ComplexConjugate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ComplexConjugate"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FermionSpinSum"],

Cell[TextData[{
  "Spinors of fermions of mass ",
  StyleBox["m",
    FontSlant->"Italic"],
  " are normalized to have square 2 ",
  StyleBox["m",
    FontSlant->"Italic"],
  " or -2 ",
  StyleBox["m",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorUBar", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
      RowBox[{"SpinorU", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
  CellLabel->"In[27]:=  (13:14:47 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"u\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"u", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[27]=  (13:14:47 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
  CellLabel->"In[28]:=  (13:14:48 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{"m", "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[28]=  (13:14:48 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
  CellLabel->"In[29]:=  (13:14:49 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "m"}], TraditionalForm]], "Output",
  CellLabel->"Out[29]=  (13:14:49 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorVBar", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
      RowBox[{"SpinorV", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
  CellLabel->"In[24]:=  (13:14:32 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"v\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"v", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (13:14:32 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
  CellLabel->"In[25]:=  (13:14:33 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], "-", "m"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[25]=  (13:14:33 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
  CellLabel->"In[26]:=  (13:14:34 on 9/15/02)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", "m"}], TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (13:14:34 on 9/15/02)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"Spinor", "[", 
          RowBox[{"k1", ",", "m"}], "]"}], ".", 
        RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}], ".", 
        RowBox[{"Spinor", "[", 
          RowBox[{"p1", ",", "m"}], "]"}]}]}]], "Input",
  CellLabel->"In[603]:=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["k1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[603]=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. \
Internally they are converted to Spinor objects.\
\>", "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ct", "=", 
      RowBox[{"ComplexConjugate", "[", "t", "]"}]}]], "Input",
  CellLabel->"In[604]:=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox["p1",
              "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
            "\[NoBreak]", ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox["k1",
              "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
            "\[NoBreak]", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[604]=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FermionSpinSum", "[", 
      RowBox[{"t", " ", "ct"}], "]"}]], "Input",
  CellLabel->"In[605]:=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"m", "+", 
                  RowBox[{
                    FormBox["\<\"\[Gamma]\"\>",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["k1",
                      "TraditionalForm"]}]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], ".", 
              SuperscriptBox["\[Gamma]", "5"], ".", 
              RowBox[{"(", 
                RowBox[{"m", "+", 
                  RowBox[{
                    FormBox["\<\"\[Gamma]\"\>",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p1",
                      "TraditionalForm"]}]}], ")"}], ".", 
              SuperscriptBox["\[Gamma]", "5"], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}]}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[605]=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", 
      RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
  CellLabel->"In[606]:=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p1",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}], "-", 
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p1",
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[606]=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", " ", "ct"}], "]"}]], "Input",
  CellLabel->"In[607]:=  (21:53:26 on 5/16/01)",
  CellTags->"FermionSpinSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmp", "ObjectName",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmp"],

Cell["\<\
FeynAmp[q, amp] is the head of a Feynman amplitude. amp denotes the \
analytical expression for the amplitude and q is the integration variable. \
FeynAmp[q1, q2, amp] denotes a two-loop amplitude.\
\>", "Text",
  CellTags->"FeynAmp"],

Cell["\<\
FeynAmp has no functional properties and serves just as a head. \
There are however special typesetting rules attached.\
\>", "Text",
  CellTags->"FeynAmp"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Amplitude",
    ButtonData:>"Amplitude",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Amplitude"],
  "."
}], "Text",
  CellTags->"FeynAmp"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynAmp"],

Cell["\<\
This is a 1-loop gluon self-energy amplitude (ignoring factors of \
(2 \[Pi])).\
\>", "Text",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{"q", ",", 
        RowBox[{
          RowBox[{"GV", "[", 
            RowBox[{"p", ",", "\[Mu]", ",", "a", ",", " ", 
              RowBox[{"q", "-", "p"}], ",", "\[Alpha]", ",", "c", ",", " ", 
              RowBox[{"-", "q"}], ",", "\[Beta]", ",", "e"}], "]"}], " ", 
          RowBox[{"GP", "[", 
            RowBox[{
              RowBox[{"p", "-", "q"}], ",", " ", "\[Alpha]", ",", "c", ",", 
              " ", "\[Rho]", ",", "d"}], "]"}], 
          RowBox[{"GV", "[", 
            RowBox[{
              RowBox[{"-", "p"}], ",", "\[Nu]", ",", "b", ",", " ", 
              RowBox[{"p", "-", "q"}], ",", "\[Rho]", ",", "d", ",", " ", "q",
               ",", "\[Sigma]", ",", "f"}], "]"}], " ", 
          RowBox[{"GP", "[", 
            RowBox[{
            "q", ",", " ", "\[Beta]", ",", "e", ",", " ", "\[Sigma]", ",", 
              "f"}], "]"}]}]}], "]"}]], "Input",
  CellLabel->"In[632]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynAmp"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        StyleBox[
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], "q"}],
          ZeroWidthTimes->True], "(", 
        FormBox[
          RowBox[{"-", 
            FractionBox[
              RowBox[{
                SubsuperscriptBox["g", "s", "2"], " ", 
                RowBox[{
                  SuperscriptBox["V", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], "(", 
                  RowBox[{
                    FormBox[
                      RowBox[{"-", "p"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"p", "-", "q"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["V", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Alpha]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], "(", 
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"q", "-", "p"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"-", "q"}],
                      "TraditionalForm"]}], ")"}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Beta]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["\[Delta]", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["\[Delta]", 
                  RowBox[{
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["f",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["f",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
                SuperscriptBox["q", "2"]}]]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[632]=  (21:53:32 on 5/16/01)"]
}, Closed]],

Cell["This is a generic 2-loop amplitude.", "Text",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{
        SubscriptBox["q", "1"], ",", 
        SubscriptBox["q", "2"], ",", "anyexpression"}], "]"}]], "Input",
  CellLabel->"In[633]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynAmp"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        RowBox[{
          StyleBox[
            RowBox[{
              SuperscriptBox["\[DifferentialD]", "D"], 
              SubscriptBox["q", "1"]}],
            ZeroWidthTimes->True], "\[Integral]", 
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], 
            SubscriptBox["q", "2"]}]}], "(", "anyexpression", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[633]=  (21:53:32 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmpDenominator", "ObjectName",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmpDenominator"],

Cell["\<\
FeynAmpDenominator[ PropagatorDenominator[ ... ], \
PropagatorDenominator[ ... ], ...] is the head of the denominators of the \
propagators, i.e., FeynAmpDenominator[x] is the representation of 1/x .\
\>", \
"Text",
  CellTags->"FeynAmpDenominator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text",
  CellTags->"FeynAmpDenominator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "]"}]], "Input",
  CellLabel->"In[611]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[611]=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{"p", ",", "m"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"p", "-", "q"}], ",", "m"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[612]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["p", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"p", "-", "q"}], ")"}], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[612]=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{"FeynAmpDenominator", "[", 
          RowBox[{"PropagatorDenominator", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "]"}]}], ";"}]], "Input",
  CellLabel->"In[613]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"t", "//", "FCI"}], "]"}]], "Input",
  CellLabel->"In[614]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}], ",", "m"}], "]"}], 
      "]"}]], "Output",
  CellLabel->"Out[614]//StandardForm=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"t", "//", "FCE"}], "]"}]], "Input",
  CellLabel->"In[615]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"{", 
        RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Output",
  CellLabel->"Out[615]//StandardForm=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
  CellLabel->"In[616]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorCombine", "ObjectName",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmpDenominatorCombine"],

Cell["\<\
FeynAmpDenominatorCombine[expr] expands expr with respect to \
FeynAmpDenominator and combines products of FeynAmpDenominator in expr into \
one FeynAmpDenominator.\
\>", "Text",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorSplit",
    ButtonData:>"FeynAmpDenominatorSplit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSplit"],
  "."
}], "Text",
  CellTags->"FeynAmpDenominatorCombine"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", " ", "=", " ", 
      RowBox[{
        RowBox[{"FAD", "[", "q", "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{"q", "-", "p"}], "]"}]}]}]], "Input",
  CellLabel->"In[608]:=  (21:53:26 on 5/16/01)",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          FractionBox["1", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "]"],
              Editable->False]],
          "TraditionalForm"], " ", 
        FormBox[
          FractionBox["1", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", "p"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q], 2], "]"],
              Editable->False]],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[608]=  (21:53:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[609]:=  (21:53:26 on 5/16/01)",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q"}]}], "]"}]], "Output",
  CellLabel->"Out[609]//StandardForm=  (21:53:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorSplit", "[", "%", "]"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[610]:=  (21:53:26 on 5/16/01)",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", "q", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q"}], "]"}]}]], "Output",
  CellLabel->"Out[610]//StandardForm=  (21:53:27 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorSimplify", "ObjectName",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell["\<\
FeynAmpDenominatorSimplify[exp] tries to simplify each \
PropagatorDenominator in a canonical way. FeynAmpDenominatorSimplify[exp, q1] \
simplifies all FeynAmpDenominator's in exp in a canonical way, including some \
translation of momenta. FeynAmpDenominatorSimplify[exp, q1, q2] additionally \
removes integrals with no mass scale.\
\>", "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell["FDS can be used as an alias.", "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData["FDS"], "Input",
  CellLabel->"In[617]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox["FeynAmpDenominatorSimplify", TraditionalForm]], "Output",
  CellLabel->"Out[617]=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text",
  CellTags->"FeynAmpDenominatorSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[TextData[{
  "The cornerstone of dimensional regularization is that ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "n"], "k", " ", 
              RowBox[{
                RowBox[{"f", "(", "k", ")"}], "/", 
                SuperscriptBox["k", 
                  RowBox[{"2", "m"}]]}]}]}], "=", " ", 
          RowBox[{"0", " ", "."}]}], TraditionalForm]]]
}], "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "k", "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]], "Input",
  CellLabel->"In[618]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[618]=  (21:53:27 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This brings ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], ")"}]}], 
        TraditionalForm]]],
  " into a standard form."
}], "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ",", 
            RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}]}], "]"}], ",", "k"}], "]"}]], "Input",
  CellLabel->"In[619]:=  (21:53:27 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "1"], "+", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[619]=  (21:53:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"FeynAmpDenominatorSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"FAD", "[", 
              RowBox[{
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"]}], ",", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "2"]}]}], "]"}], " ", 
            RowBox[{"SPD", "[", 
              RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]}]], "Input",\

  CellLabel->"In[620]:=  (21:53:28 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            FormBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"],
              "TraditionalForm"], "2"], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          SubsuperscriptBox["p", "2", "2"], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[620]=  (21:53:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r", "=", 
      RowBox[{"SPC", "[", 
        RowBox[{"t", ",", "k", ",", 
          RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[621]:=  (21:53:28 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SubsuperscriptBox["p", "2", "2"], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "+", 
                  SubscriptBox["p", "1"], "-", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[621]=  (21:53:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{"r", ",", "k"}], "]"}]], "Input",
  CellLabel->"In[622]:=  (21:53:29 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox[
            SubscriptBox["p", "1"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "2"],
            "TraditionalForm"]}], 
        RowBox[{
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "1"], "+", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[622]=  (21:53:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FDS", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"k", "-", "p1"}], ",", 
              RowBox[{"k", "-", "p2"}]}], "]"}], 
          RowBox[{
            RowBox[{"SPD", "[", 
              RowBox[{"k", ",", "OPEDelta"}], "]"}], "^", "2"}]}], ",", "k"}],
       "]"}]], "Input",
  CellLabel->"In[623]:=  (21:53:30 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p1", "+", "p2"}], ")"}], 
              "2"]}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p2",
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p1", "+", "p2"}], ")"}], 
              "2"]}]], "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"], 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p1", "+", "p2"}], ")"}], 
              "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[623]=  (21:53:30 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "r"}], "]"}]], "Input",
  CellLabel->"In[624]:=  (21:53:30 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorSplit", "ObjectName",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmpDenominatorSplit"],

Cell["\<\
FeynAmpDenominatorSplit[expr] splits all FeynAmpDenominator[a,b, \
...] in expr into FeynAmpDenominator[a]*FeynAmpDenominator[b] ... . \
FeynAmpDenominatorSplit[expr, q1] splits all FeynAmpDenominator in expr into \
a product of two, one containing q1 and other momenta, the second without q1.\
\
\>", "Text",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorCombine",
    ButtonData:>"FeynAmpDenominatorCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorCombine"],
  "."
}], "Text",
  CellTags->"FeynAmpDenominatorSplit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"q1", ",", 
            RowBox[{"q1", "-", "p"}], ",", 
            RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
            RowBox[{"q2", "-", "p"}]}], "]"}], "//", "FCI"}]}]], "Input",
  CellLabel->"In[625]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "1", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"], ".", 
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "2", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[625]=  (21:53:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "//", "Head"}]], "Input",
  CellLabel->"In[626]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    FormBox["FeynAmpDenominator", TraditionalForm]], "Output",
  CellLabel->"Out[626]=  (21:53:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSplit", "[", "t", "]"}]], "Input",
  CellLabel->"In[627]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], " ", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"], " ", 
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "2", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[627]=  (21:53:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[628]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", "q1", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q1"}], "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"q1", "-", "q2"}], "]"}], " ", 
      RowBox[{"FAD", "[", "q2", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q2"}], "]"}]}]], "Output",
  CellLabel->"Out[628]//StandardForm=  (21:53:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorSplit", "[", 
          RowBox[{"t", ",", "q1"}], "]"}], "//", "FCE"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[629]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{"q2", ",", 
          RowBox[{
            RowBox[{"-", "p"}], "+", "q2"}]}], "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"q1", ",", 
          RowBox[{
            RowBox[{"-", "p"}], "+", "q1"}], ",", 
          RowBox[{"q1", "-", "q2"}]}], "]"}]}]], "Output",
  CellLabel->"Out[629]//StandardForm=  (21:53:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[630]:=  (21:53:31 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q1", ",", "q2", ",", 
        RowBox[{"q1", "-", "q2"}], ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q1"}], ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q2"}]}], "]"}]], "Output",
  CellLabel->"Out[630]//StandardForm=  (21:53:32 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[631]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynAmpDenominatorSplit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynAmpList", "ObjectName",
  CellTags->"FeynAmpList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList[info][FeynAmp[...], FeynAmp[...], ...] is a head of a \
list of Feynman amplitudes.\
\>", "Text",
  CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList has no functional properties and serves just as a head.\
\
\>", "Text",
  CellTags->"FeynAmpList"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmp",
    ButtonData:>"FeynAmp",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmp"],
  "."
}], "Text",
  CellTags->"FeynAmpList"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynCalc", "ObjectName",
  CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynCalc"],

Cell["FeynCalc is simply a symbol with a usage definition.", "Text",
  CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "FeynCalc"}]], "Input",
  CellLabel->"In[36]:=  (13:24:28 on 9/3/02)",
  CellTags->"FeynCalc"],

Cell[BoxData["\<\"For installation notes visit www.feyncalc.org\\n For a list \
of availabe objects type $FeynCalcStuff, which contains a list of all \
functions and options in StringForm. You can get on-line information by \
?function, e.g., ?Contract.\\n There are several useful functions for short \
input, type $FCS for a list of short commands. Then type, e.g., ?GA.\\n \\n \
To get rid of the start-up messages put the line \\n $FeynCalcStartupMessages \
= False; \\n  into your init.m or the HighEnergyPhysics/FeynCalcConfig.m \
file.\"\>"], "Print",
  CellTags->"FeynCalc"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynCalcExternal", "ObjectName",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynCalcExternal"],

Cell["\<\
FeynCalcExternal[exp] translates exp from the internal FeynCalc \
representation to a shorthand form.\
\>", "Text",
  CellTags->"FeynCalcExternal"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text",
  CellTags->"FeynCalcExternal"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynCalcExternal", "[", 
      RowBox[{"DiracGamma", "[", "5", "]"}], "]"}]], "Input",
  CellLabel->"In[650]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
  CellLabel->"Out[650]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[651]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    RowBox[{"GA", "[", "5", "]"}]], "Output",
  CellLabel->"Out[651]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[652]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], ",", 
          
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[652]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[653]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[653]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
  CellLabel->"In[654]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[654]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[655]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[655]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[656]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcExternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[656]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynCalcForm", "ObjectName",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynCalcForm"],

Cell[TextData[{
  "FeynCalcForm[expr] changes the printed output to a an easy-to-read form. \
It allows a readable output also when running a terminal based ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session. Whether the result of FeynCalcForm[expr] is displayed or not, \
depends on the setting of $PrePrint. $PrePrint = FeynCalcForm forces \
displaying everything after applying FeynCalcForm. In order to change to the \
normal (internal) Mathematica OutputForm, do: ($PrePrint=.)."
}], "Text",
  CellTags->"FeynCalcForm"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FC",
    ButtonData:>"FC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FC"],
  ", ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text",
  CellTags->"FeynCalcForm"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynCalcForm"],

Cell["This is the normal notebook display:", "Text",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"SUNT", "[", "b", "]"}], ".", 
        RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
  CellLabel->"In[32]:=  (14:41:53 on 9/3/02)",
  CellTags->"FeynCalcForm"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[32]=  (14:41:53 on 9/3/02)",
  CellTags->"FeynCalcForm"]
}, Closed]],

Cell["\<\
This is the shorthand (terminal) display (easy-to-read form):\
\>", \
"Text",
  CellTags->"FeynCalcForm"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$PrePrint", " ", "=", " ", "FeynCalcForm"}], ";"}]], "Input",
  CellLabel->"In[33]:=  (14:41:54 on 9/3/02)",
  CellTags->"FeynCalcForm"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"$FrontEnd", ",", 
          RowBox[{"Evaluate", "[", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"Options", "[", 
                    RowBox[{
                    "$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
                    "]"}], "/.", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], 
                    "\[Rule]", 
                    RowBox[{"(", 
                      RowBox[{"\"\<Output\>\"", "\[Rule]", "OutputForm"}], 
                      ")"}]}]}], ")"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",\

  CellLabel->"In[34]:=  (14:41:55 on 9/3/02)",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"SUNT", "[", "b", "]"}], ".", 
        RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
  CellLabel->"In[35]:=  (14:41:57 on 9/3/02)",
  CellTags->"FeynCalcForm"],

Cell[OutputFormData["\<\
SUNTrace[SUNT[SUNIndex[a]] . SUNT[SUNIndex[b]] . \
SUNT[SUNIndex[c]]]\
\>", "\<\
tr[T[a] T[b] T[c]]\
\>"], "Output",
  CellLabel->"Out[35]=  (14:41:57 on 9/3/02)",
  CellTags->"FeynCalcForm"]
}, Closed]],

Cell["Reset to normal notebook display:", "Text",
  CellTags->"FeynCalcForm"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$PrePrint", "=."}], ";"}]], "Input",
  CellLabel->"In[36]:=  (14:41:59 on 9/3/02)",
  CellTags->"FeynCalcForm"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"$FrontEnd", ",", 
          RowBox[{"Evaluate", "[", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"Options", "[", 
                    RowBox[{
                    "$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
                    "]"}], "/.", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], 
                    "\[Rule]", 
                    RowBox[{"(", 
                      RowBox[{
                      "\"\<Output\>\"", "\[Rule]", "TraditionalForm"}], 
                      ")"}]}]}], ")"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",\

  CellLabel->"In[37]:=  (14:41:59 on 9/3/02)",
  CellTags->"FeynCalcForm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynCalcInternal", "ObjectName",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynCalcInternal"],

Cell["\<\
FeynCalcInternal[exp] translates exp into the internal FeynCalc \
(abstract data-type) representation.\
\>", "Text",
  CellTags->"FeynCalcInternal"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  "."
}], "Text",
  CellTags->"FeynCalcInternal"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[657]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], ",", 
          
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[657]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[658]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[658]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
  CellLabel->"In[659]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[659]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[660]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[660]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[661]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
  CellLabel->"Out[661]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCI", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"SD", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ",", 
          RowBox[{"SUND", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
          RowBox[{"FAD", "[", "q", "]"}], ",", 
          RowBox[{"LC", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
            "]"}]}], "}"}], "]"}]], "Input",
  CellLabel->"In[662]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          SubscriptBox["d", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          FractionBox["1", 
            SuperscriptBox["q", "2"]], ",", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[662]=  (21:53:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[663]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalcInternal"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], ",", 
        RowBox[{"SUND", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
        RowBox[{"SUNF", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
        RowBox[{"FeynAmpDenominator", "[", 
          RowBox[{"PropagatorDenominator", "[", 
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], "]"}], ",",
         
        RowBox[{"Eps", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]}], 
      "}"}]], "Output",
  CellLabel->"Out[663]//StandardForm=  (21:53:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynCalc2FORM", "ObjectName",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynCalc2FORM"],

Cell["\<\
FeynCalc2FORM[expr] displays expr in FORM syntax. \
FeynCalc2FORM[file, x] writes x in FORM syntax to a file. FeynCalc2FORM[file, \
x==y] writes x=y to a file in FORM syntax.\
\>", "Text",
  CellTags->"FeynCalc2FORM"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FORM2FeynCalc"],
  "."
}], "Text",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynCalc2FORM", "]"}]], "Input",
  CellLabel->"In[634]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"EpsDiscard", "\[Rule]", "False"}], ",", 
          RowBox[{"FORMEpilog", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"FORMProlog", "\[Rule]", "\<\"write statistics;\"\>"}], ",",
           
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"\<\"\\\\[Alpha]\"\>", "\[Rule]", "\<\"al\"\>"}], ",",
                 
                RowBox[{"\<\"\\\\[Beta]\"\>", "\[Rule]", "\<\"be\"\>"}], ",", 
                
                RowBox[{"\<\"\\\\[Gamma]\"\>", "\[Rule]", "\<\"ga\"\>"}], ",",
                 
                RowBox[{"\<\"\\\\[Delta]\"\>", "\[Rule]", "\<\"de\"\>"}], ",",
                 
                RowBox[{"\<\"\\\\[Mu]\"\>", "\[Rule]", "\<\"mu\"\>"}], ",", 
                RowBox[{"\<\"\\\\[Nu]\"\>", "\[Rule]", "\<\"nu\"\>"}], ",", 
                RowBox[{"\<\"\\\\[Rho]\"\>", "\[Rule]", "\<\"ro\"\>"}], ",", 
                RowBox[{"\<\"\\\\[Sigma]\"\>", "\[Rule]", "\<\"si\"\>"}]}], 
              "}"}]}], ",", 
          RowBox[{"TraceDimension", "\[Rule]", "4"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[634]=  (21:53:32 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
      RowBox[{"FV", "[", 
        RowBox[{"p", ",", "\[Rho]"}], "]"}], " ", 
      RowBox[{
        RowBox[{"y", "^", "2"}], "/", "d"}]}]], "Input",
  CellLabel->"In[635]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["y", "2"], " ", 
          SuperscriptBox["p", "\[Rho]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "d"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[635]=  (21:53:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
  CellLabel->"In[636]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox["\<\"(y^2*d_(mu,nu)*p(ro))/d\"\>", TraditionalForm]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"LC", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]", ",", "\[Rho]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[637]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Alpha]", "\[NoBreak]", "\[Beta]", "\[NoBreak]", "\[Delta]", 
          "\[NoBreak]", "\[Rho]"}]], TraditionalForm]], "Output",
  CellLabel->"Out[637]=  (21:53:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
  CellLabel->"In[638]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox["\<\"-i_*e_(al,be,de,ro)\"\>", TraditionalForm]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[639]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[639]=  (21:53:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
  CellLabel->"In[640]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox["\<\"g_(0,mu)*g_(0,nu)*g_(0,ro)*g_(0,si)\"\>", 
      TraditionalForm]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], 
      RowBox[{"DiracTrace", "[", 
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Rho]"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[641]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Rho]"]}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[641]=  (21:53:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
  CellLabel->"In[642]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox["\<\"g_(0,mu)*g_(0,nu)*g_(1,mu)*g_(1,ro)\"\>", 
      TraditionalForm]], "Print"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", 
        RowBox[{"\"\<fc2ftest.f\>\"", ",", 
          RowBox[{
            RowBox[{"MT", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
            RowBox[{"FV", "[", 
              RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[643]:=  (21:53:33 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReadList", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
              "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
          "\"\<fc2ftest.f\>\""}], ",", "String"}], "]"}]], "Input",
  CellLabel->"In[644]:=  (21:53:34 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "\<\"d_(mu,nu)*p(mu)\"\>", "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[644]=  (21:53:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"Tr", "[", 
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
            "]"}], ".", 
          RowBox[{"GS", "[", 
            RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[645]:=  (21:53:34 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[645]=  (21:53:34 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", 
        RowBox[{"\"\<fc2ftest.f\>\"", ",", 
          RowBox[{"L", " ", "==", "t"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[646]:=  (21:53:34 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<fc2ftest.f\>\""}], ",", "String"}], "]"}], "]"}]], "Input",
  CellLabel->"In[647]:=  (21:53:34 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"Indices \.03\.bc,\.03\.bd,\.03\[CapitalAAcute],\.03\
\[CapitalATilde];\"\>"},
            {"\<\"Vectors p,q;\"\>"},
            {"\<\"\"\>"},
            {"\<\"write statistics;\"\>"},
            {"\<\"\"\>"},
            {"\<\"Local L = ( \"\>"},
            {"\<\"4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)\
*d_(ro,si)*q.p+\"\>"},
            {"\<\"d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*\
q(mu)-\"\>"},
            {"\<\"d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*\
q(nu)+\"\>"},
            {"\<\"d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*\
q(ro)-\"\>"},
            {"\<\"d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*\
q(si)) ); \"\>"},
            {"\<\"   \"\>"},
            {"\<\"Print; \"\>"},
            {"\<\".end\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "Indices \
\.03\.bc,\.03\.bd,\.03\[CapitalAAcute],\.03\[CapitalATilde];", "Vectors p,q;",
           "", "write statistics;", "", "Local L = ( ", 
          "4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)*d_(ro,\
si)*q.p+", 
          "d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*q(mu)-\
", "d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*q(nu)+", 
          "d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*q(ro)-\
", "d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*q(si)) ); ", 
          "   ", "Print; ", ".end"}]], TraditionalForm]], "Output",
  CellLabel->"Out[647]//TableForm=  (21:53:34 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"FileNames", "[", "\"\<fc2ftest.f\>\"", "]"}], "=!=", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"DeleteFile", "[", "\"\<fc2ftest.f\>\"", "]"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[648]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalc2FORM"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
  CellLabel->"In[649]:=  (21:53:35 on 5/16/01)",
  CellTags->"FeynCalc2FORM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynmanParameterNames", "ObjectName",
  CellTags->"FeynmanParameterNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynmanParameterNames"],

Cell["\<\
FeynmanParameterNames is an option for FeynmanParametrize and \
FeynmanParametrize.\
\>", "Text",
  CellTags->"FeynmanParameterNames"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynmanParametrize",
    ButtonData:>"FeynmanParametrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynmanParametrize"],
  ", ",
  ButtonBox["FeynmanParametrize",
    ButtonData:>"FeynmanParametrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynmanParametrize"],
  "."
}], "Text",
  CellTags->"FeynmanParameterNames"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynmanParametrize ***unfinished***", "ObjectName",
  CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynmanParametrize"],

Cell["\<\
FeynmanParametrize[exp,k] introduces feynman parameters for all \
one-loop integrals in exp (k = integration momentum).\
\>", "Text",
  CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynmanParametrize", "]"}]], "Input",
  CellLabel->"In[5]:=  (23:56:41 on 9/19/02)",
  CellTags->"FeynmanParametrize"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"FeynmanParameterNames", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[5]=  (23:56:41 on 9/19/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynmanParametrize"],

Cell[BoxData[""], "Input",
  CellLabel->"In[635]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynmanParametrize"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynRule", "ObjectName",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynRule"],

Cell["\<\
FeynRule[lag, {fields}] derives the Feynman rule corresponding to \
the field configuration fields of the lagrangian lag.\
\>", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynRule", "]"}]], "Input",
  CellLabel->"In[87]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Anti5", "\[Rule]", 
            InterpretationBox[
              RowBox[{"-", "\[Infinity]"}],
              DirectedInfinity[ -1]]}], ",", 
          RowBox[{"Contract", "\[Rule]", "False"}], ",", 
          RowBox[{"Factor1", "\[Rule]", "False"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}], ",", 
          RowBox[{"InitialFunction", "\[Rule]", "Identity"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[87]="]
}, Closed]],

Cell["FeynRule does not calculate propagator Feynman rules.", "Text",
  CellTags->"FeynRule"],

Cell["\<\
The option ZeroMomentumInsertion can be used for twist-2 and higher \
twist operators.\
\>", "Text",
  CellTags->"FeynRule"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Lagrangian",
    ButtonData:>"Lagrangian",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Lagrangian"],
  "."
}], "Text",
  CellTags->"FeynRule"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gou", "=", 
      RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]}]], "Input",
  CellLabel->"In[88]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[88]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gop", "=", 
      RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]}]], "Input",
  CellLabel->"In[89]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Gamma]"}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[89]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "gop", "]"}]], "Input",
  CellLabel->"In[90]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Gamma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Gamma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SubscriptBox["g", "s"], " ", 
                RowBox[{
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Beta]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b1",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Gamma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c6",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b1", "\[NoBreak]", "c6"}]]}]}], 
            ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SubscriptBox["g", "s"], " ", 
                RowBox[{
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b2",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c7",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"b", "\[NoBreak]", "b2", "\[NoBreak]", "c7"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[90]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{"%", ",", "QuantumField"}], "]"}]], "Input",
  CellLabel->"In[91]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b2",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b1",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c6",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["c7",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[91]="]
}, Closed]],

Cell["2-gluon Feynman rules (unpolarized)", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[92]:=",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[92]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2u", "=", 
      RowBox[{"FullSimplify", "/@", 
        RowBox[{"Factor2", "[", 
          RowBox[{"FeynRule", "[", 
            RowBox[{"gou", ",", "fi", ",", 
              RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
          "]"}]}]}]], "Input",
  CellLabel->"In[670]:=  (21:53:36 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"]}]], 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "m"]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "m"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}]}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]}], "-", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], "-", 
                      RowBox[{
                        SuperscriptBox[
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]], " ", 
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[670]=  (21:53:41 on 5/16/01)"]
}, Closed]],

Cell["2-gluon Feynman rules (polarized)", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[671]:=  (21:53:41 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[671]=  (21:53:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2p", "=", 
      RowBox[{"FullSimplify", "/@", 
        RowBox[{"Factor2", "[", 
          RowBox[{"FeynRule", "[", 
            RowBox[{"gop", ",", "fi", ",", 
              RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
          "]"}]}]}]], "Input",
  CellLabel->"In[672]:=  (21:53:41 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}]], 
          RowBox[{"(", 
            RowBox[{"\[ImaginaryI]", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "m"], " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          SuperscriptBox["\[Epsilon]", 
                            RowBox[{
                              FormBox[
                                FormBox["\[Nu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}]], " ", 
                          SuperscriptBox[
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["\[Mu]",
                                "TraditionalForm"],
                              "TraditionalForm"]]}], "+", 
                        RowBox[{
                          SuperscriptBox["\[Epsilon]", 
                            RowBox[{
                              FormBox[
                                FormBox["\[Mu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox[
                                FormBox["\[Nu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}]], " ", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}]}]}], ")"}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], "m"], " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          SuperscriptBox["\[Epsilon]", 
                            RowBox[{
                              FormBox[
                                FormBox["\[Mu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}]], " ", 
                          SuperscriptBox[
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["\[Nu]",
                                "TraditionalForm"],
                              "TraditionalForm"]]}], "+", 
                        RowBox[{
                          SuperscriptBox["\[Epsilon]", 
                            RowBox[{
                              FormBox[
                                FormBox["\[Mu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox[
                                FormBox["\[Nu]",
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"], "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}]], " ", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ",
               
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[672]=  (21:53:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Calc", "[", 
        RowBox[{"f2p", "/.", 
          RowBox[{"p", "\[Rule]", 
            RowBox[{"-", "q"}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[673]:=  (21:53:45 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[673]=  (21:53:46 on 5/16/01)"]
}, Closed]],

Cell["\<\
Compare with the Feynman rule tabulated in \
Twist2GluonOperator.\
\>", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"q", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[674]:=  (21:53:46 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[674]=  (21:53:46 on 5/16/01)"]
}, Closed]],

Cell["quark-quark Feynman rule (unpolarized)", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qo", "=", 
      RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
  CellLabel->"In[675]:=  (21:53:46 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[675]=  (21:53:46 on 5/16/01)"]
}, Closed]],

Cell["quark-quark -gluon-gluon Feynman rule (unpolarized)", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qo", "=", 
      RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
  CellLabel->"In[676]:=  (21:53:46 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[676]=  (21:53:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qggf", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", "QuarkField", "]"}], "[", "p", "]"}],
           ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}], "[", "q", 
            "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "r", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "s", "]"}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[677]:=  (21:53:46 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FormBox["\[Psi]",
              "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
            "\[NoBreak]", ")"}], ",", 
          RowBox[{
            FormBox[
              OverscriptBox["\[Psi]", "_"],
              "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "q", 
            "\[NoBreak]", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "s", ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[677]=  (21:53:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"n4", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{"qo", ",", "qggf", ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], 
        "]"}]}]], "Input",
  CellLabel->"In[678]:=  (21:53:47 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "j"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "j"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[678]=  (21:53:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"s", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]}]], "Input",
  CellLabel->"In[679]:=  (21:53:52 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["s",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[679]=  (21:53:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"n4", "-", "t4"}], "/.", 
            RowBox[{"OPEm", "\[Rule]", "5"}]}], "/.", 
          RowBox[{"s", "\[Rule]", 
            RowBox[{
              RowBox[{"-", "p"}], "-", "q", "-", "r"}]}]}], "/.", 
        RowBox[{"D", "\[Rule]", "4"}]}], "]"}]], "Input",
  CellLabel->"In[680]:=  (21:53:52 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[680]=  (21:53:55 on 5/16/01)"]
}, Closed]],

Cell["\<\
 In general equality can be shown by Timing[Factor2[ \
FCE[Calc[ChangeDimension[FCE[OPESumExplicit[n4-t4]],4]/.s\[Rule]-p-q-r]]]] \
but it is a little bit slow ...\
\>", "Text",
  CellTags->"FeynRule"],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"qggf", ",", "n2", ",", "n4"}], "]"}]], "Input",
  CellLabel->"In[681]:=  (21:53:55 on 5/16/01)",
  CellTags->"FeynRule"],

Cell["QCD vertices", "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fii", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Rho]", "}"}], ",", 
                RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[682]:=  (21:53:55 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[682]=  (21:53:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g3", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{
          RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fii"}], 
        "]"}]}]], "Input",
  CellLabel->"In[683]:=  (21:53:55 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[683]=  (21:53:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[684]:=  (21:53:58 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", "r"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"r", "-", "p"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"p", "-", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[684]=  (21:53:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"g3", "-", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[685]:=  (21:53:58 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[685]=  (21:53:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi4", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Rho]", "}"}], ",", 
                RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Sigma]", "}"}], ",", 
                RowBox[{"{", "d", "}"}]}], "]"}], "[", "s", "]"}]}], 
        "}"}]}]], "Input",
  CellLabel->"In[686]:=  (21:53:59 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "s", ")"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[686]=  (21:53:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g4", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{
          RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fi4"}], 
        "]"}]}]], "Input",
  CellLabel->"In[687]:=  (21:53:59 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[687]=  (21:54:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[688]:=  (21:54:03 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[688]=  (21:54:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"g4", "-", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[689]:=  (21:54:05 on 5/16/01)",
  CellTags->"FeynRule"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[689]=  (21:54:05 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "f2p", ",", "f2u", ",", "f3", ",", "f32", ",", "fi", ",", "fi4", ",", 
        "fii", ",", "g3", ",", "g4", ",", "gop", ",", "gou", ",", "n3", ",", 
        "nf3", ",", "n4", ",", "np2", ",", "npf3", ",", "p33", ",", "pf3", 
        ",", "pn3", ",", "pqo", ",", "qf", ",", "qp", ",", "qgf", ",", "qo", 
        ",", "t4"}], "]"}]], "Input",
  CellLabel->"In[690]:=  (21:54:05 on 5/16/01)",
  CellTags->"FeynRule"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FI", "ObjectName",
  CellTags->"FI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FI"],

Cell["\<\
FI changes the output format to InputForm. This is useful to see \
the internal representation of FeynCalc objects. To change back to \
FeynCalcForm use FC.\
\>", "Text",
  CellTags->"FI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcForm",
    ButtonData:>"FeynCalcForm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcForm"],
  ", ",
  ButtonBox["FC",
    ButtonData:>"FC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FC"],
  ", ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text",
  CellTags->"FI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FieldDerivative", "ObjectName",
  CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FieldDerivative"],

Cell["\<\
FieldDerivative[f[x],x,li1,li2,...] is the derivative of f[x] with \
respect to space-time variables x and with Lorentz indices li1, li2, ..., \
where li1, li2, ... have head LorentzIndex. \
FieldDerivative[f[x],x,li1,li2,...] can be given as \
FieldDerivative[f[x],x,{l1,l2,...}], where l1 is li1 without the head, ... \
FieldDerivative is defined only for objects with head QuantumField[...]. If \
the space-time derivative of other objects is wanted, the corresponding rule \
must be specified.\
\>", "Text",
  CellTags->"FieldDerivative"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  "."
}], "Text",
  CellTags->"FieldDerivative"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"B", ",", 
            RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "[", "y", "]"}], ".", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"C", ",", 
            RowBox[{"{", "\[Rho]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"D", ",", 
            RowBox[{"{", "\[Sigma]", "}"}]}], "]"}], "[", "y", 
        "]"}]}]], "Input",
  CellLabel->"In[2]:=",
  CellTags->"FieldDerivative"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
        RowBox[{
          SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
        RowBox[{
          SubscriptBox[
            FormBox["C",
              "TraditionalForm"], "\[Rho]"], "(", "x", ")"}], ".", 
        RowBox[{
          SubscriptBox[
            FormBox["D",
              "TraditionalForm"], "\[Sigma]"], "(", "y", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[2]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FieldDerivative", "[", 
        RowBox[{"%", ",", "x", ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",\

  CellLabel->"In[3]:=",
  CellTags->"FieldDerivative"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
          RowBox[{
            SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["C",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["D",
                "TraditionalForm"], "\[Sigma]"], "(", "y", ")"}]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["C",
                "TraditionalForm"], "\[Rho]"], "(", "x", ")"}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["D",
                "TraditionalForm"], "\[Sigma]"], "(", "y", ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[3]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FieldDerivative", "[", 
        RowBox[{"%", ",", "y", ",", 
          RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",\

  CellLabel->"In[4]:=",
  CellTags->"FieldDerivative"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
          RowBox[{
            SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["C",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["D",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "y", ")"}]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["B", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "y", ")"}]}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["C",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["D",
                "TraditionalForm"], "\[Sigma]"], "(", "y", ")"}]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["C",
                "TraditionalForm"], "\[Rho]"], "(", "x", ")"}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["D",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "y", ")"}]}]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "x", ")"}]}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["B", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[Null]"]}], ")"}], 
            "\[InvisibleApplication]", 
            RowBox[{"(", "y", ")"}]}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["C",
                "TraditionalForm"], "\[Rho]"], "(", "x", ")"}], ".", 
          RowBox[{
            SubscriptBox[
              FormBox["D",
                "TraditionalForm"], "\[Sigma]"], "(", "y", ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[4]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FieldStrength", "ObjectName",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FieldStrength"],

Cell[TextData[{
  "FieldStrength[\[Mu], \[Nu], a] is the field strength tensor ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            SubsuperscriptBox["A", "\[Nu]", "a"]}], "-", 
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Nu]"], 
            SubsuperscriptBox["A", "\[Mu]", "a"]}], "+", 
          RowBox[{
            SubscriptBox["g", "s"], " ", 
            SubsuperscriptBox["A", "\[Mu]", "b1"], 
            SubsuperscriptBox["A", "\[Nu]", "c1"]}]}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          SuperscriptBox["f", 
            RowBox[{"a", " ", "b1", " ", "c1"}]]}], TraditionalForm]]],
  ". FieldStrength[\[Mu], \[Nu]] is the field strength tensor ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            SubsuperscriptBox["A", "\[Nu]", "\[Null]"]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", "\[Nu]"], 
              SubsuperscriptBox["A", "\[Mu]", "\[Null]"]}], "."}]}], 
        TraditionalForm]]],
  " The name of the field (A) and the coupling constant (g) can be set \
through the options or by additional arguments. The first two indices are \
interpreted as type LorentzIndex, except OPEDelta, which is converted to \
Momentum[OPEDelta]."
}], "Text",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FieldStrength", "]"}]], "Input",
  CellLabel->"In[691]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"IndexPosition", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], ",", 
          RowBox[{"Symbol", "\[Rule]", "\<\"F\"\>"}], ",", 
          RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[691]=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuantumField"],
  "."
}], "Text",
  CellTags->"FieldStrength"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[692]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "\[Null]"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[692]=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
  CellLabel->"In[693]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "a"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[693]=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[694]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[694]=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[695]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b12",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c47",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b12", "\[NoBreak]", "c47"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[695]=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[696]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], "-", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[696]//StandardForm=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "OPEDelta", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[697]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"Momentum", "[", "OPEDelta", "]"}]}], "]"}], "-", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[697]//StandardForm=  (21:54:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
        RowBox[{"CouplingConstant", "\[Rule]", 
          RowBox[{"-", "Gstrong"}]}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[698]:=  (21:54:06 on 5/16/01)",
  CellTags->"FieldStrength"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b13",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c48",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b13", "\[NoBreak]", "c48"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[698]=  (21:54:06 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FinalFunction", "ObjectName",
  CellTags->"FinalFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FinalFunction"],

Cell["FinalFunction is an option for OneLoopSum.", "Text",
  CellTags->"FinalFunction"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"FinalFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FinalSubstitutions", "ObjectName",
  CellTags->"FinalSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FinalSubstitutions"],

Cell["\<\
FinalSubstitutions is an option for OneLoop and OneLoopSum and \
Write2. All substitutions indicated hereby are done at the end of the \
calculation.\
\>", "Text",
  CellTags->"FinalSubstitutions"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  ", ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  "."
}], "Text",
  CellTags->"FinalSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FORM", "ObjectName",
  CellTags->"FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FORM"],

Cell["\<\
FORM is an option for RHI. If set to True a FORM file is generated \
and run from Mathematica (provided R. Hamberg's FORM-program is installed \
correctly ... ).\
\>", "Text",
  CellTags->"FORM"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  ", ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  "."
}], "Text",
  CellTags->"FORM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FORMEpilog", "ObjectName",
  CellTags->"FORMEpilog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FORMEpilog"],

Cell["\<\
FORMEpilog is an option for FeynCalc2FORM. It may be set to a \
string which is put at the end of the FORM-file.\
\>", "Text",
  CellTags->"FORMEpilog"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  ", ",
  ButtonBox["FORMProlog",
    ButtonData:>"FORMProlog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FORMProlog"],
  "."
}], "Text",
  CellTags->"FORMEpilog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FORMProlog", "ObjectName",
  CellTags->"FORMProlog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FORMProlog"],

Cell["\<\
FORMProlog is an option for FeynCalc2FORM. It may be set to a \
string which is put after the type declarations of the FORM-file.\
\>", "Text",\

  CellTags->"FORMProlog"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  ", ",
  ButtonBox["FORMEpilog",
    ButtonData:>"FORMEpilog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FORMEpilog"],
  "."
}], "Text",
  CellTags->"FORMProlog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FORM2FeynCalc", "ObjectName",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FORM2FeynCalc"],

Cell["\<\
FORM2FeynCalc[expr] translates the FORM expr into FeynCalc \
notation. FORM2FeynCalc[file] translates the FORM expresssions in file into \
FeynCalcnotation. FORM2FeynCalc[file, x1, x2, ...] reads in a file in \
FORM-format and translates the assignments for the variables a, b, ... into \
FeynCalc syntax. If the option Set is True, the variables x1, x2 are assigned \
to the right hand sides defined in the FORM-file.\
\>", "Text",
  CellTags->"FORM2FeynCalc"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  "."
}], "Text",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FORM2FeynCalc", "]"}]], "Input",
  CellLabel->"In[699]:=  (21:54:06 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Dot", "\[Rule]", "Times"}], ",", 
          RowBox[{"HoldForm", "\[Rule]", "False"}], ",", 
          RowBox[{"LorentzIndex", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"mu", ",", "nu", ",", "al", ",", "be"}], "}"}]}], ",", 
          RowBox[{"Set", "\[Rule]", "False"}], ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Vectors", "\[Rule]", "Automatic"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[699]=  (21:54:06 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", "\"\<p.q + 2*x m^2\>\"", "]"}]], "Input",
  CellLabel->"In[700]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", "x", " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[700]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[701]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    RowBox[{
      RowBox[{"2", " ", 
        SuperscriptBox["m", "2"], " ", "x"}], "+", 
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Output",
  CellLabel->"Out[701]//StandardForm=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell["\<\
Functions are automatically converted right, but bracketed \
expressions need to be substituted explicitly.\
\>", "Text",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", 
      RowBox[{"\"\<x +f(z)+ log(x)^2+[li2(1-x)]\>\"", ",", 
        RowBox[{"Replace", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
            "\"\<[li2(1-x)]\>\"", "\[Rule]", "\"\<PolyLog[2,1-x]\>\""}], 
            "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[702]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}], "+", "x", "+", 
        RowBox[{"f", "(", "z", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[702]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[703]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    RowBox[{"x", "+", 
      RowBox[{"f", "[", "z", "]"}], "+", 
      SuperscriptBox[
        RowBox[{"Log", "[", "x", "]"}], "2"], "+", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}]}]], "Output",
  CellLabel->"Out[703]//StandardForm=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
    "FORM2FeynCalc", "[", "\"\<x + [(1)]*y -[(-1)^m]\>\"", "]"}]], "Input",
  CellLabel->"In[704]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "+", 
        RowBox[{"y", " ", 
          RowBox[{"Hold", "[", "1", "]"}]}], "-", 
        RowBox[{"Hold", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[704]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
  CellLabel->"In[705]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "-", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], "+", "y"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[705]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
    "FORM2FeynCalc", "[", "\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", "]"}]], "Input",
  CellLabel->"In[706]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["p", "mu"], " ", 
          SuperscriptBox["q", "nu"]}], "+", 
        SuperscriptBox["g", 
          RowBox[{"mu", "\[NoBreak]", "nu"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[706]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[707]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "mu"}], "]"}], " ", 
        RowBox[{"FV", "[", 
          RowBox[{"q", ",", "nu"}], "]"}]}], "+", 
      RowBox[{"MT", "[", 
        RowBox[{"mu", ",", "nu"}], "]"}]}]], "Output",
  CellLabel->"Out[707]//StandardForm=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", 
      RowBox[{"\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", ",", 
        RowBox[{"Replace", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"mu", "\[Rule]", "\[Mu]"}], ",", 
              RowBox[{"nu", "\[Rule]", "\[Nu]"}]}], "}"}]}]}], "]"}]], "Input",\

  CellLabel->"In[708]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], " ", 
          SuperscriptBox["q", "\[Nu]"]}], "+", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[708]=  (21:54:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
    "FORM2FeynCalc", "[", 
      "\"\<i_*az*bz*aM^2*D1*[(1)]*b_G1 * ( 4*eperp(mu,nu)*avec.bvec*blam \
)\>\"", "]"}]], "Input",
  CellLabel->"In[709]:=  (21:54:07 on 5/16/01)",
  CellTags->"FORM2FeynCalc"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["aM", "2"], " ", "az", " ", "blam", " ", "bz", " ", 
        "b$G1", " ", "D1", " ", 
        RowBox[{"eperp", "(", 
          RowBox[{"mu", ",", "nu"}], ")"}], " ", 
        RowBox[{"Hold", "[", "1", "]"}], " ", 
        FormBox[
          RowBox[{
            FormBox["avec",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["bvec",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[709]=  (21:54:08 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FourDivergence", "ObjectName",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FourDivergence"],

Cell["\<\
FourDivergence[exp, FourVector[p, mu]] calculates the partial \
derivative of exp w.r.t. p(mu). FourDivergence[exp, FourVector[p, mu], \
FourVector[p,nu], ...]gives the multiple derivative.\
\>", "Text",
  CellTags->"FourDivergence"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RussianTrick",
    ButtonData:>"RussianTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RussianTrick"],
  "."
}], "Text",
  CellTags->"FourDivergence"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
  CellLabel->"In[710]:=  (21:54:08 on 5/16/01)",
  CellTags->"FourDivergence"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[710]=  (21:54:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourDivergence", "[", 
      RowBox[{"t", ",", 
        RowBox[{"FourVector", "[", 
          RowBox[{"q", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[711]:=  (21:54:08 on 5/16/01)",
  CellTags->"FourDivergence"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[711]=  (21:54:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "-", "k"}], ",", "q"}], "]"}]}]], "Input",
  CellLabel->"In[712]:=  (21:54:08 on 5/16/01)",
  CellTags->"FourDivergence"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", 
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"]}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[712]=  (21:54:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourDivergence", "[", 
      RowBox[{"t", ",", 
        RowBox[{"FourVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",\

  CellLabel->"In[713]:=  (21:54:08 on 5/16/01)",
  CellTags->"FourDivergence"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[713]=  (21:54:09 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[714]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourDivergence"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FourLaplacian", "ObjectName",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FourLaplacian"],

Cell[TextData[{
  "FourLaplacian[exp, p, q] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[PartialD]", 
          RowBox[{"/", 
            RowBox[{"\[PartialD]", 
              SubscriptBox["p", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{"\[PartialD]", 
          RowBox[{"/", 
            RowBox[{"\[PartialD]", 
              SubscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  "exp."
}], "Text",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FourLaplacian", "]"}]], "Input",
  CellLabel->"In[715]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourLaplacian"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[715]=  (21:54:09 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourDivergence",
    ButtonData:>"FourDivergence",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourDivergence"],
  ", ",
  ButtonBox["RussianTrick",
    ButtonData:>"RussianTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RussianTrick"],
  "."
}], "Text",
  CellTags->"FourLaplacian"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"q", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[716]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourLaplacian"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["q",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[716]=  (21:54:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourLaplacian", "[", 
      RowBox[{"%", ",", "q", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[717]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourLaplacian"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "D"}], TraditionalForm]], "Output",
  CellLabel->"Out[717]=  (21:54:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SOD", "[", "q", "]"}], "^", "OPEm"}], 
        RowBox[{"FAD", "[", 
          RowBox[{"q", ",", 
            RowBox[{"q", "-", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[718]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourLaplacian"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], "m"], 
        RowBox[{
          SuperscriptBox["q", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[718]=  (21:54:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourLaplacian", "[", 
      RowBox[{"%", ",", "q", ",", " ", "q"}], "]"}]], "Input",
  CellLabel->"In[719]:=  (21:54:09 on 5/16/01)",
  CellTags->"FourLaplacian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "1"}]]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", "D", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"12", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", "D", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"12", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[719]=  (21:54:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FourVector", "ObjectName",
  CellTags->"FourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FourVector"],

Cell[TextData[{
  "FourVector[p, ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the four-dimensional vector p with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ". A vector with space-time Dimension D is obtained by supplying the option \
Dimension \[Rule] D."
}], "Text",
  CellTags->"FourVector"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  "."
}], "Text",
  CellTags->"FourVector"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[720]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    FormBox[
      SubscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[720]=  (21:54:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[721]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", "\[NoBreak]", 
          FormBox[
            TagBox[
              RowBox[{"p", "-", "q"}],
              HoldForm],
            "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[721]=  (21:54:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[722]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
  CellLabel->"Out[722]//StandardForm=  (21:54:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[723]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", 
        RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
  CellLabel->"Out[723]//StandardForm=  (21:54:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"FourVector", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[724]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
  CellLabel->"Out[724]//StandardForm=  (21:54:10 on 5/16/01)"]
}, Closed]],

Cell["\<\
There is no special function to expand momenta in FourVector. Since \
FourVector is turned into Pair internally ExpandScalarProduct may be \
used.\
\>", "Text",
  CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[725]:=  (21:54:10 on 5/16/01)",
  CellTags->"FourVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[725]=  (21:54:11 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FreeIndex", "ObjectName",
  CellTags->"FreeIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FreeIndex"],

Cell["\<\
FreeIndex is a datatype which is recognized by Contract. Possible \
use: DataType[mu, FreeIndex] = True.\
\>", "Text",
  CellTags->"FreeIndex"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  "."
}], "Text",
  CellTags->"FreeIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FreeQ2", "ObjectName",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FreeQ2"],

Cell["\<\
FreeQ2[expr, {form1, form2, ...}] yields True if expr does not \
contain any occurence of form1, form2, ... and False otherwise. FreeQ2[expr, \
form] is the same as FreeQ[expr, form].\
\>", "Text",
  CellTags->"FreeQ2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SelectFree",
    ButtonData:>"SelectFree",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SelectFree"],
  ", ",
  ButtonBox["SelectNotFree",
    ButtonData:>"SelectNotFree",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SelectNotFree"],
  "."
}], "Text",
  CellTags->"FreeQ2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"x", "+", 
          RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"a", ",", "x"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[726]:=  (21:54:11 on 5/16/01)",
  CellTags->"FreeQ2"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[726]=  (21:54:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"x", "+", 
          RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[727]:=  (21:54:11 on 5/16/01)",
  CellTags->"FreeQ2"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[727]=  (21:54:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{"x", ",", " ", "y"}], "]"}]], "Input",
  CellLabel->"In[728]:=  (21:54:11 on 5/16/01)",
  CellTags->"FreeQ2"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[728]=  (21:54:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", " ", "f"}], "]"}]], "Input",
  CellLabel->"In[729]:=  (21:54:11 on 5/16/01)",
  CellTags->"FreeQ2"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[729]=  (21:54:11 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FRH", "ObjectName",
  CellTags->"FRH"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FRH"],

Cell["\<\
FRH[exp_] := FixedPoint[ReleaseHold, exp], i.e., FRH removes all \
HoldForm and Hold in exp.\
\>", "Text",
  CellTags->"FRH"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  "."
}], "Text",
  CellTags->"FRH"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FRH"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hold", "[", 
      RowBox[{"1", "-", "1", " ", "-", " ", 
        RowBox[{"Hold", "[", 
          RowBox[{"2", "-", "2"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[733]:=  (21:54:12 on 5/16/01)",
  CellTags->"FRH"],

Cell[BoxData[
    FormBox[
      RowBox[{"Hold", "[", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"Hold", "[", 
              RowBox[{"2", "-", "2"}], "]"}]}], "+", "1", "-", "1"}], "]"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[733]=  (21:54:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", "%", "]"}]], "Input",
  CellLabel->"In[734]:=  (21:54:12 on 5/16/01)",
  CellTags->"FRH"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[734]=  (21:54:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{"Solve", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"x", "^", "3"}], "-", "x", "-", "1"}], "==", "0"}], 
          "]"}], ",", "x", ",", 
        RowBox[{"IsolateNames", "\[Rule]", "H"}]}], "]"}]], "Input",
  CellLabel->"In[735]:=  (21:54:12 on 5/16/01)",
  CellTags->"FRH"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "3", ")"}],
                HoldForm]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "6", ")"}],
                HoldForm]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "7", ")"}],
                HoldForm]}], "}"}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[735]=  (21:54:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", 
      RowBox[{"H", "[", "3", "]"}], "]"}]], "Input",
  CellLabel->"In[736]:=  (21:54:12 on 5/16/01)",
  CellTags->"FRH"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RadicalBox[
            RowBox[{
              FractionBox["27", "2"], "-", 
              FractionBox[
                RowBox[{"3", " ", 
                  SqrtBox["69"]}], "2"]}], "3"]}], "+", 
        FractionBox[
          RadicalBox[
            RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{"(", 
                RowBox[{"9", "+", 
                  SqrtBox["69"]}], ")"}]}], "3"], 
          SuperscriptBox["3", 
            RowBox[{"2", "/", "3"}]]]}], TraditionalForm]], "Output",
  CellLabel->"Out[736]=  (21:54:12 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FromTFi", "ObjectName",
  CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FromTFi"],

Cell["\<\
FromTFi[expr, q1, q2, p] translates the TFi notatation from the \
TARCER package to the usual FeynCalc notation. See TFi for details on the \
conventions.\
\>", "Text",
  CellTags->"FromTFi"],

Cell[TextData[{
  "See also: ",
  ButtonBox["TFi",
    ButtonData:>"TFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TFi"],
  ", ",
  ButtonBox["ToTFi",
    ButtonData:>"ToTFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ToTFi"],
  "."
}], "Text",
  CellTags->"FromTFi"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{"q1", ",", 
          RowBox[{"q1", "-", "p"}], ",", 
          RowBox[{"{", 
            RowBox[{"q2", ",", "M"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
          RowBox[{"q1", "-", "q2"}]}], "]"}], "//", "ToTFi"}]], "Input",
  CellLabel->"In[730]:=  (21:54:11 on 5/16/01)",
  CellTags->"FromTFi"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          SPD[ p, p], {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output",
  CellLabel->"Out[730]=  (21:54:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FromTFi", "[", 
      RowBox[{
        RowBox[{"TFi", "[", 
          RowBox[{"D", ",", 
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], ",", 
            RowBox[{"SOD", "[", "p", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "M"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1",
         ",", "q2", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[731]:=  (21:54:12 on 5/16/01)",
  CellTags->"FromTFi"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "2", "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
              "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[731]=  (21:54:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FromTFi", "[", 
      RowBox[{
        RowBox[{"TFi", "[", 
          RowBox[{"D", ",", 
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], ",", 
            RowBox[{"SOD", "[", "p", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "M"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1",
         ",", "q2", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[732]:=  (21:54:12 on 5/16/01)",
  CellTags->"FromTFi"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "2"],
            "TraditionalForm"]}], 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2", "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
              "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[732]=  (21:54:12 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FUNCTION", "ObjectName",
  CellTags->"FUNCTION"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FUNCTION"],

Cell["\<\
FUNCTION[exp, string] is a head of an expression to be declared a \
function (of type string), if used in Write2.\
\>", "Text",
  CellTags->"FUNCTION"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  "."
}], "Text",
  CellTags->"FUNCTION"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FunctionalD", "ObjectName",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FunctionalD"],

Cell["\<\
FunctionalD[expr, {QuantumField[name, {mu}, {a}][p], ...}] \
calculates the functional derivative of expr with respect to the field list \
(with incoming momenta p, etc.) and does the fourier transform. \
FunctionalD[expr, {QuantumField[name, {mu},{a}], ...}] calculates the \
functional derivative and does partial integration but omits the x-space \
delta functions.\
\>", "Text",
  CellTags->"FunctionalD"],

Cell["FunctionalD is a low level function used in FeynRule.", "Text",
  CellTags->"FunctionalD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynRule"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuantumField"],
  "."
}], "Text",
  CellTags->"FunctionalD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FunctionalD"],

Cell[TextData[{
  "Instead of the usual ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"\[Delta]\[Phi]", "(", "x", ")"}], "/", 
            RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
          RowBox[{
            SuperscriptBox["\[Delta]", 
              RowBox[{"(", "D", ")"}]], "(", 
            RowBox[{"x", "-", "y"}], ")"}]}], TraditionalForm]]],
  " the arguments and the \[Delta] function are omitted, i.e., for the \
program for simplicity: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Delta]\[Phi]", "/", "\[Delta]\[Phi]"}], "=", "1"}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[737]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[737]=  (21:54:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"QuantumField", "[", "\[Phi]", "]"}], "^", "2"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[738]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "\[Phi]"}], TraditionalForm]], "Output",
  CellLabel->"Out[738]=  (21:54:13 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Instead of the usual ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"\[Delta]", " ", 
                RowBox[{
                  SubscriptBox["\[PartialD]", "\[Mu]"], 
                  RowBox[{"\[Phi]", "(", "x", ")"}]}]}], ")"}], "/", 
            RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            RowBox[{
              SuperscriptBox["\[Delta]", 
                RowBox[{"(", "D", ")"}]], "(", 
              RowBox[{"x", "-", "y"}], ")"}]}]}], TraditionalForm]]],
  " the arguments are omitted, and the ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
  "operator is specified by default to be an integration by parts operator, \
i.e., the right hand side will be just ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"-", 
            FormBox[
              SubscriptBox["\[PartialD]", "\[Mu]"],
              "TraditionalForm"]}], ","}], TraditionalForm]]],
  " or, more precisely (by default) ",
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], TraditionalForm]]],
  "."
}], "Text",
  ZeroWidthTimes->True,
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], 
        ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[739]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[739]=  (21:54:13 on 5/16/01)"]
}, Closed]],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"S", "[", "\[Phi]", "]"}], " ", "=", 
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"\[Integral]", " ", 
            RowBox[{
              SuperscriptBox["d", "D"], 
              RowBox[{"x", " ", "[", " ", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[PartialD]", "\[Mu]"], 
                      RowBox[{"\[Phi]", "(", "x", ")"}]}], " ", 
                    RowBox[{
                      SuperscriptBox["\[PartialD]", "\[Mu]"], 
                      RowBox[{"\[Phi]", "(", "x", ")"}]}]}], "-", 
                  RowBox[{
                    SuperscriptBox["m", "2"], 
                    RowBox[{"\[Phi]", "(", "x", ")"}], " ", 
                    RowBox[{"\[Phi]", "(", "y", ")"}]}]}], "]"}]}]}]}]}], 
      TraditionalForm]]]], "Text",
  ZeroWidthTimes->True,
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"s", "[", "\[Phi]", "]"}], "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{
                  RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], 
                "]"}], ".", 
              RowBox[{"QuantumField", "[", 
                RowBox[{
                  RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], 
                "]"}]}], "-", 
            RowBox[{
              RowBox[{"m", "^", "2"}], " ", 
              RowBox[{
                RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ".", 
                RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}]}]}], ")"}], 
        "/", "2"}]}]], "Input",
  CellLabel->"In[740]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}], ".", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], "-", 
            RowBox[{
              SuperscriptBox["m", "2"], " ", 
              RowBox[{"\[Phi]", ".", "\[Phi]"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[740]=  (21:54:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"s", "[", "\[Phi]", "]"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[741]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[Phi]"}], " ", 
          SuperscriptBox["m", "2"]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[741]=  (21:54:13 on 5/16/01)"]
}, Closed]],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"S", "[", "A", "]"}], " ", "=", " ", 
        RowBox[{"-", 
          RowBox[{"\[Integral]", " ", 
            RowBox[{
              SuperscriptBox["d", "D"], "x", " ", 
              FractionBox["1", "4"], " ", 
              RowBox[{
                SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], "(", "x", ")"}], 
              " ", 
              RowBox[{
                SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}]}]}]}], 
      TraditionalForm]]]], "Text",
  ZeroWidthTimes->True,
  CellTags->"FunctionalD"],

Cell[TextData[StyleBox["First approach",
  FontWeight->"Bold"]], "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"F1", "=", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
          RowBox[{"{", 
            RowBox[{"A", ",", "b", ",", "c"}], "}"}], ",", "1", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[742]:=  (21:54:13 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[742]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"F2", "=", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
          RowBox[{"{", 
            RowBox[{"A", ",", "d", ",", "e"}], "}"}], ",", "1", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[743]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[743]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"S", "[", "A", "]"}], "=", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"1", "/", "4"}]}], 
        RowBox[{"F1", ".", "F2"}]}]}]], "Input",
  CellLabel->"In[744]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
            ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[744]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "In order to derive the equation of motion the functional derivative of ",
  Cell[BoxData[
      FormBox["S", TraditionalForm]]],
  " with respect to ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["A", "\[Sigma]", "g"], TraditionalForm]]],
  "has to be set to zero. Bearing in mind that for FeynCalc we have to be \
precise as to where which operators (coming from the substitution of the \
derivative of the \[Delta] function) act:, act with the functional derivative \
operator on the first field strength:"
}], "Text",
  CellTags->"FunctionalD"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            FormBox[
              RowBox[{
                FormBox[
                  RowBox[{
                    FormBox[
                      RowBox[{"0", " ", "=", " ", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{"\[Delta]", "S"}], ")"}], "/", 
                              RowBox[{"(", 
                                RowBox[{"\[Delta]", 
                                  RowBox[{
                                    SubsuperscriptBox["A", "\[Sigma]", "g"], 
                                    "(", "y", ")"}]}], ")"}]}],
                            "TraditionalForm"], "=", 
                          RowBox[{"-", 
                            RowBox[{"2", "/", "4"}]}]}]}],
                      "TraditionalForm"], 
                    RowBox[{"\[Integral]", 
                      RowBox[{
                        SuperscriptBox["d", "D"], "x"}]}]}],
                  "TraditionalForm"], " ", 
                RowBox[{"(", 
                  RowBox[{"\[Delta]", "/", 
                    RowBox[{"(", 
                      RowBox[{"\[Delta]", 
                        RowBox[{
                          SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", 
                          ")"}]}], ")"}]}]}]}],
              "TraditionalForm"], " ", 
            RowBox[{
              SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}], ")"}], 
        TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{" ", 
              SubsuperscriptBox["F", "a", 
                RowBox[{",", "\[Mu]\[Nu]"}]]}],
            "TraditionalForm"], "(", "x", ")"}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->True,
  CellTags->"FunctionalD"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{"See", " ", "what", " ", "happens", " ", "with", " ", "just", 
        RowBox[{
          FormBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"\[Delta]", 
                  RowBox[{"S", "[", "A", "]"}]}], ")"}], "/", 
              RowBox[{"(", 
                RowBox[{"\[Delta]", 
                  SubsuperscriptBox["A", "\[Sigma]", "g"]}], ")"}]}],
            "TraditionalForm"], "."}]}], TraditionalForm]]]], "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Ag", "=", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "\[Sigma]", "}"}], ",", 
          RowBox[{"{", "g", "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[745]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["A", 
        FormBox[
          FormBox["\[Sigma]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["g",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[745]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"FunctionalD", "[", 
        RowBox[{"F1", ",", "Ag"}], "]"}]}]], "Input",
  CellLabel->"In[746]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[746]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell["\<\
In order to minimize the number of dummy indices, replace b \[Rule] \
c.\
\>", "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"t1", " ", "/.", " ", 
        RowBox[{"b", "\[Rule]", "c"}]}]}]], "Input",
  CellLabel->"In[747]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[747]=  (21:54:14 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Instead of inserting the definition for the second ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], TraditionalForm]]],
  ", introduce a QuantumField object with antisymmetry built into the Lorentz \
indices:"
}], "Text",
  CellTags->"FunctionalD"],

Cell[BoxData[
    RowBox[{"F", "/:", " ", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
        "pard___", ",", "F", ",", "\[Beta]_", ",", "\[Alpha]_", ",", "s_"}], 
        "]"}], ":=", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"QuantumField", "[", 
            RowBox[{
            "pard", ",", "F", ",", "\[Alpha]", ",", "\[Beta]", ",", "s"}], 
            "]"}]}], "/;", 
        RowBox[{"!", 
          RowBox[{"OrderedQ", "[", 
            RowBox[{"{", 
              RowBox[{"\[Beta]", ",", "\[Alpha]"}], "}"}], 
            "]"}]}]}]}]], "Input",
  CellLabel->"In[748]:=  (21:54:14 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"F", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[749]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]}], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[749]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"\[Mu]", "\[RuleDelayed]", "\[Nu]"}], ",", 
          RowBox[{"\[Nu]", "\[RuleDelayed]", "\[Mu]"}]}], "}"}]}]], "Input",
  CellLabel->"In[750]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["F", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]}], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[750]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{
        RowBox[{"Contract", "[", 
          RowBox[{"ExpandPartialD", "[", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{"1", "/", "2"}]}], " ", 
              RowBox[{"t1", ".", 
                RowBox[{"QuantumField", "[", 
                  RowBox[{"F", ",", 
                    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
                    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
                    RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]}]}], "]"}], 
          "]"}], "/.", 
        RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
  CellLabel->"In[751]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["F", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["F", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[751]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", " ", "=", " ", 
      RowBox[{"t2", " ", "/.", " ", 
        RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
  CellLabel->"In[752]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[752]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{
        RowBox[{"FCE", "[", "t3", "]"}], "/.", " ", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "c", ",", "g"}], "]"}], "\[Rule]", 
          RowBox[{"-", 
            RowBox[{"SUNF", "[", 
              RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]}]], "Input",
  CellLabel->"In[753]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[753]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Since the variational derivative vanishes t4 implies that 0 = ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["D", "\[Mu]"], 
          SubsuperscriptBox["F", "g", "\[Mu]\[Sigma]"]}], TraditionalForm]]],
  " ."
}], "Text",
  CellTags->"FunctionalD"],

Cell[TextData[StyleBox["Second approach",
  FontWeight->"Bold"]], "Text",
  CellTags->"FunctionalD"],

Cell["\<\
It is of course also possible to do the functional deriviate on the \
S[A] with both field strength tensors inserted.\
\>", "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"S", "[", "A", "]"}]], "Input",
  CellLabel->"In[754]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
            ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[754]=  (21:54:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r1", "=", 
      RowBox[{"FunctionalD", "[", 
        RowBox[{
          RowBox[{"S", "[", "A", "]"}], ",", "Ag"}], "]"}]}]], "Input",
  CellLabel->"In[755]:=  (21:54:15 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[755]=  (21:54:18 on 5/16/01)"]
}, Closed]],

Cell["\<\
This is just funcional derivatves and partial integration and \
simple contraction of indices. No attempt is made to rename dummy indices \
(since this is difficult in general ...).\
\>", "Text",
  CellTags->"FunctionalD"],

Cell["\<\
With a general replacement rule only valid for commuting fields the \
color indices can be canonicalized a bit more. The idea is to use the \
commutative properties of the vector fields, and canonicalize the color \
indices by a trick. This function will work on any commuting product of \
fields.\
\>", "Text",
  CellTags->"FunctionalD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "symfun", "]"}], ";"}]], "Input",
  CellLabel->"In[756]:=  (21:54:18 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"symfun", "[", 
        RowBox[{"z_", ",", "fieldname_Symbol"}], "]"}], ":=", 
      RowBox[{"Expand", "[", 
        RowBox[{"SUNSimplify", "[", 
          RowBox[{"FixedPoint", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"Collect2", "[", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{"DotSimplify", "[", 
                          RowBox[{"#1", "/.", 
                            RowBox[{"Times", "\[Rule]", "Dot"}]}], "]"}], "/.",
                         
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"(", "qi___", ")"}], ".", 
                                  RowBox[{"QuantumField", "[", 
                                    RowBox[{
                                    "par1___", ",", "fieldname", ",", "li1_", 
                                      ",", "sui1_"}], "]"}], ".", 
                                  RowBox[{"QuantumField", "[", 
                                    RowBox[{
                                    "par2___", ",", "fieldname", ",", "li2_", 
                                      ",", "sui2_"}], "]"}], ".", "qf___"}], 
                                " ", ")"}], "any_"}], ")"}], "\[RuleDelayed]",
                           
                          RowBox[{
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"(", 
                                  RowBox[{"qi", ".", 
                                    RowBox[{"QuantumField", "[", 
                                      RowBox[{
                                      "par1", ",", "A", ",", "li1", ",", 
                                        "sui2"}], "]"}], ".", 
                                    RowBox[{"QuantumField", "[", 
                                      RowBox[{
                                      "par2", ",", "fieldname", ",", "li2", 
                                        ",", "sui1"}], "]"}], ".", "qf"}], 
                                  " ", ")"}], 
                                RowBox[{"(", 
                                  RowBox[{"any", "/.", 
                                    RowBox[{"{", 
                                      RowBox[{
                                        RowBox[{
                                        "sui1", "\[RuleDelayed]", "sui2"}], 
                                        ",", 
                                        RowBox[{
                                        "sui2", "\[RuleDelayed]", "sui1"}]}], 
                                      "}"}]}], ")"}]}], ")"}], "/;", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{"!", 
                                  RowBox[{"(", 
                                    RowBox[{"FreeQ2", "[", 
                                      RowBox[{"any", ",", 
                                        RowBox[{"{", 
                                        RowBox[{"sui1", ",", "su2"}], "}"}]}],
                                       "]"}], ")"}]}], ")"}], "&&", 
                              RowBox[{"!", 
                                RowBox[{"(", 
                                  RowBox[{"OrderedQ", "[", 
                                    RowBox[{"{", 
                                      RowBox[{"sui1", ",", "sui2"}], "}"}], 
                                    "]"}], ")"}]}]}]}]}]}], "/.", 
                      RowBox[{"Dot", "\[Rule]", "Times"}]}], ",", 
                    "QuantumField"}], "]"}], "&"}], ",", "z", ",", "42"}], 
            "]"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[757]:=  (21:54:18 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r2", " ", "=", " ", 
      RowBox[{"symfun", "[", 
        RowBox[{"r1", ",", "A"}], "]"}]}]], "Input",
  CellLabel->"In[758]:=  (21:54:18 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[758]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r3", "=", 
      RowBox[{"r2", "/.", 
        RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
  CellLabel->"In[759]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"2", " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[759]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Inspection reveals that still terms are the same. Gather the terms with \
two ",
  Cell[BoxData[
      FormBox[
        RowBox[{"f", " "}], TraditionalForm]]],
  "'s:"
}], "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"twof", "=", 
      RowBox[{"Select", "[", 
        RowBox[{"r3", ",", 
          RowBox[{
            RowBox[{
              RowBox[{"Count", "[", 
                RowBox[{"#", ",", 
                  RowBox[{"SUNF", "[", "__", "]"}]}], "]"}], "===", "2"}], 
            "&"}]}], "]"}]}]], "Input",
  CellLabel->"In[760]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["3", "4"]}], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[760]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"twofnew", "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
            "twof", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
            "+", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"twof", "[", 
                  RowBox[{"[", "2", "]"}], "]"}], "/.", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"b", "\[RuleDelayed]", "c"}], ",", 
                    RowBox[{"c", "\[RuleDelayed]", "b"}]}], "}"}]}], ")"}]}], 
          ")"}], "/.", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"a", "\[RuleDelayed]", "c"}], ",", 
            RowBox[{"c", "\[RuleDelayed]", "a"}]}], "}"}]}]}]], "Input",
  CellLabel->"In[761]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["e",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[761]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r4", " ", "=", " ", 
      RowBox[{"r3", "-", "twof", "+", "twofnew"}]}]], "Input",
  CellLabel->"In[762]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"2", " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[762]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell["\<\
Check that this is now indeed the same as the t4 result from the \
first attempt.\
\>", "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData["t4"], "Input",
  CellLabel->"In[763]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[763]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w0", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"FieldStrength", "[", 
            RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "g", ",", 
              RowBox[{"{", 
                RowBox[{"A", ",", "a", ",", "b"}], "}"}], ",", "1"}], "]"}]}],
         "+", 
        RowBox[{
          RowBox[{"QuantumField", "[", 
            RowBox[{"A", ",", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
              RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], 
          RowBox[{"FieldStrength", "[", 
            RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "a", ",", 
              RowBox[{"{", 
                RowBox[{"A", ",", "b", ",", "d"}], "}"}], ",", "1"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]], "Input",
  CellLabel->"In[764]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}], 
            RowBox[{"g", "\[NoBreak]", 
              FormBox[
                RowBox[{"{", 
                  RowBox[{"A", ",", "a", ",", "b"}], "}"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["1",
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}], 
            RowBox[{"a", "\[NoBreak]", 
              FormBox[
                RowBox[{"{", 
                  RowBox[{"A", ",", "b", ",", "d"}], "}"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["1",
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[764]=  (21:54:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w1", "=", 
      RowBox[{"Explicit", "[", "w0", "]"}]}]], "Input",
  CellLabel->"In[765]:=  (21:54:20 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["g",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["g",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"g", "\[NoBreak]", "a", "\[NoBreak]", "b"}]]}]}], 
            ")"}]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "d"}]]}]}], 
            ")"}], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[765]=  (21:54:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w2", "=", 
      RowBox[{
        RowBox[{"ExpandPartialD", "[", "w1", "]"}], "/.", 
        RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
  CellLabel->"In[766]:=  (21:54:21 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[766]=  (21:54:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dif1", " ", "=", " ", 
      RowBox[{"w2", "-", "r4"}]}]], "Input",
  CellLabel->"In[767]:=  (21:54:21 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[767]=  (21:54:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dif2", "=", 
      RowBox[{"symfun", "[", 
        RowBox[{"dif1", ",", "A"}], "]"}]}]], "Input",
  CellLabel->"In[768]:=  (21:54:21 on 5/16/01)",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[768]=  (21:54:22 on 5/16/01)"]
}, Closed]],

Cell["quod erat demonstrandum.", "Text",
  CellTags->"FunctionalD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Unset", "[", 
        RowBox[{"s", "[", "\[Phi]", "]"}], "]"}], ";", 
      RowBox[{"Unset", "[", 
        RowBox[{"S", "[", "A", "]"}], "]"}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{
        "Ag", ",", "F1", ",", "F2", ",", "t1", ",", "t2", ",", "t3", ",", 
          "t4", ",", "F", ",", "r1", ",", "r2", ",", "r3", ",", "r4", ",", 
          "symfun", ",", "twof", ",", "twofnew", ",", "w0", ",", "w1", ",", 
          "w2", ",", "dif1", ",", "dif2", ",", "dif3"}], "]"}]}]], "Input",
  CellLabel->"In[769]:=  (21:54:22 on 5/16/01)",
  CellTags->"FunctionalD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples of funcional differentiation as used in FeynRule", \
"Subsubsection",
  CellTags->"FunctionalD"],

Cell["This is a part of the QCD Lagrangian.", "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"(", 
        RowBox[{"Gstrong", "*", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", " ", 
                RowBox[{"LorentzIndex", "[", "li1", "]"}], ",", " ", " ", 
                RowBox[{"SUNIndex", "[", "si2", "]"}]}], "]"}], ".", "  ", 
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", " ", 
                RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", " ", " ",
                 
                RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}], " ", ".", 
            "  ", 
            RowBox[{"QuantumField", "[", 
              RowBox[{
                RowBox[{"PartialD", "[", 
                  RowBox[{"LorentzIndex", "[", "li1", "]"}], "]"}], ",", " ", 
                "GaugeField", ",", " ", " ", " ", 
                RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", 
                RowBox[{"SUNIndex", "[", "si1", "]"}]}], "]"}]}], "*", "  ", 
          RowBox[{"SUNF", "[", 
            RowBox[{
              RowBox[{"SUNIndex", "[", "si1", "]"}], ",", " ", 
              RowBox[{"SUNIndex", "[", "si2", "]"}], ",", " ", 
              RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}]}], ")"}], "/", 
      "4"}]], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["li1",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["si2",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["li2",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["si4",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["si1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si4",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[6]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{"%", ",", " ", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Mu]", "}"}], ",", 
                  RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", " ",
             
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Nu]", "}"}], ",", 
                  RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", " ",
             
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Rho]", "}"}], ",", 
                  RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], 
          "}"}]}], "]"}]], "Input",
  CellLabel->"In[7]:=",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["si1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si4",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[7]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Calc", "[", 
        RowBox[{"%", "//", "Calc"}], "]"}], "//", "Factor"}]], "Input",
  CellLabel->"In[8]:=",
  CellTags->"FunctionalD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[8]="]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FV", "ObjectName",
  CellTags->"FV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FV"],

Cell[TextData[{
  "FV[",
  StyleBox["p",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the four-dimensional vector ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
  ". "
}], "Text",
  CellTags->"FV"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  "."
}], "Text",
  CellTags->"FV"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[773]:=  (21:54:24 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[773]=  (21:54:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[774]:=  (21:54:24 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[774]=  (21:54:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[775]:=  (21:54:24 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
  CellLabel->"Out[775]//StandardForm=  (21:54:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[776]:=  (21:54:24 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[776]//StandardForm=  (21:54:24 on 5/16/01)"]
}, Closed]],

Cell["\<\
There is no special function to expand momenta in FV; \
ExpandScalarProduct does the job.\
\>", "Text",
  CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[777]:=  (21:54:24 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[777]=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[778]:=  (21:54:25 on 5/16/01)",
  CellTags->"FV"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "-", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[778]//StandardForm=  (21:54:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FVD", "ObjectName",
  CellTags->"FVD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FVD"],

Cell[TextData[{
  "FVD[",
  StyleBox["p",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the D-dimensional vector ",
  StyleBox["p",
    FontSlant->"Italic"],
  " with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "."
}], "Text",
  CellTags->"FVD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  "."
}], "Text",
  CellTags->"FVD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[779]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[779]=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[780]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[780]=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FVD", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[781]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
  CellLabel->"Out[781]//StandardForm=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"FVD", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[782]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[782]//StandardForm=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell["There is no special function to expand momenta in FVD.", "Text",
  CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FVD", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[783]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[783]=  (21:54:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[784]:=  (21:54:25 on 5/16/01)",
  CellTags->"FVD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}]}], "]"}], "-", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"q", ",", "D"}], "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[784]//StandardForm=  (21:54:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GA", "ObjectName",
  CellTags->"GA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GA"],

Cell[TextData[{
  "GA[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] can be used as input for a 4-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Gamma]", "\[Mu]"], " "}], TraditionalForm]]],
  "and is transformed into DiracGamma[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] by FeynCalcInternal (=FCI).GA[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", "\[Nu]", ","}], TraditionalForm]]],
  " ...] is a short form for GA[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "].GA[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "]. ... ."
}], "Text",
  CellTags->"GA"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GAD"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GS"],
  "."
}], "Text",
  CellTags->"GA"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GA"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[785]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[785]=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[786]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[786]=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", 
        RowBox[{"GA", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[787]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
  CellLabel->"Out[787]//StandardForm=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input",
  CellLabel->"In[788]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[788]=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[789]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Rho]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Sigma]", "]"}]}]], "Output",
  CellLabel->"Out[789]//StandardForm=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Beta]", "]"}]}]], "Input",
  CellLabel->"In[790]:=  (21:54:26 on 5/16/01)",
  CellTags->"GA"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[790]=  (21:54:26 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GAD", "ObjectName",
  CellTags->"GAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GAD"],

Cell[TextData[{
  "GAD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] can be used as input for a D-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Gamma]", "\[Mu]"], " "}], TraditionalForm]]],
  "and is transformed into DiracGamma[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ",D],D] by FeynCalcInternal (=FCI).GAD[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
  "] is a short form for GAD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "].GAD[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "]. ... ."
}], "Text",
  CellTags->"GAD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GS"],
  "."
}], "Text",
  CellTags->"GAD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GAD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[791]:=  (21:54:26 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[791]=  (21:54:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
  CellLabel->"In[792]:=  (21:54:26 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[792]=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[793]:=  (21:54:27 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
  CellLabel->"Out[793]//StandardForm=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input",
  CellLabel->"In[794]:=  (21:54:27 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[794]=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[795]:=  (21:54:27 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Rho]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Sigma]", "]"}]}]], "Output",
  CellLabel->"Out[795]//StandardForm=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", "\[Alpha]", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GAD", "[", "\[Beta]", "]"}]}]], "Input",
  CellLabel->"In[796]:=  (21:54:27 on 5/16/01)",
  CellTags->"GAD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[796]=  (21:54:27 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GammaEpsilon", "ObjectName",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GammaEpsilon"],

Cell["\<\
GammaEpsilon[exp] gives a series expansion of Gamma[exp] in Epsilon \
up to order 6 (where EulerGamma is neglected).\
\>", "Text",
  CellTags->"GammaEpsilon"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GammaExpand",
    ButtonData:>"GammaExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GammaExpand"],
  ", ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Series2"],
  "."
}], "Text",
  CellTags->"GammaEpsilon"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GammaEpsilon"],

Cell["\<\
If the argument is of the form (1+a Epsilon) the result is not \
calculated but tabulated.\
\>", "Text",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", 
      RowBox[{"1", "+", 
        RowBox[{"a", " ", "Epsilon"}]}], "]"}]], "Input",
  CellLabel->"In[797]:=  (21:54:27 on 5/16/01)",
  CellTags->"GammaEpsilon"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$749", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "5"]}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["5",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], " ", 
                SuperscriptBox["a", "5"]}], "-", 
              RowBox[{
                FractionBox["1", "36"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], " ", 
                SuperscriptBox["a", "5"]}]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "5"]}], "+", 
        RowBox[{
          FractionBox["1", "160"], " ", 
          SuperscriptBox["a", "4"], " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "4"]}], "-", 
        RowBox[{
          FractionBox["1", "3"], " ", 
          SuperscriptBox["a", "3"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          FractionBox["1", "12"], " ", 
          SuperscriptBox["a", "2"], " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[797]=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", 
      RowBox[{"1", "-", 
        RowBox[{"Epsilon", "/", "2"}]}], "]"}]], "Input",
  CellLabel->"In[798]:=  (21:54:27 on 5/16/01)",
  CellTags->"GammaEpsilon"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$750", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Pi]", "2"], " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]]}], "1152"], "+", 
              FractionBox[
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["5",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], "160"]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "5"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "4"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "4"]}], "2560"], "+", 
        RowBox[{
          FractionBox["1", "24"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"]}], "48"], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[798]=  (21:54:27 on 5/16/01)"]
}, Closed]],

Cell["For other arguments the expansion is calculated.", "Text",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", "Epsilon", "]"}]], "Input",
  CellLabel->"In[799]:=  (21:54:27 on 5/16/01)",
  CellTags->"GammaEpsilon"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$752", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          FractionBox["1", "120"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FractionBox["5", "3"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                RowBox[{
                  SuperscriptBox[
                    TagBox["\[Psi]",
                      PolyGamma], 
                    RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], "-", 
              RowBox[{"24", " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["5",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "4"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "4"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "3"]}], "160"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "2", ")"}]], "(", "1", ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}], "12"], "+", 
        FractionBox["1", 
          FormBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[799]=  (21:54:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", "x", "]"}]], "Input",
  CellLabel->"In[800]:=  (21:54:28 on 5/16/01)",
  CellTags->"GammaEpsilon"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CapitalGamma]", "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[800]=  (21:54:28 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GammaExpand", "ObjectName",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GammaExpand"],

Cell["\<\
GammaExpand[exp] rewrites Gamma[n + m] in exp (where n has Head \
Integer).\
\>", "Text",
  CellTags->"GammaExpand"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GammaEpsilon",
    ButtonData:>"GammaEpsilon",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GammaEpsilon"],
  "."
}], "Text",
  CellTags->"GammaExpand"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{"2", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
  CellLabel->"In[801]:=  (21:54:28 on 5/16/01)",
  CellTags->"GammaExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}], " ", 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[801]=  (21:54:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{
          RowBox[{"-", "3"}], "+", "Epsilon"}], "]"}], "]"}]], "Input",
  CellLabel->"In[802]:=  (21:54:28 on 5/16/01)",
  CellTags->"GammaExpand"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "2"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "1"}], ")"}], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], TraditionalForm]], "Output",
  CellLabel->"Out[802]=  (21:54:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{"1", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
  CellLabel->"In[803]:=  (21:54:28 on 5/16/01)",
  CellTags->"GammaExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "+", "1"}], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[803]=  (21:54:28 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gamma1", "ObjectName",
  CellTags->"Gamma1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Gamma1"],

Cell["\<\
Gamma1[al,ga, be,de] is a special product of Gamma functions \
expanded up to order Epsilon^2.\
\>", "Text",
  CellTags->"Gamma1"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Gamma2",
    ButtonData:>"Gamma2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma2"],
  ", ",
  ButtonBox["Gamma3",
    ButtonData:>"Gamma3",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma3"],
  "."
}], "Text",
  CellTags->"Gamma1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gamma2", "ObjectName",
  CellTags->"Gamma2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Gamma2"],

Cell["\<\
Gamma2[x,y] is a special product of Gamma functions expanded up to \
order Epsilon^3 when positive integer arguments are given.\
\>", "Text",
  CellTags->"Gamma2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Gamma1",
    ButtonData:>"Gamma1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma1"],
  ", ",
  ButtonBox["Gamma3",
    ButtonData:>"Gamma3",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma3"],
  "."
}], "Text",
  CellTags->"Gamma2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gamma3", "ObjectName",
  CellTags->"Gamma3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Gamma3"],

Cell["\<\
Gamma3[al,be,ga,ep] is a special product of Gamma functions \
expanded up to order Epsilon^n when positive integer arguments are given (the \
order n is determined by the option EpsilonOrder).\
\>", "Text",
  CellTags->"Gamma3"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Gamma1",
    ButtonData:>"Gamma1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma1"],
  ", ",
  ButtonBox["Gamma2",
    ButtonData:>"Gamma2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gamma2"],
  "."
}], "Text",
  CellTags->"Gamma3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gauge", "ObjectName",
  CellTags->"Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Gauge"],

Cell["\<\
Gauge is an option for GluonProgagator. If set to 1 the 't Hooft \
Feynman gauge is used.\
\>", "Text",
  CellTags->"Gauge"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["$Gauge",
    ButtonData:>"$Gauge",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$Gauge"],
  ", ",
  ButtonBox["GluonProgagator",
    ButtonData:>"GluonProgagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonProgagator"],
  "."
}], "Text",
  CellTags->"Gauge"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GaugeField", "ObjectName",
  CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GaugeField"],

Cell["\<\
GaugeField is just a name. No functional properties are associated \
with it. GaugeField is used as default setting for the option QuantumField of \
FieldStrength.\
\>", "Text",
  CellTags->"GaugeField"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FieldStrength"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuantumField"],
  "."
}], "Text",
  CellTags->"GaugeField"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell[BoxData["GaugeField"], "Input",
  CellLabel->"In[812]:=  (21:54:29 on 5/16/01)",
  CellTags->"GaugeField"],

Cell[BoxData[
    FormBox["A", TraditionalForm]], "Output",
  CellLabel->"Out[812]=  (21:54:29 on 5/16/01)",
  CellTags->"GaugeField"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
  CellLabel->"In[813]:=  (21:54:30 on 5/16/01)",
  CellTags->"GaugeField"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        FormBox["A",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[813]=  (21:54:30 on 5/16/01)",
  CellTags->"GaugeField"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GaugeXi", "ObjectName",
  CellTags->"GaugeXi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GaugeXi"],

Cell["GaugeXi is a head for gauge parameters.", "Text",
  CellTags->"GaugeXi"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Gauge",
    ButtonData:>"Gauge",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gauge"],
  ", ",
  ButtonBox["GaugeField",
    ButtonData:>"GaugeField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GaugeField"],
  "."
}], "Text",
  CellTags->"GaugeXi"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GA5", "ObjectName",
  CellTags->"GA5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GA5"],

Cell["GA5 is equivalent to DiracGamma[5] and denotes gamma5.", "Text",
  CellTags->"GA5"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GS"],
  "."
}], "Text",
  CellTags->"GA5"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GA5"],

Cell[CellGroupData[{

Cell[BoxData["GA5"], "Input",
  CellLabel->"In[56]:=  (15:15:03 on 9/3/02)",
  CellTags->"GA5"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
  CellLabel->"Out[56]=  (15:15:03 on 9/3/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "Tr"}]], "Input",
  CellLabel->"In[57]:=  (15:15:04 on 9/3/02)",
  CellTags->"GA5"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[57]=  (15:15:04 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GGV", "ObjectName",
  CellTags->"GGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GGV"],

Cell["GGV is equivalent to GluonGhostVertex.", "Text",
  CellTags->"GGV"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text",
  CellTags->"GGV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GhostPropagator", "ObjectName",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GhostPropagator"],

Cell[TextData[{
  "GhostPropagator[p, a, b] gives the ghost propagator where \"a\" and \"b\" \
are the color indices. GhostPropagator[p] omits the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"GhostPropagator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GHP",
    ButtonData:>"GHP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GHP"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text",
  CellTags->"GhostPropagator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", 
      RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[804]:=  (21:54:28 on 5/16/01)",
  CellTags->"GhostPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[804]=  (21:54:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", "p", "]"}]], "Input",
  CellLabel->"In[805]:=  (21:54:28 on 5/16/01)",
  CellTags->"GhostPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox["\[ImaginaryI]", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[805]=  (21:54:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
  CellLabel->"In[806]:=  (21:54:29 on 5/16/01)",
  CellTags->"GhostPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["ci1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["ci2",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[806]=  (21:54:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GhostPropagator", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", " ", 
        "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[807]:=  (21:54:29 on 5/16/01)",
  CellTags->"GhostPropagator"],

Cell[BoxData[
    RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
  CellLabel->"Out[807]//StandardForm=  (21:54:29 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GHP", "ObjectName",
  CellTags->"GHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GHP"],

Cell[TextData[{
  "GHP[p, a, b] gives the ghost propagator where \"a\" and \"b\" are the \
color indices. GHP[p] omits the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"GHP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text",
  CellTags->"GHP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GHP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", 
      RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[808]:=  (21:54:29 on 5/16/01)",
  CellTags->"GHP"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[808]=  (21:54:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", "p", "]"}]], "Input",
  CellLabel->"In[809]:=  (21:54:29 on 5/16/01)",
  CellTags->"GHP"],

Cell[BoxData[
    FormBox[
      FractionBox["\[ImaginaryI]", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[809]=  (21:54:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
  CellLabel->"In[810]:=  (21:54:29 on 5/16/01)",
  CellTags->"GHP"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["ci1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["ci2",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[810]=  (21:54:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"GHP", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "]"}], 
      "]"}]], "Input",
  CellLabel->"In[811]:=  (21:54:29 on 5/16/01)",
  CellTags->"GHP"],

Cell[BoxData[
    RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
  CellLabel->"Out[811]//StandardForm=  (21:54:29 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GluonField", "ObjectName",
  CellTags->"GluonField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GluonField"],

Cell["GluonField is a name of a gauge field.", "Text",
  CellTags->"GluonField"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GaugeField",
    ButtonData:>"GaugeField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GaugeField"],
  "."
}], "Text",
  CellTags->"GluonField"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GluonField"],

Cell[CellGroupData[{

Cell[BoxData["GluonField"], "Input",
  CellLabel->"In[61]:=  (15:22:17 on 9/3/02)",
  CellTags->"GluonField"],

Cell[BoxData[
    FormBox["A", TraditionalForm]], "Output",
  CellLabel->"Out[61]=  (15:22:17 on 9/3/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GluonField", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
  CellLabel->"In[63]:=  (15:22:29 on 9/3/02)",
  CellTags->"GluonField"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        FormBox["A",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[63]=  (15:22:29 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GluonGhostVertex", "ObjectName",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GluonGhostVertex"],

Cell["\<\
GluonGhostVertex[{p,mu,a}, {q,nu,b}, {k,rho,c}] or \
GluonGhostVertex[ p,mu,a , q,nu,b , k,rho,c ] yields the Gluon-Ghost vertex. \
The first argument represents the gluon and the third argument the outgoing \
ghost field (but incoming four-momentum). The dimension and the name of the \
coupling constant are determined by the options Dimension and \
CouplingConstant.\
\>", "Text",
  CellTags->"GluonGhostVertex"],

Cell[TextData[{
  "See also: ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonSelfEnergy",
    ButtonData:>"GluonSelfEnergy",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonSelfEnergy"],
  ", ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonGhostVertex", "]"}]], "Input",
  CellLabel->"In[814]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonGhostVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[814]=  (21:54:30 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonGhostVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[815]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonGhostVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        SuperscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[815]=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[816]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonGhostVertex"],

Cell[BoxData[
    RowBox[{"Gstrong", " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"k", ",", "D"}], "]"}]}], "]"}], " ", 
      RowBox[{"SUNF", "[", 
        RowBox[{
          RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[816]//StandardForm=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonGhostVertex", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "k", ",", "3"}], 
      "]"}]], "Input",
  CellLabel->"In[817]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonGhostVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        SuperscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["li1",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[817]=  (21:54:30 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GluonPropagator", "ObjectName",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GluonPropagator"],

Cell[TextData[{
  "GluonPropagator[p, {mu, a}, {nu, b}] or GluonPropagator[p, mu, a , nu, b ] \
yields the gluon propagator. GluonPropagator[p, {mu}, {nu}] or \
GluonPropagator[p, mu, nu] omits the SUNDelta. The gauge and the dimension is \
determined by the option Gauge and Dimension. The following settings of Gauge \
are possible: 1 for the Feynman gauge; ",
  Cell[BoxData[
      FormBox["\[Alpha]", TraditionalForm]]],
  " for the general covariant gauge; {Momentum[n] ,1} for the axial gauge."
}], "Text",
  CellTags->"GluonPropagator"],

Cell["GP can be used as an abbreviation of GluonPropagator.", "Text",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData["GP"], "Input",
  CellLabel->"In[818]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox["GluonPropagator", TraditionalForm]], "Output",
  CellLabel->"Out[818]=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonPropagator", "]"}]], "Input",
  CellLabel->"In[819]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[819]=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonSelfEnergy",
    ButtonData:>"GluonSelfEnergy",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonSelfEnergy"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text",
  CellTags->"GluonPropagator"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
      "]"}]], "Input",
  CellLabel->"In[820]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[820]=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
  CellLabel->"In[821]:=  (21:54:30 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], 
                SuperscriptBox["p", "2"]], "-", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[821]=  (21:54:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"Gauge", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"Momentum", "[", "n", "]"}], ",", "1"}], "}"}]}]}], 
      "]"}]], "Input",
  CellLabel->"In[822]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["n",
                    "TraditionalForm"], "2"]}], "+", 
              FractionBox[
                RowBox[{
                  SuperscriptBox[
                    FormBox["n",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["n",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["n",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["n",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["n",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], 
                RowBox[{
                  FormBox["n",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[822]=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[823]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[823]=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell["\<\
This is a convenient way to enter amplitudes by hand (GP is an \
abbreviation GluonPropagator).\
\>", "Text",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GP", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
  CellLabel->"In[824]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["li1",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["li2",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["ci1",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["ci2",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[824]=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GP", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[825]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"-", "\[ImaginaryI]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"li3", ",", "li4"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
  CellLabel->"Out[825]//StandardForm=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "1"}]}], "]"}]], "Input",
  CellLabel->"In[826]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"11", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["19", "6"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[826]=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "2"}]}], "]"}]], "Input",
  CellLabel->"In[827]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "3"]}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  FractionBox["1", "6"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[827]=  (21:54:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "3"}]}], "]"}]], "Input",
  CellLabel->"In[828]:=  (21:54:31 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            SubscriptBox["T", "f"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["4", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], "-", 
                FractionBox[
                  RowBox[{"4", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[828]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "4"}]}], "]"}]], "Input",
  CellLabel->"In[829]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"10", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["10", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[829]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "5"}]}], "]"}]], "Input",
  CellLabel->"In[830]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"10", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["10", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubsuperscriptBox["g", "s", "2"]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["S", "n"], " ", 
            SubscriptBox["T", "f"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["4", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], "-", 
                FractionBox[
                  RowBox[{"4", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubsuperscriptBox["g", "s", "2"]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[830]=  (21:54:32 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GluonSelfEnergy", "ObjectName",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GluonSelfEnergy"],

Cell["\<\
GluonSelfEnergy[{mu, a}, {nu,b}] yields the 1-loop Gluon \
selfenergy.\
\>", "Text",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonSelfEnergy", "]"}]], "Input",
  CellLabel->"In[24]:=  (0:23:22 on 9/20/02)",
  CellTags->"GluonSelfEnergy"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  RowBox[{"log", "(", 
                    RowBox[{
                      SuperscriptBox["\[Mu]", "2"], " ", "_"}], ")"}], 
                  "\[RuleDelayed]", "0"}], ",", 
                RowBox[{
                  TagBox["\[DoubledGamma]",
                    (EulerGamma&)], "\[RuleDelayed]", 
                  RowBox[{"log", "(", 
                    RowBox[{"4", " ", "\[Pi]"}], ")"}]}]}], "}"}]}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
          RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (0:23:22 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text",
  CellTags->"GluonSelfEnergy"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonSelfEnergy", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[25]:=  (0:23:57 on 9/20/02)",
  CellTags->"GluonSelfEnergy"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox["20", 
                RowBox[{"3", " ", 
                  TagBox["\[CurlyEpsilon]",
                    TraditionalForm]}]], "-", 
              FractionBox["62", "9"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox["20", "9"], "-", 
              FractionBox["8", 
                RowBox[{"3", " ", 
                  TagBox["\[CurlyEpsilon]",
                    TraditionalForm]}]]}], ")"}], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[25]=  (0:23:57 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GluonVertex", "ObjectName",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GluonVertex"],

Cell["Note: All momenta are flowing into the vertex.", "Text",
  CellTags->"GluonVertex"],

Cell["\<\
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or GluonVertex[p,mu,a , \
q,nu,b , k,la,c ] yields the 3-gluon vertex. GluonVertex[{p,mu}, {q,nu}, \
{k,la}] yields the 3-gluon vertex without color structure and the coupling \
constant. GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
GluonVertex[{mu,a}, {nu,b}, {la,c}, {si,d}] or GluonVertex[p,mu,a , q,nu,b , \
k,la,c , s,si,d] or GluonVertex[ mu,a , nu,b , la,c , si,d ] yields the \
4-gluon vertex. The dimension and the name of the coupling constant are \
determined by the options Dimension and CouplingConstant.\
\>", "Text",
  CellTags->"GluonVertex"],

Cell["GV can be used as an abbreviation of GluonVertex.", "Text",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData["GV"], "Input",
  CellLabel->"In[831]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox["GluonVertex", TraditionalForm]], "Output",
  CellLabel->"Out[831]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonVertex", "]"}]], "Input",
  CellLabel->"In[832]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[832]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonGhostVertex"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  "."
}], "Text",
  CellTags->"GluonVertex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[833]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[833]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GluonVertex", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], "//", 
      "Explicit"}]], "Input",
  CellLabel->"In[834]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[834]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[835]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["V", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]"}]], 
        "(", 
        RowBox[{"p", "\[NoBreak]", 
          RowBox[{",", " "}], "\[NoBreak]", "q", "\[NoBreak]", 
          RowBox[{",", " "}], "\[NoBreak]", "r"}], ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[835]=  (21:54:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GluonVertex", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}], "//", 
      "Explicit"}]], "Input",
  CellLabel->"In[836]:=  (21:54:32 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox[
            FormBox["r",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["r",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[836]=  (21:54:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[837]:=  (21:54:33 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[837]=  (21:54:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[838]:=  (21:54:34 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[838]=  (21:54:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
      "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", "\[Rho]", ",", "c", ",",
         "\[Sigma]", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[839]:=  (21:54:34 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[839]=  (21:54:34 on 5/16/01)"]
}, Closed]],

Cell["\<\
A very convenient way to enter diagrams by hand is to label each \
line hitting a vertex by a number and put this number after the inflowing \
momentum.\
\>", "Text",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GV", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3"}], 
      "]"}]], "Input",
  CellLabel->"In[840]:=  (21:54:34 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li3",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[840]=  (21:54:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GV", "[", 
      RowBox[{
      "p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3", ",", "s", ",", 
        "4"}], "]"}]], "Input",
  CellLabel->"In[841]:=  (21:54:34 on 5/16/01)",
  CellTags->"GluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[841]=  (21:54:34 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GO", "ObjectName",
  CellTags->"GO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GO"],

Cell["GO is equivalent to Twist2GluonOperator.", "Text",
  CellTags->"GO"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text",
  CellTags->"GO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GP", "ObjectName",
  CellTags->"GP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GP"],

Cell["GP is equivalent to GluonPropagator.", "Text",
  CellTags->"GP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  "."
}], "Text",
  CellTags->"GP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GrassmannParity", "ObjectName",
  CellTags->"GrassmannParity"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GrassmannParity"],

Cell["\<\
GrassmannParity is a data type. E.g. DataType[F, GrassmannParity] = \
1 declares F to be of bosonic type and DataType[F, GrassmannParity] = -1 of \
fermionic one.\
\>", "Text",
  CellTags->"GrassmannParity"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  "."
}], "Text",
  CellTags->"GrassmannParity"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GS", "ObjectName",
  CellTags->"GS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GS"],

Cell[TextData[{
  "GS[p] can be used as input for a 4-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "p", " ", "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
          "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", "/", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"=", 
                RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
              RowBox[{
                SubscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], 
        TraditionalForm]]],
  " and is transformed into DiracGamma[Momentum[p]] by FeynCalcInternal \
(=FCI). GS[p,q, ...] is a short form for GS[p].GS[q]. ... ."
}], "Text",
  CellTags->"GS"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GAD"],
  "."
}], "Text",
  CellTags->"GS"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GS"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", "p", "]"}]], "Input",
  CellLabel->"In[847]:=  (21:54:35 on 5/16/01)",
  CellTags->"GS"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[847]=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[848]:=  (21:54:35 on 5/16/01)",
  CellTags->"GS"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", "p", "]"}], "]"}]], "Output",
  CellLabel->"Out[848]//StandardForm=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[849]:=  (21:54:35 on 5/16/01)",
  CellTags->"GS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[849]=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[850]:=  (21:54:35 on 5/16/01)",
  CellTags->"GS"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", "p", "]"}], ".", 
      RowBox[{"GS", "[", "q", "]"}], ".", 
      RowBox[{"GS", "[", "r", "]"}], ".", 
      RowBox[{"GS", "[", "s", "]"}]}]], "Output",
  CellLabel->"Out[850]//StandardForm=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", "q", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GS", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[851]:=  (21:54:35 on 5/16/01)",
  CellTags->"GS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[851]=  (21:54:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GSD", "ObjectName",
  CellTags->"GSD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GSD"],

Cell[TextData[{
  "GSD[p] can be used as input for a D-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "p", " ", "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
          "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", "/", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"=", 
                RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
              RowBox[{
                SubscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], 
        TraditionalForm]]],
  " and is transformed into DiracGamma[Momentum[p,D],D] by FeynCalcInternal \
(=FCI). GSD[p,q, ...] is a short form for GSD[p].GSD[q]. ... ."
}], "Text",
  CellTags->"GSD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GAD"],
  "."
}], "Text",
  CellTags->"GSD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"GSD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GSD", "[", "p", "]"}]], "Input",
  CellLabel->"In[842]:=  (21:54:34 on 5/16/01)",
  CellTags->"GSD"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[842]=  (21:54:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GSD", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[843]:=  (21:54:35 on 5/16/01)",
  CellTags->"GSD"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
  CellLabel->"Out[843]//StandardForm=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GSD", "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[844]:=  (21:54:35 on 5/16/01)",
  CellTags->"GSD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[844]=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[845]:=  (21:54:35 on 5/16/01)",
  CellTags->"GSD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "p", "]"}], ".", 
      RowBox[{"GSD", "[", "q", "]"}], ".", 
      RowBox[{"GSD", "[", "r", "]"}], ".", 
      RowBox[{"GSD", "[", "s", "]"}]}]], "Output",
  CellLabel->"Out[845]//StandardForm=  (21:54:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "q", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GSD", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[846]:=  (21:54:35 on 5/16/01)",
  CellTags->"GSD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[846]=  (21:54:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gstrong", "ObjectName",
  CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Gstrong"],

Cell["Gstrong denotes the strong coupling constant.", "Text",
  CellTags->"Gstrong"],

Cell[TextData[{
  "See also: ",
  ButtonBox["CovariantD",
    ButtonData:>"CovariantD",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  CellTags->"Gstrong"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Gstrong"],

Cell["\<\
Gstrong has no functional properties. Only a typesetting rule is \
defined.\
\>", "Text",
  CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell[BoxData["Gstrong"], "Input",
  CellLabel->"In[852]:=  (21:54:35 on 5/16/01)",
  CellTags->"Gstrong"],

Cell[BoxData[
    FormBox[
      SubscriptBox["g", "s"], TraditionalForm]], "Output",
  CellLabel->"Out[852]=  (21:54:35 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GTI", "ObjectName",
  CellTags->"GTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GTI"],

Cell["GTI is like RHI, but with no functional properties.", "Text",
  CellTags->"GTI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"GTI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["GV", "ObjectName",
  CellTags->"GV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"GV"],

Cell["GV is equivalent to GluonVertex.", "Text",
  CellTags->"GV"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text",
  CellTags->"GV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Hill", "ObjectName",
  CellTags->"Hill"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Hill"],

Cell["\<\
Hill[x, y] gives the Hill identity with arguments x and y. The \
returned object is 0.\
\>", "Text",
  CellTags->"Hill"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text",
  CellTags->"Hill"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Hill"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hill", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[853]:=  (21:54:35 on 5/16/01)",
  CellTags->"Hill"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"log", "(", "a", ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "a"}], ")"}], "-", 
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"log", "(", 
            FractionBox[
              RowBox[{"1", "-", "a"}], 
              RowBox[{"1", "-", "b"}]], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{"log", "(", "a", ")"}]}], "+", 
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}], "-", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], 
                  RowBox[{"1", "-", "b"}]], ")"}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}], "-", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], 
                  RowBox[{"a", " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], ")"}], " ", 
          
          RowBox[{"log", "(", 
            FractionBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
              RowBox[{"a", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", "a", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{"1", "-", "a"}], 
            RowBox[{"1", "-", "b"}]], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", "b", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["b", "a"], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
            RowBox[{"a", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "b"}], ")"}]}]], ")"}], "-", 
        FractionBox[
          SuperscriptBox["\[Pi]", "2"], "6"]}], TraditionalForm]], "Output",
  CellLabel->"Out[853]=  (21:54:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"%", " ", "/.", " ", 
          RowBox[{"a", "\[RuleDelayed]", " ", ".123"}]}], " ", "/.", " ", 
        RowBox[{"b", "\[RuleDelayed]", " ", ".656"}]}], " ", "//", " ", 
      "Chop"}]], "Input",
  CellLabel->"In[854]:=  (21:54:36 on 5/16/01)",
  CellTags->"Hill"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[854]=  (21:54:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"Hill", "[", 
            RowBox[{"x", ",", 
              RowBox[{"x", " ", "y"}]}], "]"}], "//", "PowerExpand"}], "//", 
        "SimplifyPolyLog"}], "//", "Expand"}]], "Input",
  CellLabel->"In[855]:=  (21:54:36 on 5/16/01)",
  CellTags->"Hill"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "y"}], ")"}], " ", 
          RowBox[{"log", "(", "y", ")"}]}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "y"}], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"x", " ", "y"}], ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{"1", "-", "x"}], 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}]], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], " ", "y"}], 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}]], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[855]=  (21:54:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"%", " ", "/.", " ", 
            RowBox[{"x", "\[RuleDelayed]", " ", ".34"}]}], "/.", " ", 
          RowBox[{"y", "\[Rule]", " ", ".6"}]}], "//", "N"}], "//", 
      "Chop"}]], "Input",
  CellLabel->"In[856]:=  (21:54:36 on 5/16/01)",
  CellTags->"Hill"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[856]=  (21:54:36 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HypergeometricAC", "ObjectName",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"HypergeometricAC"],

Cell["\<\
HypergeometricAC[n][exp] analytically continues Hypergeometric2F1 \
functions in exp. The second argument n refers to the equation number (n) in \
chapter 2.10 of \"Higher Transcendental Functions\" by Ergelyi, Magnus, \
Oberhettinger, Tricomi. In case of eq. (6) (p.109) the last line is returned \
for HypergeometricAC[6][exp], while the first equality is given by \
HypergeometricAC[61][exp]. ((2.10.1) is identical to eq. (9.5.7) of \"Special \
Functions & their Applications\" by N.N.Lebedev).\
\>", "Text",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricAC", "]"}]], "Input",
  CellLabel->"In[857]:=  (21:54:36 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Collect2", "\[Rule]", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[857]=  (21:54:36 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypExplicit",
    ButtonData:>"HypExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypExplicit"],
  ", ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricIR"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricSE"],
  ", ",
  ButtonBox["ToHypergeometric",
    ButtonData:>"ToHypergeometric",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ToHypergeometric"],
  "."
}], "Text",
  CellTags->"HypergeometricAC"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"HypergeometricAC"],

Cell["These are all transformation rules currently built in.", "Text",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "1", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[858]:=  (21:54:36 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], 
            " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox[
                              RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{
                              RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                              "\[Gamma]", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        RowBox[{"1", "-", "z"}],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                "\[Gamma]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], 
              ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Alpha]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox["\[Beta]",
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{
                            "\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", 
                              "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        RowBox[{"1", "-", "z"}],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[858]=  (21:54:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "2", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[859]:=  (21:54:38 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Alpha]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
                              
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        FractionBox["1", "z"],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "z"}], ")"}], 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Beta]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Beta]", "-", "\[Gamma]", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{
                              RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", 
                              "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        FractionBox["1", "z"],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "z"}], ")"}], 
              RowBox[{"-", "\[Beta]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[859]=  (21:54:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "3", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[860]:=  (21:54:38 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Alpha]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        FractionBox["1", 
                          RowBox[{"1", "-", "z"}]],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Beta]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{
                              RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", 
                              "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        FractionBox["1", 
                          RowBox[{"1", "-", "z"}]],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{"-", "\[Beta]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[860]=  (21:54:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "4", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[861]:=  (21:54:38 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], 
              ")"}], " ", 
            TagBox[
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[InvisiblePrefixScriptBase]", 
                      FormBox["2",
                        "TraditionalForm"]], 
                    SubscriptBox["F", 
                      FormBox["1",
                        "TraditionalForm"]]}], "\[InvisibleApplication]", 
                  RowBox[{"(", 
                    RowBox[{
                      TagBox[
                        TagBox[
                          RowBox[{
                            TagBox["\[Alpha]",
                              Hypergeometric2F1,
                              Editable->True], ",", 
                            TagBox[
                              RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
                              
                              Hypergeometric2F1,
                              Editable->True]}],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        TagBox[
                          TagBox[
                            RowBox[{
                            "\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", 
                              "1"}],
                            Hypergeometric2F1,
                            Editable->True],
                          InterpretTemplate[ {
                            SlotSequence[ 1]}&]],
                        Hypergeometric2F1,
                        Editable->False], ";", 
                      TagBox[
                        RowBox[{"-", 
                          FractionBox[
                            RowBox[{"1", "-", "z"}], "z"]}],
                        Hypergeometric2F1,
                        Editable->True]}], ")"}]}],
                InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                Editable->False],
              Hypergeometric2F1], " ", 
            SuperscriptBox["z", 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]], "+", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]], 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{
                  RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                  "\[Gamma]"}]], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], 
              " ", 
              RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox[
                                RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"1", "-", "\[Alpha]"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{
                                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", 
                                "+", "\[Gamma]", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          RowBox[{"-", 
                            FractionBox[
                              RowBox[{"1", "-", "z"}], "z"]}],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox["z", 
                RowBox[{"\[Alpha]", "-", "\[Gamma]"}]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[861]=  (21:54:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "6", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[862]:=  (21:54:39 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{"-", "\[Beta]"}]], " ", 
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["\[Beta]",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["\[Gamma]",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"-", 
                      FractionBox["z", 
                        RowBox[{"1", "-", "z"}]]}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1]}], TraditionalForm]], "Output",
  CellLabel->"Out[862]=  (21:54:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "61", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[863]:=  (21:54:39 on 5/16/01)",
  CellTags->"HypergeometricAC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{"-", "\[Alpha]"}]], " ", 
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["\[Alpha]",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["\[Gamma]",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"-", 
                      FractionBox["z", 
                        RowBox[{"1", "-", "z"}]]}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1]}], TraditionalForm]], "Output",
  CellLabel->"Out[863]=  (21:54:39 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HypergeometricIR", "ObjectName",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"HypergeometricIR"],

Cell["\<\
HypergeometricIR[exp, t] substitutes for all \
Hypergeometric2F1[a,b,c,z] in exp by its Euler integral reprentation. The \
factor Integratedx[t, 0, 1] can be omitted by setting the option Integratedx \
\[Rule] False.\
\>", "Text",
  CellTags->"HypergeometricIR"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricAC"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricSE"],
  ", ",
  ButtonBox["ToHypergeometric",
    ButtonData:>"ToHypergeometric",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ToHypergeometric"],
  "."
}], "Text",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricIR", "]"}]], "Input",
  CellLabel->"In[864]:=  (21:54:39 on 5/16/01)",
  CellTags->"HypergeometricIR"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Integratedx", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[864]=  (21:54:39 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricIR", "[", 
      RowBox[{
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[865]:=  (21:54:39 on 5/16/01)",
  CellTags->"HypergeometricIR"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[865]=  (21:54:40 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"t", "^", "b"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "^", "c"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "^", "a"}]}], ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[866]:=  (21:54:40 on 5/16/01)",
  CellTags->"HypergeometricIR"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"b", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"b", "+", "c", "+", "2"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[866]=  (21:54:40 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricIR", "[", 
      RowBox[{"%", ",", "t"}], "]"}]], "Input",
  CellLabel->"In[867]:=  (21:54:40 on 5/16/01)",
  CellTags->"HypergeometricIR"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
        SuperscriptBox["t", "b"], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{"t", " ", "z"}], "+", "1"}], ")"}], "a"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[867]=  (21:54:40 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HypergeometricSE", "ObjectName",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"HypergeometricSE"],

Cell[TextData[{
  "HypergeometricSE[exp, ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "] expresses Hypergeometric functions by their series expansion in terms of \
a sum (the Sum is omitted and ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  ", running from 0 to ",
  Cell[BoxData[
      FormBox["\[Infinity]", TraditionalForm]]],
  ", is the summation index)."
}], "Text",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricSE", "]"}]], "Input",
  CellLabel->"In[868]:=  (21:54:40 on 5/16/01)",
  CellTags->"HypergeometricSE"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Simplify", "\[Rule]", "FullSimplify"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[868]=  (21:54:40 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricIR"],
  "."
}], "Text",
  CellTags->"HypergeometricSE"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricSE", "[", 
      RowBox[{
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "\[Nu]"}], 
      "]"}]], "Input",
  CellLabel->"In[869]:=  (21:54:40 on 5/16/01)",
  CellTags->"HypergeometricSE"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",\

  CellLabel->"Out[869]=  (21:54:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricSE", "[", 
      RowBox[{
        RowBox[{"HypergeometricPFQ", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}], ",", 
        "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[870]:=  (21:54:41 on 5/16/01)",
  CellTags->"HypergeometricSE"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",\

  CellLabel->"Out[870]=  (21:54:41 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HypExplicit", "ObjectName",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"HypExplicit"],

Cell[TextData[{
  "HypExplicit[exp, ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "] expresses Hypergeometric functions in exp by their definition in terms \
of a sum (the Sum is omitted and ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  " is the summation index)."
}], "Text",
  CellTags->"HypExplicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricIR"],
  "."
}], "Text",
  CellTags->"HypExplicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hypergeometric2F1", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[871]:=  (21:54:41 on 5/16/01)",
  CellTags->"HypExplicit"],

Cell[BoxData[
    FormBox[
      TagBox[
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[InvisiblePrefixScriptBase]", 
                FormBox["2",
                  "TraditionalForm"]], 
              SubscriptBox["F", 
                FormBox["1",
                  "TraditionalForm"]]}], "\[InvisibleApplication]", 
            RowBox[{"(", 
              RowBox[{
                TagBox[
                  TagBox[
                    RowBox[{
                      TagBox["a",
                        Hypergeometric2F1,
                        Editable->True], ",", 
                      TagBox["b",
                        Hypergeometric2F1,
                        Editable->True]}],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox[
                  TagBox[
                    TagBox["c",
                      Hypergeometric2F1,
                      Editable->True],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox["z",
                  Hypergeometric2F1,
                  Editable->True]}], ")"}]}],
          InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
          Editable->False],
        Hypergeometric2F1], TraditionalForm]], "Output",
  CellLabel->"Out[871]=  (21:54:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypExplicit", "[", 
      RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[872]:=  (21:54:42 on 5/16/01)",
  CellTags->"HypExplicit"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",\

  CellLabel->"Out[872]=  (21:54:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricPFQ", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}]], "Input",
  CellLabel->"In[873]:=  (21:54:42 on 5/16/01)",
  CellTags->"HypExplicit"],

Cell[BoxData[
    FormBox[
      TagBox[
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[InvisiblePrefixScriptBase]", 
                FormBox["3",
                  "TraditionalForm"]], 
              SubscriptBox["F", 
                FormBox["2",
                  "TraditionalForm"]]}], "\[InvisibleApplication]", 
            RowBox[{"(", 
              RowBox[{
                TagBox[
                  TagBox[
                    RowBox[{
                      TagBox["a",
                        HypergeometricPFQ,
                        Editable->True], ",", 
                      TagBox["b",
                        HypergeometricPFQ,
                        Editable->True], ",", 
                      TagBox["c",
                        HypergeometricPFQ,
                        Editable->True]}],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  HypergeometricPFQ,
                  Editable->False], ";", 
                TagBox[
                  TagBox[
                    RowBox[{
                      TagBox["d",
                        HypergeometricPFQ,
                        Editable->True], ",", 
                      TagBox["e",
                        HypergeometricPFQ,
                        Editable->True]}],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  HypergeometricPFQ,
                  Editable->False], ";", 
                TagBox["z",
                  HypergeometricPFQ,
                  Editable->True]}], ")"}]}],
          InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
          Editable->False],
        HypergeometricPFQ], TraditionalForm]], "Output",
  CellLabel->"Out[873]=  (21:54:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypExplicit", "[", 
      RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[874]:=  (21:54:42 on 5/16/01)",
  CellTags->"HypExplicit"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output",\

  CellLabel->"Out[874]=  (21:54:42 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["HypInt", "ObjectName",
  CellTags->"HypInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"HypInt"],

Cell["\<\
HypInt[exp, t]  substitutes for all  Hypergeometric2F1[a,b,c,z] in \
exp Gamma[c]/(Gamma[b] Gamma[c-b]) Integratedx[t,0,1]  t^(b-1) (1-t)^(c-b-1) \
(1-t z)^(-a).\
\>", "Text",
  CellTags->"HypInt"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"HypInt"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hypergeometric2F1", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[14]:=  (1:49:25 on 9/22/02)",
  CellTags->"HypInt"],

Cell[BoxData[
    FormBox[
      TagBox[
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[InvisiblePrefixScriptBase]", 
                FormBox["2",
                  "TraditionalForm"]], 
              SubscriptBox["F", 
                FormBox["1",
                  "TraditionalForm"]]}], "\[InvisibleApplication]", 
            RowBox[{"(", 
              RowBox[{
                TagBox[
                  TagBox[
                    RowBox[{
                      TagBox["a",
                        Hypergeometric2F1,
                        Editable->True], ",", 
                      TagBox["b",
                        Hypergeometric2F1,
                        Editable->True]}],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox[
                  TagBox[
                    TagBox["c",
                      Hypergeometric2F1,
                      Editable->True],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox["z",
                  Hypergeometric2F1,
                  Editable->True]}], ")"}]}],
          InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
          Editable->False],
        Hypergeometric2F1], TraditionalForm]], "Output",
  CellLabel->"Out[14]=  (1:49:25 on 9/22/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypInt", "[", 
      RowBox[{"%", ",", "t"}], "]"}]], "Input",
  CellLabel->"In[15]:=  (1:49:27 on 9/22/02)",
  CellTags->"HypInt"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{
            SubsuperscriptBox["\[Integral]", 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["1",
                "TraditionalForm"]], "\[DifferentialD]", 
            TagBox[
              FormBox["t",
                "TraditionalForm"],
              TraditionalForm,
              Editable->True], "\[VeryThinSpace]"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[15]=  (1:49:29 on 9/22/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IFPD", "ObjectName",
  CellTags->"IFPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IFPD"],

Cell["IFPD[p, m] denotes (p^2 - m^2).", "Text",
  CellTags->"IFPD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text",
  CellTags->"IFPD"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IFPDOff", "ObjectName",
  CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IFPDOff"],

Cell["\<\
IFPDOff[exp_,q1_, q2_, ...] changes from IFPD representation to \
FeynAmpDenominator[ ...]. The q1, q2, ... are the integration momenta.\
\>", \
"Text",
  CellTags->"IFPDOff"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["IFPD",
    ButtonData:>"IFPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPD"],
  ", ",
  ButtonBox["IFPDOn",
    ButtonData:>"IFPDOn",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPDOn"],
  "."
}], "Text",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"IFPD", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Input",
  CellLabel->"In[86]:=  (15:34:01 on 9/3/02)",
  CellTags->"IFPDOff"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"(\"\>",
          "TraditionalForm"], 
        FormBox[
          SuperscriptBox["p", "2"],
          "TraditionalForm"], 
        FormBox["\<\" - \"\>",
          "TraditionalForm"], 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], 
        FormBox["\<\")\"\>",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[86]=  (15:34:01 on 9/3/02)",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[87]:=  (15:34:02 on 9/3/02)",
  CellTags->"IFPDOff"],

Cell[BoxData[
    RowBox[{"IFPD", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[87]//StandardForm=  (15:34:02 on 9/3/02)",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"IFPDOff", "[", 
      RowBox[{"%", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[88]:=  (15:34:04 on 9/3/02)",
  CellTags->"IFPDOff"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[88]=  (15:34:04 on 9/3/02)",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[89]:=  (15:34:06 on 9/3/02)",
  CellTags->"IFPDOff"],

Cell[BoxData[
    RowBox[{
      RowBox[{"-", 
        SuperscriptBox["m", "2"]}], "+", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[89]//StandardForm=  (15:34:06 on 9/3/02)",
  CellTags->"IFPDOff"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IFPDOn", "ObjectName",
  CellTags->"IFPDOn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IFPDOn"],

Cell["\<\
IFPDOn[exp_,q1_, q2_, ...] changes from FeynAmpDenominator[ ...] \
representation to the IFPD one (Inverse Feynman Propagator Denominator). \
I.e., FeynAmpDenominator[PropagatorDenominator[a,b]] is replaced by \
1/IFPD[a,b] and The q1, q2, ... are the integration momenta.\
\>", "Text",
  CellTags->"IFPDOn"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["IFPD",
    ButtonData:>"IFPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPD"],
  ", ",
  ButtonBox["IFPDOff",
    ButtonData:>"IFPDOff",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPDOff"],
  "."
}], "Text",
  CellTags->"IFPDOn"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IncludePair", "ObjectName",
  CellTags->"IncludePair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IncludePair"],

Cell["\<\
IncludePair is an option for FC2RHI. Possible settings are True and \
False.\
\>", "Text",
  CellTags->"IncludePair"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FC2RHI",
    ButtonData:>"FC2RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FC2RHI"],
  "."
}], "Text",
  CellTags->"IncludePair"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IndexPosition", "ObjectName",
  CellTags->"IndexPosition"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IndexPosition"],

Cell["IndexPosition is an option for FieldStrength.", "Text",
  CellTags->"IndexPosition"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FieldStrength"],
  "."
}], "Text",
  CellTags->"IndexPosition"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Indices", "ObjectName",
  CellTags->"Indices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Indices"],

Cell["\<\
Indices is an option for FORM2FeynCalc. Its default setting is \
Automatic. It may be set to a list, if the FORM-file does not contain a \
I(ndices) statement.\
\>", "Text",
  CellTags->"Indices"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FORM2FeynCalc"],
  "."
}], "Text",
  CellTags->"Indices"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["InitialFunction", "ObjectName",
  CellTags->"InitialFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"InitialFunction"],

Cell["\<\
InitialFunction is an option of FeynRule the setting of which is \
applied to the first argument of FeynRule before anything else\
\>", "Text",
  CellTags->"InitialFunction"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynRule"],
  "."
}], "Text",
  CellTags->"InitialFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["InitialSubstitutions", "ObjectName",
  CellTags->"InitialSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"InitialSubstitutions"],

Cell["\<\
InitialSubstitutions is an option for OneLoop and OneLoopSum and \
Write2. All substitutions indicated hereby are done at the end of the \
calculation.\
\>", "Text",
  CellTags->"InitialSubstitutions"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  ", ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  "."
}], "Text",
  CellTags->"InitialSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["InsideDiracTrace", "ObjectName",
  CellTags->"InsideDiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"InsideDiracTrace"],

Cell["\<\
InsideDiracTrace is and option of DiracSimplify. If set to True, \
DiracSimplify assumes to operate inside a DiracTrace, i.e., products of an \
odd number of Dirac matrices are discarded. Furthermore simple traces are \
calculated (but divided by a factor 4, i.e. :  \
DiracSimplify[DiracMatrix[a,b], InsideDiracTrace\[Rule]True]  yields \
ScalarProduct[a,b]) Traces involving more than four DiracGamma's and \
DiracGamma[5] are not performed.\
\>", "Text",
  CellTags->"InsideDiracTrace"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  "."
}], "Text",
  CellTags->"InsideDiracTrace"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IntegralTable", "ObjectName",
  CellTags->"IntegralTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IntegralTable"],

Cell["\<\
IntegralTable is an option of OneLoopSimplify, TwoLoopSimplify and \
FeynAmpDenominatorSimplify. It may be set to a list of the form : \
{FCIntegral[ ... ] \[RuleDelayed] bla, ...}.\
\>", "Text",
  CellTags->"IntegralTable"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["TwoLoopSimplify",
    ButtonData:>"TwoLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TwoLoopSimplify"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text",
  CellTags->"IntegralTable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IntegrateByParts", "ObjectName",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IntegrateByParts"],

Cell["\<\
IntegrateByParts[(1-t)^(a Epsilon -1) g[t], deriv, t] does an \
integration by parts of the definite integral over t from 0 to 1.\
\>", "Text",\

  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "IntegrateByParts", "]"}]], "Input",
  CellLabel->"In[913]:=  (21:54:57 on 5/16/01)",
  CellTags->"IntegrateByParts"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Hold", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[913]=  (21:54:57 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"IntegrateByParts", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
          RowBox[{"g", "[", "t", "]"}]}], ",", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[914]:=  (21:54:57 on 5/16/01)",
  CellTags->"IntegrateByParts"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{
              SuperscriptBox["g", "\[Prime]",
                MultilineFunction->None], "(", "t", ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "t"}], ")"}], 
              RowBox[{"a", " ", 
                TagBox["\[CurlyEpsilon]",
                  TraditionalForm]}]]}], 
          RowBox[{"a", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}]], "+", 
        FractionBox[
          RowBox[{"g", "(", "0", ")"}], 
          RowBox[{"a", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[914]=  (21:54:58 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integratedx", "ObjectName",
  CellTags->"Integratedx"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Integratedx"],

Cell["\<\
Integratedx[x, low, up] is a variable representing the integration \
operator Integrate[#, {x,low,up}]&.\
\>", "Text",
  CellTags->"Integratedx"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynmanParametrize",
    ButtonData:>"FeynmanParametrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynmanParametrize"],
  ", ",
  ButtonBox["CalculateCounterTerm",
    ButtonData:>"CalculateCounterTerm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CalculateCounterTerm"],
  ", ",
  ButtonBox["TimedIntegrate",
    ButtonData:>"TimedIntegrate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TimedIntegrate"],
  ", ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricIR"],
  ", ",
  ButtonBox["HypInt",
    ButtonData:>"HypInt",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypInt"],
  ", ",
  ButtonBox["TimedIntegrate",
    ButtonData:>"TimedIntegrate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TimedIntegrate"],
  "."
}], "Text",
  CellTags->"Integratedx"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integrate2", "ObjectName",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Integrate2"],

Cell[TextData[{
  "Integrate2 is like Integrate, but : Integrate2[a_Plus, b__] := \
Map[Integrate2[#, b]&, a] ( more linear algebra and partial fraction \
decomposition is done) Integrate2[f[x] DeltaFunction[x], x] \[Rule] f[0] \
Integrate2[f[x] DeltaFunction[x0-x], x] \[Rule] f[x0]. Integrate2[f[x] \
DeltaFunction[a + b x], x] \[Rule] Integrate[f[x] (1/Abs[b]) \
DeltaFunction[a/b + x], x], where abs[b] \[Rule] b, if b is a Symbol, and if \
b = -c, then abs[-c] \[Rule] c, i.e., the variable contained in b is supposed \
to be positive. ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          SuperscriptBox["\[Pi]", "2"]}], TraditionalForm]]],
  " is replaced by 6 Zeta2. Integrate2[1/(1-y),{y,x,1}] is intepreted as \
distribution, i.e. as Integrate2[-1/(1-y)],{y, 0, x}] \[Rule] Log[1-y]. \
Integrate2[1/(1-x),{x,0,1}] \[Rule] 0."
}], "Text",
  CellTags->"Integrate2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate3",
    ButtonData:>"Integrate3",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate3"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumT"],
  "."
}], "Text",
  CellTags->"Integrate2"],

Cell["\<\
NOTE: Since Integrate2 does do a reordering and partial fraction \
decomposition before calling the integral table of Integrate3 it will in \
general be slower compared to Integrate3 for sums of integrals. I.e., if the \
integrand has already an expanded form and if partial fraction decomposition \
is not necessary it is more effective to use Integrate3.\
\>", "Text",
  CellTags->"Integrate2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Integrate2", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{
              RowBox[{"Log", "[", "x", "]"}], "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
      "Timing"}]], "Input",
  CellLabel->"In[875]:=  (21:54:42 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.09000000000000341`", " ", "Second"}], ",", 
          RowBox[{
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "-", 
            RowBox[{
              FractionBox["3", "2"], " ", 
              RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
              RowBox[{"log", "(", "2", ")"}]}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[875]=  (21:54:42 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Since Integrate2 uses table-look-up methods it is much faster than ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s Integrate."
}], "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"2", ",", 
            RowBox[{"x", "^", "2"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[876]:=  (21:54:42 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "-", "4"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[876]=  (21:54:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"3", ",", 
            RowBox[{"-", "x"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[877]:=  (21:54:42 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "-", 
        FractionBox[
          RowBox[{"3", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "4"], "-", 
        RowBox[{"2", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "+", "1"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[877]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"3", ",", 
            RowBox[{"1", "/", 
              RowBox[{"(", 
                RowBox[{"1", "+", "x"}], ")"}]}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[878]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "2", ")"}]}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        FractionBox[
          RowBox[{"3", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "4"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "3"], "(", "2", ")"}], "3"], "-", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"2", " ", 
          RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[878]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[879]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[879]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Integrate2 does integration in a Hadamard sense, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", "1"], 
          RowBox[{
            RowBox[{"f", "(", "x", ")"}], 
            RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
  " means acutally expanding the result of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "\[Delta]", 
            RowBox[{"1", "-", "\[Delta]"}]], 
          RowBox[{
            RowBox[{"f", "(", "x", ")"}], 
            RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
  "up do ",
  Cell[BoxData[
      FormBox[
        RowBox[{"O", "(", "\[Delta]", ")"}], TraditionalForm]]],
  " and neglecting all ",
  Cell[BoxData[
      FormBox["\[Delta]", TraditionalForm]]],
  "-dependent terms. E.g. ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubsuperscriptBox["\[Integral]", "\[Delta]", 
                RowBox[{"1", "-", "\[Delta]"}]], 
              RowBox[{
                RowBox[{"1", "/", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}], " ", "=", " ", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{"-", "log"}], 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                SubsuperscriptBox["\[VerticalSeparator]", "\[Delta]", 
                  RowBox[{"1", "-", "\[Delta]"}]]}], "=", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    RowBox[{"log", "(", "\[Delta]", ")"}]}], "+", 
                  RowBox[{"log", "(", "1", ")"}]}], "\[Implies]", "0."}]}]}], 
          " "}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->False,
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[880]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[880]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "In the physics literature sometimes the \"+\" notation is used. In \
FeynCalc the ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"(", 
            FractionBox["1", 
              RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]]],
  " is represented by ",
  Cell[BoxData[
      FormBox[
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], TraditionalForm]]],
  " or just ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", "."}]}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[881]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[881]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{"1", "-", "x"}]}], "]"}], "/", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[882]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", "-", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[882]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"Log", "[", "x", "]"}], " ", 
              RowBox[{"Log", "[", 
                RowBox[{"1", "+", "x"}], "]"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[883]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "8"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[883]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Log", "[", "x", "]"}], "^", "2"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[884]:=  (21:54:43 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output",
  CellLabel->"Out[884]=  (21:54:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{"-", "x"}]}], "]"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[885]:=  (21:54:44 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "4"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[885]=  (21:54:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Log", "[", "x", "]"}], " ", 
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[886]:=  (21:54:44 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"3", "-", 
        RowBox[{"2", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[886]=  (21:54:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{"PolyLog", "[", 
            RowBox[{"3", ",", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[887]:=  (21:54:44 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"\[Zeta]", "(", "2", ")"}], "4"]}], "+", 
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "2"], "+", 
        FractionBox["3", "16"]}], TraditionalForm]], "Output",
  CellLabel->"Out[887]=  (21:54:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"Log", "[", "x", "]"}], "^", "2"}], " ", 
              RowBox[{"Log", "[", 
                RowBox[{"1", "-", "x"}], "]"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[888]:=  (21:54:44 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}], "6"], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
  CellLabel->"Out[888]=  (21:54:44 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"x", " ", 
                          RowBox[{"(", 
                            RowBox[{"1", "-", "z"}], ")"}]}], "+", "z"}], 
                      ")"}], " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x", "+", 
                        RowBox[{"x", " ", "z"}]}], ")"}]}], ")"}], "/", 
                "z"}]}], "]"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x", "+", 
              RowBox[{"x", " ", "z"}]}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[889]:=  (21:54:44 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "3"], "(", "z", ")"}], 
          RowBox[{"6", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"z", "+", "1"}], ")"}]}], 
          RowBox[{"3", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "z"}],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"1", "-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              FractionBox["1", 
                RowBox[{"z", "+", "1"}]], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], 
                  RowBox[{"z", "+", "1"}]]}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              FractionBox[
                RowBox[{"1", "-", "z"}], 
                RowBox[{"z", "+", "1"}]], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"1", "-", "z"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[889]=  (21:54:45 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Apart", "[", 
      RowBox[{
        RowBox[{"Integrate2", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"x", "^", 
                RowBox[{"(", 
                  RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  RowBox[{"1", "-", "x"}]}], "]"}]}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
        "OPEm"}], "]"}]], "Input",
  CellLabel->"In[890]:=  (21:54:45 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"\[Zeta]", "(", "2", ")"}], 
            RowBox[{"m", "-", "1"}]]}], "-", 
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], 
          SuperscriptBox["m", "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "2"}],
                    "TraditionalForm"], ")"}]}], " ", 
              RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
            RowBox[{
              SubscriptBox["S", "12"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], "+", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "m"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[890]=  (21:54:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"Log", "[", "x", "]"}], " ", 
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{"1", "+", "x"}]]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[891]:=  (21:54:46 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubsuperscriptBox["S", 
              RowBox[{"-", "1"}], "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["3", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["3", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "3"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "21"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "12"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "2"}], "-", "1", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "2", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "1", "-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "21"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "12"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"2", "-", "11"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["13", "8"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "-", 
        RowBox[{
          FractionBox["21", "8"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "-", 
        RowBox[{"2", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "12"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}]}], "+", 
        RowBox[{
          FractionBox["5", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "45"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          SuperscriptBox["\[Pi]", "4"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[891]=  (21:54:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"OPEm", "\[Rule]", "2"}]}]], "Input",
  CellLabel->"In[892]:=  (21:54:46 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["5", "4"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"3", " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        FractionBox[
          RowBox[{"5", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "2"], "-", 
        RowBox[{
          FractionBox["21", "8"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "+", 
        FractionBox[
          RowBox[{"21", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "8"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}], "12"], "-", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "45"], "-", "6"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[892]=  (21:54:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", "%", "]"}]], "Input",
  CellLabel->"In[893]:=  (21:54:47 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0.050513835402744345`", TraditionalForm]], "Output",
  CellLabel->"Out[893]=  (21:54:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NIntegrate", "[", 
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"Log", "[", "x", "]"}], " ", 
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{"1", "+", "x"}]]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[894]:=  (21:54:47 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0.050513835398271964`", TraditionalForm]], "Output",
  CellLabel->"Out[894]=  (21:54:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  FractionBox[
                    RowBox[{"1", "-", "x"}], 
                    RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "x"}], 
                      RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[895]:=  (21:54:47 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "1"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "1"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{"3", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{"3", " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "3"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "2"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "3"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "21"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "2"}]], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "12"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "21"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{"7", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"8", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{"21", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"8", " ", "m"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[895]=  (21:54:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
  CellLabel->"In[896]:=  (21:54:48 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[896]=  (21:54:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[897]:=  (21:54:48 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[897]=  (21:54:48 on 5/16/01)"]
}, Closed]],

Cell["\<\
This is the polarized non-singlet spin splitting function whose \
first moment vanishes.\
\>", "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]}]], "Input",
  CellLabel->"In[898]:=  (21:54:48 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "x"}], "+", 
                    FractionBox["3", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{"40", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "24"}], " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                    RowBox[{"48", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", "3"}], 
                  ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"88", " ", "x"}], "9"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"16", " ", 
                          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
                    FractionBox["2", "3"]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
              RowBox[{
                FractionBox["80", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["8", "9"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{"log", "(", 
                          RowBox[{"x", "+", "1"}], ")"}], " ", 
                        RowBox[{"log", "(", "x", ")"}]}], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{
                          SubscriptBox["Li", "2"], "(", 
                          RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
                RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{
                FractionBox["4", "3"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"5", " ", "x"}], "-", 
                    FractionBox["22", 
                      RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                FractionBox["4", "9"], " ", 
                RowBox[{"(", 
                  RowBox[{"53", "-", 
                    RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["536", "9"], "-", 
                    RowBox[{"16", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"88", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
                    RowBox[{"24", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", 
                    FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[898]=  (21:54:49 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Integrate2", "[", 
        RowBox[{"t", ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
      "Timing"}]], "Input",
  CellLabel->"In[899]:=  (21:54:49 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"1.0109999999999957`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[899]=  (21:54:50 on 5/16/01)"]
}, Closed]],

Cell["\<\
Expanding t with respect to x yields a form already suitable for \
Integrate3 and therefore the following is faster:\
\>", "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Integrate3", "[", 
          RowBox[{
            RowBox[{"Expand", "[", 
              RowBox[{"t", ",", "x"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
        "Expand"}], "//", "Timing"}]], "Input",
  CellLabel->"In[900]:=  (21:54:50 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.18999999999999773`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[900]=  (21:54:50 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
  CellLabel->"In[901]:=  (21:54:50 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[902]:=  (21:54:56 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[902]=  (21:54:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "5"], 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[903]:=  (21:54:56 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["6959", "10800"]}], "+", 
        FractionBox[
          RowBox[{"46", " ", 
            RowBox[{"log", "(", "2", ")"}]}], "45"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[903]=  (21:54:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "@", "%"}]], "Input",
  CellLabel->"In[904]:=  (21:54:56 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0.06419859938720329`", TraditionalForm]], "Output",
  CellLabel->"Out[904]=  (21:54:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NIntegrate", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "5"], 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[905]:=  (21:54:56 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox["0.06419859938720235`", TraditionalForm]], "Output",
  CellLabel->"Out[905]=  (21:54:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], 
          SuperscriptBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[906]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"], " ", 
              RowBox[{
                SubsuperscriptBox["S", "1", "2"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}]}], "m"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              FractionBox[
                RowBox[{"m", "-", "1"}], "2"],
              "TraditionalForm"], ")"}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              FractionBox["m", "2"],
              "TraditionalForm"], ")"}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "11"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "2", ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], "m"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[906]=  (21:54:57 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integrate3", "ObjectName",
  CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Integrate3"],

Cell["\<\
Integrate3 contains the integral table used by Integrate2. \
Integration is performed in a distributional sense. Integrate3 works more \
effectively on a sum of expressions if they are expanded or collected with \
respect to the integration variable. See the examples in Integrate2.\
\>", \
"Text",
  CellTags->"Integrate3"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  "."
}], "Text",
  CellTags->"Integrate3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "OPEm"], " ", 
          RowBox[{"Log", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[907]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "1"}], ")"}], "2"]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[907]=  (21:54:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["x", "OPEm"], " ", 
            RowBox[{"Log", "[", "x", "]"}], " ", 
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}]}], 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[908]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox["m",
            "TraditionalForm"], ")"}], "-", 
        RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox["m",
            "TraditionalForm"], ")"}], "+", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output",
  CellLabel->"Out[908]=  (21:54:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"a", 
            FractionBox[
              RowBox[{
                SuperscriptBox["x", "OPEm"], " ", 
                RowBox[{"Log", "[", "x", "]"}], " ", 
                RowBox[{"Log", "[", 
                  RowBox[{"1", "-", "x"}], "]"}]}], 
              RowBox[{"1", "-", "x"}]]}], "+", 
          RowBox[{"b", 
            FractionBox[
              RowBox[{
                SuperscriptBox["x", "OPEm"], 
                RowBox[{"PolyLog", "[", 
                  RowBox[{"3", ",", 
                    RowBox[{"-", "x"}]}], "]"}]}], 
              RowBox[{"1", "+", "x"}]]}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[909]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{
                SubscriptBox["S", "12"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "-", 
              RowBox[{
                SubscriptBox["S", "21"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "+", 
              TagBox[
                RowBox[{"\[Zeta]", "(", 
                  TagBox["3",
                    (Editable -> True)], ")"}],
                InterpretTemplate[ Zeta[ #]&]]}], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", "b", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                SuperscriptBox[
                  RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "8"], "+", 
              RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{"-", "2"}]], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"log", "(", "2", ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SubscriptBox["S", "3"], "(", 
                      FormBox["m",
                        "TraditionalForm"], ")"}], "-", 
                    RowBox[{
                      SubscriptBox["S", 
                        RowBox[{"-", "3"}]], "(", 
                      FormBox["m",
                        "TraditionalForm"], ")"}]}], ")"}]}], "+", 
              RowBox[{
                SubscriptBox["S", 
                  RowBox[{"3", "-", "1"}]], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "-", 
              RowBox[{
                FractionBox["3", "4"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{"-", "1"}]], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}], "-", 
              RowBox[{
                FractionBox["3", "4"], " ", 
                RowBox[{"log", "(", "2", ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[909]=  (21:54:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{"DeltaFunctionPrime", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[910]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[910]=  (21:54:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "x", "]"}], " ", 
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[911]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "(", "1", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[911]=  (21:54:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[912]:=  (21:54:57 on 5/16/01)",
  CellTags->"Integrate3"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[912]=  (21:54:57 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integrate4", "ObjectName",
  CellTags->"Integrate4"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Integrate4"],

Cell["\<\
Integrate4 is like Integrate5, but interruptable if $VeryVerbose \
>2.\
\>", "Text",
  CellTags->"Integrate4"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate5",
    ButtonData:>"Integrate5",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate5"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  "."
}], "Text",
  CellTags->"Integrate4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Integrate5", "ObjectName",
  CellTags->"Integrate5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Integrate5"],

Cell["Integrate5 is like Integrate2.", "Text",
  CellTags->"Integrate5"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  "."
}], "Text",
  CellTags->"Integrate5"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IntermediateSubstitutions", "ObjectName",
  CellTags->"IntermediateSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IntermediateSubstitutions"],

Cell["\<\
IntermediateSubstitutions is an option for OneLoop. All \
substitutions indicated hereby are done at an intermediate stage of the \
calculation.\
\>", "Text",
  CellTags->"IntermediateSubstitutions"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"IntermediateSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["InverseMellin", "ObjectName",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"InverseMellin"],

Cell[TextData[{
  "InverseMellin[exp, y] performs the inverse Mellin transform of polynomials \
in OPEm. The inverse transforms are not calculated but a table-lookup is \
done. WARNING: do not \"trust\" the results for the inverse Mellin transform \
involving SumT's; there is an unresolved inconsistency here (related to ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "m"}], ")"}], "."}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"InverseMellin"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["OPEm",
    ButtonData:>"OPEi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumT"],
  "."
}], "Text",
  CellTags->"InverseMellin"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}]], "Input",
  CellLabel->"In[915]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]], TraditionalForm]], "Output",
  CellLabel->"Out[915]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"OPEm", "+", "3"}], ")"}]}], ",", "y"}], "]"}]], "Input",
  CellLabel->"In[916]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["y", 
        RowBox[{"m", "+", "2"}]], TraditionalForm]], "Output",
  CellLabel->"Out[916]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{"1", ",", "y"}], "]"}]], "Input",
  CellLabel->"In[917]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "y"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[917]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"OPEm", "^", "4"}]}], ",", "y"}], "]"}]], "Input",
  CellLabel->"In[918]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "6"]}], " ", 
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SuperscriptBox["log", "3"], "(", "y", ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[918]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"1", "/", "OPEm"}], "+", "1"}], ",", "y"}], "]"}]], "Input",\

  CellLabel->"In[919]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "y"}],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox["y", 
            RowBox[{"m", "-", "1"}]]}], "+", 
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[919]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"1", "/", "i"}], "+", "1"}], ",", "y", ",", "i"}], 
      "]"}]], "Input",
  CellLabel->"In[920]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "y"}],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox["y", 
            RowBox[{"i", "-", "1"}]]}], "+", 
        SuperscriptBox["y", 
          RowBox[{"i", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[920]=  (21:54:58 on 5/16/01)"]
}, Closed]],

Cell["\<\
The inverse operation to InverseMellin is done by Integrate2.\
\>", \
"Text",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"InverseMellin", "[", 
          RowBox[{
            RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"y", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[921]:=  (21:54:58 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      FractionBox["1", "m"], TraditionalForm]], "Output",
  CellLabel->"Out[921]=  (21:54:59 on 5/16/01)"]
}, Closed]],

Cell["\<\
Below is a list of all built-in basic inverse Mellin transforms .\
\
\>", "Text",
  CellTags->"InverseMellin"],

Cell[BoxData[
    RowBox[{
      RowBox[{"list", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", 
            FractionBox["1", 
              RowBox[{"OPEm", "+", "n"}]], ",", 
            FractionBox["1", 
              RowBox[{
                RowBox[{"-", "OPEm"}], "+", "n"}]], ",", 
            RowBox[{"PolyGamma", "[", 
              RowBox[{"0", ",", "OPEm"}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"OPEm", "-", "1"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"1", "-", "OPEm"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"OPEm", "+", "1"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              SuperscriptBox["OPEm", "2"]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
            
            FractionBox[
              SuperscriptBox[
                RowBox[{"SumS", "[", 
                  RowBox[{"1", ",", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "2"], 
              "OPEm"], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"2", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"2", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
            
            RowBox[{"SumS", "[", 
              RowBox[{"3", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            SuperscriptBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{"OPEm", "-", "1"}]}], "]"}], "2"], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "2", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"2", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            SuperscriptBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "3"], ",", 
            RowBox[{
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], " ", 
              RowBox[{"SumS", "[", 
                RowBox[{"2", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "1", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], "}"}]}], 
      ";"}]], "Input",
  CellLabel->"In[922]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    RowBox[{
      RowBox[{"im", "[", "z_", "]"}], ":=", 
      RowBox[{"z", " ", "\[LongRightArrow]", 
        RowBox[{"InverseMellin", "[", 
          RowBox[{"z", ",", "y"}], "]"}]}]}]], "Input",
  CellLabel->"In[923]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"OPEm", "^", 
        RowBox[{"(", 
          RowBox[{"-", "3"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[924]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          SuperscriptBox["m", "3"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "y", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[924]=  (21:54:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"OPEm", "^", 
        RowBox[{"(", 
          RowBox[{"-", "2"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[925]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], " ", 
            RowBox[{"log", "(", "y", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[925]=  (21:54:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"PolyGamma", "[", 
        RowBox[{"0", ",", "OPEm"}], "]"}], "]"}]], "Input",
  CellLabel->"In[926]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            TagBox["\[Psi]",
              PolyGamma], 
            RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                TagBox["\[DoubledGamma]",
                  (EulerGamma&)]}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "y"}],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], "-", 
            RowBox[{
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[926]=  (21:54:59 on 5/16/01)"]
}, Closed]],

Cell[BoxData[" "], "Input",
  CellLabel->"In[927]:=  (21:54:59 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[928]:=  (21:55:00 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox["1", 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[928]=  (21:55:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"OPEm", "-", "1"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[929]:=  (21:55:00 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          RowBox[{"m", "-", "1"}]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "2"}]]}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[929]=  (21:55:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"OPEm", "+", "1"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[930]:=  (21:55:00 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          RowBox[{"m", "+", "1"}]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], "-", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "y"}], ")"}], " ", 
              SuperscriptBox["y", "m"]}], "+", 
            RowBox[{
              RowBox[{"log", "(", "y", ")"}], " ", 
              SuperscriptBox["y", "m"]}], "+", 
            SuperscriptBox["y", "m"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[930]=  (21:55:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{
              RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", 
        RowBox[{"OPEm", "^", "2"}]}], "]"}]], "Input",
  CellLabel->"In[931]:=  (21:55:00 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
                RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", "y", ")"}]}], ")"}]}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[931]=  (21:55:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{
              RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", "OPEm"}], 
      "]"}]], "Input",
  CellLabel->"In[932]:=  (21:55:01 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"log", "(", "y", ")"}], "-", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[932]=  (21:55:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"SumS", "[", 
            RowBox[{"1", ",", 
              RowBox[{
                RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "^", "2"}], "/", 
        "OPEm"}], "]"}]], "Input",
  CellLabel->"In[933]:=  (21:55:01 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubsuperscriptBox["S", "1", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}], "+", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}], "2"], "-", 
                RowBox[{"3", " ", 
                  RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}], "+", 
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", "y", ")"}]}]}], ")"}]}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[933]=  (21:55:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"2", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[934]:=  (21:55:01 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "+", 
                FractionBox[
                  RowBox[{"log", "(", "y", ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[934]=  (21:55:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"2", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", "OPEm"}], 
      "]"}]], "Input",
  CellLabel->"In[935]:=  (21:55:01 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
                RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[935]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"3", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[936]:=  (21:55:02 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}], " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]]}], "-", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}], 
                  RowBox[{"2", " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "y"}], ")"}]}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[936]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", "1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[937]:=  (21:55:02 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "11"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox[
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[937]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"im", "[", 
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[938]:=  (21:55:02 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "-", 
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox["1", 
                        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
                FractionBox[
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[938]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"im", "[", 
        RowBox[{"SumS", "[", 
          RowBox[{"2", ",", "1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[939]:=  (21:55:02 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "-", 
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox["1", 
                        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
                FractionBox[
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", "y", ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[939]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", "1", ",", "1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[940]:=  (21:55:02 on 5/16/01)",
  CellTags->"InverseMellin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "111"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[940]=  (21:55:02 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"im", ",", "list"}], "]"}], ";"}]], "Input",
  CellLabel->"In[941]:=  (21:55:03 on 5/16/01)",
  CellTags->"InverseMellin"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Isolate", "ObjectName",
  CellTags->"Isolate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Isolate"],

Cell["\<\
Isolate[expr] substitutes abbreviations KK[i] for all Plus[...] \
(sub-sums) in expr. The inserted KK[i] have head HoldForm. Isolate[expr, \
varlist] substitutes KK[i] for all subsums in expr which are free of any \
occurence of a member of the list varlist. Instead of KK any other head or a \
list of names of the abbreviations may be specified with the option \
IsolateNames.\
\>", "Text",
  CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Isolate", "]"}]], "Input",
  CellLabel->"In[942]:=  (21:55:03 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"IsolateNames", "\[Rule]", "KK"}], ",", 
          RowBox[{"IsolatePrint", "\[Rule]", "False"}], ",", 
          RowBox[{"IsolateSplit", "\[Rule]", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 1]]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[942]=  (21:55:03 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t0", "=", 
      RowBox[{"Isolate", "[", 
        RowBox[{"a", "+", "b"}], "]"}]}]], "Input",
  CellLabel->"In[943]:=  (21:55:03 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{"KK", "(", "1", ")"}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[943]=  (21:55:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Isolate", "[", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], " ", "+", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", "d"}], ")"}], " ", "f"}], " ", "+", " ", 
            "e"}], ",", "f"}], "]"}]}]], "Input",
  CellLabel->"In[944]:=  (21:55:03 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      RowBox[{"e", "+", 
        RowBox[{"f", " ", 
          TagBox[
            RowBox[{"KK", "(", "1", ")"}],
            HoldForm]}], "+", 
        RowBox[{"f", " ", 
          TagBox[
            RowBox[{"KK", "(", "2", ")"}],
            HoldForm]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[944]=  (21:55:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "t1", "]"}]], "Input",
  CellLabel->"In[945]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    RowBox[{"e", "+", 
      RowBox[{"f", " ", 
        TagBox[
          RowBox[{"KK", "[", "1", "]"}],
          HoldForm]}], "+", 
      RowBox[{"f", " ", 
        TagBox[
          RowBox[{"KK", "[", "2", "]"}],
          HoldForm]}]}]], "Output",
  CellLabel->"Out[945]//StandardForm=  (21:55:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"t0", ",", " ", "t1", ",", " ", 
        RowBox[{"ReleaseHold", "[", "t1", "]"}]}], "}"}]], "Input",
  CellLabel->"In[946]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          TagBox[
            RowBox[{"KK", "(", "1", ")"}],
            HoldForm], ",", 
          RowBox[{"e", "+", 
            RowBox[{"f", " ", 
              TagBox[
                RowBox[{"KK", "(", "1", ")"}],
                HoldForm]}], "+", 
            RowBox[{"f", " ", 
              TagBox[
                RowBox[{"KK", "(", "2", ")"}],
                HoldForm]}]}], ",", 
          RowBox[{"e", "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", "d"}], ")"}], " ", "f"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[946]=  (21:55:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"a", "[", "z", "]"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "+", 
                RowBox[{"c", " ", 
                  RowBox[{"(", 
                    RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"d", "[", "z", "]"}], " ", 
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}]}]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "d"}], "}"}], ",", 
        RowBox[{"IsolateNames", "\[Rule]", "F"}]}], "]"}]], "Input",
  CellLabel->"In[947]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"d", "(", "z", ")"}], " ", 
          TagBox[
            RowBox[{"F", "(", "3", ")"}],
            HoldForm]}], "+", 
        RowBox[{
          RowBox[{"a", "(", "z", ")"}], " ", 
          TagBox[
            RowBox[{"F", "(", "4", ")"}],
            HoldForm]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[947]=  (21:55:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"??", "F"}]], "Input",
  CellLabel->"In[948]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox["\<\"F is a fermion field.\"\>", TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {GridBox[{
                  {
                    RowBox[{
                      RowBox[{"F", "(", "3", ")"}], "=", 
                      RowBox[{"y", "+", "z"}]}]},
                  {" "},
                  {
                    RowBox[{
                      RowBox[{"F", "(", "4", ")"}], "=", 
                      RowBox[{"b", "+", 
                        RowBox[{"c", " ", 
                          RowBox[{"\<\"HoldForm\"\>", "[", 
                            RowBox[{"F", "(", "3", ")"}], "]"}]}]}]}]}
                  },
                GridBaseline->{Baseline, {1, 1}},
                ColumnWidths->0.999,
                ColumnAlignments->{Left}]}
            },
          GridBaseline->{Baseline, {1, 1}},
          ColumnAlignments->{Left}],
        Definition[ F],
        Editable->False], TraditionalForm]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{"a", "-", "b", "-", "c", "-", "d", "-", "e"}], ",", 
        RowBox[{"IsolateNames", "\[Rule]", "L"}], ",", 
        RowBox[{"IsolateSplit", "\[Rule]", "15"}]}], "]"}]], "Input",
  CellLabel->"In[949]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{"L", "(", "3", ")"}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[949]=  (21:55:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"L", "[", "2", "]"}], ",", 
        RowBox[{"L", "[", "1", "]"}]}], "}"}]], "Input",
  CellLabel->"In[950]:=  (21:55:04 on 5/16/01)",
  CellTags->"Isolate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"a", "-", "b"}], ",", 
          RowBox[{"L", "(", "1", ")"}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[950]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t0", ",", "t1", ",", "L"}], "]"}]], "Input",
  CellLabel->"In[951]:=  (21:55:05 on 5/16/01)",
  CellTags->"Isolate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IsolateHead", "ObjectName",
  CellTags->"IsolateHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IsolateHead"],

Cell["IsolateHead is equivalent to IsolateNames.", "Text",
  CellTags->"IsolateHead"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["IsolateNames",
    ButtonData:>"IsolateNames",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IsolateNames"],
  "."
}], "Text",
  CellTags->"IsolateHead"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IsolateNames", "ObjectName",
  CellTags->"IsolateNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IsolateNames"],

Cell["\<\
IsolateNames is an option for Isolate and Collect2. Its default \
setting is KK. Instead of a symbol the setting may also be a list with the \
names of the abbrevations.\
\>", "Text",
  CellTags->"IsolateNames"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  ", ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Collect2"],
  "."
}], "Text",
  CellTags->"IsolateNames"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IsolatePrint", "ObjectName",
  CellTags->"IsolatePrint"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IsolatePrint"],

Cell["\<\
IsolatePrint is an option of Isolate. If it is set to OutputForm \
(or any other *Form) the definitions of the abbreviations are printed during \
the operation of Isolate. The setting IsolatePrint \[Rule] False suppresses \
printing.\
\>", "Text",
  CellTags->"IsolatePrint"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  "."
}], "Text",
  CellTags->"IsolatePrint"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["IsolateSplit", "ObjectName",
  CellTags->"IsolateSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"IsolateSplit"],

Cell["\<\
IsolateSplit is an option for Isolate. Its setting determines the \
maximum number of characters of FortranForm[expr] which are abbreviated by \
Isolate. If the expression is larger than the indicated number, it is split \
into smaller pieces and onto each subsum Isolate is applied. With the default \
setting IsolateSplit \[Rule] Infinity no splitting is done.\
\>", "Text",
  CellTags->"IsolateSplit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  "."
}], "Text",
  CellTags->"IsolateSplit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["KeepOnly", "ObjectName",
  CellTags->"KeepOnly"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"KeepOnly"],

Cell["\<\
KeepOnly is an option of OneLoop. It may be set to B0, C0, D0 \
keeping only the corresponding coefficients. The default setting is False. If \
KeepOnly is set to {} then the part of the amplitude which is not coefficient \
of B0, C0, D0 is kept.\
\>", "Text",
  CellTags->"KeepOnly"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  "."
}], "Text",
  CellTags->"KeepOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["KK", "ObjectName",
  CellTags->"KK"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"KK"],

Cell["\<\
KK[i] is the default setting of IsolateNames, which is the head of \
abbreviations used by Isolate. A KK[i] returned by Isolate is given in \
HoldForm and can be recovered by ReleaseHold[KK[i]].\
\>", "Text",
  CellTags->"KK"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  ", ",
  ButtonBox["IsolateNames",
    ButtonData:>"IsolateNames",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IsolateNames"],
  "."
}], "Text",
  CellTags->"KK"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Kummer", "ObjectName",
  CellTags->"Kummer"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Kummer"],

Cell["\<\
Kummer[i][exp] applies Kummer relation number i (i =1, ... 24, \
94,95,96) to all Hypergeometric2F1 in exp. i = 94 corresponds to eq. 9.131.2, \
i = 95 to eq. 9.132.1 and i = 96 to eq. 9.132.2 in Gradsteyn & Ryzhik.\
\>", \
"Text",
  CellTags->"Kummer"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricAC"],
  "."
}], "Text",
  CellTags->"Kummer"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Kummer"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "2", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[952]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox["z",
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[952]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "3", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[953]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox["z", 
                          RowBox[{"1", "-", "z"}]]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[953]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "4", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[954]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox["z", 
                          RowBox[{"1", "-", "z"}]]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[954]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "6", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[955]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[955]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "6", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[956]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"a", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"b", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[956]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "7", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[957]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[957]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "7", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[958]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"a", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[958]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "8", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[959]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[959]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "8", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[960]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"b", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[960]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "10", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[961]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[961]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "10", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[962]:=  (21:55:05 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "-", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "-", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[962]=  (21:55:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "11", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[963]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[963]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "11", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[964]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "-", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "-", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[964]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "12", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[965]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"b", "-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[965]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "12", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[966]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"a", "-", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"a", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"a", "-", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[966]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "14", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[967]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[967]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "14", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[968]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[968]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "15", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[969]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "b"], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[969]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "15", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[970]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "b"], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[970]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "16", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[971]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[971]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "16", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[972]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"b", "-", "c", "+", "1"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"b", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[972]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "18", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[973]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"2", "-", "c"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"2", "-", "c"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox["z",
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[973]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "18", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[974]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox["z",
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[974]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "19", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[975]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"2", "-", "c"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"a", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"2", "-", "c"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["z", 
                        RowBox[{"z", "-", "1"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[975]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "19", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[976]:=  (21:55:06 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["z", 
                        RowBox[{"z", "-", "1"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[976]=  (21:55:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "20", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[977]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"a", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"b", "-", "c", "+", "1"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{"2", "-", "c"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"b", "-", "c", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"2", "-", "c"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["z", 
                        RowBox[{"z", "-", "1"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[977]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "20", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[978]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["z", 
                        RowBox[{"z", "-", "1"}]],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[978]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "22", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[979]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"c", "-", "a"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"c", "-", "b"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", 
                            "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[979]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "22", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[980]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[980]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "23", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[981]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"c", "-", "a"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"c", "-", "b"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"a", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", 
                            "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", 
                        FractionBox["1", 
                          RowBox[{"1", "-", "z"}]]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[981]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "23", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[982]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"1", "-", 
                        FractionBox["1", 
                          RowBox[{"1", "-", "z"}]]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[982]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "24", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[983]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox[
                          RowBox[{"c", "-", "a"}],
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox[
                          RowBox[{"c", "-", "b"}],
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox[
                        RowBox[{
                          RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"b", "-", "c"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"c", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"1", "-", "b"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{
                            RowBox[{"-", "a"}], "-", "b", "+", "c", "+", 
                            "1"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[983]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "24", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
  CellLabel->"In[984]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"c", "-", "a"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox[
                          RowBox[{"1", "-", "z"}], "z"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[984]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "94", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[985]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "+", "b", "-", "c"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox[
                                RowBox[{"c", "-", "a"}],
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"c", "-", "b"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{
                                RowBox[{"-", "a"}], "-", "b", "+", "c", "+", 
                                "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          RowBox[{"1", "-", "z"}],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c"}], ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox["a",
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox["b",
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          RowBox[{"1", "-", "z"}],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[985]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "95", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[986]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"b", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox["a",
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"c", "-", "b"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{"a", "-", "b", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          FractionBox["1", 
                            RowBox[{"1", "-", "z"}]],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{"-", "a"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox["b",
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"c", "-", "a"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{
                                RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          FractionBox["1", 
                            RowBox[{"1", "-", "z"}]],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{"-", "b"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[986]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "96", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], 
        "]"}]}]], "Input",
  CellLabel->"In[987]:=  (21:55:07 on 5/16/01)",
  CellTags->"Kummer"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[InvisiblePrefixScriptBase]", 
                  FormBox["2",
                    "TraditionalForm"]], 
                SubscriptBox["F", 
                  FormBox["1",
                    "TraditionalForm"]]}], "\[InvisibleApplication]", 
              RowBox[{"(", 
                RowBox[{
                  TagBox[
                    TagBox[
                      RowBox[{
                        TagBox["a",
                          Hypergeometric2F1,
                          Editable->True], ",", 
                        TagBox["b",
                          Hypergeometric2F1,
                          Editable->True]}],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox[
                    TagBox[
                      TagBox["c",
                        Hypergeometric2F1,
                        Editable->True],
                      InterpretTemplate[ {
                        SlotSequence[ 1]}&]],
                    Hypergeometric2F1,
                    Editable->False], ";", 
                  TagBox["z",
                    Hypergeometric2F1,
                    Editable->True]}], ")"}]}],
            InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
            Editable->False],
          Hypergeometric2F1], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "a"], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"b", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox["a",
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"a", "-", "c", "+", "1"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{"a", "-", "b", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          FractionBox["1", "z"],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox["z", 
                RowBox[{"-", "a"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "b"], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                TagBox[
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[InvisiblePrefixScriptBase]", 
                        FormBox["2",
                          "TraditionalForm"]], 
                      SubscriptBox["F", 
                        FormBox["1",
                          "TraditionalForm"]]}], "\[InvisibleApplication]", 
                    RowBox[{"(", 
                      RowBox[{
                        TagBox[
                          TagBox[
                            RowBox[{
                              TagBox["b",
                                Hypergeometric2F1,
                                Editable->True], ",", 
                              TagBox[
                                RowBox[{"b", "-", "c", "+", "1"}],
                                Hypergeometric2F1,
                                Editable->True]}],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          TagBox[
                            TagBox[
                              RowBox[{
                                RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                              Hypergeometric2F1,
                              Editable->True],
                            InterpretTemplate[ {
                              SlotSequence[ 1]}&]],
                          Hypergeometric2F1,
                          Editable->False], ";", 
                        TagBox[
                          FractionBox["1", "z"],
                          Hypergeometric2F1,
                          Editable->True]}], ")"}]}],
                  InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
                  Editable->False],
                Hypergeometric2F1], " ", 
              SuperscriptBox["z", 
                RowBox[{"-", "b"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[987]=  (21:55:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Lagrangian", "ObjectName",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Lagrangian"],

Cell["\<\
Lagrangian[\"oqu\"] gives the unpolarized OPE quark operator. \
Lagrangian[\"oqp\"] gives the polarized quark OPE operator. \
Lagrangian[\"ogu\"] gives the unpolarized gluon OPE operator. \
Lagrangian[\"ogp\"] gives the polarized gluon OPE operator. \
Lagrangian[\"ogd\"] gives the sigma-term part of the QCD lagrangian. \
Lagrangian[\"QCD\"] gives the gluon self interaction part of the QCD \
lagrangian.\
\>", "Text",
  CellTags->"Lagrangian"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynRule"],
  "."
}], "Text",
  CellTags->"Lagrangian"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}]], "Input",
  CellLabel->"In[988]:=  (21:55:07 on 5/16/01)",
  CellTags->"Lagrangian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}], "a"], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}], "a"]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[988]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell["Twist-2 operator product expansion operators", "Text",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]], "Input",
  CellLabel->"In[989]:=  (21:55:07 on 5/16/01)",
  CellTags->"Lagrangian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[989]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]], "Input",
  CellLabel->"In[990]:=  (21:55:07 on 5/16/01)",
  CellTags->"Lagrangian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Gamma]"}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[990]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]], "Input",
  CellLabel->"In[991]:=  (21:55:07 on 5/16/01)",
  CellTags->"Lagrangian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[991]=  (21:55:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<oqp\>\"", "]"}]], "Input",
  CellLabel->"In[992]:=  (21:55:07 on 5/16/01)",
  CellTags->"Lagrangian"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[992]=  (21:55:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LC", "ObjectName",
  CellTags->"LC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LC"],

Cell["\<\
LC[m,n,r,s] evaluates to 4-dimensional LeviCivita[m,n,r,s] by \
virtue of applying FeynCalcInternal. LC[m,...][p, ...] evaluates to \
4-dimensional LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",\

  CellTags->"LC"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LCD"],
  "."
}], "Text",
  CellTags->"LC"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LC"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input",
  CellLabel->"In[993]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", "\[NoBreak]", 
          "\[Sigma]"}]], TraditionalForm]], "Output",
  CellLabel->"Out[993]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input",
  CellLabel->"In[994]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[994]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[995]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[995]//StandardForm=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LC", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[996]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "p", "\[NoBreak]", 
          "q"}]], TraditionalForm]], "Output",
  CellLabel->"Out[996]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[997]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[997]//StandardForm=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"LC", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p",
             "]"}], " ", 
          RowBox[{
            RowBox[{"LC", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q",
             "]"}]}], "]"}], " "}]], "Input",
  CellLabel->"In[998]:=  (21:55:08 on 5/16/01)",
  CellTags->"LC"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"18", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"24", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[998]=  (21:55:08 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LCD", "ObjectName",
  CellTags->"LCD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LCD"],

Cell["\<\
LCD[m,n,r,s] evaluates to D-dimensional LeviCivita[m,n,r,s] by \
virtue of FeynCalcInternal. LCD[m,...][p, ...] evaluates to D-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
  CellTags->"LCD"],

Cell["\<\
You need to do SetOptions[Eps, Dimension\[Rule]D] before LCD can be \
used as D-dimensional Levi-Civita input function.\
\>", "Text",
  CellTags->"LCD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LC"],
  "."
}], "Text",
  CellTags->"LCD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LCD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[999]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input",
  CellLabel->"In[1000]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", "\[NoBreak]", 
          "\[Sigma]"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1000]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input",
  CellLabel->"In[1001]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1001]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1002]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1002]//StandardForm=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LCD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[1003]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "p", "\[NoBreak]", 
          "q"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1003]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1004]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1004]//StandardForm=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p",
             "]"}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q",
             "]"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1005]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1005]=  (21:55:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Eps", ",", 
        RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}]], "Input",
  CellLabel->"In[1006]:=  (21:55:08 on 5/16/01)",
  CellTags->"LCD"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1006]=  (21:55:08 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LeftPartialD", "ObjectName",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LeftPartialD"],

Cell[TextData[{
  "LeftPartialD[\[Mu]] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  "acting to the left."
}], "Text",
  CellTags->"LeftPartialD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"LeftPartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[14]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[14]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[15]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[Null]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[15]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[16]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[16]//StandardForm="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"LeftPartialD", "[", "\[Mu]", "]"}], "]"}]], "Input",
  CellLabel->"In[17]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    RowBox[{"LeftPartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
  CellLabel->"Out[17]//StandardForm="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Rho]", "]"}]}]], "Input",
  CellLabel->"In[18]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox["A", "\[Nu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[18]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[19]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], ".", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[19]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[20]:=",
  CellTags->"LeftPartialD"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], 
          "]"}]}]}]], "Output",
  CellLabel->"Out[20]//StandardForm="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LeftRightPartialD", "ObjectName",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LeftRightPartialD"],

Cell[TextData[{
  "LeftRightPartialD[mu] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  ", acting to the left and right. ExplicitPartialD[LeftRightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] gives 1/2 (RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] - LeftPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "])."
}], "Text",
  CellTags->"LeftRightPartialD"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD2",
    ButtonData:>"LeftRightPartialD2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD2"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"LeftRightPartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[1014]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1014]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1015]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1015]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1016]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1016]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1017]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], "2"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1017]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftRightPartialD", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1018]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1018]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1019]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            SubscriptBox["A", "\[Rho]"]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1019]=  (21:55:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LeftRightPartialD2", "ObjectName",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LeftRightPartialD2"],

Cell[TextData[{
  "LeftRightPartialD2[mu] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  ", acting to the left and right. ExplicitPartialD[LeftRightPartialD2[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] gives (RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] + LeftPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "])."
}], "Text",
  CellTags->"LeftRightPartialD2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"LeftRightPartialD2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[1020]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1020]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1021]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1021]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1022]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1022]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1023]:=  (21:55:09 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["A", "\[Nu]"]}], "+", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1023]=  (21:55:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftRightPartialD2", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1024]:=  (21:55:10 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1024]=  (21:55:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1025]:=  (21:55:11 on 5/16/01)",
  CellTags->"LeftRightPartialD2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            SubscriptBox["A", "\[Rho]"]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1025]=  (21:55:11 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LeviCivita ", "ObjectName",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LeviCivita"],

Cell[TextData[{
  "LeviCivita[",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "\[Mu]", ",", " ", "\[Nu]", ",", " ", "\[Rho]", ",", " ", 
          "\[Sigma]"}], TraditionalForm]]],
  "] is an input function for the totally antisymmetric Levi-Civita tensor. \
It evaluates automatically to the internal representation Eps[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox[
        FormBox["\[Rho]",
          "TraditionalForm"], TraditionalForm]]],
  "], LorentzIndex[\[Sigma]]] (or with a second argument in LorentzIndex for \
the Dimension, if the option Dimension of LeviCivita is changed). \
LeviCivita[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", 
          RowBox[{"\[Nu]", " ", "..."}]}], TraditionalForm]]],
  "][ ",
  Cell[BoxData[
      FormBox[
        RowBox[{"p", ",", " ", ".."}], TraditionalForm]]],
  ".] evaluates to Eps[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "], ..., Momentum[",
  Cell[BoxData[
      FormBox["p", TraditionalForm]]],
  "], ...]."
}], "Text",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "LeviCivita", "]"}]], "Input",
  CellLabel->"In[1026]:=  (21:55:11 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1026]=  (21:55:11 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LC"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LCD"],
  "."
}], "Text",
  CellTags->"LeviCivita"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeviCivita", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "]"}]], "Input",
  CellLabel->"In[1027]:=  (21:55:11 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1027]=  (21:55:11 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"OPEDelta", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[1028]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1028]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", "]"}], "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
  CellLabel->"In[1029]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["s",
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1029]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeviCivita", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Beta]", ",", "\[Delta]"}], 
      "]"}]], "Input",
  CellLabel->"In[1030]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1030]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
        RowBox[{"OPEDelta", ",", "p"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1031]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1031]//StandardForm=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t1", " ", "=", " ", 
        RowBox[{
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}], ".", 
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}]}]}], " "}]], "Input",
  CellLabel->"In[1032]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]], ".", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1032]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1033]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output",
  CellLabel->"Out[1033]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1034]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", " ", 
        RowBox[{
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}], ".", 
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}]}], "]"}], "//", "Factor2"}]], "Input",
  CellLabel->"In[1035]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1035]=  (21:55:12 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";", 
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1036]:=  (21:55:12 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LC", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "]"}]], "Input",
  CellLabel->"In[1037]:=  (21:55:13 on 5/16/01)",
  CellTags->"LeviCivita"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Alpha]", "\[NoBreak]", "\[Beta]", "\[NoBreak]", "\[Gamma]", 
          "\[NoBreak]", "\[Delta]"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1037]=  (21:55:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1038]:=  (21:55:13 on 5/16/01)",
  CellTags->"LeviCivita"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LeviCivitaSign", "ObjectName",
  CellTags->"LeviCivitaSign"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LeviCivitaSign"],

Cell["\<\
LeviCivitaSign is an option for DiracTrace and EpsChisholm. It \
determines the sign in the result of a Dirac trace of four gamma matrices and \
gamma5.\
\>", "Text",
  CellTags->"LeviCivitaSign"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["EpsChisholm",
    ButtonData:>"EpsChisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"EpsChisholm"],
  "."
}], "Text",
  CellTags->"LeviCivitaSign"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ListIntegrals  ***unfinished***", "ObjectName",
  CellTags->"ListIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ListIntegrals"],

Cell["\<\
ListIntegrals[exp, {q1, q2}] gives a list of basic integrals  \
(FeynAmpDenominator's multiplied by scalar products involving q1, q2).  Any \
non-integer exponent (e.g. the (OPEm-1) in  \
ScalarProduct[OPEDelta,q1]^(OPEm-1)) is replaced by OPEm.  If the options \
Pair is set to True only integrals involving   scalar products (not counting \
those with OPEDelta) are given; with Pair \[Rule] False those are excluded.\
\
\>", "Text",
  CellTags->"ListIntegrals"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ListIntegrals"],

Cell[BoxData[""], "Input",
  CellLabel->"In[42]:=  (0:49:11 on 9/20/02)",
  CellTags->"ListIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Li2 ", "ObjectName",
  CellTags->"Li2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Li2"],

Cell["\<\
 Li2 is an abbreviation for the dilog function, i.e., Li2 = \
PolyLog[2, #]&.\
\>", "Text",
  CellTags->"Li2"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Li3",
    ButtonData:>"Li3",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Li3"],
  "."
}], "Text",
  CellTags->"Li2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Li2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li2", "[", "x", "]"}]], "Input",
  CellLabel->"In[1039]:=  (21:55:13 on 5/16/01)",
  CellTags->"Li2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "2"], "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1039]=  (21:55:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li2", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1040]:=  (21:55:13 on 5/16/01)",
  CellTags->"Li2"],

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", "#1"}], "]"}], "&"}]], "Output",
  CellLabel->"Out[1040]//StandardForm=  (21:55:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"\[Integral]", 
        RowBox[{
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}], "x"], 
          RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input",
  CellLabel->"In[1041]:=  (21:55:13 on 5/16/01)",
  CellTags->"Li2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "2"], "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1041]=  (21:55:14 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Li3 ", "ObjectName",
  CellTags->"Li3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Li3"],

Cell["\<\
 Li3 is an abbreviation for the trilog function, i.e., Li3 = \
PolyLog[3, #]&.\
\>", "Text",
  CellTags->"Li3"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Li2",
    ButtonData:>"Li2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Li2"],
  "."
}], "Text",
  CellTags->"Li3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Li3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li3", "[", "x", "]"}]], "Input",
  CellLabel->"In[1042]:=  (21:55:14 on 5/16/01)",
  CellTags->"Li3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "3"], "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1042]=  (21:55:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li3", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1043]:=  (21:55:14 on 5/16/01)",
  CellTags->"Li3"],

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", "#1"}], "]"}], "&"}]], "Output",
  CellLabel->"Out[1043]//StandardForm=  (21:55:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D", "[", 
      RowBox[{
        RowBox[{"Li3", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1044]:=  (21:55:14 on 5/16/01)",
  CellTags->"Li3"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SubscriptBox["Li", "2"], "(", "x", ")"}], "x"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1044]=  (21:55:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"\[Integral]", 
      RowBox[{
        FractionBox[
          RowBox[{"Li2", "[", "x", "]"}], "x"], 
        RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
  CellLabel->"In[1045]:=  (21:55:15 on 5/16/01)",
  CellTags->"Li3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "3"], "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1045]=  (21:55:15 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Loop", "ObjectName",
  CellTags->"Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Loop"],

Cell["Loop is an option indicating the number of (virtual) loops.", "Text",
  CellTags->"Loop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["LorentzIndex", "ObjectName",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"LorentzIndex"],

Cell["\<\
LorentzIndex[mu] denotes a four dimensional Lorentz index. For \
other than four dimensions: LorentzIndex[mu, D] or LorentzIndex[mu], etc. \
LorentzIndex[mu, 4] simplifies to LorentzIndex[mu].\
\>", "Text",
  CellTags->"LorentzIndex"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChangeDimension",
    ButtonData:>"ChangeDimension",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ChangeDimension"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Momentum"],
  "."
}], "Text",
  CellTags->"LorentzIndex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"LorentzIndex"],

Cell["This denotes a four-dimensional Lorentz index.", "Text",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}]], "Input",
  CellLabel->"In[1046]:=  (21:55:15 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox["\[Alpha]", TraditionalForm]], "Output",
  CellLabel->"Out[1046]=  (21:55:15 on 5/16/01)"]
}, Closed]],

Cell["\<\
An optional second argument can be given for a dimension different \
from 4.\
\>", "Text",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Alpha]", ",", "n"}], "]"}]], "Input",
  CellLabel->"In[1047]:=  (21:55:15 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox["\[Alpha]", TraditionalForm]], "Output",
  CellLabel->"Out[1047]=  (21:55:15 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell["LorentzIndex[ComplexIndex[\[Alpha]]]", "Input",
  CellLabel->"In[1048]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Alpha]", "*"], TraditionalForm]], "Output",
  CellLabel->"Out[1048]=  (21:55:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
  CellLabel->"In[1049]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Mu]", "*"], TraditionalForm]], "Output",
  CellLabel->"Out[1049]=  (21:55:16 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"LorentzIndex", "[", 
        RowBox[{"ComplexIndex", "[", "\[Beta]", "]"}], "]"}], "]"}]], "Input",\

  CellLabel->"In[1050]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox["\[Beta]", TraditionalForm]], "Output",
  CellLabel->"Out[1050]=  (21:55:16 on 5/16/01)"]
}, Closed]],

Cell["\<\
Setting $LorentzIndices=True displays the dimension (if different \
from 4) as a subindex.\
\>", "Text",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
  CellLabel->"In[1051]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "n"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "4"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[1052]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubscriptBox[
            FormBox["\[Alpha]",
              "TraditionalForm"], 
            FormBox["D",
              "TraditionalForm"]], ",", 
          SubscriptBox[
            FormBox["\[Nu]",
              "TraditionalForm"], 
            FormBox["n",
              "TraditionalForm"]], ",", "\[Beta]"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1052]=  (21:55:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", " ", "=", " ", "False"}], ";"}]], "Input",
  CellLabel->"In[1053]:=  (21:55:16 on 5/16/01)",
  CellTags->"LorentzIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Lower", "ObjectName",
  CellTags->"Lower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Lower"],

Cell["\<\
Lower may be used inside LorentzIndex to indicate an covariant \
LorentzIndex.\
\>", "Text",
  CellTags->"Lower"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Upper",
    ButtonData:>"Upper",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Upper"],
  ", ",
  ButtonBox["Contract1",
    ButtonData:>"Contract1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract1"],
  "."
}], "Text",
  CellTags->"Lower"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeContext", "ObjectName",
  CellTags->"MakeContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MakeContext"],

Cell["\<\
MakeContext[string] constructs the context path of string. \
MakeContext is invoked at startup of FeynCalc. MakeContext[a, b] construct \
the context path of b defined in the context of a.\
\>", "Text",
  CellTags->"MakeContext"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc",
    ButtonData:>"FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc"],
  ", ",
  ButtonBox["CheckContext",
    ButtonData:>"CheckContext",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CheckContext"],
  ", ",
  ButtonBox["$FeynCalcStuff",
    ButtonData:>"$FeynCalcStuff",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$FeynCalcStuff"],
  ", ",
  ButtonBox["Load",
    ButtonData:>"Load",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Load"],
  "."
}], "Text",
  CellTags->"MakeContext"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Mandelstam", "ObjectName",
  CellTags->"Mandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Mandelstam"],

Cell[TextData[{
  "Mandelstam is an option for DiracTrace, OneLoop, OneLoopSum, Tr and \
TrickMandelstam.  A typical setting is Mandelstam \[Rule] {",
  "s",
  ", ",
  "t",
  ", ",
  "u",
  ", ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
  "}, which stands for  ",
  "s",
  " + ",
  "t",
  " + ",
  "u",
  " = ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox[
            StyleBox["m",
              FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
  ". If other than four-particle processes are calculated the setting should \
be: Mandelstam \[Rule] {}."
}], "Text",
  CellTags->"Mandelstam"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  ", ",
  ButtonBox["TrickMandelstam",
    ButtonData:>"TrickMandelstam",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TrickMandelstam"],
  ", ",
  ButtonBox["SetMandelstam",
    ButtonData:>"SetMandelstam",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SetMandelstam"],
  "."
}], "Text",
  CellTags->"Mandelstam"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Map2", "ObjectName",
  CellTags->"Map2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Map2"],

Cell["\<\
Map2[f, exp] is equivalent to Map if Nterms[exp] > 0, otherwise \
Map2[f, exp] gives f[exp].\
\>", "Text",
  CellTags->"Map2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["NTerms",
    ButtonData:>"NTerms",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NTerms"],
  "."
}], "Text",
  CellTags->"Map2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Map2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", 
        RowBox[{"a", "-", "b"}]}], "]"}]], "Input",
  CellLabel->"In[1054]:=  (21:55:17 on 5/16/01)",
  CellTags->"Map2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", "(", "a", ")"}], "+", 
        RowBox[{"f", "(", 
          RowBox[{"-", "b"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1054]=  (21:55:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1055]:=  (21:55:17 on 5/16/01)",
  CellTags->"Map2"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1055]=  (21:55:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1056]:=  (21:55:17 on 5/16/01)",
  CellTags->"Map2"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}], ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1056]=  (21:55:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", "1"}], "]"}]], "Input",
  CellLabel->"In[1057]:=  (21:55:17 on 5/16/01)",
  CellTags->"Map2"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1057]=  (21:55:17 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MemoryAvailable", "ObjectName",
  CellTags->"MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MemoryAvailable"],

Cell["\<\
MemoryAvailable is an option of MemSet. It can be set to an integer \
n, where n is the available amount of main memory in Mega Byte. The default \
setting is $MemoryAvailable.\
\>", "Text",
  CellTags->"MemoryAvailable"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["MemSet",
    ButtonData:>"MemSet",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MemSet"],
  ", ",
  ButtonBox["$MemoryAvailable",
    ButtonData:>"$MemoryAvailable",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$MemoryAvailable"],
  "."
}], "Text",
  CellTags->"MemoryAvailable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MemSet", "ObjectName",
  CellTags->"MemSet"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MemSet"],

Cell[TextData[{
  "MemSet[f[x_], body] is like f[x_] := f[x] = body, but dependend on the \
value of the setting of MemoryAvailable \[Rule] memorycut (memorycut - \
MemoryInUse[]/",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["10", "6"], TraditionalForm]]],
  ") MemSet[f[x_], body] may evaluate as f[x_] := body."
}], "Text",
  CellTags->"MemSet"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["MemoryAvailable",
    ButtonData:>"MemoryAvailable",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MemoryAvailable"],
  "."
}], "Text",
  CellTags->"MemSet"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MetricTensor", "ObjectName",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MetricTensor"],

Cell[TextData[{
  "MetricTensor[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor. The default dimension is 4."
}], "Text",
  CellTags->"MetricTensor"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MT"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MTD"],
  "."
}], "Text",
  CellTags->"MetricTensor"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
  CellLabel->"In[1058]:=  (21:55:17 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1058]=  (21:55:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input",
  CellLabel->"In[1059]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output",
  CellLabel->"Out[1059]=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
        RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Input",
  CellLabel->"In[1060]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1060]=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input",
  CellLabel->"In[1061]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    FormBox["n", TraditionalForm]], "Output",
  CellLabel->"Out[1061]=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1062]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1062]//StandardForm=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1063]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"a", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1063]//StandardForm=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"MetricTensor", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1064]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1064]//StandardForm=  (21:55:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"MetricTensor", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], 
      "]"}]], "Input",
  CellLabel->"In[1065]:=  (21:55:18 on 5/16/01)",
  CellTags->"MetricTensor"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1065]//StandardForm=  (21:55:18 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MLimit", "ObjectName",
  CellTags->"MLimit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MLimit"],

Cell["\<\
MLimit[expr, lims] takes multiple limits of expr using the limits \
lims.\
\>", "Text",
  CellTags->"MLimit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MLimit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MLimit", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"y", " ", 
            RowBox[{"Log", "[", "y", "]"}]}], "+", 
          RowBox[{
            RowBox[{"Sin", "[", 
              RowBox[{"x", "-", "1"}], "]"}], "/", 
            RowBox[{"(", 
              RowBox[{"x", "-", "1"}], ")"}]}]}], ",", 
        RowBox[{"x", "\[Rule]", "1"}], ",", 
        RowBox[{"y", "\[Rule]", "0"}]}], "]"}]], "Input",
  CellLabel->"In[50]:=  (0:57:29 on 9/20/02)",
  CellTags->"MLimit"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[50]=  (0:57:29 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Momentum", "ObjectName",
  CellTags->"Momentum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Momentum"],

Cell["\<\
Momentum[p] is the head of a four momentum (p). The internal \
representation of a four-dimensional p is Momentum[p]. For other than four \
dimensions: Momentum[p, dim]. Momentum[p, 4] simplifies to Momentum[p].\
\>", \
"Text",
  CellTags->"Momentum"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumExpand"],
  "."
}], "Text",
  CellTags->"Momentum"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Momentum"],

Cell["This is a four dimensional momentum.", "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Momentum", "[", "p", "]"}]], "Input",
  CellLabel->"In[1081]:=  (21:55:20 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    FormBox["p", TraditionalForm]], "Output",
  CellLabel->"Out[1081]=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell["\<\
As an optional second argument the dimension must be specified if \
it is different from 4.\
\>", "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "D"}], "]"}]], "Input",
  CellLabel->"In[1082]:=  (21:55:20 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    FormBox["p", TraditionalForm]], "Output",
  CellLabel->"Out[1082]=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell["The dimension index is supressed in the output.", "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", 
        RowBox[{"p", ",", "d"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1083]:=  (21:55:20 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "d"}], "]"}]], "Output",
  CellLabel->"Out[1083]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{"Momentum", "[", 
        RowBox[{"-", "q"}], "]"}]}]], "Input",
  CellLabel->"In[1084]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "q"}], TraditionalForm]], "Output",
  CellLabel->"Out[1084]=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1085]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
  CellLabel->"Out[1085]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], " ", "+", " ", 
        RowBox[{"Momentum", "[", 
          RowBox[{"2", "q"}], "]"}]}]}]], "Input",
  CellLabel->"In[1086]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "+", 
        RowBox[{"2", " ", "q"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1086]=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1087]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "+", 
      RowBox[{"2", " ", 
        RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
  CellLabel->"Out[1087]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a2", "//", "MomentumExpand"}], "//", "StandardForm"}]], "Input",\

  CellLabel->"In[1088]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", "p", "]"}], "+", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
  CellLabel->"Out[1088]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a2", "//", "MomentumCombine"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1089]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", "+", "q"}], "]"}]], "Output",
  CellLabel->"Out[1089]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "d"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1090]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "d"}], "]"}]], "Output",
  CellLabel->"Out[1090]//StandardForm=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2"}], "]"}]], "Input",
  CellLabel->"In[1091]:=  (21:55:21 on 5/16/01)",
  CellTags->"Momentum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MomentumCombine", "ObjectName",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MomentumCombine"],

Cell["\<\
MomentumCombine[expr] is the inverse operation to MomentumExpand \
and ExpandScalarProduct. MomentumCombine combines also Pair`s.\
\>", "Text",
  CellTags->"MomentumCombine"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Momentum"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumExpand"],
  ", ",
  ButtonBox["MomentumCombine2",
    ButtonData:>"MomentumCombine2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumCombine2"],
  "."
}], "Text",
  CellTags->"MomentumCombine"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"Momentum", "[", "q", "]"}]}]}], " "}], "//", " ", 
        "MomentumCombine"}], " ", "//", " ", "StandardForm"}]], "Input",
  CellLabel->"In[1066]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", "-", 
        RowBox[{"2", " ", "q"}]}], "]"}]], "Output",
  CellLabel->"Out[1066]//StandardForm=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"FV", "[", 
            RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "+", " ", 
          RowBox[{"2", " ", 
            RowBox[{"FV", "[", 
              RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}], " "}]}]], "Input",
  CellLabel->"In[1067]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["p", "\[Mu]"], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1067]=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumCombine", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1068]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "+", 
              RowBox[{"2", " ", "q"}]}],
            "TraditionalForm"], ")"}], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1068]=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1069]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", 
            RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1069]//StandardForm=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
      "ExpandScalarProduct"}]], "Input",
  CellLabel->"In[1070]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1070]=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[1071]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "+", 
      RowBox[{"2", " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Output",
  CellLabel->"Out[1071]//StandardForm=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1072]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumCombine"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MomentumCombine2", "ObjectName",
  CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MomentumCombine2"],

Cell["\<\
MomentumCombine2[expr]  is the inverse operation to MomentumExpand \
and ExpandScalarProduct. MomentumCombine2 combines also FourVectors.\
\>", \
"Text",
  CellTags->"MomentumCombine2"],

Cell[TextData[{
  "See also: ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumCombine"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumExpand"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  "."
}], "Text",
  CellTags->"MomentumCombine2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"3", " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "+", 
      RowBox[{"2", " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Input",
  CellLabel->"In[28]:=  (17:04:55 on 9/3/02)",
  CellTags->"MomentumCombine2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"3", " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[28]=  (17:04:55 on 9/3/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumCombine2", "[", "%", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[29]:=  (17:04:57 on 9/3/02)",
  CellTags->"MomentumCombine2"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{
            RowBox[{"3", " ", "p"}], "+", 
            RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[29]//StandardForm=  (17:04:57 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MomentumExpand", "ObjectName",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MomentumExpand"],

Cell["\<\
MomentumExpand[expr] expands Momentum[a+b+ ...] in expr into \
Momentum[a] + Momentum[b] + ... .\
\>", "Text",
  CellTags->"MomentumExpand"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MomentumCombine"],
  "."
}], "Text",
  CellTags->"MomentumExpand"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumExpand", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", "q"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1073]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", "p", "]"}], "+", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
  CellLabel->"Out[1073]//StandardForm=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{
        RowBox[{"p", "+", "q"}], ",", "r"}], "]"}]], "Input",
  CellLabel->"In[1074]:=  (21:55:19 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", 
        FormBox["r",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1074]=  (21:55:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1075]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", "q"}], "]"}], ",", 
        RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1075]//StandardForm=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumExpand", "[", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "+", "q"}], ",", "r"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1076]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", 
        TagBox[
          FormBox[
            RowBox[{"p", "+", "q"}],
            "TraditionalForm"],
          TraditionalForm,
          Editable->True], ")", ".", "(", 
        TagBox[
          FormBox["r",
            "TraditionalForm"],
          TraditionalForm,
          Editable->True], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1076]=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1077]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], "+", 
          RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
        RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1077]//StandardForm=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumExpand", "[", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "+", "q"}], ",", 
          RowBox[{"r", "-", "p"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1078]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", 
        TagBox[
          FormBox[
            RowBox[{"p", "+", "q"}],
            "TraditionalForm"],
          TraditionalForm,
          Editable->True], ")", ".", "(", 
        TagBox[
          FormBox[
            RowBox[{"r", "-", "p"}],
            "TraditionalForm"],
          TraditionalForm,
          Editable->True], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1078]=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1079]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], "+", 
          RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "+", 
          RowBox[{"Momentum", "[", "r", "]"}]}]}], "]"}]], "Output",
  CellLabel->"Out[1079]//StandardForm=  (21:55:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input",
  CellLabel->"In[1080]:=  (21:55:20 on 5/16/01)",
  CellTags->"MomentumExpand"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1080]=  (21:55:20 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MT", "ObjectName",
  CellTags->"MT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MT"],

Cell[TextData[{
  "MT[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor in 4 dimensions."
}], "Text",
  CellTags->"MT"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MTD"],
  "."
}], "Text",
  CellTags->"MT"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
  CellLabel->"In[1092]:=  (21:55:21 on 5/16/01)",
  CellTags->"MT"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1092]=  (21:55:21 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1093]:=  (21:55:22 on 5/16/01)",
  CellTags->"MT"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output",
  CellLabel->"Out[1093]=  (21:55:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1094]:=  (21:55:22 on 5/16/01)",
  CellTags->"MT"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1094]//StandardForm=  (21:55:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"MT", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1095]:=  (21:55:22 on 5/16/01)",
  CellTags->"MT"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1095]//StandardForm=  (21:55:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"FCI", "[", 
          RowBox[{"MT", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1096]:=  (21:55:22 on 5/16/01)",
  CellTags->"MT"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1096]//StandardForm=  (21:55:22 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MTD", "ObjectName",
  CellTags->"MTD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"MTD"],

Cell[TextData[{
  "MTD[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor in D dimensions."
}], "Text",
  CellTags->"MTD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MT"],
  "."
}], "Text",
  CellTags->"MTD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"MTD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
  CellLabel->"In[1097]:=  (21:55:22 on 5/16/01)",
  CellTags->"MTD"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1097]=  (21:55:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1098]:=  (21:55:22 on 5/16/01)",
  CellTags->"MTD"],

Cell[BoxData[
    FormBox["D", TraditionalForm]], "Output",
  CellLabel->"Out[1098]=  (21:55:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1099]:=  (21:55:22 on 5/16/01)",
  CellTags->"MTD"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Output",
  CellLabel->"Out[1099]//StandardForm=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1100]:=  (21:55:23 on 5/16/01)",
  CellTags->"MTD"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1100]//StandardForm=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"FCI", "[", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1101]:=  (21:55:23 on 5/16/01)",
  CellTags->"MTD"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
  CellLabel->"Out[1101]//StandardForm=  (21:55:23 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NegativeInteger", "ObjectName",
  CellTags->"NegativeInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NegativeInteger"],

Cell["\<\
NegativeInteger is a data type. E.g. DataType[n, NegativeInteger] \
can be set to True.\
\>", "Text",
  CellTags->"NegativeInteger"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  "."
}], "Text",
  CellTags->"NegativeInteger"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Nf", "ObjectName",
  CellTags->"Nf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Nf"],

Cell["Nf denotes the number of flavors.", "Text",
  CellTags->"Nf"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Amplitude",
    ButtonData:>"Amplitude",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Amplitude"],
  ", ",
  ButtonBox["CounterTerm",
    ButtonData:>"CounterTerm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CounterTerm"],
  ", ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Convolute"],
  ", ",
  ButtonBox["AnomalousDimension",
    ButtonData:>"AnomalousDimension",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AnomalousDimension"],
  ", ",
  ButtonBox["SplittingFunction",
    ButtonData:>"SplittingFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SplittingFunction"],
  "."
}], "Text",
  CellTags->"Nf"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Nielsen", "ObjectName",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Nielsen"],

Cell["Nielsen[i,j, x] denotes Nielsen's polylogarithm.", "Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Nielsen", "]"}]], "Input",
  CellLabel->"In[1102]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PolyLog", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1102]=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text",
  CellTags->"Nielsen"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "2", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1103]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", 
          RowBox[{
            FormBox["1",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["2",
              "TraditionalForm"]}]], "(", "x", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1103]=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell["\<\
Numerical evaluation is done via N[Nielsen[n_,p_,x_]] := \
(-1)^(n+p-1)/(n-1)!/p! NIntegrate[Log[1-x t]^p Log[t]^(n-1)/t,{t,0,1}];\
\>", \
"Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", 
      RowBox[{"Nielsen", "[", 
        RowBox[{"1", ",", "2", ",", ".45"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1104]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox["0.07287162612104647`", TraditionalForm]], "Output",
  CellLabel->"Out[1104]=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell["Some special values are built in.", "Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", "0"}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"-", "1"}]}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"1", "/", "2"}]}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Input",
  CellLabel->"In[1105]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"0", ",", 
          FractionBox[
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "8"], ",", 
          FractionBox[
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "8"], ",", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1105]=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "2", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1106]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "+", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1106]=  (21:55:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "3", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1107]:=  (21:55:23 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "6"]}], " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "4"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1107]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"3", ",", "1", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1108]:=  (21:55:24 on 5/16/01)",
  CellTags->"Nielsen"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "4"], "(", "x", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1108]=  (21:55:24 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NonCommFreeQ", "ObjectName",
  CellTags->"NonCommFreeQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NonCommFreeQ"],

Cell["\<\
NonCommFreeQ[exp] yields True if exp contains no non-commutative \
objects (i.e. those objects which are listed in $NonComm) or only \
non-commutative objects inside DiracTrace's or SUNTrace's.\
\>", "Text",
  CellTags->"NonCommFreeQ"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["$NonComm",
    ButtonData:>"$NonComm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$NonComm"],
  ", ",
  ButtonBox["NonCommQ",
    ButtonData:>"NonCommQ",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NonCommQ"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["SUNTrace",
    ButtonData:>"SUNTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNTrace"],
  "."
}], "Text",
  CellTags->"NonCommFreeQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NonCommQ", "ObjectName",
  CellTags->"NonCommQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NonCommQ"],

Cell["\<\
NonCommQ[exp] yields True if exp contains non-commutative objects \
(i.e. those objects which are listed in $NonComm) not inside DiracTrace's or \
SUNTrace's.\
\>", "Text",
  CellTags->"NonCommQ"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["$NonComm",
    ButtonData:>"$NonComm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$NonComm"],
  ", ",
  ButtonBox["NonCommFreeQ",
    ButtonData:>"NonCommFreeQ",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NonCommFreeQ"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["SUNTrace",
    ButtonData:>"SUNTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNTrace"],
  "."
}], "Text",
  CellTags->"NonCommQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NonCommutative", "ObjectName",
  CellTags->"NonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NonCommutative"],

Cell["\<\
NonCommutative is a data type which may be used, e.g.,  as: \
DataType[x, NonCommutative] = True.\
\>", "Text",
  CellTags->"NonCommutative"],

Cell[TextData[{
  "See also:",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  "."
}], "Text",
  CellTags->"NonCommutative"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NTerms", "ObjectName",
  CellTags->"NTerms"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NTerms"],

Cell["\<\
NTerms[x] is equivalent to Length if x is a sum; otherwise \
NTerms[x] returns 1, except NTerms[0] \[Rule] 0.\
\>", "Text",
  CellTags->"NTerms"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"NTerms"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", 
      RowBox[{"a", "-", "b"}], "]"}]], "Input",
  CellLabel->"In[1109]:=  (21:55:24 on 5/16/01)",
  CellTags->"NTerms"],

Cell[BoxData[
    FormBox["2", TraditionalForm]], "Output",
  CellLabel->"Out[1109]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", 
      RowBox[{"a", " ", "b", " ", "c"}], "]"}]], "Input",
  CellLabel->"In[1110]:=  (21:55:24 on 5/16/01)",
  CellTags->"NTerms"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1110]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", "9", "]"}]], "Input",
  CellLabel->"In[1111]:=  (21:55:24 on 5/16/01)",
  CellTags->"NTerms"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1111]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", "0", "]"}]], "Input",
  CellLabel->"In[1112]:=  (21:55:24 on 5/16/01)",
  CellTags->"NTerms"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1112]=  (21:55:24 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NumberOfMetricTensors", "ObjectName",
  CellTags->"NumberOfMetricTensors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NumberOfMetricTensors"],

Cell["NumberOfMetricTensors is an option of Tdec.", "Text",
  CellTags->"NumberOfMetricTensors"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Tdec",
    ButtonData:>"Tdec",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tdec"],
  "."
}], "Text",
  CellTags->"NonCommQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NumericalFactor", "ObjectName",
  CellTags->"NumericalFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NumericalFactor"],

Cell["\<\
NumericalFactor[expr] gives the overall numerical factor of expr.\
\
\>", "Text",
  CellTags->"NumericalFactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NumericQ1", "ObjectName",
  CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"NumericQ1"],

Cell["\<\
NumericQ1[x,{a,b,..}] is like NumericQ, but assumes that {a,b,..} \
are numeric quantities.\
\>", "Text",
  CellTags->"NumericQ1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NumericQ", "[", 
      RowBox[{
        RowBox[{"3", " ", "a"}], "+", 
        RowBox[{"Log", "[", "b", "]"}], "+", 
        SuperscriptBox["c", "2"]}], "]"}]], "Input",
  CellLabel->"In[54]:=  (1:01:37 on 9/20/02)",
  CellTags->"NumericQ1"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[54]=  (1:01:37 on 9/20/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NumericQ1", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"3", " ", "a"}], "+", 
          RowBox[{"Log", "[", "b", "]"}], "+", 
          SuperscriptBox["c", "2"]}], ",", 
        RowBox[{"{", "}"}]}], "]"}]], "Input",
  CellLabel->"In[57]:=  (1:02:04 on 9/20/02)",
  CellTags->"NumericQ1"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[57]=  (1:02:04 on 9/20/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NumericQ1", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"3", " ", "a"}], "+", 
          RowBox[{"Log", "[", "b", "]"}], "+", 
          SuperscriptBox["c", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[58]:=  (1:02:10 on 9/20/02)",
  CellTags->"NumericQ1"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[58]=  (1:02:10 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OneLoop  ***unfinished***", "ObjectName",
  CellTags->"OneLoop"],

Cell[TextData[{
  "See also the old FeynCalc documentation at ",
  ButtonBox["http://www.mertig.com/oldfc/",
    ButtonData:>{
      URL[ "http://www.mertig.com/oldfc/"], None},
    ButtonStyle->"Hyperlink"],
  ". NOTICE: While OneLoop is restricted to 't Hooft Feynman gauge the \
function ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  " does not have this restriction (but is usually slower). OneLoop handles \
selfenergies, vertex and box-graphs (those only up to 3rd rank tensor in the \
integration variable)."
}], "Text",
  CellTags->"OneLoop"],

Cell[TextData[{
  "WARNING: If you encounter anomalies: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
          "The", " ", "default", " ", "setting", " ", "of", " ", "$West", " ",
             "is", " ", "True"}], ",", " ", 
          RowBox[{"i", ".", "e", "."}], ",", " ", 
          RowBox[{"the", " ", "way", " ", 
            RowBox[{"tr", "(", 
              RowBox[{
                SuperscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["\[Gamma]", "\[Nu]"], 
                SuperscriptBox["\[Gamma]", "\[Rho]"], 
                SuperscriptBox["\[Gamma]", "\[Sigma]"], 
                SuperscriptBox["\[Gamma]", "\[Tau]"], 
                SuperscriptBox["\[Gamma]", "\[Lambda]"], 
                SuperscriptBox["\[Gamma]", "5"]}], ")"}]}]}], 
        TraditionalForm]]],
  " is calculated in D dimensions has changed compared to the old FeynCalc \
version. Please keep in mind that the issue of ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "schemes is inherintly tricky."
}], "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OneLoop"],

Cell[TextData[{
  "OneLoop[",
  "q",
  ", amplitude] calculates the one-loop Feynman diagram amplitude (",
  StyleBox["n",
    FontSlant->"Italic"],
  "-point, where ",
  StyleBox["n",
    FontSlant->"Italic"],
  "\[LessEqual]4 and the highest tensor rank of the integration momenta \
(after cancellation of scalar products) may be 3; unless OneLoopSimplify is \
used)."
}], "Text",
  CellTags->"OneLoop"],

Cell["\<\
The argument q denotes the integration variable, i.e., the loop \
momentum. OneLoop[name, q, amplitude] has as first argument a name of the \
amplitude. If the second argument has head FeynAmp then OneLoop[q, \
FeynAmp[name, k, expr]] and OneLoop[FeynAmp[name, k, expr]] tranform to \
OneLoop[name, k, expr].\
\>", "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "OneLoop", "]"}]], "Input",
  CellLabel->"In[1113]:=  (21:55:24 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Apart2", "\[Rule]", "True"}], ",", 
          RowBox[{"CancelQP", "\[Rule]", "True"}], ",", 
          RowBox[{"DenominatorOrder", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
          RowBox[{"InitialSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IntermediateSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IsolateHead", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"OneLoopSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"Prefactor", "\[Rule]", "1"}], ",", 
          RowBox[{"ReduceGamma", "\[Rule]", "False"}], ",", 
          RowBox[{"ReduceToScalars", "\[Rule]", "False"}], ",", 
          RowBox[{"SmallVariables", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"WriteOut", "\[Rule]", "False"}], ",", 
          RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1113]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TIDL"],
  ", ",
  ButtonBox["$LimitTo4",
    ButtonData:>"$LimitTo4",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$LimitTo4"],
  "."
}], "Text",
  CellTags->"OneLoop"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"-", "I"}], "/", 
          RowBox[{"Pi", "^", "2"}]}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{"{", 
            RowBox[{"q", ",", "m"}], "}"}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1114]:=  (21:55:24 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["\[ImaginaryI]", 
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1114]=  (21:55:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q", ",", "%"}], "]"}]], "Input",
  CellLabel->"In[1115]:=  (21:55:24 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1115]=  (21:55:35 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"mf", "/:", 
        RowBox[{"MakeBoxes", "[", 
          RowBox[{"mf", ",", "TraditionalForm"}], "]"}], "=", 
        RowBox[{"InterpretationBox", "[", 
          RowBox[{
            RowBox[{"SubscriptBox", "[", 
              RowBox[{"\"\<m\>\"", ",", "\"\<f\>\""}], "]"}], ",", "mf"}], 
          "]"}]}], ";"}]], "Input",
  CellLabel->"In[1116]:=  (21:55:35 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[TextData[{
  "Remember that FAD[{q,mf},{q-k,mf}] is a fast possibility to enter ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["q", "2"], "-", 
                    SuperscriptBox["mf", "2"]}], ")"}], " ", 
                RowBox[{"(", " ", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
                    SuperscriptBox["mf", "2"]}], ")"}]}], " ", ")"}], "."}]}],
         TraditionalForm]]]
}], "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"I", " ", 
          FractionBox[
            SuperscriptBox["el", "2"], 
            RowBox[{"16", " ", 
              SuperscriptBox["Pi", "4"], 
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}]}]], 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "mf"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"q", "-", "k"}], ",", "mf"}], "}"}]}], "]"}], 
          RowBox[{"DiracTrace", "[", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"mf", "+", 
                  RowBox[{"GS", "[", 
                    RowBox[{"q", "-", "k"}], "]"}]}], ")"}], ".", 
              RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
              RowBox[{"(", 
                RowBox[{"mf", "+", 
                  RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
              RowBox[{"GA", "[", "\[Mu]", "]"}]}], "]"}]}], " ", "//", " ", 
        "FCI"}]}]], "Input",
  CellLabel->"In[1117]:=  (21:55:35 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["el", "2"], " ", 
          RowBox[{"tr", "(", 
            FormBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "+", 
                    RowBox[{
                      FormBox["\<\"\[Gamma]\"\>",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", "(", "\[NoBreak]", 
                      FormBox[
                        RowBox[{"q", "-", 
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"]}],
                        "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}], ".",
                 
                SuperscriptBox["\[Gamma]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "+", 
                    RowBox[{
                      FormBox["\<\"\[Gamma]\"\>",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], ")"}], ".", 
                SuperscriptBox["\[Gamma]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], ")"}]}], 
        RowBox[{"16", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "D"}], ")"}], " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["m", "f"],
                    mf], "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", 
                      FormBox[
                        TagBox[
                          StyleBox["\<\"k\"\>",
                            FontSlant->"Italic"],
                          (StyleForm[ #, FontSlant -> "Italic"]&)],
                        "TraditionalForm"]}], ")"}], "2"], "-", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["m", "f"],
                    mf], "2"]}], ")"}]}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1117]=  (21:55:35 on 5/16/01)"]
}, Closed]],

Cell["\<\
The input to OneLoop may be in 4 dimensions, since the function \
changes the dimension of the objects automatically to the setting of the \
Dimension option (D by default).\
\>", "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q", ",", "t"}], "]"}]], "Input",
  CellLabel->"In[1118]:=  (21:55:35 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"16", " ", 
              SuperscriptBox["\[Pi]", "2"]}]], 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["el", "2"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FractionBox["8", "3"], " ", 
                    RowBox[{
                      SubscriptBox["B", "0"], "(", "0", ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ")"}], " ", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"]}], "-", 
                  RowBox[{
                    FractionBox["8", "3"], " ", 
                    RowBox[{
                      SubscriptBox["B", "0"], "(", 
                      SuperscriptBox[
                        FormBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"],
                          "TraditionalForm"], "2"], ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ")"}], " ", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"]}], "-", 
                  RowBox[{
                    FractionBox["4", "3"], " ", 
                    RowBox[{
                      SubscriptBox["B", "0"], "(", 
                      SuperscriptBox[
                        FormBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"],
                          "TraditionalForm"], "2"], ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ",", 
                      SuperscriptBox[
                        InterpretationBox[
                          SubscriptBox["m", "f"],
                          mf], "2"], ")"}], " ", 
                    SuperscriptBox[
                      FormBox[
                        FormBox[
                          TagBox[
                            StyleBox["\<\"k\"\>",
                              FontSlant->"Italic"],
                            (StyleForm[ #, FontSlant -> "Italic"]&)],
                          "TraditionalForm"],
                        "TraditionalForm"], "2"]}], "+", 
                  FractionBox[
                    RowBox[{"4", " ", 
                      SuperscriptBox[
                        FormBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"],
                          "TraditionalForm"], "2"]}], "9"]}], ")"}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1118]=  (21:55:46 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
  CellLabel->"In[1119]:=  (21:55:46 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["el", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"6", " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "2"]}], "+", 
                SuperscriptBox[
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "2"], "-", 
                RowBox[{"3", " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", 
                    SuperscriptBox[
                      FormBox[
                        FormBox[
                          TagBox[
                            StyleBox["\<\"k\"\>",
                              FontSlant->"Italic"],
                            (StyleForm[ #, FontSlant -> "Italic"]&)],
                          "TraditionalForm"],
                        "TraditionalForm"], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"2", " ", 
                        SuperscriptBox[
                          InterpretationBox[
                            SubscriptBox["m", "f"],
                            mf], "2"]}], "+", 
                      SuperscriptBox[
                        FormBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"],
                          "TraditionalForm"], "2"]}], ")"}]}]}], ")"}]}], 
          RowBox[{"36", " ", 
            SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1119]=  (21:55:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"k", ",", "r"}], "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", 
      "FCI"}]], "Input",
  CellLabel->"In[1120]:=  (21:55:47 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1120]=  (21:55:47 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"k", ",", "%"}], "]"}]], "Input",
  CellLabel->"In[1121]:=  (21:55:47 on 5/16/01)",
  CellTags->"OneLoop"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", 
                    SuperscriptBox["m", "2"], ",", 
                    SuperscriptBox["m", "2"], ")"}], " ", 
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}], 
                RowBox[{"2", " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]]}], "+", 
            FractionBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]], "+", 
            RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], ",", "0", ",", 
                SuperscriptBox["m", "2"], ")"}], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1121]=  (21:55:56 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[1122]:=  (21:55:56 on 5/16/01)",
  CellTags->"OneLoop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OneLoopSimplify", "ObjectName",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OneLoopSimplify"],

Cell["\<\
OneLoopSimplify[amp, q] simplifies the one-loop amplitude amp. The \
second argument denotes the integration momentum.\
\>", "Text",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "OneLoopSimplify", "]"}]], "Input",
  CellLabel->"In[1123]:=  (21:55:56 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSimplify", "\[Rule]", "True"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IntegralTable", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"OPE1Loop", "\[Rule]", "False"}], ",", 
          RowBox[{"ScalarProductCancel", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1123]=  (21:55:56 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TIDL"],
  "."
}], "Text",
  CellTags->"OneLoopSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"k", ",", "r"}], "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", 
      "FCI"}]], "Input",
  CellLabel->"In[1124]:=  (21:55:56 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1124]=  (21:55:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input",
  CellLabel->"In[1125]:=  (21:55:56 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["m", "2"], "+", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}], ")"}], " ", 
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["r",
                "TraditionalForm"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", "p"}], ")"}], "2"]}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]], "-", 
        FractionBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["r",
              "TraditionalForm"]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1125]=  (21:55:56 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{
        RowBox[{"%", "/.", 
          RowBox[{"m", "\[Rule]", "0"}]}], ",", "k"}], "]"}]], "Input",
  CellLabel->"In[1126]:=  (21:55:56 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], 
        RowBox[{"2", " ", 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p"}], ")"}], "2"]}]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1126]=  (21:55:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", "k", ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "1"]}], ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "2"]}]}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1127]:=  (21:55:57 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1127]=  (21:55:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", " ", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input",
  CellLabel->"In[1128]:=  (21:55:57 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "1", "2"]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "1", "2"], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "2", "2"], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "1", "2"]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1128]=  (21:55:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FCE", "[", "%", "]"}], "/.", 
        RowBox[{
          RowBox[{"SPD", "[", 
            SubscriptBox["p", "1"], "]"}], "\[Rule]", "0"}]}], "//", 
      "FCI"}]], "Input",
  CellLabel->"In[1129]:=  (21:55:58 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubsuperscriptBox["p", 
              FormBox["1",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SubsuperscriptBox["p", "2", "2"]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}]], "-", 
        FractionBox[
          SubsuperscriptBox["p", 
            FormBox["1",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubsuperscriptBox["p", 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1129]=  (21:55:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"k", "-", 
                SubscriptBox["p", "1"]}], ",", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "2"]}]}], "]"}], " ", 
          RowBox[{
            RowBox[{"SP", "[", 
              RowBox[{"k", ",", "l"}], "]"}], "^", "2"}]}], ",", "k"}], 
      "]"}]], "Input",
  CellLabel->"In[1130]:=  (21:55:58 on 5/16/01)",
  CellTags->"OneLoopSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"D", " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["l",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{"2", " ", "D", " ", 
                RowBox[{
                  FormBox["l",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["l",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}]}], "-", 
              RowBox[{"4", " ", 
                RowBox[{
                  FormBox["l",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["l",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{"D", " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["l",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["l",
                    "TraditionalForm"], "2"], " ", 
                SubsuperscriptBox["p", "1", "2"]}], "+", 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  FormBox["l",
                    "TraditionalForm"], "2"], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["l",
                    "TraditionalForm"], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"4", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], " ", 
              RowBox[{
                SuperscriptBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "2"], ".", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox[
                        TagBox[
                          StyleBox["\<\"k\"\>",
                            FontSlant->"Italic"],
                          (StyleForm[ #, FontSlant -> "Italic"]&)],
                        "TraditionalForm"], "-", 
                      SubscriptBox["p", "1"], "+", 
                      SubscriptBox["p", "2"]}], ")"}], "2"]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1130]=  (21:56:00 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OneLoopSum", "ObjectName",
  CellTags->"OneLoopSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OneLoopSum"],

Cell["\<\
OneLoopSum[ FeynAmp[ ... ], FeynAmp[ ... ] , ...] will calculate a \
list of Feynman amplitudes by replacing FeynAmp step by step by \
OneLoop.\
\>", "Text",
  CellTags->"OneLoopSum"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"OneLoopSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPE", "ObjectName",
  CellTags->"OPE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPE"],

Cell["OPE is used internally in OPE1Loop.", "Text",
  CellTags->"OPE"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPE1Loop",
    ButtonData:>"OPE1Loop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPE1Loop"],
  "."
}], "Text",
  CellTags->"OPE"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEDelta", "ObjectName",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEDelta"],

Cell["\<\
OPEDelta is a lightlike axial vector as used e.g. in the operator \
product expansion in QCD.\
\>", "Text",
  CellTags->"OPEDelta"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text",
  CellTags->"OPEDelta"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEDelta"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[1131]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input",
  CellLabel->"In[1132]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEDelta"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1132]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"OPEDelta", ",", "OPEDelta"}], "]"}]], "Input",
  CellLabel->"In[1133]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEDelta"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1133]=  (21:56:00 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo", "ObjectName",
  CellTags->"OPEi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEi"],

Cell["\<\
OPEi, etc. are variables with DataType PositiveInteger which are \
used in functions relating to the operator product expansion.\
\>", "Text",
  CellTags->"OPEi"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData["OPEi"], "Input",
  CellLabel->"In[1134]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox["i", TraditionalForm]], "Output",
  CellLabel->"Out[1134]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{
      "OPEi", ",", " ", "OPEj", ",", "OPEk", ",", "OPEl", ",", " ", "OPEm", 
        ",", " ", "OPEn", ",", " ", "OPEo", ",", " ", "PositiveInteger"}], 
      "]"}]], "Input",
  CellLabel->"In[1135]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", 
          "True", ",", "True"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1135]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEi"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEj"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEk"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEl"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEm"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEn"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEo"}], ")"}]}]}], "}"}], "]"}]], "Input",
  CellLabel->"In[1136]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1136]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell["Re has been changed:", "Text",
  CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "3"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "2"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "1"}]}], ",", " ", " ", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", "1"}]}], "}"}]], "Input",
  CellLabel->"In[1137]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1137]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">",
           " ", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">",
           " ", "1"}], ",", 
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">",
           " ", "2"}]}], "}"}]], "Input",
  CellLabel->"In[1138]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1138]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "3"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "2"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "1"}]}], ",", " ", " ", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", "1"}]}], "}"}]], "Input",
  CellLabel->"In[1139]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEi"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1139]=  (21:56:00 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEInsert  ***unfinished***", "ObjectName",
  CellTags->"OPEInsert"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEInsert"],

Cell["\<\
OPEInsert[diagram_String, name_, rhifile_String] or \
OPEInsert[diagram_String, name_]. The setting of the option EpsContract (4 or \
D) determines whether the Levi-Civita tensors are contracted in 4 or D \
dimensions.\
\>", "Text",
  CellTags->"OPEInsert"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"OPEInsert"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEInsert"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEInsert"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEInt  ***unfinished***", "ObjectName",
  CellTags->"OPEInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEInt"],

Cell["\<\
OPEInt[expr, q, p, x].  The dimension is changed to the one \
indicated by the option  Dimension. The setting of the option EpsContract \
determines  the dimension in which the Levi-Civita tensors are \
contracted.\
\>", "Text",
  CellTags->"OPEInt"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"OPEInt"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEInt"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEInt"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEIntDelta  ***unfinished***", "ObjectName",
  CellTags->"OPEIntDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEIntDelta"],

Cell[TextData[{
  "OPEIntDelta[expr, x, m] introduces the delta(1-x) (DeltaFunction[1-x]). \
The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntDelta by (4+Epsilon)."
}], "Text",
  CellTags->"OPEIntDelta"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"OPEIntDelta"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEIntDelta"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEIntDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEIntegrate  ***unfinished***", "ObjectName",
  CellTags->"OPEIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEIntegrate"],

Cell["\<\
OPEIntegrate[expr, q, x].  The dimension is changed to the one \
indicated by the option  Dimension. The setting of the option EpsContract \
determines  the dimension in which the Levi-Civita tensors are \
contracted.\
\>", "Text",
  CellTags->"OPEIntegrate"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"OPEIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEIntegrate"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEIntegrateDelta  ***unfinished***", "ObjectName",
  CellTags->"OPEIntegrateDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
  "OPEIntegrateDelta[expr, x, m] introduces the \[Delta](1-x) \
(DeltaFunction[1-x]). The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntegrateDelta by (4+Epsilon)."
}], "Text",
  CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"OPEIntegrateDelta"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEIntegrateDelta"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEIntegrateDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPEIntegrate2  ***unfinished***", "ObjectName",
  CellTags->"OPEIntegrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPEIntegrate2"],

Cell["\<\
OPEIntegrate2[exp, k] does special loop (tensorial) integrations.  \
Only the residue is calculated.\
\>", "Text",
  CellTags->"OPEIntegrate2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEIntegrate",
    ButtonData:>"OPEIntegrate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEIntegrate"],
  "."
}], "Text",
  CellTags->"OPEIntegrate2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPEIntegrate2"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1131]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPEIntegrate2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPESumExplicit", "ObjectName",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPESumExplicit"],

Cell["OPESumExplicit[exp] calculates OPESum's.", "Text",
  CellTags->"OPESumExplicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESum"],
  ", ",
  ButtonBox["OPESumSimplify",
    ButtonData:>"OPESumSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESumSimplify"],
  "."
}], "Text",
  CellTags->"OPESumExplicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"A", "^", "i"}], 
            RowBox[{"B", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "i", "-", "3"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
              RowBox[{"m", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[1140]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPESumExplicit"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], 
            RowBox[{"m", "-", "3"}]], 
          RowBox[{
            SuperscriptBox["A", "i"], " ", 
            SuperscriptBox["B", 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[1140]=  (21:56:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1141]:=  (21:56:00 on 5/16/01)",
  CellTags->"OPESumExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          FormBox[
            SuperscriptBox["A", 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{"A", "-", "B"}]], "-", 
        FractionBox[
          FormBox[
            SuperscriptBox["B", 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{"A", "-", "B"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1141]=  (21:56:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", "^", "i"}], 
            RowBox[{"b", "^", 
              RowBox[{"(", 
                RowBox[{"j", "-", "i"}], ")"}]}], 
            RowBox[{"c", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
              RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[1142]:=  (21:56:01 on 5/16/01)",
  CellTags->"OPESumExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"j", "=", "0"}], 
                RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", "i"], " ", 
            SuperscriptBox["b", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["c", 
              RowBox[{
                RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1142]=  (21:56:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
  CellLabel->"In[1143]:=  (21:56:01 on 5/16/01)",
  CellTags->"OPESumExplicit"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"c", " ", 
              FormBox[
                SuperscriptBox["a", 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"a", "-", "c"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["a", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"c", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1143]=  (21:56:01 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
  CellLabel->"In[1144]:=  (21:56:01 on 5/16/01)",
  CellTags->"OPESumExplicit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPESum", "ObjectName",
  CellTags->"OPESum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPESum"],

Cell["\<\
OPESum[exp, {i, 0, m}] denotes a symbolic sum.The syntax is the \
same as for Sum.\
\>", "Text",
  CellTags->"OPESum"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESumExplicit",
    ButtonData:>"OPESumExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESumExplicit"],
  ", ",
  ButtonBox["OPESumSimplify",
    ButtonData:>"OPESumSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESumSimplify"],
  "."
}], "Text",
  CellTags->"OPESum"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPESum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"SO", "[", "p", "]"}], "^", "OPEi"}], 
            RowBox[{
              RowBox[{"SO", "[", "k", "]"}], "^", 
              RowBox[{"(", 
                RowBox[{"OPEm", "-", "OPEi", "-", "3"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", 
              RowBox[{"OPEm", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[1145]:=  (21:56:01 on 5/16/01)",
  CellTags->"OPESum"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], 
            RowBox[{"m", "-", "3"}]], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"]}], ")"}], 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], "i"]}]}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[1145]=  (21:56:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1146]:=  (21:56:01 on 5/16/01)",
  CellTags->"OPESum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["p",
                "TraditionalForm"]}]}]], "-", 
        FractionBox[
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["p",
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1146]=  (21:56:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", "^", "i"}], 
            RowBox[{"b", "^", 
              RowBox[{"(", 
                RowBox[{"j", "-", "i"}], ")"}]}], 
            RowBox[{"c", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
              RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[1147]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"j", "=", "0"}], 
                RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", "i"], " ", 
            SuperscriptBox["b", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["c", 
              RowBox[{
                RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1147]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
  CellLabel->"In[1148]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"c", " ", 
              FormBox[
                SuperscriptBox["a", 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"a", "-", "c"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["a", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"c", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1148]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}]}]], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPESumSimplify", "ObjectName",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPESumSimplify"],

Cell["OPESumSimplify[exp] simplifies OPESum's in exp.", "Text",
  CellTags->"OPESumSimplify"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESum"],
  ", ",
  ButtonBox["OPESumExplicit",
    ButtonData:>"OPESumExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESumExplicit"],
  "."
}], "Text",
  CellTags->"OPESumSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["\<\
OPESum[(-SOD[p])^(OPEi+1) \
SOD[p-q]^(OPEm-OPEi-2),{OPEi,0,OPEm}]\
\>", "Input",
  CellLabel->"In[1150]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESumSimplify"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], "m"], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", 
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\<\"\[CapitalDelta]\"\>",
                        "TraditionalForm"], 
                      FormBox["\<\"\[CenterDot]\"\>",
                        "TraditionalForm"], 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}]}], ")"}], 
              RowBox[{"i", "+", "1"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"(\"\>",
                    "TraditionalForm"], 
                  FormBox[
                    RowBox[{"p", "-", "q"}],
                    "TraditionalForm"], 
                  FormBox["\<\")\"\>",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[1150]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell["OPESumSimplify[%]", "Input",
  CellLabel->"In[1151]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESumSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], "m"], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", "(", "\[NoBreak]", 
                        FormBox[
                          RowBox[{"p", "-", "q"}],
                          "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
              HoldForm],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1151]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumSimplify", "[", 
      RowBox[{
        RowBox[{"OPESum", "[", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", "OPEm"}], "}"}], "]"}], " ", 
        RowBox[{"a", "^", "OPEi"}]}], "]"}]], "Input",
  CellLabel->"In[1152]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESumSimplify"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], "m"], 
          FormBox[
            SuperscriptBox["a", "i"],
            "TraditionalForm"]}],
        HoldForm], TraditionalForm]], "Output",
  CellLabel->"Out[1152]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumSimplify", "[", 
      RowBox[{
        RowBox[{"OPESum", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"j", ",", "0", ",", "i"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"i", ",", "0", ",", "m"}], "}"}]}], "]"}], " ", 
        RowBox[{"a", "^", 
          RowBox[{"(", 
            RowBox[{"j", "-", "i"}], ")"}]}], " ", 
        RowBox[{"b", "^", "i"}]}], "]"}]], "Input",
  CellLabel->"In[1153]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESumSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"i", "=", "0"}], "m"], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["b", "i"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1153]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1154]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPESumSimplify"],

Cell[BoxData[
    RowBox[{"OPESum", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["a", 
            RowBox[{
              RowBox[{"-", "i"}], "+", "j"}]], " ", 
          SuperscriptBox["b", "i"]}], ",", 
        RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"j", ",", "0", ",", "i"}], "}"}]}], "]"}]], "Output",
  CellLabel->"Out[1154]//StandardForm=  (21:56:02 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPE1Loop  ***unfinished***", "ObjectName",
  CellTags->"OPE1Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPE1Loop"],

Cell["\<\
OPE1Loop[q1, amp].  OPE1Loop[{q1,q2}, amp] does sub-loop  \
decomposition.\
\>", "Text",
  CellTags->"OPE1Loop"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESum"],
  "."
}], "Text",
  CellTags->"OPE1Loop"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPE1Loop"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPE1Loop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPE2AI  ***unfinished***", "ObjectName",
  CellTags->"OPE2AI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPE2AI"],

Cell["OPE2AI[tabname, listtodo, q1,q2,p] is ....", "Text",
  CellTags->"OPE2AI"],

Cell[BoxData[
    RowBox[{"Options", "[", "OPE2AI", "]"}]], "Input",
  CellLabel->"In[15]:=  (14:15:04 on 9/20/02)",
  CellTags->"OPE2AI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESum"],
  "."
}], "Text",
  CellTags->"OPE2AI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPE2AI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPE2AI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OPE2TID  ***unfinished***", "ObjectName",
  CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OPE2TID"],

Cell["\<\
OPE2TID[exp, k1, k2, p]. The setting of the option EpsContract \
determines the dimension in which the Levi-Civita tensors are \
contracted.\
\>", "Text",
  CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "OPE2TID", "]"}]], "Input",
  CellLabel->"In[14]:=  (14:14:45 on 9/20/02)",
  CellTags->"OPE2TID"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Uncontract", "\[Rule]", "False"}], ",", 
          RowBox[{"Contract", "\[Rule]", "True"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"EpsContract", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[14]=  (14:14:45 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPESum"],
  "."
}], "Text",
  CellTags->"OPE2TID"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"OPE2TID"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"OPE2TID"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["OptionsSelect", "ObjectName",
  CellTags->"OptionsSelect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"OptionsSelect"],

Cell["\<\
OptionsSelect[function,opts] returns the option settings of opts \
accepted by function.  When an option occurs several times in opts, the first \
setting is selected.\
\>", "Text",
  CellTags->"OptionsSelect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pair", "ObjectName",
  CellTags->"Pair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Pair"],

Cell[TextData[{
  "Pair[x, y] is the head of a special pairing used in the internal \
representation: x and y may have heads LorentzIndex or Momentum. If both x \
and y have head LorentzIndex, the metric tensor is understood. If x and y \
have head Momentum, a scalar product is meant. If one of x and y has head \
LorentzIndex and the other Momentum, a Lorentz vector ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
  " is understood."
}], "Text",
  CellTags->"Pair"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MT"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"MTD"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SPD"],
  "."
}], "Text",
  CellTags->"Pair"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Pair"],

Cell["This represents a four-dimensional metric tensor.", "Text",
  CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1155]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1155]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell["This is a D-dimensional metric tensor.", "Text",
  CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1156]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1156]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1157]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Alpha]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1157]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "q", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1158]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1158]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1159]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1159]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1160]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1160]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "2"}]], "Input",
  CellLabel->"In[1161]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "4"], TraditionalForm]], "Output",
  CellLabel->"Out[1161]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "3"}]], "Input",
  CellLabel->"In[1162]:=  (21:56:02 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "6"], TraditionalForm]], "Output",
  CellLabel->"Out[1162]=  (21:56:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", 
              RowBox[{"n", "-", "4"}]}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], "//", 
      "Contract"}]], "Input",
  CellLabel->"In[1163]:=  (21:56:03 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1163]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"p", "-", "q"}], "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1164]:=  (21:56:03 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1164]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"-", "q"}], "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", "+", " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "q", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1165]:=  (21:56:03 on 5/16/01)",
  CellTags->"Pair"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1165]=  (21:56:03 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PairCollect", "ObjectName",
  CellTags->"PairCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PairCollect"],

Cell["\<\
PairCollect is an option for DiracTrace specifying if the result is \
collected with respect to Pair's.\
\>", "Text",
  CellTags->"PairCollect"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  "."
}], "Text",
  CellTags->"PairCollect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PairContract", "ObjectName",
  CellTags->"PairContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PairContract"],

Cell["\<\
PairContract is like Pair, but with (local) contraction properties.\
\
\>", "Text",
  CellTags->"PairContract"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  "."
}], "Text",
  CellTags->"PairContract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PairContract2", "ObjectName",
  CellTags->"PairContract2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PairContract2"],

Cell["\<\
PairContract2 is like Pair, but with local contraction properties \
among PairContract2's.\
\>", "Text",
  CellTags->"PairContract2"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["PairContract",
    ButtonData:>"PairContract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PairContract"],
  "."
}], "Text",
  CellTags->"PairContract2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PairContract3", "ObjectName",
  CellTags->"PairContract3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PairContract3"],

Cell["\<\
PairContract3 is like Pair, but with local contraction properties \
among PairContract3's.\
\>", "Text",
  CellTags->"PairContract3"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["PairContract",
    ButtonData:>"PairContract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PairContract"],
  "."
}], "Text",
  CellTags->"PairContract3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PartialD", "ObjectName",
  CellTags->"PartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PartialD"],

Cell[TextData[{
  "PartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes the four-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], "."}], TraditionalForm]]],
  " PartialD is used to denote derivative fields. PartialD[LorentzIndex[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", 
          StyleBox["D",
            FontSlant->"Plain"]}], TraditionalForm]]],
  "]] denotes the ",
  Cell[BoxData[
      FormBox[
        StyleBox["D",
          FontSlant->"Plain"], TraditionalForm]]],
  "-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], "."}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->True,
  CellTags->"PartialD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"PartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"PartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[1166]:=  (21:56:03 on 5/16/01)",
  CellTags->"PartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1166]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1167]:=  (21:56:03 on 5/16/01)",
  CellTags->"PartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1167]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[1168]:=  (21:56:03 on 5/16/01)",
  CellTags->"PartialD"],

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftPartialD", "[", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], " ", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[1168]//StandardForm=  (21:56:03 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PartialDRelations", "ObjectName",
  CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PartialDRelations"],

Cell["\<\
PartialDRelations is an option for ExpandPartialD. It is a list of \
rules applied by ExpandPartialD at the end.\
\>", "Text",
  ZeroWidthTimes->True,
  CellTags->"PartialDRelations"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text",
  CellTags->"PartialDRelations"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"B", ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[17]:=",
  CellTags->"PartialDRelations"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox["B", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[17]="]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", 
      RowBox[{"%", ",", " ", 
        RowBox[{"PartialDRelations", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"A", "\[Rule]", "C"}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[18]:=",
  CellTags->"PartialDRelations"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            FormBox["C",
              "TraditionalForm"], "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["B", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["C",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], ".", 
          SubscriptBox["B", "\[Mu]"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[18]="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PartialFourVector", "ObjectName",
  CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PartialFourVector"],

Cell["\<\
PartialFourVector[exp, FourVector[p, mu]]  calculates the partial \
derivative of exp w.r.t. p(mu). PartialFourVector[exp, FourVector[p, mu], \
FourVector[p,nu], ...]  gives the multiple derivative.\
\>", "Text",
  ZeroWidthTimes->True,
  CellTags->"PartialFourVector"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  "."
}], "Text",
  CellTags->"PartialFourVector"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PartialFourVector", "[", 
      RowBox[{
        RowBox[{"a", " ", 
          SuperscriptBox["\[ExponentialE]", 
            SuperscriptBox[
              RowBox[{"FourVector", "[", 
                RowBox[{"p", ",", "\[Nu]"}], "]"}], "2"]]}], ",", 
        RowBox[{"FourVector", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[32]:=  (14:35:59 on 9/20/02)",
  CellTags->"PartialFourVector"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "a", " ", 
        SuperscriptBox["\[ExponentialE]", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (14:35:59 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PartialIntegrate", "ObjectName",
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PartialIntegrate"],

Cell["\<\
 PartialIntegrate[exp, ap, t] does a partial integration of the \
definite integral Integrate[exp,{t,0,1}], with ap the factor that is to be \
integrated and exp/ap the factor that is to be differentiated.\
\>", "Text",
  ZeroWidthTimes->True,
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "PartialIntegrate", "]"}]], "Input",
  CellLabel->"In[10]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Integrate", "\[Rule]", "Integrate"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[10]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PartialIntegrate", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "x", "]"}], 
          RowBox[{"g", "[", "x", "]"}]}], ",", 
        RowBox[{"g", "[", "x", "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[11]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", "-", "b", "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"3", " ", "a"}], "+", "2"}], ")"}], " ", "a"}], "-",
             "a", "+", 
            RowBox[{
              FractionBox["2", "3"], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"3", " ", "a"}], "+", "2"}], ")"}]}]}], 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"3", " ", "a"}], "+", "2"}], ")"}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"3", " ", "b"}], "+", "2"}], ")"}], " ", "b"}], "-",
             "b", "+", 
            RowBox[{
              FractionBox["2", "3"], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"3", " ", "b"}], "+", "2"}], ")"}]}]}], 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"3", " ", "b"}], "+", "2"}], ")"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[11]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PartialIntegrate", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "x", "]"}], 
          RowBox[{"g", "[", "x", "]"}]}], ",", 
        RowBox[{"g", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[12]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["5", "3"]}], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"-", "1"}], "+", 
            FractionBox[
              RowBox[{"5", " ", 
                RowBox[{"log", "(", "5", ")"}]}], "3"]}], 
          RowBox[{"log", "(", "5", ")"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[12]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"f", "[", "x_", "]"}], "=", 
      RowBox[{"Integrate", "[", 
        RowBox[{
          RowBox[{"Log", "[", 
            RowBox[{
              RowBox[{"3", "x"}], "+", "2"}], "]"}], ",", "x"}], 
        "]"}]}]], "Input",
  CellLabel->"In[13]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", "x"}], "-", 
        "x", "+", 
        RowBox[{
          FractionBox["2", "3"], " ", 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[13]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"g", "[", "x_", "]"}], "=", 
      RowBox[{"D", "[", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"Log", "[", 
              RowBox[{
                RowBox[{"3", "x"}], "+", "2"}], "]"}]], ",", "x"}], 
        "]"}]}]], "Input",
  CellLabel->"In[14]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["3", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{
                RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[14]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PartialIntegrate", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"f", "[", "x", "]"}], 
            RowBox[{"g", "[", "x", "]"}]}], ",", 
          RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}], "//", 
      "Simplify"}]], "Input",
  CellLabel->"In[15]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["1", 
          RowBox[{"log", "(", "5", ")"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[15]=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Integrate", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"f", "[", "x", "]"}], 
            RowBox[{"g", "[", "x", "]"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
      "Simplify"}]], "Input",
  CellLabel->"In[16]:=  (16:42:05 on 9/20/02)",
  CellTags->"PartialIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["1", 
          RowBox[{"log", "(", "5", ")"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[16]=  (16:42:06 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"f", ",", "g"}], "]"}]], "Input",
  CellLabel->"In[17]:=  (16:42:06 on 9/20/02)",
  CellTags->"PartialIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PartitHead", "ObjectName",
  CellTags->"PartitHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PartitHead"],

Cell["\<\
PartitHead[expr, h] returns a list {ex1, h[ex2]} with ex1 free of \
expressions with head h, and h[ex2] having head h.\
\>", "Text",
  CellTags->"PartitHead"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PauliSigma", "ObjectName",
  CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PauliSigma"],

Cell["\<\
PauliSigma denotes the vector of the 3 Pauli matrices. \
PauliSigma[1], PauliSigma[2], PauliSigma[3] give the explicit Pauli matrices. \
PauliSigma[] yields {PauliSigma[1], PauliSigma[2], PauliSigma[3]}.\
\>", \
"Text",
  CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PauliSigma", "[", "]"}], "//", "TableForm"}]], "Input",
  CellLabel->"In[50]:=  (18:29:08 on 9/3/02)",
  CellTags->"PauliSigma"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {GridBox[{
                  {"0"},
                  {"1"}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}], GridBox[{
                  {"1"},
                  {"0"}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}]},
            {GridBox[{
                  {"0"},
                  {
                    RowBox[{"-", "\[ImaginaryI]"}]}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}], GridBox[{
                  {"\[ImaginaryI]"},
                  {"0"}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}]},
            {GridBox[{
                  {"1"},
                  {"0"}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}], GridBox[{
                  {"0"},
                  {
                    RowBox[{"-", "1"}]}
                  },
                RowSpacings->0.25,
                ColumnSpacings->1,
                RowAlignments->Baseline,
                ColumnAlignments->{Left}]}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {{{0, 1}, {1, 0}}, {{0, 
          Complex[ 0, -1]}, {
          Complex[ 0, 1], 0}}, {{1, 0}, {0, -1}}}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[50]//TableForm=  (18:29:08 on 9/3/02)",
  CellTags->"PauliSigma"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PaVe", "ObjectName",
  CellTags->"PaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PaVe"],

Cell["\<\
 PaVe[ i,j,... {p10,p12,...},{m1^2, mw^2, ...} ] denotes the \
invariant (and scalar) Passarino-Veltman integrals, i.e. the coefficient \
functions of the tensor integral decomposition. Joining plist and mlist gives \
the same conventions as for A0, B0, C0, D0. Automatic simlifications are \
performed for the coefficient functions of two-point integrals and for the \
scalar integrals.\
\>", "Text",
  CellTags->"PaVe"],

Cell[TextData[{
  "For a more detailed description see the manual (no changes from there \
basically): ",
  ButtonBox["http://www.mertig.com/oldfc",
    ButtonData:>{
      URL[ "http://www.mertig.com/oldfc"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  CellTags->"PaVe"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text",
  CellTags->"PaVe"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PaVe"],

Cell["Some of the PaVe's reduce to special cases.", "Text",
  CellTags->"PaVe"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVe", "[", 
      RowBox[{"0", ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], 
          "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1169]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVe"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", "\[NoBreak]",
         "c", "\[NoBreak]", ",", "\[NoBreak]", "d", "\[NoBreak]", ",", 
        "\[NoBreak]", "e", "\[NoBreak]", ",", "\[NoBreak]", "f", "\[NoBreak]",
         ",", "\[NoBreak]", "m1", "\[NoBreak]", ",", "\[NoBreak]", "m2", 
        "\[NoBreak]", ",", "\[NoBreak]", "m3", "\[NoBreak]", ",", 
        "\[NoBreak]", "m4", "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1169]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVe", "[", 
      RowBox[{"0", ",", "0", ",", 
        RowBox[{"{", "pp", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"m", "^", "2"}], ",", 
            RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1170]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVe"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "pp", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["m", "2"]}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["M", "2"]}], "-", "pp"}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["M", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox["M", "2"]}], "+", 
              SuperscriptBox["M", "2"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox["M", "2"], "+", "pp"}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            SuperscriptBox["M", "2"], "-", 
                            SuperscriptBox["m", "2"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "pp", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}], "-", 
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "0", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                      RowBox[{"2", " ", "pp"}]], "-", 
                    RowBox[{
                      FractionBox["1", "2"], " ", 
                      RowBox[{
                        SubscriptBox["B", "0"], "(", "pp", ",", 
                        SuperscriptBox["m", "2"], ",", 
                        SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1170]=  (21:56:03 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PaVeOrderList", "ObjectName",
  CellTags->"PaVeOrderList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PaVeOrderList"],

Cell["\<\
PaVeOrderList is an option for PaVeOrder and PaVeReduce, specifying \
in which order the arguments of D0 are to be permuted.\
\>", "Text",
  CellTags->"PaVeOrderList"],

Cell[TextData[{
  "See also: ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeOrder"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"D0"],
  "."
}], "Text",
  CellTags->"PaVeOrderList"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PaVeOrder", "ObjectName",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PaVeOrder"],

Cell[TextData[{
  "PaVeOrder[expr] orders the arguments of all D0 in expr in a standard way. \
PaVeOrder[expr, PaVeOrderList \[Rule] { {..., s, u, ...}, {... ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox["m", "1"], "2"], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox["m", "2"], "2"], TraditionalForm]]],
  " ...}, ...}] orders the arguments of all D0 in expr according to the \
specified ordering lists. The lists may contain only a subsequence of the \
D0-variables."
}], "Text",
  CellTags->"PaVeOrder"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text",
  CellTags->"PaVeOrder"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
            "me2", ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
      "]"}]], "Input",
  CellLabel->"In[1171]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", "\[NoBreak]",
         "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", "\[NoBreak]", ",", 
        "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "me2", "\[NoBreak]", ",", 
        "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1171]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
            "me2", ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
      "]"}]], "Input",
  CellLabel->"In[1172]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", "\[NoBreak]",
         "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", "\[NoBreak]", ",", 
        "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "me2", "\[NoBreak]", ",", 
        "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1172]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"D0", "[", 
            RowBox[{
            "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "m12",
               ",", "m22", ",", "m32", ",", "m42"}], "]"}], "+", 
          RowBox[{"D0", "[", 
            RowBox[{
            "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", 
              ",", "me2", ",", "0", ",", "me2", ",", "0"}], "]"}]}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"f", ",", "e"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[1173]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeOrder"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
          "\[NoBreak]", ",", "\[NoBreak]", "d", "\[NoBreak]", ",", 
          "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]", "b", 
          "\[NoBreak]", ",", "\[NoBreak]", "f", "\[NoBreak]", ",", 
          "\[NoBreak]", "e", "\[NoBreak]", ",", "\[NoBreak]", "m22", 
          "\[NoBreak]", ",", "\[NoBreak]", "m12", "\[NoBreak]", ",", 
          "\[NoBreak]", "m42", "\[NoBreak]", ",", "\[NoBreak]", "m32", 
          "\[NoBreak]", ")"}], "+", 
        RowBox[{
          FormBox[
            SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
          "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", 
          "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", 
          "\[NoBreak]", ",", "\[NoBreak]", "mw2", "\[NoBreak]", ",", 
          "\[NoBreak]", "me2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
          "\[NoBreak]", ",", "\[NoBreak]", 
          FormBox["0",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
          FormBox["0",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1173]=  (21:56:03 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PaVeReduce", "ObjectName",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PaVeReduce"],

Cell["\<\
PaVeReduce[expr] reduces all Passarino-Veltman integrals (i.e. all \
PaVe's) in expr down to scalar A0, B0, C0 and D0.\
\>", "Text",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "PaVeReduce", "]"}]], "Input",
  CellLabel->"In[1174]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "True"}], ",", 
          RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PaVeOrderList", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1174]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "WriteOutPaVe"}]], "Input",
  CellLabel->"In[1175]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox["\<\"WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If \
set to a string, the results of all Passarino-Veltman PaVe's are stored in \
files with names generated from this string and the arguments of PaVe.\"\>", 
      TraditionalForm]], "Print"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FRH",
    ButtonData:>"FRH",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FRH"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text",
  CellTags->"PaVeReduce"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeReduce", "[", 
      RowBox[{
        RowBox[{"PaVe", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"{", 
              RowBox[{"s", ",", 
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"m", "^", "2"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}], ",", 
        RowBox[{"IsolateHead", "\[Rule]", "L"}]}], "]"}]], "Input",
  CellLabel->"In[1176]:=  (21:56:03 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"8", " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"3", " ", 
                  SuperscriptBox["M", "2"]}], "-", 
                RowBox[{"2", " ", "s"}]}], ")"}], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "s", 
              "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["M", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["M", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["m", "2"], "-", 
                SuperscriptBox["M", "2"]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox["m", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"8", " ", 
                  SuperscriptBox["m", "4"]}], "-", 
                RowBox[{"10", " ", 
                  SuperscriptBox["M", "2"], " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"2", " ", "s", " ", 
                  SuperscriptBox["m", "2"]}], "+", 
                RowBox[{
                  SuperscriptBox["M", "2"], " ", "s"}]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox["m", "2"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], 
        "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"6", " ", 
                  SuperscriptBox["M", "2"]}], "-", "s"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "s", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], 
        "+", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1176]=  (21:56:03 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"L", "[", "10", "]"}]], "Input",
  CellLabel->"In[1177]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1177]=  (21:56:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", "%", "]"}]], "Input",
  CellLabel->"In[1178]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1178]=  (21:56:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeReduce", "[", 
      RowBox[{
        RowBox[{"PaVe", "[", 
          RowBox[{"2", ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "mw2", ",", 
                "t"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "0", ",", 
                "mw2"}], "}"}]}], "]"}], ",", 
        RowBox[{"WriteOutPaVe", "\[Rule]", "\"\<p\>\""}]}], "]"}]], "Input",
  CellLabel->"In[1179]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", "mw2", ",", "mw2", ")"}], 
          RowBox[{"mw2", "-", "t"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["B", "0"], "(", "t", ",", "mw2", ",", "me2", ")"}], 
          RowBox[{"mw2", "-", "t"}]], "+", 
        FractionBox["2", 
          RowBox[{"mw2", "-", "t"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1179]=  (21:56:04 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\""}], ",", "String"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[1180]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - B0[t, mw2, \
SmallVariable[me2]]/\"\>"},
            {"\<\"  (mw2 - t)\"\>"},
            {"\<\"  ) \"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - B0[t, mw2, \
SmallVariable[me2]]/", "  (mw2 - t)", "  ) "}]], TraditionalForm]], "Output",
  CellLabel->"Out[1180]//TableForm=  (21:56:04 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "/@", 
        RowBox[{
        "FileNames", "[", "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\"", "]"}]}], 
      ";"}]], "Input",
  CellLabel->"In[1181]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    RowBox[{
      RowBox[{"se", "=", 
        RowBox[{"SmallVariable", "[", "ME2", "]"}]}], ";"}]], "Input",
  CellLabel->"In[1182]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"d122", "=", 
      RowBox[{"PaVeReduce", "[", 
        RowBox[{
          RowBox[{"PaVe", "[", 
            RowBox[{"1", ",", "2", ",", "2", ",", 
              RowBox[{"{", 
                RowBox[{
                "se", ",", "MW2", ",", "MW2", ",", "se", ",", "S", ",", "T"}],
                 "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "se", ",", "0", ",", "se"}], "}"}]}], "]"}],
           ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"S", ",", "T", ",", "U", ",", 
                RowBox[{"2", " ", "MW2"}]}], "}"}]}], ",", 
          RowBox[{"IsolateHead", "\[Rule]", "F"}]}], "]"}]}]], "Input",
  CellLabel->"In[1183]:=  (21:56:04 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox["T", "2"], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
              "\[NoBreak]", ",", "\[NoBreak]", "MW2", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "T", 
              "\[NoBreak]", ",", "\[NoBreak]", "S", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "3"]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"MW2", "-", "S"}], ")"}], "2"], " ", "T", " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
              "\[NoBreak]", ",", "\[NoBreak]", "S", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"S", " ", "U"}]}], ")"}], "3"]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox["T", "2"], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "T", 
              "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "2"]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"4", " ", "S", " ", "MW2"}], "+", 
                RowBox[{"2", " ", 
                  SuperscriptBox["S", "2"]}], "+", 
                RowBox[{"S", " ", "U"}]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "S", ",", "0", ",", "0", ")"}], 
            " ", "S"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["MW2", "5"]}], "-", 
                  RowBox[{"5", " ", "S", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"U", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"16", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["MW2", "3"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", "S", " ", 
                    SuperscriptBox["U", "2"], " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "2"], " ", "U", " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "4"], " ", "MW2"}], "+", 
                  RowBox[{"8", " ", 
                    SuperscriptBox["S", "3"], " ", "U", " ", "MW2"}], "+", 
                  RowBox[{
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["U", "3"]}], "+", 
                  RowBox[{
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["U", "2"]}]}], ")"}], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "MW2", ",", "0", ",", "ME2", 
                ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{"MW2", "-", "S"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"MW2", "+", "S"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"9", " ", "S", " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"U", " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"4", " ", "S", " ", "U", " ", "MW2"}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "3"]}], "+", 
                  RowBox[{"3", " ", "S", " ", 
                    SuperscriptBox["U", "2"]}], "+", 
                  RowBox[{"5", " ", 
                    SuperscriptBox["S", "2"], " ", "U"}]}], ")"}], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "T", ",", "ME2", ",", "ME2", 
                ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"MW2", "+", "S"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", 
                    SuperscriptBox["MW2", "6"]}], "-", 
                  RowBox[{"8", " ", "S", " ", 
                    SuperscriptBox["MW2", "5"]}], "-", 
                  RowBox[{"2", " ", "T", " ", 
                    SuperscriptBox["MW2", "5"]}], "+", 
                  RowBox[{"12", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["MW2", "4"]}], "+", 
                  RowBox[{"20", " ", "S", " ", "T", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"8", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"6", " ", "S", " ", 
                    SuperscriptBox["T", "2"], " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"36", " ", 
                    SuperscriptBox["S", "2"], " ", "T", " ", 
                    SuperscriptBox["MW2", "3"]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "4"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"6", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "2"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"20", " ", 
                    SuperscriptBox["S", "3"], " ", "T", " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "3"], " ", "MW2"}], "-", 
                  RowBox[{"6", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["T", "2"], " ", "MW2"}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "4"], " ", "T", " ", "MW2"}], "-", 
                  RowBox[{
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "4"]}]}], ")"}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                  "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
                "\[NoBreak]", ",", "\[NoBreak]", "MW2", "\[NoBreak]", ",", 
                "\[NoBreak]", "T", "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["ME2",
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["0",
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["ME2",
                  "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "3"]}], ")"}]}], "-", 
        FractionBox[
          RowBox[{"MW2", "+", "S"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"S", " ", "U"}]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1183]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"\"\<fctd122.for\>\"", ",", 
          RowBox[{"d122res", "==", "d122"}], ",", 
          RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[1184]:=  (21:56:06 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<fctd122.for\>\""}], ",", "String"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1185]:=  (21:56:06 on 5/16/01)",
  CellTags->"PaVeReduce"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"        d122res = -(MW2 + S)/(2D0*(4*MW2 - T)*(MW2**2 - \
S*U)) - \"\>"},
            {"\<\"     &  ((4*MW2**5 - 5*MW2**4*S - 16*MW2**3*S**2 + \"\>"},
            {"\<\"     &       4*MW2**2*S**3 + 4*MW2*S**4 - MW2**4*U - \
\"\>"},
            {"\<\"     &       4*MW2**2*S**2*U + 8*MW2*S**3*U + \"\>"},
            {"\<\"     &       4*MW2**2*S*U**2 + S**3*U**2 + \
S**2*U**3)*\"\>"},
            {"\<\"     &     B0(MW2,ME2,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2 - S)*(4*MW2 - T)**2*(MW2**2 - S*U)**2) + \
\"\>"},
            {"\<\"     &  (S*(MW2**2 - 4*MW2*S + 2*S**2 + \
S*U)*B0(S,Null,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2 - S)*(MW2**2 - S*U)**2) + \"\>"},
            {"\<\"     &  ((MW2 + S)*(4*MW2**3 - 9*MW2**2*S + 2*S**3 - \
\"\>"},
            {"\<\"     &       MW2**2*U - 4*MW2*S*U + 5*S**2*U + 3*S*U**2)*\"\
\>"},
            {"\<\"     &     B0(T,ME2,ME2))/\"\>"},
            {"\<\"     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**2) - \"\>"},
            {"\<\"     &  ((MW2 + S)*(2*MW2**6 - 8*MW2**5*S + 12*MW2**4*S**2 \
- \"\>"},
            {"\<\"     &       8*MW2**3*S**3 + 2*MW2**2*S**4 - 2*MW2**5*T + \
\"\>"},
            {"\<\"     &       20*MW2**4*S*T - 36*MW2**3*S**2*T + \"\>"},
            {"\<\"     &       20*MW2**2*S**3*T - 2*MW2*S**4*T - \"\>"},
            {"\<\"     &       6*MW2**3*S*T**2 + 6*MW2**2*S**2*T**2 - \"\>"},
            {"\<\"     &       6*MW2*S**3*T**2 + 4*MW2*S**2*T**3 - \
S**2*T**4)*\"\>"},
            {"\<\"     &     C0(MW2,MW2,T,ME2,Null,ME2))/\"\>"},
            {"\<\"     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**3) + \"\>"},
            {"\<\"     &  ((MW2 - \
S)**2*S**2*T*C0(MW2,S,ME2,ME2,Null,Null))/\"\>"},
            {"\<\"     &   (MW2**2 - S*U)**3 - \"\>"},
            {"\<\"     &  ((MW2 - S)*S**2*T**2*C0(T,ME2,ME2,ME2,ME2,Null))/\"\
\>"},
            {"\<\"     &   (2D0*(MW2**2 - S*U)**3) + \"\>"},
            {"\<\"     &  ((MW2 - S)*S**3*T**2*\"\>"},
            {"\<\"     &     \
D0(MW2,MW2,ME2,ME2,T,S,ME2,Null,ME2,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2**2 - S*U)**3)\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "        d122res = -(MW2 + S)/(2D0*(4*MW2 - T)*(MW2**2 - S*U)) - ", 
          "     &  ((4*MW2**5 - 5*MW2**4*S - 16*MW2**3*S**2 + ", 
          "     &       4*MW2**2*S**3 + 4*MW2*S**4 - MW2**4*U - ", 
          "     &       4*MW2**2*S**2*U + 8*MW2*S**3*U + ", 
          "     &       4*MW2**2*S*U**2 + S**3*U**2 + S**2*U**3)*", 
          "     &     B0(MW2,ME2,Null))/", 
          "     &   (2D0*(MW2 - S)*(4*MW2 - T)**2*(MW2**2 - S*U)**2) + ", 
          "     &  (S*(MW2**2 - 4*MW2*S + 2*S**2 + S*U)*B0(S,Null,Null))/", 
          "     &   (2D0*(MW2 - S)*(MW2**2 - S*U)**2) + ", 
          "     &  ((MW2 + S)*(4*MW2**3 - 9*MW2**2*S + 2*S**3 - ", 
          "     &       MW2**2*U - 4*MW2*S*U + 5*S**2*U + 3*S*U**2)*", 
          "     &     B0(T,ME2,ME2))/", 
          "     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**2) - ", 
          "     &  ((MW2 + S)*(2*MW2**6 - 8*MW2**5*S + 12*MW2**4*S**2 - ", 
          "     &       8*MW2**3*S**3 + 2*MW2**2*S**4 - 2*MW2**5*T + ", 
          "     &       20*MW2**4*S*T - 36*MW2**3*S**2*T + ", 
          "     &       20*MW2**2*S**3*T - 2*MW2*S**4*T - ", 
          "     &       6*MW2**3*S*T**2 + 6*MW2**2*S**2*T**2 - ", 
          "     &       6*MW2*S**3*T**2 + 4*MW2*S**2*T**3 - S**2*T**4)*", 
          "     &     C0(MW2,MW2,T,ME2,Null,ME2))/", 
          "     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**3) + ", 
          "     &  ((MW2 - S)**2*S**2*T*C0(MW2,S,ME2,ME2,Null,Null))/", 
          "     &   (MW2**2 - S*U)**3 - ", 
          "     &  ((MW2 - S)*S**2*T**2*C0(T,ME2,ME2,ME2,ME2,Null))/", 
          "     &   (2D0*(MW2**2 - S*U)**3) + ", 
          "     &  ((MW2 - S)*S**3*T**2*", 
          "     &     D0(MW2,MW2,ME2,ME2,T,S,ME2,Null,ME2,Null))/", 
          "     &   (2D0*(MW2**2 - S*U)**3)"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1185]//TableForm=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "/@", 
        RowBox[{"FileNames", "[", "\"\<fctd122.for\>\"", "]"}]}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{"d122", ",", "se"}], "]"}], ";"}]], "Input",
  CellLabel->"In[1186]:=  (21:56:06 on 5/16/01)",
  CellTags->"PaVeReduce"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PD", "ObjectName",
  CellTags->"PD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PD"],

Cell["PD is an abbreviation for PropagatorDenominator.", "Text",
  CellTags->"PD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  ", ",
  ButtonBox["IFPD",
    ButtonData:>"IFPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPD"],
  "."
}], "Text",
  CellTags->"PD"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PlusDistribution", "ObjectName",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PlusDistribution"],

Cell["\<\
PlusDistribution[1/(1-x)] denotes a distribution (in the sense of \
the \"+\" prescription).\
\>", "Text",
  CellTags->"PlusDistribution"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Integrate2"],
  "."
}], "Text",
  CellTags->"PlusDistribution"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{"1", "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1187]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", 
          FractionBox["1", 
            RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",\

  CellLabel->"Out[1187]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1188]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", 
          FractionBox[
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",\

  CellLabel->"Out[1188]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1189]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1189]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}], "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1190]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1190]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Log", "[", 
                RowBox[{"1", "-", "x"}], "]"}], "^", "2"}], "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1191]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1191]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"x", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1192]:=  (21:56:06 on 5/16/01)",
  CellTags->"PlusDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
        SubscriptBox[
          RowBox[{"(", 
            FractionBox[
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "x"}], ")"}], 
              RowBox[{"1", "-", "x"}]], ")"}], "+"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1192]=  (21:56:06 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Polarization", "ObjectName",
  CellTags->"Polarization"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Polarization"],

Cell["\<\
Polarization[k] = Polarization[k, I] is the head of a polarization \
momentum with (incoming) momentum k. A slashed polarization vector (e1(k) \
slash) has to be entered as DiracSlash[Polarization[k]]. The internal \
representation for a polarization vector e1 corresponding to a boson with \
four momentum k is: Momentum[ Polarization[ k, I ] ]. With this notation \
transversality of polarization vectors is provided, i.e. , Pair[ Momentum[k], \
Momentum[ Polarization[k, I] ] ] yields 0. Polarization[k,-I] denotes the \
complex conjugate polarization. Polarization is also an option of various \
functions related to the operator product expansion. The setting 0 denotes \
the unpolarized and 1 the polarized case.\
\>", "Text",
  CellTags->"Polarization"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PolarizationVector",
    ButtonData:>"PolarizationVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PolarizationVector"],
  ", ",
  ButtonBox["PolarizationSum",
    ButtonData:>"PolarizationSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PolarizationSum"],
  ", ",
  ButtonBox["DoPolarizationSums",
    ButtonData:>"DoPolarizationSums",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DoPolarizationSums"],
  "."
}], "Text",
  CellTags->"Polarization"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Polarization"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Polarization", "[", "k", "]"}]], "Input",
  CellLabel->"In[1193]:=  (21:56:06 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1193]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Polarization", "[", "k", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1194]:=  (21:56:06 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    RowBox[{"Polarization", "[", 
      RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}]], "Output",
  CellLabel->"Out[1194]//StandardForm=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Polarization", "[", "k", "]"}], "//", 
      "ComplexConjugate"}]], "Input",
  CellLabel->"In[1195]:=  (21:56:06 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          InterpretationBox[
            SuperscriptBox["\<\"\[CurlyEpsilon]\"\>", "\<\"*\"\>"],
            Superscript[ "\[CurlyEpsilon]", "*"],
            Editable->False],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1195]=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1196]:=  (21:56:06 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    RowBox[{"Polarization", "[", 
      RowBox[{"k", ",", 
        RowBox[{"-", "\[ImaginaryI]"}]}], "]"}]], "Output",
  CellLabel->"Out[1196]//StandardForm=  (21:56:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", 
      RowBox[{"Polarization", "[", "k", "]"}], "]"}]], "Input",
  CellLabel->"In[1197]:=  (21:56:07 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1197]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", 
        RowBox[{"Polarization", "[", "k", "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1198]:=  (21:56:07 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{"Polarization", "[", 
          RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}], "]"}]], "Output",\

  CellLabel->"Out[1198]//StandardForm=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", " ", 
      RowBox[{
        RowBox[{"Momentum", "[", "k", "]"}], ",", " ", 
        RowBox[{"Momentum", "[", " ", 
          RowBox[{"Polarization", "[", 
            RowBox[{"k", ",", " ", "I"}], "]"}], " ", "]"}]}], " ", 
      "]"}]], "Input",
  CellLabel->"In[1199]:=  (21:56:07 on 5/16/01)",
  CellTags->"Polarization"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1199]=  (21:56:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PolarizationSum", "ObjectName",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PolarizationSum"],

Cell["\<\
PolarizationSum[ mu,nu, ... ] defines (as abbreviations) several \
polarization sums. The first two arguments are the interpreted as Lorentz \
indices, all further ones are momenta. PolarizationSum performs no \
calculations.\
\>", "Text",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "PolarizationSum", "]"}]], "Input",
  CellLabel->"In[1200]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1200]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Polarization"],
  "."
}], "Text",
  CellTags->"PolarizationSum"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[1201]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationSum"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1201]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k"}], "]"}]], "Input",
  CellLabel->"In[1202]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], 
          SuperscriptBox[
            FormBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"],
              "TraditionalForm"], "2"]], "-", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1202]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", "n"}], "]"}]], "Input",
  CellLabel->"In[1203]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["n",
                "TraditionalForm"], "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["n",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["n",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["n",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], 
          RowBox[{
            FormBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["n",
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1203]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", 
        RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}]}], "]"}]], "Input",
  CellLabel->"In[1204]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationSum"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SubsuperscriptBox["p", "1", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SubsuperscriptBox["p", "2", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        SubscriptBox["p", "1"], "-", 
                        SubscriptBox["p", "2"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox[
                FormBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        SubscriptBox["p", "1"], "-", 
                        SubscriptBox["p", "2"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], 
          RowBox[{
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1204]=  (21:56:07 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PolarizationUncontract", "ObjectName",
  CellTags->"PolarizationUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PolarizationUncontract"],

Cell["\<\
PolarizationUncontract does Uncontract on scalar products involving \
polarization vectors.\
\>", "Text",
  CellTags->"PolarizationUncontract"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Polarization"],
  ", ",
  ButtonBox["Uncontract",
    ButtonData:>"Uncontract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Uncontract"],
  "."
}], "Text",
  CellTags->"PolarizationUncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PolarizationVector", "ObjectName",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PolarizationVector"],

Cell["PolarizationVector[p, mu] gives a polarization vector.", "Text",
  CellTags->"PolarizationVector"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Polarization"],
  "."
}], "Text",
  CellTags->"PolarizationVector"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationVector", "[", 
      RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
  CellLabel->"In[1205]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1205]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Conjugate", "[", 
      RowBox[{"PolarizationVector", "[", 
        RowBox[{"k", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1206]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["\[CurlyEpsilon]", "\[Mu]", "*"], "(", 
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1206]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "A polarization vector ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
        TraditionalForm]]],
  "is a special four-vector."
}], "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolarizationVector", "[", 
        RowBox[{"k", ",", "mu"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1207]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "mu", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"Polarization", "[", 
            RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
      "]"}]], "Output",
  CellLabel->"Out[1207]//StandardForm=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "The transverality property ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], 
          "=", "0"}], TraditionalForm]]],
  " is built in."
}], "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
  CellLabel->"In[1208]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1208]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a1", "]"}]], "Input",
  CellLabel->"In[1209]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1209]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
  CellLabel->"In[1210]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
          FormBox[
            RowBox[{
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "-", "p"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1210]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a2", "]"}]], "Input",
  CellLabel->"In[1211]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", "p"}],
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1211]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Depending on the alphabetical ordering of the momenta simplifcations are \
done, e.g., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
              FormBox[
                RowBox[{"k", "-", "p"}],
                "TraditionalForm"], ")"}]}], "=", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"k", "-", "p", "-", "k"}], ")"}], "\[Mu]"]}], 
              RowBox[{
                SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
                FormBox[
                  RowBox[{"k", "-", "p"}],
                  "TraditionalForm"], ")"}]}], "=", 
            RowBox[{
              RowBox[{
                SuperscriptBox["k", "\[Mu]"], 
                RowBox[{
                  SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
                  FormBox[
                    RowBox[{"k", "-", "p"}],
                    "TraditionalForm"], ")"}]}], "=", 
              RowBox[{
                FormBox["k",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                RowBox[{
                  RowBox[{"(", "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                      "\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
                        FormBox[
                          RowBox[{"k", "-", "p"}],
                          "TraditionalForm"], "\[NoBreak]", ")"}],
                      "TraditionalForm"], "\[NoBreak]", ")"}], "."}]}]}]}]}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}]], "Input",
  CellLabel->"In[1212]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
          FormBox[
            RowBox[{
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "-", "p"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1212]=  (21:56:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a3", "]"}]], "Input",
  CellLabel->"In[1213]:=  (21:56:07 on 5/16/01)",
  CellTags->"PolarizationVector"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          FormBox[
            TagBox[
              StyleBox["\<\"k\"\>",
                FontSlant->"Italic"],
              (StyleForm[ #, FontSlant -> "Italic"]&)],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(",
         "\[NoBreak]", 
        FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", "p"}],
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1213]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "a3"}], "]"}]], "Input",
  CellLabel->"In[1214]:=  (21:56:08 on 5/16/01)",
  CellTags->"PolarizationVector"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PositiveInteger", "ObjectName",
  CellTags->"PositiveInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PositiveInteger"],

Cell["\<\
PositiveInteger is a data type. E.g. DataType[OPEm, \
PositiveInteger] gives True.\
\>", "Text",
  CellTags->"PositiveInteger"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  "."
}], "Text",
  CellTags->"PositiveInteger"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PositiveNumber", "ObjectName",
  CellTags->"PositiveNumber"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PositiveNumber"],

Cell["\<\
PositiveNumber is a data type. E.g. DataType[Epsilon, \
PositiveNumber] = True (by default). \
\>", "Text",
  CellTags->"PositiveNumber"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  "."
}], "Text",
  CellTags->"PositiveNumber"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PowerFactor", "ObjectName",
  CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PowerFactor"],

Cell[TextData[{
  "PowerFactor[exp] replaces ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["x", "a"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        SuperscriptBox["y", "a"], TraditionalForm]]],
  "with ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"PowerFactor"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PowerSimplify",
    ButtonData:>"PowerSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PowerSimplify"],
  "."
}], "Text",
  CellTags->"PowerFactor"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      SuperscriptBox["x", "a"], 
      SuperscriptBox["y", "a"]}]], "Input",
  CellLabel->"In[47]:=  (17:11:39 on 9/20/02)",
  CellTags->"PowerFactor"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["x", "a"], " ", 
        SuperscriptBox["y", "a"]}], TraditionalForm]], "Output",
  CellLabel->"Out[47]=  (17:11:39 on 9/20/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerFactor", "[", "%", "]"}]], "Input",
  CellLabel->"In[48]:=  (17:11:41 on 9/20/02)",
  CellTags->"PowerFactor"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]], "Output",
  CellLabel->"Out[48]=  (17:11:41 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PowerSimplify", "ObjectName",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PowerSimplify"],

Cell[TextData[{
  "PowerSimplify[exp] simplifies",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "x"}], ")"}], "a"]}], TraditionalForm]]],
  " to ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "a"], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["x", "a"], TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"y", "-", "x"}], ")"}], "n"], TraditionalForm]]],
  " to ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"x", "-", "y"}], ")"}], "n"], TraditionalForm]]],
  "; thus assuming that the exponent is an integer (even if it is symbolic). \
Furthermore ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], 
          RowBox[{"a", "+", "n"}]], TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"a", "+", "n"}]], TraditionalForm]]],
  " are expanded and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"2", " ", "m"}]], TraditionalForm]]],
  " \[Rule] ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
  " and (-1)^(",
  StyleBox["n",
    FontSlant->"Italic"],
  " ",
  StyleBox["m",
    FontSlant->"Italic"],
  ") \[Rule] 1 and (-1)^(",
  StyleBox["n",
    FontSlant->"Italic"],
  " ",
  StyleBox["m",
    FontSlant->"Italic"],
  ") \[Rule] ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          FormBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}],
            "TraditionalForm"], "m"], TraditionalForm]]],
  " for ",
  StyleBox["n",
    FontSlant->"Italic"],
  " even and odd respectively and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], 
          RowBox[{"-", "n"}]], TraditionalForm]]],
  " \[Rule] ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[ExponentialE]", 
          StyleBox[
            RowBox[{"\[ImaginaryI]", 
              StyleBox["m",
                FontSlant->"Italic"], "\[Pi]"}]]], TraditionalForm]]],
  " \[Rule] ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"PowerSimplify"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["OPEm",
    ButtonData:>"OPEi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEm"],
  "."
}], "Text",
  CellTags->"PowerSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1215]:=  (21:56:08 on 5/16/01)",
  CellTags->"PowerSimplify"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1215]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"OPEm", "+", "2"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1216]:=  (21:56:08 on 5/16/01)",
  CellTags->"PowerSimplify"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
  CellLabel->"Out[1216]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"OPEm", "-", "2"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1217]:=  (21:56:08 on 5/16/01)",
  CellTags->"PowerSimplify"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
  CellLabel->"Out[1217]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{"I", "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
  CellLabel->"In[1218]:=  (21:56:08 on 5/16/01)",
  CellTags->"PowerSimplify"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
  CellLabel->"Out[1218]=  (21:56:08 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Power2", "ObjectName",
  CellTags->"Power2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Power2"],

Cell["\<\
Power2[x, y] represents x^y.  Sometimes Power2 is more useful than \
the Mathematica Power. Power2[-a,b] simplifies to (-1)^b Power2[a,b] (if no \
Epsilon is in b ...).\
\>", "Text",
  CellTags->"Power2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PowerFactor",
    ButtonData:>"PowerFactor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PowerFactor"],
  "."
}], "Text",
  CellTags->"Power2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Power2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Power", "[", 
      RowBox[{
        RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
  CellLabel->"In[50]:=  (17:13:33 on 9/20/02)",
  CellTags->"Power2"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "a"}], ")"}], "b"], TraditionalForm]], "Output",
  CellLabel->"Out[50]=  (17:13:33 on 9/20/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Power2", "[", 
      RowBox[{
        RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
  CellLabel->"In[51]:=  (17:13:38 on 9/20/02)",
  CellTags->"Power2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", 
        FormBox[
          SuperscriptBox["a", "b"],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[51]=  (17:13:39 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PostFortranFile", "ObjectName",
  CellTags->"PostFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PostFortranFile"],

Cell["\<\
PostFortranFile is an option for Write2 which may be set to a file \
name (or a list of file names) or a string,  which will be put at the end of \
the generated Fortran file.\
\>", "Text",
  CellTags->"PostFortranFile"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  ", ",
  ButtonBox["PreFortranFile",
    ButtonData:>"PreFortranFile",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PreFortranFile"],
  "."
}], "Text",
  CellTags->"PostFortranFile"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Prefactor", "ObjectName",
  CellTags->"Prefactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Prefactor"],

Cell["\<\
Prefactor is an option for OneLoop and OneLoopSum. If set as option \
of OneLoop, the amplitude is multiplied by Prefactor before calculation; if \
specified as option of OneLoopSum, after calculation in the final result as a \
global factor.\
\>", "Text",
  CellTags->"Prefactor"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"Prefactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PreFortranFile", "ObjectName",
  CellTags->"PreFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PreFortranFile"],

Cell["\<\
PreFortranFile is an option for Write2 which may be set to a file \
name (or a list of file names) or a string, which will be put at the \
beginning of the generated Fortran file.\
\>", "Text",
  CellTags->"PreFortranFile"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  ", ",
  ButtonBox["PostFortranFile",
    ButtonData:>"PostFortranFile",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PostFortranFile"],
  "."
}], "Text",
  CellTags->"PreFortranFile"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PropagatorDenominator", "ObjectName",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PropagatorDenominator"],

Cell[TextData[{
  "PropagatorDenominator[q, m] is a factor of the denominator of a \
propagator. If p is supposed to be D-dimensional enter: \
PropagatorDenominator[Momentum[q, D], m]. What is meant is ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "1", "/", 
          RowBox[{"(", 
            RowBox[{"q", "^", "2"}]}]}], TraditionalForm]]],
  " - ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  "). PropagatorDenominator[p] evaluates to PropagatorDenominator[p, 0]. "
}], "Text",
  CellTags->"PropagatorDenominator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["PropagatorDenominatorExplicit",
    ButtonData:>"PropagatorDenominatorExplicit",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominatorExplicit"],
  ", ",
  ButtonBox["IFPD",
    ButtonData:>"IFPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"IFPD"],
  "."
}], "Text",
  CellTags->"PropagatorDenominator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{"p", ",", "m"}], "]"}]}]], "Input",
  CellLabel->"In[1222]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1222]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", "p", "]"}]], "Input",
  CellLabel->"In[1223]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[1223]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t1", "=", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{"q", ",", "m"}], "]"}]}], " "}]], "Input",
  CellLabel->"In[1224]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["q", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1224]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", "t1", "]"}], "]"}]], "Input",
  CellLabel->"In[1225]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1225]//StandardForm=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"t1", ",", "D"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1226]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1226]//StandardForm=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PropagatorDenominatorExplicit", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1227]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1227]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
  CellLabel->"In[1228]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"],

Cell[BoxData[
    FractionBox["1", 
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "q", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]]], "Output",
  CellLabel->"Out[1228]//StandardForm=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1229]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["PropagatorDenominatorExplicit", "ObjectName",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"PropagatorDenominatorExplicit"],

Cell["\<\
PropagatorDenominatorExplicit[exp] changes each occurence of \
PropagatorDenominator[a,b] in exp into 1/(ScalarProduct[a,a]-b^2) and \
replaces FeynAmpDenominator by Identity.\
\>", "Text",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text",
  CellTags->"PropagatorDenominatorExplicit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"q", ",", "m"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}], "//", 
      "FCI"}]], "Input",
  CellLabel->"In[1219]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1219]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PropagatorDenominatorExplicit", "[", "%", "]"}], "//", 
      "FCE"}]], "Input",
  CellLabel->"In[1220]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "-", 
              RowBox[{"2", " ", 
                FormBox[
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"]}], "+", 
              FormBox[
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1220]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1221]:=  (21:56:08 on 5/16/01)",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[BoxData[
    FractionBox["1", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"SPD", "[", 
              RowBox[{"q", ",", "q"}], "]"}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"SPD", "[", 
                RowBox[{"p", ",", "q"}], "]"}]}], "+", 
            RowBox[{"SPD", "[", 
              RowBox[{"q", ",", "q"}], "]"}]}], ")"}]}]]], "Output",
  CellLabel->"Out[1221]//StandardForm=  (21:56:08 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QGV", "ObjectName",
  CellTags->"QGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QGV"],

Cell["QGV is equivalent to QuarkGluonVertex.", "Text",
  CellTags->"QGV"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuarkGluonVertex",
    ButtonData:>"QuarkGluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkGluonVertex"],
  "."
}], "Text",
  CellTags->"QGV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QO", "ObjectName",
  CellTags->"QO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QO"],

Cell["QO is equivalent to Twist2QuarkOperator.", "Text",
  CellTags->"QO"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text",
  CellTags->"QO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QP", "ObjectName",
  CellTags->"QP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QP"],

Cell["\<\
QP is an alias for QuarkPropagator. QP[p] is the massless quark \
propagator. QP[{p,m}] gives the  quark propagator with mass m.\
\>", "Text",
  CellTags->"QP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuarkPropagator",
    ButtonData:>"QuarkPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkPropagator"],
  "."
}], "Text",
  CellTags->"QP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QuantumField", "ObjectName",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QuantumField"],

Cell[TextData[{
  "QuantumField is the head of quantized fields and their derivatives. \
QuantumField[par, ftype, {lorind}, {sunind}] denotes a quantum field of type \
ftype with (possible) Lorentz-indices lorind and SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") indices sunind. The optional first argument par denotes a partial \
derivative acting on the field."
}], "Text",
  CellTags->"QuantumField"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynRule"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  "."
}], "Text",
  CellTags->"QuantumField"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"QuantumField"],

Cell["This denotes a scalar field.", "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "S", "]"}]], "Input",
  CellLabel->"In[1230]:=  (21:56:08 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox["S", TraditionalForm]], "Output",
  CellLabel->"Out[1230]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}]], "Input",
  CellLabel->"In[1231]:=  (21:56:08 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
  CellLabel->"Out[1231]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "QuarkField", "]"}]], "Input",
  CellLabel->"In[1232]:=  (21:56:08 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox["\[Psi]", TraditionalForm]], "Output",
  CellLabel->"Out[1232]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell["This is a field with a Lorentz index.", "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"B", ",", 
        RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1233]:=  (21:56:08 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      SubscriptBox["B", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[1233]=  (21:56:08 on 5/16/01)"]
}, Closed]],

Cell["Color indices should be put after the Lorentz ones.", "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1234]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        FormBox["A",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1234]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1235]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1235]//StandardForm=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["A", "\[CapitalDelta]", "a"], TraditionalForm]]],
  " is a short form for ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[CapitalDelta]", "^", "\[Mu]"}], " ", 
          RowBox[{
            SubsuperscriptBox["A", "\[Mu]", "a"], "."}]}], 
        TraditionalForm]]],
  " "
}], "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"A", ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1236]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["A", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1236]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell["\<\
 The first list of indices is usually interpreted as type \
LorentzIndex, except for OPEDelta, which gets converted to type Momentum. \
\
\>", "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "OPEDelta", "}"}], ",", 
          RowBox[{"{", "a", "}"}]}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1237]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"A", ",", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1237]//StandardForm=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell["Derivatives of fields are denoted as follows.", "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "A", ",", 
        RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1238]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[Null]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1238]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "S"}], 
      "]"}]], "Input",
  CellLabel->"In[1239]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]], 
        SubsuperscriptBox["S", "\[Null]", "\[Null]"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1239]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "A", ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1240]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\[PartialD]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]],
          "TraditionalForm"], 
        SubsuperscriptBox["A", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1240]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PartialD", "[", "OPEDelta", "]"}], "^", "OPEm"}], ",", "A",
         ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1241]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["\[PartialD]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox["m",
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1241]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"QuantumField", "[", "A", "]"}], "]"}], " ", "===", " ", 
      RowBox[{"QuantumField", "[", "A", "]"}]}]], "Input",
  CellLabel->"In[1242]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuantumField"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[1242]=  (21:56:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QuarkField", "ObjectName",
  CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QuarkField"],

Cell["\<\
QuarkField is the name of a fermionic field. QuarkField is just a \
name with no functional properties. Only typesetting rules are \
attached.\
\>", "Text",
  CellTags->"QuarkField"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiQuarkField",
    ButtonData:>"AntiQuarkField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AntiQuarkField"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuantumField"],
  "."
}], "Text",
  CellTags->"QuarkField"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell[BoxData["QuarkField"], "Input",
  CellLabel->"In[1243]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkField"],

Cell[BoxData[
    FormBox["\[Psi]", TraditionalForm]], "Output",
  CellLabel->"Out[1243]=  (21:56:09 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QuarkGluonVertex", "ObjectName",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QuarkGluonVertex"],

Cell[TextData[{
  "QuarkGluonVertex[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", 
          StyleBox["a",
            FontSlant->"Plain"]}], TraditionalForm]]],
  "] gives the Feynman rule for the quark-gluon vertex. "
}], "Text",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "QuarkGluonVertex", "]"}]], "Input",
  CellLabel->"In[1244]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1244]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text",
  CellTags->"QuarkGluonVertex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a"}], "]"}]], "Input",
  CellLabel->"In[1245]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1245]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1246]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output",
  CellLabel->"Out[1246]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "2"}]}], "]"}]], "Input",
  CellLabel->"In[1247]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"3", " ", "\[ImaginaryI]", " ", 
          SubscriptBox["C", "A"], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output",
  CellLabel->"Out[1247]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "3"}]}], "]"}]], "Input",
  CellLabel->"In[1248]:=  (21:56:09 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "A"], "+", 
              SubscriptBox["C", "F"]}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output",
  CellLabel->"Out[1248]=  (21:56:09 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", "k", "}"}], ",", 
        RowBox[{"OPE", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1249]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", "\[CapitalKoppa]", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "2"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1249]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", "k", "}"}], ",", 
        RowBox[{"OPE", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1250]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1250]=  (21:56:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QuarkMass", "ObjectName",
  CellTags->"QuarkMass"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QuarkMass"],

Cell["QuarkMass is an option of Amplitude.", "Text",
  CellTags->"QuarkMass"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Amplitude",
    ButtonData:>"Amplitude",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Amplitude"],
  "."
}], "Text",
  CellTags->"QuarkMass"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["QuarkPropagator", "ObjectName",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"QuarkPropagator"],

Cell["\<\
QuarkPropagator[p] is the massless quark propagator. \
QuarkPropagator[{p,m}] or gives the quark propagator with mass m.\
\>", "Text",\

  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "QuarkPropagator", "]"}]], "Input",
  CellLabel->"In[1251]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkPropagator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Loop", "\[Rule]", "0"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1251]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["QuarkGluonVertex",
    ButtonData:>"QuarkGluonVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"QuarkGluonVertex"],
  "."
}], "Text",
  CellTags->"QuarkPropagator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", "p", "]"}]], "Input",
  CellLabel->"In[1252]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[1252]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", 
      RowBox[{"{", 
        RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Input",
  CellLabel->"In[1253]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{"m", "+", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], ")"}]}], 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1253]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
  CellLabel->"In[1254]:=  (21:56:10 on 5/16/01)",
  CellTags->"QuarkPropagator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["p",
              "TraditionalForm"]}]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[1254]=  (21:56:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ReduceGamma", "ObjectName",
  CellTags->"ReduceGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ReduceGamma"],

Cell["\<\
ReduceGamma is an option of OneLoop. If set to True all \
DiracMatrix[6] and DiracMatrix[7] (i.e. all ChiralityProjector) are reduced \
to Gamma5.\
\>", "Text",
  CellTags->"ReduceGamma"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"ReduceGamma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ReduceToScalars", "ObjectName",
  CellTags->"ReduceToScalars"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ReduceToScalars"],

Cell["\<\
ReduceToScalars is an option for OneLoop  and OneLoopSum that \
specifies whether the result will be reduced to scalar A0, B0, C0 and D0 \
scalar integrals.\
\>", "Text",
  CellTags->"ReduceToScalars"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"ReduceToScalars"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Rename", "ObjectName",
  CellTags->"Rename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Rename"],

Cell["\<\
Rename is an option for Contract. If set to True, dummy indices in \
Eps are renamed, using $MU[i].\
\>", "Text",
  CellTags->"Rename"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  "."
}], "Text",
  CellTags->"Rename"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RHI  ***unfinished***", "ObjectName",
  CellTags->"RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RHI"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, \
mark1 \[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). \
The exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
  CellTags->"RHI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI2FC",
    ButtonData:>"RHI2FC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI2FC"],
  "."
}], "Text",
  CellTags->"RHI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RHI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RHI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RHI2FC  ***unfinished***", "ObjectName",
  CellTags->"RHI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RHI2FC"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, \
mark1 \[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). \
The exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
  CellTags->"RHI2FC"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI2FC",
    ButtonData:>"RHI2FC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI2FC"],
  "."
}], "Text",
  CellTags->"RHI2FC"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RHI2FC"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RHI2FC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RHM  ***unfinished***", "ObjectName",
  CellTags->"RHM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RHM"],

Cell["RHM[] is like RHI[], gives Gamma functions.", "Text",
  CellTags->"RHM"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"RHM"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RHM"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RHM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RHO  ***unfinished***", "ObjectName",
  CellTags->"RHO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RHO"],

Cell["\<\
RHO[i] with i from 1 to 4 is an abbreviation for the 4 operators \
(eq. (3.2.17) -- (3.2.20)) defined in R.Hambergs thesis. The Lorentz indices \
are suppressed. The explicit expressions may be recovered by RHO[i, mu, nu, \
p].\
\>", "Text",
  CellTags->"RHO"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHI",
    ButtonData:>"RHI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHI"],
  "."
}], "Text",
  CellTags->"RHO"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RHO"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RHO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RHP  ***unfinished***", "ObjectName",
  CellTags->"RHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RHP"],

Cell["\<\
RHP[i, mu, nu, p] with i from 1 to 4 gives the projectors for \
RHO[i]. RHP[mu, nu, p] gives Sum[RHP[i,mu, nu, p] RHO[i], {i, 4}] collected \
with respect to mu and nu.\
\>", "Text",
  CellTags->"RHP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RHO",
    ButtonData:>"RHO",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"RHO"],
  "."
}], "Text",
  CellTags->"RHP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RHP"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RHP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RTL  ***unfinished***", "ObjectName",
  CellTags->"RTL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RTL"],

Cell["\<\
RTL[exp] inserts the list of known TLI integrals into exp, \
substitutes D\[RightArrow]4+Epsilon and expands around Epsilon up to the \
finite part.\
\>", "Text",
  CellTags->"RTL"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  ", ",
  ButtonBox["Epsilon",
    ButtonData:>"Epsilon",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Epsilon"],
  "."
}], "Text",
  CellTags->"RTL"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RTL"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1149]:=  (21:56:02 on 5/16/01)",
  CellTags->"RTL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RightPartialD", "ObjectName",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RightPartialD"],

Cell[TextData[{
  "RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
  ", acting to the right."
}], "Text",
  CellTags->"RightPartialD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LeftPartialD"],
  "."
}], "Text",
  CellTags->"RightPartialD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"RightPartialD", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[1255]:=  (21:56:10 on 5/16/01)",
  CellTags->"RightPartialD"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1255]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Input",
  CellLabel->"In[1256]:=  (21:56:10 on 5/16/01)",
  CellTags->"RightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1256]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
  CellLabel->"In[1257]:=  (21:56:10 on 5/16/01)",
  CellTags->"RightPartialD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[Null]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1257]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1258]:=  (21:56:10 on 5/16/01)",
  CellTags->"RightPartialD"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1258]//StandardForm=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1259]:=  (21:56:10 on 5/16/01)",
  CellTags->"RightPartialD"],

Cell[BoxData[
    RowBox[{"RightPartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
  CellLabel->"Out[1259]//StandardForm=  (21:56:10 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["RussianTrick", "ObjectName",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"RussianTrick"],

Cell["\<\
RussianTrick[exp, k, {q1,q2,p}] (=RussianTrick[exp,p,p,{q1,q2,p}]) \
does the integration by parts where p is the external momentum. \
RussianTrick[exp, k,l, {q1,q2,p}] (=RussianTrick[exp,k,l]) does integration \
by parts where l is the momentum to be differentiated.\
\>", "Text",
  CellTags->"RussianTrick"],

Cell["The result is an expression which is vanishing.", "Text",
  CellTags->"RussianTrick"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourDivergence",
    ButtonData:>"FourDivergence",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourDivergence"],
  ", ",
  ButtonBox["FourLaplacian",
    ButtonData:>"FourLaplacian",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FourLaplacian"],
  "."
}], "Text",
  CellTags->"RussianTrick"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"RHI", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
            "OPEm", ",", "0", ",", " ", "0", ",", "0", ",", " ", "0"}], "}"}],
           ",", " ", 
          RowBox[{"{", 
            RowBox[{
            "1", ",", " ", "1", ",", " ", "1", ",", " ", "1", ",", " ", "1"}],
             "}"}]}], "]"}]}]], "Input",
  CellLabel->"In[1260]:=  (21:56:10 on 5/16/01)",
  CellTags->"RussianTrick"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        StyleBox["T",
          FontFamily->"Times",
          FontWeight->"Bold"], 
        StyleBox[
          RowBox[{"1", "1", "1", "1", "1"}],
          FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
          FormBox["m",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1260]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "//", "RHI2FC"}]], "Input",
  CellLabel->"In[1261]:=  (21:56:10 on 5/16/01)",
  CellTags->"RussianTrick"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"],
                "TraditionalForm"]}], ")"}], "m"], 
        RowBox[{
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "1", "2"], ".", 
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "2", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1261]=  (21:56:10 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"RussianTrick", "[", 
      RowBox[{
        RowBox[{"%", "//", "RHI2FC"}], ",", "q2"}], "]"}]], "Input",
  CellLabel->"In[1262]:=  (21:56:10 on 5/16/01)",
  CellTags->"RussianTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}], ")"}], "m"], 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
              ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
              ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", 
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", 
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"]}], ")"}], "2"]}]]}], "-", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], ")"}], "m"], 
          RowBox[{
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "1", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"]}]], 
        "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"4", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "1", "2"], ".", 
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "2", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"]}]], 
        "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"],
                  "TraditionalForm"]}], ")"}], "m"], 
          RowBox[{
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "2", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"],
                    "TraditionalForm"]}], ")"}], "m"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}], 
          RowBox[{
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "2", "2"], ".", 
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "1", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], 
              "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1262]=  (21:56:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FC2RHI", "[", "%", "]"}]], "Input",
  CellLabel->"In[1263]:=  (21:56:12 on 5/16/01)",
  CellTags->"RussianTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"0", "1", "1", "1", "2"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "-", 
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"1", "0", "1", "1", "2"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "-", 
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"1", "0", "1", "2", "1"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "4"}], ")"}], " ", 
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "1", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"], " ", 
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "2", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1263]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{"%", ",", "t"}], "]"}]], "Input",
  CellLabel->"In[1264]:=  (21:56:13 on 5/16/01)",
  CellTags->"RussianTrick"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "1", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"], "\[Rule]", 
          FractionBox[
            RowBox[{
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"0", "1", "1", "1", "2"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "-", 
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"1", "0", "1", "1", "2"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "-", 
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"1", "0", "1", "2", "1"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                FormBox[
                  SubsuperscriptBox[
                    StyleBox["T",
                      FontFamily->"Times",
                      FontWeight->"Bold"], 
                    StyleBox[
                      RowBox[{"1", "1", "1", "2", "1"}],
                      FontColor->RGBColor[0, 0, 1]], 
                    RowBox[{
                      FormBox["m",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"]}]],
                  "TraditionalForm"]}]}], 
            RowBox[{"4", "-", "D"}]]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1264]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[1265]:=  (21:56:13 on 5/16/01)",
  CellTags->"RussianTrick"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ScalarGluonVertex", "ObjectName",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ScalarGluonVertex"],

Cell["\<\
ScalarGluonVertex[{p}, {q}, {mu,a}] or ScalarGluonVertex[ p,  q,  \
mu, a ] yields the scalar-scalar-gluon vertex (p and q are incoming momenta).
ScalarGluonVertex[{mu,a}, {nu,b}] yields the scalar-scalar-gluon-gluon vertex \
(p and q are incoming momenta).
The dimension  and the name of the coupling constant are determined by the \
options Dimension and CouplingConstant.\
\>", "Text",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "ScalarGluonVertex", "]"}]], "Input",
  CellLabel->"In[3]:=  (20:40:25 on 9/20/02)",
  CellTags->"ScalarGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (20:40:26 on 9/20/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", "p", "}"}], ",", " ", 
        RowBox[{"{", "q", "}"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"mu", ",", " ", "a"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[4]:=  (20:41:31 on 9/20/02)",
  CellTags->"ScalarGluonVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          SuperscriptBox[
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], 
            FormBox[
              FormBox["mu",
                "TraditionalForm"],
              "TraditionalForm"]], ")"}], " ", 
        SubscriptBox["T", 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (20:41:31 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ScalarProduct", "ObjectName",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ScalarProduct"],

Cell[TextData[{
  "ScalarProduct[p, q] is the input for scalar product. ScalarProduct[p] is \
equivalent to ScalarProduct[p, p]. Expansion of sums of momenta in \
ScalarProduct is done with ExpandScalarProduct. Scalar product may be set, \
e.g., ScalarProduct[a, b] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  "; but a and b must not contain sums. Note that ScalarProduct[a, b] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  " actually sets also: Pair[Momentum[a, ___], Momentum[b, ___]] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  ". It is encouraged to always set ScalarProduct's ",
  StyleBox["before",
    FontWeight->"Bold"],
  " any calculation. This improves the performance of FeynCalc ."
}], "Text",
  CellTags->"ScalarProduct"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ClearScalarProducts",
    ButtonData:>"ClearScalarProducts",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ClearScalarProducts"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["ScalarProductCancel",
    ButtonData:>"ScalarProductCancel",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProductCancel"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SPD"],
  "."
}], "Text",
  CellTags->"ScalarProduct"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[1266]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1266]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{
        RowBox[{"p", "+", "q"}], ",", 
        RowBox[{"-", "q"}]}], "]"}]], "Input",
  CellLabel->"In[1267]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"(", 
          RowBox[{
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "+", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1267]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[1268]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1268]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", "q", "]"}]], "Input",
  CellLabel->"In[1269]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["q",
          "TraditionalForm"], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1269]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1270]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1270]//StandardForm=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1271]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1271]//StandardForm=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          SubscriptBox["p", "1"], ",", 
          SubscriptBox["p", "2"]}], "]"}], " ", "=", " ", 
      RowBox[{"s", "/", "2"}]}]], "Input",
  CellLabel->"In[1272]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      FractionBox["s", "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1272]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", " ", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "-", "q"}], ",", 
          RowBox[{
            SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1273]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["s", "2"], "+", 
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "1"],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "2"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1273]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "-", "q"}], ",", 
          RowBox[{
            SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1274]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["s", "2"], "+", 
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "1"],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "2"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1274]=  (21:56:13 on 5/16/01)"]
}, Closed]],

Cell[BoxData["ClearScalarProducts"], "Input",
  CellLabel->"In[1275]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProduct"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ScalarProductCancel", "ObjectName",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ScalarProductCancel"],

Cell["\<\
ScalarProductCancel[exp, q1, q2, ...] cancels scalar products with \
propagators. ScalarProductCancel[exp] cancels simple cases.\
\>", "Text",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData["SPC"], "Input",
  CellLabel->"In[1276]:=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox["ScalarProductCancel", TraditionalForm]], "Output",
  CellLabel->"Out[1276]=  (21:56:13 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProductCancel", "//", "Options"}]], "Input",
  CellLabel->"In[1277]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1277]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "SPC"}]], "Input",
  CellLabel->"In[1278]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox["\<\"SPC is an abbreviation for ScalarProductCancel.\"\>", 
      TraditionalForm]], "Print",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ClearScalarProducts",
    ButtonData:>"ClearScalarProducts",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ClearScalarProducts"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SPD"],
  "."
}], "Text",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ScalarProductCancel"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"q", ",", "p"}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}]}], "//",
         "FCI"}]}]], "Input",
  CellLabel->"In[1279]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1279]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProductCancel", "[", 
      RowBox[{"t1", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[1280]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}]}]]}], "+", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1280]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{"%", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[1281]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1281]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPC", "[", 
      RowBox[{"t1", ",", "q", ",", 
        RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1282]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1282]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"q2", ",", "p"}], "]"}], 
          RowBox[{"SPD", "[", 
            RowBox[{"q1", ",", "p"}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q1", ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q2", ",", "m"}], "}"}], ",", 
              RowBox[{"q1", "-", "p"}], ",", 
              RowBox[{"q2", "-", "p"}], ",", 
              RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",\

  CellLabel->"In[1283]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], ".", 
          
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1283]=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPC", "[", 
      RowBox[{"t2", ",", "q1", ",", "q2", ",", 
        RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1284]:=  (21:56:14 on 5/16/01)",
  CellTags->"ScalarProductCancel"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          RowBox[{"4", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", 
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"]}], ")"}], "2"]}]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
              ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", 
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "1"]}], ")"}], "2"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                SuperscriptBox["m", "2"], "2"]}], "-", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "1", "2"], ".", 
            SubsuperscriptBox[
              StyleBox["\<\"q\"\>",
                FontSlant->"Italic"], "2", "2"], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"],
                 "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "4"], "4"], "+", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                SuperscriptBox["m", "2"]}], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"], "4"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
            ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "2"], "-", 
                  SubscriptBox[
                    StyleBox["\<\"q\"\>",
                      FontSlant->"Italic"], "1"]}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1284]=  (21:56:17 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
  CellLabel->"In[1285]:=  (21:56:17 on 5/16/01)",
  CellTags->"ScalarProductCancel"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ScaleMu", "ObjectName",
  CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ScaleMu"],

Cell["\<\
ScaleMu is the mass scale used for dimensional regularization of \
loop integrals.\
\>", "Text",
  CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell[BoxData["ScaleMu"], "Input",
  CellLabel->"In[29]:=  (15:43:20 on 9/21/02)",
  CellTags->"ScaleMu"],

Cell[BoxData[
    FormBox["\[Mu]", TraditionalForm]], "Output",
  CellLabel->"Out[29]=  (15:43:20 on 9/21/02)",
  CellTags->"ScaleMu"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Schouten", "ObjectName",
  CellTags->"Schouten"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Schouten"],

Cell["\<\
Schouten[expr] applies the Schouten identity on at most 42 terms in \
a sum. If Schouten should operate on larger expression you can give a second \
argument, e.g.: Schouten[expr, 4711] which will work on sums with less than \
4711 terms.
Schouten is also an option of Contract and DiracTrace. It may be set to an \
integer indicating the maximum number of terms onto which the function \
Schouten will be applied .\
\>", "Text",
  CellTags->"Schouten"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  "."
}], "Text",
  CellTags->"Schouten"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Schouten"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"Sum", "[", 
          RowBox[{
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"a", "=", "1"}], "4"], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"SP", "[", 
                    RowBox[{"k", ",", 
                      RowBox[{"q", "[", "a", "]"}]}], "]"}], 
                  RowBox[{"(", 
                    RowBox[{
                      FractionBox["1", "6"], 
                      RowBox[{"Eps", "[", 
                        RowBox[{
                          RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
                          RowBox[{"LorentzIndex", "[", "b", "]"}], ",", 
                          RowBox[{"LorentzIndex", "[", "c", "]"}], ",", 
                          RowBox[{"LorentzIndex", "[", "d", "]"}]}], "]"}], 
                      RowBox[{"Eps", "[", 
                        RowBox[{
                          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
                          RowBox[{"Momentum", "[", 
                            RowBox[{"q", "[", "b", "]"}], "]"}], ",", 
                          RowBox[{"Momentum", "[", 
                            RowBox[{"q", "[", "c", "]"}], "]"}], ",", 
                          RowBox[{"Momentum", "[", 
                            RowBox[{"q", "[", "d", "]"}], "]"}]}], "]"}]}], 
                    ")"}]}], ")"}]}], ",", 
            RowBox[{"{", 
              RowBox[{"b", ",", "1", ",", "4"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"c", ",", "1", ",", "4"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"d", ",", "1", ",", "4"}], "}"}]}], "]"}], "-", 
        RowBox[{
          RowBox[{"Eps", "[", 
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"q", "[", "1", "]"}], "]"}], ",", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", "[", "2", "]"}], "]"}], ",", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", "[", "3", "]"}], "]"}], ",", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", "[", "4", "]"}], "]"}]}], "]"}], "*", 
          RowBox[{"Pair", "[", 
            RowBox[{
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
              RowBox[{"Momentum", "[", "k", "]"}]}], "]"}]}]}]}]], "Input",
  CellLabel->"In[3]:=  (19:39:26 on 9/21/02)",
  CellTags->"Schouten"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            SuperscriptBox["\[Epsilon]", 
              RowBox[{
                FormBox[
                  RowBox[{"q", "(", "1", ")"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{"q", "(", "2", ")"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{"q", "(", "3", ")"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{"q", "(", "4", ")"}],
                  "TraditionalForm"]}]]}], " ", 
          SuperscriptBox[
            FormBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "3", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "2", ")"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "1", ")"}],
                "TraditionalForm"]}]], " ", 
          FormBox[
            RowBox[{
              FormBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"q", "(", "4", ")"}],
                "TraditionalForm"]}],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (19:39:27 on 9/21/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "//", "Schouten"}]], "Input",
  CellLabel->"In[4]:=  (19:39:27 on 9/21/02)",
  CellTags->"Schouten"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (19:39:27 on 9/21/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input",
  CellLabel->"In[7]:=  (19:39:49 on 9/21/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SD", "ObjectName",
  CellTags->"SD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SD"],

Cell[TextData[{
  "SD[i, j] is the (FeynCalc-external) Kronecker-delta for SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") with color indices i and j. SD[i,j] is transformed into \
SUNDelta[SUNIndex[i],SUNIndex[j]] by FeynCalcInternal."
}], "Text",
  CellTags->"SD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  "."
}], "Text",
  CellTags->"SD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1286]:=  (21:56:17 on 5/16/01)",
  CellTags->"SD"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1286]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1287]:=  (21:56:17 on 5/16/01)",
  CellTags->"SD"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1287]//StandardForm=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1288]:=  (21:56:17 on 5/16/01)",
  CellTags->"SD"],

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1288]//StandardForm=  (21:56:17 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SelectFree", "ObjectName",
  CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SelectFree"],

Cell["\<\
SelectFree[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, \
{a,b, ...}]&], except the special cases: SelectFree[a, b] returns a and \
SelectFree[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text",
  CellTags->"SelectFree"],

Cell["SelectFree is equivalent to Select1.", "Text",
  CellTags->"SelectFree"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FreeQ2",
    ButtonData:>"FreeQ2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FreeQ2"],
  ", ",
  ButtonBox["SelectNotFree",
    ButtonData:>"SelectNotFree",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SelectNotFree"],
  "."
}], "Text",
  CellTags->"SelectFree"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "b", "+", 
          RowBox[{"f", "[", "a", "]"}], "+", "d"}], ",", "a"}], 
      "]"}]], "Input",
  CellLabel->"In[1289]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox[
      RowBox[{"b", "+", "d"}], TraditionalForm]], "Output",
  CellLabel->"Out[1289]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"x", " ", "y"}], ",", " ", "x"}], "]"}]], "Input",
  CellLabel->"In[1290]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox["y", TraditionalForm]], "Output",
  CellLabel->"Out[1290]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"2", " ", "x", " ", "y", " ", "z", " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1291]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "z"}], TraditionalForm]], "Output",
  CellLabel->"Out[1291]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1292]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox["a", TraditionalForm]], "Output",
  CellLabel->"Out[1292]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"a", ",", "a"}], "]"}]], "Input",
  CellLabel->"In[1293]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1293]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"1", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[1294]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1294]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1295]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectFree"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1295]=  (21:56:17 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SelectGraphs", "ObjectName",
  CellTags->"SelectGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SelectGraphs"],

Cell["\<\
SelectGraphs is an option for OneLoopSum indicating that only a \
slected set of graphs of the list provided to OneLoopSum is to be calculated. \
Possible settings are: SelectGraphs\[RightArrow]{ i, j,  ... } or \
SelectGraphs\[RightArrow]{ a, {b, c}, ... } which indicates the graphs to be \
taken from the list provided to OneLoopSum. In the second setting the list \
{b, c} indicates that all amplitudes from b to c should be taken.\
\>", "Text",\

  CellTags->"SelectGraphs"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"SelectGraphs"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SelectNotFree", "ObjectName",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, x] returns that part of expr which is not free \
of any occurance of x.\
\>", "Text",
  CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, a, b, ...] is equivalent to Select[expr, \
!FreeQ2[#, {a, b, ...}]&], except the special cases: SelectNotFree[a, b] \
returns 1 and SelectNotFree[a, a] returns a (where a is not a product or a \
sum).\
\>", "Text",
  CellTags->"SelectNotFree"],

Cell["SelectNotFree is equivalent to Select2.", "Text",
  CellTags->"SelectNotFree"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FreeQ2",
    ButtonData:>"FreeQ2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FreeQ2"],
  ", ",
  ButtonBox["SelectFree",
    ButtonData:>"SelectFree",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SelectFree"],
  "."
}], "Text",
  CellTags->"SelectNotFree"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "b", "+", 
          RowBox[{"f", "[", "a", "]"}]}], ",", "a"}], "]"}]], "Input",
  CellLabel->"In[1296]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", "+", 
        RowBox[{"f", "(", "a", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1296]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"2", " ", "x", " ", "y", " ", 
          RowBox[{"f", "[", "x", "]"}], " ", "z"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1297]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", " ", "y", " ", 
        RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1297]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1298]:=  (21:56:17 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1298]=  (21:56:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "x"}], ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1299]:=  (21:56:18 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1299]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"a", ",", "a"}], "]"}]], "Input",
  CellLabel->"In[1300]:=  (21:56:18 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox["a", TraditionalForm]], "Output",
  CellLabel->"Out[1300]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"1", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[1301]:=  (21:56:18 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output",
  CellLabel->"Out[1301]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1302]:=  (21:56:18 on 5/16/01)",
  CellTags->"SelectNotFree"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1302]=  (21:56:18 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SelectSplit", "ObjectName",
  CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SelectSplit"],

Cell["\<\
SelectSplit[l, p] Construct list of mutually exclusive subsets from \
l in which every element li satisfies a criterium pj[li] with pj from p and \
appends the subset of remaining unmatched elements.\
\>", "Text",
  CellTags->"SelectSplit"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectSplit", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            SuperscriptBox["a", "2"], ",", 
            SuperscriptBox["b", "3"], ",", 
            SuperscriptBox["c", "4"], ",", 
            SuperscriptBox["d", "5"], ",", 
            SuperscriptBox["e", "6"], ",", 
            RowBox[{"f", "+", "g"}], ",", 
            SuperscriptBox["h", "4"]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"MatchQ", "[", 
                RowBox[{"#", ",", 
                  RowBox[{"_", "^", "2"}]}], "]"}], "&"}], ",", 
            RowBox[{
              RowBox[{"MatchQ", "[", 
                RowBox[{"#", ",", 
                  RowBox[{"_", "^", "4"}]}], "]"}], "&"}], ",", 
            RowBox[{
              RowBox[{"FreeQ", "[", 
                RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[29]:=  (21:16:30 on 9/20/02)",
  CellTags->"SelectSplit"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            SuperscriptBox["a", "2"], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              SuperscriptBox["c", "4"], ",", 
              SuperscriptBox["h", "4"]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"f", "+", "g"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              SuperscriptBox["b", "3"], ",", 
              SuperscriptBox["d", "5"], ",", 
              SuperscriptBox["e", "6"]}], "}"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[29]=  (21:16:30 on 9/20/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectSplit", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            SuperscriptBox["a", "2"], ",", 
            SuperscriptBox["b", "3"], ",", 
            SuperscriptBox["c", "4"], ",", 
            SuperscriptBox["d", "5"], ",", 
            SuperscriptBox["e", "6"], ",", 
            RowBox[{"f", "+", "g"}], ",", 
            SuperscriptBox["h", "4"]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"FreeQ", "[", 
                RowBox[{"#", ",", "Plus"}], "]"}], "&"}], ",", 
            RowBox[{
              RowBox[{"FreeQ", "[", 
                RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[30]:=  (21:16:46 on 9/20/02)",
  CellTags->"SelectSplit"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              SuperscriptBox["a", "2"], ",", 
              SuperscriptBox["b", "3"], ",", 
              SuperscriptBox["c", "4"], ",", 
              SuperscriptBox["d", "5"], ",", 
              SuperscriptBox["e", "6"], ",", 
              SuperscriptBox["h", "4"]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"f", "+", "g"}], "}"}], ",", 
          RowBox[{"{", "}"}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[30]=  (21:16:46 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Select1", "ObjectName",
  CellTags->"Select1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Select1"],

Cell["\<\
Select1[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, \
{a,b, ...}]&], except the special cases: Select1[a, b] returns a and \
Select1[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text",
  CellTags->"Select1"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Select2",
    ButtonData:>"Select2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Select2"],
  "."
}], "Text",
  CellTags->"Select1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Select2", "ObjectName",
  CellTags->"Select2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Select2"],

Cell["\<\
Select2[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, \
{a,b, ...}]&], except the special cases: Select2[a, b] returns 1 and  \
Select2[a,a] returns a (where a is not a product or a sum).\
\>", "Text",
  CellTags->"Select2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Select1",
    ButtonData:>"Select1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Select1"],
  "."
}], "Text",
  CellTags->"Select2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Series2", "ObjectName",
  CellTags->"Series2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Series2"],

Cell["\<\
 Series2 performs a series expansion around 0. Series2 is (up to \
the Gamma-bug) equivalent to Series, except that it applies Normal on the \
result and has an option FinalSubstitutions. Series2[f, e, n] is equivalent \
to Series2[f, {e, 0, n}].\
\>", "Text",
  CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Series2", "]"}]], "Input",
  CellLabel->"In[1303]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                TagBox["\[DoubledGamma]",
                  (EulerGamma&)], "\[Rule]", "0"}], "}"}]}], ",", 
          RowBox[{"Print", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1303]=  (21:56:18 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"x", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", 
        "1"}], "]"}]], "Input",
  CellLabel->"In[1304]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "\[Delta]", " ", 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
        "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[1304]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], 
      "]"}]], "Input",
  CellLabel->"In[1305]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", "x"}], "12"], "+", 
        FractionBox["1", "x"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1305]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1306]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{
          FractionBox["1", "x"], "+", 
          InterpretationBox[
            RowBox[{"O", "(", 
              SuperscriptBox["x", "0"], ")"}],
            SeriesData[ x, 0, {}, -1, 0, 1]]}],
        SeriesData[ x, 0, {1}, -1, 0, 1]], TraditionalForm]], "Output",
  CellLabel->"Out[1306]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2"}], 
      "]"}]], "Input",
  CellLabel->"In[1307]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "3"]}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox["x", "2"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", "x"}], "12"], "+", 
        FractionBox["1", "x"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1307]=  (21:56:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Series2", "[", 
        RowBox[{
          RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2", ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "]"}], "//", "FullSimplify"}]], "Input",
  CellLabel->"In[1308]:=  (21:56:18 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "12"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "2"}], " ", 
              SuperscriptBox[
                TagBox["\[DoubledGamma]",
                  (EulerGamma&)], "3"], " ", 
              SuperscriptBox["x", "2"]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Pi]", "2"], "-", 
                  RowBox[{"4", " ", "x", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}]}], ")"}], " ", "x"}], 
            "+", 
            RowBox[{"6", " ", 
              SuperscriptBox[
                TagBox["\[DoubledGamma]",
                  (EulerGamma&)], "2"], " ", "x"}], "-", 
            RowBox[{
              TagBox["\[DoubledGamma]",
                (EulerGamma&)], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["\[Pi]", "2"], " ", 
                    SuperscriptBox["x", "2"]}], "+", "12"}], ")"}]}], "+", 
            FractionBox["12", "x"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1308]=  (21:56:20 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Series", "[", 
          RowBox[{
            RowBox[{"Gamma", "[", "x", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "4"}], "}"}]}], "]"}], "//", 
        "Normal"}], "//", "FullSimplify"}]], "Input",
  CellLabel->"In[1309]:=  (21:56:20 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "12"]}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"2", " ", 
                SuperscriptBox[
                  TagBox["\[DoubledGamma]",
                    (EulerGamma&)], "3"]}], "+", 
              RowBox[{
                TagBox["\[DoubledGamma]",
                  (EulerGamma&)], " ", 
                SuperscriptBox["\[Pi]", "2"]}], "+", 
              RowBox[{"4", " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}], " ", 
          SuperscriptBox["x", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "12"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"6", " ", 
                SuperscriptBox[
                  TagBox["\[DoubledGamma]",
                    (EulerGamma&)], "2"]}], "+", 
              SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", "x"}], "-", 
        TagBox["\[DoubledGamma]",
          (EulerGamma&)], "+", 
        FractionBox["1", "x"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1309]=  (21:56:21 on 5/16/01)"]
}, Closed]],

Cell["\<\
There is a table of expansions of special hypergeometric functions.\
\
\>", "Text",
  CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"HypergeometricPFQ", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", 
                RowBox[{"OPEm", "-", "1"}], ",", 
                RowBox[{
                  RowBox[{"Epsilon", "/", "2"}], "+", "OPEm"}]}], "}"}], ",", 
            
            RowBox[{"{", 
              RowBox[{"OPEm", ",", 
                RowBox[{"OPEm", "+", "Epsilon"}]}], "}"}], ",", "1"}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"Epsilon", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1310]:=  (21:56:22 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "1", ")"}]], "(", "m", ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"m", "-", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"m", "-", "1"}], ")"}]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[1310]=  (21:56:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"Series2", "[", 
        RowBox[{
          RowBox[{"HypergeometricPFQ", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", " ", "OPEm", ",", " ", 
                  RowBox[{
                    RowBox[{"Epsilon", "/", "2"}], " ", "+", " ", "OPEm"}]}], 
                "}"}], ",", " ", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"1", " ", "+", " ", "OPEm"}], ",", " ", 
                  RowBox[{"Epsilon", " ", "+", " ", "OPEm"}]}], "}"}], ",", 
              " ", " ", "1"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"Epsilon", ",", "0", ",", "1"}], "}"}]}], 
        "]"}]}]], "Input",
  CellLabel->"In[1311]:=  (21:56:22 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          SuperscriptBox[
            RowBox[{
              SuperscriptBox[
                TagBox["\[Psi]",
                  PolyGamma], 
                RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "2"]}], "+", 
        FractionBox[
          RowBox[{"2", " ", "m"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "1", ")"}]], "(", "m", ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          RowBox[{
            SubscriptBox["S", "11"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "24"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          SuperscriptBox["\[Pi]", "2"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1311]=  (21:56:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hypergeometric2F1", "[", 
      RowBox[{"1", ",", " ", "Epsilon", ",", " ", 
        RowBox[{"1", " ", "+", " ", 
          RowBox[{"2", " ", "Epsilon"}]}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1312]:=  (21:56:22 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      TagBox[
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[InvisiblePrefixScriptBase]", 
                FormBox["2",
                  "TraditionalForm"]], 
              SubscriptBox["F", 
                FormBox["1",
                  "TraditionalForm"]]}], "\[InvisibleApplication]", 
            RowBox[{"(", 
              RowBox[{
                TagBox[
                  TagBox[
                    RowBox[{
                      TagBox["1",
                        Hypergeometric2F1,
                        Editable->True], ",", 
                      TagBox[
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm],
                        Hypergeometric2F1,
                        Editable->True]}],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox[
                  TagBox[
                    TagBox[
                      RowBox[{
                        RowBox[{"2", " ", 
                          TagBox["\[CurlyEpsilon]",
                            TraditionalForm]}], "+", "1"}],
                      Hypergeometric2F1,
                      Editable->True],
                    InterpretTemplate[ {
                      SlotSequence[ 1]}&]],
                  Hypergeometric2F1,
                  Editable->False], ";", 
                TagBox["x",
                  Hypergeometric2F1,
                  Editable->True]}], ")"}]}],
          InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
          Editable->False],
        Hypergeometric2F1], TraditionalForm]], "Output",
  CellLabel->"Out[1312]=  (21:56:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{"%", ",", 
        RowBox[{"{", 
          RowBox[{"Epsilon", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1313]:=  (21:56:22 on 5/16/01)",
  CellTags->"Series2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "6"]}], " ", 
                RowBox[{
                  SuperscriptBox["log", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{
                FractionBox["1", "6"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{
                  SubscriptBox["Li", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox["Li", "3"], "(", "x", ")"}]}], "-", 
              RowBox[{"2", " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "2"]}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{"3", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              FractionBox[
                SuperscriptBox["\[Pi]", "2"], "6"]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "-", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[1313]=  (21:56:22 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "There are over 100 more special expansions of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[ThinSpace]", "2"], 
          SubscriptBox["F", "1"]}], TraditionalForm]]],
  " tabulated in Series2.m. The interested user can consult the source code \
(search for HYPERLIST)."
}], "Text",
  CellTags->"Series2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Series3", "ObjectName",
  CellTags->"Series3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Series3"],

Cell["\<\
Series3 performs a series expansion around 0. Series3 is equivalent \
to Series, except that it applies Normal on the result and that some Series \
bugs are fixed. Series3[f, e, n] is equivalent to  Series3[f, {e, 0, \
n}].\
\>", "Text",
  CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Series3", "]"}]], "Input",
  CellLabel->"In[38]:=  (21:21:14 on 9/20/02)",
  CellTags->"Series3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[38]=  (21:21:14 on 9/20/02)",
  CellTags->"Series3"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Series2"],
  "."
}], "Text",
  CellTags->"Series3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"x", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", 
        "1"}], "]"}]], "Input",
  CellLabel->"In[53]:=  (21:23:51 on 9/20/02)",
  CellTags->"Series3"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "\[Delta]", " ", 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
        "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[53]=  (21:23:51 on 9/20/02)",
  CellTags->"Series3"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Series3", "[", 
        RowBox[{
          RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}], "//",
       "FullSimplify"}]], "Input",
  CellLabel->"In[50]:=  (21:23:16 on 9/20/02)",
  CellTags->"Series3"],

Cell[BoxData[
    FormBox[
      FractionBox["1", "x"], TraditionalForm]], "Output",
  CellLabel->"Out[50]=  (21:23:16 on 9/20/02)",
  CellTags->"Series3"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SetMandelstam", "ObjectName",
  CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SetMandelstam"],

Cell["\<\
SetMandelstam[s, t, u, p1, p2, p3, p4, m1, m2, m3, m4] defines the \
Mandelstam variables  s=(p1+p2)^2, t=(p1+p3)^2, u=(p1+p4)^2 and sets the pi \
on-shell: p1^2=m1^2, p2^2=m2^2, p3^2=m3^2, p4^2=m4^2. Note that p1 +  p2 + p3 \
+ p4 = 0 is assumed.
SetMandelstam[x, {p1, p2, p3, p4, p5}, {m1, m2, m3, m4, m5}] defines x[i, j] \
= (pi+pj)^2 and sets the pi on-shell. The pi satisfy: p1 + p2 + p3 + p4 + p5 \
= 0.\
\>", "Text",
  CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SetMandelstam", "]"}]], "Input",
  CellLabel->"In[58]:=  (21:26:53 on 9/20/02)",
  CellTags->"SetMandelstam"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"4", ",", "D"}], "}"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[58]=  (21:26:53 on 9/20/02)",
  CellTags->"SetMandelstam"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Mandelstam",
    ButtonData:>"Mandelstam",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Mandelstam"],
  "."
}], "Text",
  CellTags->"SetMandelstam"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SimplifyDeltaFunction", "ObjectName",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SimplifyDeltaFunction"],

Cell["\<\
SimplifyDeltaFunction[exp, x] simplifies f[x]*DeltaFunction[1-x] \
toLimit[f[x],x\[Rule]1] DeltaFunction[1-x] and applies a list of \
transformation rules for DeltaFunctionPrime[1-x]*x^(OPEm-1)*f[x] where \
x^(OPEm-1) is suppressed in exp.\
\>", "Text",
  CellTags->"SimplifyDeltaFunction"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["DeltaFunctionPrime",
    ButtonData:>"DeltaFunctionPrime",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeltaFunctionPrime"],
  "."
}], "Text",
  CellTags->"SimplifyDeltaFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"g", "[", "x", "]"}], " ", 
      RowBox[{"DeltaFunction", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1314]:=  (21:56:22 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1314]=  (21:56:22 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1315]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            UnderscriptBox["lim", 
              RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
            RowBox[{"g", "(", "x", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1315]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"g", "[", "x", "]"}], 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1316]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1316]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1317]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              UnderscriptBox["lim", 
                RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
              RowBox[{"g", "(", "x", ")"}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              UnderscriptBox["lim", 
                RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
              RowBox[{
                SuperscriptBox["g", "\[Prime]",
                  MultilineFunction->None], "(", "x", ")"}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1317]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"x", " ", 
      RowBox[{"Log", "[", "x", "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1318]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", " ", 
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1318]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1319]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Delta]", "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1319]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1320]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1320]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1321]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1321]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Log", "[", "x", "]"}], 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1322]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1322]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1323]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1323]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
  CellLabel->"In[1324]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1324]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1325]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyDeltaFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1325]=  (21:56:23 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SimplifyGTI  ***unfinished***", "ObjectName",
  CellTags->"SimplifyGTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SimplifyGTI"],

Cell["SimplifyGTI simplifies GTI's.", "Text",
  CellTags->"SimplifyGTI"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GTI",
    ButtonData:>"GTI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GTI"],
  "."
}], "Text",
  CellTags->"SimplifyGTI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SimplifyGTI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[6]:=  (21:30:51 on 9/20/02)",
  CellTags->"SimplifyGTI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SimplifyPolyLog", "ObjectName",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SimplifyPolyLog"],

Cell["\<\
SimplifyPolyLog[y] performs several simplifications assuming that \
the variables occuring in the Log and PolyLog functions are between 0 and 1. \
\
\>", "Text",
  CellTags->"SimplifyPolyLog"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Nielsen",
    ButtonData:>"Nielsen",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Nielsen"],
  "."
}], "Text",
  CellTags->"SimplifyPolyLog"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    RowBox[{
      RowBox[{"sip", "[", "y_", "]"}], " ", ":=", " ", 
      RowBox[{"y", " ", "==", " ", 
        RowBox[{"SimplifyPolyLog", "[", "y", "]"}]}]}]], "Input",
  CellLabel->"In[1326]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "/", "x"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1327]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["1", "x"], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1327]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", "x"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1328]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", "x", ")"}], "==", 
        RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1328]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", 
            RowBox[{"x", "^", "2"}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1329]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", 
            SuperscriptBox["x", "2"]}], ")"}], "==", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"x", "+", "1"}], ")"}]}], "+", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1329]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1330]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          SuperscriptBox["x", "2"], ")"}], "==", 
        RowBox[{
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1330]=  (21:56:23 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1331]:=  (21:56:23 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1331]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"x", "/", 
            RowBox[{"(", 
              RowBox[{"x", "-", "1"}], ")"}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1332]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["x", 
            RowBox[{"x", "-", "1"}]], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1332]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Nielsen", "[", 
        RowBox[{"1", ",", "2", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1333]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", 
            RowBox[{
              FormBox["1",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["2",
                "TraditionalForm"]}]], "(", 
          FormBox[
            RowBox[{"-", 
              FractionBox["x", 
                RowBox[{"1", "-", "x"}]]}],
            "TraditionalForm"], ")"}], "==", 
        RowBox[{
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1333]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"-", 
            RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1334]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"-", 
            FractionBox["1", "x"]}], ")"}], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1334]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1335]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[1335]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1336]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          SuperscriptBox["x", "2"], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
          RowBox[{"4", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "x"}],
                "TraditionalForm"], ")"}]}], "+", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", "x"}], ")"}]}], "+", 
          RowBox[{"4", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1336]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1337]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1337]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", 
            RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1338]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", 
            FractionBox["1", "x"]}], ")"}], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1338]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"4", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1339]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "4"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "24"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "4"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "x"}],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["3",
                  "TraditionalForm"]}]], "(", "x", ")"}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["2",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", "x", ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1339]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Log", "[", 
        RowBox[{"a", "+", 
          RowBox[{"b", "/", "c"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1340]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"a", "+", 
            FractionBox["b", "c"]}], ")"}], "==", 
        RowBox[{"log", "(", 
          FractionBox[
            RowBox[{"b", "+", 
              RowBox[{"a", " ", "c"}]}], "c"], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1340]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Log", "[", 
        RowBox[{"1", "/", "x"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1341]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          FractionBox["1", "x"], ")"}], "==", 
        RowBox[{"-", 
          RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1341]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcTanh", "[", "x", "]"}], "]"}]], "Input",
  CellLabel->"In[1342]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["tanh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"log", "(", 
            RowBox[{"-", 
              FractionBox[
                RowBox[{"x", "+", "1"}], 
                RowBox[{"1", "-", "x"}]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1342]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcSinh", "[", "x", "]"}], "]"}]], "Input",
  CellLabel->"In[1343]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["sinh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{"log", "(", 
          RowBox[{"x", "+", 
            SqrtBox[
              RowBox[{
                SuperscriptBox["x", "2"], "+", "1"}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1343]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcCosh", "[", "x", "]"}], "]"}]], "Input",
  CellLabel->"In[1344]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["cosh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{"log", "(", 
          RowBox[{"x", "+", 
            SqrtBox[
              RowBox[{
                SuperscriptBox["x", "2"], "-", "1"}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1344]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "sip", "]"}]], "Input",
  CellLabel->"In[1345]:=  (21:56:24 on 5/16/01)",
  CellTags->"SimplifyPolyLog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Simplify2  ***unfinished***", "ObjectName",
  CellTags->"Simplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Simplify2"],

Cell["Simplify2 is a special ordering function.", "Text",
  CellTags->"Simplify2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Simplify2"],

Cell[BoxData[""], "Input",
  CellLabel->"In[10]:=  (21:33:08 on 9/20/02)",
  CellTags->"Simplify2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SmallDelta", "ObjectName",
  CellTags->"SmallDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SmallDelta"],

Cell["SmallDelta denotes some small positive number.", "Text",
  CellTags->"SmallDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SmallEpsilon", "ObjectName",
  CellTags->"SmallEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SmallEpsilon"],

Cell["SmallEpsilon denotes some small positive number.", "Text",
  CellTags->"SmallEpsilon"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SmallVariable", "ObjectName",
  CellTags->"SmallVariable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SmallVariable"],

Cell["\<\
SmallVariable[me] is the head of small (negligible) variables. This \
means any mass with this head can be neglected if it appears in a sum, but \
not as an argument of Passarino-Veltman (PaVe) functions or \
PropagatorDenominator.\
\>", "Text",
  CellTags->"SmallVariable"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text",
  CellTags->"SmallVariable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SmallVariables", "ObjectName",
  CellTags->"SmallVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SmallVariables"],

Cell["\<\
SmallVariables is an option for OneLoop. \
\"SmallVariables\[Rule]{Melectron}\" i.e. will substitute \
\"SmallVariable[Melectron]\"  for all Melectron's in the calculation.\
\>", \
"Text",
  CellTags->"SmallVariables"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"SmallVariables"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SMP", "ObjectName",
  CellTags->"SMP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SMP"],

Cell["\<\
SMP[par] substitutes a symbol for the  Standard Model parameter \
par.  SMP[] gives the list of substitutions. par should be a string; e.g., \
SMP[\"SW\"] gives  sw (in the Global` context).\
\>", "Text",
  CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SMP", "[", "]"}], "//", "InputForm"}]], "Input",
  CellLabel->"In[17]:=  (21:57:07 on 9/20/02)",
  CellTags->"SMP"],

Cell["\<\
{\"EL\" :> e, \"CW\" :> cw, \"ME\" :> me, \"MH\" :> mh, \"MW\" :> \
mw, \"SW\" :> sw}\
\>", "Output",
  CellLabel->"Out[17]//InputForm=  (21:57:07 on 9/20/02)",
  CellTags->"SMP"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["SMVertex",
    ButtonData:>"SMVertex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SMVertex"],
  "."
}], "Text",
  CellTags->"SMP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SMP", "[", "\"\<ME\>\"", "]"}]], "Input",
  CellLabel->"In[19]:=  (21:57:37 on 9/20/02)",
  CellTags->"SMP"],

Cell[BoxData[
    FormBox["me", TraditionalForm]], "Output",
  CellLabel->"Out[19]=  (21:57:37 on 9/20/02)",
  CellTags->"SMP"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SMVertex", "ObjectName",
  CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SMVertex"],

Cell["\<\
SMVertex[\"AWW\", p,mu, q,nu, k,rho] gives the photon-W-W vertex \
(p,mu correspond to the photon, q,nu to the (incoming) W+ and k,rho to the \
(incoming) W-. All momenta are flowing into the vertex.  SMVertex[\"HHH\", \
___] give the three-higgs coupling. \
\>", "Text",
  CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SMVertex", "//", "Options"}]], "Input",
  CellLabel->"In[21]:=  (22:00:47 on 9/20/02)",
  CellTags->"SMVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[21]=  (22:00:47 on 9/20/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SMVertex", "[", 
      RowBox[{
      "\"\<AWW\>\"", ",", "p", ",", "\[Mu]", ",", " ", "q", ",", "\[Nu]", ",",
         " ", "k", ",", "\[Rho]"}], "]"}]], "Input",
  CellLabel->"In[22]:=  (22:01:42 on 9/20/02)",
  CellTags->"SMVertex"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", "e", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  FormBox[
                    RowBox[{"k", "-", "q"}],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  FormBox[
                    RowBox[{"k", "-", "p"}],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  FormBox[
                    RowBox[{"p", "-", "q"}],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[22]=  (22:01:42 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Smu  ***unfinished***", "ObjectName",
  CellTags->"Smu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Smu"],

Cell["Smu  is ...", "Text",
  CellTags->"Smu"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Sn", "ObjectName",
  CellTags->"Sn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Sn"],

Cell[TextData[{
  "Sn is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Pi]", 
          RowBox[{"n", "/", "2"}]], TraditionalForm]]],
  "/",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"2", " ", "\[Pi]"}], ")"}], "n"], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"Sn"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SO", "ObjectName",
  CellTags->"SO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SO"],

Cell["\<\
SO[q] is a four-dimensional scalar product of OPEDelta with q. It \
is transformed into Pair[Momentum[q], Momentum[OPEDelta] by FCI.\
\>", "Text",\

  CellTags->"SO"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["OPEDelta",
    ButtonData:>"OPEDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEDelta"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SOD",
    ButtonData:>"SOD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SOD"],
  "."
}], "Text",
  CellTags->"SO"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SO"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SO", "[", "p", "]"}]], "Input",
  CellLabel->"In[1355]:=  (21:56:25 on 5/16/01)",
  CellTags->"SO"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1355]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SO", "[", 
      RowBox[{"p", "-", "q"}], "]"}]], "Input",
  CellLabel->"In[1356]:=  (21:56:25 on 5/16/01)",
  CellTags->"SO"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"(\"\>",
          "TraditionalForm"], 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], 
        FormBox["\<\")\"\>",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1356]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SO", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1357]:=  (21:56:25 on 5/16/01)",
  CellTags->"SO"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1357]//StandardForm=  (21:56:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SOD", "ObjectName",
  CellTags->"SOD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SOD"],

Cell["\<\
SOD[q] is a D-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q,D], Momentum[OPEDelta,D] by \
FeynCalcInternal.\
\>", "Text",
  CellTags->"SOD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEDelta",
    ButtonData:>"OPEDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEDelta"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SOD",
    ButtonData:>"SOD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SOD"],
  "."
}], "Text",
  CellTags->"SOD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SOD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SOD", "[", "p", "]"}]], "Input",
  CellLabel->"In[1346]:=  (21:56:24 on 5/16/01)",
  CellTags->"SOD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1346]=  (21:56:24 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SOD", "[", 
      RowBox[{"p", "-", "q"}], "]"}]], "Input",
  CellLabel->"In[1347]:=  (21:56:25 on 5/16/01)",
  CellTags->"SOD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"(\"\>",
          "TraditionalForm"], 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], 
        FormBox["\<\")\"\>",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1347]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SOD", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1348]:=  (21:56:25 on 5/16/01)",
  CellTags->"SOD"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"OPEDelta", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1348]//StandardForm=  (21:56:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Solve2", "ObjectName",
  CellTags->"Solve2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Solve2"],

Cell["\<\
Solve2 is equivalent to Solve, except that it works only for linear \
equations (and returns just a list) and accepts the options Factoring and \
FinalSubstitutions. Solve2 uses the \"high school algorithm\" and factors \
intermediate results. Therefore it can be drastically more useful than Solve.\
\
\>", "Text",
  CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Solve2", "]"}]], "Input",
  CellLabel->"In[1349]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1349]=  (21:56:25 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1350]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            RowBox[{
              FractionBox["1", "4"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
          RowBox[{"y", "\[Rule]", 
            RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1350]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell["\<\
If no equation sign is given the polynomials are supposed to be \
0.\
\>", "Text",
  CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"x", "+", "y"}], ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1351]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"x", "\[Rule]", 
          RowBox[{"-", "y"}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1351]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"x", "+", "y"}], ",", "x", ",", 
        RowBox[{"FinalSubstitutions", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"y", "\[Rule]", "h"}], "}"}]}]}], "]"}]], "Input",
  CellLabel->"In[1352]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"x", "\[Rule]", 
          RowBox[{"-", "h"}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1352]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}], ",", 
        RowBox[{"Factoring", "\[Rule]", "Expand"}]}], "]"}]], "Input",
  CellLabel->"In[1353]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            RowBox[{
              FractionBox["b", "2"], "-", 
              FractionBox["w", "4"]}]}], ",", 
          RowBox[{"y", "\[Rule]", 
            RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1353]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1354]:=  (21:56:25 on 5/16/01)",
  CellTags->"Solve2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"x", "\[Rule]", 
              RowBox[{
                FractionBox["1", "4"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
            RowBox[{"y", "\[Rule]", 
              RowBox[{"d", "+", "p"}]}]}], "}"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1354]=  (21:56:25 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Solve3", "ObjectName",
  CellTags->"Solve3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Solve3"],

Cell["\<\
Solve3 is equivalent to Solve, except that it works only for linear \
equations (and returns just a list) and uses the \"high school algorithm\" \
and is sometimes better than Solve for systems involving rational polyonials.\
\
\>", "Text",
  CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Solve3", "]"}]], "Input",
  CellLabel->"In[33]:=  (22:07:34 on 9/20/02)",
  CellTags->"Solve3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[33]=  (22:07:34 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["Solve2",
    ButtonData:>"Solve2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Solve2"],
  "."
}], "Text",
  CellTags->"SMP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve3", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[34]:=  (22:07:44 on 9/20/02)",
  CellTags->"Solve3"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            RowBox[{
              FractionBox["1", "4"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
          RowBox[{"y", "\[Rule]", 
            RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[34]=  (22:07:45 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SP", "ObjectName",
  CellTags->"SP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SP"],

Cell[TextData[{
  "SP[a, b] denotes a four-dimensional scalar product. SP[a, b] is \
transformed into ScalarProduct[a, b] by FeynCalcInternal. SP[p] is the same \
as SP[p, p] ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", 
          RowBox[{"=", 
            SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
  ")."
}], "Text",
  CellTags->"SP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  "."
}], "Text",
  CellTags->"SP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
      RowBox[{"SP", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[1358]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], "+", 
        FormBox[
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1358]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", 
        RowBox[{"q", "+", 
          RowBox[{"2", "p"}]}]}], "]"}]], "Input",
  CellLabel->"In[1359]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1359]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"SP", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "+", 
            RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
  CellLabel->"In[1360]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1360]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"SP", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1361]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}], "+", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1361]=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1362]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1362]//StandardForm=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1363]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "a", "]"}], ",", 
        RowBox[{"Momentum", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1363]//StandardForm=  (21:56:25 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SP", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1364]:=  (21:56:25 on 5/16/01)",
  CellTags->"SP"],

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1364]//StandardForm=  (21:56:26 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SPC", "ObjectName",
  CellTags->"SPC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SPC"],

Cell["SPC is an abbreviation for ScalarProductCancel.", "Text",
  CellTags->"SPC"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ScalarProductCancel",
    ButtonData:>"ScalarProductCancel",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProductCancel"],
  "."
}], "Text",
  CellTags->"SPC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SPD", "ObjectName",
  CellTags->"SPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SPD"],

Cell[TextData[{
  "SPD[a, b] denotes a D-dimensional scalar product. SPD[a, b] is transformed \
into ScalarProduct[a, b,Dimension\[Rule]D] by FeynCalcInternal. SPD[p] is the \
same as SPD[p,p] ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", 
          RowBox[{"=", 
            SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
  ")."
}], "Text",
  CellTags->"SPD"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PD",
    ButtonData:>"PD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PD"],
  ", ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ScalarProduct"],
  "."
}], "Text",
  CellTags->"SPD"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SPD"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SPD", "[", 
        RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
      RowBox[{"SPD", "[", "q", "]"}]}]], "Input",
  CellLabel->"In[1365]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], "+", 
        FormBox[
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1365]=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", 
        RowBox[{"q", "+", 
          RowBox[{"2", "p"}]}]}], "]"}]], "Input",
  CellLabel->"In[1366]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", "\[CenterDot]",
         "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1366]=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"SPD", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "+", 
            RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
  CellLabel->"In[1367]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1367]=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"SPD", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1368]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}], "+", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1368]=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SPD", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1369]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1369]//StandardForm=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SPD", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1370]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"a", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1370]//StandardForm=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1371]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1371]//StandardForm=  (21:56:26 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{
            RowBox[{"SP", "[", 
              RowBox[{"p", ",", "q"}], "]"}], ",", " ", "D"}], "]"}], "]"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1372]:=  (21:56:26 on 5/16/01)",
  CellTags->"SPD"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Output",
  CellLabel->"Out[1372]//StandardForm=  (21:56:26 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Spinor", "ObjectName",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Spinor"],

Cell[TextData[{
  "Spinor[p, m, o] is the head of Dirac spinors. Which of the spinors ",
  StyleBox["u",
    FontSlant->"Italic"],
  ", ",
  StyleBox["v",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["u", "_"], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{",", " "}], TraditionalForm]]],
  "or ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["v", "_"], " "}], TraditionalForm]]],
  "is understood, depends on the sign of the momentum (p) argument and the \
relative position of DiracSlash[p]: Spinor[sign p, mass] is that spinor which \
yields: sign*mass*Spinor[p, mass] if the Dirac equation is applied (by \
DiracEquation or DiracSimplify).\nThe optional argument o can be used for \
additional degrees of freedom. If no optional argument o is supplied, a 1 is \
subsituted in.\nSpinors of fermions of mass ",
  StyleBox["m",
    FontSlant->"Italic"],
  " are normalized to have square ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["u", "_"], TraditionalForm]]],
  StyleBox["u",
    FontSlant->"Italic"],
  "=2 ",
  StyleBox["m",
    FontSlant->"Italic"],
  " and  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["v", "_"], " "}], TraditionalForm]]],
  StyleBox["v=",
    FontSlant->"Italic"],
  "-2 ",
  StyleBox["m",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->"Spinor"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Spinor", "[", "p", "]"}]], "Input",
  CellLabel->"In[1373]:=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1373]=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1374]:=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1374]=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]],

Cell["\<\
FeynCalc uses covariant normalization (as opposed to e.g. the \
normalization used in Bjorken&Drell).\
\>", "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Spinor", "[", 
          RowBox[{"p", ",", "m"}], "]"}], ".", 
        RowBox[{"Spinor", "[", 
          RowBox[{"p", ",", "m"}], "]"}]}], "//", "DiracSimplify"}]], "Input",\

  CellLabel->"In[4]:=  (18:38:07 on 9/19/02)",
  CellTags->"Spinor"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "m"}], TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (18:38:07 on 9/19/02)",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"Spinor", "[", 
          RowBox[{
            RowBox[{"-", "p"}], ",", "m"}], "]"}], ".", 
        RowBox[{"DiracSlash", "[", "p", "]"}]}], "]"}]], "Input",
  CellLabel->"In[3]:=  (18:37:43 on 9/19/02)",
  CellTags->"Spinor"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "m"}], " ", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            RowBox[{"-", "p"}],
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (18:37:44 on 9/19/02)",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", "p", "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1376]:=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1376]//StandardForm=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]],

Cell["\<\
By convention, ChangeDimension does not operate on momenta in \
Spinor's (but on e.g.  DiracSlash[Momentum[p]]). \
\>", "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{
          RowBox[{"Spinor", "[", "p", "]"}], ",", "D"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1377]:=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1377]//StandardForm=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1378]:=  (21:56:26 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1378]//StandardForm=  (21:56:27 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]],

Cell["SmallVariable's are discarded by Spinor.", "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{"p", ",", 
          RowBox[{"SmallVariable", "[", "m", "]"}]}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1379]:=  (21:56:27 on 5/16/01)",
  CellTags->"Spinor"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1379]//StandardForm=  (21:56:27 on 5/16/01)",
  CellTags->"Spinor"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinorCollect", "ObjectName",
  CellTags->"SpinorCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinorCollect"],

Cell["\<\
SpinorCollect is an option for FermionSpinSum. If set to False the  \
argument of FermionSpinSum has to be already collected w.r.t. Spinor.\
\>", \
"Text",
  CellTags->"SpinorCollect"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  "."
}], "Text",
  CellTags->"SpinorCollect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinorUBar", "ObjectName",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinorUBar"],

Cell[TextData[{
  "SpinorUBar[p, m] denotes a ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["u", "_"], TraditionalForm]]],
  "-spinor."
}], "Text",
  CellTags->"SpinorUBar"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text",
  CellTags->"SpinorUBar"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SpinorUBar"],

Cell["One argument only assumes a massless spinor.", "Text",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", "p", "]"}]], "Input",
  CellLabel->"In[1380]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorUBar"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"u\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1380]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1381]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorUBar"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"u\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1381]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorUBar", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1382]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorUBar"],

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1382]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorUBar", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1383]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorUBar"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1383]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorUBar", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1384]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorUBar"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1384]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinorU", "ObjectName",
  CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinorU"],

Cell["SpinorU[p, m, optarg] denotes a u-spinor.", "Text",
  CellTags->"SpinorU"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text",
  CellTags->"SpinorU"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorU", "[", "p", "]"}]], "Input",
  CellLabel->"In[1385]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorU"],

Cell[BoxData[
    FormBox[
      RowBox[{"u", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1385]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorU", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1386]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorU"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "u", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ",", 
        "\[NoBreak]", "m", "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1386]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorU", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1387]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorU"],

Cell[BoxData[
    RowBox[{"SpinorU", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1387]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorU", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1388]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorU"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1388]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorU", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[20]:=  (13:11:44 on 9/15/02)",
  CellTags->"SpinorU"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[20]//StandardForm=  (13:11:44 on 9/15/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinorVBar", "ObjectName",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinorVBar"],

Cell[TextData[{
  "SpinorVBar[p, m] denotes a ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["v", "_"], TraditionalForm]]],
  "(p,m)-spinor."
}], "Text",
  CellTags->"SpinorVBar"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorUBar"],
  "."
}], "Text",
  CellTags->"SpinorVBar"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", "p", "]"}]], "Input",
  CellLabel->"In[1390]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorVBar"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"v\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1390]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1391]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorVBar"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"v\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1391]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorVBar", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1392]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorVBar"],

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1392]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorVBar", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1393]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorVBar"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1393]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorVBar", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1394]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorVBar"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1394]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinorV", "ObjectName",
  CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinorV"],

Cell["SpinorV[p, m, optarg] denotes a v-spinor.", "Text",
  CellTags->"SpinorV"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text",
  CellTags->"SpinorV"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorV", "[", "p", "]"}]], "Input",
  CellLabel->"In[1395]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorV"],

Cell[BoxData[
    FormBox[
      RowBox[{"v", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1395]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorV", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1396]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorV"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "v", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ",", 
        "\[NoBreak]", "m", "\[NoBreak]", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1396]=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorV", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1397]:=  (21:56:27 on 5/16/01)",
  CellTags->"SpinorV"],

Cell[BoxData[
    RowBox[{"SpinorV", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output",
  CellLabel->"Out[1397]//StandardForm=  (21:56:27 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorV", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1398]:=  (21:56:28 on 5/16/01)",
  CellTags->"SpinorV"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1398]//StandardForm=  (21:56:28 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorV", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1399]:=  (21:56:28 on 5/16/01)",
  CellTags->"SpinorV"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output",
  CellLabel->"Out[1399]//StandardForm=  (21:56:28 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SpinPolarizationSum", "ObjectName",
  CellTags->"SpinPolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SpinPolarizationSum"],

Cell["\<\
SpinPolarizationSum is an option for FermionSpinSum. The set (pure) \
function acts on the usual spin sum.\
\>", "Text",
  CellTags->"SpinPolarizationSum"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  "."
}], "Text",
  CellTags->"SpinPolarizationSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SPL", "ObjectName",
  CellTags->"SPL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SPL"],

Cell["SPL is an abbreviation for SimplifyPolyLog.", "Text",
  CellTags->"SPL"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text",
  CellTags->"SPL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SplittingFunction", "ObjectName",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SplittingFunction"],

Cell[TextData[{
  "SplittingFunction[pxy] is a database of splitting functions in the ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["MS", "_"], TraditionalForm]]],
  " scheme."
}], "Text",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SplittingFunction", "]"}]], "Input",
  CellLabel->"In[1400]:=  (21:56:28 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Polarization", "\[Rule]", "1"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1400]=  (21:56:28 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AnomalousDimension",
    ButtonData:>"AnomalousDimension",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AnomalousDimension"],
  "."
}], "Text",
  CellTags->"SplittingFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SplittingFunction"],

Cell["Unpolarized case:", "Text",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SplittingFunction", ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1401]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell["\<\
In general the argument should be a string, but if the variables \
Pqq, etc. have no value, you can omit the \"\".\
\>", "Text",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
  CellLabel->"In[1402]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{"6", " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{"8", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1402]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
  CellLabel->"In[1403]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"16", " ", 
              SuperscriptBox["x", "2"]}], "-", 
            RowBox[{"16", " ", "x"}], "+", "8"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1403]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
  CellLabel->"In[1404]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"4", " ", "x"}], "-", "8", "+", 
            FractionBox["8", "x"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1404]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
  CellLabel->"In[1405]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"8", " ", 
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["x", "2"]}], "+", "x", "+", 
              RowBox[{
                FractionBox["11", "12"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"], "-", "2", "+", 
              FractionBox["1", "x"]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["8", "3"], " ", 
          SubscriptBox["N", "f"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1405]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
  CellLabel->"In[1406]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"7", "-", 
                  RowBox[{"4", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            
            RowBox[{"4", " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{"log", "(", "x", ")"}], 
                    RowBox[{"1", "-", "x"}]], "+", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox[
                        RowBox[{"log", "(", 
                          RowBox[{"1", "-", "x"}], ")"}], 
                        RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "-",
             "4"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1406]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
  CellLabel->"In[1407]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "2"}], " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "2"}], " ", "x"}], "+", "4", "-", 
                  FractionBox["4", "x"]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
            "2", "-", 
            FractionBox["4", "x"]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1407]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
  CellLabel->"In[1408]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "8"}], " ", 
                    SuperscriptBox["x", "2"]}], "+", 
                  RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1408]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
  CellLabel->"In[1409]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell["Polarized case:", "Text",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SplittingFunction", ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1410]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
  CellLabel->"In[1411]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{"6", " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{"8", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1411]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
  CellLabel->"In[1412]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"16", " ", "x"}], "-", "8"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1412]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
  CellLabel->"In[1413]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{"8", "-", 
            RowBox[{"4", " ", "x"}]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1413]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
  CellLabel->"In[1414]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "16"}], " ", "x"}], "+", 
              RowBox[{
                FractionBox["22", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{"8", " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", "8"}], 
            ")"}]}], "-", 
        RowBox[{
          FractionBox["8", "3"], " ", 
          SubscriptBox["N", "f"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1414]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
  CellLabel->"In[1415]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"8", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}], "+", 
            RowBox[{"2", " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"7", "-", 
                  RowBox[{"4", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            
            RowBox[{"4", " ", 
              RowBox[{"log", "(", "x", ")"}], " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
            RowBox[{"4", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox[
                    RowBox[{"log", "(", 
                      RowBox[{"1", "-", "x"}], ")"}], 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1415]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
  CellLabel->"In[1416]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
            "2"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1416]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agqd", "]"}]], "Input",
  CellLabel->"In[1417]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "2"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1417]=  (21:56:29 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
  CellLabel->"In[1418]:=  (21:56:29 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"4", "-", 
                  RowBox[{"8", " ", "x"}]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1418]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqgd", "]"}]], "Input",
  CellLabel->"In[1419]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"4", "-", 
                  RowBox[{"8", " ", "x"}]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1419]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
  CellLabel->"In[1420]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["67", "9"], "-", 
                    RowBox[{"4", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
              "-", 
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"log", "(", "x", ")"}], 
                      RowBox[{"1", "-", "x"}]], "+", 
                    SubscriptBox[
                      RowBox[{"(", 
                        FractionBox[
                          RowBox[{"log", "(", 
                            RowBox[{"1", "-", "x"}], ")"}], 
                          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], 
              "+", "2"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["20", "9"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1420]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aggd", "]"}]], "Input",
  CellLabel->"In[1421]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["67", "9"], "-", 
                    RowBox[{"4", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
              "-", 
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"log", "(", "x", ")"}], 
                      RowBox[{"1", "-", "x"}]], "+", 
                    SubscriptBox[
                      RowBox[{"(", 
                        FractionBox[
                          RowBox[{"log", "(", 
                            RowBox[{"1", "-", "x"}], ")"}], 
                          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], 
              "+", "2"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["20", "9"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1421]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQQS", "]"}]], "Input",
  CellLabel->"In[1422]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "16"}], " ", 
              RowBox[{"(", 
                RowBox[{"x", "+", "1"}], ")"}], " ", 
              RowBox[{
                SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"48", " ", "x"}], "-", "16"}], ")"}], " ", 
              RowBox[{"log", "(", "x", ")"}]}], "+", 
            RowBox[{"16", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1422]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]], "Input",
  CellLabel->"In[1423]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "x"}], "+", 
                    FractionBox["3", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{"40", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "24"}], " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                    RowBox[{"48", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", "3"}], 
                  ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"88", " ", "x"}], "9"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"16", " ", 
                          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
                    FractionBox["2", "3"]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
              RowBox[{
                FractionBox["80", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["8", "9"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{"log", "(", 
                          RowBox[{"x", "+", "1"}], ")"}], " ", 
                        RowBox[{"log", "(", "x", ")"}]}], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{
                          SubscriptBox["Li", "2"], "(", 
                          RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
                RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{
                FractionBox["4", "3"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"5", " ", "x"}], "-", 
                    FractionBox["22", 
                      RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                FractionBox["4", "9"], " ", 
                RowBox[{"(", 
                  RowBox[{"53", "-", 
                    RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["536", "9"], "-", 
                    RowBox[{"16", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"88", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
                    RowBox[{"24", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", 
                    FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1423]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQG", "]"}]], "Input",
  CellLabel->"In[1424]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"16", "-", 
                    RowBox[{"16", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"8", "-", 
                    RowBox[{"16", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "x"}], "-", "2"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{"54", " ", "x"}], "-", 
              RowBox[{"16", " ", "x", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
              RowBox[{"18", " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", "44"}], ")"}]}], "+", 
        RowBox[{"4", " ", 
          SubscriptBox["C", "A"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"4", "-", 
                    RowBox[{"8", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "16"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "8"}], " ", "x"}], "-", "4"}], ")"}], " ", 
                
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"44", " ", "x"}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"32", " ", "x"}], "+", "4"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], " ",
                 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], " ",
                 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", "48"}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1424]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PGQ", "]"}]], "Input",
  CellLabel->"In[1425]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "A"], " ", 
              SubscriptBox["C", "F"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"16", "-", 
                    RowBox[{"8", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"8", " ", "x"}], "3"], "+", 
                    FractionBox["80", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "32"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "+", "16"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{"280", " ", "x"}], "9"], "+", 
              RowBox[{"16", " ", "x", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"32", "-", 
                    RowBox[{"104", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", 
              FractionBox["328", "9"]}], ")"}]}], "+", 
        RowBox[{
          SubsuperscriptBox["C", "F", "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "16"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "8"}], " ", "x"}], "-", "16"}], ")"}], " ",
                 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"8", "-", 
                    RowBox[{"4", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{"32", " ", "x"}], "-", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"32", " ", "x"}], "+", "64"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"36", " ", "x"}], "+", "48"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", "68"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], " ", 
              SubscriptBox["T", "f"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"32", " ", "x"}], "9"]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"32", " ", "x"}], "3"], "-", 
                    FractionBox["64", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              FractionBox["128", "9"]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1425]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PGG", "]"}]], "Input",
  CellLabel->"In[1426]:=  (21:56:30 on 5/16/01)",
  CellTags->"SplittingFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox["8", 
                        RowBox[{"x", "+", "1"}]]}], "+", "32", "+", 
                    FractionBox["8", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["232", "3"], "-", 
                    FractionBox[
                      RowBox[{"536", " ", "x"}], "3"]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "-", 
                    FractionBox["32", 
                      RowBox[{"1", "-", "x"}]], "-", "32"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "+", 
                    FractionBox["32", 
                      RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"388", " ", "x"}], "9"], "+", 
              RowBox[{
                FractionBox["64", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "-", 
                    RowBox[{"16", " ", 
                      SubscriptBox[
                        RowBox[{"(", 
                          FractionBox["1", 
                            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
                    FractionBox["16", 
                      RowBox[{"x", "+", "1"}]]}], ")"}]}], "+", 
              RowBox[{
                FractionBox["536", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "+", 
                    FractionBox["32", 
                      RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", 
              RowBox[{"24", " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}], "-", 
              FractionBox["148", "9"]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"608", " ", "x"}], "9"], "-", 
              RowBox[{
                FractionBox["32", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"32", " ", "x"}], "3"]}], "-", 
                    FractionBox["32", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              RowBox[{
                FractionBox["160", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["448", "9"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        RowBox[{
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "16"}], ")"}], 
                " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "80"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{"80", " ", "x"}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", "80"}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1426]=  (21:56:30 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["StandardMatrixElement", "ObjectName",
  CellTags->"StandardMatrixElement"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"StandardMatrixElement"],

Cell["\<\
StandardMatrixElement[ ... ] is the head for matrix element \
abbreviations.\
\>", "Text",
  CellTags->"StandardMatrixElement"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"StandardMatrixElement"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SubContext", "ObjectName",
  CellTags->"SubContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SubContext"],

Cell["\<\
SubContext[fun] gives the sub-directory (context) in \
HighEnergyPhysics.\
\>", "Text",
  CellTags->"SubContext"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc",
    ButtonData:>"FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc"],
  ", ",
  ButtonBox["Load",
    ButtonData:>"Load",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Load"],
  "."
}], "Text",
  CellTags->"SubContext"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SubLoop", "ObjectName",
  CellTags->"SubLoop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SubLoop"],

Cell["\<\
SubLoop is an option for OPE1Loop. If set to True, sub 1-loop \
tensorintegral decomposition is performed.\
\>", "Text",
  CellTags->"SubLoop"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPE1Loop",
    ButtonData:>"OPE1Loop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPE1Loop"],
  "."
}], "Text",
  CellTags->"SubLoop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SumP", "ObjectName",
  CellTags->"SumP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SumP"],

Cell[TextData[{
  "SumP[k, m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["2", 
            RowBox[{
              StyleBox["k",
                FontSlant->"Plain"], "-", "1"}]], 
          RowBox[{
            SubsuperscriptBox["\[Sum]", 
              RowBox[{
                StyleBox["i",
                  FontSlant->"Plain"], "=", "1"}], 
              RowBox[{"2", 
                StyleBox["m",
                  FontSlant->"Plain"]}]], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "+", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], 
                    StyleBox["i",
                      FontSlant->"Plain"]]}], ")"}], "/", 
              SuperscriptBox[
                StyleBox["i",
                  FontSlant->"Plain"], 
                StyleBox["k",
                  FontSlant->"Plain"]]}]}]}], TraditionalForm]]]
}], "Text",
  CellTags->"SumP"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumT"],
  "."
}], "Text",
  CellTags->"SumP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SumP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1427]:=  (21:56:30 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1427]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1428]:=  (21:56:30 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1428]=  (21:56:30 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1429]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["m",
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1429]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1430]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output",
  CellLabel->"Out[1430]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "8"], 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", 
        "i"}]}]], "Input",
  CellLabel->"In[1431]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output",
  CellLabel->"Out[1431]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", 
      RowBox[{"SumP", "[", 
        RowBox[{"1", ",", 
          RowBox[{"n", "/", "2"}]}], "]"}], "]"}]], "Input",
  CellLabel->"In[1432]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              FractionBox[
                RowBox[{"n", "-", "1"}], "2"],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              FractionBox["n", "2"],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1432]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"n", "\[Rule]", "8"}]}]], "Input",
  CellLabel->"In[1433]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumP"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output",
  CellLabel->"Out[1433]=  (21:56:31 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SumS", "ObjectName",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SumS"],

Cell[TextData[{
  "SumS[1, m] is the harmonic number ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{
              StyleBox["i",
                FontSlant->"Italic"], "=", "1"}], 
            StyleBox["m",
              FontSlant->"Italic"]], 
          StyleBox[" ",
            FontSlant->"Italic"], 
          RowBox[{
            StyleBox["i",
              FontSlant->"Italic"], "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"=", 
              RowBox[{
                SubscriptBox["S", "1"], "(", "m", ")"}]}], " ", ")"}], "."}], 
        TraditionalForm]]],
  " SumS[1,1,m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{
              StyleBox["i",
                FontSlant->"Italic"], "=", "1"}], 
            StyleBox["m",
              FontSlant->"Italic"]], 
          SubscriptBox["S", "1"]}], TraditionalForm]]],
  "(",
  StyleBox["i",
    FontSlant->"Italic"],
  ")/",
  StyleBox["i",
    FontSlant->"Italic"],
  ". SumS[k,l,m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", "l"], "(", "i", ")"}], "/", 
            SuperscriptBox["i", "k"]}]}], TraditionalForm]]],
  ". SumS[r, n] represents Sum[Sign[r]^i/i^Abs[r], {i, 1, n}]. SumS[r,s, n] \
is Sum[Sign[r]^k/k^Abs[r] Sign[s]^j/j^Abs[s], {k, 1, n}, {j, 1, k}], etc."
}], "Text",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SumS", "]"}]], "Input",
  CellLabel->"In[1434]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Reduce", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1434]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumP",
    ButtonData:>"SumP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumP"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumT"],
  "."
}], "Text",
  CellTags->"SumS"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1435]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1435]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1436]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1436]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1437]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", 
          RowBox[{"-", "1"}]], "(", "m", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1437]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "m", ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1438]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", "m"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1438]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
        RowBox[{"m", "+", "2"}], ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1439]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "+", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "2"}], ")"}], "3"]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1439]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SumS", ",", 
          RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1440]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
        RowBox[{"m", "+", "2"}]}], "]"}]], "Input",
  CellLabel->"In[1441]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", 
          SuperscriptBox["m", "3"]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "2"}], ")"}], "3"]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1441]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SumS", ",", 
          RowBox[{"Reduce", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1442]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1443]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output",
  CellLabel->"Out[1443]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "4"], 
      RowBox[{"1", "/", "i"}]}]], "Input",
  CellLabel->"In[1444]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output",
  CellLabel->"Out[1444]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1445]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "12"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1445]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", "1", ",", "11"}], "]"}]], "Input",
  CellLabel->"In[1446]:=  (21:56:31 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      FractionBox["31276937512951", "4260000729600"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1446]=  (21:56:31 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1447]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1447]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1448]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumS"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1448]=  (21:56:32 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SumT", "ObjectName",
  CellTags->"SumT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SumT"],

Cell[TextData[{
  "SumT[1, m] is the alternative harmonic number ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], " ", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", 
            RowBox[{"i", " "}]}]}], TraditionalForm]]],
  " SumT[r, n] represents Sum[(-1)^i/i^r, {i,1,n}], SumT[r,s, n] is Sum[1/k^r \
(-1)^j/j^s, {k, 1, n}, {j, 1, k}]."
}], "Text",
  CellTags->"SumT"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumP",
    ButtonData:>"SumP",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumP"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SumS"],
  "."
}], "Text",
  CellTags->"SumT"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SumT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1449]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "1"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1449]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1450]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1450]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "m"}], "]"}]], "Input",
  CellLabel->"In[1451]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "1"], "(", "m", ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1451]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "m", ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1452]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["S", "~"], "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], "m"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1452]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1453]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1453]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "4"], 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
  CellLabel->"In[1454]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1454]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
  CellLabel->"In[1455]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1455]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", "42"}], "]"}]], "Input",
  CellLabel->"In[1456]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["38987958697055013360489864298703621429610152138683927", 
          "10512121660702378405316004964483761080879190528000000"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1456]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1457]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1457]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
  CellLabel->"In[1458]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1458]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], 
        RowBox[{"m", "-", "1"}]], 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
  CellLabel->"In[1459]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  TagBox["\[Psi]",
                    PolyGamma], 
                  RowBox[{"(", "0", ")"}]], "(", 
                FractionBox["m", "2"], ")"}], "-", 
              RowBox[{
                SuperscriptBox[
                  TagBox["\[Psi]",
                    PolyGamma], 
                  RowBox[{"(", "0", ")"}]], "(", 
                FractionBox[
                  RowBox[{"m", "+", "1"}], "2"], ")"}]}], ")"}]}], "-", 
        FractionBox[
          RowBox[{"log", "(", "4", ")"}], "2"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1459]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", "12"}], "]"}]], "Input",
  CellLabel->"In[1460]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["57561743656913", "21300003648000"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1460]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
        RowBox[{"-", "2"}], ",", "42"}], "]"}]], "Input",
  CellLabel->"In[1461]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["38987958697055013360489864298703621429610152138683927", 
          "10512121660702378405316004964483761080879190528000000"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1461]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Array", "[", 
      RowBox[{"SumT", ",", "6"}], "]"}]], "Input",
  CellLabel->"In[1462]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", 
            FractionBox["5", "8"]}], ",", 
          RowBox[{"-", 
            FractionBox["179", "216"]}], ",", 
          RowBox[{"-", 
            FractionBox["1207", "1728"]}], ",", 
          RowBox[{"-", 
            FractionBox["170603", "216000"]}], ",", 
          RowBox[{"-", 
            FractionBox["155903", "216000"]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1462]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Array", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"SumS", "[", 
            RowBox[{
              RowBox[{"-", "2"}], ",", "1", ",", "#1"}], "]"}], "&"}], ",", 
        "6"}], "]"}]], "Input",
  CellLabel->"In[1463]:=  (21:56:32 on 5/16/01)",
  CellTags->"SumT"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", 
            FractionBox["5", "8"]}], ",", 
          RowBox[{"-", 
            FractionBox["179", "216"]}], ",", 
          RowBox[{"-", 
            FractionBox["1207", "1728"]}], ",", 
          RowBox[{"-", 
            FractionBox["170603", "216000"]}], ",", 
          RowBox[{"-", 
            FractionBox["155903", "216000"]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1463]=  (21:56:32 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUND", "ObjectName",
  CellTags->"SUND"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUND"],

Cell[TextData[{
  "SUND[a,b,c] are the symmetric SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["d", "abc"], "."}], TraditionalForm]]]
}], "Text",
  CellTags->"SUND"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"SUND"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUND"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[1474]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      SubscriptBox["d", 
        RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1474]=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"tt", "=", 
      RowBox[{"SUND", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[1475]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                SubscriptBox["T", 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], "\[NoBreak]", ")"}]}], "+", 
        RowBox[{"2", " ", 
          RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                SubscriptBox["T", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], "\[NoBreak]", ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1475]=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"c", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1476]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      SubscriptBox["d", 
        RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1476]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1477]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1477]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1478]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
          RowBox[{"4", "-", 
            SubsuperscriptBox["C", "A", "2"]}], ")"}], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1478]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "Factor2"}]], "Input",
  CellLabel->"In[1479]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["N", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SuperscriptBox["N", "2"]}], ")"}]}], "N"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1479]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "Factor2"}]], "Input",
  CellLabel->"In[1480]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"4", "-", 
                SuperscriptBox["N", "2"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], "N"]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1480]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUND", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1481]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
  CellLabel->"Out[1481]//StandardForm=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1482]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1482]//StandardForm=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUND", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
        "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1483]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
  CellLabel->"Out[1483]//StandardForm=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", "tt", "]"}]], "Input",
  CellLabel->"In[1484]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUND"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNDelta", "ObjectName",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNDelta"],

Cell[TextData[{
  "SUNDelta[a, b] is the Kronecker-delta for SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") with color indices a and b."
}], "Text",
  CellTags->"SUNDelta"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitSUNIndex",
    ButtonData:>"ExplicitSUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExplicitSUNIndex"],
  ", ",
  ButtonBox["SD",
    ButtonData:>"SD",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SD"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  "."
}], "Text",
  CellTags->"SUNDelta"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1464]:=  (21:56:32 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1464]=  (21:56:32 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", " ", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"b", ",", "c"}], "]"}]}], " ", "]"}]], "Input",
  CellLabel->"In[1465]:=  (21:56:32 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1465]=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1466]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1466]=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNDelta", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1467]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1467]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1468]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1468]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNDelta", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1469]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output",
  CellLabel->"Out[1469]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SD", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1470]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1470]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell["\<\
Symbolic arguments to SUNDelta are transformed into the data type \
SUNIndex and integer arguments are transformed to ExplicitSUNIndex. The \
difference is that SUNSimplify will only sum over symbolic indices.\
\>", \
"Text",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "2"}], "]"}], 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"c", ",", "2"}], "]"}]}], "//", "SUNSimplify"}]], "Input",
  CellLabel->"In[1471]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox["2",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox["2",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1471]=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1472]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], " ", 
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[1472]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"1", ",", "2"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1473]:=  (21:56:33 on 5/16/01)",
  CellTags->"SUNDelta"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
        RowBox[{"ExplicitSUNIndex", "[", "2", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1473]//StandardForm=  (21:56:33 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNDeltaContract", "ObjectName",
  CellTags->"SUNDeltaContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNDeltaContract"],

Cell[TextData[{
  "SUNDeltaContract[expr] substitues for all SUNDelta in expr \
SUNDeltaContract, contracts the SUN(N) indices and resubstitutes SUNDelta. \
SUNDeltaContract[i, j] is the Kronecker-delta for SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") with contraction properties. SUNDeltaContract wraps also the head \
SUNIndex around its arguments."
}], "Text",
  CellTags->"SUNDeltaContract"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  "."
}], "Text",
  CellTags->"SUNDeltaContract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNF", "ObjectName",
  CellTags->"SUNF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNF"],

Cell[TextData[{
  "SUNF[a,b,c] are the structure constants of SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  "). The arguments a,b,c should be of symbolic type."
}], "Text",
  CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNF", "]"}]], "Input",
  CellLabel->"In[1485]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1485]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUND",
    ButtonData:>"SUND",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUND"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["SUNT",
    ButtonData:>"SUNT",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNT"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  "."
}], "Text",
  CellTags->"SUNF"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "x"}], "+", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"b", ",", "a", ",", "c"}], "]"}], "y"}]}]}]], "Input",
  CellLabel->"In[1486]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], "+", 
        RowBox[{"y", " ", 
          SubscriptBox["f", 
            RowBox[{"b", "\[NoBreak]", "a", "\[NoBreak]", "c"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1486]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1487]:=  (21:56:34 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"y", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1487]=  (21:56:34 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1488]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1488]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "a", ",", "b"}], "]"}]], "Input",
  CellLabel->"In[1489]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{"a", "\[NoBreak]", "a", "\[NoBreak]", "b"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1489]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "a", ",", "b"}], "]"}], "//", "Calc"}]], "Input",
  CellLabel->"In[1490]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1490]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This is a consequence of the usual choice for the normalization of the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["T", "a"], " ", "generators", "in", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"see", " ", " ", "SUNT"}], ")"}], "."}]}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1491]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1491]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1492]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1492]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1493]:=  (21:56:35 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1493]=  (21:56:35 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1494]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
  CellLabel->"Out[1494]//StandardForm=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1495]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1495]//StandardForm=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
        "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1496]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
  CellLabel->"Out[1496]//StandardForm=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Input",
  CellLabel->"In[1497]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{"b", "\[NoBreak]", "a", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1497]=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"b", ",", "a", ",", "c"}], "]"}], "//", "FCI"}]], "Input",
  CellLabel->"In[1498]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNF"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1498]=  (21:56:36 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNFJacobi", "ObjectName",
  CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNFJacobi"],

Cell["\<\
SUNFJacobi is an option for SUNSimplify, indicating whether the \
Jacobi identity should be used.\
\>", "Text",
  CellTags->"SUNFJacobi"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"SUNFJacobi"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
        RowBox[{"SUNF", "[", 
          RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"SUNSimplify", "[", 
          RowBox[{"#", ",", 
            RowBox[{"SUNFJacobi", "\[Rule]", "False"}]}], "]"}], 
        "&"}]}]], "Input",
  CellLabel->"In[83]:=  (19:09:26 on 9/3/02)",
  CellTags->"SUNFJacobi"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["f",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[83]=  (19:09:26 on 9/3/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
        RowBox[{"SUNF", "[", 
          RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"SUNSimplify", "[", 
          RowBox[{"#", ",", 
            RowBox[{"SUNFJacobi", "\[Rule]", "True"}]}], "]"}], 
        "&"}]}]], "Input",
  CellLabel->"In[84]:=  (19:09:28 on 9/3/02)",
  CellTags->"SUNFJacobi"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["f",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["f",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[84]=  (19:09:28 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNIndex", "ObjectName",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNIndex"],

Cell[TextData[{
  "SUNIndex is the head of SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") indices."
}], "Text",
  CellTags->"SUNIndex"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitSUNIndex",
    ButtonData:>"ExplicitSUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ExplicitSUNIndex"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  "."
}], "Text",
  CellTags->"SUNIndex"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNIndex", "[", "i", "]"}]], "Input",
  CellLabel->"In[1499]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNIndex"],

Cell[BoxData[
    FormBox["i", TraditionalForm]], "Output",
  CellLabel->"Out[1499]=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1500]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNIndex"],

Cell[BoxData[
    RowBox[{"SUNIndex", "[", "i", "]"}]], "Output",
  CellLabel->"Out[1500]//StandardForm=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1501]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNIndex"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "i", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "j", "]"}]}], "]"}]], "Output",
  CellLabel->"Out[1501]//StandardForm=  (21:56:36 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNIndexRename", "ObjectName",
  CellTags->"SUNIndexRename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNIndexRename"],

Cell["\<\
SUNIndexRename is an option of SUNSimplify. If set to False, no \
automatic renaming of dummy indices is done.\
\>", "Text",
  CellTags->"SUNIndexRename"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  "."
}], "Text",
  CellTags->"SUNIndexRename"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNN", "ObjectName",
  CellTags->"SUNN"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNN"],

Cell[TextData[{
  "SUNN denotes the number of colors. Trick[SUNDelta[a, a]] yields (",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["SUNN", "2"], TraditionalForm]]],
  " -1)."
}], "Text",
  CellTags->"SUNN"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNIndex"],
  ", ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CF"],
  "."
}], "Text",
  CellTags->"SUNN"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNN"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
            RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[6]:=  (20:52:23 on 9/3/02)",
  CellTags->"SUNN"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["N", "2"], "-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[6]=  (20:52:23 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNNToCACF", "ObjectName",
  CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNNToCACF"],

Cell["\<\
SUNNToCACF is an option of SUNSimplify. If set to True, the Casimir \
operator eigenvalues CA (=N) and CF (=(N^2-1)/(2 N)) are introduced.\
\>", \
"Text",
  CellTags->"SUNNToCACF"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  ", ",
  ButtonBox["SUNN",
    ButtonData:>"SUNN",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNN"],
  ", ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CF"],
  "."
}], "Text",
  CellTags->"SUNNToCACF"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
            RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[8]:=  (20:54:26 on 9/3/02)",
  CellTags->"SUNNToCACF"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
  CellLabel->"Out[8]=  (20:54:27 on 9/3/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNSimplify", "ObjectName",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNSimplify"],

Cell["\<\
SUNSimplify simplifies products of SUNT (and complex conjugated) \
matrices. Basic renaming of dummy indices is done. If the option SUNTrace is \
set to False, then any SUNT-matrices are taken out of DiracTrace[...]; \
otherwise a color-trace is taken (by SUNTrace) before taking the SUN-objects \
in front of DiracTrace[...].\
\>", "Text",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNSimplify", "]"}]], "Input",
  CellLabel->"In[1502]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Expanding", "\[Rule]", "False"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"SUNIndexRename", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNFJacobi", "\[Rule]", "False"}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1502]=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Trick"],
  "."
}], "Text",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"b", ",", "c"}], "]"}]}]}]], "Input",
  CellLabel->"In[1503]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[Delta]", 
          RowBox[{"a", "\[NoBreak]", "b"}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{"b", "\[NoBreak]", "c"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1503]=  (21:56:36 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
  CellLabel->"In[1504]:=  (21:56:36 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1504]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"SUNT", "[", "a", "]"}]}]}]], "Input",
  CellLabel->"In[1505]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1505]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t2", "]"}]], "Input",
  CellLabel->"In[1506]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output",
  CellLabel->"Out[1506]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1507]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["N", "2"], "-", "1"}], 
        RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1507]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "r", ",", "s"}], "]"}], 
        RowBox[{"SUNF", "[", 
          RowBox[{"b", ",", "r", ",", "s"}], "]"}]}]}]], "Input",
  CellLabel->"In[1508]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "r", "\[NoBreak]", "s"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"b", "\[NoBreak]", "r", "\[NoBreak]", "s"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1508]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t3", "]"}]], "Input",
  CellLabel->"In[1509]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1509]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", ".", " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
  CellLabel->"In[1510]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], ".", 
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1510]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t4", "]"}]], "Input",
  CellLabel->"In[1511]:=  (21:56:37 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1511]=  (21:56:37 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
  CellLabel->"In[1512]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"d", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1512]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t5", "]"}]], "Input",
  CellLabel->"In[1513]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1513]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
  CellLabel->"In[1514]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["d", 
          RowBox[{"b", "\[NoBreak]", "c", "\[NoBreak]", "d"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1514]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t6", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1515]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1515]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "Factor2"}]], "Input",
  CellLabel->"In[1516]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["N", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SuperscriptBox["N", "2"]}], ")"}]}], "N"], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1516]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "FullSimplify"}]], "Input",
  CellLabel->"In[1517]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["N", "2"], "-", "4"}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "N"], TraditionalForm]], "Output",
  CellLabel->"Out[1517]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1518]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1518]=  (21:56:38 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1519]:=  (21:56:38 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "+", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1519]=  (21:56:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNT", "[", 
          RowBox[{"c", ",", "b", ",", "a"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1520]:=  (21:56:39 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1520]=  (21:56:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t7", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "e"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"c", ",", "d", ",", "e"}], "]"}]}], "+", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "z"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"c", ",", "d", ",", "z"}], "]"}]}]}]}]], "Input",
  CellLabel->"In[1521]:=  (21:56:39 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "e"}]], " ", 
          SubscriptBox["f", 
            RowBox[{"c", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}], "+", 
        RowBox[{
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "z"}]], " ", 
          SubscriptBox["f", 
            RowBox[{"c", "\[NoBreak]", "d", "\[NoBreak]", "z"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1521]=  (21:56:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t7", ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1522]:=  (21:56:39 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1522]=  (21:56:39 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t7", ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
        RowBox[{"SUNIndexRename", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1523]:=  (21:56:39 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["z",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["z",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1523]=  (21:56:39 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"1", "-", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"i", ",", "i"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1528]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t8", "=", 
      RowBox[{"DiracTrace", "[", 
        RowBox[{
          RowBox[{"f", "[", 
            RowBox[{"SUNIndex", "[", "a", "]"}], "]"}], 
          RowBox[{
            RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
            RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}], "]"}]}]], "Input",\

  CellLabel->"In[11]:=",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], " ", 
            RowBox[{"f", "(", 
              FormBox["a",
                "TraditionalForm"], ")"}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[11]=",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t8", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[13]:=",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], " ", 
            RowBox[{"f", "(", 
              FormBox["a",
                "TraditionalForm"], ")"}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[13]=",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t8", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[12]:=",
  CellTags->"SUNSimplify"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"f", "(", 
          FormBox["a",
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[12]=",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6", ",", "t7", 
        ",", "t8"}], "]"}]], "Input",
  CellLabel->"In[1530]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNT", "ObjectName",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNT"],

Cell[TextData[{
  "SUNT[a] is the SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "a"], TraditionalForm]]],
  " generator in the fundamental representation."
}], "Text",
  CellTags->"SUNT"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CF"],
  ", ",
  ButtonBox["SUND",
    ButtonData:>"SUND",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUND"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text",
  CellTags->"SUNT"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNT", "[", "a", "]"}]], "Input",
  CellLabel->"In[1531]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      SubscriptBox["T", 
        FormBox["a",
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1531]=  (21:56:40 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "Since ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "a"], TraditionalForm]]],
  " is a noncommutative object, products have to separated by a dot (.)."
}], "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNT", "[", "a", "]"}], ".", " ", 
      RowBox[{"SUNT", "[", "b", "]"}], ".", " ", 
      RowBox[{"SUNT", "[", "c", "]"}]}]], "Input",
  CellLabel->"In[1532]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["b",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["c",
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1532]=  (21:56:40 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNT", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
  CellLabel->"In[1533]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["b",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["c",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["d",
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1533]=  (21:56:40 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1534]:=  (21:56:40 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          SubscriptBox["T", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1534]=  (21:56:40 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1535]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["C", "F", "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1535]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1536]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox["C", "A"], "-", 
            RowBox[{"2", " ", 
              SubscriptBox["C", "F"]}]}], ")"}], " ", 
        SubscriptBox["T", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1536]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1537]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          SubscriptBox["T", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1537]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "The normalizaton of the generators is chosen in the standard way, \
therefore ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"tr", "(", 
            RowBox[{
              SubscriptBox["T", "a"], 
              SubscriptBox["T", "b"]}], ")"}], " ", "=", " ", 
          RowBox[{
            RowBox[{"1", "/", "2"}], " ", 
            RowBox[{
              SubscriptBox["\[Delta]", "ab"], "."}]}]}], TraditionalForm]]]
}], "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1538]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1538]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "In case you want ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "f"], TraditionalForm]]],
  ", you need to include a factor 2Tf inside the trace."
}], "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"2", " ", "Tf", " ", 
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1539]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1539]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNTrace", "[", 
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1540]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[1540]//StandardForm=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1541]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    RowBox[{"SUNT", "[", 
      RowBox[{"SUNIndex", "[", "a", "]"}], "]"}]], "Output",
  CellLabel->"Out[1541]//StandardForm=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", "FCE"}], "//",
       "StandardForm"}]], "Input",
  CellLabel->"In[1542]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNT"],

Cell[BoxData[
    RowBox[{"SUNT", "[", "a", "]"}]], "Output",
  CellLabel->"Out[1542]//StandardForm=  (21:56:41 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SUNTrace", "ObjectName",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SUNTrace"],

Cell["SUNTrace[expr] calculates the color-trace.", "Text",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNTrace", "]"}]], "Input",
  CellLabel->"In[1543]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1543]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"SUNTrace"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1544]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], "2"], TraditionalForm]], "Output",
  CellLabel->"Out[1544]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1545]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1545]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1546]:=  (21:56:41 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubscriptBox["d", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1546]=  (21:56:41 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1547]:=  (21:56:42 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1547]=  (21:56:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"SUNTrace", "[", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[1548]:=  (21:56:42 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "e"}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "+", 
        RowBox[{
          FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "e"}]]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1548]=  (21:56:42 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t1", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1549]:=  (21:56:42 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1549]=  (21:56:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"SUNTrace", "[", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}], ",",
           
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
  CellLabel->"In[1550]:=  (21:56:43 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["1", "4"], " ", 
                  SubscriptBox["d", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], "+", 
                RowBox[{
                  FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}], ")"}]}], 
          RowBox[{"2", " ", "N"}]], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox["d", 
            RowBox[{"c101", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "8"]}], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c102"}]]}], "-", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c102"}]]}]}], 
            ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "c101"}]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "8"]}], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c103"}]]}], "-", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c103"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1550]=  (21:56:43 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1551]:=  (21:56:43 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output",\

  CellLabel->"Out[1551]=  (21:56:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1552]:=  (21:56:48 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1552]=  (21:56:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1553]:=  (21:56:48 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1553]=  (21:56:48 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "d", ",", "c"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1554]:=  (21:56:48 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1554]=  (21:56:49 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
  CellLabel->"In[1555]:=  (21:56:49 on 5/16/01)",
  CellTags->"SUNTrace"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SubsuperscriptBox["C", "A", "2"]}], ")"}]}], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox["C", "A"], "-", 
            RowBox[{"2", " ", 
              SubscriptBox["C", "F"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1555]=  (21:56:49 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SymbolicSum2", "ObjectName",
  CellTags->"SymbolicSum2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SymbolicSum2"],

Cell["\<\
SymbolicSum2 is similar to SymbolicSum \
(Algegra`SymbolicSum`SymbolicSum was a function to do symbolic summation. It \
was obsolete from version 3 - all functionality is now autoloaded by Sum), \
but extended to several double sums.\
\>", "Text",
  CellTags->"SymbolicSum2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["SymbolicSum3", "ObjectName",
  CellTags->"SymbolicSum3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"SymbolicSum3"],

Cell["\<\
SymbolicSymbolicSum3 is similar to SymbolicSum \
(Algegra`SymbolicSum`SymbolicSum was a function to do symbolic summation. It \
was obsolete from version 3 - all functionality is now autoloaded by Sum), \
but extended to several double sums.\
\>", "Text",
  CellTags->"SymbolicSum3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Symmetrize", "ObjectName",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Symmetrize"],

Cell["\<\
Symmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect \
to the variables a1,a2, ... \
\>", "Text",
  CellTags->"Symmetrize"],

Cell[TextData[{
  "See also: ",
  ButtonBox["AntiSymmetrize",
    ButtonData:>"AntiSymmetrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"AntiSymmetrize"],
  "."
}], "Text",
  CellTags->"Symmetrize"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Symmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"a", ",", "b"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1556]:=  (21:56:49 on 5/16/01)",
  CellTags->"Symmetrize"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"a", ",", "b"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1556]=  (21:56:49 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Symmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1557]:=  (21:56:49 on 5/16/01)",
  CellTags->"Symmetrize"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "6"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "y", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "z", ",", "y"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "x", ",", "y"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1557]=  (21:56:49 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TBox, Tbox", "ObjectName",
  CellTags->{"TBox", "Tbox"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->{"TBox", "Tbox"}],

Cell["\<\
TBox[a, b, ...] or Tbox[a, b, ...] produces a RowBox[{a,b, ...}] \
where a, b, ... are boxed in TraditionalForm. TBox and Tbox are used \
internally by FeynCalc to produce the typeset output in TraditionalForm\
\>", \
"Text",
  CellTags->{"TBox", "Tbox"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Tdec", "ObjectName",
  CellTags->"Tdec"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Tdec"],

Cell["\<\
Tdec[{q,mu}, {p}]; Tdec[{{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] \
or Tdec[exp, {{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] calculates the \
tensorial decomposition formulas. The more common ones are saved in \
TIDL.\
\>", "Text",
  CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Tdec", "]"}]], "Input",
  CellLabel->"In[1558]:=  (21:56:49 on 5/16/01)",
  CellTags->"Tdec"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "True"}], ",", 
          RowBox[{"List", "\[Rule]", "True"}], ",", 
          RowBox[{"NumberOfMetricTensors", "\[Rule]", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 1]]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1558]=  (21:56:50 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TIDL"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text",
  CellTags->"Tdec"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Tdec"],

Cell[TextData[{
  "Check that ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "D"], 
              RowBox[{"f", "(", 
                RowBox[{"p", ",", "q"}], ")"}], 
              SuperscriptBox["q", "\[Mu]"]}]}], "=", " ", 
          RowBox[{
            FractionBox[
              SuperscriptBox["p", "\[Mu]"], 
              SuperscriptBox["p", "2"]], 
            RowBox[{"\[Integral]", 
              RowBox[{
                SuperscriptBox["d", "D"], 
                RowBox[{"f", "(", 
                  RowBox[{"p", ",", "q"}], ")"}], 
                RowBox[{"p", "\[CenterDot]", "q"}]}]}]}]}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tdec", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1559]:=  (21:56:51 on 5/16/01)",
  CellTags->"Tdec"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"X1", "\[Rule]", 
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"]}], ",", 
              RowBox[{"X2", "\[Rule]", 
                FormBox[
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"]}]}], "}"}], ",", 
          FractionBox[
            RowBox[{"X2", " ", 
              SuperscriptBox["p", "\[Mu]"]}], "X1"]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1559]=  (21:56:51 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
      RowBox[{"%", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
  CellLabel->"In[1560]:=  (21:56:51 on 5/16/01)",
  CellTags->"Tdec"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], " ", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"]}], 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"]], TraditionalForm]], "Output",
  CellLabel->"Out[1560]=  (21:56:51 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "This calculates integral transformation for any ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Integral]", 
          RowBox[{
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "1"], 
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "2"], 
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "3"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"f", "(", 
            RowBox[{"p", ",", 
              RowBox[{
                SubscriptBox["q", 
                  RowBox[{"1", ","}]], 
                SubscriptBox["q", "2"]}], ",", 
              SubscriptBox["q", "3"]}], ")"}], " ", 
          SubsuperscriptBox["q", "1", "\[Mu]"], 
          SubsuperscriptBox["q", "2", "\[Nu]"], 
          SubsuperscriptBox["q", "3", "\[Rho]"]}], TraditionalForm]]],
  "."
}], "Text",
  CellTags->"Tdec"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{"Tdec", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "1"], ",", "\[Mu]"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "2"], ",", "\[Nu]"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "3"], ",", "\[Rho]"}], "}"}]}], "}"}], 
            ",", 
            RowBox[{"{", "p", "}"}]}], "]"}]}], ";"}]], "Input",
  CellLabel->"In[1561]:=  (21:56:51 on 5/16/01)",
  CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
      RowBox[{"t", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
  CellLabel->"In[1562]:=  (21:56:52 on 5/16/01)",
  CellTags->"Tdec"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Rho]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "3"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Nu]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "2"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Mu]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "1"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
              RowBox[{"(", 
                RowBox[{"D", "+", "1"}], ")"}], " ", 
              SuperscriptBox[
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "3"]}]], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", 
                  SuperscriptBox["D", "2"]}], ")"}], " ", 
              SuperscriptBox["p", "\[Mu]"], " ", 
              SuperscriptBox["p", "\[Nu]"], " ", 
              SuperscriptBox["p", "\[Rho]"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "+", 
                  RowBox[{"2", " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}]}], ")"}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1562]=  (21:56:52 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{"%", " ", 
            RowBox[{"FVD", "[", 
              RowBox[{"p", ",", "\[Mu]"}], "]"}]}], 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "\[Rho]"}], "]"}]}], "]"}], "//", 
      "Factor"}]], "Input",
  CellLabel->"In[1563]:=  (21:56:52 on 5/16/01)",
  CellTags->"Tdec"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "1"],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "2"],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "3"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1563]=  (21:56:53 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
  CellLabel->"In[1564]:=  (21:56:53 on 5/16/01)",
  CellTags->"Tdec"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TensorFunction", "ObjectName",
  CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TensorFunction"],

Cell["\<\
Tensorfunction[t, mu, nu, ...] transform into t[LorentzIndex[mu], \
LorentzIndex[nu], ...], i.e., it can be used as an unspecified tensoriell \
function t. A symmetric tensor can be obtained by Tensorfunction[{t, \"S\"}, \
mu, nu, ...], and an antisymmteric one by Tensorfunction[{t, \"A\"}, mu, nu, \
...].\
\>", "Text",
  CellTags->"TensorFunction"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Symmetrize",
    ButtonData:>"Symmetrize",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Symmetrize"],
  "."
}], "Text",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
      "]"}]], "Input",
  CellLabel->"In[1565]:=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"Tensorfunction", "(", 
        RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1565]=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1566]:=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
      "]"}]], "Output",
  CellLabel->"Out[1566]//StandardForm=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "%"}], "]"}]], "Input",
  CellLabel->"In[1567]:=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        RowBox[{"Tensorfunction", "(", 
          RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1567]=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1568]:=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", 
      RowBox[{"Tensorfunction", "[", 
        RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
        "]"}]}]], "Output",
  CellLabel->"Out[1568]//StandardForm=  (21:56:53 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Alpha]", ",", 
        "\[Beta]"}], "]"}]], "Input",
  CellLabel->"In[1569]:=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"Tensorfunction", "(", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Alpha]", ",", 
          "\[Beta]"}], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1569]=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Beta]", ",", 
        "\[Alpha]"}], "]"}]], "Input",
  CellLabel->"In[1570]:=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"Tensorfunction", "(", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", 
          "\[Alpha]"}], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1570]=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1571]:=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", 
        "\[Alpha]"}], "]"}]], "Output",
  CellLabel->"Out[1571]//StandardForm=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Attributes", "[", "f", "]"}]], "Input",
  CellLabel->"In[1572]:=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1572]=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[BoxData[
    RowBox[{"ClearAttributes", "[", 
      RowBox[{"f", ",", "Orderless"}], "]"}]], "Input",
  CellLabel->"In[1573]:=  (21:56:54 on 5/16/01)",
  CellTags->"TensorFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Tf", "ObjectName",
  CellTags->"Tf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Tf"],

Cell[TextData[{
  "Tf is the color factor ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "f"], TraditionalForm]]],
  ". It is 1/2 for SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ")."
}], "Text",
  CellTags->"Tf"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Tr2",
    ButtonData:>"Tr2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr2"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonSelfEnergy",
    ButtonData:>"GluonSelfEnergy",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"GluonSelfEnergy"],
  "."
}], "Text",
  CellTags->"Tf"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TFi", "ObjectName",
  CellTags->"TFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TFi"],

Cell["\<\
TFi[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is the \
\.02loop d-dimensional integral 1/( (q1^2 - m1^2)^n1 (q2^2 - m2^2)^n2 \
((q1-p)^2 - m3^2)^n3 *((q2-p)^2 - m4^2)^n4 * ((q1-q2)^2 - m5^2)^n5 ) . TFi[d, \
pp, {x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as additional \
factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z* (q2.p)^v*(q1.q2)^w. \
TFi[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as \
additional factors in the numerator (OPEDelta.q1)^a * (OPEDelta.q2)^b; dp is \
(OPEDelta.p).\
\>", "Text",
  CellTags->"TFi"],

Cell[TextData[{
  "TFi is similar to TFI from the TARCER package, see ",
  " ",
  ButtonBox["hep-ph/9801383.",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  CellTags->"TFi"],

Cell["\<\
The function TarcerRecurse from the TARCER package recognize TFi \
(as well as TFI, which is defined in the HighEnergyPhysics`Tarcer` \
context).\
\>", "Text",
  CellTags->"TFi"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ToTFi",
    ButtonData:>"ToTFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"ToTFi"],
  ", ",
  ButtonBox["FromTFi",
    ButtonData:>"FromTFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FromTFi"],
  "."
}], "Text",
  CellTags->"TFi"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"M", "^", "2"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"2", ",", "m1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m2"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"3", ",", "m3"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m4"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m5"}], "}"}]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1574]:=  (21:56:54 on 5/16/01)",
  CellTags->"TFi"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"2", ",", "m1"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m2"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"3", ",", "m3"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m4"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m5"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          Power[ M, 2], {{2, m1}, {1, m2}, {3, m3}, {1, m4}, {1, m5}}],
        Editable->True], TraditionalForm]], "Output",
  CellLabel->"Out[1574]=  (21:56:55 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"M", "^", "2"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", 
            ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1575]:=  (21:56:55 on 5/16/01)",
  CellTags->"TFi"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"2", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          Power[ M, 2], {{2, M}, {1, 0}, {1, 0}, {1, 0}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output",
  CellLabel->"Out[1575]=  (21:56:55 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ThreeVector", "ObjectName",
  CellTags->"ThreeVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ThreeVector"],

Cell["ThreeVector[p] is the three dimensional vector p.", "Text",
  CellTags->"ThreeVector"],

Cell[TextData[{
  "See also: ",
  ButtonBox["DotProduct",
    ButtonData:>"DotProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotProduct"],
  ", ",
  ButtonBox["CrossProduct",
    ButtonData:>"CrossProduct",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"CrossProduct"],
  "."
}], "Text",
  CellTags->"ThreeVector"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TID", "ObjectName",
  CellTags->"TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TID"],

Cell["TID[amp,q] performs a tensorial decomposition.", "Text",
  CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TID", "]"}]], "Input",
  CellLabel->"In[1587]:=  (21:56:57 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"Contract", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}], ",", 
          RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"Isolate", "\[Rule]", "False"}], ",", 
          RowBox[{"ScalarProductCancel", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1587]=  (21:56:57 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TIDL"],
  "."
}], "Text",
  CellTags->"TID"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "1"]}], ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "2"]}]}], "]"}], 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1588]:=  (21:56:57 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1588]=  (21:56:57 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Factor", "/@", 
        RowBox[{"TID", "[", 
          RowBox[{"%", ",", "k"}], "]"}]}], "//", "FCE"}]], "Input",
  CellLabel->"In[1589]:=  (21:56:57 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{
              FormBox[
                FractionBox["1", 
                  RowBox[{
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ k, 2], "]"],
                        Editable->False], ")"}], " ", 
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"], "-", 
                                SubscriptBox["p", "1"]}], ")"}], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ 
                            Plus[ k, 
                              Times[ -1, 
                                Subscript[ p, 1]]], 2], "]"],
                        Editable->False], ")"}]}]],
                "TraditionalForm"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["2",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["1",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}]}], ")"}]}], 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"]}], "-", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], ")"}]}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                FractionBox["1", 
                  RowBox[{
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ k, 2], "]"],
                        Editable->False], ")"}], " ", 
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"], "-", 
                                SubscriptBox["p", "1"]}], ")"}], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ 
                            Plus[ k, 
                              Times[ -1, 
                                Subscript[ p, 1]]], 2], "]"],
                        Editable->False], ")"}], " ", 
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"], "-", 
                                SubscriptBox["p", "2"]}], ")"}], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ 
                            Plus[ k, 
                              Times[ -1, 
                                Subscript[ p, 2]]], 2], "]"],
                        Editable->False], ")"}]}]],
                "TraditionalForm"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["2",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "+", 
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["1",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["1",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["2",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"]}]}], ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"]}], "-", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], ")"}]}], ")"}]}], 
        "-", 
        FractionBox[
          RowBox[{
            FormBox[
              FractionBox["1", 
                RowBox[{
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          FormBox[
                            TagBox[
                              StyleBox["\<\"k\"\>",
                                FontSlant->"Italic"],
                              (StyleForm[ #, FontSlant -> "Italic"]&)],
                            "TraditionalForm"], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ k, 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                TagBox[
                                  StyleBox["\<\"k\"\>",
                                    FontSlant->"Italic"],
                                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                                "TraditionalForm"], "-", 
                              SubscriptBox["p", "2"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 2]]], 2], "]"],
                      Editable->False], ")"}]}]],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            FormBox[
              FractionBox["1", 
                RowBox[{
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                TagBox[
                                  StyleBox["\<\"k\"\>",
                                    FontSlant->"Italic"],
                                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                                "TraditionalForm"], "-", 
                              SubscriptBox["p", "1"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 1]]], 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                TagBox[
                                  StyleBox["\<\"k\"\>",
                                    FontSlant->"Italic"],
                                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                                "TraditionalForm"], "-", 
                              SubscriptBox["p", "2"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 2]]], 2], "]"],
                      Editable->False], ")"}]}]],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}], "-", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], ")"}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1589]=  (21:56:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}], 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1590]:=  (21:56:58 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox[
            FormBox[
              TagBox[
                StyleBox["\<\"k\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)],
              "TraditionalForm"], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "-", "p"}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1590]=  (21:56:58 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TID", "[", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input",
  CellLabel->"In[1591]:=  (21:56:58 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox[
                FormBox[
                  TagBox[
                    StyleBox["\<\"k\"\>",
                      FontSlant->"Italic"],
                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                  "TraditionalForm"], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        StyleBox["\<\"k\"\>",
                          FontSlant->"Italic"],
                        (StyleForm[ #, FontSlant -> "Italic"]&)],
                      "TraditionalForm"], "-", "p"}], ")"}], "2"]}]}]], "-", 
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p"}], ")"}], "2"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1591]=  (21:56:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TID", "[", 
      RowBox[{"%%", ",", "k", ",", 
        RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "True"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1592]:=  (21:56:59 on 5/16/01)",
  CellTags->"TID"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{"2", " ", 
          RowBox[{
            SuperscriptBox[
              FormBox[
                TagBox[
                  StyleBox["\<\"k\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)],
                "TraditionalForm"], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox[
                    TagBox[
                      StyleBox["\<\"k\"\>",
                        FontSlant->"Italic"],
                      (StyleForm[ #, FontSlant -> "Italic"]&)],
                    "TraditionalForm"], "-", "p"}], ")"}], "2"]}]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1592]=  (21:56:59 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TIDL", "ObjectName",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TIDL"],

Cell[" TIDL is a database of tensorial reduction formalae.", "Text",
  CellTags->"TIDL"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TID"]
}], "Text",
  CellTags->"TIDL"]
}, Closed]],

Cell["Examples", "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "\[Mu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[TextData[{
  "In any n-dimensional integral ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "n"], "\[NegativeVeryThinSpace]", " ", 
              SuperscriptBox["q", "\[Mu]"], " ", 
              RowBox[{"f", "(", 
                RowBox[{"q", ",", "p"}], ")"}], " ", "the", " ", 
              SuperscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  "can be replaced by"
}], "Text",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1576]:=  (21:56:55 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}]}], 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]], TraditionalForm]], "Output",
  CellLabel->"Out[1576]=  (21:56:55 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "\[Mu]\[Nu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1577]:=  (21:56:55 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"D", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "4"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1577]=  (21:56:55 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1578]:=  (21:56:55 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}], " ", 
              SubsuperscriptBox["p", "1", "2"]}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], "-", 
            RowBox[{
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"]}], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], "-", 
            RowBox[{
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1578]=  (21:56:56 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1579]:=  (21:56:56 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", "2", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "-", 
                RowBox[{"2", " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], " ", 
                  SubsuperscriptBox["p", "1", "2"]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"], " ", 
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"D", "-", "2"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", "D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], "2"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "3"]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "-", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], "2"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "3"]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "-", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", "D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1579]=  (21:56:56 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1580]:=  (21:56:56 on 5/16/01)",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1581]:=  (21:56:56 on 5/16/01)",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"], ",", 
            SubscriptBox["p", "3"]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1582]:=  (21:56:56 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["3",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}]}], ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}], ")"}]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "3"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "3", "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "3"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "3", "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], "/",
           
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1582]=  (21:56:57 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",\

  CellLabel->"In[1583]:=  (21:56:57 on 5/16/01)",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",\

  CellLabel->"In[1584]:=  (21:56:57 on 5/16/01)",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection",
  CellTags->"TIDL"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r", ",", "s"}], "}"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[1585]:=  (21:56:57 on 5/16/01)",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"Length", "[", "%", "]"}], ",", " ", 
        RowBox[{"LeafCount", "[", "%", "]"}], ",", 
        RowBox[{"ByteCount", "[", "%", "]"}]}], "}"}]], "Input",
  CellLabel->"In[1586]:=  (21:56:57 on 5/16/01)",
  CellTags->"TIDL"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", "29", ",", "320"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1586]=  (21:56:57 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TimedIntegrate", "ObjectName",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TimedIntegrate"],

Cell["\<\
 TimedIntegrate[exp, vars] is like Integrate, but stops after the \
number of seconds specified by the option Timing. Options of Integrate can be \
given and are passed on.\
\>", "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TimedIntegrate", "]"}]], "Input",
  CellLabel->"In[63]:=  (23:07:08 on 9/20/02)",
  CellTags->"TimedIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Timing", "\[Rule]", "10"}], ",", 
          RowBox[{"Assumptions", "\[Rule]", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], ">", "0"}]}], ",", 
          RowBox[{"Integrate", "\[Rule]", "Integrate"}], ",", 
          RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[63]=  (23:07:08 on 9/20/02)",
  CellTags->"TimedIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TimedIntegrate"],

Cell["This should reach to be done", "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TimedIntegrate", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"x", "^", "5"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
        RowBox[{"Timing", "\[Rule]", "1"}]}], "]"}]], "Input",
  CellLabel->"In[70]:=  (23:09:09 on 9/20/02)",
  CellTags->"TimedIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "5"}], TraditionalForm]], "Output",
  CellLabel->"Out[70]=  (23:09:09 on 9/20/02)",
  CellTags->"TimedIntegrate"]
}, Closed]],

Cell["This shouldn't", "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TimedIntegrate", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"Cos", "[", 
            RowBox[{"x", "^", "5"}], "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
        RowBox[{"Timing", "\[Rule]", "10"}], ",", 
        RowBox[{"Integrate", "\[Rule]", "int"}]}], "]"}]], "Input",
  CellLabel->"In[74]:=  (23:11:11 on 9/20/02)",
  CellTags->"TimedIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"int", "(", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"cos", "(", 
              SuperscriptBox["x", "5"], ")"}], ")"}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"Assumptions", "\[Rule]", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], ">", "0"}]}]}], ")"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[74]=  (23:11:11 on 9/20/02)",
  CellTags->"TimedIntegrate"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TLI  ***unfinished***", "ObjectName",
  CellTags->"TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TLI"],

Cell["\<\
TLI[{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}].  The exponents of \
the numerator scalar product are (dl = OPEDelta):  v: k1.k1, w: k2.k2,  x: \
p.k1, y: p.k2, z: k1.k2.  a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: \
dl.(k1-k2).
TLI[{mu1, ...}, {nu1, ...}][{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}] is a \
tensor integral, where mu1 belongs to k1 and nu1 to k2.
TLI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  TLI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}].
TLI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
TLI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
  CellTags->"TLI"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TLI", "]"}]], "Input",
  CellLabel->"In[76]:=  (23:30:25 on 9/20/02)",
  CellTags->"TLI"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"EpsilonOrder", "\[Rule]", "0"}], ",", 
          RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[76]=  (23:30:25 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI2FC",
    ButtonData:>"TLI2FC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI2FC"],
  "."
}], "Text",
  CellTags->"TLI"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TLI"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1588]:=  (21:56:57 on 5/16/01)",
  CellTags->"TLI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TLIFP  ***unfinished***", "ObjectName",
  CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TLIFP"],

Cell["TLIFP[exp] does Feynman-Parametrizations of TLI's in exp.", "Text",
  CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TLIFP", "]"}]], "Input",
  CellLabel->"In[78]:=  (23:32:57 on 9/20/02)",
  CellTags->"TLIFP"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FeynmanParameterNames", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"x", ",", "t", ",", "u", ",", "s", ",", "y"}], "}"}]}], 
          ",", 
          RowBox[{"GammaExpand", "\[Rule]", "True"}], ",", 
          RowBox[{"Momentum", "\[Rule]", "p"}], ",", 
          RowBox[{"Print", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[78]=  (23:32:57 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  "."
}], "Text",
  CellTags->"TLIFP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TLIFP"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1588]:=  (21:56:57 on 5/16/01)",
  CellTags->"TLIFP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TLIHYP  ***unfinished***", "ObjectName",
  CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TLIHYP"],

Cell["\<\
TLIHYP[exp] expresses TLI's in exp. in terms of hypergeometric \
functions, where possible.\
\>", "Text",
  CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TLIHYP", "]"}]], "Input",
  CellLabel->"In[3]:=  (23:38:40 on 9/20/02)",
  CellTags->"TLIHYP"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FeynmanParameterNames", "\[Rule]", "x"}], ",", 
          RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (23:38:40 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  "."
}], "Text",
  CellTags->"TLIHYP"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TLIHYP"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1588]:=  (21:56:57 on 5/16/01)",
  CellTags->"TLIHYP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TLI2", "ObjectName",
  CellTags->"TLI2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TLI2"],

Cell["like TLI, but without any functional properties.", "Text",
  CellTags->"TLI2"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  "."
}], "Text",
  CellTags->"TLI2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TLI2FC  ***unfinished***", "ObjectName",
  CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TLI2FC"],

Cell["\<\
TLI2FC[exp] transforms all TLI-integrals in exp to the FAD \
form.\
\>", "Text",
  CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TLI2FC", "]"}]], "Input",
  CellLabel->"In[7]:=  (23:44:39 on 9/20/02)",
  CellTags->"TLI2FC"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
          RowBox[{"Momentum", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"q1", ",", "q2", ",", "p"}], "}"}]}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[7]=  (23:44:39 on 9/20/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TLI",
    ButtonData:>"TLI",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TLI"],
  "."
}], "Text",
  CellTags->"TLI2FC"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TLI2FC"],

Cell[BoxData[""], "Input",
  CellLabel->"In[1588]:=  (21:56:57 on 5/16/01)",
  CellTags->"TLI2FC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ToDistribution", "ObjectName",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ToDistribution"],

Cell["\<\
ToDistribution[exp,x] replaces (1-x)^(a Epsilon - 1) in exp by 1/(a \
Epsilon) DeltaFunction[1-x] + 1/(1-x) + a Epsilon Log[1-x]/(1-x) + 1/2 a^2 \
Epsilon^2 Log[1-x]^2/(1-x)] and (1-x)^(a Epsilon - 2) in exp by -1/(a \
Epsilon) DeltaFunctionPrime[1-x] + 1/(1-x)^2 + (a Epsilon) Log[1-x]/(1-x)^2 + \
a^2 Epsilon^2/2 Log[1-x]^2/(1-x)^2 + a^3 Epsilon^3/6 \
Log[1-x]^3/(1-x)^2.\
\>", "Text",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "ToDistribution", "]"}]], "Input",
  CellLabel->"In[1593]:=  (21:56:59 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1593]=  (21:56:59 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PlusDistribution",
    ButtonData:>"PlusDistribution",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PlusDistribution"],
  "."
}], "Text",
  CellTags->"ToDistribution"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"ToDistribution", ",", 
        RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1594]:=  (21:56:59 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1594]=  (21:56:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToDistribution", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"Epsilon", "-", "1"}], ")"}]}], ",", "x"}], 
      "]"}]], "Input",
  CellLabel->"In[1595]:=  (21:56:59 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "6"], " ", 
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["log", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
        RowBox[{
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
        SubscriptBox[
          RowBox[{"(", 
            FractionBox["1", 
              RowBox[{"1", "-", "x"}]], ")"}], "+"], "+", 
        FractionBox[
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1595]=  (21:56:59 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"ToDistribution", ",", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}]}], "]"}]], "Input",\

  CellLabel->"In[1596]:=  (21:57:00 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1596]=  (21:57:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToDistribution", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], 
      "]"}]], "Input",
  CellLabel->"In[1597]:=  (21:57:00 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "3"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"6", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "2"]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "2"]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1597]=  (21:57:00 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Integrate", "[", 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], 
              RowBox[{"Epsilon", "-", "2"}]], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
            RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], ",", 
        "Epsilon", ",", "3"}], "]"}]], "Input",
  CellLabel->"In[1598]:=  (21:57:00 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "-", 
        SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], "-", 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[1598]=  (21:57:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "@", "Integrate"}]], "Input",
  CellLabel->"In[1599]:=  (21:57:01 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Assumptions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"GenerateConditions", "\[Rule]", "Automatic"}], ",", 
          RowBox[{"PrincipalValue", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1599]=  (21:57:01 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"ToDistribution", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", 
              RowBox[{"(", 
                RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1600]:=  (21:57:01 on 5/16/01)",
  CellTags->"ToDistribution"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "-", 
        SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], "-", 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
  CellLabel->"Out[1600]=  (21:57:01 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ToHypergeometric", "ObjectName",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ToHypergeometric"],

Cell[TextData[{
  "ToHypergeometric[",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox[
            StyleBox["t",
              FontSlant->"Plain"], 
            StyleBox["b",
              FontSlant->"Plain"]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                StyleBox["t",
                  FontSlant->"Plain"]}], ")"}], 
            StyleBox["c",
              FontSlant->"Plain"]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{
                  StyleBox["t",
                    FontSlant->"Plain"], " ", 
                  StyleBox["z",
                    FontSlant->"Plain"]}]}], ")"}], 
            StyleBox["a",
              FontSlant->"Plain"]]}], TraditionalForm]],
    FontVariations->{"CompatibilityType"->0}],
  ", ",
  "t",
  "] returns ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"c", "+", "1"}], ")"}], "/", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"b", "+", "c", "+", "2"}], ")"}]}], " ", 
          FormBox[
            RowBox[{
              SubscriptBox["\[Null]", "2"], 
              RowBox[{
                SubscriptBox["F", "1"], "(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "a"}], ",", 
                    RowBox[{"b", "+", "1"}]}], ";", 
                  RowBox[{"b", "+", "c", "+", "2"}], ";", 
                  RowBox[{"-", "z"}]}], ")"}]}],
            "TraditionalForm"]}], TraditionalForm]]],
  ". Remember that Re ",
  StyleBox["b",
    FontSlant->"Italic"],
  " >0 and Re (",
  StyleBox["c",
    FontSlant->"Italic"],
  "-",
  StyleBox["b",
    FontSlant->"Italic"],
  ") > 0 should hold (need not be set in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ")."
}], "Text",
  CellTags->"ToHypergeometric"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricAC"],
  ", ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricIR"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"HypergeometricSE"],
  "."
}], "Text",
  CellTags->"ToHypergeometric"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["t", "b"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[1601]:=  (21:57:01 on 5/16/01)",
  CellTags->"ToHypergeometric"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"b", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"b", "+", "c", "+", "2"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", "z"}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1601]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{"w", " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{"c", "-", "b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
  CellLabel->"In[1602]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToHypergeometric"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"w", " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox["z",
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}], 
        RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1602]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["t", "b"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"u", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], 
      "]"}]], "Input",
  CellLabel->"In[1603]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToHypergeometric"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["u", "a"], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox[
                            RowBox[{"-", "a"}],
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox[
                            RowBox[{"b", "+", "1"}],
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox[
                          RowBox[{"b", "+", "c", "+", "2"}],
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      RowBox[{"-", 
                        FractionBox["z", "u"]}],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1603]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{"w", " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{"c", "-", "b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"u", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
  CellLabel->"In[1604]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToHypergeometric"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["u", 
            RowBox[{"-", "a"}]], " ", "w", " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}], " ", 
          TagBox[
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[InvisiblePrefixScriptBase]", 
                    FormBox["2",
                      "TraditionalForm"]], 
                  SubscriptBox["F", 
                    FormBox["1",
                      "TraditionalForm"]]}], "\[InvisibleApplication]", 
                RowBox[{"(", 
                  RowBox[{
                    TagBox[
                      TagBox[
                        RowBox[{
                          TagBox["a",
                            Hypergeometric2F1,
                            Editable->True], ",", 
                          TagBox["b",
                            Hypergeometric2F1,
                            Editable->True]}],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      TagBox[
                        TagBox["c",
                          Hypergeometric2F1,
                          Editable->True],
                        InterpretTemplate[ {
                          SlotSequence[ 1]}&]],
                      Hypergeometric2F1,
                      Editable->False], ";", 
                    TagBox[
                      FractionBox["z", "u"],
                      Hypergeometric2F1,
                      Editable->True]}], ")"}]}],
              InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&],
              Editable->False],
            Hypergeometric2F1]}], 
        RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1604]=  (21:57:02 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ToLarin", "ObjectName",
  CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ToLarin"],

Cell[TextData[{
  "ToLarin[exp] translates all ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
          SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]]],
  " in exp into ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], "/", "6"}], " ", 
          SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Lambda]\[Sigma]"], 
          SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
          SuperscriptBox["\[Gamma]", "\[Lambda]"], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Sigma]"], "."}]}], 
        TraditionalForm]]]
}], "Text",
  CellTags->"ToLarin"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Eps"],
  "."
}], "Text",
  CellTags->"ToLarin"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "5", "]"}]}]], "Input",
  CellLabel->"In[1605]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToLarin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1605]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToLarin", "[", "%", "]"}]], "Input",
  CellLabel->"In[1606]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToLarin"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "6"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du1",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du2",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du3",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1606]=  (21:57:02 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ToTFi", "ObjectName",
  CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ToTFi"],

Cell["\<\
ToTFi[expr, q1, q2, p] translates FeynCalc 2-loop self energy type \
integrals into the TFi notatation, which can be used as input for the \
function TarcerRecurse from the TARCER package. See TFi for details on the \
conventions.\
\>", "Text",
  CellTags->"ToTFi"],

Cell[TextData[{
  "See also: ",
  ButtonBox["TFi",
    ButtonData:>"TFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"TFi"],
  ", ",
  ButtonBox["FromTFi",
    ButtonData:>"FromTFi",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FromTFi"],
  "."
}], "Text",
  CellTags->"ToTFi"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q1", ",", 
        RowBox[{"q1", "-", "p"}], ",", 
        RowBox[{"{", 
          RowBox[{"q2", ",", "M"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
        RowBox[{"q1", "-", "q2"}]}], "]"}]], "Input",
  CellLabel->"In[1607]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1", "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q1, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"],
                 "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q1], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "1"], "-", 
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2"]}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ q1, 
                    Times[ -1, q2]], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SubsuperscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2", "2"], 
                "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q2, 2], "-", 
                Power[ M, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SubscriptBox[
                        StyleBox["\<\"q\"\>",
                          FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"],
                 "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q2], 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output",
  CellLabel->"Out[1607]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToTFi", "[", 
      RowBox[{"%", ",", "q1", ",", "q2", ",", "p"}], "]"}]], "Input",
  CellLabel->"In[1608]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          SPD[ p, p], {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output",
  CellLabel->"Out[1608]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1609]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"SPD", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]], "Output",
  CellLabel->"Out[1609]//StandardForm=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SOD", "[", "q1", "]"}], " ", 
        RowBox[{"SOD", "[", "q2", "]"}], 
        RowBox[{"FAD", "[", 
          RowBox[{"q1", ",", 
            RowBox[{"q1", "-", "p"}], ",", 
            RowBox[{"{", 
              RowBox[{"q2", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
            RowBox[{"q1", "-", "q2"}]}], "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1610]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "1"],
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox[
              SubscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2"],
              "TraditionalForm"]}]}], 
        RowBox[{
          SubsuperscriptBox[
            StyleBox["\<\"q\"\>",
              FontSlant->"Italic"], "1", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", "p"}], ")"}], "2"], ".", 
          
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox["\<\"q\"\>",
                  FontSlant->"Italic"], "2", "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["\<\"q\"\>",
                        FontSlant->"Italic"], "2"], "-", "p"}], ")"}], "2"], 
              "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "1"], "-", 
                SubscriptBox[
                  StyleBox["\<\"q\"\>",
                    FontSlant->"Italic"], "2"]}], ")"}], "2"]}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1610]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToTFi", "[", "%", "]"}]], "Input",
  CellLabel->"In[1611]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")", " ", "1", "1"}]],
        TFi[ D, 
          SPD[ p, p], 
          SOD[ p], {1, 1}, {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output",
  CellLabel->"Out[1611]=  (21:57:02 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1612]:=  (21:57:02 on 5/16/01)",
  CellTags->"ToTFi"],

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"SPD", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        RowBox[{"SOD", "[", "p", "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]], "Output",
  CellLabel->"Out[1612]//StandardForm=  (21:57:02 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Tr", "ObjectName",
  CellTags->"Tr"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Tr"],

Cell[TextData[{
  "Tr[exp] calculates the Dirac trace of exp. Depending on the setting of the \
option SUNTrace also a trace over SU(",
  StyleBox["N",
    FontSlant->"Italic"],
  ") objects is performed."
}], "Text",
  CellTags->"Tr"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Tr", "]"}]], "Input",
  CellLabel->"In[1613]:=  (21:57:02 on 5/16/01)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
          RowBox[{"LeviCivitaSign", "\[Rule]", 
            RowBox[{"-", "1"}]}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PairCollect", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "442"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
          RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1613]=  (21:57:02 on 5/16/01)",
  CellTags->"Tr"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["SUNTrace",
    ButtonData:>"SUNTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNTrace"],
  "."
}], "Text",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Tr"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
  CellLabel->"In[2]:=  (11:03:09 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[2]=  (11:03:09 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", "%", "]"}]], "Input",
  CellLabel->"In[3]:=  (11:03:10 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[3]=  (11:03:11 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"GSD", "[", "q", "]"}], "-", "m"}], ")"}], ".", 
        RowBox[{"GAD", "[", "\[Nu]", "]"}]}], "]"}]], "Input",
  CellLabel->"In[4]:=  (11:03:13 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[4]=  (11:03:13 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GA", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
        "]"}], "]"}]], "Input",
  CellLabel->"In[5]:=  (11:03:14 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[5]=  (11:03:14 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GAD", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", 
          ",", "\[Xi]", ",", "5"}], "]"}], "]"}]], "Input",
  CellLabel->"In[6]:=  (11:03:20 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Xi]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Xi]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Xi]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Xi]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Xi]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[6]=  (11:03:20 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["$West"], "Input",
  CellLabel->"In[7]:=  (11:03:24 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[7]=  (11:03:24 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "$West"}]], "Input",
  CellLabel->"In[8]:=  (11:03:26 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
  "\"\<If $West is set to True (which is the default), traces involving more \
than 4 Dirac matrices and gamma5 are calculated recursively according to \
formula (A.5) from Comp. Phys. Comm 77 (1993) 286-298, which is based on the \
Breitenlohner Maison gamma5-scheme.\>\""], "Print",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  CellTags->"Tr"]
}, Closed]],

Cell["\<\
$West=False should be set before any calculation is done (because \
intermediate results from trace calculations are cached).\
\>", "Text",
  CellTags->"Tr"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$West", "=", "False"}]], "Input",
  CellLabel->"In[9]:=  (11:03:31 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[9]=  (11:03:31 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"GAD", "[", 
          RowBox[{
            SubscriptBox["\[Nu]", "1"], ",", 
            SubscriptBox["\[Nu]", "2"], ",", 
            SubscriptBox["\[Nu]", "3"], ",", 
            SubscriptBox["\[Nu]", "4"], ",", 
            SubscriptBox["\[Nu]", "5"], ",", 
            SubscriptBox["\[Nu]", "6"]}], "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}]}], "]"}]], "Input",
  CellLabel->"In[10]:=  (11:03:48 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "1"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "2"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "1"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "3"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "2"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "3"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "4"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "5"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "4"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "6"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  SubscriptBox["\[Nu]", "5"], "\[NoBreak]", 
                  SubscriptBox["\[Nu]", "6"]}]]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[10]=  (11:03:48 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
  CellLabel->"In[11]:=  (11:03:52 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["r",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[11]=  (11:03:52 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
          RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"GS", "[", "q", "]"}], "+", "m"}], ")"}], ".", 
          RowBox[{"GA", "[", "\[Mu]", "]"}]}], ",", 
        RowBox[{"Factoring", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[12]:=  (11:03:57 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"8", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              SuperscriptBox["m", "2"]}], "-", 
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[12]=  (11:03:58 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[13]:=  (11:04:04 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[13]=  (11:04:04 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[14]:=  (11:04:08 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    RowBox[{"4", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
  CellLabel->"Out[14]//StandardForm=  (11:04:08 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
          RowBox[{"GA", "[", "\[Beta]", "]"}], ".", 
          RowBox[{"GAD", "[", "\[Mu]", "]"}]}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[15]:=  (11:04:17 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "D", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[15]=  (11:04:17 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[16]:=  (11:04:22 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    RowBox[{"4", " ", "D", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
  CellLabel->"Out[16]//StandardForm=  (11:04:22 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
        RowBox[{
          RowBox[{"SUNT", "[", "b", "]"}], ".", 
          RowBox[{"SUNT", "[", "c", "]"}]}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"c", ",", "b"}], "]"}]}]}]], "Input",
  CellLabel->"In[25]:=  (18:16:06 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox["b",
              "TraditionalForm"]], ".", 
          SubscriptBox["T", 
            FormBox["c",
              "TraditionalForm"]]}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{"b", "\[NoBreak]", "c"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[25]=  (18:16:06 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"t", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[26]:=  (18:16:07 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SubscriptBox["C", "F"], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[26]=  (18:16:08 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"t", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[27]:=  (18:16:11 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["C", "F"], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[27]=  (18:16:12 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"1", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[2]:=  (20:04:29 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output",
  CellLabel->"Out[2]=  (20:04:29 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"1", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[5]:=  (20:04:55 on 9/4/02)",
  CellTags->"Tr"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SubscriptBox["C", "A"]}], TraditionalForm]], "Output",
  CellLabel->"Out[5]=  (20:04:56 on 9/4/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
  CellLabel->"In[26]:=  (11:06:56 on 9/4/02)",
  CellTags->"Tr"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TraceDimension", "ObjectName",
  CellTags->"TraceDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TraceDimension"],

Cell["\<\
TraceDimension is an option for FeynCalc2FORM. If set to 4: trace, \
if set to n: tracen.\
\>", "Text",
  CellTags->"TraceDimension"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynCalc2FORM"],
  "."
}], "Text",
  CellTags->"TraceDimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TraceOfOne", "ObjectName",
  CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TraceOfOne"],

Cell["\<\
TraceOfOne is an option for Tr and DiracTrace. Its setting \
determines the value of the unit trace.\
\>", "Text",
  CellTags->"TraceOfOne"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  "."
}], "Text",
  CellTags->"TraceOfOne"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"1", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
        RowBox[{"TraceOfOne", "\[Rule]", "tr1"}]}], "]"}]], "Input",
  CellLabel->"In[12]:=  (21:24:21 on 9/4/02)",
  CellTags->"TraceOfOne"],

Cell[BoxData[
    FormBox["tr1", TraditionalForm]], "Output",
  CellLabel->"Out[12]=  (21:24:21 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"1", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
        RowBox[{"TraceOfOne", "\[Rule]", "tr2"}]}], "]"}]], "Input",
  CellLabel->"In[13]:=  (21:24:23 on 9/4/02)",
  CellTags->"TraceOfOne"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", "tr2"}], TraditionalForm]], "Output",
  CellLabel->"Out[13]=  (21:24:23 on 9/4/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Trick", "ObjectName",
  CellTags->"Trick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Trick"],

Cell["\<\
Trick[exp] performs several basic simplifications without \
expansion. Trick[exp] uses Contract, DotSimplify and SUNDeltaContract.\
\>", \
"Text",
  CellTags->"Trick"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DotSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text",
  CellTags->"Trick"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
  CellTags->"Trick"],

Cell[TextData[{
  "This calculates ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", " ", "\[Nu]"}]], 
          SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
  "and ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
  " in D dimensions."
}], "Text",
  ZeroWidthTimes->True,
  CellTags->"Trick"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"GA", "[", "\[Mu]", "]"}], " ", 
            RowBox[{"MT", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
  CellLabel->"In[1631]:=  (21:57:05 on 5/16/01)",
  CellTags->"Trick"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", "D"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1631]=  (21:57:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
  CellLabel->"In[1632]:=  (21:57:05 on 5/16/01)",
  CellTags->"Trick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"q", "-", "p"}],
                "TraditionalForm"], ")"}], "\[Nu]"], ")"}], " ", 
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"p", "+", "r"}],
                "TraditionalForm"], ")"}], "\[Mu]"], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1632]=  (21:57:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", "%", "]"}]], "Input",
  CellLabel->"In[1633]:=  (21:57:05 on 5/16/01)",
  CellTags->"Trick"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1633]=  (21:57:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{"c", ".", "b", ".", "a", " ", ".", " ", 
        RowBox[{"GA", "[", "d", "]"}], ".", 
        RowBox[{"GA", "[", "e", "]"}]}], "]"}]], "Input",
  CellLabel->"In[1634]:=  (21:57:05 on 5/16/01)",
  CellTags->"Trick"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", " ", "b", " ", "c", " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1634]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
  CellLabel->"In[1635]:=  (21:57:06 on 5/16/01)",
  CellTags->"Trick"],

Cell[BoxData[
    RowBox[{"a", " ", "b", " ", "c", " ", 
      RowBox[{
        RowBox[{"GA", "[", "d", "]"}], ".", 
        RowBox[{"GA", "[", "e", "]"}]}]}]], "Output",
  CellLabel->"Out[1635]//StandardForm=  (21:57:06 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TrickIntegrate", "ObjectName",
  CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TrickIntegrate"],

Cell["\<\
 TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] does an integration \
trick for the definite integral of ((1-t)^(a Epsilon -1) g[t])  from 0 to 1. \
TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] gives  g[1]/a/Epsilon + \
Hold[Integrate][(1-t)^(a Epsilon-1) (g[t]-g[1]),{t,0,1}]. TrickIntegrate[t^(a \
Epsilon -1) f[t], t] gives f[0]/a/Epsilon + Hold[Integrate][t^(a Epsilon-1) \
(f[t]-f[0]),{t,0,1}], provided g[1] and f[0] do exist.\
\>", "Text",
  CellTags->"TrickIntegrate"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TrickIntegrate", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
          RowBox[{"g", "[", "t", "]"}]}], ",", "t"}], "]"}]], "Input",
  CellLabel->"In[11]:=  (23:51:03 on 9/20/02)",
  CellTags->"TrickIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            UnderscriptBox["lim", 
              RowBox[{"t", "\[Rule]", "0"}]], "\[ThinSpace]", 
            RowBox[{"g", "(", 
              RowBox[{"1", "-", "t"}], ")"}]}], 
          RowBox[{"a", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}]], "+", 
        RowBox[{
          RowBox[{"Hold", "[", "Integrate", "]"}], "[", 
          RowBox[{
            RowBox[{
              SuperscriptBox["t", 
                RowBox[{
                  RowBox[{"a", " ", 
                    TagBox["\[CurlyEpsilon]",
                      TraditionalForm]}], "-", "1"}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"g", "(", 
                    RowBox[{"1", "-", "t"}], ")"}], "-", 
                  RowBox[{
                    UnderscriptBox["lim", 
                      RowBox[{"t", "\[Rule]", "0"}]], "\[ThinSpace]", 
                    RowBox[{"g", "(", 
                      RowBox[{"1", "-", "t"}], ")"}]}]}], ")"}]}], ",", 
            RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[11]=  (23:51:03 on 9/20/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TrickMandelstam", "ObjectName",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TrickMandelstam"],

Cell[TextData[{
  "TrickMandelstam[expr, {s, t, u, ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "1", "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "2", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "3", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox["m",
            FontSlant->"Plain"], "4", "2"], TraditionalForm]]],
  "}] simplifies all sums in expr so that one of the Mandelstam variables s, \
t or u is eliminated by the relation ",
  StyleBox["s",
    FontSlant->"Italic"],
  " + ",
  StyleBox["t",
    FontSlant->"Italic"],
  " + ",
  StyleBox["u",
    FontSlant->"Italic"],
  " = ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox["m",
            FontSlant->"Italic"], "1", "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
  " . The trick is that the resulting sum has the most short number of \
terms."
}], "Text",
  CellTags->"TrickMandelstam"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"TrickMandelstam", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"s", "+", "t", "-", "u"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", 
                  SubsuperscriptBox["M", "W", "2"]}], "-", "t", "-", "u"}], 
              ")"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"s", ",", "t", ",", "u", ",", 
              RowBox[{"2", 
                SubsuperscriptBox["M", "W", "2"]}]}], "}"}]}], "]"}], "//", 
      "Factor2"}]], "Input",
  CellLabel->"In[1629]:=  (21:57:05 on 5/16/01)",
  CellTags->"TrickMandelstam"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "s", " ", 
        RowBox[{"(", 
          RowBox[{"u", "-", 
            SubsuperscriptBox["M", "W", "2"]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1629]=  (21:57:05 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TrickMandelstam", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "s"}], " ", "-", " ", 
          RowBox[{"s", "^", "2"}], " ", "+", " ", 
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "t"}], " ", "-", " ", 
          RowBox[{"s", "t"}], " ", "+", " ", 
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "u"}], " ", "-", " ", 
          RowBox[{"s", "u"}]}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"s", ",", "t", ",", "u", ",", 
            RowBox[{"2", 
              RowBox[{"M", "^", "2"}]}]}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1630]:=  (21:57:05 on 5/16/01)",
  CellTags->"TrickMandelstam"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SuperscriptBox["M", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["M", "2"], "-", "s"}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1630]=  (21:57:05 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Tr2", "ObjectName",
  CellTags->"Tr2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Tr2"],

Cell["\<\
If exp contains DiracTrace's, Tr2[exp] simplifies exp and does the \
Dirac traces unless more that 4 gamma matrices and DiracGamma[5] occur. \
Tr2[exp] also separates the color-strucure, and takes the color trace if Tf \
occurs in exp. If exp does not contain DiracTrace's, Tr2[exp] takes the Dirac \
trace.\
\>", "Text",
  CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Tr2", "]"}]], "Input",
  CellLabel->"In[18]:=  (21:02:39 on 9/3/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Factoring", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[18]=  (21:02:39 on 9/3/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  ", ",
  ButtonBox["Tf",
    ButtonData:>"Tf",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tf"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["SUNTrace",
    ButtonData:>"SUNTrace",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SUNTrace"],
  "."
}], "Text",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", "a", "]"}], "//", "Tr2"}]], "Input",
  CellLabel->"In[24]:=  (14:12:41 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "a"}], TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (14:12:41 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", "a", "]"}]], "Input",
  CellLabel->"In[25]:=  (14:12:44 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "a"}], TraditionalForm]], "Output",
  CellLabel->"Out[25]=  (14:12:44 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{
          RowBox[{"a", "[", 
            RowBox[{"SUNIndex", "[", "i", "]"}], "]"}], "Tf"}], "]"}], "//", 
      "Tr2"}]], "Input",
  CellLabel->"In[32]:=  (14:13:51 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{"a", "(", 
          FormBox["i",
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (14:13:51 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{"a", "[", 
          RowBox[{"SUNIndex", "[", "i", "]"}], "]"}], "]"}], "//", 
      "Tr2"}]], "Input",
  CellLabel->"In[30]:=  (14:13:42 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"a", "(", 
          FormBox["i",
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[30]=  (14:13:42 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{"a", " ", "Tf"}], "]"}], "//", "Tr2"}]], "Input",
  CellLabel->"In[31]:=  (14:13:45 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "a", " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
  CellLabel->"Out[31]=  (14:13:45 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"a", " ", "Tf"}], ",", 
        RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[33]:=  (14:13:59 on 9/21/02)",
  CellTags->"Tr2"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "a", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
  CellLabel->"Out[33]=  (14:13:59 on 9/21/02)",
  CellTags->"Tr2"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist2AlienOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
  CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist2AlienOperator"],

Cell["\<\
Twist2AlienOperator[p, 0] : (7);   \
Twist2AlienOperator[p1,p2,{p3,mu,a}, 0] (p1: incoming quark momentum, p3: \
incoming gluon (count1)).\
\>", "Text",
  CellTags->"Twist2AlienOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2AlienOperator", "[", 
      RowBox[{"p", ",", " ", "0"}], "]"}]], "Input",
  CellLabel->"In[35]:=  (14:17:04 on 9/21/02)",
  CellTags->"Twist2AlienOperator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"2", " ", 
          SubscriptBox["C", "F"], " ", 
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox["2", "m"], "-", 
              FractionBox["1", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["2", 
                RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output",
  CellLabel->"Out[35]=  (14:17:04 on 9/21/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2AlienOperator", "[", 
      RowBox[{"p1", ",", "p2", ",", 
        RowBox[{"{", 
          RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", " ", "0"}], 
      "]"}]], "Input",
  CellLabel->"In[36]:=  (14:17:20 on 9/21/02)",
  CellTags->"Twist2AlienOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], 
        RowBox[{"(", 
          RowBox[{"\[ImaginaryI]", " ", 
            RowBox[{"(", 
              RowBox[{"1", "+", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
            SubsuperscriptBox["g", "s", "3"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{
              SubscriptBox["T", 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"]}], ")"}]}], " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["mu",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      FractionBox["1", 
                        RowBox[{"m", "-", "1"}]], "-", 
                      FractionBox["1", "m"]}], ")"}], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p3",
                              "TraditionalForm"]}], ")"}]}], ")"}], 
                    RowBox[{"m", "-", "2"}]]}], "+", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      FractionBox["2", "m"], "-", 
                      FractionBox["1", 
                        RowBox[{"m", "+", "1"}]], "-", 
                      FractionBox["2", 
                        RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      FractionBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p1",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{"m", "-", "1"}]], 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p1",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p3",
                              "TraditionalForm"]}]}]], "-", 
                      FractionBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p3",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{"m", "-", "1"}]], 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p1",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p3",
                              "TraditionalForm"]}]}]]}], ")"}]}]}], ")"}]}], 
          ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[36]=  (14:17:20 on 9/21/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist2CounterOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
  CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist2CounterOperator"],

Cell["\<\
Twist2CounterOperator[p,mu,nu,a,b,5]; Twist2CounterOperator[p, 7] : \
(7);  Twist2CounterOperator[p1,p2,{p3,mu,a}, 1] (p1: incoming quark momentum, \
p3: incoming gluon (count1)).\
\>", "Text",
  CellTags->"Twist2CounterOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2CounterOperator", "[", 
      RowBox[{"p", ",", "mu", ",", "nu", ",", "a", ",", "b", ",", "5"}], 
      "]"}]], "Input",
  CellLabel->"In[38]:=  (14:19:33 on 9/21/02)",
  CellTags->"Twist2CounterOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"2", " ", 
              TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "+", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
              SubscriptBox["C", "A"], " ", 
              SubsuperscriptBox["g", "s", "2"], " ", 
              SubscriptBox["S", "n"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FractionBox["8", "m"], "-", 
                        FractionBox["24", 
                          RowBox[{"m", "+", "1"}]], "+", 
                        FractionBox["24", 
                          RowBox[{"m", "+", "2"}]], "-", 
                        FractionBox["4", 
                          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["mu",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["nu",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{"m", "-", "2"}]]}], "+", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          FractionBox["6", "m"]}], "+", 
                        FractionBox["16", 
                          RowBox[{"m", "+", "1"}]], "-", 
                        FractionBox["16", 
                          RowBox[{"m", "+", "2"}]], "+", 
                        FractionBox["6", 
                          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["mu",
                                "TraditionalForm"],
                              "TraditionalForm"]], " ", 
                          SuperscriptBox[
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["nu",
                                "TraditionalForm"],
                              "TraditionalForm"]]}], "+", 
                        RowBox[{
                          SuperscriptBox[
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["mu",
                                "TraditionalForm"],
                              "TraditionalForm"]], " ", 
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox["nu",
                                "TraditionalForm"],
                              "TraditionalForm"]]}]}], ")"}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{"m", "-", "1"}]]}], "+", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FractionBox["8", "m"], "-", 
                        FractionBox["12", 
                          RowBox[{"m", "+", "1"}]], "+", 
                        FractionBox["8", 
                          RowBox[{"m", "+", "2"}]], "-", 
                        FractionBox["8", 
                          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
                    SuperscriptBox["g", 
                      RowBox[{"mu", "\[NoBreak]", "nu"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "m"]}]}], ")"}], " ",
               
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[38]=  (14:19:33 on 9/21/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2CounterOperator", "[", 
      RowBox[{"p", ",", "7"}], "]"}]], "Input",
  CellLabel->"In[39]:=  (14:19:43 on 9/21/02)",
  CellTags->"Twist2CounterOperator"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
          SubscriptBox["C", "F"], " ", 
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox["2", "m"], "-", 
              FractionBox["1", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["2", 
                RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output",
  CellLabel->"Out[39]=  (14:19:43 on 9/21/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2CounterOperator", "[", 
      RowBox[{"p1", ",", "p2", ",", 
        RowBox[{"{", 
          RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", "1"}], 
      "]"}]], "Input",
  CellLabel->"In[40]:=  (14:19:55 on 9/21/02)",
  CellTags->"Twist2CounterOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"2", " ", 
              TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "+", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubscriptBox["C", "A"], "-", 
                  RowBox[{"2", " ", 
                    SubscriptBox["C", "F"]}]}], ")"}], " ", 
              SubsuperscriptBox["g", "s", "3"], " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox["2", "m"], "-", 
                  FractionBox["1", 
                    RowBox[{"m", "+", "1"}]], "-", 
                  FractionBox["2", 
                    RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
              SubscriptBox["S", "n"], " ", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["mu",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p1",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{"m", "-", "1"}]], 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p1",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p2",
                          "TraditionalForm"]}]}]], "-", 
                  FractionBox[
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p2",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      RowBox[{"m", "-", "1"}]], 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p1",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p2",
                          "TraditionalForm"]}]}]]}], ")"}], " ", 
              SubscriptBox["T", 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[40]=  (14:19:56 on 9/21/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist2GluonOperator", "ObjectName",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell["\<\
Twist2GluonOperator[{p, mu, a}, {nu, b}] or Twist2GluonOperator[p, \
{mu, a}, {nu, b}] or Twist2GluonOperator[p, mu,a, nu,b] yields the 2-gluon \
operator (p is ingoing momentum corresponding to Lorentz index mu). \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}] or Twist2GluonOperator[ \
p,mu,a , q,nu,b , k,la,c ] gives the 3-gluon operator. \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
Twist2GluonOperator[p,mu,a , q,nu,b , k,la,c , s,si,d] yields the 4-Gluon \
operator. The dimension is determined by the option and Dimension. The \
setting of the option Polarization (unpolarized: 0; polarized: 1) determines \
whether the unpolarized or polarized feynman rule is returned. With the \
setting Explicit to False the color-structure and the (1+(-1)^OPEm) (for \
polarized: (1-(-1)^OPEm)) is extracted and the color indices are omitted in \
the arguments of Twist2GluonOperator.\
\>", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text",
  CellTags->"Twist2GluonOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion ", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}], ",", " ", 
        RowBox[{"ZeroMomentumInsertion", " ", "\[Rule]", " ", "True"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1636]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1636]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell["2-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1637]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1637]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1638]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1638]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1639]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"]}], "G2"], "(", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1639]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1640]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1640]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell["3-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1641]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G3"], "(", 
            FormBox["q",
              "TraditionalForm"], ",", 
            FormBox["r",
              "TraditionalForm"], ",", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1641]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1642]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Mu]",
              "TraditionalForm"]}], "G3"], "(", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1642]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1643]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}]}], ")"}]}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1643]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell["4-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1644]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c112",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1644]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1645]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Sigma]",
              "TraditionalForm"]}], "G4"], "(", 
        FormBox["p",
          "TraditionalForm"], ",", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["s",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1645]=  (21:57:06 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1646]:=  (21:57:06 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["s",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["s",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], " ",
                   
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], " ",
                   
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1646]=  (21:57:06 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1647]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1647]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell["2-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1648]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1648]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1649]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1649]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
  CellLabel->"In[1650]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"]}], "G2"], "(", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1650]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1651]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1651]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell["3-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1652]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G3"], "(", 
            FormBox["q",
              "TraditionalForm"], ",", 
            FormBox["r",
              "TraditionalForm"], ",", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[1652]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1653]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Mu]",
              "TraditionalForm"]}], "G3"], "(", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1653]=  (21:57:07 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1654]:=  (21:57:07 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["r",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["r",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1654]=  (21:57:08 on 5/16/01)"]
}, Closed]],

Cell["4-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1655]:=  (21:57:08 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c113",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1655]=  (21:57:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1656]:=  (21:57:08 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Sigma]",
              "TraditionalForm"]}], "G4"], "(", 
        FormBox["p",
          "TraditionalForm"], ",", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["s",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1656]=  (21:57:08 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1657]:=  (21:57:08 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["s",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["s",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], " ",
                   
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["s",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], " ",
                   
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["s",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1657]=  (21:57:08 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Advanced Examples", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell["The setting All for Explicit performs the sums.", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Twist2GluonOperator", ",", " ", 
          RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
          RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1658]:=  (21:57:09 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Twist2GluonOperator", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[1659]:=  (21:57:09 on 5/16/01)",
  CellTags->"Twist2GluonOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero-momentum insertion", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1660]:=  (21:57:09 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1660]=  (21:57:09 on 5/16/01)"]
}, Closed]],

Cell["2-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1661]:=  (21:57:09 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]}]}], ")"}], " ", 
              FormBox[
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              FormBox[
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1661]=  (21:57:09 on 5/16/01)"]
}, Closed]],

Cell["3-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1662]:=  (21:57:09 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]]}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "+", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            FractionBox[
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}], " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], ")"}], "m"]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}], "m"], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]}], "+", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}], ")"}], "m"], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]}]}], ")"}]}], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1662]=  (21:57:10 on 5/16/01)"]
}, Closed]],

Cell["4-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1663]:=  (21:57:10 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c114",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1663]=  (21:57:10 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero-momentum insertion", "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "\[Rule]", " ", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1664]:=  (21:57:10 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1664]=  (21:57:11 on 5/16/01)"]
}, Closed]],

Cell["2-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1665]:=  (21:57:11 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1665]=  (21:57:11 on 5/16/01)"]
}, Closed]],

Cell["3-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Twist2GluonOperator", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[1666]:=  (21:57:11 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[1667]:=  (21:57:12 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["\[Mu]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["r",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["q",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["r",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[Mu]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["\[Mu]",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1667]=  (21:57:12 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
  CellLabel->"In[1668]:=  (21:57:12 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["\[Mu]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["r",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["q",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["r",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[Mu]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["\[Mu]",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1668]=  (21:57:12 on 5/16/01)"]
}, Closed]],

Cell["4-gluon legs", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
  CellLabel->"In[1669]:=  (21:57:12 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c115",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1669]=  (21:57:12 on 5/16/01)"]
}, Closed]],

Cell["Suppress the lengthy output.", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", 
      RowBox[{
        RowBox[{"Twist2GluonOperator", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
            RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], ",", "6"}], 
      "]"}]], "Input",
  CellLabel->"In[1670]:=  (21:57:12 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "+", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}]}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], "+", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}], ")"}], 
              RowBox[{"m", "-", "2"}]]}], "+", 
          RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
            " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]], " ", 
            SuperscriptBox[
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]], " ", 
            RowBox[{"(", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}]}], 
          "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{"j", "-", "i"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["s",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                  "TraditionalForm"]}], ")"}], " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    SuperscriptBox[
                      FormBox["s",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}]}], ")"}]}]}],
        (Short[ #, 6]&)], TraditionalForm]], "Output",
  CellLabel->"Out[1670]//Short=  (21:57:13 on 5/16/01)"]
}, Closed]],

Cell["\<\
The setting Explicit \[Rule] All performs the sums.\
\>", "Text",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Twist2GluonOperator", ",", " ", 
          RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
          RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1671]:=  (21:57:13 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", 
      RowBox[{
        RowBox[{"Twist2GluonOperator", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ",", 
        "5"}], "]"}]], "Input",
  CellLabel->"In[1672]:=  (21:57:13 on 5/16/01)",
  CellTags->"Twist2GluonOperator"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]]}], "+", 
              RowBox[{"\[LeftSkeleton]", "20", "\[RightSkeleton]"}], "+", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], "-", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], "m"]}], "+", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], "m"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}]}], "+", 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "m"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}]}]}], ")"}]}], 
                RowBox[{
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]}]]}], ")"}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}],
        (Short[ #, 5]&)], TraditionalForm]], "Output",
  CellLabel->"Out[1672]//Short=  (21:57:13 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist2QuarkOperator", "ObjectName",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell["\<\
Twist2QuarkOperator[p] or Twist2QuarkOperator[p,_,_] yields the \
quark-antiquark operator (p is momentum in the direction of the incoming \
quark). Twist2QuarkOperator[{p,q}] yields the 2-quark operator for non-zero \
momentum insertion (p is momentum in the direction of the incoming quark). \
Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a] is the quark-antiquark-gluon \
operator, where p1 is the incoming quark, p2 the incoming antiquark and p3 \
denotes the incoming gluon momentum. Twist2QuarkOperator[{p1}, {p2}, {p3, mu, \
a}, {p4, nu, b}] or Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}, {p4, \
nu, b}] or Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a, p4, nu, b] gives the \
Quark-Quark-Gluon-Gluon-operator. The setting of the option Polarization \
(unpolarized: 0; polarized: 1) determines whether the unpolarized or \
polarized operator is returned\
\>", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text",
  CellTags->"Twist2QuarkOperator"]
}, Closed]],

Cell["Examples", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1673]:=  (21:57:13 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1673]=  (21:57:13 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
  CellLabel->"In[1674]:=  (21:57:13 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SuperscriptBox["\[Gamma]", "5"]}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1674]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark-gluon operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1675]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubscriptBox["g", "s"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1675]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1676]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["g", "s", "2"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"j", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", "1"}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], "i"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"j", "-", "i"}]]}],
                      "TraditionalForm"]}], ")"}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "m"], " ", 
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"j", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", "1"}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "i"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"j", "-", "i"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1676]=  (21:57:14 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", " ", 
        RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1677]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1677]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
  CellLabel->"In[1678]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1678]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark-gluon operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1679]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1679]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1680]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox[
                                  FormBox[
                                    TagBox[
                                      StyleBox["\<\"k\"\>",
                                        FontSlant->"Italic"],
                                      (
                                      StyleForm[ #, FontSlant -> "Italic"]&)],
                                    
                                    "TraditionalForm"],
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1680]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell["This shows the FeynCalcExternal (FCE) form.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Twist2QuarkOperator", "[", "p", "]"}], "//", "FCE"}], "//", 
      "StandardForm"}]], "Input",
  CellLabel->"In[1681]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "OPEDelta", "]"}], " ", 
      SuperscriptBox[
        RowBox[{"SOD", "[", "p", "]"}], 
        RowBox[{
          RowBox[{"-", "1"}], "+", "OPEm"}]]}]], "Output",
  CellLabel->"Out[1681]//StandardForm=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2QuarkOperator", "[", 
      RowBox[{
        RowBox[{"{", "p", "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
  CellLabel->"In[1682]:=  (21:57:14 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubscriptBox["g", "s"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "1"}]], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]], "+", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "m"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "1"}]]}], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]]}], ")"}]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1682]=  (21:57:14 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2QuarkOperator", "[", 
      RowBox[{
        RowBox[{"{", "p", "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
  CellLabel->"In[1683]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["g", "s", "2"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        FormBox[
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{"-", "1"}], ")"}], 
                            RowBox[{"m", "-", "1"}]],
                          "TraditionalForm"], " ", 
                        FormBox[
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}], 
                            RowBox[{"m", "-", "1"}]],
                          "TraditionalForm"]}], 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox[
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}]}]], "-", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}]}]], "+", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox[
                                  FormBox[
                                    TagBox[
                                      StyleBox["\<\"k\"\>",
                                        FontSlant->"Italic"],
                                      (
                                      StyleForm[ #, FontSlant -> "Italic"]&)],
                                    
                                    "TraditionalForm"],
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox[
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}]}]]}], ")"}]}], 
              "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        FormBox[
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{"-", "1"}], ")"}], 
                            RowBox[{"m", "-", "1"}]],
                          "TraditionalForm"], " ", 
                        FormBox[
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}], 
                            RowBox[{"m", "-", "1"}]],
                          "TraditionalForm"]}], 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}]}]], "-", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]], "+", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}]}]]}], ")"}]}]}],
             ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1683]=  (21:57:15 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero momentum insertion", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1684]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1684]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell["\<\
With the setting ZeroMomentumInsertion \[Rule] False a non-zero \
momentum is assumed to flow into the operator vertex. \
\>", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell["\<\
This is the Feynman rule associated with the quark-antiquark \
operator, where p is the momentum of the incoming quark and q the momentum of \
the antiquark. The momentum flowing into the operator vertex is thus \
-p-q.\
\>", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
  CellLabel->"In[1685]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"1", "-", "m"}]], " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], "-", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1685]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell["This is the quark-antiquark-gluon operator vertex.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1686]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"2", "-", "m"}]], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], ")"}]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1686]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell["This shows the FeynCalcExternal form.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", "t2", "]"}], "]"}]], "Input",
  CellLabel->"In[1687]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    RowBox[{
      SuperscriptBox["2", 
        RowBox[{"2", "-", "OPEm"}]], " ", "Gstrong", " ", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}], ".", 
        RowBox[{"GSD", "[", "OPEDelta", "]"}]}], " ", 
      RowBox[{"FVD", "[", 
        RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", 
                    RowBox[{"SOD", "[", "k", "]"}]}], "+", 
                  RowBox[{"SOD", "[", "p", "]"}], "-", 
                  RowBox[{"SOD", "[", "q", "]"}]}], ")"}], 
              RowBox[{
                RowBox[{"-", "2"}], "-", "OPEi", "+", "OPEm"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"SOD", "[", "k", "]"}], "+", 
                  RowBox[{"SOD", "[", "p", "]"}], "-", 
                  RowBox[{"SOD", "[", "q", "]"}]}], ")"}], "OPEi"]}], ",", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", 
              RowBox[{
                RowBox[{"-", "2"}], "+", "OPEm"}]}], "}"}]}], 
        "]"}]}]], "Output",
  CellLabel->"Out[1687]//StandardForm=  (21:57:15 on 5/16/01)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero momentum insertion", "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], 
      "]"}]], "Input",
  CellLabel->"In[1688]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1688]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
  CellLabel->"In[1689]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"1", "-", "m"}]], " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], "-", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1689]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell["Quark-antiquark-gluon operator.", "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",\

  CellLabel->"In[1690]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"2", "-", "m"}]], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                FormBox[
                                  TagBox[
                                    StyleBox["\<\"k\"\>",
                                      FontSlant->"Italic"],
                                    (StyleForm[ #, FontSlant -> "Italic"]&)],
                                  "TraditionalForm"],
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], ")"}]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            FormBox[
                              TagBox[
                                StyleBox["\<\"k\"\>",
                                  FontSlant->"Italic"],
                                (StyleForm[ #, FontSlant -> "Italic"]&)],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1690]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}],
         "]"}], ";"}]], "Input",
  CellLabel->"In[1691]:=  (21:57:15 on 5/16/01)",
  CellTags->"Twist2QuarkOperator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist3QuarkOperator", "ObjectName",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist3QuarkOperator"],

Cell["\<\
Twist3QuarkOperator[p] or  Twist3QuarkOperator[p,_,_]  yields the  \
2-quark operator (p is momentum in the direction of the fermion number flow). \
Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a] Quark-Quark-Gluon-operator, \
where p1 is the incoming quark, p2 the incoming antiquark and p3 denotes the \
(incoming) gluon momentum.  Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, \
a}, {p4, nu, b}] or Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a, p4, nu, b] \
 gives the Quark-Quark-Gluon-Gluon-operator. The setting of the option \
Polarization (unpolarized: 0; polarized: 1) determines whether the \
uppolarized or polarized operator is returned.\
\>", "Text",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist3QuarkOperator", "//", "Options"}]], "Input",
  CellLabel->"In[42]:=  (14:28:18 on 9/21/02)",
  CellTags->"Twist3QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[42]=  (14:28:18 on 9/21/02)",
  CellTags->"Twist3QuarkOperator"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  ", ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text",
  CellTags->"Twist3QuarkOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist3QuarkOperator", "[", "p", "]"}]], "Input",
  CellLabel->"In[44]:=  (14:28:38 on 9/21/02)",
  CellTags->"Twist3QuarkOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[44]=  (14:28:38 on 9/21/02)",
  CellTags->"Twist3QuarkOperator"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Twist4GluonOperator", "ObjectName",
  CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Twist4GluonOperator"],

Cell["\<\
Twist4GluonOperator[{oa, ob, oc, od}, {p1,la1,a1}, {p2,la2,a2}, \
{p3,la3,a3}, {p4,la4,a4}].\
\>", "Text",
  CellTags->"Twist4GluonOperator"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  ", ",
  ButtonBox["Twist3QuarkOperator",
    ButtonData:>"Twist3QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist3QuarkOperator"],
  ", ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text",
  CellTags->"Twist4GluonOperator"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist4GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"oa", ",", " ", "ob", ",", " ", "oc", ",", " ", "od"}], 
          "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"p1", ",", "la1", ",", "a1"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"p2", ",", "la2", ",", "a2"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"p3", ",", "la3", ",", "a3"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"p4", ",", "la4", ",", "a4"}], "}"}]}], "]"}]], "Input",
  CellLabel->"In[51]:=  (14:33:07 on 9/21/02)",
  CellTags->"Twist4GluonOperator"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oa"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "ob"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "od"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "oc"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la3"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la2", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p2",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p2",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la1", "\[NoBreak]", "la2"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p1",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p1",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la1",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la2",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["p4",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"la3", "\[NoBreak]", "la4"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p3",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p3",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la3",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["la4",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p3",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p4",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a1", ",", "oa"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a2", ",", "ob"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a3", ",", "oc"}], ")"}], " ", 
          RowBox[{"SUND", "(", 
            RowBox[{"a4", ",", "od"}], ")"}]}]}], TraditionalForm]], "Output",\

  CellLabel->"Out[51]=  (14:33:07 on 9/21/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TwoLoopSimplify  ***unfinished*** (examples missing)", "ObjectName",
  CellTags->"TwoLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"TwoLoopSimplify"],

Cell["\<\
TwoLoopSimplify[amplitude,{qu1,qu2}] simplifies the 2-loop \
amplitude (qu1 and qu2 denote the integration momenta). \
TwoLoopSimplify[amplitude] transforms to TwoLoopSimplify[amplitude, \
{Global`q1, Global`q2}], i.e., the integration momenta in amplitude must be \
named q1 and q2.\
\>", "Text",
  CellTags->"TwoLoopSimplify"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text",
  CellTags->"TwoLoopSimplify"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"TwoLoopSimplify"],

Cell[BoxData[""], "Input",
  CellLabel->"In[3]:=  (14:37:42 on 9/21/02)",
  CellTags->"TwoLoopSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Uncontract", "ObjectName",
  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Uncontract"],

Cell["\<\
Uncontract[exp, q1, q2, ...] uncontracts Eps and DiracGamma. \
Uncontract[exp, q1, q2, Pair \[Rule] {p}] uncontracts also p.q1 and p.q2; the \
option Pair \[Rule] All uncontracts all momenta except OPEDelta.\
\>", "Text",\

  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Uncontract", "]"}]], "Input",
  CellLabel->"In[1692]:=  (21:57:15 on 5/16/01)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"Pair", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1692]=  (21:57:15 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract"],
  "."
}], "Text",
  CellTags->"Uncontract"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
  CellLabel->"In[54]:=  (22:14:55 on 9/4/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]], TraditionalForm]], "Output",
  CellLabel->"Out[54]=  (22:14:55 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t1", ",", " ", "p"}], "]"}]], "Input",
  CellLabel->"In[55]:=  (22:14:59 on 9/4/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$501", "\[InvisibleApplication]", 
                  RowBox[{"(", "1", ")"}]}],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$501", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[55]=  (22:14:59 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracSlash", "[", "p", "]"}]}]], "Input",
  CellLabel->"In[56]:=  (22:15:10 on 9/4/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[56]=  (22:15:10 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t2", ",", " ", "p"}], "]"}]], "Input",
  CellLabel->"In[57]:=  (22:15:12 on 9/4/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox[
              RowBox[{"$AL$502", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$502", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[57]=  (22:15:13 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t1", ",", " ", "p", ",", "q"}], "]"}]], "Input",
  CellLabel->"In[58]:=  (22:15:34 on 9/4/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$504", "\[InvisibleApplication]", 
                  RowBox[{"(", "1", ")"}]}],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$503", "\[InvisibleApplication]", 
                  RowBox[{"(", "1", ")"}]}],
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$503", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$504", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[58]=  (22:15:34 on 9/4/02)"]
}, Closed]],

Cell["By default scalar products are not uncontracted.", "Text",
  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", " ", "q"}], "]"}]], "Input",
  CellLabel->"In[1698]:=  (21:57:16 on 5/16/01)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1698]=  (21:57:16 on 5/16/01)"]
}, Closed]],

Cell["\<\
With the option Pair\[Rule]All they are \
\[OpenCurlyDoubleQuote]uncontracted \[CloseCurlyDoubleQuote].\
\>", "Text",
  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
  CellLabel->"In[15]:=  (17:33:45 on 9/21/02)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$30", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$30", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[15]=  (17:33:45 on 9/21/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p", ",", "q"}], "]"}], "^", "2"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
  CellLabel->"In[1700]:=  (21:57:16 on 5/16/01)",
  CellTags->"Uncontract"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$4046", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$4046", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$4046", "\[InvisibleApplication]", 
                RowBox[{"(", "2", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$4046", "\[InvisibleApplication]", 
                RowBox[{"(", "2", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1700]=  (21:57:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
  CellLabel->"In[1701]:=  (21:57:16 on 5/16/01)",
  CellTags->"Uncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["UnDeclareNonCommutative", "ObjectName",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"UnDeclareNonCommutative"],

Cell["\<\
UnDeclareNonCommutative[a, b, ...] undeclares a,b, ... to be \
noncommutative, i.e., DataType[a,b, ..., NonCommutative] is set to \
False.\
\>", "Text",
  CellTags->"UnDeclareNonCommutative"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DeclareNonCommutative"],
  "."
}], "Text",
  CellTags->"UnDeclareNonCommutative"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
  CellLabel->"In[1702]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell["\<\
As a side-effect of DeclareNonCommutative x is declared to be of \
DataType NonCommutative.\
\>", "Text",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["DataType[x,NonCommutative]", "Input",
  CellLabel->"In[1703]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[1703]=  (21:57:16 on 5/16/01)"]
}, Closed]],

Cell["The inverse operation is UnDeclareNonCommutative.", "Text",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", "x", "]"}]], "Input",
  CellLabel->"In[1704]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
  CellLabel->"In[1705]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[1705]=  (21:57:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[1706]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
  CellLabel->"In[1707]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1707]=  (21:57:16 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input",
  CellLabel->"In[1708]:=  (21:57:16 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
  CellLabel->"In[1709]:=  (21:57:17 on 5/16/01)",
  CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1709]=  (21:57:17 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Upper", "ObjectName",
  CellTags->"Upper"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Upper"],

Cell["\<\
Upper may be used inside LorentzIndex to indicate an contravariant \
LorentzIndex.\
\>", "Text",
  CellTags->"Upper"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Lower",
    ButtonData:>"Lower",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Lower"],
  ", ",
  ButtonBox["Contract1",
    ButtonData:>"Contract1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Contract1"],
  "."
}], "Text",
  CellTags->"Upper"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["UVPart  ***unfinished*** (examples missing)", "ObjectName",
  CellTags->"UVPart"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"UVPart"],

Cell["\<\
UVPart[exp, q] discards ultraviolet finite integrals  (q = \
integration momentum).\
\>", "Text",
  CellTags->"UVPart"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"UVPart"],

Cell[BoxData[""], "Input",
  CellLabel->"In[3]:=  (14:37:42 on 9/21/02)",
  CellTags->"UVPart"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Vectors", "ObjectName",
  CellTags->"Vectors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Vectors"],

Cell["\<\
Vectors is an option for FORM2FeynCalc. Its default setting is \
Automatic. It may be set to a list, if the FORM-file does not contain a \
V(ectors) statement.\
\>", "Text",
  CellTags->"Vectors"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FORM2FeynCalc"],
  "."
}], "Text",
  CellTags->"Vectors"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["WriteOut", "ObjectName",
  CellTags->"WriteOut"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"WriteOut"],

Cell["\<\
WriteOut is an option for OneLoop.  If set to True, the result of \
OneLoop will be written to a file called \"name.res\", where name is the \
first argument of OneLoop.\
\>", "Text",
  CellTags->"WriteOut"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"WriteOut"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["WriteOutPaVe", "ObjectName",
  CellTags->"WriteOutPaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"WriteOutPaVe"],

Cell["\<\
WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If set to \
a string, the results of all Passarino-Veltman PaVe's are stored in files \
with names generated from this string and the arguments of PaVe.\
\>", "Text",\

  CellTags->"WriteOutPaVe"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["OneLoopSum",
    ButtonData:>"OneLoopSum",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoopSum"],
  "."
}], "Text",
  CellTags->"WriteOutPaVe"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Write2", "ObjectName",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Write2"],

Cell["\<\
Write2[file, val1 = expr1, val2 = expr2, ...] writes the settings \
val1 = expr1, val2 = expr2 in sequence followed by a newline, to the \
specified output file. Setting the option FormatType of Write2 to FortranForm \
results in FORTRAN syntax output.\
\>", "Text",
  CellTags->"Write2"],

Cell["\<\
The continuation character for FORTRAN output can be controlled by \
changing $FortranContinuationCharacter.\
\>", "Text",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FullForm", "[", "$FortranContinuationCharacter", "]"}]], "Input",\

  CellLabel->"In[1710]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      TagBox[
        StyleBox["\"\<&\>\"",
          ShowSpecialCharacters->False,
          ShowStringCharacters->True,
          NumberMarks->True],
        FullForm], TraditionalForm]], "Output",
  CellLabel->"Out[1710]//FullForm=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Write2", "]"}]], "Input",
  CellLabel->"In[1711]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
          RowBox[{"D0Convention", "\[Rule]", "0"}], ",", 
          RowBox[{"PageWidth", "\[Rule]", "62"}], ",", 
          RowBox[{"PreFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"PostFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"StringReplace", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1711]=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Attributes", "[", "Write2", "]"}]], "Input",
  CellLabel->"In[1712]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "HoldRest", "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[1712]=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Isolate"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text",
  CellTags->"Write2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"Collect", "[", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"a", "-", "c"}], ")"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"a", "-", "b"}], ")"}], "2"]}], ")"}], "2"], ",", 
          "a", ",", "Factor"}], "]"}]}]], "Input",
  CellLabel->"In[1713]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SuperscriptBox["a", "4"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              RowBox[{"c", " ", "b"}], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              SuperscriptBox["c", "2"]}], ")"}], "2"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1713]=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"tempfilename", "=", 
        RowBox[{
          RowBox[{"ToString", "[", "$SessionID", "]"}], "<>", 
          "\"\<.s\>\""}]}], ";"}]], "Input",
  CellLabel->"In[1714]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell["This writes the assignment r=t to a file.", "Text",
  CellTags->"Write2"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1715]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell["This shows the contents of the file.", "Text",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}],
           ",", "String"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1716]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + \
\"\>"},
            {"\<\" (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)\"\>"},
            {"\<\"       );\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + ", 
          " (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)", "       );"}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1716]//TableForm=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeleteFile", "[", 
      RowBox[{
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
            "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
      "]"}]], "Input",
  CellLabel->"In[1717]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"x", "+", 
        RowBox[{"Isolate", "[", 
          RowBox[{"t", ",", "a", ",", 
            RowBox[{"IsolateNames", "\[Rule]", "W"}]}], "]"}]}]}]], "Input",
  CellLabel->"In[1718]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SuperscriptBox["a", "4"]}], "-", 
        RowBox[{"8", " ", 
          TagBox[
            RowBox[{"W", "(", "1", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{"8", " ", 
          TagBox[
            RowBox[{"W", "(", "3", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          TagBox[
            RowBox[{"W", "(", "1", ")"}],
            HoldForm], " ", 
          TagBox[
            RowBox[{"W", "(", "2", ")"}],
            HoldForm], " ", "a"}], "+", 
        SuperscriptBox[
          TagBox[
            RowBox[{"W", "(", "2", ")"}],
            HoldForm], "2"], "+", "x"}], TraditionalForm]], "Output",
  CellLabel->"Out[1718]=  (21:57:17 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t2"}]}], "]"}], ";"}]], "Input",
  CellLabel->"In[1719]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}],
           ",", "String"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1720]:=  (21:57:17 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"W[1] = (b + c\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"W[2] = (b^2 + c^2\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"W[3] = (b^2 + b*c + c^2\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"r = ( 4*a^4 + x - 8*a^3*HoldForm[W[1]] - \
4*a*HoldForm[W[1]]*\"\>"},
            {"\<\"  HoldForm[W[2]] + HoldForm[W[2]]^2 + \
8*a^2*HoldForm[W[3]]\"\>"},
            {"\<\"       );\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"W[1] = (b + c", "       );", "W[2] = (b^2 + c^2", 
          "       );", "W[3] = (b^2 + b*c + c^2", "       );", 
          "r = ( 4*a^4 + x - 8*a^3*HoldForm[W[1]] - 4*a*HoldForm[W[1]]*", 
          "  HoldForm[W[2]] + HoldForm[W[2]]^2 + 8*a^2*HoldForm[W[3]]", 
          "       );"}]], TraditionalForm]], "Output",
  CellLabel->"Out[1720]//TableForm=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{"DeleteFile", "[", 
      RowBox[{
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
            "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
      "]"}]], "Input",
  CellLabel->"In[1721]:=  (21:57:18 on 5/16/01)",
  CellTags->"Write2"],

Cell["\<\
This is how to write out the expression t2 in Fortran format.\
\>", \
"Text",
  CellTags->"Write2"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t2"}], ",", 
          RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], 
      ";"}]], "Input",
  CellLabel->"In[1722]:=  (21:57:18 on 5/16/01)",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}],
           ",", "String"}], "]"}], "]"}]], "Input",
  CellLabel->"In[1723]:=  (21:57:18 on 5/16/01)",
  CellTags->"Write2"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"        W(1)= b + c\"\>"},
            {"\<\"        W(2)= b**2 + c**2\"\>"},
            {"\<\"        W(3)= b**2 + b*c + c**2\"\>"},
            {"\<\"        r = 4*a**4 + x - 8*a**3*W(1) - 4*a*W(1)*W(2) + \
W(2)**2 + \"\>"},
            {"\<\"     &  8*a**2*W(3)\"\>"},
            {"\<\"                  \"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"        W(1)= b + c", "        W(2)= b**2 + c**2", 
          "        W(3)= b**2 + b*c + c**2", 
          "        r = 4*a**4 + x - 8*a**3*W(1) - 4*a*W(1)*W(2) + W(2)**2 + ",
           "     &  8*a**2*W(3)", "                  "}]], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1723]//TableForm=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "[", 
        RowBox[{
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
              "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
        "]"}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{"t", ",", "t2", ",", "tempfilename"}], "]"}], ";"}]], "Input",\

  CellLabel->"In[1724]:=  (21:57:18 on 5/16/01)",
  CellTags->"Write2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["XYT", "ObjectName",
  CellTags->"XYT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"XYT"],

Cell[TextData[{
  "XYT[exp, x,y] transforms  ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"x", " ", "y"}], ")"}], "m"], TraditionalForm]]],
  " away ..."
}], "Text",
  CellTags->"XYT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["ZeroMomentumInsertion", "ObjectName",
  CellTags->"ZeroMomentumInsertion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"ZeroMomentumInsertion"],

Cell["\<\
ZeroMomentumInsertion is an option of FeynRule, Twist2GluonOperator \
and Twist2QuarkOperator.\
\>", "Text",
  CellTags->"ZeroMomentumInsertion"],

Cell[TextData[{
  "See also:  ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynRule"],
  ", ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2GluonOperator"],
  ", ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text",
  CellTags->"ZeroMomentumInsertion"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Zeta2", "ObjectName",
  CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"Zeta2"],

Cell["Zeta2 denotes Zeta[2]. N[Zeta[2]] evaluates to N[Zeta[2]].", "Text",
  CellTags->"Zeta2"],

Cell[TextData[{
  "See also: ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text",
  CellTags->"Zeta2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell[BoxData["Zeta2"], "Input",
  CellLabel->"In[1725]:=  (21:57:18 on 5/16/01)",
  CellTags->"Zeta2"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Zeta]", "(", "2", ")"}], TraditionalForm]], "Output",
  CellLabel->"Out[1725]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", "Zeta2", "]"}]], "Input",
  CellLabel->"In[1726]:=  (21:57:18 on 5/16/01)",
  CellTags->"Zeta2"],

Cell[BoxData[
    FormBox["1.6449340668482262`", TraditionalForm]], "Output",
  CellLabel->"Out[1726]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyPolyLog", "[", 
      RowBox[{"Pi", "^", "2"}], "]"}]], "Input",
  CellLabel->"In[1727]:=  (21:57:18 on 5/16/01)",
  CellTags->"Zeta2"],

Cell[BoxData[
    FormBox[
      RowBox[{"6", " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
  CellLabel->"Out[1727]=  (21:57:18 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$Abbreviations", "ObjectName",
  CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$Abbreviations"],

Cell["\<\
$Abbreviations are a list of string substitution rules used when \
generating names for storing intermediate results. It is used by OneLoop and \
PaVeReduce.The elements of the list should be of the form \"name\" \[Rule] \
\"abbreviation\".\
\>", "Text",
  CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$Abbreviations", "//", "StandardForm"}]], "Input",
  CellLabel->"In[67]:=  (22:23:48 on 9/4/02)",
  CellTags->"$Abbreviations"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"\<\", \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"^\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"{\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"/\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"Subscript\"\>", "\[Rule]", "\<\"su\"\>"}], ",", 
        RowBox[{"\<\"SmallVariable\"\>", "\[Rule]", "\<\"sma\"\>"}], ",", 
        RowBox[{"\<\"}\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"[\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"]\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"*\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\" \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"\\n\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
        RowBox[{"\<\"\\r\"\>", "\[Rule]", "\<\"\"\>"}]}], "}"}]], "Output",
  CellLabel->"Out[67]//StandardForm=  (22:23:48 on 9/4/02)",
  CellTags->"$Abbreviations"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Abbreviation",
    ButtonData:>"Abbreviation",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Abbreviation"],
  ", ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["WriteOut",
    ButtonData:>"WriteOut",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"WriteOut"],
  ", ",
  ButtonBox["WriteOutPaVe",
    ButtonData:>"WriteOutPaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"WriteOutPaVe"],
  "."
}], "Text",
  CellTags->"$Abbreviations"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$AL", "ObjectName",
  CellTags->"$AL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$AL"],

Cell["\<\
$AL is the head for dummy indices which may be introduced by \
Uncontract.\
\>", "Text",
  CellTags->"$AL"],

Cell[CellGroupData[{

Cell[BoxData["$AL"], "Input",
  CellLabel->"In[43]:=  (22:09:10 on 9/4/02)",
  CellTags->"$AL"],

Cell[BoxData[
    FormBox["$AL", TraditionalForm]], "Output",
  CellLabel->"Out[43]=  (22:09:10 on 9/4/02)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
  CellLabel->"In[59]:=  (22:15:58 on 9/4/02)",
  CellTags->"$AL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$505", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$505", "\[InvisibleApplication]", 
                RowBox[{"(", "1", ")"}]}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[59]=  (22:15:58 on 9/4/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$AL", "=", "\[Mu]"}], ";"}]], "Input",
  CellLabel->"In[60]:=  (22:16:19 on 9/4/02)",
  CellTags->"$AL"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
  CellLabel->"In[61]:=  (22:16:22 on 9/4/02)",
  CellTags->"$AL"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"\[Mu]$506", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"\[Mu]$506", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[61]=  (22:16:22 on 9/4/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$AL", "=."}], ";"}]], "Input",
  CellLabel->"In[62]:=  (22:16:53 on 9/4/02)",
  CellTags->"$AL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$BreitMaison", "ObjectName",
  CellTags->"$BreitMaison"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$BreitMaison"],

Cell[TextData[{
  "The Breitenlohner-Maison ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " scheme is currently not supported by the Dirac algebra functions.Use \
Tracer if you need it (PHI supplies the functions FCToTracer and \
TracerToFC)."
}], "Text",
  CellTags->"$BreitMaison"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  ", ",
  ButtonBox["$Larin",
    ButtonData:>"$Larin",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$Larin"],
  ", ",
  ButtonBox["$West",
    ButtonData:>"$West",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$West"],
  "."
}], "Text",
  CellTags->"$BreitMaison"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$Color", "ObjectName",
  CellTags->"$Color"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$Color"],

Cell["\<\
$Color is False by default. If set to True, some special variables \
will be colored.\
\>", "Text",
  CellTags->"$Color"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$Covariant", "ObjectName",
  CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$Covariant"],

Cell["\<\
The boolean setting of $Covariant determines whether lorentz \
indices are displayed as lower indices (True) or as upper ones (False).\
\>", \
"Text",
  CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell[BoxData["$Covariant"], "Input",
  CellLabel->"In[20]:=  (22:39:55 on 9/4/02)",
  CellTags->"$Covariant"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[20]=  (22:39:55 on 9/4/02)",
  CellTags->"$Covariant"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[22]:=  (22:40:50 on 9/4/02)",
  CellTags->"$Covariant"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[22]=  (22:40:50 on 9/4/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$Covariant", "=", "True"}], ";"}]], "Input",
  CellLabel->"In[23]:=  (22:40:52 on 9/4/02)",
  CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
  CellLabel->"In[24]:=  (22:40:58 on 9/4/02)",
  CellTags->"$Covariant"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
  CellLabel->"Out[24]=  (22:40:58 on 9/4/02)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$Covariant", "=", "False"}], ";"}]], "Input",
  CellLabel->"In[25]:=  (22:41:02 on 9/4/02)",
  CellTags->"$Covariant"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$FCS", "ObjectName",
  CellTags->"$FCS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$FCS"],

Cell["\<\
$FCS is a list of functions with a short name. E.g. GA[nu] can be \
used instead of DiracGamma[nu].\
\>", "Text",
  CellTags->"$FCS"],

Cell[CellGroupData[{

Cell[BoxData["$FCS"], "Input",
  CellLabel->"In[10]:=  (22:38:18 on 9/4/02)",
  CellTags->"$FCS"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"\<\"FAD\"\>", ",", "\<\"FV\"\>", ",", "\<\"FVD\"\>", 
          ",", "\<\"GA\"\>", ",", "\<\"GA5\"\>", ",", "\<\"GS\"\>", 
          ",", "\<\"GSD\"\>", ",", "\<\"LC\"\>", ",", "\<\"LCD\"\>", 
          ",", "\<\"MT\"\>", ",", "\<\"MTD\"\>", ",", "\<\"SD\"\>", 
          ",", "\<\"SOD\"\>", ",", "\<\"SP\"\>", ",", "\<\"SPC\"\>", 
          ",", "\<\"SPD\"\>", ",", "\<\"SPL\"\>", ",", "\<\"FCI\"\>", 
          ",", "\<\"FCE\"\>", ",", "\<\"FI\"\>", ",", "\<\"FC\"\>", 
          ",", "\<\"GGV\"\>", ",", "\<\"GP\"\>", ",", "\<\"QGV\"\>", 
          ",", "\<\"QO\"\>"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[10]=  (22:38:18 on 9/4/02)",
  CellTags->"$FCS"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$FCT", "ObjectName",
  CellTags->"$FCT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$FCT"],

Cell["\<\
Setting the global variable $FCT to True changes the default \
typesetting rules for the noncommutative multiplication operator Dot[].\
\>", \
"Text",
  CellTags->"$FCT"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$FCT", " ", "=", " ", "True"}], ";"}]], "Input",
  CellLabel->"In[1728]:=  (21:57:18 on 5/16/01)",
  CellTags->"$FCT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
      RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[1729]:=  (21:57:18 on 5/16/01)",
  CellTags->"$FCT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1729]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$FCT", " ", "=", " ", "False"}], ";"}]], "Input",
  CellLabel->"In[1730]:=  (21:57:18 on 5/16/01)",
  CellTags->"$FCT"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
      RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
  CellLabel->"In[1731]:=  (21:57:18 on 5/16/01)",
  CellTags->"$FCT"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1731]=  (21:57:18 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$FeynCalcStuff", "ObjectName",
  CellTags->"$FeynCalcStuff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$FeynCalcStuff"],

Cell["$FeynCalcStuff is the list of availabe stuff in FeynCalc.", "Text",
  CellTags->"$FeynCalcStuff"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$FeynCalcVersion", "ObjectName",
  CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$FeynCalcVersion"],

Cell["\<\
$FeynCalcVersion is a string that represents the version of \
FeynCalc.\
\>", "Text",
  CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell[BoxData["$FeynCalcVersion"], "Input",
  CellLabel->"In[32]:=  (22:45:14 on 9/4/02)",
  CellTags->"$FeynCalcVersion"],

Cell[BoxData[
    FormBox["\<\"4.1.1\"\>", TraditionalForm]], "Output",
  CellLabel->"Out[32]=  (22:45:14 on 9/4/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$FortranContinuationCharacter", "ObjectName",
  CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$FortranContinuationCharacter"],

Cell["\<\
$FortranContinuationCharacter is the continuation character used in \
Write2.\
\>", "Text",
  CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell[BoxData["$FortranContinuationCharacter"], "Input",
  CellLabel->"In[35]:=  (22:46:53 on 9/4/02)",
  CellTags->"$FortranContinuationCharacter"],

Cell[BoxData[
    FormBox["\<\"&\"\>", TraditionalForm]], "Output",
  CellLabel->"Out[35]=  (22:46:53 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Write2"],
  "."
}], "Text",
  CellTags->"$FortranContinuationCharacter"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$Gauge", "ObjectName",
  CellTags->"$Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$Gauge"],

Cell["\<\
$Gauge(= 1/xi) is a constant specifying the gauge fixing parameter \
of QED in Lorentz gauge.  The usual choice is Feynman gauge, $Gauge=1. Notice \
that $Gauge is used by some functions, the option Gauge by others.\
\>", \
"Text",
  CellTags->"$Gauge"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Gauge",
    ButtonData:>"Gauge",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Gauge"],
  "."
}], "Text",
  CellTags->"$Gauge"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$Larin", "ObjectName",
  CellTags->"$Larin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$Larin"],

Cell[TextData[{
  "If set to True, the Larin-Gorishny-Atkyampo-DelBurgo-scheme for ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " in ",
  StyleBox["D",
    FontSlant->"Italic"],
  "-dimensions is used, i.e. before evaluating traces (but after moving ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " anticommuting in all dimensions to the right of the Dirac string) a \
product  gamma[mu].gamma5 is substituted to  -I/6 Eps[mu,al,be,si] \
gamma[al,be,si], where all indices live in ",
  StyleBox["D",
    FontSlant->"Italic"],
  "-dimensions now. Especially the Levic-Civita tensor is taken to be ",
  StyleBox["D",
    FontSlant->"Italic"],
  "-dimensional, i.e., contraction of two Eps's results in ",
  StyleBox["D",
    FontSlant->"Italic"],
  "'s. This has (FOR ONE AXIAL-VECTOR-CURRENT ONLY, it is not so clear if \
this scheme also works for more than one fermion line involving ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  ") the same effect as the Breitenlohner-Maison-'t Hooft-Veltman scheme."
}], "Text",
  CellTags->"$Larin"],

Cell[CellGroupData[{

Cell[BoxData["$Larin"], "Input",
  CellLabel->"In[40]:=  (22:52:58 on 9/4/02)",
  CellTags->"$Larin"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[40]=  (22:52:58 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also:  ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  ", ",
  ButtonBox["$BreitMaison",
    ButtonData:>"$BreitMaison",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$BreitMaison"],
  ", ",
  ButtonBox["$West",
    ButtonData:>"$West",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$West"],
  "."
}], "Text",
  CellTags->"$Larin"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$LimitTo4", "ObjectName",
  CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$LimitTo4"],

Cell["\<\
$LimitTo4 is a global variable with default setting True. If set to \
False, no limit Dimension \[Rule] 4 is performed after tensor integral \
decomposition.\
\>", "Text",
  CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell[BoxData["$LimitTo4"], "Input",
  CellLabel->"In[43]:=  (22:57:54 on 9/4/02)",
  CellTags->"$LimitTo4"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[43]=  (22:57:54 on 9/4/02)",
  CellTags->"$LimitTo4"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"PaVeReduce"],
  ", ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OneLoop"],
  "."
}], "Text",
  CellTags->"$LimitTo4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$LorentzIndices", "ObjectName",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$LorentzIndices"],

Cell["\<\
$LorentzIndices is a global variable. If set to True the dimension \
of LorentzIndex is displayed as an index.\
\>", "Text",
  CellTags->"$LorentzIndices"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
  CellLabel->"In[1732]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Alpha]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
  CellLabel->"In[1733]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Alpha]",
                "TraditionalForm"], 
              FormBox["n",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1733]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1734]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[1734]//StandardForm=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"p", ",", "\[Beta]"}], "]"}], 
        RowBox[{"GAD", "[", "\[Mu]", "]"}]}], "//", "FCI"}]], "Input",
  CellLabel->"In[1735]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Mu]",
                "TraditionalForm"], 
              FormBox["D",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["D",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["D",
                  "TraditionalForm"]],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            SubscriptBox[
              FormBox["\[Beta]",
                "TraditionalForm"], 
              FormBox["D",
                "TraditionalForm"]],
            "TraditionalForm"]]}], TraditionalForm]], "Output",
  CellLabel->"Out[1735]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input",
  CellLabel->"In[1736]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}]}], "]"}]}]], "Output",
  CellLabel->"Out[1736]//StandardForm=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
  CellLabel->"In[1737]:=  (21:57:18 on 5/16/01)",
  CellTags->"$LorentzIndices"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$MIntegrate", "ObjectName",
  CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$MIntegrate"],

Cell["\<\
$MIntegrate is a global list of integrations done by Mathematica \
inside OPEIntDelta.\
\>", "Text",
  CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell[BoxData["$MIntegrate"], "Input",
  CellLabel->"In[47]:=  (23:02:11 on 9/4/02)",
  CellTags->"$MIntegrate"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[47]=  (23:02:11 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEIntDelta",
    ButtonData:>"OPEIntDelta",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"OPEIntDelta"],
  "."
}], "Text",
  CellTags->"$MIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$MU", "ObjectName",
  CellTags->"$MU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$MU"],

Cell["\<\
$MU is the head for dummy indices which may be introduced by \
Chisholm (and evtl. Contract and DiracReduce).\
\>", "Text",
  CellTags->"$MU"],

Cell[CellGroupData[{

Cell[BoxData["$MU"], "Input",
  CellLabel->"In[54]:=  (23:07:57 on 9/4/02)",
  CellTags->"$MU"],

Cell[BoxData[
    FormBox["$MU", TraditionalForm]], "Output",
  CellLabel->"Out[54]=  (23:07:57 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Chisholm",
    ButtonData:>"Chisholm",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Chisholm"],
  "."
}], "Text",
  CellTags->"$MU"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$MemoryAvailable", "ObjectName",
  CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$MemoryAvailable"],

Cell[TextData[{
  "$MemoryAvailable is  a global variable which is set to an integer ",
  StyleBox["n",
    FontSlant->"Italic"],
  ", where ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is the available amount of main memory in MB. The default is 128. It \
should be increased if possible. The higher $MemoryAvailable can be, the more \
intermediate steps do not have to be repeated by FeynCalc."
}], "Text",
  CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell[BoxData["$MemoryAvailable"], "Input",
  CellLabel->"In[56]:=  (23:09:15 on 9/4/02)",
  CellTags->"$MemoryAvailable"],

Cell[BoxData[
    FormBox["256", TraditionalForm]], "Output",
  CellLabel->"Out[56]=  (23:09:15 on 9/4/02)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$NonComm", "ObjectName",
  CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$NonComm"],

Cell["\<\
$NonComm contains a list of all non-commutative heads \
present.\
\>", "Text",
  CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell[BoxData["$NonComm"], "Input",
  CellLabel->"In[68]:=  (23:14:49 on 9/4/02)",
  CellTags->"$NonComm"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "DiracGamma", ",", "DiracGammaT", ",", "DiracMatrix", ",", 
          "DiracSlash", ",", "DiracSigma", ",", "DiracSpinor", ",", "C", ",", 
          "FieldStrength", ",", "GA", ",", "GS", ",", "GSD", ",", 
          "RightPartialD", ",", "LeftRightPartialD", ",", 
          "LeftRightPartialD2", ",", "PartialD", ",", "QuantumField", ",", 
          "SUNT", ",", "Spinor", ",", "SpinorU", ",", "SpinorUBar", ",", 
          "SpinorV", ",", "SpinorVBar", ",", "ChiralityProjector", ",", 
          "OPESum"}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[68]=  (23:14:49 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["NonCommQ",
    ButtonData:>"NonCommQ",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NonCommQ"],
  ", ",
  ButtonBox["NonCommutative",
    ButtonData:>"NonCommutative",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"NonCommutative"],
  "."
}], "Text",
  CellTags->"$NonComm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$PairBrackets", "ObjectName",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$PairBrackets"],

Cell["\<\
$PairBrackets determines whether brackets are drawn around scalar \
products in the notebook interface.\
\>", "Text",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell[BoxData["$PairBrackets"], "Input",
  CellLabel->"In[1738]:=  (21:57:18 on 5/16/01)",
  CellTags->"$PairBrackets"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[1738]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], 
      RowBox[{"SP", "[", 
        RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
  CellLabel->"In[1739]:=  (21:57:18 on 5/16/01)",
  CellTags->"$PairBrackets"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], " ", 
        FormBox[
          RowBox[{
            FormBox["r",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["s",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1739]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$PairBrackets", "=", "True"}]], "Input",
  CellLabel->"In[1740]:=  (21:57:18 on 5/16/01)",
  CellTags->"$PairBrackets"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output",
  CellLabel->"Out[1740]=  (21:57:18 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], 
      RowBox[{"SP", "[", 
        RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
  CellLabel->"In[1741]:=  (21:57:18 on 5/16/01)",
  CellTags->"$PairBrackets"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], " ", 
        FormBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            
            FormBox["s",
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"]}], TraditionalForm]], "Output",
  CellLabel->"Out[1741]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$PairBrackets", "=", "False"}]], "Input",
  CellLabel->"In[1742]:=  (21:57:19 on 5/16/01)",
  CellTags->"$PairBrackets"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[1742]=  (21:57:19 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$SpinorMinimal", "ObjectName",
  CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$SpinorMinimal"],

Cell["\<\
$SpinorMinimal is a global switch for an additional simplification \
attempt in DiracSimplify for more than one Spinor-line. The default is False, \
since otherwise it costs too much time.\
\>", "Text",
  CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell[BoxData["$SpinorMinimal"], "Input",
  CellLabel->"In[76]:=  (23:20:02 on 9/4/02)",
  CellTags->"$SpinorMinimal"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output",
  CellLabel->"Out[76]=  (23:20:02 on 9/4/02)"]
}, Closed]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text",
  CellTags->"$SpinorMinimal"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$VeryVerbose", "ObjectName",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$VeryVerbose"],

Cell["\<\
$VeryVerbose is a global variable with default setting 0. If set to \
1, 2, ..., less and more intermediate comments and informations are displayed \
during calculations.\
\>", "Text",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell[BoxData["$VeryVerbose"], "Input",
  CellLabel->"In[1744]:=  (21:57:19 on 5/16/01)",
  CellTags->"$VeryVerbose"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output",
  CellLabel->"Out[1744]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$VeryVerbose", " ", "=", " ", "3"}], ";"}]], "Input",
  CellLabel->"In[1745]:=  (21:57:19 on 5/16/01)",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"x", "-", "y", "-", "z"}], ")"}], "^", "6"}], "]"}], 
        ",", "x"}], "]"}]], "Input",
  CellLabel->"In[1746]:=  (21:57:19 on 5/16/01)",
  CellTags->"$VeryVerbose"],

Cell["\<\
expanding. inhomogeneous part; LeafCount = 43; factored. \
#1#2#3#4#5#6\
\>", "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{"\<\"length \"\>", "\[InvisibleSpace]", "21", 
          "\[InvisibleSpace]", "\<\".\"\>"}],
        SequenceForm[ "length ", 21, "."],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{"\<\"collecting \"\>", "\[InvisibleSpace]", "6", 
          "\[InvisibleSpace]", "\<\" terms.\"\>"}],
        SequenceForm[ "collecting ", 6, " terms."],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["x", "6"], "-", 
        RowBox[{"6", " ", 
          RowBox[{"(", 
            RowBox[{"y", "+", "z"}], ")"}], " ", 
          SuperscriptBox["x", "5"]}], "+", 
        RowBox[{"15", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "2"], " ", 
          SuperscriptBox["x", "4"]}], "-", 
        RowBox[{"20", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "3"], " ", 
          SuperscriptBox["x", "3"]}], "+", 
        RowBox[{"15", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "4"], " ", 
          SuperscriptBox["x", "2"]}], "-", 
        RowBox[{"6", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "5"], " ", "x"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"y", "+", "z"}], ")"}], "6"]}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[1746]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$VeryVerbose", "=", "0"}], ";"}]], "Input",
  CellLabel->"In[1747]:=  (21:57:19 on 5/16/01)",
  CellTags->"$VeryVerbose"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["$V0",
    ButtonData:>"$V0",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"$V0"],
  "."
}], "Text",
  CellTags->"$V0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$West", "ObjectName",
  CellTags->"$West"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"$West"],

Cell[TextData[{
  "If $West is set to True (which is the default), traces involving more than \
4 Dirac matrices and ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " are calculated recursively according to formula (A.5) from Comp. Phys. \
Comm 77 (1993) 286-298, which is based on the Breitenlohner Maison ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "-scheme."
}], "Text",
  CellTags->"$West"],

Cell[TextData[{
  "See examples in ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Tr"],
  "."
}], "Text",
  CellTags->"$West"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Evaluation time and memory usage", "Commentary"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AbsoluteTime", "[", "]"}], "-", "absolutestarttime"}]], "Input",\

  CellLabel->"In[1749]:=  (21:57:19 on 5/16/01)"],

Cell[BoxData[
    FormBox["330.`8.2381", TraditionalForm]], "Output",
  CellLabel->"Out[1749]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MemoryInUse", "[", "]"}]], "Input",
  CellLabel->"In[1750]:=  (21:57:19 on 5/16/01)"],

Cell[BoxData[
    FormBox["15181296", TraditionalForm]], "Output",
  CellLabel->"Out[1750]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input",
  CellLabel->"In[1751]:=  (21:57:19 on 5/16/01)"],

Cell[BoxData[
    FormBox["19141128", TraditionalForm]], "Output",
  CellLabel->"Out[1751]=  (21:57:19 on 5/16/01)"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TimeUsed", "[", "]"}]], "Input",
  CellLabel->"In[1752]:=  (21:57:20 on 5/16/01)"],

Cell[BoxData[
    FormBox["219.15599999999986`", TraditionalForm]], "Output",
  CellLabel->"Out[1752]=  (21:57:20 on 5/16/01)"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Functions (experimental)", "Section",
  CellTags->"Experimental"],

Cell[CellGroupData[{

Cell["DoPolarizationSums", "ObjectName",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"DoPolarizationSums"],

Cell["\<\
***EXPERIMENTAL***
DoPolarizationSums[exp] sums over vector polarizations for expressions with a \
factor of the form
Pair[LorentzIndex[rho1_], Momentum[Polarization[p_, -I]]]*
Pair[LorentzIndex[rho2_], Momentum[Polarization[p_, I]]].\
\>", "Text",
  CellTags->"DoPolarizationSums"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Polarization"],
  ", ",
  ButtonBox["Uncontract",
    ButtonData:>"Uncontract",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"Uncontract"],
  "."
}], "Text",
  CellTags->"DoPolarizationSums"]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"DoPolarizationSums"],

Cell["\<\
In the following (and indeed everywhere else within FeynCalc), \
notice that Lorentz indices written as super or subscripts are not to be \
taken as such. Instead, by convention, when two indices are contracted one is \
always lower and the other upper.\
\>", "Text",
  CellTags->"DoPolarizationSums"],

Cell["\<\
FeynCalc uses the following normalization of Polarization \
vectors:\
\>", "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"Polarization", "[", 
              RowBox[{"p4", ",", 
                RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
      
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"Polarization", "[", 
              RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
        "]"}]}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubsuperscriptBox["\[CurlyEpsilon]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "*"], "(", 
          FormBox[
            SubscriptBox[
              TagBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], "(", 
          FormBox[
            SubscriptBox[
              TagBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    RowBox[{"%", "//", "Contract"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[TextData[{
  "DoPolarizationSums is chosen in such a way as to be consistent with this \
normalization and with ",
  Cell[BoxData[
      FormBox[
        UnderscriptBox["\[Sum]", "Polarizations"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[CurlyEpsilon]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", 
            FormBox[
              SubscriptBox[
                TagBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{
            SubscriptBox[
              RowBox[{"(", 
                SuperscriptBox["\[CurlyEpsilon]", "\[Nu]"], ")"}], 
              FormBox["",
                "TraditionalForm"]], "(", 
            FormBox[
              SubscriptBox[
                TagBox[
                  StyleBox["\<\"p\"\>",
                    FontSlant->"Italic"],
                  (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
              "TraditionalForm"], ")"}]}], TraditionalForm]]],
  "=",
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          FormBox[
            SuperscriptBox[
              SubscriptBox["g", "\[Mu]"], 
              RowBox[{"\[NoBreak]", 
                FormBox["\[Nu]",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}], TraditionalForm]]],
  ":"
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[TextData[{
  "We can choose e.g. the following polarization vectors, labeled with a \
subscript, {",
  Cell[BoxData[
      FormBox[
        SubscriptBox["e", "1"], TraditionalForm]]],
  ",",
  Cell[BoxData[
      FormBox[
        SubscriptBox["e", "2"], TraditionalForm]]],
  ",",
  Cell[BoxData[
      FormBox[
        SubscriptBox["e", "3"], TraditionalForm]]],
  ",",
  Cell[BoxData[
      FormBox[
        SubscriptBox["e", "4"], TraditionalForm]]],
  "} = {(\[ImaginaryI],0,0,0), (0,\[ImaginaryI],0,0), (0,0,\[ImaginaryI],0), \
(0,0,0,\[ImaginaryI])}."
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"ee", ",", "tt"}], "]"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ev", "[", "i_", "]"}], ":=", 
        RowBox[{"ReplacePart", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", "I", 
            ",", "i"}], "]"}]}], ";"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"ev", "[", "1", "]"}], ",", 
        RowBox[{"ev", "[", "2", "]"}], ",", 
        RowBox[{"ev", "[", "3", "]"}], ",", 
        RowBox[{"ev", "[", "4", "]"}]}], "}"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"\[ImaginaryI]", "0", "0", "0"},
            {"0", "\[ImaginaryI]", "0", "0"},
            {"0", "0", "\[ImaginaryI]", "0"},
            {"0", "0", "0", "\[ImaginaryI]"}
            },
          ColumnAlignments->{Decimal}], "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[TextData[{
  StyleBox["ee[i][\[Mu]]", "Input"],
  " is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
  ":"
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"ee", "[", "i_", "]"}], "[", "mu_", "]"}], ":=", 
        RowBox[{
          RowBox[{"ev", "[", "i", "]"}], "[", 
          RowBox[{"[", "mu", "]"}], "]"}]}], ";"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    RowBox[{
      RowBox[{"tt", "[", 
        RowBox[{"i_", ",", "j_", ",", "mu_", ",", "nu_"}], "]"}], ":=", 
      RowBox[{
        RowBox[{
          RowBox[{"ee", "[", "i", "]"}], "[", "mu", "]"}], 
        RowBox[{
          RowBox[{"ee", "[", "j", "]"}], "[", "nu", "]"}]}]}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[TextData[{
  StyleBox["tt[i,j,\[Mu],\[Nu]]", "Input"],
  " is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          SubscriptBox["e", "j"], "\[Nu]"], TraditionalForm]]],
  ":"
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        UnderscriptBox["\[Sum]", 
          RowBox[{"i", ",", "j"}]], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                SubscriptBox["e", "i"], "\[Mu]"],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["e", 
                RowBox[{"j", ",", "\[Nu]"}]],
              "TraditionalForm"]}]}], TraditionalForm]]],
  "=",
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          FormBox[
            SuperscriptBox[
              SubscriptBox["g", "\[Mu]"], 
              RowBox[{"\[NoBreak]", 
                FormBox["\[Nu]",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}], TraditionalForm]]],
  ":"
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "4"], 
          RowBox[{"tt", "[", 
            RowBox[{"i", ",", "i", ",", "mu", ",", "nu"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"mu", ",", "1", ",", "4"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"nu", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {
              RowBox[{"-", "1"}], "0", "0", "0"},
            {"0", 
              RowBox[{"-", "1"}], "0", "0"},
            {"0", "0", 
              RowBox[{"-", "1"}], "0"},
            {"0", "0", "0", 
              RowBox[{"-", "1"}]}
            },
          ColumnAlignments->{Decimal}], "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        UnderscriptBox["\[Sum]", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}]], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                SubscriptBox["e", "i"], "\[Mu]"],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["e", 
                RowBox[{"j", ",", "\[Nu]"}]],
              "TraditionalForm"]}]}], TraditionalForm]]],
  "=",
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          FormBox[
            SuperscriptBox[
              SubscriptBox["g", "i"], "j"],
            "TraditionalForm"]}], TraditionalForm]]],
  ":"
}], "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"mu", "=", "1"}], "4"], 
          RowBox[{"tt", "[", 
            RowBox[{"i", ",", "j", ",", "mu", ",", "mu"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {
              RowBox[{"-", "1"}], "0", "0", "0"},
            {"0", 
              RowBox[{"-", "1"}], "0", "0"},
            {"0", "0", 
              RowBox[{"-", "1"}], "0"},
            {"0", "0", "0", 
              RowBox[{"-", "1"}]}
            },
          ColumnAlignments->{Decimal}], "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest", "=", 
      RowBox[{
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}]}], "]"}], 
        " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", 
              RowBox[{"Polarization", "[", 
                RowBox[{"p4", ",", 
                  RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], 
        " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
              RowBox[{"Polarization", "[", 
                RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
          "]"}]}]}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", 
            FormBox[
              SuperscriptBox["\[Mu]", "*"],
              "TraditionalForm"]}]], " ", 
        RowBox[{
          SubsuperscriptBox["\[CurlyEpsilon]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "*"], "(", 
          FormBox[
            SubscriptBox[
              TagBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Mu]", "*"],
                "TraditionalForm"],
              "TraditionalForm"]], "(", 
          FormBox[
            SubscriptBox[
              TagBox[
                StyleBox["\<\"p\"\>",
                  FontSlant->"Italic"],
                (StyleForm[ #, FontSlant -> "Italic"]&)], "4"],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest", "//", "DoPolarizationSums"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox[
          FormBox[
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", 
                FormBox[
                  SuperscriptBox["\[Mu]", "*"],
                  "TraditionalForm"]}]],
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "Contract"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "4"}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest", "//", "Contract"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "DoPolarizationSums"}]], "Input",
  CellTags->"DoPolarizationSums"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "4"}], TraditionalForm]], "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"ee", ",", "tt", ",", "ev", ",", "ptest"}], "]"}]], "Input",
  CellLabel->"In[1701]:=  (21:57:16 on 5/16/01)",
  CellTags->"DoPolarizationSums"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynmanDoIntegrals ***unfinished***", "ObjectName",
  CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynmanDoIntegrals"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanDoIntegrals[exp, moms, vars] attempts to evaluate integrals over \
Feynman parameters vars in an expression exp as produced e.g. with \
FeynmanReduce. The variables given must be atomic quantities (AtomQ). If vars \
is omitted all variables in the integrals will be integrated. If vars is \
given, only the variables in vars will be integrated. moms is a list of all \
external momenta. The integrals in exp must be given in the form \
Integratedx[x, low, up].int, where x is the integration variable, low and up \
are the integration limits and int the integrand. The interval [low,up] is \
assumed to include integration bounds put by possible DeltaFunctions and \
moreover it is assumed that up \[GreaterEqual] 0 and that up > low. The two \
options FCIntegrate and FCNIntegrate determine which integration will be \
applied to integrals judged respectively analytically and numerically doable. \
This judging is a very rough one. Using (TimedIntegrate[##, \
Integrate\[Rule]Integrate]&) or (TimedIntegrate[##, \
Integrate\[Rule]NIntegrate]&) as the setting of one or both allows for finer \
judging. Those that are judged neither analytically nor numerically doable \
are left unevaluated, but can of course be attempted evaluated by a simple \
sustitution. Beside the explicit options of FeynmanDoIntegrals options of the \
integration functions specified (FCIntegrate and FCNIntegrate) may be given \
and are passed on to these.\
\>", "Text",
  CellTags->"FeynmanDoIntegrals"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynmanReduce",
    ButtonData:>"FeynmanReduce",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynmanReduce"],
  ", ",
  ButtonBox["FeynmanParametrize1",
    ButtonData:>"FeynmanParametrize1",
    ButtonStyle->"AddOnsLink",
    ButtonNote->"FeynmanParametrize1"],
  "."
}], "Text",
  CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynmanDoIntegrals", "]"}]], "Input",
  CellLabel->"In[2]:=  (23:51:12 on 9/19/02)",
  CellTags->"FeynmanDoIntegrals"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"NIntegrate", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"EpsilonOrder", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"-", "2"}], ",", "0"}], "}"}]}], ",", 
          RowBox[{"FCIntegrate", "\[Rule]", "TimedIntegrate"}], ",", 
          RowBox[{"Simplify", "\[Rule]", "True"}], ",", 
          RowBox[{"Expand", "\[Rule]", "True"}], ",", 
          RowBox[{"Series", "\[Rule]", "True"}], ",", 
          RowBox[{"FCNIntegrate", "\[Rule]", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"Sequence", "@@", 
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                          "HighEnergyPhysics`fcloops`FeynmanDoIntegrals`\
Private`Integratedx", "@@", "#1"}], "&"}], ")"}], "/@", 
                      RowBox[{"{", "##2", "}"}]}]}], ".", "#1"}], "&"}], 
              ")"}]}]}], "}"}], TraditionalForm]], "Output",
  CellLabel->"Out[2]=  (23:51:13 on 9/19/02)",
  CellTags->"FeynmanDoIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynmanDoIntegrals"],

Cell[BoxData[""], "Input",
  CellLabel->"In[635]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynmanDoIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynmanParametrize1 ***unfinished***", "ObjectName",
  CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynmanParametrize1"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanParametrize1[exp,k,Method->Denominator] introduces Feynman parameters \
for all one-loop integrals in exp (k = integration momentum) using formula \
(11.A.1) from \"The Quantum Theory of Fields\" vol. 1 by Steven Weinberg.  \
FeynmanParametrize1[exp,k,Method->Exp] introduces Feynman parameters for all \
one-loop integrals in exp (k = integration momentum) using 1/(A-I eps) = I \
Integrate[Exp[-I x (A-I eps)],{x,0,Infinity}, \
Assumptions->{Arg[A]==0,Arg[eps]==0}]. In this case, when the option \
Integrate is set to True, odd factors of k-tensors are dropped and even \
factors are replaced according to Itzykson&Zuber (8-117).\
\>", "Text",
  CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynmanParametrize1", "]"}]], "Input",
  CellLabel->"In[7]:=  (0:06:22 on 9/20/02)",
  CellTags->"FeynmanParametrize1"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FeynmanParameterNames", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "}"}]}], 
          ",", 
          RowBox[{"Method", "\[Rule]", "Denominator"}], ",", 
          RowBox[{"Integrate", "\[Rule]", "True"}], ",", 
          RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
          RowBox[{"CompleteSquare", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[7]=  (0:06:22 on 9/20/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynmanParametrize1"],

Cell[BoxData[""], "Input",
  CellLabel->"In[635]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynmanParametrize1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FeynmanReduce ***unfinished***", "ObjectName",
  CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
  CellTags->"FeynmanReduce"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanReduce[exp,params] takes a Feynman parameterized expression exp (as \
e.g. generated with FeynmanParametrize1) and a list of Feynman parameters as \
input and attempts to simplify the expression. If no parameters are given, \
Integratedx variables in the expression will be used. Currently, reduction of \
exponentials is implemented. This will work on terms of the form \
E^p1[a,b,c,...]*p2[a,b,c,...], where p1 and p2 are fractions of polynomials \
in the Feynman parameters a,b,c,... If the option Expand is set to True, \
FeynmanReduce will attempt to bring the expression exp into a sum of such \
terms and operate on the terms one by one.\
\>", "Text",
  CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynmanReduce", "]"}]], "Input",
  CellLabel->"In[9]:=  (0:07:39 on 9/20/02)",
  CellTags->"FeynmanReduce"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
          RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output",
  CellLabel->"Out[9]=  (0:07:39 on 9/20/02)"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"FeynmanReduce"],

Cell[BoxData[""], "Input",
  CellLabel->"In[635]:=  (21:53:32 on 5/16/01)",
  CellTags->"FeynmanReduce"]
}, Closed]]
}, Closed]],

Cell["\<\
Other objects that are considered to be ok or under active \
development:\
\>", "Text",
  CellTags->"Experimental"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
      "ChargeConjugationMatrix", ",", " ", "DiracGammaT", ",", " ", 
        "SquareAmplitude"}], "}"}]], "Input",
  CellTags->"Experimental"],

Cell["Objects that are under consideration for being discarded:", "Text",
  CellTags->"Experimental"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
      "$OPEWard", ",", "$AchmedRoss", ",", "Contract2", ",", "Contract3", ",",
         "SpinSumExternalMomentum", ",", "SelectedGraphs"}], "}"}]], "Input",
  CellTags->"Experimental"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"AllBrackets",
ShowPageBreaks->False,
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
CellGrouping->Manual,
WindowSize->{774, 671},
WindowMargins->{{Automatic, 96}, {Automatic, 4}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{597.563, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintSelectionHighlighting"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->True,
"FacingPages"->True,
"FirstPageFace"->Left,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{"MakeContentsNonDefaultOptions" -> {AutomaticCellTagging -> \
FromContent, PageNumbers -> False, SaveOptions -> True, SelectCellStyles -> \
{"Section", "Subsection"}, TOCCellStyles -> {"Contents", "Index"}}},
CellLabelAutoDelete->True,
ShowCellTags->True,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Functions"->{
    Cell[2365, 78, 53, 1, 70, "Section",
      CellTags->"Functions"]},
  "Abbreviation"->{
    Cell[2948, 105, 86, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"Abbreviation"],
    Cell[3059, 111, 64, 1, 49, "Subsubsection",
      CellTags->"Abbreviation"],
    Cell[3126, 114, 330, 6, 56, "Text",
      CellTags->"Abbreviation"],
    Cell[3459, 122, 1114, 43, 56, "Text",
      CellTags->"Abbreviation"],
    Cell[4610, 170, 61, 1, 49, "Subsubsection",
      CellTags->"Abbreviation"],
    Cell[4696, 175, 318, 8, 56, "Input",
      CellTags->"Abbreviation"]},
  "Amplitude"->{
    Cell[6007, 218, 56, 1, 39, "ObjectName",
      CellTags->"Amplitude"],
    Cell[6088, 223, 61, 1, 49, "Subsubsection",
      CellTags->"Amplitude"],
    Cell[6152, 226, 530, 9, 90, "Text",
      CellTags->"Amplitude"],
    Cell[6707, 239, 144, 3, 56, "Input",
      CellTags->"Amplitude"],
    Cell[6854, 244, 394, 10, 70, "Output",
      CellTags->"Amplitude"],
    Cell[7263, 257, 190, 9, 35, "Text",
      CellTags->"Amplitude"],
    Cell[7490, 271, 58, 1, 49, "Subsubsection",
      CellTags->"Amplitude"],
    Cell[7551, 274, 133, 3, 56, "Input",
      CellTags->"Amplitude"],
    Cell[7687, 279, 94, 1, 39, "Text",
      CellTags->"Amplitude"],
    Cell[7806, 284, 150, 3, 56, "Input",
      CellTags->"Amplitude"],
    Cell[7959, 289, 4845, 132, 70, "Output",
      CellTags->"Amplitude"],
    Cell[12841, 426, 137, 3, 52, "Input",
      CellTags->"Amplitude"],
    Cell[12981, 431, 12314, 304, 70, "Output",
      CellTags->"Amplitude"],
    Cell[25310, 738, 220, 8, 35, "Text",
      CellTags->"Amplitude"],
    Cell[25555, 750, 373, 9, 56, "Input",
      CellTags->"Amplitude"],
    Cell[25931, 761, 32526, 732, 70, "Output",
      CellTags->"Amplitude"]},
  "Amputate"->{
    Cell[58518, 1500, 54, 1, 39, "ObjectName",
      CellTags->"Amputate"],
    Cell[58597, 1505, 60, 1, 49, "Subsubsection",
      CellTags->"Amputate"],
    Cell[58660, 1508, 216, 5, 56, "Text",
      CellTags->"Amputate"],
    Cell[58901, 1517, 142, 3, 56, "Input",
      CellTags->"Amputate"],
    Cell[59399, 1534, 708, 28, 35, "Text",
      CellTags->"Amputate"],
    Cell[60144, 1567, 57, 1, 49, "Subsubsection",
      CellTags->"Amputate"],
    Cell[60226, 1572, 206, 5, 70, "Input",
      CellTags->"Amputate"],
    Cell[61080, 1601, 163, 4, 70, "Input",
      CellTags->"Amputate"]},
  "AnomalousDimension"->{
    Cell[62263, 1641, 74, 1, 39, "ObjectName",
      CellTags->"AnomalousDimension"],
    Cell[62362, 1646, 70, 1, 49, "Subsubsection",
      CellTags->"AnomalousDimension"],
    Cell[62435, 1649, 144, 4, 39, "Text",
      CellTags->"AnomalousDimension"],
    Cell[62604, 1657, 162, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[63057, 1672, 438, 18, 35, "Text",
      CellTags->"AnomalousDimension"],
    Cell[63532, 1695, 67, 1, 49, "Subsubsection",
      CellTags->"AnomalousDimension"],
    Cell[63602, 1698, 64, 1, 39, "Text",
      CellTags->"AnomalousDimension"],
    Cell[63691, 1703, 243, 5, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[64225, 1720, 249, 8, 70, "Text",
      CellTags->"AnomalousDimension"],
    Cell[64499, 1732, 161, 3, 70, "Input",
      CellTags->"AnomalousDimension"],
    Cell[65235, 1756, 248, 8, 70, "Text",
      CellTags->"AnomalousDimension"],
    Cell[65508, 1768, 160, 3, 70, "Input",
      CellTags->"AnomalousDimension"],
    Cell[66006, 1785, 248, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[66279, 1797, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[66776, 1814, 248, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[67049, 1826, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[67957, 1855, 249, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[68231, 1867, 161, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[69238, 1897, 249, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[69512, 1909, 161, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[78214, 2131, 248, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[78487, 2143, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[84081, 2289, 248, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[84354, 2301, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[92855, 2520, 248, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[93128, 2532, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[99809, 2706, 275, 8, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[100109, 2718, 160, 3, 56, "Input",
      CellTags->"AnomalousDimension"],
    Cell[106420, 2870, 341, 9, 38, "Text",
      CellTags->"AnomalousDimension"],
    Cell[106786, 2883, 362, 10, 56, "Input",
      CellTags->"AnomalousDimension"]},
  "AntiCommutator"->{
    Cell[107465, 2908, 66, 1, 39, "ObjectName",
      CellTags->"AntiCommutator"],
    Cell[107556, 2913, 66, 1, 49, "Subsubsection",
      CellTags->"AntiCommutator"],
    Cell[107625, 2916, 146, 4, 39, "Text",
      CellTags->"AntiCommutator"],
    Cell[107774, 2922, 642, 24, 39, "Text",
      CellTags->"AntiCommutator"],
    Cell[108453, 2951, 63, 1, 49, "Subsubsection",
      CellTags->"AntiCommutator"],
    Cell[108519, 2954, 210, 9, 70, "Text",
      CellTags->"AntiCommutator"],
    Cell[108732, 2965, 182, 4, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[108939, 2973, 176, 4, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[109411, 2990, 153, 3, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[109797, 3005, 347, 9, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[110710, 3037, 340, 9, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[111403, 3060, 261, 6, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[111667, 3068, 105, 1, 70, "Text",
      CellTags->"AntiCommutator"],
    Cell[111797, 3073, 215, 5, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[112160, 3087, 220, 5, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[112647, 3106, 304, 8, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[113126, 3124, 300, 7, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[113795, 3146, 283, 6, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[114081, 3154, 214, 5, 70, "Input",
      CellTags->"AntiCommutator"],
    Cell[114298, 3161, 276, 7, 70, "Input",
      CellTags->"AntiCommutator"]},
  "AntiQuarkField"->{
    Cell[114623, 3174, 66, 1, 39, "ObjectName",
      CellTags->"AntiQuarkField"],
    Cell[114714, 3179, 66, 1, 49, "Subsubsection",
      CellTags->"AntiQuarkField"],
    Cell[114783, 3182, 200, 5, 39, "Text",
      CellTags->"AntiQuarkField"],
    Cell[114986, 3189, 336, 14, 39, "Text",
      CellTags->"AntiQuarkField"],
    Cell[115359, 3208, 63, 1, 49, "Subsubsection",
      CellTags->"AntiQuarkField"],
    Cell[115447, 3213, 119, 2, 70, "Input",
      CellTags->"AntiQuarkField"]},
  "AntiSymmetrize"->{
    Cell[115770, 3227, 66, 1, 39, "ObjectName",
      CellTags->"AntiSymmetrize"],
    Cell[115861, 3232, 66, 1, 49, "Subsubsection",
      CellTags->"AntiSymmetrize"],
    Cell[115930, 3235, 154, 4, 39, "Text",
      CellTags->"AntiSymmetrize"],
    Cell[116087, 3241, 197, 8, 39, "Text",
      CellTags->"AntiSymmetrize"],
    Cell[116321, 3254, 63, 1, 49, "Subsubsection",
      CellTags->"AntiSymmetrize"],
    Cell[116409, 3259, 301, 8, 70, "Input",
      CellTags->"AntiSymmetrize"],
    Cell[117125, 3285, 321, 8, 70, "Input",
      CellTags->"AntiSymmetrize"]},
  "Anti5"->{
    Cell[118277, 3321, 48, 1, 39, "ObjectName",
      CellTags->"Anti5"],
    Cell[118350, 3326, 57, 1, 49, "Subsubsection",
      CellTags->"Anti5"],
    Cell[118410, 3329, 528, 16, 56, "Text",
      CellTags->"Anti5"],
    Cell[118941, 3347, 176, 7, 39, "Text",
      CellTags->"Anti5"],
    Cell[119120, 3356, 456, 19, 39, "Text",
      CellTags->"Anti5"],
    Cell[119613, 3380, 54, 1, 49, "Subsubsection",
      CellTags->"Anti5"],
    Cell[119692, 3385, 191, 5, 70, "Input",
      CellTags->"Anti5"],
    Cell[120130, 3402, 130, 3, 70, "Input",
      CellTags->"Anti5"],
    Cell[120654, 3422, 181, 5, 70, "Input",
      CellTags->"Anti5"],
    Cell[121193, 3443, 236, 6, 70, "Input",
      CellTags->"Anti5"],
    Cell[121827, 3465, 157, 4, 70, "Input",
      CellTags->"Anti5"],
    Cell[122824, 3500, 165, 4, 70, "Input",
      CellTags->"Anti5"],
    Cell[123829, 3535, 188, 5, 70, "Input",
      CellTags->"Anti5"],
    Cell[124835, 3569, 381, 14, 70, "Text",
      CellTags->"Anti5"],
    Cell[125241, 3587, 159, 4, 70, "Input",
      CellTags->"Anti5"],
    Cell[125652, 3603, 130, 3, 70, "Input",
      CellTags->"Anti5"]},
  "Apart1"->{
    Cell[126200, 3625, 50, 1, 39, "ObjectName",
      CellTags->"Apart1"],
    Cell[126275, 3630, 58, 1, 49, "Subsubsection",
      CellTags->"Apart1"],
    Cell[126336, 3633, 170, 4, 39, "Text",
      CellTags->"Apart1"],
    Cell[126509, 3639, 298, 14, 39, "Text",
      CellTags->"Apart1"],
    Cell[126844, 3658, 55, 1, 49, "Subsubsection",
      CellTags->"Apart1"],
    Cell[126902, 3661, 10585, 269, 70, "Input",
      CellTags->"Apart1"],
    Cell[137512, 3934, 259, 7, 70, "Input",
      CellTags->"Apart1"],
    Cell[138042, 3954, 258, 7, 70, "Input",
      CellTags->"Apart1"],
    Cell[138570, 3974, 158, 4, 70, "Input",
      CellTags->"Apart1"],
    Cell[138854, 3985, 171, 4, 70, "Input",
      CellTags->"Apart1"]},
  "Apart2"->{
    Cell[139074, 3995, 50, 1, 39, "ObjectName",
      CellTags->"Apart2"],
    Cell[139149, 4000, 58, 1, 49, "Subsubsection",
      CellTags->"Apart2"],
    Cell[139210, 4003, 111, 4, 39, "Text",
      CellTags->"Apart2"],
    Cell[139324, 4009, 325, 14, 39, "Text",
      CellTags->"Apart2"],
    Cell[139686, 4028, 55, 1, 49, "Subsubsection",
      CellTags->"Apart2"],
    Cell[139766, 4033, 316, 9, 70, "Input",
      CellTags->"Apart2"],
    Cell[141640, 4085, 133, 3, 70, "Input",
      CellTags->"Apart2"],
    Cell[142720, 4120, 173, 4, 70, "Input",
      CellTags->"Apart2"]},
  "Apart3"->{
    Cell[143581, 4151, 50, 1, 39, "ObjectName",
      CellTags->"Apart3"],
    Cell[143656, 4156, 58, 1, 49, "Subsubsection",
      CellTags->"Apart3"],
    Cell[143717, 4159, 123, 4, 39, "Text",
      CellTags->"Apart3"],
    Cell[143843, 4165, 184, 9, 39, "Text",
      CellTags->"Apart3"],
    Cell[144064, 4179, 55, 1, 49, "Subsubsection",
      CellTags->"Apart3"],
    Cell[144122, 4182, 10593, 269, 70, "Input",
      CellTags->"Apart3"],
    Cell[154740, 4455, 259, 7, 70, "Input",
      CellTags->"Apart3"],
    Cell[155270, 4475, 258, 7, 70, "Input",
      CellTags->"Apart3"],
    Cell[155795, 4495, 158, 4, 70, "Input",
      CellTags->"Apart3"],
    Cell[156079, 4506, 171, 4, 70, "Input",
      CellTags->"Apart3"]},
  "A0"->{
    Cell[156299, 4516, 66, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"A0"],
    Cell[156390, 4522, 54, 1, 49, "Subsubsection",
      CellTags->"A0"],
    Cell[156447, 4525, 381, 10, 37, "Text",
      CellTags->"A0"],
    Cell[156853, 4539, 135, 3, 56, "Input",
      CellTags->"A0"],
    Cell[157190, 4552, 480, 24, 35, "Text",
      CellTags->"A0"],
    Cell[157707, 4581, 51, 1, 49, "Subsubsection",
      CellTags->"A0"],
    Cell[157761, 4584, 270, 11, 39, "Text",
      CellTags->"A0"],
    Cell[158056, 4599, 152, 4, 57, "Input",
      CellTags->"A0"],
    Cell[158581, 4618, 229, 6, 52, "Input",
      CellTags->"A0"],
    Cell[158835, 4628, 152, 4, 57, "Input",
      CellTags->"A0"],
    Cell[159386, 4647, 230, 6, 52, "Input",
      CellTags->"A0"],
    Cell[159619, 4655, 241, 8, 35, "Text",
      CellTags->"A0"],
    Cell[159885, 4667, 124, 3, 56, "Input",
      CellTags->"A0"],
    Cell[160156, 4679, 197, 5, 52, "Input",
      CellTags->"A0"]},
  "A0ToB0"->{
    Cell[160524, 4695, 74, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"A0ToB0"],
    Cell[160623, 4701, 58, 1, 49, "Subsubsection",
      CellTags->"A0ToB0"],
    Cell[160684, 4704, 712, 27, 39, "Text",
      CellTags->"A0ToB0"],
    Cell[161421, 4735, 139, 3, 56, "Input",
      CellTags->"A0ToB0"],
    Cell[161763, 4748, 586, 29, 35, "Text",
      CellTags->"A0ToB0"]},
  "BackgroundGluonVertex"->{
    Cell[162398, 4783, 80, 1, 39, "ObjectName",
      CellTags->"BackgroundGluonVertex"],
    Cell[162503, 4788, 73, 1, 49, "Subsubsection",
      CellTags->"BackgroundGluonVertex"],
    Cell[162579, 4791, 1050, 19, 141, "Text",
      CellTags->"BackgroundGluonVertex"],
    Cell[163654, 4814, 169, 3, 56, "Input",
      CellTags->"BackgroundGluonVertex"],
    Cell[164192, 4831, 214, 9, 35, "Text",
      CellTags->"BackgroundGluonVertex"],
    Cell[164443, 4845, 70, 1, 49, "Subsubsection",
      CellTags->"BackgroundGluonVertex"],
    Cell[164538, 4850, 428, 11, 70, "Input",
      CellTags->"BackgroundGluonVertex"],
    Cell[168906, 4970, 513, 12, 70, "Input",
      CellTags->"BackgroundGluonVertex"],
    Cell[180388, 5258, 482, 11, 70, "Input",
      CellTags->"BackgroundGluonVertex"],
    Cell[185058, 5383, 569, 13, 70, "Input",
      CellTags->"BackgroundGluonVertex"]},
  "Bracket"->{
    Cell[196792, 5676, 52, 1, 39, "ObjectName",
      CellTags->"Bracket"],
    Cell[196869, 5681, 59, 1, 49, "Subsubsection",
      CellTags->"Bracket"],
    Cell[196931, 5684, 73, 1, 39, "Text",
      CellTags->"Bracket"],
    Cell[197007, 5687, 194, 9, 39, "Text",
      CellTags->"Bracket"]},
  "BReduce"->{
    Cell[197250, 5702, 76, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"BReduce"],
    Cell[197351, 5708, 59, 1, 49, "Subsubsection",
      CellTags->"BReduce"],
    Cell[197413, 5711, 149, 4, 39, "Text",
      CellTags->"BReduce"],
    Cell[197565, 5717, 587, 29, 39, "Text",
      CellTags->"BReduce"],
    Cell[198189, 5751, 56, 1, 49, "Subsubsection",
      CellTags->"BReduce"],
    Cell[198248, 5754, 275, 11, 39, "Text",
      CellTags->"BReduce"],
    Cell[198548, 5769, 166, 4, 56, "Input",
      CellTags->"BReduce"],
    Cell[199394, 5800, 95, 3, 35, "Text",
      CellTags->"BReduce"],
    Cell[199514, 5807, 220, 5, 56, "Input",
      CellTags->"BReduce"]},
  "B0"->{
    Cell[200157, 5831, 42, 1, 39, "ObjectName",
      CellTags->"B0"],
    Cell[200224, 5836, 54, 1, 49, "Subsubsection",
      CellTags->"B0"],
    Cell[200281, 5839, 403, 9, 56, "Text",
      CellTags->"B0"],
    Cell[200709, 5852, 131, 3, 56, "Input",
      CellTags->"B0"],
    Cell[201180, 5868, 486, 24, 35, "Text",
      CellTags->"B0"],
    Cell[201703, 5897, 51, 1, 49, "Subsubsection",
      CellTags->"B0"],
    Cell[201779, 5902, 288, 8, 70, "Input",
      CellTags->"B0"],
    Cell[202467, 5926, 287, 7, 70, "Input",
      CellTags->"B0"],
    Cell[202779, 5937, 192, 5, 70, "Input",
      CellTags->"B0"],
    Cell[203301, 5957, 222, 6, 70, "Input",
      CellTags->"B0"],
    Cell[203853, 5978, 220, 6, 70, "Input",
      CellTags->"B0"]},
  "B0Real"->{
    Cell[204385, 5999, 74, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"B0Real"],
    Cell[204484, 6005, 58, 1, 49, "Subsubsection",
      CellTags->"B0Real"],
    Cell[204545, 6008, 189, 5, 39, "Text",
      CellTags->"B0Real"],
    Cell[204737, 6015, 165, 8, 39, "Text",
      CellTags->"B0Real"],
    Cell[204939, 6028, 55, 1, 49, "Subsubsection",
      CellTags->"B0Real"],
    Cell[204997, 6031, 84, 1, 39, "Text",
      CellTags->"B0Real"],
    Cell[205106, 6036, 165, 4, 56, "Input",
      CellTags->"B0Real"],
    Cell[205505, 6052, 130, 3, 52, "Input",
      CellTags->"B0Real"],
    Cell[205973, 6068, 93, 3, 35, "Text",
      CellTags->"B0Real"],
    Cell[206091, 6075, 218, 5, 56, "Input",
      CellTags->"B0Real"]},
  "B0Unique"->{
    Cell[206600, 6095, 78, 2, 39, "ObjectName",
      PageBreakAbove->True,
      CellTags->"B0Unique"],
    Cell[206703, 6101, 60, 1, 49, "Subsubsection",
      CellTags->"B0Unique"],
    Cell[206766, 6104, 187, 5, 39, "Text",
      CellTags->"B0Unique"],
    Cell[206956, 6111, 167, 8, 39, "Text",
      CellTags->"B0Unique"],
    Cell[207160, 6124, 57, 1, 49, "Subsubsection",
      CellTags->"B0Unique"],
    Cell[207220, 6127, 74, 1, 39, "Text",
      CellTags->"B0Unique"],
    Cell[207319, 6132, 167, 4, 56, "Input",
      CellTags->"B0Unique"],
    Cell[207753, 6149, 132, 3, 52, "Input",
      CellTags->"B0Unique"],
    Cell[208223, 6165, 89, 3, 35, "Text",
      CellTags->"B0Unique"],
    Cell[208337, 6172, 223, 5, 56, "Input",
      CellTags->"B0Unique"]},
  "B00"->{
    Cell[208818, 6191, 44, 1, 39, "ObjectName",
      CellTags->"B00"],
    Cell[208887, 6196, 55, 1, 49, "Subsubsection",
      CellTags->"B00"],
    Cell[208945, 6199, 303, 8, 56, "Text",
      CellTags->"B00"],
    Cell[209273, 6211, 133, 3, 56, "Input",
      CellTags->"B00"],
    Cell[209610, 6224, 379, 19, 35, "Text",
      CellTags->"B00"],
    Cell[210026, 6248, 52, 1, 49, "Subsubsection",
      CellTags->"B00"],
    Cell[210081, 6251, 215, 7, 70, "Text",
      CellTags->"B00"],
    Cell[210321, 6262, 126, 3, 70, "Input",
      CellTags->"B00"],
    Cell[210670, 6277, 259, 7, 70, "Input",
      CellTags->"B00"],
    Cell[214693, 6381, 259, 7, 70, "Input",
      CellTags->"B00"],
    Cell[216953, 6449, 314, 8, 70, "Input",
      CellTags->"B00"],
    Cell[218058, 6486, 224, 6, 70, "Input",
      CellTags->"B00"],
    Cell[219200, 6523, 302, 8, 70, "Input",
      CellTags->"B00"]},
  "B1"->{
    Cell[220478, 6565, 42, 1, 39, "ObjectName",
      CellTags->"B1"],
    Cell[220545, 6570, 54, 1, 49, "Subsubsection",
      CellTags->"B1"],
    Cell[220602, 6573, 243, 7, 39, "Text",
      CellTags->"B1"],
    Cell[220870, 6584, 131, 3, 56, "Input",
      CellTags->"B1"],
    Cell[221205, 6597, 612, 29, 35, "Text",
      CellTags->"B1"],
    Cell[221854, 6631, 51, 1, 49, "Subsubsection",
      CellTags->"B1"],
    Cell[221930, 6636, 257, 7, 70, "Input",
      CellTags->"B1"],
    Cell[223729, 6691, 312, 8, 70, "Input",
      CellTags->"B1"],
    Cell[224441, 6715, 231, 6, 70, "Input",
      CellTags->"B1"],
    Cell[224697, 6725, 257, 7, 70, "Input",
      CellTags->"B1"],
    Cell[225474, 6753, 222, 6, 70, "Input",
      CellTags->"B1"],
    Cell[226239, 6780, 192, 5, 70, "Input",
      CellTags->"B1"],
    Cell[226974, 6806, 287, 7, 70, "Input",
      CellTags->"B1"],
    Cell[228267, 6844, 336, 9, 70, "Input",
      CellTags->"B1"]},
  "B11"->{
    Cell[229190, 6876, 44, 1, 39, "ObjectName",
      CellTags->"B11"],
    Cell[229259, 6881, 55, 1, 49, "Subsubsection",
      CellTags->"B11"],
    Cell[229317, 6884, 542, 17, 56, "Text",
      CellTags->"B11"],
    Cell[229884, 6905, 133, 3, 56, "Input",
      CellTags->"B11"],
    Cell[230221, 6918, 484, 24, 35, "Text",
      CellTags->"B11"],
    Cell[230742, 6947, 52, 1, 49, "Subsubsection",
      CellTags->"B11"],
    Cell[230797, 6950, 233, 7, 70, "Text",
      CellTags->"B11"],
    Cell[231055, 6961, 126, 3, 70, "Input",
      CellTags->"B11"],
    Cell[231404, 6976, 259, 7, 70, "Input",
      CellTags->"B11"],
    Cell[235618, 7086, 259, 7, 70, "Input",
      CellTags->"B11"],
    Cell[237553, 7146, 208, 5, 70, "Input",
      CellTags->"B11"],
    Cell[237988, 7163, 259, 7, 70, "Input",
      CellTags->"B11"],
    Cell[239038, 7199, 207, 5, 70, "Input",
      CellTags->"B11"],
    Cell[239471, 7216, 224, 6, 70, "Input",
      CellTags->"B11"],
    Cell[240045, 7237, 302, 8, 70, "Input",
      CellTags->"B11"]},
  "CA"->{
    Cell[240753, 7263, 42, 1, 39, "ObjectName",
      CellTags->"CA"],
    Cell[240820, 7268, 54, 1, 49, "Subsubsection",
      CellTags->"CA"],
    Cell[240877, 7271, 208, 9, 39, "Text",
      CellTags->"CA"],
    Cell[241088, 7282, 297, 14, 39, "Text",
      CellTags->"CA"],
    Cell[241422, 7301, 51, 1, 49, "Subsubsection",
      CellTags->"CA"],
    Cell[241498, 7306, 95, 2, 70, "Input",
      CellTags->"CA"],
    Cell[241767, 7318, 210, 5, 70, "Input",
      CellTags->"CA"],
    Cell[242125, 7332, 97, 2, 70, "Input",
      CellTags->"CA"]},
  "Calc"->{
    Cell[242394, 7345, 46, 1, 39, "ObjectName",
      CellTags->"Calc"],
    Cell[242465, 7350, 56, 1, 49, "Subsubsection",
      CellTags->"Calc"],
    Cell[242524, 7353, 234, 5, 56, "Text",
      CellTags->"Calc"],
    Cell[242761, 7360, 440, 19, 39, "Text",
      CellTags->"Calc"],
    Cell[243238, 7384, 62, 1, 50, "Subsubsection",
      CellTags->"Calc"],
    Cell[243303, 7387, 382, 13, 40, "Text",
      CellTags->"Calc"],
    Cell[243710, 7404, 347, 9, 56, "Input",
      CellTags->"Calc"],
    Cell[244239, 7422, 244, 9, 35, "Text",
      CellTags->"Calc"],
    Cell[244508, 7435, 312, 8, 56, "Input",
      CellTags->"Calc"],
    Cell[245326, 7464, 395, 11, 52, "Input",
      CellTags->"Calc"],
    Cell[246379, 7500, 129, 3, 52, "Input",
      CellTags->"Calc"],
    Cell[247324, 7532, 258, 6, 52, "Input",
      CellTags->"Calc"],
    Cell[249114, 7589, 476, 14, 52, "Input",
      CellTags->"Calc"]},
  "CalculateCounterTerm"->{
    Cell[249762, 7614, 95, 1, 39, "ObjectName",
      CellTags->"CalculateCounterTerm"],
    Cell[249882, 7619, 72, 1, 49, "Subsubsection",
      CellTags->"CalculateCounterTerm"],
    Cell[249957, 7622, 111, 1, 39, "Text",
      CellTags->"CalculateCounterTerm"],
    Cell[250105, 7628, 78, 1, 50, "Subsubsection",
      CellTags->"CalculateCounterTerm"],
    Cell[250208, 7633, 149, 3, 56, "Input",
      CellTags->"CalculateCounterTerm"]},
  "Info3241266069-5635180"->{
    Cell[250360, 7638, 133, 2, 37, "Print",
      CellTags->"Info3241266069-5635180"]},
  "CancelQP"->{
    Cell[250554, 7647, 54, 1, 39, "ObjectName",
      CellTags->"CancelQP"],
    Cell[250633, 7652, 60, 1, 49, "Subsubsection",
      CellTags->"CancelQP"],
    Cell[250696, 7655, 335, 13, 39, "Text",
      CellTags->"CancelQP"],
    Cell[251068, 7673, 66, 1, 50, "Subsubsection",
      CellTags->"CancelQP"],
    Cell[251159, 7678, 706, 20, 56, "Input",
      CellTags->"CancelQP"],
    Cell[251868, 7700, 945, 26, 71, "Output",
      CellTags->"CancelQP"],
    Cell[252850, 7731, 218, 5, 52, "Input",
      CellTags->"CancelQP"],
    Cell[253071, 7738, 2930, 74, 109, "Output",
      CellTags->"CancelQP"],
    Cell[256038, 7817, 219, 5, 52, "Input",
      CellTags->"CancelQP"],
    Cell[256260, 7824, 4327, 107, 109, "Output",
      CellTags->"CancelQP"],
    Cell[260624, 7936, 690, 19, 52, "Input",
      CellTags->"CancelQP"],
    Cell[261317, 7957, 1217, 36, 71, "Output",
      CellTags->"CancelQP"],
    Cell[262571, 7998, 218, 5, 52, "Input",
      CellTags->"CancelQP"],
    Cell[262792, 8005, 9798, 230, 123, "Output",
      CellTags->"CancelQP"],
    Cell[272627, 8240, 219, 5, 52, "Input",
      CellTags->"CancelQP"],
    Cell[272849, 8247, 11087, 261, 221, "Output",
      CellTags->"CancelQP"],
    Cell[283951, 8511, 133, 3, 52, "Input",
      CellTags->"CancelQP"]},
  "Cases2"->{
    Cell[284133, 8520, 50, 1, 39, "ObjectName",
      CellTags->"Cases2"],
    Cell[284208, 8525, 58, 1, 49, "Subsubsection",
      CellTags->"Cases2"],
    Cell[284269, 8528, 294, 6, 56, "Text",
      CellTags->"Cases2"],
    Cell[284600, 8539, 64, 1, 50, "Subsubsection",
      CellTags->"Cases2"],
    Cell[284689, 8544, 374, 10, 70, "Input",
      CellTags->"Cases2"],
    Cell[285425, 8570, 390, 11, 70, "Input",
      CellTags->"Cases2"],
    Cell[286181, 8597, 522, 14, 70, "Input",
      CellTags->"Cases2"],
    Cell[287158, 8629, 397, 10, 70, "Input",
      CellTags->"Cases2"]},
  "CF"->{
    Cell[288292, 8667, 42, 1, 39, "ObjectName",
      CellTags->"CF"],
    Cell[288359, 8672, 54, 1, 49, "Subsubsection",
      CellTags->"CF"],
    Cell[288416, 8675, 308, 13, 39, "Text",
      CellTags->"CF"],
    Cell[288727, 8690, 297, 14, 39, "Text",
      CellTags->"CF"],
    Cell[289061, 8709, 51, 1, 49, "Subsubsection",
      CellTags->"CF"],
    Cell[289137, 8714, 95, 2, 56, "Input",
      CellTags->"CF"],
    Cell[289406, 8726, 215, 5, 52, "Input",
      CellTags->"CF"],
    Cell[289884, 8744, 97, 2, 52, "Input",
      CellTags->"CF"],
    Cell[290129, 8755, 219, 5, 52, "Input",
      CellTags->"CF"]},
  "ChangeDimension"->{
    Cell[290546, 8772, 68, 1, 39, "ObjectName",
      CellTags->"ChangeDimension"],
    Cell[290639, 8777, 67, 1, 49, "Subsubsection",
      CellTags->"ChangeDimension"],
    Cell[290709, 8780, 211, 5, 56, "Text",
      CellTags->"ChangeDimension"],
    Cell[290923, 8787, 562, 24, 39, "Text",
      CellTags->"ChangeDimension"],
    Cell[291522, 8816, 64, 1, 49, "Subsubsection",
      CellTags->"ChangeDimension"],
    Cell[291589, 8819, 220, 5, 56, "Text",
      CellTags->"ChangeDimension"],
    Cell[291834, 8828, 254, 6, 56, "Input",
      CellTags->"ChangeDimension"],
    Cell[292349, 8848, 183, 4, 52, "Input",
      CellTags->"ChangeDimension"],
    Cell[292793, 8866, 144, 3, 52, "Input",
      CellTags->"ChangeDimension"],
    Cell[293250, 8881, 122, 4, 35, "Text",
      CellTags->"ChangeDimension"],
    Cell[293397, 8889, 242, 6, 56, "Input",
      CellTags->"ChangeDimension"],
    Cell[293890, 8905, 106, 1, 35, "Text",
      CellTags->"ChangeDimension"],
    Cell[294021, 8910, 582, 15, 56, "Input",
      CellTags->"ChangeDimension"],
    Cell[295350, 8950, 185, 4, 52, "Input",
      CellTags->"ChangeDimension"],
    Cell[295815, 8968, 265, 6, 52, "Input",
      CellTags->"ChangeDimension"],
    Cell[296338, 8986, 135, 4, 35, "Text",
      CellTags->"ChangeDimension"],
    Cell[296476, 8992, 251, 6, 56, "Input",
      CellTags->"ChangeDimension"],
    Cell[296730, 9000, 220, 5, 56, "Text",
      CellTags->"ChangeDimension"],
    Cell[296953, 9007, 244, 6, 56, "Input",
      CellTags->"ChangeDimension"],
    Cell[297222, 9017, 262, 6, 70, "Input",
      CellTags->"ChangeDimension"],
    Cell[298189, 9050, 219, 5, 70, "Input",
      CellTags->"ChangeDimension"],
    Cell[299113, 9082, 213, 5, 70, "Input",
      CellTags->"ChangeDimension"],
    Cell[299708, 9104, 173, 4, 70, "Input",
      CellTags->"ChangeDimension"],
    Cell[300030, 9116, 244, 6, 70, "Input",
      CellTags->"ChangeDimension"],
    Cell[300277, 9124, 194, 5, 70, "Input",
      CellTags->"ChangeDimension"]},
  "ChargeConjugationMatrix"->{
    Cell[300520, 9135, 84, 1, 39, "ObjectName",
      CellTags->"ChargeConjugationMatrix"],
    Cell[300629, 9140, 75, 1, 49, "Subsubsection",
      CellTags->"ChargeConjugationMatrix"],
    Cell[300707, 9143, 182, 6, 39, "Text",
      CellTags->"ChargeConjugationMatrix"],
    Cell[300892, 9151, 261, 9, 39, "Text",
      CellTags->"ChargeConjugationMatrix"],
    Cell[301190, 9165, 72, 1, 49, "Subsubsection",
      CellTags->"ChargeConjugationMatrix"],
    Cell[301287, 9170, 250, 5, 70, "Input",
      CellTags->"ChargeConjugationMatrix"],
    Cell[301924, 9192, 148, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrix"],
    Cell[302397, 9210, 245, 5, 70, "Input",
      CellTags->"ChargeConjugationMatrix"],
    Cell[302919, 9228, 148, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrix"],
    Cell[303305, 9243, 162, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrix"]},
  "ChargeConjugationMatrixInv"->{
    Cell[303661, 9258, 90, 1, 39, "ObjectName",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[303776, 9263, 78, 1, 49, "Subsubsection",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[303857, 9266, 198, 5, 39, "Text",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[304058, 9273, 255, 9, 39, "Text",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[304350, 9287, 75, 1, 49, "Subsubsection",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[304450, 9292, 253, 5, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[305090, 9314, 151, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[305566, 9332, 248, 5, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[306091, 9350, 151, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[306480, 9365, 195, 4, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[306905, 9381, 147, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"],
    Cell[307200, 9393, 168, 3, 70, "Input",
      CellTags->"ChargeConjugationMatrixInv"]},
  "CheckContext"->{
    Cell[307562, 9408, 62, 1, 39, "ObjectName",
      CellTags->"CheckContext"],
    Cell[307649, 9413, 64, 1, 49, "Subsubsection",
      CellTags->"CheckContext"],
    Cell[307716, 9416, 165, 4, 39, "Text",
      CellTags->"CheckContext"],
    Cell[307884, 9422, 343, 14, 39, "Text",
      CellTags->"CheckContext"],
    Cell[308264, 9441, 61, 1, 49, "Subsubsection",
      CellTags->"CheckContext"],
    Cell[308328, 9444, 143, 4, 39, "Text",
      CellTags->"CheckContext"],
    Cell[308496, 9452, 164, 3, 56, "Input",
      CellTags->"CheckContext"],
    Cell[308788, 9462, 88, 1, 35, "Text",
      CellTags->"CheckContext"],
    Cell[308901, 9467, 115, 2, 56, "Input",
      CellTags->"CheckContext"],
    Cell[309175, 9478, 164, 3, 52, "Input",
      CellTags->"CheckContext"]},
  "ChiralityProjector"->{
    Cell[309512, 9492, 74, 1, 39, "ObjectName",
      CellTags->"ChiralityProjector"],
    Cell[309611, 9497, 70, 1, 49, "Subsubsection",
      CellTags->"ChiralityProjector"],
    Cell[309684, 9500, 577, 19, 39, "Text",
      CellTags->"ChiralityProjector"],
    Cell[310264, 9521, 442, 19, 39, "Text",
      CellTags->"ChiralityProjector"],
    Cell[310743, 9545, 67, 1, 49, "Subsubsection",
      CellTags->"ChiralityProjector"],
    Cell[310835, 9550, 316, 8, 70, "Input",
      CellTags->"ChiralityProjector"],
    Cell[311440, 9572, 142, 3, 70, "Input",
      CellTags->"ChiralityProjector"],
    Cell[311871, 9589, 233, 5, 70, "Input",
      CellTags->"ChiralityProjector"]},
  "Chisholm"->{
    Cell[312611, 9616, 54, 1, 39, "ObjectName",
      CellTags->"Chisholm"],
    Cell[312690, 9621, 60, 1, 49, "Subsubsection",
      CellTags->"Chisholm"],
    Cell[312753, 9624, 143, 4, 39, "Text",
      CellTags->"Chisholm"],
    Cell[312899, 9630, 339, 14, 39, "Text",
      CellTags->"Chisholm"],
    Cell[313275, 9649, 57, 1, 49, "Subsubsection",
      CellTags->"Chisholm"],
    Cell[313357, 9654, 249, 6, 70, "Input",
      CellTags->"Chisholm"],
    Cell[316376, 9745, 220, 5, 70, "Input",
      CellTags->"Chisholm"],
    Cell[316895, 9763, 171, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[318971, 9826, 78, 1, 70, "Text",
      CellTags->"Chisholm"],
    Cell[319074, 9831, 162, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[319385, 9844, 171, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[321483, 9909, 162, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[321794, 9922, 206, 5, 70, "Input",
      CellTags->"Chisholm"],
    Cell[322896, 9958, 138, 3, 70, "Input",
      CellTags->"Chisholm"],
    Cell[325261, 10031, 280, 7, 70, "Input",
      CellTags->"Chisholm"],
    Cell[326012, 10055, 171, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[342197, 10505, 171, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[358360, 10953, 86, 1, 70, "Text",
      CellTags->"Chisholm"],
    Cell[358471, 10958, 162, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[358806, 10972, 162, 4, 70, "Input",
      CellTags->"Chisholm"],
    Cell[359119, 10984, 234, 6, 70, "Input",
      CellTags->"Chisholm"]},
  "ChisholmSpinor"->{
    Cell[359402, 10996, 66, 1, 39, "ObjectName",
      CellTags->"ChisholmSpinor"],
    Cell[359493, 11001, 66, 1, 49, "Subsubsection",
      CellTags->"ChisholmSpinor"],
    Cell[359562, 11004, 297, 6, 56, "Text",
      CellTags->"ChisholmSpinor"],
    Cell[359862, 11012, 327, 14, 39, "Text",
      CellTags->"ChisholmSpinor"],
    Cell[360226, 11031, 63, 1, 49, "Subsubsection",
      CellTags->"ChisholmSpinor"],
    Cell[360314, 11036, 364, 9, 56, "Input",
      CellTags->"ChisholmSpinor"],
    Cell[360681, 11047, 897, 23, 51, "Output",
      CellTags->"ChisholmSpinor"],
    Cell[361615, 11075, 142, 3, 56, "Input",
      CellTags->"ChisholmSpinor"],
    Cell[361760, 11080, 6642, 166, 119, "Output",
      CellTags->"ChisholmSpinor"]},
  "ClearScalarProducts"->{
    Cell[368463, 11253, 76, 1, 39, "ObjectName",
      CellTags->"ClearScalarProducts"],
    Cell[368564, 11258, 71, 1, 49, "Subsubsection",
      CellTags->"ClearScalarProducts"],
    Cell[368638, 11261, 146, 4, 39, "Text",
      CellTags->"ClearScalarProducts"],
    Cell[368787, 11267, 526, 23, 39, "Text",
      CellTags->"ClearScalarProducts"],
    Cell[369350, 11295, 68, 1, 49, "Subsubsection",
      CellTags->"ClearScalarProducts"],
    Cell[369443, 11300, 236, 6, 70, "Input",
      CellTags->"ClearScalarProducts"],
    Cell[369855, 11316, 253, 6, 70, "Input",
      CellTags->"ClearScalarProducts"],
    Cell[370262, 11330, 129, 2, 70, "Input",
      CellTags->"ClearScalarProducts"],
    Cell[370416, 11336, 253, 6, 70, "Input",
      CellTags->"ClearScalarProducts"],
    Cell[370892, 11354, 180, 4, 70, "Input",
      CellTags->"ClearScalarProducts"]},
  "Collecting"->{
    Cell[371319, 11372, 58, 1, 39, "ObjectName",
      CellTags->"Collecting"],
    Cell[371402, 11377, 62, 1, 49, "Subsubsection",
      CellTags->"Collecting"],
    Cell[371467, 11380, 208, 5, 39, "Text",
      CellTags->"Collecting"],
    Cell[371678, 11387, 443, 18, 39, "Text",
      CellTags->"Collecting"]},
  "Collect2"->{
    Cell[372170, 11411, 54, 1, 39, "ObjectName",
      CellTags->"Collect2"],
    Cell[372249, 11416, 60, 1, 49, "Subsubsection",
      CellTags->"Collect2"],
    Cell[372312, 11419, 614, 10, 90, "Text",
      CellTags->"Collect2"],
    Cell[372951, 11433, 143, 3, 56, "Input",
      CellTags->"Collect2"],
    Cell[373558, 11451, 189, 9, 35, "Text",
      CellTags->"Collect2"],
    Cell[373784, 11465, 57, 1, 49, "Subsubsection",
      CellTags->"Collect2"],
    Cell[373866, 11470, 542, 14, 66, "Input",
      CellTags->"Collect2"],
    Cell[375398, 11518, 222, 5, 52, "Input",
      CellTags->"Collect2"],
    Cell[376581, 11555, 223, 5, 52, "Input",
      CellTags->"Collect2"],
    Cell[377794, 11594, 652, 18, 53, "Input",
      CellTags->"Collect2"],
    Cell[379085, 11635, 388, 10, 53, "Input",
      CellTags->"Collect2"],
    Cell[380511, 11682, 140, 3, 52, "Input",
      CellTags->"Collect2"],
    Cell[381652, 11717, 162, 4, 52, "Input",
      CellTags->"Collect2"],
    Cell[381839, 11725, 377, 10, 57, "Input",
      CellTags->"Collect2"],
    Cell[383388, 11772, 181, 4, 52, "Input",
      CellTags->"Collect2"]},
  "Collect3"->{
    Cell[384417, 11806, 54, 1, 39, "ObjectName",
      CellTags->"Collect3"],
    Cell[384496, 11811, 60, 1, 49, "Subsubsection",
      CellTags->"Collect3"],
    Cell[384559, 11814, 810, 23, 56, "Text",
      CellTags->"Collect3"],
    Cell[385394, 11841, 141, 3, 56, "Input",
      CellTags->"Collect3"],
    Cell[385811, 11856, 309, 14, 35, "Text",
      CellTags->"Collect3"],
    Cell[386157, 11875, 57, 1, 49, "Subsubsection",
      CellTags->"Collect3"],
    Cell[386239, 11880, 650, 18, 57, "Input",
      CellTags->"Collect3"],
    Cell[387576, 11923, 360, 10, 53, "Input",
      CellTags->"Collect3"]},
  "Combinations"->{
    Cell[393558, 12077, 62, 1, 39, "ObjectName",
      CellTags->"Combinations"],
    Cell[393645, 12082, 64, 1, 49, "Subsubsection",
      CellTags->"Combinations"],
    Cell[393712, 12085, 245, 5, 56, "Text",
      CellTags->"Combinations"],
    Cell[393994, 12095, 61, 1, 49, "Subsubsection",
      CellTags->"Combinations"],
    Cell[394080, 12100, 302, 8, 56, "Input",
      CellTags->"Combinations"]},
  "Combine"->{
    Cell[394907, 12131, 52, 1, 39, "ObjectName",
      CellTags->"Combine"],
    Cell[394984, 12136, 59, 1, 49, "Subsubsection",
      CellTags->"Combine"],
    Cell[395046, 12139, 316, 6, 56, "Text",
      CellTags->"Combine"],
    Cell[395387, 12149, 141, 3, 56, "Input",
      CellTags->"Combine"],
    Cell[395735, 12162, 188, 9, 35, "Text",
      CellTags->"Combine"],
    Cell[395960, 12176, 56, 1, 49, "Subsubsection",
      CellTags->"Combine"],
    Cell[396041, 12181, 363, 11, 70, "Input",
      CellTags->"Combine"],
    Cell[396782, 12207, 125, 4, 70, "Text",
      CellTags->"Combine"],
    Cell[396932, 12215, 364, 11, 70, "Input",
      CellTags->"Combine"],
    Cell[397661, 12240, 176, 4, 70, "Text",
      CellTags->"Combine"],
    Cell[397862, 12248, 443, 12, 70, "Input",
      CellTags->"Combine"]},
  "CombineGraphs"->{
    Cell[398716, 12278, 64, 1, 39, "ObjectName",
      CellTags->"CombineGraphs"],
    Cell[398805, 12283, 65, 1, 49, "Subsubsection",
      CellTags->"CombineGraphs"],
    Cell[398873, 12286, 87, 1, 39, "Text",
      CellTags->"CombineGraphs"],
    Cell[398963, 12289, 197, 8, 39, "Text",
      CellTags->"CombineGraphs"]},
  "Commutator"->{
    Cell[399209, 12303, 58, 1, 39, "ObjectName",
      CellTags->"Commutator"],
    Cell[399292, 12308, 62, 1, 49, "Subsubsection",
      CellTags->"Commutator"],
    Cell[399357, 12311, 139, 4, 39, "Text",
      CellTags->"Commutator"],
    Cell[399499, 12317, 650, 24, 39, "Text",
      CellTags->"Commutator"],
    Cell[400186, 12346, 59, 1, 49, "Subsubsection",
      CellTags->"Commutator"],
    Cell[400248, 12349, 199, 4, 70, "Input",
      CellTags->"Commutator"],
    Cell[400472, 12357, 168, 4, 70, "Input",
      CellTags->"Commutator"],
    Cell[400881, 12373, 149, 3, 70, "Input",
      CellTags->"Commutator"],
    Cell[401263, 12388, 304, 8, 70, "Input",
      CellTags->"Commutator"],
    Cell[402012, 12412, 201, 4, 70, "Input",
      CellTags->"Commutator"],
    Cell[402216, 12418, 69, 1, 70, "Text",
      CellTags->"Commutator"],
    Cell[402288, 12421, 269, 6, 70, "Input",
      CellTags->"Commutator"],
    Cell[402582, 12431, 557, 15, 70, "Input",
      CellTags->"Commutator"],
    Cell[404066, 12475, 142, 3, 70, "Input",
      CellTags->"Commutator"],
    Cell[404334, 12485, 141, 3, 70, "Input",
      CellTags->"Commutator"],
    Cell[404478, 12490, 191, 4, 70, "Input",
      CellTags->"Commutator"]},
  "CommutatorExplicit"->{
    Cell[404718, 12500, 74, 1, 39, "ObjectName",
      CellTags->"CommutatorExplicit"],
    Cell[404817, 12505, 70, 1, 49, "Subsubsection",
      CellTags->"CommutatorExplicit"],
    Cell[404890, 12508, 158, 4, 39, "Text",
      CellTags->"CommutatorExplicit"],
    Cell[405051, 12514, 319, 14, 39, "Text",
      CellTags->"CommutatorExplicit"],
    Cell[405407, 12533, 67, 1, 49, "Subsubsection",
      CellTags->"CommutatorExplicit"],
    Cell[405477, 12536, 207, 4, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[405709, 12544, 176, 4, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[406126, 12560, 157, 3, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[406516, 12575, 238, 6, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[407194, 12599, 157, 3, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[407840, 12622, 203, 5, 70, "Input",
      CellTags->"CommutatorExplicit"],
    Cell[408488, 12643, 209, 4, 70, "Input",
      CellTags->"CommutatorExplicit"]},
  "CompleteSquare"->{
    Cell[408746, 12653, 66, 1, 39, "ObjectName",
      CellTags->"CompleteSquare"],
    Cell[408837, 12658, 66, 1, 49, "Subsubsection",
      CellTags->"CompleteSquare"],
    Cell[408906, 12661, 1277, 82, 56, "Text",
      CellTags->"CompleteSquare"],
    Cell[410220, 12748, 63, 1, 49, "Subsubsection",
      CellTags->"CompleteSquare"],
    Cell[411241, 12784, 206, 5, 52, "Input",
      CellTags->"CompleteSquare"],
    Cell[412372, 12823, 220, 6, 52, "Input",
      CellTags->"CompleteSquare"],
    Cell[412739, 12838, 186, 4, 52, "Input",
      CellTags->"CompleteSquare"],
    Cell[413588, 12866, 168, 4, 52, "Input",
      CellTags->"CompleteSquare"]},
  "ComplexConjugate"->{
    Cell[413805, 12876, 70, 1, 39, "ObjectName",
      CellTags->"ComplexConjugate"],
    Cell[413900, 12881, 68, 1, 49, "Subsubsection",
      CellTags->"ComplexConjugate"],
    Cell[413971, 12884, 417, 8, 73, "Text",
      CellTags->"ComplexConjugate"],
    Cell[414391, 12894, 482, 19, 39, "Text",
      CellTags->"ComplexConjugate"],
    Cell[414910, 12918, 65, 1, 49, "Subsubsection",
      CellTags->"ComplexConjugate"],
    Cell[415000, 12923, 233, 5, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[415625, 12945, 149, 3, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[416164, 12963, 184, 4, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[416652, 12980, 153, 3, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[417443, 13007, 219, 5, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[418437, 13041, 153, 3, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[419365, 13073, 227, 5, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[420141, 13101, 327, 8, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[420755, 13121, 323, 8, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[421668, 13149, 153, 3, 70, "Input",
      CellTags->"ComplexConjugate"],
    Cell[422545, 13174, 167, 4, 70, "Text",
      CellTags->"ComplexConjugate"]},
  "Contract"->{
    Cell[422761, 13184, 54, 1, 39, "ObjectName",
      CellTags->"Contract"],
    Cell[422840, 13189, 60, 1, 49, "Subsubsection",
      CellTags->"Contract"],
    Cell[422903, 13192, 414, 8, 73, "Text",
      CellTags->"Contract"],
    Cell[423342, 13204, 143, 3, 56, "Input",
      CellTags->"Contract"],
    Cell[424131, 13225, 456, 19, 35, "Text",
      CellTags->"Contract"],
    Cell[424624, 13249, 57, 1, 49, "Subsubsection",
      CellTags->"Contract"],
    Cell[424706, 13254, 279, 7, 56, "Input",
      CellTags->"Contract"],
    Cell[425277, 13275, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[425739, 13294, 240, 6, 52, "Input",
      CellTags->"Contract"],
    Cell[426333, 13316, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[426793, 13333, 176, 4, 52, "Input",
      CellTags->"Contract"],
    Cell[427171, 13347, 84, 1, 35, "Text",
      CellTags->"Contract"],
    Cell[427280, 13352, 137, 3, 56, "Input",
      CellTags->"Contract"],
    Cell[427543, 13362, 209, 5, 52, "Text",
      CellTags->"Contract"],
    Cell[427777, 13371, 277, 7, 56, "Input",
      CellTags->"Contract"],
    Cell[428392, 13393, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[428677, 13405, 277, 7, 52, "Input",
      CellTags->"Contract"],
    Cell[429292, 13427, 273, 7, 52, "Input",
      CellTags->"Contract"],
    Cell[429804, 13446, 142, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[430254, 13463, 335, 9, 52, "Input",
      CellTags->"Contract"],
    Cell[431235, 13497, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[432247, 13530, 323, 8, 52, "Input",
      CellTags->"Contract"],
    Cell[433375, 13567, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[434157, 13595, 400, 10, 52, "Input",
      CellTags->"Contract"],
    Cell[435921, 13651, 137, 3, 52, "Input",
      CellTags->"Contract"],
    Cell[436347, 13668, 508, 13, 52, "Input",
      CellTags->"Contract"],
    Cell[437355, 13699, 171, 4, 52, "Input",
      CellTags->"Contract"],
    Cell[438190, 13727, 237, 6, 52, "Input",
      CellTags->"Contract"]},
  "Contract1"->{
    Cell[438476, 13739, 56, 1, 39, "ObjectName",
      CellTags->"Contract1"],
    Cell[438557, 13744, 61, 1, 49, "Subsubsection",
      CellTags->"Contract1"],
    Cell[438621, 13747, 168, 4, 39, "Text",
      CellTags->"Contract1"],
    Cell[438792, 13753, 421, 18, 39, "Text",
      CellTags->"Contract1"],
    Cell[439250, 13776, 58, 1, 49, "Subsubsection",
      CellTags->"Contract1"],
    Cell[439333, 13781, 430, 12, 56, "Input",
      CellTags->"Contract1"],
    Cell[439766, 13795, 291, 8, 37, "Output",
      CellTags->"Contract1"],
    Cell[440094, 13808, 430, 12, 52, "Input",
      CellTags->"Contract1"],
    Cell[440527, 13822, 628, 20, 38, "Output",
      CellTags->"Contract1"],
    Cell[441192, 13847, 328, 9, 52, "Input",
      CellTags->"Contract1"],
    Cell[441523, 13858, 131, 3, 37, "Output",
      CellTags->"Contract1"],
    Cell[441691, 13866, 328, 9, 52, "Input",
      CellTags->"Contract1"],
    Cell[442022, 13877, 391, 11, 38, "Output",
      CellTags->"Contract1"]},
  "Convolute"->{
    Cell[442474, 13895, 56, 1, 39, "ObjectName",
      CellTags->"Convolute"],
    Cell[442555, 13900, 61, 1, 49, "Subsubsection",
      CellTags->"Convolute"],
    Cell[442619, 13903, 1660, 50, 58, "Text",
      CellTags->"Convolute"],
    Cell[444304, 13957, 145, 3, 56, "Input",
      CellTags->"Convolute"],
    Cell[444936, 13977, 355, 14, 35, "Text",
      CellTags->"Convolute"],
    Cell[445328, 13996, 58, 1, 49, "Subsubsection",
      CellTags->"Convolute"],
    Cell[445411, 14001, 166, 4, 56, "Input",
      CellTags->"Convolute"],
    Cell[445783, 14016, 166, 4, 52, "Input",
      CellTags->"Convolute"],
    Cell[446184, 14032, 166, 4, 52, "Input",
      CellTags->"Convolute"],
    Cell[446525, 14046, 224, 6, 64, "Input",
      CellTags->"Convolute"],
    Cell[447027, 14065, 277, 7, 64, "Input",
      CellTags->"Convolute"],
    Cell[447582, 14085, 224, 6, 64, "Input",
      CellTags->"Convolute"],
    Cell[448175, 14107, 282, 8, 64, "Input",
      CellTags->"Convolute"],
    Cell[449134, 14140, 233, 6, 52, "Input",
      CellTags->"Convolute"],
    Cell[449806, 14164, 266, 7, 52, "Input",
      CellTags->"Convolute"],
    Cell[450428, 14187, 291, 8, 64, "Input",
      CellTags->"Convolute"],
    Cell[451181, 14213, 324, 9, 64, "Input",
      CellTags->"Convolute"],
    Cell[452244, 14248, 295, 8, 64, "Input",
      CellTags->"Convolute"],
    Cell[453427, 14286, 233, 6, 52, "Input",
      CellTags->"Convolute"],
    Cell[453949, 14306, 233, 6, 52, "Input",
      CellTags->"Convolute"],
    Cell[454588, 14329, 320, 9, 64, "Input",
      CellTags->"Convolute"],
    Cell[455487, 14361, 202, 5, 52, "Input",
      CellTags->"Convolute"],
    Cell[455992, 14381, 238, 6, 52, "Input",
      CellTags->"Convolute"],
    Cell[456543, 14402, 260, 7, 64, "Input",
      CellTags->"Convolute"],
    Cell[457380, 14431, 262, 7, 64, "Input",
      CellTags->"Convolute"],
    Cell[458071, 14456, 291, 8, 64, "Input",
      CellTags->"Convolute"],
    Cell[459098, 14490, 262, 7, 64, "Input",
      CellTags->"Convolute"],
    Cell[459907, 14518, 233, 6, 52, "Input",
      CellTags->"Convolute"],
    Cell[460429, 14538, 202, 5, 52, "Input",
      CellTags->"Convolute"],
    Cell[460883, 14556, 353, 10, 64, "Input",
      CellTags->"Convolute"]},
  "ConvoluteTable"->{
    Cell[462354, 14605, 83, 1, 39, "ObjectName",
      CellTags->"ConvoluteTable"],
    Cell[462462, 14610, 66, 1, 49, "Subsubsection",
      CellTags->"ConvoluteTable"],
    Cell[462531, 14613, 104, 1, 39, "Text",
      CellTags->"ConvoluteTable"],
    Cell[462660, 14618, 150, 3, 56, "Input",
      CellTags->"ConvoluteTable"],
    Cell[462813, 14623, 499, 13, 70, "Output",
      CellTags->"ConvoluteTable"],
    Cell[463327, 14639, 345, 14, 35, "Text",
      CellTags->"ConvoluteTable"],
    Cell[463709, 14658, 63, 1, 49, "Subsubsection",
      CellTags->"ConvoluteTable"],
    Cell[463797, 14663, 175, 4, 56, "Input",
      CellTags->"ConvoluteTable"],
    Cell[464177, 14678, 175, 4, 52, "Input",
      CellTags->"ConvoluteTable"],
    Cell[464586, 14694, 175, 4, 52, "Input",
      CellTags->"ConvoluteTable"]},
  "CovariantFieldDerivative"->{
    Cell[464959, 14710, 86, 1, 39, "ObjectName",
      CellTags->"CovariantFieldDerivative"],
    Cell[465070, 14715, 76, 1, 49, "Subsubsection",
      CellTags->"CovariantFieldDerivative"],
    Cell[465149, 14718, 375, 7, 73, "Text",
      CellTags->"CovariantFieldDerivative"],
    Cell[465527, 14727, 493, 19, 39, "Text",
      CellTags->"CovariantFieldDerivative"],
    Cell[466057, 14751, 73, 1, 49, "Subsubsection",
      CellTags->"CovariantFieldDerivative"],
    Cell[466155, 14756, 377, 10, 43, "Input",
      CellTags->"CovariantFieldDerivative"]},
  "CounterT"->{
    Cell[467226, 14795, 54, 1, 39, "ObjectName",
      CellTags->"CounterT"],
    Cell[467305, 14800, 60, 1, 49, "Subsubsection",
      CellTags->"CounterT"],
    Cell[467368, 14803, 147, 4, 39, "Text",
      CellTags->"CounterT"],
    Cell[467518, 14809, 483, 19, 39, "Text",
      CellTags->"CounterT"],
    Cell[468038, 14833, 57, 1, 49, "Subsubsection",
      CellTags->"CounterT"],
    Cell[468098, 14836, 271, 5, 56, "Input",
      CellTags->"CounterT"]},
  "CounterTerm"->{
    Cell[468418, 14847, 60, 1, 39, "ObjectName",
      CellTags->"CounterTerm"],
    Cell[468503, 14852, 63, 1, 49, "Subsubsection",
      CellTags->"CounterTerm"],
    Cell[468569, 14855, 217, 5, 56, "Text",
      CellTags->"CounterTerm"],
    Cell[468789, 14862, 621, 24, 39, "Text",
      CellTags->"CounterTerm"],
    Cell[469447, 14891, 60, 1, 49, "Subsubsection",
      CellTags->"CounterTerm"],
    Cell[469532, 14896, 143, 3, 56, "Input",
      CellTags->"CounterTerm"]},
  "CouplingConstant"->{
    Cell[471892, 14960, 70, 1, 39, "ObjectName",
      CellTags->"CouplingConstant"],
    Cell[471987, 14965, 68, 1, 49, "Subsubsection",
      CellTags->"CouplingConstant"],
    Cell[472058, 14968, 658, 11, 115, "Text",
      CellTags->"CouplingConstant"],
    Cell[472719, 14981, 341, 14, 39, "Text",
      CellTags->"CouplingConstant"]},
  "CovariantD"->{
    Cell[473109, 15001, 58, 1, 39, "ObjectName",
      CellTags->"CovariantD"],
    Cell[473192, 15006, 62, 1, 49, "Subsubsection",
      CellTags->"CovariantD"],
    Cell[473257, 15009, 1213, 19, 216, "Text",
      CellTags->"CovariantD"],
    Cell[474495, 15032, 147, 3, 56, "Input",
      CellTags->"CovariantD"],
    Cell[475151, 15051, 188, 5, 35, "Text",
      CellTags->"CovariantD"],
    Cell[475342, 15058, 488, 19, 39, "Text",
      CellTags->"CovariantD"],
    Cell[475867, 15082, 59, 1, 49, "Subsubsection",
      CellTags->"CovariantD"],
    Cell[475951, 15087, 145, 3, 56, "Input",
      CellTags->"CovariantD"],
    Cell[476274, 15100, 182, 4, 52, "Input",
      CellTags->"CovariantD"],
    Cell[476682, 15115, 217, 5, 52, "Input",
      CellTags->"CovariantD"],
    Cell[478216, 15163, 165, 4, 35, "Text",
      CellTags->"CovariantD"],
    Cell[478406, 15171, 148, 3, 56, "Input",
      CellTags->"CovariantD"],
    Cell[478789, 15186, 185, 4, 52, "Input",
      CellTags->"CovariantD"],
    Cell[479257, 15203, 240, 5, 52, "Input",
      CellTags->"CovariantD"],
    Cell[480769, 15250, 220, 5, 52, "Input",
      CellTags->"CovariantD"],
    Cell[482230, 15296, 224, 5, 52, "Input",
      CellTags->"CovariantD"],
    Cell[484317, 15355, 675, 22, 38, "Text",
      CellTags->"CovariantD"],
    Cell[485017, 15381, 258, 6, 56, "Input",
      CellTags->"CovariantD"],
    Cell[485954, 15411, 372, 11, 35, "Text",
      CellTags->"CovariantD"],
    Cell[486351, 15426, 258, 6, 56, "Input",
      CellTags->"CovariantD"],
    Cell[489808, 15516, 207, 7, 35, "Text",
      CellTags->"CovariantD"],
    Cell[490040, 15527, 258, 6, 56, "Input",
      CellTags->"CovariantD"],
    Cell[499156, 15739, 218, 5, 52, "Input",
      CellTags->"CovariantD"],
    Cell[499737, 15760, 238, 6, 52, "Input",
      CellTags->"CovariantD"],
    Cell[506358, 15918, 270, 7, 52, "Input",
      CellTags->"CovariantD"],
    Cell[507596, 15955, 287, 7, 52, "Input",
      CellTags->"CovariantD"]},
  "CrossProduct"->{
    Cell[508881, 15995, 62, 1, 39, "ObjectName",
      CellTags->"CrossProduct"],
    Cell[508968, 16000, 64, 1, 49, "Subsubsection",
      CellTags->"CrossProduct"],
    Cell[509035, 16003, 146, 4, 39, "Text",
      CellTags->"CrossProduct"],
    Cell[509184, 16009, 331, 14, 39, "Text",
      CellTags->"CrossProduct"],
    Cell[509552, 16028, 61, 1, 49, "Subsubsection",
      CellTags->"CrossProduct"],
    Cell[509638, 16033, 389, 10, 56, "Input",
      CellTags->"CrossProduct"],
    Cell[510030, 16045, 893, 25, 52, "Output",
      CellTags->"CrossProduct"]},
  "C0"->{
    Cell[510984, 16077, 42, 1, 39, "ObjectName",
      CellTags->"C0"],
    Cell[511051, 16082, 54, 1, 49, "Subsubsection",
      CellTags->"C0"],
    Cell[511108, 16085, 461, 10, 73, "Text",
      CellTags->"C0"],
    Cell[511572, 16097, 501, 24, 39, "Text",
      CellTags->"C0"],
    Cell[512110, 16126, 51, 1, 49, "Subsubsection",
      CellTags->"C0"],
    Cell[512186, 16131, 217, 6, 56, "Input",
      CellTags->"C0"],
    Cell[512950, 16154, 245, 6, 52, "Input",
      CellTags->"C0"],
    Cell[513742, 16177, 404, 11, 52, "Input",
      CellTags->"C0"]},
  "DataType"->{
    Cell[514717, 16207, 54, 1, 39, "ObjectName",
      CellTags->"DataType"],
    Cell[514796, 16212, 60, 1, 49, "Subsubsection",
      CellTags->"DataType"],
    Cell[514859, 16215, 347, 7, 56, "Text",
      CellTags->"DataType"],
    Cell[515209, 16224, 896, 32, 56, "Text",
      CellTags->"DataType"],
    Cell[516108, 16258, 313, 13, 39, "Text",
      CellTags->"DataType"],
    Cell[516424, 16273, 231, 9, 39, "Text",
      CellTags->"DataType"],
    Cell[516692, 16287, 57, 1, 49, "Subsubsection",
      CellTags->"DataType"],
    Cell[516752, 16290, 75, 1, 70, "Text",
      CellTags->"DataType"],
    Cell[516830, 16293, 269, 7, 70, "Input",
      CellTags->"DataType"],
    Cell[517124, 16304, 279, 8, 56, "Input",
      CellTags->"DataType"],
    Cell[517406, 16314, 298, 9, 70, "Output",
      CellTags->"DataType"],
    Cell[517719, 16326, 182, 5, 52, "Text",
      CellTags->"DataType"],
    Cell[517926, 16335, 140, 3, 56, "Input",
      CellTags->"DataType"],
    Cell[518069, 16340, 258, 7, 70, "Output",
      CellTags->"DataType"],
    Cell[518342, 16350, 339, 10, 52, "Input",
      CellTags->"DataType"],
    Cell[518684, 16362, 518, 14, 56, "Input",
      CellTags->"DataType"],
    Cell[519205, 16378, 504, 14, 56, "Input",
      CellTags->"DataType"],
    Cell[519734, 16396, 426, 13, 56, "Input",
      CellTags->"DataType"],
    Cell[520163, 16411, 436, 13, 70, "Output",
      CellTags->"DataType"],
    Cell[520636, 16429, 135, 3, 52, "Input",
      CellTags->"DataType"],
    Cell[520774, 16434, 371, 11, 70, "Output",
      CellTags->"DataType"],
    Cell[521182, 16450, 135, 3, 52, "Input",
      CellTags->"DataType"],
    Cell[521320, 16455, 215, 6, 70, "Output",
      CellTags->"DataType"],
    Cell[521550, 16464, 234, 6, 52, "Input",
      CellTags->"DataType"],
    Cell[521787, 16472, 237, 7, 56, "Input",
      CellTags->"DataType"],
    Cell[522027, 16481, 550, 15, 56, "Input",
      CellTags->"DataType"],
    Cell[522602, 16500, 368, 12, 56, "Input",
      CellTags->"DataType"],
    Cell[522973, 16514, 372, 11, 70, "Output",
      CellTags->"DataType"],
    Cell[523382, 16530, 133, 3, 52, "Input",
      CellTags->"DataType"],
    Cell[523518, 16535, 290, 9, 70, "Output",
      CellTags->"DataType"],
    Cell[523823, 16547, 188, 5, 52, "Input",
      CellTags->"DataType"],
    Cell[524014, 16554, 270, 7, 56, "Input",
      CellTags->"DataType"],
    Cell[524287, 16563, 340, 10, 56, "Input",
      CellTags->"DataType"],
    Cell[524630, 16575, 117, 4, 39, "Text",
      CellTags->"DataType"],
    Cell[524772, 16583, 181, 4, 56, "Input",
      CellTags->"DataType"],
    Cell[524956, 16589, 135, 3, 70, "Output",
      CellTags->"DataType"],
    Cell[525106, 16595, 84, 1, 35, "Text",
      CellTags->"DataType"],
    Cell[525215, 16600, 287, 8, 56, "Input",
      CellTags->"DataType"],
    Cell[525505, 16610, 132, 3, 70, "Output",
      CellTags->"DataType"],
    Cell[525674, 16618, 283, 8, 52, "Input",
      CellTags->"DataType"],
    Cell[525960, 16628, 567, 17, 70, "Output",
      CellTags->"DataType"]},
  "DB0"->{
    Cell[526588, 16652, 44, 1, 39, "ObjectName",
      CellTags->"DB0"],
    Cell[526657, 16657, 55, 1, 49, "Subsubsection",
      CellTags->"DB0"],
    Cell[526715, 16660, 147, 4, 39, "Text",
      CellTags->"DB0"],
    Cell[526865, 16666, 169, 9, 39, "Text",
      CellTags->"DB0"],
    Cell[527071, 16680, 52, 1, 49, "Subsubsection",
      CellTags->"DB0"],
    Cell[527148, 16685, 422, 12, 57, "Input",
      CellTags->"DB0"]},
  "DB1"->{
    Cell[527930, 16714, 44, 1, 39, "ObjectName",
      CellTags->"DB1"],
    Cell[527999, 16719, 55, 1, 49, "Subsubsection",
      CellTags->"DB1"],
    Cell[528057, 16722, 120, 4, 39, "Text",
      CellTags->"DB1"],
    Cell[528180, 16728, 169, 9, 39, "Text",
      CellTags->"DB1"],
    Cell[528386, 16742, 52, 1, 49, "Subsubsection",
      CellTags->"DB1"],
    Cell[528463, 16747, 422, 12, 57, "Input",
      CellTags->"DB1"]},
  "DeclareNonCommutative"->{
    Cell[530638, 16811, 80, 1, 39, "ObjectName",
      CellTags->"DeclareNonCommutative"],
    Cell[530743, 16816, 73, 1, 49, "Subsubsection",
      CellTags->"DeclareNonCommutative"],
    Cell[530819, 16819, 195, 5, 39, "Text",
      CellTags->"DeclareNonCommutative"],
    Cell[531017, 16826, 370, 14, 39, "Text",
      CellTags->"DeclareNonCommutative"],
    Cell[531424, 16845, 70, 1, 49, "Subsubsection",
      CellTags->"DeclareNonCommutative"],
    Cell[531497, 16848, 163, 3, 70, "Input",
      CellTags->"DeclareNonCommutative"],
    Cell[531663, 16853, 153, 4, 70, "Text",
      CellTags->"DeclareNonCommutative"],
    Cell[531841, 16861, 190, 4, 70, "Input",
      CellTags->"DeclareNonCommutative"],
    Cell[532160, 16872, 190, 4, 70, "Input",
      CellTags->"DeclareNonCommutative"],
    Cell[532375, 16880, 227, 5, 70, "Input",
      CellTags->"DeclareNonCommutative"],
    Cell[532824, 16895, 202, 4, 70, "Input",
      CellTags->"DeclareNonCommutative"],
    Cell[533051, 16903, 227, 5, 70, "Input",
      CellTags->"DeclareNonCommutative"]},
  "DeltaFunction"->{
    Cell[533549, 16922, 64, 1, 39, "ObjectName",
      CellTags->"DeltaFunction"],
    Cell[533638, 16927, 65, 1, 49, "Subsubsection",
      CellTags->"DeltaFunction"],
    Cell[533706, 16930, 370, 11, 64, "Text",
      CellTags->"DeltaFunction"],
    Cell[534079, 16943, 635, 24, 39, "Text",
      CellTags->"DeltaFunction"],
    Cell[534751, 16972, 62, 1, 49, "Subsubsection",
      CellTags->"DeltaFunction"],
    Cell[534838, 16977, 174, 4, 56, "Input",
      CellTags->"DeltaFunction"],
    Cell[535277, 16994, 387, 10, 52, "Input",
      CellTags->"DeltaFunction"],
    Cell[535844, 17014, 354, 9, 52, "Input",
      CellTags->"DeltaFunction"],
    Cell[536378, 17033, 387, 10, 52, "Input",
      CellTags->"DeltaFunction"],
    Cell[536945, 17053, 247, 6, 52, "Input",
      CellTags->"DeltaFunction"]},
  "DeltaFunctionDoublePrime"->{
    Cell[537364, 17070, 86, 1, 39, "ObjectName",
      CellTags->"DeltaFunctionDoublePrime"],
    Cell[537475, 17075, 76, 1, 49, "Subsubsection",
      CellTags->"DeltaFunctionDoublePrime"],
    Cell[537554, 17078, 261, 8, 39, "Text",
      CellTags->"DeltaFunctionDoublePrime"],
    Cell[537818, 17088, 231, 8, 39, "Text",
      CellTags->"DeltaFunctionDoublePrime"]},
  "DeltaFunctionPrime"->{
    Cell[538098, 17102, 74, 1, 39, "ObjectName",
      CellTags->"DeltaFunctionPrime"],
    Cell[538197, 17107, 70, 1, 49, "Subsubsection",
      CellTags->"DeltaFunctionPrime"],
    Cell[538270, 17110, 240, 7, 39, "Text",
      CellTags->"DeltaFunctionPrime"],
    Cell[538513, 17119, 618, 23, 39, "Text",
      CellTags->"DeltaFunctionPrime"],
    Cell[539168, 17147, 67, 1, 49, "Subsubsection",
      CellTags->"DeltaFunctionPrime"],
    Cell[539260, 17152, 184, 4, 70, "Input",
      CellTags->"DeltaFunctionPrime"],
    Cell[539746, 17170, 397, 10, 70, "Input",
      CellTags->"DeltaFunctionPrime"],
    Cell[540402, 17193, 393, 10, 70, "Input",
      CellTags->"DeltaFunctionPrime"]},
  "DenominatorOrder"->{
    Cell[540967, 17214, 70, 1, 39, "ObjectName",
      CellTags->"DenominatorOrder"],
    Cell[541062, 17219, 68, 1, 49, "Subsubsection",
      CellTags->"DenominatorOrder"],
    Cell[541133, 17222, 176, 4, 39, "Text",
      CellTags->"DenominatorOrder"],
    Cell[541312, 17228, 349, 13, 39, "Text",
      CellTags->"DenominatorOrder"]},
  "Dimension"->{
    Cell[541710, 17247, 56, 1, 39, "ObjectName",
      CellTags->"Dimension"],
    Cell[541791, 17252, 61, 1, 49, "Subsubsection",
      CellTags->"Dimension"],
    Cell[541855, 17255, 247, 5, 56, "Text",
      CellTags->"Dimension"],
    Cell[542127, 17264, 148, 3, 56, "Input",
      CellTags->"Dimension"],
    Cell[542586, 17281, 420, 11, 52, "Input",
      CellTags->"Dimension"],
    Cell[543607, 17315, 132, 4, 35, "Text",
      CellTags->"Dimension"],
    Cell[543764, 17323, 429, 11, 56, "Input",
      CellTags->"Dimension"],
    Cell[544708, 17351, 167, 4, 35, "Text",
      CellTags->"Dimension"],
    Cell[544878, 17357, 165, 4, 56, "Input",
      CellTags->"Dimension"],
    Cell[545068, 17365, 384, 9, 56, "Input",
      CellTags->"Dimension"],
    Cell[546385, 17408, 138, 3, 52, "Input",
      CellTags->"Dimension"],
    Cell[547051, 17428, 166, 4, 52, "Input",
      CellTags->"Dimension"]},
  "DimensionalReduction"->{
    Cell[547266, 17438, 78, 1, 39, "ObjectName",
      CellTags->"DimensionalReduction"],
    Cell[547369, 17443, 72, 1, 49, "Subsubsection",
      CellTags->"DimensionalReduction"],
    Cell[547444, 17446, 124, 4, 39, "Text",
      CellTags->"DimensionalReduction"],
    Cell[547571, 17452, 323, 13, 39, "Text",
      CellTags->"DimensionalReduction"]},
  "DiracBasis"->{
    Cell[547943, 17471, 58, 1, 39, "ObjectName",
      CellTags->"DiracBasis"],
    Cell[548026, 17476, 62, 1, 49, "Subsubsection",
      CellTags->"DiracBasis"],
    Cell[548091, 17479, 300, 6, 56, "Text",
      CellTags->"DiracBasis"],
    Cell[548394, 17487, 196, 8, 39, "Text",
      CellTags->"DiracBasis"]},
  "DiracCanonical"->{
    Cell[548639, 17501, 66, 1, 39, "ObjectName",
      CellTags->"DiracCanonical"],
    Cell[548730, 17506, 66, 1, 49, "Subsubsection",
      CellTags->"DiracCanonical"],
    Cell[548799, 17509, 172, 4, 39, "Text",
      CellTags->"DiracCanonical"],
    Cell[548974, 17515, 206, 8, 39, "Text",
      CellTags->"DiracCanonical"]},
  "DiracEquation"->{
    Cell[549229, 17529, 64, 1, 39, "ObjectName",
      CellTags->"DiracEquation"],
    Cell[549318, 17534, 65, 1, 49, "Subsubsection",
      CellTags->"DiracEquation"],
    Cell[549386, 17537, 161, 4, 39, "Text",
      CellTags->"DiracEquation"],
    Cell[549550, 17543, 212, 9, 39, "Text",
      CellTags->"DiracEquation"],
    Cell[549799, 17557, 62, 1, 49, "Subsubsection",
      CellTags->"DiracEquation"],
    Cell[549864, 17560, 335, 9, 56, "Input",
      CellTags->"DiracEquation"],
    Cell[550202, 17571, 144, 3, 56, "Input",
      CellTags->"DiracEquation"]},
  "DiracGamma"->{
    Cell[550395, 17580, 58, 1, 39, "ObjectName",
      CellTags->"DiracGamma"],
    Cell[550478, 17585, 62, 1, 49, "Subsubsection",
      CellTags->"DiracGamma"],
    Cell[550543, 17588, 852, 25, 73, "Text",
      CellTags->"DiracGamma"],
    Cell[551398, 17615, 734, 29, 39, "Text",
      CellTags->"DiracGamma"],
    Cell[552169, 17649, 59, 1, 49, "Subsubsection",
      CellTags->"DiracGamma"],
    Cell[552253, 17654, 141, 3, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[552577, 17667, 191, 4, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[553034, 17683, 368, 13, 70, "Text",
      CellTags->"DiracGamma"],
    Cell[553427, 17700, 204, 5, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[553954, 17719, 332, 8, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[555015, 17753, 270, 8, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[555608, 17775, 256, 7, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[556425, 17803, 176, 4, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[557513, 17838, 358, 9, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[558534, 17870, 170, 4, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[559905, 17911, 173, 4, 70, "Input",
      CellTags->"DiracGamma"],
    Cell[561769, 17966, 205, 5, 70, "Input",
      CellTags->"DiracGamma"]},
  "DiracGammaCombine"->{
    Cell[562023, 17977, 72, 1, 39, "ObjectName",
      CellTags->"DiracGammaCombine"],
    Cell[562120, 17982, 69, 1, 49, "Subsubsection",
      CellTags->"DiracGammaCombine"],
    Cell[562192, 17985, 136, 4, 39, "Text",
      CellTags->"DiracGammaCombine"],
    Cell[562331, 17991, 867, 34, 39, "Text",
      CellTags->"DiracGammaCombine"],
    Cell[563235, 18030, 66, 1, 49, "Subsubsection",
      CellTags->"DiracGammaCombine"],
    Cell[563326, 18035, 262, 6, 56, "Input",
      CellTags->"DiracGammaCombine"],
    Cell[563991, 18058, 150, 3, 52, "Input",
      CellTags->"DiracGammaCombine"],
    Cell[564379, 18072, 324, 8, 52, "Input",
      CellTags->"DiracGammaCombine"],
    Cell[565172, 18099, 150, 3, 52, "Input",
      CellTags->"DiracGammaCombine"],
    Cell[565622, 18115, 155, 3, 52, "Input",
      CellTags->"DiracGammaCombine"]},
  "DiracGammaExpand"->{
    Cell[566450, 18144, 70, 1, 39, "ObjectName",
      CellTags->"DiracGammaExpand"],
    Cell[566545, 18149, 68, 1, 49, "Subsubsection",
      CellTags->"DiracGammaExpand"],
    Cell[566616, 18152, 193, 5, 56, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[566812, 18159, 869, 34, 39, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[567718, 18198, 65, 1, 49, "Subsubsection",
      CellTags->"DiracGammaExpand"],
    Cell[567808, 18203, 375, 9, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[568890, 18236, 128, 4, 70, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[569043, 18244, 149, 3, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[569521, 18259, 102, 1, 70, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[569648, 18264, 153, 3, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[570691, 18296, 91, 1, 70, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[570807, 18301, 246, 5, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[571404, 18320, 196, 4, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[572052, 18339, 107, 1, 70, "Text",
      CellTags->"DiracGammaExpand"],
    Cell[572184, 18344, 150, 3, 70, "Input",
      CellTags->"DiracGammaExpand"],
    Cell[573099, 18373, 142, 3, 70, "Input",
      CellTags->"DiracGammaExpand"]},
  "DiracGammaT"->{
    Cell[573290, 18382, 60, 1, 39, "ObjectName",
      CellTags->"DiracGammaT"],
    Cell[573375, 18387, 63, 1, 49, "Subsubsection",
      CellTags->"DiracGammaT"],
    Cell[573441, 18390, 209, 5, 56, "Text",
      CellTags->"DiracGammaT"],
    Cell[573653, 18397, 201, 9, 39, "Text",
      CellTags->"DiracGammaT"],
    Cell[573891, 18411, 60, 1, 49, "Subsubsection",
      CellTags->"DiracGammaT"],
    Cell[573976, 18416, 190, 4, 56, "Input",
      CellTags->"DiracGammaT"],
    Cell[574459, 18434, 141, 3, 52, "Input",
      CellTags->"DiracGammaT"],
    Cell[574885, 18451, 209, 6, 52, "Input",
      CellTags->"DiracGammaT"]},
  "DiracMatrix"->{
    Cell[575485, 18475, 60, 1, 39, "ObjectName",
      CellTags->"DiracMatrix"],
    Cell[575570, 18480, 63, 1, 49, "Subsubsection",
      CellTags->"DiracMatrix"],
    Cell[575636, 18483, 1375, 48, 56, "Text",
      CellTags->"DiracMatrix"],
    Cell[577014, 18533, 1146, 49, 39, "Text",
      CellTags->"DiracMatrix"],
    Cell[578197, 18587, 60, 1, 49, "Subsubsection",
      CellTags->"DiracMatrix"],
    Cell[578282, 18592, 147, 3, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[578692, 18607, 640, 20, 70, "Text",
      CellTags->"DiracMatrix"],
    Cell[579357, 18631, 220, 5, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[580046, 18656, 196, 5, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[580462, 18671, 177, 4, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[580826, 18685, 140, 3, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[581132, 18697, 139, 3, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[581458, 18710, 170, 4, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[581905, 18726, 184, 4, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[582398, 18743, 203, 5, 70, "Input",
      CellTags->"DiracMatrix"],
    Cell[583062, 18768, 140, 3, 70, "Input",
      CellTags->"DiracMatrix"]},
  "DiracOrder"->{
    Cell[583554, 18786, 58, 1, 39, "ObjectName",
      CellTags->"DiracOrder"],
    Cell[583637, 18791, 62, 1, 49, "Subsubsection",
      CellTags->"DiracOrder"],
    Cell[583702, 18794, 205, 5, 39, "Text",
      CellTags->"DiracOrder"],
    Cell[583910, 18801, 335, 14, 39, "Text",
      CellTags->"DiracOrder"],
    Cell[584282, 18820, 59, 1, 49, "Subsubsection",
      CellTags->"DiracOrder"],
    Cell[584366, 18825, 213, 5, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[584843, 18843, 142, 3, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[585602, 18868, 94, 1, 70, "Text",
      CellTags->"DiracOrder"],
    Cell[585721, 18873, 213, 5, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[586242, 18891, 142, 3, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[587023, 18918, 208, 5, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[587513, 18934, 215, 8, 70, "Text",
      CellTags->"DiracOrder"],
    Cell[587753, 18946, 142, 3, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[588411, 18970, 204, 5, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[588915, 18988, 142, 3, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[589416, 19007, 229, 6, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[589941, 19025, 319, 10, 70, "Text",
      CellTags->"DiracOrder"],
    Cell[590285, 19039, 266, 7, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[592185, 19098, 141, 3, 70, "Input",
      CellTags->"DiracOrder"],
    Cell[592938, 19124, 229, 6, 70, "Input",
      CellTags->"DiracOrder"]},
  "DiracReduce"->{
    Cell[593216, 19136, 60, 1, 39, "ObjectName",
      CellTags->"DiracReduce"],
    Cell[593301, 19141, 63, 1, 49, "Subsubsection",
      CellTags->"DiracReduce"],
    Cell[593367, 19144, 663, 11, 90, "Text",
      CellTags->"DiracReduce"],
    Cell[594055, 19159, 149, 3, 56, "Input",
      CellTags->"DiracReduce"],
    Cell[594586, 19176, 210, 9, 35, "Text",
      CellTags->"DiracReduce"],
    Cell[594833, 19190, 60, 1, 49, "Subsubsection",
      CellTags->"DiracReduce"],
    Cell[594918, 19195, 209, 5, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[595384, 19212, 144, 3, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[595918, 19231, 223, 5, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[596440, 19249, 144, 3, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[597832, 19289, 249, 6, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[598431, 19310, 172, 4, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[601097, 19377, 303, 7, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[601572, 19394, 165, 4, 70, "Input",
      CellTags->"DiracReduce"],
    Cell[601887, 19406, 188, 4, 70, "Input",
      CellTags->"DiracReduce"]},
  "DiracSigma"->{
    Cell[602124, 19416, 58, 1, 39, "ObjectName",
      CellTags->"DiracSigma"],
    Cell[602207, 19421, 62, 1, 49, "Subsubsection",
      CellTags->"DiracSigma"],
    Cell[602272, 19424, 214, 5, 56, "Text",
      CellTags->"DiracSigma"],
    Cell[602489, 19431, 224, 9, 39, "Text",
      CellTags->"DiracSigma"],
    Cell[602750, 19445, 59, 1, 49, "Subsubsection",
      CellTags->"DiracSigma"],
    Cell[602834, 19450, 285, 7, 70, "Input",
      CellTags->"DiracSigma"],
    Cell[603360, 19469, 150, 3, 70, "Input",
      CellTags->"DiracSigma"],
    Cell[604585, 19508, 285, 7, 70, "Input",
      CellTags->"DiracSigma"],
    Cell[605137, 19528, 272, 7, 70, "Input",
      CellTags->"DiracSigma"],
    Cell[605630, 19546, 150, 3, 70, "Input",
      CellTags->"DiracSigma"],
    Cell[607231, 19591, 176, 4, 70, "Input",
      CellTags->"DiracSigma"]},
  "DiracSigmaExplicit"->{
    Cell[607456, 19601, 74, 1, 39, "ObjectName",
      CellTags->"DiracSigmaExplicit"],
    Cell[607555, 19606, 70, 1, 49, "Subsubsection",
      CellTags->"DiracSigmaExplicit"],
    Cell[607628, 19609, 189, 5, 39, "Text",
      CellTags->"DiracSigmaExplicit"],
    Cell[607820, 19616, 208, 9, 39, "Text",
      CellTags->"DiracSigmaExplicit"],
    Cell[608065, 19630, 67, 1, 49, "Subsubsection",
      CellTags->"DiracSigmaExplicit"],
    Cell[608157, 19635, 259, 6, 70, "Input",
      CellTags->"DiracSigmaExplicit"],
    Cell[608657, 19653, 157, 3, 70, "Input",
      CellTags->"DiracSigmaExplicit"]},
  "DiracSimpCombine"->{
    Cell[609913, 19694, 70, 1, 39, "ObjectName",
      CellTags->"DiracSimpCombine"],
    Cell[610008, 19699, 68, 1, 49, "Subsubsection",
      CellTags->"DiracSimpCombine"],
    Cell[610079, 19702, 210, 5, 56, "Text",
      CellTags->"DiracSimpCombine"],
    Cell[610292, 19709, 208, 8, 39, "Text",
      CellTags->"DiracSimpCombine"]},
  "DiracSimplify"->{
    Cell[610549, 19723, 64, 1, 39, "ObjectName",
      CellTags->"DiracSimplify"],
    Cell[610638, 19728, 65, 1, 49, "Subsubsection",
      CellTags->"DiracSimplify"],
    Cell[610706, 19731, 383, 7, 73, "Text",
      CellTags->"DiracSimplify"],
    Cell[611114, 19742, 153, 3, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[611949, 19763, 455, 19, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[612441, 19787, 62, 1, 49, "Subsubsection",
      CellTags->"DiracSimplify"],
    Cell[612506, 19790, 100, 1, 39, "Text",
      CellTags->"DiracSimplify"],
    Cell[612631, 19795, 215, 5, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[613154, 19813, 148, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[613626, 19830, 97, 1, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[613748, 19835, 216, 5, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[614272, 19853, 148, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[614766, 19872, 211, 5, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[615259, 19888, 192, 7, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[615476, 19899, 148, 3, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[616140, 19923, 207, 5, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[616647, 19941, 148, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[617154, 19960, 390, 10, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[618496, 20001, 123, 4, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[618644, 20009, 148, 3, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[621695, 20098, 145, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[622777, 20131, 319, 7, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[623767, 20162, 148, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[624063, 20174, 460, 12, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[625266, 20210, 147, 3, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[648057, 20867, 118, 4, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[648200, 20875, 299, 7, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[649106, 20904, 142, 4, 35, "Text",
      CellTags->"DiracSimplify"],
    Cell[649273, 20912, 344, 8, 56, "Input",
      CellTags->"DiracSimplify"],
    Cell[650400, 20946, 316, 7, 52, "Input",
      CellTags->"DiracSimplify"],
    Cell[650842, 20960, 219, 5, 52, "Input",
      CellTags->"DiracSimplify"]},
  "DiracSimplify2"->{
    Cell[651110, 20971, 66, 1, 39, "ObjectName",
      CellTags->"DiracSimplify2"],
    Cell[651201, 20976, 66, 1, 49, "Subsubsection",
      CellTags->"DiracSimplify2"],
    Cell[651270, 20979, 237, 7, 39, "Text",
      CellTags->"DiracSimplify2"],
    Cell[651510, 20988, 213, 9, 39, "Text",
      CellTags->"DiracSimplify2"],
    Cell[651760, 21002, 63, 1, 49, "Subsubsection",
      CellTags->"DiracSimplify2"],
    Cell[651848, 21007, 263, 6, 56, "Input",
      CellTags->"DiracSimplify2"],
    Cell[652629, 21031, 149, 3, 52, "Input",
      CellTags->"DiracSimplify2"]},
  "DiracSlash"->{
    Cell[653954, 21077, 58, 1, 39, "ObjectName",
      CellTags->"DiracSlash"],
    Cell[654037, 21082, 62, 1, 49, "Subsubsection",
      CellTags->"DiracSlash"],
    Cell[654102, 21085, 515, 16, 40, "Text",
      CellTags->"DiracSlash"],
    Cell[654642, 21105, 147, 3, 56, "Input",
      CellTags->"DiracSlash"],
    Cell[655078, 21120, 941, 39, 35, "Text",
      CellTags->"DiracSlash"],
    Cell[656056, 21164, 59, 1, 49, "Subsubsection",
      CellTags->"DiracSlash"],
    Cell[656118, 21167, 265, 9, 40, "Text",
      CellTags->"DiracSlash"],
    Cell[656408, 21180, 141, 3, 56, "Input",
      CellTags->"DiracSlash"],
    Cell[656872, 21197, 209, 5, 52, "Input",
      CellTags->"DiracSlash"],
    Cell[657730, 21226, 168, 4, 52, "Input",
      CellTags->"DiracSlash"],
    Cell[658547, 21254, 133, 3, 52, "Input",
      CellTags->"DiracSlash"],
    Cell[658908, 21268, 182, 5, 52, "Input",
      CellTags->"DiracSlash"],
    Cell[659299, 21283, 261, 7, 52, "Input",
      CellTags->"DiracSlash"]},
  "DiracSpinor"->{
    Cell[659853, 21304, 60, 1, 39, "ObjectName",
      CellTags->"DiracSpinor"],
    Cell[659938, 21309, 63, 1, 49, "Subsubsection",
      CellTags->"DiracSpinor"],
    Cell[660004, 21312, 289, 6, 56, "Text",
      CellTags->"DiracSpinor"],
    Cell[660296, 21320, 182, 8, 39, "Text",
      CellTags->"DiracSpinor"]},
  "DiracSubstitute67"->{
    Cell[660527, 21334, 72, 1, 39, "ObjectName",
      CellTags->"DiracSubstitute67"],
    Cell[660624, 21339, 69, 1, 49, "Subsubsection",
      CellTags->"DiracSubstitute67"],
    Cell[660696, 21342, 220, 5, 56, "Text",
      CellTags->"DiracSubstitute67"],
    Cell[660919, 21349, 335, 13, 39, "Text",
      CellTags->"DiracSubstitute67"]},
  "DiracTrace"->{
    Cell[661303, 21368, 58, 1, 39, "ObjectName",
      CellTags->"DiracTrace"],
    Cell[661386, 21373, 62, 1, 49, "Subsubsection",
      CellTags->"DiracTrace"],
    Cell[661451, 21376, 329, 6, 56, "Text",
      CellTags->"DiracTrace"],
    Cell[661805, 21386, 147, 3, 56, "Input",
      CellTags->"DiracTrace"],
    Cell[662654, 21408, 306, 12, 35, "Text",
      CellTags->"DiracTrace"],
    Cell[662963, 21422, 176, 9, 39, "Text",
      CellTags->"DiracTrace"],
    Cell[663176, 21436, 59, 1, 49, "Subsubsection",
      CellTags->"DiracTrace"],
    Cell[663260, 21441, 211, 5, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[663820, 21461, 252, 6, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[664535, 21484, 181, 4, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[665619, 21516, 361, 9, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[666794, 21554, 395, 9, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[680311, 21891, 223, 5, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[681389, 21925, 347, 8, 70, "Input",
      CellTags->"DiracTrace"],
    Cell[681996, 21946, 139, 3, 70, "Input",
      CellTags->"DiracTrace"]},
  "DiracTraceEvaluate"->{
    Cell[682385, 21962, 74, 1, 39, "ObjectName",
      CellTags->"DiracTraceEvaluate"],
    Cell[682484, 21967, 70, 1, 49, "Subsubsection",
      CellTags->"DiracTraceEvaluate"],
    Cell[682557, 21970, 163, 4, 39, "Text",
      CellTags->"DiracTraceEvaluate"],
    Cell[682723, 21976, 303, 13, 39, "Text",
      CellTags->"DiracTraceEvaluate"]},
  "DiracTrick"->{
    Cell[683075, 21995, 58, 1, 39, "ObjectName",
      CellTags->"DiracTrick"],
    Cell[683158, 22000, 62, 1, 49, "Subsubsection",
      CellTags->"DiracTrick"],
    Cell[683223, 22003, 223, 5, 56, "Text",
      CellTags->"DiracTrick"],
    Cell[683471, 22012, 147, 3, 56, "Input",
      CellTags->"DiracTrick"],
    Cell[683825, 22025, 461, 19, 35, "Text",
      CellTags->"DiracTrick"],
    Cell[684323, 22049, 59, 1, 49, "Subsubsection",
      CellTags->"DiracTrick"],
    Cell[684385, 22052, 97, 1, 70, "Text",
      CellTags->"DiracTrick"],
    Cell[684507, 22057, 212, 5, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[685027, 22075, 142, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[685493, 22092, 94, 1, 70, "Text",
      CellTags->"DiracTrick"],
    Cell[685612, 22097, 213, 5, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[686133, 22115, 142, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[686657, 22135, 208, 5, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[687147, 22151, 189, 7, 70, "Text",
      CellTags->"DiracTrick"],
    Cell[687361, 22162, 142, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[688019, 22186, 204, 5, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[688523, 22204, 142, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[689024, 22223, 387, 10, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[690385, 22266, 142, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[691464, 22299, 136, 3, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[694503, 22388, 457, 12, 70, "Input",
      CellTags->"DiracTrick"],
    Cell[695703, 22424, 141, 3, 70, "Input",
      CellTags->"DiracTrick"]},
  "Divideout"->{
    Cell[720700, 23054, 56, 1, 39, "ObjectName",
      CellTags->"Divideout"],
    Cell[720781, 23059, 61, 1, 49, "Subsubsection",
      CellTags->"Divideout"],
    Cell[720845, 23062, 138, 4, 39, "Text",
      CellTags->"Divideout"],
    Cell[720986, 23068, 303, 13, 39, "Text",
      CellTags->"Divideout"]},
  "DOT"->{
    Cell[721338, 23087, 44, 1, 39, "ObjectName",
      CellTags->"DOT"],
    Cell[721407, 23092, 55, 1, 49, "Subsubsection",
      CellTags->"DOT"],
    Cell[721465, 23095, 351, 9, 106, "Text",
      CellTags->"DOT"],
    Cell[721819, 23106, 189, 8, 39, "Text",
      CellTags->"DOT"]},
  "DotExpand"->{
    Cell[722057, 23120, 56, 1, 39, "ObjectName",
      CellTags->"DotExpand"],
    Cell[722138, 23125, 61, 1, 49, "Subsubsection",
      CellTags->"DotExpand"],
    Cell[722202, 23128, 86, 1, 39, "Text",
      CellTags->"DotExpand"],
    Cell[722291, 23131, 624, 23, 39, "Text",
      CellTags->"DotExpand"],
    Cell[722952, 23159, 58, 1, 49, "Subsubsection",
      CellTags->"DotExpand"],
    Cell[723035, 23164, 359, 10, 43, "Input",
      CellTags->"DotExpand"],
    Cell[723944, 23191, 256, 7, 43, "Input",
      CellTags->"DotExpand"],
    Cell[724225, 23202, 359, 10, 43, "Input",
      CellTags->"DotExpand"],
    Cell[725323, 23238, 258, 7, 43, "Input",
      CellTags->"DotExpand"],
    Cell[725606, 23249, 359, 10, 43, "Input",
      CellTags->"DotExpand"]},
  "DotPower"->{
    Cell[726561, 23280, 54, 1, 39, "ObjectName",
      CellTags->"DotPower"],
    Cell[726640, 23285, 60, 1, 49, "Subsubsection",
      CellTags->"DotPower"],
    Cell[726703, 23288, 192, 5, 39, "Text",
      CellTags->"DotPower"],
    Cell[726898, 23295, 194, 8, 39, "Text",
      CellTags->"DotPower"]},
  "DotProduct"->{
    Cell[727141, 23309, 58, 1, 39, "ObjectName",
      CellTags->"DotProduct"],
    Cell[727224, 23314, 62, 1, 49, "Subsubsection",
      CellTags->"DotProduct"],
    Cell[727289, 23317, 220, 5, 56, "Text",
      CellTags->"DotProduct"],
    Cell[727512, 23324, 335, 14, 39, "Text",
      CellTags->"DotProduct"],
    Cell[727884, 23343, 59, 1, 49, "Subsubsection",
      CellTags->"DotProduct"],
    Cell[727968, 23348, 281, 7, 56, "Input",
      CellTags->"DotProduct"]},
  "DotSimplifyRelations"->{
    Cell[728709, 23374, 78, 1, 39, "ObjectName",
      CellTags->"DotSimplifyRelations"],
    Cell[728812, 23379, 72, 1, 49, "Subsubsection",
      CellTags->"DotSimplifyRelations"],
    Cell[728887, 23382, 242, 5, 56, "Text",
      CellTags->"DotSimplifyRelations"],
    Cell[729132, 23389, 206, 8, 39, "Text",
      CellTags->"DotSimplifyRelations"]},
  "DotSimplify"->{
    Cell[729387, 23403, 60, 1, 39, "ObjectName",
      CellTags->"DotSimplify"],
    Cell[729472, 23408, 63, 1, 49, "Subsubsection",
      CellTags->"DotSimplify"],
    Cell[729538, 23411, 325, 7, 56, "Text",
      CellTags->"DotSimplify"],
    Cell[729888, 23422, 149, 3, 56, "Input",
      CellTags->"DotSimplify"],
    Cell[730415, 23439, 447, 19, 35, "Text",
      CellTags->"DotSimplify"],
    Cell[730899, 23463, 60, 1, 49, "Subsubsection",
      CellTags->"DotSimplify"],
    Cell[730984, 23468, 417, 11, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[731973, 23500, 144, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[732837, 23527, 190, 4, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[733052, 23535, 259, 7, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[733596, 23556, 144, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[734024, 23572, 198, 5, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[734370, 23586, 144, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[734879, 23604, 198, 5, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[735102, 23613, 144, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[735530, 23629, 202, 5, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[735880, 23643, 144, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[736347, 23659, 202, 5, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[736574, 23668, 365, 9, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[737138, 23687, 153, 3, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[737316, 23694, 180, 4, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[737672, 23708, 255, 6, 70, "Input",
      CellTags->"DotSimplify"],
    Cell[738090, 23722, 202, 4, 70, "Input",
      CellTags->"DotSimplify"]},
  "DummyIndex"->{
    Cell[738341, 23732, 58, 1, 39, "ObjectName",
      CellTags->"DummyIndex"],
    Cell[738424, 23737, 62, 1, 49, "Subsubsection",
      CellTags->"DummyIndex"],
    Cell[738489, 23740, 194, 5, 39, "Text",
      CellTags->"DummyIndex"],
    Cell[738686, 23747, 193, 8, 39, "Text",
      CellTags->"DummyIndex"]},
  "D0"->{
    Cell[738928, 23761, 42, 1, 39, "ObjectName",
      CellTags->"D0"],
    Cell[738995, 23766, 54, 1, 49, "Subsubsection",
      CellTags->"D0"],
    Cell[739052, 23769, 537, 12, 73, "Text",
      CellTags->"D0"],
    Cell[739592, 23783, 501, 24, 39, "Text",
      CellTags->"D0"],
    Cell[740130, 23812, 51, 1, 49, "Subsubsection",
      CellTags->"D0"],
    Cell[740206, 23817, 380, 10, 56, "Input",
      CellTags->"D0"],
    Cell[741888, 23866, 592, 15, 72, "Input",
      CellTags->"D0"],
    Cell[743791, 23921, 132, 3, 52, "Input",
      CellTags->"D0"]},
  "D0Convention"->{
    Cell[745249, 23965, 62, 1, 39, "ObjectName",
      CellTags->"D0Convention"],
    Cell[745336, 23970, 64, 1, 49, "Subsubsection",
      CellTags->"D0Convention"],
    Cell[745403, 23973, 308, 6, 56, "Text",
      CellTags->"D0Convention"],
    Cell[745714, 23981, 285, 13, 39, "Text",
      CellTags->"D0Convention"]},
  "Eps"->{
    Cell[746048, 24000, 44, 1, 39, "ObjectName",
      CellTags->"Eps"],
    Cell[746117, 24005, 55, 1, 49, "Subsubsection",
      CellTags->"Eps"],
    Cell[746175, 24008, 360, 9, 56, "Text",
      CellTags->"Eps"],
    Cell[746560, 24021, 133, 3, 56, "Input",
      CellTags->"Eps"],
    Cell[746896, 24034, 529, 24, 35, "Text",
      CellTags->"Eps"],
    Cell[747462, 24063, 52, 1, 49, "Subsubsection",
      CellTags->"Eps"],
    Cell[747539, 24068, 374, 8, 70, "Input",
      CellTags->"Eps"],
    Cell[748618, 24103, 366, 8, 70, "Input",
      CellTags->"Eps"],
    Cell[749630, 24136, 222, 6, 70, "Input",
      CellTags->"Eps"],
    Cell[750086, 24153, 174, 4, 70, "Input",
      CellTags->"Eps"],
    Cell[750408, 24166, 174, 4, 70, "Input",
      CellTags->"Eps"],
    Cell[750730, 24178, 232, 6, 70, "Input",
      CellTags->"Eps"],
    Cell[750987, 24188, 549, 13, 70, "Input",
      CellTags->"Eps"],
    Cell[752241, 24228, 156, 4, 70, "Input",
      CellTags->"Eps"],
    Cell[752546, 24240, 232, 6, 70, "Input",
      CellTags->"Eps"],
    Cell[752803, 24250, 549, 13, 70, "Input",
      CellTags->"Eps"],
    Cell[754057, 24290, 192, 5, 70, "Input",
      CellTags->"Eps"],
    Cell[754631, 24312, 541, 14, 70, "Input",
      CellTags->"Eps"],
    Cell[756743, 24377, 1225, 29, 70, "Input",
      CellTags->"Eps"],
    Cell[760259, 24473, 231, 6, 70, "Input",
      CellTags->"Eps"],
    Cell[760936, 24498, 156, 4, 70, "Input",
      CellTags->"Eps"],
    Cell[761218, 24509, 200, 5, 70, "Input",
      CellTags->"Eps"]},
  "EpsChisholm"->{
    Cell[761467, 24520, 60, 1, 39, "ObjectName",
      CellTags->"EpsChisholm"],
    Cell[761552, 24525, 63, 1, 49, "Subsubsection",
      CellTags->"EpsChisholm"],
    Cell[761618, 24528, 167, 4, 39, "Text",
      CellTags->"EpsChisholm"],
    Cell[761788, 24534, 195, 9, 39, "Text",
      CellTags->"EpsChisholm"],
    Cell[762020, 24548, 60, 1, 49, "Subsubsection",
      CellTags->"EpsChisholm"],
    Cell[762105, 24553, 251, 6, 70, "Input",
      CellTags->"EpsChisholm"],
    Cell[765124, 24644, 143, 3, 70, "Input",
      CellTags->"EpsChisholm"]},
  "EpsContract"->{
    Cell[766078, 24679, 60, 1, 39, "ObjectName",
      CellTags->"EpsContract"],
    Cell[766163, 24684, 63, 1, 49, "Subsubsection",
      CellTags->"EpsContract"],
    Cell[766229, 24687, 228, 5, 56, "Text",
      CellTags->"EpsContract"],
    Cell[766460, 24694, 300, 14, 39, "Text",
      CellTags->"EpsContract"],
    Cell[766797, 24713, 60, 1, 49, "Subsubsection",
      CellTags->"EpsContract"],
    Cell[766882, 24718, 555, 13, 76, "Input",
      CellTags->"EpsContract"],
    Cell[768140, 24758, 245, 6, 56, "Input",
      CellTags->"EpsContract"],
    Cell[769224, 24794, 244, 6, 56, "Input",
      CellTags->"EpsContract"]},
  "EpsDiscard"->{
    Cell[769773, 24816, 58, 1, 39, "ObjectName",
      CellTags->"EpsDiscard"],
    Cell[769856, 24821, 62, 1, 49, "Subsubsection",
      CellTags->"EpsDiscard"],
    Cell[769921, 24824, 170, 4, 39, "Text",
      CellTags->"EpsDiscard"],
    Cell[770094, 24830, 209, 9, 39, "Text",
      CellTags->"EpsDiscard"]},
  "EpsEvaluate"->{
    Cell[770352, 24845, 60, 1, 39, "ObjectName",
      CellTags->"EpsEvaluate"],
    Cell[770437, 24850, 63, 1, 49, "Subsubsection",
      CellTags->"EpsEvaluate"],
    Cell[770503, 24853, 174, 4, 39, "Text",
      CellTags->"EpsEvaluate"],
    Cell[770680, 24859, 537, 24, 39, "Text",
      CellTags->"EpsEvaluate"],
    Cell[771254, 24888, 60, 1, 49, "Subsubsection",
      CellTags->"EpsEvaluate"],
    Cell[771339, 24893, 410, 11, 70, "Input",
      CellTags->"EpsEvaluate"],
    Cell[772425, 24930, 143, 3, 70, "Input",
      CellTags->"EpsEvaluate"],
    Cell[773797, 24975, 144, 3, 70, "Input",
      CellTags->"EpsEvaluate"]},
  "EpsUncontract"->{
    Cell[774681, 25001, 64, 1, 39, "ObjectName",
      CellTags->"EpsUncontract"],
    Cell[774770, 25006, 65, 1, 49, "Subsubsection",
      CellTags->"EpsUncontract"],
    Cell[774838, 25009, 118, 4, 39, "Text",
      CellTags->"EpsUncontract"],
    Cell[774959, 25015, 308, 14, 39, "Text",
      CellTags->"EpsUncontract"]},
  "Epsilon"->{
    Cell[775316, 25035, 52, 1, 39, "ObjectName",
      CellTags->"Epsilon"],
    Cell[775393, 25040, 59, 1, 49, "Subsubsection",
      CellTags->"Epsilon"],
    Cell[775455, 25043, 247, 9, 39, "Text",
      CellTags->"Epsilon"],
    Cell[775705, 25054, 188, 9, 39, "Text",
      CellTags->"Epsilon"],
    Cell[775930, 25068, 56, 1, 49, "Subsubsection",
      CellTags->"Epsilon"],
    Cell[776011, 25073, 105, 2, 56, "Input",
      CellTags->"Epsilon"],
    Cell[776296, 25084, 117, 4, 35, "Text",
      CellTags->"Epsilon"],
    Cell[776438, 25092, 702, 19, 76, "Input",
      CellTags->"Epsilon"]},
  "EpsilonOrder"->{
    Cell[777422, 25125, 62, 1, 39, "ObjectName",
      CellTags->"EpsilonOrder"],
    Cell[777509, 25130, 64, 1, 49, "Subsubsection",
      CellTags->"EpsilonOrder"],
    Cell[777576, 25133, 333, 12, 39, "Text",
      CellTags->"EpsilonOrder"],
    Cell[777912, 25147, 310, 14, 39, "Text",
      CellTags->"EpsilonOrder"]},
  "Expanding"->{
    Cell[778271, 25167, 56, 1, 39, "ObjectName",
      CellTags->"Expanding"],
    Cell[778352, 25172, 61, 1, 49, "Subsubsection",
      CellTags->"Expanding"],
    Cell[778416, 25175, 379, 7, 73, "Text",
      CellTags->"Expanding"],
    Cell[778798, 25184, 694, 29, 39, "Text",
      CellTags->"Expanding"]},
  "ExpandPartialD"->{
    Cell[779541, 25219, 66, 1, 39, "ObjectName",
      CellTags->"ExpandPartialD"],
    Cell[779632, 25224, 66, 1, 49, "Subsubsection",
      CellTags->"ExpandPartialD"],
    Cell[779701, 25227, 191, 5, 39, "Text",
      CellTags->"ExpandPartialD"],
    Cell[781789, 25284, 776, 28, 35, "Text",
      CellTags->"ExpandPartialD"],
    Cell[782602, 25317, 63, 1, 49, "Subsubsection",
      CellTags->"ExpandPartialD"],
    Cell[782690, 25322, 410, 10, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[783515, 25350, 124, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[784651, 25389, 118, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[785727, 25422, 281, 7, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[786385, 25446, 124, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[787393, 25481, 487, 13, 63, "Input",
      CellTags->"ExpandPartialD"],
    Cell[788265, 25510, 124, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[790959, 25588, 199, 6, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[791468, 25608, 127, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[792905, 25651, 125, 3, 43, "Input",
      CellTags->"ExpandPartialD"],
    Cell[795359, 25719, 345, 9, 39, "Input",
      CellTags->"ExpandPartialD"],
    Cell[797312, 25781, 125, 3, 39, "Input",
      CellTags->"ExpandPartialD"]},
  "ScalarProductCancel"->{
    Cell[797585, 25795, 122, 1, 39, "ObjectName",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[797732, 25800, 96, 1, 49, "Subsubsection",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[797831, 25803, 253, 5, 64, "Text",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798109, 25812, 185, 3, 56, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798297, 25817, 254, 6, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798566, 25826, 495, 19, 35, "Text",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799098, 25850, 93, 1, 49, "Subsubsection",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799216, 25855, 304, 7, 56, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799523, 25864, 501, 12, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[800061, 25881, 178, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[800242, 25886, 1811, 49, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802090, 25940, 223, 5, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802316, 25947, 408, 11, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802761, 25963, 184, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802948, 25968, 446, 12, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[803431, 25985, 227, 5, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[803661, 25992, 313, 8, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804011, 26005, 184, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804198, 26010, 577, 18, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804812, 26033, 286, 7, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805101, 26042, 475, 12, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805613, 26059, 215, 4, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805831, 26065, 805, 22, 38, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[2217834, 71708, 76, 1, 39, "ObjectName",
      CellTags->"ScalarProductCancel"],
    Cell[2217935, 71713, 71, 1, 49, "Subsubsection",
      CellTags->"ScalarProductCancel"],
    Cell[2218009, 71716, 187, 4, 39, "Text",
      CellTags->"ScalarProductCancel"],
    Cell[2218221, 71724, 114, 2, 56, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2218338, 71728, 162, 3, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2218537, 71736, 162, 3, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2218702, 71741, 457, 10, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2219196, 71756, 134, 3, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2219333, 71761, 154, 3, 70, "Print",
      CellTags->"ScalarProductCancel"],
    Cell[2219502, 71767, 805, 33, 35, "Text",
      CellTags->"ScalarProductCancel"],
    Cell[2220344, 71805, 68, 1, 49, "Subsubsection",
      CellTags->"ScalarProductCancel"],
    Cell[2220415, 71808, 257, 9, 39, "Text",
      CellTags->"ScalarProductCancel"],
    Cell[2220697, 71821, 541, 15, 56, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2221241, 71838, 611, 18, 54, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2221889, 71861, 188, 4, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2222080, 71867, 1126, 33, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2223243, 71905, 194, 4, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2223440, 71911, 916, 27, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2224393, 71943, 222, 5, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2224618, 71950, 916, 27, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2225571, 71982, 714, 19, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2226288, 72003, 2080, 60, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2228405, 72068, 234, 5, 52, "Input",
      CellTags->"ScalarProductCancel"],
    Cell[2228642, 72075, 6568, 176, 70, "Output",
      CellTags->"ScalarProductCancel"],
    Cell[2235225, 72254, 199, 5, 52, "Input",
      CellTags->"ScalarProductCancel"]},
  "ExpandScalarProduct"->{
    Cell[797585, 25795, 122, 1, 39, "ObjectName",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[797732, 25800, 96, 1, 49, "Subsubsection",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[797831, 25803, 253, 5, 64, "Text",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798109, 25812, 185, 3, 56, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798297, 25817, 254, 6, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[798566, 25826, 495, 19, 35, "Text",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799098, 25850, 93, 1, 49, "Subsubsection",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799216, 25855, 304, 7, 56, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[799523, 25864, 501, 12, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[800061, 25881, 178, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[800242, 25886, 1811, 49, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802090, 25940, 223, 5, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802316, 25947, 408, 11, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802761, 25963, 184, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[802948, 25968, 446, 12, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[803431, 25985, 227, 5, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[803661, 25992, 313, 8, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804011, 26005, 184, 3, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804198, 26010, 577, 18, 70, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[804812, 26033, 286, 7, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805101, 26042, 475, 12, 37, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805613, 26059, 215, 4, 52, "Input",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
    Cell[805831, 26065, 805, 22, 38, "Output",
      CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]},
  "Expand2"->{
    Cell[806697, 26094, 52, 1, 39, "ObjectName",
      CellTags->"Expand2"],
    Cell[806774, 26099, 59, 1, 49, "Subsubsection",
      CellTags->"Expand2"],
    Cell[806836, 26102, 165, 4, 39, "Text",
      CellTags->"Expand2"],
    Cell[807038, 26111, 56, 1, 49, "Subsubsection",
      CellTags->"Expand2"],
    Cell[807119, 26116, 734, 21, 56, "Input",
      CellTags->"Expand2"]},
  "Explicit"->{
    Cell[808587, 26164, 54, 1, 39, "ObjectName",
      CellTags->"Explicit"],
    Cell[808666, 26169, 60, 1, 49, "Subsubsection",
      CellTags->"Explicit"],
    Cell[808729, 26172, 407, 9, 56, "Text",
      CellTags->"Explicit"],
    Cell[809139, 26183, 354, 14, 39, "Text",
      CellTags->"Explicit"],
    Cell[809530, 26202, 57, 1, 49, "Subsubsection",
      CellTags->"Explicit"],
    Cell[809612, 26207, 271, 6, 70, "Input",
      CellTags->"Explicit"],
    Cell[811349, 26262, 137, 3, 70, "Input",
      CellTags->"Explicit"],
    Cell[816026, 26400, 220, 5, 70, "Input",
      CellTags->"Explicit"],
    Cell[817476, 26448, 137, 3, 70, "Input",
      CellTags->"Explicit"],
    Cell[821800, 26570, 187, 4, 70, "Input",
      CellTags->"Explicit"],
    Cell[822224, 26586, 137, 3, 70, "Input",
      CellTags->"Explicit"]},
  "ExplicitLorentzIndex"->{
    Cell[824490, 26663, 78, 1, 39, "ObjectName",
      CellTags->"ExplicitLorentzIndex"],
    Cell[824593, 26668, 72, 1, 49, "Subsubsection",
      CellTags->"ExplicitLorentzIndex"],
    Cell[824668, 26671, 142, 4, 39, "Text",
      CellTags->"ExplicitLorentzIndex"],
    Cell[824813, 26677, 317, 13, 39, "Text",
      CellTags->"ExplicitLorentzIndex"],
    Cell[825167, 26695, 69, 1, 49, "Subsubsection",
      CellTags->"ExplicitLorentzIndex"],
    Cell[825261, 26700, 263, 6, 56, "Input",
      CellTags->"ExplicitLorentzIndex"],
    Cell[825797, 26720, 146, 3, 56, "Input",
      CellTags->"ExplicitLorentzIndex"]},
  "ExplicitPartialD"->{
    Cell[826257, 26737, 70, 1, 39, "ObjectName",
      CellTags->"ExplicitPartialD"],
    Cell[826352, 26742, 68, 1, 49, "Subsubsection",
      CellTags->"ExplicitPartialD"],
    Cell[826423, 26745, 163, 4, 39, "Text",
      CellTags->"ExplicitPartialD"],
    Cell[826589, 26751, 508, 18, 39, "Text",
      CellTags->"ExplicitPartialD"],
    Cell[827134, 26774, 65, 1, 49, "Subsubsection",
      CellTags->"ExplicitPartialD"],
    Cell[827224, 26779, 215, 4, 56, "Input",
      CellTags->"ExplicitPartialD"],
    Cell[828130, 26808, 218, 5, 52, "Input",
      CellTags->"ExplicitPartialD"],
    Cell[828904, 26835, 225, 5, 52, "Input",
      CellTags->"ExplicitPartialD"],
    Cell[829714, 26861, 203, 6, 39, "Input",
      CellTags->"ExplicitPartialD"],
    Cell[830229, 26881, 129, 3, 39, "Input",
      CellTags->"ExplicitPartialD"]},
  "ExplicitSUNIndex"->{
    Cell[832085, 26936, 70, 1, 39, "ObjectName",
      CellTags->"ExplicitSUNIndex"],
    Cell[832180, 26941, 68, 1, 49, "Subsubsection",
      CellTags->"ExplicitSUNIndex"],
    Cell[832251, 26944, 184, 6, 39, "Text",
      CellTags->"ExplicitSUNIndex"],
    Cell[832438, 26952, 526, 23, 39, "Text",
      CellTags->"ExplicitSUNIndex"],
    Cell[833001, 26980, 65, 1, 49, "Subsubsection",
      CellTags->"ExplicitSUNIndex"],
    Cell[833091, 26985, 153, 3, 56, "Input",
      CellTags->"ExplicitSUNIndex"]},
  "ExtraFactor"->{
    Cell[833917, 27017, 60, 1, 39, "ObjectName",
      CellTags->"ExtraFactor"],
    Cell[834002, 27022, 63, 1, 49, "Subsubsection",
      CellTags->"ExtraFactor"],
    Cell[834068, 27025, 202, 5, 39, "Text",
      CellTags->"ExtraFactor"],
    Cell[834273, 27032, 206, 8, 39, "Text",
      CellTags->"ExtraFactor"]},
  "ExtraVariables"->{
    Cell[834528, 27046, 66, 1, 39, "ObjectName",
      CellTags->"ExtraVariables"],
    Cell[834619, 27051, 66, 1, 49, "Subsubsection",
      CellTags->"ExtraVariables"],
    Cell[834688, 27054, 214, 5, 56, "Text",
      CellTags->"ExtraVariables"],
    Cell[834905, 27061, 321, 14, 39, "Text",
      CellTags->"ExtraVariables"]},
  "FactorFull"->{
    Cell[835275, 27081, 58, 1, 39, "ObjectName",
      CellTags->"FactorFull"],
    Cell[835358, 27086, 62, 1, 49, "Subsubsection",
      CellTags->"FactorFull"],
    Cell[835423, 27089, 549, 24, 39, "Text",
      CellTags->"FactorFull"],
    Cell[835975, 27115, 191, 9, 39, "Text",
      CellTags->"FactorFull"]},
  "Factoring"->{
    Cell[836215, 27130, 56, 1, 39, "ObjectName",
      CellTags->"Factoring"],
    Cell[836296, 27135, 61, 1, 49, "Subsubsection",
      CellTags->"Factoring"],
    Cell[836360, 27138, 236, 5, 56, "Text",
      CellTags->"Factoring"],
    Cell[836599, 27145, 415, 19, 39, "Text",
      CellTags->"Factoring"]},
  "Factorout"->{
    Cell[837063, 27170, 56, 1, 39, "ObjectName",
      CellTags->"Factorout"],
    Cell[837144, 27175, 61, 1, 49, "Subsubsection",
      CellTags->"Factorout"],
    Cell[837208, 27178, 75, 1, 39, "Text",
      CellTags->"Factorout"],
    Cell[837286, 27181, 187, 9, 39, "Text",
      CellTags->"Factorout"]},
  "FactorTime"->{
    Cell[837522, 27196, 58, 1, 39, "ObjectName",
      CellTags->"FactorTime"],
    Cell[837605, 27201, 62, 1, 49, "Subsubsection",
      CellTags->"FactorTime"],
    Cell[837670, 27204, 167, 4, 39, "Text",
      CellTags->"FactorTime"],
    Cell[837840, 27210, 191, 9, 39, "Text",
      CellTags->"FactorTime"]},
  "Factor1"->{
    Cell[838080, 27225, 52, 1, 39, "ObjectName",
      CellTags->"Factor1"],
    Cell[838157, 27230, 59, 1, 49, "Subsubsection",
      CellTags->"Factor1"],
    Cell[838219, 27233, 145, 4, 39, "Text",
      CellTags->"Factor1"],
    Cell[838367, 27239, 188, 9, 39, "Text",
      CellTags->"Factor1"],
    Cell[838592, 27253, 56, 1, 49, "Subsubsection",
      CellTags->"Factor1"],
    Cell[838673, 27258, 266, 8, 56, "Input",
      CellTags->"Factor1"],
    Cell[839233, 27280, 270, 7, 52, "Input",
      CellTags->"Factor1"],
    Cell[840256, 27315, 313, 9, 52, "Input",
      CellTags->"Factor1"],
    Cell[840803, 27336, 134, 3, 52, "Input",
      CellTags->"Factor1"],
    Cell[841231, 27353, 135, 3, 52, "Input",
      CellTags->"Factor1"],
    Cell[841578, 27366, 172, 4, 52, "Input",
      CellTags->"Factor1"]},
  "Factor2"->{
    Cell[841799, 27376, 52, 1, 39, "ObjectName",
      CellTags->"Factor2"],
    Cell[841876, 27381, 59, 1, 49, "Subsubsection",
      CellTags->"Factor2"],
    Cell[841938, 27384, 399, 7, 73, "Text",
      CellTags->"Factor2"],
    Cell[842362, 27395, 141, 3, 56, "Input",
      CellTags->"Factor2"],
    Cell[842784, 27410, 191, 9, 35, "Text",
      CellTags->"Factor2"],
    Cell[843012, 27424, 56, 1, 49, "Subsubsection",
      CellTags->"Factor2"],
    Cell[843093, 27429, 267, 8, 70, "Input",
      CellTags->"Factor2"],
    Cell[843655, 27451, 271, 7, 70, "Input",
      CellTags->"Factor2"],
    Cell[844503, 27481, 314, 9, 70, "Input",
      CellTags->"Factor2"],
    Cell[845052, 27502, 135, 3, 70, "Input",
      CellTags->"Factor2"],
    Cell[845482, 27519, 136, 3, 70, "Input",
      CellTags->"Factor2"],
    Cell[845853, 27534, 210, 5, 70, "Input",
      CellTags->"Factor2"],
    Cell[846336, 27551, 173, 4, 70, "Input",
      CellTags->"Factor2"]},
  "FAD"->{
    Cell[846558, 27561, 44, 1, 39, "ObjectName",
      CellTags->"FAD"],
    Cell[846627, 27566, 55, 1, 49, "Subsubsection",
      CellTags->"FAD"],
    Cell[846685, 27569, 338, 7, 56, "Text",
      CellTags->"FAD"],
    Cell[847026, 27578, 865, 34, 39, "Text",
      CellTags->"FAD"],
    Cell[847928, 27617, 52, 1, 49, "Subsubsection",
      CellTags->"FAD"],
    Cell[848005, 27622, 183, 5, 70, "Input",
      CellTags->"FAD"],
    Cell[849139, 27657, 247, 7, 70, "Input",
      CellTags->"FAD"],
    Cell[850471, 27696, 307, 9, 70, "Input",
      CellTags->"FAD"],
    Cell[850997, 27716, 269, 8, 70, "Input",
      CellTags->"FAD"],
    Cell[851929, 27746, 341, 9, 70, "Input",
      CellTags->"FAD"]},
  "FC"->{
    Cell[852957, 27779, 42, 1, 39, "ObjectName",
      CellTags->"FC"],
    Cell[853024, 27784, 54, 1, 49, "Subsubsection",
      CellTags->"FC"],
    Cell[853081, 27787, 120, 4, 39, "Text",
      CellTags->"FC"],
    Cell[853204, 27793, 588, 24, 39, "Text",
      CellTags->"FC"],
    Cell[853829, 27822, 51, 1, 49, "Subsubsection",
      CellTags->"FC"],
    Cell[853905, 27827, 93, 2, 56, "Input",
      CellTags->"FC"],
    Cell[854001, 27831, 92, 2, 52, "Output",
      CellTags->"FC"],
    Cell[854130, 27838, 273, 7, 52, "Input",
      CellTags->"FC"],
    Cell[854406, 27847, 123, 2, 52, "Output",
      CellTags->"FC"],
    Cell[854544, 27852, 93, 2, 52, "Input",
      CellTags->"FC"],
    Cell[854662, 27858, 273, 7, 56, "Input",
      CellTags->"FC"],
    Cell[854938, 27867, 427, 12, 52, "Output",
      CellTags->"FC"]},
  "FCE"->{
    Cell[855426, 27886, 44, 1, 39, "ObjectName",
      CellTags->"FCE"],
    Cell[855495, 27891, 55, 1, 49, "Subsubsection",
      CellTags->"FCE"],
    Cell[855553, 27894, 127, 4, 39, "Text",
      CellTags->"FCE"],
    Cell[855683, 27900, 73, 1, 39, "Text",
      CellTags->"FCE"],
    Cell[855759, 27903, 460, 19, 39, "Text",
      CellTags->"FCE"],
    Cell[856256, 27927, 52, 1, 49, "Subsubsection",
      CellTags->"FCE"],
    Cell[856333, 27932, 325, 9, 70, "Input",
      CellTags->"FCE"],
    Cell[856998, 27956, 132, 3, 70, "Input",
      CellTags->"FCE"],
    Cell[857383, 27971, 517, 13, 70, "Input",
      CellTags->"FCE"],
    Cell[858709, 28011, 132, 3, 70, "Input",
      CellTags->"FCE"],
    Cell[859394, 28033, 123, 3, 70, "Input",
      CellTags->"FCE"],
    Cell[860769, 28078, 132, 3, 70, "Input",
      CellTags->"FCE"],
    Cell[861970, 28111, 166, 4, 70, "Input",
      CellTags->"FCE"]},
  "FCI"->{
    Cell[862713, 28136, 44, 1, 39, "ObjectName",
      CellTags->"FCI"],
    Cell[862782, 28141, 55, 1, 49, "Subsubsection",
      CellTags->"FCI"],
    Cell[862840, 28144, 162, 5, 39, "Text",
      CellTags->"FCI"],
    Cell[863005, 28151, 460, 19, 39, "Text",
      CellTags->"FCI"],
    Cell[863502, 28175, 52, 1, 49, "Subsubsection",
      CellTags->"FCI"],
    Cell[863579, 28180, 517, 13, 70, "Input",
      CellTags->"FCI"],
    Cell[864905, 28220, 132, 3, 70, "Input",
      CellTags->"FCI"],
    Cell[865590, 28242, 123, 3, 70, "Input",
      CellTags->"FCI"],
    Cell[866965, 28287, 132, 3, 70, "Input",
      CellTags->"FCI"],
    Cell[868166, 28320, 166, 4, 70, "Input",
      CellTags->"FCI"]},
  "FCIntegral"->{
    Cell[868909, 28345, 58, 1, 39, "ObjectName",
      CellTags->"FCIntegral"],
    Cell[868992, 28350, 62, 1, 49, "Subsubsection",
      CellTags->"FCIntegral"],
    Cell[869057, 28353, 210, 5, 56, "Text",
      CellTags->"FCIntegral"],
    Cell[869270, 28360, 383, 14, 39, "Text",
      CellTags->"FCIntegral"]},
  "FCIntegrate"->{
    Cell[869702, 28380, 60, 1, 39, "ObjectName",
      CellTags->"FCIntegrate"],
    Cell[869787, 28385, 63, 1, 49, "Subsubsection",
      CellTags->"FCIntegrate"],
    Cell[869853, 28388, 288, 6, 56, "Text",
      CellTags->"FCIntegrate"],
    Cell[870144, 28396, 207, 9, 39, "Text",
      CellTags->"FCIntegrate"]},
  "FCNIntegrate"->{
    Cell[870400, 28411, 62, 1, 39, "ObjectName",
      CellTags->"FCNIntegrate"],
    Cell[870487, 28416, 64, 1, 49, "Subsubsection",
      CellTags->"FCNIntegrate"],
    Cell[870554, 28419, 421, 8, 73, "Text",
      CellTags->"FCNIntegrate"],
    Cell[870978, 28429, 205, 9, 39, "Text",
      CellTags->"FCNIntegrate"]},
  "FC2RHI"->{
    Cell[871232, 28444, 67, 1, 39, "ObjectName",
      CellTags->"FC2RHI"],
    Cell[871324, 28449, 58, 1, 49, "Subsubsection",
      CellTags->"FC2RHI"],
    Cell[871385, 28452, 307, 6, 56, "Text",
      CellTags->"FC2RHI"],
    Cell[871717, 28462, 138, 3, 56, "Input",
      CellTags->"FC2RHI"],
    Cell[872189, 28478, 175, 9, 35, "Text",
      CellTags->"FC2RHI"],
    Cell[872401, 28492, 55, 1, 49, "Subsubsection",
      CellTags->"FC2RHI"],
    Cell[872459, 28495, 97, 2, 56, "Input",
      CellTags->"FC2RHI"]},
  "FC2TLI"->{
    Cell[872605, 28503, 67, 1, 39, "ObjectName",
      CellTags->"FC2TLI"],
    Cell[872697, 28508, 58, 1, 49, "Subsubsection",
      CellTags->"FC2TLI"],
    Cell[872758, 28511, 259, 6, 56, "Text",
      CellTags->"FC2TLI"],
    Cell[873042, 28521, 138, 3, 56, "Input",
      CellTags->"FC2TLI"],
    Cell[873514, 28537, 175, 9, 35, "Text",
      CellTags->"FC2TLI"],
    Cell[873726, 28551, 55, 1, 49, "Subsubsection",
      CellTags->"FC2TLI"],
    Cell[873784, 28554, 97, 2, 56, "Input",
      CellTags->"FC2TLI"]},
  "FDS"->{
    Cell[873930, 28562, 44, 1, 39, "ObjectName",
      CellTags->"FDS"],
    Cell[873999, 28567, 55, 1, 49, "Subsubsection",
      CellTags->"FDS"],
    Cell[874057, 28570, 83, 1, 39, "Text",
      CellTags->"FDS"],
    Cell[874143, 28573, 241, 9, 39, "Text",
      CellTags->"FDS"]},
  "FermionSpinSum"->{
    Cell[874433, 28588, 66, 1, 39, "ObjectName",
      CellTags->"FermionSpinSum"],
    Cell[874524, 28593, 66, 1, 49, "Subsubsection",
      CellTags->"FermionSpinSum"],
    Cell[874593, 28596, 121, 4, 39, "Text",
      CellTags->"FermionSpinSum"],
    Cell[874739, 28604, 155, 3, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[875255, 28620, 564, 24, 35, "Text",
      CellTags->"FermionSpinSum"],
    Cell[875856, 28649, 63, 1, 49, "Subsubsection",
      CellTags->"FermionSpinSum"],
    Cell[875922, 28652, 275, 12, 39, "Text",
      CellTags->"FermionSpinSum"],
    Cell[876222, 28668, 353, 9, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[876578, 28679, 608, 15, 50, "Output",
      CellTags->"FermionSpinSum"],
    Cell[877223, 28699, 174, 4, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[877400, 28705, 468, 13, 50, "Output",
      CellTags->"FermionSpinSum"],
    Cell[877905, 28723, 174, 4, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[878082, 28729, 164, 4, 50, "Output",
      CellTags->"FermionSpinSum"],
    Cell[878283, 28738, 353, 9, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[878639, 28749, 608, 15, 37, "Output",
      CellTags->"FermionSpinSum"],
    Cell[879284, 28769, 174, 4, 52, "Input",
      CellTags->"FermionSpinSum"],
    Cell[879461, 28775, 468, 13, 37, "Output",
      CellTags->"FermionSpinSum"],
    Cell[879966, 28793, 174, 4, 52, "Input",
      CellTags->"FermionSpinSum"],
    Cell[880143, 28799, 188, 5, 37, "Output",
      CellTags->"FermionSpinSum"],
    Cell[880368, 28809, 398, 10, 52, "Input",
      CellTags->"FermionSpinSum"],
    Cell[880769, 28821, 833, 20, 70, "Output",
      CellTags->"FermionSpinSum"],
    Cell[881617, 28844, 165, 4, 35, "Text",
      CellTags->"FermionSpinSum"],
    Cell[881807, 28852, 179, 4, 56, "Input",
      CellTags->"FermionSpinSum"],
    Cell[881989, 28858, 889, 21, 70, "Output",
      CellTags->"FermionSpinSum"],
    Cell[882915, 28884, 177, 4, 52, "Input",
      CellTags->"FermionSpinSum"],
    Cell[883095, 28890, 1621, 40, 70, "Output",
      CellTags->"FermionSpinSum"],
    Cell[884753, 28935, 180, 4, 52, "Input",
      CellTags->"FermionSpinSum"],
    Cell[884936, 28941, 1264, 36, 70, "Output",
      CellTags->"FermionSpinSum"],
    Cell[886215, 28980, 173, 4, 52, "Input",
      CellTags->"FermionSpinSum"]},
  "FeynAmp"->{
    Cell[886437, 28990, 52, 1, 39, "ObjectName",
      CellTags->"FeynAmp"],
    Cell[886514, 28995, 59, 1, 49, "Subsubsection",
      CellTags->"FeynAmp"],
    Cell[886576, 28998, 244, 5, 56, "Text",
      CellTags->"FeynAmp"],
    Cell[886823, 29005, 166, 4, 39, "Text",
      CellTags->"FeynAmp"],
    Cell[886992, 29011, 194, 9, 39, "Text",
      CellTags->"FeynAmp"],
    Cell[887223, 29025, 56, 1, 49, "Subsubsection",
      CellTags->"FeynAmp"],
    Cell[887282, 29028, 126, 4, 70, "Text",
      CellTags->"FeynAmp"],
    Cell[887433, 29036, 1007, 22, 70, "Input",
      CellTags->"FeynAmp"],
    Cell[894015, 29201, 74, 1, 70, "Text",
      CellTags->"FeynAmp"],
    Cell[894114, 29206, 240, 6, 70, "Input",
      CellTags->"FeynAmp"]},
  "FeynAmpDenominator"->{
    Cell[894906, 29234, 74, 1, 39, "ObjectName",
      CellTags->"FeynAmpDenominator"],
    Cell[895005, 29239, 70, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominator"],
    Cell[895078, 29242, 259, 6, 56, "Text",
      CellTags->"FeynAmpDenominator"],
    Cell[895340, 29250, 361, 14, 39, "Text",
      CellTags->"FeynAmpDenominator"],
    Cell[895738, 29269, 67, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominator"],
    Cell[895830, 29274, 238, 5, 70, "Input",
      CellTags->"FeynAmpDenominator"],
    Cell[896334, 29292, 387, 9, 70, "Input",
      CellTags->"FeynAmpDenominator"],
    Cell[897256, 29321, 297, 7, 70, "Input",
      CellTags->"FeynAmpDenominator"],
    Cell[897578, 29332, 181, 4, 70, "Input",
      CellTags->"FeynAmpDenominator"],
    Cell[898101, 29350, 181, 4, 70, "Input",
      CellTags->"FeynAmpDenominator"],
    Cell[898479, 29363, 166, 4, 70, "Input",
      CellTags->"FeynAmpDenominator"]},
  "FeynAmpDenominatorCombine"->{
    Cell[898694, 29373, 88, 1, 39, "ObjectName",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[898807, 29378, 77, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[898887, 29381, 229, 5, 56, "Text",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[899119, 29388, 254, 9, 39, "Text",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[899410, 29402, 74, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[899509, 29407, 282, 7, 70, "Input",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[900805, 29447, 249, 6, 70, "Input",
      CellTags->"FeynAmpDenominatorCombine"],
    Cell[901299, 29465, 247, 6, 70, "Input",
      CellTags->"FeynAmpDenominatorCombine"]},
  "FeynAmpDenominatorSimplify"->{
    Cell[901849, 29486, 90, 1, 39, "ObjectName",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[901964, 29491, 78, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[902045, 29494, 399, 7, 73, "Text",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[902447, 29503, 86, 1, 39, "Text",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[902558, 29508, 120, 2, 56, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[902829, 29517, 231, 9, 35, "Text",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[903097, 29531, 75, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[903175, 29534, 507, 14, 70, "Text",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[903707, 29552, 336, 8, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[904169, 29567, 575, 18, 70, "Text",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[904769, 29589, 401, 10, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[906035, 29629, 562, 15, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[909892, 29739, 258, 6, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[912377, 29811, 189, 4, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[913665, 29852, 464, 13, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"],
    Cell[918070, 29975, 179, 4, 70, "Input",
      CellTags->"FeynAmpDenominatorSimplify"]},
  "FeynAmpDenominatorSplit"->{
    Cell[918298, 29985, 84, 1, 39, "ObjectName",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[918407, 29990, 75, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[918485, 29993, 358, 7, 73, "Text",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[918846, 30002, 258, 9, 39, "Text",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[919141, 30016, 72, 1, 49, "Subsubsection",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[919238, 30021, 372, 9, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[920828, 30070, 144, 3, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[921137, 30082, 167, 3, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[922522, 30125, 182, 4, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[923205, 30148, 277, 7, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[923929, 30173, 247, 6, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"],
    Cell[924519, 30192, 149, 3, 70, "Input",
      CellTags->"FeynAmpDenominatorSplit"]},
  "FeynAmpList"->{
    Cell[924717, 30201, 60, 1, 39, "ObjectName",
      CellTags->"FeynAmpList"],
    Cell[924802, 30206, 63, 1, 49, "Subsubsection",
      CellTags->"FeynAmpList"],
    Cell[924868, 30209, 146, 4, 39, "Text",
      CellTags->"FeynAmpList"],
    Cell[925017, 30215, 120, 4, 39, "Text",
      CellTags->"FeynAmpList"],
    Cell[925140, 30221, 192, 9, 39, "Text",
      CellTags->"FeynAmpList"]},
  "FeynCalc"->{
    Cell[925381, 30236, 54, 1, 39, "ObjectName",
      CellTags->"FeynCalc"],
    Cell[925460, 30241, 60, 1, 49, "Subsubsection",
      CellTags->"FeynCalc"],
    Cell[925523, 30244, 92, 1, 39, "Text",
      CellTags->"FeynCalc"],
    Cell[925640, 30249, 125, 3, 56, "Input",
      CellTags->"FeynCalc"],
    Cell[925768, 30254, 582, 8, 208, "Print",
      CellTags->"FeynCalc"]},
  "FeynCalcExternal"->{
    Cell[926411, 30269, 70, 1, 39, "ObjectName",
      CellTags->"FeynCalcExternal"],
    Cell[926506, 30274, 68, 1, 49, "Subsubsection",
      CellTags->"FeynCalcExternal"],
    Cell[926577, 30277, 157, 4, 39, "Text",
      CellTags->"FeynCalcExternal"],
    Cell[926737, 30283, 224, 9, 39, "Text",
      CellTags->"FeynCalcExternal"],
    Cell[926998, 30297, 65, 1, 49, "Subsubsection",
      CellTags->"FeynCalcExternal"],
    Cell[927088, 30302, 194, 4, 56, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[927465, 30316, 145, 3, 52, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[927772, 30328, 530, 13, 52, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[929111, 30368, 145, 3, 52, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[929809, 30390, 149, 3, 52, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[931210, 30435, 145, 3, 52, "Input",
      CellTags->"FeynCalcExternal"],
    Cell[932424, 30468, 199, 5, 52, "Input",
      CellTags->"FeynCalcExternal"]},
  "FeynCalcForm"->{
    Cell[933200, 30494, 62, 1, 39, "ObjectName",
      CellTags->"FeynCalcForm"],
    Cell[933287, 30499, 64, 1, 49, "Subsubsection",
      CellTags->"FeynCalcForm"],
    Cell[933354, 30502, 540, 10, 73, "Text",
      CellTags->"FeynCalcForm"],
    Cell[933897, 30514, 466, 19, 39, "Text",
      CellTags->"FeynCalcForm"],
    Cell[934400, 30538, 61, 1, 49, "Subsubsection",
      CellTags->"FeynCalcForm"],
    Cell[934464, 30541, 80, 1, 25, "Text",
      CellTags->"FeynCalcForm"],
    Cell[934569, 30546, 287, 7, 56, "Input",
      CellTags->"FeynCalcForm"],
    Cell[934859, 30555, 761, 23, 37, "Output",
      CellTags->"FeynCalcForm"],
    Cell[935635, 30581, 115, 4, 21, "Text",
      CellTags->"FeynCalcForm"],
    Cell[935753, 30587, 178, 4, 43, "Input",
      CellTags->"FeynCalcForm"],
    Cell[935934, 30593, 867, 22, 63, "Input",
      CellTags->"FeynCalcForm"],
    Cell[936826, 30619, 287, 7, 56, "Input",
      CellTags->"FeynCalcForm"],
    Cell[937116, 30628, 216, 7, 35, "Output",
      CellTags->"FeynCalcForm"],
    Cell[937347, 30638, 77, 1, 21, "Text",
      CellTags->"FeynCalcForm"],
    Cell[937427, 30641, 153, 4, 43, "Input",
      CellTags->"FeynCalcForm"],
    Cell[937583, 30647, 895, 23, 63, "Input",
      CellTags->"FeynCalcForm"]},
  "FeynCalcInternal"->{
    Cell[938527, 30676, 70, 1, 39, "ObjectName",
      CellTags->"FeynCalcInternal"],
    Cell[938622, 30681, 68, 1, 49, "Subsubsection",
      CellTags->"FeynCalcInternal"],
    Cell[938693, 30684, 158, 4, 39, "Text",
      CellTags->"FeynCalcInternal"],
    Cell[938854, 30690, 434, 19, 39, "Text",
      CellTags->"FeynCalcInternal"],
    Cell[939325, 30714, 65, 1, 49, "Subsubsection",
      CellTags->"FeynCalcInternal"],
    Cell[939415, 30719, 530, 13, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[940754, 30759, 145, 3, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[941452, 30781, 149, 3, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[942853, 30826, 145, 3, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[944067, 30859, 199, 5, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[944819, 30883, 621, 15, 70, "Input",
      CellTags->"FeynCalcInternal"],
    Cell[947603, 30967, 145, 3, 70, "Input",
      CellTags->"FeynCalcInternal"]},
  "FeynCalc2FORM"->{
    Cell[949116, 31009, 64, 1, 39, "ObjectName",
      CellTags->"FeynCalc2FORM"],
    Cell[949205, 31014, 65, 1, 49, "Subsubsection",
      CellTags->"FeynCalc2FORM"],
    Cell[949273, 31017, 227, 5, 56, "Text",
      CellTags->"FeynCalc2FORM"],
    Cell[949503, 31024, 212, 9, 39, "Text",
      CellTags->"FeynCalc2FORM"],
    Cell[949740, 31037, 153, 3, 56, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[951185, 31074, 62, 1, 49, "Subsubsection",
      CellTags->"FeynCalc2FORM"],
    Cell[951272, 31079, 328, 9, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[951970, 31104, 169, 4, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[952267, 31116, 241, 6, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[952814, 31135, 169, 4, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[953107, 31147, 255, 6, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[953825, 31170, 169, 4, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[954141, 31183, 356, 9, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[955099, 31214, 169, 4, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[955393, 31225, 401, 10, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[955819, 31239, 406, 10, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[956429, 31260, 385, 10, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[965732, 31518, 252, 6, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[966009, 31528, 460, 11, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[968238, 31583, 380, 10, 70, "Input",
      CellTags->"FeynCalc2FORM"],
    Cell[968621, 31595, 161, 4, 70, "Input",
      CellTags->"FeynCalc2FORM"]},
  "FeynmanParameterNames"->{
    Cell[968831, 31605, 80, 1, 39, "ObjectName",
      CellTags->"FeynmanParameterNames"],
    Cell[968936, 31610, 73, 1, 49, "Subsubsection",
      CellTags->"FeynmanParameterNames"],
    Cell[969012, 31613, 144, 4, 39, "Text",
      CellTags->"FeynmanParameterNames"],
    Cell[969159, 31619, 385, 14, 39, "Text",
      CellTags->"FeynmanParameterNames"]},
  "FeynmanParametrize"->{
    Cell[969593, 31639, 91, 1, 39, "ObjectName",
      CellTags->"FeynmanParametrize"],
    Cell[969709, 31644, 70, 1, 49, "Subsubsection",
      CellTags->"FeynmanParametrize"],
    Cell[969782, 31647, 177, 4, 39, "Text",
      CellTags->"FeynmanParametrize"],
    Cell[969984, 31655, 161, 3, 56, "Input",
      CellTags->"FeynmanParametrize"],
    Cell[970466, 31673, 67, 1, 49, "Subsubsection",
      CellTags->"FeynmanParametrize"],
    Cell[970536, 31676, 109, 2, 56, "Input",
      CellTags->"FeynmanParametrize"]},
  "FeynRule"->{
    Cell[970694, 31684, 54, 1, 39, "ObjectName",
      CellTags->"FeynRule"],
    Cell[970773, 31689, 60, 1, 49, "Subsubsection",
      CellTags->"FeynRule"],
    Cell[970836, 31692, 169, 4, 39, "Text",
      CellTags->"FeynRule"],
    Cell[971030, 31700, 119, 3, 43, "Input",
      CellTags->"FeynRule"],
    Cell[971923, 31725, 93, 1, 35, "Text",
      CellTags->"FeynRule"],
    Cell[972019, 31728, 134, 4, 39, "Text",
      CellTags->"FeynRule"],
    Cell[972156, 31734, 198, 9, 39, "Text",
      CellTags->"FeynRule"],
    Cell[972391, 31748, 57, 1, 49, "Subsubsection",
      CellTags->"FeynRule"],
    Cell[972473, 31753, 154, 4, 43, "Input",
      CellTags->"FeynRule"],
    Cell[973516, 31786, 154, 4, 43, "Input",
      CellTags->"FeynRule"],
    Cell[975051, 31832, 115, 3, 43, "Input",
      CellTags->"FeynRule"],
    Cell[980928, 31995, 149, 4, 43, "Input",
      CellTags->"FeynRule"],
    Cell[984713, 32116, 75, 1, 39, "Text",
      CellTags->"FeynRule"],
    Cell[984813, 32121, 601, 16, 43, "Input",
      CellTags->"FeynRule"],
    Cell[986371, 32173, 371, 9, 56, "Input",
      CellTags->"FeynRule"],
    Cell[993145, 32341, 73, 1, 35, "Text",
      CellTags->"FeynRule"],
    Cell[993243, 32346, 625, 16, 56, "Input",
      CellTags->"FeynRule"],
    Cell[994849, 32398, 371, 9, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1002951, 32580, 268, 7, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1004690, 32634, 113, 4, 35, "Text",
      CellTags->"FeynRule"],
    Cell[1004828, 32642, 424, 10, 56, "Input",
      CellTags->"FeynRule"],
    Cell[1006825, 32702, 78, 1, 35, "Text",
      CellTags->"FeynRule"],
    Cell[1006928, 32707, 177, 4, 56, "Input",
      CellTags->"FeynRule"],
    Cell[1007929, 32738, 91, 1, 35, "Text",
      CellTags->"FeynRule"],
    Cell[1008045, 32743, 177, 4, 56, "Input",
      CellTags->"FeynRule"],
    Cell[1009068, 32776, 863, 22, 72, "Input",
      CellTags->"FeynRule"],
    Cell[1011269, 32843, 277, 7, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1018898, 33041, 501, 12, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1026755, 33224, 445, 12, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1027326, 33243, 211, 5, 52, "Text",
      CellTags->"FeynRule"],
    Cell[1027540, 33250, 176, 4, 56, "Input",
      CellTags->"FeynRule"],
    Cell[1027719, 33256, 52, 1, 39, "Text",
      CellTags->"FeynRule"],
    Cell[1027796, 33261, 854, 21, 76, "Input",
      CellTags->"FeynRule"],
    Cell[1030027, 33330, 260, 7, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1033154, 33421, 455, 11, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1036625, 33522, 240, 6, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1037013, 33537, 1084, 26, 72, "Input",
      CellTags->"FeynRule"],
    Cell[1039872, 33623, 260, 7, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1044991, 33769, 542, 13, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1054668, 34016, 240, 6, 52, "Input",
      CellTags->"FeynRule"],
    Cell[1055034, 34029, 475, 9, 72, "Input",
      CellTags->"FeynRule"]},
  "FI"->{
    Cell[1055558, 34044, 42, 1, 39, "ObjectName",
      CellTags->"FI"],
    Cell[1055625, 34049, 54, 1, 49, "Subsubsection",
      CellTags->"FI"],
    Cell[1055682, 34052, 198, 5, 56, "Text",
      CellTags->"FI"],
    Cell[1055883, 34059, 588, 24, 39, "Text",
      CellTags->"FI"]},
  "FieldDerivative"->{
    Cell[1056520, 34089, 68, 1, 39, "ObjectName",
      CellTags->"FieldDerivative"],
    Cell[1056613, 34094, 67, 1, 49, "Subsubsection",
      CellTags->"FieldDerivative"],
    Cell[1056683, 34097, 552, 10, 90, "Text",
      CellTags->"FieldDerivative"],
    Cell[1057238, 34109, 337, 14, 39, "Text",
      CellTags->"FieldDerivative"],
    Cell[1057612, 34128, 64, 1, 49, "Subsubsection",
      CellTags->"FieldDerivative"],
    Cell[1057701, 34133, 720, 20, 43, "Input",
      CellTags->"FieldDerivative"],
    Cell[1058984, 34176, 238, 7, 39, "Input",
      CellTags->"FieldDerivative"],
    Cell[1061249, 34246, 238, 7, 39, "Input",
      CellTags->"FieldDerivative"]},
  "FieldStrength"->{
    Cell[1067332, 34422, 64, 1, 39, "ObjectName",
      CellTags->"FieldStrength"],
    Cell[1067421, 34427, 65, 1, 49, "Subsubsection",
      CellTags->"FieldStrength"],
    Cell[1067489, 34430, 1453, 37, 75, "Text",
      CellTags->"FieldStrength"],
    Cell[1068967, 34471, 153, 3, 56, "Input",
      CellTags->"FieldStrength"],
    Cell[1069686, 34492, 209, 9, 35, "Text",
      CellTags->"FieldStrength"],
    Cell[1069932, 34506, 62, 1, 49, "Subsubsection",
      CellTags->"FieldStrength"],
    Cell[1070019, 34511, 182, 4, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1070444, 34527, 192, 4, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1070873, 34543, 237, 5, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1072089, 34585, 247, 5, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1074443, 34662, 284, 6, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1075381, 34689, 287, 6, 70, "Input",
      CellTags->"FieldStrength"],
    Cell[1076320, 34716, 339, 7, 70, "Input",
      CellTags->"FieldStrength"]},
  "FinalFunction"->{
    Cell[1078790, 34797, 64, 1, 39, "ObjectName",
      CellTags->"FinalFunction"],
    Cell[1078879, 34802, 65, 1, 49, "Subsubsection",
      CellTags->"FinalFunction"],
    Cell[1078947, 34805, 87, 1, 39, "Text",
      CellTags->"FinalFunction"],
    Cell[1079037, 34808, 203, 9, 39, "Text",
      CellTags->"FinalFunction"]},
  "FinalSubstitutions"->{
    Cell[1079289, 34823, 74, 1, 39, "ObjectName",
      CellTags->"FinalSubstitutions"],
    Cell[1079388, 34828, 70, 1, 49, "Subsubsection",
      CellTags->"FinalSubstitutions"],
    Cell[1079461, 34831, 207, 5, 39, "Text",
      CellTags->"FinalSubstitutions"],
    Cell[1079671, 34838, 439, 19, 39, "Text",
      CellTags->"FinalSubstitutions"]},
  "FORM"->{
    Cell[1080159, 34863, 46, 1, 39, "ObjectName",
      CellTags->"FORM"],
    Cell[1080230, 34868, 56, 1, 49, "Subsubsection",
      CellTags->"FORM"],
    Cell[1080289, 34871, 205, 5, 56, "Text",
      CellTags->"FORM"],
    Cell[1080497, 34878, 308, 14, 39, "Text",
      CellTags->"FORM"]},
  "FORMEpilog"->{
    Cell[1080854, 34898, 58, 1, 39, "ObjectName",
      CellTags->"FORMEpilog"],
    Cell[1080937, 34903, 62, 1, 49, "Subsubsection",
      CellTags->"FORMEpilog"],
    Cell[1081002, 34906, 162, 4, 39, "Text",
      CellTags->"FORMEpilog"],
    Cell[1081167, 34912, 335, 14, 39, "Text",
      CellTags->"FORMEpilog"]},
  "FORMProlog"->{
    Cell[1081551, 34932, 58, 1, 39, "ObjectName",
      CellTags->"FORMProlog"],
    Cell[1081634, 34937, 62, 1, 49, "Subsubsection",
      CellTags->"FORMProlog"],
    Cell[1081699, 34940, 181, 5, 39, "Text",
      CellTags->"FORMProlog"],
    Cell[1081883, 34947, 335, 14, 39, "Text",
      CellTags->"FORMProlog"]},
  "FORM2FeynCalc"->{
    Cell[1082267, 34967, 64, 1, 39, "ObjectName",
      CellTags->"FORM2FeynCalc"],
    Cell[1082356, 34972, 65, 1, 49, "Subsubsection",
      CellTags->"FORM2FeynCalc"],
    Cell[1082424, 34975, 471, 8, 73, "Text",
      CellTags->"FORM2FeynCalc"],
    Cell[1082898, 34985, 212, 9, 39, "Text",
      CellTags->"FORM2FeynCalc"],
    Cell[1083135, 34998, 153, 3, 56, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1084094, 35026, 62, 1, 49, "Subsubsection",
      CellTags->"FORM2FeynCalc"],
    Cell[1084181, 35031, 167, 3, 56, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1084834, 35054, 142, 3, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1085239, 35068, 160, 4, 35, "Text",
      CellTags->"FORM2FeynCalc"],
    Cell[1085424, 35076, 384, 9, 76, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1086201, 35101, 142, 3, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1086713, 35119, 180, 4, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1087285, 35140, 145, 3, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1087711, 35157, 180, 4, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1088233, 35176, 142, 3, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1088756, 35195, 401, 10, 52, "Input",
      CellTags->"FORM2FeynCalc"],
    Cell[1089518, 35221, 231, 6, 72, "Input",
      CellTags->"FORM2FeynCalc"]},
  "FourDivergence"->{
    Cell[1090453, 35252, 66, 1, 39, "ObjectName",
      CellTags->"FourDivergence"],
    Cell[1090544, 35257, 66, 1, 49, "Subsubsection",
      CellTags->"FourDivergence"],
    Cell[1090613, 35260, 243, 5, 56, "Text",
      CellTags->"FourDivergence"],
    Cell[1090859, 35267, 210, 9, 39, "Text",
      CellTags->"FourDivergence"],
    Cell[1091106, 35281, 63, 1, 49, "Subsubsection",
      CellTags->"FourDivergence"],
    Cell[1091194, 35286, 204, 5, 70, "Input",
      CellTags->"FourDivergence"],
    Cell[1091706, 35305, 254, 6, 70, "Input",
      CellTags->"FourDivergence"],
    Cell[1092285, 35327, 235, 6, 70, "Input",
      CellTags->"FourDivergence"],
    Cell[1093115, 35355, 289, 8, 70, "Input",
      CellTags->"FourDivergence"],
    Cell[1093530, 35370, 140, 3, 70, "Input",
      CellTags->"FourDivergence"]},
  "FourLaplacian"->{
    Cell[1093719, 35379, 64, 1, 39, "ObjectName",
      CellTags->"FourLaplacian"],
    Cell[1093808, 35384, 65, 1, 49, "Subsubsection",
      CellTags->"FourLaplacian"],
    Cell[1093876, 35387, 491, 16, 40, "Text",
      CellTags->"FourLaplacian"],
    Cell[1094392, 35407, 153, 3, 56, "Input",
      CellTags->"FourLaplacian"],
    Cell[1094748, 35420, 347, 14, 35, "Text",
      CellTags->"FourLaplacian"],
    Cell[1095132, 35439, 62, 1, 49, "Subsubsection",
      CellTags->"FourLaplacian"],
    Cell[1095219, 35444, 163, 4, 70, "Input",
      CellTags->"FourLaplacian"],
    Cell[1095605, 35460, 184, 4, 70, "Input",
      CellTags->"FourLaplacian"],
    Cell[1095964, 35474, 325, 9, 70, "Input",
      CellTags->"FourLaplacian"],
    Cell[1096910, 35507, 189, 4, 70, "Input",
      CellTags->"FourLaplacian"]},
  "FourVector"->{
    Cell[1102868, 35673, 58, 1, 39, "ObjectName",
      CellTags->"FourVector"],
    Cell[1102951, 35678, 62, 1, 49, "Subsubsection",
      CellTags->"FourVector"],
    Cell[1103016, 35681, 353, 10, 56, "Text",
      CellTags->"FourVector"],
    Cell[1103372, 35693, 389, 19, 39, "Text",
      CellTags->"FourVector"],
    Cell[1103798, 35717, 59, 1, 49, "Subsubsection",
      CellTags->"FourVector"],
    Cell[1103882, 35722, 172, 4, 70, "Input",
      CellTags->"FourVector"],
    Cell[1104232, 35736, 201, 5, 70, "Input",
      CellTags->"FourVector"],
    Cell[1104806, 35758, 217, 5, 70, "Input",
      CellTags->"FourVector"],
    Cell[1105224, 35773, 272, 6, 70, "Input",
      CellTags->"FourVector"],
    Cell[1105750, 35790, 255, 6, 70, "Input",
      CellTags->"FourVector"],
    Cell[1106184, 35804, 196, 5, 70, "Text",
      CellTags->"FourVector"],
    Cell[1106405, 35813, 255, 6, 70, "Input",
      CellTags->"FourVector"]},
  "FreeIndex"->{
    Cell[1107241, 35845, 56, 1, 39, "ObjectName",
      CellTags->"FreeIndex"],
    Cell[1107322, 35850, 61, 1, 49, "Subsubsection",
      CellTags->"FreeIndex"],
    Cell[1107386, 35853, 153, 4, 39, "Text",
      CellTags->"FreeIndex"],
    Cell[1107542, 35859, 313, 14, 39, "Text",
      CellTags->"FreeIndex"]},
  "FreeQ2"->{
    Cell[1107904, 35879, 50, 1, 39, "ObjectName",
      CellTags->"FreeQ2"],
    Cell[1107979, 35884, 58, 1, 49, "Subsubsection",
      CellTags->"FreeQ2"],
    Cell[1108040, 35887, 229, 5, 56, "Text",
      CellTags->"FreeQ2"],
    Cell[1108272, 35894, 331, 14, 39, "Text",
      CellTags->"FreeQ2"],
    Cell[1108640, 35913, 55, 1, 49, "Subsubsection",
      CellTags->"FreeQ2"],
    Cell[1108720, 35918, 300, 8, 70, "Input",
      CellTags->"FreeQ2"],
    Cell[1109172, 35935, 295, 8, 70, "Input",
      CellTags->"FreeQ2"],
    Cell[1109618, 35952, 165, 4, 70, "Input",
      CellTags->"FreeQ2"],
    Cell[1109934, 35965, 199, 5, 70, "Input",
      CellTags->"FreeQ2"]},
  "FRH"->{
    Cell[1110309, 35981, 44, 1, 39, "ObjectName",
      CellTags->"FRH"],
    Cell[1110378, 35986, 55, 1, 49, "Subsubsection",
      CellTags->"FRH"],
    Cell[1110436, 35989, 135, 4, 39, "Text",
      CellTags->"FRH"],
    Cell[1110574, 35995, 184, 9, 39, "Text",
      CellTags->"FRH"],
    Cell[1110795, 36009, 52, 1, 49, "Subsubsection",
      CellTags->"FRH"],
    Cell[1110872, 36014, 243, 6, 56, "Input",
      CellTags->"FRH"],
    Cell[1111446, 36035, 127, 3, 52, "Input",
      CellTags->"FRH"],
    Cell[1111721, 36047, 373, 10, 52, "Input",
      CellTags->"FRH"],
    Cell[1112791, 36083, 159, 4, 52, "Input",
      CellTags->"FRH"]},
  "FromTFi"->{
    Cell[1113688, 36116, 52, 1, 39, "ObjectName",
      CellTags->"FromTFi"],
    Cell[1113765, 36121, 59, 1, 49, "Subsubsection",
      CellTags->"FromTFi"],
    Cell[1113827, 36124, 201, 5, 56, "Text",
      CellTags->"FromTFi"],
    Cell[1114031, 36131, 280, 13, 39, "Text",
      CellTags->"FromTFi"],
    Cell[1114348, 36149, 56, 1, 49, "Subsubsection",
      CellTags->"FromTFi"],
    Cell[1114429, 36154, 453, 12, 70, "Input",
      CellTags->"FromTFi"],
    Cell[1116055, 36207, 873, 22, 70, "Input",
      CellTags->"FromTFi"],
    Cell[1118397, 36276, 951, 24, 70, "Input",
      CellTags->"FromTFi"]},
  "FUNCTION"->{
    Cell[1121228, 36359, 54, 1, 39, "ObjectName",
      CellTags->"FUNCTION"],
    Cell[1121307, 36364, 60, 1, 49, "Subsubsection",
      CellTags->"FUNCTION"],
    Cell[1121370, 36367, 161, 4, 39, "Text",
      CellTags->"FUNCTION"],
    Cell[1121534, 36373, 186, 9, 39, "Text",
      CellTags->"FUNCTION"]},
  "FunctionalD"->{
    Cell[1121769, 36388, 60, 1, 39, "ObjectName",
      CellTags->"FunctionalD"],
    Cell[1121854, 36393, 63, 1, 49, "Subsubsection",
      CellTags->"FunctionalD"],
    Cell[1121920, 36396, 419, 8, 73, "Text",
      CellTags->"FunctionalD"],
    Cell[1122342, 36406, 96, 1, 39, "Text",
      CellTags->"FunctionalD"],
    Cell[1122441, 36409, 327, 14, 39, "Text",
      CellTags->"FunctionalD"],
    Cell[1122805, 36428, 60, 1, 49, "Subsubsection",
      CellTags->"FunctionalD"],
    Cell[1122868, 36431, 692, 20, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1123585, 36455, 270, 6, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1124003, 36470, 301, 7, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1124462, 36485, 1498, 44, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1125985, 36533, 359, 9, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1126683, 36556, 968, 24, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1127676, 36584, 843, 22, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1129497, 36638, 259, 6, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1130581, 36673, 590, 16, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1131174, 36691, 99, 2, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1131298, 36697, 371, 8, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1133500, 36768, 371, 8, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1135702, 36839, 268, 8, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1140281, 36968, 579, 13, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1140863, 36983, 1926, 50, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1142792, 37035, 534, 13, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1143351, 37052, 280, 7, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1144018, 37077, 202, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1147429, 37181, 123, 4, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1147577, 37189, 213, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1150999, 37293, 303, 8, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1151305, 37303, 652, 18, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1151982, 37325, 278, 7, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1152800, 37355, 289, 7, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1153679, 37386, 722, 17, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1158574, 37535, 221, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1160822, 37608, 428, 11, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1162896, 37674, 294, 9, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1163193, 37685, 100, 2, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1163296, 37689, 168, 4, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1163489, 37697, 133, 3, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1167955, 37823, 237, 6, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1201917, 38837, 232, 5, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1202152, 38844, 348, 7, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1202503, 38853, 164, 4, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1202670, 38859, 3947, 76, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1206642, 38939, 206, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1224232, 39475, 201, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1233655, 39769, 225, 8, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1233905, 39781, 401, 11, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1238171, 39917, 828, 22, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1240869, 40005, 186, 4, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1248397, 40240, 132, 4, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1248554, 40248, 104, 2, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1250326, 40307, 957, 22, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1252815, 40377, 169, 4, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1257894, 40521, 246, 6, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1266759, 40799, 170, 4, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1278278, 41159, 200, 5, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1278604, 41171, 67, 1, 70, "Text",
      CellTags->"FunctionalD"],
    Cell[1278674, 41174, 617, 13, 70, "Input",
      CellTags->"FunctionalD"],
    Cell[1279328, 41192, 111, 2, 49, "Subsubsection",
      CellTags->"FunctionalD"],
    Cell[1279442, 41196, 80, 1, 39, "Text",
      CellTags->"FunctionalD"],
    Cell[1282749, 41295, 949, 24, 63, "Input",
      CellTags->"FunctionalD"],
    Cell[1296986, 41653, 177, 5, 43, "Input",
      CellTags->"FunctionalD"]},
  "FV"->{
    Cell[1300105, 41748, 42, 1, 39, "ObjectName",
      CellTags->"FV"],
    Cell[1300172, 41753, 54, 1, 49, "Subsubsection",
      CellTags->"FV"],
    Cell[1300229, 41756, 298, 13, 39, "Text",
      CellTags->"FV"],
    Cell[1300530, 41771, 615, 29, 39, "Text",
      CellTags->"FV"],
    Cell[1301182, 41805, 51, 1, 49, "Subsubsection",
      CellTags->"FV"],
    Cell[1301258, 41810, 156, 4, 70, "Input",
      CellTags->"FV"],
    Cell[1301594, 41824, 185, 5, 70, "Input",
      CellTags->"FV"],
    Cell[1302072, 41843, 197, 5, 70, "Input",
      CellTags->"FV"],
    Cell[1302462, 41858, 242, 7, 70, "Input",
      CellTags->"FV"],
    Cell[1302963, 41875, 131, 4, 70, "Text",
      CellTags->"FV"],
    Cell[1303119, 41883, 239, 6, 70, "Input",
      CellTags->"FV"],
    Cell[1303915, 41913, 135, 3, 70, "Input",
      CellTags->"FV"]},
  "FVD"->{
    Cell[1304546, 41935, 44, 1, 39, "ObjectName",
      CellTags->"FVD"],
    Cell[1304615, 41940, 55, 1, 49, "Subsubsection",
      CellTags->"FVD"],
    Cell[1304673, 41943, 334, 15, 39, "Text",
      CellTags->"FVD"],
    Cell[1305010, 41960, 613, 29, 39, "Text",
      CellTags->"FVD"],
    Cell[1305660, 41994, 52, 1, 49, "Subsubsection",
      CellTags->"FVD"],
    Cell[1305737, 41999, 158, 4, 70, "Input",
      CellTags->"FVD"],
    Cell[1306075, 42013, 187, 5, 70, "Input",
      CellTags->"FVD"],
    Cell[1306555, 42032, 199, 5, 70, "Input",
      CellTags->"FVD"],
    Cell[1306948, 42047, 244, 7, 70, "Input",
      CellTags->"FVD"],
    Cell[1307513, 42066, 89, 1, 70, "Text",
      CellTags->"FVD"],
    Cell[1307627, 42071, 241, 6, 70, "Input",
      CellTags->"FVD"],
    Cell[1308425, 42101, 136, 3, 70, "Input",
      CellTags->"FVD"]},
  "GA"->{
    Cell[1309189, 42127, 42, 1, 39, "ObjectName",
      CellTags->"GA"],
    Cell[1309256, 42132, 54, 1, 49, "Subsubsection",
      CellTags->"GA"],
    Cell[1309313, 42135, 708, 24, 57, "Text",
      CellTags->"GA"],
    Cell[1310024, 42161, 402, 19, 39, "Text",
      CellTags->"GA"],
    Cell[1310463, 42185, 51, 1, 49, "Subsubsection",
      CellTags->"GA"],
    Cell[1310539, 42190, 129, 3, 70, "Input",
      CellTags->"GA"],
    Cell[1310855, 42203, 259, 7, 70, "Input",
      CellTags->"GA"],
    Cell[1311521, 42227, 208, 5, 70, "Input",
      CellTags->"GA"],
    Cell[1311946, 42242, 199, 5, 70, "Input",
      CellTags->"GA"],
    Cell[1312510, 42262, 246, 6, 70, "Input",
      CellTags->"GA"],
    Cell[1313084, 42281, 301, 8, 70, "Input",
      CellTags->"GA"]},
  "GAD"->{
    Cell[1313844, 42308, 44, 1, 39, "ObjectName",
      CellTags->"GAD"],
    Cell[1313913, 42313, 55, 1, 49, "Subsubsection",
      CellTags->"GAD"],
    Cell[1313971, 42316, 725, 24, 57, "Text",
      CellTags->"GAD"],
    Cell[1314699, 42342, 400, 19, 39, "Text",
      CellTags->"GAD"],
    Cell[1315136, 42366, 52, 1, 49, "Subsubsection",
      CellTags->"GAD"],
    Cell[1315213, 42371, 131, 3, 70, "Input",
      CellTags->"GAD"],
    Cell[1315531, 42384, 262, 7, 70, "Input",
      CellTags->"GAD"],
    Cell[1316200, 42408, 210, 5, 70, "Input",
      CellTags->"GAD"],
    Cell[1316687, 42425, 201, 5, 70, "Input",
      CellTags->"GAD"],
    Cell[1317253, 42445, 248, 6, 70, "Input",
      CellTags->"GAD"],
    Cell[1317833, 42464, 305, 8, 70, "Input",
      CellTags->"GAD"]},
  "GammaEpsilon"->{
    Cell[1318597, 42491, 62, 1, 39, "ObjectName",
      CellTags->"GammaEpsilon"],
    Cell[1318684, 42496, 64, 1, 49, "Subsubsection",
      CellTags->"GammaEpsilon"],
    Cell[1318751, 42499, 168, 4, 39, "Text",
      CellTags->"GammaEpsilon"],
    Cell[1318922, 42505, 322, 14, 39, "Text",
      CellTags->"GammaEpsilon"],
    Cell[1319281, 42524, 61, 1, 49, "Subsubsection",
      CellTags->"GammaEpsilon"],
    Cell[1319345, 42527, 142, 4, 70, "Text",
      CellTags->"GammaEpsilon"],
    Cell[1319512, 42535, 207, 5, 70, "Input",
      CellTags->"GammaEpsilon"],
    Cell[1321840, 42604, 207, 5, 70, "Input",
      CellTags->"GammaEpsilon"],
    Cell[1323857, 42664, 92, 1, 70, "Text",
      CellTags->"GammaEpsilon"],
    Cell[1323974, 42669, 151, 3, 70, "Input",
      CellTags->"GammaEpsilon"],
    Cell[1326081, 42733, 145, 3, 70, "Input",
      CellTags->"GammaEpsilon"]},
  "GammaExpand"->{
    Cell[1326446, 42749, 60, 1, 39, "ObjectName",
      CellTags->"GammaExpand"],
    Cell[1326531, 42754, 63, 1, 49, "Subsubsection",
      CellTags->"GammaExpand"],
    Cell[1326597, 42757, 126, 4, 39, "Text",
      CellTags->"GammaExpand"],
    Cell[1326726, 42763, 207, 9, 39, "Text",
      CellTags->"GammaExpand"],
    Cell[1326970, 42777, 60, 1, 49, "Subsubsection",
      CellTags->"GammaExpand"],
    Cell[1327055, 42782, 224, 5, 70, "Input",
      CellTags->"GammaExpand"],
    Cell[1327732, 42806, 240, 6, 70, "Input",
      CellTags->"GammaExpand"],
    Cell[1328803, 42841, 224, 5, 70, "Input",
      CellTags->"GammaExpand"]},
  "Gamma1"->{
    Cell[1329336, 42861, 50, 1, 39, "ObjectName",
      CellTags->"Gamma1"],
    Cell[1329411, 42866, 58, 1, 49, "Subsubsection",
      CellTags->"Gamma1"],
    Cell[1329472, 42869, 140, 4, 39, "Text",
      CellTags->"Gamma1"],
    Cell[1329615, 42875, 298, 14, 39, "Text",
      CellTags->"Gamma1"]},
  "Gamma2"->{
    Cell[1329962, 42895, 50, 1, 39, "ObjectName",
      CellTags->"Gamma2"],
    Cell[1330037, 42900, 58, 1, 49, "Subsubsection",
      CellTags->"Gamma2"],
    Cell[1330098, 42903, 173, 4, 39, "Text",
      CellTags->"Gamma2"],
    Cell[1330274, 42909, 298, 14, 39, "Text",
      CellTags->"Gamma2"]},
  "Gamma3"->{
    Cell[1330621, 42929, 50, 1, 39, "ObjectName",
      CellTags->"Gamma3"],
    Cell[1330696, 42934, 58, 1, 49, "Subsubsection",
      CellTags->"Gamma3"],
    Cell[1330757, 42937, 238, 5, 56, "Text",
      CellTags->"Gamma3"],
    Cell[1330998, 42944, 298, 14, 39, "Text",
      CellTags->"Gamma3"]},
  "Gauge"->{
    Cell[1331345, 42964, 48, 1, 39, "ObjectName",
      CellTags->"Gauge"],
    Cell[1331418, 42969, 57, 1, 49, "Subsubsection",
      CellTags->"Gauge"],
    Cell[1331478, 42972, 134, 4, 39, "Text",
      CellTags->"Gauge"],
    Cell[1331615, 42978, 324, 14, 39, "Text",
      CellTags->"Gauge"]},
  "GaugeField"->{
    Cell[1331988, 42998, 58, 1, 39, "ObjectName",
      CellTags->"GaugeField"],
    Cell[1332071, 43003, 62, 1, 49, "Subsubsection",
      CellTags->"GaugeField"],
    Cell[1332136, 43006, 213, 5, 56, "Text",
      CellTags->"GaugeField"],
    Cell[1332352, 43013, 341, 14, 39, "Text",
      CellTags->"GaugeField"],
    Cell[1332730, 43032, 59, 1, 49, "Subsubsection",
      CellTags->"GaugeField"],
    Cell[1332814, 43037, 111, 2, 70, "Input",
      CellTags->"GaugeField"],
    Cell[1332928, 43041, 134, 3, 70, "Output",
      CellTags->"GaugeField"],
    Cell[1333099, 43049, 279, 6, 70, "Input",
      CellTags->"GaugeField"],
    Cell[1333381, 43057, 417, 14, 70, "Output",
      CellTags->"GaugeField"]},
  "GaugeXi"->{
    Cell[1333859, 43078, 52, 1, 39, "ObjectName",
      CellTags->"GaugeXi"],
    Cell[1333936, 43083, 59, 1, 49, "Subsubsection",
      CellTags->"GaugeXi"],
    Cell[1333998, 43086, 78, 1, 39, "Text",
      CellTags->"GaugeXi"],
    Cell[1334079, 43089, 308, 14, 39, "Text",
      CellTags->"GaugeXi"]},
  "GA5"->{
    Cell[1334436, 43109, 44, 1, 39, "ObjectName",
      CellTags->"GA5"],
    Cell[1334505, 43114, 55, 1, 49, "Subsubsection",
      CellTags->"GA5"],
    Cell[1334563, 43117, 89, 1, 39, "Text",
      CellTags->"GA5"],
    Cell[1334655, 43120, 397, 19, 39, "Text",
      CellTags->"GA5"],
    Cell[1335089, 43144, 52, 1, 49, "Subsubsection",
      CellTags->"GA5"],
    Cell[1335166, 43149, 95, 2, 56, "Input",
      CellTags->"GA5"],
    Cell[1335442, 43161, 120, 3, 52, "Input",
      CellTags->"GA5"]},
  "GGV"->{
    Cell[1335732, 43175, 44, 1, 39, "ObjectName",
      CellTags->"GGV"],
    Cell[1335801, 43180, 55, 1, 49, "Subsubsection",
      CellTags->"GGV"],
    Cell[1335859, 43183, 73, 1, 39, "Text",
      CellTags->"GGV"],
    Cell[1335935, 43186, 211, 9, 39, "Text",
      CellTags->"GGV"]},
  "GhostPropagator"->{
    Cell[1336195, 43201, 68, 1, 39, "ObjectName",
      CellTags->"GhostPropagator"],
    Cell[1336288, 43206, 67, 1, 49, "Subsubsection",
      CellTags->"GhostPropagator"],
    Cell[1336358, 43209, 349, 9, 39, "Text",
      CellTags->"GhostPropagator"],
    Cell[1336710, 43220, 469, 19, 39, "Text",
      CellTags->"GhostPropagator"],
    Cell[1337216, 43244, 64, 1, 49, "Subsubsection",
      CellTags->"GhostPropagator"],
    Cell[1337305, 43249, 188, 4, 70, "Input",
      CellTags->"GhostPropagator"],
    Cell[1338060, 43275, 151, 3, 70, "Input",
      CellTags->"GhostPropagator"],
    Cell[1338426, 43289, 188, 4, 70, "Input",
      CellTags->"GhostPropagator"],
    Cell[1339185, 43315, 305, 8, 70, "Input",
      CellTags->"GhostPropagator"]},
  "GHP"->{
    Cell[1339816, 43338, 44, 1, 39, "ObjectName",
      CellTags->"GHP"],
    Cell[1339885, 43343, 55, 1, 49, "Subsubsection",
      CellTags->"GHP"],
    Cell[1339943, 43346, 313, 9, 39, "Text",
      CellTags->"GHP"],
    Cell[1340259, 43357, 493, 19, 39, "Text",
      CellTags->"GHP"],
    Cell[1340789, 43381, 52, 1, 49, "Subsubsection",
      CellTags->"GHP"],
    Cell[1340866, 43386, 164, 4, 70, "Input",
      CellTags->"GHP"],
    Cell[1341597, 43412, 127, 3, 70, "Input",
      CellTags->"GHP"],
    Cell[1341939, 43426, 164, 4, 70, "Input",
      CellTags->"GHP"],
    Cell[1342674, 43452, 282, 8, 70, "Input",
      CellTags->"GHP"]},
  "GluonField"->{
    Cell[1343282, 43475, 58, 1, 39, "ObjectName",
      CellTags->"GluonField"],
    Cell[1343365, 43480, 62, 1, 49, "Subsubsection",
      CellTags->"GluonField"],
    Cell[1343430, 43483, 80, 1, 39, "Text",
      CellTags->"GluonField"],
    Cell[1343513, 43486, 200, 9, 39, "Text",
      CellTags->"GluonField"],
    Cell[1343750, 43500, 59, 1, 49, "Subsubsection",
      CellTags->"GluonField"],
    Cell[1343834, 43505, 109, 2, 56, "Input",
      CellTags->"GluonField"],
    Cell[1344089, 43516, 277, 6, 52, "Input",
      CellTags->"GluonField"]},
  "GluonGhostVertex"->{
    Cell[1344819, 43544, 70, 1, 39, "ObjectName",
      CellTags->"GluonGhostVertex"],
    Cell[1344914, 43549, 68, 1, 49, "Subsubsection",
      CellTags->"GluonGhostVertex"],
    Cell[1344985, 43552, 425, 8, 73, "Text",
      CellTags->"GluonGhostVertex"],
    Cell[1345413, 43562, 625, 23, 39, "Text",
      CellTags->"GluonGhostVertex"],
    Cell[1346063, 43589, 159, 3, 56, "Input",
      CellTags->"GluonGhostVertex"],
    Cell[1346567, 43607, 65, 1, 49, "Subsubsection",
      CellTags->"GluonGhostVertex"],
    Cell[1346657, 43612, 413, 10, 70, "Input",
      CellTags->"GluonGhostVertex"],
    Cell[1348112, 43660, 145, 3, 70, "Input",
      CellTags->"GluonGhostVertex"],
    Cell[1348856, 43683, 227, 5, 70, "Input",
      CellTags->"GluonGhostVertex"]},
  "GluonPropagator"->{
    Cell[1350153, 43728, 68, 1, 39, "ObjectName",
      CellTags->"GluonPropagator"],
    Cell[1350246, 43733, 67, 1, 49, "Subsubsection",
      CellTags->"GluonPropagator"],
    Cell[1350316, 43736, 543, 10, 73, "Text",
      CellTags->"GluonPropagator"],
    Cell[1350862, 43748, 100, 1, 39, "Text",
      CellTags->"GluonPropagator"],
    Cell[1350987, 43753, 108, 2, 56, "Input",
      CellTags->"GluonPropagator"],
    Cell[1351257, 43764, 157, 3, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1351909, 43783, 634, 24, 35, "Text",
      CellTags->"GluonPropagator"],
    Cell[1352580, 43812, 73, 1, 50, "Subsubsection",
      CellTags->"GluonPropagator"],
    Cell[1352678, 43817, 223, 5, 56, "Input",
      CellTags->"GluonPropagator"],
    Cell[1353872, 43855, 272, 5, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1355878, 43913, 383, 9, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1360585, 44042, 196, 4, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1361377, 44067, 150, 4, 35, "Text",
      CellTags->"GluonPropagator"],
    Cell[1361552, 44075, 175, 4, 56, "Input",
      CellTags->"GluonPropagator"],
    Cell[1362698, 44112, 285, 8, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1363384, 44136, 276, 5, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1365756, 44201, 276, 5, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1368134, 44267, 276, 5, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1370514, 44332, 276, 5, 52, "Input",
      CellTags->"GluonPropagator"],
    Cell[1372886, 44397, 276, 5, 52, "Input",
      CellTags->"GluonPropagator"]},
  "GluonSelfEnergy"->{
    Cell[1377211, 44514, 68, 1, 39, "ObjectName",
      CellTags->"GluonSelfEnergy"],
    Cell[1377304, 44519, 67, 1, 49, "Subsubsection",
      CellTags->"GluonSelfEnergy"],
    Cell[1377374, 44522, 125, 4, 39, "Text",
      CellTags->"GluonSelfEnergy"],
    Cell[1377524, 44530, 155, 3, 56, "Input",
      CellTags->"GluonSelfEnergy"],
    Cell[1378653, 44561, 634, 24, 35, "Text",
      CellTags->"GluonSelfEnergy"],
    Cell[1379324, 44590, 73, 1, 50, "Subsubsection",
      CellTags->"GluonSelfEnergy"],
    Cell[1379422, 44595, 304, 8, 56, "Input",
      CellTags->"GluonSelfEnergy"]},
  "GluonVertex"->{
    Cell[1383057, 44702, 60, 1, 39, "ObjectName",
      CellTags->"GluonVertex"],
    Cell[1383142, 44707, 63, 1, 49, "Subsubsection",
      CellTags->"GluonVertex"],
    Cell[1383208, 44710, 89, 1, 39, "Text",
      CellTags->"GluonVertex"],
    Cell[1383300, 44713, 622, 10, 90, "Text",
      CellTags->"GluonVertex"],
    Cell[1383925, 44725, 92, 1, 39, "Text",
      CellTags->"GluonVertex"],
    Cell[1384042, 44730, 104, 2, 56, "Input",
      CellTags->"GluonVertex"],
    Cell[1384304, 44741, 149, 3, 52, "Input",
      CellTags->"GluonVertex"],
    Cell[1384894, 44759, 360, 14, 35, "Text",
      CellTags->"GluonVertex"],
    Cell[1385291, 44778, 60, 1, 49, "Subsubsection",
      CellTags->"GluonVertex"],
    Cell[1385376, 44783, 403, 10, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1387245, 44842, 459, 12, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1392244, 44989, 373, 10, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1393062, 45016, 429, 12, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1397072, 45146, 490, 12, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1406697, 45392, 450, 12, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1416288, 45638, 266, 6, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1425673, 45876, 203, 5, 70, "Text",
      CellTags->"GluonVertex"],
    Cell[1425901, 45885, 208, 5, 70, "Input",
      CellTags->"GluonVertex"],
    Cell[1427574, 45939, 237, 6, 70, "Input",
      CellTags->"GluonVertex"]},
  "GO"->{
    Cell[1436928, 46181, 42, 1, 39, "ObjectName",
      CellTags->"GO"],
    Cell[1436995, 46186, 54, 1, 49, "Subsubsection",
      CellTags->"GO"],
    Cell[1437052, 46189, 74, 1, 39, "Text",
      CellTags->"GO"],
    Cell[1437129, 46192, 219, 9, 39, "Text",
      CellTags->"GO"]},
  "GP"->{
    Cell[1437397, 46207, 42, 1, 39, "ObjectName",
      CellTags->"GP"],
    Cell[1437464, 46212, 54, 1, 49, "Subsubsection",
      CellTags->"GP"],
    Cell[1437521, 46215, 70, 1, 39, "Text",
      CellTags->"GP"],
    Cell[1437594, 46218, 207, 9, 39, "Text",
      CellTags->"GP"]},
  "GrassmannParity"->{
    Cell[1437850, 46233, 68, 1, 39, "ObjectName",
      CellTags->"GrassmannParity"],
    Cell[1437943, 46238, 67, 1, 49, "Subsubsection",
      CellTags->"GrassmannParity"],
    Cell[1438013, 46241, 217, 5, 56, "Text",
      CellTags->"GrassmannParity"],
    Cell[1438233, 46248, 199, 9, 39, "Text",
      CellTags->"GrassmannParity"]},
  "GS"->{
    Cell[1438481, 46263, 42, 1, 39, "ObjectName",
      CellTags->"GS"],
    Cell[1438548, 46268, 54, 1, 49, "Subsubsection",
      CellTags->"GS"],
    Cell[1438605, 46271, 727, 18, 57, "Text",
      CellTags->"GS"],
    Cell[1439335, 46291, 525, 24, 39, "Text",
      CellTags->"GS"],
    Cell[1439897, 46320, 51, 1, 49, "Subsubsection",
      CellTags->"GS"],
    Cell[1439973, 46325, 125, 3, 70, "Input",
      CellTags->"GS"],
    Cell[1440326, 46339, 203, 6, 70, "Input",
      CellTags->"GS"],
    Cell[1440738, 46355, 172, 4, 70, "Input",
      CellTags->"GS"],
    Cell[1441617, 46385, 220, 6, 70, "Input",
      CellTags->"GS"],
    Cell[1442145, 46404, 288, 8, 70, "Input",
      CellTags->"GS"]},
  "GSD"->{
    Cell[1443061, 46437, 44, 1, 39, "ObjectName",
      CellTags->"GSD"],
    Cell[1443130, 46442, 55, 1, 49, "Subsubsection",
      CellTags->"GSD"],
    Cell[1443188, 46445, 736, 18, 57, "Text",
      CellTags->"GSD"],
    Cell[1443927, 46465, 526, 24, 39, "Text",
      CellTags->"GSD"],
    Cell[1444490, 46494, 52, 1, 49, "Subsubsection",
      CellTags->"GSD"],
    Cell[1444567, 46499, 127, 3, 70, "Input",
      CellTags->"GSD"],
    Cell[1444922, 46513, 205, 6, 70, "Input",
      CellTags->"GSD"],
    Cell[1445396, 46531, 174, 4, 70, "Input",
      CellTags->"GSD"],
    Cell[1446277, 46561, 222, 6, 70, "Input",
      CellTags->"GSD"],
    Cell[1446811, 46580, 292, 8, 70, "Input",
      CellTags->"GSD"]},
  "Gstrong"->{
    Cell[1447731, 46613, 52, 1, 39, "ObjectName",
      CellTags->"Gstrong"],
    Cell[1447808, 46618, 59, 1, 49, "Subsubsection",
      CellTags->"Gstrong"],
    Cell[1447870, 46621, 84, 1, 39, "Text",
      CellTags->"Gstrong"],
    Cell[1447957, 46624, 360, 15, 39, "Text",
      CellTags->"Gstrong"],
    Cell[1448354, 46644, 56, 1, 49, "Subsubsection",
      CellTags->"Gstrong"],
    Cell[1448413, 46647, 122, 4, 70, "Text",
      CellTags->"Gstrong"],
    Cell[1448560, 46655, 105, 2, 70, "Input",
      CellTags->"Gstrong"]},
  "GTI"->{
    Cell[1448863, 46669, 44, 1, 39, "ObjectName",
      CellTags->"GTI"],
    Cell[1448932, 46674, 55, 1, 49, "Subsubsection",
      CellTags->"GTI"],
    Cell[1448990, 46677, 86, 1, 39, "Text",
      CellTags->"GTI"],
    Cell[1449079, 46680, 172, 9, 39, "Text",
      CellTags->"GTI"]},
  "GV"->{
    Cell[1449300, 46695, 42, 1, 39, "ObjectName",
      CellTags->"GV"],
    Cell[1449367, 46700, 54, 1, 49, "Subsubsection",
      CellTags->"GV"],
    Cell[1449424, 46703, 66, 1, 39, "Text",
      CellTags->"GV"],
    Cell[1449493, 46706, 195, 9, 39, "Text",
      CellTags->"GV"]},
  "Hill"->{
    Cell[1449737, 46721, 46, 1, 39, "ObjectName",
      CellTags->"Hill"],
    Cell[1449808, 46726, 56, 1, 49, "Subsubsection",
      CellTags->"Hill"],
    Cell[1449867, 46729, 130, 4, 39, "Text",
      CellTags->"Hill"],
    Cell[1450000, 46735, 209, 9, 39, "Text",
      CellTags->"Hill"],
    Cell[1450246, 46749, 53, 1, 49, "Subsubsection",
      CellTags->"Hill"],
    Cell[1450324, 46754, 156, 4, 56, "Input",
      CellTags->"Hill"],
    Cell[1453280, 46840, 322, 8, 52, "Input",
      CellTags->"Hill"],
    Cell[1453750, 46857, 325, 9, 52, "Input",
      CellTags->"Hill"],
    Cell[1455413, 46910, 333, 9, 52, "Input",
      CellTags->"Hill"]},
  "HypergeometricAC"->{
    Cell[1455918, 46930, 70, 1, 39, "ObjectName",
      CellTags->"HypergeometricAC"],
    Cell[1456013, 46935, 68, 1, 49, "Subsubsection",
      CellTags->"HypergeometricAC"],
    Cell[1456084, 46938, 554, 9, 90, "Text",
      CellTags->"HypergeometricAC"],
    Cell[1456663, 46951, 159, 3, 56, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1457027, 46964, 641, 24, 39, "Text",
      CellTags->"HypergeometricAC"],
    Cell[1457705, 46993, 65, 1, 49, "Subsubsection",
      CellTags->"HypergeometricAC"],
    Cell[1457773, 46996, 102, 1, 70, "Text",
      CellTags->"HypergeometricAC"],
    Cell[1457900, 47001, 311, 7, 70, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1463412, 47136, 311, 7, 70, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1468848, 47269, 311, 7, 70, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1474337, 47403, 311, 7, 70, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1480480, 47550, 311, 7, 70, "Input",
      CellTags->"HypergeometricAC"],
    Cell[1482785, 47615, 312, 7, 70, "Input",
      CellTags->"HypergeometricAC"]},
  "HypergeometricIR"->{
    Cell[1485116, 47682, 70, 1, 39, "ObjectName",
      CellTags->"HypergeometricIR"],
    Cell[1485211, 47687, 68, 1, 49, "Subsubsection",
      CellTags->"HypergeometricIR"],
    Cell[1485282, 47690, 272, 6, 56, "Text",
      CellTags->"HypergeometricIR"],
    Cell[1485557, 47698, 512, 19, 39, "Text",
      CellTags->"HypergeometricIR"],
    Cell[1486094, 47721, 159, 3, 56, "Input",
      CellTags->"HypergeometricIR"],
    Cell[1486496, 47737, 65, 1, 49, "Subsubsection",
      CellTags->"HypergeometricIR"],
    Cell[1486586, 47742, 288, 7, 70, "Input",
      CellTags->"HypergeometricIR"],
    Cell[1487711, 47777, 492, 14, 70, "Input",
      CellTags->"HypergeometricIR"],
    Cell[1490442, 47853, 180, 4, 70, "Input",
      CellTags->"HypergeometricIR"]},
  "HypergeometricSE"->{
    Cell[1491091, 47878, 70, 1, 39, "ObjectName",
      CellTags->"HypergeometricSE"],
    Cell[1491186, 47883, 68, 1, 49, "Subsubsection",
      CellTags->"HypergeometricSE"],
    Cell[1491257, 47886, 440, 13, 56, "Text",
      CellTags->"HypergeometricSE"],
    Cell[1491722, 47903, 159, 3, 56, "Input",
      CellTags->"HypergeometricSE"],
    Cell[1492094, 47916, 224, 9, 35, "Text",
      CellTags->"HypergeometricSE"],
    Cell[1492355, 47930, 65, 1, 49, "Subsubsection",
      CellTags->"HypergeometricSE"],
    Cell[1492445, 47935, 292, 7, 70, "Input",
      CellTags->"HypergeometricSE"],
    Cell[1493544, 47967, 430, 11, 70, "Input",
      CellTags->"HypergeometricSE"]},
  "HypExplicit"->{
    Cell[1495119, 48011, 60, 1, 39, "ObjectName",
      CellTags->"HypExplicit"],
    Cell[1495204, 48016, 63, 1, 49, "Subsubsection",
      CellTags->"HypExplicit"],
    Cell[1495270, 48019, 339, 10, 39, "Text",
      CellTags->"HypExplicit"],
    Cell[1495612, 48031, 219, 9, 39, "Text",
      CellTags->"HypExplicit"],
    Cell[1495868, 48045, 60, 1, 49, "Subsubsection",
      CellTags->"HypExplicit"],
    Cell[1495953, 48050, 196, 4, 70, "Input",
      CellTags->"HypExplicit"],
    Cell[1497714, 48103, 174, 4, 70, "Input",
      CellTags->"HypExplicit"],
    Cell[1498695, 48132, 316, 8, 70, "Input",
      CellTags->"HypExplicit"],
    Cell[1500861, 48196, 174, 4, 70, "Input",
      CellTags->"HypExplicit"]},
  "HypInt"->{
    Cell[1502180, 48233, 50, 1, 39, "ObjectName",
      CellTags->"HypInt"],
    Cell[1502255, 48238, 58, 1, 49, "Subsubsection",
      CellTags->"HypInt"],
    Cell[1502316, 48241, 207, 5, 56, "Text",
      CellTags->"HypInt"],
    Cell[1502560, 48251, 55, 1, 49, "Subsubsection",
      CellTags->"HypInt"],
    Cell[1502640, 48256, 189, 4, 56, "Input",
      CellTags->"HypInt"],
    Cell[1504392, 48309, 158, 4, 52, "Input",
      CellTags->"HypInt"]},
  "IFPD"->{
    Cell[1505796, 48354, 46, 1, 39, "ObjectName",
      CellTags->"IFPD"],
    Cell[1505867, 48359, 56, 1, 49, "Subsubsection",
      CellTags->"IFPD"],
    Cell[1505926, 48362, 67, 1, 39, "Text",
      CellTags->"IFPD"],
    Cell[1505996, 48365, 227, 9, 39, "Text",
      CellTags->"IFPD"]},
  "IFPDOff"->{
    Cell[1506272, 48380, 52, 1, 39, "ObjectName",
      CellTags->"IFPDOff"],
    Cell[1506349, 48385, 59, 1, 49, "Subsubsection",
      CellTags->"IFPDOff"],
    Cell[1506411, 48388, 185, 5, 39, "Text",
      CellTags->"IFPDOff"],
    Cell[1506599, 48395, 293, 14, 39, "Text",
      CellTags->"IFPDOff"],
    Cell[1506929, 48414, 56, 1, 49, "Subsubsection",
      CellTags->"IFPDOff"],
    Cell[1507010, 48419, 198, 5, 56, "Input",
      CellTags->"IFPDOff"],
    Cell[1507211, 48426, 491, 16, 51, "Output",
      CellTags->"IFPDOff"],
    Cell[1507739, 48447, 134, 3, 56, "Input",
      CellTags->"IFPDOff"],
    Cell[1507876, 48452, 213, 5, 37, "Output",
      CellTags->"IFPDOff"],
    Cell[1508126, 48462, 160, 4, 52, "Input",
      CellTags->"IFPDOff"],
    Cell[1508289, 48468, 267, 8, 38, "Output",
      CellTags->"IFPDOff"],
    Cell[1508593, 48481, 134, 3, 52, "Input",
      CellTags->"IFPDOff"],
    Cell[1508730, 48486, 339, 9, 37, "Output",
      CellTags->"IFPDOff"]},
  "IFPDOn"->{
    Cell[1509130, 48502, 50, 1, 39, "ObjectName",
      CellTags->"IFPDOn"],
    Cell[1509205, 48507, 58, 1, 49, "Subsubsection",
      CellTags->"IFPDOn"],
    Cell[1509266, 48510, 318, 6, 56, "Text",
      CellTags->"IFPDOn"],
    Cell[1509587, 48518, 295, 14, 39, "Text",
      CellTags->"IFPDOn"]},
  "IncludePair"->{
    Cell[1509931, 48538, 60, 1, 39, "ObjectName",
      CellTags->"IncludePair"],
    Cell[1510016, 48543, 63, 1, 49, "Subsubsection",
      CellTags->"IncludePair"],
    Cell[1510082, 48546, 127, 4, 39, "Text",
      CellTags->"IncludePair"],
    Cell[1510212, 48552, 189, 9, 39, "Text",
      CellTags->"IncludePair"]},
  "IndexPosition"->{
    Cell[1510450, 48567, 64, 1, 39, "ObjectName",
      CellTags->"IndexPosition"],
    Cell[1510539, 48572, 65, 1, 49, "Subsubsection",
      CellTags->"IndexPosition"],
    Cell[1510607, 48575, 90, 1, 39, "Text",
      CellTags->"IndexPosition"],
    Cell[1510700, 48578, 212, 9, 39, "Text",
      CellTags->"IndexPosition"]},
  "Indices"->{
    Cell[1510961, 48593, 52, 1, 39, "ObjectName",
      CellTags->"Indices"],
    Cell[1511038, 48598, 59, 1, 49, "Subsubsection",
      CellTags->"Indices"],
    Cell[1511100, 48601, 206, 5, 56, "Text",
      CellTags->"Indices"],
    Cell[1511309, 48608, 206, 9, 39, "Text",
      CellTags->"Indices"]},
  "InitialFunction"->{
    Cell[1511564, 48623, 68, 1, 39, "ObjectName",
      CellTags->"InitialFunction"],
    Cell[1511657, 48628, 67, 1, 49, "Subsubsection",
      CellTags->"InitialFunction"],
    Cell[1511727, 48631, 184, 4, 39, "Text",
      CellTags->"InitialFunction"],
    Cell[1511914, 48637, 199, 9, 39, "Text",
      CellTags->"InitialFunction"]},
  "InitialSubstitutions"->{
    Cell[1512162, 48652, 78, 1, 39, "ObjectName",
      CellTags->"InitialSubstitutions"],
    Cell[1512265, 48657, 72, 1, 49, "Subsubsection",
      CellTags->"InitialSubstitutions"],
    Cell[1512340, 48660, 211, 5, 39, "Text",
      CellTags->"InitialSubstitutions"],
    Cell[1512554, 48667, 441, 19, 39, "Text",
      CellTags->"InitialSubstitutions"]},
  "InsideDiracTrace"->{
    Cell[1513044, 48692, 70, 1, 39, "ObjectName",
      CellTags->"InsideDiracTrace"],
    Cell[1513139, 48697, 68, 1, 49, "Subsubsection",
      CellTags->"InsideDiracTrace"],
    Cell[1513210, 48700, 499, 9, 73, "Text",
      CellTags->"InsideDiracTrace"],
    Cell[1513712, 48711, 341, 14, 39, "Text",
      CellTags->"InsideDiracTrace"]},
  "IntegralTable"->{
    Cell[1514102, 48731, 64, 1, 39, "ObjectName",
      CellTags->"IntegralTable"],
    Cell[1514191, 48736, 65, 1, 49, "Subsubsection",
      CellTags->"IntegralTable"],
    Cell[1514259, 48739, 234, 5, 56, "Text",
      CellTags->"IntegralTable"],
    Cell[1514496, 48746, 533, 19, 39, "Text",
      CellTags->"IntegralTable"]},
  "IntegrateByParts"->{
    Cell[1515078, 48771, 70, 1, 39, "ObjectName",
      CellTags->"IntegrateByParts"],
    Cell[1515173, 48776, 68, 1, 49, "Subsubsection",
      CellTags->"IntegrateByParts"],
    Cell[1515244, 48779, 187, 5, 39, "Text",
      CellTags->"IntegrateByParts"],
    Cell[1515456, 48788, 159, 3, 56, "Input",
      CellTags->"IntegrateByParts"],
    Cell[1515851, 48804, 65, 1, 49, "Subsubsection",
      CellTags->"IntegrateByParts"],
    Cell[1515941, 48809, 670, 19, 70, "Input",
      CellTags->"IntegrateByParts"]},
  "Integratedx"->{
    Cell[1517462, 48859, 60, 1, 39, "ObjectName",
      CellTags->"Integratedx"],
    Cell[1517547, 48864, 63, 1, 49, "Subsubsection",
      CellTags->"Integratedx"],
    Cell[1517613, 48867, 155, 4, 39, "Text",
      CellTags->"Integratedx"],
    Cell[1517771, 48873, 915, 34, 39, "Text",
      CellTags->"Integratedx"]},
  "Integrate2"->{
    Cell[1518735, 48913, 58, 1, 39, "ObjectName",
      CellTags->"Integrate2"],
    Cell[1518818, 48918, 62, 1, 49, "Subsubsection",
      CellTags->"Integrate2"],
    Cell[1518883, 48921, 885, 17, 107, "Text",
      CellTags->"Integrate2"],
    Cell[1519771, 48940, 551, 24, 39, "Text",
      CellTags->"Integrate2"],
    Cell[1520325, 48966, 406, 7, 73, "Text",
      CellTags->"Integrate2"],
    Cell[1520768, 48978, 59, 1, 49, "Subsubsection",
      CellTags->"Integrate2"],
    Cell[1520852, 48983, 532, 15, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1521998, 49019, 196, 6, 70, "Text",
      CellTags->"Integrate2"],
    Cell[1522219, 49029, 342, 9, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1522860, 49052, 337, 9, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1523768, 49083, 414, 11, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1524991, 49122, 384, 10, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1525533, 49140, 1942, 54, 70, "Text",
      CellTags->"Integrate2"],
    Cell[1527500, 49198, 331, 9, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1527957, 49214, 755, 25, 70, "Text",
      CellTags->"Integrate2"],
    Cell[1528737, 49243, 386, 10, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1529271, 49262, 476, 13, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1529962, 49286, 516, 14, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1530847, 49317, 402, 11, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1531579, 49343, 436, 12, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1532530, 49375, 380, 10, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1533158, 49398, 342, 9, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1534002, 49427, 553, 15, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1535185, 49465, 966, 25, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1541938, 49655, 596, 16, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1543683, 49709, 635, 17, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1553948, 50013, 174, 4, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1555731, 50067, 132, 3, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1556031, 50079, 533, 14, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1556732, 50102, 877, 23, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1562590, 50276, 183, 4, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1562924, 50289, 450, 12, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1563500, 50308, 138, 4, 70, "Text",
      CellTags->"Integrate2"],
    Cell[1563663, 50316, 179, 4, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1571162, 50504, 284, 8, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1571689, 50523, 166, 4, 70, "Text",
      CellTags->"Integrate2"],
    Cell[1571880, 50531, 405, 11, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1572529, 50553, 158, 4, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1572712, 50561, 384, 10, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1573276, 50581, 400, 11, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1574018, 50608, 127, 3, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1574312, 50620, 400, 11, 70, "Input",
      CellTags->"Integrate2"],
    Cell[1574879, 50640, 471, 13, 70, "Input",
      CellTags->"Integrate2"]},
  "Integrate3"->{
    Cell[1581677, 50845, 58, 1, 39, "ObjectName",
      CellTags->"Integrate3"],
    Cell[1581760, 50850, 62, 1, 49, "Subsubsection",
      CellTags->"Integrate3"],
    Cell[1581825, 50853, 334, 7, 56, "Text",
      CellTags->"Integrate3"],
    Cell[1582162, 50862, 200, 9, 39, "Text",
      CellTags->"Integrate3"],
    Cell[1582399, 50876, 59, 1, 49, "Subsubsection",
      CellTags->"Integrate3"],
    Cell[1582483, 50881, 340, 9, 70, "Input",
      CellTags->"Integrate3"],
    Cell[1583127, 50905, 489, 13, 70, "Input",
      CellTags->"Integrate3"],
    Cell[1584392, 50947, 858, 23, 70, "Input",
      CellTags->"Integrate3"],
    Cell[1588386, 51058, 320, 8, 70, "Input",
      CellTags->"Integrate3"],
    Cell[1588854, 51075, 389, 10, 70, "Input",
      CellTags->"Integrate3"],
    Cell[1589502, 51098, 331, 9, 70, "Input",
      CellTags->"Integrate3"]},
  "Integrate4"->{
    Cell[1590005, 51118, 58, 1, 39, "ObjectName",
      CellTags->"Integrate4"],
    Cell[1590088, 51123, 62, 1, 49, "Subsubsection",
      CellTags->"Integrate4"],
    Cell[1590153, 51126, 120, 4, 39, "Text",
      CellTags->"Integrate4"],
    Cell[1590276, 51132, 326, 14, 39, "Text",
      CellTags->"Integrate4"]},
  "Integrate5"->{
    Cell[1590651, 51152, 58, 1, 39, "ObjectName",
      CellTags->"Integrate5"],
    Cell[1590734, 51157, 62, 1, 49, "Subsubsection",
      CellTags->"Integrate5"],
    Cell[1590799, 51160, 72, 1, 39, "Text",
      CellTags->"Integrate5"],
    Cell[1590874, 51163, 200, 9, 39, "Text",
      CellTags->"Integrate5"]},
  "IntermediateSubstitutions"->{
    Cell[1591123, 51178, 88, 1, 39, "ObjectName",
      CellTags->"IntermediateSubstitutions"],
    Cell[1591236, 51183, 77, 1, 49, "Subsubsection",
      CellTags->"IntermediateSubstitutions"],
    Cell[1591316, 51186, 209, 5, 39, "Text",
      CellTags->"IntermediateSubstitutions"],
    Cell[1591528, 51193, 206, 9, 39, "Text",
      CellTags->"IntermediateSubstitutions"]},
  "InverseMellin"->{
    Cell[1591783, 51208, 64, 1, 39, "ObjectName",
      CellTags->"InverseMellin"],
    Cell[1591872, 51213, 65, 1, 49, "Subsubsection",
      CellTags->"InverseMellin"],
    Cell[1591940, 51216, 578, 14, 71, "Text",
      CellTags->"InverseMellin"],
    Cell[1592521, 51232, 662, 29, 39, "Text",
      CellTags->"InverseMellin"],
    Cell[1593220, 51266, 62, 1, 49, "Subsubsection",
      CellTags->"InverseMellin"],
    Cell[1593307, 51271, 206, 5, 56, "Input",
      CellTags->"InverseMellin"],
    Cell[1593718, 51287, 270, 7, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1594193, 51305, 174, 4, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1594728, 51325, 237, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1595339, 51348, 239, 7, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1596041, 51374, 251, 7, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1596733, 51398, 116, 4, 35, "Text",
      CellTags->"InverseMellin"],
    Cell[1596874, 51406, 354, 9, 56, "Input",
      CellTags->"InverseMellin"],
    Cell[1597379, 51423, 120, 4, 35, "Text",
      CellTags->"InverseMellin"],
    Cell[1597502, 51429, 4033, 103, 216, "Input",
      CellTags->"InverseMellin"],
    Cell[1601538, 51534, 290, 7, 56, "Input",
      CellTags->"InverseMellin"],
    Cell[1601853, 51545, 221, 6, 56, "Input",
      CellTags->"InverseMellin"],
    Cell[1602577, 51571, 221, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1603256, 51596, 208, 5, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1604472, 51634, 105, 2, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1604602, 51640, 234, 6, 56, "Input",
      CellTags->"InverseMellin"],
    Cell[1605484, 51671, 330, 9, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1606464, 51705, 330, 9, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1607675, 51744, 329, 9, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1608957, 51785, 307, 9, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1609971, 51820, 346, 10, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1611516, 51867, 234, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1612665, 51904, 277, 8, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1613898, 51944, 234, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1615328, 51988, 244, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1616276, 52020, 272, 7, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1618005, 52071, 272, 7, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1619659, 52120, 254, 6, 52, "Input",
      CellTags->"InverseMellin"],
    Cell[1620709, 52153, 194, 5, 52, "Input",
      CellTags->"InverseMellin"]},
  "Isolate"->{
    Cell[1620952, 52164, 52, 1, 39, "ObjectName",
      CellTags->"Isolate"],
    Cell[1621029, 52169, 59, 1, 49, "Subsubsection",
      CellTags->"Isolate"],
    Cell[1621091, 52172, 425, 8, 73, "Text",
      CellTags->"Isolate"],
    Cell[1621541, 52184, 141, 3, 56, "Input",
      CellTags->"Isolate"],
    Cell[1622135, 52204, 56, 1, 49, "Subsubsection",
      CellTags->"Isolate"],
    Cell[1622216, 52209, 192, 5, 56, "Input",
      CellTags->"Isolate"],
    Cell[1622624, 52226, 476, 13, 52, "Input",
      CellTags->"Isolate"],
    Cell[1623503, 52257, 141, 3, 52, "Input",
      CellTags->"Isolate"],
    Cell[1624012, 52277, 218, 5, 52, "Input",
      CellTags->"Isolate"],
    Cell[1625073, 52313, 698, 19, 52, "Input",
      CellTags->"Isolate"],
    Cell[1626234, 52352, 120, 3, 52, "Input",
      CellTags->"Isolate"],
    Cell[1627395, 52389, 324, 7, 52, "Input",
      CellTags->"Isolate"],
    Cell[1627934, 52408, 219, 6, 52, "Input",
      CellTags->"Isolate"],
    Cell[1628402, 52425, 172, 4, 52, "Input",
      CellTags->"Isolate"]},
  "IsolateHead"->{
    Cell[1628623, 52435, 60, 1, 39, "ObjectName",
      CellTags->"IsolateHead"],
    Cell[1628708, 52440, 63, 1, 49, "Subsubsection",
      CellTags->"IsolateHead"],
    Cell[1628774, 52443, 85, 1, 39, "Text",
      CellTags->"IsolateHead"],
    Cell[1628862, 52446, 207, 9, 39, "Text",
      CellTags->"IsolateHead"]},
  "IsolateNames"->{
    Cell[1629118, 52461, 62, 1, 39, "ObjectName",
      CellTags->"IsolateNames"],
    Cell[1629205, 52466, 64, 1, 49, "Subsubsection",
      CellTags->"IsolateNames"],
    Cell[1629272, 52469, 221, 5, 56, "Text",
      CellTags->"IsolateNames"],
    Cell[1629496, 52476, 313, 14, 39, "Text",
      CellTags->"IsolateNames"]},
  "IsolatePrint"->{
    Cell[1629858, 52496, 62, 1, 39, "ObjectName",
      CellTags->"IsolatePrint"],
    Cell[1629945, 52501, 64, 1, 49, "Subsubsection",
      CellTags->"IsolatePrint"],
    Cell[1630012, 52504, 285, 6, 56, "Text",
      CellTags->"IsolatePrint"],
    Cell[1630300, 52512, 193, 9, 39, "Text",
      CellTags->"IsolatePrint"]},
  "IsolateSplit"->{
    Cell[1630542, 52527, 62, 1, 39, "ObjectName",
      CellTags->"IsolateSplit"],
    Cell[1630629, 52532, 64, 1, 49, "Subsubsection",
      CellTags->"IsolateSplit"],
    Cell[1630696, 52535, 414, 7, 73, "Text",
      CellTags->"IsolateSplit"],
    Cell[1631113, 52544, 193, 9, 39, "Text",
      CellTags->"IsolateSplit"]},
  "KeepOnly"->{
    Cell[1631355, 52559, 54, 1, 39, "ObjectName",
      CellTags->"KeepOnly"],
    Cell[1631434, 52564, 60, 1, 49, "Subsubsection",
      CellTags->"KeepOnly"],
    Cell[1631497, 52567, 294, 6, 56, "Text",
      CellTags->"KeepOnly"],
    Cell[1631794, 52575, 495, 24, 39, "Text",
      CellTags->"KeepOnly"]},
  "KK"->{
    Cell[1632338, 52605, 42, 1, 39, "ObjectName",
      CellTags->"KK"],
    Cell[1632405, 52610, 54, 1, 49, "Subsubsection",
      CellTags->"KK"],
    Cell[1632462, 52613, 236, 5, 56, "Text",
      CellTags->"KK"],
    Cell[1632701, 52620, 315, 14, 39, "Text",
      CellTags->"KK"]},
  "Kummer"->{
    Cell[1633065, 52640, 50, 1, 39, "ObjectName",
      CellTags->"Kummer"],
    Cell[1633140, 52645, 58, 1, 49, "Subsubsection",
      CellTags->"Kummer"],
    Cell[1633201, 52648, 263, 6, 56, "Text",
      CellTags->"Kummer"],
    Cell[1633467, 52656, 214, 9, 39, "Text",
      CellTags->"Kummer"],
    Cell[1633718, 52670, 55, 1, 49, "Subsubsection",
      CellTags->"Kummer"],
    Cell[1633798, 52675, 402, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1637750, 52783, 402, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1641736, 52892, 402, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1645722, 53001, 457, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1649745, 53110, 561, 13, 70, "Input",
      CellTags->"Kummer"],
    Cell[1653984, 53222, 457, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1657998, 53331, 561, 13, 70, "Input",
      CellTags->"Kummer"],
    Cell[1662258, 53444, 457, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1666272, 53553, 561, 13, 70, "Input",
      CellTags->"Kummer"],
    Cell[1670532, 53666, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1674898, 53785, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1679601, 53911, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1683944, 54030, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1688624, 54156, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1692931, 54274, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1697679, 54401, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1702045, 54520, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1706834, 54648, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1711108, 54765, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1715805, 54891, 588, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1720079, 55008, 756, 19, 70, "Input",
      CellTags->"Kummer"],
    Cell[1724923, 55137, 626, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1729305, 55254, 403, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1733258, 55362, 626, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1737736, 55481, 403, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1741681, 55589, 626, 15, 70, "Input",
      CellTags->"Kummer"],
    Cell[1746159, 55708, 403, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1750104, 55816, 690, 17, 70, "Input",
      CellTags->"Kummer"],
    Cell[1754661, 55937, 458, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1758685, 56046, 690, 17, 70, "Input",
      CellTags->"Kummer"],
    Cell[1763400, 56171, 458, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1767488, 56282, 690, 17, 70, "Input",
      CellTags->"Kummer"],
    Cell[1772130, 56405, 458, 11, 70, "Input",
      CellTags->"Kummer"],
    Cell[1776145, 56514, 403, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1783201, 56688, 403, 10, 70, "Input",
      CellTags->"Kummer"],
    Cell[1790349, 56865, 403, 10, 70, "Input",
      CellTags->"Kummer"]},
  "Lagrangian"->{
    Cell[1797525, 57044, 58, 1, 39, "ObjectName",
      CellTags->"Lagrangian"],
    Cell[1797608, 57049, 62, 1, 49, "Subsubsection",
      CellTags->"Lagrangian"],
    Cell[1797673, 57052, 455, 9, 73, "Text",
      CellTags->"Lagrangian"],
    Cell[1798131, 57063, 194, 9, 39, "Text",
      CellTags->"Lagrangian"],
    Cell[1798362, 57077, 59, 1, 49, "Subsubsection",
      CellTags->"Lagrangian"],
    Cell[1798446, 57082, 151, 3, 70, "Input",
      CellTags->"Lagrangian"],
    Cell[1799029, 57101, 86, 1, 70, "Text",
      CellTags->"Lagrangian"],
    Cell[1799140, 57106, 151, 3, 70, "Input",
      CellTags->"Lagrangian"],
    Cell[1800204, 57138, 151, 3, 70, "Input",
      CellTags->"Lagrangian"],
    Cell[1801760, 57183, 151, 3, 70, "Input",
      CellTags->"Lagrangian"],
    Cell[1802757, 57215, 151, 3, 70, "Input",
      CellTags->"Lagrangian"]},
  "LC"->{
    Cell[1803827, 57250, 42, 1, 39, "ObjectName",
      CellTags->"LC"],
    Cell[1803894, 57255, 54, 1, 49, "Subsubsection",
      CellTags->"LC"],
    Cell[1803951, 57258, 243, 6, 56, "Text",
      CellTags->"LC"],
    Cell[1804197, 57266, 297, 14, 39, "Text",
      CellTags->"LC"],
    Cell[1804531, 57285, 51, 1, 49, "Subsubsection",
      CellTags->"LC"],
    Cell[1804607, 57290, 199, 5, 70, "Input",
      CellTags->"LC"],
    Cell[1805107, 57308, 122, 3, 70, "Input",
      CellTags->"LC"],
    Cell[1805934, 57338, 131, 3, 70, "Input",
      CellTags->"LC"],
    Cell[1806475, 57355, 221, 6, 70, "Input",
      CellTags->"LC"],
    Cell[1806985, 57374, 161, 4, 70, "Input",
      CellTags->"LC"],
    Cell[1807598, 57394, 474, 13, 70, "Input",
      CellTags->"LC"]},
  "LCD"->{
    Cell[1808717, 57433, 44, 1, 39, "ObjectName",
      CellTags->"LCD"],
    Cell[1808786, 57438, 55, 1, 49, "Subsubsection",
      CellTags->"LCD"],
    Cell[1808844, 57441, 235, 5, 56, "Text",
      CellTags->"LCD"],
    Cell[1809082, 57448, 162, 4, 39, "Text",
      CellTags->"LCD"],
    Cell[1809247, 57454, 295, 14, 39, "Text",
      CellTags->"LCD"],
    Cell[1809579, 57473, 52, 1, 49, "Subsubsection",
      CellTags->"LCD"],
    Cell[1809634, 57476, 232, 6, 70, "Input",
      CellTags->"LCD"],
    Cell[1809891, 57486, 202, 5, 70, "Input",
      CellTags->"LCD"],
    Cell[1810395, 57504, 124, 3, 70, "Input",
      CellTags->"LCD"],
    Cell[1811225, 57534, 133, 3, 70, "Input",
      CellTags->"LCD"],
    Cell[1811893, 57555, 224, 6, 70, "Input",
      CellTags->"LCD"],
    Cell[1812407, 57574, 163, 4, 70, "Input",
      CellTags->"LCD"],
    Cell[1813085, 57596, 494, 13, 70, "Input",
      CellTags->"LCD"],
    Cell[1814125, 57630, 207, 5, 70, "Input",
      CellTags->"LCD"]},
  "LeftPartialD"->{
    Cell[1814582, 57649, 62, 1, 39, "ObjectName",
      CellTags->"LeftPartialD"],
    Cell[1814669, 57654, 64, 1, 49, "Subsubsection",
      CellTags->"LeftPartialD"],
    Cell[1814736, 57657, 358, 12, 42, "Text",
      CellTags->"LeftPartialD"],
    Cell[1815097, 57671, 616, 24, 39, "Text",
      CellTags->"LeftPartialD"],
    Cell[1815750, 57700, 70, 1, 50, "Subsubsection",
      CellTags->"LeftPartialD"],
    Cell[1815845, 57705, 275, 7, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1816493, 57729, 123, 3, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1817064, 57752, 121, 3, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1817511, 57768, 168, 4, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1817874, 57782, 409, 10, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1818699, 57810, 123, 3, 43, "Input",
      CellTags->"LeftPartialD"],
    Cell[1819837, 57849, 121, 3, 43, "Input",
      CellTags->"LeftPartialD"]},
  "LeftRightPartialD"->{
    Cell[1820943, 57884, 72, 1, 39, "ObjectName",
      CellTags->"LeftRightPartialD"],
    Cell[1821040, 57889, 69, 1, 49, "Subsubsection",
      CellTags->"LeftRightPartialD"],
    Cell[1821112, 57892, 661, 21, 59, "Text",
      CellTags->"LeftRightPartialD"],
    Cell[1821776, 57915, 893, 33, 39, "Text",
      CellTags->"LeftRightPartialD"],
    Cell[1822706, 57953, 66, 1, 49, "Subsubsection",
      CellTags->"LeftRightPartialD"],
    Cell[1822797, 57958, 160, 3, 56, "Input",
      CellTags->"LeftRightPartialD"],
    Cell[1823289, 57976, 155, 3, 56, "Input",
      CellTags->"LeftRightPartialD"],
    Cell[1824136, 58004, 310, 7, 56, "Input",
      CellTags->"LeftRightPartialD"],
    Cell[1824849, 58028, 153, 3, 56, "Input",
      CellTags->"LeftRightPartialD"],
    Cell[1825877, 58062, 444, 10, 56, "Input",
      CellTags->"LeftRightPartialD"],
    Cell[1826767, 58090, 153, 3, 56, "Input",
      CellTags->"LeftRightPartialD"]},
  "LeftRightPartialD2"->{
    Cell[1828010, 58131, 74, 1, 39, "ObjectName",
      CellTags->"LeftRightPartialD2"],
    Cell[1828109, 58136, 70, 1, 49, "Subsubsection",
      CellTags->"LeftRightPartialD2"],
    Cell[1828182, 58139, 660, 21, 59, "Text",
      CellTags->"LeftRightPartialD2"],
    Cell[1828845, 58162, 751, 29, 39, "Text",
      CellTags->"LeftRightPartialD2"],
    Cell[1829633, 58196, 67, 1, 49, "Subsubsection",
      CellTags->"LeftRightPartialD2"],
    Cell[1829725, 58201, 162, 3, 70, "Input",
      CellTags->"LeftRightPartialD2"],
    Cell[1830219, 58219, 156, 3, 70, "Input",
      CellTags->"LeftRightPartialD2"],
    Cell[1830932, 58244, 312, 7, 70, "Input",
      CellTags->"LeftRightPartialD2"],
    Cell[1831647, 58268, 154, 3, 70, "Input",
      CellTags->"LeftRightPartialD2"],
    Cell[1832590, 58301, 446, 10, 70, "Input",
      CellTags->"LeftRightPartialD2"],
    Cell[1833482, 58329, 154, 3, 70, "Input",
      CellTags->"LeftRightPartialD2"]},
  "LeviCivita"->{
    Cell[1834603, 58367, 59, 1, 39, "ObjectName",
      CellTags->"LeviCivita"],
    Cell[1834687, 58372, 62, 1, 49, "Subsubsection",
      CellTags->"LeviCivita"],
    Cell[1834752, 58375, 1314, 41, 73, "Text",
      CellTags->"LeviCivita"],
    Cell[1836091, 58420, 148, 3, 56, "Input",
      CellTags->"LeviCivita"],
    Cell[1836443, 58433, 506, 24, 35, "Text",
      CellTags->"LeviCivita"],
    Cell[1836986, 58462, 59, 1, 49, "Subsubsection",
      CellTags->"LeviCivita"],
    Cell[1837070, 58467, 223, 5, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1838006, 58499, 245, 6, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1838849, 58528, 216, 5, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1839531, 58552, 222, 5, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1839902, 58566, 297, 8, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1840597, 58588, 488, 13, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1842457, 58647, 141, 3, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1842748, 58658, 374, 9, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1843147, 58671, 506, 13, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1844014, 58699, 374, 9, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1844413, 58712, 215, 5, 70, "Input",
      CellTags->"LeviCivita"],
    Cell[1844915, 58728, 138, 3, 70, "Input",
      CellTags->"LeviCivita"]},
  "LeviCivitaSign"->{
    Cell[1845102, 58737, 66, 1, 39, "ObjectName",
      CellTags->"LeviCivitaSign"],
    Cell[1845193, 58742, 66, 1, 49, "Subsubsection",
      CellTags->"LeviCivitaSign"],
    Cell[1845262, 58745, 206, 5, 39, "Text",
      CellTags->"LeviCivitaSign"],
    Cell[1845471, 58752, 333, 14, 39, "Text",
      CellTags->"LeviCivitaSign"]},
  "ListIntegrals"->{
    Cell[1845853, 58772, 82, 1, 39, "ObjectName",
      CellTags->"ListIntegrals"],
    Cell[1845960, 58777, 65, 1, 49, "Subsubsection",
      CellTags->"ListIntegrals"],
    Cell[1846028, 58780, 473, 9, 73, "Text",
      CellTags->"ListIntegrals"],
    Cell[1846538, 58794, 62, 1, 49, "Subsubsection",
      CellTags->"ListIntegrals"],
    Cell[1846603, 58797, 102, 2, 56, "Input",
      CellTags->"ListIntegrals"]},
  "Li2"->{
    Cell[1846754, 58805, 45, 1, 39, "ObjectName",
      CellTags->"Li2"],
    Cell[1846824, 58810, 55, 1, 49, "Subsubsection",
      CellTags->"Li2"],
    Cell[1846882, 58813, 120, 4, 39, "Text",
      CellTags->"Li2"],
    Cell[1847005, 58819, 165, 8, 39, "Text",
      CellTags->"Li2"],
    Cell[1847207, 58832, 52, 1, 49, "Subsubsection",
      CellTags->"Li2"],
    Cell[1847284, 58837, 128, 3, 70, "Input",
      CellTags->"Li2"],
    Cell[1847624, 58852, 135, 3, 70, "Input",
      CellTags->"Li2"],
    Cell[1847979, 58866, 317, 9, 70, "Input",
      CellTags->"Li2"]},
  "Li3"->{
    Cell[1848532, 58889, 45, 1, 39, "ObjectName",
      CellTags->"Li3"],
    Cell[1848602, 58894, 55, 1, 49, "Subsubsection",
      CellTags->"Li3"],
    Cell[1848660, 58897, 121, 4, 39, "Text",
      CellTags->"Li3"],
    Cell[1848784, 58903, 165, 8, 39, "Text",
      CellTags->"Li3"],
    Cell[1848986, 58916, 52, 1, 49, "Subsubsection",
      CellTags->"Li3"],
    Cell[1849063, 58921, 128, 3, 70, "Input",
      CellTags->"Li3"],
    Cell[1849403, 58936, 135, 3, 70, "Input",
      CellTags->"Li3"],
    Cell[1849758, 58950, 189, 5, 70, "Input",
      CellTags->"Li3"],
    Cell[1850193, 58968, 252, 7, 70, "Input",
      CellTags->"Li3"]},
  "Loop"->{
    Cell[1850681, 58989, 46, 1, 39, "ObjectName",
      CellTags->"Loop"],
    Cell[1850752, 58994, 56, 1, 49, "Subsubsection",
      CellTags->"Loop"],
    Cell[1850811, 58997, 95, 1, 39, "Text",
      CellTags->"Loop"]},
  "LorentzIndex"->{
    Cell[1850955, 59004, 62, 1, 39, "ObjectName",
      CellTags->"LorentzIndex"],
    Cell[1851042, 59009, 64, 1, 49, "Subsubsection",
      CellTags->"LorentzIndex"],
    Cell[1851109, 59012, 244, 5, 56, "Text",
      CellTags->"LorentzIndex"],
    Cell[1851356, 59019, 337, 14, 39, "Text",
      CellTags->"LorentzIndex"],
    Cell[1851730, 59038, 61, 1, 49, "Subsubsection",
      CellTags->"LorentzIndex"],
    Cell[1851794, 59041, 90, 1, 70, "Text",
      CellTags->"LorentzIndex"],
    Cell[1851909, 59046, 153, 3, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1852196, 59056, 128, 4, 70, "Text",
      CellTags->"LorentzIndex"],
    Cell[1852349, 59064, 180, 4, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1852685, 59077, 131, 2, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1853000, 59089, 197, 4, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1853378, 59103, 246, 6, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1853757, 59116, 142, 4, 70, "Text",
      CellTags->"LorentzIndex"],
    Cell[1853902, 59122, 169, 4, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1854096, 59130, 412, 10, 70, "Input",
      CellTags->"LorentzIndex"],
    Cell[1855005, 59160, 180, 4, 70, "Input",
      CellTags->"LorentzIndex"]},
  "Lower"->{
    Cell[1855234, 59170, 48, 1, 39, "ObjectName",
      CellTags->"Lower"],
    Cell[1855307, 59175, 57, 1, 49, "Subsubsection",
      CellTags->"Lower"],
    Cell[1855367, 59178, 123, 4, 39, "Text",
      CellTags->"Lower"],
    Cell[1855493, 59184, 435, 19, 39, "Text",
      CellTags->"Lower"]},
  "MakeContext"->{
    Cell[1855977, 59209, 60, 1, 39, "ObjectName",
      CellTags->"MakeContext"],
    Cell[1856062, 59214, 63, 1, 49, "Subsubsection",
      CellTags->"MakeContext"],
    Cell[1856128, 59217, 239, 5, 56, "Text",
      CellTags->"MakeContext"],
    Cell[1856370, 59224, 573, 24, 39, "Text",
      CellTags->"MakeContext"]},
  "Mandelstam"->{
    Cell[1856992, 59254, 58, 1, 39, "ObjectName",
      CellTags->"Mandelstam"],
    Cell[1857075, 59259, 62, 1, 49, "Subsubsection",
      CellTags->"Mandelstam"],
    Cell[1857140, 59262, 1845, 73, 73, "Text",
      CellTags->"Mandelstam"],
    Cell[1858988, 59337, 821, 34, 39, "Text",
      CellTags->"Mandelstam"]},
  "Map2"->{
    Cell[1859858, 59377, 46, 1, 39, "ObjectName",
      CellTags->"Map2"],
    Cell[1859929, 59382, 56, 1, 49, "Subsubsection",
      CellTags->"Map2"],
    Cell[1859988, 59385, 136, 4, 39, "Text",
      CellTags->"Map2"],
    Cell[1860127, 59391, 182, 9, 39, "Text",
      CellTags->"Map2"],
    Cell[1860346, 59405, 53, 1, 49, "Subsubsection",
      CellTags->"Map2"],
    Cell[1860424, 59410, 186, 5, 70, "Input",
      CellTags->"Map2"],
    Cell[1860880, 59428, 157, 4, 70, "Input",
      CellTags->"Map2"],
    Cell[1861218, 59442, 227, 6, 70, "Input",
      CellTags->"Map2"],
    Cell[1861703, 59461, 157, 4, 70, "Input",
      CellTags->"Map2"]},
  "MemoryAvailable"->{
    Cell[1862065, 59477, 68, 1, 39, "ObjectName",
      CellTags->"MemoryAvailable"],
    Cell[1862158, 59482, 67, 1, 49, "Subsubsection",
      CellTags->"MemoryAvailable"],
    Cell[1862228, 59485, 231, 5, 56, "Text",
      CellTags->"MemoryAvailable"],
    Cell[1862462, 59492, 337, 14, 39, "Text",
      CellTags->"MemoryAvailable"]},
  "MemSet"->{
    Cell[1862848, 59512, 50, 1, 39, "ObjectName",
      CellTags->"MemSet"],
    Cell[1862923, 59517, 58, 1, 49, "Subsubsection",
      CellTags->"MemSet"],
    Cell[1862984, 59520, 358, 9, 56, "Text",
      CellTags->"MemSet"],
    Cell[1863345, 59531, 211, 9, 39, "Text",
      CellTags->"MemSet"]},
  "MetricTensor"->{
    Cell[1863605, 59546, 62, 1, 39, "ObjectName",
      CellTags->"MetricTensor"],
    Cell[1863692, 59551, 64, 1, 49, "Subsubsection",
      CellTags->"MetricTensor"],
    Cell[1863759, 59554, 227, 7, 39, "Text",
      CellTags->"MetricTensor"],
    Cell[1863989, 59563, 637, 29, 39, "Text",
      CellTags->"MetricTensor"],
    Cell[1864663, 59597, 61, 1, 49, "Subsubsection",
      CellTags->"MetricTensor"],
    Cell[1864749, 59602, 186, 4, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1865170, 59618, 169, 4, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1865488, 59631, 239, 5, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1866167, 59655, 169, 4, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1866485, 59668, 218, 5, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1866985, 59685, 273, 6, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1867602, 59705, 256, 6, 70, "Input",
      CellTags->"MetricTensor"],
    Cell[1868048, 59721, 320, 8, 70, "Input",
      CellTags->"MetricTensor"]},
  "MLimit"->{
    Cell[1868583, 59741, 50, 1, 39, "ObjectName",
      CellTags->"MLimit"],
    Cell[1868658, 59746, 58, 1, 49, "Subsubsection",
      CellTags->"MLimit"],
    Cell[1868719, 59749, 119, 4, 39, "Text",
      CellTags->"MLimit"],
    Cell[1868875, 59758, 55, 1, 49, "Subsubsection",
      CellTags->"MLimit"],
    Cell[1868955, 59763, 512, 14, 56, "Input",
      CellTags->"MLimit"]},
  "Momentum"->{
    Cell[1869637, 59788, 54, 1, 39, "ObjectName",
      CellTags->"Momentum"],
    Cell[1869716, 59793, 60, 1, 49, "Subsubsection",
      CellTags->"Momentum"],
    Cell[1869779, 59796, 261, 6, 56, "Text",
      CellTags->"Momentum"],
    Cell[1870043, 59804, 573, 24, 39, "Text",
      CellTags->"Momentum"],
    Cell[1870653, 59833, 57, 1, 49, "Subsubsection",
      CellTags->"Momentum"],
    Cell[1870713, 59836, 76, 1, 70, "Text",
      CellTags->"Momentum"],
    Cell[1870814, 59841, 138, 3, 70, "Input",
      CellTags->"Momentum"],
    Cell[1871079, 59851, 139, 4, 70, "Text",
      CellTags->"Momentum"],
    Cell[1871243, 59859, 165, 4, 70, "Input",
      CellTags->"Momentum"],
    Cell[1871535, 59870, 87, 1, 70, "Text",
      CellTags->"Momentum"],
    Cell[1871647, 59875, 206, 5, 70, "Input",
      CellTags->"Momentum"],
    Cell[1872049, 59890, 190, 5, 70, "Input",
      CellTags->"Momentum"],
    Cell[1872410, 59905, 139, 3, 70, "Input",
      CellTags->"Momentum"],
    Cell[1872740, 59918, 313, 8, 70, "Input",
      CellTags->"Momentum"],
    Cell[1873318, 59939, 139, 3, 70, "Input",
      CellTags->"Momentum"],
    Cell[1873750, 59955, 182, 5, 70, "Input",
      CellTags->"Momentum"],
    Cell[1874167, 59971, 188, 5, 70, "Input",
      CellTags->"Momentum"],
    Cell[1874551, 59986, 263, 7, 70, "Input",
      CellTags->"Momentum"],
    Cell[1874988, 60001, 164, 4, 70, "Input",
      CellTags->"Momentum"]},
  "MomentumCombine"->{
    Cell[1875201, 60011, 68, 1, 39, "ObjectName",
      CellTags->"MomentumCombine"],
    Cell[1875294, 60016, 67, 1, 49, "Subsubsection",
      CellTags->"MomentumCombine"],
    Cell[1875364, 60019, 184, 4, 39, "Text",
      CellTags->"MomentumCombine"],
    Cell[1875551, 60025, 627, 23, 39, "Text",
      CellTags->"MomentumCombine"],
    Cell[1876215, 60053, 64, 1, 49, "Subsubsection",
      CellTags->"MomentumCombine"],
    Cell[1876304, 60058, 391, 10, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1876920, 60079, 378, 10, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1877573, 60102, 153, 3, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1878153, 60124, 199, 5, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1878698, 60143, 206, 5, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1879505, 60173, 149, 3, 70, "Input",
      CellTags->"MomentumCombine"],
    Cell[1880140, 60192, 143, 3, 70, "Input",
      CellTags->"MomentumCombine"]},
  "MomentumCombine2"->{
    Cell[1880332, 60201, 70, 1, 39, "ObjectName",
      CellTags->"MomentumCombine2"],
    Cell[1880427, 60206, 68, 1, 49, "Subsubsection",
      CellTags->"MomentumCombine2"],
    Cell[1880498, 60209, 195, 5, 39, "Text",
      CellTags->"MomentumCombine2"],
    Cell[1880696, 60216, 631, 23, 39, "Text",
      CellTags->"MomentumCombine2"],
    Cell[1881364, 60244, 65, 1, 49, "Subsubsection",
      CellTags->"MomentumCombine2"],
    Cell[1881454, 60249, 517, 13, 56, "Input",
      CellTags->"MomentumCombine2"],
    Cell[1882612, 60288, 197, 5, 52, "Input",
      CellTags->"MomentumCombine2"]},
  "MomentumExpand"->{
    Cell[1883209, 60310, 66, 1, 39, "ObjectName",
      CellTags->"MomentumExpand"],
    Cell[1883300, 60315, 66, 1, 49, "Subsubsection",
      CellTags->"MomentumExpand"],
    Cell[1883369, 60318, 150, 4, 39, "Text",
      CellTags->"MomentumExpand"],
    Cell[1883522, 60324, 372, 14, 39, "Text",
      CellTags->"MomentumExpand"],
    Cell[1883931, 60343, 63, 1, 49, "Subsubsection",
      CellTags->"MomentumExpand"],
    Cell[1884019, 60348, 268, 7, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1884522, 60366, 205, 5, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1885114, 60387, 144, 3, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1885563, 60403, 254, 6, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1886288, 60430, 144, 3, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1886780, 60447, 285, 7, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1887569, 60476, 144, 3, 70, "Input",
      CellTags->"MomentumExpand"],
    Cell[1888163, 60496, 140, 3, 70, "Input",
      CellTags->"MomentumExpand"]},
  "MT"->{
    Cell[1889144, 60530, 42, 1, 39, "ObjectName",
      CellTags->"MT"],
    Cell[1889211, 60535, 54, 1, 49, "Subsubsection",
      CellTags->"MT"],
    Cell[1889268, 60538, 195, 7, 39, "Text",
      CellTags->"MT"],
    Cell[1889466, 60547, 657, 29, 39, "Text",
      CellTags->"MT"],
    Cell[1890160, 60581, 51, 1, 49, "Subsubsection",
      CellTags->"MT"],
    Cell[1890236, 60586, 166, 4, 70, "Input",
      CellTags->"MT"],
    Cell[1890637, 60602, 317, 8, 70, "Input",
      CellTags->"MT"],
    Cell[1891103, 60619, 194, 5, 70, "Input",
      CellTags->"MT"],
    Cell[1891487, 60634, 239, 7, 70, "Input",
      CellTags->"MT"],
    Cell[1892008, 60653, 279, 8, 70, "Input",
      CellTags->"MT"]},
  "MTD"->{
    Cell[1892501, 60673, 44, 1, 39, "ObjectName",
      CellTags->"MTD"],
    Cell[1892570, 60678, 55, 1, 49, "Subsubsection",
      CellTags->"MTD"],
    Cell[1892628, 60681, 197, 7, 39, "Text",
      CellTags->"MTD"],
    Cell[1892828, 60690, 655, 29, 39, "Text",
      CellTags->"MTD"],
    Cell[1893520, 60724, 52, 1, 49, "Subsubsection",
      CellTags->"MTD"],
    Cell[1893597, 60729, 168, 4, 70, "Input",
      CellTags->"MTD"],
    Cell[1894000, 60745, 320, 8, 70, "Input",
      CellTags->"MTD"],
    Cell[1894469, 60762, 216, 6, 70, "Input",
      CellTags->"MTD"],
    Cell[1894889, 60778, 254, 7, 70, "Input",
      CellTags->"MTD"],
    Cell[1895500, 60799, 289, 8, 70, "Input",
      CellTags->"MTD"]},
  "NegativeInteger"->{
    Cell[1896012, 60819, 68, 1, 39, "ObjectName",
      CellTags->"NegativeInteger"],
    Cell[1896105, 60824, 67, 1, 49, "Subsubsection",
      CellTags->"NegativeInteger"],
    Cell[1896175, 60827, 142, 4, 39, "Text",
      CellTags->"NegativeInteger"],
    Cell[1896320, 60833, 199, 9, 39, "Text",
      CellTags->"NegativeInteger"]},
  "Nf"->{
    Cell[1896568, 60848, 42, 1, 39, "ObjectName",
      CellTags->"Nf"],
    Cell[1896635, 60853, 54, 1, 49, "Subsubsection",
      CellTags->"Nf"],
    Cell[1896692, 60856, 67, 1, 39, "Text",
      CellTags->"Nf"],
    Cell[1896762, 60859, 738, 29, 39, "Text",
      CellTags->"Nf"]},
  "Nielsen"->{
    Cell[1897549, 60894, 52, 1, 39, "ObjectName",
      CellTags->"Nielsen"],
    Cell[1897626, 60899, 59, 1, 49, "Subsubsection",
      CellTags->"Nielsen"],
    Cell[1897688, 60902, 87, 1, 39, "Text",
      CellTags->"Nielsen"],
    Cell[1897800, 60907, 142, 3, 56, "Input",
      CellTags->"Nielsen"],
    Cell[1898148, 60920, 212, 9, 35, "Text",
      CellTags->"Nielsen"],
    Cell[1898397, 60934, 56, 1, 49, "Subsubsection",
      CellTags->"Nielsen"],
    Cell[1898478, 60939, 173, 4, 70, "Input",
      CellTags->"Nielsen"],
    Cell[1898995, 60958, 179, 5, 70, "Text",
      CellTags->"Nielsen"],
    Cell[1899199, 60967, 209, 5, 70, "Input",
      CellTags->"Nielsen"],
    Cell[1899554, 60979, 72, 1, 70, "Text",
      CellTags->"Nielsen"],
    Cell[1899651, 60984, 557, 14, 70, "Input",
      CellTags->"Nielsen"],
    Cell[1901013, 61027, 227, 5, 70, "Input",
      CellTags->"Nielsen"],
    Cell[1902100, 61062, 227, 5, 70, "Input",
      CellTags->"Nielsen"],
    Cell[1903409, 61103, 227, 5, 70, "Input",
      CellTags->"Nielsen"]},
  "NonCommFreeQ"->{
    Cell[1903872, 61122, 62, 1, 39, "ObjectName",
      CellTags->"NonCommFreeQ"],
    Cell[1903959, 61127, 64, 1, 49, "Subsubsection",
      CellTags->"NonCommFreeQ"],
    Cell[1904026, 61130, 245, 5, 56, "Text",
      CellTags->"NonCommFreeQ"],
    Cell[1904274, 61137, 562, 24, 39, "Text",
      CellTags->"NonCommFreeQ"]},
  "NonCommQ"->{
    Cell[1904885, 61167, 54, 1, 39, "ObjectName",
      CellTags->"NonCommQ"],
    Cell[1904964, 61172, 60, 1, 49, "Subsubsection",
      CellTags->"NonCommQ"],
    Cell[1905027, 61175, 206, 5, 56, "Text",
      CellTags->"NonCommQ"],
    Cell[1905236, 61182, 570, 24, 39, "Text",
      CellTags->"NonCommQ"],
    Cell[1908482, 61331, 180, 9, 39, "Text",
      CellTags->"NonCommQ"]},
  "NonCommutative"->{
    Cell[1905855, 61212, 66, 1, 39, "ObjectName",
      CellTags->"NonCommutative"],
    Cell[1905946, 61217, 66, 1, 49, "Subsubsection",
      CellTags->"NonCommutative"],
    Cell[1906015, 61220, 151, 4, 39, "Text",
      CellTags->"NonCommutative"],
    Cell[1906169, 61226, 356, 14, 39, "Text",
      CellTags->"NonCommutative"]},
  "NTerms"->{
    Cell[1906574, 61246, 50, 1, 39, "ObjectName",
      CellTags->"NTerms"],
    Cell[1906649, 61251, 58, 1, 49, "Subsubsection",
      CellTags->"NTerms"],
    Cell[1906710, 61254, 155, 4, 39, "Text",
      CellTags->"NTerms"],
    Cell[1906902, 61263, 55, 1, 49, "Subsubsection",
      CellTags->"NTerms"],
    Cell[1906982, 61268, 161, 4, 70, "Input",
      CellTags->"NTerms"],
    Cell[1907292, 61281, 171, 4, 70, "Input",
      CellTags->"NTerms"],
    Cell[1907612, 61294, 134, 3, 70, "Input",
      CellTags->"NTerms"],
    Cell[1907895, 61306, 134, 3, 70, "Input",
      CellTags->"NTerms"]},
  "NumberOfMetricTensors"->{
    Cell[1908202, 61320, 80, 1, 39, "ObjectName",
      CellTags->"NumberOfMetricTensors"],
    Cell[1908307, 61325, 73, 1, 49, "Subsubsection",
      CellTags->"NumberOfMetricTensors"],
    Cell[1908383, 61328, 96, 1, 39, "Text",
      CellTags->"NumberOfMetricTensors"]},
  "NumericalFactor"->{
    Cell[1908711, 61346, 68, 1, 39, "ObjectName",
      CellTags->"NumericalFactor"],
    Cell[1908804, 61351, 67, 1, 49, "Subsubsection",
      CellTags->"NumericalFactor"],
    Cell[1908874, 61354, 122, 4, 39, "Text",
      CellTags->"NumericalFactor"]},
  "NumericQ1"->{
    Cell[1909045, 61364, 56, 1, 39, "ObjectName",
      CellTags->"NumericQ1"],
    Cell[1909126, 61369, 61, 1, 49, "Subsubsection",
      CellTags->"NumericQ1"],
    Cell[1909190, 61372, 140, 4, 39, "Text",
      CellTags->"NumericQ1"],
    Cell[1909367, 61381, 58, 1, 49, "Subsubsection",
      CellTags->"NumericQ1"],
    Cell[1909450, 61386, 268, 7, 57, "Input",
      CellTags->"NumericQ1"],
    Cell[1909868, 61402, 328, 9, 53, "Input",
      CellTags->"NumericQ1"],
    Cell[1910346, 61420, 374, 10, 53, "Input",
      CellTags->"NumericQ1"]},
  "OneLoop"->{
    Cell[1910893, 61441, 70, 1, 39, "ObjectName",
      CellTags->"OneLoop"],
    Cell[1910966, 61444, 642, 16, 73, "Text",
      CellTags->"OneLoop"],
    Cell[1911611, 61462, 1110, 27, 56, "Text",
      CellTags->"OneLoop"],
    Cell[1912746, 61493, 59, 1, 49, "Subsubsection",
      CellTags->"OneLoop"],
    Cell[1912808, 61496, 405, 13, 56, "Text",
      CellTags->"OneLoop"],
    Cell[1913216, 61511, 355, 7, 73, "Text",
      CellTags->"OneLoop"],
    Cell[1913596, 61522, 142, 3, 56, "Input",
      CellTags->"OneLoop"],
    Cell[1915148, 61558, 854, 39, 35, "Text",
      CellTags->"OneLoop"],
    Cell[1916039, 61602, 56, 1, 49, "Subsubsection",
      CellTags->"OneLoop"],
    Cell[1916120, 61607, 342, 10, 56, "Input",
      CellTags->"OneLoop"],
    Cell[1916892, 61635, 163, 4, 52, "Input",
      CellTags->"OneLoop"],
    Cell[1917456, 61654, 440, 11, 52, "Input",
      CellTags->"OneLoop"],
    Cell[1917899, 61667, 733, 20, 39, "Text",
      CellTags->"OneLoop"],
    Cell[1918657, 61691, 1146, 30, 74, "Input",
      CellTags->"OneLoop"],
    Cell[1922879, 61802, 220, 5, 52, "Text",
      CellTags->"OneLoop"],
    Cell[1923124, 61811, 163, 4, 56, "Input",
      CellTags->"OneLoop"],
    Cell[1927399, 61917, 141, 3, 52, "Input",
      CellTags->"OneLoop"],
    Cell[1930497, 61997, 416, 12, 52, "Input",
      CellTags->"OneLoop"],
    Cell[1932204, 62051, 163, 4, 52, "Input",
      CellTags->"OneLoop"],
    Cell[1935128, 62131, 134, 3, 52, "Input",
      CellTags->"OneLoop"]},
  "OneLoopSimplify"->{
    Cell[1935311, 62140, 68, 1, 39, "ObjectName",
      CellTags->"OneLoopSimplify"],
    Cell[1935404, 62145, 67, 1, 49, "Subsubsection",
      CellTags->"OneLoopSimplify"],
    Cell[1935474, 62148, 173, 4, 39, "Text",
      CellTags->"OneLoopSimplify"],
    Cell[1935672, 62156, 158, 3, 56, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1936669, 62181, 409, 19, 35, "Text",
      CellTags->"OneLoopSimplify"],
    Cell[1937115, 62205, 64, 1, 49, "Subsubsection",
      CellTags->"OneLoopSimplify"],
    Cell[1937204, 62210, 424, 12, 56, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1938919, 62264, 179, 4, 52, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1941436, 62337, 246, 6, 52, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1942699, 62377, 485, 12, 52, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1945034, 62448, 184, 4, 52, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1958726, 62810, 312, 9, 52, "Input",
      CellTags->"OneLoopSimplify"],
    Cell[1968466, 63076, 533, 15, 52, "Input",
      CellTags->"OneLoopSimplify"]},
  "OneLoopSum"->{
    Cell[1973284, 63206, 58, 1, 39, "ObjectName",
      CellTags->"OneLoopSum"],
    Cell[1973367, 63211, 62, 1, 49, "Subsubsection",
      CellTags->"OneLoopSum"],
    Cell[1973432, 63214, 193, 5, 39, "Text",
      CellTags->"OneLoopSum"],
    Cell[1973628, 63221, 191, 9, 39, "Text",
      CellTags->"OneLoopSum"]},
  "OPE"->{
    Cell[1973868, 63236, 44, 1, 39, "ObjectName",
      CellTags->"OPE"],
    Cell[1973937, 63241, 55, 1, 49, "Subsubsection",
      CellTags->"OPE"],
    Cell[1973995, 63244, 70, 1, 39, "Text",
      CellTags->"OPE"],
    Cell[1974068, 63247, 187, 9, 39, "Text",
      CellTags->"OPE"]},
  "OPEDelta"->{
    Cell[1974304, 63262, 54, 1, 39, "ObjectName",
      CellTags->"OPEDelta"],
    Cell[1974383, 63267, 60, 1, 49, "Subsubsection",
      CellTags->"OPEDelta"],
    Cell[1974446, 63270, 141, 4, 39, "Text",
      CellTags->"OPEDelta"],
    Cell[1974590, 63276, 225, 9, 39, "Text",
      CellTags->"OPEDelta"],
    Cell[1974852, 63290, 57, 1, 49, "Subsubsection",
      CellTags->"OPEDelta"],
    Cell[1974934, 63295, 178, 4, 70, "Input",
      CellTags->"OPEDelta"],
    Cell[1975352, 63311, 165, 4, 70, "Input",
      CellTags->"OPEDelta"],
    Cell[1975666, 63324, 173, 4, 70, "Input",
      CellTags->"OPEDelta"]},
  "OPEi"->{
    Cell[1976012, 63339, 82, 1, 39, "ObjectName",
      CellTags->"OPEi"],
    Cell[1976119, 63344, 56, 1, 49, "Subsubsection",
      CellTags->"OPEi"],
    Cell[1976178, 63347, 172, 4, 39, "Text",
      CellTags->"OPEi"],
    Cell[1976387, 63356, 53, 1, 49, "Subsubsection",
      CellTags->"OPEi"],
    Cell[1976465, 63361, 100, 2, 56, "Input",
      CellTags->"OPEi"],
    Cell[1976714, 63372, 304, 7, 52, "Input",
      CellTags->"OPEi"],
    Cell[1977314, 63392, 1365, 40, 72, "Input",
      CellTags->"OPEi"],
    Cell[1978930, 63443, 56, 1, 35, "Text",
      CellTags->"OPEi"],
    Cell[1979011, 63448, 632, 17, 56, "Input",
      CellTags->"OPEi"],
    Cell[1979902, 63477, 662, 19, 52, "Input",
      CellTags->"OPEi"],
    Cell[1980795, 63508, 632, 17, 52, "Input",
      CellTags->"OPEi"]},
  "OPEInsert"->{
    Cell[1981710, 63539, 74, 1, 39, "ObjectName",
      CellTags->"OPEInsert"],
    Cell[1981809, 63544, 61, 1, 49, "Subsubsection",
      CellTags->"OPEInsert"],
    Cell[1981873, 63547, 267, 6, 56, "Text",
      CellTags->"OPEInsert"],
    Cell[1982143, 63555, 178, 9, 39, "Text",
      CellTags->"OPEInsert"],
    Cell[1982358, 63569, 58, 1, 49, "Subsubsection",
      CellTags->"OPEInsert"],
    Cell[1982419, 63572, 101, 2, 56, "Input",
      CellTags->"OPEInsert"]},
  "OPEInt"->{
    Cell[1982569, 63580, 68, 1, 39, "ObjectName",
      CellTags->"OPEInt"],
    Cell[1982662, 63585, 58, 1, 49, "Subsubsection",
      CellTags->"OPEInt"],
    Cell[1982723, 63588, 260, 6, 56, "Text",
      CellTags->"OPEInt"],
    Cell[1982986, 63596, 175, 9, 39, "Text",
      CellTags->"OPEInt"],
    Cell[1983198, 63610, 55, 1, 49, "Subsubsection",
      CellTags->"OPEInt"],
    Cell[1983256, 63613, 98, 2, 56, "Input",
      CellTags->"OPEInt"]},
  "OPEIntDelta"->{
    Cell[1983403, 63621, 78, 1, 39, "ObjectName",
      CellTags->"OPEIntDelta"],
    Cell[1983506, 63626, 63, 1, 49, "Subsubsection",
      CellTags->"OPEIntDelta"],
    Cell[1983572, 63629, 484, 10, 81, "Text",
      CellTags->"OPEIntDelta"],
    Cell[1984059, 63641, 180, 9, 39, "Text",
      CellTags->"OPEIntDelta"],
    Cell[1984276, 63655, 60, 1, 49, "Subsubsection",
      CellTags->"OPEIntDelta"],
    Cell[1984339, 63658, 103, 2, 56, "Input",
      CellTags->"OPEIntDelta"]},
  "OPEIntegrate"->{
    Cell[1984491, 63666, 80, 1, 39, "ObjectName",
      CellTags->"OPEIntegrate"],
    Cell[1984596, 63671, 64, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrate"],
    Cell[1984663, 63674, 269, 6, 56, "Text",
      CellTags->"OPEIntegrate"],
    Cell[1984935, 63682, 181, 9, 39, "Text",
      CellTags->"OPEIntegrate"],
    Cell[1985153, 63696, 61, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrate"],
    Cell[1985217, 63699, 104, 2, 56, "Input",
      CellTags->"OPEIntegrate"]},
  "OPEIntegrateDelta"->{
    Cell[1985370, 63707, 90, 1, 39, "ObjectName",
      CellTags->"OPEIntegrateDelta"],
    Cell[1985485, 63712, 69, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrateDelta"],
    Cell[1985557, 63715, 505, 10, 81, "Text",
      CellTags->"OPEIntegrateDelta"],
    Cell[1986065, 63727, 186, 9, 39, "Text",
      CellTags->"OPEIntegrateDelta"],
    Cell[1986288, 63741, 66, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrateDelta"],
    Cell[1986357, 63744, 109, 2, 56, "Input",
      CellTags->"OPEIntegrateDelta"]},
  "OPEIntegrate2"->{
    Cell[1986515, 63752, 82, 1, 39, "ObjectName",
      CellTags->"OPEIntegrate2"],
    Cell[1986622, 63757, 65, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrate2"],
    Cell[1986690, 63760, 153, 4, 39, "Text",
      CellTags->"OPEIntegrate2"],
    Cell[1986846, 63766, 209, 9, 39, "Text",
      CellTags->"OPEIntegrate2"],
    Cell[1987092, 63780, 62, 1, 49, "Subsubsection",
      CellTags->"OPEIntegrate2"],
    Cell[1987157, 63783, 105, 2, 56, "Input",
      CellTags->"OPEIntegrate2"]},
  "OPESumExplicit"->{
    Cell[1987311, 63791, 66, 1, 39, "ObjectName",
      CellTags->"OPESumExplicit"],
    Cell[1987402, 63796, 66, 1, 49, "Subsubsection",
      CellTags->"OPESumExplicit"],
    Cell[1987471, 63799, 86, 1, 39, "Text",
      CellTags->"OPESumExplicit"],
    Cell[1987560, 63802, 330, 14, 39, "Text",
      CellTags->"OPESumExplicit"],
    Cell[1987927, 63821, 63, 1, 49, "Subsubsection",
      CellTags->"OPESumExplicit"],
    Cell[1988015, 63826, 482, 13, 70, "Input",
      CellTags->"OPESumExplicit"],
    Cell[1988991, 63859, 151, 3, 70, "Input",
      CellTags->"OPESumExplicit"],
    Cell[1989674, 63884, 686, 18, 70, "Input",
      CellTags->"OPESumExplicit"],
    Cell[1991276, 63934, 151, 3, 70, "Input",
      CellTags->"OPESumExplicit"],
    Cell[1994259, 64025, 194, 5, 70, "Input",
      CellTags->"OPESumExplicit"]},
  "OPESum"->{
    Cell[1994502, 64036, 50, 1, 39, "ObjectName",
      CellTags->"OPESum"],
    Cell[1994577, 64041, 58, 1, 49, "Subsubsection",
      CellTags->"OPESum"],
    Cell[1994638, 64044, 128, 4, 39, "Text",
      CellTags->"OPESum"],
    Cell[1994769, 64050, 346, 14, 39, "Text",
      CellTags->"OPESum"],
    Cell[1995152, 64069, 55, 1, 49, "Subsubsection",
      CellTags->"OPESum"],
    Cell[1995232, 64074, 571, 15, 70, "Input",
      CellTags->"OPESum"],
    Cell[1997182, 64131, 143, 3, 70, "Input",
      CellTags->"OPESum"],
    Cell[2000212, 64218, 678, 18, 70, "Input",
      CellTags->"OPESum"],
    Cell[2001806, 64268, 143, 3, 70, "Input",
      CellTags->"OPESum"],
    Cell[2004781, 64359, 186, 5, 70, "Input",
      CellTags->"OPESum"]},
  "OPESumSimplify"->{
    Cell[2005016, 64370, 66, 1, 39, "ObjectName",
      CellTags->"OPESumSimplify"],
    Cell[2005107, 64375, 66, 1, 49, "Subsubsection",
      CellTags->"OPESumSimplify"],
    Cell[2005176, 64378, 93, 1, 39, "Text",
      CellTags->"OPESumSimplify"],
    Cell[2005272, 64381, 330, 14, 39, "Text",
      CellTags->"OPESumSimplify"],
    Cell[2005639, 64400, 63, 1, 49, "Subsubsection",
      CellTags->"OPESumSimplify"],
    Cell[2005727, 64405, 170, 5, 70, "Input",
      CellTags->"OPESumSimplify"],
    Cell[2007321, 64453, 114, 2, 70, "Input",
      CellTags->"OPESumSimplify"],
    Cell[2009096, 64501, 328, 8, 70, "Input",
      CellTags->"OPESumSimplify"],
    Cell[2009794, 64526, 535, 14, 70, "Input",
      CellTags->"OPESumSimplify"],
    Cell[2011087, 64568, 144, 3, 70, "Input",
      CellTags->"OPESumSimplify"]},
  "OPE1Loop"->{
    Cell[2011755, 64592, 72, 1, 39, "ObjectName",
      CellTags->"OPE1Loop"],
    Cell[2011852, 64597, 60, 1, 49, "Subsubsection",
      CellTags->"OPE1Loop"],
    Cell[2011915, 64600, 122, 4, 39, "Text",
      CellTags->"OPE1Loop"],
    Cell[2012040, 64606, 186, 9, 39, "Text",
      CellTags->"OPE1Loop"],
    Cell[2012263, 64620, 57, 1, 49, "Subsubsection",
      CellTags->"OPE1Loop"],
    Cell[2012323, 64623, 100, 2, 56, "Input",
      CellTags->"OPE1Loop"]},
  "OPE2AI"->{
    Cell[2012472, 64631, 68, 1, 39, "ObjectName",
      CellTags->"OPE2AI"],
    Cell[2012565, 64636, 58, 1, 49, "Subsubsection",
      CellTags->"OPE2AI"],
    Cell[2012626, 64639, 80, 1, 39, "Text",
      CellTags->"OPE2AI"],
    Cell[2012709, 64642, 138, 3, 56, "Input",
      CellTags->"OPE2AI"],
    Cell[2012850, 64647, 184, 9, 39, "Text",
      CellTags->"OPE2AI"],
    Cell[2013071, 64661, 55, 1, 49, "Subsubsection",
      CellTags->"OPE2AI"],
    Cell[2013129, 64664, 98, 2, 56, "Input",
      CellTags->"OPE2AI"]},
  "OPE2TID"->{
    Cell[2013276, 64672, 70, 1, 39, "ObjectName",
      CellTags->"OPE2TID"],
    Cell[2013371, 64677, 59, 1, 49, "Subsubsection",
      CellTags->"OPE2TID"],
    Cell[2013433, 64680, 187, 5, 39, "Text",
      CellTags->"OPE2TID"],
    Cell[2013645, 64689, 140, 3, 56, "Input",
      CellTags->"OPE2TID"],
    Cell[2014186, 64706, 185, 9, 35, "Text",
      CellTags->"OPE2TID"],
    Cell[2014408, 64720, 56, 1, 49, "Subsubsection",
      CellTags->"OPE2TID"],
    Cell[2014467, 64723, 99, 2, 56, "Input",
      CellTags->"OPE2TID"]},
  "OptionsSelect"->{
    Cell[2014615, 64731, 64, 1, 39, "ObjectName",
      CellTags->"OptionsSelect"],
    Cell[2014704, 64736, 65, 1, 49, "Subsubsection",
      CellTags->"OptionsSelect"],
    Cell[2014772, 64739, 220, 5, 56, "Text",
      CellTags->"OptionsSelect"]},
  "Pair"->{
    Cell[2015041, 64750, 46, 1, 39, "ObjectName",
      CellTags->"Pair"],
    Cell[2015112, 64755, 56, 1, 49, "Subsubsection",
      CellTags->"Pair"],
    Cell[2015171, 64758, 512, 11, 73, "Text",
      CellTags->"Pair"],
    Cell[2015686, 64771, 1082, 49, 39, "Text",
      CellTags->"Pair"],
    Cell[2016805, 64825, 53, 1, 49, "Subsubsection",
      CellTags->"Pair"],
    Cell[2016861, 64828, 85, 1, 70, "Text",
      CellTags->"Pair"],
    Cell[2016971, 64833, 260, 6, 70, "Input",
      CellTags->"Pair"],
    Cell[2017444, 64849, 74, 1, 70, "Text",
      CellTags->"Pair"],
    Cell[2017543, 64854, 322, 8, 70, "Input",
      CellTags->"Pair"],
    Cell[2018305, 64881, 250, 6, 70, "Input",
      CellTags->"Pair"],
    Cell[2018884, 64903, 239, 6, 70, "Input",
      CellTags->"Pair"],
    Cell[2019432, 64923, 239, 6, 70, "Input",
      CellTags->"Pair"],
    Cell[2019895, 64941, 270, 7, 70, "Input",
      CellTags->"Pair"],
    Cell[2020554, 64965, 272, 7, 70, "Input",
      CellTags->"Pair"],
    Cell[2021050, 64984, 272, 7, 70, "Input",
      CellTags->"Pair"],
    Cell[2021546, 65003, 376, 10, 70, "Input",
      CellTags->"Pair"],
    Cell[2022071, 65022, 328, 8, 70, "Input",
      CellTags->"Pair"],
    Cell[2022826, 65048, 460, 12, 70, "Input",
      CellTags->"Pair"]},
  "PairCollect"->{
    Cell[2023459, 65071, 60, 1, 39, "ObjectName",
      CellTags->"PairCollect"],
    Cell[2023544, 65076, 63, 1, 49, "Subsubsection",
      CellTags->"PairCollect"],
    Cell[2023610, 65079, 154, 4, 39, "Text",
      CellTags->"PairCollect"],
    Cell[2023767, 65085, 309, 14, 39, "Text",
      CellTags->"PairCollect"]},
  "PairContract"->{
    Cell[2024125, 65105, 62, 1, 39, "ObjectName",
      CellTags->"PairContract"],
    Cell[2024212, 65110, 64, 1, 49, "Subsubsection",
      CellTags->"PairContract"],
    Cell[2024279, 65113, 121, 4, 39, "Text",
      CellTags->"PairContract"],
    Cell[2024403, 65119, 297, 13, 39, "Text",
      CellTags->"PairContract"]},
  "PairContract2"->{
    Cell[2024749, 65138, 64, 1, 39, "ObjectName",
      CellTags->"PairContract2"],
    Cell[2024838, 65143, 65, 1, 49, "Subsubsection",
      CellTags->"PairContract2"],
    Cell[2024906, 65146, 143, 4, 39, "Text",
      CellTags->"PairContract2"],
    Cell[2025052, 65152, 430, 18, 39, "Text",
      CellTags->"PairContract2"]},
  "PairContract3"->{
    Cell[2025531, 65176, 64, 1, 39, "ObjectName",
      CellTags->"PairContract3"],
    Cell[2025620, 65181, 65, 1, 49, "Subsubsection",
      CellTags->"PairContract3"],
    Cell[2025688, 65184, 143, 4, 39, "Text",
      CellTags->"PairContract3"],
    Cell[2025834, 65190, 430, 18, 39, "Text",
      CellTags->"PairContract3"]},
  "PartialD"->{
    Cell[2026313, 65214, 54, 1, 39, "ObjectName",
      CellTags->"PartialD"],
    Cell[2026392, 65219, 60, 1, 49, "Subsubsection",
      CellTags->"PartialD"],
    Cell[2026455, 65222, 1019, 35, 58, "Text",
      CellTags->"PartialD"],
    Cell[2027477, 65259, 624, 24, 39, "Text",
      CellTags->"PartialD"],
    Cell[2028138, 65288, 66, 1, 50, "Subsubsection",
      CellTags->"PartialD"],
    Cell[2028229, 65293, 280, 7, 70, "Input",
      CellTags->"PartialD"],
    Cell[2028907, 65317, 144, 3, 70, "Input",
      CellTags->"PartialD"],
    Cell[2029449, 65337, 142, 3, 70, "Input",
      CellTags->"PartialD"]},
  "PartialDRelations"->{
    Cell[2029987, 65356, 72, 1, 39, "ObjectName",
      CellTags->"PartialDRelations"],
    Cell[2030084, 65361, 69, 1, 49, "Subsubsection",
      CellTags->"PartialDRelations"],
    Cell[2030156, 65364, 193, 5, 39, "Text",
      CellTags->"PartialDRelations"],
    Cell[2030352, 65371, 753, 29, 39, "Text",
      CellTags->"PartialDRelations"],
    Cell[2031142, 65405, 75, 1, 50, "Subsubsection",
      CellTags->"PartialDRelations"],
    Cell[2031242, 65410, 381, 10, 43, "Input",
      CellTags->"PartialDRelations"],
    Cell[2032038, 65438, 281, 7, 39, "Input",
      CellTags->"PartialDRelations"]},
  "PartialFourVector"->{
    Cell[2033470, 65487, 72, 1, 39, "ObjectName",
      CellTags->"PartialFourVector"],
    Cell[2033567, 65492, 69, 1, 49, "Subsubsection",
      CellTags->"PartialFourVector"],
    Cell[2033639, 65495, 279, 6, 56, "Text",
      CellTags->"PartialFourVector"],
    Cell[2033921, 65503, 435, 19, 39, "Text",
      CellTags->"PartialFourVector"],
    Cell[2034393, 65527, 75, 1, 50, "Subsubsection",
      CellTags->"PartialFourVector"],
    Cell[2034493, 65532, 458, 11, 59, "Input",
      CellTags->"PartialFourVector"]},
  "PartialIntegrate"->{
    Cell[2035491, 65566, 70, 1, 39, "ObjectName",
      CellTags->"PartialIntegrate"],
    Cell[2035586, 65571, 68, 1, 49, "Subsubsection",
      CellTags->"PartialIntegrate"],
    Cell[2035657, 65574, 285, 6, 56, "Text",
      CellTags->"PartialIntegrate"],
    Cell[2035967, 65584, 158, 3, 56, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2036128, 65589, 224, 6, 37, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2036401, 65601, 74, 1, 50, "Subsubsection",
      CellTags->"PartialIntegrate"],
    Cell[2036500, 65606, 389, 10, 43, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2036892, 65618, 1237, 35, 61, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2038166, 65658, 319, 8, 39, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2038488, 65668, 453, 13, 62, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2038978, 65686, 344, 10, 39, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2039325, 65698, 491, 15, 53, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2039853, 65718, 371, 11, 53, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2040227, 65731, 479, 14, 57, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2040743, 65750, 371, 10, 39, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2041117, 65762, 226, 6, 56, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2041380, 65773, 392, 11, 39, "Input",
      CellTags->"PartialIntegrate"],
    Cell[2041775, 65786, 226, 6, 56, "Output",
      CellTags->"PartialIntegrate"],
    Cell[2042016, 65795, 168, 4, 39, "Input",
      CellTags->"PartialIntegrate"]},
  "PartitHead"->{
    Cell[2042233, 65805, 58, 1, 39, "ObjectName",
      CellTags->"PartitHead"],
    Cell[2042316, 65810, 62, 1, 49, "Subsubsection",
      CellTags->"PartitHead"],
    Cell[2042381, 65813, 168, 4, 39, "Text",
      CellTags->"PartitHead"]},
  "PauliSigma"->{
    Cell[2042598, 65823, 58, 1, 39, "ObjectName",
      CellTags->"PauliSigma"],
    Cell[2042681, 65828, 62, 1, 49, "Subsubsection",
      CellTags->"PauliSigma"],
    Cell[2042746, 65831, 255, 6, 56, "Text",
      CellTags->"PauliSigma"],
    Cell[2043026, 65841, 170, 4, 56, "Input",
      CellTags->"PauliSigma"],
    Cell[2043199, 65847, 1990, 60, 126, "Output",
      CellTags->"PauliSigma"]},
  "PaVe"->{
    Cell[2045250, 65914, 46, 1, 39, "ObjectName",
      CellTags->"PaVe"],
    Cell[2045321, 65919, 56, 1, 49, "Subsubsection",
      CellTags->"PaVe"],
    Cell[2045380, 65922, 431, 8, 73, "Text",
      CellTags->"PaVe"],
    Cell[2045814, 65932, 278, 8, 39, "Text",
      CellTags->"PaVe"],
    Cell[2046095, 65942, 194, 9, 39, "Text",
      CellTags->"PaVe"],
    Cell[2046326, 65956, 53, 1, 49, "Subsubsection",
      CellTags->"PaVe"],
    Cell[2046382, 65959, 79, 1, 70, "Text",
      CellTags->"PaVe"],
    Cell[2046486, 65964, 369, 10, 70, "Input",
      CellTags->"PaVe"],
    Cell[2047571, 65993, 333, 9, 70, "Input",
      CellTags->"PaVe"]},
  "PaVeOrderList"->{
    Cell[2050521, 66071, 64, 1, 39, "ObjectName",
      CellTags->"PaVeOrderList"],
    Cell[2050610, 66076, 65, 1, 49, "Subsubsection",
      CellTags->"PaVeOrderList"],
    Cell[2050678, 66079, 177, 4, 39, "Text",
      CellTags->"PaVeOrderList"],
    Cell[2050858, 66085, 529, 23, 39, "Text",
      CellTags->"PaVeOrderList"]},
  "PaVeOrder"->{
    Cell[2051436, 66114, 56, 1, 39, "ObjectName",
      CellTags->"PaVeOrder"],
    Cell[2051517, 66119, 61, 1, 49, "Subsubsection",
      CellTags->"PaVeOrder"],
    Cell[2051581, 66122, 600, 16, 56, "Text",
      CellTags->"PaVeOrder"],
    Cell[2052184, 66140, 199, 9, 39, "Text",
      CellTags->"PaVeOrder"],
    Cell[2052420, 66154, 58, 1, 49, "Subsubsection",
      CellTags->"PaVeOrder"],
    Cell[2052503, 66159, 491, 12, 70, "Input",
      CellTags->"PaVeOrder"],
    Cell[2053812, 66194, 491, 12, 70, "Input",
      CellTags->"PaVeOrder"],
    Cell[2055121, 66229, 851, 20, 70, "Input",
      CellTags->"PaVeOrder"]},
  "PaVeReduce"->{
    Cell[2057471, 66286, 58, 1, 39, "ObjectName",
      CellTags->"PaVeReduce"],
    Cell[2057554, 66291, 62, 1, 49, "Subsubsection",
      CellTags->"PaVeReduce"],
    Cell[2057619, 66294, 168, 4, 39, "Text",
      CellTags->"PaVeReduce"],
    Cell[2057812, 66302, 148, 3, 56, "Input",
      CellTags->"PaVeReduce"],
    Cell[2058505, 66324, 134, 3, 52, "Input",
      CellTags->"PaVeReduce"],
    Cell[2058938, 66336, 302, 14, 35, "Text",
      CellTags->"PaVeReduce"],
    Cell[2059277, 66355, 59, 1, 49, "Subsubsection",
      CellTags->"PaVeReduce"],
    Cell[2059361, 66360, 643, 16, 56, "Input",
      CellTags->"PaVeReduce"],
    Cell[2064102, 66489, 134, 3, 52, "Input",
      CellTags->"PaVeReduce"],
    Cell[2064418, 66502, 135, 3, 52, "Input",
      CellTags->"PaVeReduce"],
    Cell[2064735, 66515, 607, 15, 52, "Input",
      CellTags->"PaVeReduce"],
    Cell[2065892, 66550, 487, 12, 72, "Input",
      CellTags->"PaVeReduce"],
    Cell[2066996, 66582, 255, 7, 56, "Input",
      CellTags->"PaVeReduce"],
    Cell[2067254, 66591, 199, 5, 56, "Input",
      CellTags->"PaVeReduce"],
    Cell[2067478, 66600, 790, 19, 76, "Input",
      CellTags->"PaVeReduce"],
    Cell[2080093, 66896, 321, 8, 52, "Input",
      CellTags->"PaVeReduce"],
    Cell[2080439, 66908, 459, 11, 56, "Input",
      CellTags->"PaVeReduce"],
    Cell[2085122, 67002, 299, 7, 52, "Input",
      CellTags->"PaVeReduce"]},
  "PD"->{
    Cell[2085470, 67015, 42, 1, 39, "ObjectName",
      CellTags->"PD"],
    Cell[2085537, 67020, 54, 1, 49, "Subsubsection",
      CellTags->"PD"],
    Cell[2085594, 67023, 82, 1, 39, "Text",
      CellTags->"PD"],
    Cell[2085679, 67026, 333, 14, 39, "Text",
      CellTags->"PD"]},
  "PlusDistribution"->{
    Cell[2086061, 67046, 70, 1, 39, "ObjectName",
      CellTags->"PlusDistribution"],
    Cell[2086156, 67051, 68, 1, 49, "Subsubsection",
      CellTags->"PlusDistribution"],
    Cell[2086227, 67054, 148, 4, 39, "Text",
      CellTags->"PlusDistribution"],
    Cell[2086378, 67060, 206, 9, 39, "Text",
      CellTags->"PlusDistribution"],
    Cell[2086621, 67074, 65, 1, 49, "Subsubsection",
      CellTags->"PlusDistribution"],
    Cell[2086711, 67079, 241, 6, 70, "Input",
      CellTags->"PlusDistribution"],
    Cell[2087220, 67099, 308, 8, 70, "Input",
      CellTags->"PlusDistribution"],
    Cell[2087871, 67123, 398, 10, 70, "Input",
      CellTags->"PlusDistribution"],
    Cell[2088418, 67142, 473, 12, 70, "Input",
      CellTags->"PlusDistribution"],
    Cell[2089040, 67163, 510, 13, 70, "Input",
      CellTags->"PlusDistribution"],
    Cell[2089699, 67185, 376, 10, 70, "Input",
      CellTags->"PlusDistribution"]},
  "Polarization"->{
    Cell[2090582, 67217, 62, 1, 39, "ObjectName",
      CellTags->"Polarization"],
    Cell[2090669, 67222, 64, 1, 49, "Subsubsection",
      CellTags->"Polarization"],
    Cell[2090736, 67225, 770, 12, 107, "Text",
      CellTags->"Polarization"],
    Cell[2091509, 67239, 517, 19, 39, "Text",
      CellTags->"Polarization"],
    Cell[2092063, 67263, 61, 1, 49, "Subsubsection",
      CellTags->"Polarization"],
    Cell[2092149, 67268, 146, 3, 70, "Input",
      CellTags->"Polarization"],
    Cell[2092778, 67290, 192, 5, 70, "Input",
      CellTags->"Polarization"],
    Cell[2093182, 67305, 196, 5, 70, "Input",
      CellTags->"Polarization"],
    Cell[2094067, 67335, 237, 6, 70, "Input",
      CellTags->"Polarization"],
    Cell[2094540, 67352, 187, 4, 70, "Input",
      CellTags->"Polarization"],
    Cell[2095419, 67379, 235, 6, 70, "Input",
      CellTags->"Polarization"],
    Cell[2095954, 67398, 364, 9, 70, "Input",
      CellTags->"Polarization"]},
  "PolarizationSum"->{
    Cell[2096491, 67418, 68, 1, 39, "ObjectName",
      CellTags->"PolarizationSum"],
    Cell[2096584, 67423, 67, 1, 49, "Subsubsection",
      CellTags->"PolarizationSum"],
    Cell[2096654, 67426, 280, 6, 56, "Text",
      CellTags->"PolarizationSum"],
    Cell[2096959, 67436, 158, 3, 56, "Input",
      CellTags->"PolarizationSum"],
    Cell[2097321, 67449, 211, 9, 35, "Text",
      CellTags->"PolarizationSum"],
    Cell[2097569, 67463, 64, 1, 49, "Subsubsection",
      CellTags->"PolarizationSum"],
    Cell[2097658, 67468, 187, 4, 56, "Input",
      CellTags->"PolarizationSum"],
    Cell[2098101, 67485, 197, 4, 52, "Input",
      CellTags->"PolarizationSum"],
    Cell[2099829, 67539, 207, 4, 52, "Input",
      CellTags->"PolarizationSum"],
    Cell[2103963, 67659, 296, 7, 52, "Input",
      CellTags->"PolarizationSum"]},
  "PolarizationUncontract"->{
    Cell[2114309, 67943, 82, 1, 39, "ObjectName",
      CellTags->"PolarizationUncontract"],
    Cell[2114416, 67948, 74, 1, 49, "Subsubsection",
      CellTags->"PolarizationUncontract"],
    Cell[2114493, 67951, 153, 4, 39, "Text",
      CellTags->"PolarizationUncontract"],
    Cell[2114649, 67957, 344, 14, 39, "Text",
      CellTags->"PolarizationUncontract"]},
  "PolarizationVector"->{
    Cell[2115042, 67977, 74, 1, 39, "ObjectName",
      CellTags->"PolarizationVector"],
    Cell[2115141, 67982, 70, 1, 49, "Subsubsection",
      CellTags->"PolarizationVector"],
    Cell[2115214, 67985, 104, 1, 39, "Text",
      CellTags->"PolarizationVector"],
    Cell[2115321, 67988, 441, 18, 39, "Text",
      CellTags->"PolarizationVector"],
    Cell[2115799, 68011, 67, 1, 49, "Subsubsection",
      CellTags->"PolarizationVector"],
    Cell[2115891, 68016, 189, 4, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2116551, 68039, 231, 5, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2117241, 68061, 265, 9, 70, "Text",
      CellTags->"PolarizationVector"],
    Cell[2117531, 68074, 227, 5, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2118114, 68092, 547, 18, 70, "Text",
      CellTags->"PolarizationVector"],
    Cell[2118686, 68114, 338, 8, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2119834, 68151, 149, 3, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2120132, 68163, 371, 9, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2121358, 68202, 149, 3, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2122455, 68235, 1987, 53, 70, "Text",
      CellTags->"PolarizationVector"],
    Cell[2124467, 68292, 371, 9, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2125433, 68323, 149, 3, 70, "Input",
      CellTags->"PolarizationVector"],
    Cell[2126530, 68356, 185, 4, 70, "Input",
      CellTags->"PolarizationVector"]},
  "PositiveInteger"->{
    Cell[2126764, 68366, 68, 1, 39, "ObjectName",
      CellTags->"PositiveInteger"],
    Cell[2126857, 68371, 67, 1, 49, "Subsubsection",
      CellTags->"PositiveInteger"],
    Cell[2126927, 68374, 137, 4, 39, "Text",
      CellTags->"PositiveInteger"],
    Cell[2127067, 68380, 199, 9, 39, "Text",
      CellTags->"PositiveInteger"]},
  "PositiveNumber"->{
    Cell[2127315, 68395, 66, 1, 39, "ObjectName",
      CellTags->"PositiveNumber"],
    Cell[2127406, 68400, 66, 1, 49, "Subsubsection",
      CellTags->"PositiveNumber"],
    Cell[2127475, 68403, 147, 4, 39, "Text",
      CellTags->"PositiveNumber"],
    Cell[2127625, 68409, 198, 9, 39, "Text",
      CellTags->"PositiveNumber"]},
  "PowerFactor"->{
    Cell[2127872, 68424, 60, 1, 39, "ObjectName",
      CellTags->"PowerFactor"],
    Cell[2127957, 68429, 63, 1, 49, "Subsubsection",
      CellTags->"PowerFactor"],
    Cell[2128023, 68432, 422, 16, 39, "Text",
      CellTags->"PowerFactor"],
    Cell[2128448, 68450, 210, 9, 39, "Text",
      CellTags->"PowerFactor"],
    Cell[2128695, 68464, 60, 1, 49, "Subsubsection",
      CellTags->"PowerFactor"],
    Cell[2128780, 68469, 178, 5, 56, "Input",
      CellTags->"PowerFactor"],
    Cell[2129192, 68486, 142, 3, 52, "Input",
      CellTags->"PowerFactor"]},
  "PowerSimplify"->{
    Cell[2129593, 68503, 64, 1, 39, "ObjectName",
      CellTags->"PowerSimplify"],
    Cell[2129682, 68508, 65, 1, 49, "Subsubsection",
      CellTags->"PowerSimplify"],
    Cell[2129750, 68511, 2968, 112, 73, "Text",
      CellTags->"PowerSimplify"],
    Cell[2132721, 68625, 305, 14, 39, "Text",
      CellTags->"PowerSimplify"],
    Cell[2133063, 68644, 62, 1, 49, "Subsubsection",
      CellTags->"PowerSimplify"],
    Cell[2133150, 68649, 288, 8, 56, "Input",
      CellTags->"PowerSimplify"],
    Cell[2133587, 68666, 293, 8, 52, "Input",
      CellTags->"PowerSimplify"],
    Cell[2134112, 68686, 293, 8, 52, "Input",
      CellTags->"PowerSimplify"],
    Cell[2134637, 68706, 233, 6, 52, "Input",
      CellTags->"PowerSimplify"]},
  "Power2"->{
    Cell[2135126, 68726, 50, 1, 39, "ObjectName",
      CellTags->"Power2"],
    Cell[2135201, 68731, 58, 1, 49, "Subsubsection",
      CellTags->"Power2"],
    Cell[2135262, 68734, 214, 5, 56, "Text",
      CellTags->"Power2"],
    Cell[2135479, 68741, 199, 9, 39, "Text",
      CellTags->"Power2"],
    Cell[2135715, 68755, 55, 1, 49, "Subsubsection",
      CellTags->"Power2"],
    Cell[2135795, 68760, 182, 5, 56, "Input",
      CellTags->"Power2"],
    Cell[2136207, 68777, 183, 5, 52, "Input",
      CellTags->"Power2"]},
  "PostFortranFile"->{
    Cell[2136756, 68800, 68, 1, 39, "ObjectName",
      CellTags->"PostFortranFile"],
    Cell[2136849, 68805, 67, 1, 49, "Subsubsection",
      CellTags->"PostFortranFile"],
    Cell[2136919, 68808, 230, 5, 56, "Text",
      CellTags->"PostFortranFile"],
    Cell[2137152, 68815, 331, 14, 39, "Text",
      CellTags->"PostFortranFile"]},
  "Prefactor"->{
    Cell[2137532, 68835, 56, 1, 39, "ObjectName",
      CellTags->"Prefactor"],
    Cell[2137613, 68840, 61, 1, 49, "Subsubsection",
      CellTags->"Prefactor"],
    Cell[2137677, 68843, 291, 6, 56, "Text",
      CellTags->"Prefactor"],
    Cell[2137971, 68851, 316, 14, 39, "Text",
      CellTags->"Prefactor"]},
  "PreFortranFile"->{
    Cell[2138336, 68871, 66, 1, 39, "ObjectName",
      CellTags->"PreFortranFile"],
    Cell[2138427, 68876, 66, 1, 49, "Subsubsection",
      CellTags->"PreFortranFile"],
    Cell[2138496, 68879, 233, 5, 56, "Text",
      CellTags->"PreFortranFile"],
    Cell[2138732, 68886, 333, 14, 39, "Text",
      CellTags->"PreFortranFile"]},
  "PropagatorDenominator"->{
    Cell[2139114, 68906, 80, 1, 39, "ObjectName",
      CellTags->"PropagatorDenominator"],
    Cell[2139219, 68911, 73, 1, 49, "Subsubsection",
      CellTags->"PropagatorDenominator"],
    Cell[2139295, 68914, 569, 15, 56, "Text",
      CellTags->"PropagatorDenominator"],
    Cell[2139867, 68931, 526, 19, 39, "Text",
      CellTags->"PropagatorDenominator"],
    Cell[2140430, 68955, 70, 1, 49, "Subsubsection",
      CellTags->"PropagatorDenominator"],
    Cell[2140525, 68960, 215, 5, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2141007, 68978, 164, 3, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2141375, 68992, 247, 6, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2141889, 69011, 190, 4, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2142360, 69027, 231, 5, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2142872, 69044, 173, 3, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2143367, 69062, 155, 3, 70, "Input",
      CellTags->"PropagatorDenominator"],
    Cell[2143896, 69079, 149, 3, 70, "Input",
      CellTags->"PropagatorDenominator"]},
  "PropagatorDenominatorExplicit"->{
    Cell[2144094, 69088, 96, 1, 39, "ObjectName",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2144215, 69093, 81, 1, 49, "Subsubsection",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2144299, 69096, 244, 5, 56, "Text",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2144546, 69103, 402, 14, 39, "Text",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2144985, 69122, 78, 1, 49, "Subsubsection",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2145088, 69127, 383, 11, 70, "Input",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2145917, 69157, 217, 5, 70, "Input",
      CellTags->"PropagatorDenominatorExplicit"],
    Cell[2147342, 69202, 159, 3, 70, "Input",
      CellTags->"PropagatorDenominatorExplicit"]},
  "QGV"->{
    Cell[2148238, 69232, 44, 1, 39, "ObjectName",
      CellTags->"QGV"],
    Cell[2148307, 69237, 55, 1, 49, "Subsubsection",
      CellTags->"QGV"],
    Cell[2148365, 69240, 73, 1, 39, "Text",
      CellTags->"QGV"],
    Cell[2148441, 69243, 211, 9, 39, "Text",
      CellTags->"QGV"]},
  "QO"->{
    Cell[2148701, 69258, 42, 1, 39, "ObjectName",
      CellTags->"QO"],
    Cell[2148768, 69263, 54, 1, 49, "Subsubsection",
      CellTags->"QO"],
    Cell[2148825, 69266, 74, 1, 39, "Text",
      CellTags->"QO"],
    Cell[2148902, 69269, 219, 9, 39, "Text",
      CellTags->"QO"]},
  "QP"->{
    Cell[2149170, 69284, 42, 1, 39, "ObjectName",
      CellTags->"QP"],
    Cell[2149237, 69289, 54, 1, 49, "Subsubsection",
      CellTags->"QP"],
    Cell[2149294, 69292, 170, 4, 39, "Text",
      CellTags->"QP"],
    Cell[2149467, 69298, 207, 9, 39, "Text",
      CellTags->"QP"]},
  "QuantumField"->{
    Cell[2149723, 69313, 62, 1, 39, "ObjectName",
      CellTags->"QuantumField"],
    Cell[2149810, 69318, 64, 1, 49, "Subsubsection",
      CellTags->"QuantumField"],
    Cell[2149877, 69321, 410, 9, 56, "Text",
      CellTags->"QuantumField"],
    Cell[2150290, 69332, 454, 19, 39, "Text",
      CellTags->"QuantumField"],
    Cell[2150781, 69356, 61, 1, 49, "Subsubsection",
      CellTags->"QuantumField"],
    Cell[2150845, 69359, 72, 1, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2150942, 69364, 146, 3, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2151237, 69376, 159, 3, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2151577, 69389, 155, 3, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2151864, 69399, 81, 1, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2151970, 69404, 206, 5, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2152333, 69417, 95, 1, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2152453, 69422, 254, 6, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2153139, 69448, 142, 3, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2153568, 69461, 409, 14, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2154002, 69479, 248, 6, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2154572, 69499, 188, 5, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2154785, 69508, 293, 7, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2155355, 69525, 89, 1, 70, "Text",
      CellTags->"QuantumField"],
    Cell[2155469, 69530, 261, 6, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2156203, 69556, 228, 6, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2156762, 69577, 306, 7, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2157604, 69606, 349, 9, 70, "Input",
      CellTags->"QuantumField"],
    Cell[2158497, 69637, 273, 6, 70, "Input",
      CellTags->"QuantumField"]},
  "QuarkField"->{
    Cell[2158946, 69654, 58, 1, 39, "ObjectName",
      CellTags->"QuarkField"],
    Cell[2159029, 69659, 62, 1, 49, "Subsubsection",
      CellTags->"QuarkField"],
    Cell[2159094, 69662, 192, 5, 39, "Text",
      CellTags->"QuarkField"],
    Cell[2159289, 69669, 344, 14, 39, "Text",
      CellTags->"QuarkField"],
    Cell[2159670, 69688, 59, 1, 49, "Subsubsection",
      CellTags->"QuarkField"],
    Cell[2159754, 69693, 112, 2, 70, "Input",
      CellTags->"QuarkField"]},
  "QuarkGluonVertex"->{
    Cell[2160044, 69706, 70, 1, 39, "ObjectName",
      CellTags->"QuarkGluonVertex"],
    Cell[2160139, 69711, 68, 1, 49, "Subsubsection",
      CellTags->"QuarkGluonVertex"],
    Cell[2160210, 69714, 286, 9, 39, "Text",
      CellTags->"QuarkGluonVertex"],
    Cell[2160521, 69727, 160, 3, 56, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2161184, 69746, 209, 9, 35, "Text",
      CellTags->"QuarkGluonVertex"],
    Cell[2161430, 69760, 65, 1, 49, "Subsubsection",
      CellTags->"QuarkGluonVertex"],
    Cell[2161520, 69765, 185, 4, 70, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2162261, 69791, 245, 5, 70, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2163408, 69827, 245, 5, 70, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2164418, 69859, 245, 5, 70, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2165530, 69894, 373, 9, 70, "Input",
      CellTags->"QuarkGluonVertex"],
    Cell[2168925, 69985, 374, 9, 70, "Input",
      CellTags->"QuarkGluonVertex"]},
  "QuarkMass"->{
    Cell[2169879, 70018, 56, 1, 39, "ObjectName",
      CellTags->"QuarkMass"],
    Cell[2169960, 70023, 61, 1, 49, "Subsubsection",
      CellTags->"QuarkMass"],
    Cell[2170024, 70026, 77, 1, 39, "Text",
      CellTags->"QuarkMass"],
    Cell[2170104, 70029, 196, 9, 39, "Text",
      CellTags->"QuarkMass"]},
  "QuarkPropagator"->{
    Cell[2170349, 70044, 68, 1, 39, "ObjectName",
      CellTags->"QuarkPropagator"],
    Cell[2170442, 70049, 67, 1, 49, "Subsubsection",
      CellTags->"QuarkPropagator"],
    Cell[2170512, 70052, 177, 5, 39, "Text",
      CellTags->"QuarkPropagator"],
    Cell[2170714, 70061, 158, 3, 56, "Input",
      CellTags->"QuarkPropagator"],
    Cell[2171425, 70081, 364, 14, 35, "Text",
      CellTags->"QuarkPropagator"],
    Cell[2171826, 70100, 64, 1, 49, "Subsubsection",
      CellTags->"QuarkPropagator"],
    Cell[2171915, 70105, 152, 3, 70, "Input",
      CellTags->"QuarkPropagator"],
    Cell[2172520, 70126, 208, 5, 70, "Input",
      CellTags->"QuarkPropagator"],
    Cell[2173332, 70153, 189, 4, 70, "Input",
      CellTags->"QuarkPropagator"]},
  "ReduceGamma"->{
    Cell[2173998, 70177, 60, 1, 39, "ObjectName",
      CellTags->"ReduceGamma"],
    Cell[2174083, 70182, 63, 1, 49, "Subsubsection",
      CellTags->"ReduceGamma"],
    Cell[2174149, 70185, 197, 5, 39, "Text",
      CellTags->"ReduceGamma"],
    Cell[2174349, 70192, 192, 9, 39, "Text",
      CellTags->"ReduceGamma"]},
  "ReduceToScalars"->{
    Cell[2174590, 70207, 68, 1, 39, "ObjectName",
      CellTags->"ReduceToScalars"],
    Cell[2174683, 70212, 67, 1, 49, "Subsubsection",
      CellTags->"ReduceToScalars"],
    Cell[2174753, 70215, 211, 5, 56, "Text",
      CellTags->"ReduceToScalars"],
    Cell[2174967, 70222, 322, 14, 39, "Text",
      CellTags->"ReduceToScalars"]},
  "Rename"->{
    Cell[2175338, 70242, 50, 1, 39, "ObjectName",
      CellTags->"Rename"],
    Cell[2175413, 70247, 58, 1, 49, "Subsubsection",
      CellTags->"Rename"],
    Cell[2175474, 70250, 145, 4, 39, "Text",
      CellTags->"Rename"],
    Cell[2175622, 70256, 190, 9, 39, "Text",
      CellTags->"Rename"]},
  "RHI"->{
    Cell[2175861, 70271, 62, 1, 39, "ObjectName",
      CellTags->"RHI"],
    Cell[2175948, 70276, 55, 1, 49, "Subsubsection",
      CellTags->"RHI"],
    Cell[2176006, 70279, 612, 12, 156, "Text",
      CellTags->"RHI"],
    Cell[2176621, 70293, 181, 9, 39, "Text",
      CellTags->"RHI"],
    Cell[2176839, 70307, 52, 1, 49, "Subsubsection",
      CellTags->"RHI"],
    Cell[2176894, 70310, 95, 2, 56, "Input",
      CellTags->"RHI"]},
  "RHI2FC"->{
    Cell[2177038, 70318, 68, 1, 39, "ObjectName",
      CellTags->"RHI2FC"],
    Cell[2177131, 70323, 58, 1, 49, "Subsubsection",
      CellTags->"RHI2FC"],
    Cell[2177192, 70326, 615, 12, 156, "Text",
      CellTags->"RHI2FC"],
    Cell[2177810, 70340, 184, 9, 39, "Text",
      CellTags->"RHI2FC"],
    Cell[2178031, 70354, 55, 1, 49, "Subsubsection",
      CellTags->"RHI2FC"],
    Cell[2178089, 70357, 98, 2, 56, "Input",
      CellTags->"RHI2FC"]},
  "RHM"->{
    Cell[2178236, 70365, 62, 1, 39, "ObjectName",
      CellTags->"RHM"],
    Cell[2178323, 70370, 55, 1, 49, "Subsubsection",
      CellTags->"RHM"],
    Cell[2178381, 70373, 78, 1, 39, "Text",
      CellTags->"RHM"],
    Cell[2178462, 70376, 172, 9, 39, "Text",
      CellTags->"RHM"],
    Cell[2178671, 70390, 52, 1, 49, "Subsubsection",
      CellTags->"RHM"],
    Cell[2178726, 70393, 95, 2, 56, "Input",
      CellTags->"RHM"]},
  "RHO"->{
    Cell[2178870, 70401, 62, 1, 39, "ObjectName",
      CellTags->"RHO"],
    Cell[2178957, 70406, 55, 1, 49, "Subsubsection",
      CellTags->"RHO"],
    Cell[2179015, 70409, 270, 6, 56, "Text",
      CellTags->"RHO"],
    Cell[2179288, 70417, 172, 9, 39, "Text",
      CellTags->"RHO"],
    Cell[2179497, 70431, 52, 1, 49, "Subsubsection",
      CellTags->"RHO"],
    Cell[2179552, 70434, 95, 2, 56, "Input",
      CellTags->"RHO"]},
  "RHP"->{
    Cell[2179696, 70442, 62, 1, 39, "ObjectName",
      CellTags->"RHP"],
    Cell[2179783, 70447, 55, 1, 49, "Subsubsection",
      CellTags->"RHP"],
    Cell[2179841, 70450, 211, 5, 56, "Text",
      CellTags->"RHP"],
    Cell[2180055, 70457, 172, 9, 39, "Text",
      CellTags->"RHP"],
    Cell[2180264, 70471, 52, 1, 49, "Subsubsection",
      CellTags->"RHP"],
    Cell[2180319, 70474, 95, 2, 56, "Input",
      CellTags->"RHP"]},
  "RTL"->{
    Cell[2180463, 70482, 62, 1, 39, "ObjectName",
      CellTags->"RTL"],
    Cell[2180550, 70487, 55, 1, 49, "Subsubsection",
      CellTags->"RTL"],
    Cell[2180608, 70490, 191, 5, 39, "Text",
      CellTags->"RTL"],
    Cell[2180802, 70497, 289, 14, 39, "Text",
      CellTags->"RTL"],
    Cell[2181128, 70516, 52, 1, 49, "Subsubsection",
      CellTags->"RTL"],
    Cell[2181183, 70519, 95, 2, 56, "Input",
      CellTags->"RTL"]},
  "RightPartialD"->{
    Cell[2181327, 70527, 64, 1, 39, "ObjectName",
      CellTags->"RightPartialD"],
    Cell[2181416, 70532, 65, 1, 49, "Subsubsection",
      CellTags->"RightPartialD"],
    Cell[2181484, 70535, 275, 10, 40, "Text",
      CellTags->"RightPartialD"],
    Cell[2181762, 70547, 467, 19, 39, "Text",
      CellTags->"RightPartialD"],
    Cell[2182266, 70571, 62, 1, 49, "Subsubsection",
      CellTags->"RightPartialD"],
    Cell[2182353, 70576, 152, 3, 70, "Input",
      CellTags->"RightPartialD"],
    Cell[2182833, 70594, 302, 7, 70, "Input",
      CellTags->"RightPartialD"],
    Cell[2183534, 70618, 149, 3, 70, "Input",
      CellTags->"RightPartialD"],
    Cell[2184156, 70641, 143, 3, 70, "Input",
      CellTags->"RightPartialD"],
    Cell[2184650, 70657, 198, 5, 70, "Input",
      CellTags->"RightPartialD"]},
  "RussianTrick"->{
    Cell[2185093, 70674, 62, 1, 39, "ObjectName",
      CellTags->"RussianTrick"],
    Cell[2185180, 70679, 64, 1, 49, "Subsubsection",
      CellTags->"RussianTrick"],
    Cell[2185247, 70682, 319, 6, 56, "Text",
      CellTags->"RussianTrick"],
    Cell[2185569, 70690, 91, 1, 39, "Text",
      CellTags->"RussianTrick"],
    Cell[2185663, 70693, 349, 14, 39, "Text",
      CellTags->"RussianTrick"],
    Cell[2186049, 70712, 61, 1, 49, "Subsubsection",
      CellTags->"RussianTrick"],
    Cell[2186135, 70717, 469, 13, 70, "Input",
      CellTags->"RussianTrick"],
    Cell[2187263, 70757, 136, 3, 70, "Input",
      CellTags->"RussianTrick"],
    Cell[2189023, 70812, 209, 5, 70, "Input",
      CellTags->"RussianTrick"],
    Cell[2198423, 71068, 140, 3, 70, "Input",
      CellTags->"RussianTrick"],
    Cell[2202203, 71188, 167, 4, 70, "Input",
      CellTags->"RussianTrick"],
    Cell[2206519, 71308, 139, 3, 70, "Input",
      CellTags->"RussianTrick"]},
  "ScalarGluonVertex"->{
    Cell[2206707, 71317, 72, 1, 39, "ObjectName",
      CellTags->"ScalarGluonVertex"],
    Cell[2206804, 71322, 69, 1, 49, "Subsubsection",
      CellTags->"ScalarGluonVertex"],
    Cell[2206876, 71325, 431, 8, 89, "Text",
      CellTags->"ScalarGluonVertex"],
    Cell[2207332, 71337, 159, 3, 56, "Input",
      CellTags->"ScalarGluonVertex"],
    Cell[2207834, 71355, 66, 1, 49, "Subsubsection",
      CellTags->"ScalarGluonVertex"],
    Cell[2207925, 71360, 324, 8, 56, "Input",
      CellTags->"ScalarGluonVertex"]},
  "ScalarProduct"->{
    Cell[2208906, 71395, 64, 1, 39, "ObjectName",
      CellTags->"ScalarProduct"],
    Cell[2208995, 71400, 65, 1, 49, "Subsubsection",
      CellTags->"ScalarProduct"],
    Cell[2209063, 71403, 875, 21, 90, "Text",
      CellTags->"ScalarProduct"],
    Cell[2209941, 71426, 952, 38, 39, "Text",
      CellTags->"ScalarProduct"],
    Cell[2210930, 71469, 62, 1, 49, "Subsubsection",
      CellTags->"ScalarProduct"],
    Cell[2211017, 71474, 175, 4, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2211501, 71492, 228, 6, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2212202, 71517, 175, 4, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2212601, 71533, 148, 3, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2212973, 71548, 216, 5, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2213463, 71565, 278, 7, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2214077, 71586, 301, 8, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2214552, 71604, 358, 9, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2216125, 71653, 343, 9, 70, "Input",
      CellTags->"ScalarProduct"],
    Cell[2217661, 71700, 124, 2, 70, "Input",
      CellTags->"ScalarProduct"]},
  "ScaleMu"->{
    Cell[2235473, 72265, 52, 1, 39, "ObjectName",
      CellTags->"ScaleMu"],
    Cell[2235550, 72270, 59, 1, 49, "Subsubsection",
      CellTags->"ScaleMu"],
    Cell[2235612, 72273, 129, 4, 39, "Text",
      CellTags->"ScaleMu"],
    Cell[2235766, 72281, 104, 2, 56, "Input",
      CellTags->"ScaleMu"],
    Cell[2235873, 72285, 134, 3, 37, "Output",
      CellTags->"ScaleMu"]},
  "Schouten"->{
    Cell[2236068, 72295, 54, 1, 39, "ObjectName",
      CellTags->"Schouten"],
    Cell[2236147, 72300, 60, 1, 49, "Subsubsection",
      CellTags->"Schouten"],
    Cell[2236210, 72303, 463, 9, 98, "Text",
      CellTags->"Schouten"],
    Cell[2236676, 72314, 311, 13, 39, "Text",
      CellTags->"Schouten"],
    Cell[2237024, 72332, 57, 1, 49, "Subsubsection",
      CellTags->"Schouten"],
    Cell[2237106, 72337, 2467, 54, 169, "Input",
      CellTags->"Schouten"],
    Cell[2268586, 73198, 131, 3, 52, "Input",
      CellTags->"Schouten"]},
  "SD"->{
    Cell[2268998, 73216, 42, 1, 39, "ObjectName",
      CellTags->"SD"],
    Cell[2269065, 73221, 54, 1, 49, "Subsubsection",
      CellTags->"SD"],
    Cell[2269122, 73224, 271, 7, 56, "Text",
      CellTags->"SD"],
    Cell[2269396, 73233, 186, 9, 39, "Text",
      CellTags->"SD"],
    Cell[2269619, 73247, 51, 1, 49, "Subsubsection",
      CellTags->"SD"],
    Cell[2269695, 73252, 153, 4, 70, "Input",
      CellTags->"SD"],
    Cell[2270068, 73267, 162, 4, 70, "Input",
      CellTags->"SD"],
    Cell[2270508, 73283, 162, 4, 70, "Input",
      CellTags->"SD"]},
  "SelectFree"->{
    Cell[2270884, 73299, 58, 1, 39, "ObjectName",
      CellTags->"SelectFree"],
    Cell[2270967, 73304, 62, 1, 49, "Subsubsection",
      CellTags->"SelectFree"],
    Cell[2271032, 73307, 256, 5, 56, "Text",
      CellTags->"SelectFree"],
    Cell[2271291, 73314, 78, 1, 39, "Text",
      CellTags->"SelectFree"],
    Cell[2271372, 73317, 323, 14, 39, "Text",
      CellTags->"SelectFree"],
    Cell[2271732, 73336, 59, 1, 49, "Subsubsection",
      CellTags->"SelectFree"],
    Cell[2271816, 73341, 261, 7, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2272253, 73358, 203, 5, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2272605, 73372, 329, 8, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2273110, 73390, 169, 4, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2273428, 73403, 169, 4, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2273746, 73416, 169, 4, 70, "Input",
      CellTags->"SelectFree"],
    Cell[2274064, 73429, 203, 5, 70, "Input",
      CellTags->"SelectFree"]},
  "SelectGraphs"->{
    Cell[2274440, 73445, 62, 1, 39, "ObjectName",
      CellTags->"SelectGraphs"],
    Cell[2274527, 73450, 64, 1, 49, "Subsubsection",
      CellTags->"SelectGraphs"],
    Cell[2274594, 73453, 489, 9, 73, "Text",
      CellTags->"SelectGraphs"],
    Cell[2275086, 73464, 202, 9, 39, "Text",
      CellTags->"SelectGraphs"]},
  "SelectNotFree"->{
    Cell[2275337, 73479, 64, 1, 39, "ObjectName",
      CellTags->"SelectNotFree"],
    Cell[2275426, 73484, 65, 1, 49, "Subsubsection",
      CellTags->"SelectNotFree"],
    Cell[2275494, 73487, 144, 4, 39, "Text",
      CellTags->"SelectNotFree"],
    Cell[2275641, 73493, 273, 6, 56, "Text",
      CellTags->"SelectNotFree"],
    Cell[2275917, 73501, 84, 1, 39, "Text",
      CellTags->"SelectNotFree"],
    Cell[2276004, 73504, 317, 14, 39, "Text",
      CellTags->"SelectNotFree"],
    Cell[2276358, 73523, 62, 1, 49, "Subsubsection",
      CellTags->"SelectNotFree"],
    Cell[2276445, 73528, 250, 6, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2276905, 73545, 330, 8, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2277455, 73564, 175, 4, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2277779, 73577, 204, 5, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2278132, 73591, 175, 4, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2278456, 73604, 175, 4, 70, "Input",
      CellTags->"SelectNotFree"],
    Cell[2278780, 73617, 209, 5, 70, "Input",
      CellTags->"SelectNotFree"]},
  "SelectSplit"->{
    Cell[2279194, 73634, 60, 1, 39, "ObjectName",
      CellTags->"SelectSplit"],
    Cell[2279279, 73639, 63, 1, 49, "Subsubsection",
      CellTags->"SelectSplit"],
    Cell[2279345, 73642, 250, 5, 56, "Text",
      CellTags->"SelectSplit"],
    Cell[2279632, 73652, 60, 1, 49, "Subsubsection",
      CellTags->"SelectSplit"],
    Cell[2279717, 73657, 1004, 27, 57, "Input",
      CellTags->"SelectSplit"],
    Cell[2281398, 73709, 808, 22, 53, "Input",
      CellTags->"SelectSplit"]},
  "Select1"->{
    Cell[2282853, 73755, 52, 1, 39, "ObjectName",
      CellTags->"Select1"],
    Cell[2282930, 73760, 59, 1, 49, "Subsubsection",
      CellTags->"Select1"],
    Cell[2282992, 73763, 244, 5, 56, "Text",
      CellTags->"Select1"],
    Cell[2283239, 73770, 188, 9, 39, "Text",
      CellTags->"Select1"]},
  "Select2"->{
    Cell[2283476, 73785, 52, 1, 39, "ObjectName",
      CellTags->"Select2"],
    Cell[2283553, 73790, 59, 1, 49, "Subsubsection",
      CellTags->"Select2"],
    Cell[2283615, 73793, 246, 5, 56, "Text",
      CellTags->"Select2"],
    Cell[2283864, 73800, 188, 9, 39, "Text",
      CellTags->"Select2"]},
  "Series2"->{
    Cell[2284101, 73815, 52, 1, 39, "ObjectName",
      CellTags->"Series2"],
    Cell[2284178, 73820, 59, 1, 49, "Subsubsection",
      CellTags->"Series2"],
    Cell[2284240, 73823, 293, 6, 56, "Text",
      CellTags->"Series2"],
    Cell[2284558, 73833, 142, 3, 56, "Input",
      CellTags->"Series2"],
    Cell[2285287, 73857, 56, 1, 49, "Subsubsection",
      CellTags->"Series2"],
    Cell[2285368, 73862, 425, 12, 70, "Input",
      CellTags->"Series2"],
    Cell[2286196, 73891, 218, 6, 70, "Input",
      CellTags->"Series2"],
    Cell[2286714, 73911, 270, 7, 70, "Input",
      CellTags->"Series2"],
    Cell[2287408, 73935, 218, 6, 70, "Input",
      CellTags->"Series2"],
    Cell[2288229, 73964, 346, 8, 70, "Input",
      CellTags->"Series2"],
    Cell[2289996, 74015, 369, 10, 70, "Input",
      CellTags->"Series2"],
    Cell[2291675, 74065, 116, 4, 70, "Text",
      CellTags->"Series2"],
    Cell[2291816, 74073, 672, 18, 70, "Input",
      CellTags->"Series2"],
    Cell[2293226, 74118, 799, 20, 70, "Input",
      CellTags->"Series2"],
    Cell[2295469, 74185, 285, 6, 70, "Input",
      CellTags->"Series2"],
    Cell[2297592, 74246, 239, 6, 70, "Input",
      CellTags->"Series2"],
    Cell[2300642, 74328, 355, 10, 70, "Text",
      CellTags->"Series2"]},
  "Series3"->{
    Cell[2301046, 74344, 52, 1, 39, "ObjectName",
      CellTags->"Series3"],
    Cell[2301123, 74349, 59, 1, 49, "Subsubsection",
      CellTags->"Series3"],
    Cell[2301185, 74352, 270, 6, 56, "Text",
      CellTags->"Series3"],
    Cell[2301480, 74362, 140, 3, 56, "Input",
      CellTags->"Series3"],
    Cell[2301623, 74367, 316, 8, 37, "Output",
      CellTags->"Series3"],
    Cell[2301954, 74378, 188, 9, 35, "Text",
      CellTags->"Series3"],
    Cell[2302179, 74392, 56, 1, 49, "Subsubsection",
      CellTags->"Series3"],
    Cell[2302260, 74397, 423, 12, 56, "Input",
      CellTags->"Series3"],
    Cell[2302686, 74411, 384, 11, 53, "Output",
      CellTags->"Series3"],
    Cell[2303107, 74427, 259, 7, 52, "Input",
      CellTags->"Series3"],
    Cell[2303369, 74436, 155, 4, 53, "Output",
      CellTags->"Series3"]},
  "SetMandelstam"->{
    Cell[2303585, 74447, 64, 1, 39, "ObjectName",
      CellTags->"SetMandelstam"],
    Cell[2303674, 74452, 65, 1, 49, "Subsubsection",
      CellTags->"SetMandelstam"],
    Cell[2303742, 74455, 463, 9, 98, "Text",
      CellTags->"SetMandelstam"],
    Cell[2304230, 74468, 152, 3, 56, "Input",
      CellTags->"SetMandelstam"],
    Cell[2304385, 74473, 277, 8, 37, "Output",
      CellTags->"SetMandelstam"],
    Cell[2304677, 74484, 203, 9, 35, "Text",
      CellTags->"SetMandelstam"]},
  "SimplifyDeltaFunction"->{
    Cell[2304929, 74499, 80, 1, 39, "ObjectName",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2305034, 74504, 73, 1, 49, "Subsubsection",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2305110, 74507, 300, 6, 56, "Text",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2305413, 74515, 370, 14, 39, "Text",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2305820, 74534, 70, 1, 49, "Subsubsection",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2305915, 74539, 244, 6, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2306494, 74560, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2307184, 74584, 244, 6, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2307802, 74606, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2309026, 74644, 261, 6, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2309673, 74666, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2310135, 74683, 315, 8, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2310886, 74709, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2311378, 74727, 354, 9, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2312214, 74755, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2312559, 74768, 315, 8, 70, "Input",
      CellTags->"SimplifyDeltaFunction"],
    Cell[2313310, 74794, 196, 4, 70, "Input",
      CellTags->"SimplifyDeltaFunction"]},
  "SimplifyGTI"->{
    Cell[2313826, 74814, 78, 1, 39, "ObjectName",
      CellTags->"SimplifyGTI"],
    Cell[2313929, 74819, 63, 1, 49, "Subsubsection",
      CellTags->"SimplifyGTI"],
    Cell[2313995, 74822, 72, 1, 39, "Text",
      CellTags->"SimplifyGTI"],
    Cell[2314070, 74825, 180, 9, 39, "Text",
      CellTags->"SimplifyGTI"],
    Cell[2314287, 74839, 60, 1, 49, "Subsubsection",
      CellTags->"SimplifyGTI"],
    Cell[2314350, 74842, 100, 2, 56, "Input",
      CellTags->"SimplifyGTI"]},
  "SimplifyPolyLog"->{
    Cell[2314499, 74850, 68, 1, 39, "ObjectName",
      CellTags->"SimplifyPolyLog"],
    Cell[2314592, 74855, 67, 1, 49, "Subsubsection",
      CellTags->"SimplifyPolyLog"],
    Cell[2314662, 74858, 202, 5, 39, "Text",
      CellTags->"SimplifyPolyLog"],
    Cell[2314867, 74865, 196, 9, 39, "Text",
      CellTags->"SimplifyPolyLog"],
    Cell[2315100, 74879, 64, 1, 49, "Subsubsection",
      CellTags->"SimplifyPolyLog"],
    Cell[2315167, 74882, 265, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2315457, 74892, 238, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2316580, 74927, 207, 5, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2317358, 74953, 271, 7, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2318484, 74989, 238, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2319549, 75023, 338, 9, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2320763, 75062, 306, 8, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2321915, 75099, 348, 9, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2323260, 75144, 266, 7, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2324181, 75175, 238, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2324571, 75190, 238, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2326412, 75246, 338, 9, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2328666, 75313, 271, 7, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2330772, 75377, 338, 9, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2334896, 75495, 234, 6, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2335545, 75519, 203, 5, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2336052, 75538, 178, 4, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2336742, 75563, 178, 4, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2337363, 75586, 178, 4, 70, "Input",
      CellTags->"SimplifyPolyLog"],
    Cell[2337962, 75607, 144, 3, 70, "Input",
      CellTags->"SimplifyPolyLog"]},
  "Simplify2"->{
    Cell[2338155, 75616, 74, 1, 39, "ObjectName",
      CellTags->"Simplify2"],
    Cell[2338254, 75621, 61, 1, 49, "Subsubsection",
      CellTags->"Simplify2"],
    Cell[2338318, 75624, 82, 1, 39, "Text",
      CellTags->"Simplify2"],
    Cell[2338437, 75630, 58, 1, 49, "Subsubsection",
      CellTags->"Simplify2"],
    Cell[2338498, 75633, 99, 2, 56, "Input",
      CellTags->"Simplify2"]},
  "SmallDelta"->{
    Cell[2338646, 75641, 58, 1, 39, "ObjectName",
      CellTags->"SmallDelta"],
    Cell[2338729, 75646, 62, 1, 49, "Subsubsection",
      CellTags->"SmallDelta"],
    Cell[2338794, 75649, 88, 1, 39, "Text",
      CellTags->"SmallDelta"]},
  "SmallEpsilon"->{
    Cell[2338931, 75656, 62, 1, 39, "ObjectName",
      CellTags->"SmallEpsilon"],
    Cell[2339018, 75661, 64, 1, 49, "Subsubsection",
      CellTags->"SmallEpsilon"],
    Cell[2339085, 75664, 92, 1, 39, "Text",
      CellTags->"SmallEpsilon"]},
  "SmallVariable"->{
    Cell[2339226, 75671, 64, 1, 39, "ObjectName",
      CellTags->"SmallVariable"],
    Cell[2339315, 75676, 65, 1, 49, "Subsubsection",
      CellTags->"SmallVariable"],
    Cell[2339383, 75679, 284, 6, 56, "Text",
      CellTags->"SmallVariable"],
    Cell[2339670, 75687, 470, 19, 39, "Text",
      CellTags->"SmallVariable"]},
  "SmallVariables"->{
    Cell[2340189, 75712, 66, 1, 39, "ObjectName",
      CellTags->"SmallVariables"],
    Cell[2340280, 75717, 66, 1, 49, "Subsubsection",
      CellTags->"SmallVariables"],
    Cell[2340349, 75720, 228, 6, 56, "Text",
      CellTags->"SmallVariables"],
    Cell[2340580, 75728, 195, 9, 39, "Text",
      CellTags->"SmallVariables"]},
  "SMP"->{
    Cell[2340824, 75743, 44, 1, 39, "ObjectName",
      CellTags->"SMP"],
    Cell[2340893, 75748, 55, 1, 49, "Subsubsection",
      CellTags->"SMP"],
    Cell[2340951, 75751, 233, 5, 56, "Text",
      CellTags->"SMP"],
    Cell[2341209, 75760, 157, 4, 56, "Input",
      CellTags->"SMP"],
    Cell[2341369, 75766, 189, 5, 39, "Output",
      CellTags->"SMP"],
    Cell[2341573, 75774, 180, 8, 35, "Text",
      CellTags->"SMP"],
    Cell[2341790, 75787, 52, 1, 49, "Subsubsection",
      CellTags->"SMP"],
    Cell[2341867, 75792, 135, 3, 56, "Input",
      CellTags->"SMP"],
    Cell[2342005, 75797, 127, 3, 37, "Output",
      CellTags->"SMP"],
    Cell[2357337, 76401, 174, 8, 35, "Text",
      CellTags->"SMP"]},
  "SMVertex"->{
    Cell[2342193, 75807, 54, 1, 39, "ObjectName",
      CellTags->"SMVertex"],
    Cell[2342272, 75812, 60, 1, 49, "Subsubsection",
      CellTags->"SMVertex"],
    Cell[2342335, 75815, 305, 6, 56, "Text",
      CellTags->"SMVertex"],
    Cell[2342665, 75825, 138, 3, 56, "Input",
      CellTags->"SMVertex"],
    Cell[2343117, 75843, 57, 1, 49, "Subsubsection",
      CellTags->"SMVertex"],
    Cell[2343199, 75848, 262, 6, 56, "Input",
      CellTags->"SMVertex"]},
  "Smu"->{
    Cell[2345123, 75906, 62, 1, 39, "ObjectName",
      CellTags->"Smu"],
    Cell[2345210, 75911, 55, 1, 49, "Subsubsection",
      CellTags->"Smu"],
    Cell[2345268, 75914, 46, 1, 39, "Text",
      CellTags->"Smu"]},
  "Sn"->{
    Cell[2345363, 75921, 42, 1, 39, "ObjectName",
      CellTags->"Sn"],
    Cell[2345430, 75926, 54, 1, 49, "Subsubsection",
      CellTags->"Sn"],
    Cell[2345487, 75929, 343, 14, 39, "Text",
      CellTags->"Sn"]},
  "SO"->{
    Cell[2345879, 75949, 42, 1, 39, "ObjectName",
      CellTags->"SO"],
    Cell[2345946, 75954, 54, 1, 49, "Subsubsection",
      CellTags->"SO"],
    Cell[2346003, 75957, 176, 5, 39, "Text",
      CellTags->"SO"],
    Cell[2346182, 75964, 632, 28, 39, "Text",
      CellTags->"SO"],
    Cell[2346851, 75997, 51, 1, 49, "Subsubsection",
      CellTags->"SO"],
    Cell[2346927, 76002, 126, 3, 70, "Input",
      CellTags->"SO"],
    Cell[2347411, 76021, 153, 4, 70, "Input",
      CellTags->"SO"],
    Cell[2348073, 76046, 204, 6, 70, "Input",
      CellTags->"SO"]},
  "SOD"->{
    Cell[2348582, 76066, 44, 1, 39, "ObjectName",
      CellTags->"SOD"],
    Cell[2348651, 76071, 55, 1, 49, "Subsubsection",
      CellTags->"SOD"],
    Cell[2348709, 76074, 192, 5, 56, "Text",
      CellTags->"SOD"],
    Cell[2348904, 76081, 535, 24, 39, "Text",
      CellTags->"SOD"],
    Cell[2349476, 76110, 52, 1, 49, "Subsubsection",
      CellTags->"SOD"],
    Cell[2349553, 76115, 128, 3, 70, "Input",
      CellTags->"SOD"],
    Cell[2350039, 76134, 155, 4, 70, "Input",
      CellTags->"SOD"],
    Cell[2350703, 76159, 206, 6, 70, "Input",
      CellTags->"SOD"]},
  "Solve2"->{
    Cell[2351276, 76181, 50, 1, 39, "ObjectName",
      CellTags->"Solve2"],
    Cell[2351351, 76186, 58, 1, 49, "Subsubsection",
      CellTags->"Solve2"],
    Cell[2351412, 76189, 347, 7, 56, "Text",
      CellTags->"Solve2"],
    Cell[2351784, 76200, 140, 3, 56, "Input",
      CellTags->"Solve2"],
    Cell[2352274, 76218, 55, 1, 49, "Subsubsection",
      CellTags->"Solve2"],
    Cell[2352354, 76223, 491, 14, 70, "Input",
      CellTags->"Solve2"],
    Cell[2353312, 76254, 114, 4, 70, "Text",
      CellTags->"Solve2"],
    Cell[2353451, 76262, 190, 5, 70, "Input",
      CellTags->"Solve2"],
    Cell[2353878, 76279, 322, 8, 70, "Input",
      CellTags->"Solve2"],
    Cell[2354437, 76299, 549, 15, 70, "Input",
      CellTags->"Solve2"],
    Cell[2355397, 76331, 490, 14, 70, "Input",
      CellTags->"Solve2"]},
  "Solve3"->{
    Cell[2356454, 76368, 50, 1, 39, "ObjectName",
      CellTags->"Solve3"],
    Cell[2356529, 76373, 58, 1, 49, "Subsubsection",
      CellTags->"Solve3"],
    Cell[2356590, 76376, 272, 6, 56, "Text",
      CellTags->"Solve3"],
    Cell[2356887, 76386, 138, 3, 56, "Input",
      CellTags->"Solve3"],
    Cell[2357548, 76414, 55, 1, 49, "Subsubsection",
      CellTags->"Solve3"],
    Cell[2357628, 76419, 489, 14, 56, "Input",
      CellTags->"Solve3"]},
  "SP"->{
    Cell[2358628, 76454, 42, 1, 39, "ObjectName",
      CellTags->"SP"],
    Cell[2358695, 76459, 54, 1, 49, "Subsubsection",
      CellTags->"SP"],
    Cell[2358752, 76462, 353, 11, 56, "Text",
      CellTags->"SP"],
    Cell[2359108, 76475, 462, 19, 39, "Text",
      CellTags->"SP"],
    Cell[2359607, 76499, 51, 1, 49, "Subsubsection",
      CellTags->"SP"],
    Cell[2359683, 76504, 225, 6, 70, "Input",
      CellTags->"SP"],
    Cell[2360458, 76533, 237, 7, 70, "Input",
      CellTags->"SP"],
    Cell[2361186, 76559, 290, 8, 70, "Input",
      CellTags->"SP"],
    Cell[2362029, 76589, 205, 5, 70, "Input",
      CellTags->"SP"],
    Cell[2362804, 76617, 194, 5, 70, "Input",
      CellTags->"SP"],
    Cell[2363188, 76632, 235, 7, 70, "Input",
      CellTags->"SP"],
    Cell[2363697, 76651, 271, 8, 70, "Input",
      CellTags->"SP"]},
  "SPC"->{
    Cell[2364182, 76671, 44, 1, 39, "ObjectName",
      CellTags->"SPC"],
    Cell[2364251, 76676, 55, 1, 49, "Subsubsection",
      CellTags->"SPC"],
    Cell[2364309, 76679, 82, 1, 39, "Text",
      CellTags->"SPC"],
    Cell[2364394, 76682, 220, 9, 39, "Text",
      CellTags->"SPC"]},
  "SPD"->{
    Cell[2364663, 76697, 44, 1, 39, "ObjectName",
      CellTags->"SPD"],
    Cell[2364732, 76702, 55, 1, 49, "Subsubsection",
      CellTags->"SPD"],
    Cell[2364790, 76705, 372, 11, 56, "Text",
      CellTags->"SPD"],
    Cell[2365165, 76718, 565, 24, 39, "Text",
      CellTags->"SPD"],
    Cell[2365767, 76747, 52, 1, 49, "Subsubsection",
      CellTags->"SPD"],
    Cell[2365844, 76752, 228, 6, 70, "Input",
      CellTags->"SPD"],
    Cell[2366622, 76781, 239, 7, 70, "Input",
      CellTags->"SPD"],
    Cell[2367352, 76807, 292, 8, 70, "Input",
      CellTags->"SPD"],
    Cell[2368197, 76837, 207, 5, 70, "Input",
      CellTags->"SPD"],
    Cell[2368974, 76865, 196, 5, 70, "Input",
      CellTags->"SPD"],
    Cell[2369361, 76880, 237, 7, 70, "Input",
      CellTags->"SPD"],
    Cell[2369934, 76901, 273, 8, 70, "Input",
      CellTags->"SPD"],
    Cell[2370398, 76919, 332, 9, 70, "Input",
      CellTags->"SPD"]},
  "Spinor"->{
    Cell[2370945, 76940, 50, 1, 39, "ObjectName",
      CellTags->"Spinor"],
    Cell[2371020, 76945, 58, 1, 49, "Subsubsection",
      CellTags->"Spinor"],
    Cell[2371081, 76948, 1421, 49, 123, "Text",
      CellTags->"Spinor"],
    Cell[2372505, 76999, 829, 34, 39, "Text",
      CellTags->"Spinor"],
    Cell[2373371, 77038, 55, 1, 49, "Subsubsection",
      CellTags->"Spinor"],
    Cell[2373451, 77043, 134, 3, 56, "Input",
      CellTags->"Spinor"],
    Cell[2373588, 77048, 264, 7, 70, "Output",
      CellTags->"Spinor"],
    Cell[2373889, 77060, 161, 4, 52, "Input",
      CellTags->"Spinor"],
    Cell[2374053, 77066, 309, 7, 70, "Output",
      CellTags->"Spinor"],
    Cell[2374377, 77076, 147, 4, 35, "Text",
      CellTags->"Spinor"],
    Cell[2374549, 77084, 303, 9, 56, "Input",
      CellTags->"Spinor"],
    Cell[2374855, 77095, 155, 4, 50, "Output",
      CellTags->"Spinor"],
    Cell[2375047, 77104, 306, 8, 52, "Input",
      CellTags->"Spinor"],
    Cell[2375356, 77114, 393, 10, 37, "Output",
      CellTags->"Spinor"],
    Cell[2375786, 77129, 173, 4, 52, "Input",
      CellTags->"Spinor"],
    Cell[2375962, 77135, 234, 6, 70, "Output",
      CellTags->"Spinor"],
    Cell[2376211, 77144, 159, 4, 35, "Text",
      CellTags->"Spinor"],
    Cell[2376395, 77152, 259, 7, 56, "Input",
      CellTags->"Spinor"],
    Cell[2376657, 77161, 234, 6, 37, "Output",
      CellTags->"Spinor"],
    Cell[2376928, 77172, 202, 5, 52, "Input",
      CellTags->"Spinor"],
    Cell[2377133, 77179, 234, 6, 70, "Output",
      CellTags->"Spinor"],
    Cell[2377382, 77188, 78, 1, 35, "Text",
      CellTags->"Spinor"],
    Cell[2377485, 77193, 257, 7, 56, "Input",
      CellTags->"Spinor"],
    Cell[2377745, 77202, 234, 6, 70, "Output",
      CellTags->"Spinor"]},
  "SpinorCollect"->{
    Cell[2378040, 77215, 64, 1, 39, "ObjectName",
      CellTags->"SpinorCollect"],
    Cell[2378129, 77220, 65, 1, 49, "Subsubsection",
      CellTags->"SpinorCollect"],
    Cell[2378197, 77223, 194, 5, 39, "Text",
      CellTags->"SpinorCollect"],
    Cell[2378394, 77230, 323, 13, 39, "Text",
      CellTags->"SpinorCollect"]},
  "SpinorUBar"->{
    Cell[2378766, 77249, 58, 1, 39, "ObjectName",
      CellTags->"SpinorUBar"],
    Cell[2378849, 77254, 62, 1, 49, "Subsubsection",
      CellTags->"SpinorUBar"],
    Cell[2378914, 77257, 183, 7, 39, "Text",
      CellTags->"SpinorUBar"],
    Cell[2379100, 77266, 548, 24, 39, "Text",
      CellTags->"SpinorUBar"],
    Cell[2379685, 77295, 59, 1, 49, "Subsubsection",
      CellTags->"SpinorUBar"],
    Cell[2379747, 77298, 86, 1, 39, "Text",
      CellTags->"SpinorUBar"],
    Cell[2379858, 77303, 142, 3, 56, "Input",
      CellTags->"SpinorUBar"],
    Cell[2380318, 77320, 169, 4, 52, "Input",
      CellTags->"SpinorUBar"],
    Cell[2380850, 77339, 210, 5, 52, "Input",
      CellTags->"SpinorUBar"],
    Cell[2381258, 77354, 251, 7, 52, "Input",
      CellTags->"SpinorUBar"],
    Cell[2381761, 77373, 287, 8, 52, "Input",
      CellTags->"SpinorUBar"]},
  "SpinorU"->{
    Cell[2382324, 77395, 52, 1, 39, "ObjectName",
      CellTags->"SpinorU"],
    Cell[2382401, 77400, 59, 1, 49, "Subsubsection",
      CellTags->"SpinorU"],
    Cell[2382463, 77403, 80, 1, 39, "Text",
      CellTags->"SpinorU"],
    Cell[2382546, 77406, 554, 24, 39, "Text",
      CellTags->"SpinorU"],
    Cell[2383137, 77435, 56, 1, 49, "Subsubsection",
      CellTags->"SpinorU"],
    Cell[2383218, 77440, 136, 3, 56, "Input",
      CellTags->"SpinorU"],
    Cell[2383584, 77454, 163, 4, 52, "Input",
      CellTags->"SpinorU"],
    Cell[2384024, 77470, 204, 5, 52, "Input",
      CellTags->"SpinorU"],
    Cell[2384423, 77485, 245, 7, 52, "Input",
      CellTags->"SpinorU"],
    Cell[2384920, 77504, 279, 8, 52, "Input",
      CellTags->"SpinorU"]},
  "SpinorVBar"->{
    Cell[2385473, 77526, 58, 1, 39, "ObjectName",
      CellTags->"SpinorVBar"],
    Cell[2385556, 77531, 62, 1, 49, "Subsubsection",
      CellTags->"SpinorVBar"],
    Cell[2385621, 77534, 188, 7, 39, "Text",
      CellTags->"SpinorVBar"],
    Cell[2385812, 77543, 548, 24, 39, "Text",
      CellTags->"SpinorVBar"],
    Cell[2386397, 77572, 59, 1, 49, "Subsubsection",
      CellTags->"SpinorVBar"],
    Cell[2386481, 77577, 142, 3, 70, "Input",
      CellTags->"SpinorVBar"],
    Cell[2386941, 77594, 169, 4, 70, "Input",
      CellTags->"SpinorVBar"],
    Cell[2387473, 77613, 210, 5, 70, "Input",
      CellTags->"SpinorVBar"],
    Cell[2387881, 77628, 251, 7, 70, "Input",
      CellTags->"SpinorVBar"],
    Cell[2388410, 77648, 287, 8, 70, "Input",
      CellTags->"SpinorVBar"]},
  "SpinorV"->{
    Cell[2388999, 77671, 52, 1, 39, "ObjectName",
      CellTags->"SpinorV"],
    Cell[2389076, 77676, 59, 1, 49, "Subsubsection",
      CellTags->"SpinorV"],
    Cell[2389138, 77679, 80, 1, 39, "Text",
      CellTags->"SpinorV"],
    Cell[2389221, 77682, 554, 24, 39, "Text",
      CellTags->"SpinorV"],
    Cell[2389812, 77711, 56, 1, 49, "Subsubsection",
      CellTags->"SpinorV"],
    Cell[2389893, 77716, 136, 3, 70, "Input",
      CellTags->"SpinorV"],
    Cell[2390259, 77730, 163, 4, 70, "Input",
      CellTags->"SpinorV"],
    Cell[2390699, 77746, 204, 5, 70, "Input",
      CellTags->"SpinorV"],
    Cell[2391098, 77761, 245, 7, 70, "Input",
      CellTags->"SpinorV"],
    Cell[2391621, 77781, 281, 8, 70, "Input",
      CellTags->"SpinorV"]},
  "SpinPolarizationSum"->{
    Cell[2392204, 77804, 76, 1, 39, "ObjectName",
      CellTags->"SpinPolarizationSum"],
    Cell[2392305, 77809, 71, 1, 49, "Subsubsection",
      CellTags->"SpinPolarizationSum"],
    Cell[2392379, 77812, 165, 4, 39, "Text",
      CellTags->"SpinPolarizationSum"],
    Cell[2392547, 77818, 215, 8, 39, "Text",
      CellTags->"SpinPolarizationSum"]},
  "SPL"->{
    Cell[2392811, 77832, 44, 1, 39, "ObjectName",
      CellTags->"SPL"],
    Cell[2392880, 77837, 55, 1, 49, "Subsubsection",
      CellTags->"SPL"],
    Cell[2392938, 77840, 78, 1, 39, "Text",
      CellTags->"SPL"],
    Cell[2393019, 77843, 208, 9, 39, "Text",
      CellTags->"SPL"]},
  "SplittingFunction"->{
    Cell[2393276, 77858, 72, 1, 39, "ObjectName",
      CellTags->"SplittingFunction"],
    Cell[2393373, 77863, 69, 1, 49, "Subsubsection",
      CellTags->"SplittingFunction"],
    Cell[2393445, 77866, 231, 7, 39, "Text",
      CellTags->"SplittingFunction"],
    Cell[2393701, 77877, 162, 3, 56, "Input",
      CellTags->"SplittingFunction"],
    Cell[2394070, 77890, 231, 9, 35, "Text",
      CellTags->"SplittingFunction"],
    Cell[2394338, 77904, 66, 1, 49, "Subsubsection",
      CellTags->"SplittingFunction"],
    Cell[2394407, 77907, 66, 1, 70, "Text",
      CellTags->"SplittingFunction"],
    Cell[2394476, 77910, 264, 6, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2394743, 77918, 171, 4, 70, "Text",
      CellTags->"SplittingFunction"],
    Cell[2394939, 77926, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2395819, 77955, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2396362, 77975, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2396858, 77993, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2398115, 78031, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2399769, 78079, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2400704, 78109, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2401547, 78135, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2401708, 78140, 64, 1, 70, "Text",
      CellTags->"SplittingFunction"],
    Cell[2401775, 78143, 264, 6, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2402064, 78153, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2402944, 78182, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2403407, 78200, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2403862, 78217, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2405111, 78256, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2406900, 78308, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2407711, 78335, 159, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2408450, 78360, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2409188, 78385, 159, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2409927, 78410, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2411907, 78466, 159, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2413888, 78522, 159, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2414886, 78554, 160, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2422367, 78741, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2426314, 78845, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"],
    Cell[2431161, 78973, 158, 3, 70, "Input",
      CellTags->"SplittingFunction"]},
  "StandardMatrixElement"->{
    Cell[2437046, 79126, 80, 1, 39, "ObjectName",
      CellTags->"StandardMatrixElement"],
    Cell[2437151, 79131, 73, 1, 49, "Subsubsection",
      CellTags->"StandardMatrixElement"],
    Cell[2437227, 79134, 137, 4, 39, "Text",
      CellTags->"StandardMatrixElement"],
    Cell[2437367, 79140, 202, 9, 39, "Text",
      CellTags->"StandardMatrixElement"]},
  "SubContext"->{
    Cell[2437618, 79155, 58, 1, 39, "ObjectName",
      CellTags->"SubContext"],
    Cell[2437701, 79160, 62, 1, 49, "Subsubsection",
      CellTags->"SubContext"],
    Cell[2437766, 79163, 123, 4, 39, "Text",
      CellTags->"SubContext"],
    Cell[2437892, 79169, 302, 14, 39, "Text",
      CellTags->"SubContext"]},
  "SubLoop"->{
    Cell[2438243, 79189, 52, 1, 39, "ObjectName",
      CellTags->"SubLoop"],
    Cell[2438320, 79194, 59, 1, 49, "Subsubsection",
      CellTags->"SubLoop"],
    Cell[2438382, 79197, 153, 4, 39, "Text",
      CellTags->"SubLoop"],
    Cell[2438538, 79203, 191, 9, 39, "Text",
      CellTags->"SubLoop"]},
  "SumP"->{
    Cell[2438778, 79218, 46, 1, 39, "ObjectName",
      CellTags->"SumP"],
    Cell[2438849, 79223, 56, 1, 49, "Subsubsection",
      CellTags->"SumP"],
    Cell[2438908, 79226, 1002, 31, 44, "Text",
      CellTags->"SumP"],
    Cell[2439913, 79259, 284, 14, 39, "Text",
      CellTags->"SumP"],
    Cell[2440234, 79278, 53, 1, 49, "Subsubsection",
      CellTags->"SumP"],
    Cell[2440312, 79283, 186, 5, 70, "Input",
      CellTags->"SumP"],
    Cell[2440839, 79303, 186, 5, 70, "Input",
      CellTags->"SumP"],
    Cell[2441366, 79323, 157, 4, 70, "Input",
      CellTags->"SumP"],
    Cell[2441833, 79341, 157, 4, 70, "Input",
      CellTags->"SumP"],
    Cell[2442166, 79355, 384, 12, 70, "Input",
      CellTags->"SumP"],
    Cell[2442726, 79377, 229, 6, 70, "Input",
      CellTags->"SumP"],
    Cell[2443958, 79418, 156, 4, 70, "Input",
      CellTags->"SumP"]},
  "SumS"->{
    Cell[2444314, 79434, 46, 1, 39, "ObjectName",
      CellTags->"SumS"],
    Cell[2444385, 79439, 56, 1, 49, "Subsubsection",
      CellTags->"SumS"],
    Cell[2444444, 79442, 1713, 57, 59, "Text",
      CellTags->"SumS"],
    Cell[2446182, 79503, 136, 3, 56, "Input",
      CellTags->"SumS"],
    Cell[2446523, 79516, 284, 14, 35, "Text",
      CellTags->"SumS"],
    Cell[2446844, 79535, 53, 1, 49, "Subsubsection",
      CellTags->"SumS"],
    Cell[2446922, 79540, 186, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2447395, 79559, 186, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2447868, 79578, 181, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2448284, 79595, 210, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2448845, 79616, 239, 6, 70, "Input",
      CellTags->"SumS"],
    Cell[2449520, 79640, 235, 6, 70, "Input",
      CellTags->"SumS"],
    Cell[2449780, 79650, 186, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2450608, 79679, 236, 6, 70, "Input",
      CellTags->"SumS"],
    Cell[2450869, 79689, 157, 4, 70, "Input",
      CellTags->"SumS"],
    Cell[2451202, 79703, 215, 6, 70, "Input",
      CellTags->"SumS"],
    Cell[2451593, 79719, 196, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2452077, 79738, 178, 4, 70, "Input",
      CellTags->"SumS"],
    Cell[2452461, 79753, 181, 5, 70, "Input",
      CellTags->"SumS"],
    Cell[2452841, 79769, 157, 4, 70, "Input",
      CellTags->"SumS"]},
  "SumT"->{
    Cell[2453221, 79786, 46, 1, 39, "ObjectName",
      CellTags->"SumT"],
    Cell[2453292, 79791, 56, 1, 49, "Subsubsection",
      CellTags->"SumT"],
    Cell[2453351, 79794, 536, 15, 56, "Text",
      CellTags->"SumT"],
    Cell[2453890, 79811, 284, 14, 39, "Text",
      CellTags->"SumT"],
    Cell[2454211, 79830, 53, 1, 49, "Subsubsection",
      CellTags->"SumT"],
    Cell[2454289, 79835, 186, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2454793, 79855, 186, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2455297, 79875, 157, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2455701, 79892, 210, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2456397, 79918, 157, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2456753, 79933, 303, 9, 70, "Input",
      CellTags->"SumT"],
    Cell[2457255, 79953, 196, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2457770, 79973, 168, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2458258, 79990, 157, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2458614, 80005, 181, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2458994, 80021, 332, 10, 70, "Input",
      CellTags->"SumT"],
    Cell[2460265, 80064, 168, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2460664, 80080, 192, 5, 70, "Input",
      CellTags->"SumT"],
    Cell[2461176, 80098, 161, 4, 70, "Input",
      CellTags->"SumT"],
    Cell[2461930, 80125, 304, 9, 70, "Input",
      CellTags->"SumT"]},
  "SUND"->{
    Cell[2462851, 80159, 46, 1, 39, "ObjectName",
      CellTags->"SUND"],
    Cell[2462922, 80164, 56, 1, 49, "Subsubsection",
      CellTags->"SUND"],
    Cell[2462981, 80167, 246, 10, 39, "Text",
      CellTags->"SUND"],
    Cell[2463230, 80179, 425, 19, 39, "Text",
      CellTags->"SUND"],
    Cell[2463692, 80203, 53, 1, 49, "Subsubsection",
      CellTags->"SUND"],
    Cell[2463770, 80208, 167, 4, 56, "Input",
      CellTags->"SUND"],
    Cell[2464176, 80224, 254, 6, 52, "Input",
      CellTags->"SUND"],
    Cell[2466073, 80281, 167, 4, 52, "Input",
      CellTags->"SUND"],
    Cell[2466479, 80297, 167, 4, 52, "Input",
      CellTags->"SUND"],
    Cell[2466795, 80310, 320, 8, 52, "Input",
      CellTags->"SUND"],
    Cell[2467455, 80333, 460, 12, 52, "Input",
      CellTags->"SUND"],
    Cell[2468325, 80363, 460, 12, 52, "Input",
      CellTags->"SUND"],
    Cell[2469465, 80401, 215, 6, 52, "Input",
      CellTags->"SUND"],
    Cell[2469882, 80417, 249, 7, 52, "Input",
      CellTags->"SUND"],
    Cell[2470456, 80437, 287, 8, 52, "Input",
      CellTags->"SUND"],
    Cell[2470923, 80453, 132, 3, 52, "Input",
      CellTags->"SUND"]},
  "SUNDelta"->{
    Cell[2471104, 80462, 54, 1, 39, "ObjectName",
      CellTags->"SUNDelta"],
    Cell[2471183, 80467, 60, 1, 49, "Subsubsection",
      CellTags->"SUNDelta"],
    Cell[2471246, 80470, 178, 6, 39, "Text",
      CellTags->"SUNDelta"],
    Cell[2471427, 80478, 779, 33, 39, "Text",
      CellTags->"SUNDelta"],
    Cell[2472243, 80516, 57, 1, 49, "Subsubsection",
      CellTags->"SUNDelta"],
    Cell[2472325, 80521, 165, 4, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2472710, 80536, 316, 8, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2473458, 80563, 247, 6, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2474137, 80588, 206, 5, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2474539, 80603, 247, 7, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2475064, 80622, 283, 8, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2475537, 80640, 241, 7, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2476034, 80657, 259, 6, 70, "Text",
      CellTags->"SUNDelta"],
    Cell[2476318, 80667, 380, 10, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2477362, 80703, 138, 3, 70, "Input",
      CellTags->"SUNDelta"],
    Cell[2477981, 80723, 247, 7, 70, "Input",
      CellTags->"SUNDelta"]},
  "SUNDeltaContract"->{
    Cell[2478546, 80744, 70, 1, 39, "ObjectName",
      CellTags->"SUNDeltaContract"],
    Cell[2478641, 80749, 68, 1, 49, "Subsubsection",
      CellTags->"SUNDeltaContract"],
    Cell[2478712, 80752, 403, 9, 56, "Text",
      CellTags->"SUNDeltaContract"],
    Cell[2479118, 80763, 320, 14, 39, "Text",
      CellTags->"SUNDeltaContract"]},
  "SUNF"->{
    Cell[2479487, 80783, 46, 1, 39, "ObjectName",
      CellTags->"SUNF"],
    Cell[2479558, 80788, 56, 1, 49, "Subsubsection",
      CellTags->"SUNF"],
    Cell[2479617, 80791, 196, 6, 39, "Text",
      CellTags->"SUNF"],
    Cell[2479838, 80801, 136, 3, 56, "Input",
      CellTags->"SUNF"],
    Cell[2480181, 80814, 764, 34, 35, "Text",
      CellTags->"SUNF"],
    Cell[2480982, 80853, 53, 1, 49, "Subsubsection",
      CellTags->"SUNF"],
    Cell[2481060, 80858, 362, 10, 70, "Input",
      CellTags->"SUNF"],
    Cell[2481845, 80885, 131, 3, 70, "Input",
      CellTags->"SUNF"],
    Cell[2483124, 80928, 138, 3, 70, "Input",
      CellTags->"SUNF"],
    Cell[2483927, 80957, 167, 4, 70, "Input",
      CellTags->"SUNF"],
    Cell[2484333, 80973, 200, 5, 70, "Input",
      CellTags->"SUNF"],
    Cell[2484660, 80985, 375, 11, 70, "Text",
      CellTags->"SUNF"],
    Cell[2485060, 81000, 222, 5, 70, "Input",
      CellTags->"SUNF"],
    Cell[2487025, 81056, 320, 8, 70, "Input",
      CellTags->"SUNF"],
    Cell[2487852, 81085, 287, 7, 70, "Input",
      CellTags->"SUNF"],
    Cell[2489906, 81144, 215, 6, 70, "Input",
      CellTags->"SUNF"],
    Cell[2490323, 81160, 249, 7, 70, "Input",
      CellTags->"SUNF"],
    Cell[2490897, 81180, 287, 8, 70, "Input",
      CellTags->"SUNF"],
    Cell[2491386, 81198, 167, 4, 70, "Input",
      CellTags->"SUNF"],
    Cell[2491792, 81214, 199, 5, 70, "Input",
      CellTags->"SUNF"]},
  "SUNFJacobi"->{
    Cell[2492615, 81245, 58, 1, 39, "ObjectName",
      CellTags->"SUNFJacobi"],
    Cell[2492698, 81250, 62, 1, 49, "Subsubsection",
      CellTags->"SUNFJacobi"],
    Cell[2492763, 81253, 147, 4, 39, "Text",
      CellTags->"SUNFJacobi"],
    Cell[2492913, 81259, 305, 13, 39, "Text",
      CellTags->"SUNFJacobi"],
    Cell[2493255, 81277, 59, 1, 49, "Subsubsection",
      CellTags->"SUNFJacobi"],
    Cell[2493339, 81282, 462, 13, 56, "Input",
      CellTags->"SUNFJacobi"],
    Cell[2494832, 81333, 461, 13, 52, "Input",
      CellTags->"SUNFJacobi"]},
  "SUNIndex"->{
    Cell[2497376, 81416, 54, 1, 39, "ObjectName",
      CellTags->"SUNIndex"],
    Cell[2497455, 81421, 60, 1, 49, "Subsubsection",
      CellTags->"SUNIndex"],
    Cell[2497518, 81424, 141, 6, 39, "Text",
      CellTags->"SUNIndex"],
    Cell[2497662, 81432, 437, 18, 39, "Text",
      CellTags->"SUNIndex"],
    Cell[2498136, 81455, 57, 1, 49, "Subsubsection",
      CellTags->"SUNIndex"],
    Cell[2498218, 81460, 138, 3, 70, "Input",
      CellTags->"SUNIndex"],
    Cell[2498505, 81472, 138, 3, 70, "Input",
      CellTags->"SUNIndex"],
    Cell[2498812, 81484, 247, 7, 70, "Input",
      CellTags->"SUNIndex"]},
  "SUNIndexRename"->{
    Cell[2499361, 81505, 66, 1, 39, "ObjectName",
      CellTags->"SUNIndexRename"],
    Cell[2499452, 81510, 66, 1, 49, "Subsubsection",
      CellTags->"SUNIndexRename"],
    Cell[2499521, 81513, 164, 4, 39, "Text",
      CellTags->"SUNIndexRename"],
    Cell[2499688, 81519, 327, 14, 39, "Text",
      CellTags->"SUNIndexRename"]},
  "SUNN"->{
    Cell[2500064, 81539, 46, 1, 39, "ObjectName",
      CellTags->"SUNN"],
    Cell[2500135, 81544, 56, 1, 49, "Subsubsection",
      CellTags->"SUNN"],
    Cell[2500194, 81547, 216, 7, 39, "Text",
      CellTags->"SUNN"],
    Cell[2500413, 81556, 632, 29, 39, "Text",
      CellTags->"SUNN"],
    Cell[2501082, 81590, 53, 1, 49, "Subsubsection",
      CellTags->"SUNN"],
    Cell[2501160, 81595, 375, 9, 56, "Input",
      CellTags->"SUNN"]},
  "SUNNToCACF"->{
    Cell[2501761, 81617, 58, 1, 39, "ObjectName",
      CellTags->"SUNNToCACF"],
    Cell[2501844, 81622, 62, 1, 49, "Subsubsection",
      CellTags->"SUNNToCACF"],
    Cell[2501909, 81625, 190, 5, 39, "Text",
      CellTags->"SUNNToCACF"],
    Cell[2502102, 81632, 626, 29, 39, "Text",
      CellTags->"SUNNToCACF"],
    Cell[2502765, 81666, 59, 1, 49, "Subsubsection",
      CellTags->"SUNNToCACF"],
    Cell[2502849, 81671, 380, 9, 56, "Input",
      CellTags->"SUNNToCACF"]},
  "SUNSimplify"->{
    Cell[2503491, 81694, 60, 1, 39, "ObjectName",
      CellTags->"SUNSimplify"],
    Cell[2503576, 81699, 63, 1, 49, "Subsubsection",
      CellTags->"SUNSimplify"],
    Cell[2503642, 81702, 377, 7, 73, "Text",
      CellTags->"SUNSimplify"],
    Cell[2504044, 81713, 150, 3, 56, "Input",
      CellTags->"SUNSimplify"],
    Cell[2504779, 81733, 186, 9, 35, "Text",
      CellTags->"SUNSimplify"],
    Cell[2505002, 81747, 60, 1, 49, "Subsubsection",
      CellTags->"SUNSimplify"],
    Cell[2505087, 81752, 301, 8, 56, "Input",
      CellTags->"SUNSimplify"],
    Cell[2505714, 81774, 145, 3, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2506291, 81796, 231, 6, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2506856, 81818, 145, 3, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2507176, 81831, 220, 5, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2507660, 81849, 308, 8, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2508325, 81872, 145, 3, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2508977, 81896, 323, 8, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2509657, 81919, 145, 3, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2510054, 81934, 313, 8, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2510724, 81957, 145, 3, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2511376, 81981, 313, 8, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2512046, 82004, 217, 5, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2512412, 82018, 467, 12, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2513289, 82048, 472, 12, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2514387, 82084, 294, 7, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2516448, 82143, 294, 7, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2518463, 82201, 327, 8, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2519107, 82223, 533, 14, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2520245, 82258, 218, 5, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2521507, 82301, 280, 6, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2523827, 82373, 243, 6, 52, "Input",
      CellTags->"SUNSimplify"],
    Cell[2524095, 82383, 388, 11, 43, "Input",
      CellTags->"SUNSimplify"],
    Cell[2524486, 82396, 706, 21, 37, "Output",
      CellTags->"SUNSimplify"],
    Cell[2525229, 82422, 193, 5, 39, "Input",
      CellTags->"SUNSimplify"],
    Cell[2525425, 82429, 706, 21, 37, "Output",
      CellTags->"SUNSimplify"],
    Cell[2526168, 82455, 192, 5, 39, "Input",
      CellTags->"SUNSimplify"],
    Cell[2526363, 82462, 734, 22, 37, "Output",
      CellTags->"SUNSimplify"],
    Cell[2527112, 82487, 249, 6, 52, "Input",
      CellTags->"SUNSimplify"]},
  "SUNT"->{
    Cell[2527410, 82499, 46, 1, 39, "ObjectName",
      CellTags->"SUNT"],
    Cell[2527481, 82504, 56, 1, 49, "Subsubsection",
      CellTags->"SUNT"],
    Cell[2527540, 82507, 254, 10, 39, "Text",
      CellTags->"SUNT"],
    Cell[2527797, 82519, 737, 34, 39, "Text",
      CellTags->"SUNT"],
    Cell[2528571, 82558, 53, 1, 49, "Subsubsection",
      CellTags->"SUNT"],
    Cell[2528649, 82563, 130, 3, 70, "Input",
      CellTags->"SUNT"],
    Cell[2528979, 82576, 216, 7, 70, "Text",
      CellTags->"SUNT"],
    Cell[2529220, 82587, 247, 6, 70, "Input",
      CellTags->"SUNT"],
    Cell[2529890, 82612, 177, 4, 70, "Input",
      CellTags->"SUNT"],
    Cell[2530564, 82638, 290, 7, 70, "Input",
      CellTags->"SUNT"],
    Cell[2531227, 82662, 221, 5, 70, "Input",
      CellTags->"SUNT"],
    Cell[2531633, 82677, 211, 5, 70, "Input",
      CellTags->"SUNT"],
    Cell[2532376, 82704, 290, 7, 70, "Input",
      CellTags->"SUNT"],
    Cell[2533017, 82726, 491, 15, 70, "Text",
      CellTags->"SUNT"],
    Cell[2533533, 82745, 198, 5, 70, "Input",
      CellTags->"SUNT"],
    Cell[2534186, 82768, 210, 7, 70, "Text",
      CellTags->"SUNT"],
    Cell[2534421, 82779, 240, 6, 70, "Input",
      CellTags->"SUNT"],
    Cell[2535168, 82806, 248, 7, 70, "Input",
      CellTags->"SUNT"],
    Cell[2535752, 82827, 208, 6, 70, "Input",
      CellTags->"SUNT"],
    Cell[2536164, 82843, 242, 7, 70, "Input",
      CellTags->"SUNT"]},
  "SUNTrace"->{
    Cell[2536595, 82861, 54, 1, 39, "ObjectName",
      CellTags->"SUNTrace"],
    Cell[2536674, 82866, 60, 1, 49, "Subsubsection",
      CellTags->"SUNTrace"],
    Cell[2536737, 82869, 82, 1, 39, "Text",
      CellTags->"SUNTrace"],
    Cell[2536844, 82874, 144, 3, 56, "Input",
      CellTags->"SUNTrace"],
    Cell[2537195, 82887, 303, 14, 35, "Text",
      CellTags->"SUNTrace"],
    Cell[2537535, 82906, 57, 1, 49, "Subsubsection",
      CellTags->"SUNTrace"],
    Cell[2537617, 82911, 202, 5, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2538296, 82936, 212, 5, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2539284, 82970, 288, 7, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2540800, 83019, 222, 5, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2541962, 83058, 334, 8, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2548598, 83256, 214, 5, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2549752, 83295, 355, 9, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2567651, 83751, 214, 5, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2568969, 83795, 324, 8, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2569442, 83812, 324, 8, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2569915, 83829, 324, 8, 70, "Input",
      CellTags->"SUNTrace"],
    Cell[2570388, 83846, 324, 8, 70, "Input",
      CellTags->"SUNTrace"]},
  "SymbolicSum2"->{
    Cell[2571516, 83885, 62, 1, 39, "ObjectName",
      CellTags->"SymbolicSum2"],
    Cell[2571603, 83890, 64, 1, 49, "Subsubsection",
      CellTags->"SymbolicSum2"],
    Cell[2571670, 83893, 285, 6, 56, "Text",
      CellTags->"SymbolicSum2"]},
  "SymbolicSum3"->{
    Cell[2572004, 83905, 62, 1, 39, "ObjectName",
      CellTags->"SymbolicSum3"],
    Cell[2572091, 83910, 64, 1, 49, "Subsubsection",
      CellTags->"SymbolicSum3"],
    Cell[2572158, 83913, 293, 6, 56, "Text",
      CellTags->"SymbolicSum3"]},
  "Symmetrize"->{
    Cell[2572500, 83925, 58, 1, 39, "ObjectName",
      CellTags->"Symmetrize"],
    Cell[2572583, 83930, 62, 1, 49, "Subsubsection",
      CellTags->"Symmetrize"],
    Cell[2572648, 83933, 146, 4, 39, "Text",
      CellTags->"Symmetrize"],
    Cell[2572797, 83939, 205, 8, 39, "Text",
      CellTags->"Symmetrize"],
    Cell[2573039, 83952, 59, 1, 49, "Subsubsection",
      CellTags->"Symmetrize"],
    Cell[2573123, 83957, 294, 8, 70, "Input",
      CellTags->"Symmetrize"],
    Cell[2573833, 83983, 314, 8, 70, "Input",
      CellTags->"Symmetrize"]},
  "TBox"->{
    Cell[2574979, 84019, 62, 1, 39, "ObjectName",
      CellTags->{"TBox", "Tbox"}],
    Cell[2575066, 84024, 66, 1, 49, "Subsubsection",
      CellTags->{"TBox", "Tbox"}],
    Cell[2575135, 84027, 265, 6, 56, "Text",
      CellTags->{"TBox", "Tbox"}]},
  "Tbox"->{
    Cell[2574979, 84019, 62, 1, 39, "ObjectName",
      CellTags->{"TBox", "Tbox"}],
    Cell[2575066, 84024, 66, 1, 49, "Subsubsection",
      CellTags->{"TBox", "Tbox"}],
    Cell[2575135, 84027, 265, 6, 56, "Text",
      CellTags->{"TBox", "Tbox"}]},
  "Tdec"->{
    Cell[2575449, 84039, 46, 1, 39, "ObjectName",
      CellTags->"Tdec"],
    Cell[2575520, 84044, 56, 1, 49, "Subsubsection",
      CellTags->"Tdec"],
    Cell[2575579, 84047, 259, 6, 56, "Text",
      CellTags->"Tdec"],
    Cell[2575863, 84057, 136, 3, 56, "Input",
      CellTags->"Tdec"],
    Cell[2576541, 84076, 422, 19, 35, "Text",
      CellTags->"Tdec"],
    Cell[2577000, 84100, 53, 1, 49, "Subsubsection",
      CellTags->"Tdec"],
    Cell[2577056, 84103, 756, 23, 70, "Text",
      CellTags->"Tdec"],
    Cell[2577837, 84130, 250, 7, 70, "Input",
      CellTags->"Tdec"],
    Cell[2579029, 84169, 245, 7, 70, "Input",
      CellTags->"Tdec"],
    Cell[2579881, 84199, 923, 28, 70, "Text",
      CellTags->"Tdec"],
    Cell[2580807, 84229, 697, 19, 70, "Input",
      CellTags->"Tdec"],
    Cell[2581529, 84252, 245, 7, 70, "Input",
      CellTags->"Tdec"],
    Cell[2595977, 84623, 469, 13, 70, "Input",
      CellTags->"Tdec"],
    Cell[2597216, 84662, 153, 4, 70, "Input",
      CellTags->"Tdec"]},
  "TensorFunction"->{
    Cell[2597418, 84672, 66, 1, 39, "ObjectName",
      CellTags->"TensorFunction"],
    Cell[2597509, 84677, 66, 1, 49, "Subsubsection",
      CellTags->"TensorFunction"],
    Cell[2597578, 84680, 361, 7, 56, "Text",
      CellTags->"TensorFunction"],
    Cell[2597942, 84689, 204, 9, 39, "Text",
      CellTags->"TensorFunction"],
    Cell[2598183, 84703, 63, 1, 49, "Subsubsection",
      CellTags->"TensorFunction"],
    Cell[2598271, 84708, 217, 5, 56, "Input",
      CellTags->"TensorFunction"],
    Cell[2598491, 84715, 253, 6, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2598781, 84726, 144, 3, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2598928, 84731, 232, 5, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2599197, 84741, 241, 6, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2599441, 84749, 475, 14, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2599953, 84768, 144, 3, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2600100, 84773, 421, 10, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2600558, 84788, 277, 7, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2600838, 84797, 319, 8, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2601194, 84810, 277, 7, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2601474, 84819, 319, 8, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2601830, 84832, 144, 3, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2601977, 84837, 292, 7, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2602306, 84849, 146, 3, 52, "Input",
      CellTags->"TensorFunction"],
    Cell[2602455, 84854, 161, 4, 70, "Output",
      CellTags->"TensorFunction"],
    Cell[2602631, 84861, 186, 4, 52, "Input",
      CellTags->"TensorFunction"]},
  "Tf"->{
    Cell[2602866, 84871, 42, 1, 39, "ObjectName",
      CellTags->"Tf"],
    Cell[2602933, 84876, 54, 1, 49, "Subsubsection",
      CellTags->"Tf"],
    Cell[2602990, 84879, 231, 10, 40, "Text",
      CellTags->"Tf"],
    Cell[2603224, 84891, 446, 18, 39, "Text",
      CellTags->"Tf"]},
  "TFi"->{
    Cell[2603719, 84915, 44, 1, 39, "ObjectName",
      CellTags->"TFi"],
    Cell[2603788, 84920, 55, 1, 49, "Subsubsection",
      CellTags->"TFi"],
    Cell[2603846, 84923, 579, 10, 90, "Text",
      CellTags->"TFi"],
    Cell[2604428, 84935, 250, 8, 39, "Text",
      CellTags->"TFi"],
    Cell[2604681, 84945, 188, 5, 39, "Text",
      CellTags->"TFi"],
    Cell[2604872, 84952, 288, 13, 39, "Text",
      CellTags->"TFi"],
    Cell[2605197, 84970, 52, 1, 49, "Subsubsection",
      CellTags->"TFi"],
    Cell[2605274, 84975, 629, 17, 70, "Input",
      CellTags->"TFi"],
    Cell[2607089, 85033, 365, 10, 70, "Input",
      CellTags->"TFi"]},
  "ThreeVector"->{
    Cell[2608654, 85086, 60, 1, 39, "ObjectName",
      CellTags->"ThreeVector"],
    Cell[2608739, 85091, 63, 1, 49, "Subsubsection",
      CellTags->"ThreeVector"],
    Cell[2608805, 85094, 92, 1, 39, "Text",
      CellTags->"ThreeVector"],
    Cell[2608900, 85097, 326, 13, 39, "Text",
      CellTags->"ThreeVector"]},
  "TID"->{
    Cell[2609275, 85116, 44, 1, 39, "ObjectName",
      CellTags->"TID"],
    Cell[2609344, 85121, 55, 1, 49, "Subsubsection",
      CellTags->"TID"],
    Cell[2609402, 85124, 81, 1, 39, "Text",
      CellTags->"TID"],
    Cell[2609508, 85129, 134, 3, 56, "Input",
      CellTags->"TID"],
    Cell[2610390, 85151, 316, 14, 35, "Text",
      CellTags->"TID"],
    Cell[2610743, 85170, 52, 1, 49, "Subsubsection",
      CellTags->"TID"],
    Cell[2610820, 85175, 458, 12, 56, "Input",
      CellTags->"TID"],
    Cell[2612877, 85239, 224, 6, 52, "Input",
      CellTags->"TID"],
    Cell[2631762, 85680, 332, 9, 52, "Input",
      CellTags->"TID"],
    Cell[2633278, 85730, 155, 4, 52, "Input",
      CellTags->"TID"],
    Cell[2635891, 85810, 236, 6, 52, "Input",
      CellTags->"TID"]},
  "TIDL"->{
    Cell[2637189, 85854, 46, 1, 39, "ObjectName",
      CellTags->"TIDL"],
    Cell[2637260, 85859, 56, 1, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2637319, 85862, 88, 1, 39, "Text",
      CellTags->"TIDL"],
    Cell[2637410, 85865, 166, 8, 39, "Text",
      CellTags->"TIDL"],
    Cell[2637591, 85876, 53, 1, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2637669, 85881, 171, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2637843, 85888, 507, 14, 70, "Text",
      CellTags->"TIDL"],
    Cell[2638375, 85906, 250, 7, 70, "Input",
      CellTags->"TIDL"],
    Cell[2639319, 85941, 176, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2639520, 85950, 390, 11, 70, "Input",
      CellTags->"TIDL"],
    Cell[2642929, 86051, 171, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2643125, 86060, 345, 10, 70, "Input",
      CellTags->"TIDL"],
    Cell[2647829, 86196, 176, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2648030, 86205, 485, 14, 70, "Input",
      CellTags->"TIDL"],
    Cell[2690041, 87177, 182, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2690226, 87184, 548, 15, 70, "Input",
      CellTags->"TIDL"],
    Cell[2690811, 87204, 190, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2691004, 87211, 637, 17, 70, "Input",
      CellTags->"TIDL"],
    Cell[2691678, 87233, 171, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2691874, 87242, 387, 11, 70, "Input",
      CellTags->"TIDL"],
    Cell[2717579, 87860, 176, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2717758, 87867, 473, 14, 70, "Input",
      CellTags->"TIDL"],
    Cell[2718268, 87886, 182, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2718453, 87893, 560, 16, 70, "Input",
      CellTags->"TIDL"],
    Cell[2719050, 87914, 190, 5, 49, "Subsubsection",
      CellTags->"TIDL"],
    Cell[2719243, 87921, 664, 18, 70, "Input",
      CellTags->"TIDL"],
    Cell[2719932, 87943, 287, 7, 70, "Input",
      CellTags->"TIDL"]},
  "TimedIntegrate"->{
    Cell[2720468, 87964, 66, 1, 39, "ObjectName",
      CellTags->"TimedIntegrate"],
    Cell[2720559, 87969, 66, 1, 49, "Subsubsection",
      CellTags->"TimedIntegrate"],
    Cell[2720628, 87972, 226, 5, 56, "Text",
      CellTags->"TimedIntegrate"],
    Cell[2720879, 87981, 154, 3, 56, "Input",
      CellTags->"TimedIntegrate"],
    Cell[2721036, 87986, 511, 13, 37, "Output",
      CellTags->"TimedIntegrate"],
    Cell[2721596, 88005, 63, 1, 49, "Subsubsection",
      CellTags->"TimedIntegrate"],
    Cell[2721662, 88008, 74, 1, 25, "Text",
      CellTags->"TimedIntegrate"],
    Cell[2721761, 88013, 362, 9, 56, "Input",
      CellTags->"TimedIntegrate"],
    Cell[2722126, 88024, 159, 4, 37, "Output",
      CellTags->"TimedIntegrate"],
    Cell[2722300, 88031, 60, 1, 21, "Text",
      CellTags->"TimedIntegrate"],
    Cell[2722385, 88036, 458, 11, 56, "Input",
      CellTags->"TimedIntegrate"],
    Cell[2722846, 88049, 555, 15, 39, "Output",
      CellTags->"TimedIntegrate"]},
  "TLI"->{
    Cell[2723462, 88071, 62, 1, 39, "ObjectName",
      CellTags->"TLI"],
    Cell[2723549, 88076, 55, 1, 49, "Subsubsection",
      CellTags->"TLI"],
    Cell[2723607, 88079, 617, 12, 131, "Text",
      CellTags->"TLI"],
    Cell[2724249, 88095, 132, 3, 56, "Input",
      CellTags->"TLI"],
    Cell[2724661, 88110, 181, 9, 35, "Text",
      CellTags->"TLI"],
    Cell[2724879, 88124, 52, 1, 49, "Subsubsection",
      CellTags->"TLI"],
    Cell[2724934, 88127, 95, 2, 56, "Input",
      CellTags->"TLI"]},
  "TLIFP"->{
    Cell[2725078, 88135, 66, 1, 39, "ObjectName",
      CellTags->"TLIFP"],
    Cell[2725169, 88140, 57, 1, 49, "Subsubsection",
      CellTags->"TLIFP"],
    Cell[2725229, 88143, 94, 1, 39, "Text",
      CellTags->"TLIFP"],
    Cell[2725348, 88148, 136, 3, 56, "Input",
      CellTags->"TLIFP"],
    Cell[2725996, 88168, 174, 9, 35, "Text",
      CellTags->"TLIFP"],
    Cell[2726207, 88182, 54, 1, 49, "Subsubsection",
      CellTags->"TLIFP"],
    Cell[2726264, 88185, 97, 2, 56, "Input",
      CellTags->"TLIFP"]},
  "TLIHYP"->{
    Cell[2726410, 88193, 68, 1, 39, "ObjectName",
      CellTags->"TLIHYP"],
    Cell[2726503, 88198, 58, 1, 49, "Subsubsection",
      CellTags->"TLIHYP"],
    Cell[2726564, 88201, 137, 4, 39, "Text",
      CellTags->"TLIHYP"],
    Cell[2726726, 88209, 137, 3, 56, "Input",
      CellTags->"TLIHYP"],
    Cell[2727151, 88224, 175, 9, 35, "Text",
      CellTags->"TLIHYP"],
    Cell[2727363, 88238, 55, 1, 49, "Subsubsection",
      CellTags->"TLIHYP"],
    Cell[2727421, 88241, 98, 2, 56, "Input",
      CellTags->"TLIHYP"]},
  "TLI2"->{
    Cell[2727568, 88249, 46, 1, 39, "ObjectName",
      CellTags->"TLI2"],
    Cell[2727639, 88254, 56, 1, 49, "Subsubsection",
      CellTags->"TLI2"],
    Cell[2727698, 88257, 84, 1, 39, "Text",
      CellTags->"TLI2"],
    Cell[2727785, 88260, 173, 9, 39, "Text",
      CellTags->"TLI2"]},
  "TLI2FC"->{
    Cell[2728007, 88275, 68, 1, 39, "ObjectName",
      CellTags->"TLI2FC"],
    Cell[2728100, 88280, 58, 1, 49, "Subsubsection",
      CellTags->"TLI2FC"],
    Cell[2728161, 88283, 112, 4, 39, "Text",
      CellTags->"TLI2FC"],
    Cell[2728298, 88291, 137, 3, 56, "Input",
      CellTags->"TLI2FC"],
    Cell[2728857, 88309, 175, 9, 35, "Text",
      CellTags->"TLI2FC"],
    Cell[2729069, 88323, 55, 1, 49, "Subsubsection",
      CellTags->"TLI2FC"],
    Cell[2729127, 88326, 98, 2, 56, "Input",
      CellTags->"TLI2FC"]},
  "ToDistribution"->{
    Cell[2729274, 88334, 66, 1, 39, "ObjectName",
      CellTags->"ToDistribution"],
    Cell[2729365, 88339, 66, 1, 49, "Subsubsection",
      CellTags->"ToDistribution"],
    Cell[2729434, 88342, 427, 8, 73, "Text",
      CellTags->"ToDistribution"],
    Cell[2729886, 88354, 156, 3, 56, "Input",
      CellTags->"ToDistribution"],
    Cell[2730260, 88367, 222, 9, 35, "Text",
      CellTags->"ToDistribution"],
    Cell[2730519, 88381, 63, 1, 49, "Subsubsection",
      CellTags->"ToDistribution"],
    Cell[2730607, 88386, 258, 6, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2731113, 88404, 347, 10, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2733164, 88468, 245, 6, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2733649, 88486, 347, 10, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2735718, 88552, 563, 14, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2736749, 88585, 146, 3, 70, "Input",
      CellTags->"ToDistribution"],
    Cell[2737308, 88604, 513, 14, 70, "Input",
      CellTags->"ToDistribution"]},
  "ToHypergeometric"->{
    Cell[2738313, 88639, 70, 1, 39, "ObjectName",
      CellTags->"ToHypergeometric"],
    Cell[2738408, 88644, 68, 1, 49, "Subsubsection",
      CellTags->"ToHypergeometric"],
    Cell[2738479, 88647, 2065, 67, 56, "Text",
      CellTags->"ToHypergeometric"],
    Cell[2740547, 88716, 512, 19, 39, "Text",
      CellTags->"ToHypergeometric"],
    Cell[2741096, 88740, 65, 1, 49, "Subsubsection",
      CellTags->"ToHypergeometric"],
    Cell[2741186, 88745, 496, 14, 70, "Input",
      CellTags->"ToHypergeometric"],
    Cell[2743922, 88821, 603, 16, 70, "Input",
      CellTags->"ToHypergeometric"],
    Cell[2746513, 88893, 496, 14, 70, "Input",
      CellTags->"ToHypergeometric"],
    Cell[2749334, 88971, 603, 16, 70, "Input",
      CellTags->"ToHypergeometric"]},
  "ToLarin"->{
    Cell[2752060, 89048, 52, 1, 39, "ObjectName",
      CellTags->"ToLarin"],
    Cell[2752137, 89053, 59, 1, 49, "Subsubsection",
      CellTags->"ToLarin"],
    Cell[2752199, 89056, 693, 20, 39, "Text",
      CellTags->"ToLarin"],
    Cell[2752895, 89078, 176, 9, 39, "Text",
      CellTags->"ToLarin"],
    Cell[2753108, 89092, 56, 1, 49, "Subsubsection",
      CellTags->"ToLarin"],
    Cell[2753189, 89097, 269, 6, 70, "Input",
      CellTags->"ToLarin"],
    Cell[2753975, 89124, 136, 3, 70, "Input",
      CellTags->"ToLarin"]},
  "ToTFi"->{
    Cell[2755650, 89180, 48, 1, 39, "ObjectName",
      CellTags->"ToTFi"],
    Cell[2755723, 89185, 57, 1, 49, "Subsubsection",
      CellTags->"ToTFi"],
    Cell[2755783, 89188, 275, 6, 56, "Text",
      CellTags->"ToTFi"],
    Cell[2756061, 89196, 284, 13, 39, "Text",
      CellTags->"ToTFi"],
    Cell[2756382, 89214, 54, 1, 49, "Subsubsection",
      CellTags->"ToTFi"],
    Cell[2756461, 89219, 404, 11, 70, "Input",
      CellTags->"ToTFi"],
    Cell[2760058, 89313, 181, 4, 70, "Input",
      CellTags->"ToTFi"],
    Cell[2761413, 89358, 135, 3, 70, "Input",
      CellTags->"ToTFi"],
    Cell[2762246, 89385, 574, 15, 70, "Input",
      CellTags->"ToTFi"],
    Cell[2764949, 89466, 132, 3, 70, "Input",
      CellTags->"ToTFi"],
    Cell[2766298, 89511, 135, 3, 70, "Input",
      CellTags->"ToTFi"]},
  "Tr"->{
    Cell[2767271, 89543, 42, 1, 39, "ObjectName",
      CellTags->"Tr"],
    Cell[2767338, 89548, 54, 1, 49, "Subsubsection",
      CellTags->"Tr"],
    Cell[2767395, 89551, 235, 7, 39, "Text",
      CellTags->"Tr"],
    Cell[2767655, 89562, 132, 3, 56, "Input",
      CellTags->"Tr"],
    Cell[2767790, 89567, 764, 17, 70, "Output",
      CellTags->"Tr"],
    Cell[2768569, 89587, 578, 23, 35, "Text",
      CellTags->"Tr"],
    Cell[2769184, 89615, 51, 1, 49, "Subsubsection",
      CellTags->"Tr"],
    Cell[2769260, 89620, 157, 4, 56, "Input",
      CellTags->"Tr"],
    Cell[2769420, 89626, 232, 6, 37, "Output",
      CellTags->"Tr"],
    Cell[2769689, 89637, 122, 3, 52, "Input",
      CellTags->"Tr"],
    Cell[2769814, 89642, 235, 7, 37, "Output",
      CellTags->"Tr"],
    Cell[2770086, 89654, 449, 12, 52, "Input",
      CellTags->"Tr"],
    Cell[2770538, 89668, 1768, 51, 39, "Output",
      CellTags->"Tr"],
    Cell[2772343, 89724, 252, 7, 52, "Input",
      CellTags->"Tr"],
    Cell[2772598, 89733, 789, 23, 37, "Output",
      CellTags->"Tr"],
    Cell[2773424, 89761, 284, 7, 52, "Input",
      CellTags->"Tr"],
    Cell[2773711, 89770, 13052, 322, 56, "Output",
      CellTags->"Tr"],
    Cell[2786800, 90097, 95, 2, 52, "Input",
      CellTags->"Tr"],
    Cell[2786898, 90101, 126, 3, 37, "Output",
      CellTags->"Tr"],
    Cell[2787061, 90109, 115, 3, 52, "Input",
      CellTags->"Tr"],
    Cell[2787179, 90114, 368, 7, 75, "Print",
      CellTags->"Tr"],
    Cell[2787562, 90124, 167, 4, 39, "Text",
      CellTags->"Tr"],
    Cell[2787754, 90132, 124, 3, 56, "Input",
      CellTags->"Tr"],
    Cell[2787881, 90137, 127, 3, 37, "Output",
      CellTags->"Tr"],
    Cell[2788045, 90145, 508, 13, 52, "Input",
      CellTags->"Tr"],
    Cell[2788556, 90160, 6804, 170, 37, "Output",
      CellTags->"Tr"],
    Cell[2795397, 90335, 205, 5, 52, "Input",
      CellTags->"Tr"],
    Cell[2795605, 90342, 1629, 46, 37, "Output",
      CellTags->"Tr"],
    Cell[2797271, 90393, 537, 14, 52, "Input",
      CellTags->"Tr"],
    Cell[2797811, 90409, 485, 14, 39, "Output",
      CellTags->"Tr"],
    Cell[2798333, 90428, 269, 7, 52, "Input",
      CellTags->"Tr"],
    Cell[2798605, 90437, 241, 7, 37, "Output",
      CellTags->"Tr"],
    Cell[2798883, 90449, 129, 3, 52, "Input",
      CellTags->"Tr"],
    Cell[2799015, 90454, 207, 5, 37, "Output",
      CellTags->"Tr"],
    Cell[2799259, 90464, 410, 10, 52, "Input",
      CellTags->"Tr"],
    Cell[2799672, 90476, 251, 7, 37, "Output",
      CellTags->"Tr"],
    Cell[2799960, 90488, 129, 3, 52, "Input",
      CellTags->"Tr"],
    Cell[2800092, 90493, 217, 5, 37, "Output",
      CellTags->"Tr"],
    Cell[2800346, 90503, 407, 11, 52, "Input",
      CellTags->"Tr"],
    Cell[2801325, 90536, 253, 6, 52, "Input",
      CellTags->"Tr"],
    Cell[2801874, 90556, 252, 6, 52, "Input",
      CellTags->"Tr"],
    Cell[2802460, 90577, 252, 6, 52, "Input",
      CellTags->"Tr"],
    Cell[2802857, 90592, 251, 6, 52, "Input",
      CellTags->"Tr"],
    Cell[2803286, 90607, 148, 4, 52, "Input",
      CellTags->"Tr"]},
  "TraceDimension"->{
    Cell[2803483, 90617, 66, 1, 39, "ObjectName",
      CellTags->"TraceDimension"],
    Cell[2803574, 90622, 66, 1, 49, "Subsubsection",
      CellTags->"TraceDimension"],
    Cell[2803643, 90625, 143, 4, 39, "Text",
      CellTags->"TraceDimension"],
    Cell[2803789, 90631, 213, 9, 39, "Text",
      CellTags->"TraceDimension"]},
  "TraceOfOne"->{
    Cell[2804051, 90646, 58, 1, 39, "ObjectName",
      CellTags->"TraceOfOne"],
    Cell[2804134, 90651, 62, 1, 49, "Subsubsection",
      CellTags->"TraceOfOne"],
    Cell[2804199, 90654, 150, 4, 39, "Text",
      CellTags->"TraceOfOne"],
    Cell[2804352, 90660, 295, 13, 39, "Text",
      CellTags->"TraceOfOne"],
    Cell[2804684, 90678, 59, 1, 49, "Subsubsection",
      CellTags->"TraceOfOne"],
    Cell[2804768, 90683, 317, 7, 56, "Input",
      CellTags->"TraceOfOne"],
    Cell[2805233, 90699, 316, 7, 52, "Input",
      CellTags->"TraceOfOne"]},
  "Trick"->{
    Cell[2805776, 90719, 48, 1, 39, "ObjectName",
      CellTags->"Trick"],
    Cell[2805849, 90724, 57, 1, 49, "Subsubsection",
      CellTags->"Trick"],
    Cell[2805909, 90727, 177, 5, 39, "Text",
      CellTags->"Trick"],
    Cell[2806089, 90734, 678, 29, 39, "Text",
      CellTags->"Trick"],
    Cell[2806804, 90768, 63, 1, 50, "Subsubsection",
      CellTags->"Trick"],
    Cell[2806870, 90771, 420, 15, 70, "Text",
      CellTags->"Trick"],
    Cell[2807315, 90790, 428, 11, 70, "Input",
      CellTags->"Trick"],
    Cell[2808112, 90818, 397, 11, 70, "Input",
      CellTags->"Trick"],
    Cell[2809168, 90854, 132, 3, 70, "Input",
      CellTags->"Trick"],
    Cell[2810117, 90886, 269, 6, 70, "Input",
      CellTags->"Trick"],
    Cell[2810917, 90913, 165, 4, 70, "Input",
      CellTags->"Trick"]},
  "TrickIntegrate"->{
    Cell[2811380, 90931, 66, 1, 39, "ObjectName",
      CellTags->"TrickIntegrate"],
    Cell[2811471, 90936, 66, 1, 49, "Subsubsection",
      CellTags->"TrickIntegrate"],
    Cell[2811540, 90939, 484, 8, 73, "Text",
      CellTags->"TrickIntegrate"],
    Cell[2812061, 90952, 63, 1, 49, "Subsubsection",
      CellTags->"TrickIntegrate"],
    Cell[2812149, 90957, 450, 12, 56, "Input",
      CellTags->"TrickIntegrate"]},
  "TrickMandelstam"->{
    Cell[2813926, 91011, 68, 1, 39, "ObjectName",
      CellTags->"TrickMandelstam"],
    Cell[2814019, 91016, 67, 1, 49, "Subsubsection",
      CellTags->"TrickMandelstam"],
    Cell[2814089, 91019, 1509, 56, 56, "Text",
      CellTags->"TrickMandelstam"],
    Cell[2815635, 91080, 64, 1, 49, "Subsubsection",
      CellTags->"TrickMandelstam"],
    Cell[2815724, 91085, 664, 18, 57, "Input",
      CellTags->"TrickMandelstam"],
    Cell[2816707, 91118, 721, 18, 52, "Input",
      CellTags->"TrickMandelstam"]},
  "Tr2"->{
    Cell[2817769, 91153, 44, 1, 39, "ObjectName",
      CellTags->"Tr2"],
    Cell[2817838, 91158, 55, 1, 49, "Subsubsection",
      CellTags->"Tr2"],
    Cell[2817896, 91161, 350, 7, 56, "Text",
      CellTags->"Tr2"],
    Cell[2818271, 91172, 131, 3, 56, "Input",
      CellTags->"Tr2"],
    Cell[2818405, 91177, 206, 6, 37, "Output",
      CellTags->"Tr2"],
    Cell[2818626, 91186, 645, 28, 35, "Text",
      CellTags->"Tr2"],
    Cell[2819308, 91219, 52, 1, 49, "Subsubsection",
      CellTags->"Tr2"],
    Cell[2819385, 91224, 163, 4, 56, "Input",
      CellTags->"Tr2"],
    Cell[2819551, 91230, 153, 4, 37, "Output",
      CellTags->"Tr2"],
    Cell[2819741, 91239, 125, 3, 56, "Input",
      CellTags->"Tr2"],
    Cell[2819869, 91244, 153, 4, 37, "Output",
      CellTags->"Tr2"],
    Cell[2820059, 91253, 276, 8, 56, "Input",
      CellTags->"Tr2"],
    Cell[2820338, 91263, 314, 9, 39, "Output",
      CellTags->"Tr2"],
    Cell[2820689, 91277, 247, 7, 56, "Input",
      CellTags->"Tr2"],
    Cell[2820939, 91286, 238, 7, 37, "Output",
      CellTags->"Tr2"],
    Cell[2821214, 91298, 193, 5, 56, "Input",
      CellTags->"Tr2"],
    Cell[2821410, 91305, 191, 5, 39, "Output",
      CellTags->"Tr2"],
    Cell[2821638, 91315, 227, 6, 56, "Input",
      CellTags->"Tr2"],
    Cell[2821868, 91323, 229, 6, 39, "Output",
      CellTags->"Tr2"]},
  "Twist2AlienOperator"->{
    Cell[2822158, 91336, 117, 2, 39, "ObjectName",
      CellTags->"Twist2AlienOperator"],
    Cell[2822300, 91342, 71, 1, 49, "Subsubsection",
      CellTags->"Twist2AlienOperator"],
    Cell[2822374, 91345, 197, 5, 39, "Text",
      CellTags->"Twist2AlienOperator"],
    Cell[2822608, 91355, 68, 1, 49, "Subsubsection",
      CellTags->"Twist2AlienOperator"],
    Cell[2822701, 91360, 190, 4, 56, "Input",
      CellTags->"Twist2AlienOperator"],
    Cell[2824098, 91402, 291, 7, 52, "Input",
      CellTags->"Twist2AlienOperator"]},
  "Twist2CounterOperator"->{
    Cell[2829439, 91531, 121, 2, 39, "ObjectName",
      CellTags->"Twist2CounterOperator"],
    Cell[2829585, 91537, 73, 1, 49, "Subsubsection",
      CellTags->"Twist2CounterOperator"],
    Cell[2829661, 91540, 240, 5, 56, "Text",
      CellTags->"Twist2CounterOperator"],
    Cell[2829938, 91550, 70, 1, 49, "Subsubsection",
      CellTags->"Twist2CounterOperator"],
    Cell[2830033, 91555, 238, 5, 56, "Input",
      CellTags->"Twist2CounterOperator"],
    Cell[2836392, 91707, 189, 4, 52, "Input",
      CellTags->"Twist2CounterOperator"],
    Cell[2837959, 91754, 290, 7, 52, "Input",
      CellTags->"Twist2CounterOperator"]},
  "Twist2GluonOperator"->{
    Cell[2842425, 91868, 76, 1, 39, "ObjectName",
      CellTags->"Twist2GluonOperator"],
    Cell[2842526, 91873, 71, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[2842600, 91876, 965, 15, 141, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2843568, 91893, 236, 9, 39, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2843841, 91907, 100, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[2843966, 91912, 403, 8, 76, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2844874, 91936, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2844962, 91941, 325, 8, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2846503, 91991, 380, 9, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2848479, 92052, 263, 6, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2849173, 92076, 318, 7, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2850498, 92116, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2850586, 92121, 420, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2852654, 92186, 390, 10, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2853674, 92220, 445, 11, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2870976, 92655, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2871064, 92660, 507, 12, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2878836, 92863, 467, 12, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2880077, 92903, 522, 13, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2917631, 93824, 101, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[2917757, 93829, 317, 7, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2918579, 93852, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2918667, 93857, 325, 8, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2920223, 93908, 380, 9, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2924791, 94036, 263, 6, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2925485, 94060, 318, 7, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2929387, 94166, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2929475, 94171, 420, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2931628, 94238, 390, 10, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2932648, 94272, 445, 11, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2960396, 94969, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[2960484, 94974, 507, 12, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2968271, 95178, 467, 12, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[2969512, 95218, 522, 13, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3022578, 96563, 77, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[3022658, 96566, 98, 1, 39, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3022759, 96569, 344, 7, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3023106, 96578, 463, 12, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3023606, 96595, 103, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[3023734, 96600, 333, 7, 76, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3024571, 96623, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3024659, 96628, 385, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3030559, 96774, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3030647, 96779, 420, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3123019, 98660, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3123107, 98665, 563, 13, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3130947, 98870, 105, 1, 49, "Subsubsection",
      CellTags->"Twist2GluonOperator"],
    Cell[3131077, 98875, 333, 7, 76, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3131914, 98898, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3132002, 98903, 385, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3137197, 99046, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3137263, 99049, 463, 12, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3137751, 99065, 390, 10, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3142974, 99210, 445, 11, 52, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3148230, 99354, 63, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3148318, 99359, 563, 13, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3156139, 99562, 79, 1, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3156243, 99567, 634, 16, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3169210, 99885, 110, 3, 35, "Text",
      CellTags->"Twist2GluonOperator"],
    Cell[3169323, 99890, 344, 7, 56, "Input",
      CellTags->"Twist2GluonOperator"],
    Cell[3169692, 99901, 522, 13, 56, "Input",
      CellTags->"Twist2GluonOperator"]},
  "Twist2QuarkOperator"->{
    Cell[3179029, 100121, 76, 1, 39, "ObjectName",
      CellTags->"Twist2QuarkOperator"],
    Cell[3179130, 100126, 71, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3179204, 100129, 981, 15, 141, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3180188, 100146, 236, 9, 39, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3180439, 100158, 68, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3180532, 100163, 99, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3180656, 100168, 317, 7, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3181477, 100191, 76, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3181578, 100196, 188, 4, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3182606, 100227, 82, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3182713, 100232, 369, 10, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3185262, 100304, 88, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3185375, 100309, 457, 12, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3193326, 100496, 101, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3193452, 100501, 322, 7, 56, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3194278, 100524, 76, 1, 35, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3194379, 100529, 188, 4, 56, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3195231, 100555, 82, 1, 35, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3195338, 100560, 369, 10, 56, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3197812, 100630, 88, 1, 35, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3197925, 100635, 457, 12, 56, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3206134, 100824, 94, 1, 35, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3206253, 100829, 238, 6, 56, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3206811, 100849, 439, 10, 52, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3210210, 100944, 523, 12, 72, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3228050, 101326, 103, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3228178, 101331, 318, 7, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3229001, 101354, 179, 4, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3229183, 101360, 278, 6, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3229486, 101370, 248, 6, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3230879, 101411, 101, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3231005, 101416, 369, 10, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3235509, 101526, 88, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3235622, 101531, 188, 4, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3237184, 101576, 105, 1, 49, "Subsubsection",
      CellTags->"Twist2QuarkOperator"],
    Cell[3237314, 101581, 318, 7, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3238137, 101604, 76, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3238238, 101609, 248, 6, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3239631, 101650, 82, 1, 70, "Text",
      CellTags->"Twist2QuarkOperator"],
    Cell[3239738, 101655, 369, 10, 70, "Input",
      CellTags->"Twist2QuarkOperator"],
    Cell[3244242, 101765, 252, 6, 70, "Input",
      CellTags->"Twist2QuarkOperator"]},
  "Twist3QuarkOperator"->{
    Cell[3244543, 101777, 76, 1, 39, "ObjectName",
      CellTags->"Twist3QuarkOperator"],
    Cell[3244644, 101782, 71, 1, 49, "Subsubsection",
      CellTags->"Twist3QuarkOperator"],
    Cell[3244718, 101785, 769, 12, 107, "Text",
      CellTags->"Twist3QuarkOperator"],
    Cell[3245512, 101801, 160, 3, 56, "Input",
      CellTags->"Twist3QuarkOperator"],
    Cell[3245675, 101806, 392, 10, 37, "Output",
      CellTags->"Twist3QuarkOperator"],
    Cell[3246082, 101819, 389, 14, 35, "Text",
      CellTags->"Twist3QuarkOperator"],
    Cell[3246508, 101838, 68, 1, 49, "Subsubsection",
      CellTags->"Twist3QuarkOperator"],
    Cell[3246601, 101843, 158, 3, 56, "Input",
      CellTags->"Twist3QuarkOperator"],
    Cell[3246762, 101848, 912, 25, 38, "Output",
      CellTags->"Twist3QuarkOperator"]},
  "Twist4GluonOperator"->{
    Cell[3247735, 101880, 76, 1, 39, "ObjectName",
      CellTags->"Twist4GluonOperator"],
    Cell[3247836, 101885, 71, 1, 49, "Subsubsection",
      CellTags->"Twist4GluonOperator"],
    Cell[3247910, 101888, 151, 4, 39, "Text",
      CellTags->"Twist4GluonOperator"],
    Cell[3248064, 101894, 542, 19, 39, "Text",
      CellTags->"Twist4GluonOperator"],
    Cell[3248643, 101918, 68, 1, 49, "Subsubsection",
      CellTags->"Twist4GluonOperator"],
    Cell[3248736, 101923, 621, 15, 56, "Input",
      CellTags->"Twist4GluonOperator"]},
  "TwoLoopSimplify"->{
    Cell[3406809, 106007, 105, 1, 39, "ObjectName",
      CellTags->"TwoLoopSimplify"],
    Cell[3406939, 106012, 67, 1, 49, "Subsubsection",
      CellTags->"TwoLoopSimplify"],
    Cell[3407009, 106015, 338, 7, 56, "Text",
      CellTags->"TwoLoopSimplify"],
    Cell[3407350, 106024, 220, 9, 39, "Text",
      CellTags->"TwoLoopSimplify"],
    Cell[3407607, 106038, 64, 1, 49, "Subsubsection",
      CellTags->"TwoLoopSimplify"],
    Cell[3407674, 106041, 104, 2, 56, "Input",
      CellTags->"TwoLoopSimplify"]},
  "Uncontract"->{
    Cell[3407827, 106049, 58, 1, 39, "ObjectName",
      CellTags->"Uncontract"],
    Cell[3407910, 106054, 62, 1, 49, "Subsubsection",
      CellTags->"Uncontract"],
    Cell[3407975, 106057, 259, 6, 56, "Text",
      CellTags->"Uncontract"],
    Cell[3408259, 106067, 148, 3, 56, "Input",
      CellTags->"Uncontract"],
    Cell[3408835, 106085, 194, 9, 35, "Text",
      CellTags->"Uncontract"],
    Cell[3409066, 106099, 59, 1, 49, "Subsubsection",
      CellTags->"Uncontract"],
    Cell[3409150, 106104, 279, 7, 56, "Input",
      CellTags->"Uncontract"],
    Cell[3410010, 106134, 172, 4, 52, "Input",
      CellTags->"Uncontract"],
    Cell[3411268, 106175, 167, 4, 52, "Input",
      CellTags->"Uncontract"],
    Cell[3411756, 106193, 172, 4, 52, "Input",
      CellTags->"Uncontract"],
    Cell[3412631, 106223, 182, 4, 52, "Input",
      CellTags->"Uncontract"],
    Cell[3414337, 106275, 90, 1, 35, "Text",
      CellTags->"Uncontract"],
    Cell[3414452, 106280, 251, 6, 56, "Input",
      CellTags->"Uncontract"],
    Cell[3414990, 106298, 153, 4, 35, "Text",
      CellTags->"Uncontract"],
    Cell[3415168, 106306, 294, 7, 56, "Input",
      CellTags->"Uncontract"],
    Cell[3416208, 106341, 329, 8, 52, "Input",
      CellTags->"Uncontract"],
    Cell[3417863, 106393, 166, 4, 52, "Input",
      CellTags->"Uncontract"]},
  "UnDeclareNonCommutative"->{
    Cell[3418078, 106403, 84, 1, 39, "ObjectName",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3418187, 106408, 75, 1, 49, "Subsubsection",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3418265, 106411, 201, 5, 39, "Text",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3418469, 106418, 366, 14, 39, "Text",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3418872, 106437, 72, 1, 49, "Subsubsection",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3418947, 106440, 166, 3, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3419116, 106445, 154, 4, 70, "Text",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3419295, 106453, 132, 2, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3419557, 106462, 104, 1, 70, "Text",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3419664, 106465, 168, 3, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3419857, 106472, 193, 4, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3420181, 106483, 193, 4, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3420399, 106491, 203, 4, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3420791, 106504, 195, 4, 70, "Input",
      CellTags->"UnDeclareNonCommutative"],
    Cell[3421011, 106512, 203, 4, 70, "Input",
      CellTags->"UnDeclareNonCommutative"]},
  "Upper"->{
    Cell[3421451, 106529, 48, 1, 39, "ObjectName",
      CellTags->"Upper"],
    Cell[3421524, 106534, 57, 1, 49, "Subsubsection",
      CellTags->"Upper"],
    Cell[3421584, 106537, 127, 4, 39, "Text",
      CellTags->"Upper"],
    Cell[3421714, 106543, 429, 18, 39, "Text",
      CellTags->"Upper"]},
  "UVPart"->{
    Cell[3422192, 106567, 87, 1, 39, "ObjectName",
      CellTags->"UVPart"],
    Cell[3422304, 106572, 58, 1, 49, "Subsubsection",
      CellTags->"UVPart"],
    Cell[3422365, 106575, 129, 4, 39, "Text",
      CellTags->"UVPart"],
    Cell[3422531, 106584, 55, 1, 49, "Subsubsection",
      CellTags->"UVPart"],
    Cell[3422589, 106587, 95, 2, 56, "Input",
      CellTags->"UVPart"]},
  "Vectors"->{
    Cell[3422733, 106595, 52, 1, 39, "ObjectName",
      CellTags->"Vectors"],
    Cell[3422810, 106600, 59, 1, 49, "Subsubsection",
      CellTags->"Vectors"],
    Cell[3422872, 106603, 206, 5, 56, "Text",
      CellTags->"Vectors"],
    Cell[3423081, 106610, 200, 8, 39, "Text",
      CellTags->"Vectors"]},
  "WriteOut"->{
    Cell[3423330, 106624, 54, 1, 39, "ObjectName",
      CellTags->"WriteOut"],
    Cell[3423409, 106629, 60, 1, 49, "Subsubsection",
      CellTags->"WriteOut"],
    Cell[3423472, 106632, 217, 5, 56, "Text",
      CellTags->"WriteOut"],
    Cell[3423692, 106639, 183, 8, 39, "Text",
      CellTags->"WriteOut"]},
  "WriteOutPaVe"->{
    Cell[3423924, 106653, 62, 1, 39, "ObjectName",
      CellTags->"WriteOutPaVe"],
    Cell[3424011, 106658, 64, 1, 49, "Subsubsection",
      CellTags->"WriteOutPaVe"],
    Cell[3424078, 106661, 263, 6, 56, "Text",
      CellTags->"WriteOutPaVe"],
    Cell[3424344, 106669, 430, 18, 39, "Text",
      CellTags->"WriteOutPaVe"]},
  "Write2"->{
    Cell[3424823, 106693, 50, 1, 39, "ObjectName",
      CellTags->"Write2"],
    Cell[3424898, 106698, 58, 1, 49, "Subsubsection",
      CellTags->"Write2"],
    Cell[3424959, 106701, 298, 6, 56, "Text",
      CellTags->"Write2"],
    Cell[3425260, 106709, 154, 4, 39, "Text",
      CellTags->"Write2"],
    Cell[3425439, 106717, 166, 4, 56, "Input",
      CellTags->"Write2"],
    Cell[3425931, 106736, 140, 3, 52, "Input",
      CellTags->"Write2"],
    Cell[3426748, 106759, 143, 3, 52, "Input",
      CellTags->"Write2"],
    Cell[3427052, 106770, 313, 14, 35, "Text",
      CellTags->"Write2"],
    Cell[3427402, 106789, 55, 1, 49, "Subsubsection",
      CellTags->"Write2"],
    Cell[3427482, 106794, 553, 15, 60, "Input",
      CellTags->"Write2"],
    Cell[3429089, 106843, 259, 7, 52, "Input",
      CellTags->"Write2"],
    Cell[3429351, 106852, 79, 1, 39, "Text",
      CellTags->"Write2"],
    Cell[3429433, 106855, 227, 6, 56, "Input",
      CellTags->"Write2"],
    Cell[3429663, 106863, 74, 1, 39, "Text",
      CellTags->"Write2"],
    Cell[3429762, 106868, 446, 11, 56, "Input",
      CellTags->"Write2"],
    Cell[3430871, 106900, 350, 9, 56, "Input",
      CellTags->"Write2"],
    Cell[3431246, 106913, 283, 7, 56, "Input",
      CellTags->"Write2"],
    Cell[3432400, 106951, 228, 6, 52, "Input",
      CellTags->"Write2"],
    Cell[3432653, 106961, 446, 11, 56, "Input",
      CellTags->"Write2"],
    Cell[3434163, 107001, 350, 9, 56, "Input",
      CellTags->"Write2"],
    Cell[3434516, 107012, 109, 4, 39, "Text",
      CellTags->"Write2"],
    Cell[3434628, 107018, 301, 8, 56, "Input",
      CellTags->"Write2"],
    Cell[3434954, 107030, 446, 11, 56, "Input",
      CellTags->"Write2"],
    Cell[3436308, 107066, 483, 13, 56, "Input",
      CellTags->"Write2"]},
  "XYT"->{
    Cell[3436840, 107085, 44, 1, 39, "ObjectName",
      CellTags->"XYT"],
    Cell[3436909, 107090, 55, 1, 49, "Subsubsection",
      CellTags->"XYT"],
    Cell[3436967, 107093, 241, 9, 39, "Text",
      CellTags->"XYT"]},
  "ZeroMomentumInsertion"->{
    Cell[3437257, 107108, 80, 1, 39, "ObjectName",
      CellTags->"ZeroMomentumInsertion"],
    Cell[3437362, 107113, 73, 1, 49, "Subsubsection",
      CellTags->"ZeroMomentumInsertion"],
    Cell[3437438, 107116, 155, 4, 39, "Text",
      CellTags->"ZeroMomentumInsertion"],
    Cell[3437596, 107122, 505, 18, 39, "Text",
      CellTags->"ZeroMomentumInsertion"]},
  "Zeta2"->{
    Cell[3438150, 107146, 48, 1, 39, "ObjectName",
      CellTags->"Zeta2"],
    Cell[3438223, 107151, 57, 1, 49, "Subsubsection",
      CellTags->"Zeta2"],
    Cell[3438283, 107154, 95, 1, 39, "Text",
      CellTags->"Zeta2"],
    Cell[3438381, 107157, 203, 8, 39, "Text",
      CellTags->"Zeta2"],
    Cell[3438621, 107170, 54, 1, 49, "Subsubsection",
      CellTags->"Zeta2"],
    Cell[3438700, 107175, 102, 2, 70, "Input",
      CellTags->"Zeta2"],
    Cell[3438989, 107187, 132, 3, 70, "Input",
      CellTags->"Zeta2"],
    Cell[3439288, 107199, 170, 4, 70, "Input",
      CellTags->"Zeta2"]},
  "$Abbreviations"->{
    Cell[3439698, 107216, 66, 1, 39, "ObjectName",
      CellTags->"$Abbreviations"],
    Cell[3439789, 107221, 66, 1, 49, "Subsubsection",
      CellTags->"$Abbreviations"],
    Cell[3439858, 107224, 294, 6, 56, "Text",
      CellTags->"$Abbreviations"],
    Cell[3440177, 107234, 154, 3, 56, "Input",
      CellTags->"$Abbreviations"],
    Cell[3440334, 107239, 961, 17, 107, "Output",
      CellTags->"$Abbreviations"],
    Cell[3441310, 107259, 705, 29, 35, "Text",
      CellTags->"$Abbreviations"]},
  "$AL"->{
    Cell[3442064, 107294, 44, 1, 39, "ObjectName",
      CellTags->"$AL"],
    Cell[3442133, 107299, 55, 1, 49, "Subsubsection",
      CellTags->"$AL"],
    Cell[3442191, 107302, 117, 4, 39, "Text",
      CellTags->"$AL"],
    Cell[3442333, 107310, 95, 2, 56, "Input",
      CellTags->"$AL"],
    Cell[3442576, 107321, 286, 7, 52, "Input",
      CellTags->"$AL"],
    Cell[3443587, 107354, 146, 4, 52, "Input",
      CellTags->"$AL"],
    Cell[3443758, 107362, 286, 7, 56, "Input",
      CellTags->"$AL"],
    Cell[3444665, 107393, 138, 4, 52, "Input",
      CellTags->"$AL"]},
  "$BreitMaison"->{
    Cell[3444852, 107403, 62, 1, 39, "ObjectName",
      CellTags->"$BreitMaison"],
    Cell[3444939, 107408, 64, 1, 49, "Subsubsection",
      CellTags->"$BreitMaison"],
    Cell[3445006, 107411, 332, 9, 56, "Text",
      CellTags->"$BreitMaison"],
    Cell[3445341, 107422, 403, 19, 39, "Text",
      CellTags->"$BreitMaison"]},
  "$Color"->{
    Cell[3445793, 107447, 50, 1, 39, "ObjectName",
      CellTags->"$Color"],
    Cell[3445868, 107452, 58, 1, 49, "Subsubsection",
      CellTags->"$Color"],
    Cell[3445929, 107455, 131, 4, 39, "Text",
      CellTags->"$Color"]},
  "$Covariant"->{
    Cell[3446109, 107465, 58, 1, 39, "ObjectName",
      CellTags->"$Covariant"],
    Cell[3446192, 107470, 62, 1, 49, "Subsubsection",
      CellTags->"$Covariant"],
    Cell[3446257, 107473, 186, 5, 39, "Text",
      CellTags->"$Covariant"],
    Cell[3446468, 107482, 109, 2, 56, "Input",
      CellTags->"$Covariant"],
    Cell[3446580, 107486, 136, 3, 50, "Output",
      CellTags->"$Covariant"],
    Cell[3446753, 107494, 135, 3, 52, "Input",
      CellTags->"$Covariant"],
    Cell[3447051, 107505, 159, 4, 52, "Input",
      CellTags->"$Covariant"],
    Cell[3447235, 107513, 135, 3, 56, "Input",
      CellTags->"$Covariant"],
    Cell[3447531, 107524, 160, 4, 52, "Input",
      CellTags->"$Covariant"]},
  "$FCS"->{
    Cell[3447740, 107534, 46, 1, 39, "ObjectName",
      CellTags->"$FCS"],
    Cell[3447811, 107539, 56, 1, 49, "Subsubsection",
      CellTags->"$FCS"],
    Cell[3447870, 107542, 143, 4, 39, "Text",
      CellTags->"$FCS"],
    Cell[3448038, 107550, 97, 2, 56, "Input",
      CellTags->"$FCS"],
    Cell[3448138, 107554, 739, 13, 69, "Output",
      CellTags->"$FCS"]},
  "$FCT"->{
    Cell[3448938, 107574, 46, 1, 39, "ObjectName",
      CellTags->"$FCT"],
    Cell[3449009, 107579, 56, 1, 49, "Subsubsection",
      CellTags->"$FCT"],
    Cell[3449068, 107582, 180, 5, 39, "Text",
      CellTags->"$FCT"],
    Cell[3449251, 107589, 160, 4, 56, "Input",
      CellTags->"$FCT"],
    Cell[3449436, 107597, 206, 5, 56, "Input",
      CellTags->"$FCT"],
    Cell[3449878, 107612, 161, 4, 52, "Input",
      CellTags->"$FCT"],
    Cell[3450064, 107620, 206, 5, 56, "Input",
      CellTags->"$FCT"]},
  "$FeynCalcStuff"->{
    Cell[3450552, 107639, 66, 1, 39, "ObjectName",
      CellTags->"$FeynCalcStuff"],
    Cell[3450643, 107644, 66, 1, 49, "Subsubsection",
      CellTags->"$FeynCalcStuff"],
    Cell[3450712, 107647, 103, 1, 39, "Text",
      CellTags->"$FeynCalcStuff"]},
  "$FeynCalcVersion"->{
    Cell[3450864, 107654, 70, 1, 39, "ObjectName",
      CellTags->"$FeynCalcVersion"],
    Cell[3450959, 107659, 68, 1, 49, "Subsubsection",
      CellTags->"$FeynCalcVersion"],
    Cell[3451030, 107662, 127, 4, 39, "Text",
      CellTags->"$FeynCalcVersion"],
    Cell[3451182, 107670, 121, 2, 56, "Input",
      CellTags->"$FeynCalcVersion"]},
  "$FortranContinuationCharacter"->{
    Cell[3451485, 107683, 96, 1, 39, "ObjectName",
      CellTags->"$FortranContinuationCharacter"],
    Cell[3451606, 107688, 81, 1, 49, "Subsubsection",
      CellTags->"$FortranContinuationCharacter"],
    Cell[3451690, 107691, 146, 4, 39, "Text",
      CellTags->"$FortranContinuationCharacter"],
    Cell[3451861, 107699, 147, 2, 56, "Input",
      CellTags->"$FortranContinuationCharacter"],
    Cell[3452140, 107708, 207, 9, 35, "Text",
      CellTags->"$FortranContinuationCharacter"]},
  "$Gauge"->{
    Cell[3452396, 107723, 50, 1, 39, "ObjectName",
      CellTags->"$Gauge"],
    Cell[3452471, 107728, 58, 1, 49, "Subsubsection",
      CellTags->"$Gauge"],
    Cell[3452532, 107731, 263, 6, 56, "Text",
      CellTags->"$Gauge"],
    Cell[3452798, 107739, 181, 9, 39, "Text",
      CellTags->"$Gauge"]},
  "$Larin"->{
    Cell[3453028, 107754, 50, 1, 39, "ObjectName",
      CellTags->"$Larin"],
    Cell[3453103, 107759, 58, 1, 49, "Subsubsection",
      CellTags->"$Larin"],
    Cell[3453164, 107762, 1184, 30, 107, "Text",
      CellTags->"$Larin"],
    Cell[3454373, 107796, 101, 2, 56, "Input",
      CellTags->"$Larin"],
    Cell[3454602, 107805, 409, 18, 35, "Text",
      CellTags->"$Larin"]},
  "$LimitTo4"->{
    Cell[3455060, 107829, 56, 1, 39, "ObjectName",
      CellTags->"$LimitTo4"],
    Cell[3455141, 107834, 61, 1, 49, "Subsubsection",
      CellTags->"$LimitTo4"],
    Cell[3455205, 107837, 206, 5, 39, "Text",
      CellTags->"$LimitTo4"],
    Cell[3455436, 107846, 107, 2, 56, "Input",
      CellTags->"$LimitTo4"],
    Cell[3455546, 107850, 134, 3, 37, "Output",
      CellTags->"$LimitTo4"],
    Cell[3455695, 107856, 424, 19, 35, "Text",
      CellTags->"$LimitTo4"]},
  "$LorentzIndices"->{
    Cell[3456168, 107881, 68, 1, 39, "ObjectName",
      CellTags->"$LorentzIndices"],
    Cell[3456261, 107886, 67, 1, 49, "Subsubsection",
      CellTags->"$LorentzIndices"],
    Cell[3456331, 107889, 165, 4, 39, "Text",
      CellTags->"$LorentzIndices"],
    Cell[3456499, 107895, 172, 4, 56, "Input",
      CellTags->"$LorentzIndices"],
    Cell[3456696, 107903, 391, 9, 56, "Input",
      CellTags->"$LorentzIndices"],
    Cell[3458021, 107946, 145, 3, 52, "Input",
      CellTags->"$LorentzIndices"],
    Cell[3458717, 107968, 361, 9, 52, "Input",
      CellTags->"$LorentzIndices"],
    Cell[3460307, 108021, 145, 3, 52, "Input",
      CellTags->"$LorentzIndices"],
    Cell[3461214, 108047, 173, 4, 52, "Input",
      CellTags->"$LorentzIndices"]},
  "$MIntegrate"->{
    Cell[3461436, 108057, 60, 1, 39, "ObjectName",
      CellTags->"$MIntegrate"],
    Cell[3461521, 108062, 63, 1, 49, "Subsubsection",
      CellTags->"$MIntegrate"],
    Cell[3461587, 108065, 137, 4, 39, "Text",
      CellTags->"$MIntegrate"],
    Cell[3461749, 108073, 111, 2, 56, "Input",
      CellTags->"$MIntegrate"],
    Cell[3462006, 108083, 204, 9, 35, "Text",
      CellTags->"$MIntegrate"]},
  "$MU"->{
    Cell[3462259, 108098, 44, 1, 39, "ObjectName",
      CellTags->"$MU"],
    Cell[3462328, 108103, 55, 1, 49, "Subsubsection",
      CellTags->"$MU"],
    Cell[3462386, 108106, 152, 4, 39, "Text",
      CellTags->"$MU"],
    Cell[3462563, 108114, 95, 2, 56, "Input",
      CellTags->"$MU"],
    Cell[3462784, 108123, 187, 9, 35, "Text",
      CellTags->"$MU"]},
  "$MemoryAvailable"->{
    Cell[3463020, 108138, 70, 1, 39, "ObjectName",
      CellTags->"$MemoryAvailable"],
    Cell[3463115, 108143, 68, 1, 49, "Subsubsection",
      CellTags->"$MemoryAvailable"],
    Cell[3463186, 108146, 444, 11, 56, "Text",
      CellTags->"$MemoryAvailable"],
    Cell[3463655, 108161, 121, 2, 56, "Input",
      CellTags->"$MemoryAvailable"]},
  "$NonComm"->{
    Cell[3463948, 108174, 54, 1, 39, "ObjectName",
      CellTags->"$NonComm"],
    Cell[3464027, 108179, 60, 1, 49, "Subsubsection",
      CellTags->"$NonComm"],
    Cell[3464090, 108182, 112, 4, 39, "Text",
      CellTags->"$NonComm"],
    Cell[3464227, 108190, 105, 2, 56, "Input",
      CellTags->"$NonComm"],
    Cell[3465014, 108209, 330, 14, 35, "Text",
      CellTags->"$NonComm"]},
  "$PairBrackets"->{
    Cell[3465393, 108229, 64, 1, 39, "ObjectName",
      CellTags->"$PairBrackets"],
    Cell[3465482, 108234, 65, 1, 49, "Subsubsection",
      CellTags->"$PairBrackets"],
    Cell[3465550, 108237, 156, 4, 39, "Text",
      CellTags->"$PairBrackets"],
    Cell[3465731, 108245, 118, 2, 56, "Input",
      CellTags->"$PairBrackets"],
    Cell[3466002, 108256, 250, 7, 52, "Input",
      CellTags->"$PairBrackets"],
    Cell[3466908, 108289, 146, 3, 52, "Input",
      CellTags->"$PairBrackets"],
    Cell[3467206, 108301, 250, 7, 52, "Input",
      CellTags->"$PairBrackets"],
    Cell[3468188, 108334, 147, 3, 52, "Input",
      CellTags->"$PairBrackets"]},
  "$SpinorMinimal"->{
    Cell[3468512, 108348, 66, 1, 39, "ObjectName",
      CellTags->"$SpinorMinimal"],
    Cell[3468603, 108353, 66, 1, 49, "Subsubsection",
      CellTags->"$SpinorMinimal"],
    Cell[3468672, 108356, 242, 5, 56, "Text",
      CellTags->"$SpinorMinimal"],
    Cell[3468939, 108365, 117, 2, 56, "Input",
      CellTags->"$SpinorMinimal"],
    Cell[3469184, 108374, 213, 9, 35, "Text",
      CellTags->"$SpinorMinimal"]},
  "$VeryVerbose"->{
    Cell[3469446, 108389, 62, 1, 39, "ObjectName",
      CellTags->"$VeryVerbose"],
    Cell[3469533, 108394, 64, 1, 49, "Subsubsection",
      CellTags->"$VeryVerbose"],
    Cell[3469600, 108397, 222, 5, 56, "Text",
      CellTags->"$VeryVerbose"],
    Cell[3469847, 108406, 116, 2, 56, "Input",
      CellTags->"$VeryVerbose"],
    Cell[3470090, 108415, 173, 4, 52, "Input",
      CellTags->"$VeryVerbose"],
    Cell[3470288, 108423, 326, 9, 56, "Input",
      CellTags->"$VeryVerbose"],
    Cell[3472376, 108489, 163, 4, 52, "Input",
      CellTags->"$VeryVerbose"]},
  "$V0"->{
    Cell[3472542, 108495, 172, 9, 39, "Text",
      CellTags->"$V0"]},
  "$West"->{
    Cell[3472763, 108510, 48, 1, 39, "ObjectName",
      CellTags->"$West"],
    Cell[3472836, 108515, 57, 1, 49, "Subsubsection",
      CellTags->"$West"],
    Cell[3472896, 108518, 492, 13, 56, "Text",
      CellTags->"$West"],
    Cell[3473391, 108533, 170, 8, 39, "Text",
      CellTags->"$West"]},
  "Experimental"->{
    Cell[3474844, 108599, 71, 1, 56, "Section",
      CellTags->"Experimental"],
    Cell[3497544, 109351, 125, 4, 35, "Text",
      CellTags->"Experimental"],
    Cell[3497672, 109357, 189, 5, 43, "Input",
      CellTags->"Experimental"],
    Cell[3497864, 109364, 101, 1, 39, "Text",
      CellTags->"Experimental"],
    Cell[3497968, 109367, 231, 5, 43, "Input",
      CellTags->"Experimental"]},
  "DoPolarizationSums"->{
    Cell[3474940, 108604, 74, 1, 45, "ObjectName",
      CellTags->"DoPolarizationSums"],
    Cell[3475039, 108609, 70, 1, 49, "Subsubsection",
      CellTags->"DoPolarizationSums"],
    Cell[3475112, 108612, 292, 7, 114, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3475407, 108621, 340, 14, 39, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3475784, 108640, 67, 1, 49, "Subsubsection",
      CellTags->"DoPolarizationSums"],
    Cell[3475854, 108643, 311, 6, 56, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3476168, 108651, 126, 4, 39, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3476319, 108659, 644, 17, 63, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3476966, 108678, 996, 29, 40, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3477965, 108709, 94, 2, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3478062, 108713, 115, 3, 37, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3478192, 108719, 1599, 48, 40, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3479794, 108769, 608, 21, 39, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3480405, 108792, 124, 3, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3480532, 108797, 331, 9, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3480888, 108810, 273, 7, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3481164, 108819, 398, 10, 89, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3481577, 108832, 233, 9, 42, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3481813, 108843, 286, 8, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3482102, 108853, 341, 9, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3482446, 108864, 359, 13, 42, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3482808, 108879, 869, 30, 40, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3483702, 108913, 469, 12, 64, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3484174, 108927, 470, 14, 89, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3484659, 108944, 767, 27, 40, "Text",
      CellTags->"DoPolarizationSums"],
    Cell[3485451, 108975, 468, 12, 64, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3485922, 108989, 470, 14, 89, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3486429, 109008, 1000, 25, 83, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3487432, 109035, 1213, 35, 41, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3488682, 109075, 108, 2, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3488793, 109079, 385, 11, 41, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3489215, 109095, 94, 2, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3489312, 109099, 115, 3, 37, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3489464, 109107, 98, 2, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3489565, 109111, 115, 3, 37, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3489717, 109119, 104, 2, 43, "Input",
      CellTags->"DoPolarizationSums"],
    Cell[3489824, 109123, 115, 3, 37, "Output",
      CellTags->"DoPolarizationSums"],
    Cell[3489954, 109129, 199, 4, 56, "Input",
      CellTags->"DoPolarizationSums"]},
  "FeynmanDoIntegrals"->{
    Cell[3490202, 109139, 91, 1, 39, "ObjectName",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3490318, 109144, 70, 1, 49, "Subsubsection",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3490391, 109147, 1519, 23, 217, "Text",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3491913, 109172, 370, 14, 39, "Text",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3492308, 109190, 161, 3, 56, "Input",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3492472, 109195, 1218, 28, 75, "Output",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3493739, 109229, 67, 1, 49, "Subsubsection",
      CellTags->"FeynmanDoIntegrals"],
    Cell[3493809, 109232, 109, 2, 56, "Input",
      CellTags->"FeynmanDoIntegrals"]},
  "FeynmanParametrize1"->{
    Cell[3493967, 109240, 93, 1, 39, "ObjectName",
      CellTags->"FeynmanParametrize1"],
    Cell[3494085, 109245, 71, 1, 49, "Subsubsection",
      CellTags->"FeynmanParametrize1"],
    Cell[3494159, 109248, 718, 12, 132, "Text",
      CellTags->"FeynmanParametrize1"],
    Cell[3494902, 109264, 162, 3, 56, "Input",
      CellTags->"FeynmanParametrize1"],
    Cell[3495679, 109288, 68, 1, 49, "Subsubsection",
      CellTags->"FeynmanParametrize1"],
    Cell[3495750, 109291, 110, 2, 56, "Input",
      CellTags->"FeynmanParametrize1"]},
  "FeynmanReduce"->{
    Cell[3495909, 109299, 81, 1, 39, "ObjectName",
      CellTags->"FeynmanReduce"],
    Cell[3496015, 109304, 65, 1, 49, "Subsubsection",
      CellTags->"FeynmanReduce"],
    Cell[3496083, 109307, 724, 12, 132, "Text",
      CellTags->"FeynmanReduce"],
    Cell[3496832, 109323, 150, 3, 56, "Input",
      CellTags->"FeynmanReduce"],
    Cell[3497348, 109342, 62, 1, 49, "Subsubsection",
      CellTags->"FeynmanReduce"],
    Cell[3497413, 109345, 104, 2, 56, "Input",
      CellTags->"FeynmanReduce"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Functions", 3500242, 109429},
  {"Abbreviation", 3500334, 109432},
  {"Amplitude", 3500842, 109446},
  {"Amputate", 3502029, 109479},
  {"AnomalousDimension", 3502649, 109496},
  {"AntiCommutator", 3505140, 109557},
  {"AntiQuarkField", 3506758, 109598},
  {"AntiSymmetrize", 3507272, 109611},
  {"Anti5", 3507855, 109626},
  {"Apart1", 3509007, 109659},
  {"Apart2", 3509753, 109680},
  {"Apart3", 3510353, 109697},
  {"A0", 3511094, 109718},
  {"A0ToB0", 3512092, 109748},
  {"BackgroundGluonVertex", 3512516, 109760},
  {"Bracket", 3513415, 109781},
  {"BReduce", 3513727, 109790},
  {"B0", 3514429, 109810},
  {"B0Real", 3515198, 109833},
  {"B0Unique", 3515965, 109855},
  {"B00", 3516747, 109877},
  {"B1", 3517658, 109904},
  {"B11", 3518625, 109933},
  {"CA", 3519673, 109964},
  {"Calc", 3520237, 109981},
  {"CalculateCounterTerm", 3521181, 110008},
  {"Info3241266069-5635180", 3521654, 110019},
  {"CancelQP", 3521758, 110022},
  {"Cases2", 3523059, 110057},
  {"CF", 3523659, 110074},
  {"ChangeDimension", 3524302, 110093},
  {"ChargeConjugationMatrix", 3526347, 110144},
  {"ChargeConjugationMatrixInv", 3527278, 110165},
  {"CheckContext", 3528407, 110190},
  {"ChiralityProjector", 3529218, 110211},
  {"Chisholm", 3529917, 110228},
  {"ChisholmSpinor", 3531493, 110271},
  {"ClearScalarProducts", 3532265, 110290},
  {"Collecting", 3533151, 110311},
  {"Collect2", 3533482, 110320},
  {"Collect3", 3534630, 110351},
  {"Combinations", 3535263, 110368},
  {"Combine", 3535686, 110379},
  {"CombineGraphs", 3536528, 110402},
  {"Commutator", 3536871, 110411},
  {"CommutatorExplicit", 3538130, 110444},
  {"CompleteSquare", 3539178, 110469},
  {"ComplexConjugate", 3539862, 110486},
  {"Contract", 3541207, 110519},
  {"Contract1", 3543313, 110576},
  {"Convolute", 3544331, 110603},
  {"ConvoluteTable", 3546624, 110664},
  {"CovariantFieldDerivative", 3547476, 110685},
  {"CounterT", 3548051, 110698},
  {"CounterTerm", 3548532, 110711},
  {"CouplingConstant", 3549036, 110724},
  {"CovariantD", 3549395, 110733},
  {"CrossProduct", 3551409, 110786},
  {"C0", 3551986, 110801},
  {"DataType", 3552566, 110818},
  {"DB0", 3555643, 110901},
  {"DB1", 3556086, 110914},
  {"DeclareNonCommutative", 3556547, 110927},
  {"DeltaFunction", 3557629, 110952},
  {"DeltaFunctionDoublePrime", 3558472, 110973},
  {"DeltaFunctionPrime", 3558868, 110982},
  {"DenominatorOrder", 3559584, 110999},
  {"Dimension", 3559940, 111008},
  {"DimensionalReduction", 3560879, 111033},
  {"DiracBasis", 3561252, 111042},
  {"DiracCanonical", 3561588, 111051},
  {"DiracEquation", 3561939, 111060},
  {"DiracGamma", 3562527, 111075},
  {"DiracGammaCombine", 3563863, 111110},
  {"DiracGammaExpand", 3564735, 111131},
  {"DiracGammaT", 3566079, 111164},
  {"DiracMatrix", 3566731, 111181},
  {"DiracOrder", 3568002, 111214},
  {"DiracReduce", 3569556, 111255},
  {"DiracSigma", 3570747, 111286},
  {"DiracSigmaExplicit", 3571626, 111309},
  {"DiracSimpCombine", 3572255, 111324},
  {"DiracSimplify", 3572614, 111333},
  {"DiracSimplify2", 3575098, 111396},
  {"DiracSlash", 3575693, 111411},
  {"DiracSpinor", 3576718, 111438},
  {"DiracSubstitute67", 3577061, 111447},
  {"DiracTrace", 3577422, 111456},
  {"DiracTraceEvaluate", 3578605, 111487},
  {"DiracTrick", 3578970, 111496},
  {"Divideout", 3580674, 111541},
  {"DOT", 3580996, 111550},
  {"DotExpand", 3581300, 111559},
  {"DotPower", 3582086, 111580},
  {"DotProduct", 3582410, 111589},
  {"DotSimplifyRelations", 3582912, 111602},
  {"DotSimplify", 3583285, 111611},
  {"DummyIndex", 3585093, 111658},
  {"D0", 3585417, 111667},
  {"D0Convention", 3586002, 111684},
  {"Eps", 3586336, 111693},
  {"EpsChisholm", 3587896, 111738},
  {"EpsContract", 3588471, 111753},
  {"EpsDiscard", 3589124, 111770},
  {"EpsEvaluate", 3589457, 111779},
  {"EpsUncontract", 3590113, 111796},
  {"Epsilon", 3590455, 111805},
  {"EpsilonOrder", 3591076, 111822},
  {"Expanding", 3591417, 111831},
  {"ExpandPartialD", 3591750, 111840},
  {"ScalarProductCancel", 3593157, 111875},
  {"ExpandScalarProduct", 3597937, 111972},
  {"Expand2", 3600518, 112019},
  {"Explicit", 3600918, 112030},
  {"ExplicitLorentzIndex", 3601778, 112053},
  {"ExplicitPartialD", 3602422, 112068},
  {"ExplicitSUNIndex", 3603284, 112089},
  {"ExtraFactor", 3603813, 112102},
  {"ExtraVariables", 3604153, 112111},
  {"FactorFull", 3604502, 112120},
  {"Factoring", 3604834, 112129},
  {"Factorout", 3605162, 112138},
  {"FactorTime", 3605489, 112147},
  {"Factor1", 3605818, 112156},
  {"Factor2", 3606653, 112179},
  {"FAD", 3607630, 112206},
  {"FC", 3608348, 112227},
  {"FCE", 3609194, 112252},
  {"FCI", 3610119, 112279},
  {"FCIntegral", 3610846, 112300},
  {"FCIntegrate", 3611180, 112309},
  {"FCNIntegrate", 3611518, 112318},
  {"FC2RHI", 3611854, 112327},
  {"FC2TLI", 3612388, 112342},
  {"FDS", 3612919, 112357},
  {"FermionSpinSum", 3613226, 112366},
  {"FeynAmp", 3615599, 112425},
  {"FeynAmpDenominator", 3616370, 112446},
  {"FeynAmpDenominatorCombine", 3617345, 112469},
  {"FeynAmpDenominatorSimplify", 3618124, 112486},
  {"FeynAmpDenominatorSplit", 3619645, 112519},
  {"FeynAmpList", 3620749, 112544},
  {"FeynCalc", 3621159, 112555},
  {"FeynCalcExternal", 3621564, 112566},
  {"FeynCalcForm", 3622586, 112591},
  {"FeynCalcInternal", 3623877, 112624},
  {"FeynCalc2FORM", 3624902, 112649},
  {"FeynmanParameterNames", 3626612, 112692},
  {"FeynmanParametrize", 3626997, 112701},
  {"FeynRule", 3627535, 112714},
  {"FI", 3630615, 112797},
  {"FieldDerivative", 3630925, 112806},
  {"FieldStrength", 3631622, 112823},
  {"FinalFunction", 3632702, 112850},
  {"FinalSubstitutions", 3633057, 112859},
  {"FORM", 3633420, 112868},
  {"FORMEpilog", 3633733, 112877},
  {"FORMProlog", 3634070, 112886},
  {"FORM2FeynCalc", 3634410, 112895},
  {"FourDivergence", 3635809, 112930},
  {"FourLaplacian", 3636657, 112951},
  {"FourVector", 3637494, 112972},
  {"FreeIndex", 3638453, 112997},
  {"FreeQ2", 3638782, 113006},
  {"FRH", 3639468, 113025},
  {"FromTFi", 3640131, 113044},
  {"FUNCTION", 3640760, 113061},
  {"FunctionalD", 3641089, 113070},
  {"FV", 3645879, 113193},
  {"FVD", 3646736, 113218},
  {"GA", 3647603, 113243},
  {"GAD", 3648392, 113266},
  {"GammaEpsilon", 3649201, 113289},
  {"GammaExpand", 3650104, 113312},
  {"Gamma1", 3650759, 113329},
  {"Gamma2", 3651076, 113338},
  {"Gamma3", 3651393, 113347},
  {"Gauge", 3651709, 113356},
  {"GaugeField", 3652026, 113365},
  {"GaugeXi", 3652755, 113384},
  {"GA5", 3653072, 113393},
  {"GGV", 3653589, 113408},
  {"GhostPropagator", 3653901, 113417},
  {"GHP", 3654668, 113436},
  {"GluonField", 3655334, 113455},
  {"GluonGhostVertex", 3655913, 113470},
  {"GluonPropagator", 3656702, 113489},
  {"GluonSelfEnergy", 3658382, 113530},
  {"GluonVertex", 3658993, 113545},
  {"GO", 3660505, 113584},
  {"GP", 3660800, 113593},
  {"GrassmannParity", 3661108, 113602},
  {"GS", 3661456, 113611},
  {"GSD", 3662176, 113632},
  {"Gstrong", 3662910, 113653},
  {"GTI", 3663455, 113668},
  {"GV", 3663754, 113677},
  {"Hill", 3664051, 113686},
  {"HypergeometricAC", 3664731, 113705},
  {"HypergeometricIR", 3665851, 113732},
  {"HypergeometricSE", 3666641, 113751},
  {"HypExplicit", 3667343, 113768},
  {"HypInt", 3668077, 113787},
  {"IFPD", 3668545, 113800},
  {"IFPDOff", 3668853, 113809},
  {"IFPDOn", 3669852, 113836},
  {"IncludePair", 3670174, 113845},
  {"IndexPosition", 3670517, 113854},
  {"Indices", 3670861, 113863},
  {"InitialFunction", 3671190, 113872},
  {"InitialSubstitutions", 3671556, 113881},
  {"InsideDiracTrace", 3671939, 113890},
  {"IntegralTable", 3672303, 113899},
  {"IntegrateByParts", 3672658, 113908},
  {"Integratedx", 3673194, 113921},
  {"Integrate2", 3673535, 113930},
  {"Integrate3", 3676902, 114017},
  {"Integrate4", 3677787, 114040},
  {"Integrate5", 3678124, 114049},
  {"IntermediateSubstitutions", 3678474, 114058},
  {"InverseMellin", 3678873, 114067},
  {"Isolate", 3681630, 114136},
  {"IsolateHead", 3682705, 114165},
  {"IsolateNames", 3683046, 114174},
  {"IsolatePrint", 3683393, 114183},
  {"IsolateSplit", 3683739, 114192},
  {"KeepOnly", 3684081, 114201},
  {"KK", 3684402, 114210},
  {"Kummer", 3684703, 114219},
  {"Lagrangian", 3687767, 114302},
  {"LC", 3688639, 114325},
  {"LCD", 3689428, 114348},
  {"LeftPartialD", 3690446, 114377},
  {"LeftRightPartialD", 3691439, 114402},
  {"LeftRightPartialD2", 3692409, 114425},
  {"LeviCivita", 3693382, 114448},
  {"LeviCivitaSign", 3694812, 114485},
  {"ListIntegrals", 3695168, 114494},
  {"Li2", 3695597, 114505},
  {"Li3", 3696185, 114522},
  {"Loop", 3696844, 114541},
  {"LorentzIndex", 3697087, 114548},
  {"Lower", 3698379, 114581},
  {"MakeContext", 3698697, 114590},
  {"Mandelstam", 3699038, 114599},
  {"Map2", 3699371, 114608},
  {"MemoryAvailable", 3700050, 114627},
  {"MemSet", 3700403, 114636},
  {"MetricTensor", 3700725, 114645},
  {"MLimit", 3701784, 114672},
  {"Momentum", 3702184, 114683},
  {"MomentumCombine", 3703645, 114722},
  {"MomentumCombine2", 3704673, 114747},
  {"MomentumExpand", 3705295, 114762},
  {"MT", 3706376, 114789},
  {"MTD", 3707095, 114810},
  {"NegativeInteger", 3707836, 114831},
  {"Nf", 3708184, 114840},
  {"Nielsen", 3708485, 114849},
  {"NonCommFreeQ", 3709555, 114878},
  {"NonCommQ", 3709898, 114887},
  {"NonCommutative", 3710305, 114898},
  {"NTerms", 3710654, 114907},
  {"NumberOfMetricTensors", 3711285, 114924},
  {"NumericalFactor", 3711582, 114931},
  {"NumericQ1", 3711856, 114938},
  {"OneLoop", 3712422, 114953},
  {"OneLoopSimplify", 3713948, 114994},
  {"OneLoopSum", 3715053, 115021},
  {"OPE", 3715382, 115030},
  {"OPEDelta", 3715687, 115039},
  {"OPEi", 3716316, 115056},
  {"OPEInsert", 3717135, 115079},
  {"OPEInt", 3717622, 115092},
  {"OPEIntDelta", 3718095, 115105},
  {"OPEIntegrate", 3718601, 115118},
  {"OPEIntegrateDelta", 3719117, 115131},
  {"OPEIntegrate2", 3719660, 115144},
  {"OPESumExplicit", 3720179, 115157},
  {"OPESum", 3721022, 115178},
  {"OPESumSimplify", 3721794, 115199},
  {"OPE1Loop", 3722638, 115220},
  {"OPE2AI", 3723119, 115233},
  {"OPE2TID", 3723660, 115248},
  {"OptionsSelect", 3724215, 115263},
  {"Pair", 3724478, 115270},
  {"PairCollect", 3725793, 115307},
  {"PairContract", 3726136, 115316},
  {"PairContract2", 3726484, 115325},
  {"PairContract3", 3726836, 115334},
  {"PartialD", 3727183, 115343},
  {"PartialDRelations", 3727828, 115360},
  {"PartialFourVector", 3728460, 115375},
  {"PartialIntegrate", 3729007, 115388},
  {"PartitHead", 3730637, 115427},
  {"PauliSigma", 3730897, 115434},
  {"PaVe", 3731309, 115445},
  {"PaVeOrderList", 3731984, 115464},
  {"PaVeOrder", 3732332, 115473},
  {"PaVeReduce", 3732979, 115490},
  {"PD", 3734398, 115527},
  {"PlusDistribution", 3734708, 115536},
  {"Polarization", 3735662, 115559},
  {"PolarizationSum", 3736653, 115584},
  {"PolarizationUncontract", 3737520, 115605},
  {"PolarizationVector", 3737913, 115614},
  {"PositiveInteger", 3739484, 115651},
  {"PositiveNumber", 3739844, 115660},
  {"PowerFactor", 3740197, 115669},
  {"PowerSimplify", 3740782, 115684},
  {"Power2", 3741537, 115703},
  {"PostFortranFile", 3742088, 115718},
  {"Prefactor", 3742444, 115727},
  {"PreFortranFile", 3742781, 115736},
  {"PropagatorDenominator", 3743145, 115745},
  {"PropagatorDenominatorExplicit", 3744345, 115772},
  {"QGV", 3745143, 115789},
  {"QO", 3745442, 115798},
  {"QP", 3745737, 115807},
  {"QuantumField", 3746043, 115816},
  {"QuarkField", 3747966, 115865},
  {"QuarkGluonVertex", 3748470, 115878},
  {"QuarkMass", 3749500, 115903},
  {"QuarkPropagator", 3749836, 115912},
  {"ReduceGamma", 3750611, 115931},
  {"ReduceToScalars", 3750956, 115940},
  {"Rename", 3751309, 115949},
  {"RHI", 3751622, 115958},
  {"RHI2FC", 3752074, 115971},
  {"RHM", 3752541, 115984},
  {"RHO", 3752987, 115997},
  {"RHP", 3753434, 116010},
  {"RTL", 3753881, 116023},
  {"RightPartialD", 3754339, 116036},
  {"RussianTrick", 3755178, 116057},
  {"ScalarGluonVertex", 3756168, 116082},
  {"ScalarProduct", 3756711, 116095},
  {"ScaleMu", 3757945, 116126},
  {"Schouten", 3758343, 116137},
  {"SD", 3758899, 116152},
  {"SelectFree", 3759486, 116169},
  {"SelectGraphs", 3760523, 116196},
  {"SelectNotFree", 3760870, 116205},
  {"SelectSplit", 3762024, 116234},
  {"Select1", 3762528, 116247},
  {"Select2", 3762849, 116256},
  {"Series2", 3763170, 116265},
  {"Series3", 3764464, 116300},
  {"SetMandelstam", 3765321, 116323},
  {"SimplifyDeltaFunction", 3765841, 116336},
  {"SimplifyGTI", 3767374, 116371},
  {"SimplifyPolyLog", 3767881, 116384},
  {"Simplify2", 3769965, 116435},
  {"SmallDelta", 3770379, 116446},
  {"SmallEpsilon", 3770640, 116453},
  {"SmallVariable", 3770908, 116460},
  {"SmallVariables", 3771261, 116469},
  {"SMP", 3771606, 116478},
  {"SMVertex", 3772340, 116499},
  {"Smu", 3772819, 116512},
  {"Sn", 3773049, 116519},
  {"SO", 3773278, 116526},
  {"SOD", 3773859, 116543},
  {"Solve2", 3774451, 116560},
  {"Solve3", 3775288, 116583},
  {"SP", 3775755, 116596},
  {"SPC", 3776613, 116621},
  {"SPD", 3776913, 116630},
  {"Spinor", 3777856, 116657},
  {"SpinorCollect", 3779655, 116706},
  {"SpinorUBar", 3780004, 116715},
  {"SpinorU", 3780882, 116738},
  {"SpinorVBar", 3781657, 116759},
  {"SpinorV", 3782460, 116780},
  {"SpinPolarizationSum", 3783244, 116801},
  {"SPL", 3783609, 116810},
  {"SplittingFunction", 3783923, 116819},
  {"StandardMatrixElement", 3786904, 116890},
  {"SubContext", 3787284, 116899},
  {"SubLoop", 3787618, 116908},
  {"SumP", 3787936, 116917},
  {"SumS", 3788821, 116942},
  {"SumT", 3790273, 116983},
  {"SUND", 3791725, 117024},
  {"SUNDelta", 3792898, 117057},
  {"SUNDeltaContract", 3794140, 117090},
  {"SUNF", 3794495, 117099},
  {"SUNFJacobi", 3795952, 117140},
  {"SUNIndex", 3796527, 117155},
  {"SUNIndexRename", 3797167, 117172},
  {"SUNN", 3797514, 117181},
  {"SUNNToCACF", 3797976, 117194},
  {"SUNSimplify", 3798475, 117207},
  {"SUNT", 3801244, 117278},
  {"SUNTrace", 3802697, 117319},
  {"SymbolicSum2", 3804084, 117356},
  {"SymbolicSum3", 3804352, 117363},
  {"Symmetrize", 3804618, 117370},
  {"TBox", 3805186, 117385},
  {"Tbox", 3805452, 117392},
  {"Tdec", 3805718, 117399},
  {"TensorFunction", 3806754, 117428},
  {"Tf", 3808573, 117473},
  {"TFi", 3808872, 117482},
  {"ThreeVector", 3809540, 117501},
  {"TID", 3809873, 117510},
  {"TIDL", 3810673, 117533},
  {"TimedIntegrate", 3812717, 117588},
  {"TLI", 3813720, 117613},
  {"TLIFP", 3814241, 117628},
  {"TLIHYP", 3814774, 117643},
  {"TLI2", 3815313, 117658},
  {"TLI2FC", 3815620, 117667},
  {"ToDistribution", 3816169, 117682},
  {"ToHypergeometric", 3817267, 117709},
  {"ToLarin", 3818053, 117728},
  {"ToTFi", 3818602, 117743},
  {"Tr", 3819424, 117766},
  {"TraceDimension", 3822532, 117855},
  {"TraceOfOne", 3822884, 117864},
  {"Trick", 3823454, 117879},
  {"TrickIntegrate", 3824288, 117902},
  {"TrickMandelstam", 3824735, 117913},
  {"Tr2", 3825260, 117926},
  {"Twist2AlienOperator", 3826642, 117965},
  {"Twist2CounterOperator", 3827206, 117978},
  {"Twist2GluonOperator", 3827868, 117993},
  {"Twist2QuarkOperator", 3833186, 118116},
  {"Twist3QuarkOperator", 3836815, 118199},
  {"Twist4GluonOperator", 3837649, 118218},
  {"TwoLoopSimplify", 3838213, 118231},
  {"Uncontract", 3838747, 118244},
  {"UnDeclareNonCommutative", 3840118, 118279},
  {"Upper", 3841512, 118310},
  {"UVPart", 3841829, 118319},
  {"Vectors", 3842231, 118330},
  {"WriteOut", 3842557, 118339},
  {"WriteOutPaVe", 3842891, 118348},
  {"Write2", 3843236, 118357},
  {"XYT", 3845040, 118406},
  {"ZeroMomentumInsertion", 3845293, 118413},
  {"Zeta2", 3845673, 118422},
  {"$Abbreviations", 3846295, 118439},
  {"$AL", 3846812, 118452},
  {"$BreitMaison", 3847410, 118469},
  {"$Color", 3847755, 118478},
  {"$Covariant", 3848006, 118485},
  {"$FCS", 3848732, 118504},
  {"$FCT", 3849116, 118515},
  {"$FeynCalcStuff", 3849653, 118530},
  {"$FeynCalcVersion", 3849934, 118537},
  {"$FortranContinuationCharacter", 3850318, 118546},
  {"$Gauge", 3850827, 118557},
  {"$Larin", 3851147, 118566},
  {"$LimitTo4", 3851548, 118577},
  {"$LorentzIndices", 3852045, 118590},
  {"$MIntegrate", 3852822, 118609},
  {"$MU", 3853238, 118620},
  {"$MemoryAvailable", 3853626, 118631},
  {"$NonComm", 3853990, 118640},
  {"$PairBrackets", 3854402, 118651},
  {"$SpinorMinimal", 3855083, 118668},
  {"$VeryVerbose", 3855523, 118679},
  {"$V0", 3856105, 118694},
  {"$West", 3856189, 118697},
  {"Experimental", 3856512, 118706},
  {"DoPolarizationSums", 3856938, 118717},
  {"FeynmanDoIntegrals", 3860268, 118794},
  {"FeynmanParametrize1", 3861007, 118811},
  {"FeynmanReduce", 3861570, 118824}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 54, 1, 105, "Title"],
Cell[1784, 55, 88, 1, 64, "Subtitle"],
Cell[1875, 58, 120, 3, 66, "Subtitle",
  PageBreakBelow->True],
Cell[1998, 63, 81, 1, 53, "Subsubtitle"],
Cell[2082, 66, 258, 8, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2365, 78, 53, 1, 70, "Section",
  CellTags->"Functions"],

Cell[CellGroupData[{
Cell[2443, 83, 54, 0, 33, "Commentary"],
Cell[2500, 85, 89, 3, 25, "Text"],

Cell[CellGroupData[{
Cell[2614, 92, 154, 3, 43, "Input"],
Cell[2771, 97, 128, 2, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2948, 105, 86, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"Abbreviation"],

Cell[CellGroupData[{
Cell[3059, 111, 64, 1, 49, "Subsubsection",
  CellTags->"Abbreviation"],
Cell[3126, 114, 330, 6, 56, "Text",
  CellTags->"Abbreviation"],
Cell[3459, 122, 1114, 43, 56, "Text",
  CellTags->"Abbreviation"]
}, Closed]],

Cell[CellGroupData[{
Cell[4610, 170, 61, 1, 49, "Subsubsection",
  CellTags->"Abbreviation"],

Cell[CellGroupData[{
Cell[4696, 175, 318, 8, 56, "Input",
  CellTags->"Abbreviation"],
Cell[5017, 185, 929, 26, 56, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[6007, 218, 56, 1, 39, "ObjectName",
  CellTags->"Amplitude"],

Cell[CellGroupData[{
Cell[6088, 223, 61, 1, 49, "Subsubsection",
  CellTags->"Amplitude"],
Cell[6152, 226, 530, 9, 90, "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{
Cell[6707, 239, 144, 3, 56, "Input",
  CellTags->"Amplitude"],
Cell[6854, 244, 394, 10, 70, "Output",
  CellTags->"Amplitude"]
}, Closed]],
Cell[7263, 257, 190, 9, 35, "Text",
  CellTags->"Amplitude"]
}, Closed]],

Cell[CellGroupData[{
Cell[7490, 271, 58, 1, 49, "Subsubsection",
  CellTags->"Amplitude"],
Cell[7551, 274, 133, 3, 56, "Input",
  CellTags->"Amplitude"],
Cell[7687, 279, 94, 1, 39, "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{
Cell[7806, 284, 150, 3, 56, "Input",
  CellTags->"Amplitude"],
Cell[7959, 289, 4845, 132, 70, "Output",
  CellTags->"Amplitude"]
}, Closed]],

Cell[CellGroupData[{
Cell[12841, 426, 137, 3, 52, "Input",
  CellTags->"Amplitude"],
Cell[12981, 431, 12314, 304, 70, "Output",
  CellTags->"Amplitude"]
}, Closed]],
Cell[25310, 738, 220, 8, 35, "Text",
  CellTags->"Amplitude"],

Cell[CellGroupData[{
Cell[25555, 750, 373, 9, 56, "Input",
  CellTags->"Amplitude"],
Cell[25931, 761, 32526, 732, 70, "Output",
  CellTags->"Amplitude"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[58518, 1500, 54, 1, 39, "ObjectName",
  CellTags->"Amputate"],

Cell[CellGroupData[{
Cell[58597, 1505, 60, 1, 49, "Subsubsection",
  CellTags->"Amputate"],
Cell[58660, 1508, 216, 5, 56, "Text",
  CellTags->"Amputate"],

Cell[CellGroupData[{
Cell[58901, 1517, 142, 3, 56, "Input",
  CellTags->"Amputate"],
Cell[59046, 1522, 338, 9, 70, "Output"]
}, Closed]],
Cell[59399, 1534, 708, 28, 35, "Text",
  CellTags->"Amputate"]
}, Closed]],

Cell[CellGroupData[{
Cell[60144, 1567, 57, 1, 49, "Subsubsection",
  CellTags->"Amputate"],

Cell[CellGroupData[{
Cell[60226, 1572, 206, 5, 70, "Input",
  CellTags->"Amputate"],
Cell[60435, 1579, 608, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[61080, 1601, 163, 4, 70, "Input",
  CellTags->"Amputate"],
Cell[61246, 1607, 956, 27, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[62263, 1641, 74, 1, 39, "ObjectName",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[62362, 1646, 70, 1, 49, "Subsubsection",
  CellTags->"AnomalousDimension"],
Cell[62435, 1649, 144, 4, 39, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[62604, 1657, 162, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[62769, 1662, 273, 7, 70, "Output"]
}, Closed]],
Cell[63057, 1672, 438, 18, 35, "Text",
  CellTags->"AnomalousDimension"]
}, Closed]],

Cell[CellGroupData[{
Cell[63532, 1695, 67, 1, 49, "Subsubsection",
  CellTags->"AnomalousDimension"],
Cell[63602, 1698, 64, 1, 39, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[63691, 1703, 243, 5, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[63937, 1710, 273, 7, 70, "Output"]
}, Closed]],
Cell[64225, 1720, 249, 8, 70, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[64499, 1732, 161, 3, 70, "Input",
  CellTags->"AnomalousDimension"],
Cell[64663, 1737, 557, 16, 70, "Output"]
}, Closed]],
Cell[65235, 1756, 248, 8, 70, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[65508, 1768, 160, 3, 70, "Input",
  CellTags->"AnomalousDimension"],
Cell[65671, 1773, 320, 9, 70, "Output"]
}, Closed]],
Cell[66006, 1785, 248, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[66279, 1797, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[66442, 1802, 319, 9, 70, "Output"]
}, Closed]],
Cell[66776, 1814, 248, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[67049, 1826, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[67212, 1831, 730, 21, 70, "Output"]
}, Closed]],
Cell[67957, 1855, 249, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[68231, 1867, 161, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[68395, 1872, 828, 22, 70, "Output"]
}, Closed]],
Cell[69238, 1897, 249, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[69512, 1909, 161, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[69676, 1914, 8523, 214, 70, "Output"]
}, Closed]],
Cell[78214, 2131, 248, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[78487, 2143, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[78650, 2148, 5416, 138, 70, "Output"]
}, Closed]],
Cell[84081, 2289, 248, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[84354, 2301, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[84517, 2306, 8323, 211, 70, "Output"]
}, Closed]],
Cell[92855, 2520, 248, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[93128, 2532, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[93291, 2537, 6503, 166, 70, "Output"]
}, Closed]],
Cell[99809, 2706, 275, 8, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[100109, 2718, 160, 3, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[100272, 2723, 6133, 144, 155, "Output"]
}, Closed]],
Cell[106420, 2870, 341, 9, 38, "Text",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[106786, 2883, 362, 10, 56, "Input",
  CellTags->"AnomalousDimension"],
Cell[107151, 2895, 253, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[107465, 2908, 66, 1, 39, "ObjectName",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{
Cell[107556, 2913, 66, 1, 49, "Subsubsection",
  CellTags->"AntiCommutator"],
Cell[107625, 2916, 146, 4, 39, "Text",
  CellTags->"AntiCommutator"],
Cell[107774, 2922, 642, 24, 39, "Text",
  CellTags->"AntiCommutator"]
}, Closed]],

Cell[CellGroupData[{
Cell[108453, 2951, 63, 1, 49, "Subsubsection",
  CellTags->"AntiCommutator"],
Cell[108519, 2954, 210, 9, 70, "Text",
  CellTags->"AntiCommutator"],
Cell[108732, 2965, 182, 4, 70, "Input",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{
Cell[108939, 2973, 176, 4, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[109118, 2979, 256, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[109411, 2990, 153, 3, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[109567, 2995, 193, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[109797, 3005, 347, 9, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[110147, 3016, 526, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[110710, 3037, 340, 9, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[111053, 3048, 335, 9, 70, "Output"]
}, Closed]],
Cell[111403, 3060, 261, 6, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[111667, 3068, 105, 1, 70, "Text",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{
Cell[111797, 3073, 215, 5, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[112015, 3080, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[112160, 3087, 220, 5, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[112383, 3094, 227, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[112647, 3106, 304, 8, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[112954, 3116, 135, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[113126, 3124, 300, 7, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[113429, 3133, 351, 10, 70, "Output"]
}, Closed]],
Cell[113795, 3146, 283, 6, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[114081, 3154, 214, 5, 70, "Input",
  CellTags->"AntiCommutator"],
Cell[114298, 3161, 276, 7, 70, "Input",
  CellTags->"AntiCommutator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[114623, 3174, 66, 1, 39, "ObjectName",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{
Cell[114714, 3179, 66, 1, 49, "Subsubsection",
  CellTags->"AntiQuarkField"],
Cell[114783, 3182, 200, 5, 39, "Text",
  CellTags->"AntiQuarkField"],
Cell[114986, 3189, 336, 14, 39, "Text",
  CellTags->"AntiQuarkField"]
}, Closed]],

Cell[CellGroupData[{
Cell[115359, 3208, 63, 1, 49, "Subsubsection",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{
Cell[115447, 3213, 119, 2, 70, "Input",
  CellTags->"AntiQuarkField"],
Cell[115569, 3217, 140, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[115770, 3227, 66, 1, 39, "ObjectName",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{
Cell[115861, 3232, 66, 1, 49, "Subsubsection",
  CellTags->"AntiSymmetrize"],
Cell[115930, 3235, 154, 4, 39, "Text",
  CellTags->"AntiSymmetrize"],
Cell[116087, 3241, 197, 8, 39, "Text",
  CellTags->"AntiSymmetrize"]
}, Closed]],

Cell[CellGroupData[{
Cell[116321, 3254, 63, 1, 49, "Subsubsection",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{
Cell[116409, 3259, 301, 8, 70, "Input",
  CellTags->"AntiSymmetrize"],
Cell[116713, 3269, 375, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[117125, 3285, 321, 8, 70, "Input",
  CellTags->"AntiSymmetrize"],
Cell[117449, 3295, 767, 19, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[118277, 3321, 48, 1, 39, "ObjectName",
  CellTags->"Anti5"],

Cell[CellGroupData[{
Cell[118350, 3326, 57, 1, 49, "Subsubsection",
  CellTags->"Anti5"],
Cell[118410, 3329, 528, 16, 56, "Text",
  CellTags->"Anti5"],
Cell[118941, 3347, 176, 7, 39, "Text",
  CellTags->"Anti5"],
Cell[119120, 3356, 456, 19, 39, "Text",
  CellTags->"Anti5"]
}, Closed]],

Cell[CellGroupData[{
Cell[119613, 3380, 54, 1, 49, "Subsubsection",
  CellTags->"Anti5"],

Cell[CellGroupData[{
Cell[119692, 3385, 191, 5, 70, "Input",
  CellTags->"Anti5"],
Cell[119886, 3392, 207, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[120130, 3402, 130, 3, 70, "Input",
  CellTags->"Anti5"],
Cell[120263, 3407, 354, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[120654, 3422, 181, 5, 70, "Input",
  CellTags->"Anti5"],
Cell[120838, 3429, 318, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[121193, 3443, 236, 6, 70, "Input",
  CellTags->"Anti5"],
Cell[121432, 3451, 358, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[121827, 3465, 157, 4, 70, "Input",
  CellTags->"Anti5"],
Cell[121987, 3471, 800, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[122824, 3500, 165, 4, 70, "Input",
  CellTags->"Anti5"],
Cell[122992, 3506, 800, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[123829, 3535, 188, 5, 70, "Input",
  CellTags->"Anti5"],
Cell[124020, 3542, 800, 24, 70, "Output"]
}, Closed]],
Cell[124835, 3569, 381, 14, 70, "Text",
  CellTags->"Anti5"],

Cell[CellGroupData[{
Cell[125241, 3587, 159, 4, 70, "Input",
  CellTags->"Anti5"],
Cell[125403, 3593, 212, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[125652, 3603, 130, 3, 70, "Input",
  CellTags->"Anti5"],
Cell[125785, 3608, 354, 10, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[126200, 3625, 50, 1, 39, "ObjectName",
  CellTags->"Apart1"],

Cell[CellGroupData[{
Cell[126275, 3630, 58, 1, 49, "Subsubsection",
  CellTags->"Apart1"],
Cell[126336, 3633, 170, 4, 39, "Text",
  CellTags->"Apart1"],
Cell[126509, 3639, 298, 14, 39, "Text",
  CellTags->"Apart1"]
}, Closed]],

Cell[CellGroupData[{
Cell[126844, 3658, 55, 1, 49, "Subsubsection",
  CellTags->"Apart1"],
Cell[126902, 3661, 10585, 269, 70, "Input",
  CellTags->"Apart1"],

Cell[CellGroupData[{
Cell[137512, 3934, 259, 7, 70, "Input",
  CellTags->"Apart1"],
Cell[137774, 3943, 231, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[138042, 3954, 258, 7, 70, "Input",
  CellTags->"Apart1"],
Cell[138303, 3963, 230, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[138570, 3974, 158, 4, 70, "Input",
  CellTags->"Apart1"],
Cell[138731, 3980, 108, 2, 70, "Output"]
}, Closed]],
Cell[138854, 3985, 171, 4, 70, "Input",
  CellTags->"Apart1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[139074, 3995, 50, 1, 39, "ObjectName",
  CellTags->"Apart2"],

Cell[CellGroupData[{
Cell[139149, 4000, 58, 1, 49, "Subsubsection",
  CellTags->"Apart2"],
Cell[139210, 4003, 111, 4, 39, "Text",
  CellTags->"Apart2"],
Cell[139324, 4009, 325, 14, 39, "Text",
  CellTags->"Apart2"]
}, Closed]],

Cell[CellGroupData[{
Cell[139686, 4028, 55, 1, 49, "Subsubsection",
  CellTags->"Apart2"],

Cell[CellGroupData[{
Cell[139766, 4033, 316, 9, 70, "Input",
  CellTags->"Apart2"],
Cell[140085, 4044, 1518, 36, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[141640, 4085, 133, 3, 70, "Input",
  CellTags->"Apart2"],
Cell[141776, 4090, 907, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[142720, 4120, 173, 4, 70, "Input",
  CellTags->"Apart2"],
Cell[142896, 4126, 624, 18, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[143581, 4151, 50, 1, 39, "ObjectName",
  CellTags->"Apart3"],

Cell[CellGroupData[{
Cell[143656, 4156, 58, 1, 49, "Subsubsection",
  CellTags->"Apart3"],
Cell[143717, 4159, 123, 4, 39, "Text",
  CellTags->"Apart3"],
Cell[143843, 4165, 184, 9, 39, "Text",
  CellTags->"Apart3"]
}, Closed]],

Cell[CellGroupData[{
Cell[144064, 4179, 55, 1, 49, "Subsubsection",
  CellTags->"Apart3"],
Cell[144122, 4182, 10593, 269, 70, "Input",
  CellTags->"Apart3"],

Cell[CellGroupData[{
Cell[154740, 4455, 259, 7, 70, "Input",
  CellTags->"Apart3"],
Cell[155002, 4464, 231, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[155270, 4475, 258, 7, 70, "Input",
  CellTags->"Apart3"],
Cell[155531, 4484, 227, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[155795, 4495, 158, 4, 70, "Input",
  CellTags->"Apart3"],
Cell[155956, 4501, 108, 2, 70, "Output"]
}, Closed]],
Cell[156079, 4506, 171, 4, 70, "Input",
  CellTags->"Apart3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[156299, 4516, 66, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[156390, 4522, 54, 1, 49, "Subsubsection",
  CellTags->"A0"],
Cell[156447, 4525, 381, 10, 37, "Text",
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[156853, 4539, 135, 3, 56, "Input",
  CellTags->"A0"],
Cell[156991, 4544, 184, 5, 70, "Output"]
}, Closed]],
Cell[157190, 4552, 480, 24, 35, "Text",
  CellTags->"A0"]
}, Closed]],

Cell[CellGroupData[{
Cell[157707, 4581, 51, 1, 49, "Subsubsection",
  CellTags->"A0"],
Cell[157761, 4584, 270, 11, 39, "Text",
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[158056, 4599, 152, 4, 57, "Input",
  CellTags->"A0"],
Cell[158211, 4605, 355, 10, 39, "Output"]
}, Closed]],
Cell[158581, 4618, 229, 6, 52, "Input",
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[158835, 4628, 152, 4, 57, "Input",
  CellTags->"A0"],
Cell[158990, 4634, 381, 10, 39, "Output"]
}, Closed]],
Cell[159386, 4647, 230, 6, 52, "Input",
  CellTags->"A0"],
Cell[159619, 4655, 241, 8, 35, "Text",
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[159885, 4667, 124, 3, 56, "Input",
  CellTags->"A0"],
Cell[160012, 4672, 107, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[160156, 4679, 197, 5, 52, "Input",
  CellTags->"A0"],
Cell[160356, 4686, 107, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[160524, 4695, 74, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"A0ToB0"],

Cell[CellGroupData[{
Cell[160623, 4701, 58, 1, 49, "Subsubsection",
  CellTags->"A0ToB0"],
Cell[160684, 4704, 712, 27, 39, "Text",
  CellTags->"A0ToB0"],

Cell[CellGroupData[{
Cell[161421, 4735, 139, 3, 56, "Input",
  CellTags->"A0ToB0"],
Cell[161563, 4740, 185, 5, 37, "Output"]
}, Closed]],
Cell[161763, 4748, 586, 29, 35, "Text",
  CellTags->"A0ToB0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[162398, 4783, 80, 1, 39, "ObjectName",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{
Cell[162503, 4788, 73, 1, 49, "Subsubsection",
  CellTags->"BackgroundGluonVertex"],
Cell[162579, 4791, 1050, 19, 141, "Text",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{
Cell[163654, 4814, 169, 3, 56, "Input",
  CellTags->"BackgroundGluonVertex"],
Cell[163826, 4819, 351, 9, 70, "Output"]
}, Closed]],
Cell[164192, 4831, 214, 9, 35, "Text",
  CellTags->"BackgroundGluonVertex"]
}, Closed]],

Cell[CellGroupData[{
Cell[164443, 4845, 70, 1, 49, "Subsubsection",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{
Cell[164538, 4850, 428, 11, 70, "Input",
  CellTags->"BackgroundGluonVertex"],
Cell[164969, 4863, 3900, 102, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[168906, 4970, 513, 12, 70, "Input",
  CellTags->"BackgroundGluonVertex"],
Cell[169422, 4984, 10929, 269, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[180388, 5258, 482, 11, 70, "Input",
  CellTags->"BackgroundGluonVertex"],
Cell[180873, 5271, 4148, 107, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[185058, 5383, 569, 13, 70, "Input",
  CellTags->"BackgroundGluonVertex"],
Cell[185630, 5398, 11101, 271, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[196792, 5676, 52, 1, 39, "ObjectName",
  CellTags->"Bracket"],

Cell[CellGroupData[{
Cell[196869, 5681, 59, 1, 49, "Subsubsection",
  CellTags->"Bracket"],
Cell[196931, 5684, 73, 1, 39, "Text",
  CellTags->"Bracket"],
Cell[197007, 5687, 194, 9, 39, "Text",
  CellTags->"Bracket"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[197250, 5702, 76, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"BReduce"],

Cell[CellGroupData[{
Cell[197351, 5708, 59, 1, 49, "Subsubsection",
  CellTags->"BReduce"],
Cell[197413, 5711, 149, 4, 39, "Text",
  CellTags->"BReduce"],
Cell[197565, 5717, 587, 29, 39, "Text",
  CellTags->"BReduce"]
}, Closed]],

Cell[CellGroupData[{
Cell[198189, 5751, 56, 1, 49, "Subsubsection",
  CellTags->"BReduce"],
Cell[198248, 5754, 275, 11, 39, "Text",
  CellTags->"BReduce"],

Cell[CellGroupData[{
Cell[198548, 5769, 166, 4, 56, "Input",
  CellTags->"BReduce"],
Cell[198717, 5775, 194, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[198948, 5785, 108, 2, 39, "Input"],
Cell[199059, 5789, 320, 8, 37, "Output"]
}, Closed]],
Cell[199394, 5800, 95, 3, 35, "Text",
  CellTags->"BReduce"],

Cell[CellGroupData[{
Cell[199514, 5807, 220, 5, 56, "Input",
  CellTags->"BReduce"],
Cell[199737, 5814, 359, 10, 53, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[200157, 5831, 42, 1, 39, "ObjectName",
  CellTags->"B0"],

Cell[CellGroupData[{
Cell[200224, 5836, 54, 1, 49, "Subsubsection",
  CellTags->"B0"],
Cell[200281, 5839, 403, 9, 56, "Text",
  CellTags->"B0"],

Cell[CellGroupData[{
Cell[200709, 5852, 131, 3, 56, "Input",
  CellTags->"B0"],
Cell[200843, 5857, 322, 8, 70, "Output"]
}, Closed]],
Cell[201180, 5868, 486, 24, 35, "Text",
  CellTags->"B0"]
}, Closed]],

Cell[CellGroupData[{
Cell[201703, 5897, 51, 1, 49, "Subsubsection",
  CellTags->"B0"],

Cell[CellGroupData[{
Cell[201779, 5902, 288, 8, 70, "Input",
  CellTags->"B0"],
Cell[202070, 5912, 382, 11, 70, "Output"]
}, Closed]],
Cell[202467, 5926, 287, 7, 70, "Input",
  CellTags->"B0"],

Cell[CellGroupData[{
Cell[202779, 5937, 192, 5, 70, "Input",
  CellTags->"B0"],
Cell[202974, 5944, 290, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[203301, 5957, 222, 6, 70, "Input",
  CellTags->"B0"],
Cell[203526, 5965, 290, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[203853, 5978, 220, 6, 70, "Input",
  CellTags->"B0"],
Cell[204076, 5986, 248, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[204385, 5999, 74, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"B0Real"],

Cell[CellGroupData[{
Cell[204484, 6005, 58, 1, 49, "Subsubsection",
  CellTags->"B0Real"],
Cell[204545, 6008, 189, 5, 39, "Text",
  CellTags->"B0Real"],
Cell[204737, 6015, 165, 8, 39, "Text",
  CellTags->"B0Real"]
}, Closed]],

Cell[CellGroupData[{
Cell[204939, 6028, 55, 1, 49, "Subsubsection",
  CellTags->"B0Real"],
Cell[204997, 6031, 84, 1, 39, "Text",
  CellTags->"B0Real"],

Cell[CellGroupData[{
Cell[205106, 6036, 165, 4, 56, "Input",
  CellTags->"B0Real"],
Cell[205274, 6042, 194, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[205505, 6052, 130, 3, 52, "Input",
  CellTags->"B0Real"],
Cell[205638, 6057, 320, 8, 37, "Output"]
}, Closed]],
Cell[205973, 6068, 93, 3, 35, "Text",
  CellTags->"B0Real"],

Cell[CellGroupData[{
Cell[206091, 6075, 218, 5, 56, "Input",
  CellTags->"B0Real"],
Cell[206312, 6082, 227, 6, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[206600, 6095, 78, 2, 39, "ObjectName",
  PageBreakAbove->True,
  CellTags->"B0Unique"],

Cell[CellGroupData[{
Cell[206703, 6101, 60, 1, 49, "Subsubsection",
  CellTags->"B0Unique"],
Cell[206766, 6104, 187, 5, 39, "Text",
  CellTags->"B0Unique"],
Cell[206956, 6111, 167, 8, 39, "Text",
  CellTags->"B0Unique"]
}, Closed]],

Cell[CellGroupData[{
Cell[207160, 6124, 57, 1, 49, "Subsubsection",
  CellTags->"B0Unique"],
Cell[207220, 6127, 74, 1, 39, "Text",
  CellTags->"B0Unique"],

Cell[CellGroupData[{
Cell[207319, 6132, 167, 4, 56, "Input",
  CellTags->"B0Unique"],
Cell[207489, 6138, 227, 6, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[207753, 6149, 132, 3, 52, "Input",
  CellTags->"B0Unique"],
Cell[207888, 6154, 320, 8, 37, "Output"]
}, Closed]],
Cell[208223, 6165, 89, 3, 35, "Text",
  CellTags->"B0Unique"],

Cell[CellGroupData[{
Cell[208337, 6172, 223, 5, 56, "Input",
  CellTags->"B0Unique"],
Cell[208563, 6179, 194, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[208818, 6191, 44, 1, 39, "ObjectName",
  CellTags->"B00"],

Cell[CellGroupData[{
Cell[208887, 6196, 55, 1, 49, "Subsubsection",
  CellTags->"B00"],
Cell[208945, 6199, 303, 8, 56, "Text",
  CellTags->"B00"],

Cell[CellGroupData[{
Cell[209273, 6211, 133, 3, 56, "Input",
  CellTags->"B00"],
Cell[209409, 6216, 186, 5, 70, "Output"]
}, Closed]],
Cell[209610, 6224, 379, 19, 35, "Text",
  CellTags->"B00"]
}, Closed]],

Cell[CellGroupData[{
Cell[210026, 6248, 52, 1, 49, "Subsubsection",
  CellTags->"B00"],
Cell[210081, 6251, 215, 7, 70, "Text",
  CellTags->"B00"],

Cell[CellGroupData[{
Cell[210321, 6262, 126, 3, 70, "Input",
  CellTags->"B00"],
Cell[210450, 6267, 183, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[210670, 6277, 259, 7, 70, "Input",
  CellTags->"B00"],
Cell[210932, 6286, 3724, 90, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[214693, 6381, 259, 7, 70, "Input",
  CellTags->"B00"],
Cell[214955, 6390, 1961, 54, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[216953, 6449, 314, 8, 70, "Input",
  CellTags->"B00"],
Cell[217270, 6459, 751, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[218058, 6486, 224, 6, 70, "Input",
  CellTags->"B00"],
Cell[218285, 6494, 878, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[219200, 6523, 302, 8, 70, "Input",
  CellTags->"B00"],
Cell[219505, 6533, 912, 25, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[220478, 6565, 42, 1, 39, "ObjectName",
  CellTags->"B1"],

Cell[CellGroupData[{
Cell[220545, 6570, 54, 1, 49, "Subsubsection",
  CellTags->"B1"],
Cell[220602, 6573, 243, 7, 39, "Text",
  CellTags->"B1"],

Cell[CellGroupData[{
Cell[220870, 6584, 131, 3, 56, "Input",
  CellTags->"B1"],
Cell[221004, 6589, 186, 5, 70, "Output"]
}, Closed]],
Cell[221205, 6597, 612, 29, 35, "Text",
  CellTags->"B1"]
}, Closed]],

Cell[CellGroupData[{
Cell[221854, 6631, 51, 1, 49, "Subsubsection",
  CellTags->"B1"],

Cell[CellGroupData[{
Cell[221930, 6636, 257, 7, 70, "Input",
  CellTags->"B1"],
Cell[222190, 6645, 1502, 41, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[223729, 6691, 312, 8, 70, "Input",
  CellTags->"B1"],
Cell[224044, 6701, 382, 11, 70, "Output"]
}, Closed]],
Cell[224441, 6715, 231, 6, 70, "Input",
  CellTags->"B1"],

Cell[CellGroupData[{
Cell[224697, 6725, 257, 7, 70, "Input",
  CellTags->"B1"],
Cell[224957, 6734, 480, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[225474, 6753, 222, 6, 70, "Input",
  CellTags->"B1"],
Cell[225699, 6761, 503, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[226239, 6780, 192, 5, 70, "Input",
  CellTags->"B1"],
Cell[226434, 6787, 503, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[226974, 6806, 287, 7, 70, "Input",
  CellTags->"B1"],
Cell[227264, 6815, 966, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[228267, 6844, 336, 9, 70, "Input",
  CellTags->"B1"],
Cell[228606, 6855, 523, 14, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[229190, 6876, 44, 1, 39, "ObjectName",
  CellTags->"B11"],

Cell[CellGroupData[{
Cell[229259, 6881, 55, 1, 49, "Subsubsection",
  CellTags->"B11"],
Cell[229317, 6884, 542, 17, 56, "Text",
  CellTags->"B11"],

Cell[CellGroupData[{
Cell[229884, 6905, 133, 3, 56, "Input",
  CellTags->"B11"],
Cell[230020, 6910, 186, 5, 70, "Output"]
}, Closed]],
Cell[230221, 6918, 484, 24, 35, "Text",
  CellTags->"B11"]
}, Closed]],

Cell[CellGroupData[{
Cell[230742, 6947, 52, 1, 49, "Subsubsection",
  CellTags->"B11"],
Cell[230797, 6950, 233, 7, 70, "Text",
  CellTags->"B11"],

Cell[CellGroupData[{
Cell[231055, 6961, 126, 3, 70, "Input",
  CellTags->"B11"],
Cell[231184, 6966, 183, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[231404, 6976, 259, 7, 70, "Input",
  CellTags->"B11"],
Cell[231666, 6985, 3915, 96, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[235618, 7086, 259, 7, 70, "Input",
  CellTags->"B11"],
Cell[235880, 7095, 1636, 46, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[237553, 7146, 208, 5, 70, "Input",
  CellTags->"B11"],
Cell[237764, 7153, 187, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[237988, 7163, 259, 7, 70, "Input",
  CellTags->"B11"],
Cell[238250, 7172, 751, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[239038, 7199, 207, 5, 70, "Input",
  CellTags->"B11"],
Cell[239248, 7206, 186, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[239471, 7216, 224, 6, 70, "Input",
  CellTags->"B11"],
Cell[239698, 7224, 310, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[240045, 7237, 302, 8, 70, "Input",
  CellTags->"B11"],
Cell[240350, 7247, 342, 9, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[240753, 7263, 42, 1, 39, "ObjectName",
  CellTags->"CA"],

Cell[CellGroupData[{
Cell[240820, 7268, 54, 1, 49, "Subsubsection",
  CellTags->"CA"],
Cell[240877, 7271, 208, 9, 39, "Text",
  CellTags->"CA"],
Cell[241088, 7282, 297, 14, 39, "Text",
  CellTags->"CA"]
}, Closed]],

Cell[CellGroupData[{
Cell[241422, 7301, 51, 1, 49, "Subsubsection",
  CellTags->"CA"],

Cell[CellGroupData[{
Cell[241498, 7306, 95, 2, 70, "Input",
  CellTags->"CA"],
Cell[241596, 7310, 134, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[241767, 7318, 210, 5, 70, "Input",
  CellTags->"CA"],
Cell[241980, 7325, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[242125, 7332, 97, 2, 70, "Input",
  CellTags->"CA"],
Cell[242225, 7336, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[242394, 7345, 46, 1, 39, "ObjectName",
  CellTags->"Calc"],

Cell[CellGroupData[{
Cell[242465, 7350, 56, 1, 49, "Subsubsection",
  CellTags->"Calc"],
Cell[242524, 7353, 234, 5, 56, "Text",
  CellTags->"Calc"],
Cell[242761, 7360, 440, 19, 39, "Text",
  CellTags->"Calc"]
}, Closed]],

Cell[CellGroupData[{
Cell[243238, 7384, 62, 1, 50, "Subsubsection",
  CellTags->"Calc"],
Cell[243303, 7387, 382, 13, 40, "Text",
  CellTags->"Calc"],

Cell[CellGroupData[{
Cell[243710, 7404, 347, 9, 56, "Input",
  CellTags->"Calc"],
Cell[244060, 7415, 164, 4, 70, "Output"]
}, Closed]],
Cell[244239, 7422, 244, 9, 35, "Text",
  CellTags->"Calc"],

Cell[CellGroupData[{
Cell[244508, 7435, 312, 8, 56, "Input",
  CellTags->"Calc"],
Cell[244823, 7445, 466, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[245326, 7464, 395, 11, 52, "Input",
  CellTags->"Calc"],
Cell[245724, 7477, 618, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[246379, 7500, 129, 3, 52, "Input",
  CellTags->"Calc"],
Cell[246511, 7505, 776, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[247324, 7532, 258, 6, 52, "Input",
  CellTags->"Calc"],
Cell[247585, 7540, 1492, 44, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[249114, 7589, 476, 14, 52, "Input",
  CellTags->"Calc"],
Cell[249593, 7605, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[249762, 7614, 95, 1, 39, "ObjectName",
  CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{
Cell[249882, 7619, 72, 1, 49, "Subsubsection",
  CellTags->"CalculateCounterTerm"],
Cell[249957, 7622, 111, 1, 39, "Text",
  CellTags->"CalculateCounterTerm"]
}, Closed]],

Cell[CellGroupData[{
Cell[250105, 7628, 78, 1, 50, "Subsubsection",
  CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{
Cell[250208, 7633, 149, 3, 56, "Input",
  CellTags->"CalculateCounterTerm"],
Cell[250360, 7638, 133, 2, 37, "Print",
  CellTags->"Info3241266069-5635180"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[250554, 7647, 54, 1, 39, "ObjectName",
  CellTags->"CancelQP"],

Cell[CellGroupData[{
Cell[250633, 7652, 60, 1, 49, "Subsubsection",
  CellTags->"CancelQP"],
Cell[250696, 7655, 335, 13, 39, "Text",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[251068, 7673, 66, 1, 50, "Subsubsection",
  CellTags->"CancelQP"],

Cell[CellGroupData[{
Cell[251159, 7678, 706, 20, 56, "Input",
  CellTags->"CancelQP"],
Cell[251868, 7700, 945, 26, 71, "Output",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[252850, 7731, 218, 5, 52, "Input",
  CellTags->"CancelQP"],
Cell[253071, 7738, 2930, 74, 109, "Output",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[256038, 7817, 219, 5, 52, "Input",
  CellTags->"CancelQP"],
Cell[256260, 7824, 4327, 107, 109, "Output",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[260624, 7936, 690, 19, 52, "Input",
  CellTags->"CancelQP"],
Cell[261317, 7957, 1217, 36, 71, "Output",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[262571, 7998, 218, 5, 52, "Input",
  CellTags->"CancelQP"],
Cell[262792, 8005, 9798, 230, 123, "Output",
  CellTags->"CancelQP"]
}, Closed]],

Cell[CellGroupData[{
Cell[272627, 8240, 219, 5, 52, "Input",
  CellTags->"CancelQP"],
Cell[272849, 8247, 11087, 261, 221, "Output",
  CellTags->"CancelQP"]
}, Closed]],
Cell[283951, 8511, 133, 3, 52, "Input",
  CellTags->"CancelQP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[284133, 8520, 50, 1, 39, "ObjectName",
  CellTags->"Cases2"],

Cell[CellGroupData[{
Cell[284208, 8525, 58, 1, 49, "Subsubsection",
  CellTags->"Cases2"],
Cell[284269, 8528, 294, 6, 56, "Text",
  CellTags->"Cases2"]
}, Closed]],

Cell[CellGroupData[{
Cell[284600, 8539, 64, 1, 50, "Subsubsection",
  CellTags->"Cases2"],

Cell[CellGroupData[{
Cell[284689, 8544, 374, 10, 70, "Input",
  CellTags->"Cases2"],
Cell[285066, 8556, 322, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[285425, 8570, 390, 11, 70, "Input",
  CellTags->"Cases2"],
Cell[285818, 8583, 326, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[286181, 8597, 522, 14, 70, "Input",
  CellTags->"Cases2"],
Cell[286706, 8613, 415, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[287158, 8629, 397, 10, 70, "Input",
  CellTags->"Cases2"],
Cell[287558, 8641, 673, 19, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[288292, 8667, 42, 1, 39, "ObjectName",
  CellTags->"CF"],

Cell[CellGroupData[{
Cell[288359, 8672, 54, 1, 49, "Subsubsection",
  CellTags->"CF"],
Cell[288416, 8675, 308, 13, 39, "Text",
  CellTags->"CF"],
Cell[288727, 8690, 297, 14, 39, "Text",
  CellTags->"CF"]
}, Closed]],

Cell[CellGroupData[{
Cell[289061, 8709, 51, 1, 49, "Subsubsection",
  CellTags->"CF"],

Cell[CellGroupData[{
Cell[289137, 8714, 95, 2, 56, "Input",
  CellTags->"CF"],
Cell[289235, 8718, 134, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[289406, 8726, 215, 5, 52, "Input",
  CellTags->"CF"],
Cell[289624, 8733, 223, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[289884, 8744, 97, 2, 52, "Input",
  CellTags->"CF"],
Cell[289984, 8748, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[290129, 8755, 219, 5, 52, "Input",
  CellTags->"CF"],
Cell[290351, 8762, 134, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[290546, 8772, 68, 1, 39, "ObjectName",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[290639, 8777, 67, 1, 49, "Subsubsection",
  CellTags->"ChangeDimension"],
Cell[290709, 8780, 211, 5, 56, "Text",
  CellTags->"ChangeDimension"],
Cell[290923, 8787, 562, 24, 39, "Text",
  CellTags->"ChangeDimension"]
}, Closed]],

Cell[CellGroupData[{
Cell[291522, 8816, 64, 1, 49, "Subsubsection",
  CellTags->"ChangeDimension"],
Cell[291589, 8819, 220, 5, 56, "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[291834, 8828, 254, 6, 56, "Input",
  CellTags->"ChangeDimension"],
Cell[292091, 8836, 221, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[292349, 8848, 183, 4, 52, "Input",
  CellTags->"ChangeDimension"],
Cell[292535, 8854, 221, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[292793, 8866, 144, 3, 52, "Input",
  CellTags->"ChangeDimension"],
Cell[292940, 8871, 295, 7, 70, "Output"]
}, Closed]],
Cell[293250, 8881, 122, 4, 35, "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[293397, 8889, 242, 6, 56, "Input",
  CellTags->"ChangeDimension"],
Cell[293642, 8897, 233, 5, 70, "Output"]
}, Closed]],
Cell[293890, 8905, 106, 1, 35, "Text",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[294021, 8910, 582, 15, 56, "Input",
  CellTags->"ChangeDimension"],
Cell[294606, 8927, 707, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[295350, 8950, 185, 4, 52, "Input",
  CellTags->"ChangeDimension"],
Cell[295538, 8956, 240, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[295815, 8968, 265, 6, 52, "Input",
  CellTags->"ChangeDimension"],
Cell[296083, 8976, 240, 7, 70, "Output"]
}, Closed]],
Cell[296338, 8986, 135, 4, 35, "Text",
  CellTags->"ChangeDimension"],
Cell[296476, 8992, 251, 6, 56, "Input",
  CellTags->"ChangeDimension"],
Cell[296730, 9000, 220, 5, 56, "Text",
  CellTags->"ChangeDimension"],
Cell[296953, 9007, 244, 6, 56, "Input",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[297222, 9017, 262, 6, 70, "Input",
  CellTags->"ChangeDimension"],
Cell[297487, 9025, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[298189, 9050, 219, 5, 70, "Input",
  CellTags->"ChangeDimension"],
Cell[298411, 9057, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[299113, 9082, 213, 5, 70, "Input",
  CellTags->"ChangeDimension"],
Cell[299329, 9089, 342, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[299708, 9104, 173, 4, 70, "Input",
  CellTags->"ChangeDimension"],
Cell[299884, 9110, 131, 3, 70, "Output"]
}, Closed]],
Cell[300030, 9116, 244, 6, 70, "Input",
  CellTags->"ChangeDimension"],
Cell[300277, 9124, 194, 5, 70, "Input",
  CellTags->"ChangeDimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[300520, 9135, 84, 1, 39, "ObjectName",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{
Cell[300629, 9140, 75, 1, 49, "Subsubsection",
  CellTags->"ChargeConjugationMatrix"],
Cell[300707, 9143, 182, 6, 39, "Text",
  CellTags->"ChargeConjugationMatrix"],
Cell[300892, 9151, 261, 9, 39, "Text",
  CellTags->"ChargeConjugationMatrix"]
}, Closed]],

Cell[CellGroupData[{
Cell[301190, 9165, 72, 1, 49, "Subsubsection",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{
Cell[301287, 9170, 250, 5, 70, "Input",
  CellTags->"ChargeConjugationMatrix"],
Cell[301540, 9177, 347, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[301924, 9192, 148, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrix"],
Cell[302075, 9197, 285, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[302397, 9210, 245, 5, 70, "Input",
  CellTags->"ChargeConjugationMatrix"],
Cell[302645, 9217, 237, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[302919, 9228, 148, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrix"],
Cell[303070, 9233, 198, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[303305, 9243, 162, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrix"],
Cell[303470, 9248, 130, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[303661, 9258, 90, 1, 39, "ObjectName",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{
Cell[303776, 9263, 78, 1, 49, "Subsubsection",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[303857, 9266, 198, 5, 39, "Text",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[304058, 9273, 255, 9, 39, "Text",
  CellTags->"ChargeConjugationMatrixInv"]
}, Closed]],

Cell[CellGroupData[{
Cell[304350, 9287, 75, 1, 49, "Subsubsection",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{
Cell[304450, 9292, 253, 5, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[304706, 9299, 347, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[305090, 9314, 151, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[305244, 9319, 285, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[305566, 9332, 248, 5, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[305817, 9339, 237, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[306091, 9350, 151, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[306245, 9355, 198, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[306480, 9365, 195, 4, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[306678, 9371, 190, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[306905, 9381, 147, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[307055, 9386, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[307200, 9393, 168, 3, 70, "Input",
  CellTags->"ChargeConjugationMatrixInv"],
Cell[307371, 9398, 130, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[307562, 9408, 62, 1, 39, "ObjectName",
  CellTags->"CheckContext"],

Cell[CellGroupData[{
Cell[307649, 9413, 64, 1, 49, "Subsubsection",
  CellTags->"CheckContext"],
Cell[307716, 9416, 165, 4, 39, "Text",
  CellTags->"CheckContext"],
Cell[307884, 9422, 343, 14, 39, "Text",
  CellTags->"CheckContext"]
}, Closed]],

Cell[CellGroupData[{
Cell[308264, 9441, 61, 1, 49, "Subsubsection",
  CellTags->"CheckContext"],
Cell[308328, 9444, 143, 4, 39, "Text",
  CellTags->"CheckContext"],

Cell[CellGroupData[{
Cell[308496, 9452, 164, 3, 56, "Input",
  CellTags->"CheckContext"],
Cell[308663, 9457, 110, 2, 37, "Output"]
}, Closed]],
Cell[308788, 9462, 88, 1, 35, "Text",
  CellTags->"CheckContext"],

Cell[CellGroupData[{
Cell[308901, 9467, 115, 2, 56, "Input",
  CellTags->"CheckContext"],
Cell[309019, 9471, 119, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[309175, 9478, 164, 3, 52, "Input",
  CellTags->"CheckContext"],
Cell[309342, 9483, 109, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[309512, 9492, 74, 1, 39, "ObjectName",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{
Cell[309611, 9497, 70, 1, 49, "Subsubsection",
  CellTags->"ChiralityProjector"],
Cell[309684, 9500, 577, 19, 39, "Text",
  CellTags->"ChiralityProjector"],
Cell[310264, 9521, 442, 19, 39, "Text",
  CellTags->"ChiralityProjector"]
}, Closed]],

Cell[CellGroupData[{
Cell[310743, 9545, 67, 1, 49, "Subsubsection",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{
Cell[310835, 9550, 316, 8, 70, "Input",
  CellTags->"ChiralityProjector"],
Cell[311154, 9560, 249, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[311440, 9572, 142, 3, 70, "Input",
  CellTags->"ChiralityProjector"],
Cell[311585, 9577, 249, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[311871, 9589, 233, 5, 70, "Input",
  CellTags->"ChiralityProjector"],
Cell[312107, 9596, 443, 13, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[312611, 9616, 54, 1, 39, "ObjectName",
  CellTags->"Chisholm"],

Cell[CellGroupData[{
Cell[312690, 9621, 60, 1, 49, "Subsubsection",
  CellTags->"Chisholm"],
Cell[312753, 9624, 143, 4, 39, "Text",
  CellTags->"Chisholm"],
Cell[312899, 9630, 339, 14, 39, "Text",
  CellTags->"Chisholm"]
}, Closed]],

Cell[CellGroupData[{
Cell[313275, 9649, 57, 1, 49, "Subsubsection",
  CellTags->"Chisholm"],

Cell[CellGroupData[{
Cell[313357, 9654, 249, 6, 70, "Input",
  CellTags->"Chisholm"],
Cell[313609, 9662, 2730, 78, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[316376, 9745, 220, 5, 70, "Input",
  CellTags->"Chisholm"],
Cell[316599, 9752, 259, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[316895, 9763, 171, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[317069, 9769, 1887, 54, 70, "Output"]
}, Closed]],
Cell[318971, 9826, 78, 1, 70, "Text",
  CellTags->"Chisholm"],

Cell[CellGroupData[{
Cell[319074, 9831, 162, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[319239, 9837, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[319385, 9844, 171, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[319559, 9850, 1887, 54, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[321483, 9909, 162, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[321648, 9915, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[321794, 9922, 206, 5, 70, "Input",
  CellTags->"Chisholm"],
Cell[322003, 9929, 856, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[322896, 9958, 138, 3, 70, "Input",
  CellTags->"Chisholm"],
Cell[323037, 9963, 2187, 63, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[325261, 10031, 280, 7, 70, "Input",
  CellTags->"Chisholm"],
Cell[325544, 10040, 431, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[326012, 10055, 171, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[326186, 10061, 15974, 439, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[342197, 10505, 171, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[342371, 10511, 15974, 439, 70, "Output"]
}, Closed]],
Cell[358360, 10953, 86, 1, 70, "Text",
  CellTags->"Chisholm"],

Cell[CellGroupData[{
Cell[358471, 10958, 162, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[358636, 10964, 133, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[358806, 10972, 162, 4, 70, "Input",
  CellTags->"Chisholm"],
Cell[358971, 10978, 133, 3, 70, "Output"]
}, Closed]],
Cell[359119, 10984, 234, 6, 70, "Input",
  CellTags->"Chisholm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[359402, 10996, 66, 1, 39, "ObjectName",
  CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{
Cell[359493, 11001, 66, 1, 49, "Subsubsection",
  CellTags->"ChisholmSpinor"],
Cell[359562, 11004, 297, 6, 56, "Text",
  CellTags->"ChisholmSpinor"],
Cell[359862, 11012, 327, 14, 39, "Text",
  CellTags->"ChisholmSpinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[360226, 11031, 63, 1, 49, "Subsubsection",
  CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{
Cell[360314, 11036, 364, 9, 56, "Input",
  CellTags->"ChisholmSpinor"],
Cell[360681, 11047, 897, 23, 51, "Output",
  CellTags->"ChisholmSpinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[361615, 11075, 142, 3, 56, "Input",
  CellTags->"ChisholmSpinor"],
Cell[361760, 11080, 6642, 166, 119, "Output",
  CellTags->"ChisholmSpinor"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[368463, 11253, 76, 1, 39, "ObjectName",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{
Cell[368564, 11258, 71, 1, 49, "Subsubsection",
  CellTags->"ClearScalarProducts"],
Cell[368638, 11261, 146, 4, 39, "Text",
  CellTags->"ClearScalarProducts"],
Cell[368787, 11267, 526, 23, 39, "Text",
  CellTags->"ClearScalarProducts"]
}, Closed]],

Cell[CellGroupData[{
Cell[369350, 11295, 68, 1, 49, "Subsubsection",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{
Cell[369443, 11300, 236, 6, 70, "Input",
  CellTags->"ClearScalarProducts"],
Cell[369682, 11308, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[369855, 11316, 253, 6, 70, "Input",
  CellTags->"ClearScalarProducts"],
Cell[370111, 11324, 136, 3, 70, "Output"]
}, Closed]],
Cell[370262, 11330, 129, 2, 70, "Input",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{
Cell[370416, 11336, 253, 6, 70, "Input",
  CellTags->"ClearScalarProducts"],
Cell[370672, 11344, 183, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[370892, 11354, 180, 4, 70, "Input",
  CellTags->"ClearScalarProducts"],
Cell[371075, 11360, 183, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[371319, 11372, 58, 1, 39, "ObjectName",
  CellTags->"Collecting"],

Cell[CellGroupData[{
Cell[371402, 11377, 62, 1, 49, "Subsubsection",
  CellTags->"Collecting"],
Cell[371467, 11380, 208, 5, 39, "Text",
  CellTags->"Collecting"],
Cell[371678, 11387, 443, 18, 39, "Text",
  CellTags->"Collecting"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[372170, 11411, 54, 1, 39, "ObjectName",
  CellTags->"Collect2"],

Cell[CellGroupData[{
Cell[372249, 11416, 60, 1, 49, "Subsubsection",
  CellTags->"Collect2"],
Cell[372312, 11419, 614, 10, 90, "Text",
  CellTags->"Collect2"],

Cell[CellGroupData[{
Cell[372951, 11433, 143, 3, 56, "Input",
  CellTags->"Collect2"],
Cell[373097, 11438, 446, 10, 70, "Output"]
}, Closed]],
Cell[373558, 11451, 189, 9, 35, "Text",
  CellTags->"Collect2"]
}, Closed]],

Cell[CellGroupData[{
Cell[373784, 11465, 57, 1, 49, "Subsubsection",
  CellTags->"Collect2"],

Cell[CellGroupData[{
Cell[373866, 11470, 542, 14, 66, "Input",
  CellTags->"Collect2"],
Cell[374411, 11486, 950, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[375398, 11518, 222, 5, 52, "Input",
  CellTags->"Collect2"],
Cell[375623, 11525, 921, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[376581, 11555, 223, 5, 52, "Input",
  CellTags->"Collect2"],
Cell[376807, 11562, 950, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[377794, 11594, 652, 18, 53, "Input",
  CellTags->"Collect2"],
Cell[378449, 11614, 599, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[379085, 11635, 388, 10, 53, "Input",
  CellTags->"Collect2"],
Cell[379476, 11647, 998, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[380511, 11682, 140, 3, 52, "Input",
  CellTags->"Collect2"],
Cell[380654, 11687, 983, 27, 70, "Output"]
}, Closed]],
Cell[381652, 11717, 162, 4, 52, "Input",
  CellTags->"Collect2"],

Cell[CellGroupData[{
Cell[381839, 11725, 377, 10, 57, "Input",
  CellTags->"Collect2"],
Cell[382219, 11737, 1132, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[383388, 11772, 181, 4, 52, "Input",
  CellTags->"Collect2"],
Cell[383572, 11778, 784, 21, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[384417, 11806, 54, 1, 39, "ObjectName",
  CellTags->"Collect3"],

Cell[CellGroupData[{
Cell[384496, 11811, 60, 1, 49, "Subsubsection",
  CellTags->"Collect3"],
Cell[384559, 11814, 810, 23, 56, "Text",
  CellTags->"Collect3"],

Cell[CellGroupData[{
Cell[385394, 11841, 141, 3, 56, "Input",
  CellTags->"Collect3"],
Cell[385538, 11846, 258, 7, 37, "Output"]
}, Closed]],
Cell[385811, 11856, 309, 14, 35, "Text",
  CellTags->"Collect3"]
}, Closed]],

Cell[CellGroupData[{
Cell[386157, 11875, 57, 1, 49, "Subsubsection",
  CellTags->"Collect3"],

Cell[CellGroupData[{
Cell[386239, 11880, 650, 18, 57, "Input",
  CellTags->"Collect3"],
Cell[386892, 11900, 647, 18, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[387576, 11923, 360, 10, 53, "Input",
  CellTags->"Collect3"],
Cell[387939, 11935, 5558, 135, 137, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[393558, 12077, 62, 1, 39, "ObjectName",
  CellTags->"Combinations"],

Cell[CellGroupData[{
Cell[393645, 12082, 64, 1, 49, "Subsubsection",
  CellTags->"Combinations"],
Cell[393712, 12085, 245, 5, 56, "Text",
  CellTags->"Combinations"]
}, Closed]],

Cell[CellGroupData[{
Cell[393994, 12095, 61, 1, 49, "Subsubsection",
  CellTags->"Combinations"],

Cell[CellGroupData[{
Cell[394080, 12100, 302, 8, 56, "Input",
  CellTags->"Combinations"],
Cell[394385, 12110, 461, 14, 89, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[394907, 12131, 52, 1, 39, "ObjectName",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[394984, 12136, 59, 1, 49, "Subsubsection",
  CellTags->"Combine"],
Cell[395046, 12139, 316, 6, 56, "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[395387, 12149, 141, 3, 56, "Input",
  CellTags->"Combine"],
Cell[395531, 12154, 189, 5, 70, "Output"]
}, Closed]],
Cell[395735, 12162, 188, 9, 35, "Text",
  CellTags->"Combine"]
}, Closed]],

Cell[CellGroupData[{
Cell[395960, 12176, 56, 1, 49, "Subsubsection",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[396041, 12181, 363, 11, 70, "Input",
  CellTags->"Combine"],
Cell[396407, 12194, 360, 10, 70, "Output"]
}, Closed]],
Cell[396782, 12207, 125, 4, 70, "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[396932, 12215, 364, 11, 70, "Input",
  CellTags->"Combine"],
Cell[397299, 12228, 347, 9, 70, "Output"]
}, Closed]],
Cell[397661, 12240, 176, 4, 70, "Text",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[397862, 12248, 443, 12, 70, "Input",
  CellTags->"Combine"],
Cell[398308, 12262, 347, 9, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[398716, 12278, 64, 1, 39, "ObjectName",
  CellTags->"CombineGraphs"],

Cell[CellGroupData[{
Cell[398805, 12283, 65, 1, 49, "Subsubsection",
  CellTags->"CombineGraphs"],
Cell[398873, 12286, 87, 1, 39, "Text",
  CellTags->"CombineGraphs"],
Cell[398963, 12289, 197, 8, 39, "Text",
  CellTags->"CombineGraphs"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[399209, 12303, 58, 1, 39, "ObjectName",
  CellTags->"Commutator"],

Cell[CellGroupData[{
Cell[399292, 12308, 62, 1, 49, "Subsubsection",
  CellTags->"Commutator"],
Cell[399357, 12311, 139, 4, 39, "Text",
  CellTags->"Commutator"],
Cell[399499, 12317, 650, 24, 39, "Text",
  CellTags->"Commutator"]
}, Closed]],

Cell[CellGroupData[{
Cell[400186, 12346, 59, 1, 49, "Subsubsection",
  CellTags->"Commutator"],
Cell[400248, 12349, 199, 4, 70, "Input",
  CellTags->"Commutator"],

Cell[CellGroupData[{
Cell[400472, 12357, 168, 4, 70, "Input",
  CellTags->"Commutator"],
Cell[400643, 12363, 201, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[400881, 12373, 149, 3, 70, "Input",
  CellTags->"Commutator"],
Cell[401033, 12378, 193, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[401263, 12388, 304, 8, 70, "Input",
  CellTags->"Commutator"],
Cell[401570, 12398, 427, 11, 70, "Output"]
}, Closed]],
Cell[402012, 12412, 201, 4, 70, "Input",
  CellTags->"Commutator"],
Cell[402216, 12418, 69, 1, 70, "Text",
  CellTags->"Commutator"],
Cell[402288, 12421, 269, 6, 70, "Input",
  CellTags->"Commutator"],

Cell[CellGroupData[{
Cell[402582, 12431, 557, 15, 70, "Input",
  CellTags->"Commutator"],
Cell[403142, 12448, 887, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[404066, 12475, 142, 3, 70, "Input",
  CellTags->"Commutator"],
Cell[404211, 12480, 108, 2, 70, "Output"]
}, Closed]],
Cell[404334, 12485, 141, 3, 70, "Input",
  CellTags->"Commutator"],
Cell[404478, 12490, 191, 4, 70, "Input",
  CellTags->"Commutator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[404718, 12500, 74, 1, 39, "ObjectName",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{
Cell[404817, 12505, 70, 1, 49, "Subsubsection",
  CellTags->"CommutatorExplicit"],
Cell[404890, 12508, 158, 4, 39, "Text",
  CellTags->"CommutatorExplicit"],
Cell[405051, 12514, 319, 14, 39, "Text",
  CellTags->"CommutatorExplicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[405407, 12533, 67, 1, 49, "Subsubsection",
  CellTags->"CommutatorExplicit"],
Cell[405477, 12536, 207, 4, 70, "Input",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{
Cell[405709, 12544, 176, 4, 70, "Input",
  CellTags->"CommutatorExplicit"],
Cell[405888, 12550, 201, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[406126, 12560, 157, 3, 70, "Input",
  CellTags->"CommutatorExplicit"],
Cell[406286, 12565, 193, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[406516, 12575, 238, 6, 70, "Input",
  CellTags->"CommutatorExplicit"],
Cell[406757, 12583, 400, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[407194, 12599, 157, 3, 70, "Input",
  CellTags->"CommutatorExplicit"],
Cell[407354, 12604, 449, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[407840, 12622, 203, 5, 70, "Input",
  CellTags->"CommutatorExplicit"],
Cell[408046, 12629, 427, 11, 70, "Output"]
}, Closed]],
Cell[408488, 12643, 209, 4, 70, "Input",
  CellTags->"CommutatorExplicit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[408746, 12653, 66, 1, 39, "ObjectName",
  CellTags->"CompleteSquare"],

Cell[CellGroupData[{
Cell[408837, 12658, 66, 1, 49, "Subsubsection",
  CellTags->"CompleteSquare"],
Cell[408906, 12661, 1277, 82, 56, "Text",
  CellTags->"CompleteSquare"]
}, Closed]],

Cell[CellGroupData[{
Cell[410220, 12748, 63, 1, 49, "Subsubsection",
  CellTags->"CompleteSquare"],

Cell[CellGroupData[{
Cell[410308, 12753, 359, 10, 43, "Input"],
Cell[410670, 12765, 534, 14, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[411241, 12784, 206, 5, 52, "Input",
  CellTags->"CompleteSquare"],
Cell[411450, 12791, 885, 27, 55, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[412372, 12823, 220, 6, 52, "Input",
  CellTags->"CompleteSquare"],
Cell[412595, 12831, 107, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[412739, 12838, 186, 4, 52, "Input",
  CellTags->"CompleteSquare"],
Cell[412928, 12844, 645, 19, 55, "Output"]
}, Closed]],
Cell[413588, 12866, 168, 4, 52, "Input",
  CellTags->"CompleteSquare"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[413805, 12876, 70, 1, 39, "ObjectName",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{
Cell[413900, 12881, 68, 1, 49, "Subsubsection",
  CellTags->"ComplexConjugate"],
Cell[413971, 12884, 417, 8, 73, "Text",
  CellTags->"ComplexConjugate"],
Cell[414391, 12894, 482, 19, 39, "Text",
  CellTags->"ComplexConjugate"]
}, Closed]],

Cell[CellGroupData[{
Cell[414910, 12918, 65, 1, 49, "Subsubsection",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{
Cell[415000, 12923, 233, 5, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[415236, 12930, 352, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[415625, 12945, 149, 3, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[415777, 12950, 350, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[416164, 12963, 184, 4, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[416351, 12969, 264, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[416652, 12980, 153, 3, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[416808, 12985, 598, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[417443, 13007, 219, 5, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[417665, 13014, 735, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[418437, 13041, 153, 3, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[418593, 13046, 735, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[419365, 13073, 227, 5, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[419595, 13080, 509, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[420141, 13101, 327, 8, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[420471, 13111, 247, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[420755, 13121, 323, 8, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[421081, 13131, 550, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[421668, 13149, 153, 3, 70, "Input",
  CellTags->"ComplexConjugate"],
Cell[421824, 13154, 706, 17, 70, "Output"]
}, Closed]],
Cell[422545, 13174, 167, 4, 70, "Text",
  CellTags->"ComplexConjugate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[422761, 13184, 54, 1, 39, "ObjectName",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[422840, 13189, 60, 1, 49, "Subsubsection",
  CellTags->"Contract"],
Cell[422903, 13192, 414, 8, 73, "Text",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[423342, 13204, 143, 3, 56, "Input",
  CellTags->"Contract"],
Cell[423488, 13209, 628, 13, 70, "Output"]
}, Closed]],
Cell[424131, 13225, 456, 19, 35, "Text",
  CellTags->"Contract"]
}, Closed]],

Cell[CellGroupData[{
Cell[424624, 13249, 57, 1, 49, "Subsubsection",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[424706, 13254, 279, 7, 56, "Input",
  CellTags->"Contract"],
Cell[424988, 13263, 252, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[425277, 13275, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[425417, 13280, 285, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[425739, 13294, 240, 6, 52, "Input",
  CellTags->"Contract"],
Cell[425982, 13302, 314, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[426333, 13316, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[426473, 13321, 283, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[426793, 13333, 176, 4, 52, "Input",
  CellTags->"Contract"],
Cell[426972, 13339, 184, 5, 70, "Output"]
}, Closed]],
Cell[427171, 13347, 84, 1, 35, "Text",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[427280, 13352, 137, 3, 56, "Input",
  CellTags->"Contract"],
Cell[427420, 13357, 108, 2, 70, "Output"]
}, Closed]],
Cell[427543, 13362, 209, 5, 52, "Text",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[427777, 13371, 277, 7, 56, "Input",
  CellTags->"Contract"],
Cell[428057, 13380, 298, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[428392, 13393, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[428532, 13398, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[428677, 13405, 277, 7, 52, "Input",
  CellTags->"Contract"],
Cell[428957, 13414, 298, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[429292, 13427, 273, 7, 52, "Input",
  CellTags->"Contract"],
Cell[429568, 13436, 199, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[429804, 13446, 142, 3, 52, "Input",
  CellTags->"Contract"],
Cell[429949, 13451, 268, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[430254, 13463, 335, 9, 52, "Input",
  CellTags->"Contract"],
Cell[430592, 13474, 606, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[431235, 13497, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[431375, 13502, 835, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[432247, 13530, 323, 8, 52, "Input",
  CellTags->"Contract"],
Cell[432573, 13540, 765, 22, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[433375, 13567, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[433515, 13572, 605, 18, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[434157, 13595, 400, 10, 52, "Input",
  CellTags->"Contract"],
Cell[434560, 13607, 1324, 39, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[435921, 13651, 137, 3, 52, "Input",
  CellTags->"Contract"],
Cell[436061, 13656, 249, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[436347, 13668, 508, 13, 52, "Input",
  CellTags->"Contract"],
Cell[436858, 13683, 460, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[437355, 13699, 171, 4, 52, "Input",
  CellTags->"Contract"],
Cell[437529, 13705, 646, 19, 70, "Output"]
}, Closed]],
Cell[438190, 13727, 237, 6, 52, "Input",
  CellTags->"Contract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[438476, 13739, 56, 1, 39, "ObjectName",
  CellTags->"Contract1"],

Cell[CellGroupData[{
Cell[438557, 13744, 61, 1, 49, "Subsubsection",
  CellTags->"Contract1"],
Cell[438621, 13747, 168, 4, 39, "Text",
  CellTags->"Contract1"],
Cell[438792, 13753, 421, 18, 39, "Text",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{
Cell[439250, 13776, 58, 1, 49, "Subsubsection",
  CellTags->"Contract1"],

Cell[CellGroupData[{
Cell[439333, 13781, 430, 12, 56, "Input",
  CellTags->"Contract1"],
Cell[439766, 13795, 291, 8, 37, "Output",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{
Cell[440094, 13808, 430, 12, 52, "Input",
  CellTags->"Contract1"],
Cell[440527, 13822, 628, 20, 38, "Output",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{
Cell[441192, 13847, 328, 9, 52, "Input",
  CellTags->"Contract1"],
Cell[441523, 13858, 131, 3, 37, "Output",
  CellTags->"Contract1"]
}, Closed]],

Cell[CellGroupData[{
Cell[441691, 13866, 328, 9, 52, "Input",
  CellTags->"Contract1"],
Cell[442022, 13877, 391, 11, 38, "Output",
  CellTags->"Contract1"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[442474, 13895, 56, 1, 39, "ObjectName",
  CellTags->"Convolute"],

Cell[CellGroupData[{
Cell[442555, 13900, 61, 1, 49, "Subsubsection",
  CellTags->"Convolute"],
Cell[442619, 13903, 1660, 50, 58, "Text",
  CellTags->"Convolute"],

Cell[CellGroupData[{
Cell[444304, 13957, 145, 3, 56, "Input",
  CellTags->"Convolute"],
Cell[444452, 13962, 469, 12, 70, "Output"]
}, Closed]],
Cell[444936, 13977, 355, 14, 35, "Text",
  CellTags->"Convolute"]
}, Closed]],

Cell[CellGroupData[{
Cell[445328, 13996, 58, 1, 49, "Subsubsection",
  CellTags->"Convolute"],

Cell[CellGroupData[{
Cell[445411, 14001, 166, 4, 56, "Input",
  CellTags->"Convolute"],
Cell[445580, 14007, 166, 4, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[445783, 14016, 166, 4, 52, "Input",
  CellTags->"Convolute"],
Cell[445952, 14022, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[446184, 14032, 166, 4, 52, "Input",
  CellTags->"Convolute"],
Cell[446353, 14038, 135, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[446525, 14046, 224, 6, 64, "Input",
  CellTags->"Convolute"],
Cell[446752, 14054, 238, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[447027, 14065, 277, 7, 64, "Input",
  CellTags->"Convolute"],
Cell[447307, 14074, 238, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[447582, 14085, 224, 6, 64, "Input",
  CellTags->"Convolute"],
Cell[447809, 14093, 329, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[448175, 14107, 282, 8, 64, "Input",
  CellTags->"Convolute"],
Cell[448460, 14117, 637, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[449134, 14140, 233, 6, 52, "Input",
  CellTags->"Convolute"],
Cell[449370, 14148, 399, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[449806, 14164, 266, 7, 52, "Input",
  CellTags->"Convolute"],
Cell[450075, 14173, 316, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[450428, 14187, 291, 8, 64, "Input",
  CellTags->"Convolute"],
Cell[450722, 14197, 422, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[451181, 14213, 324, 9, 64, "Input",
  CellTags->"Convolute"],
Cell[451508, 14224, 699, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[452244, 14248, 295, 8, 64, "Input",
  CellTags->"Convolute"],
Cell[452542, 14258, 848, 23, 53, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[453427, 14286, 233, 6, 52, "Input",
  CellTags->"Convolute"],
Cell[453663, 14294, 249, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[453949, 14306, 233, 6, 52, "Input",
  CellTags->"Convolute"],
Cell[454185, 14314, 366, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[454588, 14329, 320, 9, 64, "Input",
  CellTags->"Convolute"],
Cell[454911, 14340, 539, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[455487, 14361, 202, 5, 52, "Input",
  CellTags->"Convolute"],
Cell[455692, 14368, 263, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[455992, 14381, 238, 6, 52, "Input",
  CellTags->"Convolute"],
Cell[456233, 14389, 273, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[456543, 14402, 260, 7, 64, "Input",
  CellTags->"Convolute"],
Cell[456806, 14411, 537, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[457380, 14431, 262, 7, 64, "Input",
  CellTags->"Convolute"],
Cell[457645, 14440, 389, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[458071, 14456, 291, 8, 64, "Input",
  CellTags->"Convolute"],
Cell[458365, 14466, 696, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[459098, 14490, 262, 7, 64, "Input",
  CellTags->"Convolute"],
Cell[459363, 14499, 507, 14, 53, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[459907, 14518, 233, 6, 52, "Input",
  CellTags->"Convolute"],
Cell[460143, 14526, 249, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[460429, 14538, 202, 5, 52, "Input",
  CellTags->"Convolute"],
Cell[460634, 14545, 212, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[460883, 14556, 353, 10, 64, "Input",
  CellTags->"Convolute"],
Cell[461239, 14568, 1054, 30, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[462354, 14605, 83, 1, 39, "ObjectName",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{
Cell[462462, 14610, 66, 1, 49, "Subsubsection",
  CellTags->"ConvoluteTable"],
Cell[462531, 14613, 104, 1, 39, "Text",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{
Cell[462660, 14618, 150, 3, 56, "Input",
  CellTags->"ConvoluteTable"],
Cell[462813, 14623, 499, 13, 70, "Output",
  CellTags->"ConvoluteTable"]
}, Closed]],
Cell[463327, 14639, 345, 14, 35, "Text",
  CellTags->"ConvoluteTable"]
}, Closed]],

Cell[CellGroupData[{
Cell[463709, 14658, 63, 1, 49, "Subsubsection",
  CellTags->"ConvoluteTable"],

Cell[CellGroupData[{
Cell[463797, 14663, 175, 4, 56, "Input",
  CellTags->"ConvoluteTable"],
Cell[463975, 14669, 165, 4, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[464177, 14678, 175, 4, 52, "Input",
  CellTags->"ConvoluteTable"],
Cell[464355, 14684, 194, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[464586, 14694, 175, 4, 52, "Input",
  CellTags->"ConvoluteTable"],
Cell[464764, 14700, 134, 3, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[464959, 14710, 86, 1, 39, "ObjectName",
  CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{
Cell[465070, 14715, 76, 1, 49, "Subsubsection",
  CellTags->"CovariantFieldDerivative"],
Cell[465149, 14718, 375, 7, 73, "Text",
  CellTags->"CovariantFieldDerivative"],
Cell[465527, 14727, 493, 19, 39, "Text",
  CellTags->"CovariantFieldDerivative"]
}, Closed]],

Cell[CellGroupData[{
Cell[466057, 14751, 73, 1, 49, "Subsubsection",
  CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{
Cell[466155, 14756, 377, 10, 43, "Input",
  CellTags->"CovariantFieldDerivative"],
Cell[466535, 14768, 630, 20, 25, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[467226, 14795, 54, 1, 39, "ObjectName",
  CellTags->"CounterT"],

Cell[CellGroupData[{
Cell[467305, 14800, 60, 1, 49, "Subsubsection",
  CellTags->"CounterT"],
Cell[467368, 14803, 147, 4, 39, "Text",
  CellTags->"CounterT"],
Cell[467518, 14809, 483, 19, 39, "Text",
  CellTags->"CounterT"]
}, Closed]],

Cell[CellGroupData[{
Cell[468038, 14833, 57, 1, 49, "Subsubsection",
  CellTags->"CounterT"],
Cell[468098, 14836, 271, 5, 56, "Input",
  CellTags->"CounterT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[468418, 14847, 60, 1, 39, "ObjectName",
  CellTags->"CounterTerm"],

Cell[CellGroupData[{
Cell[468503, 14852, 63, 1, 49, "Subsubsection",
  CellTags->"CounterTerm"],
Cell[468569, 14855, 217, 5, 56, "Text",
  CellTags->"CounterTerm"],
Cell[468789, 14862, 621, 24, 39, "Text",
  CellTags->"CounterTerm"]
}, Closed]],

Cell[CellGroupData[{
Cell[469447, 14891, 60, 1, 49, "Subsubsection",
  CellTags->"CounterTerm"],

Cell[CellGroupData[{
Cell[469532, 14896, 143, 3, 56, "Input",
  CellTags->"CounterTerm"],
Cell[469678, 14901, 2153, 52, 65, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[471892, 14960, 70, 1, 39, "ObjectName",
  CellTags->"CouplingConstant"],

Cell[CellGroupData[{
Cell[471987, 14965, 68, 1, 49, "Subsubsection",
  CellTags->"CouplingConstant"],
Cell[472058, 14968, 658, 11, 115, "Text",
  CellTags->"CouplingConstant"],
Cell[472719, 14981, 341, 14, 39, "Text",
  CellTags->"CouplingConstant"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[473109, 15001, 58, 1, 39, "ObjectName",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[473192, 15006, 62, 1, 49, "Subsubsection",
  CellTags->"CovariantD"],
Cell[473257, 15009, 1213, 19, 216, "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[474495, 15032, 147, 3, 56, "Input",
  CellTags->"CovariantD"],
Cell[474645, 15037, 491, 11, 70, "Output"]
}, Closed]],
Cell[475151, 15051, 188, 5, 35, "Text",
  CellTags->"CovariantD"],
Cell[475342, 15058, 488, 19, 39, "Text",
  CellTags->"CovariantD"]
}, Closed]],

Cell[CellGroupData[{
Cell[475867, 15082, 59, 1, 49, "Subsubsection",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[475951, 15087, 145, 3, 56, "Input",
  CellTags->"CovariantD"],
Cell[476099, 15092, 138, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[476274, 15100, 182, 4, 52, "Input",
  CellTags->"CovariantD"],
Cell[476459, 15106, 186, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[476682, 15115, 217, 5, 52, "Input",
  CellTags->"CovariantD"],
Cell[476902, 15122, 1299, 38, 70, "Output"]
}, Closed]],
Cell[478216, 15163, 165, 4, 35, "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[478406, 15171, 148, 3, 56, "Input",
  CellTags->"CovariantD"],
Cell[478557, 15176, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[478789, 15186, 185, 4, 52, "Input",
  CellTags->"CovariantD"],
Cell[478977, 15192, 243, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[479257, 15203, 240, 5, 52, "Input",
  CellTags->"CovariantD"],
Cell[479500, 15210, 1232, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[480769, 15250, 220, 5, 52, "Input",
  CellTags->"CovariantD"],
Cell[480992, 15257, 1201, 34, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[482230, 15296, 224, 5, 52, "Input",
  CellTags->"CovariantD"],
Cell[482457, 15303, 1845, 49, 70, "Output"]
}, Closed]],
Cell[484317, 15355, 675, 22, 38, "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[485017, 15381, 258, 6, 56, "Input",
  CellTags->"CovariantD"],
Cell[485278, 15389, 661, 19, 70, "Output"]
}, Closed]],
Cell[485954, 15411, 372, 11, 35, "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[486351, 15426, 258, 6, 56, "Input",
  CellTags->"CovariantD"],
Cell[486612, 15434, 3181, 79, 70, "Output"]
}, Closed]],
Cell[489808, 15516, 207, 7, 35, "Text",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[490040, 15527, 258, 6, 56, "Input",
  CellTags->"CovariantD"],
Cell[490301, 15535, 8818, 199, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[499156, 15739, 218, 5, 52, "Input",
  CellTags->"CovariantD"],
Cell[499377, 15746, 323, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[499737, 15760, 238, 6, 52, "Input",
  CellTags->"CovariantD"],
Cell[499978, 15768, 6343, 145, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[506358, 15918, 270, 7, 52, "Input",
  CellTags->"CovariantD"],
Cell[506631, 15927, 928, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[507596, 15955, 287, 7, 52, "Input",
  CellTags->"CovariantD"],
Cell[507886, 15964, 934, 24, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[508881, 15995, 62, 1, 39, "ObjectName",
  CellTags->"CrossProduct"],

Cell[CellGroupData[{
Cell[508968, 16000, 64, 1, 49, "Subsubsection",
  CellTags->"CrossProduct"],
Cell[509035, 16003, 146, 4, 39, "Text",
  CellTags->"CrossProduct"],
Cell[509184, 16009, 331, 14, 39, "Text",
  CellTags->"CrossProduct"]
}, Closed]],

Cell[CellGroupData[{
Cell[509552, 16028, 61, 1, 49, "Subsubsection",
  CellTags->"CrossProduct"],

Cell[CellGroupData[{
Cell[509638, 16033, 389, 10, 56, "Input",
  CellTags->"CrossProduct"],
Cell[510030, 16045, 893, 25, 52, "Output",
  CellTags->"CrossProduct"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[510984, 16077, 42, 1, 39, "ObjectName",
  CellTags->"C0"],

Cell[CellGroupData[{
Cell[511051, 16082, 54, 1, 49, "Subsubsection",
  CellTags->"C0"],
Cell[511108, 16085, 461, 10, 73, "Text",
  CellTags->"C0"],
Cell[511572, 16097, 501, 24, 39, "Text",
  CellTags->"C0"]
}, Closed]],

Cell[CellGroupData[{
Cell[512110, 16126, 51, 1, 49, "Subsubsection",
  CellTags->"C0"],

Cell[CellGroupData[{
Cell[512186, 16131, 217, 6, 56, "Input",
  CellTags->"C0"],
Cell[512406, 16139, 507, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[512950, 16154, 245, 6, 52, "Input",
  CellTags->"C0"],
Cell[513198, 16162, 507, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[513742, 16177, 404, 11, 52, "Input",
  CellTags->"C0"],
Cell[514149, 16190, 507, 10, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[514717, 16207, 54, 1, 39, "ObjectName",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[514796, 16212, 60, 1, 49, "Subsubsection",
  CellTags->"DataType"],
Cell[514859, 16215, 347, 7, 56, "Text",
  CellTags->"DataType"],
Cell[515209, 16224, 896, 32, 56, "Text",
  CellTags->"DataType"],
Cell[516108, 16258, 313, 13, 39, "Text",
  CellTags->"DataType"],
Cell[516424, 16273, 231, 9, 39, "Text",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{
Cell[516692, 16287, 57, 1, 49, "Subsubsection",
  CellTags->"DataType"],
Cell[516752, 16290, 75, 1, 70, "Text",
  CellTags->"DataType"],
Cell[516830, 16293, 269, 7, 70, "Input",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[517124, 16304, 279, 8, 56, "Input",
  CellTags->"DataType"],
Cell[517406, 16314, 298, 9, 70, "Output",
  CellTags->"DataType"]
}, Closed]],
Cell[517719, 16326, 182, 5, 52, "Text",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[517926, 16335, 140, 3, 56, "Input",
  CellTags->"DataType"],
Cell[518069, 16340, 258, 7, 70, "Output",
  CellTags->"DataType"]
}, Closed]],
Cell[518342, 16350, 339, 10, 52, "Input",
  CellTags->"DataType"],
Cell[518684, 16362, 518, 14, 56, "Input",
  CellTags->"DataType"],
Cell[519205, 16378, 504, 14, 56, "Input",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[519734, 16396, 426, 13, 56, "Input",
  CellTags->"DataType"],
Cell[520163, 16411, 436, 13, 70, "Output",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{
Cell[520636, 16429, 135, 3, 52, "Input",
  CellTags->"DataType"],
Cell[520774, 16434, 371, 11, 70, "Output",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{
Cell[521182, 16450, 135, 3, 52, "Input",
  CellTags->"DataType"],
Cell[521320, 16455, 215, 6, 70, "Output",
  CellTags->"DataType"]
}, Closed]],
Cell[521550, 16464, 234, 6, 52, "Input",
  CellTags->"DataType"],
Cell[521787, 16472, 237, 7, 56, "Input",
  CellTags->"DataType"],
Cell[522027, 16481, 550, 15, 56, "Input",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[522602, 16500, 368, 12, 56, "Input",
  CellTags->"DataType"],
Cell[522973, 16514, 372, 11, 70, "Output",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{
Cell[523382, 16530, 133, 3, 52, "Input",
  CellTags->"DataType"],
Cell[523518, 16535, 290, 9, 70, "Output",
  CellTags->"DataType"]
}, Closed]],
Cell[523823, 16547, 188, 5, 52, "Input",
  CellTags->"DataType"],
Cell[524014, 16554, 270, 7, 56, "Input",
  CellTags->"DataType"],
Cell[524287, 16563, 340, 10, 56, "Input",
  CellTags->"DataType"],
Cell[524630, 16575, 117, 4, 39, "Text",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[524772, 16583, 181, 4, 56, "Input",
  CellTags->"DataType"],
Cell[524956, 16589, 135, 3, 70, "Output",
  CellTags->"DataType"]
}, Closed]],
Cell[525106, 16595, 84, 1, 35, "Text",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[525215, 16600, 287, 8, 56, "Input",
  CellTags->"DataType"],
Cell[525505, 16610, 132, 3, 70, "Output",
  CellTags->"DataType"]
}, Closed]],

Cell[CellGroupData[{
Cell[525674, 16618, 283, 8, 52, "Input",
  CellTags->"DataType"],
Cell[525960, 16628, 567, 17, 70, "Output",
  CellTags->"DataType"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[526588, 16652, 44, 1, 39, "ObjectName",
  CellTags->"DB0"],

Cell[CellGroupData[{
Cell[526657, 16657, 55, 1, 49, "Subsubsection",
  CellTags->"DB0"],
Cell[526715, 16660, 147, 4, 39, "Text",
  CellTags->"DB0"],
Cell[526865, 16666, 169, 9, 39, "Text",
  CellTags->"DB0"]
}, Closed]],

Cell[CellGroupData[{
Cell[527071, 16680, 52, 1, 49, "Subsubsection",
  CellTags->"DB0"],

Cell[CellGroupData[{
Cell[527148, 16685, 422, 12, 57, "Input",
  CellTags->"DB0"],
Cell[527573, 16699, 296, 8, 39, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[527930, 16714, 44, 1, 39, "ObjectName",
  CellTags->"DB1"],

Cell[CellGroupData[{
Cell[527999, 16719, 55, 1, 49, "Subsubsection",
  CellTags->"DB1"],
Cell[528057, 16722, 120, 4, 39, "Text",
  CellTags->"DB1"],
Cell[528180, 16728, 169, 9, 39, "Text",
  CellTags->"DB1"]
}, Closed]],

Cell[CellGroupData[{
Cell[528386, 16742, 52, 1, 49, "Subsubsection",
  CellTags->"DB1"],

Cell[CellGroupData[{
Cell[528463, 16747, 422, 12, 57, "Input",
  CellTags->"DB1"],
Cell[528888, 16761, 1689, 43, 61, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[530638, 16811, 80, 1, 39, "ObjectName",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{
Cell[530743, 16816, 73, 1, 49, "Subsubsection",
  CellTags->"DeclareNonCommutative"],
Cell[530819, 16819, 195, 5, 39, "Text",
  CellTags->"DeclareNonCommutative"],
Cell[531017, 16826, 370, 14, 39, "Text",
  CellTags->"DeclareNonCommutative"]
}, Closed]],

Cell[CellGroupData[{
Cell[531424, 16845, 70, 1, 49, "Subsubsection",
  CellTags->"DeclareNonCommutative"],
Cell[531497, 16848, 163, 3, 70, "Input",
  CellTags->"DeclareNonCommutative"],
Cell[531663, 16853, 153, 4, 70, "Text",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{
Cell[531841, 16861, 190, 4, 70, "Input",
  CellTags->"DeclareNonCommutative"],
Cell[532034, 16867, 111, 2, 70, "Output"]
}, Closed]],
Cell[532160, 16872, 190, 4, 70, "Input",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{
Cell[532375, 16880, 227, 5, 70, "Input",
  CellTags->"DeclareNonCommutative"],
Cell[532605, 16887, 204, 5, 70, "Output"]
}, Closed]],
Cell[532824, 16895, 202, 4, 70, "Input",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{
Cell[533051, 16903, 227, 5, 70, "Input",
  CellTags->"DeclareNonCommutative"],
Cell[533281, 16910, 207, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[533549, 16922, 64, 1, 39, "ObjectName",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{
Cell[533638, 16927, 65, 1, 49, "Subsubsection",
  CellTags->"DeltaFunction"],
Cell[533706, 16930, 370, 11, 64, "Text",
  CellTags->"DeltaFunction"],
Cell[534079, 16943, 635, 24, 39, "Text",
  CellTags->"DeltaFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[534751, 16972, 62, 1, 49, "Subsubsection",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{
Cell[534838, 16977, 174, 4, 56, "Input",
  CellTags->"DeltaFunction"],
Cell[535015, 16983, 225, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[535277, 16994, 387, 10, 52, "Input",
  CellTags->"DeltaFunction"],
Cell[535667, 17006, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[535844, 17014, 354, 9, 52, "Input",
  CellTags->"DeltaFunction"],
Cell[536201, 17025, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[536378, 17033, 387, 10, 52, "Input",
  CellTags->"DeltaFunction"],
Cell[536768, 17045, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[536945, 17053, 247, 6, 52, "Input",
  CellTags->"DeltaFunction"],
Cell[537195, 17061, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[537364, 17070, 86, 1, 39, "ObjectName",
  CellTags->"DeltaFunctionDoublePrime"],

Cell[CellGroupData[{
Cell[537475, 17075, 76, 1, 49, "Subsubsection",
  CellTags->"DeltaFunctionDoublePrime"],
Cell[537554, 17078, 261, 8, 39, "Text",
  CellTags->"DeltaFunctionDoublePrime"],
Cell[537818, 17088, 231, 8, 39, "Text",
  CellTags->"DeltaFunctionDoublePrime"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[538098, 17102, 74, 1, 39, "ObjectName",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{
Cell[538197, 17107, 70, 1, 49, "Subsubsection",
  CellTags->"DeltaFunctionPrime"],
Cell[538270, 17110, 240, 7, 39, "Text",
  CellTags->"DeltaFunctionPrime"],
Cell[538513, 17119, 618, 23, 39, "Text",
  CellTags->"DeltaFunctionPrime"]
}, Closed]],

Cell[CellGroupData[{
Cell[539168, 17147, 67, 1, 49, "Subsubsection",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{
Cell[539260, 17152, 184, 4, 70, "Input",
  CellTags->"DeltaFunctionPrime"],
Cell[539447, 17158, 262, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[539746, 17170, 397, 10, 70, "Input",
  CellTags->"DeltaFunctionPrime"],
Cell[540146, 17182, 219, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[540402, 17193, 393, 10, 70, "Input",
  CellTags->"DeltaFunctionPrime"],
Cell[540798, 17205, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[540967, 17214, 70, 1, 39, "ObjectName",
  CellTags->"DenominatorOrder"],

Cell[CellGroupData[{
Cell[541062, 17219, 68, 1, 49, "Subsubsection",
  CellTags->"DenominatorOrder"],
Cell[541133, 17222, 176, 4, 39, "Text",
  CellTags->"DenominatorOrder"],
Cell[541312, 17228, 349, 13, 39, "Text",
  CellTags->"DenominatorOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[541710, 17247, 56, 1, 39, "ObjectName",
  CellTags->"Dimension"],

Cell[CellGroupData[{
Cell[541791, 17252, 61, 1, 49, "Subsubsection",
  CellTags->"Dimension"],
Cell[541855, 17255, 247, 5, 56, "Text",
  CellTags->"Dimension"],

Cell[CellGroupData[{
Cell[542127, 17264, 148, 3, 56, "Input",
  CellTags->"Dimension"],
Cell[542278, 17269, 271, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[542586, 17281, 420, 11, 52, "Input",
  CellTags->"Dimension"],
Cell[543009, 17294, 583, 18, 70, "Output"]
}, Closed]],
Cell[543607, 17315, 132, 4, 35, "Text",
  CellTags->"Dimension"],

Cell[CellGroupData[{
Cell[543764, 17323, 429, 11, 56, "Input",
  CellTags->"Dimension"],
Cell[544196, 17336, 497, 12, 70, "Output"]
}, Closed]],
Cell[544708, 17351, 167, 4, 35, "Text",
  CellTags->"Dimension"],
Cell[544878, 17357, 165, 4, 56, "Input",
  CellTags->"Dimension"],

Cell[CellGroupData[{
Cell[545068, 17365, 384, 9, 56, "Input",
  CellTags->"Dimension"],
Cell[545455, 17376, 893, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[546385, 17408, 138, 3, 52, "Input",
  CellTags->"Dimension"],
Cell[546526, 17413, 510, 12, 70, "Output"]
}, Closed]],
Cell[547051, 17428, 166, 4, 52, "Input",
  CellTags->"Dimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[547266, 17438, 78, 1, 39, "ObjectName",
  CellTags->"DimensionalReduction"],

Cell[CellGroupData[{
Cell[547369, 17443, 72, 1, 49, "Subsubsection",
  CellTags->"DimensionalReduction"],
Cell[547444, 17446, 124, 4, 39, "Text",
  CellTags->"DimensionalReduction"],
Cell[547571, 17452, 323, 13, 39, "Text",
  CellTags->"DimensionalReduction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[547943, 17471, 58, 1, 39, "ObjectName",
  CellTags->"DiracBasis"],

Cell[CellGroupData[{
Cell[548026, 17476, 62, 1, 49, "Subsubsection",
  CellTags->"DiracBasis"],
Cell[548091, 17479, 300, 6, 56, "Text",
  CellTags->"DiracBasis"],
Cell[548394, 17487, 196, 8, 39, "Text",
  CellTags->"DiracBasis"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[548639, 17501, 66, 1, 39, "ObjectName",
  CellTags->"DiracCanonical"],

Cell[CellGroupData[{
Cell[548730, 17506, 66, 1, 49, "Subsubsection",
  CellTags->"DiracCanonical"],
Cell[548799, 17509, 172, 4, 39, "Text",
  CellTags->"DiracCanonical"],
Cell[548974, 17515, 206, 8, 39, "Text",
  CellTags->"DiracCanonical"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[549229, 17529, 64, 1, 39, "ObjectName",
  CellTags->"DiracEquation"],

Cell[CellGroupData[{
Cell[549318, 17534, 65, 1, 49, "Subsubsection",
  CellTags->"DiracEquation"],
Cell[549386, 17537, 161, 4, 39, "Text",
  CellTags->"DiracEquation"],
Cell[549550, 17543, 212, 9, 39, "Text",
  CellTags->"DiracEquation"]
}, Closed]],

Cell[CellGroupData[{
Cell[549799, 17557, 62, 1, 49, "Subsubsection",
  CellTags->"DiracEquation"],
Cell[549864, 17560, 335, 9, 56, "Input",
  CellTags->"DiracEquation"],
Cell[550202, 17571, 144, 3, 56, "Input",
  CellTags->"DiracEquation"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[550395, 17580, 58, 1, 39, "ObjectName",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{
Cell[550478, 17585, 62, 1, 49, "Subsubsection",
  CellTags->"DiracGamma"],
Cell[550543, 17588, 852, 25, 73, "Text",
  CellTags->"DiracGamma"],
Cell[551398, 17615, 734, 29, 39, "Text",
  CellTags->"DiracGamma"]
}, Closed]],

Cell[CellGroupData[{
Cell[552169, 17649, 59, 1, 49, "Subsubsection",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{
Cell[552253, 17654, 141, 3, 70, "Input",
  CellTags->"DiracGamma"],
Cell[552397, 17659, 143, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[552577, 17667, 191, 4, 70, "Input",
  CellTags->"DiracGamma"],
Cell[552771, 17673, 248, 7, 70, "Output"]
}, Closed]],
Cell[553034, 17683, 368, 13, 70, "Text",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{
Cell[553427, 17700, 204, 5, 70, "Input",
  CellTags->"DiracGamma"],
Cell[553634, 17707, 283, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[553954, 17719, 332, 8, 70, "Input",
  CellTags->"DiracGamma"],
Cell[554289, 17729, 689, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[555015, 17753, 270, 8, 70, "Input",
  CellTags->"DiracGamma"],
Cell[555288, 17763, 283, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[555608, 17775, 256, 7, 70, "Input",
  CellTags->"DiracGamma"],
Cell[555867, 17784, 521, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[556425, 17803, 176, 4, 70, "Input",
  CellTags->"DiracGamma"],
Cell[556604, 17809, 872, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[557513, 17838, 358, 9, 70, "Input",
  CellTags->"DiracGamma"],
Cell[557874, 17849, 623, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[558534, 17870, 170, 4, 70, "Input",
  CellTags->"DiracGamma"],
Cell[558707, 17876, 1161, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[559905, 17911, 173, 4, 70, "Input",
  CellTags->"DiracGamma"],
Cell[560081, 17917, 1673, 46, 70, "Output"]
}, Closed]],
Cell[561769, 17966, 205, 5, 70, "Input",
  CellTags->"DiracGamma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[562023, 17977, 72, 1, 39, "ObjectName",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{
Cell[562120, 17982, 69, 1, 49, "Subsubsection",
  CellTags->"DiracGammaCombine"],
Cell[562192, 17985, 136, 4, 39, "Text",
  CellTags->"DiracGammaCombine"],
Cell[562331, 17991, 867, 34, 39, "Text",
  CellTags->"DiracGammaCombine"]
}, Closed]],

Cell[CellGroupData[{
Cell[563235, 18030, 66, 1, 49, "Subsubsection",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{
Cell[563326, 18035, 262, 6, 56, "Input",
  CellTags->"DiracGammaCombine"],
Cell[563591, 18043, 363, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[563991, 18058, 150, 3, 52, "Input",
  CellTags->"DiracGammaCombine"],
Cell[564144, 18063, 198, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[564379, 18072, 324, 8, 52, "Input",
  CellTags->"DiracGammaCombine"],
Cell[564706, 18082, 429, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[565172, 18099, 150, 3, 52, "Input",
  CellTags->"DiracGammaCombine"],
Cell[565325, 18104, 260, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[565622, 18115, 155, 3, 52, "Input",
  CellTags->"DiracGammaCombine"],
Cell[565780, 18120, 609, 17, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[566450, 18144, 70, 1, 39, "ObjectName",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[566545, 18149, 68, 1, 49, "Subsubsection",
  CellTags->"DiracGammaExpand"],
Cell[566616, 18152, 193, 5, 56, "Text",
  CellTags->"DiracGammaExpand"],
Cell[566812, 18159, 869, 34, 39, "Text",
  CellTags->"DiracGammaExpand"]
}, Closed]],

Cell[CellGroupData[{
Cell[567718, 18198, 65, 1, 49, "Subsubsection",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[567808, 18203, 375, 9, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[568186, 18214, 689, 19, 70, "Output"]
}, Closed]],
Cell[568890, 18236, 128, 4, 70, "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[569043, 18244, 149, 3, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[569195, 18249, 311, 7, 70, "Output"]
}, Closed]],
Cell[569521, 18259, 102, 1, 70, "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[569648, 18264, 153, 3, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[569804, 18269, 872, 24, 70, "Output"]
}, Closed]],
Cell[570691, 18296, 91, 1, 70, "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[570807, 18301, 246, 5, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[571056, 18308, 311, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[571404, 18320, 196, 4, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[571603, 18326, 434, 10, 70, "Output"]
}, Closed]],
Cell[572052, 18339, 107, 1, 70, "Text",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[572184, 18344, 150, 3, 70, "Input",
  CellTags->"DiracGammaExpand"],
Cell[572337, 18349, 747, 21, 70, "Output"]
}, Closed]],
Cell[573099, 18373, 142, 3, 70, "Input",
  CellTags->"DiracGammaExpand"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[573290, 18382, 60, 1, 39, "ObjectName",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{
Cell[573375, 18387, 63, 1, 49, "Subsubsection",
  CellTags->"DiracGammaT"],
Cell[573441, 18390, 209, 5, 56, "Text",
  CellTags->"DiracGammaT"],
Cell[573653, 18397, 201, 9, 39, "Text",
  CellTags->"DiracGammaT"]
}, Closed]],

Cell[CellGroupData[{
Cell[573891, 18411, 60, 1, 49, "Subsubsection",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{
Cell[573976, 18416, 190, 4, 56, "Input",
  CellTags->"DiracGammaT"],
Cell[574169, 18422, 253, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[574459, 18434, 141, 3, 52, "Input",
  CellTags->"DiracGammaT"],
Cell[574603, 18439, 245, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[574885, 18451, 209, 6, 52, "Input",
  CellTags->"DiracGammaT"],
Cell[575097, 18459, 327, 9, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[575485, 18475, 60, 1, 39, "ObjectName",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{
Cell[575570, 18480, 63, 1, 49, "Subsubsection",
  CellTags->"DiracMatrix"],
Cell[575636, 18483, 1375, 48, 56, "Text",
  CellTags->"DiracMatrix"],
Cell[577014, 18533, 1146, 49, 39, "Text",
  CellTags->"DiracMatrix"]
}, Closed]],

Cell[CellGroupData[{
Cell[578197, 18587, 60, 1, 49, "Subsubsection",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{
Cell[578282, 18592, 147, 3, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[578432, 18597, 245, 7, 70, "Output"]
}, Closed]],
Cell[578692, 18607, 640, 20, 70, "Text",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{
Cell[579357, 18631, 220, 5, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[579580, 18638, 429, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[580046, 18656, 196, 5, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[580245, 18663, 180, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[580462, 18671, 177, 4, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[580642, 18677, 147, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[580826, 18685, 140, 3, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[580969, 18690, 126, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[581132, 18697, 139, 3, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[581274, 18702, 147, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[581458, 18710, 170, 4, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[581631, 18716, 237, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[581905, 18726, 184, 4, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[582092, 18732, 269, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[582398, 18743, 203, 5, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[582604, 18750, 421, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[583062, 18768, 140, 3, 70, "Input",
  CellTags->"DiracMatrix"],
Cell[583205, 18773, 288, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[583554, 18786, 58, 1, 39, "ObjectName",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[583637, 18791, 62, 1, 49, "Subsubsection",
  CellTags->"DiracOrder"],
Cell[583702, 18794, 205, 5, 39, "Text",
  CellTags->"DiracOrder"],
Cell[583910, 18801, 335, 14, 39, "Text",
  CellTags->"DiracOrder"]
}, Closed]],

Cell[CellGroupData[{
Cell[584282, 18820, 59, 1, 49, "Subsubsection",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[584366, 18825, 213, 5, 70, "Input",
  CellTags->"DiracOrder"],
Cell[584582, 18832, 224, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[584843, 18843, 142, 3, 70, "Input",
  CellTags->"DiracOrder"],
Cell[584988, 18848, 599, 17, 70, "Output"]
}, Closed]],
Cell[585602, 18868, 94, 1, 70, "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[585721, 18873, 213, 5, 70, "Input",
  CellTags->"DiracOrder"],
Cell[585937, 18880, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[586242, 18891, 142, 3, 70, "Input",
  CellTags->"DiracOrder"],
Cell[586387, 18896, 599, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[587023, 18918, 208, 5, 70, "Input",
  CellTags->"DiracOrder"],
Cell[587234, 18925, 264, 6, 70, "Output"]
}, Closed]],
Cell[587513, 18934, 215, 8, 70, "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[587753, 18946, 142, 3, 70, "Input",
  CellTags->"DiracOrder"],
Cell[587898, 18951, 476, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[588411, 18970, 204, 5, 70, "Input",
  CellTags->"DiracOrder"],
Cell[588618, 18977, 260, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[588915, 18988, 142, 3, 70, "Input",
  CellTags->"DiracOrder"],
Cell[589060, 18993, 319, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[589416, 19007, 229, 6, 70, "Input",
  CellTags->"DiracOrder"],
Cell[589648, 19015, 278, 7, 70, "Output"]
}, Closed]],
Cell[589941, 19025, 319, 10, 70, "Text",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[590285, 19039, 266, 7, 70, "Input",
  CellTags->"DiracOrder"],
Cell[590554, 19048, 1594, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[592185, 19098, 141, 3, 70, "Input",
  CellTags->"DiracOrder"],
Cell[592329, 19103, 594, 18, 70, "Output"]
}, Closed]],
Cell[592938, 19124, 229, 6, 70, "Input",
  CellTags->"DiracOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[593216, 19136, 60, 1, 39, "ObjectName",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{
Cell[593301, 19141, 63, 1, 49, "Subsubsection",
  CellTags->"DiracReduce"],
Cell[593367, 19144, 663, 11, 90, "Text",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{
Cell[594055, 19159, 149, 3, 56, "Input",
  CellTags->"DiracReduce"],
Cell[594207, 19164, 364, 9, 70, "Output"]
}, Closed]],
Cell[594586, 19176, 210, 9, 35, "Text",
  CellTags->"DiracReduce"]
}, Closed]],

Cell[CellGroupData[{
Cell[594833, 19190, 60, 1, 49, "Subsubsection",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{
Cell[594918, 19195, 209, 5, 70, "Input",
  CellTags->"DiracReduce"],
Cell[595130, 19202, 217, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[595384, 19212, 144, 3, 70, "Input",
  CellTags->"DiracReduce"],
Cell[595531, 19217, 350, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[595918, 19231, 223, 5, 70, "Input",
  CellTags->"DiracReduce"],
Cell[596144, 19238, 259, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[596440, 19249, 144, 3, 70, "Input",
  CellTags->"DiracReduce"],
Cell[596587, 19254, 1208, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[597832, 19289, 249, 6, 70, "Input",
  CellTags->"DiracReduce"],
Cell[598084, 19297, 310, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[598431, 19310, 172, 4, 70, "Input",
  CellTags->"DiracReduce"],
Cell[598606, 19316, 2454, 56, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[601097, 19377, 303, 7, 70, "Input",
  CellTags->"DiracReduce"],
Cell[601403, 19386, 132, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[601572, 19394, 165, 4, 70, "Input",
  CellTags->"DiracReduce"],
Cell[601740, 19400, 132, 3, 70, "Output"]
}, Closed]],
Cell[601887, 19406, 188, 4, 70, "Input",
  CellTags->"DiracReduce"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[602124, 19416, 58, 1, 39, "ObjectName",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{
Cell[602207, 19421, 62, 1, 49, "Subsubsection",
  CellTags->"DiracSigma"],
Cell[602272, 19424, 214, 5, 56, "Text",
  CellTags->"DiracSigma"],
Cell[602489, 19431, 224, 9, 39, "Text",
  CellTags->"DiracSigma"]
}, Closed]],

Cell[CellGroupData[{
Cell[602750, 19445, 59, 1, 49, "Subsubsection",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{
Cell[602834, 19450, 285, 7, 70, "Input",
  CellTags->"DiracSigma"],
Cell[603122, 19459, 201, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[603360, 19469, 150, 3, 70, "Input",
  CellTags->"DiracSigma"],
Cell[603513, 19474, 1035, 29, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[604585, 19508, 285, 7, 70, "Input",
  CellTags->"DiracSigma"],
Cell[604873, 19517, 227, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[605137, 19528, 272, 7, 70, "Input",
  CellTags->"DiracSigma"],
Cell[605412, 19537, 181, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[605630, 19546, 150, 3, 70, "Input",
  CellTags->"DiracSigma"],
Cell[605783, 19551, 1433, 37, 70, "Output"]
}, Closed]],
Cell[607231, 19591, 176, 4, 70, "Input",
  CellTags->"DiracSigma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[607456, 19601, 74, 1, 39, "ObjectName",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{
Cell[607555, 19606, 70, 1, 49, "Subsubsection",
  CellTags->"DiracSigmaExplicit"],
Cell[607628, 19609, 189, 5, 39, "Text",
  CellTags->"DiracSigmaExplicit"],
Cell[607820, 19616, 208, 9, 39, "Text",
  CellTags->"DiracSigmaExplicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[608065, 19630, 67, 1, 49, "Subsubsection",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{
Cell[608157, 19635, 259, 6, 70, "Input",
  CellTags->"DiracSigmaExplicit"],
Cell[608419, 19643, 201, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[608657, 19653, 157, 3, 70, "Input",
  CellTags->"DiracSigmaExplicit"],
Cell[608817, 19658, 1035, 29, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[609913, 19694, 70, 1, 39, "ObjectName",
  CellTags->"DiracSimpCombine"],

Cell[CellGroupData[{
Cell[610008, 19699, 68, 1, 49, "Subsubsection",
  CellTags->"DiracSimpCombine"],
Cell[610079, 19702, 210, 5, 56, "Text",
  CellTags->"DiracSimpCombine"],
Cell[610292, 19709, 208, 8, 39, "Text",
  CellTags->"DiracSimpCombine"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[610549, 19723, 64, 1, 39, "ObjectName",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[610638, 19728, 65, 1, 49, "Subsubsection",
  CellTags->"DiracSimplify"],
Cell[610706, 19731, 383, 7, 73, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[611114, 19742, 153, 3, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[611270, 19747, 664, 13, 70, "Output"]
}, Closed]],
Cell[611949, 19763, 455, 19, 35, "Text",
  CellTags->"DiracSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[612441, 19787, 62, 1, 49, "Subsubsection",
  CellTags->"DiracSimplify"],
Cell[612506, 19790, 100, 1, 39, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[612631, 19795, 215, 5, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[612849, 19802, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[613154, 19813, 148, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[613305, 19818, 306, 9, 70, "Output"]
}, Closed]],
Cell[613626, 19830, 97, 1, 35, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[613748, 19835, 216, 5, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[613967, 19842, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[614272, 19853, 148, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[614423, 19858, 306, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[614766, 19872, 211, 5, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[614980, 19879, 264, 6, 70, "Output"]
}, Closed]],
Cell[615259, 19888, 192, 7, 35, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[615476, 19899, 148, 3, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[615627, 19904, 476, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[616140, 19923, 207, 5, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[616350, 19930, 260, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[616647, 19941, 148, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[616798, 19946, 319, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[617154, 19960, 390, 10, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[617547, 19972, 934, 26, 70, "Output"]
}, Closed]],
Cell[618496, 20001, 123, 4, 35, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[618644, 20009, 148, 3, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[618795, 20014, 2863, 79, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[621695, 20098, 145, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[621843, 20103, 897, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[622777, 20131, 319, 7, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[623099, 20140, 631, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[623767, 20162, 148, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[623918, 20167, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[624063, 20174, 460, 12, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[624526, 20188, 703, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[625266, 20210, 147, 3, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[625416, 20215, 22626, 649, 70, "Output"]
}, Closed]],
Cell[648057, 20867, 118, 4, 35, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[648200, 20875, 299, 7, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[648502, 20884, 589, 17, 70, "Output"]
}, Closed]],
Cell[649106, 20904, 142, 4, 35, "Text",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[649273, 20912, 344, 8, 56, "Input",
  CellTags->"DiracSimplify"],
Cell[649620, 20922, 743, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[650400, 20946, 316, 7, 52, "Input",
  CellTags->"DiracSimplify"],
Cell[650719, 20955, 108, 2, 70, "Output"]
}, Closed]],
Cell[650842, 20960, 219, 5, 52, "Input",
  CellTags->"DiracSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[651110, 20971, 66, 1, 39, "ObjectName",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{
Cell[651201, 20976, 66, 1, 49, "Subsubsection",
  CellTags->"DiracSimplify2"],
Cell[651270, 20979, 237, 7, 39, "Text",
  CellTags->"DiracSimplify2"],
Cell[651510, 20988, 213, 9, 39, "Text",
  CellTags->"DiracSimplify2"]
}, Closed]],

Cell[CellGroupData[{
Cell[651760, 21002, 63, 1, 49, "Subsubsection",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{
Cell[651848, 21007, 263, 6, 56, "Input",
  CellTags->"DiracSimplify2"],
Cell[652114, 21015, 478, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[652629, 21031, 149, 3, 52, "Input",
  CellTags->"DiracSimplify2"],
Cell[652781, 21036, 1112, 34, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[653954, 21077, 58, 1, 39, "ObjectName",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{
Cell[654037, 21082, 62, 1, 49, "Subsubsection",
  CellTags->"DiracSlash"],
Cell[654102, 21085, 515, 16, 40, "Text",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{
Cell[654642, 21105, 147, 3, 56, "Input",
  CellTags->"DiracSlash"],
Cell[654792, 21110, 271, 7, 70, "Output"]
}, Closed]],
Cell[655078, 21120, 941, 39, 35, "Text",
  CellTags->"DiracSlash"]
}, Closed]],

Cell[CellGroupData[{
Cell[656056, 21164, 59, 1, 49, "Subsubsection",
  CellTags->"DiracSlash"],
Cell[656118, 21167, 265, 9, 40, "Text",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{
Cell[656408, 21180, 141, 3, 56, "Input",
  CellTags->"DiracSlash"],
Cell[656552, 21185, 283, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[656872, 21197, 209, 5, 52, "Input",
  CellTags->"DiracSlash"],
Cell[657084, 21204, 609, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[657730, 21226, 168, 4, 52, "Input",
  CellTags->"DiracSlash"],
Cell[657901, 21232, 609, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[658547, 21254, 133, 3, 52, "Input",
  CellTags->"DiracSlash"],
Cell[658683, 21259, 188, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[658908, 21268, 182, 5, 52, "Input",
  CellTags->"DiracSlash"],
Cell[659093, 21275, 169, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[659299, 21283, 261, 7, 52, "Input",
  CellTags->"DiracSlash"],
Cell[659563, 21292, 229, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[659853, 21304, 60, 1, 39, "ObjectName",
  CellTags->"DiracSpinor"],

Cell[CellGroupData[{
Cell[659938, 21309, 63, 1, 49, "Subsubsection",
  CellTags->"DiracSpinor"],
Cell[660004, 21312, 289, 6, 56, "Text",
  CellTags->"DiracSpinor"],
Cell[660296, 21320, 182, 8, 39, "Text",
  CellTags->"DiracSpinor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[660527, 21334, 72, 1, 39, "ObjectName",
  CellTags->"DiracSubstitute67"],

Cell[CellGroupData[{
Cell[660624, 21339, 69, 1, 49, "Subsubsection",
  CellTags->"DiracSubstitute67"],
Cell[660696, 21342, 220, 5, 56, "Text",
  CellTags->"DiracSubstitute67"],
Cell[660919, 21349, 335, 13, 39, "Text",
  CellTags->"DiracSubstitute67"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[661303, 21368, 58, 1, 39, "ObjectName",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{
Cell[661386, 21373, 62, 1, 49, "Subsubsection",
  CellTags->"DiracTrace"],
Cell[661451, 21376, 329, 6, 56, "Text",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{
Cell[661805, 21386, 147, 3, 56, "Input",
  CellTags->"DiracTrace"],
Cell[661955, 21391, 684, 14, 56, "Output"]
}, Closed]],
Cell[662654, 21408, 306, 12, 35, "Text",
  CellTags->"DiracTrace"],
Cell[662963, 21422, 176, 9, 39, "Text",
  CellTags->"DiracTrace"]
}, Closed]],

Cell[CellGroupData[{
Cell[663176, 21436, 59, 1, 49, "Subsubsection",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{
Cell[663260, 21441, 211, 5, 70, "Input",
  CellTags->"DiracTrace"],
Cell[663474, 21448, 309, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[663820, 21461, 252, 6, 70, "Input",
  CellTags->"DiracTrace"],
Cell[664075, 21469, 423, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[664535, 21484, 181, 4, 70, "Input",
  CellTags->"DiracTrace"],
Cell[664719, 21490, 863, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[665619, 21516, 361, 9, 70, "Input",
  CellTags->"DiracTrace"],
Cell[665983, 21527, 774, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[666794, 21554, 395, 9, 70, "Input",
  CellTags->"DiracTrace"],
Cell[667192, 21565, 13082, 321, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[680311, 21891, 223, 5, 70, "Input",
  CellTags->"DiracTrace"],
Cell[680537, 21898, 815, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[681389, 21925, 347, 8, 70, "Input",
  CellTags->"DiracTrace"],
Cell[681739, 21935, 220, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[681996, 21946, 139, 3, 70, "Input",
  CellTags->"DiracTrace"],
Cell[682138, 21951, 186, 4, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[682385, 21962, 74, 1, 39, "ObjectName",
  CellTags->"DiracTraceEvaluate"],

Cell[CellGroupData[{
Cell[682484, 21967, 70, 1, 49, "Subsubsection",
  CellTags->"DiracTraceEvaluate"],
Cell[682557, 21970, 163, 4, 39, "Text",
  CellTags->"DiracTraceEvaluate"],
Cell[682723, 21976, 303, 13, 39, "Text",
  CellTags->"DiracTraceEvaluate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[683075, 21995, 58, 1, 39, "ObjectName",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[683158, 22000, 62, 1, 49, "Subsubsection",
  CellTags->"DiracTrick"],
Cell[683223, 22003, 223, 5, 56, "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[683471, 22012, 147, 3, 56, "Input",
  CellTags->"DiracTrick"],
Cell[683621, 22017, 189, 5, 70, "Output"]
}, Closed]],
Cell[683825, 22025, 461, 19, 35, "Text",
  CellTags->"DiracTrick"]
}, Closed]],

Cell[CellGroupData[{
Cell[684323, 22049, 59, 1, 49, "Subsubsection",
  CellTags->"DiracTrick"],
Cell[684385, 22052, 97, 1, 70, "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[684507, 22057, 212, 5, 70, "Input",
  CellTags->"DiracTrick"],
Cell[684722, 22064, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[685027, 22075, 142, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[685172, 22080, 306, 9, 70, "Output"]
}, Closed]],
Cell[685493, 22092, 94, 1, 70, "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[685612, 22097, 213, 5, 70, "Input",
  CellTags->"DiracTrick"],
Cell[685828, 22104, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[686133, 22115, 142, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[686278, 22120, 342, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[686657, 22135, 208, 5, 70, "Input",
  CellTags->"DiracTrick"],
Cell[686868, 22142, 264, 6, 70, "Output"]
}, Closed]],
Cell[687147, 22151, 189, 7, 70, "Text",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[687361, 22162, 142, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[687506, 22167, 476, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[688019, 22186, 204, 5, 70, "Input",
  CellTags->"DiracTrick"],
Cell[688226, 22193, 260, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[688523, 22204, 142, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[688668, 22209, 319, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[689024, 22223, 387, 10, 70, "Input",
  CellTags->"DiracTrick"],
Cell[689414, 22235, 934, 26, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[690385, 22266, 142, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[690530, 22271, 897, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[691464, 22299, 136, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[691603, 22304, 2863, 79, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[694503, 22388, 457, 12, 70, "Input",
  CellTags->"DiracTrick"],
Cell[694963, 22402, 703, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[695703, 22424, 141, 3, 70, "Input",
  CellTags->"DiracTrick"],
Cell[695847, 22429, 24792, 618, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[720700, 23054, 56, 1, 39, "ObjectName",
  CellTags->"Divideout"],

Cell[CellGroupData[{
Cell[720781, 23059, 61, 1, 49, "Subsubsection",
  CellTags->"Divideout"],
Cell[720845, 23062, 138, 4, 39, "Text",
  CellTags->"Divideout"],
Cell[720986, 23068, 303, 13, 39, "Text",
  CellTags->"Divideout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[721338, 23087, 44, 1, 39, "ObjectName",
  CellTags->"DOT"],

Cell[CellGroupData[{
Cell[721407, 23092, 55, 1, 49, "Subsubsection",
  CellTags->"DOT"],
Cell[721465, 23095, 351, 9, 106, "Text",
  CellTags->"DOT"],
Cell[721819, 23106, 189, 8, 39, "Text",
  CellTags->"DOT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[722057, 23120, 56, 1, 39, "ObjectName",
  CellTags->"DotExpand"],

Cell[CellGroupData[{
Cell[722138, 23125, 61, 1, 49, "Subsubsection",
  CellTags->"DotExpand"],
Cell[722202, 23128, 86, 1, 39, "Text",
  CellTags->"DotExpand"],
Cell[722291, 23131, 624, 23, 39, "Text",
  CellTags->"DotExpand"]
}, Closed]],

Cell[CellGroupData[{
Cell[722952, 23159, 58, 1, 49, "Subsubsection",
  CellTags->"DotExpand"],

Cell[CellGroupData[{
Cell[723035, 23164, 359, 10, 43, "Input",
  CellTags->"DotExpand"],
Cell[723397, 23176, 532, 12, 23, "Output"]
}, Closed]],
Cell[723944, 23191, 256, 7, 43, "Input",
  CellTags->"DotExpand"],

Cell[CellGroupData[{
Cell[724225, 23202, 359, 10, 43, "Input",
  CellTags->"DotExpand"],
Cell[724587, 23214, 721, 21, 23, "Output"]
}, Closed]],
Cell[725323, 23238, 258, 7, 43, "Input",
  CellTags->"DotExpand"],

Cell[CellGroupData[{
Cell[725606, 23249, 359, 10, 43, "Input",
  CellTags->"DotExpand"],
Cell[725968, 23261, 532, 12, 23, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[726561, 23280, 54, 1, 39, "ObjectName",
  CellTags->"DotPower"],

Cell[CellGroupData[{
Cell[726640, 23285, 60, 1, 49, "Subsubsection",
  CellTags->"DotPower"],
Cell[726703, 23288, 192, 5, 39, "Text",
  CellTags->"DotPower"],
Cell[726898, 23295, 194, 8, 39, "Text",
  CellTags->"DotPower"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[727141, 23309, 58, 1, 39, "ObjectName",
  CellTags->"DotProduct"],

Cell[CellGroupData[{
Cell[727224, 23314, 62, 1, 49, "Subsubsection",
  CellTags->"DotProduct"],
Cell[727289, 23317, 220, 5, 56, "Text",
  CellTags->"DotProduct"],
Cell[727512, 23324, 335, 14, 39, "Text",
  CellTags->"DotProduct"]
}, Closed]],

Cell[CellGroupData[{
Cell[727884, 23343, 59, 1, 49, "Subsubsection",
  CellTags->"DotProduct"],

Cell[CellGroupData[{
Cell[727968, 23348, 281, 7, 56, "Input",
  CellTags->"DotProduct"],
Cell[728252, 23357, 396, 10, 39, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[728709, 23374, 78, 1, 39, "ObjectName",
  CellTags->"DotSimplifyRelations"],

Cell[CellGroupData[{
Cell[728812, 23379, 72, 1, 49, "Subsubsection",
  CellTags->"DotSimplifyRelations"],
Cell[728887, 23382, 242, 5, 56, "Text",
  CellTags->"DotSimplifyRelations"],
Cell[729132, 23389, 206, 8, 39, "Text",
  CellTags->"DotSimplifyRelations"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[729387, 23403, 60, 1, 39, "ObjectName",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[729472, 23408, 63, 1, 49, "Subsubsection",
  CellTags->"DotSimplify"],
Cell[729538, 23411, 325, 7, 56, "Text",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[729888, 23422, 149, 3, 56, "Input",
  CellTags->"DotSimplify"],
Cell[730040, 23427, 360, 9, 70, "Output"]
}, Closed]],
Cell[730415, 23439, 447, 19, 35, "Text",
  CellTags->"DotSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[730899, 23463, 60, 1, 49, "Subsubsection",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[730984, 23468, 417, 11, 70, "Input",
  CellTags->"DotSimplify"],
Cell[731404, 23481, 532, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[731973, 23500, 144, 3, 70, "Input",
  CellTags->"DotSimplify"],
Cell[732120, 23505, 702, 19, 70, "Output"]
}, Closed]],
Cell[732837, 23527, 190, 4, 70, "Input",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[733052, 23535, 259, 7, 70, "Input",
  CellTags->"DotSimplify"],
Cell[733314, 23544, 245, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[733596, 23556, 144, 3, 70, "Input",
  CellTags->"DotSimplify"],
Cell[733743, 23561, 244, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[734024, 23572, 198, 5, 70, "Input",
  CellTags->"DotSimplify"],
Cell[734225, 23579, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[734370, 23586, 144, 3, 70, "Input",
  CellTags->"DotSimplify"],
Cell[734517, 23591, 347, 10, 70, "Output"]
}, Closed]],
Cell[734879, 23604, 198, 5, 70, "Input",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[735102, 23613, 144, 3, 70, "Input",
  CellTags->"DotSimplify"],
Cell[735249, 23618, 244, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[735530, 23629, 202, 5, 70, "Input",
  CellTags->"DotSimplify"],
Cell[735735, 23636, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[735880, 23643, 144, 3, 70, "Input",
  CellTags->"DotSimplify"],
Cell[736027, 23648, 305, 8, 70, "Output"]
}, Closed]],
Cell[736347, 23659, 202, 5, 70, "Input",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[736574, 23668, 365, 9, 70, "Input",
  CellTags->"DotSimplify"],
Cell[736942, 23679, 181, 5, 70, "Output"]
}, Closed]],
Cell[737138, 23687, 153, 3, 70, "Input",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[737316, 23694, 180, 4, 70, "Input",
  CellTags->"DotSimplify"],
Cell[737499, 23700, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[737672, 23708, 255, 6, 70, "Input",
  CellTags->"DotSimplify"],
Cell[737930, 23716, 145, 3, 70, "Output"]
}, Closed]],
Cell[738090, 23722, 202, 4, 70, "Input",
  CellTags->"DotSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[738341, 23732, 58, 1, 39, "ObjectName",
  CellTags->"DummyIndex"],

Cell[CellGroupData[{
Cell[738424, 23737, 62, 1, 49, "Subsubsection",
  CellTags->"DummyIndex"],
Cell[738489, 23740, 194, 5, 39, "Text",
  CellTags->"DummyIndex"],
Cell[738686, 23747, 193, 8, 39, "Text",
  CellTags->"DummyIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[738928, 23761, 42, 1, 39, "ObjectName",
  CellTags->"D0"],

Cell[CellGroupData[{
Cell[738995, 23766, 54, 1, 49, "Subsubsection",
  CellTags->"D0"],
Cell[739052, 23769, 537, 12, 73, "Text",
  CellTags->"D0"],
Cell[739592, 23783, 501, 24, 39, "Text",
  CellTags->"D0"]
}, Closed]],

Cell[CellGroupData[{
Cell[740130, 23812, 51, 1, 49, "Subsubsection",
  CellTags->"D0"],

Cell[CellGroupData[{
Cell[740206, 23817, 380, 10, 56, "Input",
  CellTags->"D0"],
Cell[740589, 23829, 1262, 32, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[741888, 23866, 592, 15, 72, "Input",
  CellTags->"D0"],
Cell[742483, 23883, 1271, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[743791, 23921, 132, 3, 52, "Input",
  CellTags->"D0"],
Cell[743926, 23926, 1262, 32, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[745249, 23965, 62, 1, 39, "ObjectName",
  CellTags->"D0Convention"],

Cell[CellGroupData[{
Cell[745336, 23970, 64, 1, 49, "Subsubsection",
  CellTags->"D0Convention"],
Cell[745403, 23973, 308, 6, 56, "Text",
  CellTags->"D0Convention"],
Cell[745714, 23981, 285, 13, 39, "Text",
  CellTags->"D0Convention"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[746048, 24000, 44, 1, 39, "ObjectName",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[746117, 24005, 55, 1, 49, "Subsubsection",
  CellTags->"Eps"],
Cell[746175, 24008, 360, 9, 56, "Text",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[746560, 24021, 133, 3, 56, "Input",
  CellTags->"Eps"],
Cell[746696, 24026, 185, 5, 70, "Output"]
}, Closed]],
Cell[746896, 24034, 529, 24, 35, "Text",
  CellTags->"Eps"]
}, Closed]],

Cell[CellGroupData[{
Cell[747462, 24063, 52, 1, 49, "Subsubsection",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[747539, 24068, 374, 8, 70, "Input",
  CellTags->"Eps"],
Cell[747916, 24078, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[748618, 24103, 366, 8, 70, "Input",
  CellTags->"Eps"],
Cell[748987, 24113, 606, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[749630, 24136, 222, 6, 70, "Input",
  CellTags->"Eps"],
Cell[749855, 24144, 194, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[750086, 24153, 174, 4, 70, "Input",
  CellTags->"Eps"],
Cell[750263, 24159, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[750408, 24166, 174, 4, 70, "Input",
  CellTags->"Eps"],
Cell[750585, 24172, 130, 3, 70, "Output"]
}, Closed]],
Cell[750730, 24178, 232, 6, 70, "Input",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[750987, 24188, 549, 13, 70, "Input",
  CellTags->"Eps"],
Cell[751539, 24203, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[752241, 24228, 156, 4, 70, "Input",
  CellTags->"Eps"],
Cell[752400, 24234, 131, 3, 70, "Output"]
}, Closed]],
Cell[752546, 24240, 232, 6, 70, "Input",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[752803, 24250, 549, 13, 70, "Input",
  CellTags->"Eps"],
Cell[753355, 24265, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[754057, 24290, 192, 5, 70, "Input",
  CellTags->"Eps"],
Cell[754252, 24297, 342, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[754631, 24312, 541, 14, 70, "Input",
  CellTags->"Eps"],
Cell[755175, 24328, 1531, 44, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[756743, 24377, 1225, 29, 70, "Input",
  CellTags->"Eps"],
Cell[757971, 24408, 2251, 60, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[760259, 24473, 231, 6, 70, "Input",
  CellTags->"Eps"],
Cell[760493, 24481, 406, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[760936, 24498, 156, 4, 70, "Input",
  CellTags->"Eps"],
Cell[761095, 24504, 108, 2, 70, "Output"]
}, Closed]],
Cell[761218, 24509, 200, 5, 70, "Input",
  CellTags->"Eps"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[761467, 24520, 60, 1, 39, "ObjectName",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{
Cell[761552, 24525, 63, 1, 49, "Subsubsection",
  CellTags->"EpsChisholm"],
Cell[761618, 24528, 167, 4, 39, "Text",
  CellTags->"EpsChisholm"],
Cell[761788, 24534, 195, 9, 39, "Text",
  CellTags->"EpsChisholm"]
}, Closed]],

Cell[CellGroupData[{
Cell[762020, 24548, 60, 1, 49, "Subsubsection",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{
Cell[762105, 24553, 251, 6, 70, "Input",
  CellTags->"EpsChisholm"],
Cell[762359, 24561, 2728, 78, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[765124, 24644, 143, 3, 70, "Input",
  CellTags->"EpsChisholm"],
Cell[765270, 24649, 747, 23, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[766078, 24679, 60, 1, 39, "ObjectName",
  CellTags->"EpsContract"],

Cell[CellGroupData[{
Cell[766163, 24684, 63, 1, 49, "Subsubsection",
  CellTags->"EpsContract"],
Cell[766229, 24687, 228, 5, 56, "Text",
  CellTags->"EpsContract"],
Cell[766460, 24694, 300, 14, 39, "Text",
  CellTags->"EpsContract"]
}, Closed]],

Cell[CellGroupData[{
Cell[766797, 24713, 60, 1, 49, "Subsubsection",
  CellTags->"EpsContract"],

Cell[CellGroupData[{
Cell[766882, 24718, 555, 13, 76, "Input",
  CellTags->"EpsContract"],
Cell[767440, 24733, 663, 20, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[768140, 24758, 245, 6, 56, "Input",
  CellTags->"EpsContract"],
Cell[768388, 24766, 799, 23, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[769224, 24794, 244, 6, 56, "Input",
  CellTags->"EpsContract"],
Cell[769471, 24802, 129, 3, 37, "Output"]
}, Closed]],
Cell[769615, 24808, 109, 2, 43, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[769773, 24816, 58, 1, 39, "ObjectName",
  CellTags->"EpsDiscard"],

Cell[CellGroupData[{
Cell[769856, 24821, 62, 1, 49, "Subsubsection",
  CellTags->"EpsDiscard"],
Cell[769921, 24824, 170, 4, 39, "Text",
  CellTags->"EpsDiscard"],
Cell[770094, 24830, 209, 9, 39, "Text",
  CellTags->"EpsDiscard"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[770352, 24845, 60, 1, 39, "ObjectName",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{
Cell[770437, 24850, 63, 1, 49, "Subsubsection",
  CellTags->"EpsEvaluate"],
Cell[770503, 24853, 174, 4, 39, "Text",
  CellTags->"EpsEvaluate"],
Cell[770680, 24859, 537, 24, 39, "Text",
  CellTags->"EpsEvaluate"]
}, Closed]],

Cell[CellGroupData[{
Cell[771254, 24888, 60, 1, 49, "Subsubsection",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{
Cell[771339, 24893, 410, 11, 70, "Input",
  CellTags->"EpsEvaluate"],
Cell[771752, 24906, 636, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[772425, 24930, 143, 3, 70, "Input",
  CellTags->"EpsEvaluate"],
Cell[772571, 24935, 1189, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[773797, 24975, 144, 3, 70, "Input",
  CellTags->"EpsEvaluate"],
Cell[773944, 24980, 676, 14, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[774681, 25001, 64, 1, 39, "ObjectName",
  CellTags->"EpsUncontract"],

Cell[CellGroupData[{
Cell[774770, 25006, 65, 1, 49, "Subsubsection",
  CellTags->"EpsUncontract"],
Cell[774838, 25009, 118, 4, 39, "Text",
  CellTags->"EpsUncontract"],
Cell[774959, 25015, 308, 14, 39, "Text",
  CellTags->"EpsUncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[775316, 25035, 52, 1, 39, "ObjectName",
  CellTags->"Epsilon"],

Cell[CellGroupData[{
Cell[775393, 25040, 59, 1, 49, "Subsubsection",
  CellTags->"Epsilon"],
Cell[775455, 25043, 247, 9, 39, "Text",
  CellTags->"Epsilon"],
Cell[775705, 25054, 188, 9, 39, "Text",
  CellTags->"Epsilon"]
}, Closed]],

Cell[CellGroupData[{
Cell[775930, 25068, 56, 1, 49, "Subsubsection",
  CellTags->"Epsilon"],

Cell[CellGroupData[{
Cell[776011, 25073, 105, 2, 56, "Input",
  CellTags->"Epsilon"],
Cell[776119, 25077, 162, 4, 37, "Output"]
}, Closed]],
Cell[776296, 25084, 117, 4, 35, "Text",
  CellTags->"Epsilon"],

Cell[CellGroupData[{
Cell[776438, 25092, 702, 19, 76, "Input",
  CellTags->"Epsilon"],
Cell[777143, 25113, 218, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[777422, 25125, 62, 1, 39, "ObjectName",
  CellTags->"EpsilonOrder"],

Cell[CellGroupData[{
Cell[777509, 25130, 64, 1, 49, "Subsubsection",
  CellTags->"EpsilonOrder"],
Cell[777576, 25133, 333, 12, 39, "Text",
  CellTags->"EpsilonOrder"],
Cell[777912, 25147, 310, 14, 39, "Text",
  CellTags->"EpsilonOrder"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[778271, 25167, 56, 1, 39, "ObjectName",
  CellTags->"Expanding"],

Cell[CellGroupData[{
Cell[778352, 25172, 61, 1, 49, "Subsubsection",
  CellTags->"Expanding"],
Cell[778416, 25175, 379, 7, 73, "Text",
  CellTags->"Expanding"],
Cell[778798, 25184, 694, 29, 39, "Text",
  CellTags->"Expanding"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[779541, 25219, 66, 1, 39, "ObjectName",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{
Cell[779632, 25224, 66, 1, 49, "Subsubsection",
  CellTags->"ExpandPartialD"],
Cell[779701, 25227, 191, 5, 39, "Text",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{
Cell[779917, 25236, 100, 2, 29, "Input"],
Cell[780020, 25240, 1754, 41, 140, "Output"]
}, Closed]],
Cell[781789, 25284, 776, 28, 35, "Text",
  CellTags->"ExpandPartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[782602, 25317, 63, 1, 49, "Subsubsection",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{
Cell[782690, 25322, 410, 10, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[783103, 25334, 375, 11, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[783515, 25350, 124, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[783642, 25355, 972, 29, 26, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[784651, 25389, 118, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[784772, 25394, 918, 23, 56, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[785727, 25422, 281, 7, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[786011, 25431, 337, 10, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[786385, 25446, 124, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[786512, 25451, 844, 25, 41, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[787393, 25481, 487, 13, 63, "Input",
  CellTags->"ExpandPartialD"],
Cell[787883, 25496, 345, 9, 31, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[788265, 25510, 124, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[788392, 25515, 2530, 68, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[790959, 25588, 199, 6, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[791161, 25596, 270, 7, 30, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[791468, 25608, 127, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[791598, 25613, 1270, 33, 30, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[792905, 25651, 125, 3, 43, "Input",
  CellTags->"ExpandPartialD"],
Cell[793033, 25656, 2289, 58, 26, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[795359, 25719, 345, 9, 39, "Input",
  CellTags->"ExpandPartialD"],
Cell[795707, 25730, 1568, 46, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[797312, 25781, 125, 3, 39, "Input",
  CellTags->"ExpandPartialD"],
Cell[797440, 25786, 84, 2, 23, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[797585, 25795, 122, 1, 39, "ObjectName",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{
Cell[797732, 25800, 96, 1, 49, "Subsubsection",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[797831, 25803, 253, 5, 64, "Text",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{
Cell[798109, 25812, 185, 3, 56, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[798297, 25817, 254, 6, 37, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],
Cell[798566, 25826, 495, 19, 35, "Text",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[799098, 25850, 93, 1, 49, "Subsubsection",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{
Cell[799216, 25855, 304, 7, 56, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[799523, 25864, 501, 12, 37, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[800061, 25881, 178, 3, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[800242, 25886, 1811, 49, 37, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[802090, 25940, 223, 5, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[802316, 25947, 408, 11, 70, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[802761, 25963, 184, 3, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[802948, 25968, 446, 12, 70, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[803431, 25985, 227, 5, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[803661, 25992, 313, 8, 70, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[804011, 26005, 184, 3, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[804198, 26010, 577, 18, 70, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[804812, 26033, 286, 7, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[805101, 26042, 475, 12, 37, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]],

Cell[CellGroupData[{
Cell[805613, 26059, 215, 4, 52, "Input",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[805831, 26065, 805, 22, 38, "Output",
  CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[806697, 26094, 52, 1, 39, "ObjectName",
  CellTags->"Expand2"],

Cell[CellGroupData[{
Cell[806774, 26099, 59, 1, 49, "Subsubsection",
  CellTags->"Expand2"],
Cell[806836, 26102, 165, 4, 39, "Text",
  CellTags->"Expand2"]
}, Closed]],

Cell[CellGroupData[{
Cell[807038, 26111, 56, 1, 49, "Subsubsection",
  CellTags->"Expand2"],

Cell[CellGroupData[{
Cell[807119, 26116, 734, 21, 56, "Input",
  CellTags->"Expand2"],
Cell[807856, 26139, 670, 18, 38, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[808587, 26164, 54, 1, 39, "ObjectName",
  CellTags->"Explicit"],

Cell[CellGroupData[{
Cell[808666, 26169, 60, 1, 49, "Subsubsection",
  CellTags->"Explicit"],
Cell[808729, 26172, 407, 9, 56, "Text",
  CellTags->"Explicit"],
Cell[809139, 26183, 354, 14, 39, "Text",
  CellTags->"Explicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[809530, 26202, 57, 1, 49, "Subsubsection",
  CellTags->"Explicit"],

Cell[CellGroupData[{
Cell[809612, 26207, 271, 6, 70, "Input",
  CellTags->"Explicit"],
Cell[809886, 26215, 1426, 42, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[811349, 26262, 137, 3, 70, "Input",
  CellTags->"Explicit"],
Cell[811489, 26267, 4500, 128, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[816026, 26400, 220, 5, 70, "Input",
  CellTags->"Explicit"],
Cell[816249, 26407, 1190, 36, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[817476, 26448, 137, 3, 70, "Input",
  CellTags->"Explicit"],
Cell[817616, 26453, 4147, 112, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[821800, 26570, 187, 4, 70, "Input",
  CellTags->"Explicit"],
Cell[821990, 26576, 197, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[822224, 26586, 137, 3, 70, "Input",
  CellTags->"Explicit"],
Cell[822364, 26591, 2065, 65, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[824490, 26663, 78, 1, 39, "ObjectName",
  CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{
Cell[824593, 26668, 72, 1, 49, "Subsubsection",
  CellTags->"ExplicitLorentzIndex"],
Cell[824668, 26671, 142, 4, 39, "Text",
  CellTags->"ExplicitLorentzIndex"],
Cell[824813, 26677, 317, 13, 39, "Text",
  CellTags->"ExplicitLorentzIndex"]
}, Closed]],

Cell[CellGroupData[{
Cell[825167, 26695, 69, 1, 49, "Subsubsection",
  CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{
Cell[825261, 26700, 263, 6, 56, "Input",
  CellTags->"ExplicitLorentzIndex"],
Cell[825527, 26708, 233, 7, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[825797, 26720, 146, 3, 56, "Input",
  CellTags->"ExplicitLorentzIndex"],
Cell[825946, 26725, 250, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[826257, 26737, 70, 1, 39, "ObjectName",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{
Cell[826352, 26742, 68, 1, 49, "Subsubsection",
  CellTags->"ExplicitPartialD"],
Cell[826423, 26745, 163, 4, 39, "Text",
  CellTags->"ExplicitPartialD"],
Cell[826589, 26751, 508, 18, 39, "Text",
  CellTags->"ExplicitPartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[827134, 26774, 65, 1, 49, "Subsubsection",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{
Cell[827224, 26779, 215, 4, 56, "Input",
  CellTags->"ExplicitPartialD"],
Cell[827442, 26785, 651, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[828130, 26808, 218, 5, 52, "Input",
  CellTags->"ExplicitPartialD"],
Cell[828351, 26815, 516, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[828904, 26835, 225, 5, 52, "Input",
  CellTags->"ExplicitPartialD"],
Cell[829132, 26842, 545, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[829714, 26861, 203, 6, 39, "Input",
  CellTags->"ExplicitPartialD"],
Cell[829920, 26869, 272, 7, 30, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[830229, 26881, 129, 3, 39, "Input",
  CellTags->"ExplicitPartialD"],
Cell[830361, 26886, 1663, 43, 30, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[832085, 26936, 70, 1, 39, "ObjectName",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{
Cell[832180, 26941, 68, 1, 49, "Subsubsection",
  CellTags->"ExplicitSUNIndex"],
Cell[832251, 26944, 184, 6, 39, "Text",
  CellTags->"ExplicitSUNIndex"],
Cell[832438, 26952, 526, 23, 39, "Text",
  CellTags->"ExplicitSUNIndex"]
}, Closed]],

Cell[CellGroupData[{
Cell[833001, 26980, 65, 1, 49, "Subsubsection",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{
Cell[833091, 26985, 153, 3, 56, "Input",
  CellTags->"ExplicitSUNIndex"],
Cell[833247, 26990, 108, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[833392, 26997, 219, 6, 39, "Input"],
Cell[833614, 27005, 242, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[833917, 27017, 60, 1, 39, "ObjectName",
  CellTags->"ExtraFactor"],

Cell[CellGroupData[{
Cell[834002, 27022, 63, 1, 49, "Subsubsection",
  CellTags->"ExtraFactor"],
Cell[834068, 27025, 202, 5, 39, "Text",
  CellTags->"ExtraFactor"],
Cell[834273, 27032, 206, 8, 39, "Text",
  CellTags->"ExtraFactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[834528, 27046, 66, 1, 39, "ObjectName",
  CellTags->"ExtraVariables"],

Cell[CellGroupData[{
Cell[834619, 27051, 66, 1, 49, "Subsubsection",
  CellTags->"ExtraVariables"],
Cell[834688, 27054, 214, 5, 56, "Text",
  CellTags->"ExtraVariables"],
Cell[834905, 27061, 321, 14, 39, "Text",
  CellTags->"ExtraVariables"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[835275, 27081, 58, 1, 39, "ObjectName",
  CellTags->"FactorFull"],

Cell[CellGroupData[{
Cell[835358, 27086, 62, 1, 49, "Subsubsection",
  CellTags->"FactorFull"],
Cell[835423, 27089, 549, 24, 39, "Text",
  CellTags->"FactorFull"],
Cell[835975, 27115, 191, 9, 39, "Text",
  CellTags->"FactorFull"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[836215, 27130, 56, 1, 39, "ObjectName",
  CellTags->"Factoring"],

Cell[CellGroupData[{
Cell[836296, 27135, 61, 1, 49, "Subsubsection",
  CellTags->"Factoring"],
Cell[836360, 27138, 236, 5, 56, "Text",
  CellTags->"Factoring"],
Cell[836599, 27145, 415, 19, 39, "Text",
  CellTags->"Factoring"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[837063, 27170, 56, 1, 39, "ObjectName",
  CellTags->"Factorout"],

Cell[CellGroupData[{
Cell[837144, 27175, 61, 1, 49, "Subsubsection",
  CellTags->"Factorout"],
Cell[837208, 27178, 75, 1, 39, "Text",
  CellTags->"Factorout"],
Cell[837286, 27181, 187, 9, 39, "Text",
  CellTags->"Factorout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[837522, 27196, 58, 1, 39, "ObjectName",
  CellTags->"FactorTime"],

Cell[CellGroupData[{
Cell[837605, 27201, 62, 1, 49, "Subsubsection",
  CellTags->"FactorTime"],
Cell[837670, 27204, 167, 4, 39, "Text",
  CellTags->"FactorTime"],
Cell[837840, 27210, 191, 9, 39, "Text",
  CellTags->"FactorTime"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[838080, 27225, 52, 1, 39, "ObjectName",
  CellTags->"Factor1"],

Cell[CellGroupData[{
Cell[838157, 27230, 59, 1, 49, "Subsubsection",
  CellTags->"Factor1"],
Cell[838219, 27233, 145, 4, 39, "Text",
  CellTags->"Factor1"],
Cell[838367, 27239, 188, 9, 39, "Text",
  CellTags->"Factor1"]
}, Closed]],

Cell[CellGroupData[{
Cell[838592, 27253, 56, 1, 49, "Subsubsection",
  CellTags->"Factor1"],

Cell[CellGroupData[{
Cell[838673, 27258, 266, 8, 56, "Input",
  CellTags->"Factor1"],
Cell[838942, 27268, 254, 7, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[839233, 27280, 270, 7, 52, "Input",
  CellTags->"Factor1"],
Cell[839506, 27289, 713, 21, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[840256, 27315, 313, 9, 52, "Input",
  CellTags->"Factor1"],
Cell[840572, 27326, 194, 5, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[840803, 27336, 134, 3, 52, "Input",
  CellTags->"Factor1"],
Cell[840940, 27341, 254, 7, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[841231, 27353, 135, 3, 52, "Input",
  CellTags->"Factor1"],
Cell[841369, 27358, 194, 5, 38, "Output"]
}, Closed]],
Cell[841578, 27366, 172, 4, 52, "Input",
  CellTags->"Factor1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[841799, 27376, 52, 1, 39, "ObjectName",
  CellTags->"Factor2"],

Cell[CellGroupData[{
Cell[841876, 27381, 59, 1, 49, "Subsubsection",
  CellTags->"Factor2"],
Cell[841938, 27384, 399, 7, 73, "Text",
  CellTags->"Factor2"],

Cell[CellGroupData[{
Cell[842362, 27395, 141, 3, 56, "Input",
  CellTags->"Factor2"],
Cell[842506, 27400, 263, 7, 37, "Output"]
}, Closed]],
Cell[842784, 27410, 191, 9, 35, "Text",
  CellTags->"Factor2"]
}, Closed]],

Cell[CellGroupData[{
Cell[843012, 27424, 56, 1, 49, "Subsubsection",
  CellTags->"Factor2"],

Cell[CellGroupData[{
Cell[843093, 27429, 267, 8, 70, "Input",
  CellTags->"Factor2"],
Cell[843363, 27439, 255, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[843655, 27451, 271, 7, 70, "Input",
  CellTags->"Factor2"],
Cell[843929, 27460, 537, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[844503, 27481, 314, 9, 70, "Input",
  CellTags->"Factor2"],
Cell[844820, 27492, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[845052, 27502, 135, 3, 70, "Input",
  CellTags->"Factor2"],
Cell[845190, 27507, 255, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[845482, 27519, 136, 3, 70, "Input",
  CellTags->"Factor2"],
Cell[845621, 27524, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[845853, 27534, 210, 5, 70, "Input",
  CellTags->"Factor2"],
Cell[846066, 27541, 255, 7, 70, "Output"]
}, Closed]],
Cell[846336, 27551, 173, 4, 70, "Input",
  CellTags->"Factor2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[846558, 27561, 44, 1, 39, "ObjectName",
  CellTags->"FAD"],

Cell[CellGroupData[{
Cell[846627, 27566, 55, 1, 49, "Subsubsection",
  CellTags->"FAD"],
Cell[846685, 27569, 338, 7, 56, "Text",
  CellTags->"FAD"],
Cell[847026, 27578, 865, 34, 39, "Text",
  CellTags->"FAD"]
}, Closed]],

Cell[CellGroupData[{
Cell[847928, 27617, 52, 1, 49, "Subsubsection",
  CellTags->"FAD"],

Cell[CellGroupData[{
Cell[848005, 27622, 183, 5, 70, "Input",
  CellTags->"FAD"],
Cell[848191, 27629, 911, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[849139, 27657, 247, 7, 70, "Input",
  CellTags->"FAD"],
Cell[849389, 27666, 1045, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[850471, 27696, 307, 9, 70, "Input",
  CellTags->"FAD"],
Cell[850781, 27707, 179, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[850997, 27716, 269, 8, 70, "Input",
  CellTags->"FAD"],
Cell[851269, 27726, 623, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[851929, 27746, 341, 9, 70, "Input",
  CellTags->"FAD"],
Cell[852273, 27757, 623, 15, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[852957, 27779, 42, 1, 39, "ObjectName",
  CellTags->"FC"],

Cell[CellGroupData[{
Cell[853024, 27784, 54, 1, 49, "Subsubsection",
  CellTags->"FC"],
Cell[853081, 27787, 120, 4, 39, "Text",
  CellTags->"FC"],
Cell[853204, 27793, 588, 24, 39, "Text",
  CellTags->"FC"]
}, Closed]],

Cell[CellGroupData[{
Cell[853829, 27822, 51, 1, 49, "Subsubsection",
  CellTags->"FC"],

Cell[CellGroupData[{
Cell[853905, 27827, 93, 2, 56, "Input",
  CellTags->"FC"],
Cell[854001, 27831, 92, 2, 52, "Output",
  CellTags->"FC"]
}, Closed]],

Cell[CellGroupData[{
Cell[854130, 27838, 273, 7, 52, "Input",
  CellTags->"FC"],
Cell[854406, 27847, 123, 2, 52, "Output",
  CellTags->"FC"]
}, Closed]],
Cell[854544, 27852, 93, 2, 52, "Input",
  CellTags->"FC"],

Cell[CellGroupData[{
Cell[854662, 27858, 273, 7, 56, "Input",
  CellTags->"FC"],
Cell[854938, 27867, 427, 12, 52, "Output",
  CellTags->"FC"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[855426, 27886, 44, 1, 39, "ObjectName",
  CellTags->"FCE"],

Cell[CellGroupData[{
Cell[855495, 27891, 55, 1, 49, "Subsubsection",
  CellTags->"FCE"],
Cell[855553, 27894, 127, 4, 39, "Text",
  CellTags->"FCE"],
Cell[855683, 27900, 73, 1, 39, "Text",
  CellTags->"FCE"],
Cell[855759, 27903, 460, 19, 39, "Text",
  CellTags->"FCE"]
}, Closed]],

Cell[CellGroupData[{
Cell[856256, 27927, 52, 1, 49, "Subsubsection",
  CellTags->"FCE"],

Cell[CellGroupData[{
Cell[856333, 27932, 325, 9, 70, "Input",
  CellTags->"FCE"],
Cell[856661, 27943, 300, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[856998, 27956, 132, 3, 70, "Input",
  CellTags->"FCE"],
Cell[857133, 27961, 213, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[857383, 27971, 517, 13, 70, "Input",
  CellTags->"FCE"],
Cell[857903, 27986, 769, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[858709, 28011, 132, 3, 70, "Input",
  CellTags->"FCE"],
Cell[858844, 28016, 513, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[859394, 28033, 123, 3, 70, "Input",
  CellTags->"FCE"],
Cell[859520, 28038, 1212, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[860769, 28078, 132, 3, 70, "Input",
  CellTags->"FCE"],
Cell[860904, 28083, 1029, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[861970, 28111, 166, 4, 70, "Input",
  CellTags->"FCE"],
Cell[862139, 28117, 513, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[862713, 28136, 44, 1, 39, "ObjectName",
  CellTags->"FCI"],

Cell[CellGroupData[{
Cell[862782, 28141, 55, 1, 49, "Subsubsection",
  CellTags->"FCI"],
Cell[862840, 28144, 162, 5, 39, "Text",
  CellTags->"FCI"],
Cell[863005, 28151, 460, 19, 39, "Text",
  CellTags->"FCI"]
}, Closed]],

Cell[CellGroupData[{
Cell[863502, 28175, 52, 1, 49, "Subsubsection",
  CellTags->"FCI"],

Cell[CellGroupData[{
Cell[863579, 28180, 517, 13, 70, "Input",
  CellTags->"FCI"],
Cell[864099, 28195, 769, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[864905, 28220, 132, 3, 70, "Input",
  CellTags->"FCI"],
Cell[865040, 28225, 513, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[865590, 28242, 123, 3, 70, "Input",
  CellTags->"FCI"],
Cell[865716, 28247, 1212, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[866965, 28287, 132, 3, 70, "Input",
  CellTags->"FCI"],
Cell[867100, 28292, 1029, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[868166, 28320, 166, 4, 70, "Input",
  CellTags->"FCI"],
Cell[868335, 28326, 513, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[868909, 28345, 58, 1, 39, "ObjectName",
  CellTags->"FCIntegral"],

Cell[CellGroupData[{
Cell[868992, 28350, 62, 1, 49, "Subsubsection",
  CellTags->"FCIntegral"],
Cell[869057, 28353, 210, 5, 56, "Text",
  CellTags->"FCIntegral"],
Cell[869270, 28360, 383, 14, 39, "Text",
  CellTags->"FCIntegral"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[869702, 28380, 60, 1, 39, "ObjectName",
  CellTags->"FCIntegrate"],

Cell[CellGroupData[{
Cell[869787, 28385, 63, 1, 49, "Subsubsection",
  CellTags->"FCIntegrate"],
Cell[869853, 28388, 288, 6, 56, "Text",
  CellTags->"FCIntegrate"],
Cell[870144, 28396, 207, 9, 39, "Text",
  CellTags->"FCIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[870400, 28411, 62, 1, 39, "ObjectName",
  CellTags->"FCNIntegrate"],

Cell[CellGroupData[{
Cell[870487, 28416, 64, 1, 49, "Subsubsection",
  CellTags->"FCNIntegrate"],
Cell[870554, 28419, 421, 8, 73, "Text",
  CellTags->"FCNIntegrate"],
Cell[870978, 28429, 205, 9, 39, "Text",
  CellTags->"FCNIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[871232, 28444, 67, 1, 39, "ObjectName",
  CellTags->"FC2RHI"],

Cell[CellGroupData[{
Cell[871324, 28449, 58, 1, 49, "Subsubsection",
  CellTags->"FC2RHI"],
Cell[871385, 28452, 307, 6, 56, "Text",
  CellTags->"FC2RHI"],

Cell[CellGroupData[{
Cell[871717, 28462, 138, 3, 56, "Input",
  CellTags->"FC2RHI"],
Cell[871858, 28467, 316, 8, 37, "Output"]
}, Closed]],
Cell[872189, 28478, 175, 9, 35, "Text",
  CellTags->"FC2RHI"]
}, Closed]],

Cell[CellGroupData[{
Cell[872401, 28492, 55, 1, 49, "Subsubsection",
  CellTags->"FC2RHI"],
Cell[872459, 28495, 97, 2, 56, "Input",
  CellTags->"FC2RHI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[872605, 28503, 67, 1, 39, "ObjectName",
  CellTags->"FC2TLI"],

Cell[CellGroupData[{
Cell[872697, 28508, 58, 1, 49, "Subsubsection",
  CellTags->"FC2TLI"],
Cell[872758, 28511, 259, 6, 56, "Text",
  CellTags->"FC2TLI"],

Cell[CellGroupData[{
Cell[873042, 28521, 138, 3, 56, "Input",
  CellTags->"FC2TLI"],
Cell[873183, 28526, 316, 8, 37, "Output"]
}, Closed]],
Cell[873514, 28537, 175, 9, 35, "Text",
  CellTags->"FC2TLI"]
}, Closed]],

Cell[CellGroupData[{
Cell[873726, 28551, 55, 1, 49, "Subsubsection",
  CellTags->"FC2TLI"],
Cell[873784, 28554, 97, 2, 56, "Input",
  CellTags->"FC2TLI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[873930, 28562, 44, 1, 39, "ObjectName",
  CellTags->"FDS"],

Cell[CellGroupData[{
Cell[873999, 28567, 55, 1, 49, "Subsubsection",
  CellTags->"FDS"],
Cell[874057, 28570, 83, 1, 39, "Text",
  CellTags->"FDS"],
Cell[874143, 28573, 241, 9, 39, "Text",
  CellTags->"FDS"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[874433, 28588, 66, 1, 39, "ObjectName",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{
Cell[874524, 28593, 66, 1, 49, "Subsubsection",
  CellTags->"FermionSpinSum"],
Cell[874593, 28596, 121, 4, 39, "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{
Cell[874739, 28604, 155, 3, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[874897, 28609, 343, 8, 70, "Output"]
}, Closed]],
Cell[875255, 28620, 564, 24, 35, "Text",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[875856, 28649, 63, 1, 49, "Subsubsection",
  CellTags->"FermionSpinSum"],
Cell[875922, 28652, 275, 12, 39, "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{
Cell[876222, 28668, 353, 9, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[876578, 28679, 608, 15, 50, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[877223, 28699, 174, 4, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[877400, 28705, 468, 13, 50, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[877905, 28723, 174, 4, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[878082, 28729, 164, 4, 50, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[878283, 28738, 353, 9, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[878639, 28749, 608, 15, 37, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[879284, 28769, 174, 4, 52, "Input",
  CellTags->"FermionSpinSum"],
Cell[879461, 28775, 468, 13, 37, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[879966, 28793, 174, 4, 52, "Input",
  CellTags->"FermionSpinSum"],
Cell[880143, 28799, 188, 5, 37, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[880368, 28809, 398, 10, 52, "Input",
  CellTags->"FermionSpinSum"],
Cell[880769, 28821, 833, 20, 70, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],
Cell[881617, 28844, 165, 4, 35, "Text",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{
Cell[881807, 28852, 179, 4, 56, "Input",
  CellTags->"FermionSpinSum"],
Cell[881989, 28858, 889, 21, 70, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[882915, 28884, 177, 4, 52, "Input",
  CellTags->"FermionSpinSum"],
Cell[883095, 28890, 1621, 40, 70, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[884753, 28935, 180, 4, 52, "Input",
  CellTags->"FermionSpinSum"],
Cell[884936, 28941, 1264, 36, 70, "Output",
  CellTags->"FermionSpinSum"]
}, Closed]],
Cell[886215, 28980, 173, 4, 52, "Input",
  CellTags->"FermionSpinSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[886437, 28990, 52, 1, 39, "ObjectName",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{
Cell[886514, 28995, 59, 1, 49, "Subsubsection",
  CellTags->"FeynAmp"],
Cell[886576, 28998, 244, 5, 56, "Text",
  CellTags->"FeynAmp"],
Cell[886823, 29005, 166, 4, 39, "Text",
  CellTags->"FeynAmp"],
Cell[886992, 29011, 194, 9, 39, "Text",
  CellTags->"FeynAmp"]
}, Closed]],

Cell[CellGroupData[{
Cell[887223, 29025, 56, 1, 49, "Subsubsection",
  CellTags->"FeynAmp"],
Cell[887282, 29028, 126, 4, 70, "Text",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{
Cell[887433, 29036, 1007, 22, 70, "Input",
  CellTags->"FeynAmp"],
Cell[888443, 29060, 5557, 138, 70, "Output"]
}, Closed]],
Cell[894015, 29201, 74, 1, 70, "Text",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{
Cell[894114, 29206, 240, 6, 70, "Input",
  CellTags->"FeynAmp"],
Cell[894357, 29214, 488, 13, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[894906, 29234, 74, 1, 39, "ObjectName",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{
Cell[895005, 29239, 70, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominator"],
Cell[895078, 29242, 259, 6, 56, "Text",
  CellTags->"FeynAmpDenominator"],
Cell[895340, 29250, 361, 14, 39, "Text",
  CellTags->"FeynAmpDenominator"]
}, Closed]],

Cell[CellGroupData[{
Cell[895738, 29269, 67, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{
Cell[895830, 29274, 238, 5, 70, "Input",
  CellTags->"FeynAmpDenominator"],
Cell[896071, 29281, 226, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[896334, 29292, 387, 9, 70, "Input",
  CellTags->"FeynAmpDenominator"],
Cell[896724, 29303, 517, 15, 70, "Output"]
}, Closed]],
Cell[897256, 29321, 297, 7, 70, "Input",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{
Cell[897578, 29332, 181, 4, 70, "Input",
  CellTags->"FeynAmpDenominator"],
Cell[897762, 29338, 302, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[898101, 29350, 181, 4, 70, "Input",
  CellTags->"FeynAmpDenominator"],
Cell[898285, 29356, 179, 4, 70, "Output"]
}, Closed]],
Cell[898479, 29363, 166, 4, 70, "Input",
  CellTags->"FeynAmpDenominator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[898694, 29373, 88, 1, 39, "ObjectName",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{
Cell[898807, 29378, 77, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorCombine"],
Cell[898887, 29381, 229, 5, 56, "Text",
  CellTags->"FeynAmpDenominatorCombine"],
Cell[899119, 29388, 254, 9, 39, "Text",
  CellTags->"FeynAmpDenominatorCombine"]
}, Closed]],

Cell[CellGroupData[{
Cell[899410, 29402, 74, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{
Cell[899509, 29407, 282, 7, 70, "Input",
  CellTags->"FeynAmpDenominatorCombine"],
Cell[899794, 29416, 974, 26, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[900805, 29447, 249, 6, 70, "Input",
  CellTags->"FeynAmpDenominatorCombine"],
Cell[901057, 29455, 205, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[901299, 29465, 247, 6, 70, "Input",
  CellTags->"FeynAmpDenominatorCombine"],
Cell[901549, 29473, 239, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[901849, 29486, 90, 1, 39, "ObjectName",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{
Cell[901964, 29491, 78, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[902045, 29494, 399, 7, 73, "Text",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[902447, 29503, 86, 1, 39, "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{
Cell[902558, 29508, 120, 2, 56, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[902681, 29512, 133, 2, 70, "Output"]
}, Closed]],
Cell[902829, 29517, 231, 9, 35, "Text",
  CellTags->"FeynAmpDenominatorSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[903097, 29531, 75, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[903175, 29534, 507, 14, 70, "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{
Cell[903707, 29552, 336, 8, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[904046, 29562, 108, 2, 70, "Output"]
}, Closed]],
Cell[904169, 29567, 575, 18, 70, "Text",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{
Cell[904769, 29589, 401, 10, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[905173, 29601, 825, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[906035, 29629, 562, 15, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[906600, 29646, 3255, 88, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[909892, 29739, 258, 6, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[910153, 29747, 2187, 59, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[912377, 29811, 189, 4, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[912569, 29817, 1059, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[913665, 29852, 464, 13, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"],
Cell[914132, 29867, 3923, 105, 70, "Output"]
}, Closed]],
Cell[918070, 29975, 179, 4, 70, "Input",
  CellTags->"FeynAmpDenominatorSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[918298, 29985, 84, 1, 39, "ObjectName",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{
Cell[918407, 29990, 75, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[918485, 29993, 358, 7, 73, "Text",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[918846, 30002, 258, 9, 39, "Text",
  CellTags->"FeynAmpDenominatorSplit"]
}, Closed]],

Cell[CellGroupData[{
Cell[919141, 30016, 72, 1, 49, "Subsubsection",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{
Cell[919238, 30021, 372, 9, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[919613, 30032, 1178, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[920828, 30070, 144, 3, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[920975, 30075, 125, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[921137, 30082, 167, 3, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[921307, 30087, 1178, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[922522, 30125, 182, 4, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[922707, 30131, 461, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[923205, 30148, 277, 7, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[923485, 30157, 407, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[923929, 30173, 247, 6, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"],
Cell[924179, 30181, 325, 8, 70, "Output"]
}, Closed]],
Cell[924519, 30192, 149, 3, 70, "Input",
  CellTags->"FeynAmpDenominatorSplit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[924717, 30201, 60, 1, 39, "ObjectName",
  CellTags->"FeynAmpList"],

Cell[CellGroupData[{
Cell[924802, 30206, 63, 1, 49, "Subsubsection",
  CellTags->"FeynAmpList"],
Cell[924868, 30209, 146, 4, 39, "Text",
  CellTags->"FeynAmpList"],
Cell[925017, 30215, 120, 4, 39, "Text",
  CellTags->"FeynAmpList"],
Cell[925140, 30221, 192, 9, 39, "Text",
  CellTags->"FeynAmpList"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[925381, 30236, 54, 1, 39, "ObjectName",
  CellTags->"FeynCalc"],

Cell[CellGroupData[{
Cell[925460, 30241, 60, 1, 49, "Subsubsection",
  CellTags->"FeynCalc"],
Cell[925523, 30244, 92, 1, 39, "Text",
  CellTags->"FeynCalc"],

Cell[CellGroupData[{
Cell[925640, 30249, 125, 3, 56, "Input",
  CellTags->"FeynCalc"],
Cell[925768, 30254, 582, 8, 208, "Print",
  CellTags->"FeynCalc"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[926411, 30269, 70, 1, 39, "ObjectName",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{
Cell[926506, 30274, 68, 1, 49, "Subsubsection",
  CellTags->"FeynCalcExternal"],
Cell[926577, 30277, 157, 4, 39, "Text",
  CellTags->"FeynCalcExternal"],
Cell[926737, 30283, 224, 9, 39, "Text",
  CellTags->"FeynCalcExternal"]
}, Closed]],

Cell[CellGroupData[{
Cell[926998, 30297, 65, 1, 49, "Subsubsection",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{
Cell[927088, 30302, 194, 4, 56, "Input",
  CellTags->"FeynCalcExternal"],
Cell[927285, 30308, 143, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[927465, 30316, 145, 3, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[927613, 30321, 122, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[927772, 30328, 530, 13, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[928305, 30343, 769, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[929111, 30368, 145, 3, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[929259, 30373, 513, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[929809, 30390, 149, 3, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[929961, 30395, 1212, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[931210, 30435, 145, 3, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[931358, 30440, 1029, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[932424, 30468, 199, 5, 52, "Input",
  CellTags->"FeynCalcExternal"],
Cell[932626, 30475, 513, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[933200, 30494, 62, 1, 39, "ObjectName",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{
Cell[933287, 30499, 64, 1, 49, "Subsubsection",
  CellTags->"FeynCalcForm"],
Cell[933354, 30502, 540, 10, 73, "Text",
  CellTags->"FeynCalcForm"],
Cell[933897, 30514, 466, 19, 39, "Text",
  CellTags->"FeynCalcForm"]
}, Closed]],

Cell[CellGroupData[{
Cell[934400, 30538, 61, 1, 49, "Subsubsection",
  CellTags->"FeynCalcForm"],
Cell[934464, 30541, 80, 1, 25, "Text",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{
Cell[934569, 30546, 287, 7, 56, "Input",
  CellTags->"FeynCalcForm"],
Cell[934859, 30555, 761, 23, 37, "Output",
  CellTags->"FeynCalcForm"]
}, Closed]],
Cell[935635, 30581, 115, 4, 21, "Text",
  CellTags->"FeynCalcForm"],
Cell[935753, 30587, 178, 4, 43, "Input",
  CellTags->"FeynCalcForm"],
Cell[935934, 30593, 867, 22, 63, "Input",
  CellTags->"FeynCalcForm"],

Cell[CellGroupData[{
Cell[936826, 30619, 287, 7, 56, "Input",
  CellTags->"FeynCalcForm"],
Cell[937116, 30628, 216, 7, 35, "Output",
  CellTags->"FeynCalcForm"]
}, Closed]],
Cell[937347, 30638, 77, 1, 21, "Text",
  CellTags->"FeynCalcForm"],
Cell[937427, 30641, 153, 4, 43, "Input",
  CellTags->"FeynCalcForm"],
Cell[937583, 30647, 895, 23, 63, "Input",
  CellTags->"FeynCalcForm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[938527, 30676, 70, 1, 39, "ObjectName",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{
Cell[938622, 30681, 68, 1, 49, "Subsubsection",
  CellTags->"FeynCalcInternal"],
Cell[938693, 30684, 158, 4, 39, "Text",
  CellTags->"FeynCalcInternal"],
Cell[938854, 30690, 434, 19, 39, "Text",
  CellTags->"FeynCalcInternal"]
}, Closed]],

Cell[CellGroupData[{
Cell[939325, 30714, 65, 1, 49, "Subsubsection",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{
Cell[939415, 30719, 530, 13, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[939948, 30734, 769, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[940754, 30759, 145, 3, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[940902, 30764, 513, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[941452, 30781, 149, 3, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[941604, 30786, 1212, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[942853, 30826, 145, 3, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[943001, 30831, 1029, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[944067, 30859, 199, 5, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[944269, 30866, 513, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[944819, 30883, 621, 15, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[945443, 30900, 2123, 62, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[947603, 30967, 145, 3, 70, "Input",
  CellTags->"FeynCalcInternal"],
Cell[947751, 30972, 1304, 30, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[949116, 31009, 64, 1, 39, "ObjectName",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[949205, 31014, 65, 1, 49, "Subsubsection",
  CellTags->"FeynCalc2FORM"],
Cell[949273, 31017, 227, 5, 56, "Text",
  CellTags->"FeynCalc2FORM"],
Cell[949503, 31024, 212, 9, 39, "Text",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[949740, 31037, 153, 3, 56, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[949896, 31042, 1240, 26, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[951185, 31074, 62, 1, 49, "Subsubsection",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[951272, 31079, 328, 9, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[951603, 31090, 330, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[951970, 31104, 169, 4, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[952142, 31110, 88, 1, 70, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[952267, 31116, 241, 6, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[952511, 31124, 266, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[952814, 31135, 169, 4, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[952986, 31141, 84, 1, 70, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[953107, 31147, 255, 6, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[953365, 31155, 423, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[953825, 31170, 169, 4, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[953997, 31176, 107, 2, 70, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[954141, 31183, 356, 9, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[954500, 31194, 562, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[955099, 31214, 169, 4, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[955271, 31220, 107, 2, 70, "Print"]
}, Closed]],
Cell[955393, 31225, 401, 10, 70, "Input",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[955819, 31239, 406, 10, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[956228, 31251, 164, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[956429, 31260, 385, 10, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[956817, 31272, 8900, 243, 70, "Output"]
}, Closed]],
Cell[965732, 31518, 252, 6, 70, "Input",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[966009, 31528, 460, 11, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[966472, 31541, 1751, 39, 70, "Output"]
}, Closed]],
Cell[968238, 31583, 380, 10, 70, "Input",
  CellTags->"FeynCalc2FORM"],
Cell[968621, 31595, 161, 4, 70, "Input",
  CellTags->"FeynCalc2FORM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[968831, 31605, 80, 1, 39, "ObjectName",
  CellTags->"FeynmanParameterNames"],

Cell[CellGroupData[{
Cell[968936, 31610, 73, 1, 49, "Subsubsection",
  CellTags->"FeynmanParameterNames"],
Cell[969012, 31613, 144, 4, 39, "Text",
  CellTags->"FeynmanParameterNames"],
Cell[969159, 31619, 385, 14, 39, "Text",
  CellTags->"FeynmanParameterNames"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[969593, 31639, 91, 1, 39, "ObjectName",
  CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{
Cell[969709, 31644, 70, 1, 49, "Subsubsection",
  CellTags->"FeynmanParametrize"],
Cell[969782, 31647, 177, 4, 39, "Text",
  CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{
Cell[969984, 31655, 161, 3, 56, "Input",
  CellTags->"FeynmanParametrize"],
Cell[970148, 31660, 269, 7, 37, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[970466, 31673, 67, 1, 49, "Subsubsection",
  CellTags->"FeynmanParametrize"],
Cell[970536, 31676, 109, 2, 56, "Input",
  CellTags->"FeynmanParametrize"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[970694, 31684, 54, 1, 39, "ObjectName",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[970773, 31689, 60, 1, 49, "Subsubsection",
  CellTags->"FeynRule"],
Cell[970836, 31692, 169, 4, 39, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[971030, 31700, 119, 3, 43, "Input",
  CellTags->"FeynRule"],
Cell[971152, 31705, 756, 17, 42, "Output"]
}, Closed]],
Cell[971923, 31725, 93, 1, 35, "Text",
  CellTags->"FeynRule"],
Cell[972019, 31728, 134, 4, 39, "Text",
  CellTags->"FeynRule"],
Cell[972156, 31734, 198, 9, 39, "Text",
  CellTags->"FeynRule"]
}, Closed]],

Cell[CellGroupData[{
Cell[972391, 31748, 57, 1, 49, "Subsubsection",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[972473, 31753, 154, 4, 43, "Input",
  CellTags->"FeynRule"],
Cell[972630, 31759, 849, 22, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[973516, 31786, 154, 4, 43, "Input",
  CellTags->"FeynRule"],
Cell[973673, 31792, 1341, 35, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[975051, 31832, 115, 3, 43, "Input",
  CellTags->"FeynRule"],
Cell[975169, 31837, 5722, 153, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[980928, 31995, 149, 4, 43, "Input",
  CellTags->"FeynRule"],
Cell[981080, 32001, 3618, 112, 27, "Output"]
}, Closed]],
Cell[984713, 32116, 75, 1, 39, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[984813, 32121, 601, 16, 43, "Input",
  CellTags->"FeynRule"],
Cell[985417, 32139, 917, 29, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[986371, 32173, 371, 9, 56, "Input",
  CellTags->"FeynRule"],
Cell[986745, 32184, 6385, 154, 70, "Output"]
}, Closed]],
Cell[993145, 32341, 73, 1, 35, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[993243, 32346, 625, 16, 56, "Input",
  CellTags->"FeynRule"],
Cell[993871, 32364, 941, 29, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[994849, 32398, 371, 9, 52, "Input",
  CellTags->"FeynRule"],
Cell[995223, 32409, 7691, 166, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1002951, 32580, 268, 7, 52, "Input",
  CellTags->"FeynRule"],
Cell[1003222, 32589, 1453, 42, 70, "Output"]
}, Closed]],
Cell[1004690, 32634, 113, 4, 35, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[1004828, 32642, 424, 10, 56, "Input",
  CellTags->"FeynRule"],
Cell[1005255, 32654, 1555, 45, 70, "Output"]
}, Closed]],
Cell[1006825, 32702, 78, 1, 35, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[1006928, 32707, 177, 4, 56, "Input",
  CellTags->"FeynRule"],
Cell[1007108, 32713, 806, 22, 70, "Output"]
}, Closed]],
Cell[1007929, 32738, 91, 1, 35, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[1008045, 32743, 177, 4, 56, "Input",
  CellTags->"FeynRule"],
Cell[1008225, 32749, 806, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1009068, 32776, 863, 22, 72, "Input",
  CellTags->"FeynRule"],
Cell[1009934, 32800, 1298, 38, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1011269, 32843, 277, 7, 52, "Input",
  CellTags->"FeynRule"],
Cell[1011549, 32852, 7312, 184, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1018898, 33041, 501, 12, 52, "Input",
  CellTags->"FeynRule"],
Cell[1019402, 33055, 7316, 164, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1026755, 33224, 445, 12, 52, "Input",
  CellTags->"FeynRule"],
Cell[1027203, 33238, 108, 2, 70, "Output"]
}, Closed]],
Cell[1027326, 33243, 211, 5, 52, "Text",
  CellTags->"FeynRule"],
Cell[1027540, 33250, 176, 4, 56, "Input",
  CellTags->"FeynRule"],
Cell[1027719, 33256, 52, 1, 39, "Text",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[1027796, 33261, 854, 21, 76, "Input",
  CellTags->"FeynRule"],
Cell[1028653, 33284, 1337, 41, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1030027, 33330, 260, 7, 52, "Input",
  CellTags->"FeynRule"],
Cell[1030290, 33339, 2827, 77, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1033154, 33421, 455, 11, 52, "Input",
  CellTags->"FeynRule"],
Cell[1033612, 33434, 2976, 83, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1036625, 33522, 240, 6, 52, "Input",
  CellTags->"FeynRule"],
Cell[1036868, 33530, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1037013, 33537, 1084, 26, 72, "Input",
  CellTags->"FeynRule"],
Cell[1038100, 33565, 1735, 53, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1039872, 33623, 260, 7, 52, "Input",
  CellTags->"FeynRule"],
Cell[1040135, 33632, 4819, 132, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1044991, 33769, 542, 13, 52, "Input",
  CellTags->"FeynRule"],
Cell[1045536, 33784, 9095, 227, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1054668, 34016, 240, 6, 52, "Input",
  CellTags->"FeynRule"],
Cell[1054911, 34024, 108, 2, 70, "Output"]
}, Closed]],
Cell[1055034, 34029, 475, 9, 72, "Input",
  CellTags->"FeynRule"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1055558, 34044, 42, 1, 39, "ObjectName",
  CellTags->"FI"],

Cell[CellGroupData[{
Cell[1055625, 34049, 54, 1, 49, "Subsubsection",
  CellTags->"FI"],
Cell[1055682, 34052, 198, 5, 56, "Text",
  CellTags->"FI"],
Cell[1055883, 34059, 588, 24, 39, "Text",
  CellTags->"FI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1056520, 34089, 68, 1, 39, "ObjectName",
  CellTags->"FieldDerivative"],

Cell[CellGroupData[{
Cell[1056613, 34094, 67, 1, 49, "Subsubsection",
  CellTags->"FieldDerivative"],
Cell[1056683, 34097, 552, 10, 90, "Text",
  CellTags->"FieldDerivative"],
Cell[1057238, 34109, 337, 14, 39, "Text",
  CellTags->"FieldDerivative"]
}, Closed]],

Cell[CellGroupData[{
Cell[1057612, 34128, 64, 1, 49, "Subsubsection",
  CellTags->"FieldDerivative"],

Cell[CellGroupData[{
Cell[1057701, 34133, 720, 20, 43, "Input",
  CellTags->"FieldDerivative"],
Cell[1058424, 34155, 523, 16, 25, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1058984, 34176, 238, 7, 39, "Input",
  CellTags->"FieldDerivative"],
Cell[1059225, 34185, 1987, 56, 26, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1061249, 34246, 238, 7, 39, "Input",
  CellTags->"FieldDerivative"],
Cell[1061490, 34255, 5781, 160, 48, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1067332, 34422, 64, 1, 39, "ObjectName",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{
Cell[1067421, 34427, 65, 1, 49, "Subsubsection",
  CellTags->"FieldStrength"],
Cell[1067489, 34430, 1453, 37, 75, "Text",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{
Cell[1068967, 34471, 153, 3, 56, "Input",
  CellTags->"FieldStrength"],
Cell[1069123, 34476, 548, 13, 70, "Output"]
}, Closed]],
Cell[1069686, 34492, 209, 9, 35, "Text",
  CellTags->"FieldStrength"]
}, Closed]],

Cell[CellGroupData[{
Cell[1069932, 34506, 62, 1, 49, "Subsubsection",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{
Cell[1070019, 34511, 182, 4, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1070204, 34517, 203, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1070444, 34527, 192, 4, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1070639, 34533, 197, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1070873, 34543, 237, 5, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1071113, 34550, 939, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1072089, 34585, 247, 5, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1072339, 34592, 2067, 65, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1074443, 34662, 284, 6, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1074730, 34670, 614, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1075381, 34689, 287, 6, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1075671, 34697, 612, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1076320, 34716, 339, 7, 70, "Input",
  CellTags->"FieldStrength"],
Cell[1076662, 34725, 2067, 65, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1078790, 34797, 64, 1, 39, "ObjectName",
  CellTags->"FinalFunction"],

Cell[CellGroupData[{
Cell[1078879, 34802, 65, 1, 49, "Subsubsection",
  CellTags->"FinalFunction"],
Cell[1078947, 34805, 87, 1, 39, "Text",
  CellTags->"FinalFunction"],
Cell[1079037, 34808, 203, 9, 39, "Text",
  CellTags->"FinalFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1079289, 34823, 74, 1, 39, "ObjectName",
  CellTags->"FinalSubstitutions"],

Cell[CellGroupData[{
Cell[1079388, 34828, 70, 1, 49, "Subsubsection",
  CellTags->"FinalSubstitutions"],
Cell[1079461, 34831, 207, 5, 39, "Text",
  CellTags->"FinalSubstitutions"],
Cell[1079671, 34838, 439, 19, 39, "Text",
  CellTags->"FinalSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1080159, 34863, 46, 1, 39, "ObjectName",
  CellTags->"FORM"],

Cell[CellGroupData[{
Cell[1080230, 34868, 56, 1, 49, "Subsubsection",
  CellTags->"FORM"],
Cell[1080289, 34871, 205, 5, 56, "Text",
  CellTags->"FORM"],
Cell[1080497, 34878, 308, 14, 39, "Text",
  CellTags->"FORM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1080854, 34898, 58, 1, 39, "ObjectName",
  CellTags->"FORMEpilog"],

Cell[CellGroupData[{
Cell[1080937, 34903, 62, 1, 49, "Subsubsection",
  CellTags->"FORMEpilog"],
Cell[1081002, 34906, 162, 4, 39, "Text",
  CellTags->"FORMEpilog"],
Cell[1081167, 34912, 335, 14, 39, "Text",
  CellTags->"FORMEpilog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1081551, 34932, 58, 1, 39, "ObjectName",
  CellTags->"FORMProlog"],

Cell[CellGroupData[{
Cell[1081634, 34937, 62, 1, 49, "Subsubsection",
  CellTags->"FORMProlog"],
Cell[1081699, 34940, 181, 5, 39, "Text",
  CellTags->"FORMProlog"],
Cell[1081883, 34947, 335, 14, 39, "Text",
  CellTags->"FORMProlog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1082267, 34967, 64, 1, 39, "ObjectName",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{
Cell[1082356, 34972, 65, 1, 49, "Subsubsection",
  CellTags->"FORM2FeynCalc"],
Cell[1082424, 34975, 471, 8, 73, "Text",
  CellTags->"FORM2FeynCalc"],
Cell[1082898, 34985, 212, 9, 39, "Text",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{
Cell[1083135, 34998, 153, 3, 56, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1083291, 35003, 754, 17, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1084094, 35026, 62, 1, 49, "Subsubsection",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{
Cell[1084181, 35031, 167, 3, 56, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1084351, 35036, 446, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1084834, 35054, 142, 3, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1084979, 35059, 245, 6, 70, "Output"]
}, Closed]],
Cell[1085239, 35068, 160, 4, 35, "Text",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{
Cell[1085424, 35076, 384, 9, 76, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1085811, 35087, 353, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1086201, 35101, 142, 3, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1086346, 35106, 330, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1086713, 35119, 180, 4, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1086896, 35125, 352, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1087285, 35140, 145, 3, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1087433, 35145, 241, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1087711, 35157, 180, 4, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1087894, 35163, 302, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1088233, 35176, 142, 3, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1088378, 35181, 341, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1088756, 35195, 401, 10, 52, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1089160, 35207, 321, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1089518, 35221, 231, 6, 72, "Input",
  CellTags->"FORM2FeynCalc"],
Cell[1089752, 35229, 640, 16, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1090453, 35252, 66, 1, 39, "ObjectName",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{
Cell[1090544, 35257, 66, 1, 49, "Subsubsection",
  CellTags->"FourDivergence"],
Cell[1090613, 35260, 243, 5, 56, "Text",
  CellTags->"FourDivergence"],
Cell[1090859, 35267, 210, 9, 39, "Text",
  CellTags->"FourDivergence"]
}, Closed]],

Cell[CellGroupData[{
Cell[1091106, 35281, 63, 1, 49, "Subsubsection",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{
Cell[1091194, 35286, 204, 5, 70, "Input",
  CellTags->"FourDivergence"],
Cell[1091401, 35293, 268, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1091706, 35305, 254, 6, 70, "Input",
  CellTags->"FourDivergence"],
Cell[1091963, 35313, 285, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1092285, 35327, 235, 6, 70, "Input",
  CellTags->"FourDivergence"],
Cell[1092523, 35335, 555, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1093115, 35355, 289, 8, 70, "Input",
  CellTags->"FourDivergence"],
Cell[1093407, 35365, 108, 2, 70, "Output"]
}, Closed]],
Cell[1093530, 35370, 140, 3, 70, "Input",
  CellTags->"FourDivergence"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1093719, 35379, 64, 1, 39, "ObjectName",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{
Cell[1093808, 35384, 65, 1, 49, "Subsubsection",
  CellTags->"FourLaplacian"],
Cell[1093876, 35387, 491, 16, 40, "Text",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{
Cell[1094392, 35407, 153, 3, 56, "Input",
  CellTags->"FourLaplacian"],
Cell[1094548, 35412, 185, 5, 70, "Output"]
}, Closed]],
Cell[1094748, 35420, 347, 14, 35, "Text",
  CellTags->"FourLaplacian"]
}, Closed]],

Cell[CellGroupData[{
Cell[1095132, 35439, 62, 1, 49, "Subsubsection",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{
Cell[1095219, 35444, 163, 4, 70, "Input",
  CellTags->"FourLaplacian"],
Cell[1095385, 35450, 183, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1095605, 35460, 184, 4, 70, "Input",
  CellTags->"FourLaplacian"],
Cell[1095792, 35466, 135, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1095964, 35474, 325, 9, 70, "Input",
  CellTags->"FourLaplacian"],
Cell[1096292, 35485, 581, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1096910, 35507, 189, 4, 70, "Input",
  CellTags->"FourLaplacian"],
Cell[1097102, 35513, 5705, 153, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1102868, 35673, 58, 1, 39, "ObjectName",
  CellTags->"FourVector"],

Cell[CellGroupData[{
Cell[1102951, 35678, 62, 1, 49, "Subsubsection",
  CellTags->"FourVector"],
Cell[1103016, 35681, 353, 10, 56, "Text",
  CellTags->"FourVector"],
Cell[1103372, 35693, 389, 19, 39, "Text",
  CellTags->"FourVector"]
}, Closed]],

Cell[CellGroupData[{
Cell[1103798, 35717, 59, 1, 49, "Subsubsection",
  CellTags->"FourVector"],

Cell[CellGroupData[{
Cell[1103882, 35722, 172, 4, 70, "Input",
  CellTags->"FourVector"],
Cell[1104057, 35728, 138, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1104232, 35736, 201, 5, 70, "Input",
  CellTags->"FourVector"],
Cell[1104436, 35743, 333, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1104806, 35758, 217, 5, 70, "Input",
  CellTags->"FourVector"],
Cell[1105026, 35765, 161, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1105224, 35773, 272, 6, 70, "Input",
  CellTags->"FourVector"],
Cell[1105499, 35781, 214, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1105750, 35790, 255, 6, 70, "Input",
  CellTags->"FourVector"],
Cell[1106008, 35798, 161, 3, 70, "Output"]
}, Closed]],
Cell[1106184, 35804, 196, 5, 70, "Text",
  CellTags->"FourVector"],

Cell[CellGroupData[{
Cell[1106405, 35813, 255, 6, 70, "Input",
  CellTags->"FourVector"],
Cell[1106663, 35821, 517, 17, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1107241, 35845, 56, 1, 39, "ObjectName",
  CellTags->"FreeIndex"],

Cell[CellGroupData[{
Cell[1107322, 35850, 61, 1, 49, "Subsubsection",
  CellTags->"FreeIndex"],
Cell[1107386, 35853, 153, 4, 39, "Text",
  CellTags->"FreeIndex"],
Cell[1107542, 35859, 313, 14, 39, "Text",
  CellTags->"FreeIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1107904, 35879, 50, 1, 39, "ObjectName",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{
Cell[1107979, 35884, 58, 1, 49, "Subsubsection",
  CellTags->"FreeQ2"],
Cell[1108040, 35887, 229, 5, 56, "Text",
  CellTags->"FreeQ2"],
Cell[1108272, 35894, 331, 14, 39, "Text",
  CellTags->"FreeQ2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1108640, 35913, 55, 1, 49, "Subsubsection",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{
Cell[1108720, 35918, 300, 8, 70, "Input",
  CellTags->"FreeQ2"],
Cell[1109023, 35928, 112, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1109172, 35935, 295, 8, 70, "Input",
  CellTags->"FreeQ2"],
Cell[1109470, 35945, 111, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1109618, 35952, 165, 4, 70, "Input",
  CellTags->"FreeQ2"],
Cell[1109786, 35958, 111, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1109934, 35965, 199, 5, 70, "Input",
  CellTags->"FreeQ2"],
Cell[1110136, 35972, 112, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1110309, 35981, 44, 1, 39, "ObjectName",
  CellTags->"FRH"],

Cell[CellGroupData[{
Cell[1110378, 35986, 55, 1, 49, "Subsubsection",
  CellTags->"FRH"],
Cell[1110436, 35989, 135, 4, 39, "Text",
  CellTags->"FRH"],
Cell[1110574, 35995, 184, 9, 39, "Text",
  CellTags->"FRH"]
}, Closed]],

Cell[CellGroupData[{
Cell[1110795, 36009, 52, 1, 49, "Subsubsection",
  CellTags->"FRH"],

Cell[CellGroupData[{
Cell[1110872, 36014, 243, 6, 56, "Input",
  CellTags->"FRH"],
Cell[1111118, 36022, 291, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1111446, 36035, 127, 3, 52, "Input",
  CellTags->"FRH"],
Cell[1111576, 36040, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1111721, 36047, 373, 10, 52, "Input",
  CellTags->"FRH"],
Cell[1112097, 36059, 657, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1112791, 36083, 159, 4, 52, "Input",
  CellTags->"FRH"],
Cell[1112953, 36089, 674, 20, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1113688, 36116, 52, 1, 39, "ObjectName",
  CellTags->"FromTFi"],

Cell[CellGroupData[{
Cell[1113765, 36121, 59, 1, 49, "Subsubsection",
  CellTags->"FromTFi"],
Cell[1113827, 36124, 201, 5, 56, "Text",
  CellTags->"FromTFi"],
Cell[1114031, 36131, 280, 13, 39, "Text",
  CellTags->"FromTFi"]
}, Closed]],

Cell[CellGroupData[{
Cell[1114348, 36149, 56, 1, 49, "Subsubsection",
  CellTags->"FromTFi"],

Cell[CellGroupData[{
Cell[1114429, 36154, 453, 12, 70, "Input",
  CellTags->"FromTFi"],
Cell[1114885, 36168, 1133, 34, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1116055, 36207, 873, 22, 70, "Input",
  CellTags->"FromTFi"],
Cell[1116931, 36231, 1429, 40, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1118397, 36276, 951, 24, 70, "Input",
  CellTags->"FromTFi"],
Cell[1119351, 36302, 1816, 50, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1121228, 36359, 54, 1, 39, "ObjectName",
  CellTags->"FUNCTION"],

Cell[CellGroupData[{
Cell[1121307, 36364, 60, 1, 49, "Subsubsection",
  CellTags->"FUNCTION"],
Cell[1121370, 36367, 161, 4, 39, "Text",
  CellTags->"FUNCTION"],
Cell[1121534, 36373, 186, 9, 39, "Text",
  CellTags->"FUNCTION"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1121769, 36388, 60, 1, 39, "ObjectName",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1121854, 36393, 63, 1, 49, "Subsubsection",
  CellTags->"FunctionalD"],
Cell[1121920, 36396, 419, 8, 73, "Text",
  CellTags->"FunctionalD"],
Cell[1122342, 36406, 96, 1, 39, "Text",
  CellTags->"FunctionalD"],
Cell[1122441, 36409, 327, 14, 39, "Text",
  CellTags->"FunctionalD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1122805, 36428, 60, 1, 49, "Subsubsection",
  CellTags->"FunctionalD"],
Cell[1122868, 36431, 692, 20, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1123585, 36455, 270, 6, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1123858, 36463, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1124003, 36470, 301, 7, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1124307, 36479, 140, 3, 70, "Output"]
}, Closed]],
Cell[1124462, 36485, 1498, 44, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1125985, 36533, 359, 9, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1126347, 36544, 321, 9, 70, "Output"]
}, Closed]],
Cell[1126683, 36556, 968, 24, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1127676, 36584, 843, 22, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1128522, 36608, 938, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1129497, 36638, 259, 6, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1129759, 36646, 807, 24, 70, "Output"]
}, Closed]],
Cell[1130581, 36673, 590, 16, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1131174, 36691, 99, 2, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1131298, 36697, 371, 8, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1131672, 36707, 1791, 56, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1133500, 36768, 371, 8, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1133874, 36778, 1791, 56, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1135702, 36839, 268, 8, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1135973, 36849, 4293, 116, 70, "Output"]
}, Closed]],
Cell[1140281, 36968, 579, 13, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1140863, 36983, 1926, 50, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1142792, 37035, 534, 13, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1143351, 37052, 280, 7, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1143634, 37061, 347, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1144018, 37077, 202, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1144223, 37084, 3191, 94, 70, "Output"]
}, Closed]],
Cell[1147429, 37181, 123, 4, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1147577, 37189, 213, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1147793, 37196, 3191, 94, 70, "Output"]
}, Closed]],
Cell[1150999, 37293, 303, 8, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1151305, 37303, 652, 18, 70, "Input",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1151982, 37325, 278, 7, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1152263, 37334, 500, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1152800, 37355, 289, 7, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1153092, 37364, 550, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1153679, 37386, 722, 17, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1154404, 37405, 4133, 125, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1158574, 37535, 221, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1158798, 37542, 1987, 61, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1160822, 37608, 428, 11, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1161253, 37621, 1628, 50, 70, "Output"]
}, Closed]],
Cell[1162896, 37674, 294, 9, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1163193, 37685, 100, 2, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1163296, 37689, 168, 4, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1163489, 37697, 133, 3, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1163625, 37702, 4293, 116, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1167955, 37823, 237, 6, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1168195, 37831, 33707, 1003, 70, "Output"]
}, Closed]],
Cell[1201917, 38837, 232, 5, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1202152, 38844, 348, 7, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1202503, 38853, 164, 4, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1202670, 38859, 3947, 76, 70, "Input",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1206642, 38939, 206, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1206851, 38946, 17344, 524, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1224232, 39475, 201, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1224436, 39482, 9204, 284, 70, "Output"]
}, Closed]],
Cell[1233655, 39769, 225, 8, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1233905, 39781, 401, 11, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1234309, 39794, 3825, 118, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1238171, 39917, 828, 22, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1239002, 39941, 1830, 59, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1240869, 40005, 186, 4, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1241058, 40011, 7324, 226, 70, "Output"]
}, Closed]],
Cell[1248397, 40240, 132, 4, 70, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1248554, 40248, 104, 2, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1248661, 40252, 1628, 50, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1250326, 40307, 957, 22, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1251286, 40331, 1492, 41, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1252815, 40377, 169, 4, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1252987, 40383, 4870, 133, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1257894, 40521, 246, 6, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1258143, 40529, 8579, 265, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1266759, 40799, 170, 4, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1266932, 40805, 11309, 349, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1278278, 41159, 200, 5, 70, "Input",
  CellTags->"FunctionalD"],
Cell[1278481, 41166, 108, 2, 70, "Output"]
}, Closed]],
Cell[1278604, 41171, 67, 1, 70, "Text",
  CellTags->"FunctionalD"],
Cell[1278674, 41174, 617, 13, 70, "Input",
  CellTags->"FunctionalD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1279328, 41192, 111, 2, 49, "Subsubsection",
  CellTags->"FunctionalD"],
Cell[1279442, 41196, 80, 1, 39, "Text",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[1279547, 41201, 1308, 28, 89, "Input"],
Cell[1280858, 41231, 1854, 59, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1282749, 41295, 949, 24, 63, "Input",
  CellTags->"FunctionalD"],
Cell[1283701, 41321, 13248, 327, 81, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1296986, 41653, 177, 5, 43, "Input",
  CellTags->"FunctionalD"],
Cell[1297166, 41660, 2878, 81, 39, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1300105, 41748, 42, 1, 39, "ObjectName",
  CellTags->"FV"],

Cell[CellGroupData[{
Cell[1300172, 41753, 54, 1, 49, "Subsubsection",
  CellTags->"FV"],
Cell[1300229, 41756, 298, 13, 39, "Text",
  CellTags->"FV"],
Cell[1300530, 41771, 615, 29, 39, "Text",
  CellTags->"FV"]
}, Closed]],

Cell[CellGroupData[{
Cell[1301182, 41805, 51, 1, 49, "Subsubsection",
  CellTags->"FV"],

Cell[CellGroupData[{
Cell[1301258, 41810, 156, 4, 70, "Input",
  CellTags->"FV"],
Cell[1301417, 41816, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1301594, 41824, 185, 5, 70, "Input",
  CellTags->"FV"],
Cell[1301782, 41831, 253, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1302072, 41843, 197, 5, 70, "Input",
  CellTags->"FV"],
Cell[1302272, 41850, 153, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1302462, 41858, 242, 7, 70, "Input",
  CellTags->"FV"],
Cell[1302707, 41867, 241, 5, 70, "Output"]
}, Closed]],
Cell[1302963, 41875, 131, 4, 70, "Text",
  CellTags->"FV"],

Cell[CellGroupData[{
Cell[1303119, 41883, 239, 6, 70, "Input",
  CellTags->"FV"],
Cell[1303361, 41891, 517, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1303915, 41913, 135, 3, 70, "Input",
  CellTags->"FV"],
Cell[1304053, 41918, 432, 10, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1304546, 41935, 44, 1, 39, "ObjectName",
  CellTags->"FVD"],

Cell[CellGroupData[{
Cell[1304615, 41940, 55, 1, 49, "Subsubsection",
  CellTags->"FVD"],
Cell[1304673, 41943, 334, 15, 39, "Text",
  CellTags->"FVD"],
Cell[1305010, 41960, 613, 29, 39, "Text",
  CellTags->"FVD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1305660, 41994, 52, 1, 49, "Subsubsection",
  CellTags->"FVD"],

Cell[CellGroupData[{
Cell[1305737, 41999, 158, 4, 70, "Input",
  CellTags->"FVD"],
Cell[1305898, 42005, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1306075, 42013, 187, 5, 70, "Input",
  CellTags->"FVD"],
Cell[1306265, 42020, 253, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1306555, 42032, 199, 5, 70, "Input",
  CellTags->"FVD"],
Cell[1306757, 42039, 154, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1306948, 42047, 244, 7, 70, "Input",
  CellTags->"FVD"],
Cell[1307195, 42056, 303, 7, 70, "Output"]
}, Closed]],
Cell[1307513, 42066, 89, 1, 70, "Text",
  CellTags->"FVD"],

Cell[CellGroupData[{
Cell[1307627, 42071, 241, 6, 70, "Input",
  CellTags->"FVD"],
Cell[1307871, 42079, 517, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1308425, 42101, 136, 3, 70, "Input",
  CellTags->"FVD"],
Cell[1308564, 42106, 564, 14, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1309189, 42127, 42, 1, 39, "ObjectName",
  CellTags->"GA"],

Cell[CellGroupData[{
Cell[1309256, 42132, 54, 1, 49, "Subsubsection",
  CellTags->"GA"],
Cell[1309313, 42135, 708, 24, 57, "Text",
  CellTags->"GA"],
Cell[1310024, 42161, 402, 19, 39, "Text",
  CellTags->"GA"]
}, Closed]],

Cell[CellGroupData[{
Cell[1310463, 42185, 51, 1, 49, "Subsubsection",
  CellTags->"GA"],

Cell[CellGroupData[{
Cell[1310539, 42190, 129, 3, 70, "Input",
  CellTags->"GA"],
Cell[1310671, 42195, 147, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1310855, 42203, 259, 7, 70, "Input",
  CellTags->"GA"],
Cell[1311117, 42212, 367, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1311521, 42227, 208, 5, 70, "Input",
  CellTags->"GA"],
Cell[1311732, 42234, 177, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1311946, 42242, 199, 5, 70, "Input",
  CellTags->"GA"],
Cell[1312148, 42249, 325, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1312510, 42262, 246, 6, 70, "Input",
  CellTags->"GA"],
Cell[1312759, 42270, 288, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1313084, 42281, 301, 8, 70, "Input",
  CellTags->"GA"],
Cell[1313388, 42291, 395, 10, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1313844, 42308, 44, 1, 39, "ObjectName",
  CellTags->"GAD"],

Cell[CellGroupData[{
Cell[1313913, 42313, 55, 1, 49, "Subsubsection",
  CellTags->"GAD"],
Cell[1313971, 42316, 725, 24, 57, "Text",
  CellTags->"GAD"],
Cell[1314699, 42342, 400, 19, 39, "Text",
  CellTags->"GAD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1315136, 42366, 52, 1, 49, "Subsubsection",
  CellTags->"GAD"],

Cell[CellGroupData[{
Cell[1315213, 42371, 131, 3, 70, "Input",
  CellTags->"GAD"],
Cell[1315347, 42376, 147, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1315531, 42384, 262, 7, 70, "Input",
  CellTags->"GAD"],
Cell[1315796, 42393, 367, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1316200, 42408, 210, 5, 70, "Input",
  CellTags->"GAD"],
Cell[1316413, 42415, 237, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1316687, 42425, 201, 5, 70, "Input",
  CellTags->"GAD"],
Cell[1316891, 42432, 325, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1317253, 42445, 248, 6, 70, "Input",
  CellTags->"GAD"],
Cell[1317504, 42453, 292, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1317833, 42464, 305, 8, 70, "Input",
  CellTags->"GAD"],
Cell[1318141, 42474, 395, 10, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1318597, 42491, 62, 1, 39, "ObjectName",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{
Cell[1318684, 42496, 64, 1, 49, "Subsubsection",
  CellTags->"GammaEpsilon"],
Cell[1318751, 42499, 168, 4, 39, "Text",
  CellTags->"GammaEpsilon"],
Cell[1318922, 42505, 322, 14, 39, "Text",
  CellTags->"GammaEpsilon"]
}, Closed]],

Cell[CellGroupData[{
Cell[1319281, 42524, 61, 1, 49, "Subsubsection",
  CellTags->"GammaEpsilon"],
Cell[1319345, 42527, 142, 4, 70, "Text",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{
Cell[1319512, 42535, 207, 5, 70, "Input",
  CellTags->"GammaEpsilon"],
Cell[1319722, 42542, 2081, 57, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1321840, 42604, 207, 5, 70, "Input",
  CellTags->"GammaEpsilon"],
Cell[1322050, 42611, 1792, 50, 70, "Output"]
}, Closed]],
Cell[1323857, 42664, 92, 1, 70, "Text",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{
Cell[1323974, 42669, 151, 3, 70, "Input",
  CellTags->"GammaEpsilon"],
Cell[1324128, 42674, 1916, 54, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1326081, 42733, 145, 3, 70, "Input",
  CellTags->"GammaEpsilon"],
Cell[1326229, 42738, 156, 4, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1326446, 42749, 60, 1, 39, "ObjectName",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{
Cell[1326531, 42754, 63, 1, 49, "Subsubsection",
  CellTags->"GammaExpand"],
Cell[1326597, 42757, 126, 4, 39, "Text",
  CellTags->"GammaExpand"],
Cell[1326726, 42763, 207, 9, 39, "Text",
  CellTags->"GammaExpand"]
}, Closed]],

Cell[CellGroupData[{
Cell[1326970, 42777, 60, 1, 49, "Subsubsection",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{
Cell[1327055, 42782, 224, 5, 70, "Input",
  CellTags->"GammaExpand"],
Cell[1327282, 42789, 413, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1327732, 42806, 240, 6, 70, "Input",
  CellTags->"GammaExpand"],
Cell[1327975, 42814, 791, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1328803, 42841, 224, 5, 70, "Input",
  CellTags->"GammaExpand"],
Cell[1329030, 42848, 245, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1329336, 42861, 50, 1, 39, "ObjectName",
  CellTags->"Gamma1"],

Cell[CellGroupData[{
Cell[1329411, 42866, 58, 1, 49, "Subsubsection",
  CellTags->"Gamma1"],
Cell[1329472, 42869, 140, 4, 39, "Text",
  CellTags->"Gamma1"],
Cell[1329615, 42875, 298, 14, 39, "Text",
  CellTags->"Gamma1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1329962, 42895, 50, 1, 39, "ObjectName",
  CellTags->"Gamma2"],

Cell[CellGroupData[{
Cell[1330037, 42900, 58, 1, 49, "Subsubsection",
  CellTags->"Gamma2"],
Cell[1330098, 42903, 173, 4, 39, "Text",
  CellTags->"Gamma2"],
Cell[1330274, 42909, 298, 14, 39, "Text",
  CellTags->"Gamma2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1330621, 42929, 50, 1, 39, "ObjectName",
  CellTags->"Gamma3"],

Cell[CellGroupData[{
Cell[1330696, 42934, 58, 1, 49, "Subsubsection",
  CellTags->"Gamma3"],
Cell[1330757, 42937, 238, 5, 56, "Text",
  CellTags->"Gamma3"],
Cell[1330998, 42944, 298, 14, 39, "Text",
  CellTags->"Gamma3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1331345, 42964, 48, 1, 39, "ObjectName",
  CellTags->"Gauge"],

Cell[CellGroupData[{
Cell[1331418, 42969, 57, 1, 49, "Subsubsection",
  CellTags->"Gauge"],
Cell[1331478, 42972, 134, 4, 39, "Text",
  CellTags->"Gauge"],
Cell[1331615, 42978, 324, 14, 39, "Text",
  CellTags->"Gauge"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1331988, 42998, 58, 1, 39, "ObjectName",
  CellTags->"GaugeField"],

Cell[CellGroupData[{
Cell[1332071, 43003, 62, 1, 49, "Subsubsection",
  CellTags->"GaugeField"],
Cell[1332136, 43006, 213, 5, 56, "Text",
  CellTags->"GaugeField"],
Cell[1332352, 43013, 341, 14, 39, "Text",
  CellTags->"GaugeField"]
}, Closed]],

Cell[CellGroupData[{
Cell[1332730, 43032, 59, 1, 49, "Subsubsection",
  CellTags->"GaugeField"],

Cell[CellGroupData[{
Cell[1332814, 43037, 111, 2, 70, "Input",
  CellTags->"GaugeField"],
Cell[1332928, 43041, 134, 3, 70, "Output",
  CellTags->"GaugeField"]
}, Closed]],

Cell[CellGroupData[{
Cell[1333099, 43049, 279, 6, 70, "Input",
  CellTags->"GaugeField"],
Cell[1333381, 43057, 417, 14, 70, "Output",
  CellTags->"GaugeField"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1333859, 43078, 52, 1, 39, "ObjectName",
  CellTags->"GaugeXi"],

Cell[CellGroupData[{
Cell[1333936, 43083, 59, 1, 49, "Subsubsection",
  CellTags->"GaugeXi"],
Cell[1333998, 43086, 78, 1, 39, "Text",
  CellTags->"GaugeXi"],
Cell[1334079, 43089, 308, 14, 39, "Text",
  CellTags->"GaugeXi"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1334436, 43109, 44, 1, 39, "ObjectName",
  CellTags->"GA5"],

Cell[CellGroupData[{
Cell[1334505, 43114, 55, 1, 49, "Subsubsection",
  CellTags->"GA5"],
Cell[1334563, 43117, 89, 1, 39, "Text",
  CellTags->"GA5"],
Cell[1334655, 43120, 397, 19, 39, "Text",
  CellTags->"GA5"]
}, Closed]],

Cell[CellGroupData[{
Cell[1335089, 43144, 52, 1, 49, "Subsubsection",
  CellTags->"GA5"],

Cell[CellGroupData[{
Cell[1335166, 43149, 95, 2, 56, "Input",
  CellTags->"GA5"],
Cell[1335264, 43153, 141, 3, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1335442, 43161, 120, 3, 52, "Input",
  CellTags->"GA5"],
Cell[1335565, 43166, 106, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1335732, 43175, 44, 1, 39, "ObjectName",
  CellTags->"GGV"],

Cell[CellGroupData[{
Cell[1335801, 43180, 55, 1, 49, "Subsubsection",
  CellTags->"GGV"],
Cell[1335859, 43183, 73, 1, 39, "Text",
  CellTags->"GGV"],
Cell[1335935, 43186, 211, 9, 39, "Text",
  CellTags->"GGV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1336195, 43201, 68, 1, 39, "ObjectName",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{
Cell[1336288, 43206, 67, 1, 49, "Subsubsection",
  CellTags->"GhostPropagator"],
Cell[1336358, 43209, 349, 9, 39, "Text",
  CellTags->"GhostPropagator"],
Cell[1336710, 43220, 469, 19, 39, "Text",
  CellTags->"GhostPropagator"]
}, Closed]],

Cell[CellGroupData[{
Cell[1337216, 43244, 64, 1, 49, "Subsubsection",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{
Cell[1337305, 43249, 188, 4, 70, "Input",
  CellTags->"GhostPropagator"],
Cell[1337496, 43255, 527, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1338060, 43275, 151, 3, 70, "Input",
  CellTags->"GhostPropagator"],
Cell[1338214, 43280, 175, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1338426, 43289, 188, 4, 70, "Input",
  CellTags->"GhostPropagator"],
Cell[1338617, 43295, 531, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1339185, 43315, 305, 8, 70, "Input",
  CellTags->"GhostPropagator"],
Cell[1339493, 43325, 262, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1339816, 43338, 44, 1, 39, "ObjectName",
  CellTags->"GHP"],

Cell[CellGroupData[{
Cell[1339885, 43343, 55, 1, 49, "Subsubsection",
  CellTags->"GHP"],
Cell[1339943, 43346, 313, 9, 39, "Text",
  CellTags->"GHP"],
Cell[1340259, 43357, 493, 19, 39, "Text",
  CellTags->"GHP"]
}, Closed]],

Cell[CellGroupData[{
Cell[1340789, 43381, 52, 1, 49, "Subsubsection",
  CellTags->"GHP"],

Cell[CellGroupData[{
Cell[1340866, 43386, 164, 4, 70, "Input",
  CellTags->"GHP"],
Cell[1341033, 43392, 527, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1341597, 43412, 127, 3, 70, "Input",
  CellTags->"GHP"],
Cell[1341727, 43417, 175, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1341939, 43426, 164, 4, 70, "Input",
  CellTags->"GHP"],
Cell[1342106, 43432, 531, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1342674, 43452, 282, 8, 70, "Input",
  CellTags->"GHP"],
Cell[1342959, 43462, 262, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1343282, 43475, 58, 1, 39, "ObjectName",
  CellTags->"GluonField"],

Cell[CellGroupData[{
Cell[1343365, 43480, 62, 1, 49, "Subsubsection",
  CellTags->"GluonField"],
Cell[1343430, 43483, 80, 1, 39, "Text",
  CellTags->"GluonField"],
Cell[1343513, 43486, 200, 9, 39, "Text",
  CellTags->"GluonField"]
}, Closed]],

Cell[CellGroupData[{
Cell[1343750, 43500, 59, 1, 49, "Subsubsection",
  CellTags->"GluonField"],

Cell[CellGroupData[{
Cell[1343834, 43505, 109, 2, 56, "Input",
  CellTags->"GluonField"],
Cell[1343946, 43509, 106, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1344089, 43516, 277, 6, 52, "Input",
  CellTags->"GluonField"],
Cell[1344369, 43524, 389, 13, 40, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1344819, 43544, 70, 1, 39, "ObjectName",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{
Cell[1344914, 43549, 68, 1, 49, "Subsubsection",
  CellTags->"GluonGhostVertex"],
Cell[1344985, 43552, 425, 8, 73, "Text",
  CellTags->"GluonGhostVertex"],
Cell[1345413, 43562, 625, 23, 39, "Text",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{
Cell[1346063, 43589, 159, 3, 56, "Input",
  CellTags->"GluonGhostVertex"],
Cell[1346225, 43594, 293, 7, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1346567, 43607, 65, 1, 49, "Subsubsection",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{
Cell[1346657, 43612, 413, 10, 70, "Input",
  CellTags->"GluonGhostVertex"],
Cell[1347073, 43624, 1002, 31, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1348112, 43660, 145, 3, 70, "Input",
  CellTags->"GluonGhostVertex"],
Cell[1348260, 43665, 559, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1348856, 43683, 227, 5, 70, "Input",
  CellTags->"GluonGhostVertex"],
Cell[1349086, 43690, 1006, 31, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1350153, 43728, 68, 1, 39, "ObjectName",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{
Cell[1350246, 43733, 67, 1, 49, "Subsubsection",
  CellTags->"GluonPropagator"],
Cell[1350316, 43736, 543, 10, 73, "Text",
  CellTags->"GluonPropagator"],
Cell[1350862, 43748, 100, 1, 39, "Text",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{
Cell[1350987, 43753, 108, 2, 56, "Input",
  CellTags->"GluonPropagator"],
Cell[1351098, 43757, 122, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1351257, 43764, 157, 3, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1351417, 43769, 477, 11, 70, "Output"]
}, Closed]],
Cell[1351909, 43783, 634, 24, 35, "Text",
  CellTags->"GluonPropagator"]
}, Closed]],

Cell[CellGroupData[{
Cell[1352580, 43812, 73, 1, 50, "Subsubsection",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{
Cell[1352678, 43817, 223, 5, 56, "Input",
  CellTags->"GluonPropagator"],
Cell[1352904, 43824, 931, 26, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1353872, 43855, 272, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1354147, 43862, 1694, 46, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1355878, 43913, 383, 9, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1356264, 43924, 4284, 113, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1360585, 44042, 196, 4, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1360784, 44048, 578, 16, 70, "Output"]
}, Closed]],
Cell[1361377, 44067, 150, 4, 35, "Text",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{
Cell[1361552, 44075, 175, 4, 56, "Input",
  CellTags->"GluonPropagator"],
Cell[1361730, 44081, 931, 26, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1362698, 44112, 285, 8, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1362986, 44122, 361, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1363384, 44136, 276, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1363663, 44143, 2056, 53, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1365756, 44201, 276, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1366035, 44208, 2062, 54, 58, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1368134, 44267, 276, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1368413, 44274, 2064, 53, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1370514, 44332, 276, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1370793, 44339, 2056, 53, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1372886, 44397, 276, 5, 52, "Input",
  CellTags->"GluonPropagator"],
Cell[1373165, 44404, 3985, 103, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1377211, 44514, 68, 1, 39, "ObjectName",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{
Cell[1377304, 44519, 67, 1, 49, "Subsubsection",
  CellTags->"GluonSelfEnergy"],
Cell[1377374, 44522, 125, 4, 39, "Text",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{
Cell[1377524, 44530, 155, 3, 56, "Input",
  CellTags->"GluonSelfEnergy"],
Cell[1377682, 44535, 956, 23, 58, "Output"]
}, Closed]],
Cell[1378653, 44561, 634, 24, 35, "Text",
  CellTags->"GluonSelfEnergy"]
}, Closed]],

Cell[CellGroupData[{
Cell[1379324, 44590, 73, 1, 50, "Subsubsection",
  CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{
Cell[1379422, 44595, 304, 8, 56, "Input",
  CellTags->"GluonSelfEnergy"],
Cell[1379729, 44605, 3267, 90, 53, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1383057, 44702, 60, 1, 39, "ObjectName",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{
Cell[1383142, 44707, 63, 1, 49, "Subsubsection",
  CellTags->"GluonVertex"],
Cell[1383208, 44710, 89, 1, 39, "Text",
  CellTags->"GluonVertex"],
Cell[1383300, 44713, 622, 10, 90, "Text",
  CellTags->"GluonVertex"],
Cell[1383925, 44725, 92, 1, 39, "Text",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{
Cell[1384042, 44730, 104, 2, 56, "Input",
  CellTags->"GluonVertex"],
Cell[1384149, 44734, 118, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1384304, 44741, 149, 3, 52, "Input",
  CellTags->"GluonVertex"],
Cell[1384456, 44746, 423, 10, 70, "Output"]
}, Closed]],
Cell[1384894, 44759, 360, 14, 35, "Text",
  CellTags->"GluonVertex"]
}, Closed]],

Cell[CellGroupData[{
Cell[1385291, 44778, 60, 1, 49, "Subsubsection",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{
Cell[1385376, 44783, 403, 10, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1385782, 44795, 1426, 42, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1387245, 44842, 459, 12, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1387707, 44856, 4500, 128, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1392244, 44989, 373, 10, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1392620, 45001, 405, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1393062, 45016, 429, 12, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1393494, 45030, 3541, 111, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1397072, 45146, 490, 12, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1397565, 45160, 9095, 227, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1406697, 45392, 450, 12, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1407150, 45406, 9101, 227, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1416288, 45638, 266, 6, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1416557, 45646, 9101, 227, 70, "Output"]
}, Closed]],
Cell[1425673, 45876, 203, 5, 70, "Text",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{
Cell[1425901, 45885, 208, 5, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1426112, 45892, 1425, 42, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1427574, 45939, 237, 6, 70, "Input",
  CellTags->"GluonVertex"],
Cell[1427814, 45947, 9053, 227, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1436928, 46181, 42, 1, 39, "ObjectName",
  CellTags->"GO"],

Cell[CellGroupData[{
Cell[1436995, 46186, 54, 1, 49, "Subsubsection",
  CellTags->"GO"],
Cell[1437052, 46189, 74, 1, 39, "Text",
  CellTags->"GO"],
Cell[1437129, 46192, 219, 9, 39, "Text",
  CellTags->"GO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1437397, 46207, 42, 1, 39, "ObjectName",
  CellTags->"GP"],

Cell[CellGroupData[{
Cell[1437464, 46212, 54, 1, 49, "Subsubsection",
  CellTags->"GP"],
Cell[1437521, 46215, 70, 1, 39, "Text",
  CellTags->"GP"],
Cell[1437594, 46218, 207, 9, 39, "Text",
  CellTags->"GP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1437850, 46233, 68, 1, 39, "ObjectName",
  CellTags->"GrassmannParity"],

Cell[CellGroupData[{
Cell[1437943, 46238, 67, 1, 49, "Subsubsection",
  CellTags->"GrassmannParity"],
Cell[1438013, 46241, 217, 5, 56, "Text",
  CellTags->"GrassmannParity"],
Cell[1438233, 46248, 199, 9, 39, "Text",
  CellTags->"GrassmannParity"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1438481, 46263, 42, 1, 39, "ObjectName",
  CellTags->"GS"],

Cell[CellGroupData[{
Cell[1438548, 46268, 54, 1, 49, "Subsubsection",
  CellTags->"GS"],
Cell[1438605, 46271, 727, 18, 57, "Text",
  CellTags->"GS"],
Cell[1439335, 46291, 525, 24, 39, "Text",
  CellTags->"GS"]
}, Closed]],

Cell[CellGroupData[{
Cell[1439897, 46320, 51, 1, 49, "Subsubsection",
  CellTags->"GS"],

Cell[CellGroupData[{
Cell[1439973, 46325, 125, 3, 70, "Input",
  CellTags->"GS"],
Cell[1440101, 46330, 188, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1440326, 46339, 203, 6, 70, "Input",
  CellTags->"GS"],
Cell[1440532, 46347, 169, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1440738, 46355, 172, 4, 70, "Input",
  CellTags->"GS"],
Cell[1440913, 46361, 667, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1441617, 46385, 220, 6, 70, "Input",
  CellTags->"GS"],
Cell[1441840, 46393, 268, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1442145, 46404, 288, 8, 70, "Input",
  CellTags->"GS"],
Cell[1442436, 46414, 564, 16, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1443061, 46437, 44, 1, 39, "ObjectName",
  CellTags->"GSD"],

Cell[CellGroupData[{
Cell[1443130, 46442, 55, 1, 49, "Subsubsection",
  CellTags->"GSD"],
Cell[1443188, 46445, 736, 18, 57, "Text",
  CellTags->"GSD"],
Cell[1443927, 46465, 526, 24, 39, "Text",
  CellTags->"GSD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1444490, 46494, 52, 1, 49, "Subsubsection",
  CellTags->"GSD"],

Cell[CellGroupData[{
Cell[1444567, 46499, 127, 3, 70, "Input",
  CellTags->"GSD"],
Cell[1444697, 46504, 188, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1444922, 46513, 205, 6, 70, "Input",
  CellTags->"GSD"],
Cell[1445130, 46521, 229, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1445396, 46531, 174, 4, 70, "Input",
  CellTags->"GSD"],
Cell[1445573, 46537, 667, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1446277, 46561, 222, 6, 70, "Input",
  CellTags->"GSD"],
Cell[1446502, 46569, 272, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1446811, 46580, 292, 8, 70, "Input",
  CellTags->"GSD"],
Cell[1447106, 46590, 564, 16, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1447731, 46613, 52, 1, 39, "ObjectName",
  CellTags->"Gstrong"],

Cell[CellGroupData[{
Cell[1447808, 46618, 59, 1, 49, "Subsubsection",
  CellTags->"Gstrong"],
Cell[1447870, 46621, 84, 1, 39, "Text",
  CellTags->"Gstrong"],
Cell[1447957, 46624, 360, 15, 39, "Text",
  CellTags->"Gstrong"]
}, Closed]],

Cell[CellGroupData[{
Cell[1448354, 46644, 56, 1, 49, "Subsubsection",
  CellTags->"Gstrong"],
Cell[1448413, 46647, 122, 4, 70, "Text",
  CellTags->"Gstrong"],

Cell[CellGroupData[{
Cell[1448560, 46655, 105, 2, 70, "Input",
  CellTags->"Gstrong"],
Cell[1448668, 46659, 134, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1448863, 46669, 44, 1, 39, "ObjectName",
  CellTags->"GTI"],

Cell[CellGroupData[{
Cell[1448932, 46674, 55, 1, 49, "Subsubsection",
  CellTags->"GTI"],
Cell[1448990, 46677, 86, 1, 39, "Text",
  CellTags->"GTI"],
Cell[1449079, 46680, 172, 9, 39, "Text",
  CellTags->"GTI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1449300, 46695, 42, 1, 39, "ObjectName",
  CellTags->"GV"],

Cell[CellGroupData[{
Cell[1449367, 46700, 54, 1, 49, "Subsubsection",
  CellTags->"GV"],
Cell[1449424, 46703, 66, 1, 39, "Text",
  CellTags->"GV"],
Cell[1449493, 46706, 195, 9, 39, "Text",
  CellTags->"GV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1449737, 46721, 46, 1, 39, "ObjectName",
  CellTags->"Hill"],

Cell[CellGroupData[{
Cell[1449808, 46726, 56, 1, 49, "Subsubsection",
  CellTags->"Hill"],
Cell[1449867, 46729, 130, 4, 39, "Text",
  CellTags->"Hill"],
Cell[1450000, 46735, 209, 9, 39, "Text",
  CellTags->"Hill"]
}, Closed]],

Cell[CellGroupData[{
Cell[1450246, 46749, 53, 1, 49, "Subsubsection",
  CellTags->"Hill"],

Cell[CellGroupData[{
Cell[1450324, 46754, 156, 4, 56, "Input",
  CellTags->"Hill"],
Cell[1450483, 46760, 2760, 75, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1453280, 46840, 322, 8, 52, "Input",
  CellTags->"Hill"],
Cell[1453605, 46850, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1453750, 46857, 325, 9, 52, "Input",
  CellTags->"Hill"],
Cell[1454078, 46868, 1298, 37, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1455413, 46910, 333, 9, 52, "Input",
  CellTags->"Hill"],
Cell[1455749, 46921, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1455918, 46930, 70, 1, 39, "ObjectName",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{
Cell[1456013, 46935, 68, 1, 49, "Subsubsection",
  CellTags->"HypergeometricAC"],
Cell[1456084, 46938, 554, 9, 90, "Text",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{
Cell[1456663, 46951, 159, 3, 56, "Input",
  CellTags->"HypergeometricAC"],
Cell[1456825, 46956, 187, 5, 37, "Output"]
}, Closed]],
Cell[1457027, 46964, 641, 24, 39, "Text",
  CellTags->"HypergeometricAC"]
}, Closed]],

Cell[CellGroupData[{
Cell[1457705, 46993, 65, 1, 49, "Subsubsection",
  CellTags->"HypergeometricAC"],
Cell[1457773, 46996, 102, 1, 70, "Text",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{
Cell[1457900, 47001, 311, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1458214, 47010, 5161, 121, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1463412, 47136, 311, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1463726, 47145, 5085, 119, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1468848, 47269, 311, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1469162, 47278, 5138, 120, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1474337, 47403, 311, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1474651, 47412, 5792, 133, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1480480, 47550, 311, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1480794, 47559, 1954, 51, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1482785, 47615, 312, 7, 70, "Input",
  CellTags->"HypergeometricAC"],
Cell[1483100, 47624, 1955, 51, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1485116, 47682, 70, 1, 39, "ObjectName",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{
Cell[1485211, 47687, 68, 1, 49, "Subsubsection",
  CellTags->"HypergeometricIR"],
Cell[1485282, 47690, 272, 6, 56, "Text",
  CellTags->"HypergeometricIR"],
Cell[1485557, 47698, 512, 19, 39, "Text",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{
Cell[1486094, 47721, 159, 3, 56, "Input",
  CellTags->"HypergeometricIR"],
Cell[1486256, 47726, 191, 5, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1486496, 47737, 65, 1, 49, "Subsubsection",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{
Cell[1486586, 47742, 288, 7, 70, "Input",
  CellTags->"HypergeometricIR"],
Cell[1486877, 47751, 797, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1487711, 47777, 492, 14, 70, "Input",
  CellTags->"HypergeometricIR"],
Cell[1488206, 47793, 2199, 55, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1490442, 47853, 180, 4, 70, "Input",
  CellTags->"HypergeometricIR"],
Cell[1490625, 47859, 405, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1491091, 47878, 70, 1, 39, "ObjectName",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{
Cell[1491186, 47883, 68, 1, 49, "Subsubsection",
  CellTags->"HypergeometricSE"],
Cell[1491257, 47886, 440, 13, 56, "Text",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{
Cell[1491722, 47903, 159, 3, 56, "Input",
  CellTags->"HypergeometricSE"],
Cell[1491884, 47908, 195, 5, 70, "Output"]
}, Closed]],
Cell[1492094, 47916, 224, 9, 35, "Text",
  CellTags->"HypergeometricSE"]
}, Closed]],

Cell[CellGroupData[{
Cell[1492355, 47930, 65, 1, 49, "Subsubsection",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{
Cell[1492445, 47935, 292, 7, 70, "Input",
  CellTags->"HypergeometricSE"],
Cell[1492740, 47944, 767, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1493544, 47967, 430, 11, 70, "Input",
  CellTags->"HypergeometricSE"],
Cell[1493977, 47980, 1081, 24, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1495119, 48011, 60, 1, 39, "ObjectName",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{
Cell[1495204, 48016, 63, 1, 49, "Subsubsection",
  CellTags->"HypExplicit"],
Cell[1495270, 48019, 339, 10, 39, "Text",
  CellTags->"HypExplicit"],
Cell[1495612, 48031, 219, 9, 39, "Text",
  CellTags->"HypExplicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[1495868, 48045, 60, 1, 49, "Subsubsection",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{
Cell[1495953, 48050, 196, 4, 70, "Input",
  CellTags->"HypExplicit"],
Cell[1496152, 48056, 1525, 42, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1497714, 48103, 174, 4, 70, "Input",
  CellTags->"HypExplicit"],
Cell[1497891, 48109, 767, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1498695, 48132, 316, 8, 70, "Input",
  CellTags->"HypExplicit"],
Cell[1499014, 48142, 1810, 49, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1500861, 48196, 174, 4, 70, "Input",
  CellTags->"HypExplicit"],
Cell[1501038, 48202, 1081, 24, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1502180, 48233, 50, 1, 39, "ObjectName",
  CellTags->"HypInt"],

Cell[CellGroupData[{
Cell[1502255, 48238, 58, 1, 49, "Subsubsection",
  CellTags->"HypInt"],
Cell[1502316, 48241, 207, 5, 56, "Text",
  CellTags->"HypInt"]
}, Closed]],

Cell[CellGroupData[{
Cell[1502560, 48251, 55, 1, 49, "Subsubsection",
  CellTags->"HypInt"],

Cell[CellGroupData[{
Cell[1502640, 48256, 189, 4, 56, "Input",
  CellTags->"HypInt"],
Cell[1502832, 48262, 1523, 42, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1504392, 48309, 158, 4, 52, "Input",
  CellTags->"HypInt"],
Cell[1504553, 48315, 1182, 32, 61, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1505796, 48354, 46, 1, 39, "ObjectName",
  CellTags->"IFPD"],

Cell[CellGroupData[{
Cell[1505867, 48359, 56, 1, 49, "Subsubsection",
  CellTags->"IFPD"],
Cell[1505926, 48362, 67, 1, 39, "Text",
  CellTags->"IFPD"],
Cell[1505996, 48365, 227, 9, 39, "Text",
  CellTags->"IFPD"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1506272, 48380, 52, 1, 39, "ObjectName",
  CellTags->"IFPDOff"],

Cell[CellGroupData[{
Cell[1506349, 48385, 59, 1, 49, "Subsubsection",
  CellTags->"IFPDOff"],
Cell[1506411, 48388, 185, 5, 39, "Text",
  CellTags->"IFPDOff"],
Cell[1506599, 48395, 293, 14, 39, "Text",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{
Cell[1506929, 48414, 56, 1, 49, "Subsubsection",
  CellTags->"IFPDOff"],

Cell[CellGroupData[{
Cell[1507010, 48419, 198, 5, 56, "Input",
  CellTags->"IFPDOff"],
Cell[1507211, 48426, 491, 16, 51, "Output",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{
Cell[1507739, 48447, 134, 3, 56, "Input",
  CellTags->"IFPDOff"],
Cell[1507876, 48452, 213, 5, 37, "Output",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{
Cell[1508126, 48462, 160, 4, 52, "Input",
  CellTags->"IFPDOff"],
Cell[1508289, 48468, 267, 8, 38, "Output",
  CellTags->"IFPDOff"]
}, Closed]],

Cell[CellGroupData[{
Cell[1508593, 48481, 134, 3, 52, "Input",
  CellTags->"IFPDOff"],
Cell[1508730, 48486, 339, 9, 37, "Output",
  CellTags->"IFPDOff"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1509130, 48502, 50, 1, 39, "ObjectName",
  CellTags->"IFPDOn"],

Cell[CellGroupData[{
Cell[1509205, 48507, 58, 1, 49, "Subsubsection",
  CellTags->"IFPDOn"],
Cell[1509266, 48510, 318, 6, 56, "Text",
  CellTags->"IFPDOn"],
Cell[1509587, 48518, 295, 14, 39, "Text",
  CellTags->"IFPDOn"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1509931, 48538, 60, 1, 39, "ObjectName",
  CellTags->"IncludePair"],

Cell[CellGroupData[{
Cell[1510016, 48543, 63, 1, 49, "Subsubsection",
  CellTags->"IncludePair"],
Cell[1510082, 48546, 127, 4, 39, "Text",
  CellTags->"IncludePair"],
Cell[1510212, 48552, 189, 9, 39, "Text",
  CellTags->"IncludePair"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1510450, 48567, 64, 1, 39, "ObjectName",
  CellTags->"IndexPosition"],

Cell[CellGroupData[{
Cell[1510539, 48572, 65, 1, 49, "Subsubsection",
  CellTags->"IndexPosition"],
Cell[1510607, 48575, 90, 1, 39, "Text",
  CellTags->"IndexPosition"],
Cell[1510700, 48578, 212, 9, 39, "Text",
  CellTags->"IndexPosition"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1510961, 48593, 52, 1, 39, "ObjectName",
  CellTags->"Indices"],

Cell[CellGroupData[{
Cell[1511038, 48598, 59, 1, 49, "Subsubsection",
  CellTags->"Indices"],
Cell[1511100, 48601, 206, 5, 56, "Text",
  CellTags->"Indices"],
Cell[1511309, 48608, 206, 9, 39, "Text",
  CellTags->"Indices"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1511564, 48623, 68, 1, 39, "ObjectName",
  CellTags->"InitialFunction"],

Cell[CellGroupData[{
Cell[1511657, 48628, 67, 1, 49, "Subsubsection",
  CellTags->"InitialFunction"],
Cell[1511727, 48631, 184, 4, 39, "Text",
  CellTags->"InitialFunction"],
Cell[1511914, 48637, 199, 9, 39, "Text",
  CellTags->"InitialFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1512162, 48652, 78, 1, 39, "ObjectName",
  CellTags->"InitialSubstitutions"],

Cell[CellGroupData[{
Cell[1512265, 48657, 72, 1, 49, "Subsubsection",
  CellTags->"InitialSubstitutions"],
Cell[1512340, 48660, 211, 5, 39, "Text",
  CellTags->"InitialSubstitutions"],
Cell[1512554, 48667, 441, 19, 39, "Text",
  CellTags->"InitialSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1513044, 48692, 70, 1, 39, "ObjectName",
  CellTags->"InsideDiracTrace"],

Cell[CellGroupData[{
Cell[1513139, 48697, 68, 1, 49, "Subsubsection",
  CellTags->"InsideDiracTrace"],
Cell[1513210, 48700, 499, 9, 73, "Text",
  CellTags->"InsideDiracTrace"],
Cell[1513712, 48711, 341, 14, 39, "Text",
  CellTags->"InsideDiracTrace"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1514102, 48731, 64, 1, 39, "ObjectName",
  CellTags->"IntegralTable"],

Cell[CellGroupData[{
Cell[1514191, 48736, 65, 1, 49, "Subsubsection",
  CellTags->"IntegralTable"],
Cell[1514259, 48739, 234, 5, 56, "Text",
  CellTags->"IntegralTable"],
Cell[1514496, 48746, 533, 19, 39, "Text",
  CellTags->"IntegralTable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1515078, 48771, 70, 1, 39, "ObjectName",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{
Cell[1515173, 48776, 68, 1, 49, "Subsubsection",
  CellTags->"IntegrateByParts"],
Cell[1515244, 48779, 187, 5, 39, "Text",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{
Cell[1515456, 48788, 159, 3, 56, "Input",
  CellTags->"IntegrateByParts"],
Cell[1515618, 48793, 184, 5, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1515851, 48804, 65, 1, 49, "Subsubsection",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{
Cell[1515941, 48809, 670, 19, 70, "Input",
  CellTags->"IntegrateByParts"],
Cell[1516614, 48830, 787, 22, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1517462, 48859, 60, 1, 39, "ObjectName",
  CellTags->"Integratedx"],

Cell[CellGroupData[{
Cell[1517547, 48864, 63, 1, 49, "Subsubsection",
  CellTags->"Integratedx"],
Cell[1517613, 48867, 155, 4, 39, "Text",
  CellTags->"Integratedx"],
Cell[1517771, 48873, 915, 34, 39, "Text",
  CellTags->"Integratedx"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1518735, 48913, 58, 1, 39, "ObjectName",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1518818, 48918, 62, 1, 49, "Subsubsection",
  CellTags->"Integrate2"],
Cell[1518883, 48921, 885, 17, 107, "Text",
  CellTags->"Integrate2"],
Cell[1519771, 48940, 551, 24, 39, "Text",
  CellTags->"Integrate2"],
Cell[1520325, 48966, 406, 7, 73, "Text",
  CellTags->"Integrate2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1520768, 48978, 59, 1, 49, "Subsubsection",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1520852, 48983, 532, 15, 70, "Input",
  CellTags->"Integrate2"],
Cell[1521387, 49000, 596, 16, 70, "Output"]
}, Closed]],
Cell[1521998, 49019, 196, 6, 70, "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1522219, 49029, 342, 9, 70, "Input",
  CellTags->"Integrate2"],
Cell[1522564, 49040, 259, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1522860, 49052, 337, 9, 70, "Input",
  CellTags->"Integrate2"],
Cell[1523200, 49063, 531, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1523768, 49083, 414, 11, 70, "Input",
  CellTags->"Integrate2"],
Cell[1524185, 49096, 769, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1524991, 49122, 384, 10, 70, "Input",
  CellTags->"Integrate2"],
Cell[1525378, 49134, 140, 3, 70, "Output"]
}, Closed]],
Cell[1525533, 49140, 1942, 54, 70, "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1527500, 49198, 331, 9, 70, "Input",
  CellTags->"Integrate2"],
Cell[1527834, 49209, 108, 2, 70, "Output"]
}, Closed]],
Cell[1527957, 49214, 755, 25, 70, "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1528737, 49243, 386, 10, 70, "Input",
  CellTags->"Integrate2"],
Cell[1529126, 49255, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1529271, 49262, 476, 13, 70, "Input",
  CellTags->"Integrate2"],
Cell[1529750, 49277, 175, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1529962, 49286, 516, 14, 70, "Input",
  CellTags->"Integrate2"],
Cell[1530481, 49302, 329, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1530847, 49317, 402, 11, 70, "Input",
  CellTags->"Integrate2"],
Cell[1531252, 49330, 290, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1531579, 49343, 436, 12, 70, "Input",
  CellTags->"Integrate2"],
Cell[1532018, 49357, 475, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1532530, 49375, 380, 10, 70, "Input",
  CellTags->"Integrate2"],
Cell[1532913, 49387, 208, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1533158, 49398, 342, 9, 70, "Input",
  CellTags->"Integrate2"],
Cell[1533503, 49409, 462, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1534002, 49427, 553, 15, 70, "Input",
  CellTags->"Integrate2"],
Cell[1534558, 49444, 590, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1535185, 49465, 966, 25, 70, "Input",
  CellTags->"Integrate2"],
Cell[1536154, 49492, 5747, 158, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1541938, 49655, 596, 16, 70, "Input",
  CellTags->"Integrate2"],
Cell[1542537, 49673, 1109, 31, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1543683, 49709, 635, 17, 70, "Input",
  CellTags->"Integrate2"],
Cell[1544321, 49728, 9590, 280, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1553948, 50013, 174, 4, 70, "Input",
  CellTags->"Integrate2"],
Cell[1554125, 50019, 1569, 43, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1555731, 50067, 132, 3, 70, "Input",
  CellTags->"Integrate2"],
Cell[1555866, 50072, 128, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1556031, 50079, 533, 14, 70, "Input",
  CellTags->"Integrate2"],
Cell[1556567, 50095, 128, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1556732, 50102, 877, 23, 70, "Input",
  CellTags->"Integrate2"],
Cell[1557612, 50127, 4941, 144, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1562590, 50276, 183, 4, 70, "Input",
  CellTags->"Integrate2"],
Cell[1562776, 50282, 111, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1562924, 50289, 450, 12, 70, "Input",
  CellTags->"Integrate2"],
Cell[1563377, 50303, 108, 2, 70, "Output"]
}, Closed]],
Cell[1563500, 50308, 138, 4, 70, "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1563663, 50316, 179, 4, 70, "Input",
  CellTags->"Integrate2"],
Cell[1563845, 50322, 7280, 177, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1571162, 50504, 284, 8, 70, "Input",
  CellTags->"Integrate2"],
Cell[1571449, 50514, 225, 6, 70, "Output"]
}, Closed]],
Cell[1571689, 50523, 166, 4, 70, "Text",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1571880, 50531, 405, 11, 70, "Input",
  CellTags->"Integrate2"],
Cell[1572288, 50544, 226, 6, 70, "Output"]
}, Closed]],
Cell[1572529, 50553, 158, 4, 70, "Input",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1572712, 50561, 384, 10, 70, "Input",
  CellTags->"Integrate2"],
Cell[1573099, 50573, 140, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1573276, 50581, 400, 11, 70, "Input",
  CellTags->"Integrate2"],
Cell[1573679, 50594, 302, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1574018, 50608, 127, 3, 70, "Input",
  CellTags->"Integrate2"],
Cell[1574148, 50613, 127, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1574312, 50620, 400, 11, 70, "Input",
  CellTags->"Integrate2"],
Cell[1574715, 50633, 127, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1574879, 50640, 471, 13, 70, "Input",
  CellTags->"Integrate2"],
Cell[1575353, 50655, 6263, 183, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1581677, 50845, 58, 1, 39, "ObjectName",
  CellTags->"Integrate3"],

Cell[CellGroupData[{
Cell[1581760, 50850, 62, 1, 49, "Subsubsection",
  CellTags->"Integrate3"],
Cell[1581825, 50853, 334, 7, 56, "Text",
  CellTags->"Integrate3"],
Cell[1582162, 50862, 200, 9, 39, "Text",
  CellTags->"Integrate3"]
}, Closed]],

Cell[CellGroupData[{
Cell[1582399, 50876, 59, 1, 49, "Subsubsection",
  CellTags->"Integrate3"],

Cell[CellGroupData[{
Cell[1582483, 50881, 340, 9, 70, "Input",
  CellTags->"Integrate3"],
Cell[1582826, 50892, 264, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1583127, 50905, 489, 13, 70, "Input",
  CellTags->"Integrate3"],
Cell[1583619, 50920, 736, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1584392, 50947, 858, 23, 70, "Input",
  CellTags->"Integrate3"],
Cell[1585253, 50972, 3096, 81, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1588386, 51058, 320, 8, 70, "Input",
  CellTags->"Integrate3"],
Cell[1588709, 51068, 108, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1588854, 51075, 389, 10, 70, "Input",
  CellTags->"Integrate3"],
Cell[1589246, 51087, 219, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1589502, 51098, 331, 9, 70, "Input",
  CellTags->"Integrate3"],
Cell[1589836, 51109, 108, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1590005, 51118, 58, 1, 39, "ObjectName",
  CellTags->"Integrate4"],

Cell[CellGroupData[{
Cell[1590088, 51123, 62, 1, 49, "Subsubsection",
  CellTags->"Integrate4"],
Cell[1590153, 51126, 120, 4, 39, "Text",
  CellTags->"Integrate4"],
Cell[1590276, 51132, 326, 14, 39, "Text",
  CellTags->"Integrate4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1590651, 51152, 58, 1, 39, "ObjectName",
  CellTags->"Integrate5"],

Cell[CellGroupData[{
Cell[1590734, 51157, 62, 1, 49, "Subsubsection",
  CellTags->"Integrate5"],
Cell[1590799, 51160, 72, 1, 39, "Text",
  CellTags->"Integrate5"],
Cell[1590874, 51163, 200, 9, 39, "Text",
  CellTags->"Integrate5"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1591123, 51178, 88, 1, 39, "ObjectName",
  CellTags->"IntermediateSubstitutions"],

Cell[CellGroupData[{
Cell[1591236, 51183, 77, 1, 49, "Subsubsection",
  CellTags->"IntermediateSubstitutions"],
Cell[1591316, 51186, 209, 5, 39, "Text",
  CellTags->"IntermediateSubstitutions"],
Cell[1591528, 51193, 206, 9, 39, "Text",
  CellTags->"IntermediateSubstitutions"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1591783, 51208, 64, 1, 39, "ObjectName",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1591872, 51213, 65, 1, 49, "Subsubsection",
  CellTags->"InverseMellin"],
Cell[1591940, 51216, 578, 14, 71, "Text",
  CellTags->"InverseMellin"],
Cell[1592521, 51232, 662, 29, 39, "Text",
  CellTags->"InverseMellin"]
}, Closed]],

Cell[CellGroupData[{
Cell[1593220, 51266, 62, 1, 49, "Subsubsection",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1593307, 51271, 206, 5, 56, "Input",
  CellTags->"InverseMellin"],
Cell[1593516, 51278, 165, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1593718, 51287, 270, 7, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1593991, 51296, 165, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1594193, 51305, 174, 4, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1594370, 51311, 321, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1594728, 51325, 237, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1594968, 51333, 334, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1595339, 51348, 239, 7, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1595581, 51357, 423, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1596041, 51374, 251, 7, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1596295, 51383, 423, 12, 70, "Output"]
}, Closed]],
Cell[1596733, 51398, 116, 4, 35, "Text",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1596874, 51406, 354, 9, 56, "Input",
  CellTags->"InverseMellin"],
Cell[1597231, 51417, 133, 3, 70, "Output"]
}, Closed]],
Cell[1597379, 51423, 120, 4, 35, "Text",
  CellTags->"InverseMellin"],
Cell[1597502, 51429, 4033, 103, 216, "Input",
  CellTags->"InverseMellin"],
Cell[1601538, 51534, 290, 7, 56, "Input",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1601853, 51545, 221, 6, 56, "Input",
  CellTags->"InverseMellin"],
Cell[1602077, 51553, 463, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1602577, 51571, 221, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1602801, 51579, 418, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1603256, 51596, 208, 5, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1603467, 51603, 990, 28, 70, "Output"]
}, Closed]],
Cell[1604472, 51634, 105, 2, 52, "Input",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1604602, 51640, 234, 6, 56, "Input",
  CellTags->"InverseMellin"],
Cell[1604839, 51648, 608, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1605484, 51671, 330, 9, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1605817, 51682, 610, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1606464, 51705, 330, 9, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1606797, 51716, 841, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1607675, 51744, 329, 9, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1608007, 51755, 913, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1608957, 51785, 307, 9, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1609267, 51796, 667, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1609971, 51820, 346, 10, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1610320, 51832, 1159, 30, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1611516, 51867, 234, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1611753, 51875, 875, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1612665, 51904, 277, 8, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1612945, 51914, 916, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1613898, 51944, 234, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1614135, 51952, 1156, 31, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1615328, 51988, 244, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1615575, 51996, 664, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1616276, 52020, 272, 7, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1616551, 52029, 1417, 37, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1618005, 52071, 272, 7, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1618280, 52080, 1342, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1619659, 52120, 254, 6, 52, "Input",
  CellTags->"InverseMellin"],
Cell[1619916, 52128, 778, 22, 70, "Output"]
}, Closed]],
Cell[1620709, 52153, 194, 5, 52, "Input",
  CellTags->"InverseMellin"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1620952, 52164, 52, 1, 39, "ObjectName",
  CellTags->"Isolate"],

Cell[CellGroupData[{
Cell[1621029, 52169, 59, 1, 49, "Subsubsection",
  CellTags->"Isolate"],
Cell[1621091, 52172, 425, 8, 73, "Text",
  CellTags->"Isolate"],

Cell[CellGroupData[{
Cell[1621541, 52184, 141, 3, 56, "Input",
  CellTags->"Isolate"],
Cell[1621685, 52189, 401, 9, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1622135, 52204, 56, 1, 49, "Subsubsection",
  CellTags->"Isolate"],

Cell[CellGroupData[{
Cell[1622216, 52209, 192, 5, 56, "Input",
  CellTags->"Isolate"],
Cell[1622411, 52216, 176, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1622624, 52226, 476, 13, 52, "Input",
  CellTags->"Isolate"],
Cell[1623103, 52241, 363, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1623503, 52257, 141, 3, 52, "Input",
  CellTags->"Isolate"],
Cell[1623647, 52262, 328, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1624012, 52277, 218, 5, 52, "Input",
  CellTags->"Isolate"],
Cell[1624233, 52284, 803, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1625073, 52313, 698, 19, 52, "Input",
  CellTags->"Isolate"],
Cell[1625774, 52334, 423, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1626234, 52352, 120, 3, 52, "Input",
  CellTags->"Isolate"],
Cell[1626357, 52357, 86, 1, 70, "Print"],
Cell[1626446, 52360, 912, 24, 70, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[1627395, 52389, 324, 7, 52, "Input",
  CellTags->"Isolate"],
Cell[1627722, 52398, 175, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1627934, 52408, 219, 6, 52, "Input",
  CellTags->"Isolate"],
Cell[1628156, 52416, 231, 6, 70, "Output"]
}, Closed]],
Cell[1628402, 52425, 172, 4, 52, "Input",
  CellTags->"Isolate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1628623, 52435, 60, 1, 39, "ObjectName",
  CellTags->"IsolateHead"],

Cell[CellGroupData[{
Cell[1628708, 52440, 63, 1, 49, "Subsubsection",
  CellTags->"IsolateHead"],
Cell[1628774, 52443, 85, 1, 39, "Text",
  CellTags->"IsolateHead"],
Cell[1628862, 52446, 207, 9, 39, "Text",
  CellTags->"IsolateHead"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1629118, 52461, 62, 1, 39, "ObjectName",
  CellTags->"IsolateNames"],

Cell[CellGroupData[{
Cell[1629205, 52466, 64, 1, 49, "Subsubsection",
  CellTags->"IsolateNames"],
Cell[1629272, 52469, 221, 5, 56, "Text",
  CellTags->"IsolateNames"],
Cell[1629496, 52476, 313, 14, 39, "Text",
  CellTags->"IsolateNames"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1629858, 52496, 62, 1, 39, "ObjectName",
  CellTags->"IsolatePrint"],

Cell[CellGroupData[{
Cell[1629945, 52501, 64, 1, 49, "Subsubsection",
  CellTags->"IsolatePrint"],
Cell[1630012, 52504, 285, 6, 56, "Text",
  CellTags->"IsolatePrint"],
Cell[1630300, 52512, 193, 9, 39, "Text",
  CellTags->"IsolatePrint"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1630542, 52527, 62, 1, 39, "ObjectName",
  CellTags->"IsolateSplit"],

Cell[CellGroupData[{
Cell[1630629, 52532, 64, 1, 49, "Subsubsection",
  CellTags->"IsolateSplit"],
Cell[1630696, 52535, 414, 7, 73, "Text",
  CellTags->"IsolateSplit"],
Cell[1631113, 52544, 193, 9, 39, "Text",
  CellTags->"IsolateSplit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1631355, 52559, 54, 1, 39, "ObjectName",
  CellTags->"KeepOnly"],

Cell[CellGroupData[{
Cell[1631434, 52564, 60, 1, 49, "Subsubsection",
  CellTags->"KeepOnly"],
Cell[1631497, 52567, 294, 6, 56, "Text",
  CellTags->"KeepOnly"],
Cell[1631794, 52575, 495, 24, 39, "Text",
  CellTags->"KeepOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1632338, 52605, 42, 1, 39, "ObjectName",
  CellTags->"KK"],

Cell[CellGroupData[{
Cell[1632405, 52610, 54, 1, 49, "Subsubsection",
  CellTags->"KK"],
Cell[1632462, 52613, 236, 5, 56, "Text",
  CellTags->"KK"],
Cell[1632701, 52620, 315, 14, 39, "Text",
  CellTags->"KK"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1633065, 52640, 50, 1, 39, "ObjectName",
  CellTags->"Kummer"],

Cell[CellGroupData[{
Cell[1633140, 52645, 58, 1, 49, "Subsubsection",
  CellTags->"Kummer"],
Cell[1633201, 52648, 263, 6, 56, "Text",
  CellTags->"Kummer"],
Cell[1633467, 52656, 214, 9, 39, "Text",
  CellTags->"Kummer"]
}, Closed]],

Cell[CellGroupData[{
Cell[1633718, 52670, 55, 1, 49, "Subsubsection",
  CellTags->"Kummer"],

Cell[CellGroupData[{
Cell[1633798, 52675, 402, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1634203, 52687, 3510, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1637750, 52783, 402, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1638155, 52795, 3544, 92, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1641736, 52892, 402, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1642141, 52904, 3544, 92, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1645722, 53001, 457, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1646182, 53014, 3526, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1649745, 53110, 561, 13, 70, "Input",
  CellTags->"Kummer"],
Cell[1650309, 53125, 3638, 92, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1653984, 53222, 457, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1654444, 53235, 3517, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1657998, 53331, 561, 13, 70, "Input",
  CellTags->"Kummer"],
Cell[1658562, 53346, 3659, 93, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1662258, 53444, 457, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1662718, 53457, 3517, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1666272, 53553, 561, 13, 70, "Input",
  CellTags->"Kummer"],
Cell[1666836, 53568, 3659, 93, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1670532, 53666, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1671123, 53683, 3738, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1674898, 53785, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1675657, 53806, 3907, 100, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1679601, 53911, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1680192, 53928, 3715, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1683944, 54030, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1684703, 54051, 3884, 100, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1688624, 54156, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1689215, 54173, 3679, 96, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1692931, 54274, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1693690, 54295, 3952, 101, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1697679, 54401, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1698270, 54418, 3738, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1702045, 54520, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1702804, 54541, 3993, 102, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1706834, 54648, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1707425, 54665, 3646, 95, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1711108, 54765, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1711867, 54786, 3901, 100, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1715805, 54891, 588, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1716396, 54908, 3646, 95, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1720079, 55008, 756, 19, 70, "Input",
  CellTags->"Kummer"],
Cell[1720838, 55029, 4048, 103, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1724923, 55137, 626, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1725552, 55154, 3716, 95, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1729305, 55254, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1729711, 55266, 3510, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1733258, 55362, 626, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1733887, 55379, 3812, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1737736, 55481, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1738142, 55493, 3502, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1741681, 55589, 626, 15, 70, "Input",
  CellTags->"Kummer"],
Cell[1742310, 55606, 3812, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1746159, 55708, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1746565, 55720, 3502, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1750104, 55816, 690, 17, 70, "Input",
  CellTags->"Kummer"],
Cell[1750797, 55835, 3827, 97, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1754661, 55937, 458, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1755122, 55950, 3526, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1758685, 56046, 690, 17, 70, "Input",
  CellTags->"Kummer"],
Cell[1759378, 56065, 3985, 101, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1763400, 56171, 458, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1763861, 56184, 3590, 93, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1767488, 56282, 690, 17, 70, "Input",
  CellTags->"Kummer"],
Cell[1768181, 56301, 3912, 99, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1772130, 56405, 458, 11, 70, "Input",
  CellTags->"Kummer"],
Cell[1772591, 56418, 3517, 91, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1776145, 56514, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1776551, 56526, 6613, 157, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1783201, 56688, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1783607, 56700, 6705, 160, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1790349, 56865, 403, 10, 70, "Input",
  CellTags->"Kummer"],
Cell[1790755, 56877, 6709, 160, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1797525, 57044, 58, 1, 39, "ObjectName",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{
Cell[1797608, 57049, 62, 1, 49, "Subsubsection",
  CellTags->"Lagrangian"],
Cell[1797673, 57052, 455, 9, 73, "Text",
  CellTags->"Lagrangian"],
Cell[1798131, 57063, 194, 9, 39, "Text",
  CellTags->"Lagrangian"]
}, Closed]],

Cell[CellGroupData[{
Cell[1798362, 57077, 59, 1, 49, "Subsubsection",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{
Cell[1798446, 57082, 151, 3, 70, "Input",
  CellTags->"Lagrangian"],
Cell[1798600, 57087, 414, 11, 70, "Output"]
}, Closed]],
Cell[1799029, 57101, 86, 1, 70, "Text",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{
Cell[1799140, 57106, 151, 3, 70, "Input",
  CellTags->"Lagrangian"],
Cell[1799294, 57111, 873, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1800204, 57138, 151, 3, 70, "Input",
  CellTags->"Lagrangian"],
Cell[1800358, 57143, 1365, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1801760, 57183, 151, 3, 70, "Input",
  CellTags->"Lagrangian"],
Cell[1801914, 57188, 806, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1802757, 57215, 151, 3, 70, "Input",
  CellTags->"Lagrangian"],
Cell[1802911, 57220, 855, 23, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1803827, 57250, 42, 1, 39, "ObjectName",
  CellTags->"LC"],

Cell[CellGroupData[{
Cell[1803894, 57255, 54, 1, 49, "Subsubsection",
  CellTags->"LC"],
Cell[1803951, 57258, 243, 6, 56, "Text",
  CellTags->"LC"],
Cell[1804197, 57266, 297, 14, 39, "Text",
  CellTags->"LC"]
}, Closed]],

Cell[CellGroupData[{
Cell[1804531, 57285, 51, 1, 49, "Subsubsection",
  CellTags->"LC"],

Cell[CellGroupData[{
Cell[1804607, 57290, 199, 5, 70, "Input",
  CellTags->"LC"],
Cell[1804809, 57297, 261, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1805107, 57308, 122, 3, 70, "Input",
  CellTags->"LC"],
Cell[1805232, 57313, 665, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1805934, 57338, 131, 3, 70, "Input",
  CellTags->"LC"],
Cell[1806068, 57343, 370, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1806475, 57355, 221, 6, 70, "Input",
  CellTags->"LC"],
Cell[1806699, 57363, 249, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1806985, 57374, 161, 4, 70, "Input",
  CellTags->"LC"],
Cell[1807149, 57380, 412, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1807598, 57394, 474, 13, 70, "Input",
  CellTags->"LC"],
Cell[1808075, 57409, 581, 17, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1808717, 57433, 44, 1, 39, "ObjectName",
  CellTags->"LCD"],

Cell[CellGroupData[{
Cell[1808786, 57438, 55, 1, 49, "Subsubsection",
  CellTags->"LCD"],
Cell[1808844, 57441, 235, 5, 56, "Text",
  CellTags->"LCD"],
Cell[1809082, 57448, 162, 4, 39, "Text",
  CellTags->"LCD"],
Cell[1809247, 57454, 295, 14, 39, "Text",
  CellTags->"LCD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1809579, 57473, 52, 1, 49, "Subsubsection",
  CellTags->"LCD"],
Cell[1809634, 57476, 232, 6, 70, "Input",
  CellTags->"LCD"],

Cell[CellGroupData[{
Cell[1809891, 57486, 202, 5, 70, "Input",
  CellTags->"LCD"],
Cell[1810096, 57493, 262, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1810395, 57504, 124, 3, 70, "Input",
  CellTags->"LCD"],
Cell[1810522, 57509, 666, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1811225, 57534, 133, 3, 70, "Input",
  CellTags->"LCD"],
Cell[1811361, 57539, 495, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1811893, 57555, 224, 6, 70, "Input",
  CellTags->"LCD"],
Cell[1812120, 57563, 250, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1812407, 57574, 163, 4, 70, "Input",
  CellTags->"LCD"],
Cell[1812573, 57580, 475, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1813085, 57596, 494, 13, 70, "Input",
  CellTags->"LCD"],
Cell[1813582, 57611, 506, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1814125, 57630, 207, 5, 70, "Input",
  CellTags->"LCD"],
Cell[1814335, 57637, 186, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1814582, 57649, 62, 1, 39, "ObjectName",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{
Cell[1814669, 57654, 64, 1, 49, "Subsubsection",
  CellTags->"LeftPartialD"],
Cell[1814736, 57657, 358, 12, 42, "Text",
  CellTags->"LeftPartialD"],
Cell[1815097, 57671, 616, 24, 39, "Text",
  CellTags->"LeftPartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1815750, 57700, 70, 1, 50, "Subsubsection",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{
Cell[1815845, 57705, 275, 7, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1816123, 57714, 333, 10, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1816493, 57729, 123, 3, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1816619, 57734, 408, 13, 26, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1817064, 57752, 121, 3, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1817188, 57757, 286, 6, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1817511, 57768, 168, 4, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1817682, 57774, 155, 3, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1817874, 57782, 409, 10, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1818286, 57794, 376, 11, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1818699, 57810, 123, 3, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1818825, 57815, 975, 29, 26, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1819837, 57849, 121, 3, 43, "Input",
  CellTags->"LeftPartialD"],
Cell[1819961, 57854, 921, 23, 56, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1820943, 57884, 72, 1, 39, "ObjectName",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{
Cell[1821040, 57889, 69, 1, 49, "Subsubsection",
  CellTags->"LeftRightPartialD"],
Cell[1821112, 57892, 661, 21, 59, "Text",
  CellTags->"LeftRightPartialD"],
Cell[1821776, 57915, 893, 33, 39, "Text",
  CellTags->"LeftRightPartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1822706, 57953, 66, 1, 49, "Subsubsection",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{
Cell[1822797, 57958, 160, 3, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1822960, 57963, 292, 8, 42, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1823289, 57976, 155, 3, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1823447, 57981, 652, 18, 53, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1824136, 58004, 310, 7, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1824449, 58013, 363, 10, 42, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1824849, 58028, 153, 3, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1825005, 58033, 835, 24, 55, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1825877, 58062, 444, 10, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1826324, 58074, 406, 11, 42, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1826767, 58090, 153, 3, 56, "Input",
  CellTags->"LeftRightPartialD"],
Cell[1826923, 58095, 1026, 29, 53, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1828010, 58131, 74, 1, 39, "ObjectName",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{
Cell[1828109, 58136, 70, 1, 49, "Subsubsection",
  CellTags->"LeftRightPartialD2"],
Cell[1828182, 58139, 660, 21, 59, "Text",
  CellTags->"LeftRightPartialD2"],
Cell[1828845, 58162, 751, 29, 39, "Text",
  CellTags->"LeftRightPartialD2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1829633, 58196, 67, 1, 49, "Subsubsection",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{
Cell[1829725, 58201, 162, 3, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1829890, 58206, 292, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1830219, 58219, 156, 3, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1830378, 58224, 517, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1830932, 58244, 312, 7, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1831247, 58253, 363, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1831647, 58268, 154, 3, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1831804, 58273, 749, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1832590, 58301, 446, 10, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1833039, 58313, 406, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1833482, 58329, 154, 3, 70, "Input",
  CellTags->"LeftRightPartialD2"],
Cell[1833639, 58334, 903, 26, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1834603, 58367, 59, 1, 39, "ObjectName",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1834687, 58372, 62, 1, 49, "Subsubsection",
  CellTags->"LeviCivita"],
Cell[1834752, 58375, 1314, 41, 73, "Text",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1836091, 58420, 148, 3, 56, "Input",
  CellTags->"LeviCivita"],
Cell[1836242, 58425, 186, 5, 70, "Output"]
}, Closed]],
Cell[1836443, 58433, 506, 24, 35, "Text",
  CellTags->"LeviCivita"]
}, Closed]],

Cell[CellGroupData[{
Cell[1836986, 58462, 59, 1, 49, "Subsubsection",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1837070, 58467, 223, 5, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1837296, 58474, 673, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1838006, 58499, 245, 6, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1838254, 58507, 558, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1838849, 58528, 216, 5, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1839068, 58535, 426, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1839531, 58552, 222, 5, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1839756, 58559, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1839902, 58566, 297, 8, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1840202, 58576, 358, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1840597, 58588, 488, 13, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1841088, 58603, 1332, 39, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1842457, 58647, 141, 3, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1842601, 58652, 132, 3, 70, "Output"]
}, Closed]],
Cell[1842748, 58658, 374, 9, 70, "Input",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1843147, 58671, 506, 13, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1843656, 58686, 343, 10, 70, "Output"]
}, Closed]],
Cell[1844014, 58699, 374, 9, 70, "Input",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1844413, 58712, 215, 5, 70, "Input",
  CellTags->"LeviCivita"],
Cell[1844631, 58719, 269, 6, 70, "Output"]
}, Closed]],
Cell[1844915, 58728, 138, 3, 70, "Input",
  CellTags->"LeviCivita"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1845102, 58737, 66, 1, 39, "ObjectName",
  CellTags->"LeviCivitaSign"],

Cell[CellGroupData[{
Cell[1845193, 58742, 66, 1, 49, "Subsubsection",
  CellTags->"LeviCivitaSign"],
Cell[1845262, 58745, 206, 5, 39, "Text",
  CellTags->"LeviCivitaSign"],
Cell[1845471, 58752, 333, 14, 39, "Text",
  CellTags->"LeviCivitaSign"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1845853, 58772, 82, 1, 39, "ObjectName",
  CellTags->"ListIntegrals"],

Cell[CellGroupData[{
Cell[1845960, 58777, 65, 1, 49, "Subsubsection",
  CellTags->"ListIntegrals"],
Cell[1846028, 58780, 473, 9, 73, "Text",
  CellTags->"ListIntegrals"]
}, Closed]],

Cell[CellGroupData[{
Cell[1846538, 58794, 62, 1, 49, "Subsubsection",
  CellTags->"ListIntegrals"],
Cell[1846603, 58797, 102, 2, 56, "Input",
  CellTags->"ListIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1846754, 58805, 45, 1, 39, "ObjectName",
  CellTags->"Li2"],

Cell[CellGroupData[{
Cell[1846824, 58810, 55, 1, 49, "Subsubsection",
  CellTags->"Li2"],
Cell[1846882, 58813, 120, 4, 39, "Text",
  CellTags->"Li2"],
Cell[1847005, 58819, 165, 8, 39, "Text",
  CellTags->"Li2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1847207, 58832, 52, 1, 49, "Subsubsection",
  CellTags->"Li2"],

Cell[CellGroupData[{
Cell[1847284, 58837, 128, 3, 70, "Input",
  CellTags->"Li2"],
Cell[1847415, 58842, 172, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1847624, 58852, 135, 3, 70, "Input",
  CellTags->"Li2"],
Cell[1847762, 58857, 180, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1847979, 58866, 317, 9, 70, "Input",
  CellTags->"Li2"],
Cell[1848299, 58877, 172, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1848532, 58889, 45, 1, 39, "ObjectName",
  CellTags->"Li3"],

Cell[CellGroupData[{
Cell[1848602, 58894, 55, 1, 49, "Subsubsection",
  CellTags->"Li3"],
Cell[1848660, 58897, 121, 4, 39, "Text",
  CellTags->"Li3"],
Cell[1848784, 58903, 165, 8, 39, "Text",
  CellTags->"Li3"]
}, Closed]],

Cell[CellGroupData[{
Cell[1848986, 58916, 52, 1, 49, "Subsubsection",
  CellTags->"Li3"],

Cell[CellGroupData[{
Cell[1849063, 58921, 128, 3, 70, "Input",
  CellTags->"Li3"],
Cell[1849194, 58926, 172, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1849403, 58936, 135, 3, 70, "Input",
  CellTags->"Li3"],
Cell[1849541, 58941, 180, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1849758, 58950, 189, 5, 70, "Input",
  CellTags->"Li3"],
Cell[1849950, 58957, 206, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1850193, 58968, 252, 7, 70, "Input",
  CellTags->"Li3"],
Cell[1850448, 58977, 172, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1850681, 58989, 46, 1, 39, "ObjectName",
  CellTags->"Loop"],

Cell[CellGroupData[{
Cell[1850752, 58994, 56, 1, 49, "Subsubsection",
  CellTags->"Loop"],
Cell[1850811, 58997, 95, 1, 39, "Text",
  CellTags->"Loop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1850955, 59004, 62, 1, 39, "ObjectName",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{
Cell[1851042, 59009, 64, 1, 49, "Subsubsection",
  CellTags->"LorentzIndex"],
Cell[1851109, 59012, 244, 5, 56, "Text",
  CellTags->"LorentzIndex"],
Cell[1851356, 59019, 337, 14, 39, "Text",
  CellTags->"LorentzIndex"]
}, Closed]],

Cell[CellGroupData[{
Cell[1851730, 59038, 61, 1, 49, "Subsubsection",
  CellTags->"LorentzIndex"],
Cell[1851794, 59041, 90, 1, 70, "Text",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{
Cell[1851909, 59046, 153, 3, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1852065, 59051, 116, 2, 70, "Output"]
}, Closed]],
Cell[1852196, 59056, 128, 4, 70, "Text",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{
Cell[1852349, 59064, 180, 4, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1852532, 59070, 116, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1852685, 59077, 131, 2, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1852819, 59081, 144, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1853000, 59089, 197, 4, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1853200, 59095, 141, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1853378, 59103, 246, 6, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1853627, 59111, 115, 2, 70, "Output"]
}, Closed]],
Cell[1853757, 59116, 142, 4, 70, "Text",
  CellTags->"LorentzIndex"],
Cell[1853902, 59122, 169, 4, 70, "Input",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{
Cell[1854096, 59130, 412, 10, 70, "Input",
  CellTags->"LorentzIndex"],
Cell[1854511, 59142, 479, 15, 70, "Output"]
}, Closed]],
Cell[1855005, 59160, 180, 4, 70, "Input",
  CellTags->"LorentzIndex"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1855234, 59170, 48, 1, 39, "ObjectName",
  CellTags->"Lower"],

Cell[CellGroupData[{
Cell[1855307, 59175, 57, 1, 49, "Subsubsection",
  CellTags->"Lower"],
Cell[1855367, 59178, 123, 4, 39, "Text",
  CellTags->"Lower"],
Cell[1855493, 59184, 435, 19, 39, "Text",
  CellTags->"Lower"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1855977, 59209, 60, 1, 39, "ObjectName",
  CellTags->"MakeContext"],

Cell[CellGroupData[{
Cell[1856062, 59214, 63, 1, 49, "Subsubsection",
  CellTags->"MakeContext"],
Cell[1856128, 59217, 239, 5, 56, "Text",
  CellTags->"MakeContext"],
Cell[1856370, 59224, 573, 24, 39, "Text",
  CellTags->"MakeContext"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1856992, 59254, 58, 1, 39, "ObjectName",
  CellTags->"Mandelstam"],

Cell[CellGroupData[{
Cell[1857075, 59259, 62, 1, 49, "Subsubsection",
  CellTags->"Mandelstam"],
Cell[1857140, 59262, 1845, 73, 73, "Text",
  CellTags->"Mandelstam"],
Cell[1858988, 59337, 821, 34, 39, "Text",
  CellTags->"Mandelstam"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1859858, 59377, 46, 1, 39, "ObjectName",
  CellTags->"Map2"],

Cell[CellGroupData[{
Cell[1859929, 59382, 56, 1, 49, "Subsubsection",
  CellTags->"Map2"],
Cell[1859988, 59385, 136, 4, 39, "Text",
  CellTags->"Map2"],
Cell[1860127, 59391, 182, 9, 39, "Text",
  CellTags->"Map2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1860346, 59405, 53, 1, 49, "Subsubsection",
  CellTags->"Map2"],

Cell[CellGroupData[{
Cell[1860424, 59410, 186, 5, 70, "Input",
  CellTags->"Map2"],
Cell[1860613, 59417, 230, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1860880, 59428, 157, 4, 70, "Input",
  CellTags->"Map2"],
Cell[1861040, 59434, 141, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1861218, 59442, 227, 6, 70, "Input",
  CellTags->"Map2"],
Cell[1861448, 59450, 218, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1861703, 59461, 157, 4, 70, "Input",
  CellTags->"Map2"],
Cell[1861863, 59467, 141, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1862065, 59477, 68, 1, 39, "ObjectName",
  CellTags->"MemoryAvailable"],

Cell[CellGroupData[{
Cell[1862158, 59482, 67, 1, 49, "Subsubsection",
  CellTags->"MemoryAvailable"],
Cell[1862228, 59485, 231, 5, 56, "Text",
  CellTags->"MemoryAvailable"],
Cell[1862462, 59492, 337, 14, 39, "Text",
  CellTags->"MemoryAvailable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1862848, 59512, 50, 1, 39, "ObjectName",
  CellTags->"MemSet"],

Cell[CellGroupData[{
Cell[1862923, 59517, 58, 1, 49, "Subsubsection",
  CellTags->"MemSet"],
Cell[1862984, 59520, 358, 9, 56, "Text",
  CellTags->"MemSet"],
Cell[1863345, 59531, 211, 9, 39, "Text",
  CellTags->"MemSet"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1863605, 59546, 62, 1, 39, "ObjectName",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{
Cell[1863692, 59551, 64, 1, 49, "Subsubsection",
  CellTags->"MetricTensor"],
Cell[1863759, 59554, 227, 7, 39, "Text",
  CellTags->"MetricTensor"],
Cell[1863989, 59563, 637, 29, 39, "Text",
  CellTags->"MetricTensor"]
}, Closed]],

Cell[CellGroupData[{
Cell[1864663, 59597, 61, 1, 49, "Subsubsection",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{
Cell[1864749, 59602, 186, 4, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1864938, 59608, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1865170, 59618, 169, 4, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1865342, 59624, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1865488, 59631, 239, 5, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1865730, 59638, 400, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1866167, 59655, 169, 4, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1866339, 59661, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1866485, 59668, 218, 5, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1866706, 59675, 242, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1866985, 59685, 273, 6, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1867261, 59693, 304, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1867602, 59705, 256, 6, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1867861, 59713, 150, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1868048, 59721, 320, 8, 70, "Input",
  CellTags->"MetricTensor"],
Cell[1868371, 59731, 151, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1868583, 59741, 50, 1, 39, "ObjectName",
  CellTags->"MLimit"],

Cell[CellGroupData[{
Cell[1868658, 59746, 58, 1, 49, "Subsubsection",
  CellTags->"MLimit"],
Cell[1868719, 59749, 119, 4, 39, "Text",
  CellTags->"MLimit"]
}, Closed]],

Cell[CellGroupData[{
Cell[1868875, 59758, 55, 1, 49, "Subsubsection",
  CellTags->"MLimit"],

Cell[CellGroupData[{
Cell[1868955, 59763, 512, 14, 56, "Input",
  CellTags->"MLimit"],
Cell[1869470, 59779, 106, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1869637, 59788, 54, 1, 39, "ObjectName",
  CellTags->"Momentum"],

Cell[CellGroupData[{
Cell[1869716, 59793, 60, 1, 49, "Subsubsection",
  CellTags->"Momentum"],
Cell[1869779, 59796, 261, 6, 56, "Text",
  CellTags->"Momentum"],
Cell[1870043, 59804, 573, 24, 39, "Text",
  CellTags->"Momentum"]
}, Closed]],

Cell[CellGroupData[{
Cell[1870653, 59833, 57, 1, 49, "Subsubsection",
  CellTags->"Momentum"],
Cell[1870713, 59836, 76, 1, 70, "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{
Cell[1870814, 59841, 138, 3, 70, "Input",
  CellTags->"Momentum"],
Cell[1870955, 59846, 109, 2, 70, "Output"]
}, Closed]],
Cell[1871079, 59851, 139, 4, 70, "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{
Cell[1871243, 59859, 165, 4, 70, "Input",
  CellTags->"Momentum"],
Cell[1871411, 59865, 109, 2, 70, "Output"]
}, Closed]],
Cell[1871535, 59870, 87, 1, 70, "Text",
  CellTags->"Momentum"],

Cell[CellGroupData[{
Cell[1871647, 59875, 206, 5, 70, "Input",
  CellTags->"Momentum"],
Cell[1871856, 59882, 156, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1872049, 59890, 190, 5, 70, "Input",
  CellTags->"Momentum"],
Cell[1872242, 59897, 131, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1872410, 59905, 139, 3, 70, "Input",
  CellTags->"Momentum"],
Cell[1872552, 59910, 151, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1872740, 59918, 313, 8, 70, "Input",
  CellTags->"Momentum"],
Cell[1873056, 59928, 225, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1873318, 59939, 139, 3, 70, "Input",
  CellTags->"Momentum"],
Cell[1873460, 59944, 253, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1873750, 59955, 182, 5, 70, "Input",
  CellTags->"Momentum"],
Cell[1873935, 59962, 195, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1874167, 59971, 188, 5, 70, "Input",
  CellTags->"Momentum"],
Cell[1874358, 59978, 156, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1874551, 59986, 263, 7, 70, "Input",
  CellTags->"Momentum"],
Cell[1874817, 59995, 156, 3, 70, "Output"]
}, Closed]],
Cell[1874988, 60001, 164, 4, 70, "Input",
  CellTags->"Momentum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1875201, 60011, 68, 1, 39, "ObjectName",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{
Cell[1875294, 60016, 67, 1, 49, "Subsubsection",
  CellTags->"MomentumCombine"],
Cell[1875364, 60019, 184, 4, 39, "Text",
  CellTags->"MomentumCombine"],
Cell[1875551, 60025, 627, 23, 39, "Text",
  CellTags->"MomentumCombine"]
}, Closed]],

Cell[CellGroupData[{
Cell[1876215, 60053, 64, 1, 49, "Subsubsection",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{
Cell[1876304, 60058, 391, 10, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1876698, 60070, 185, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1876920, 60079, 378, 10, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1877301, 60091, 235, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1877573, 60102, 153, 3, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1877729, 60107, 387, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1878153, 60124, 199, 5, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1878355, 60131, 306, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1878698, 60143, 206, 5, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1878907, 60150, 561, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1879505, 60173, 149, 3, 70, "Input",
  CellTags->"MomentumCombine"],
Cell[1879657, 60178, 468, 11, 70, "Output"]
}, Closed]],
Cell[1880140, 60192, 143, 3, 70, "Input",
  CellTags->"MomentumCombine"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1880332, 60201, 70, 1, 39, "ObjectName",
  CellTags->"MomentumCombine2"],

Cell[CellGroupData[{
Cell[1880427, 60206, 68, 1, 49, "Subsubsection",
  CellTags->"MomentumCombine2"],
Cell[1880498, 60209, 195, 5, 39, "Text",
  CellTags->"MomentumCombine2"],
Cell[1880696, 60216, 631, 23, 39, "Text",
  CellTags->"MomentumCombine2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1881364, 60244, 65, 1, 49, "Subsubsection",
  CellTags->"MomentumCombine2"],

Cell[CellGroupData[{
Cell[1881454, 60249, 517, 13, 56, "Input",
  CellTags->"MomentumCombine2"],
Cell[1881974, 60264, 601, 19, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1882612, 60288, 197, 5, 52, "Input",
  CellTags->"MomentumCombine2"],
Cell[1882812, 60295, 336, 8, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1883209, 60310, 66, 1, 39, "ObjectName",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{
Cell[1883300, 60315, 66, 1, 49, "Subsubsection",
  CellTags->"MomentumExpand"],
Cell[1883369, 60318, 150, 4, 39, "Text",
  CellTags->"MomentumExpand"],
Cell[1883522, 60324, 372, 14, 39, "Text",
  CellTags->"MomentumExpand"]
}, Closed]],

Cell[CellGroupData[{
Cell[1883931, 60343, 63, 1, 49, "Subsubsection",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{
Cell[1884019, 60348, 268, 7, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1884290, 60357, 195, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1884522, 60366, 205, 5, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1884730, 60373, 347, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1885114, 60387, 144, 3, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1885261, 60392, 265, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1885563, 60403, 254, 6, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1885820, 60411, 431, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1886288, 60430, 144, 3, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1886435, 60435, 308, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1886780, 60447, 285, 7, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1887068, 60456, 464, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1887569, 60476, 144, 3, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1887716, 60481, 410, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1888163, 60496, 140, 3, 70, "Input",
  CellTags->"MomentumExpand"],
Cell[1888306, 60501, 777, 22, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1889144, 60530, 42, 1, 39, "ObjectName",
  CellTags->"MT"],

Cell[CellGroupData[{
Cell[1889211, 60535, 54, 1, 49, "Subsubsection",
  CellTags->"MT"],
Cell[1889268, 60538, 195, 7, 39, "Text",
  CellTags->"MT"],
Cell[1889466, 60547, 657, 29, 39, "Text",
  CellTags->"MT"]
}, Closed]],

Cell[CellGroupData[{
Cell[1890160, 60581, 51, 1, 49, "Subsubsection",
  CellTags->"MT"],

Cell[CellGroupData[{
Cell[1890236, 60586, 166, 4, 70, "Input",
  CellTags->"MT"],
Cell[1890405, 60592, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1890637, 60602, 317, 8, 70, "Input",
  CellTags->"MT"],
Cell[1890957, 60612, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1891103, 60619, 194, 5, 70, "Input",
  CellTags->"MT"],
Cell[1891300, 60626, 150, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1891487, 60634, 239, 7, 70, "Input",
  CellTags->"MT"],
Cell[1891729, 60643, 242, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1892008, 60653, 279, 8, 70, "Input",
  CellTags->"MT"],
Cell[1892290, 60663, 150, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1892501, 60673, 44, 1, 39, "ObjectName",
  CellTags->"MTD"],

Cell[CellGroupData[{
Cell[1892570, 60678, 55, 1, 49, "Subsubsection",
  CellTags->"MTD"],
Cell[1892628, 60681, 197, 7, 39, "Text",
  CellTags->"MTD"],
Cell[1892828, 60690, 655, 29, 39, "Text",
  CellTags->"MTD"]
}, Closed]],

Cell[CellGroupData[{
Cell[1893520, 60724, 52, 1, 49, "Subsubsection",
  CellTags->"MTD"],

Cell[CellGroupData[{
Cell[1893597, 60729, 168, 4, 70, "Input",
  CellTags->"MTD"],
Cell[1893768, 60735, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1894000, 60745, 320, 8, 70, "Input",
  CellTags->"MTD"],
Cell[1894323, 60755, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1894469, 60762, 216, 6, 70, "Input",
  CellTags->"MTD"],
Cell[1894688, 60770, 164, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1894889, 60778, 254, 7, 70, "Input",
  CellTags->"MTD"],
Cell[1895146, 60787, 317, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1895500, 60799, 289, 8, 70, "Input",
  CellTags->"MTD"],
Cell[1895792, 60809, 159, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1896012, 60819, 68, 1, 39, "ObjectName",
  CellTags->"NegativeInteger"],

Cell[CellGroupData[{
Cell[1896105, 60824, 67, 1, 49, "Subsubsection",
  CellTags->"NegativeInteger"],
Cell[1896175, 60827, 142, 4, 39, "Text",
  CellTags->"NegativeInteger"],
Cell[1896320, 60833, 199, 9, 39, "Text",
  CellTags->"NegativeInteger"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1896568, 60848, 42, 1, 39, "ObjectName",
  CellTags->"Nf"],

Cell[CellGroupData[{
Cell[1896635, 60853, 54, 1, 49, "Subsubsection",
  CellTags->"Nf"],
Cell[1896692, 60856, 67, 1, 39, "Text",
  CellTags->"Nf"],
Cell[1896762, 60859, 738, 29, 39, "Text",
  CellTags->"Nf"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1897549, 60894, 52, 1, 39, "ObjectName",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1897626, 60899, 59, 1, 49, "Subsubsection",
  CellTags->"Nielsen"],
Cell[1897688, 60902, 87, 1, 39, "Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1897800, 60907, 142, 3, 56, "Input",
  CellTags->"Nielsen"],
Cell[1897945, 60912, 188, 5, 70, "Output"]
}, Closed]],
Cell[1898148, 60920, 212, 9, 35, "Text",
  CellTags->"Nielsen"]
}, Closed]],

Cell[CellGroupData[{
Cell[1898397, 60934, 56, 1, 49, "Subsubsection",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1898478, 60939, 173, 4, 70, "Input",
  CellTags->"Nielsen"],
Cell[1898654, 60945, 326, 10, 70, "Output"]
}, Closed]],
Cell[1898995, 60958, 179, 5, 70, "Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1899199, 60967, 209, 5, 70, "Input",
  CellTags->"Nielsen"],
Cell[1899411, 60974, 128, 2, 70, "Output"]
}, Closed]],
Cell[1899554, 60979, 72, 1, 70, "Text",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1899651, 60984, 557, 14, 70, "Input",
  CellTags->"Nielsen"],
Cell[1900211, 61000, 765, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1901013, 61027, 227, 5, 70, "Input",
  CellTags->"Nielsen"],
Cell[1901243, 61034, 820, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1902100, 61062, 227, 5, 70, "Input",
  CellTags->"Nielsen"],
Cell[1902330, 61069, 1042, 29, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1903409, 61103, 227, 5, 70, "Input",
  CellTags->"Nielsen"],
Cell[1903639, 61110, 172, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1903872, 61122, 62, 1, 39, "ObjectName",
  CellTags->"NonCommFreeQ"],

Cell[CellGroupData[{
Cell[1903959, 61127, 64, 1, 49, "Subsubsection",
  CellTags->"NonCommFreeQ"],
Cell[1904026, 61130, 245, 5, 56, "Text",
  CellTags->"NonCommFreeQ"],
Cell[1904274, 61137, 562, 24, 39, "Text",
  CellTags->"NonCommFreeQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1904885, 61167, 54, 1, 39, "ObjectName",
  CellTags->"NonCommQ"],

Cell[CellGroupData[{
Cell[1904964, 61172, 60, 1, 49, "Subsubsection",
  CellTags->"NonCommQ"],
Cell[1905027, 61175, 206, 5, 56, "Text",
  CellTags->"NonCommQ"],
Cell[1905236, 61182, 570, 24, 39, "Text",
  CellTags->"NonCommQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1905855, 61212, 66, 1, 39, "ObjectName",
  CellTags->"NonCommutative"],

Cell[CellGroupData[{
Cell[1905946, 61217, 66, 1, 49, "Subsubsection",
  CellTags->"NonCommutative"],
Cell[1906015, 61220, 151, 4, 39, "Text",
  CellTags->"NonCommutative"],
Cell[1906169, 61226, 356, 14, 39, "Text",
  CellTags->"NonCommutative"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1906574, 61246, 50, 1, 39, "ObjectName",
  CellTags->"NTerms"],

Cell[CellGroupData[{
Cell[1906649, 61251, 58, 1, 49, "Subsubsection",
  CellTags->"NTerms"],
Cell[1906710, 61254, 155, 4, 39, "Text",
  CellTags->"NTerms"]
}, Closed]],

Cell[CellGroupData[{
Cell[1906902, 61263, 55, 1, 49, "Subsubsection",
  CellTags->"NTerms"],

Cell[CellGroupData[{
Cell[1906982, 61268, 161, 4, 70, "Input",
  CellTags->"NTerms"],
Cell[1907146, 61274, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1907292, 61281, 171, 4, 70, "Input",
  CellTags->"NTerms"],
Cell[1907466, 61287, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1907612, 61294, 134, 3, 70, "Input",
  CellTags->"NTerms"],
Cell[1907749, 61299, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1907895, 61306, 134, 3, 70, "Input",
  CellTags->"NTerms"],
Cell[1908032, 61311, 109, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1908202, 61320, 80, 1, 39, "ObjectName",
  CellTags->"NumberOfMetricTensors"],

Cell[CellGroupData[{
Cell[1908307, 61325, 73, 1, 49, "Subsubsection",
  CellTags->"NumberOfMetricTensors"],
Cell[1908383, 61328, 96, 1, 39, "Text",
  CellTags->"NumberOfMetricTensors"],
Cell[1908482, 61331, 180, 9, 39, "Text",
  CellTags->"NonCommQ"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1908711, 61346, 68, 1, 39, "ObjectName",
  CellTags->"NumericalFactor"],

Cell[CellGroupData[{
Cell[1908804, 61351, 67, 1, 49, "Subsubsection",
  CellTags->"NumericalFactor"],
Cell[1908874, 61354, 122, 4, 39, "Text",
  CellTags->"NumericalFactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1909045, 61364, 56, 1, 39, "ObjectName",
  CellTags->"NumericQ1"],

Cell[CellGroupData[{
Cell[1909126, 61369, 61, 1, 49, "Subsubsection",
  CellTags->"NumericQ1"],
Cell[1909190, 61372, 140, 4, 39, "Text",
  CellTags->"NumericQ1"]
}, Closed]],

Cell[CellGroupData[{
Cell[1909367, 61381, 58, 1, 49, "Subsubsection",
  CellTags->"NumericQ1"],

Cell[CellGroupData[{
Cell[1909450, 61386, 268, 7, 57, "Input",
  CellTags->"NumericQ1"],
Cell[1909721, 61395, 110, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1909868, 61402, 328, 9, 53, "Input",
  CellTags->"NumericQ1"],
Cell[1910199, 61413, 110, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1910346, 61420, 374, 10, 53, "Input",
  CellTags->"NumericQ1"],
Cell[1910723, 61432, 109, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1910893, 61441, 70, 1, 39, "ObjectName",
  CellTags->"OneLoop"],
Cell[1910966, 61444, 642, 16, 73, "Text",
  CellTags->"OneLoop"],
Cell[1911611, 61462, 1110, 27, 56, "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{
Cell[1912746, 61493, 59, 1, 49, "Subsubsection",
  CellTags->"OneLoop"],
Cell[1912808, 61496, 405, 13, 56, "Text",
  CellTags->"OneLoop"],
Cell[1913216, 61511, 355, 7, 73, "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{
Cell[1913596, 61522, 142, 3, 56, "Input",
  CellTags->"OneLoop"],
Cell[1913741, 61527, 1392, 28, 70, "Output"]
}, Closed]],
Cell[1915148, 61558, 854, 39, 35, "Text",
  CellTags->"OneLoop"]
}, Closed]],

Cell[CellGroupData[{
Cell[1916039, 61602, 56, 1, 49, "Subsubsection",
  CellTags->"OneLoop"],

Cell[CellGroupData[{
Cell[1916120, 61607, 342, 10, 56, "Input",
  CellTags->"OneLoop"],
Cell[1916465, 61619, 390, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1916892, 61635, 163, 4, 52, "Input",
  CellTags->"OneLoop"],
Cell[1917058, 61641, 383, 10, 70, "Output"]
}, Closed]],
Cell[1917456, 61654, 440, 11, 52, "Input",
  CellTags->"OneLoop"],
Cell[1917899, 61667, 733, 20, 39, "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{
Cell[1918657, 61691, 1146, 30, 74, "Input",
  CellTags->"OneLoop"],
Cell[1919806, 61723, 3058, 76, 70, "Output"]
}, Closed]],
Cell[1922879, 61802, 220, 5, 52, "Text",
  CellTags->"OneLoop"],

Cell[CellGroupData[{
Cell[1923124, 61811, 163, 4, 56, "Input",
  CellTags->"OneLoop"],
Cell[1923290, 61817, 4072, 95, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1927399, 61917, 141, 3, 52, "Input",
  CellTags->"OneLoop"],
Cell[1927543, 61922, 2917, 70, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1930497, 61997, 416, 12, 52, "Input",
  CellTags->"OneLoop"],
Cell[1930916, 62011, 1251, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1932204, 62051, 163, 4, 52, "Input",
  CellTags->"OneLoop"],
Cell[1932370, 62057, 2743, 71, 70, "Output"]
}, Closed]],
Cell[1935128, 62131, 134, 3, 52, "Input",
  CellTags->"OneLoop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1935311, 62140, 68, 1, 39, "ObjectName",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{
Cell[1935404, 62145, 67, 1, 49, "Subsubsection",
  CellTags->"OneLoopSimplify"],
Cell[1935474, 62148, 173, 4, 39, "Text",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{
Cell[1935672, 62156, 158, 3, 56, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1935833, 62161, 821, 17, 70, "Output"]
}, Closed]],
Cell[1936669, 62181, 409, 19, 35, "Text",
  CellTags->"OneLoopSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[1937115, 62205, 64, 1, 49, "Subsubsection",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{
Cell[1937204, 62210, 424, 12, 56, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1937631, 62224, 1251, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1938919, 62264, 179, 4, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1939101, 62270, 2298, 62, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1941436, 62337, 246, 6, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1941685, 62345, 977, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1942699, 62377, 485, 12, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1943187, 62391, 1810, 52, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1945034, 62448, 184, 4, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1945221, 62454, 13468, 351, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1958726, 62810, 312, 9, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1959041, 62821, 9388, 250, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1968466, 63076, 533, 15, 52, "Input",
  CellTags->"OneLoopSimplify"],
Cell[1969002, 63093, 4221, 106, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1973284, 63206, 58, 1, 39, "ObjectName",
  CellTags->"OneLoopSum"],

Cell[CellGroupData[{
Cell[1973367, 63211, 62, 1, 49, "Subsubsection",
  CellTags->"OneLoopSum"],
Cell[1973432, 63214, 193, 5, 39, "Text",
  CellTags->"OneLoopSum"],
Cell[1973628, 63221, 191, 9, 39, "Text",
  CellTags->"OneLoopSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1973868, 63236, 44, 1, 39, "ObjectName",
  CellTags->"OPE"],

Cell[CellGroupData[{
Cell[1973937, 63241, 55, 1, 49, "Subsubsection",
  CellTags->"OPE"],
Cell[1973995, 63244, 70, 1, 39, "Text",
  CellTags->"OPE"],
Cell[1974068, 63247, 187, 9, 39, "Text",
  CellTags->"OPE"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1974304, 63262, 54, 1, 39, "ObjectName",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{
Cell[1974383, 63267, 60, 1, 49, "Subsubsection",
  CellTags->"OPEDelta"],
Cell[1974446, 63270, 141, 4, 39, "Text",
  CellTags->"OPEDelta"],
Cell[1974590, 63276, 225, 9, 39, "Text",
  CellTags->"OPEDelta"]
}, Closed]],

Cell[CellGroupData[{
Cell[1974852, 63290, 57, 1, 49, "Subsubsection",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{
Cell[1974934, 63295, 178, 4, 70, "Input",
  CellTags->"OPEDelta"],
Cell[1975115, 63301, 200, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1975352, 63311, 165, 4, 70, "Input",
  CellTags->"OPEDelta"],
Cell[1975520, 63317, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1975666, 63324, 173, 4, 70, "Input",
  CellTags->"OPEDelta"],
Cell[1975842, 63330, 109, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1976012, 63339, 82, 1, 39, "ObjectName",
  CellTags->"OPEi"],

Cell[CellGroupData[{
Cell[1976119, 63344, 56, 1, 49, "Subsubsection",
  CellTags->"OPEi"],
Cell[1976178, 63347, 172, 4, 39, "Text",
  CellTags->"OPEi"]
}, Closed]],

Cell[CellGroupData[{
Cell[1976387, 63356, 53, 1, 49, "Subsubsection",
  CellTags->"OPEi"],

Cell[CellGroupData[{
Cell[1976465, 63361, 100, 2, 56, "Input",
  CellTags->"OPEi"],
Cell[1976568, 63365, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1976714, 63372, 304, 7, 52, "Input",
  CellTags->"OPEi"],
Cell[1977021, 63381, 256, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1977314, 63392, 1365, 40, 72, "Input",
  CellTags->"OPEi"],
Cell[1978682, 63434, 233, 6, 70, "Output"]
}, Closed]],
Cell[1978930, 63443, 56, 1, 35, "Text",
  CellTags->"OPEi"],

Cell[CellGroupData[{
Cell[1979011, 63448, 632, 17, 56, "Input",
  CellTags->"OPEi"],
Cell[1979646, 63467, 219, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1979902, 63477, 662, 19, 52, "Input",
  CellTags->"OPEi"],
Cell[1980567, 63498, 191, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1980795, 63508, 632, 17, 52, "Input",
  CellTags->"OPEi"],
Cell[1981430, 63527, 219, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1981710, 63539, 74, 1, 39, "ObjectName",
  CellTags->"OPEInsert"],

Cell[CellGroupData[{
Cell[1981809, 63544, 61, 1, 49, "Subsubsection",
  CellTags->"OPEInsert"],
Cell[1981873, 63547, 267, 6, 56, "Text",
  CellTags->"OPEInsert"],
Cell[1982143, 63555, 178, 9, 39, "Text",
  CellTags->"OPEInsert"]
}, Closed]],

Cell[CellGroupData[{
Cell[1982358, 63569, 58, 1, 49, "Subsubsection",
  CellTags->"OPEInsert"],
Cell[1982419, 63572, 101, 2, 56, "Input",
  CellTags->"OPEInsert"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1982569, 63580, 68, 1, 39, "ObjectName",
  CellTags->"OPEInt"],

Cell[CellGroupData[{
Cell[1982662, 63585, 58, 1, 49, "Subsubsection",
  CellTags->"OPEInt"],
Cell[1982723, 63588, 260, 6, 56, "Text",
  CellTags->"OPEInt"],
Cell[1982986, 63596, 175, 9, 39, "Text",
  CellTags->"OPEInt"]
}, Closed]],

Cell[CellGroupData[{
Cell[1983198, 63610, 55, 1, 49, "Subsubsection",
  CellTags->"OPEInt"],
Cell[1983256, 63613, 98, 2, 56, "Input",
  CellTags->"OPEInt"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1983403, 63621, 78, 1, 39, "ObjectName",
  CellTags->"OPEIntDelta"],

Cell[CellGroupData[{
Cell[1983506, 63626, 63, 1, 49, "Subsubsection",
  CellTags->"OPEIntDelta"],
Cell[1983572, 63629, 484, 10, 81, "Text",
  CellTags->"OPEIntDelta"],
Cell[1984059, 63641, 180, 9, 39, "Text",
  CellTags->"OPEIntDelta"]
}, Closed]],

Cell[CellGroupData[{
Cell[1984276, 63655, 60, 1, 49, "Subsubsection",
  CellTags->"OPEIntDelta"],
Cell[1984339, 63658, 103, 2, 56, "Input",
  CellTags->"OPEIntDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1984491, 63666, 80, 1, 39, "ObjectName",
  CellTags->"OPEIntegrate"],

Cell[CellGroupData[{
Cell[1984596, 63671, 64, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrate"],
Cell[1984663, 63674, 269, 6, 56, "Text",
  CellTags->"OPEIntegrate"],
Cell[1984935, 63682, 181, 9, 39, "Text",
  CellTags->"OPEIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[1985153, 63696, 61, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrate"],
Cell[1985217, 63699, 104, 2, 56, "Input",
  CellTags->"OPEIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1985370, 63707, 90, 1, 39, "ObjectName",
  CellTags->"OPEIntegrateDelta"],

Cell[CellGroupData[{
Cell[1985485, 63712, 69, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrateDelta"],
Cell[1985557, 63715, 505, 10, 81, "Text",
  CellTags->"OPEIntegrateDelta"],
Cell[1986065, 63727, 186, 9, 39, "Text",
  CellTags->"OPEIntegrateDelta"]
}, Closed]],

Cell[CellGroupData[{
Cell[1986288, 63741, 66, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrateDelta"],
Cell[1986357, 63744, 109, 2, 56, "Input",
  CellTags->"OPEIntegrateDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1986515, 63752, 82, 1, 39, "ObjectName",
  CellTags->"OPEIntegrate2"],

Cell[CellGroupData[{
Cell[1986622, 63757, 65, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrate2"],
Cell[1986690, 63760, 153, 4, 39, "Text",
  CellTags->"OPEIntegrate2"],
Cell[1986846, 63766, 209, 9, 39, "Text",
  CellTags->"OPEIntegrate2"]
}, Closed]],

Cell[CellGroupData[{
Cell[1987092, 63780, 62, 1, 49, "Subsubsection",
  CellTags->"OPEIntegrate2"],
Cell[1987157, 63783, 105, 2, 56, "Input",
  CellTags->"OPEIntegrate2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1987311, 63791, 66, 1, 39, "ObjectName",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{
Cell[1987402, 63796, 66, 1, 49, "Subsubsection",
  CellTags->"OPESumExplicit"],
Cell[1987471, 63799, 86, 1, 39, "Text",
  CellTags->"OPESumExplicit"],
Cell[1987560, 63802, 330, 14, 39, "Text",
  CellTags->"OPESumExplicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[1987927, 63821, 63, 1, 49, "Subsubsection",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{
Cell[1988015, 63826, 482, 13, 70, "Input",
  CellTags->"OPESumExplicit"],
Cell[1988500, 63841, 454, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1988991, 63859, 151, 3, 70, "Input",
  CellTags->"OPESumExplicit"],
Cell[1989145, 63864, 492, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1989674, 63884, 686, 18, 70, "Input",
  CellTags->"OPESumExplicit"],
Cell[1990363, 63904, 876, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1991276, 63934, 151, 3, 70, "Input",
  CellTags->"OPESumExplicit"],
Cell[1991430, 63939, 2814, 83, 70, "Output"]
}, Closed]],
Cell[1994259, 64025, 194, 5, 70, "Input",
  CellTags->"OPESumExplicit"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1994502, 64036, 50, 1, 39, "ObjectName",
  CellTags->"OPESum"],

Cell[CellGroupData[{
Cell[1994577, 64041, 58, 1, 49, "Subsubsection",
  CellTags->"OPESum"],
Cell[1994638, 64044, 128, 4, 39, "Text",
  CellTags->"OPESum"],
Cell[1994769, 64050, 346, 14, 39, "Text",
  CellTags->"OPESum"]
}, Closed]],

Cell[CellGroupData[{
Cell[1995152, 64069, 55, 1, 49, "Subsubsection",
  CellTags->"OPESum"],

Cell[CellGroupData[{
Cell[1995232, 64074, 571, 15, 70, "Input",
  CellTags->"OPESum"],
Cell[1995806, 64091, 1339, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1997182, 64131, 143, 3, 70, "Input",
  CellTags->"OPESum"],
Cell[1997328, 64136, 2847, 77, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2000212, 64218, 678, 18, 70, "Input",
  CellTags->"OPESum"],
Cell[2000893, 64238, 876, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2001806, 64268, 143, 3, 70, "Input",
  CellTags->"OPESum"],
Cell[2001952, 64273, 2814, 83, 70, "Output"]
}, Closed]],
Cell[2004781, 64359, 186, 5, 70, "Input",
  CellTags->"OPESum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2005016, 64370, 66, 1, 39, "ObjectName",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{
Cell[2005107, 64375, 66, 1, 49, "Subsubsection",
  CellTags->"OPESumSimplify"],
Cell[2005176, 64378, 93, 1, 39, "Text",
  CellTags->"OPESumSimplify"],
Cell[2005272, 64381, 330, 14, 39, "Text",
  CellTags->"OPESumSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[2005639, 64400, 63, 1, 49, "Subsubsection",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{
Cell[2005727, 64405, 170, 5, 70, "Input",
  CellTags->"OPESumSimplify"],
Cell[2005900, 64412, 1384, 36, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2007321, 64453, 114, 2, 70, "Input",
  CellTags->"OPESumSimplify"],
Cell[2007438, 64457, 1621, 39, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2009096, 64501, 328, 8, 70, "Input",
  CellTags->"OPESumSimplify"],
Cell[2009427, 64511, 330, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2009794, 64526, 535, 14, 70, "Input",
  CellTags->"OPESumSimplify"],
Cell[2010332, 64542, 718, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2011087, 64568, 144, 3, 70, "Input",
  CellTags->"OPESumSimplify"],
Cell[2011234, 64573, 460, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2011755, 64592, 72, 1, 39, "ObjectName",
  CellTags->"OPE1Loop"],

Cell[CellGroupData[{
Cell[2011852, 64597, 60, 1, 49, "Subsubsection",
  CellTags->"OPE1Loop"],
Cell[2011915, 64600, 122, 4, 39, "Text",
  CellTags->"OPE1Loop"],
Cell[2012040, 64606, 186, 9, 39, "Text",
  CellTags->"OPE1Loop"]
}, Closed]],

Cell[CellGroupData[{
Cell[2012263, 64620, 57, 1, 49, "Subsubsection",
  CellTags->"OPE1Loop"],
Cell[2012323, 64623, 100, 2, 56, "Input",
  CellTags->"OPE1Loop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2012472, 64631, 68, 1, 39, "ObjectName",
  CellTags->"OPE2AI"],

Cell[CellGroupData[{
Cell[2012565, 64636, 58, 1, 49, "Subsubsection",
  CellTags->"OPE2AI"],
Cell[2012626, 64639, 80, 1, 39, "Text",
  CellTags->"OPE2AI"],
Cell[2012709, 64642, 138, 3, 56, "Input",
  CellTags->"OPE2AI"],
Cell[2012850, 64647, 184, 9, 39, "Text",
  CellTags->"OPE2AI"]
}, Closed]],

Cell[CellGroupData[{
Cell[2013071, 64661, 55, 1, 49, "Subsubsection",
  CellTags->"OPE2AI"],
Cell[2013129, 64664, 98, 2, 56, "Input",
  CellTags->"OPE2AI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2013276, 64672, 70, 1, 39, "ObjectName",
  CellTags->"OPE2TID"],

Cell[CellGroupData[{
Cell[2013371, 64677, 59, 1, 49, "Subsubsection",
  CellTags->"OPE2TID"],
Cell[2013433, 64680, 187, 5, 39, "Text",
  CellTags->"OPE2TID"],

Cell[CellGroupData[{
Cell[2013645, 64689, 140, 3, 56, "Input",
  CellTags->"OPE2TID"],
Cell[2013788, 64694, 383, 9, 37, "Output"]
}, Closed]],
Cell[2014186, 64706, 185, 9, 35, "Text",
  CellTags->"OPE2TID"]
}, Closed]],

Cell[CellGroupData[{
Cell[2014408, 64720, 56, 1, 49, "Subsubsection",
  CellTags->"OPE2TID"],
Cell[2014467, 64723, 99, 2, 56, "Input",
  CellTags->"OPE2TID"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2014615, 64731, 64, 1, 39, "ObjectName",
  CellTags->"OptionsSelect"],

Cell[CellGroupData[{
Cell[2014704, 64736, 65, 1, 49, "Subsubsection",
  CellTags->"OptionsSelect"],
Cell[2014772, 64739, 220, 5, 56, "Text",
  CellTags->"OptionsSelect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2015041, 64750, 46, 1, 39, "ObjectName",
  CellTags->"Pair"],

Cell[CellGroupData[{
Cell[2015112, 64755, 56, 1, 49, "Subsubsection",
  CellTags->"Pair"],
Cell[2015171, 64758, 512, 11, 73, "Text",
  CellTags->"Pair"],
Cell[2015686, 64771, 1082, 49, 39, "Text",
  CellTags->"Pair"]
}, Closed]],

Cell[CellGroupData[{
Cell[2016805, 64825, 53, 1, 49, "Subsubsection",
  CellTags->"Pair"],
Cell[2016861, 64828, 85, 1, 70, "Text",
  CellTags->"Pair"],

Cell[CellGroupData[{
Cell[2016971, 64833, 260, 6, 70, "Input",
  CellTags->"Pair"],
Cell[2017234, 64841, 195, 5, 70, "Output"]
}, Closed]],
Cell[2017444, 64849, 74, 1, 70, "Text",
  CellTags->"Pair"],

Cell[CellGroupData[{
Cell[2017543, 64854, 322, 8, 70, "Input",
  CellTags->"Pair"],
Cell[2017868, 64864, 400, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2018305, 64881, 250, 6, 70, "Input",
  CellTags->"Pair"],
Cell[2018558, 64889, 289, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2018884, 64903, 239, 6, 70, "Input",
  CellTags->"Pair"],
Cell[2019126, 64911, 269, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2019432, 64923, 239, 6, 70, "Input",
  CellTags->"Pair"],
Cell[2019674, 64931, 184, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2019895, 64941, 270, 7, 70, "Input",
  CellTags->"Pair"],
Cell[2020168, 64950, 349, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2020554, 64965, 272, 7, 70, "Input",
  CellTags->"Pair"],
Cell[2020829, 64974, 184, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2021050, 64984, 272, 7, 70, "Input",
  CellTags->"Pair"],
Cell[2021325, 64993, 184, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2021546, 65003, 376, 10, 70, "Input",
  CellTags->"Pair"],
Cell[2021925, 65015, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2022071, 65022, 328, 8, 70, "Input",
  CellTags->"Pair"],
Cell[2022402, 65032, 387, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2022826, 65048, 460, 12, 70, "Input",
  CellTags->"Pair"],
Cell[2023289, 65062, 109, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2023459, 65071, 60, 1, 39, "ObjectName",
  CellTags->"PairCollect"],

Cell[CellGroupData[{
Cell[2023544, 65076, 63, 1, 49, "Subsubsection",
  CellTags->"PairCollect"],
Cell[2023610, 65079, 154, 4, 39, "Text",
  CellTags->"PairCollect"],
Cell[2023767, 65085, 309, 14, 39, "Text",
  CellTags->"PairCollect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2024125, 65105, 62, 1, 39, "ObjectName",
  CellTags->"PairContract"],

Cell[CellGroupData[{
Cell[2024212, 65110, 64, 1, 49, "Subsubsection",
  CellTags->"PairContract"],
Cell[2024279, 65113, 121, 4, 39, "Text",
  CellTags->"PairContract"],
Cell[2024403, 65119, 297, 13, 39, "Text",
  CellTags->"PairContract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2024749, 65138, 64, 1, 39, "ObjectName",
  CellTags->"PairContract2"],

Cell[CellGroupData[{
Cell[2024838, 65143, 65, 1, 49, "Subsubsection",
  CellTags->"PairContract2"],
Cell[2024906, 65146, 143, 4, 39, "Text",
  CellTags->"PairContract2"],
Cell[2025052, 65152, 430, 18, 39, "Text",
  CellTags->"PairContract2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2025531, 65176, 64, 1, 39, "ObjectName",
  CellTags->"PairContract3"],

Cell[CellGroupData[{
Cell[2025620, 65181, 65, 1, 49, "Subsubsection",
  CellTags->"PairContract3"],
Cell[2025688, 65184, 143, 4, 39, "Text",
  CellTags->"PairContract3"],
Cell[2025834, 65190, 430, 18, 39, "Text",
  CellTags->"PairContract3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2026313, 65214, 54, 1, 39, "ObjectName",
  CellTags->"PartialD"],

Cell[CellGroupData[{
Cell[2026392, 65219, 60, 1, 49, "Subsubsection",
  CellTags->"PartialD"],
Cell[2026455, 65222, 1019, 35, 58, "Text",
  CellTags->"PartialD"],
Cell[2027477, 65259, 624, 24, 39, "Text",
  CellTags->"PartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[2028138, 65288, 66, 1, 50, "Subsubsection",
  CellTags->"PartialD"],

Cell[CellGroupData[{
Cell[2028229, 65293, 280, 7, 70, "Input",
  CellTags->"PartialD"],
Cell[2028512, 65302, 358, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2028907, 65317, 144, 3, 70, "Input",
  CellTags->"PartialD"],
Cell[2029054, 65322, 358, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2029449, 65337, 142, 3, 70, "Input",
  CellTags->"PartialD"],
Cell[2029594, 65342, 332, 7, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2029987, 65356, 72, 1, 39, "ObjectName",
  CellTags->"PartialDRelations"],

Cell[CellGroupData[{
Cell[2030084, 65361, 69, 1, 49, "Subsubsection",
  CellTags->"PartialDRelations"],
Cell[2030156, 65364, 193, 5, 39, "Text",
  CellTags->"PartialDRelations"],
Cell[2030352, 65371, 753, 29, 39, "Text",
  CellTags->"PartialDRelations"]
}, Closed]],

Cell[CellGroupData[{
Cell[2031142, 65405, 75, 1, 50, "Subsubsection",
  CellTags->"PartialDRelations"],

Cell[CellGroupData[{
Cell[2031242, 65410, 381, 10, 43, "Input",
  CellTags->"PartialDRelations"],
Cell[2031626, 65422, 375, 11, 28, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2032038, 65438, 281, 7, 39, "Input",
  CellTags->"PartialDRelations"],
Cell[2032322, 65447, 1087, 33, 26, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2033470, 65487, 72, 1, 39, "ObjectName",
  CellTags->"PartialFourVector"],

Cell[CellGroupData[{
Cell[2033567, 65492, 69, 1, 49, "Subsubsection",
  CellTags->"PartialFourVector"],
Cell[2033639, 65495, 279, 6, 56, "Text",
  CellTags->"PartialFourVector"],
Cell[2033921, 65503, 435, 19, 39, "Text",
  CellTags->"PartialFourVector"]
}, Closed]],

Cell[CellGroupData[{
Cell[2034393, 65527, 75, 1, 50, "Subsubsection",
  CellTags->"PartialFourVector"],

Cell[CellGroupData[{
Cell[2034493, 65532, 458, 11, 59, "Input",
  CellTags->"PartialFourVector"],
Cell[2034954, 65545, 476, 14, 39, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2035491, 65566, 70, 1, 39, "ObjectName",
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{
Cell[2035586, 65571, 68, 1, 49, "Subsubsection",
  CellTags->"PartialIntegrate"],
Cell[2035657, 65574, 285, 6, 56, "Text",
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{
Cell[2035967, 65584, 158, 3, 56, "Input",
  CellTags->"PartialIntegrate"],
Cell[2036128, 65589, 224, 6, 37, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2036401, 65601, 74, 1, 50, "Subsubsection",
  CellTags->"PartialIntegrate"],

Cell[CellGroupData[{
Cell[2036500, 65606, 389, 10, 43, "Input",
  CellTags->"PartialIntegrate"],
Cell[2036892, 65618, 1237, 35, 61, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2038166, 65658, 319, 8, 39, "Input",
  CellTags->"PartialIntegrate"],
Cell[2038488, 65668, 453, 13, 62, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2038978, 65686, 344, 10, 39, "Input",
  CellTags->"PartialIntegrate"],
Cell[2039325, 65698, 491, 15, 53, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2039853, 65718, 371, 11, 53, "Input",
  CellTags->"PartialIntegrate"],
Cell[2040227, 65731, 479, 14, 57, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2040743, 65750, 371, 10, 39, "Input",
  CellTags->"PartialIntegrate"],
Cell[2041117, 65762, 226, 6, 56, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2041380, 65773, 392, 11, 39, "Input",
  CellTags->"PartialIntegrate"],
Cell[2041775, 65786, 226, 6, 56, "Output",
  CellTags->"PartialIntegrate"]
}, Closed]],
Cell[2042016, 65795, 168, 4, 39, "Input",
  CellTags->"PartialIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2042233, 65805, 58, 1, 39, "ObjectName",
  CellTags->"PartitHead"],

Cell[CellGroupData[{
Cell[2042316, 65810, 62, 1, 49, "Subsubsection",
  CellTags->"PartitHead"],
Cell[2042381, 65813, 168, 4, 39, "Text",
  CellTags->"PartitHead"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2042598, 65823, 58, 1, 39, "ObjectName",
  CellTags->"PauliSigma"],

Cell[CellGroupData[{
Cell[2042681, 65828, 62, 1, 49, "Subsubsection",
  CellTags->"PauliSigma"],
Cell[2042746, 65831, 255, 6, 56, "Text",
  CellTags->"PauliSigma"],

Cell[CellGroupData[{
Cell[2043026, 65841, 170, 4, 56, "Input",
  CellTags->"PauliSigma"],
Cell[2043199, 65847, 1990, 60, 126, "Output",
  CellTags->"PauliSigma"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2045250, 65914, 46, 1, 39, "ObjectName",
  CellTags->"PaVe"],

Cell[CellGroupData[{
Cell[2045321, 65919, 56, 1, 49, "Subsubsection",
  CellTags->"PaVe"],
Cell[2045380, 65922, 431, 8, 73, "Text",
  CellTags->"PaVe"],
Cell[2045814, 65932, 278, 8, 39, "Text",
  CellTags->"PaVe"],
Cell[2046095, 65942, 194, 9, 39, "Text",
  CellTags->"PaVe"]
}, Closed]],

Cell[CellGroupData[{
Cell[2046326, 65956, 53, 1, 49, "Subsubsection",
  CellTags->"PaVe"],
Cell[2046382, 65959, 79, 1, 70, "Text",
  CellTags->"PaVe"],

Cell[CellGroupData[{
Cell[2046486, 65964, 369, 10, 70, "Input",
  CellTags->"PaVe"],
Cell[2046858, 65976, 676, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2047571, 65993, 333, 9, 70, "Input",
  CellTags->"PaVe"],
Cell[2047907, 66004, 2553, 60, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2050521, 66071, 64, 1, 39, "ObjectName",
  CellTags->"PaVeOrderList"],

Cell[CellGroupData[{
Cell[2050610, 66076, 65, 1, 49, "Subsubsection",
  CellTags->"PaVeOrderList"],
Cell[2050678, 66079, 177, 4, 39, "Text",
  CellTags->"PaVeOrderList"],
Cell[2050858, 66085, 529, 23, 39, "Text",
  CellTags->"PaVeOrderList"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2051436, 66114, 56, 1, 39, "ObjectName",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{
Cell[2051517, 66119, 61, 1, 49, "Subsubsection",
  CellTags->"PaVeOrder"],
Cell[2051581, 66122, 600, 16, 56, "Text",
  CellTags->"PaVeOrder"],
Cell[2052184, 66140, 199, 9, 39, "Text",
  CellTags->"PaVeOrder"]
}, Closed]],

Cell[CellGroupData[{
Cell[2052420, 66154, 58, 1, 49, "Subsubsection",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{
Cell[2052503, 66159, 491, 12, 70, "Input",
  CellTags->"PaVeOrder"],
Cell[2052997, 66173, 778, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2053812, 66194, 491, 12, 70, "Input",
  CellTags->"PaVeOrder"],
Cell[2054306, 66208, 778, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2055121, 66229, 851, 20, 70, "Input",
  CellTags->"PaVeOrder"],
Cell[2055975, 66251, 1435, 28, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2057471, 66286, 58, 1, 39, "ObjectName",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[2057554, 66291, 62, 1, 49, "Subsubsection",
  CellTags->"PaVeReduce"],
Cell[2057619, 66294, 168, 4, 39, "Text",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[2057812, 66302, 148, 3, 56, "Input",
  CellTags->"PaVeReduce"],
Cell[2057963, 66307, 505, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2058505, 66324, 134, 3, 52, "Input",
  CellTags->"PaVeReduce"],
Cell[2058642, 66329, 281, 4, 70, "Print"]
}, Closed]],
Cell[2058938, 66336, 302, 14, 35, "Text",
  CellTags->"PaVeReduce"]
}, Closed]],

Cell[CellGroupData[{
Cell[2059277, 66355, 59, 1, 49, "Subsubsection",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[2059361, 66360, 643, 16, 56, "Input",
  CellTags->"PaVeReduce"],
Cell[2060007, 66378, 4058, 106, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2064102, 66489, 134, 3, 52, "Input",
  CellTags->"PaVeReduce"],
Cell[2064239, 66494, 142, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2064418, 66502, 135, 3, 52, "Input",
  CellTags->"PaVeReduce"],
Cell[2064556, 66507, 142, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2064735, 66515, 607, 15, 52, "Input",
  CellTags->"PaVeReduce"],
Cell[2065345, 66532, 510, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2065892, 66550, 487, 12, 72, "Input",
  CellTags->"PaVeReduce"],
Cell[2066382, 66564, 599, 15, 71, "Output"]
}, Closed]],
Cell[2066996, 66582, 255, 7, 56, "Input",
  CellTags->"PaVeReduce"],
Cell[2067254, 66591, 199, 5, 56, "Input",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[2067478, 66600, 790, 19, 76, "Input",
  CellTags->"PaVeReduce"],
Cell[2068271, 66621, 11807, 272, 70, "Output"]
}, Closed]],
Cell[2080093, 66896, 321, 8, 52, "Input",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[2080439, 66908, 459, 11, 56, "Input",
  CellTags->"PaVeReduce"],
Cell[2080901, 66921, 4206, 78, 521, "Output"]
}, Closed]],
Cell[2085122, 67002, 299, 7, 52, "Input",
  CellTags->"PaVeReduce"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2085470, 67015, 42, 1, 39, "ObjectName",
  CellTags->"PD"],

Cell[CellGroupData[{
Cell[2085537, 67020, 54, 1, 49, "Subsubsection",
  CellTags->"PD"],
Cell[2085594, 67023, 82, 1, 39, "Text",
  CellTags->"PD"],
Cell[2085679, 67026, 333, 14, 39, "Text",
  CellTags->"PD"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2086061, 67046, 70, 1, 39, "ObjectName",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{
Cell[2086156, 67051, 68, 1, 49, "Subsubsection",
  CellTags->"PlusDistribution"],
Cell[2086227, 67054, 148, 4, 39, "Text",
  CellTags->"PlusDistribution"],
Cell[2086378, 67060, 206, 9, 39, "Text",
  CellTags->"PlusDistribution"]
}, Closed]],

Cell[CellGroupData[{
Cell[2086621, 67074, 65, 1, 49, "Subsubsection",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{
Cell[2086711, 67079, 241, 6, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2086955, 67087, 228, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2087220, 67099, 308, 8, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2087531, 67109, 303, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2087871, 67123, 398, 10, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2088272, 67135, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2088418, 67142, 473, 12, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2088894, 67156, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2089040, 67163, 510, 13, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2089553, 67178, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2089699, 67185, 376, 10, 70, "Input",
  CellTags->"PlusDistribution"],
Cell[2090078, 67197, 443, 13, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2090582, 67217, 62, 1, 39, "ObjectName",
  CellTags->"Polarization"],

Cell[CellGroupData[{
Cell[2090669, 67222, 64, 1, 49, "Subsubsection",
  CellTags->"Polarization"],
Cell[2090736, 67225, 770, 12, 107, "Text",
  CellTags->"Polarization"],
Cell[2091509, 67239, 517, 19, 39, "Text",
  CellTags->"Polarization"]
}, Closed]],

Cell[CellGroupData[{
Cell[2092063, 67263, 61, 1, 49, "Subsubsection",
  CellTags->"Polarization"],

Cell[CellGroupData[{
Cell[2092149, 67268, 146, 3, 70, "Input",
  CellTags->"Polarization"],
Cell[2092298, 67273, 443, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2092778, 67290, 192, 5, 70, "Input",
  CellTags->"Polarization"],
Cell[2092973, 67297, 172, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2093182, 67305, 196, 5, 70, "Input",
  CellTags->"Polarization"],
Cell[2093381, 67312, 649, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2094067, 67335, 237, 6, 70, "Input",
  CellTags->"Polarization"],
Cell[2094307, 67343, 196, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2094540, 67352, 187, 4, 70, "Input",
  CellTags->"Polarization"],
Cell[2094730, 67358, 652, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2095419, 67379, 235, 6, 70, "Input",
  CellTags->"Polarization"],
Cell[2095657, 67387, 260, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2095954, 67398, 364, 9, 70, "Input",
  CellTags->"Polarization"],
Cell[2096321, 67409, 109, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2096491, 67418, 68, 1, 39, "ObjectName",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{
Cell[2096584, 67423, 67, 1, 49, "Subsubsection",
  CellTags->"PolarizationSum"],
Cell[2096654, 67426, 280, 6, 56, "Text",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{
Cell[2096959, 67436, 158, 3, 56, "Input",
  CellTags->"PolarizationSum"],
Cell[2097120, 67441, 186, 5, 70, "Output"]
}, Closed]],
Cell[2097321, 67449, 211, 9, 35, "Text",
  CellTags->"PolarizationSum"]
}, Closed]],

Cell[CellGroupData[{
Cell[2097569, 67463, 64, 1, 49, "Subsubsection",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{
Cell[2097658, 67468, 187, 4, 56, "Input",
  CellTags->"PolarizationSum"],
Cell[2097848, 67474, 216, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2098101, 67485, 197, 4, 52, "Input",
  CellTags->"PolarizationSum"],
Cell[2098301, 67491, 1491, 43, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2099829, 67539, 207, 4, 52, "Input",
  CellTags->"PolarizationSum"],
Cell[2100039, 67545, 3887, 109, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2103963, 67659, 296, 7, 52, "Input",
  CellTags->"PolarizationSum"],
Cell[2104262, 67668, 9986, 268, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2114309, 67943, 82, 1, 39, "ObjectName",
  CellTags->"PolarizationUncontract"],

Cell[CellGroupData[{
Cell[2114416, 67948, 74, 1, 49, "Subsubsection",
  CellTags->"PolarizationUncontract"],
Cell[2114493, 67951, 153, 4, 39, "Text",
  CellTags->"PolarizationUncontract"],
Cell[2114649, 67957, 344, 14, 39, "Text",
  CellTags->"PolarizationUncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2115042, 67977, 74, 1, 39, "ObjectName",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[2115141, 67982, 70, 1, 49, "Subsubsection",
  CellTags->"PolarizationVector"],
Cell[2115214, 67985, 104, 1, 39, "Text",
  CellTags->"PolarizationVector"],
Cell[2115321, 67988, 441, 18, 39, "Text",
  CellTags->"PolarizationVector"]
}, Closed]],

Cell[CellGroupData[{
Cell[2115799, 68011, 67, 1, 49, "Subsubsection",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[2115891, 68016, 189, 4, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2116083, 68022, 431, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2116551, 68039, 231, 5, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2116785, 68046, 441, 12, 70, "Output"]
}, Closed]],
Cell[2117241, 68061, 265, 9, 70, "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[2117531, 68074, 227, 5, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2117761, 68081, 338, 8, 70, "Output"]
}, Closed]],
Cell[2118114, 68092, 547, 18, 70, "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[2118686, 68114, 338, 8, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2119027, 68124, 770, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2119834, 68151, 149, 3, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2119986, 68156, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2120132, 68163, 371, 9, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2120506, 68174, 815, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2121358, 68202, 149, 3, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2121510, 68207, 930, 25, 70, "Output"]
}, Closed]],
Cell[2122455, 68235, 1987, 53, 70, "Text",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[2124467, 68292, 371, 9, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2124841, 68303, 555, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2125433, 68323, 149, 3, 70, "Input",
  CellTags->"PolarizationVector"],
Cell[2125585, 68328, 930, 25, 70, "Output"]
}, Closed]],
Cell[2126530, 68356, 185, 4, 70, "Input",
  CellTags->"PolarizationVector"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2126764, 68366, 68, 1, 39, "ObjectName",
  CellTags->"PositiveInteger"],

Cell[CellGroupData[{
Cell[2126857, 68371, 67, 1, 49, "Subsubsection",
  CellTags->"PositiveInteger"],
Cell[2126927, 68374, 137, 4, 39, "Text",
  CellTags->"PositiveInteger"],
Cell[2127067, 68380, 199, 9, 39, "Text",
  CellTags->"PositiveInteger"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2127315, 68395, 66, 1, 39, "ObjectName",
  CellTags->"PositiveNumber"],

Cell[CellGroupData[{
Cell[2127406, 68400, 66, 1, 49, "Subsubsection",
  CellTags->"PositiveNumber"],
Cell[2127475, 68403, 147, 4, 39, "Text",
  CellTags->"PositiveNumber"],
Cell[2127625, 68409, 198, 9, 39, "Text",
  CellTags->"PositiveNumber"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2127872, 68424, 60, 1, 39, "ObjectName",
  CellTags->"PowerFactor"],

Cell[CellGroupData[{
Cell[2127957, 68429, 63, 1, 49, "Subsubsection",
  CellTags->"PowerFactor"],
Cell[2128023, 68432, 422, 16, 39, "Text",
  CellTags->"PowerFactor"],
Cell[2128448, 68450, 210, 9, 39, "Text",
  CellTags->"PowerFactor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2128695, 68464, 60, 1, 49, "Subsubsection",
  CellTags->"PowerFactor"],

Cell[CellGroupData[{
Cell[2128780, 68469, 178, 5, 56, "Input",
  CellTags->"PowerFactor"],
Cell[2128961, 68476, 194, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2129192, 68486, 142, 3, 52, "Input",
  CellTags->"PowerFactor"],
Cell[2129337, 68491, 195, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2129593, 68503, 64, 1, 39, "ObjectName",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{
Cell[2129682, 68508, 65, 1, 49, "Subsubsection",
  CellTags->"PowerSimplify"],
Cell[2129750, 68511, 2968, 112, 73, "Text",
  CellTags->"PowerSimplify"],
Cell[2132721, 68625, 305, 14, 39, "Text",
  CellTags->"PowerSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[2133063, 68644, 62, 1, 49, "Subsubsection",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{
Cell[2133150, 68649, 288, 8, 56, "Input",
  CellTags->"PowerSimplify"],
Cell[2133441, 68659, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2133587, 68666, 293, 8, 52, "Input",
  CellTags->"PowerSimplify"],
Cell[2133883, 68676, 192, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2134112, 68686, 293, 8, 52, "Input",
  CellTags->"PowerSimplify"],
Cell[2134408, 68696, 192, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2134637, 68706, 233, 6, 52, "Input",
  CellTags->"PowerSimplify"],
Cell[2134873, 68714, 192, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2135126, 68726, 50, 1, 39, "ObjectName",
  CellTags->"Power2"],

Cell[CellGroupData[{
Cell[2135201, 68731, 58, 1, 49, "Subsubsection",
  CellTags->"Power2"],
Cell[2135262, 68734, 214, 5, 56, "Text",
  CellTags->"Power2"],
Cell[2135479, 68741, 199, 9, 39, "Text",
  CellTags->"Power2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2135715, 68755, 55, 1, 49, "Subsubsection",
  CellTags->"Power2"],

Cell[CellGroupData[{
Cell[2135795, 68760, 182, 5, 56, "Input",
  CellTags->"Power2"],
Cell[2135980, 68767, 190, 5, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2136207, 68777, 183, 5, 52, "Input",
  CellTags->"Power2"],
Cell[2136393, 68784, 302, 9, 38, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2136756, 68800, 68, 1, 39, "ObjectName",
  CellTags->"PostFortranFile"],

Cell[CellGroupData[{
Cell[2136849, 68805, 67, 1, 49, "Subsubsection",
  CellTags->"PostFortranFile"],
Cell[2136919, 68808, 230, 5, 56, "Text",
  CellTags->"PostFortranFile"],
Cell[2137152, 68815, 331, 14, 39, "Text",
  CellTags->"PostFortranFile"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2137532, 68835, 56, 1, 39, "ObjectName",
  CellTags->"Prefactor"],

Cell[CellGroupData[{
Cell[2137613, 68840, 61, 1, 49, "Subsubsection",
  CellTags->"Prefactor"],
Cell[2137677, 68843, 291, 6, 56, "Text",
  CellTags->"Prefactor"],
Cell[2137971, 68851, 316, 14, 39, "Text",
  CellTags->"Prefactor"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2138336, 68871, 66, 1, 39, "ObjectName",
  CellTags->"PreFortranFile"],

Cell[CellGroupData[{
Cell[2138427, 68876, 66, 1, 49, "Subsubsection",
  CellTags->"PreFortranFile"],
Cell[2138496, 68879, 233, 5, 56, "Text",
  CellTags->"PreFortranFile"],
Cell[2138732, 68886, 333, 14, 39, "Text",
  CellTags->"PreFortranFile"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2139114, 68906, 80, 1, 39, "ObjectName",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{
Cell[2139219, 68911, 73, 1, 49, "Subsubsection",
  CellTags->"PropagatorDenominator"],
Cell[2139295, 68914, 569, 15, 56, "Text",
  CellTags->"PropagatorDenominator"],
Cell[2139867, 68931, 526, 19, 39, "Text",
  CellTags->"PropagatorDenominator"]
}, Closed]],

Cell[CellGroupData[{
Cell[2140430, 68955, 70, 1, 49, "Subsubsection",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{
Cell[2140525, 68960, 215, 5, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2140743, 68967, 227, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2141007, 68978, 164, 3, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2141174, 68983, 164, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2141375, 68992, 247, 6, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2141625, 69000, 227, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2141889, 69011, 190, 4, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2142082, 69017, 241, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2142360, 69027, 231, 5, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2142594, 69034, 241, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2142872, 69044, 173, 3, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2143048, 69049, 282, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2143367, 69062, 155, 3, 70, "Input",
  CellTags->"PropagatorDenominator"],
Cell[2143525, 69067, 356, 9, 70, "Output"]
}, Closed]],
Cell[2143896, 69079, 149, 3, 70, "Input",
  CellTags->"PropagatorDenominator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2144094, 69088, 96, 1, 39, "ObjectName",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{
Cell[2144215, 69093, 81, 1, 49, "Subsubsection",
  CellTags->"PropagatorDenominatorExplicit"],
Cell[2144299, 69096, 244, 5, 56, "Text",
  CellTags->"PropagatorDenominatorExplicit"],
Cell[2144546, 69103, 402, 14, 39, "Text",
  CellTags->"PropagatorDenominatorExplicit"]
}, Closed]],

Cell[CellGroupData[{
Cell[2144985, 69122, 78, 1, 49, "Subsubsection",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{
Cell[2145088, 69127, 383, 11, 70, "Input",
  CellTags->"PropagatorDenominatorExplicit"],
Cell[2145474, 69140, 406, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2145917, 69157, 217, 5, 70, "Input",
  CellTags->"PropagatorDenominatorExplicit"],
Cell[2146137, 69164, 1168, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2147342, 69202, 159, 3, 70, "Input",
  CellTags->"PropagatorDenominatorExplicit"],
Cell[2147504, 69207, 673, 18, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2148238, 69232, 44, 1, 39, "ObjectName",
  CellTags->"QGV"],

Cell[CellGroupData[{
Cell[2148307, 69237, 55, 1, 49, "Subsubsection",
  CellTags->"QGV"],
Cell[2148365, 69240, 73, 1, 39, "Text",
  CellTags->"QGV"],
Cell[2148441, 69243, 211, 9, 39, "Text",
  CellTags->"QGV"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2148701, 69258, 42, 1, 39, "ObjectName",
  CellTags->"QO"],

Cell[CellGroupData[{
Cell[2148768, 69263, 54, 1, 49, "Subsubsection",
  CellTags->"QO"],
Cell[2148825, 69266, 74, 1, 39, "Text",
  CellTags->"QO"],
Cell[2148902, 69269, 219, 9, 39, "Text",
  CellTags->"QO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2149170, 69284, 42, 1, 39, "ObjectName",
  CellTags->"QP"],

Cell[CellGroupData[{
Cell[2149237, 69289, 54, 1, 49, "Subsubsection",
  CellTags->"QP"],
Cell[2149294, 69292, 170, 4, 39, "Text",
  CellTags->"QP"],
Cell[2149467, 69298, 207, 9, 39, "Text",
  CellTags->"QP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2149723, 69313, 62, 1, 39, "ObjectName",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2149810, 69318, 64, 1, 49, "Subsubsection",
  CellTags->"QuantumField"],
Cell[2149877, 69321, 410, 9, 56, "Text",
  CellTags->"QuantumField"],
Cell[2150290, 69332, 454, 19, 39, "Text",
  CellTags->"QuantumField"]
}, Closed]],

Cell[CellGroupData[{
Cell[2150781, 69356, 61, 1, 49, "Subsubsection",
  CellTags->"QuantumField"],
Cell[2150845, 69359, 72, 1, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2150942, 69364, 146, 3, 70, "Input",
  CellTags->"QuantumField"],
Cell[2151091, 69369, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2151237, 69376, 159, 3, 70, "Input",
  CellTags->"QuantumField"],
Cell[2151399, 69381, 141, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2151577, 69389, 155, 3, 70, "Input",
  CellTags->"QuantumField"],
Cell[2151735, 69394, 114, 2, 70, "Output"]
}, Closed]],
Cell[2151864, 69399, 81, 1, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2151970, 69404, 206, 5, 70, "Input",
  CellTags->"QuantumField"],
Cell[2152179, 69411, 139, 3, 70, "Output"]
}, Closed]],
Cell[2152333, 69417, 95, 1, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2152453, 69422, 254, 6, 70, "Input",
  CellTags->"QuantumField"],
Cell[2152710, 69430, 392, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2153139, 69448, 142, 3, 70, "Input",
  CellTags->"QuantumField"],
Cell[2153284, 69453, 269, 5, 70, "Output"]
}, Closed]],
Cell[2153568, 69461, 409, 14, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2154002, 69479, 248, 6, 70, "Input",
  CellTags->"QuantumField"],
Cell[2154253, 69487, 304, 9, 70, "Output"]
}, Closed]],
Cell[2154572, 69499, 188, 5, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2154785, 69508, 293, 7, 70, "Input",
  CellTags->"QuantumField"],
Cell[2155081, 69517, 259, 5, 70, "Output"]
}, Closed]],
Cell[2155355, 69525, 89, 1, 70, "Text",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[2155469, 69530, 261, 6, 70, "Input",
  CellTags->"QuantumField"],
Cell[2155733, 69538, 433, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2156203, 69556, 228, 6, 70, "Input",
  CellTags->"QuantumField"],
Cell[2156434, 69564, 291, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2156762, 69577, 306, 7, 70, "Input",
  CellTags->"QuantumField"],
Cell[2157071, 69586, 496, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2157604, 69606, 349, 9, 70, "Input",
  CellTags->"QuantumField"],
Cell[2157956, 69617, 504, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2158497, 69637, 273, 6, 70, "Input",
  CellTags->"QuantumField"],
Cell[2158773, 69645, 112, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2158946, 69654, 58, 1, 39, "ObjectName",
  CellTags->"QuarkField"],

Cell[CellGroupData[{
Cell[2159029, 69659, 62, 1, 49, "Subsubsection",
  CellTags->"QuarkField"],
Cell[2159094, 69662, 192, 5, 39, "Text",
  CellTags->"QuarkField"],
Cell[2159289, 69669, 344, 14, 39, "Text",
  CellTags->"QuarkField"]
}, Closed]],

Cell[CellGroupData[{
Cell[2159670, 69688, 59, 1, 49, "Subsubsection",
  CellTags->"QuarkField"],

Cell[CellGroupData[{
Cell[2159754, 69693, 112, 2, 70, "Input",
  CellTags->"QuarkField"],
Cell[2159869, 69697, 114, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2160044, 69706, 70, 1, 39, "ObjectName",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{
Cell[2160139, 69711, 68, 1, 49, "Subsubsection",
  CellTags->"QuarkGluonVertex"],
Cell[2160210, 69714, 286, 9, 39, "Text",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{
Cell[2160521, 69727, 160, 3, 56, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2160684, 69732, 485, 11, 70, "Output"]
}, Closed]],
Cell[2161184, 69746, 209, 9, 35, "Text",
  CellTags->"QuarkGluonVertex"]
}, Closed]],

Cell[CellGroupData[{
Cell[2161430, 69760, 65, 1, 49, "Subsubsection",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{
Cell[2161520, 69765, 185, 4, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2161708, 69771, 516, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2162261, 69791, 245, 5, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2162509, 69798, 862, 24, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2163408, 69827, 245, 5, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2163656, 69834, 725, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2164418, 69859, 245, 5, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2164666, 69866, 827, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2165530, 69894, 373, 9, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2165906, 69905, 2982, 75, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2168925, 69985, 374, 9, 70, "Input",
  CellTags->"QuarkGluonVertex"],
Cell[2169302, 69996, 516, 15, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2169879, 70018, 56, 1, 39, "ObjectName",
  CellTags->"QuarkMass"],

Cell[CellGroupData[{
Cell[2169960, 70023, 61, 1, 49, "Subsubsection",
  CellTags->"QuarkMass"],
Cell[2170024, 70026, 77, 1, 39, "Text",
  CellTags->"QuarkMass"],
Cell[2170104, 70029, 196, 9, 39, "Text",
  CellTags->"QuarkMass"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2170349, 70044, 68, 1, 39, "ObjectName",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{
Cell[2170442, 70049, 67, 1, 49, "Subsubsection",
  CellTags->"QuarkPropagator"],
Cell[2170512, 70052, 177, 5, 39, "Text",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{
Cell[2170714, 70061, 158, 3, 56, "Input",
  CellTags->"QuarkPropagator"],
Cell[2170875, 70066, 535, 12, 70, "Output"]
}, Closed]],
Cell[2171425, 70081, 364, 14, 35, "Text",
  CellTags->"QuarkPropagator"]
}, Closed]],

Cell[CellGroupData[{
Cell[2171826, 70100, 64, 1, 49, "Subsubsection",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{
Cell[2171915, 70105, 152, 3, 70, "Input",
  CellTags->"QuarkPropagator"],
Cell[2172070, 70110, 413, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2172520, 70126, 208, 5, 70, "Input",
  CellTags->"QuarkPropagator"],
Cell[2172731, 70133, 564, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2173332, 70153, 189, 4, 70, "Input",
  CellTags->"QuarkPropagator"],
Cell[2173524, 70159, 413, 11, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2173998, 70177, 60, 1, 39, "ObjectName",
  CellTags->"ReduceGamma"],

Cell[CellGroupData[{
Cell[2174083, 70182, 63, 1, 49, "Subsubsection",
  CellTags->"ReduceGamma"],
Cell[2174149, 70185, 197, 5, 39, "Text",
  CellTags->"ReduceGamma"],
Cell[2174349, 70192, 192, 9, 39, "Text",
  CellTags->"ReduceGamma"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2174590, 70207, 68, 1, 39, "ObjectName",
  CellTags->"ReduceToScalars"],

Cell[CellGroupData[{
Cell[2174683, 70212, 67, 1, 49, "Subsubsection",
  CellTags->"ReduceToScalars"],
Cell[2174753, 70215, 211, 5, 56, "Text",
  CellTags->"ReduceToScalars"],
Cell[2174967, 70222, 322, 14, 39, "Text",
  CellTags->"ReduceToScalars"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2175338, 70242, 50, 1, 39, "ObjectName",
  CellTags->"Rename"],

Cell[CellGroupData[{
Cell[2175413, 70247, 58, 1, 49, "Subsubsection",
  CellTags->"Rename"],
Cell[2175474, 70250, 145, 4, 39, "Text",
  CellTags->"Rename"],
Cell[2175622, 70256, 190, 9, 39, "Text",
  CellTags->"Rename"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2175861, 70271, 62, 1, 39, "ObjectName",
  CellTags->"RHI"],

Cell[CellGroupData[{
Cell[2175948, 70276, 55, 1, 49, "Subsubsection",
  CellTags->"RHI"],
Cell[2176006, 70279, 612, 12, 156, "Text",
  CellTags->"RHI"],
Cell[2176621, 70293, 181, 9, 39, "Text",
  CellTags->"RHI"]
}, Closed]],

Cell[CellGroupData[{
Cell[2176839, 70307, 52, 1, 49, "Subsubsection",
  CellTags->"RHI"],
Cell[2176894, 70310, 95, 2, 56, "Input",
  CellTags->"RHI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2177038, 70318, 68, 1, 39, "ObjectName",
  CellTags->"RHI2FC"],

Cell[CellGroupData[{
Cell[2177131, 70323, 58, 1, 49, "Subsubsection",
  CellTags->"RHI2FC"],
Cell[2177192, 70326, 615, 12, 156, "Text",
  CellTags->"RHI2FC"],
Cell[2177810, 70340, 184, 9, 39, "Text",
  CellTags->"RHI2FC"]
}, Closed]],

Cell[CellGroupData[{
Cell[2178031, 70354, 55, 1, 49, "Subsubsection",
  CellTags->"RHI2FC"],
Cell[2178089, 70357, 98, 2, 56, "Input",
  CellTags->"RHI2FC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2178236, 70365, 62, 1, 39, "ObjectName",
  CellTags->"RHM"],

Cell[CellGroupData[{
Cell[2178323, 70370, 55, 1, 49, "Subsubsection",
  CellTags->"RHM"],
Cell[2178381, 70373, 78, 1, 39, "Text",
  CellTags->"RHM"],
Cell[2178462, 70376, 172, 9, 39, "Text",
  CellTags->"RHM"]
}, Closed]],

Cell[CellGroupData[{
Cell[2178671, 70390, 52, 1, 49, "Subsubsection",
  CellTags->"RHM"],
Cell[2178726, 70393, 95, 2, 56, "Input",
  CellTags->"RHM"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2178870, 70401, 62, 1, 39, "ObjectName",
  CellTags->"RHO"],

Cell[CellGroupData[{
Cell[2178957, 70406, 55, 1, 49, "Subsubsection",
  CellTags->"RHO"],
Cell[2179015, 70409, 270, 6, 56, "Text",
  CellTags->"RHO"],
Cell[2179288, 70417, 172, 9, 39, "Text",
  CellTags->"RHO"]
}, Closed]],

Cell[CellGroupData[{
Cell[2179497, 70431, 52, 1, 49, "Subsubsection",
  CellTags->"RHO"],
Cell[2179552, 70434, 95, 2, 56, "Input",
  CellTags->"RHO"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2179696, 70442, 62, 1, 39, "ObjectName",
  CellTags->"RHP"],

Cell[CellGroupData[{
Cell[2179783, 70447, 55, 1, 49, "Subsubsection",
  CellTags->"RHP"],
Cell[2179841, 70450, 211, 5, 56, "Text",
  CellTags->"RHP"],
Cell[2180055, 70457, 172, 9, 39, "Text",
  CellTags->"RHP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2180264, 70471, 52, 1, 49, "Subsubsection",
  CellTags->"RHP"],
Cell[2180319, 70474, 95, 2, 56, "Input",
  CellTags->"RHP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2180463, 70482, 62, 1, 39, "ObjectName",
  CellTags->"RTL"],

Cell[CellGroupData[{
Cell[2180550, 70487, 55, 1, 49, "Subsubsection",
  CellTags->"RTL"],
Cell[2180608, 70490, 191, 5, 39, "Text",
  CellTags->"RTL"],
Cell[2180802, 70497, 289, 14, 39, "Text",
  CellTags->"RTL"]
}, Closed]],

Cell[CellGroupData[{
Cell[2181128, 70516, 52, 1, 49, "Subsubsection",
  CellTags->"RTL"],
Cell[2181183, 70519, 95, 2, 56, "Input",
  CellTags->"RTL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2181327, 70527, 64, 1, 39, "ObjectName",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{
Cell[2181416, 70532, 65, 1, 49, "Subsubsection",
  CellTags->"RightPartialD"],
Cell[2181484, 70535, 275, 10, 40, "Text",
  CellTags->"RightPartialD"],
Cell[2181762, 70547, 467, 19, 39, "Text",
  CellTags->"RightPartialD"]
}, Closed]],

Cell[CellGroupData[{
Cell[2182266, 70571, 62, 1, 49, "Subsubsection",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{
Cell[2182353, 70576, 152, 3, 70, "Input",
  CellTags->"RightPartialD"],
Cell[2182508, 70581, 288, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2182833, 70594, 302, 7, 70, "Input",
  CellTags->"RightPartialD"],
Cell[2183138, 70603, 359, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2183534, 70618, 149, 3, 70, "Input",
  CellTags->"RightPartialD"],
Cell[2183686, 70623, 433, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2184156, 70641, 143, 3, 70, "Input",
  CellTags->"RightPartialD"],
Cell[2184302, 70646, 311, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2184650, 70657, 198, 5, 70, "Input",
  CellTags->"RightPartialD"],
Cell[2184851, 70664, 181, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2185093, 70674, 62, 1, 39, "ObjectName",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{
Cell[2185180, 70679, 64, 1, 49, "Subsubsection",
  CellTags->"RussianTrick"],
Cell[2185247, 70682, 319, 6, 56, "Text",
  CellTags->"RussianTrick"],
Cell[2185569, 70690, 91, 1, 39, "Text",
  CellTags->"RussianTrick"],
Cell[2185663, 70693, 349, 14, 39, "Text",
  CellTags->"RussianTrick"]
}, Closed]],

Cell[CellGroupData[{
Cell[2186049, 70712, 61, 1, 49, "Subsubsection",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{
Cell[2186135, 70717, 469, 13, 70, "Input",
  CellTags->"RussianTrick"],
Cell[2186607, 70732, 619, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2187263, 70757, 136, 3, 70, "Input",
  CellTags->"RussianTrick"],
Cell[2187402, 70762, 1584, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2189023, 70812, 209, 5, 70, "Input",
  CellTags->"RussianTrick"],
Cell[2189235, 70819, 9151, 244, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2198423, 71068, 140, 3, 70, "Input",
  CellTags->"RussianTrick"],
Cell[2198566, 71073, 3600, 110, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2202203, 71188, 167, 4, 70, "Input",
  CellTags->"RussianTrick"],
Cell[2202373, 71194, 4131, 111, 70, "Output"]
}, Closed]],
Cell[2206519, 71308, 139, 3, 70, "Input",
  CellTags->"RussianTrick"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2206707, 71317, 72, 1, 39, "ObjectName",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{
Cell[2206804, 71322, 69, 1, 49, "Subsubsection",
  CellTags->"ScalarGluonVertex"],
Cell[2206876, 71325, 431, 8, 89, "Text",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{
Cell[2207332, 71337, 159, 3, 56, "Input",
  CellTags->"ScalarGluonVertex"],
Cell[2207494, 71342, 291, 7, 37, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2207834, 71355, 66, 1, 49, "Subsubsection",
  CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{
Cell[2207925, 71360, 324, 8, 56, "Input",
  CellTags->"ScalarGluonVertex"],
Cell[2208252, 71370, 593, 18, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2208906, 71395, 64, 1, 39, "ObjectName",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{
Cell[2208995, 71400, 65, 1, 49, "Subsubsection",
  CellTags->"ScalarProduct"],
Cell[2209063, 71403, 875, 21, 90, "Text",
  CellTags->"ScalarProduct"],
Cell[2209941, 71426, 952, 38, 39, "Text",
  CellTags->"ScalarProduct"]
}, Closed]],

Cell[CellGroupData[{
Cell[2210930, 71469, 62, 1, 49, "Subsubsection",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{
Cell[2211017, 71474, 175, 4, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2211195, 71480, 269, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2211501, 71492, 228, 6, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2211732, 71500, 433, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2212202, 71517, 175, 4, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2212380, 71523, 184, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2212601, 71533, 148, 3, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2212752, 71538, 184, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2212973, 71548, 216, 5, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2213192, 71555, 234, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2213463, 71565, 278, 7, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2213744, 71574, 296, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2214077, 71586, 301, 8, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2214381, 71596, 134, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2214552, 71604, 358, 9, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2214913, 71615, 1175, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2216125, 71653, 343, 9, 70, "Input",
  CellTags->"ScalarProduct"],
Cell[2216471, 71664, 1175, 33, 70, "Output"]
}, Closed]],
Cell[2217661, 71700, 124, 2, 70, "Input",
  CellTags->"ScalarProduct"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2217834, 71708, 76, 1, 39, "ObjectName",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{
Cell[2217935, 71713, 71, 1, 49, "Subsubsection",
  CellTags->"ScalarProductCancel"],
Cell[2218009, 71716, 187, 4, 39, "Text",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{
Cell[2218221, 71724, 114, 2, 56, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2218338, 71728, 162, 3, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2218537, 71736, 162, 3, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2218702, 71741, 457, 10, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2219196, 71756, 134, 3, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2219333, 71761, 154, 3, 70, "Print",
  CellTags->"ScalarProductCancel"]
}, Closed]],
Cell[2219502, 71767, 805, 33, 35, "Text",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2220344, 71805, 68, 1, 49, "Subsubsection",
  CellTags->"ScalarProductCancel"],
Cell[2220415, 71808, 257, 9, 39, "Text",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{
Cell[2220697, 71821, 541, 15, 56, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2221241, 71838, 611, 18, 54, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2221889, 71861, 188, 4, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2222080, 71867, 1126, 33, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2223243, 71905, 194, 4, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2223440, 71911, 916, 27, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2224393, 71943, 222, 5, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2224618, 71950, 916, 27, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2225571, 71982, 714, 19, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2226288, 72003, 2080, 60, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],

Cell[CellGroupData[{
Cell[2228405, 72068, 234, 5, 52, "Input",
  CellTags->"ScalarProductCancel"],
Cell[2228642, 72075, 6568, 176, 70, "Output",
  CellTags->"ScalarProductCancel"]
}, Closed]],
Cell[2235225, 72254, 199, 5, 52, "Input",
  CellTags->"ScalarProductCancel"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2235473, 72265, 52, 1, 39, "ObjectName",
  CellTags->"ScaleMu"],

Cell[CellGroupData[{
Cell[2235550, 72270, 59, 1, 49, "Subsubsection",
  CellTags->"ScaleMu"],
Cell[2235612, 72273, 129, 4, 39, "Text",
  CellTags->"ScaleMu"],

Cell[CellGroupData[{
Cell[2235766, 72281, 104, 2, 56, "Input",
  CellTags->"ScaleMu"],
Cell[2235873, 72285, 134, 3, 37, "Output",
  CellTags->"ScaleMu"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2236068, 72295, 54, 1, 39, "ObjectName",
  CellTags->"Schouten"],

Cell[CellGroupData[{
Cell[2236147, 72300, 60, 1, 49, "Subsubsection",
  CellTags->"Schouten"],
Cell[2236210, 72303, 463, 9, 98, "Text",
  CellTags->"Schouten"],
Cell[2236676, 72314, 311, 13, 39, "Text",
  CellTags->"Schouten"]
}, Closed]],

Cell[CellGroupData[{
Cell[2237024, 72332, 57, 1, 49, "Subsubsection",
  CellTags->"Schouten"],

Cell[CellGroupData[{
Cell[2237106, 72337, 2467, 54, 169, "Input",
  CellTags->"Schouten"],
Cell[2239576, 72393, 28973, 800, 201, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2268586, 73198, 131, 3, 52, "Input",
  CellTags->"Schouten"],
Cell[2268720, 73203, 106, 2, 37, "Output"]
}, Closed]],
Cell[2268841, 73208, 108, 2, 39, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2268998, 73216, 42, 1, 39, "ObjectName",
  CellTags->"SD"],

Cell[CellGroupData[{
Cell[2269065, 73221, 54, 1, 49, "Subsubsection",
  CellTags->"SD"],
Cell[2269122, 73224, 271, 7, 56, "Text",
  CellTags->"SD"],
Cell[2269396, 73233, 186, 9, 39, "Text",
  CellTags->"SD"]
}, Closed]],

Cell[CellGroupData[{
Cell[2269619, 73247, 51, 1, 49, "Subsubsection",
  CellTags->"SD"],

Cell[CellGroupData[{
Cell[2269695, 73252, 153, 4, 70, "Input",
  CellTags->"SD"],
Cell[2269851, 73258, 180, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2270068, 73267, 162, 4, 70, "Input",
  CellTags->"SD"],
Cell[2270233, 73273, 238, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2270508, 73283, 162, 4, 70, "Input",
  CellTags->"SD"],
Cell[2270673, 73289, 150, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2270884, 73299, 58, 1, 39, "ObjectName",
  CellTags->"SelectFree"],

Cell[CellGroupData[{
Cell[2270967, 73304, 62, 1, 49, "Subsubsection",
  CellTags->"SelectFree"],
Cell[2271032, 73307, 256, 5, 56, "Text",
  CellTags->"SelectFree"],
Cell[2271291, 73314, 78, 1, 39, "Text",
  CellTags->"SelectFree"],
Cell[2271372, 73317, 323, 14, 39, "Text",
  CellTags->"SelectFree"]
}, Closed]],

Cell[CellGroupData[{
Cell[2271732, 73336, 59, 1, 49, "Subsubsection",
  CellTags->"SelectFree"],

Cell[CellGroupData[{
Cell[2271816, 73341, 261, 7, 70, "Input",
  CellTags->"SelectFree"],
Cell[2272080, 73350, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2272253, 73358, 203, 5, 70, "Input",
  CellTags->"SelectFree"],
Cell[2272459, 73365, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2272605, 73372, 329, 8, 70, "Input",
  CellTags->"SelectFree"],
Cell[2272937, 73382, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2273110, 73390, 169, 4, 70, "Input",
  CellTags->"SelectFree"],
Cell[2273282, 73396, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2273428, 73403, 169, 4, 70, "Input",
  CellTags->"SelectFree"],
Cell[2273600, 73409, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2273746, 73416, 169, 4, 70, "Input",
  CellTags->"SelectFree"],
Cell[2273918, 73422, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2274064, 73429, 203, 5, 70, "Input",
  CellTags->"SelectFree"],
Cell[2274270, 73436, 109, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2274440, 73445, 62, 1, 39, "ObjectName",
  CellTags->"SelectGraphs"],

Cell[CellGroupData[{
Cell[2274527, 73450, 64, 1, 49, "Subsubsection",
  CellTags->"SelectGraphs"],
Cell[2274594, 73453, 489, 9, 73, "Text",
  CellTags->"SelectGraphs"],
Cell[2275086, 73464, 202, 9, 39, "Text",
  CellTags->"SelectGraphs"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2275337, 73479, 64, 1, 39, "ObjectName",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{
Cell[2275426, 73484, 65, 1, 49, "Subsubsection",
  CellTags->"SelectNotFree"],
Cell[2275494, 73487, 144, 4, 39, "Text",
  CellTags->"SelectNotFree"],
Cell[2275641, 73493, 273, 6, 56, "Text",
  CellTags->"SelectNotFree"],
Cell[2275917, 73501, 84, 1, 39, "Text",
  CellTags->"SelectNotFree"],
Cell[2276004, 73504, 317, 14, 39, "Text",
  CellTags->"SelectNotFree"]
}, Closed]],

Cell[CellGroupData[{
Cell[2276358, 73523, 62, 1, 49, "Subsubsection",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{
Cell[2276445, 73528, 250, 6, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2276698, 73536, 170, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2276905, 73545, 330, 8, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2277238, 73555, 180, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2277455, 73564, 175, 4, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2277633, 73570, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2277779, 73577, 204, 5, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2277986, 73584, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2278132, 73591, 175, 4, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2278310, 73597, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2278456, 73604, 175, 4, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2278634, 73610, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2278780, 73617, 209, 5, 70, "Input",
  CellTags->"SelectNotFree"],
Cell[2278992, 73624, 141, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2279194, 73634, 60, 1, 39, "ObjectName",
  CellTags->"SelectSplit"],

Cell[CellGroupData[{
Cell[2279279, 73639, 63, 1, 49, "Subsubsection",
  CellTags->"SelectSplit"],
Cell[2279345, 73642, 250, 5, 56, "Text",
  CellTags->"SelectSplit"]
}, Closed]],

Cell[CellGroupData[{
Cell[2279632, 73652, 60, 1, 49, "Subsubsection",
  CellTags->"SelectSplit"],

Cell[CellGroupData[{
Cell[2279717, 73657, 1004, 27, 57, "Input",
  CellTags->"SelectSplit"],
Cell[2280724, 73686, 637, 18, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2281398, 73709, 808, 22, 53, "Input",
  CellTags->"SelectSplit"],
Cell[2282209, 73733, 583, 15, 39, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2282853, 73755, 52, 1, 39, "ObjectName",
  CellTags->"Select1"],

Cell[CellGroupData[{
Cell[2282930, 73760, 59, 1, 49, "Subsubsection",
  CellTags->"Select1"],
Cell[2282992, 73763, 244, 5, 56, "Text",
  CellTags->"Select1"],
Cell[2283239, 73770, 188, 9, 39, "Text",
  CellTags->"Select1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2283476, 73785, 52, 1, 39, "ObjectName",
  CellTags->"Select2"],

Cell[CellGroupData[{
Cell[2283553, 73790, 59, 1, 49, "Subsubsection",
  CellTags->"Select2"],
Cell[2283615, 73793, 246, 5, 56, "Text",
  CellTags->"Select2"],
Cell[2283864, 73800, 188, 9, 39, "Text",
  CellTags->"Select2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2284101, 73815, 52, 1, 39, "ObjectName",
  CellTags->"Series2"],

Cell[CellGroupData[{
Cell[2284178, 73820, 59, 1, 49, "Subsubsection",
  CellTags->"Series2"],
Cell[2284240, 73823, 293, 6, 56, "Text",
  CellTags->"Series2"],

Cell[CellGroupData[{
Cell[2284558, 73833, 142, 3, 56, "Input",
  CellTags->"Series2"],
Cell[2284703, 73838, 535, 13, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2285287, 73857, 56, 1, 49, "Subsubsection",
  CellTags->"Series2"],

Cell[CellGroupData[{
Cell[2285368, 73862, 425, 12, 70, "Input",
  CellTags->"Series2"],
Cell[2285796, 73876, 363, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2286196, 73891, 218, 6, 70, "Input",
  CellTags->"Series2"],
Cell[2286417, 73899, 260, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2286714, 73911, 270, 7, 70, "Input",
  CellTags->"Series2"],
Cell[2286987, 73920, 384, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2287408, 73935, 218, 6, 70, "Input",
  CellTags->"Series2"],
Cell[2287629, 73943, 563, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2288229, 73964, 346, 8, 70, "Input",
  CellTags->"Series2"],
Cell[2288578, 73974, 1381, 36, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2289996, 74015, 369, 10, 70, "Input",
  CellTags->"Series2"],
Cell[2290368, 74027, 1292, 35, 70, "Output"]
}, Closed]],
Cell[2291675, 74065, 116, 4, 70, "Text",
  CellTags->"Series2"],

Cell[CellGroupData[{
Cell[2291816, 74073, 672, 18, 70, "Input",
  CellTags->"Series2"],
Cell[2292491, 74093, 698, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2293226, 74118, 799, 20, 70, "Input",
  CellTags->"Series2"],
Cell[2294028, 74140, 1404, 40, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2295469, 74185, 285, 6, 70, "Input",
  CellTags->"Series2"],
Cell[2295757, 74193, 1798, 48, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2297592, 74246, 239, 6, 70, "Input",
  CellTags->"Series2"],
Cell[2297834, 74254, 2793, 71, 70, "Output"]
}, Closed]],
Cell[2300642, 74328, 355, 10, 70, "Text",
  CellTags->"Series2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2301046, 74344, 52, 1, 39, "ObjectName",
  CellTags->"Series3"],

Cell[CellGroupData[{
Cell[2301123, 74349, 59, 1, 49, "Subsubsection",
  CellTags->"Series3"],
Cell[2301185, 74352, 270, 6, 56, "Text",
  CellTags->"Series3"],

Cell[CellGroupData[{
Cell[2301480, 74362, 140, 3, 56, "Input",
  CellTags->"Series3"],
Cell[2301623, 74367, 316, 8, 37, "Output",
  CellTags->"Series3"]
}, Closed]],
Cell[2301954, 74378, 188, 9, 35, "Text",
  CellTags->"Series3"]
}, Closed]],

Cell[CellGroupData[{
Cell[2302179, 74392, 56, 1, 49, "Subsubsection",
  CellTags->"Series3"],

Cell[CellGroupData[{
Cell[2302260, 74397, 423, 12, 56, "Input",
  CellTags->"Series3"],
Cell[2302686, 74411, 384, 11, 53, "Output",
  CellTags->"Series3"]
}, Closed]],

Cell[CellGroupData[{
Cell[2303107, 74427, 259, 7, 52, "Input",
  CellTags->"Series3"],
Cell[2303369, 74436, 155, 4, 53, "Output",
  CellTags->"Series3"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2303585, 74447, 64, 1, 39, "ObjectName",
  CellTags->"SetMandelstam"],

Cell[CellGroupData[{
Cell[2303674, 74452, 65, 1, 49, "Subsubsection",
  CellTags->"SetMandelstam"],
Cell[2303742, 74455, 463, 9, 98, "Text",
  CellTags->"SetMandelstam"],

Cell[CellGroupData[{
Cell[2304230, 74468, 152, 3, 56, "Input",
  CellTags->"SetMandelstam"],
Cell[2304385, 74473, 277, 8, 37, "Output",
  CellTags->"SetMandelstam"]
}, Closed]],
Cell[2304677, 74484, 203, 9, 35, "Text",
  CellTags->"SetMandelstam"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2304929, 74499, 80, 1, 39, "ObjectName",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{
Cell[2305034, 74504, 73, 1, 49, "Subsubsection",
  CellTags->"SimplifyDeltaFunction"],
Cell[2305110, 74507, 300, 6, 56, "Text",
  CellTags->"SimplifyDeltaFunction"],
Cell[2305413, 74515, 370, 14, 39, "Text",
  CellTags->"SimplifyDeltaFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2305820, 74534, 70, 1, 49, "Subsubsection",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{
Cell[2305915, 74539, 244, 6, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2306162, 74547, 295, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2306494, 74560, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2306693, 74566, 454, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2307184, 74584, 244, 6, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2307431, 74592, 334, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2307802, 74606, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2308001, 74612, 988, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2309026, 74644, 261, 6, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2309290, 74652, 346, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2309673, 74666, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2309872, 74672, 226, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2310135, 74683, 315, 8, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2310453, 74693, 396, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2310886, 74709, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2311085, 74715, 256, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2311378, 74727, 354, 9, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2311735, 74738, 442, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2312214, 74755, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2312413, 74761, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2312559, 74768, 315, 8, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2312877, 74778, 396, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2313310, 74794, 196, 4, 70, "Input",
  CellTags->"SimplifyDeltaFunction"],
Cell[2313509, 74800, 256, 7, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2313826, 74814, 78, 1, 39, "ObjectName",
  CellTags->"SimplifyGTI"],

Cell[CellGroupData[{
Cell[2313929, 74819, 63, 1, 49, "Subsubsection",
  CellTags->"SimplifyGTI"],
Cell[2313995, 74822, 72, 1, 39, "Text",
  CellTags->"SimplifyGTI"],
Cell[2314070, 74825, 180, 9, 39, "Text",
  CellTags->"SimplifyGTI"]
}, Closed]],

Cell[CellGroupData[{
Cell[2314287, 74839, 60, 1, 49, "Subsubsection",
  CellTags->"SimplifyGTI"],
Cell[2314350, 74842, 100, 2, 56, "Input",
  CellTags->"SimplifyGTI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2314499, 74850, 68, 1, 39, "ObjectName",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{
Cell[2314592, 74855, 67, 1, 49, "Subsubsection",
  CellTags->"SimplifyPolyLog"],
Cell[2314662, 74858, 202, 5, 39, "Text",
  CellTags->"SimplifyPolyLog"],
Cell[2314867, 74865, 196, 9, 39, "Text",
  CellTags->"SimplifyPolyLog"]
}, Closed]],

Cell[CellGroupData[{
Cell[2315100, 74879, 64, 1, 49, "Subsubsection",
  CellTags->"SimplifyPolyLog"],
Cell[2315167, 74882, 265, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{
Cell[2315457, 74892, 238, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2315698, 74900, 845, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2316580, 74927, 207, 5, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2316790, 74934, 531, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2317358, 74953, 271, 7, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2317632, 74962, 815, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2318484, 74989, 238, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2318725, 74997, 787, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2319549, 75023, 338, 9, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2319890, 75034, 836, 23, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2320763, 75062, 306, 8, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2321072, 75072, 806, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2321915, 75099, 348, 9, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2322266, 75110, 957, 29, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2323260, 75144, 266, 7, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2323529, 75153, 615, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2324181, 75175, 238, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2324422, 75183, 112, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2324571, 75190, 238, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2324812, 75198, 1563, 43, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2326412, 75246, 338, 9, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2326753, 75257, 1876, 51, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2328666, 75313, 271, 7, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2328940, 75322, 1795, 50, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2330772, 75377, 338, 9, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2331113, 75388, 3746, 102, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2334896, 75495, 234, 6, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2335133, 75503, 375, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2335545, 75519, 203, 5, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2335751, 75526, 264, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2336052, 75538, 178, 4, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2336233, 75544, 472, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2336742, 75563, 178, 4, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2336923, 75569, 403, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2337363, 75586, 178, 4, 70, "Input",
  CellTags->"SimplifyPolyLog"],
Cell[2337544, 75592, 403, 12, 70, "Output"]
}, Closed]],
Cell[2337962, 75607, 144, 3, 70, "Input",
  CellTags->"SimplifyPolyLog"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2338155, 75616, 74, 1, 39, "ObjectName",
  CellTags->"Simplify2"],

Cell[CellGroupData[{
Cell[2338254, 75621, 61, 1, 49, "Subsubsection",
  CellTags->"Simplify2"],
Cell[2338318, 75624, 82, 1, 39, "Text",
  CellTags->"Simplify2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2338437, 75630, 58, 1, 49, "Subsubsection",
  CellTags->"Simplify2"],
Cell[2338498, 75633, 99, 2, 56, "Input",
  CellTags->"Simplify2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2338646, 75641, 58, 1, 39, "ObjectName",
  CellTags->"SmallDelta"],

Cell[CellGroupData[{
Cell[2338729, 75646, 62, 1, 49, "Subsubsection",
  CellTags->"SmallDelta"],
Cell[2338794, 75649, 88, 1, 39, "Text",
  CellTags->"SmallDelta"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2338931, 75656, 62, 1, 39, "ObjectName",
  CellTags->"SmallEpsilon"],

Cell[CellGroupData[{
Cell[2339018, 75661, 64, 1, 49, "Subsubsection",
  CellTags->"SmallEpsilon"],
Cell[2339085, 75664, 92, 1, 39, "Text",
  CellTags->"SmallEpsilon"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2339226, 75671, 64, 1, 39, "ObjectName",
  CellTags->"SmallVariable"],

Cell[CellGroupData[{
Cell[2339315, 75676, 65, 1, 49, "Subsubsection",
  CellTags->"SmallVariable"],
Cell[2339383, 75679, 284, 6, 56, "Text",
  CellTags->"SmallVariable"],
Cell[2339670, 75687, 470, 19, 39, "Text",
  CellTags->"SmallVariable"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2340189, 75712, 66, 1, 39, "ObjectName",
  CellTags->"SmallVariables"],

Cell[CellGroupData[{
Cell[2340280, 75717, 66, 1, 49, "Subsubsection",
  CellTags->"SmallVariables"],
Cell[2340349, 75720, 228, 6, 56, "Text",
  CellTags->"SmallVariables"],
Cell[2340580, 75728, 195, 9, 39, "Text",
  CellTags->"SmallVariables"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2340824, 75743, 44, 1, 39, "ObjectName",
  CellTags->"SMP"],

Cell[CellGroupData[{
Cell[2340893, 75748, 55, 1, 49, "Subsubsection",
  CellTags->"SMP"],
Cell[2340951, 75751, 233, 5, 56, "Text",
  CellTags->"SMP"],

Cell[CellGroupData[{
Cell[2341209, 75760, 157, 4, 56, "Input",
  CellTags->"SMP"],
Cell[2341369, 75766, 189, 5, 39, "Output",
  CellTags->"SMP"]
}, Closed]],
Cell[2341573, 75774, 180, 8, 35, "Text",
  CellTags->"SMP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2341790, 75787, 52, 1, 49, "Subsubsection",
  CellTags->"SMP"],

Cell[CellGroupData[{
Cell[2341867, 75792, 135, 3, 56, "Input",
  CellTags->"SMP"],
Cell[2342005, 75797, 127, 3, 37, "Output",
  CellTags->"SMP"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2342193, 75807, 54, 1, 39, "ObjectName",
  CellTags->"SMVertex"],

Cell[CellGroupData[{
Cell[2342272, 75812, 60, 1, 49, "Subsubsection",
  CellTags->"SMVertex"],
Cell[2342335, 75815, 305, 6, 56, "Text",
  CellTags->"SMVertex"],

Cell[CellGroupData[{
Cell[2342665, 75825, 138, 3, 56, "Input",
  CellTags->"SMVertex"],
Cell[2342806, 75830, 262, 7, 37, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2343117, 75843, 57, 1, 49, "Subsubsection",
  CellTags->"SMVertex"],

Cell[CellGroupData[{
Cell[2343199, 75848, 262, 6, 56, "Input",
  CellTags->"SMVertex"],
Cell[2343464, 75856, 1598, 43, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2345123, 75906, 62, 1, 39, "ObjectName",
  CellTags->"Smu"],

Cell[CellGroupData[{
Cell[2345210, 75911, 55, 1, 49, "Subsubsection",
  CellTags->"Smu"],
Cell[2345268, 75914, 46, 1, 39, "Text",
  CellTags->"Smu"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2345363, 75921, 42, 1, 39, "ObjectName",
  CellTags->"Sn"],

Cell[CellGroupData[{
Cell[2345430, 75926, 54, 1, 49, "Subsubsection",
  CellTags->"Sn"],
Cell[2345487, 75929, 343, 14, 39, "Text",
  CellTags->"Sn"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2345879, 75949, 42, 1, 39, "ObjectName",
  CellTags->"SO"],

Cell[CellGroupData[{
Cell[2345946, 75954, 54, 1, 49, "Subsubsection",
  CellTags->"SO"],
Cell[2346003, 75957, 176, 5, 39, "Text",
  CellTags->"SO"],
Cell[2346182, 75964, 632, 28, 39, "Text",
  CellTags->"SO"]
}, Closed]],

Cell[CellGroupData[{
Cell[2346851, 75997, 51, 1, 49, "Subsubsection",
  CellTags->"SO"],

Cell[CellGroupData[{
Cell[2346927, 76002, 126, 3, 70, "Input",
  CellTags->"SO"],
Cell[2347056, 76007, 318, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2347411, 76021, 153, 4, 70, "Input",
  CellTags->"SO"],
Cell[2347567, 76027, 469, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2348073, 76046, 204, 6, 70, "Input",
  CellTags->"SO"],
Cell[2348280, 76054, 241, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2348582, 76066, 44, 1, 39, "ObjectName",
  CellTags->"SOD"],

Cell[CellGroupData[{
Cell[2348651, 76071, 55, 1, 49, "Subsubsection",
  CellTags->"SOD"],
Cell[2348709, 76074, 192, 5, 56, "Text",
  CellTags->"SOD"],
Cell[2348904, 76081, 535, 24, 39, "Text",
  CellTags->"SOD"]
}, Closed]],

Cell[CellGroupData[{
Cell[2349476, 76110, 52, 1, 49, "Subsubsection",
  CellTags->"SOD"],

Cell[CellGroupData[{
Cell[2349553, 76115, 128, 3, 70, "Input",
  CellTags->"SOD"],
Cell[2349684, 76120, 318, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2350039, 76134, 155, 4, 70, "Input",
  CellTags->"SOD"],
Cell[2350197, 76140, 469, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2350703, 76159, 206, 6, 70, "Input",
  CellTags->"SOD"],
Cell[2350912, 76167, 303, 7, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2351276, 76181, 50, 1, 39, "ObjectName",
  CellTags->"Solve2"],

Cell[CellGroupData[{
Cell[2351351, 76186, 58, 1, 49, "Subsubsection",
  CellTags->"Solve2"],
Cell[2351412, 76189, 347, 7, 56, "Text",
  CellTags->"Solve2"],

Cell[CellGroupData[{
Cell[2351784, 76200, 140, 3, 56, "Input",
  CellTags->"Solve2"],
Cell[2351927, 76205, 298, 7, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2352274, 76218, 55, 1, 49, "Subsubsection",
  CellTags->"Solve2"],

Cell[CellGroupData[{
Cell[2352354, 76223, 491, 14, 70, "Input",
  CellTags->"Solve2"],
Cell[2352848, 76239, 449, 12, 70, "Output"]
}, Closed]],
Cell[2353312, 76254, 114, 4, 70, "Text",
  CellTags->"Solve2"],

Cell[CellGroupData[{
Cell[2353451, 76262, 190, 5, 70, "Input",
  CellTags->"Solve2"],
Cell[2353644, 76269, 197, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2353878, 76279, 322, 8, 70, "Input",
  CellTags->"Solve2"],
Cell[2354203, 76289, 197, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2354437, 76299, 549, 15, 70, "Input",
  CellTags->"Solve2"],
Cell[2354989, 76316, 371, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2355397, 76331, 490, 14, 70, "Input",
  CellTags->"Solve2"],
Cell[2355890, 76347, 503, 14, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2356454, 76368, 50, 1, 39, "ObjectName",
  CellTags->"Solve3"],

Cell[CellGroupData[{
Cell[2356529, 76373, 58, 1, 49, "Subsubsection",
  CellTags->"Solve3"],
Cell[2356590, 76376, 272, 6, 56, "Text",
  CellTags->"Solve3"],

Cell[CellGroupData[{
Cell[2356887, 76386, 138, 3, 56, "Input",
  CellTags->"Solve3"],
Cell[2357028, 76391, 294, 7, 37, "Output"]
}, Closed]],
Cell[2357337, 76401, 174, 8, 35, "Text",
  CellTags->"SMP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2357548, 76414, 55, 1, 49, "Subsubsection",
  CellTags->"Solve3"],

Cell[CellGroupData[{
Cell[2357628, 76419, 489, 14, 56, "Input",
  CellTags->"Solve3"],
Cell[2358120, 76435, 447, 12, 53, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2358628, 76454, 42, 1, 39, "ObjectName",
  CellTags->"SP"],

Cell[CellGroupData[{
Cell[2358695, 76459, 54, 1, 49, "Subsubsection",
  CellTags->"SP"],
Cell[2358752, 76462, 353, 11, 56, "Text",
  CellTags->"SP"],
Cell[2359108, 76475, 462, 19, 39, "Text",
  CellTags->"SP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2359607, 76499, 51, 1, 49, "Subsubsection",
  CellTags->"SP"],

Cell[CellGroupData[{
Cell[2359683, 76504, 225, 6, 70, "Input",
  CellTags->"SP"],
Cell[2359911, 76512, 510, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2360458, 76533, 237, 7, 70, "Input",
  CellTags->"SP"],
Cell[2360698, 76542, 451, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2361186, 76559, 290, 8, 70, "Input",
  CellTags->"SP"],
Cell[2361479, 76569, 513, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2362029, 76589, 205, 5, 70, "Input",
  CellTags->"SP"],
Cell[2362237, 76596, 530, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2362804, 76617, 194, 5, 70, "Input",
  CellTags->"SP"],
Cell[2363001, 76624, 150, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2363188, 76632, 235, 7, 70, "Input",
  CellTags->"SP"],
Cell[2363426, 76641, 234, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2363697, 76651, 271, 8, 70, "Input",
  CellTags->"SP"],
Cell[2363971, 76661, 150, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2364182, 76671, 44, 1, 39, "ObjectName",
  CellTags->"SPC"],

Cell[CellGroupData[{
Cell[2364251, 76676, 55, 1, 49, "Subsubsection",
  CellTags->"SPC"],
Cell[2364309, 76679, 82, 1, 39, "Text",
  CellTags->"SPC"],
Cell[2364394, 76682, 220, 9, 39, "Text",
  CellTags->"SPC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2364663, 76697, 44, 1, 39, "ObjectName",
  CellTags->"SPD"],

Cell[CellGroupData[{
Cell[2364732, 76702, 55, 1, 49, "Subsubsection",
  CellTags->"SPD"],
Cell[2364790, 76705, 372, 11, 56, "Text",
  CellTags->"SPD"],
Cell[2365165, 76718, 565, 24, 39, "Text",
  CellTags->"SPD"]
}, Closed]],

Cell[CellGroupData[{
Cell[2365767, 76747, 52, 1, 49, "Subsubsection",
  CellTags->"SPD"],

Cell[CellGroupData[{
Cell[2365844, 76752, 228, 6, 70, "Input",
  CellTags->"SPD"],
Cell[2366075, 76760, 510, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2366622, 76781, 239, 7, 70, "Input",
  CellTags->"SPD"],
Cell[2366864, 76790, 451, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2367352, 76807, 292, 8, 70, "Input",
  CellTags->"SPD"],
Cell[2367647, 76817, 513, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2368197, 76837, 207, 5, 70, "Input",
  CellTags->"SPD"],
Cell[2368407, 76844, 530, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2368974, 76865, 196, 5, 70, "Input",
  CellTags->"SPD"],
Cell[2369173, 76872, 151, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2369361, 76880, 237, 7, 70, "Input",
  CellTags->"SPD"],
Cell[2369601, 76889, 296, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2369934, 76901, 273, 8, 70, "Input",
  CellTags->"SPD"],
Cell[2370210, 76911, 151, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2370398, 76919, 332, 9, 70, "Input",
  CellTags->"SPD"],
Cell[2370733, 76930, 151, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2370945, 76940, 50, 1, 39, "ObjectName",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[2371020, 76945, 58, 1, 49, "Subsubsection",
  CellTags->"Spinor"],
Cell[2371081, 76948, 1421, 49, 123, "Text",
  CellTags->"Spinor"],
Cell[2372505, 76999, 829, 34, 39, "Text",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2373371, 77038, 55, 1, 49, "Subsubsection",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[2373451, 77043, 134, 3, 56, "Input",
  CellTags->"Spinor"],
Cell[2373588, 77048, 264, 7, 70, "Output",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2373889, 77060, 161, 4, 52, "Input",
  CellTags->"Spinor"],
Cell[2374053, 77066, 309, 7, 70, "Output",
  CellTags->"Spinor"]
}, Closed]],
Cell[2374377, 77076, 147, 4, 35, "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[2374549, 77084, 303, 9, 56, "Input",
  CellTags->"Spinor"],
Cell[2374855, 77095, 155, 4, 50, "Output",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2375047, 77104, 306, 8, 52, "Input",
  CellTags->"Spinor"],
Cell[2375356, 77114, 393, 10, 37, "Output",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2375786, 77129, 173, 4, 52, "Input",
  CellTags->"Spinor"],
Cell[2375962, 77135, 234, 6, 70, "Output",
  CellTags->"Spinor"]
}, Closed]],
Cell[2376211, 77144, 159, 4, 35, "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[2376395, 77152, 259, 7, 56, "Input",
  CellTags->"Spinor"],
Cell[2376657, 77161, 234, 6, 37, "Output",
  CellTags->"Spinor"]
}, Closed]],

Cell[CellGroupData[{
Cell[2376928, 77172, 202, 5, 52, "Input",
  CellTags->"Spinor"],
Cell[2377133, 77179, 234, 6, 70, "Output",
  CellTags->"Spinor"]
}, Closed]],
Cell[2377382, 77188, 78, 1, 35, "Text",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[2377485, 77193, 257, 7, 56, "Input",
  CellTags->"Spinor"],
Cell[2377745, 77202, 234, 6, 70, "Output",
  CellTags->"Spinor"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2378040, 77215, 64, 1, 39, "ObjectName",
  CellTags->"SpinorCollect"],

Cell[CellGroupData[{
Cell[2378129, 77220, 65, 1, 49, "Subsubsection",
  CellTags->"SpinorCollect"],
Cell[2378197, 77223, 194, 5, 39, "Text",
  CellTags->"SpinorCollect"],
Cell[2378394, 77230, 323, 13, 39, "Text",
  CellTags->"SpinorCollect"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2378766, 77249, 58, 1, 39, "ObjectName",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{
Cell[2378849, 77254, 62, 1, 49, "Subsubsection",
  CellTags->"SpinorUBar"],
Cell[2378914, 77257, 183, 7, 39, "Text",
  CellTags->"SpinorUBar"],
Cell[2379100, 77266, 548, 24, 39, "Text",
  CellTags->"SpinorUBar"]
}, Closed]],

Cell[CellGroupData[{
Cell[2379685, 77295, 59, 1, 49, "Subsubsection",
  CellTags->"SpinorUBar"],
Cell[2379747, 77298, 86, 1, 39, "Text",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{
Cell[2379858, 77303, 142, 3, 56, "Input",
  CellTags->"SpinorUBar"],
Cell[2380003, 77308, 278, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2380318, 77320, 169, 4, 52, "Input",
  CellTags->"SpinorUBar"],
Cell[2380490, 77326, 323, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2380850, 77339, 210, 5, 52, "Input",
  CellTags->"SpinorUBar"],
Cell[2381063, 77346, 158, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2381258, 77354, 251, 7, 52, "Input",
  CellTags->"SpinorUBar"],
Cell[2381512, 77363, 212, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2381761, 77373, 287, 8, 52, "Input",
  CellTags->"SpinorUBar"],
Cell[2382051, 77383, 212, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2382324, 77395, 52, 1, 39, "ObjectName",
  CellTags->"SpinorU"],

Cell[CellGroupData[{
Cell[2382401, 77400, 59, 1, 49, "Subsubsection",
  CellTags->"SpinorU"],
Cell[2382463, 77403, 80, 1, 39, "Text",
  CellTags->"SpinorU"],
Cell[2382546, 77406, 554, 24, 39, "Text",
  CellTags->"SpinorU"]
}, Closed]],

Cell[CellGroupData[{
Cell[2383137, 77435, 56, 1, 49, "Subsubsection",
  CellTags->"SpinorU"],

Cell[CellGroupData[{
Cell[2383218, 77440, 136, 3, 56, "Input",
  CellTags->"SpinorU"],
Cell[2383357, 77445, 190, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2383584, 77454, 163, 4, 52, "Input",
  CellTags->"SpinorU"],
Cell[2383750, 77460, 237, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2384024, 77470, 204, 5, 52, "Input",
  CellTags->"SpinorU"],
Cell[2384231, 77477, 155, 3, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2384423, 77485, 245, 7, 52, "Input",
  CellTags->"SpinorU"],
Cell[2384671, 77494, 212, 5, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2384920, 77504, 279, 8, 52, "Input",
  CellTags->"SpinorU"],
Cell[2385202, 77514, 210, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2385473, 77526, 58, 1, 39, "ObjectName",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{
Cell[2385556, 77531, 62, 1, 49, "Subsubsection",
  CellTags->"SpinorVBar"],
Cell[2385621, 77534, 188, 7, 39, "Text",
  CellTags->"SpinorVBar"],
Cell[2385812, 77543, 548, 24, 39, "Text",
  CellTags->"SpinorVBar"]
}, Closed]],

Cell[CellGroupData[{
Cell[2386397, 77572, 59, 1, 49, "Subsubsection",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{
Cell[2386481, 77577, 142, 3, 70, "Input",
  CellTags->"SpinorVBar"],
Cell[2386626, 77582, 278, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2386941, 77594, 169, 4, 70, "Input",
  CellTags->"SpinorVBar"],
Cell[2387113, 77600, 323, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2387473, 77613, 210, 5, 70, "Input",
  CellTags->"SpinorVBar"],
Cell[2387686, 77620, 158, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2387881, 77628, 251, 7, 70, "Input",
  CellTags->"SpinorVBar"],
Cell[2388135, 77637, 238, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2388410, 77648, 287, 8, 70, "Input",
  CellTags->"SpinorVBar"],
Cell[2388700, 77658, 238, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2388999, 77671, 52, 1, 39, "ObjectName",
  CellTags->"SpinorV"],

Cell[CellGroupData[{
Cell[2389076, 77676, 59, 1, 49, "Subsubsection",
  CellTags->"SpinorV"],
Cell[2389138, 77679, 80, 1, 39, "Text",
  CellTags->"SpinorV"],
Cell[2389221, 77682, 554, 24, 39, "Text",
  CellTags->"SpinorV"]
}, Closed]],

Cell[CellGroupData[{
Cell[2389812, 77711, 56, 1, 49, "Subsubsection",
  CellTags->"SpinorV"],

Cell[CellGroupData[{
Cell[2389893, 77716, 136, 3, 70, "Input",
  CellTags->"SpinorV"],
Cell[2390032, 77721, 190, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2390259, 77730, 163, 4, 70, "Input",
  CellTags->"SpinorV"],
Cell[2390425, 77736, 237, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2390699, 77746, 204, 5, 70, "Input",
  CellTags->"SpinorV"],
Cell[2390906, 77753, 155, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2391098, 77761, 245, 7, 70, "Input",
  CellTags->"SpinorV"],
Cell[2391346, 77770, 238, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2391621, 77781, 281, 8, 70, "Input",
  CellTags->"SpinorV"],
Cell[2391905, 77791, 238, 6, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2392204, 77804, 76, 1, 39, "ObjectName",
  CellTags->"SpinPolarizationSum"],

Cell[CellGroupData[{
Cell[2392305, 77809, 71, 1, 49, "Subsubsection",
  CellTags->"SpinPolarizationSum"],
Cell[2392379, 77812, 165, 4, 39, "Text",
  CellTags->"SpinPolarizationSum"],
Cell[2392547, 77818, 215, 8, 39, "Text",
  CellTags->"SpinPolarizationSum"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2392811, 77832, 44, 1, 39, "ObjectName",
  CellTags->"SPL"],

Cell[CellGroupData[{
Cell[2392880, 77837, 55, 1, 49, "Subsubsection",
  CellTags->"SPL"],
Cell[2392938, 77840, 78, 1, 39, "Text",
  CellTags->"SPL"],
Cell[2393019, 77843, 208, 9, 39, "Text",
  CellTags->"SPL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2393276, 77858, 72, 1, 39, "ObjectName",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{
Cell[2393373, 77863, 69, 1, 49, "Subsubsection",
  CellTags->"SplittingFunction"],
Cell[2393445, 77866, 231, 7, 39, "Text",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{
Cell[2393701, 77877, 162, 3, 56, "Input",
  CellTags->"SplittingFunction"],
Cell[2393866, 77882, 189, 5, 70, "Output"]
}, Closed]],
Cell[2394070, 77890, 231, 9, 35, "Text",
  CellTags->"SplittingFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2394338, 77904, 66, 1, 49, "Subsubsection",
  CellTags->"SplittingFunction"],
Cell[2394407, 77907, 66, 1, 70, "Text",
  CellTags->"SplittingFunction"],
Cell[2394476, 77910, 264, 6, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2394743, 77918, 171, 4, 70, "Text",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{
Cell[2394939, 77926, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2395100, 77931, 682, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2395819, 77955, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2395980, 77960, 345, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2396362, 77975, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2396523, 77980, 298, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2396858, 77993, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2397019, 77998, 1059, 28, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2398115, 78031, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2398276, 78036, 1456, 38, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2399769, 78079, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2399930, 78084, 737, 20, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2400704, 78109, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2400865, 78114, 667, 18, 70, "Output"]
}, Closed]],
Cell[2401547, 78135, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2401708, 78140, 64, 1, 70, "Text",
  CellTags->"SplittingFunction"],
Cell[2401775, 78143, 264, 6, 70, "Input",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{
Cell[2402064, 78153, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2402225, 78158, 682, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2402944, 78182, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2403105, 78187, 265, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2403407, 78200, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2403568, 78205, 257, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2403862, 78217, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2404023, 78222, 1051, 29, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2405111, 78256, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2405272, 78261, 1591, 42, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2406900, 78308, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2407061, 78313, 613, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2407711, 78335, 159, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2407873, 78340, 540, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2408450, 78360, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2408611, 78365, 540, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2409188, 78385, 159, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2409350, 78390, 540, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2409927, 78410, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2410088, 78415, 1782, 46, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2411907, 78466, 159, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2412069, 78471, 1782, 46, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2413888, 78522, 159, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2414050, 78527, 799, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2414886, 78554, 160, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2415049, 78559, 7281, 177, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2422367, 78741, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2422528, 78746, 3749, 94, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2426314, 78845, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2426475, 78850, 4649, 118, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2431161, 78973, 158, 3, 70, "Input",
  CellTags->"SplittingFunction"],
Cell[2431322, 78978, 5663, 141, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2437046, 79126, 80, 1, 39, "ObjectName",
  CellTags->"StandardMatrixElement"],

Cell[CellGroupData[{
Cell[2437151, 79131, 73, 1, 49, "Subsubsection",
  CellTags->"StandardMatrixElement"],
Cell[2437227, 79134, 137, 4, 39, "Text",
  CellTags->"StandardMatrixElement"],
Cell[2437367, 79140, 202, 9, 39, "Text",
  CellTags->"StandardMatrixElement"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2437618, 79155, 58, 1, 39, "ObjectName",
  CellTags->"SubContext"],

Cell[CellGroupData[{
Cell[2437701, 79160, 62, 1, 49, "Subsubsection",
  CellTags->"SubContext"],
Cell[2437766, 79163, 123, 4, 39, "Text",
  CellTags->"SubContext"],
Cell[2437892, 79169, 302, 14, 39, "Text",
  CellTags->"SubContext"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2438243, 79189, 52, 1, 39, "ObjectName",
  CellTags->"SubLoop"],

Cell[CellGroupData[{
Cell[2438320, 79194, 59, 1, 49, "Subsubsection",
  CellTags->"SubLoop"],
Cell[2438382, 79197, 153, 4, 39, "Text",
  CellTags->"SubLoop"],
Cell[2438538, 79203, 191, 9, 39, "Text",
  CellTags->"SubLoop"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2438778, 79218, 46, 1, 39, "ObjectName",
  CellTags->"SumP"],

Cell[CellGroupData[{
Cell[2438849, 79223, 56, 1, 49, "Subsubsection",
  CellTags->"SumP"],
Cell[2438908, 79226, 1002, 31, 44, "Text",
  CellTags->"SumP"],
Cell[2439913, 79259, 284, 14, 39, "Text",
  CellTags->"SumP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2440234, 79278, 53, 1, 49, "Subsubsection",
  CellTags->"SumP"],

Cell[CellGroupData[{
Cell[2440312, 79283, 186, 5, 70, "Input",
  CellTags->"SumP"],
Cell[2440501, 79290, 301, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2440839, 79303, 186, 5, 70, "Input",
  CellTags->"SumP"],
Cell[2441028, 79310, 301, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2441366, 79323, 157, 4, 70, "Input",
  CellTags->"SumP"],
Cell[2441526, 79329, 270, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2441833, 79341, 157, 4, 70, "Input",
  CellTags->"SumP"],
Cell[2441993, 79347, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2442166, 79355, 384, 12, 70, "Input",
  CellTags->"SumP"],
Cell[2442553, 79369, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2442726, 79377, 229, 6, 70, "Input",
  CellTags->"SumP"],
Cell[2442958, 79385, 963, 28, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2443958, 79418, 156, 4, 70, "Input",
  CellTags->"SumP"],
Cell[2444117, 79424, 136, 3, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2444314, 79434, 46, 1, 39, "ObjectName",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[2444385, 79439, 56, 1, 49, "Subsubsection",
  CellTags->"SumS"],
Cell[2444444, 79442, 1713, 57, 59, "Text",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[2446182, 79503, 136, 3, 56, "Input",
  CellTags->"SumS"],
Cell[2446321, 79508, 187, 5, 70, "Output"]
}, Closed]],
Cell[2446523, 79516, 284, 14, 35, "Text",
  CellTags->"SumS"]
}, Closed]],

Cell[CellGroupData[{
Cell[2446844, 79535, 53, 1, 49, "Subsubsection",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[2446922, 79540, 186, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2447111, 79547, 247, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2447395, 79559, 186, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2447584, 79566, 247, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2447868, 79578, 181, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2448052, 79585, 195, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2448284, 79595, 210, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2448497, 79602, 311, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2448845, 79616, 239, 6, 70, "Input",
  CellTags->"SumS"],
Cell[2449087, 79624, 418, 13, 70, "Output"]
}, Closed]],
Cell[2449520, 79640, 235, 6, 70, "Input",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[2449780, 79650, 186, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2449969, 79657, 624, 19, 70, "Output"]
}, Closed]],
Cell[2450608, 79679, 236, 6, 70, "Input",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[2450869, 79689, 157, 4, 70, "Input",
  CellTags->"SumS"],
Cell[2451029, 79695, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2451202, 79703, 215, 6, 70, "Input",
  CellTags->"SumS"],
Cell[2451420, 79711, 136, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2451593, 79719, 196, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2451792, 79726, 248, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2452077, 79738, 178, 4, 70, "Input",
  CellTags->"SumS"],
Cell[2452258, 79744, 166, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2452461, 79753, 181, 5, 70, "Input",
  CellTags->"SumS"],
Cell[2452645, 79760, 159, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2452841, 79769, 157, 4, 70, "Input",
  CellTags->"SumS"],
Cell[2453001, 79775, 159, 4, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2453221, 79786, 46, 1, 39, "ObjectName",
  CellTags->"SumT"],

Cell[CellGroupData[{
Cell[2453292, 79791, 56, 1, 49, "Subsubsection",
  CellTags->"SumT"],
Cell[2453351, 79794, 536, 15, 56, "Text",
  CellTags->"SumT"],
Cell[2453890, 79811, 284, 14, 39, "Text",
  CellTags->"SumT"]
}, Closed]],

Cell[CellGroupData[{
Cell[2454211, 79830, 53, 1, 49, "Subsubsection",
  CellTags->"SumT"],

Cell[CellGroupData[{
Cell[2454289, 79835, 186, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2454478, 79842, 278, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2454793, 79855, 186, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2454982, 79862, 278, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2455297, 79875, 157, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2455457, 79881, 207, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2455701, 79892, 210, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2455914, 79899, 446, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2456397, 79918, 157, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2456557, 79924, 159, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2456753, 79933, 303, 9, 70, "Input",
  CellTags->"SumT"],
Cell[2457059, 79944, 159, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2457255, 79953, 196, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2457454, 79960, 279, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2457770, 79973, 168, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2457941, 79979, 280, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2458258, 79990, 157, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2458418, 79996, 159, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2458614, 80005, 181, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2458798, 80012, 159, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2458994, 80021, 332, 10, 70, "Input",
  CellTags->"SumT"],
Cell[2459329, 80033, 899, 26, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2460265, 80064, 168, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2460436, 80070, 191, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2460664, 80080, 192, 5, 70, "Input",
  CellTags->"SumT"],
Cell[2460859, 80087, 280, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2461176, 80098, 161, 4, 70, "Input",
  CellTags->"SumT"],
Cell[2461340, 80104, 553, 16, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2461930, 80125, 304, 9, 70, "Input",
  CellTags->"SumT"],
Cell[2462237, 80136, 553, 16, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2462851, 80159, 46, 1, 39, "ObjectName",
  CellTags->"SUND"],

Cell[CellGroupData[{
Cell[2462922, 80164, 56, 1, 49, "Subsubsection",
  CellTags->"SUND"],
Cell[2462981, 80167, 246, 10, 39, "Text",
  CellTags->"SUND"],
Cell[2463230, 80179, 425, 19, 39, "Text",
  CellTags->"SUND"]
}, Closed]],

Cell[CellGroupData[{
Cell[2463692, 80203, 53, 1, 49, "Subsubsection",
  CellTags->"SUND"],

Cell[CellGroupData[{
Cell[2463770, 80208, 167, 4, 56, "Input",
  CellTags->"SUND"],
Cell[2463940, 80214, 199, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2464176, 80224, 254, 6, 52, "Input",
  CellTags->"SUND"],
Cell[2464433, 80232, 1603, 44, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2466073, 80281, 167, 4, 52, "Input",
  CellTags->"SUND"],
Cell[2466243, 80287, 199, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2466479, 80297, 167, 4, 52, "Input",
  CellTags->"SUND"],
Cell[2466649, 80303, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2466795, 80310, 320, 8, 52, "Input",
  CellTags->"SUND"],
Cell[2467118, 80320, 300, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2467455, 80333, 460, 12, 52, "Input",
  CellTags->"SUND"],
Cell[2467918, 80347, 370, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2468325, 80363, 460, 12, 52, "Input",
  CellTags->"SUND"],
Cell[2468788, 80377, 640, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2469465, 80401, 215, 6, 52, "Input",
  CellTags->"SUND"],
Cell[2469683, 80409, 162, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2469882, 80417, 249, 7, 52, "Input",
  CellTags->"SUND"],
Cell[2470134, 80426, 285, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2470456, 80437, 287, 8, 52, "Input",
  CellTags->"SUND"],
Cell[2470746, 80447, 162, 3, 70, "Output"]
}, Closed]],
Cell[2470923, 80453, 132, 3, 52, "Input",
  CellTags->"SUND"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2471104, 80462, 54, 1, 39, "ObjectName",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{
Cell[2471183, 80467, 60, 1, 49, "Subsubsection",
  CellTags->"SUNDelta"],
Cell[2471246, 80470, 178, 6, 39, "Text",
  CellTags->"SUNDelta"],
Cell[2471427, 80478, 779, 33, 39, "Text",
  CellTags->"SUNDelta"]
}, Closed]],

Cell[CellGroupData[{
Cell[2472243, 80516, 57, 1, 49, "Subsubsection",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{
Cell[2472325, 80521, 165, 4, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2472493, 80527, 180, 4, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2472710, 80536, 316, 8, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2473029, 80546, 392, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2473458, 80563, 247, 6, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2473708, 80571, 392, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2474137, 80588, 206, 5, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2474346, 80595, 156, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2474539, 80603, 247, 7, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2474789, 80612, 238, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2475064, 80622, 283, 8, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2475350, 80632, 150, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2475537, 80640, 241, 7, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2475781, 80649, 238, 5, 70, "Output"]
}, Closed]],
Cell[2476034, 80657, 259, 6, 70, "Text",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{
Cell[2476318, 80667, 380, 10, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2476701, 80679, 624, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2477362, 80703, 138, 3, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2477503, 80708, 441, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2477981, 80723, 247, 7, 70, "Input",
  CellTags->"SUNDelta"],
Cell[2478231, 80732, 254, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2478546, 80744, 70, 1, 39, "ObjectName",
  CellTags->"SUNDeltaContract"],

Cell[CellGroupData[{
Cell[2478641, 80749, 68, 1, 49, "Subsubsection",
  CellTags->"SUNDeltaContract"],
Cell[2478712, 80752, 403, 9, 56, "Text",
  CellTags->"SUNDeltaContract"],
Cell[2479118, 80763, 320, 14, 39, "Text",
  CellTags->"SUNDeltaContract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2479487, 80783, 46, 1, 39, "ObjectName",
  CellTags->"SUNF"],

Cell[CellGroupData[{
Cell[2479558, 80788, 56, 1, 49, "Subsubsection",
  CellTags->"SUNF"],
Cell[2479617, 80791, 196, 6, 39, "Text",
  CellTags->"SUNF"],

Cell[CellGroupData[{
Cell[2479838, 80801, 136, 3, 56, "Input",
  CellTags->"SUNF"],
Cell[2479977, 80806, 189, 5, 70, "Output"]
}, Closed]],
Cell[2480181, 80814, 764, 34, 35, "Text",
  CellTags->"SUNF"]
}, Closed]],

Cell[CellGroupData[{
Cell[2480982, 80853, 53, 1, 49, "Subsubsection",
  CellTags->"SUNF"],

Cell[CellGroupData[{
Cell[2481060, 80858, 362, 10, 70, "Input",
  CellTags->"SUNF"],
Cell[2481425, 80870, 383, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2481845, 80885, 131, 3, 70, "Input",
  CellTags->"SUNF"],
Cell[2481979, 80890, 1108, 33, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2483124, 80928, 138, 3, 70, "Input",
  CellTags->"SUNF"],
Cell[2483265, 80933, 625, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2483927, 80957, 167, 4, 70, "Input",
  CellTags->"SUNF"],
Cell[2484097, 80963, 199, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2484333, 80973, 200, 5, 70, "Input",
  CellTags->"SUNF"],
Cell[2484536, 80980, 109, 2, 70, "Output"]
}, Closed]],
Cell[2484660, 80985, 375, 11, 70, "Text",
  CellTags->"SUNF"],

Cell[CellGroupData[{
Cell[2485060, 81000, 222, 5, 70, "Input",
  CellTags->"SUNF"],
Cell[2485285, 81007, 1703, 44, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2487025, 81056, 320, 8, 70, "Input",
  CellTags->"SUNF"],
Cell[2487348, 81066, 467, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2487852, 81085, 287, 7, 70, "Input",
  CellTags->"SUNF"],
Cell[2488142, 81094, 1727, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2489906, 81144, 215, 6, 70, "Input",
  CellTags->"SUNF"],
Cell[2490124, 81152, 162, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2490323, 81160, 249, 7, 70, "Input",
  CellTags->"SUNF"],
Cell[2490575, 81169, 285, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2490897, 81180, 287, 8, 70, "Input",
  CellTags->"SUNF"],
Cell[2491187, 81190, 162, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2491386, 81198, 167, 4, 70, "Input",
  CellTags->"SUNF"],
Cell[2491556, 81204, 199, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2491792, 81214, 199, 5, 70, "Input",
  CellTags->"SUNF"],
Cell[2491994, 81221, 560, 17, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2492615, 81245, 58, 1, 39, "ObjectName",
  CellTags->"SUNFJacobi"],

Cell[CellGroupData[{
Cell[2492698, 81250, 62, 1, 49, "Subsubsection",
  CellTags->"SUNFJacobi"],
Cell[2492763, 81253, 147, 4, 39, "Text",
  CellTags->"SUNFJacobi"],
Cell[2492913, 81259, 305, 13, 39, "Text",
  CellTags->"SUNFJacobi"]
}, Closed]],

Cell[CellGroupData[{
Cell[2493255, 81277, 59, 1, 49, "Subsubsection",
  CellTags->"SUNFJacobi"],

Cell[CellGroupData[{
Cell[2493339, 81282, 462, 13, 56, "Input",
  CellTags->"SUNFJacobi"],
Cell[2493804, 81297, 991, 31, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2494832, 81333, 461, 13, 52, "Input",
  CellTags->"SUNFJacobi"],
Cell[2495296, 81348, 2019, 61, 39, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2497376, 81416, 54, 1, 39, "ObjectName",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{
Cell[2497455, 81421, 60, 1, 49, "Subsubsection",
  CellTags->"SUNIndex"],
Cell[2497518, 81424, 141, 6, 39, "Text",
  CellTags->"SUNIndex"],
Cell[2497662, 81432, 437, 18, 39, "Text",
  CellTags->"SUNIndex"]
}, Closed]],

Cell[CellGroupData[{
Cell[2498136, 81455, 57, 1, 49, "Subsubsection",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{
Cell[2498218, 81460, 138, 3, 70, "Input",
  CellTags->"SUNIndex"],
Cell[2498359, 81465, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2498505, 81472, 138, 3, 70, "Input",
  CellTags->"SUNIndex"],
Cell[2498646, 81477, 129, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2498812, 81484, 247, 7, 70, "Input",
  CellTags->"SUNIndex"],
Cell[2499062, 81493, 238, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2499361, 81505, 66, 1, 39, "ObjectName",
  CellTags->"SUNIndexRename"],

Cell[CellGroupData[{
Cell[2499452, 81510, 66, 1, 49, "Subsubsection",
  CellTags->"SUNIndexRename"],
Cell[2499521, 81513, 164, 4, 39, "Text",
  CellTags->"SUNIndexRename"],
Cell[2499688, 81519, 327, 14, 39, "Text",
  CellTags->"SUNIndexRename"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2500064, 81539, 46, 1, 39, "ObjectName",
  CellTags->"SUNN"],

Cell[CellGroupData[{
Cell[2500135, 81544, 56, 1, 49, "Subsubsection",
  CellTags->"SUNN"],
Cell[2500194, 81547, 216, 7, 39, "Text",
  CellTags->"SUNN"],
Cell[2500413, 81556, 632, 29, 39, "Text",
  CellTags->"SUNN"]
}, Closed]],

Cell[CellGroupData[{
Cell[2501082, 81590, 53, 1, 49, "Subsubsection",
  CellTags->"SUNN"],

Cell[CellGroupData[{
Cell[2501160, 81595, 375, 9, 56, "Input",
  CellTags->"SUNN"],
Cell[2501538, 81606, 162, 4, 38, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2501761, 81617, 58, 1, 39, "ObjectName",
  CellTags->"SUNNToCACF"],

Cell[CellGroupData[{
Cell[2501844, 81622, 62, 1, 49, "Subsubsection",
  CellTags->"SUNNToCACF"],
Cell[2501909, 81625, 190, 5, 39, "Text",
  CellTags->"SUNNToCACF"],
Cell[2502102, 81632, 626, 29, 39, "Text",
  CellTags->"SUNNToCACF"]
}, Closed]],

Cell[CellGroupData[{
Cell[2502765, 81666, 59, 1, 49, "Subsubsection",
  CellTags->"SUNNToCACF"],

Cell[CellGroupData[{
Cell[2502849, 81671, 380, 9, 56, "Input",
  CellTags->"SUNNToCACF"],
Cell[2503232, 81682, 198, 5, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2503491, 81694, 60, 1, 39, "ObjectName",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{
Cell[2503576, 81699, 63, 1, 49, "Subsubsection",
  CellTags->"SUNSimplify"],
Cell[2503642, 81702, 377, 7, 73, "Text",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{
Cell[2504044, 81713, 150, 3, 56, "Input",
  CellTags->"SUNSimplify"],
Cell[2504197, 81718, 567, 12, 70, "Output"]
}, Closed]],
Cell[2504779, 81733, 186, 9, 35, "Text",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[2505002, 81747, 60, 1, 49, "Subsubsection",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{
Cell[2505087, 81752, 301, 8, 56, "Input",
  CellTags->"SUNSimplify"],
Cell[2505391, 81762, 286, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2505714, 81774, 145, 3, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2505862, 81779, 392, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2506291, 81796, 231, 6, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2506525, 81804, 294, 9, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2506856, 81818, 145, 3, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2507004, 81823, 135, 3, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2507176, 81831, 220, 5, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2507399, 81838, 224, 6, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2507660, 81849, 308, 8, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2507971, 81859, 317, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2508325, 81872, 145, 3, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2508473, 81877, 467, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2508977, 81896, 323, 8, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2509303, 81906, 317, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2509657, 81919, 145, 3, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2509805, 81924, 212, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2510054, 81934, 313, 8, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2510370, 81944, 317, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2510724, 81957, 145, 3, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2510872, 81962, 467, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2511376, 81981, 313, 8, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2511692, 81991, 317, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2512046, 82004, 217, 5, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2512266, 82011, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2512412, 82018, 467, 12, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2512882, 82032, 370, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2513289, 82048, 472, 12, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2513764, 82062, 586, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2514387, 82084, 294, 7, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2514684, 82093, 1727, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2516448, 82143, 294, 7, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2516745, 82152, 1681, 44, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2518463, 82201, 327, 8, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2518793, 82211, 277, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2519107, 82223, 533, 14, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2519643, 82239, 565, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2520245, 82258, 218, 5, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2520466, 82265, 1004, 31, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2521507, 82301, 280, 6, 52, "Input",
  CellTags->"SUNSimplify"],
Cell[2521790, 82309, 2022, 61, 70, "Output"]
}, Closed]],
Cell[2523827, 82373, 243, 6, 52, "Input",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{
Cell[2524095, 82383, 388, 11, 43, "Input",
  CellTags->"SUNSimplify"],
Cell[2524486, 82396, 706, 21, 37, "Output",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[2525229, 82422, 193, 5, 39, "Input",
  CellTags->"SUNSimplify"],
Cell[2525425, 82429, 706, 21, 37, "Output",
  CellTags->"SUNSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[2526168, 82455, 192, 5, 39, "Input",
  CellTags->"SUNSimplify"],
Cell[2526363, 82462, 734, 22, 37, "Output",
  CellTags->"SUNSimplify"]
}, Closed]],
Cell[2527112, 82487, 249, 6, 52, "Input",
  CellTags->"SUNSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2527410, 82499, 46, 1, 39, "ObjectName",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[2527481, 82504, 56, 1, 49, "Subsubsection",
  CellTags->"SUNT"],
Cell[2527540, 82507, 254, 10, 39, "Text",
  CellTags->"SUNT"],
Cell[2527797, 82519, 737, 34, 39, "Text",
  CellTags->"SUNT"]
}, Closed]],

Cell[CellGroupData[{
Cell[2528571, 82558, 53, 1, 49, "Subsubsection",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[2528649, 82563, 130, 3, 70, "Input",
  CellTags->"SUNT"],
Cell[2528782, 82568, 182, 5, 70, "Output"]
}, Closed]],
Cell[2528979, 82576, 216, 7, 70, "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[2529220, 82587, 247, 6, 70, "Input",
  CellTags->"SUNT"],
Cell[2529470, 82595, 383, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2529890, 82612, 177, 4, 70, "Input",
  CellTags->"SUNT"],
Cell[2530070, 82618, 457, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2530564, 82638, 290, 7, 70, "Input",
  CellTags->"SUNT"],
Cell[2530857, 82647, 333, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2531227, 82662, 221, 5, 70, "Input",
  CellTags->"SUNT"],
Cell[2531451, 82669, 145, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2531633, 82677, 211, 5, 70, "Input",
  CellTags->"SUNT"],
Cell[2531847, 82684, 492, 15, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2532376, 82704, 290, 7, 70, "Input",
  CellTags->"SUNT"],
Cell[2532669, 82713, 333, 10, 70, "Output"]
}, Closed]],
Cell[2533017, 82726, 491, 15, 70, "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[2533533, 82745, 198, 5, 70, "Input",
  CellTags->"SUNT"],
Cell[2533734, 82752, 437, 13, 70, "Output"]
}, Closed]],
Cell[2534186, 82768, 210, 7, 70, "Text",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[2534421, 82779, 240, 6, 70, "Input",
  CellTags->"SUNT"],
Cell[2534664, 82787, 467, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2535168, 82806, 248, 7, 70, "Input",
  CellTags->"SUNT"],
Cell[2535419, 82815, 296, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2535752, 82827, 208, 6, 70, "Input",
  CellTags->"SUNT"],
Cell[2535963, 82835, 164, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2536164, 82843, 242, 7, 70, "Input",
  CellTags->"SUNT"],
Cell[2536409, 82852, 125, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2536595, 82861, 54, 1, 39, "ObjectName",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{
Cell[2536674, 82866, 60, 1, 49, "Subsubsection",
  CellTags->"SUNTrace"],
Cell[2536737, 82869, 82, 1, 39, "Text",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{
Cell[2536844, 82874, 144, 3, 56, "Input",
  CellTags->"SUNTrace"],
Cell[2536991, 82879, 189, 5, 70, "Output"]
}, Closed]],
Cell[2537195, 82887, 303, 14, 35, "Text",
  CellTags->"SUNTrace"]
}, Closed]],

Cell[CellGroupData[{
Cell[2537535, 82906, 57, 1, 49, "Subsubsection",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{
Cell[2537617, 82911, 202, 5, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2537822, 82918, 437, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2538296, 82936, 212, 5, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2538511, 82943, 736, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2539284, 82970, 288, 7, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2539575, 82979, 1188, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2540800, 83019, 222, 5, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2541025, 83026, 900, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2541962, 83058, 334, 8, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2542299, 83068, 6262, 183, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2548598, 83256, 214, 5, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2548815, 83263, 900, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2549752, 83295, 355, 9, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2550110, 83306, 17504, 440, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2567651, 83751, 214, 5, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2567868, 83758, 1064, 32, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2568969, 83795, 324, 8, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2569296, 83805, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2569442, 83812, 324, 8, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2569769, 83822, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2569915, 83829, 324, 8, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2570242, 83839, 109, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2570388, 83846, 324, 8, 70, "Input",
  CellTags->"SUNTrace"],
Cell[2570715, 83856, 740, 22, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2571516, 83885, 62, 1, 39, "ObjectName",
  CellTags->"SymbolicSum2"],

Cell[CellGroupData[{
Cell[2571603, 83890, 64, 1, 49, "Subsubsection",
  CellTags->"SymbolicSum2"],
Cell[2571670, 83893, 285, 6, 56, "Text",
  CellTags->"SymbolicSum2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2572004, 83905, 62, 1, 39, "ObjectName",
  CellTags->"SymbolicSum3"],

Cell[CellGroupData[{
Cell[2572091, 83910, 64, 1, 49, "Subsubsection",
  CellTags->"SymbolicSum3"],
Cell[2572158, 83913, 293, 6, 56, "Text",
  CellTags->"SymbolicSum3"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2572500, 83925, 58, 1, 39, "ObjectName",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{
Cell[2572583, 83930, 62, 1, 49, "Subsubsection",
  CellTags->"Symmetrize"],
Cell[2572648, 83933, 146, 4, 39, "Text",
  CellTags->"Symmetrize"],
Cell[2572797, 83939, 205, 8, 39, "Text",
  CellTags->"Symmetrize"]
}, Closed]],

Cell[CellGroupData[{
Cell[2573039, 83952, 59, 1, 49, "Subsubsection",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{
Cell[2573123, 83957, 294, 8, 70, "Input",
  CellTags->"Symmetrize"],
Cell[2573420, 83967, 376, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2573833, 83983, 314, 8, 70, "Input",
  CellTags->"Symmetrize"],
Cell[2574150, 83993, 768, 19, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2574979, 84019, 62, 1, 39, "ObjectName",
  CellTags->{"TBox", "Tbox"}],

Cell[CellGroupData[{
Cell[2575066, 84024, 66, 1, 49, "Subsubsection",
  CellTags->{"TBox", "Tbox"}],
Cell[2575135, 84027, 265, 6, 56, "Text",
  CellTags->{"TBox", "Tbox"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2575449, 84039, 46, 1, 39, "ObjectName",
  CellTags->"Tdec"],

Cell[CellGroupData[{
Cell[2575520, 84044, 56, 1, 49, "Subsubsection",
  CellTags->"Tdec"],
Cell[2575579, 84047, 259, 6, 56, "Text",
  CellTags->"Tdec"],

Cell[CellGroupData[{
Cell[2575863, 84057, 136, 3, 56, "Input",
  CellTags->"Tdec"],
Cell[2576002, 84062, 524, 11, 70, "Output"]
}, Closed]],
Cell[2576541, 84076, 422, 19, 35, "Text",
  CellTags->"Tdec"]
}, Closed]],

Cell[CellGroupData[{
Cell[2577000, 84100, 53, 1, 49, "Subsubsection",
  CellTags->"Tdec"],
Cell[2577056, 84103, 756, 23, 70, "Text",
  CellTags->"Tdec"],

Cell[CellGroupData[{
Cell[2577837, 84130, 250, 7, 70, "Input",
  CellTags->"Tdec"],
Cell[2578090, 84139, 902, 25, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2579029, 84169, 245, 7, 70, "Input",
  CellTags->"Tdec"],
Cell[2579277, 84178, 589, 18, 70, "Output"]
}, Closed]],
Cell[2579881, 84199, 923, 28, 70, "Text",
  CellTags->"Tdec"],
Cell[2580807, 84229, 697, 19, 70, "Input",
  CellTags->"Tdec"],

Cell[CellGroupData[{
Cell[2581529, 84252, 245, 7, 70, "Input",
  CellTags->"Tdec"],
Cell[2581777, 84261, 14163, 357, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2595977, 84623, 469, 13, 70, "Input",
  CellTags->"Tdec"],
Cell[2596449, 84638, 752, 21, 70, "Output"]
}, Closed]],
Cell[2597216, 84662, 153, 4, 70, "Input",
  CellTags->"Tdec"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2597418, 84672, 66, 1, 39, "ObjectName",
  CellTags->"TensorFunction"],

Cell[CellGroupData[{
Cell[2597509, 84677, 66, 1, 49, "Subsubsection",
  CellTags->"TensorFunction"],
Cell[2597578, 84680, 361, 7, 56, "Text",
  CellTags->"TensorFunction"],
Cell[2597942, 84689, 204, 9, 39, "Text",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2598183, 84703, 63, 1, 49, "Subsubsection",
  CellTags->"TensorFunction"],

Cell[CellGroupData[{
Cell[2598271, 84708, 217, 5, 56, "Input",
  CellTags->"TensorFunction"],
Cell[2598491, 84715, 253, 6, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2598781, 84726, 144, 3, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2598928, 84731, 232, 5, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2599197, 84741, 241, 6, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2599441, 84749, 475, 14, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2599953, 84768, 144, 3, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2600100, 84773, 421, 10, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2600558, 84788, 277, 7, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2600838, 84797, 319, 8, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2601194, 84810, 277, 7, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2601474, 84819, 319, 8, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2601830, 84832, 144, 3, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2601977, 84837, 292, 7, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],

Cell[CellGroupData[{
Cell[2602306, 84849, 146, 3, 52, "Input",
  CellTags->"TensorFunction"],
Cell[2602455, 84854, 161, 4, 70, "Output",
  CellTags->"TensorFunction"]
}, Closed]],
Cell[2602631, 84861, 186, 4, 52, "Input",
  CellTags->"TensorFunction"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2602866, 84871, 42, 1, 39, "ObjectName",
  CellTags->"Tf"],

Cell[CellGroupData[{
Cell[2602933, 84876, 54, 1, 49, "Subsubsection",
  CellTags->"Tf"],
Cell[2602990, 84879, 231, 10, 40, "Text",
  CellTags->"Tf"],
Cell[2603224, 84891, 446, 18, 39, "Text",
  CellTags->"Tf"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2603719, 84915, 44, 1, 39, "ObjectName",
  CellTags->"TFi"],

Cell[CellGroupData[{
Cell[2603788, 84920, 55, 1, 49, "Subsubsection",
  CellTags->"TFi"],
Cell[2603846, 84923, 579, 10, 90, "Text",
  CellTags->"TFi"],
Cell[2604428, 84935, 250, 8, 39, "Text",
  CellTags->"TFi"],
Cell[2604681, 84945, 188, 5, 39, "Text",
  CellTags->"TFi"],
Cell[2604872, 84952, 288, 13, 39, "Text",
  CellTags->"TFi"]
}, Closed]],

Cell[CellGroupData[{
Cell[2605197, 84970, 52, 1, 49, "Subsubsection",
  CellTags->"TFi"],

Cell[CellGroupData[{
Cell[2605274, 84975, 629, 17, 70, "Input",
  CellTags->"TFi"],
Cell[2605906, 84994, 1146, 34, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2607089, 85033, 365, 10, 70, "Input",
  CellTags->"TFi"],
Cell[2607457, 85045, 1136, 34, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2608654, 85086, 60, 1, 39, "ObjectName",
  CellTags->"ThreeVector"],

Cell[CellGroupData[{
Cell[2608739, 85091, 63, 1, 49, "Subsubsection",
  CellTags->"ThreeVector"],
Cell[2608805, 85094, 92, 1, 39, "Text",
  CellTags->"ThreeVector"],
Cell[2608900, 85097, 326, 13, 39, "Text",
  CellTags->"ThreeVector"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2609275, 85116, 44, 1, 39, "ObjectName",
  CellTags->"TID"],

Cell[CellGroupData[{
Cell[2609344, 85121, 55, 1, 49, "Subsubsection",
  CellTags->"TID"],
Cell[2609402, 85124, 81, 1, 39, "Text",
  CellTags->"TID"],

Cell[CellGroupData[{
Cell[2609508, 85129, 134, 3, 56, "Input",
  CellTags->"TID"],
Cell[2609645, 85134, 730, 14, 70, "Output"]
}, Closed]],
Cell[2610390, 85151, 316, 14, 35, "Text",
  CellTags->"TID"]
}, Closed]],

Cell[CellGroupData[{
Cell[2610743, 85170, 52, 1, 49, "Subsubsection",
  CellTags->"TID"],

Cell[CellGroupData[{
Cell[2610820, 85175, 458, 12, 56, "Input",
  CellTags->"TID"],
Cell[2611281, 85189, 1559, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2612877, 85239, 224, 6, 52, "Input",
  CellTags->"TID"],
Cell[2613104, 85247, 18621, 428, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2631762, 85680, 332, 9, 52, "Input",
  CellTags->"TID"],
Cell[2632097, 85691, 1144, 34, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2633278, 85730, 155, 4, 52, "Input",
  CellTags->"TID"],
Cell[2633436, 85736, 2418, 69, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2635891, 85810, 236, 6, 52, "Input",
  CellTags->"TID"],
Cell[2636130, 85818, 998, 29, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2637189, 85854, 46, 1, 39, "ObjectName",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2637260, 85859, 56, 1, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2637319, 85862, 88, 1, 39, "Text",
  CellTags->"TIDL"],
Cell[2637410, 85865, 166, 8, 39, "Text",
  CellTags->"TIDL"]
}, Closed]],
Cell[2637591, 85876, 53, 1, 49, "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2637669, 85881, 171, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2637843, 85888, 507, 14, 70, "Text",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2638375, 85906, 250, 7, 70, "Input",
  CellTags->"TIDL"],
Cell[2638628, 85915, 642, 20, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2639319, 85941, 176, 5, 49, "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2639520, 85950, 390, 11, 70, "Input",
  CellTags->"TIDL"],
Cell[2639913, 85963, 2967, 82, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2642929, 86051, 171, 5, 49, "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2643125, 86060, 345, 10, 70, "Input",
  CellTags->"TIDL"],
Cell[2643473, 86072, 4307, 118, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2647829, 86196, 176, 5, 49, "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2648030, 86205, 485, 14, 70, "Input",
  CellTags->"TIDL"],
Cell[2648518, 86221, 41474, 950, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2690041, 87177, 182, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2690226, 87184, 548, 15, 70, "Input",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{
Cell[2690811, 87204, 190, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2691004, 87211, 637, 17, 70, "Input",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{
Cell[2691678, 87233, 171, 5, 49, "Subsubsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2691874, 87242, 387, 11, 70, "Input",
  CellTags->"TIDL"],
Cell[2692264, 87255, 25266, 599, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2717579, 87860, 176, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2717758, 87867, 473, 14, 70, "Input",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{
Cell[2718268, 87886, 182, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2718453, 87893, 560, 16, 70, "Input",
  CellTags->"TIDL"]
}, Closed]],

Cell[CellGroupData[{
Cell[2719050, 87914, 190, 5, 49, "Subsubsection",
  CellTags->"TIDL"],
Cell[2719243, 87921, 664, 18, 70, "Input",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2719932, 87943, 287, 7, 70, "Input",
  CellTags->"TIDL"],
Cell[2720222, 87952, 185, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2720468, 87964, 66, 1, 39, "ObjectName",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{
Cell[2720559, 87969, 66, 1, 49, "Subsubsection",
  CellTags->"TimedIntegrate"],
Cell[2720628, 87972, 226, 5, 56, "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{
Cell[2720879, 87981, 154, 3, 56, "Input",
  CellTags->"TimedIntegrate"],
Cell[2721036, 87986, 511, 13, 37, "Output",
  CellTags->"TimedIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2721596, 88005, 63, 1, 49, "Subsubsection",
  CellTags->"TimedIntegrate"],
Cell[2721662, 88008, 74, 1, 25, "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{
Cell[2721761, 88013, 362, 9, 56, "Input",
  CellTags->"TimedIntegrate"],
Cell[2722126, 88024, 159, 4, 37, "Output",
  CellTags->"TimedIntegrate"]
}, Closed]],
Cell[2722300, 88031, 60, 1, 21, "Text",
  CellTags->"TimedIntegrate"],

Cell[CellGroupData[{
Cell[2722385, 88036, 458, 11, 56, "Input",
  CellTags->"TimedIntegrate"],
Cell[2722846, 88049, 555, 15, 39, "Output",
  CellTags->"TimedIntegrate"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2723462, 88071, 62, 1, 39, "ObjectName",
  CellTags->"TLI"],

Cell[CellGroupData[{
Cell[2723549, 88076, 55, 1, 49, "Subsubsection",
  CellTags->"TLI"],
Cell[2723607, 88079, 617, 12, 131, "Text",
  CellTags->"TLI"],

Cell[CellGroupData[{
Cell[2724249, 88095, 132, 3, 56, "Input",
  CellTags->"TLI"],
Cell[2724384, 88100, 262, 7, 37, "Output"]
}, Closed]],
Cell[2724661, 88110, 181, 9, 35, "Text",
  CellTags->"TLI"]
}, Closed]],

Cell[CellGroupData[{
Cell[2724879, 88124, 52, 1, 49, "Subsubsection",
  CellTags->"TLI"],
Cell[2724934, 88127, 95, 2, 56, "Input",
  CellTags->"TLI"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2725078, 88135, 66, 1, 39, "ObjectName",
  CellTags->"TLIFP"],

Cell[CellGroupData[{
Cell[2725169, 88140, 57, 1, 49, "Subsubsection",
  CellTags->"TLIFP"],
Cell[2725229, 88143, 94, 1, 39, "Text",
  CellTags->"TLIFP"],

Cell[CellGroupData[{
Cell[2725348, 88148, 136, 3, 56, "Input",
  CellTags->"TLIFP"],
Cell[2725487, 88153, 494, 12, 37, "Output"]
}, Closed]],
Cell[2725996, 88168, 174, 9, 35, "Text",
  CellTags->"TLIFP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2726207, 88182, 54, 1, 49, "Subsubsection",
  CellTags->"TLIFP"],
Cell[2726264, 88185, 97, 2, 56, "Input",
  CellTags->"TLIFP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2726410, 88193, 68, 1, 39, "ObjectName",
  CellTags->"TLIHYP"],

Cell[CellGroupData[{
Cell[2726503, 88198, 58, 1, 49, "Subsubsection",
  CellTags->"TLIHYP"],
Cell[2726564, 88201, 137, 4, 39, "Text",
  CellTags->"TLIHYP"],

Cell[CellGroupData[{
Cell[2726726, 88209, 137, 3, 56, "Input",
  CellTags->"TLIHYP"],
Cell[2726866, 88214, 270, 7, 37, "Output"]
}, Closed]],
Cell[2727151, 88224, 175, 9, 35, "Text",
  CellTags->"TLIHYP"]
}, Closed]],

Cell[CellGroupData[{
Cell[2727363, 88238, 55, 1, 49, "Subsubsection",
  CellTags->"TLIHYP"],
Cell[2727421, 88241, 98, 2, 56, "Input",
  CellTags->"TLIHYP"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2727568, 88249, 46, 1, 39, "ObjectName",
  CellTags->"TLI2"],

Cell[CellGroupData[{
Cell[2727639, 88254, 56, 1, 49, "Subsubsection",
  CellTags->"TLI2"],
Cell[2727698, 88257, 84, 1, 39, "Text",
  CellTags->"TLI2"],
Cell[2727785, 88260, 173, 9, 39, "Text",
  CellTags->"TLI2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2728007, 88275, 68, 1, 39, "ObjectName",
  CellTags->"TLI2FC"],

Cell[CellGroupData[{
Cell[2728100, 88280, 58, 1, 49, "Subsubsection",
  CellTags->"TLI2FC"],
Cell[2728161, 88283, 112, 4, 39, "Text",
  CellTags->"TLI2FC"],

Cell[CellGroupData[{
Cell[2728298, 88291, 137, 3, 56, "Input",
  CellTags->"TLI2FC"],
Cell[2728438, 88296, 404, 10, 37, "Output"]
}, Closed]],
Cell[2728857, 88309, 175, 9, 35, "Text",
  CellTags->"TLI2FC"]
}, Closed]],

Cell[CellGroupData[{
Cell[2729069, 88323, 55, 1, 49, "Subsubsection",
  CellTags->"TLI2FC"],
Cell[2729127, 88326, 98, 2, 56, "Input",
  CellTags->"TLI2FC"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2729274, 88334, 66, 1, 39, "ObjectName",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{
Cell[2729365, 88339, 66, 1, 49, "Subsubsection",
  CellTags->"ToDistribution"],
Cell[2729434, 88342, 427, 8, 73, "Text",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{
Cell[2729886, 88354, 156, 3, 56, "Input",
  CellTags->"ToDistribution"],
Cell[2730045, 88359, 200, 5, 70, "Output"]
}, Closed]],
Cell[2730260, 88367, 222, 9, 35, "Text",
  CellTags->"ToDistribution"]
}, Closed]],

Cell[CellGroupData[{
Cell[2730519, 88381, 63, 1, 49, "Subsubsection",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{
Cell[2730607, 88386, 258, 6, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2730868, 88394, 208, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2731113, 88404, 347, 10, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2731463, 88416, 1664, 47, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2733164, 88468, 245, 6, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2733412, 88476, 200, 5, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2733649, 88486, 347, 10, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2733999, 88498, 1682, 49, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2735718, 88552, 563, 14, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2736284, 88568, 428, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2736749, 88585, 146, 3, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2736898, 88590, 373, 9, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2737308, 88604, 513, 14, 70, "Input",
  CellTags->"ToDistribution"],
Cell[2737824, 88620, 428, 12, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2738313, 88639, 70, 1, 39, "ObjectName",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{
Cell[2738408, 88644, 68, 1, 49, "Subsubsection",
  CellTags->"ToHypergeometric"],
Cell[2738479, 88647, 2065, 67, 56, "Text",
  CellTags->"ToHypergeometric"],
Cell[2740547, 88716, 512, 19, 39, "Text",
  CellTags->"ToHypergeometric"]
}, Closed]],

Cell[CellGroupData[{
Cell[2741096, 88740, 65, 1, 49, "Subsubsection",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{
Cell[2741186, 88745, 496, 14, 70, "Input",
  CellTags->"ToHypergeometric"],
Cell[2741685, 88761, 2200, 55, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2743922, 88821, 603, 16, 70, "Input",
  CellTags->"ToHypergeometric"],
Cell[2744528, 88839, 1948, 49, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2746513, 88893, 496, 14, 70, "Input",
  CellTags->"ToHypergeometric"],
Cell[2747012, 88909, 2285, 57, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2749334, 88971, 603, 16, 70, "Input",
  CellTags->"ToHypergeometric"],
Cell[2749940, 88989, 2059, 52, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2752060, 89048, 52, 1, 39, "ObjectName",
  CellTags->"ToLarin"],

Cell[CellGroupData[{
Cell[2752137, 89053, 59, 1, 49, "Subsubsection",
  CellTags->"ToLarin"],
Cell[2752199, 89056, 693, 20, 39, "Text",
  CellTags->"ToLarin"],
Cell[2752895, 89078, 176, 9, 39, "Text",
  CellTags->"ToLarin"]
}, Closed]],

Cell[CellGroupData[{
Cell[2753108, 89092, 56, 1, 49, "Subsubsection",
  CellTags->"ToLarin"],

Cell[CellGroupData[{
Cell[2753189, 89097, 269, 6, 70, "Input",
  CellTags->"ToLarin"],
Cell[2753461, 89105, 477, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2753975, 89124, 136, 3, 70, "Input",
  CellTags->"ToLarin"],
Cell[2754114, 89129, 1475, 44, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2755650, 89180, 48, 1, 39, "ObjectName",
  CellTags->"ToTFi"],

Cell[CellGroupData[{
Cell[2755723, 89185, 57, 1, 49, "Subsubsection",
  CellTags->"ToTFi"],
Cell[2755783, 89188, 275, 6, 56, "Text",
  CellTags->"ToTFi"],
Cell[2756061, 89196, 284, 13, 39, "Text",
  CellTags->"ToTFi"]
}, Closed]],

Cell[CellGroupData[{
Cell[2756382, 89214, 54, 1, 49, "Subsubsection",
  CellTags->"ToTFi"],

Cell[CellGroupData[{
Cell[2756461, 89219, 404, 11, 70, "Input",
  CellTags->"ToTFi"],
Cell[2756868, 89232, 3153, 76, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2760058, 89313, 181, 4, 70, "Input",
  CellTags->"ToTFi"],
Cell[2760242, 89319, 1134, 34, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2761413, 89358, 135, 3, 70, "Input",
  CellTags->"ToTFi"],
Cell[2761551, 89363, 658, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2762246, 89385, 574, 15, 70, "Input",
  CellTags->"ToTFi"],
Cell[2762823, 89402, 2089, 59, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2764949, 89466, 132, 3, 70, "Input",
  CellTags->"ToTFi"],
Cell[2765084, 89471, 1177, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2766298, 89511, 135, 3, 70, "Input",
  CellTags->"ToTFi"],
Cell[2766436, 89516, 774, 20, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2767271, 89543, 42, 1, 39, "ObjectName",
  CellTags->"Tr"],

Cell[CellGroupData[{
Cell[2767338, 89548, 54, 1, 49, "Subsubsection",
  CellTags->"Tr"],
Cell[2767395, 89551, 235, 7, 39, "Text",
  CellTags->"Tr"],

Cell[CellGroupData[{
Cell[2767655, 89562, 132, 3, 56, "Input",
  CellTags->"Tr"],
Cell[2767790, 89567, 764, 17, 70, "Output",
  CellTags->"Tr"]
}, Closed]],
Cell[2768569, 89587, 578, 23, 35, "Text",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2769184, 89615, 51, 1, 49, "Subsubsection",
  CellTags->"Tr"],

Cell[CellGroupData[{
Cell[2769260, 89620, 157, 4, 56, "Input",
  CellTags->"Tr"],
Cell[2769420, 89626, 232, 6, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2769689, 89637, 122, 3, 52, "Input",
  CellTags->"Tr"],
Cell[2769814, 89642, 235, 7, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2770086, 89654, 449, 12, 52, "Input",
  CellTags->"Tr"],
Cell[2770538, 89668, 1768, 51, 39, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2772343, 89724, 252, 7, 52, "Input",
  CellTags->"Tr"],
Cell[2772598, 89733, 789, 23, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2773424, 89761, 284, 7, 52, "Input",
  CellTags->"Tr"],
Cell[2773711, 89770, 13052, 322, 56, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2786800, 90097, 95, 2, 52, "Input",
  CellTags->"Tr"],
Cell[2786898, 90101, 126, 3, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2787061, 90109, 115, 3, 52, "Input",
  CellTags->"Tr"],
Cell[2787179, 90114, 368, 7, 75, "Print",
  CellTags->"Tr"]
}, Closed]],
Cell[2787562, 90124, 167, 4, 39, "Text",
  CellTags->"Tr"],

Cell[CellGroupData[{
Cell[2787754, 90132, 124, 3, 56, "Input",
  CellTags->"Tr"],
Cell[2787881, 90137, 127, 3, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2788045, 90145, 508, 13, 52, "Input",
  CellTags->"Tr"],
Cell[2788556, 90160, 6804, 170, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2795397, 90335, 205, 5, 52, "Input",
  CellTags->"Tr"],
Cell[2795605, 90342, 1629, 46, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2797271, 90393, 537, 14, 52, "Input",
  CellTags->"Tr"],
Cell[2797811, 90409, 485, 14, 39, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2798333, 90428, 269, 7, 52, "Input",
  CellTags->"Tr"],
Cell[2798605, 90437, 241, 7, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2798883, 90449, 129, 3, 52, "Input",
  CellTags->"Tr"],
Cell[2799015, 90454, 207, 5, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2799259, 90464, 410, 10, 52, "Input",
  CellTags->"Tr"],
Cell[2799672, 90476, 251, 7, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2799960, 90488, 129, 3, 52, "Input",
  CellTags->"Tr"],
Cell[2800092, 90493, 217, 5, 37, "Output",
  CellTags->"Tr"]
}, Closed]],

Cell[CellGroupData[{
Cell[2800346, 90503, 407, 11, 52, "Input",
  CellTags->"Tr"],
Cell[2800756, 90516, 532, 15, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2801325, 90536, 253, 6, 52, "Input",
  CellTags->"Tr"],
Cell[2801581, 90544, 256, 7, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2801874, 90556, 252, 6, 52, "Input",
  CellTags->"Tr"],
Cell[2802129, 90564, 294, 8, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2802460, 90577, 252, 6, 52, "Input",
  CellTags->"Tr"],
Cell[2802715, 90585, 105, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2802857, 90592, 251, 6, 52, "Input",
  CellTags->"Tr"],
Cell[2803111, 90600, 160, 4, 37, "Output"]
}, Closed]],
Cell[2803286, 90607, 148, 4, 52, "Input",
  CellTags->"Tr"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2803483, 90617, 66, 1, 39, "ObjectName",
  CellTags->"TraceDimension"],

Cell[CellGroupData[{
Cell[2803574, 90622, 66, 1, 49, "Subsubsection",
  CellTags->"TraceDimension"],
Cell[2803643, 90625, 143, 4, 39, "Text",
  CellTags->"TraceDimension"],
Cell[2803789, 90631, 213, 9, 39, "Text",
  CellTags->"TraceDimension"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2804051, 90646, 58, 1, 39, "ObjectName",
  CellTags->"TraceOfOne"],

Cell[CellGroupData[{
Cell[2804134, 90651, 62, 1, 49, "Subsubsection",
  CellTags->"TraceOfOne"],
Cell[2804199, 90654, 150, 4, 39, "Text",
  CellTags->"TraceOfOne"],
Cell[2804352, 90660, 295, 13, 39, "Text",
  CellTags->"TraceOfOne"]
}, Closed]],

Cell[CellGroupData[{
Cell[2804684, 90678, 59, 1, 49, "Subsubsection",
  CellTags->"TraceOfOne"],

Cell[CellGroupData[{
Cell[2804768, 90683, 317, 7, 56, "Input",
  CellTags->"TraceOfOne"],
Cell[2805088, 90692, 108, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2805233, 90699, 316, 7, 52, "Input",
  CellTags->"TraceOfOne"],
Cell[2805552, 90708, 163, 4, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2805776, 90719, 48, 1, 39, "ObjectName",
  CellTags->"Trick"],

Cell[CellGroupData[{
Cell[2805849, 90724, 57, 1, 49, "Subsubsection",
  CellTags->"Trick"],
Cell[2805909, 90727, 177, 5, 39, "Text",
  CellTags->"Trick"],
Cell[2806089, 90734, 678, 29, 39, "Text",
  CellTags->"Trick"]
}, Closed]],

Cell[CellGroupData[{
Cell[2806804, 90768, 63, 1, 50, "Subsubsection",
  CellTags->"Trick"],
Cell[2806870, 90771, 420, 15, 70, "Text",
  CellTags->"Trick"],

Cell[CellGroupData[{
Cell[2807315, 90790, 428, 11, 70, "Input",
  CellTags->"Trick"],
Cell[2807746, 90803, 329, 10, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2808112, 90818, 397, 11, 70, "Input",
  CellTags->"Trick"],
Cell[2808512, 90831, 619, 18, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2809168, 90854, 132, 3, 70, "Input",
  CellTags->"Trick"],
Cell[2809303, 90859, 777, 22, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2810117, 90886, 269, 6, 70, "Input",
  CellTags->"Trick"],
Cell[2810389, 90894, 491, 14, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2810917, 90913, 165, 4, 70, "Input",
  CellTags->"Trick"],
Cell[2811085, 90919, 234, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2811380, 90931, 66, 1, 39, "ObjectName",
  CellTags->"TrickIntegrate"],

Cell[CellGroupData[{
Cell[2811471, 90936, 66, 1, 49, "Subsubsection",
  CellTags->"TrickIntegrate"],
Cell[2811540, 90939, 484, 8, 73, "Text",
  CellTags->"TrickIntegrate"]
}, Closed]],

Cell[CellGroupData[{
Cell[2812061, 90952, 63, 1, 49, "Subsubsection",
  CellTags->"TrickIntegrate"],

Cell[CellGroupData[{
Cell[2812149, 90957, 450, 12, 56, "Input",
  CellTags->"TrickIntegrate"],
Cell[2812602, 90971, 1263, 33, 53, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2813926, 91011, 68, 1, 39, "ObjectName",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{
Cell[2814019, 91016, 67, 1, 49, "Subsubsection",
  CellTags->"TrickMandelstam"],
Cell[2814089, 91019, 1509, 56, 56, "Text",
  CellTags->"TrickMandelstam"]
}, Closed]],

Cell[CellGroupData[{
Cell[2815635, 91080, 64, 1, 49, "Subsubsection",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{
Cell[2815724, 91085, 664, 18, 57, "Input",
  CellTags->"TrickMandelstam"],
Cell[2816391, 91105, 279, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2816707, 91118, 721, 18, 52, "Input",
  CellTags->"TrickMandelstam"],
Cell[2817431, 91138, 277, 8, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2817769, 91153, 44, 1, 39, "ObjectName",
  CellTags->"Tr2"],

Cell[CellGroupData[{
Cell[2817838, 91158, 55, 1, 49, "Subsubsection",
  CellTags->"Tr2"],
Cell[2817896, 91161, 350, 7, 56, "Text",
  CellTags->"Tr2"],

Cell[CellGroupData[{
Cell[2818271, 91172, 131, 3, 56, "Input",
  CellTags->"Tr2"],
Cell[2818405, 91177, 206, 6, 37, "Output",
  CellTags->"Tr2"]
}, Closed]],
Cell[2818626, 91186, 645, 28, 35, "Text",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2819308, 91219, 52, 1, 49, "Subsubsection",
  CellTags->"Tr2"],

Cell[CellGroupData[{
Cell[2819385, 91224, 163, 4, 56, "Input",
  CellTags->"Tr2"],
Cell[2819551, 91230, 153, 4, 37, "Output",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2819741, 91239, 125, 3, 56, "Input",
  CellTags->"Tr2"],
Cell[2819869, 91244, 153, 4, 37, "Output",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2820059, 91253, 276, 8, 56, "Input",
  CellTags->"Tr2"],
Cell[2820338, 91263, 314, 9, 39, "Output",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2820689, 91277, 247, 7, 56, "Input",
  CellTags->"Tr2"],
Cell[2820939, 91286, 238, 7, 37, "Output",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2821214, 91298, 193, 5, 56, "Input",
  CellTags->"Tr2"],
Cell[2821410, 91305, 191, 5, 39, "Output",
  CellTags->"Tr2"]
}, Closed]],

Cell[CellGroupData[{
Cell[2821638, 91315, 227, 6, 56, "Input",
  CellTags->"Tr2"],
Cell[2821868, 91323, 229, 6, 39, "Output",
  CellTags->"Tr2"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2822158, 91336, 117, 2, 39, "ObjectName",
  CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{
Cell[2822300, 91342, 71, 1, 49, "Subsubsection",
  CellTags->"Twist2AlienOperator"],
Cell[2822374, 91345, 197, 5, 39, "Text",
  CellTags->"Twist2AlienOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[2822608, 91355, 68, 1, 49, "Subsubsection",
  CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{
Cell[2822701, 91360, 190, 4, 56, "Input",
  CellTags->"Twist2AlienOperator"],
Cell[2822894, 91366, 1167, 31, 58, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2824098, 91402, 291, 7, 52, "Input",
  CellTags->"Twist2AlienOperator"],
Cell[2824392, 91411, 4986, 113, 64, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2829439, 91531, 121, 2, 39, "ObjectName",
  CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{
Cell[2829585, 91537, 73, 1, 49, "Subsubsection",
  CellTags->"Twist2CounterOperator"],
Cell[2829661, 91540, 240, 5, 56, "Text",
  CellTags->"Twist2CounterOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[2829938, 91550, 70, 1, 49, "Subsubsection",
  CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{
Cell[2830033, 91555, 238, 5, 56, "Input",
  CellTags->"Twist2CounterOperator"],
Cell[2830274, 91562, 6081, 140, 90, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2836392, 91707, 189, 4, 52, "Input",
  CellTags->"Twist2CounterOperator"],
Cell[2836584, 91713, 1338, 36, 58, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2837959, 91754, 290, 7, 52, "Input",
  CellTags->"Twist2CounterOperator"],
Cell[2838252, 91763, 4112, 98, 64, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2842425, 91868, 76, 1, 39, "ObjectName",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2842526, 91873, 71, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],
Cell[2842600, 91876, 965, 15, 141, "Text",
  CellTags->"Twist2GluonOperator"],
Cell[2843568, 91893, 236, 9, 39, "Text",
  CellTags->"Twist2GluonOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[2843841, 91907, 100, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2843966, 91912, 403, 8, 76, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2844372, 91922, 487, 11, 70, "Output"]
}, Closed]],
Cell[2844874, 91936, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2844962, 91941, 325, 8, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2845290, 91951, 1176, 35, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2846503, 91991, 380, 9, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2846886, 92002, 1556, 45, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2848479, 92052, 263, 6, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2848745, 92060, 391, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2849173, 92076, 318, 7, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2849494, 92085, 989, 28, 70, "Output"]
}, Closed]],
Cell[2850498, 92116, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2850586, 92121, 420, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2851009, 92133, 1608, 48, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2852654, 92186, 390, 10, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2853047, 92198, 590, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2853674, 92220, 445, 11, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2854122, 92233, 16839, 419, 70, "Output"]
}, Closed]],
Cell[2870976, 92655, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2871064, 92660, 507, 12, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2871574, 92674, 7225, 184, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2878836, 92863, 467, 12, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2879306, 92877, 734, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2880077, 92903, 522, 13, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2880602, 92918, 36980, 900, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2917631, 93824, 101, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2917757, 93829, 317, 7, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2918077, 93838, 487, 11, 70, "Output"]
}, Closed]],
Cell[2918579, 93852, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2918667, 93857, 325, 8, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2918995, 93867, 1191, 36, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2920223, 93908, 380, 9, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2920606, 93919, 4148, 112, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2924791, 94036, 263, 6, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2925057, 94044, 391, 11, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2925485, 94060, 318, 7, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2925806, 94069, 3566, 94, 70, "Output"]
}, Closed]],
Cell[2929387, 94166, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2929475, 94171, 420, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2929898, 94183, 1693, 50, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2931628, 94238, 390, 10, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2932021, 94250, 590, 17, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2932648, 94272, 445, 11, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2933096, 94285, 27285, 681, 70, "Output"]
}, Closed]],
Cell[2960396, 94969, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2960484, 94974, 507, 12, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2960994, 94988, 7240, 185, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2968271, 95178, 467, 12, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2968741, 95192, 734, 21, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2969512, 95218, 522, 13, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[2970037, 95233, 52492, 1324, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3022578, 96563, 77, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],
Cell[3022658, 96566, 98, 1, 39, "Text",
  CellTags->"Twist2GluonOperator"],
Cell[3022759, 96569, 344, 7, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3023106, 96578, 463, 12, 56, "Input",
  CellTags->"Twist2GluonOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[3023606, 96595, 103, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3023734, 96600, 333, 7, 76, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3024070, 96609, 486, 11, 70, "Output"]
}, Closed]],
Cell[3024571, 96623, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3024659, 96628, 385, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3025047, 96640, 5497, 131, 70, "Output"]
}, Closed]],
Cell[3030559, 96774, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3030647, 96779, 420, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3031070, 96791, 91934, 1866, 70, "Output"]
}, Closed]],
Cell[3123019, 98660, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3123107, 98665, 563, 13, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3123673, 98680, 7225, 184, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3130947, 98870, 105, 1, 49, "Subsubsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3131077, 98875, 333, 7, 76, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3131413, 98884, 486, 11, 70, "Output"]
}, Closed]],
Cell[3131914, 98898, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3132002, 98903, 385, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3132390, 98915, 4792, 128, 70, "Output"]
}, Closed]],
Cell[3137197, 99046, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],
Cell[3137263, 99049, 463, 12, 56, "Input",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3137751, 99065, 390, 10, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3138144, 99077, 4793, 128, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3142974, 99210, 445, 11, 52, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3143422, 99223, 4793, 128, 70, "Output"]
}, Closed]],
Cell[3148230, 99354, 63, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3148318, 99359, 563, 13, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3148884, 99374, 7240, 185, 70, "Output"]
}, Closed]],
Cell[3156139, 99562, 79, 1, 35, "Text",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3156243, 99567, 634, 16, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3156880, 99585, 12315, 297, 70, "Output"]
}, Closed]],
Cell[3169210, 99885, 110, 3, 35, "Text",
  CellTags->"Twist2GluonOperator"],
Cell[3169323, 99890, 344, 7, 56, "Input",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[3169692, 99901, 522, 13, 56, "Input",
  CellTags->"Twist2GluonOperator"],
Cell[3170217, 99916, 8751, 198, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3179029, 100121, 76, 1, 39, "ObjectName",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3179130, 100126, 71, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],
Cell[3179204, 100129, 981, 15, 141, "Text",
  CellTags->"Twist2QuarkOperator"],
Cell[3180188, 100146, 236, 9, 39, "Text",
  CellTags->"Twist2QuarkOperator"]
}, Closed]],
Cell[3180439, 100158, 68, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3180532, 100163, 99, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3180656, 100168, 317, 7, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3180976, 100177, 486, 11, 70, "Output"]
}, Closed]],
Cell[3181477, 100191, 76, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3181578, 100196, 188, 4, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3181769, 100202, 822, 22, 70, "Output"]
}, Closed]],
Cell[3182606, 100227, 82, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3182713, 100232, 369, 10, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3183085, 100244, 2162, 57, 70, "Output"]
}, Closed]],
Cell[3185262, 100304, 88, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3185375, 100309, 457, 12, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3185835, 100323, 7442, 167, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3193326, 100496, 101, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3193452, 100501, 322, 7, 56, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3193777, 100510, 486, 11, 70, "Output"]
}, Closed]],
Cell[3194278, 100524, 76, 1, 35, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3194379, 100529, 188, 4, 56, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3194570, 100535, 646, 17, 70, "Output"]
}, Closed]],
Cell[3195231, 100555, 82, 1, 35, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3195338, 100560, 369, 10, 56, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3195710, 100572, 2087, 55, 70, "Output"]
}, Closed]],
Cell[3197812, 100630, 88, 1, 35, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3197925, 100635, 457, 12, 56, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3198385, 100649, 7734, 172, 70, "Output"]
}, Closed]],
Cell[3206134, 100824, 94, 1, 35, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3206253, 100829, 238, 6, 56, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3206494, 100837, 280, 7, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3206811, 100849, 439, 10, 52, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3207253, 100861, 2920, 78, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3210210, 100944, 523, 12, 72, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3210736, 100958, 17265, 362, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3228050, 101326, 103, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3228178, 101331, 318, 7, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3228499, 101340, 487, 11, 70, "Output"]
}, Closed]],
Cell[3229001, 101354, 179, 4, 70, "Text",
  CellTags->"Twist2QuarkOperator"],
Cell[3229183, 101360, 278, 6, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3229486, 101370, 248, 6, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3229737, 101378, 1127, 30, 70, "Output"]
}, Closed]],
Cell[3230879, 101411, 101, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3231005, 101416, 369, 10, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3231377, 101428, 4117, 95, 70, "Output"]
}, Closed]],
Cell[3235509, 101526, 88, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3235622, 101531, 188, 4, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3235813, 101537, 1322, 33, 70, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3237184, 101576, 105, 1, 49, "Subsubsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3237314, 101581, 318, 7, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3237635, 101590, 487, 11, 70, "Output"]
}, Closed]],
Cell[3238137, 101604, 76, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3238238, 101609, 248, 6, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3238489, 101617, 1127, 30, 70, "Output"]
}, Closed]],
Cell[3239631, 101650, 82, 1, 70, "Text",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[3239738, 101655, 369, 10, 70, "Input",
  CellTags->"Twist2QuarkOperator"],
Cell[3240110, 101667, 4117, 95, 70, "Output"]
}, Closed]],
Cell[3244242, 101765, 252, 6, 70, "Input",
  CellTags->"Twist2QuarkOperator"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3244543, 101777, 76, 1, 39, "ObjectName",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{
Cell[3244644, 101782, 71, 1, 49, "Subsubsection",
  CellTags->"Twist3QuarkOperator"],
Cell[3244718, 101785, 769, 12, 107, "Text",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{
Cell[3245512, 101801, 160, 3, 56, "Input",
  CellTags->"Twist3QuarkOperator"],
Cell[3245675, 101806, 392, 10, 37, "Output",
  CellTags->"Twist3QuarkOperator"]
}, Closed]],
Cell[3246082, 101819, 389, 14, 35, "Text",
  CellTags->"Twist3QuarkOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[3246508, 101838, 68, 1, 49, "Subsubsection",
  CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{
Cell[3246601, 101843, 158, 3, 56, "Input",
  CellTags->"Twist3QuarkOperator"],
Cell[3246762, 101848, 912, 25, 38, "Output",
  CellTags->"Twist3QuarkOperator"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3247735, 101880, 76, 1, 39, "ObjectName",
  CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{
Cell[3247836, 101885, 71, 1, 49, "Subsubsection",
  CellTags->"Twist4GluonOperator"],
Cell[3247910, 101888, 151, 4, 39, "Text",
  CellTags->"Twist4GluonOperator"],
Cell[3248064, 101894, 542, 19, 39, "Text",
  CellTags->"Twist4GluonOperator"]
}, Closed]],

Cell[CellGroupData[{
Cell[3248643, 101918, 68, 1, 49, "Subsubsection",
  CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{
Cell[3248736, 101923, 621, 15, 56, "Input",
  CellTags->"Twist4GluonOperator"],
Cell[3249360, 101940, 157388, 4060, 979, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3406809, 106007, 105, 1, 39, "ObjectName",
  CellTags->"TwoLoopSimplify"],

Cell[CellGroupData[{
Cell[3406939, 106012, 67, 1, 49, "Subsubsection",
  CellTags->"TwoLoopSimplify"],
Cell[3407009, 106015, 338, 7, 56, "Text",
  CellTags->"TwoLoopSimplify"],
Cell[3407350, 106024, 220, 9, 39, "Text",
  CellTags->"TwoLoopSimplify"]
}, Closed]],

Cell[CellGroupData[{
Cell[3407607, 106038, 64, 1, 49, "Subsubsection",
  CellTags->"TwoLoopSimplify"],
Cell[3407674, 106041, 104, 2, 56, "Input",
  CellTags->"TwoLoopSimplify"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3407827, 106049, 58, 1, 39, "ObjectName",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[3407910, 106054, 62, 1, 49, "Subsubsection",
  CellTags->"Uncontract"],
Cell[3407975, 106057, 259, 6, 56, "Text",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[3408259, 106067, 148, 3, 56, "Input",
  CellTags->"Uncontract"],
Cell[3408410, 106072, 410, 10, 70, "Output"]
}, Closed]],
Cell[3408835, 106085, 194, 9, 35, "Text",
  CellTags->"Uncontract"]
}, Closed]],

Cell[CellGroupData[{
Cell[3409066, 106099, 59, 1, 49, "Subsubsection",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[3409150, 106104, 279, 7, 56, "Input",
  CellTags->"Uncontract"],
Cell[3409432, 106113, 541, 16, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3410010, 106134, 172, 4, 52, "Input",
  CellTags->"Uncontract"],
Cell[3410185, 106140, 1046, 30, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3411268, 106175, 167, 4, 52, "Input",
  CellTags->"Uncontract"],
Cell[3411438, 106181, 281, 7, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3411756, 106193, 172, 4, 52, "Input",
  CellTags->"Uncontract"],
Cell[3411931, 106199, 663, 19, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3412631, 106223, 182, 4, 52, "Input",
  CellTags->"Uncontract"],
Cell[3412816, 106229, 1506, 43, 38, "Output"]
}, Closed]],
Cell[3414337, 106275, 90, 1, 35, "Text",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[3414452, 106280, 251, 6, 56, "Input",
  CellTags->"Uncontract"],
Cell[3414706, 106288, 269, 7, 70, "Output"]
}, Closed]],
Cell[3414990, 106298, 153, 4, 35, "Text",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[3415168, 106306, 294, 7, 56, "Input",
  CellTags->"Uncontract"],
Cell[3415465, 106315, 706, 21, 38, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3416208, 106341, 329, 8, 52, "Input",
  CellTags->"Uncontract"],
Cell[3416540, 106351, 1308, 39, 70, "Output"]
}, Closed]],
Cell[3417863, 106393, 166, 4, 52, "Input",
  CellTags->"Uncontract"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3418078, 106403, 84, 1, 39, "ObjectName",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[3418187, 106408, 75, 1, 49, "Subsubsection",
  CellTags->"UnDeclareNonCommutative"],
Cell[3418265, 106411, 201, 5, 39, "Text",
  CellTags->"UnDeclareNonCommutative"],
Cell[3418469, 106418, 366, 14, 39, "Text",
  CellTags->"UnDeclareNonCommutative"]
}, Closed]],

Cell[CellGroupData[{
Cell[3418872, 106437, 72, 1, 49, "Subsubsection",
  CellTags->"UnDeclareNonCommutative"],
Cell[3418947, 106440, 166, 3, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],
Cell[3419116, 106445, 154, 4, 70, "Text",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[3419295, 106453, 132, 2, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],
Cell[3419430, 106457, 112, 2, 70, "Output"]
}, Closed]],
Cell[3419557, 106462, 104, 1, 70, "Text",
  CellTags->"UnDeclareNonCommutative"],
Cell[3419664, 106465, 168, 3, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[3419857, 106472, 193, 4, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],
Cell[3420053, 106478, 113, 2, 70, "Output"]
}, Closed]],
Cell[3420181, 106483, 193, 4, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[3420399, 106491, 203, 4, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],
Cell[3420605, 106497, 171, 4, 70, "Output"]
}, Closed]],
Cell[3420791, 106504, 195, 4, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[3421011, 106512, 203, 4, 70, "Input",
  CellTags->"UnDeclareNonCommutative"],
Cell[3421217, 106518, 173, 4, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3421451, 106529, 48, 1, 39, "ObjectName",
  CellTags->"Upper"],

Cell[CellGroupData[{
Cell[3421524, 106534, 57, 1, 49, "Subsubsection",
  CellTags->"Upper"],
Cell[3421584, 106537, 127, 4, 39, "Text",
  CellTags->"Upper"],
Cell[3421714, 106543, 429, 18, 39, "Text",
  CellTags->"Upper"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3422192, 106567, 87, 1, 39, "ObjectName",
  CellTags->"UVPart"],

Cell[CellGroupData[{
Cell[3422304, 106572, 58, 1, 49, "Subsubsection",
  CellTags->"UVPart"],
Cell[3422365, 106575, 129, 4, 39, "Text",
  CellTags->"UVPart"]
}, Closed]],

Cell[CellGroupData[{
Cell[3422531, 106584, 55, 1, 49, "Subsubsection",
  CellTags->"UVPart"],
Cell[3422589, 106587, 95, 2, 56, "Input",
  CellTags->"UVPart"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3422733, 106595, 52, 1, 39, "ObjectName",
  CellTags->"Vectors"],

Cell[CellGroupData[{
Cell[3422810, 106600, 59, 1, 49, "Subsubsection",
  CellTags->"Vectors"],
Cell[3422872, 106603, 206, 5, 56, "Text",
  CellTags->"Vectors"],
Cell[3423081, 106610, 200, 8, 39, "Text",
  CellTags->"Vectors"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3423330, 106624, 54, 1, 39, "ObjectName",
  CellTags->"WriteOut"],

Cell[CellGroupData[{
Cell[3423409, 106629, 60, 1, 49, "Subsubsection",
  CellTags->"WriteOut"],
Cell[3423472, 106632, 217, 5, 56, "Text",
  CellTags->"WriteOut"],
Cell[3423692, 106639, 183, 8, 39, "Text",
  CellTags->"WriteOut"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3423924, 106653, 62, 1, 39, "ObjectName",
  CellTags->"WriteOutPaVe"],

Cell[CellGroupData[{
Cell[3424011, 106658, 64, 1, 49, "Subsubsection",
  CellTags->"WriteOutPaVe"],
Cell[3424078, 106661, 263, 6, 56, "Text",
  CellTags->"WriteOutPaVe"],
Cell[3424344, 106669, 430, 18, 39, "Text",
  CellTags->"WriteOutPaVe"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3424823, 106693, 50, 1, 39, "ObjectName",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3424898, 106698, 58, 1, 49, "Subsubsection",
  CellTags->"Write2"],
Cell[3424959, 106701, 298, 6, 56, "Text",
  CellTags->"Write2"],
Cell[3425260, 106709, 154, 4, 39, "Text",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3425439, 106717, 166, 4, 56, "Input",
  CellTags->"Write2"],
Cell[3425608, 106723, 286, 8, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3425931, 106736, 140, 3, 52, "Input",
  CellTags->"Write2"],
Cell[3426074, 106741, 637, 13, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3426748, 106759, 143, 3, 52, "Input",
  CellTags->"Write2"],
Cell[3426894, 106764, 143, 3, 70, "Output"]
}, Closed]],
Cell[3427052, 106770, 313, 14, 35, "Text",
  CellTags->"Write2"]
}, Closed]],

Cell[CellGroupData[{
Cell[3427402, 106789, 55, 1, 49, "Subsubsection",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3427482, 106794, 553, 15, 60, "Input",
  CellTags->"Write2"],
Cell[3428038, 106811, 1036, 29, 70, "Output"]
}, Closed]],
Cell[3429089, 106843, 259, 7, 52, "Input",
  CellTags->"Write2"],
Cell[3429351, 106852, 79, 1, 39, "Text",
  CellTags->"Write2"],
Cell[3429433, 106855, 227, 6, 56, "Input",
  CellTags->"Write2"],
Cell[3429663, 106863, 74, 1, 39, "Text",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3429762, 106868, 446, 11, 56, "Input",
  CellTags->"Write2"],
Cell[3430211, 106881, 645, 16, 71, "Output"]
}, Closed]],
Cell[3430871, 106900, 350, 9, 56, "Input",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3431246, 106913, 283, 7, 56, "Input",
  CellTags->"Write2"],
Cell[3431532, 106922, 853, 26, 70, "Output"]
}, Closed]],
Cell[3432400, 106951, 228, 6, 52, "Input",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3432653, 106961, 446, 11, 56, "Input",
  CellTags->"Write2"],
Cell[3433102, 106974, 1046, 24, 179, "Output"]
}, Closed]],
Cell[3434163, 107001, 350, 9, 56, "Input",
  CellTags->"Write2"],
Cell[3434516, 107012, 109, 4, 39, "Text",
  CellTags->"Write2"],
Cell[3434628, 107018, 301, 8, 56, "Input",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[3434954, 107030, 446, 11, 56, "Input",
  CellTags->"Write2"],
Cell[3435403, 107043, 890, 20, 125, "Output"]
}, Closed]],
Cell[3436308, 107066, 483, 13, 56, "Input",
  CellTags->"Write2"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3436840, 107085, 44, 1, 39, "ObjectName",
  CellTags->"XYT"],

Cell[CellGroupData[{
Cell[3436909, 107090, 55, 1, 49, "Subsubsection",
  CellTags->"XYT"],
Cell[3436967, 107093, 241, 9, 39, "Text",
  CellTags->"XYT"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3437257, 107108, 80, 1, 39, "ObjectName",
  CellTags->"ZeroMomentumInsertion"],

Cell[CellGroupData[{
Cell[3437362, 107113, 73, 1, 49, "Subsubsection",
  CellTags->"ZeroMomentumInsertion"],
Cell[3437438, 107116, 155, 4, 39, "Text",
  CellTags->"ZeroMomentumInsertion"],
Cell[3437596, 107122, 505, 18, 39, "Text",
  CellTags->"ZeroMomentumInsertion"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3438150, 107146, 48, 1, 39, "ObjectName",
  CellTags->"Zeta2"],

Cell[CellGroupData[{
Cell[3438223, 107151, 57, 1, 49, "Subsubsection",
  CellTags->"Zeta2"],
Cell[3438283, 107154, 95, 1, 39, "Text",
  CellTags->"Zeta2"],
Cell[3438381, 107157, 203, 8, 39, "Text",
  CellTags->"Zeta2"]
}, Closed]],

Cell[CellGroupData[{
Cell[3438621, 107170, 54, 1, 49, "Subsubsection",
  CellTags->"Zeta2"],

Cell[CellGroupData[{
Cell[3438700, 107175, 102, 2, 70, "Input",
  CellTags->"Zeta2"],
Cell[3438805, 107179, 147, 3, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3438989, 107187, 132, 3, 70, "Input",
  CellTags->"Zeta2"],
Cell[3439124, 107192, 127, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3439288, 107199, 170, 4, 70, "Input",
  CellTags->"Zeta2"],
Cell[3439461, 107205, 176, 4, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3439698, 107216, 66, 1, 39, "ObjectName",
  CellTags->"$Abbreviations"],

Cell[CellGroupData[{
Cell[3439789, 107221, 66, 1, 49, "Subsubsection",
  CellTags->"$Abbreviations"],
Cell[3439858, 107224, 294, 6, 56, "Text",
  CellTags->"$Abbreviations"],

Cell[CellGroupData[{
Cell[3440177, 107234, 154, 3, 56, "Input",
  CellTags->"$Abbreviations"],
Cell[3440334, 107239, 961, 17, 107, "Output",
  CellTags->"$Abbreviations"]
}, Closed]],
Cell[3441310, 107259, 705, 29, 35, "Text",
  CellTags->"$Abbreviations"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3442064, 107294, 44, 1, 39, "ObjectName",
  CellTags->"$AL"],

Cell[CellGroupData[{
Cell[3442133, 107299, 55, 1, 49, "Subsubsection",
  CellTags->"$AL"],
Cell[3442191, 107302, 117, 4, 39, "Text",
  CellTags->"$AL"],

Cell[CellGroupData[{
Cell[3442333, 107310, 95, 2, 56, "Input",
  CellTags->"$AL"],
Cell[3442431, 107314, 108, 2, 37, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3442576, 107321, 286, 7, 52, "Input",
  CellTags->"$AL"],
Cell[3442865, 107330, 707, 21, 38, "Output"]
}, Closed]],
Cell[3443587, 107354, 146, 4, 52, "Input",
  CellTags->"$AL"],

Cell[CellGroupData[{
Cell[3443758, 107362, 286, 7, 56, "Input",
  CellTags->"$AL"],
Cell[3444047, 107371, 603, 19, 38, "Output"]
}, Closed]],
Cell[3444665, 107393, 138, 4, 52, "Input",
  CellTags->"$AL"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3444852, 107403, 62, 1, 39, "ObjectName",
  CellTags->"$BreitMaison"],

Cell[CellGroupData[{
Cell[3444939, 107408, 64, 1, 49, "Subsubsection",
  CellTags->"$BreitMaison"],
Cell[3445006, 107411, 332, 9, 56, "Text",
  CellTags->"$BreitMaison"],
Cell[3445341, 107422, 403, 19, 39, "Text",
  CellTags->"$BreitMaison"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3445793, 107447, 50, 1, 39, "ObjectName",
  CellTags->"$Color"],

Cell[CellGroupData[{
Cell[3445868, 107452, 58, 1, 49, "Subsubsection",
  CellTags->"$Color"],
Cell[3445929, 107455, 131, 4, 39, "Text",
  CellTags->"$Color"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3446109, 107465, 58, 1, 39, "ObjectName",
  CellTags->"$Covariant"],

Cell[CellGroupData[{
Cell[3446192, 107470, 62, 1, 49, "Subsubsection",
  CellTags->"$Covariant"],
Cell[3446257, 107473, 186, 5, 39, "Text",
  CellTags->"$Covariant"],

Cell[CellGroupData[{
Cell[3446468, 107482, 109, 2, 56, "Input",
  CellTags->"$Covariant"],
Cell[3446580, 107486, 136, 3, 50, "Output",
  CellTags->"$Covariant"]
}, Closed]],

Cell[CellGroupData[{
Cell[3446753, 107494, 135, 3, 52, "Input",
  CellTags->"$Covariant"],
Cell[3446891, 107499, 145, 3, 37, "Output"]
}, Closed]],
Cell[3447051, 107505, 159, 4, 52, "Input",
  CellTags->"$Covariant"],

Cell[CellGroupData[{
Cell[3447235, 107513, 135, 3, 56, "Input",
  CellTags->"$Covariant"],
Cell[3447373, 107518, 143, 3, 39, "Output"]
}, Closed]],
Cell[3447531, 107524, 160, 4, 52, "Input",
  CellTags->"$Covariant"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3447740, 107534, 46, 1, 39, "ObjectName",
  CellTags->"$FCS"],

Cell[CellGroupData[{
Cell[3447811, 107539, 56, 1, 49, "Subsubsection",
  CellTags->"$FCS"],
Cell[3447870, 107542, 143, 4, 39, "Text",
  CellTags->"$FCS"],

Cell[CellGroupData[{
Cell[3448038, 107550, 97, 2, 56, "Input",
  CellTags->"$FCS"],
Cell[3448138, 107554, 739, 13, 69, "Output",
  CellTags->"$FCS"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3448938, 107574, 46, 1, 39, "ObjectName",
  CellTags->"$FCT"],

Cell[CellGroupData[{
Cell[3449009, 107579, 56, 1, 49, "Subsubsection",
  CellTags->"$FCT"],
Cell[3449068, 107582, 180, 5, 39, "Text",
  CellTags->"$FCT"],
Cell[3449251, 107589, 160, 4, 56, "Input",
  CellTags->"$FCT"],

Cell[CellGroupData[{
Cell[3449436, 107597, 206, 5, 56, "Input",
  CellTags->"$FCT"],
Cell[3449645, 107604, 218, 5, 70, "Output"]
}, Closed]],
Cell[3449878, 107612, 161, 4, 52, "Input",
  CellTags->"$FCT"],

Cell[CellGroupData[{
Cell[3450064, 107620, 206, 5, 56, "Input",
  CellTags->"$FCT"],
Cell[3450273, 107627, 218, 5, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3450552, 107639, 66, 1, 39, "ObjectName",
  CellTags->"$FeynCalcStuff"],

Cell[CellGroupData[{
Cell[3450643, 107644, 66, 1, 49, "Subsubsection",
  CellTags->"$FeynCalcStuff"],
Cell[3450712, 107647, 103, 1, 39, "Text",
  CellTags->"$FeynCalcStuff"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3450864, 107654, 70, 1, 39, "ObjectName",
  CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{
Cell[3450959, 107659, 68, 1, 49, "Subsubsection",
  CellTags->"$FeynCalcVersion"],
Cell[3451030, 107662, 127, 4, 39, "Text",
  CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{
Cell[3451182, 107670, 121, 2, 56, "Input",
  CellTags->"$FeynCalcVersion"],
Cell[3451306, 107674, 118, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3451485, 107683, 96, 1, 39, "ObjectName",
  CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{
Cell[3451606, 107688, 81, 1, 49, "Subsubsection",
  CellTags->"$FortranContinuationCharacter"],
Cell[3451690, 107691, 146, 4, 39, "Text",
  CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{
Cell[3451861, 107699, 147, 2, 56, "Input",
  CellTags->"$FortranContinuationCharacter"],
Cell[3452011, 107703, 114, 2, 37, "Output"]
}, Closed]],
Cell[3452140, 107708, 207, 9, 35, "Text",
  CellTags->"$FortranContinuationCharacter"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3452396, 107723, 50, 1, 39, "ObjectName",
  CellTags->"$Gauge"],

Cell[CellGroupData[{
Cell[3452471, 107728, 58, 1, 49, "Subsubsection",
  CellTags->"$Gauge"],
Cell[3452532, 107731, 263, 6, 56, "Text",
  CellTags->"$Gauge"],
Cell[3452798, 107739, 181, 9, 39, "Text",
  CellTags->"$Gauge"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3453028, 107754, 50, 1, 39, "ObjectName",
  CellTags->"$Larin"],

Cell[CellGroupData[{
Cell[3453103, 107759, 58, 1, 49, "Subsubsection",
  CellTags->"$Larin"],
Cell[3453164, 107762, 1184, 30, 107, "Text",
  CellTags->"$Larin"],

Cell[CellGroupData[{
Cell[3454373, 107796, 101, 2, 56, "Input",
  CellTags->"$Larin"],
Cell[3454477, 107800, 110, 2, 37, "Output"]
}, Closed]],
Cell[3454602, 107805, 409, 18, 35, "Text",
  CellTags->"$Larin"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3455060, 107829, 56, 1, 39, "ObjectName",
  CellTags->"$LimitTo4"],

Cell[CellGroupData[{
Cell[3455141, 107834, 61, 1, 49, "Subsubsection",
  CellTags->"$LimitTo4"],
Cell[3455205, 107837, 206, 5, 39, "Text",
  CellTags->"$LimitTo4"],

Cell[CellGroupData[{
Cell[3455436, 107846, 107, 2, 56, "Input",
  CellTags->"$LimitTo4"],
Cell[3455546, 107850, 134, 3, 37, "Output",
  CellTags->"$LimitTo4"]
}, Closed]],
Cell[3455695, 107856, 424, 19, 35, "Text",
  CellTags->"$LimitTo4"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3456168, 107881, 68, 1, 39, "ObjectName",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{
Cell[3456261, 107886, 67, 1, 49, "Subsubsection",
  CellTags->"$LorentzIndices"],
Cell[3456331, 107889, 165, 4, 39, "Text",
  CellTags->"$LorentzIndices"],
Cell[3456499, 107895, 172, 4, 56, "Input",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{
Cell[3456696, 107903, 391, 9, 56, "Input",
  CellTags->"$LorentzIndices"],
Cell[3457090, 107914, 894, 27, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3458021, 107946, 145, 3, 52, "Input",
  CellTags->"$LorentzIndices"],
Cell[3458169, 107951, 511, 12, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3458717, 107968, 361, 9, 52, "Input",
  CellTags->"$LorentzIndices"],
Cell[3459081, 107979, 1189, 37, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3460307, 108021, 145, 3, 52, "Input",
  CellTags->"$LorentzIndices"],
Cell[3460455, 108026, 744, 18, 70, "Output"]
}, Closed]],
Cell[3461214, 108047, 173, 4, 52, "Input",
  CellTags->"$LorentzIndices"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3461436, 108057, 60, 1, 39, "ObjectName",
  CellTags->"$MIntegrate"],

Cell[CellGroupData[{
Cell[3461521, 108062, 63, 1, 49, "Subsubsection",
  CellTags->"$MIntegrate"],
Cell[3461587, 108065, 137, 4, 39, "Text",
  CellTags->"$MIntegrate"],

Cell[CellGroupData[{
Cell[3461749, 108073, 111, 2, 56, "Input",
  CellTags->"$MIntegrate"],
Cell[3461863, 108077, 128, 3, 37, "Output"]
}, Closed]],
Cell[3462006, 108083, 204, 9, 35, "Text",
  CellTags->"$MIntegrate"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3462259, 108098, 44, 1, 39, "ObjectName",
  CellTags->"$MU"],

Cell[CellGroupData[{
Cell[3462328, 108103, 55, 1, 49, "Subsubsection",
  CellTags->"$MU"],
Cell[3462386, 108106, 152, 4, 39, "Text",
  CellTags->"$MU"],

Cell[CellGroupData[{
Cell[3462563, 108114, 95, 2, 56, "Input",
  CellTags->"$MU"],
Cell[3462661, 108118, 108, 2, 37, "Output"]
}, Closed]],
Cell[3462784, 108123, 187, 9, 35, "Text",
  CellTags->"$MU"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3463020, 108138, 70, 1, 39, "ObjectName",
  CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{
Cell[3463115, 108143, 68, 1, 49, "Subsubsection",
  CellTags->"$MemoryAvailable"],
Cell[3463186, 108146, 444, 11, 56, "Text",
  CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{
Cell[3463655, 108161, 121, 2, 56, "Input",
  CellTags->"$MemoryAvailable"],
Cell[3463779, 108165, 108, 2, 37, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3463948, 108174, 54, 1, 39, "ObjectName",
  CellTags->"$NonComm"],

Cell[CellGroupData[{
Cell[3464027, 108179, 60, 1, 49, "Subsubsection",
  CellTags->"$NonComm"],
Cell[3464090, 108182, 112, 4, 39, "Text",
  CellTags->"$NonComm"],

Cell[CellGroupData[{
Cell[3464227, 108190, 105, 2, 56, "Input",
  CellTags->"$NonComm"],
Cell[3464335, 108194, 664, 12, 75, "Output"]
}, Closed]],
Cell[3465014, 108209, 330, 14, 35, "Text",
  CellTags->"$NonComm"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3465393, 108229, 64, 1, 39, "ObjectName",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{
Cell[3465482, 108234, 65, 1, 49, "Subsubsection",
  CellTags->"$PairBrackets"],
Cell[3465550, 108237, 156, 4, 39, "Text",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{
Cell[3465731, 108245, 118, 2, 56, "Input",
  CellTags->"$PairBrackets"],
Cell[3465852, 108249, 113, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3466002, 108256, 250, 7, 52, "Input",
  CellTags->"$PairBrackets"],
Cell[3466255, 108265, 616, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3466908, 108289, 146, 3, 52, "Input",
  CellTags->"$PairBrackets"],
Cell[3467057, 108294, 112, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3467206, 108301, 250, 7, 52, "Input",
  CellTags->"$PairBrackets"],
Cell[3467459, 108310, 692, 19, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3468188, 108334, 147, 3, 52, "Input",
  CellTags->"$PairBrackets"],
Cell[3468338, 108339, 113, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3468512, 108348, 66, 1, 39, "ObjectName",
  CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{
Cell[3468603, 108353, 66, 1, 49, "Subsubsection",
  CellTags->"$SpinorMinimal"],
Cell[3468672, 108356, 242, 5, 56, "Text",
  CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{
Cell[3468939, 108365, 117, 2, 56, "Input",
  CellTags->"$SpinorMinimal"],
Cell[3469059, 108369, 110, 2, 37, "Output"]
}, Closed]],
Cell[3469184, 108374, 213, 9, 35, "Text",
  CellTags->"$SpinorMinimal"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3469446, 108389, 62, 1, 39, "ObjectName",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{
Cell[3469533, 108394, 64, 1, 49, "Subsubsection",
  CellTags->"$VeryVerbose"],
Cell[3469600, 108397, 222, 5, 56, "Text",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{
Cell[3469847, 108406, 116, 2, 56, "Input",
  CellTags->"$VeryVerbose"],
Cell[3469966, 108410, 109, 2, 37, "Output"]
}, Closed]],
Cell[3470090, 108415, 173, 4, 52, "Input",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{
Cell[3470288, 108423, 326, 9, 56, "Input",
  CellTags->"$VeryVerbose"],
Cell[3470617, 108434, 96, 3, 70, "Print"],
Cell[3470716, 108439, 257, 6, 70, "Print"],
Cell[3470976, 108447, 275, 6, 70, "Print"],
Cell[3471254, 108455, 1107, 31, 70, "Output"]
}, Closed]],
Cell[3472376, 108489, 163, 4, 52, "Input",
  CellTags->"$VeryVerbose"],
Cell[3472542, 108495, 172, 9, 39, "Text",
  CellTags->"$V0"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3472763, 108510, 48, 1, 39, "ObjectName",
  CellTags->"$West"],

Cell[CellGroupData[{
Cell[3472836, 108515, 57, 1, 49, "Subsubsection",
  CellTags->"$West"],
Cell[3472896, 108518, 492, 13, 56, "Text",
  CellTags->"$West"],
Cell[3473391, 108533, 170, 8, 39, "Text",
  CellTags->"$West"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3473610, 108547, 54, 0, 30, "Commentary"],

Cell[CellGroupData[{
Cell[3473689, 108551, 158, 4, 70, "Input"],
Cell[3473850, 108557, 119, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3474006, 108564, 112, 2, 70, "Input"],
Cell[3474121, 108568, 116, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3474274, 108575, 114, 2, 70, "Input"],
Cell[3474391, 108579, 116, 2, 70, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3474544, 108586, 109, 2, 70, "Input"],
Cell[3474656, 108590, 127, 2, 70, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3474844, 108599, 71, 1, 56, "Section",
  CellTags->"Experimental"],

Cell[CellGroupData[{
Cell[3474940, 108604, 74, 1, 45, "ObjectName",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{
Cell[3475039, 108609, 70, 1, 49, "Subsubsection",
  CellTags->"DoPolarizationSums"],
Cell[3475112, 108612, 292, 7, 114, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3475407, 108621, 340, 14, 39, "Text",
  CellTags->"DoPolarizationSums"]
}, Closed]],

Cell[CellGroupData[{
Cell[3475784, 108640, 67, 1, 49, "Subsubsection",
  CellTags->"DoPolarizationSums"],
Cell[3475854, 108643, 311, 6, 56, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3476168, 108651, 126, 4, 39, "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{
Cell[3476319, 108659, 644, 17, 63, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3476966, 108678, 996, 29, 40, "Output",
  CellTags->"DoPolarizationSums"],
Cell[3477965, 108709, 94, 2, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3478062, 108713, 115, 3, 37, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[3478192, 108719, 1599, 48, 40, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3479794, 108769, 608, 21, 39, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3480405, 108792, 124, 3, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3480532, 108797, 331, 9, 43, "Input",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{
Cell[3480888, 108810, 273, 7, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3481164, 108819, 398, 10, 89, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[3481577, 108832, 233, 9, 42, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3481813, 108843, 286, 8, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3482102, 108853, 341, 9, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3482446, 108864, 359, 13, 42, "Text",
  CellTags->"DoPolarizationSums"],
Cell[3482808, 108879, 869, 30, 40, "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{
Cell[3483702, 108913, 469, 12, 64, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3484174, 108927, 470, 14, 89, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[3484659, 108944, 767, 27, 40, "Text",
  CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{
Cell[3485451, 108975, 468, 12, 64, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3485922, 108989, 470, 14, 89, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3486429, 109008, 1000, 25, 83, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3487432, 109035, 1213, 35, 41, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3488682, 109075, 108, 2, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3488793, 109079, 385, 11, 41, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3489215, 109095, 94, 2, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3489312, 109099, 115, 3, 37, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3489464, 109107, 98, 2, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3489565, 109111, 115, 3, 37, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3489717, 109119, 104, 2, 43, "Input",
  CellTags->"DoPolarizationSums"],
Cell[3489824, 109123, 115, 3, 37, "Output",
  CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[3489954, 109129, 199, 4, 56, "Input",
  CellTags->"DoPolarizationSums"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3490202, 109139, 91, 1, 39, "ObjectName",
  CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{
Cell[3490318, 109144, 70, 1, 49, "Subsubsection",
  CellTags->"FeynmanDoIntegrals"],
Cell[3490391, 109147, 1519, 23, 217, "Text",
  CellTags->"FeynmanDoIntegrals"],
Cell[3491913, 109172, 370, 14, 39, "Text",
  CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{
Cell[3492308, 109190, 161, 3, 56, "Input",
  CellTags->"FeynmanDoIntegrals"],
Cell[3492472, 109195, 1218, 28, 75, "Output",
  CellTags->"FeynmanDoIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3493739, 109229, 67, 1, 49, "Subsubsection",
  CellTags->"FeynmanDoIntegrals"],
Cell[3493809, 109232, 109, 2, 56, "Input",
  CellTags->"FeynmanDoIntegrals"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3493967, 109240, 93, 1, 39, "ObjectName",
  CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{
Cell[3494085, 109245, 71, 1, 49, "Subsubsection",
  CellTags->"FeynmanParametrize1"],
Cell[3494159, 109248, 718, 12, 132, "Text",
  CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{
Cell[3494902, 109264, 162, 3, 56, "Input",
  CellTags->"FeynmanParametrize1"],
Cell[3495067, 109269, 563, 13, 56, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3495679, 109288, 68, 1, 49, "Subsubsection",
  CellTags->"FeynmanParametrize1"],
Cell[3495750, 109291, 110, 2, 56, "Input",
  CellTags->"FeynmanParametrize1"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3495909, 109299, 81, 1, 39, "ObjectName",
  CellTags->"FeynmanReduce"],

Cell[CellGroupData[{
Cell[3496015, 109304, 65, 1, 49, "Subsubsection",
  CellTags->"FeynmanReduce"],
Cell[3496083, 109307, 724, 12, 132, "Text",
  CellTags->"FeynmanReduce"],

Cell[CellGroupData[{
Cell[3496832, 109323, 150, 3, 56, "Input",
  CellTags->"FeynmanReduce"],
Cell[3496985, 109328, 314, 8, 37, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3497348, 109342, 62, 1, 49, "Subsubsection",
  CellTags->"FeynmanReduce"],
Cell[3497413, 109345, 104, 2, 56, "Input",
  CellTags->"FeynmanReduce"]
}, Closed]]
}, Closed]],
Cell[3497544, 109351, 125, 4, 35, "Text",
  CellTags->"Experimental"],
Cell[3497672, 109357, 189, 5, 43, "Input",
  CellTags->"Experimental"],
Cell[3497864, 109364, 101, 1, 39, "Text",
  CellTags->"Experimental"],
Cell[3497968, 109367, 231, 5, 43, "Input",
  CellTags->"Experimental"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

